﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Text;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.IO;
using System.Globalization;

#region Assetlink Includes
using Assetlink.Solutions;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.ReportFacade;
using Assetlink.Solutions.Facade.UGMFacade;
using Assetlink.Solutions.Facade.InfrastructureFacade;
using Assetlink.CustomControls.HelperClasses;
#endregion

namespace Assetlink.Solutions.Reports
{
    #region Directives Include

    using XML = String;
    using ERRORXML = String;

    #endregion

    public partial class AdHocReportPrintView : System.Web.UI.Page
    {
        #region Member Properties
        ALSolutionCommon    ALSolutionCommonObj     = null;
        ALParam             ALParamObj              = null;
        String              ReportID                = null;        
        XML                 ReportData              = null;        
        String              TargetEntityName        = null;
        String              TotalFor_LocalizedStr   = null;
        XML                 ReportColumnsTemplate   = null;        
        String              widthlist               = null;
        String              ReportName              = null;
        String              ReportDescription       = null;
        String              ReportGeneratedBy       = null;
        String              ReportGeneratedOn       = null;
        bool                IsAggregationRequired   = false;
        bool                IsReportDataAvailable   = true;
        String              FormatXMLPath           = null;
        String              cultureInfo             = null;

        public String       TenantName              = null;
        XML                 GroupDataTypeMap        = "";
        String              MaxSearchResultAlert    = "";
        String              FromPreviewScreen       = null;
        #endregion

        #region private properties
        private String PersonalizedFilterList
        {
            get
            { 
                object Obj = ViewState["PersonalizedFilterList"];
                if (Obj == null)
                    return String.Empty;
                else
                    return (String)Obj;
            }
            set
            {
                ViewState["PersonalizedFilterList"] = value;
            }
        }
        #endregion

        #region Standard Methods

        protected override void InitializeCulture()
        {
            ALSolutionCommonObj = new ALSolutionCommon();
            ALParamObj = new ALParam();

            ALSolutionCommonObj.httpContext = HttpContext.Current;
            ALSolutionCommonObj.InitALParam(ref ALParamObj);

            TenantName = ALParamObj.TenantName;

            Page.UICulture = Localization.GetCultureName(ALParamObj.Language, ALParamObj.TenantName);
            Localization.SetCultureForPage(ALParamObj.Language, ALParamObj.TenantName);
        }

        protected void Page_Init(object sender, EventArgs e)
        {
            try
            {
                initializeURLParameters();                
            }
            catch (Exception ExceptionObject)
            {
                redirectToErrorPage(ExceptionObject.Message);
            }
        }
        
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                Page.Header.DataBind();
                initializeControls();
                GetReportDetails();                
                SetReportHeader();
            }
            catch (Exception ExceptionObject)
            {
                redirectToErrorPage(ExceptionObject.Message);
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            try
            {
                base.Render(writer);
                StringBuilder sb = new StringBuilder();
                GenerateHTML(ref sb);
                Page.Response.Write(sb.ToString());                
                if(!String.IsNullOrEmpty(MaxSearchResultAlert))
                    Page.Response.Write("<script>alert('"+ MaxSearchResultAlert+"');</script>");
            }
            catch (Exception ExceptionObject)
            {
                redirectToErrorPage(ExceptionObject.Message);
            }
        }

        #endregion

        #region Private Methods

        private void SetReportHeader()
        {
            NameVal.InnerHtml           = ReportName;
            DescriptionVal.InnerHtml    = ReportDescription;

            if (String.IsNullOrEmpty(ReportGeneratedBy))
            {
                UserFacade userFacadeObj = new UserFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                XML userAttrsXML = "<USER><FIRSTNAME/><LASTNAME/><LOGINNAME/><MIDDLENAME/></USER>";
                XML userXML = null;
                ERRORXML errorxml_Output = userFacadeObj.GetUser(ulong.Parse(ALParamObj.UserID), userAttrsXML, null, out userXML);
                XPathDocument xpDoc_UserColl = new XPathDocument(new StringReader(userXML));
                XPathNavigator xpNav_User = xpDoc_UserColl.CreateNavigator().SelectSingleNode("USERCOLLECTION");
                ReportGeneratedBy = ALCommon.GetFormattedUserName(xpNav_User);
            }   
            if (String.IsNullOrEmpty(ReportGeneratedOn))
            {
                DateTime now = DateTime.Now;                
                ReportGeneratedOn = ALCommon.FormatDateTimeField(FormatXMLPath, now.Day.ToString(), now.Month.ToString(), now.Year.ToString(), now.Hour.ToString(), now.Minute.ToString(), now.Second.ToString(), System.Globalization.CultureInfo.GetCultureInfo(cultureInfo), null);
            }
            
            GeneratedByVal.InnerHtml = ReportGeneratedBy;
            GeneratedOnVal.InnerHtml = ReportGeneratedOn;            
        }

        private void GetReportDetails()
        {
            SearchConfiguration facade_SearchConfiguration  = null;
            StringBuilder       strbldr_SearchInputXML      = null;
            ERRORXML            errorxml_Output             = null;
            XML                 ReportConfig                = null;
            XDocument           xDoc_ReportConfig           = null;
            XElement            xEle_AdHocReport            = null;
            XElement            xEle_SearchConfig           = null;
            XElement            xEle_FormConfig             = null;
            String              str_PersonalizeFiltersPath  = null;
            String              DataXML                     = null;
            String              xml_FormControlInput        = null;
            List<ObjectInfo>    ObjInfo                     = null;
            String              FormInputXml                = null;
            InfrastructureFacadeClass ObjInfrast            = null;
            String              UserPersonalization         = null;
            String              str_TempData                = null;

            facade_SearchConfiguration = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);

            strbldr_SearchInputXML = new StringBuilder();
            strbldr_SearchInputXML.AppendFormat(@"<SEARCH>
	                <TARGETOBJECT>ADHOCREPORT</TARGETOBJECT>
	                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                <SESSIONINFO UserID='{0}' Language='{1}' TenantName='{2}' SessionID='{3}'/>
	                <TARGETENTITY>
		                <SELECTLIST>
			                <EXPRESSION>
				                <FIELDNAME>REPORTID</FIELDNAME>
				                <ALIAS>REPORTID</ALIAS>
			                </EXPRESSION>
                            <EXPRESSION>
				                <FIELDNAME>REPORTNAME</FIELDNAME>
				                <ALIAS>REPORTNAME</ALIAS>
			                </EXPRESSION>
			                <EXPRESSION>
				                <FIELDNAME>REPORTDESCRIPTION</FIELDNAME>
				                <ALIAS>REPORTDESCRIPTION</ALIAS>
			                </EXPRESSION>
                            <EXPRESSION>
				                <FIELDNAME>REPORTCONFIGXML</FIELDNAME>
				                <ALIAS>REPORTCONFIGXML</ALIAS>
			                </EXPRESSION>
		                </SELECTLIST>
                        <FILTERLIST>
			                <FILTER>
				                <FILTERID>1</FILTERID>
				                <EXPRESSION>REPORTID</EXPRESSION>
				                <OPERATOR>EQUALTO</OPERATOR>
				                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
				                <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                </FILTER>
                        </FILTERLIST>
	                </TARGETENTITY>
                </SEARCH>", ALParamObj.UserID, ALParamObj.Language, ALParamObj.TenantName, ALParamObj.SessionID, ReportID);
            errorxml_Output = facade_SearchConfiguration.GetHierarchialData(strbldr_SearchInputXML.ToString(), ref ReportConfig);
            if ((!String.IsNullOrEmpty(errorxml_Output)) || (String.IsNullOrEmpty(ReportConfig)))
            {
                throw new Exception("Err_AdhocReport_Get_Report_Configuration");
            }
            
            errorxml_Output = GetReportColumnsConfigTemplate(ReportConfig);
            if (!String.IsNullOrEmpty(errorxml_Output))
                throw new Exception(errorxml_Output);           

            #region set Personalization Form Data
            xDoc_ReportConfig   = XDocument.Parse(ReportConfig);
            xEle_AdHocReport    = xDoc_ReportConfig.Root.XPathSelectElement("ADHOCREPORT");
            xEle_SearchConfig   = xEle_AdHocReport.XPathSelectElement("REPORTCONFIGXML/CONFIGURATION");
            if (xEle_SearchConfig != null)
            {
                xEle_FormConfig = xEle_SearchConfig.XPathSelectElement("Form");
                if (xEle_FormConfig != null && !String.IsNullOrEmpty(xEle_FormConfig.ToString()))
                {   
                    str_PersonalizeFiltersPath          = String.Format("{0}{1}_PersonalizeFilters.xml", ALSolutionCommon.GetTempPath(ALParamObj.SessionID), ReportID);                    
                    if (!File.Exists(str_PersonalizeFiltersPath))
                    {
                        //Get User Personalization for report filters and save it.
                        ObjInfrast      = new InfrastructureFacadeClass(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                        errorxml_Output = ObjInfrast.GetUserPersonalization("ADHOCREPORTFILTERPERSONALIZATION", "RPT_" + ReportID, out UserPersonalization);
                        if (String.IsNullOrEmpty(errorxml_Output))
                        {
                            if (!String.IsNullOrEmpty(UserPersonalization))
                            {
                                File.WriteAllText(str_PersonalizeFiltersPath, UserPersonalization);
                                DataXML                 = ALCommon.GetInnerValue(ref UserPersonalization, "<FORMDATAXML>", "</FORMDATAXML>");
                                str_TempData            = ALCommon.GetInnerValue(ref UserPersonalization, "<ADHOCREPORTFILTERS>", "</ADHOCREPORTFILTERS>");
                                if (!String.IsNullOrEmpty(str_TempData))
                                {
                                    PersonalizedFilterList = String.Format("<SEARCH><TARGETENTITY>{0}</TARGETENTITY></SEARCH>", str_TempData);
                                }
                            }
                        }
                        else
                        {
                            //Redirect to Error Page
                        }
                    }
                    else
                    {
                        DataXML         = File.ReadAllText(str_PersonalizeFiltersPath);                       
                        str_TempData    = ALCommon.GetInnerValue(ref DataXML, "<ADHOCREPORTFILTERS>", "</ADHOCREPORTFILTERS>");
                        if (!String.IsNullOrEmpty(str_TempData))
                            PersonalizedFilterList = String.Format("<SEARCH><TARGETENTITY>{0}</TARGETENTITY></SEARCH>", str_TempData);

                        DataXML         = ALCommon.GetInnerValue(ref DataXML, "<FORMDATAXML>", "</FORMDATAXML>");
                    }

                    //if (File.Exists(str_PersonalizeFiltersPath))
                    {
                        //DataXML                                     = File.ReadAllText(str_PersonalizeFiltersPath);
                        //DataXML                                     = ALCommon.GetInnerValue(ref DataXML, "<FORMDATAXML>", "</FORMDATAXML>");
                        Frm_PersonalizeFilters.TenantName           = ALParamObj.TenantName;
                        Frm_PersonalizeFilters.ConfigurationXMLType = CustomControls.Form.InputType.InnerXML;
                        Frm_PersonalizeFilters.ConfigurationXML     = xEle_FormConfig.ToString();

                        FormFacade FormFacadeObj                    = new FormFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                        FormFacadeObj.FormName                      = xEle_FormConfig.Attribute("Name").Value;//"ODSDIGITALASSET_Report";
                        FormFacadeObj.SolutionName                  = ALParamObj.SolutionName;
                        FormFacadeObj.ShowConfiguredFields          = true;
                        FormFacadeObj.ConfigurationXMLType          = "INNERXML";
                        FormFacadeObj.ConfigurationXML              = xEle_FormConfig.ToString();

                        errorxml_Output                             = FormFacadeObj.GetFormInputXML("READONLY", DataXML,out FormInputXml, out ObjInfo);
                        if (String.IsNullOrEmpty(errorxml_Output) && !String.IsNullOrEmpty(FormInputXml))
                        {
                            Frm_PersonalizeFilters.InputXMLType = Assetlink.CustomControls.Form.InputType.InnerXML;
                            Frm_PersonalizeFilters.InputXML     = FormInputXml;
                        }
                    }
                    //else
                    //    Div_Filters.Attributes.Add("style","display:none;");
                }
                else
                    Div_PersonalizeFilters.Attributes.Add("style", "display:none;");
            }
            #endregion

            errorxml_Output = GetReportData(ReportConfig);
            if (!String.IsNullOrEmpty(errorxml_Output))
                throw new Exception(errorxml_Output);
        }

        private ERRORXML GetReportColumnsConfigTemplate(XML ReportConfig)
        {
            XDocument   xDoc_ReportConfig       = XDocument.Parse(ReportConfig);
            XElement    xEle_AdHocReport        = xDoc_ReportConfig.Root.XPathSelectElement("ADHOCREPORT");
            XElement    xEle_SearchConfig       = xEle_AdHocReport.XPathSelectElement("REPORTCONFIGXML/CONFIGURATION");
            ERRORXML    ErrorXML                = null;

            if (xEle_SearchConfig == null)
            {
                ErrorXML = "Err_AdhocReport_Invalid_Report_Configuration";
                return ErrorXML;
            }

            XElement xEle_ConfigColl            = xEle_SearchConfig.XPathSelectElement("DisplayLayout/SeriesCollection/AdHocReportsDisplayColumns/CONFIGCOLUMNSCOLLECTION");
            XElement xEle_GroupByColumn         = xEle_ConfigColl.XPathSelectElement("CONFIGCOLUMNS[GROUPBYLEVEL = '1']");            
            IEnumerable<XElement> xEles_Column  = xEle_ConfigColl.XPathSelectElements("CONFIGCOLUMNS[GROUPBYLEVEL='None']");

            IEnumerable<XElement> xEles_GroupByColumn = xEle_ConfigColl.XPathSelectElements("CONFIGCOLUMNS[GROUPBYLEVEL!='None']");

            Int32 int_ColumnCount           = 0;
            String str_DataRelationName     = null;
            String str_DataObjectName       = null;
            String str_DataFieldName        = null;
            String str_DataFieldType        = null;
            String str_DataFieldDisplayName = null;
            String str_Aggregation          = null;
            String str_Width                = null;
            String str_GroupByLevel         = null;
            String GroupedBy_LocalizedStr   = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_Preview_GroupedBy");
            String[] widthListArr           = null;

            if (xEle_SearchConfig.Element("SearchTarget") != null)
                TargetEntityName = xEle_SearchConfig.Element("SearchTarget").Attribute("ObjectName").Value;

            if (String.IsNullOrEmpty(TargetEntityName))
            {
                ErrorXML = "Err_AdhocReport_Invalid_Report_EntityName";
                return ErrorXML;
            }

            ReportName              = xEle_AdHocReport.Element("REPORTNAME").Value;
            ReportDescription       = xEle_AdHocReport.Element("REPORTDESCRIPTION").Value;
            
            ReportColumnsTemplate   = "<GRIDCOLUMNS>";

            if (!String.IsNullOrEmpty(widthlist))
                widthListArr = widthlist.Split(new String[] { "$SET$" }, StringSplitOptions.None);

            #region Create GroupDataType Map for formatting the group values as per the group data type            
            GroupDataTypeMap = "<GROUPS>";
            if (xEles_GroupByColumn != null)
            {
                foreach (XElement xEle_GroupedColumn in xEles_GroupByColumn)
                {
                    str_DataFieldType   = xEle_GroupedColumn.Element("FIELDDATATYPE").Value;
                    str_GroupByLevel    = xEle_GroupedColumn.Element("GROUPBYLEVEL").Value;
                    GroupDataTypeMap += "<GROUP DataType='" + str_DataFieldType + "' level='" + str_GroupByLevel + "'/>";
                }
            }
            GroupDataTypeMap += "</GROUPS>";
            #endregion

            if (xEle_GroupByColumn != null)
            {
                if (widthListArr != null)
                    str_Width = widthListArr[int_ColumnCount];
                else
                {
                    str_Width = "200"; //Default Width
                    if (xEle_GroupByColumn.Element("WIDTH") != null)
                    {
                        if (!String.IsNullOrEmpty(xEle_GroupByColumn.Element("WIDTH").Value))
                            str_Width = xEle_GroupByColumn.Element("WIDTH").Value;
                    }
                }   

                ReportColumnsTemplate += "<COLUMN><OBJECTNAME>" + TargetEntityName
                                        + "</OBJECTNAME><FIELDDATATYPE>TEXT"
                                        + "</FIELDDATATYPE><RELATIONSHIPNAME>" + TargetEntityName
                                        + "</RELATIONSHIPNAME><AGGREGATION>NONE"
                                        + "</AGGREGATION><FIELDNAME>GROUPDATAVALUE"
                                        + "</FIELDNAME><FIELDTITLE>" + GroupedBy_LocalizedStr
                                        + "</FIELDTITLE><WIDTH>" + str_Width
                                        + "</WIDTH></COLUMN>";
                int_ColumnCount++;
                str_Width = "";
            }

            foreach (XElement xEle_Column in xEles_Column)
            {                
                str_DataRelationName        = xEle_Column.Element("RELATIONSHIPNAME").Value;
                str_DataObjectName          = xEle_Column.Element("OBJECTNAME").Value;
                str_DataFieldName           = xEle_Column.Element("FIELDNAME").Value;
                str_DataFieldType           = xEle_Column.Element("FIELDDATATYPE").Value;
                str_DataFieldDisplayName    = xEle_Column.Element("FIELDTITLE").Value;
                str_Aggregation             = xEle_Column.Element("AGGREGATION").Value;

                if (str_Aggregation.ToUpper() != "NONE")
                    IsAggregationRequired = true;

                if (widthListArr != null)
                    str_Width = widthListArr[int_ColumnCount];
                else
                {
                    if (xEle_Column.Element("WIDTH") != null)
                    {
                        if (!String.IsNullOrEmpty(xEle_Column.Element("WIDTH").Value))
                            str_Width = xEle_Column.Element("WIDTH").Value;
                    }

                    if (String.IsNullOrEmpty(str_Width)) //Apply Default Width
                    {
                        switch (str_DataFieldType.ToUpper())
                        {
                            case Constants.DATA_TYPE_DATE:                            
                            case Constants.DATA_TYPE_FLOAT:
                            case Constants.DATA_TYPE_NUMBER:
                            case Constants.DATA_TYPE_CURRENCY:
                                str_Width = "120";
                                break;
                            case Constants.DATA_TYPE_DATETIME:
                                str_Width = "150";
                                break;
                            default:
                                str_Width = "200";
                                break;
                        }
                    }
                }

                ReportColumnsTemplate += "<COLUMN><OBJECTNAME>" + str_DataObjectName
                                            + "</OBJECTNAME><FIELDDATATYPE>" + str_DataFieldType.ToUpper()
                                            + "</FIELDDATATYPE><RELATIONSHIPNAME>" + str_DataRelationName
                                            + "</RELATIONSHIPNAME><AGGREGATION>" + str_Aggregation.ToUpper()
                                            + "</AGGREGATION><FIELDNAME>" + str_DataFieldName
                                            + "</FIELDNAME><FIELDTITLE>" + str_DataFieldDisplayName
                                            + "</FIELDTITLE><WIDTH>" + str_Width
                                            + "</WIDTH></COLUMN>";
                int_ColumnCount++;
            }
            ReportColumnsTemplate += "</GRIDCOLUMNS>";
            return ErrorXML;
        }

        private ERRORXML GetReportData(XML ReportConfig)
        {
            ERRORXML                ErrorXML                    = null;
            SearchConfiguration     facade_SearchConfiguration  = null;
            StringDictionary        dict_PlaceHolder            = null;
            bool                    ShowNote                    = false;
            Report_CommonFunctions  ObjCommonFn                 = null;
            String                  str_ShowNote                = null;
            String                  str_TotalRecords            = null;
            String                  str_MaxRecords              = null;
            String                  ReportSearchDetails         = null;
            String                  st_InsertString             = null;

            MaxSearchResultAlert    = "";
            ObjCommonFn             = new Report_CommonFunctions(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language, ALParamObj.SolutionName);
            if (IsReportDataAvailable)
            {
                ReportData          = File.ReadAllText(ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + ReportID + ".xml");
                if(File.Exists(ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + ReportID + "_SearchDetails.xml"))
                {
                    ReportSearchDetails = File.ReadAllText(ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + ReportID + "_SearchDetails.xml");

                    str_ShowNote        = ALCommon.GetInnerValue(ref ReportSearchDetails, "<SHOWMAXRESULTNOTE>", "</SHOWMAXRESULTNOTE>");
                    str_TotalRecords    = ALCommon.GetInnerValue(ref ReportSearchDetails, "<TOTALRESULTCOUNT>", "</TOTALRESULTCOUNT>");
                    str_MaxRecords      = ALCommon.GetInnerValue(ref ReportSearchDetails, "<MAXRESULTCOUNT>", "</MAXRESULTCOUNT>");

                    if (String.Compare(str_ShowNote, "TRUE", true) == 0)
                    {
                        dict_PlaceHolder = new StringDictionary();
                        dict_PlaceHolder.Add("TOTALRESULTCOUNT", str_TotalRecords);
                        dict_PlaceHolder.Add("MAXRESULTCOUNT", str_MaxRecords);                    
                        MaxSearchResultAlert = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_MaxResultNote", dict_PlaceHolder); 
                    }                   
                }
            }
            else
            {
                XDocument xDoc_ReportConfig     = XDocument.Parse(ReportConfig);
                XElement xEle_AdHocReport       = xDoc_ReportConfig.Root.XPathSelectElement("ADHOCREPORT");
                XElement xEle_SearchConfig      = xEle_AdHocReport.XPathSelectElement("REPORTCONFIGXML/CONFIGURATION");
                XML xml_DisplayConfiguration    = null;

                if (xEle_SearchConfig != null)
                    xml_DisplayConfiguration = xEle_SearchConfig.ToString();

                if (String.IsNullOrEmpty(xml_DisplayConfiguration))
                {
                    ErrorXML = "Err_AdhocReport_Invalid_Report_DisplayConfiguration";
                    return ErrorXML;
                }

                XElement xEle_SearchInput = xEle_SearchConfig.XPathSelectElement("SEARCHINPUT/INPUTXML/SEARCHCOLLECTION/SEARCH");
                if (xEle_SearchInput != null)
                {
                    XML xml_SearchInput = xEle_SearchInput.ToString();

                    String str_InsertString = "<SESSIONINFO UserID='" + ALParamObj.UserID + "' Language='" + ALParamObj.Language
                            + "' TenantName='" + ALParamObj.TenantName + "' SessionID='" + ALParamObj.SessionID + "'/>";
                    xml_SearchInput = ALCommon.InsertBefore(ref xml_SearchInput, "<TARGETENTITY>", ref str_InsertString);

                    #region Add "FROMODS" attribute to "SEARCH" node so that search operation will be performed on appropriate Table

                    XmlDocument tmpDoc = null;
                    XPathNavigator xpNav = null;

                    tmpDoc = new XmlDocument();
                    tmpDoc.LoadXml(xml_SearchInput);

                    ALReportHelper.MergePersonalizedFiltersIntoSearchXML(ref tmpDoc, PersonalizedFilterList, "", null);
                    xpNav = tmpDoc.CreateNavigator();

                    if (xpNav.SelectSingleNode("SEARCH") != null)
                        xpNav.SelectSingleNode("SEARCH").CreateAttribute("", "FromODS", "", "True");

                    #endregion
                    xml_SearchInput = tmpDoc.OuterXml;

                    XML xml_SearchOutput = null;
                    facade_SearchConfiguration = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    ERRORXML errorxml_Output = facade_SearchConfiguration.GetHierarchialData(xml_SearchInput, ref xml_SearchOutput);

                    if ((!String.IsNullOrEmpty(errorxml_Output)) || (String.IsNullOrEmpty(xml_SearchOutput)))
                    {
                        ErrorXML = "Err_AdhocReport_Get_Report_Data";
                        return ErrorXML;
                    }

                    //Max Search Result alert handling
                    dict_PlaceHolder = new StringDictionary();
                    ObjCommonFn.Get_TOPNRecords_From_SearchResult(TargetEntityName, ref xml_SearchOutput, ref ShowNote, ref dict_PlaceHolder);
                    if (ShowNote)
                    {                        
                        MaxSearchResultAlert = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_MaxResultNote", dict_PlaceHolder);
                      
                        st_InsertString = String.Format(@"<REPORTDATA>
                                                  <SHOWMAXRESULTNOTE>TRUE</SHOWMAXRESULTNOTE>
                                                  <TOTALRESULTCOUNT>{0}</TOTALRESULTCOUNT>
                                                  <MAXRESULTCOUNT>{1}</MAXRESULTCOUNT>
                                                </REPORTDATA>", dict_PlaceHolder["TOTALRESULTCOUNT"], dict_PlaceHolder["MAXRESULTCOUNT"]);
                        File.WriteAllText(ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + ReportID + "_SearchDetails.xml", st_InsertString);
                      
                    }                  

                    Report_Facade reportFacadeObj = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    ErrorXML = reportFacadeObj.GetAdHocReportGroupedDataXML(xEle_SearchConfig.ToString(), xml_SearchOutput, ref ReportData);                    
                }
                else
                    ErrorXML = "Err_AdhocReport_Invalid_NSearchInputXML_Report_Data";
            }
            return ErrorXML;
        }

        private void GenerateHTML(ref StringBuilder sb)
        {
            XslCompiledTransform    xslcomptrans_Object         = null;
            XsltArgumentList        xsltarglist_Object          = null;
            XmlDocument             xmldoc_GridColumns          = null;
            XmlDocument             xmldoc_GroupDataType        = null;
            String                  str_XSLTFilePath            = null;
            StringBuilder           strbldr_GridDataResultXML   = null;
            StringWriter            strwrtr_GridDataResult      = null;
            XPathDocument           xpathdoc_SearchResultOutput = null;            
            Formats                 Format                      = null;
            CultureInfo             CustomCulture               = null;
            XsltSettings            xsltSettingsObj             = new XsltSettings();

            str_XSLTFilePath    = ALSolutionCommon.GetXSLTPath(ALParamObj.TenantName, ALParamObj.SolutionName) + "AdHocReportPrintView.xslt";
                        
            xsltSettingsObj.EnableScript = true;            
            xslcomptrans_Object = new XslCompiledTransform(true);
            xslcomptrans_Object.Load(str_XSLTFilePath, xsltSettingsObj, null);
            
            CustomCulture = new CultureInfo(Localization.GetCultureName(ALParamObj.Language, ALParamObj.TenantName));
                        
            Format = new Formats(FormatXMLPath);

            //Initiate the object for argumets.
            xsltarglist_Object = new XsltArgumentList();
            xmldoc_GridColumns = new XmlDocument();
            xmldoc_GridColumns.LoadXml(ReportColumnsTemplate);

            xmldoc_GroupDataType = new XmlDocument();
            xmldoc_GroupDataType.LoadXml(GroupDataTypeMap);

            xsltarglist_Object.AddParam("ParamArrayGridColumns", "", xmldoc_GridColumns);
            xsltarglist_Object.AddParam("ParamGroupDataTypeMap", "", xmldoc_GroupDataType);
            xsltarglist_Object.AddParam("ParamTargetObjectName", "", TargetEntityName);
            xsltarglist_Object.AddParam("LocalizedTotalFor", "", TotalFor_LocalizedStr);
            xsltarglist_Object.AddParam("IsAggregationRequired", "", IsAggregationRequired.ToString().ToUpper());
            xsltarglist_Object.AddExtensionObject("arpv:MyUtils", new XslExtensionClass(Convert.ToUInt64(ALParamObj.SessionID), Convert.ToUInt64(ALParamObj.UserID), ALParamObj.Language, ALParamObj.TenantName, Format, CustomCulture));

            //Initiate the output string writer.
            strbldr_GridDataResultXML   = new StringBuilder();
            strwrtr_GridDataResult      = new StringWriter(strbldr_GridDataResultXML);

            //Transform the search result XML into the grid XML.
            xpathdoc_SearchResultOutput = new XPathDocument(new StringReader(ReportData));
            xslcomptrans_Object.Transform(xpathdoc_SearchResultOutput, xsltarglist_Object, strwrtr_GridDataResult);

            String Result = strbldr_GridDataResultXML.ToString();
            sb.Append(Result);
        }

        //initializeURLParameters() method is used to set the parameters from URL.
        private void initializeURLParameters()
        {
            if (String.IsNullOrEmpty(ALParamObj.SolutionName))
                ALParamObj.SolutionName = "Reports";

            ReportID            = ALSolutionCommonObj.GetParamValue("REPORTID");
            ReportGeneratedBy   = ALSolutionCommonObj.GetParamValue("ReportGeneratedBy");
            ReportGeneratedOn   = ALSolutionCommonObj.GetParamValue("ReportGeneratedOn");
            widthlist           = ALSolutionCommonObj.GetParamValue("widthlist");
            FromPreviewScreen   = ALSolutionCommonObj.GetParamValue("FromPreviewScreen");
        }

        private void initializeControls()
        {
            Title.Text = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_PrintViewPage");
            
            TotalFor_LocalizedStr = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_Preview_TotalFor");
            
            ClientLogo.Src = GetImagesPath() + "Client_Logo1.gif";

            NameLbl.InnerHtml = Localization.GetString("ALResource", ALParamObj.TenantName, "Name_Shared") + ":";
            DescriptionLbl.InnerHtml = Localization.GetString("ALResource", ALParamObj.TenantName, "Description_Shared") + ":";
            GeneratedOnLbl.InnerHtml = Localization.GetString("ALResource", ALParamObj.TenantName, "GeneratedOn_Shared") + ":";
            GeneratedByLbl.InnerHtml = Localization.GetString("ALResource", ALParamObj.TenantName, "GeneratedBy_Shared") + ":";

            if (!File.Exists(ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + ReportID + ".xml"))
                IsReportDataAvailable = false;
                             
            FormatXMLPath = ALCommon.GetFormatDetails(ALParamObj.TenantName, out cultureInfo);

            //if (String.Compare(FromPreviewScreen, "TRUE", true) == 0)
            {
                StringBuilder sb_Temp = new StringBuilder(500);
                sb_Temp.AppendFormat(@"<table width='100%' class='AdHocReportPreviewActionBar'>
                                    <tr style='height:25px;'>
                                        <td class='TextWhite' width='75px' align='left'>
                                            {0}
                                        </td>                                      
                                    </tr>
                                </table>", Localization.GetString("ALResource", ALParamObj.TenantName, "RPT_Lbl_Filters"));

                ABC_PersonliazedFilters.TenantName      = ALParamObj.TenantName;
                ABC_PersonliazedFilters.CaptionTextMode = Assetlink.CustomControls.eCaptionTextMode.HTML;
                ABC_PersonliazedFilters.CaptionHTML     = sb_Temp; 
            }
            //else
            //    Div_PersonalizeFilters.Attributes.Add("style", "display:none;");
        }

        /// This is used to redirect current page to an error page if any error occurred while working with system.
        //redirectToErrorPage() method is used to redirect current page to an error page if any error occurred while working with system.
        private void redirectToErrorPage(String ErrorMessageString)
        {
            String str_RedirectionPage = null;
            ERRORXML errorxml_Output = null;
            StringBuilder sb_ErrorPageURL = null;
            String str_ErrorPageURL = null;

            ErrorMessageString = Localization.GetString("ALErrorResource", ALParamObj.TenantName, ErrorMessageString, ErrorMessageString);

            errorxml_Output = ALSolutionCommonObj.PrepareErrorPageUrl(ALParamObj.SolutionName, null, "REPORTS", null, ErrorMessageString, null, null, ref str_ErrorPageURL);
            if (String.IsNullOrEmpty(errorxml_Output))
            {
                sb_ErrorPageURL = new StringBuilder(200);
                sb_ErrorPageURL.Append("/" + ALParamObj.TenantName + "/Common/ASPX/ShowError.aspx?" + ALSolutionCommon.AppendStandardParamforURL(ALParamObj));
                sb_ErrorPageURL.Append("&ShowOKButton=FALSE&ErrorMessage=" + ErrorMessageString);
                str_ErrorPageURL = sb_ErrorPageURL.ToString();
            }

            str_RedirectionPage = ALSolutionCommonObj.EncodeURLForResponseRedirects(str_ErrorPageURL);            
            Response.Redirect(str_RedirectionPage);
        }

        private string GetImagesPath()
        {
            return String.Format("/{0}/App_Themes/{1}/Images/", ALParamObj.TenantName, Page.Theme);
        }

        #endregion

        #region Member Classes
        public class Constants
        {
            public const string DATA_TYPE_TEXT      = "TEXT";
            public const string DATA_TYPE_STRING    = "STRING";
            public const string DATA_TYPE_NUMBER    = "NUMBER";
            public const string DATA_TYPE_FLOAT     = "FLOAT";
            public const string DATA_TYPE_CURRENCY  = "CURRENCY";
            public const string DATA_TYPE_DATE      = "DATE";
            public const string DATA_TYPE_DATETIME  = "DATETIME";
            public const string DATA_TYPE_MULTILINE = "MULTILINE";
        }

        public class XslExtensionClass
        {
            private CultureInfo CustomCulture   = null;
            private Formats     Format          = null;
            private ALCommon    ALCommonObj     = null;
            private String      TenantName      = null;

            public XslExtensionClass(Formats FormatObj, CultureInfo CustomCultureObj)
            {
                Format          = FormatObj;
                CustomCulture   = CustomCultureObj;
            }

            public XslExtensionClass(UInt64 SessionID, UInt64 UserID,String Language, String TenantName,Formats FormatObj, CultureInfo CustomCultureObj)
            {
                Format          = FormatObj;
                CustomCulture   = CustomCultureObj;
                ALCommonObj     = new ALCommon(TenantName, UserID.ToString(), SessionID.ToString(), Language);
                this.TenantName = TenantName;
            }

            public String FormatValueforDataType(String datatype, String value)
            {
                String dateFormat = "";
                String[] valueArr = null; 
                try
                {
                    switch (datatype.ToUpper())
                    {
                        case Constants.DATA_TYPE_TEXT:
                            break;
                        case Constants.DATA_TYPE_STRING:
                            if (value.Contains("$AL$"))
                            {
                                ALCommonObj.ConvertDelimiterSeparatorToSeparator(value, ",", out value, out valueArr, false, TenantName);
                            }
                            break;
                        case Constants.DATA_TYPE_NUMBER:
                            value = Format.FormatNumericField(value, CustomCulture);
                            break;
                        case Constants.DATA_TYPE_CURRENCY:
                            value = Format.FormatCurrencyField(value, CustomCulture);
                            break;
                        case Constants.DATA_TYPE_FLOAT:
                            value = Format.FormatFloatField(value, CustomCulture);
                            break;
                        case Constants.DATA_TYPE_DATE:
                            if (!String.IsNullOrEmpty(value))
                            {
                                valueArr = value.Split(new String[] { "/" }, StringSplitOptions.None);
                                value = valueArr[1] + " " + GetMonthName(valueArr[0]) + " " + valueArr[2];
                            }
                            dateFormat = Format.ColumnDateFieldFormat;
                            value = Format.FormatDateField(value, CustomCulture);
                            if (dateFormat == "")
                                value = value.Split(' ')[0]; //done to remove time bcoz when no format given it gives datetime by default
                            break;
                        case Constants.DATA_TYPE_DATETIME:
                            if (!String.IsNullOrEmpty(value))
                            {
                                String[] datetimeArr = value.Split(new String[] { " " }, StringSplitOptions.None);
                                value = datetimeArr[0];
                                valueArr = value.Split(new String[] { "/" }, StringSplitOptions.None);
                                value = valueArr[1] + " " + GetMonthName(valueArr[0]) + " " + valueArr[2];
                                if (datetimeArr.Length > 1 && datetimeArr[1] != null && datetimeArr[1] != "")
                                {
                                    String timeStamp = datetimeArr[1];
                                    String[] timeStampArr = timeStamp.Split(new String[] { ":" }, StringSplitOptions.None);
                                    if (Convert.ToInt32(timeStampArr[0]) == 0 && Convert.ToInt32(timeStampArr[1]) == 0 && Convert.ToInt32(timeStampArr[2]) == 0)
                                        value = Format.FormatDateField(value, CustomCulture);
                                    else
                                    {
                                        value += " " + datetimeArr[1];
                                        value = Format.FormatDateTimeField(value, CustomCulture);
                                    }
                                }
                                else
                                    value = Format.FormatDateField(value, CustomCulture);
                            }
                            break;
                        case Constants.DATA_TYPE_MULTILINE:
                            value = value.Replace("\n", "<BR>");
                            break;
                        default:
                            break;
                    }
                }
                catch (Exception)
                {
                    value = "";
                }

                if (value == null)
                    value = "";

                return value;

            }

            private String GetMonthName(string sMonthNum)
            {
                string GetMonthValue = "";
                switch (sMonthNum)
                {
                    case "01":
                        GetMonthValue = "January";
                        break;
                    case "02":
                        GetMonthValue = "February";
                        break;
                    case "03":
                        GetMonthValue = "March";
                        break;
                    case "04":
                        GetMonthValue = "April";
                        break;
                    case "05":
                        GetMonthValue = "May";
                        break;
                    case "06":
                        GetMonthValue = "June";
                        break;
                    case "07":
                        GetMonthValue = "July";
                        break;
                    case "08":
                        GetMonthValue = "August";
                        break;
                    case "09":
                        GetMonthValue = "September";
                        break;
                    case "10":
                        GetMonthValue = "October";
                        break;
                    case "11":
                        GetMonthValue = "November";
                        break;
                    case "12":
                        GetMonthValue = "December";
                        break;
                }
                return GetMonthValue;
            }
        }
        #endregion
    }
}
