<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:msxml="urn:schemas-microsoft-com:xslt"
                xmlns:myUtils="arpv:MyUtils"
                xmlns:random="http://www.microsoft.com/msxsl"
                extension-element-prefixes="msxml"
                exclude-result-prefixes="msxml random">

  <xsl:param name="ParamTargetObjectName"/>
  <xsl:param name="ParamArrayGridColumns" select="/.."/>
  <xsl:param name="ParamGroupDataTypeMap" select="/.."/>
  <xsl:param name="LocalizedTotalFor"/>
  <xsl:param name="IsAggregationRequired"/>
  <xsl:output method="xml" version="1.0" omit-xml-declaration="yes" indent="yes"/>

  <msxml:script implements-prefix="random">
    function GetRandomNumber()
    {      
      return Math.random();
    }

  </msxml:script>


  <xsl:template match="/">
    <xsl:element name="griddata">
      <xsl:variable name="ROOTNAME">        
        <xsl:text>GROUPDATACOLLECTION</xsl:text>
      </xsl:variable>
      <xsl:for-each select="child::*/child::*[msxml:node-set($ROOTNAME)]">
        <xsl:call-template name="CreateGridNodes">
          <xsl:with-param name="CurrLevel" select="0"></xsl:with-param>          
        </xsl:call-template>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <xsl:template name="CreateGridNodes">
    <xsl:param name="CurrLevel"></xsl:param>    
    <xsl:element name="gridnode">
      
      <xsl:attribute name="id">       
        <xsl:value-of select="random:GetRandomNumber()"/>
      </xsl:attribute>
      
      <xsl:attribute name="depth">
        <xsl:value-of select="$CurrLevel"/>
      </xsl:attribute>
      
      <xsl:variable name="varxpath_DataTargetNodeObject">
        <xsl:copy-of select="."/>
      </xsl:variable>

      <xsl:element name="data">
        
        <xsl:for-each select="$ParamArrayGridColumns/GRIDCOLUMNS/COLUMN/FIELDNAME">
          
          <xsl:variable name="varxpath_GridObjectField" select="text()"></xsl:variable>          
          <xsl:variable name="var_ColumnObjectName">
            <xsl:value-of select="parent::*/OBJECTNAME/text()"/>
          </xsl:variable>          
          <xsl:variable name="var_ColumnRelationName">
            <xsl:value-of select="parent::*/RELATIONSHIPNAME/text()"/>
          </xsl:variable>
          <xsl:variable name="var_Aggregation">
            <xsl:value-of select="parent::*/AGGREGATION/text()"/>
          </xsl:variable>
          <xsl:variable name="var_FieldDataType">
            <xsl:value-of select="parent::*/FIELDDATATYPE/text()"/>
          </xsl:variable>

          <xsl:variable name="varxpath_DataTempObjectForGroup" select="msxml:node-set($varxpath_DataTargetNodeObject)/*">
          </xsl:variable>
          
          <xsl:variable name="varxpath_DataTempObject">            
            <xsl:choose>
              <xsl:when test="($var_ColumnObjectName = $var_ColumnRelationName) or (name($varxpath_DataTempObjectForGroup) = 'GROUPDATA')">
                <xsl:copy-of select="msxml:node-set($varxpath_DataTargetNodeObject)/*"></xsl:copy-of>
              </xsl:when>
              <xsl:otherwise>                
                <xsl:copy-of select="msxml:node-set($varxpath_DataTargetNodeObject)/*/child::*[name() = $var_ColumnRelationName]/child::*[name() = $var_ColumnObjectName]"></xsl:copy-of>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>

          <xsl:variable name="varxpath_DataTargetObject" select="msxml:node-set($varxpath_DataTempObject)/*">            
          </xsl:variable>

          <xsl:element name="datafield">
            <!--Start of 'datafield' Element-->

            <xsl:variable name="varxpath_FieldNameCombination">
              <xsl:value-of select="$var_ColumnRelationName"/>
              <xsl:value-of select ="'.'"/>
              <xsl:value-of select ="$varxpath_GridObjectField"/>
            </xsl:variable>
            
            <xsl:attribute name="name">
              <xsl:value-of select="$varxpath_FieldNameCombination"/>
            </xsl:attribute>

            <xsl:element name="name">
              <!--Start of 'name' Element-->

              <xsl:attribute name="text">
                <xsl:text>$</xsl:text>
                <xsl:value-of select="$varxpath_FieldNameCombination"/>
              </xsl:attribute>

              <xsl:element name ="row">
                <!--Start of 'row' Element-->

                <xsl:element name="value">

                  <xsl:variable name="GridNodeFieldValue">
                    <xsl:choose>
                      <xsl:when test="name($varxpath_DataTargetObject) = 'GROUPDATA'">
                        <xsl:choose>
                          <xsl:when test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                            <xsl:variable name="GridNodeTempFieldValue">
                              <xsl:value-of select="$varxpath_DataTargetObject/@VALUE"/>
                            </xsl:variable>
                            <xsl:variable name="GroupDataType">
                              <xsl:value-of select="$ParamGroupDataTypeMap/GROUPS/GROUP[@level=number($CurrLevel)+1]/@DataType"/>
                            </xsl:variable>
                            <xsl:variable name="FormattedDataValue">
                              <xsl:value-of select="myUtils:FormatValueforDataType($GroupDataType, $GridNodeTempFieldValue)"/>
                            </xsl:variable>
                            <xsl:value-of select="$FormattedDataValue"/>
                          </xsl:when>
                          <xsl:otherwise>
                            <xsl:choose>
                              <xsl:when test="$var_Aggregation = 'NONE'">
                                <xsl:value-of select="''"/>
                              </xsl:when>
                              <xsl:otherwise>
                                <xsl:value-of select="$varxpath_DataTargetObject/attribute::*[name()=$varxpath_FieldNameCombination]"/>
                              </xsl:otherwise>
                            </xsl:choose>
                          </xsl:otherwise>
                        </xsl:choose>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:if test="$varxpath_GridObjectField != 'GROUPDATAVALUE'">
                          <xsl:choose>
                            <xsl:when test="($var_FieldDataType = 'CURRENCY') or ($var_FieldDataType = 'DATETIME')or ($var_FieldDataType = 'STRING')">
                              <xsl:if test="$var_FieldDataType = 'CURRENCY'">
                                <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/AMOUNT/text()"/>
                              </xsl:if>
                              <xsl:if test="$var_FieldDataType = 'DATE'">
                                <xsl:if test="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text() != ''">
                                  <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/MONTH/text()"/>
                                  <xsl:text>/</xsl:text>
                                  <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/DAY/text()"/>
                                  <xsl:text>/</xsl:text>
                                  <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text()"/>
                                </xsl:if>                                
                              </xsl:if>
                              <xsl:if test="$var_FieldDataType = 'DATETIME'">
                                <xsl:variable name="DateTimeValue">
                                  <xsl:if test="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text() != ''">
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/MONTH/text()"/>
                                    <xsl:text>/</xsl:text>
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/DAY/text()"/>
                                    <xsl:text>/</xsl:text>
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text()"/>
                                  </xsl:if>
                                  <xsl:if test="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/HOUR/text() != ''">
                                    <xsl:text> </xsl:text>
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/HOUR/text()"/>
                                    <xsl:text>:</xsl:text>
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/MINUTE/text()"/>
                                    <xsl:text>:</xsl:text>
                                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/SECOND/text()"/>
                                  </xsl:if>
                                </xsl:variable>
                                <xsl:variable name="FormattedDateTimeValue">
                                  <xsl:value-of select="myUtils:FormatValueforDataType($var_FieldDataType, $DateTimeValue)"/>
                                </xsl:variable>
                                <xsl:value-of select="$FormattedDateTimeValue"/>
                              </xsl:if>
                              <xsl:if test="$var_FieldDataType = 'STRING'">
                                <xsl:variable name="RawStringValue">
                                  <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/text()"/>
                                </xsl:variable>
                                <xsl:choose>
                                  <xsl:when test="contains($RawStringValue,'$AL$')">
                                    <xsl:value-of select="myUtils:FormatValueforDataType($var_FieldDataType, $RawStringValue)"/>
                                  </xsl:when>
                                  <xsl:otherwise>
                                    <xsl:value-of select="$RawStringValue"/>
                                  </xsl:otherwise>
                                </xsl:choose>
                              </xsl:if>
                            </xsl:when>
                            <xsl:otherwise>
                              <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/text()"/>
                            </xsl:otherwise>
                          </xsl:choose>
                        </xsl:if>
                      </xsl:otherwise>
                    </xsl:choose>
                      <!--Start of 'value' Element-->                      
                  </xsl:variable>

                  <xsl:attribute name="text">
                    <xsl:value-of select="$GridNodeFieldValue"/>
                  </xsl:attribute>

                  <!--<xsl:attribute name="id">
                        <xsl:text>$NAME$</xsl:text>
                        <xsl:value-of select="MARKETINGACTIVITYID"/>
                      </xsl:attribute>-->

                  <xsl:attribute name="tooltip">
                    <xsl:value-of select="$GridNodeFieldValue"/>
                  </xsl:attribute>

                </xsl:element>
                <!--End of 'value' Element-->
              </xsl:element>
              <!--End of 'row' Element-->
            </xsl:element>
            <!--End of 'name' Element-->

          </xsl:element>
        </xsl:for-each>
        
      </xsl:element>
      <xsl:choose>      
        <xsl:when test="count(child::GROUPDATA) > 0">
          <xsl:for-each select="child::GROUPDATA">
            <xsl:call-template name="CreateGridNodes">
              <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
            </xsl:call-template>            
          </xsl:for-each>
        </xsl:when>      
        <xsl:otherwise>
          <xsl:for-each select="child::*[local-name() = $ParamTargetObjectName]">
            <xsl:call-template name="CreateGridNodes">
              <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
            </xsl:call-template>            
          </xsl:for-each>          
        </xsl:otherwise>
      </xsl:choose>
      <xsl:call-template name="CreateAggregationFooter">
        <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
        <xsl:with-param name="varxpath_DataTargetObject" select="$varxpath_DataTargetNodeObject"></xsl:with-param>
      </xsl:call-template>
    </xsl:element>
  </xsl:template>

  <xsl:template name="CreateAggregationFooter">
    <xsl:param name="CurrLevel"></xsl:param>
    <xsl:param name="varxpath_DataTargetObject"></xsl:param>

    <xsl:variable name="varxpath_DataTempObject" select="msxml:node-set($varxpath_DataTargetObject)/*">
    </xsl:variable>

    <xsl:if test="name($varxpath_DataTempObject) = 'GROUPDATA' and $IsAggregationRequired = 'TRUE'">

      <xsl:element name="gridnode">

        <xsl:attribute name="id">
          <xsl:value-of select="random:GetRandomNumber()"/>
        </xsl:attribute>

        <xsl:attribute name="depth">
          <xsl:value-of select="$CurrLevel"/>
        </xsl:attribute>

        <xsl:element name="data">

          <xsl:for-each select="$ParamArrayGridColumns/GRIDCOLUMNS/COLUMN/FIELDNAME">

            <xsl:variable name="varxpath_GridObjectField" select="text()"></xsl:variable>
            <xsl:variable name="var_ColumnObjectName">
              <xsl:value-of select="parent::*/OBJECTNAME/text()"/>
            </xsl:variable>
            <xsl:variable name="var_ColumnRelationName">
              <xsl:value-of select="parent::*/RELATIONSHIPNAME/text()"/>
            </xsl:variable>
            <xsl:variable name="var_Aggregation">
              <xsl:value-of select="parent::*/AGGREGATION/text()"/>
            </xsl:variable>
            <xsl:variable name="var_FieldDataType">
              <xsl:value-of select="parent::*/FIELDDATATYPE/text()"/>
            </xsl:variable>

            <xsl:element name="datafield">
              <!--Start of 'datafield' Element-->

              <xsl:variable name="varxpath_FieldNameCombination">
                <xsl:value-of select="$var_ColumnRelationName"/>
                <xsl:value-of select ="'.'"/>
                <xsl:value-of select ="$varxpath_GridObjectField"/>
              </xsl:variable>

              <xsl:attribute name="name">
                <xsl:value-of select="$varxpath_FieldNameCombination"/>
              </xsl:attribute>

              <xsl:element name="name">
                <!--Start of 'name' Element-->

                <xsl:attribute name="text">
                  <xsl:text>$</xsl:text>
                  <xsl:value-of select="$varxpath_FieldNameCombination"/>
                </xsl:attribute>

                <xsl:element name ="row">
                  <!--Start of 'row' Element-->

                  <xsl:element name="value">

                    <xsl:variable name="GridNodeFieldValue">

                      <xsl:choose>
                        <xsl:when test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                          <xsl:value-of select="$LocalizedTotalFor"/>
                          <xsl:text> </xsl:text>
                          <xsl:variable name="GridNodeTempFieldValue">
                            <xsl:value-of select="$varxpath_DataTempObject/@VALUE"/>
                          </xsl:variable>
                          <xsl:variable name="FormattedDataValue">
                            <xsl:value-of select="myUtils:FormatValueforDataType($var_FieldDataType, $GridNodeTempFieldValue)"/>
                          </xsl:variable>
                          <xsl:value-of select="$FormattedDataValue"/>
                          <!--<xsl:value-of select="''"/>-->
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:choose>
                            <xsl:when test="$var_Aggregation = 'NONE'">
                              <xsl:value-of select="''"/>
                            </xsl:when>
                            <xsl:otherwise>
                              <xsl:value-of select="$varxpath_DataTempObject/attribute::*[name()=$varxpath_FieldNameCombination]"/>
                            </xsl:otherwise>
                          </xsl:choose>
                        </xsl:otherwise>
                      </xsl:choose>

                      <!--Start of 'value' Element-->
                    </xsl:variable>

                    <xsl:variable name="GroupType">
                      <xsl:choose>
                        <xsl:when test="$CurrLevel = '1'">
                          <xsl:text>Group</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:text>SubGroup</xsl:text>
                        </xsl:otherwise>
                      </xsl:choose>
                    </xsl:variable>

                    <xsl:attribute name="textcss">
                      <xsl:text>AdHocReportPreview</xsl:text>
                      <xsl:value-of select="$GroupType"/>
                      <xsl:text>AggregationFooter</xsl:text>                      
                      <xsl:if test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                        <xsl:text>Title</xsl:text>
                      </xsl:if>                        
                    </xsl:attribute>                    

                    <xsl:attribute name="cellcss">
                      <xsl:text>AdHocReportPreviewAggregationFooterCell</xsl:text>
                    </xsl:attribute>

                    <xsl:attribute name="text">
                      <xsl:value-of select="$GridNodeFieldValue"/>
                    </xsl:attribute>
                    <!--<xsl:attribute name="id">
                        <xsl:text>$NAME$</xsl:text>
                        <xsl:value-of select="MARKETINGACTIVITYID"/>
                      </xsl:attribute>-->

                    <xsl:attribute name="tooltip">
                      <xsl:value-of select="$GridNodeFieldValue"/>
                    </xsl:attribute>

                  </xsl:element>
                  <!--End of 'value' Element-->
                </xsl:element>
                <!--End of 'row' Element-->
              </xsl:element>
              <!--End of 'name' Element-->

            </xsl:element>

          </xsl:for-each>
        </xsl:element>
      </xsl:element>
    </xsl:if>
    </xsl:template>
</xsl:stylesheet>
