using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;
using System.Xml;
using System.IO;
using System.Xml.XPath;

#region AssetlinkIncludes
using Assetlink;
using Assetlink.Solutions;
using Assetlink.CustomControls;
using Assetlink.Widgets;
using Assetlink.Widgets.WidgetCommon;
using Assetlink.Solutions.Facade.DAMFacade;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.InfrastructureFacade;
using Assetlink.Solutions.Pages;
#endregion

namespace Assetlink.Solutions.DAM
{
    public partial class View_Original : System.Web.UI.Page
    {
        ALSolutionCommon ALSolutionCommonObj  = null;
        ALParam          ALParamObj           = null;
        private String   DigitalAssetID       = "";
        String           AuditingSolutionName = null;
        DAMHelper        DAMHelperObj         = null;
        String           RevisionID           = null;
        String           IsCurrentVersion     = null;
        String           MaxAccessRights      = null;
        AssetWidget      ObjAssetWidgetFacade = null;
        String           ParentSectionID      = null;
        String          str_AllowDownloadOfExpiredAssets = "false";
        

        #region Page Events

        protected override void OnPreInit(EventArgs e)
        {
            String ThemeName                = null;            

            ThemeName = ALSolutionCommonObj.GetParamValue("ThemeName");
            if (!String.IsNullOrEmpty(ThemeName))
                Page.Theme = ThemeName;

            if (String.IsNullOrEmpty(ALParamObj.SolutionName))
                ALParamObj.SolutionName = "DAM";
            base.OnPreInit(e);
        }

        protected override void InitializeCulture()
        {
            ALSolutionCommonObj                 = new ALSolutionCommon();
            ALParamObj                          = new ALParam();
            ALSolutionCommonObj.httpContext     = HttpContext.Current;
            ALSolutionCommonObj.InitALParam(ref ALParamObj);

            Page.UICulture = Localization.GetCultureName(ALParamObj.Language, ALParamObj.TenantName);
            Localization.SetCultureForPage(ALParamObj.Language, ALParamObj.TenantName);
            Page.Title = Localization.GetString("ALResource", ALParamObj.TenantName, "DAM_AccessCtr_ViewOriginal_Shared");
        }

        protected override void OnInit(EventArgs e)
        {
          
            DigitalAssetID              = ALSolutionCommonObj.GetParamValue("DigitalAssetID");
            RevisionID                  = ALSolutionCommonObj.GetParamValue("RevisionID");
            IsCurrentVersion            = ALSolutionCommonObj.GetParamValue("IsCurrentVersion");
            AuditingSolutionName        = ALSolutionCommonObj.GetParamValue("AuditingSolutionName");
            AuditingSolutionName        = Server.UrlDecode(AuditingSolutionName);
            MaxAccessRights             = ALSolutionCommonObj.GetParamValue("MaxAccessRights");
            ParentSectionID             = ALSolutionCommonObj.GetParamValue("ParentSectionID");
            str_AllowDownloadOfExpiredAssets = ALSolutionCommonObj.GetParamValue("AllowDownloadOfExpiredAssets");
            
            if (String.IsNullOrEmpty(str_AllowDownloadOfExpiredAssets))
                str_AllowDownloadOfExpiredAssets = "false";
            base.OnInit(e);
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            #region member variable
            String                  ErrorXML                        = "";
            String                  FileDownloadURL                 = "";
            String                  ErrorPageURL                    = "";
            DamCommon               ObjDamCommon                    = null;
            String                  AuditXml                        = null;
            StringBuilder           IncrementeDownloadInputXML      = null;
            StringBuilder           sb_TermsAndCSURL                = null;
            DAMCommonFunctions      DAMCommonFunctionsObj           = null;
            String                  ErrorMessage                    = null;
            #endregion

            try
            {
                Page.Header.DataBind();
                DAMHelperObj = new DAMHelper(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language,ALParamObj.SolutionName,Page.Theme);
                #region Check for view preview access
                if (!IsViewOriginal())
                {
                    ErrorXML = ALSolutionCommon.PrepareErrorXML("DAM_VIEWORIGINAL_NOACCESS","");
                    ALSolutionCommonObj.GetErrorMessage("DAM", "", "DAMSOLUTION", ErrorXML, "", ref ErrorMessage);

                    StringBuilder SBShowErrorPageUrl = new StringBuilder();

                    SBShowErrorPageUrl.AppendFormat("/{0}/Common/ASPX/ShowError.aspx?", ALParamObj.TenantName);
                    SBShowErrorPageUrl.AppendFormat("ErrorMessage={0}", ErrorMessage);
                    SBShowErrorPageUrl.AppendFormat("&SessionID={0}&UserID={1}&TenantName={2}&Language={3}", ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    SBShowErrorPageUrl.Append("&ShowOKButton=false&ErrorType=Message");
                    SBShowErrorPageUrl.AppendFormat("&ThemeName={0}", Page.Theme);      
                    Response.Redirect(ALSolutionCommonObj.EncodeURLForResponseRedirects(SBShowErrorPageUrl.ToString()));
                }
                #endregion
                #region Check IsSessionValid
                if (!IsSessionValid())
                {
                    String RedirectUrl = null;

                    DAMHelperObj.GetShowErrorPageUrlForSessionInvalid(ref RedirectUrl);

                    if (IsPostBack)
                    {
                        ALAjaxPage tempALAjaxPage = null;
                        tempALAjaxPage = this.Page as ALAjaxPage;
                        if (tempALAjaxPage != null)
                        {
                            ((ALAjaxPage)(this.Page)).Ajax.Redirect(this.Page, RedirectUrl);
                        }
                        else
                        {
                            Response.Redirect(RedirectUrl);
                        }
                    }
                    else
                    {
                        Response.Redirect(RedirectUrl);
                    }
                }
                #endregion 
                #region Check for Expired Assets
                bool IsAssetExpiryEnabled = DAMCommonFunctions.IsAssetExpiryEnabled(ALParamObj.TenantName, ALParamObj.SessionID, ALParamObj.SolutionName, ALParamObj.Language, ALParamObj.UserID, this.Page);
                if (!IsAssetExpiryEnabled || ((String.Compare(str_AllowDownloadOfExpiredAssets, "true", true) == 0) && IsAssetExpiryEnabled))
                {
                    DAMHelperObj.AllowDownloadOfExpiredAsset = true;
                }
                #endregion              

                DAMCommonFunctionsObj = new DAMCommonFunctions(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                bool IsAuditEnabled = ALSolutionCommonExtension.IsAuditingEnabled(this.Page, ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);

                if ((DAMCommonFunctions.IsTermsAndConditionsEnabled(ALParamObj.TenantName, ALParamObj.SessionID, ALParamObj.SolutionName, this.Page) || (IsAuditEnabled && DAMCommonFunctionsObj.isTrackReasonForDownload())))
                { 
                    sb_TermsAndCSURL = new StringBuilder(100);
                    sb_TermsAndCSURL = new StringBuilder();
                    sb_TermsAndCSURL.AppendFormat("/{0}/Common/ASPX/ViewOrginal_TermsandConditions.aspx?", ALParamObj.TenantName);
                    sb_TermsAndCSURL.AppendFormat("TenantName={0}&Language={1}&UserID={2}&SessionID={3}&SolutionName={4}", ALParamObj.TenantName, ALParamObj.Language, ALParamObj.UserID, ALParamObj.SessionID,ALParamObj.SolutionName);
                    sb_TermsAndCSURL.AppendFormat("&DigitalAssetID={0}&AuditingSolutionName={1}&RenditionType={2}", DigitalAssetID, Server.UrlEncode(AuditingSolutionName), "Original");
                    sb_TermsAndCSURL.AppendFormat("&ThemeName={0}&AllowDownloadOfExpiredAssets={1}&DownloadType=SingleFileDownload", Page.Theme, str_AllowDownloadOfExpiredAssets);
                    sb_TermsAndCSURL.AppendFormat("&IsCurrentVersion={0}&RevisionID={1}", IsCurrentVersion, RevisionID);
                    Response.Redirect(ALSolutionCommonObj.EncodeURLForResponseRedirects(sb_TermsAndCSURL.ToString()));
                }
                else
                {

                  ErrorXML = DAMHelperObj.PrepareDigitalAssetDownloadUrl(DigitalAssetID, "Original", "", AuditingSolutionName, IsAuditEnabled, RevisionID, ref FileDownloadURL, ref AuditXml);

                    if (String.IsNullOrEmpty(ErrorXML))
                    {
                        //Desicion: Auditing is stamped for all versions original download.
                        //but no version data will be stamped i.e. for which version the preview is downloaded.
                        if (IsAuditEnabled && !String.IsNullOrEmpty(AuditXml))
                        {
                                Session["AuditInputXml"] = AuditXml;
                        }

                        if (!String.IsNullOrEmpty(FileDownloadURL))
                        {
                            //Increment Download Count
                            IncrementeDownloadInputXML = new StringBuilder();
                            IncrementeDownloadInputXML.AppendFormat("<DIGITALASSETCOLLECTION><DIGITALASSET><DIGITALASSETID>{0}</DIGITALASSETID></DIGITALASSET></DIGITALASSETCOLLECTION>", DigitalAssetID);
                            ObjDamCommon = new DamCommon(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language, ALParamObj.SolutionName, Page.Theme);
                            ErrorXML = ObjDamCommon.IncrementDownloadCount(IncrementeDownloadInputXML.ToString());
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                ALSolutionCommonObj.PrepareErrorPageUrl("DAM", "", "", ErrorXML, "", "", "", ref ErrorPageURL);
                                ErrorPageURL = ErrorPageURL + "&JS_CloseOnOk=TRUE";
                                Response.Redirect(ErrorPageURL);
                            }
                            Response.Redirect(FileDownloadURL);
                        }
                        else
                        {
                            ErrorXML = ALSolutionCommon.PrepareErrorXML("DAM_DOWLOAD_DIGITALASSETFAILED", "");
                            ALSolutionCommonObj.PrepareErrorPageUrl("DAM", "", "DAMSOLUTION", ErrorXML, "", "", "", ref ErrorPageURL);
                            ErrorPageURL = ErrorPageURL + "&JS_CloseOnOk=TRUE";
                            Response.Redirect(ErrorPageURL);
                        }
                    }
                    else
                    {
                        if (ErrorXML.Equals("DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETLOCKED") || ErrorXML.Equals("DAMFACADE_DAMHELPER_DOWNLOADDIGITALASSET_DIGITALASSETEXPIRED")
                                || ErrorXML.Equals("DIGITALASSET_DIGITALASSETMARKEDASDELETED_Shared") || ErrorXML.Equals("DIGITALASSET_DIGITALASSETMARKEDFORPERMANANTDELETE_Shared"))
                            ErrorXML = ALSolutionCommon.PrepareErrorXML(ErrorXML, "");

                        ALSolutionCommonObj.PrepareErrorPageUrl("DAM", "", "DAMSOLUTION", ErrorXML, "", "", "", ref ErrorPageURL);
                        ErrorPageURL = ErrorPageURL + "&JS_CloseOnOk=TRUE";
                        Response.Redirect(ErrorPageURL);
                    }
                }
            }
            catch (Exception ObjException)
            {
                ErrorXML = ALSolutionCommon.PrepareErrorXML(ObjException.Message);
            }
        }

        private bool IsSessionValid()
        {
            bool m_IsSessionValid = false;
            String ErrorXml = "";
            InfrastructureFacadeClass ObjINFI = new InfrastructureFacadeClass(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);

            ErrorXml = ObjINFI.IsSessionValid(ref m_IsSessionValid);
            if (String.IsNullOrEmpty(ErrorXml))
                return m_IsSessionValid;
            return false;
        }

        bool IsViewOriginal()
        {
            if (String.IsNullOrEmpty(MaxAccessRights))
                GetMaxAccessRights();
            
            switch (MaxAccessRights)
            {
                case "DW_COMPLETE_ACCESS":
                case "DW_DELETE_PERMISSION":
                case "DW_CREATE_PERMISSION":
                case "DW_MODIFY_PERMISSION":
                case "DW_EDITSI_PERMISSION":
                case "DW_VIEW_ORIGINAL":                
                    {
                        return true;
                    }
                case "DW_VIEW_PREVIEW":
                case "DW_VIEW_THUMBNAIL":
                case "DW_NOACCESS":
                    {
                        return false;
                    }
                default:
                    {
                        return false;
                    }
            }
            //return false;
        }

        void GetMaxAccessRights()
        {
            String              AccessRights = null;
            String              SectionID = null;
            DAM_SectionFacad    ObjDAMSectionFacade = null;

            try
            {
                if (String.IsNullOrEmpty(this.MaxAccessRights))
                {
                    if (String.IsNullOrEmpty(this.ParentSectionID))
                    {
                        ObjAssetWidgetFacade = new AssetWidget(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                        ObjAssetWidgetFacade.GetSectionID(this.DigitalAssetID, ref SectionID);
                        this.ParentSectionID = SectionID;
                    }
                    ObjDAMSectionFacade = new DAM_SectionFacad(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    ObjDAMSectionFacade.GetMaxAccessRight(this.ParentSectionID, ref AccessRights);
                    MaxAccessRights = AccessRights;
                }
            }
            catch (Exception)
            {

            }
        }
        #endregion
    }
}
