using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Reflection;

#region Assetlink Include

//using MediaRequestRouter;
//using MediaRequestRouter.ALSearchWS;
//using Assetlink.Solutions;
using Assetlink.WebServices.ALSearchWS;

#endregion

namespace Assetlink
{
    using XML = System.String;
    using ERRORXML = System.String;

    public class MediaRequestRouter
    {

        #region Private variables

        String m_TenantName;
        String m_UserID;
        String m_SessionID;
        String m_Language;
        String m_InputXML;
        String m_RequestID;
        String m_ApplicationType;
        String m_ExtraInformation;
        String m_SolutionName;
        String m_MediaOutputQueuePath;
        String m_UndeliveredQueuePath;
        String m_MediaServerQueuePath;

        ERRORXML str_ErrorXML;
        String str_MediaRouterConfigPath;
        bool m_ReplaceNetworkDrivePath = true;
        #endregion

        #region Public properties

        public String TenantName
        {
            set
            {
                m_TenantName = value;
            }
            get
            {
                return m_TenantName;
            }
        }

        public String UserID
        {
            set
            {
                m_UserID = value;
            }
            get
            {
                return m_UserID;
            }
        }

        public String SessionID
        {
            set
            {
                m_SessionID = value;
            }
            get
            {
                return m_SessionID;
            }
        }

        public String Language
        {
            set
            {
                m_Language = value;
            }
            get
            {
                return m_Language;
            }
        }

        public String SolutionName
        {
            set
            {
                m_SolutionName = value;
            }
            get
            {
                return m_SolutionName;
            }
        }

        public String InputXML
        {
            set
            {
                m_InputXML = value;
            }
            get
            {
                return m_InputXML;
            }
        }

        public String RequestID
        {
            set
            {
                m_RequestID = value;
            }
            get
            {
                return m_RequestID;
            }
        }

        public String ApplicationType
        {
            set
            {
                m_ApplicationType = value;
            }
            get
            {
                return m_ApplicationType;
            }
        }

        public String ExtraInformation
        {
            set
            {
                m_ExtraInformation = value;
            }
            get
            {
                return m_ExtraInformation;
            }
        }

        public String MediaOutputQueuePath
        {
            get
            {
                return m_MediaOutputQueuePath;
            }
            set
            {
                m_MediaOutputQueuePath = value;
            }
        }

        public String UndeliveredQueuePath
        {
            get
            {
                return m_UndeliveredQueuePath;
            }
            set
            {
                m_UndeliveredQueuePath = value;
            }
        }

        public String MediaServerQueuePath
        {
            get
            {
                return m_MediaServerQueuePath;
            }
            set
            {
                m_MediaServerQueuePath = value;
            }
        }

        public bool ReplaceNetworkDrivePath
        {
            get
            {
                return m_ReplaceNetworkDrivePath;
            }
            set
            {
                m_ReplaceNetworkDrivePath = value;
            }
        }       
        #endregion

        #region Constructor

        public MediaRequestRouter(String str_TenantName, String str_UserID, String str_SessionID, String str_Language, String str_SolutionName)
        {
            m_Language = str_Language;
            m_SessionID = str_SessionID;
            m_TenantName = str_TenantName;
            m_UserID = str_UserID;
            m_SolutionName = str_SolutionName.ToUpper();

            str_MediaRouterConfigPath = ALCommon.GetInstallationDir() + "Configuration\\XML\\MediaRequestRouter.xml";
        }

        #endregion

        #region Public methods

        /// <summary>
        /// Determines whether request for the specified file format need to be sent to the media server.
        /// </summary>
        /// <param name="InputFileFormat"></param>
        /// <param name="FileFormatDetails"></param>
        /// <returns></returns>
        public bool IsMediaProcessorRequired(String str_InputFileFormat, out String ApplicationType, out String str_FileFormatDetails, out String str_JobPath)
        {
            StringBuilder sb_InputXML = new StringBuilder(200);
            XML str_SearchOutputXML = null;
            XPathDocument xpDoc_FileFormat = null;
            XPathNavigator nav_FileFormat = null;
            //String str_MediaRouterConfigPath;
            XPathDocument xpDoc_RouterConfig = null;
            XPathNavigator xpNav_Root = null;
            XPathNavigator xpNav_MediaServer = null;
            XPathNavigator xpNav_Application = null;
            XPathNavigator xpNav_ApplicationType = null;
            //String ApplicationType = null;

            try
            {
                str_JobPath = null;
                ApplicationType = null;

                sb_InputXML.AppendFormat(@" <SEARCH>
                                                <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{4}'></SESSIONINFO>
                                                <TARGETOBJECT>FILEFORMAT</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
                                                        <FILTER>
		                                                    <EXPRESSION>FILEFORMAT</EXPRESSION>
		                                                    <OPERATOR>EQUALTO</OPERATOR>
		                                                    <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
		                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                    </FILTER>
                                                    </FILTERLIST>
                                                </TARGETENTITY>
                                            </SEARCH>", m_Language, m_TenantName, m_UserID, str_InputFileFormat, m_SessionID);

                ALInstallationInfo objInstallationInfo = ALCommon.GetInstallationDetails();

                if (objInstallationInfo.IsWebServiceMachine)
                {
                    #region Search for FileFormat in DB using OMI

                    String str_OMIAssemblyPath = ALCommon.GetInstallationDir() + "\\ALServices\\bin\\ObjectManagerInterface.dll";
                    Assembly OMIAssembly = Assembly.LoadFrom(str_OMIAssemblyPath);
                    Object OMIInstance = null;
                    Type OMIType = null;
                    MethodInfo GetHierarchalDataMethodInfo = null;

                    OMIType = OMIAssembly.GetType("Assetlink.Framework.OMInterface");
                    Object[] arrObjParameterList = { ulong.Parse(m_UserID), "ENGLISH", m_TenantName};
                    OMIInstance = Activator.CreateInstance(OMIType, arrObjParameterList);

                    GetHierarchalDataMethodInfo = OMIType.GetMethod("GetHierarchialData",new Type[]{typeof(String), typeof(String).MakeByRefType()});
                    if (GetHierarchalDataMethodInfo != null)
                    {
                        Object[] arrObjParameters = { sb_InputXML.ToString(), null };
                        str_ErrorXML = (string)GetHierarchalDataMethodInfo.Invoke(OMIInstance, arrObjParameters);

                        if (str_ErrorXML == null)
                            str_SearchOutputXML = (string)arrObjParameters[1];
                        else
                            throw new Exception("Error while searching File Format. Error XML : " + str_ErrorXML);
                    }
                    else
                        throw new Exception("Error while searching File Format. Method GetHierarchialData could not be found");

                    #endregion
                }
                else
                {
                    #region Search for FileFormat in DB using WS

                    Search obj_SearchWS = new Search();
                    obj_SearchWS.Url = ALCommon.GetWebServiceURL("ALSearchWS");
                    str_ErrorXML = obj_SearchWS.GetHierarchialData(sb_InputXML.ToString(), ref str_SearchOutputXML);

                    #endregion
                }


                if (str_ErrorXML != null)
                {
                    throw new Exception("Error while searching File Format. Error XML : " + str_ErrorXML);
                }
                xpDoc_FileFormat = new XPathDocument(new StringReader(str_SearchOutputXML));
                nav_FileFormat = xpDoc_FileFormat.CreateNavigator().SelectSingleNode("/FILEFORMATCOLLECTION/FILEFORMAT/MEDIAPROCESSORREQUIRED");

                str_FileFormatDetails = str_SearchOutputXML;



                if (nav_FileFormat == null)
                {
                    throw new ApplicationException("Invalid File Format.");
                }

                

                if (nav_FileFormat.InnerXml.ToUpper() != "1")
                    return false;

                xpNav_ApplicationType = xpDoc_FileFormat.CreateNavigator().SelectSingleNode("/FILEFORMATCOLLECTION/FILEFORMAT/APPLICATIONTYPE");
                if (xpNav_ApplicationType != null)
                {
                    ApplicationType = xpNav_ApplicationType.Value;
                }

                #region Prepare Job Path

                xpDoc_RouterConfig = new XPathDocument(str_MediaRouterConfigPath);
                xpNav_Root = xpDoc_RouterConfig.CreateNavigator();
                xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [(translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='" + SolutionName.ToUpper() + "']");

                if (xpNav_MediaServer == null)
                {
                    xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [(translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='DEFAULT']");
                }

                if (xpNav_MediaServer == null)
                {
                    throw new Exception("Invalid Media Router configuration");
                }

                if (String.IsNullOrEmpty(ApplicationType))
                {
                    throw new Exception("Invalid File Format configuration");
                }

                xpNav_Application = xpNav_MediaServer.SelectSingleNode("APPLICATION [translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')= '" + ApplicationType + "']");
                if (xpNav_Application != null)
                {
                    if (xpNav_Application.SelectSingleNode("FILEPATH/REPLACEABLEPATH") != null)
                        str_JobPath = xpNav_Application.SelectSingleNode("FILEPATH/REPLACEABLEPATH").Value;
                    else
                        str_JobPath = null;
                }

                #endregion

                return true;
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
        }

        public ERRORXML SendRequestToMediaProcessor(String InputXML, String ApplicationType, String RequestID, String ExtraInformation)
        { 
          return SendRequestToMediaProcessor(InputXML, ApplicationType, RequestID, ExtraInformation, System.Messaging.MessagePriority.Normal);
        }

        /// <summary>
        /// Add message in queue with given priority.
        /// </summary>
        /// <param name="InputXML"></param>
        /// <param name="ApplicationType"></param>
        /// <param name="RequestID"></param>
        /// <param name="ExtraInformation"></param>
        /// <param name="Priority"></param>
        /// <returns></returns>
        public ERRORXML SendRequestToMediaProcessor(String InputXML, String ApplicationType, String RequestID, String ExtraInformation, System.Messaging.MessagePriority Priority)
        {
            #region member Variable
            XPathDocument   xpDoc_RouterConfig      = null;
            XPathNavigator  nav_Temp                = null;
            XPathNavigator  nav_Servers             = null;
            XPathNavigator  nav_ServerNode          = null;
            XPathNavigator  nav_ApplicationNode     = null;
            XPathNavigator  nav_FilePath            = null;
            String          str_NWDrivePath         = null;
            String          str_ReplacablePath      = null;
            String          str_RouteMessageTimeout = null;
            int             iRouteMessageTimeout    = 0;
            MSMQWrapper     objMessaging            = null;
            #endregion

            ApplicationType     = ApplicationType.ToUpper();
            m_ApplicationType   = ApplicationType;
            m_InputXML          = InputXML;
            m_RequestID         = RequestID;
            m_ExtraInformation  = ExtraInformation;

            try
            {
                str_ReplacablePath = String.Empty;
                str_NWDrivePath    = String.Empty;

                #region Read the Router Configuration
                xpDoc_RouterConfig  = new XPathDocument(str_MediaRouterConfigPath);
                nav_Servers         = xpDoc_RouterConfig.CreateNavigator().SelectSingleNode("/MEDIAPROCESSORS");

                if (String.IsNullOrEmpty(MediaOutputQueuePath))
                {
                    nav_Temp = nav_Servers.SelectSingleNode("MEDIAOUTPUTQPATH");
                    if (nav_Temp != null)
                        m_MediaOutputQueuePath = nav_Temp.Value;
                }
                if (String.IsNullOrEmpty(UndeliveredQueuePath))
                {
                    nav_Temp = nav_Servers.SelectSingleNode("UNDELIVEREDQPATH");
                    if (nav_Temp != null)
                        m_UndeliveredQueuePath = nav_Temp.Value;
                }
                nav_ServerNode = nav_Servers.SelectSingleNode("MEDIAPROCESSOR[(translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='" + m_SolutionName + "' and APPLICATION[(translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='" + ApplicationType + "']]");
                if(nav_ServerNode == null)
                {
                    //str_ErrorXML = ALSolutionCommon.PrepareErrorXML("No media server supports the specified application type");
                    str_ErrorXML = "No media server supports the specified application type";
                    return str_ErrorXML;
                }
                
                str_RouteMessageTimeout = nav_ServerNode.SelectSingleNode("ROUTEMESSAGETIMEOUT").Value;

                nav_ApplicationNode = nav_ServerNode.SelectSingleNode("APPLICATION[(translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='" + ApplicationType + "']");
                if (nav_ApplicationNode != null)
                {
                    if (String.IsNullOrEmpty(MediaServerQueuePath))
                    	m_MediaServerQueuePath  = nav_ApplicationNode.SelectSingleNode("MEDIAPROCESSORQPATH").Value;

                    nav_FilePath            = nav_ApplicationNode.SelectSingleNode("FILEPATH");
                    if (nav_FilePath != null)
                    {
                        //m_MediaServerQueuePath = nav_ServerNode.SelectSingleNode("APPLICATION[(translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'))='" + ApplicationType + "']/MEDIASERVERQPATH").Value;
                        str_ReplacablePath = nav_FilePath.SelectSingleNode("REPLACEABLEPATH").Value;
                        str_NWDrivePath = nav_FilePath.SelectSingleNode("NETWORKPATH").Value;
                    }
                    
                }
                #endregion

                #region Prepare Request XML
                String str_InputXML = PrepareRequestXML(InputXML, RequestID, ExtraInformation, str_NWDrivePath, str_ReplacablePath);
                #endregion

                #region Prepare and send the request message
                objMessaging                            = new MSMQWrapper(MediaServerQueuePath, QueueAccessMode.SendMessage);
                objMessaging.AdministrationQueuePath    = UndeliveredQueuePath;
                objMessaging.MessagePriority            = Priority;

                if (!String.IsNullOrEmpty(str_RouteMessageTimeout) && int.TryParse(str_RouteMessageTimeout, out iRouteMessageTimeout))
                    objMessaging.TimeToReachQueue = new TimeSpan(0, iRouteMessageTimeout, 0);
                else
                {                    
                    str_ErrorXML = "Invalid RouteMessageTimeout in the router configuration.";
                    return str_ErrorXML;
                }

                str_ErrorXML = objMessaging.SendMessage(TenantName + " - " + SolutionName, str_InputXML);

                #endregion

                return str_ErrorXML;
            }
            catch (Exception ExcepObj)
            {                
                str_ErrorXML = ExcepObj.Message;
                return str_ErrorXML;
            }
        }

        private String PrepareRequestXML(String InputXML, String RequestID, String ExtraInformation, String str_NWDrivePath, String str_ReplacablePath)
        {
            ALXMLDocument objXmlDoc = new ALXMLDocument("REQUEST");
            objXmlDoc.AppendChildNode("REQUEST", "REQUESTHEADER");
            objXmlDoc.AppendChildNode("REQUESTHEADER", "USERID");
            objXmlDoc.InnerText = UserID;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "SESSIONID");
            objXmlDoc.InnerText = SessionID;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "SOLUTIONNAME");
            objXmlDoc.InnerText = SolutionName;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "TENANTNAME");
            objXmlDoc.InnerText = TenantName;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "LANGUAGE");
            objXmlDoc.InnerText = Language;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "REQUESTID");
            objXmlDoc.InnerText = RequestID;
            objXmlDoc.AppendChildNode("REQUESTHEADER", "REQUESTSTATUS");
            objXmlDoc.InnerText = "NEW";
            objXmlDoc.AppendChildNode("REQUEST", "INPUTXML");
            if (!String.IsNullOrEmpty(str_ReplacablePath) && (!String.IsNullOrEmpty(str_NWDrivePath)) && ReplaceNetworkDrivePath)
            {
 				//This is done conditionally because in case the cataloger is on internal network, then Cataloger mount point
            	// folder is the work area for Media processor. In this case cataloger will give input xml in required format
            	// i.e. File will be already in Network drive format hence no need to replace.
                objXmlDoc.InnerXML = InputXML.Replace(str_ReplacablePath, str_NWDrivePath);
            }
            else
                objXmlDoc.InnerXML = InputXML;

            objXmlDoc.AppendChildNode("REQUEST", "OUTPUTQPATH");
            objXmlDoc.InnerText = MediaOutputQueuePath;
            //objXmlDoc.AppendChildNode("REQUEST", "FILEPATH");
            //objXmlDoc.AppendChildNode("FILEPATH", "REPLACEABLEPATH");
            //objXmlDoc.InnerText = str_ReplacablePath;
            //objXmlDoc.AppendChildNode("FILEPATH", "NETWORKPATH");
            //objXmlDoc.InnerText = str_NWDrivePath;
            objXmlDoc.AppendChildNode("REQUEST", "EXTRAINFORMATION");
            objXmlDoc.InnerXML = ExtraInformation;
            objXmlDoc.AppendChildNode("EXTRAINFORMATION", "FILEPATH");
            objXmlDoc.AppendChildNode("FILEPATH", "REPLACEABLEPATH");
            objXmlDoc.InnerText = str_ReplacablePath;
            objXmlDoc.AppendChildNode("FILEPATH", "NETWORKPATH");
            objXmlDoc.InnerText = str_NWDrivePath;

            return objXmlDoc.Outerxml;
        }

        public ERRORXML TransformResponseXML(ref String ResponseXML)
        {
            XmlDocument xdoc_ResponseXML = new XmlDocument();
            String str_InputXML = null;
            XmlNode xNode_Temp = null;
            String str_ReplacablePath = null;
            String str_NWDrivePath = null;

            try
            {

                xdoc_ResponseXML.LoadXml(ResponseXML);
                xNode_Temp = xdoc_ResponseXML.SelectSingleNode("/REQUEST");

                str_ReplacablePath = xNode_Temp.SelectSingleNode("EXTRAINFORMATION/FILEPATH/REPLACEABLEPATH").InnerXml;
                str_NWDrivePath = xNode_Temp.SelectSingleNode("EXTRAINFORMATION/FILEPATH/NETWORKPATH").InnerXml;

                xNode_Temp = xNode_Temp.SelectSingleNode("INPUTXML");
                str_InputXML = xNode_Temp.InnerXml.Replace(str_NWDrivePath, str_ReplacablePath);
                xNode_Temp.InnerXml = str_InputXML;

                ResponseXML = xdoc_ResponseXML.OuterXml;
                xdoc_ResponseXML = null;

                return null;
            }
            catch (Exception ExcepObj)
            {
                //str_ErrorXML = ALSolutionCommon.PrepareErrorXML(ExcepObj.Message);
                str_ErrorXML = ExcepObj.Message;
                return str_ErrorXML;
            }
        }

        public ERRORXML GetJobPath(String ApplicationType, out String str_JobPath)
        {
            //String str_MediaRouterConfigPath = null;
            XPathDocument xpDoc_RouterConfig = null;
            XPathNavigator xpNav_Root = null;
            XPathNavigator xpNav_MediaServer = null;
            XPathNavigator xpNav_ApplicationMediaServer = null;

            str_JobPath = null;

            if (String.IsNullOrEmpty(ApplicationType))
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Application Type");                
                return "Invalid Application Type";
            }

            //str_MediaRouterConfigPath = ALCommon.GetInstallationDir() + "\\Configuration\\XML\\MediaRequestRouter.xml";
            xpDoc_RouterConfig = new XPathDocument(str_MediaRouterConfigPath);
            xpNav_Root = xpDoc_RouterConfig.CreateNavigator();
            xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']");

            if (xpNav_MediaServer == null)
            {
                xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='DEFAULT']");
            }

            if (xpNav_MediaServer == null)
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");               
                return "Invalid Media Router configuration";
            }

            xpNav_ApplicationMediaServer = xpNav_MediaServer.SelectSingleNode("APPLICATION [translate(@TYPE, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')= '" + ApplicationType.ToUpper() + "']");

            if (xpNav_ApplicationMediaServer == null)
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");
                return "Invalid Media Router configuration";
            }
            
            if (xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/REPLACEABLEPATH") != null)
                str_JobPath = xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/REPLACEABLEPATH").Value;
            else
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");
                return "Invalid Media Router configuration";
            
            return null;
        }

        public ERRORXML GetNetworkPath(String ApplicationType, out String str_NetworkPath)
        {
            //String str_MediaRouterConfigPath = null;
            XPathDocument xpDoc_RouterConfig = null;
            XPathNavigator xpNav_Root = null;
            XPathNavigator xpNav_MediaServer = null;
            XPathNavigator xpNav_ApplicationMediaServer = null;

            str_NetworkPath = null;

            if (String.IsNullOrEmpty(ApplicationType))
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Application Type");                
                return "Invalid Application Type";
            }

            //str_MediaRouterConfigPath = ALCommon.GetInstallationDir() + "\\Configuration\\XML\\MediaRequestRouter.xml";
            xpDoc_RouterConfig = new XPathDocument(str_MediaRouterConfigPath);
            xpNav_Root = xpDoc_RouterConfig.CreateNavigator();
            xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']");

            if (xpNav_MediaServer == null)
            {
                xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='DEFAULT']");
            }

            if (xpNav_MediaServer == null)
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");               
                return "Invalid Media Router configuration";
            }

            xpNav_ApplicationMediaServer = xpNav_MediaServer.SelectSingleNode("APPLICATION [translate(@TYPE, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')= '" + ApplicationType.ToUpper() + "']");

            if (xpNav_ApplicationMediaServer == null)
            {
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");
                return "Invalid Media Router configuration";
            }

            if (xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/NETWORKPATH") != null)
                str_NetworkPath = xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/NETWORKPATH").Value;
            else
                //return ALSolutionCommon.PrepareErrorXML("Invalid Media Router configuration");
                return "Invalid Media Router configuration";

            return null;
        }

        public ERRORXML GetNetworkPath(String ApplicationType, out String str_NetworkPath, out String str_ReplacablePath)
        {            
            XPathDocument   xpDoc_RouterConfig  = null;
            XPathNavigator  xpNav_Root          = null;
            XPathNavigator  xpNav_MediaServer   = null;
            XPathNavigator  xpNav_ApplicationMediaServer = null;

            str_NetworkPath = null;
            str_ReplacablePath = null;

            if (String.IsNullOrEmpty(ApplicationType))
            {            
                return "Invalid Application Type";
            }

            xpDoc_RouterConfig = new XPathDocument(str_MediaRouterConfigPath);
            xpNav_Root = xpDoc_RouterConfig.CreateNavigator();
            xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']");

            if (xpNav_MediaServer == null)
            {
                xpNav_MediaServer = xpNav_Root.SelectSingleNode("/MEDIAPROCESSORS/MEDIAPROCESSOR [translate(@SOLUTIONNAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='DEFAULT']");
            }

            if (xpNav_MediaServer == null)
            {             
                return "Invalid Media Router configuration";
            }

            xpNav_ApplicationMediaServer = xpNav_MediaServer.SelectSingleNode("APPLICATION [translate(@TYPE, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')= '" + ApplicationType.ToUpper() + "']");

            if (xpNav_ApplicationMediaServer == null)
            {                
                return "Invalid Media Router configuration";
            }

            if (xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/NETWORKPATH") != null)
                str_NetworkPath = xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/NETWORKPATH").Value;
            else             
                return "Invalid Media Router configuration";

            if (xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/REPLACEABLEPATH") != null)
                str_ReplacablePath = xpNav_ApplicationMediaServer.SelectSingleNode("FILEPATH/REPLACEABLEPATH").Value;
            else
                return "Invalid Media Router configuration";

            return null;
        }

        #endregion

    }
}
