﻿#region Using Directives
#region System
using System;
using System.Xml;
using System.Text;
using System.Reflection;
using System.Configuration;
using System.Collections.Generic;
using System.Diagnostics;

#endregion
#region Assetlink
using Assetlink;
#endregion
#endregion

//---------------------------------------------------------------------------------------------------//
// File Name: DataManager.cs                                                                         //   
// Author   : Miss. Afsar Khan                                                                       // 
// Class    : DataManager                                                                            //
// Purpose  : Core class Data Access Layer. This loads appropriate Database ADO.NET assembly.    //
//---------------------------------------------------------------------------------------------------//

namespace Assetlink.DataAccessLayer
{
    /// <summary>
    /// <CreatedDate> 18-04-2005</CreatedDate>
	/// <Author>Miss Afsar Khan</Author>
	/// <Purpose>
    /// Core Class for Data Access Layer. 
    /// 1. This loads appropriate Database ADO.NET assembly.
    /// 2. Initialiaze the Connection,Command and Transaction objects.
	/// </Purpose>
    /// </summary>
    public class DataManager
    {
        #region Member Declaration

        private         string  m_DataBaseType              = null;
        private         string  m_ConnectionString          = null;
        private         string  m_ConnectionStringForRead   = null;
        private         Boolean m_TraceEnabled              = false;    //The DataProvider object holds the TraceEnabled value which indicates whether tracing should be done or no
        private         String  m_NameOfTenant              = null;
        private         String  m_InstalledPath             = null;
        private         long    m_SessionTimeOut            = 0;
        private         String  m_AssemblyName              = null;
        public          QueryProcessor DataProvider         = null;	    //The DataProvider object holds the instance of the proper class of the Database Type
        private         String  m_FTSLogicalOperator        = null;
		private			String	m_ODSConnectionStringForRead = null;
        private			String	m_ODSConnectionString		= null;
        private         int     m_RetrySleepDuration        = 0; //Database Query Retry duration      
        private         Object m_FileSystemManager          = null;
        private         IXMLRepository m_XMLManager         = null;
        public          List<String> CacheItemsToRemove     = new List<String>();
        #endregion
        
        #region Properties
        /// <summary>
        /// Name OF Tenant
        /// </summary>
        public String TenantName
        {
            get { return m_NameOfTenant; }
            set { m_NameOfTenant = value; }
        }


        /// <summary>
        /// Whether tracing is enabled for Tenant
        /// </summary>
        public Boolean TraceEnabled
        {
            get { return m_TraceEnabled; }
            set { m_TraceEnabled = value; }
        }

        public string ConnectionString
        {
            get { return m_ConnectionString; }
            set { m_ConnectionString = value; }
        }

        public string DataBaseType
        {
            get { return m_DataBaseType; }
            set { m_DataBaseType = value; }
        }

        public long SessionTimeOut
        {
            get { return m_SessionTimeOut; }
            set { m_SessionTimeOut = value; }
        }
    
        public string ConnectionStringForRead
        {
            get { return m_ConnectionStringForRead; }
            set { m_ConnectionStringForRead = value; }
        }

		public string ODSConnectionStringForRead
        {
            get { return m_ODSConnectionStringForRead; }
            set { m_ODSConnectionStringForRead = value; }
        }
        public string ODSConnectionString
        {
            get { return m_ODSConnectionString; }
            set { m_ODSConnectionString = value; }
        }

        public int RetrySleepDuration
        {
            get {
                    if (m_RetrySleepDuration == 0)
                        return 1000;
                    else
                        return m_RetrySleepDuration;
                }
            set { m_RetrySleepDuration = value; }
        }

        public Object FileSystemManager
        {
            get
            {
                return m_FileSystemManager;
            }
            set
            {
                m_FileSystemManager = value;
            }
        }

        public IXMLRepository XMLRepository
        {
            get
            {
                return m_XMLManager;
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// <CreatedDate> 18-04-2005</CreatedDate>
	    /// <Author>Miss Afsar Khan</Author>
	    /// <Purpose>
        /// Default Constructor.
	    /// </Purpose>
        /// </summary>
        public DataManager()
        {
        }

        /// <summary>
       	/// <CreatedDate> 18-04-2005</CreatedDate>
	    /// <Author>Miss Afsar Khan</Author>
	    /// <Purpose>
        /// Constructor: To initialize the DataManager object with TenantName
        /// In the Constructor of the Data Manager class the Configuration file is loaded
	    /// which contains the details of the database for the client. Get the 
	    /// Connection String and the Database type. Depending on the database, create 
	    /// an instance of SQLDataProvider or OracleDataProvider. The DataProvider object
	    /// attribute of the Data Manager class holds the instance of this object
	    /// </Purpose>
        /// </summary>
        /// <param name="TenantName">The Name of the Tenant</param>
        public DataManager(String TenantName)
        {
           Tenant     client          = null;   //Represent the Tenant Object
            bool      cacheEnabled    = false;  //Whether Cache is enabled or not.
            String    cachedItemName  = null;   //Name of Cached Item.
            
			try
			{
                cacheEnabled = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                if (cacheEnabled == true)
                {
                    //CacheManager.Error += new CacheManager.OnError(CacheError); //Subscribe to Cache Error Events
                    cachedItemName      = TenantName.ToUpper() + "_CONFIG";
                    if (CacheManager.IsCached(cachedItemName.ToUpper()) == false)
                    {   
                        client    = new Tenant(TenantName);
                        CacheManager.AddCacheItem(cachedItemName.ToUpper(), client, 
                                                  client.ConfigurationPath);
                    }
                    else
                    {
                        client = CacheManager.GetCacheItem(cachedItemName.ToUpper()) as Tenant;
                    }
                    initMembers(client);
                }
                else
                {
                    InitTenantConfiguration(TenantName);
                }

                InitQueryProcessor();

                if (DataProvider != null)
                {
                    DataProvider.OMDataManager      = this;
                    DataProvider.FTSLogicalOperator = m_FTSLogicalOperator;
                    DataProvider.TenantName         = TenantName;
                    DataProvider.DatabaseType       = DataBaseType;
                    DataProvider.InitializeDatabaseSetup(ConnectionString);

                    //Initializes the Cache
                    //if (cacheEnabled == true)
                    //{
                        initializeCache(client);
                    //}
                }
                else
                {
                    throw new ApplicationException("Unable to create DataManager Object.");
                }
                m_XMLManager = GetXMLManagerInstance();
			}
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }

            finally
            {
               // TenantXMLDocument = null;
            }
        }
        /// <summary>
        /// <CreatedDate> 18-04-2005</CreatedDate>
	    /// <Author>Miss Afsar Khan</Author>
	    /// <Purpose>
        /// In the Constructor of the Data Manager class the Configuration file is loaded
	    /// which contains the details of the database for the client. Get the 
	    /// Connection String and the Database type. Depending on the database, create 
	    /// an instance of SQLDataProvider or OracleDataProvider. The DataProvider object
	    /// attribute of the Data Manager class holds the instance of this object
	    /// </Purpose>
        /// </summary>
        /// <param name="TenantName">The name of Tenant</param>
        /// <param name="TransactionFlag">Whether Transaction is allowed or not.True: Allowed, False: Not Allowed</param>
        public DataManager(String TenantName,Boolean TransactionFlag)
		{
            bool    cacheEnabled   = false;
            Tenant  client         = null; 
            String  cachedItemName = null;
			try
			{
                cacheEnabled = true;// Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                if (cacheEnabled == true)
                {
                    //CacheManager.Error += new CacheManager.OnError(CacheError); //Subscribe to Cache Error Events
                    cachedItemName      = TenantName.ToUpper() + "_CONFIG";
                    if (CacheManager.IsCached(cachedItemName.ToUpper()) == false)
                    {
                       client = new Tenant(TenantName);
                       CacheManager.AddCacheItem(cachedItemName.ToUpper(), client, client.ConfigurationPath);
                    }
                    else
                    {
                        client = CacheManager.GetCacheItem(cachedItemName.ToUpper()) as Tenant;

                    }
                    initMembers(client);
                }
                else
                {
                    InitTenantConfiguration(TenantName);
                }

                InitQueryProcessor();

                if (DataProvider != null)
                {
                    DataProvider.OMDataManager = this;
                    DataProvider.TenantName = TenantName;
                    DataProvider.InitializeDatabaseSetup(ConnectionString, TransactionFlag);

                    //Initializes the Cache
                    //if (cacheEnabled == true)
                    //{
                       initializeCache(client);
                    //}
                }
                else
                {
                    throw new ApplicationException("Unable to create DataManager Object.");
                }
                m_XMLManager = GetXMLManagerInstance();
              
			}
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }

            finally
            {
               
            }
		}

        /// <summary>
        /// Initialize the Cache for the Database.
        /// </summary>
        private void initializeCache(Tenant client)
        {
            bool    notCached       = false;
            String  cachedItemName  = null;
            String  changedTables   = null;
            String[]tableArray      = null;

            //Check whether the items are cached or not.
            foreach (String cacheTable in client.CacheTableList)
            {
                cachedItemName = m_NameOfTenant.ToUpper() + "_" + cacheTable.ToUpper();
                if (CacheManager.IsCached(cachedItemName.ToUpper()) == false)
                {
                    //Even a single table is not cache will refresh the cache.
                    changedTables += cacheTable.ToUpper() + ",";
                    notCached = true;
                    
                }
            }
            //If not cache then refresh the list of tables.
            if (notCached == true)
            {
                changedTables = changedTables.Substring(0,changedTables.Length-1);
                tableArray    = changedTables.Split(',');
                
                DataProvider.InitializeCache(client.ConnectionString, tableArray);
                
            }
            
        }

        private void InitQueryProcessor()
        {
            Assembly        Assm                = null;
            Type            ClassType           = null;
            StringBuilder   AssemblyPath        = null;

            try
            {
                AssemblyPath = new StringBuilder();
                AssemblyPath.AppendFormat("{0}Binaries\\{1}.dll", m_InstalledPath, m_AssemblyName);

                //Load the Database Assembly
                Assm            = Assembly.LoadFrom(AssemblyPath.ToString());
                //GetType method is used to load the required class (since there can be more the one class in the assembly)
                ClassType       = Assm.GetType("Assetlink.DataAccessLayer." + m_AssemblyName, true);
                DataProvider    = (Assetlink.DataAccessLayer.QueryProcessor)Activator.CreateInstance(ClassType);
                DataProvider.OMDataManager = this;
            }
            catch(Exception ex)
            {
                throw ex;
            }
        }

        private void InitTenantConfiguration(String TenantName)
        {
            XmlDocument     TenantXMLDocument   = null;
            XmlNode         ConnNode            = null ;
            XmlNode         DBNode              = null;
            XmlNode         AssemblyNode        = null;
            StringBuilder   AppXMLPath          = null;
            XmlNode         TracingNode         = null;
            XmlNode         SessionNode         = null;
            XmlNode         tempNode            = null;
            XmlNode         RetrySleepDurationNode = null;

            m_NameOfTenant      = TenantName;
            m_InstalledPath     = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK",
                                                        "Installed Path");
            AppXMLPath = new StringBuilder();
            AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(TenantName));

            TenantXMLDocument = new XmlDocument();
            TenantXMLDocument.Load(AppXMLPath.ToString());

            //Reading the Connection String
            ConnNode            = TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING");
            ConnectionString    = Assetlink.ALCommon.Decrypt(ConnNode.InnerText);
            ConnNode            = null;

            ConnNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP");
            m_ConnectionStringForRead = Assetlink.ALCommon.Decrypt(ConnNode.InnerText);

			ConnNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORODSSP");
			m_ODSConnectionStringForRead = Assetlink.ALCommon.Decrypt(ConnNode.InnerText);

            RetrySleepDurationNode =TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/RETRYSLEEPDURATION");
            if (RetrySleepDurationNode != null && RetrySleepDurationNode.InnerText != "")
                m_RetrySleepDuration = Convert.ToInt16(RetrySleepDurationNode.InnerText);

            SessionNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/SESSION");
            if (SessionNode == null)
            {
                //Set SessionTimeOut to default value is SESSION node is not found in Tenant xml
                //The default value for SessionTimeOut is 12 Hours
                SessionTimeOut = 43200;
            }
            else
            {
                SessionTimeOut = Convert.ToInt64(SessionNode.Attributes.GetNamedItem("Timeout").InnerText);
            }
            SessionNode = null;


            //Reading the Database Type
            DBNode          = TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/DATABASETYPE");
            m_DataBaseType  = DBNode.InnerText;
            DBNode           = null;

            //Checking if tracing is enabled
            TracingNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/TRACING");
            if (TracingNode != null)
            {
                if (TracingNode.InnerText.ToUpper() == "TRUE")
                {
                    SetTracing(true);
                }
                else
                {
                    SetTracing(false);
                }
                TracingNode = null;
            }
            else
            {
                SetTracing(false);
            }

            //Checking if tracing is enabled
            tempNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FULLTEXTSEARCH/LOGICALOPERATOR");
            if (tempNode != null)
            {
                m_FTSLogicalOperator = tempNode.InnerText;
            }

            //Reading the Assembly Name
            AssemblyNode    = TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/ASSEMBLYNAME");
            m_AssemblyName  = AssemblyNode.InnerText;
            AssemblyNode    = null;

        }

        /// <summary>
		/// <CreatedDate>13-05-2011</CreatedDate>
		/// <Author>Mugdha Moolay Gadre</Author>
		/// <Purpose>
		/// In the Constructor of the Data Manager class the Configuration file is loaded
		/// which contains the details of the database for the client. Get the 
		/// Connection String and the Database type. Depending on the database, create 
		/// an instance of SQLDataProvider or OracleDataProvider. The DataProvider object
		/// attribute of the Data Manager class holds the instance of this object
		/// </Purpose>
		/// </summary>
		/// <param name="TenantName">The name of Tenant</param>
		/// /// <param name="TransactionFlag">Whether Transaction is allowed or not.True: Allowed, False: Not Allowed</param>
		/// <param name="ExecuteQueriesOnODSDB">Flag if true indicates that Queries should be executed on ODS database</param>
		public DataManager(String TenantName, Boolean TransactionFlag, Boolean ExecuteQueriesOnODSDB)
		{
            bool    cacheEnabled   = false;
            Tenant  client         = null; 
            String  cachedItemName = null;
			try
			{
                cacheEnabled = true;// Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                if (cacheEnabled == true)
                {
                    //CacheManager.Error += new CacheManager.OnError(CacheError); //Subscribe to Cache Error Events
                    cachedItemName      = TenantName.ToUpper() + "_CONFIG";
                    if (CacheManager.IsCached(cachedItemName.ToUpper()) == false)
                    {
                       client = new Tenant(TenantName);
                       CacheManager.AddCacheItem(cachedItemName.ToUpper(), client, client.ConfigurationPath);
                    }
                    else
                    {
                        client = CacheManager.GetCacheItem(cachedItemName.ToUpper()) as Tenant;

                    }
                    initMembers(client);
                }
                else
                {
                    InitTenantConfiguration(TenantName);
                }

                InitQueryProcessor();

                if (DataProvider != null)
                {
                    DataProvider.OMDataManager = this;                    
                    DataProvider.TenantName = TenantName;
					if (ExecuteQueriesOnODSDB)
                        DataProvider.InitializeDatabaseSetup(ODSConnectionString, TransactionFlag);
					else
						DataProvider.InitializeDatabaseSetup(ConnectionString, TransactionFlag);

                    //Initializes the Cache
                    //if (cacheEnabled == true)
                    //{
                       initializeCache(client);
                    //}
                }
                else
                {
                    throw new ApplicationException("Unable to create DataManager Object.");
                }
                m_XMLManager = GetXMLManagerInstance();
			}
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }

            finally
            {
               
            }
		}

        #endregion

        #region Methods
        public void SetTracing(Boolean bTraceEnabled)
        {
            m_TraceEnabled = bTraceEnabled;
        }

        public void Close()
        {
            DataProvider.Close();
        }
        private void initMembers(Tenant client)
        {
            this.ConnectionString           = client.ConnectionString;
            this.ConnectionStringForRead    = client.ConnectionStringForRead;
            this.DataBaseType               = client.DataBaseType;
            this.SessionTimeOut             = client.SessionTimeOut;
            this.m_AssemblyName             = client.AssemblyName;
            this.m_InstalledPath            = client.InstalledPath;
            this.m_NameOfTenant             = client.Name;
            this.m_FTSLogicalOperator       = client.FTSLogicalOperator;
			this.m_ODSConnectionStringForRead = client.ODSConnectionStringForRead;
            this.m_ODSConnectionString		= client.ODSConnectionString;
            this.m_RetrySleepDuration       = client.RetrySleepDuration;
            this.SetTracing(client.TraceEnabled);
        }
        #endregion
    
        #region Cache Exception Methods

        /// <summary>
        /// Log the exception caused by Cache using Standard Cache Infrastrcuture.
        /// </summary>
        /// <param name="ex">The Exception Object</param>
        private void CacheError(Exception ex)
        {
            //OMError error = new OMError(OMErrorCodes.EXCEPTION_OCCURED,ex.Message,
            //                            OMErrorObjectType.DATAACCESSLAYER,"Cache Manager",
            //                            OMErrorType.EXCEPTION,ex,null);
            //Diagnostic.ExceptionTracing.LogError(TenantName, error);
        }
        #endregion

        #region Retry Validation Function
        public static Boolean CanRetry(Exception ExceptionObj, int RetryCount, ref int IterationCount, int RetrySleepDuration)
        {
            Boolean     DoCanRetry          = false;
            String      ErrorXml            = null;
            String      MethodName          = "";
            String      StackTraceText      = "";
            if (ExceptionObj != null)
            {
                
                try
                {
                    StackTrace StackTraceObj = new StackTrace(false);
                    if (StackTraceObj != null)
                    {
                        MethodName        = StackTraceObj.GetFrame(1).GetMethod().ToString();
                        StackTraceText    = StackTraceObj.ToString().Substring(StackTraceObj.ToString().IndexOf("\r\n") + 2);
                    }

                }
                catch { }

                ErrorXml    = "<ERROR><ERRORTYPE>EXCEPTION</ERRORTYPE><MESSAGE>" + ExceptionObj.Message + "</MESSAGE></ERROR>";
                DoCanRetry  = CanRetry(ErrorXml, RetryCount, ref IterationCount, RetrySleepDuration, MethodName, StackTraceText);
            }
            return DoCanRetry;
        }

        public static Boolean CanRetry(String ErrorXml, int RetryCount, ref int IterationCount, int RetrySleepDuration)
        {
            String  MethodName      = "";
            String  StackTraceText  = "";
            try
            {
                StackTrace StackTraceObj = new StackTrace(false);
                if (StackTraceObj != null)
                {
                    MethodName          =   StackTraceObj.GetFrame(1).GetMethod().ToString();
                    StackTraceText      = StackTraceObj.ToString().Substring(StackTraceObj.ToString().IndexOf("\r\n") + 2);
                }
            }
            catch { }

            return CanRetry(ErrorXml, RetryCount, ref IterationCount, RetrySleepDuration, MethodName, StackTraceText);
        }

        private static Boolean CanRetry(String ErrorXml, int RetryCount, ref int IterationCount, int RetrySleepDuration, String MethodName, String StackTraceTextObj)
        {
            XmlDocument ErrorXmlDoc         = null;
            String      ErrorType           = null;
            String      ExceptionMessage    = null;
            Boolean     DoCanRetry          = false;
            String      ErrorNodeXpath      = null;

            if (!String.IsNullOrEmpty(ErrorXml))
            {
                try
                {
                    ErrorXmlDoc = new XmlDocument();
                    ErrorXmlDoc.LoadXml(ErrorXml);

                    if (ErrorXmlDoc.SelectSingleNode("ERROR/ERRORTYPE") != null)
                        ErrorNodeXpath = "ERROR";
                    else
                        ErrorNodeXpath = "ERRORCOLLECTION/ERROR";

                    foreach(XmlNode ErrorNode in ErrorXmlDoc.SelectNodes(ErrorNodeXpath))
                    {
                        ErrorType = ErrorNode.SelectSingleNode("ERRORTYPE").InnerText.ToUpper();
                        if (ErrorNode.SelectSingleNode("EXCEPTION/MESSAGE") != null)
                            ExceptionMessage = ErrorNode.SelectSingleNode("EXCEPTION/MESSAGE").InnerText.Trim();
                        else
                            ExceptionMessage = ErrorNode.SelectSingleNode("MESSAGE").InnerText.Trim();

                         //Error Lookup
                        if (ErrorType == "EXCEPTION")
                        {
                            if (ExceptionMessage.ToUpper().Contains("CONNECTION WAS FORCIBLY CLOSED") || ExceptionMessage.ToUpper().Contains("A TRANSPORT-LEVEL ERROR HAS OCCURRED"))
                            {
                                IterationCount++;
                                DoCanRetry  = true;
                                ErrorXml    = null;

                                WriteToEventLog("Thread ID :" + System.Threading.Thread.CurrentThread.GetHashCode() + "\r\nMethod : " + MethodName + ", " + IterationCount.ToString() + " Execution Failed\r\nError Message : " + ExceptionMessage + "\r\nStack :" + StackTraceTextObj, EventLogEntryType.Warning);
                            }
                        }

                        if (DoCanRetry == true && RetryCount == IterationCount)
                            DoCanRetry = false;
                        else
                            System.Threading.Thread.Sleep(RetrySleepDuration);

                        if (DoCanRetry)
                            break;
                    }
                }
                catch(Exception ExceptionObj)
                {
                    DoCanRetry = false;
                    WriteToEventLog("Thread ID :" + System.Threading.Thread.CurrentThread.GetHashCode() + "\r\nMethod : CanRetry Execution Failed\r\nError Message : " + ExceptionObj.Message + "\r\nStack :" + StackTraceTextObj, EventLogEntryType.Warning);
                }
            }

            return DoCanRetry;
        }


        public static void WriteToEventLog(String EventMessage, EventLogEntryType EventLogType)
        {
            String Source   = "DataAccessLayer";
            String LogName  = "SAS MOM";
            String Machine  = ".";
            String str_oldLogName = null;
            EventLog EventLogObj = null;
            try
            {
                if (EventLog.SourceExists(Source, Machine))
                {
                    str_oldLogName = EventLog.LogNameFromSourceName(Source, Machine);
                    if (String.Compare(LogName, str_oldLogName, true) != 0)
                    {
                        EventLog.DeleteEventSource(Source);
                        EventLog.CreateEventSource(Source, LogName);
                    }
                }
                else
                {
                    //if (!EventLog.SourceExists(Source, Machine))
                     EventLog.CreateEventSource(Source, LogName);
                }                

                EventLogObj = new EventLog(LogName, Machine, Source);
                EventLogObj.WriteEntry(EventMessage, EventLogType);
            }
            catch { }
        }

        #endregion

        private IXMLRepository GetXMLManagerInstance()
        {
            Assembly Assm = null;
            Type ClassType = null;
            StringBuilder AssemblyPath = null;

            try
            {
                String m_InstalledPath = ALCommon.GetInstallationDir();
                String m_AssemblyName = "XMLManager";
                AssemblyPath = new StringBuilder();
                AssemblyPath.AppendFormat("{0}Binaries\\{1}.dll", m_InstalledPath, m_AssemblyName);

                //Load the Database Assembly
                Assm = Assembly.LoadFrom(AssemblyPath.ToString());
                //GetType method is used to load the required class (since there can be more the one class in the assembly)
                ClassType = Assm.GetType("Assetlink.XMLManager", true);
                object[] args = new object[] { TenantName };
                IXMLRepository objXMLManager = (Assetlink.DataAccessLayer.IXMLRepository)Activator.CreateInstance(ClassType, args);
                return objXMLManager;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }

    public interface IXMLRepository
    {
        OMError Save(String TypeName, ulong ObjectID, String FieldName, String FileData);
        OMError Load(String TypeName, ulong ObjectID, String FieldName, out String FileData);

        OMError DeleteXML(String TypeName, ulong ObjectID, String FieldName);
        OMError DeleteXML(String TypeName, String[] ArrObjectID, String FieldName);
        OMError DeleteXML(String TypeName, ulong ObjectID);
        OMError DeleteXML(String TypeName);
        OMError CopyXSD(String TypeName, String FieldName, String SourceXSDPath);
        OMError RetrieveXSDPath(String TypeName, String FieldName, out String XSDFilePath);
        OMError DeleteXSD(String TypeName, String FieldName);
        OMError DeleteXSD(String TypeName, String[] ArrFieldName);

        OMError CommitXMLOperations();
        OMError RollbackXMLOperations();
        OMError HandleInDoubtTransactions(String Info);

        void Init(DataManager OMDataManager);

        //DataManager OMDataManager
        //{
        //    get;
        //    set;
        //}

        ulong TranscationId
        {
            get;
            set; 
        }
    }

}