#region System Includes

using System;
using System.Xml;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Xml.XPath;

#endregion

#region Assetlink Includes

using Assetlink;
using Assetlink.DataAccessLayer;

#endregion

//---------------------------------------------------------------------------------------------------//
// File Name: SQLQueryProcessor.cs                                                                   //   
// Author   : Miss. Afsar Khan                                                                       // 
// Class    : QueryProcessor                                                                         //
// Purpose  : Conatins logic SQL Server database access code.                                        //
//---------------------------------------------------------------------------------------------------//

namespace Assetlink.DataAccessLayer
{
    using ERRORXML = System.String;
    using XML = System.String;

    /// <summary>
    /// <CreatedDate>18-04-2005</CreatedDate>
    /// <purpose>
    /// This class contains all the SQL server specific functions for e.g 
    /// Converting the date into SQL specific format.
    /// Initializing the Connection object, Command Object and the Transaction Object.
    /// </purpose>
    /// <Author>Miss Afsar Khan</Author>
    /// </summary>
    public partial class SQLQueryProcessor : QueryProcessor
    {

        /// <summary>
        /// <Purpose>
        /// The CreateTable function takes InputXML as parameter.Based on the details of the Fields specified
        /// the query for the creating the table is prepared.Based on the Mandatory node it indicates whether
        /// the field value can be null or no.Based on the type the Field Type will be decided.Based on the 
        /// Primary Key node value the Primary Key of the table will be decided.
        /// Based on all these inputs the Query will be formed and the table will be created
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">
        /// The structure of this xml is as follows.Please refer to the TypeCollection.XSD
        /// <?xml version="1.0" encoding="UTF-8"?>
        ///<TYPE>
        ///<TABLENAME>AL_ENT_Plan</TABLENAME>
        /// 	<FIELDCOLLECTION>
        ///			<FIELD OperationType="NEW">
        ///		 		<FIELDNAME>PlanName</FIELDNAME>
        ///					<FIELDTYPE>STRING</FIELDTYPE>
        ///						<FIELDSIZE>
        ///								30</FIELDSIZE>
        ///								<ISMANDATORY>TRUE</ISMANDATORY>
        ///								<ISPRIMARYKEY>TRUE</ISPRIMARYKEY>
        ///
        ///							</FIELD>
        ///						<FIELD OperationType="NEW">
        ///							<FIELDNAME>Description</FIELDNAME>
        ///							<FIELDTYPE>STRING</FIELDTYPE>
        ///							<FIELDSIZE>230</FIELDSIZE>
        ///					<ISMANDATORY>FALSE</ISMANDATORY>
        ///					<ISPRIMARYKEY>FALSE</ISPRIMARYKEY>
        ///			</FIELD>
        ///	
        ///     </FIELDCOLLECTION>
        ///
        ///</TYPE>
        /// </param>

        /// <returns>
        /// This function returns Error object if there is any error occured.
        /// </returns>
        public override OMError PrepareCreateTableQuery(String InputXML, ref String CreateTableQuery)
        {
            StringBuilder   Query               = null;	    //The query that is formed for the Create table is stored in this variable
            XmlDocument     InputXMLDocument    = null;
            XmlNode         TableNameNode       = null;	    //The table Name which needs to be created is specified in the input XML
            string          DataType            = null;		//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            XmlNodeList     FieldNodes          = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
            XmlNode         TypeNameNode        = null;		//The Type node in the InputXML is saved in this node
            XmlNode         FieldCollectionNode = null;		//This has the FieldCollectionNode which holds all the information about the fields
            XmlNode         FieldNameNode       = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode         FieldTypeNode       = null;		//This variable holds the Datatype node of the individual field node specified in the FieldCollection
            XmlNode         SizeNode            = null;		//This variable holds the Size node of the individual field node specified in the FieldCollection
            XmlNode         MandatoryNode       = null;		//This variable holds Mandatory node which indicates whether the Field node is mandatory or no.If yes then the FieldValue cannot be blank
            XmlNode         PrimaryKeyNode      = null;		//This variable holds Primary Node which indicates whether the Field node is PrimaryKey or no.
            OMError         ErrorObj            = null;		//In case there is an error in the function this object will be populated

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String          XSDSourceFilePath   = null;
            String          ObjectTypeName      = null;
            IXMLRepository  ObjXMLManager       = null;

            try
            {
                Query               = new StringBuilder();
                ObjXMLManager       = OMDataManager.XMLRepository; //new XMLManager(TenantName);
                InputXMLDocument    = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                TypeNameNode        = InputXMLDocument.SelectSingleNode("TYPE");
                TableNameNode       = TypeNameNode.SelectSingleNode("TABLENAME");
                FieldCollectionNode = TypeNameNode.SelectSingleNode("FIELDCOLLECTION");
                FieldNodes          = FieldCollectionNode.SelectNodes("FIELD");
                ObjectTypeName      = TypeNameNode.SelectSingleNode("TYPENAME").InnerText.Trim().ToUpper();

                Query.AppendFormat("CREATE TABLE {0} (", TableNameNode.InnerText.ToUpper());
                foreach (XmlElement FieldNode in FieldNodes)
                {
                    FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                    FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");

                    if ((FieldTypeNode.InnerText.ToUpper() != "XML")
                            && (FieldTypeNode.InnerText.ToUpper() != "BINDER")
                            && (FieldTypeNode.InnerText.ToUpper() != "FILE"))
                    {
                        MandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                        PrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");

                        switch ((FieldTypeNode.InnerText).ToUpper())
                        {
                            case "TEXT":
                                DataType = "NVARCHAR(MAX)";
                                break;
                            case "STRING":
                                DataType = "NVARCHAR";
                                SizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                                break;

                            case "BIGINT":
                            case "INTEGER":
                                DataType = "BIGINT";
                                //SizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                                break;

                            case "BOOLEAN":
                                DataType = "NVARCHAR";
                                SizeNode = InputXMLDocument.CreateElement("FIELDSIZE");
                                SizeNode.InnerText = "1";
                                break;
                            
                            case "FLOAT":
                            case "DECIMAL":
                                DataType = "FLOAT";
                                //Following code is added by Kiran on date 03rd Jan 06.
                                //This is so because the size of FLOAT is taken as 255 if we could not mentioned the following.
                                SizeNode = InputXMLDocument.CreateElement("FIELDSIZE");
                                SizeNode.InnerText = "8";
                                break;

                            case "DATETIME":
                                DataType = "DATETIME";
                                break;

                            //To support "CURRENCY" Data type, Code added on date 14th June 2006 by Pallavi
                            case "CURRENCY":
                                DataType = "CURRENCY";
                                //1] Currency Type
                                Query.AppendFormat("{0}_C NVARCHAR (25)", FieldNameNode.InnerText.ToUpper());
                                if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                    Query.AppendFormat("NOT NULL,");
                                else
                                    Query.AppendFormat("NULL,");

                                // 2] Currency Value
                                Query.AppendFormat("{0}_A MONEY ", FieldNameNode.InnerText.ToUpper());
                                if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                    Query.AppendFormat("NOT NULL,");
                                else
                                    Query.AppendFormat("NULL,");

                                break;
                        }

                        //MandatoryNode   = FieldNode.SelectSingleNode("ISMANDATORY");
                        //PrimaryKeyNode  = FieldNode.SelectSingleNode("ISPRIMARYKEY");

                        if (DataType != "CURRENCY")
                            Query.AppendFormat("{0} {1} ", FieldNameNode.InnerText.ToUpper(), DataType);

                        if ((DataType != "BIGINT") && (DataType != "DATETIME") && (DataType != "CURRENCY") && (DataType != "NVARCHAR(MAX)") && (DataType != "FLOAT") && (DataType != "DECIMAL"))
                            Query.AppendFormat("({0}) ", SizeNode.InnerText);

                        if (DataType != "CURRENCY")
                        {
                            if ((PrimaryKeyNode != null) && (PrimaryKeyNode.InnerText == "TRUE"))
                                Query.AppendFormat("NOT NULL PRIMARY KEY,");
                            else
                            {
                                /*
                                  This code is commented out because Object Manager doesn't enforce any Database constraints. 
                                  Only an entry is done ALOM_SCHEMA for ISMANDATORY constraint.
                                 */
                                //if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                //    Query.AppendFormat("NOT NULL,");
                                //else
                                Query.AppendFormat("NULL,");
                            }
                        }
                    }
                    //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                    //else if ((FieldTypeNode.InnerText.ToUpper() == "XML") && (FieldNameNode != null)
                    //            && (FieldNode.SelectSingleNode("XSDPATH") != null))
                    //{
                    //    //XSDSourceFilePath = FieldTypeNode.Attributes["XSDFilePath"].Value;
                    //    XSDSourceFilePath = FieldNode.SelectSingleNode("XSDPATH").InnerText;
                    //    ErrorObj = ObjXMLManager.CopyXSD(ObjectTypeName, FieldNameNode.InnerText.Trim().ToUpper(), XSDSourceFilePath);
                    //    if (ErrorObj != null)
                    //        return (ErrorObj);
                    //}
                }

                Query = Query.Remove((Query.Length - 1), 1);
                Query.AppendFormat(")");
                CreateTableQuery = Query.ToString();
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.TABLE_CREATION_FAILED,
                                            ExceptionObj.Message, OMErrorObjectType.TABLE,
                                            null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
        }

        public override OMError ExecuteStoredProcedureForRead(String ProcedureName)
        {
            CommandObj.CommandText = ProcedureName;
            CommandObj.CommandType = CommandType.StoredProcedure;
            CommandObj.Connection  = ConnectionObj;
            CommandObj.Transaction = TransactionObj;

            foreach (String ParamName in ParamDictionary.Keys)
            {
                SqlParameter SqlParam = new SqlParameter(ParamName, ParamDictionary[ParamName]);
                CommandObj.Parameters.Add(SqlParam);
            }

            DataReaderObj = (SqlDataReader) CommandObj.ExecuteReader();
            return null;          

        }
        
        /// <summary>
        /// This function is an extension of the CreateRecord method. 
        /// This does a insert based on XML.
        /// </summary>
        /// <param name="EntityTypeName">Name of the Type i.e. EntityType or FacetType</param>
        /// <param name="TableName">This function takes TableName as parameter.This 
        /// specifies which table the record has to be inserted</param>
        /// <param name="InputXML">This function takes InputXML as parameter.
        /// The structure of the XML will be like this
        /// <Plan>
        ///		<PlanName>Plan F'or Tes'ting</PlanName>
        ///		<Description>This is for Testing</Description>
        ///		<Age>25</Age>
        ///		<DOB>
        ///		<Day>3</Day>
        ///		<Month>10</Month>
        ///		<Year>1979</Year>
        ///		</DOB>
        /// </Plan>
        /// </param>
        /// <returns>
        /// This function returns OMError object if there is any error occured.
        /// </returns>
        //public override OMError CreateRecord(String TypeName, String TableName, String InputXML)
        //{
        //    XmlNode DayNode;					//This contains the Day node which specifies the day in the date in InputXML
        //    XmlNode MonthNode;					//This contains the Month node which specifies the Month in the date in InputXML
        //    XmlNode YearNode;					//This contains the Year node which specifies the Year in the date in InputXML
        //    string DataType = "";				//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
        //    string Seperator = "";				//This contains the seperator value like "," which is used for query formation
        //    string FieldName;					//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
        //    string FieldType;					//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
        //    string FieldSize = "";				//This has the field size of the individual field.This is used for truncation of data
        //    XmlDocument InputXMLDocument;			//This Contains the Main Document node of the InputXML
        //    Hashtable FieldNameValueCollection;	//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
        //    XmlElement RootNode;					//Contains the Root node in the InputXML
        //    StringBuilder Query;						//The Insert Query formed is saved in this variable
        //    StringBuilder FieldList;					//The FieldList represents the list of fields.This is used to append in the Insert Query 
        //    StringBuilder FieldValues;				//The FieldValues represents the list of field vlaues.This is used to append in the Insert Query 
        //    StringBuilder Date;						//The Value of the Date is saved in this variable
        //    OMError ErrorObj = null;			//In case there is an error in the function this object will be populated
        //    XmlNode HourNode;					//This contains the Hour node which specifies the Hour in the date in InputXML
        //    XmlNode MinuteNode;					//This contains the Minute node which specifies the Minute in the date in InputXML
        //    XmlNode SecondNode;					//This contains the Second node which specifies the Second in the date in InputXML
        //    StringBuilder QuerySB = null;
        //    FieldInformation FieldDataObj = new FieldInformation(); //This is the Field Object stored in the struct, It contains the value of the FieldName and FieldType
        //    try
        //    {
        //        //StringDictionary FieldNameValueCollection = new StringDictionary(); 

        //        // Needs to change CaseInsensitiveHashCodeProvider to IEqualityComparer

        //        //In this code section the records from the ALOM_Schema is retrieved based on the TableName and kept in the HashTable. 
        //        //Later to determine the DataType of the Field this Hashtable is used.

        //        FieldNameValueCollection = new Hashtable();//new CaseInsensitiveHashCodeProvider(), new CaseInsensitiveComparer());

        //        Query = new StringBuilder();
        //        Query.AppendFormat("Select FIELDNAME,FIELDTYPE,FIELDSIZE FROM ALOM_SCHEMA WHERE TABLENAME ='{0}'", TableName.ToUpper());

        //        ErrorObj = RetrieveResult(Query.ToString());

        //        if (ErrorObj != null)
        //        {
        //            return ErrorObj;
        //        }

        //        while (DataReaderObj.Read())
        //        {
        //            FieldName = DataReaderObj.GetValue(0).ToString();
        //            FieldType = DataReaderObj.GetValue(1).ToString();
        //            FieldSize = DataReaderObj.GetValue(2).ToString();
        //            FieldDataObj.FieldType = FieldType;
        //            FieldDataObj.FieldSize = FieldSize;
        //            FieldNameValueCollection.Add(FieldName.Trim(), FieldDataObj);
        //        }
        //        DataReaderObj.Close();

        //        /*Load the Input XML. Get all the necessary data from the Input XML.Prepare the Insert Query 
        //        based on the DataType of the fields. Convert the Date into Database specific.*/

        //        InputXMLDocument = new XmlDocument();
        //        InputXMLDocument.LoadXml(InputXML);

        //        RootNode = InputXMLDocument.DocumentElement;

        //        QuerySB = new StringBuilder();
        //        QuerySB.AppendFormat("INSERT INTO {0} (", TableName.ToUpper());

        //        FieldList = new StringBuilder();
        //        FieldValues = new StringBuilder();
        //        Date = new StringBuilder();

        //        foreach (XmlElement ChildNode in RootNode.ChildNodes)
        //        {
        //            if (FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
        //            {
        //                FieldList.AppendFormat("{0} {1}", Seperator, (ChildNode.Name).ToUpper());

        //                FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
        //                DataType = FieldDataObj.FieldType;
        //                FieldSize = FieldDataObj.FieldSize;

        //                switch (DataType.ToUpper())
        //                {
        //                    case "STRING":
        //                        if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
        //                        {
        //                            FieldValues.AppendFormat("{0} N'{1}' ", Seperator, TruncateData(ReplaceSingleQuote(ChildNode.InnerText), Convert.ToInt32(FieldSize)));
        //                        }
        //                        break;
        //                    case "BIGINT":
        //                    case "INTEGER":
        //                        if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
        //                        {
        //                            FieldValues.AppendFormat("{0}{1} ", Seperator, Convert.ToString(ChildNode.InnerText));
        //                        }
        //                        else
        //                        {
        //                            if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
        //                            {
        //                                FieldValues.AppendFormat("{0}0 ", Seperator );
        //                            }
        //                        }
        //                        break;
        //                    case "BOOLEAN":
        //                        if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
        //                        {
        //                            FieldValues.AppendFormat("{0} N'{1}' ", Seperator, ChildNode.InnerText);
        //                        }
        //                        else
        //                        {
        //                            if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
        //                            {
        //                                FieldValues.AppendFormat("{0}'FALSE' ", Seperator);
        //                            }
        //                        }
        //                        break;
        //                    case "DECIMAL":
        //                        if (!(ChildNode.InnerText == null) || !(ChildNode.InnerText == ""))
        //                        {
        //                            FieldValues.AppendFormat("{0}{1} ", Seperator, Convert.ToString(ChildNode.InnerText));
        //                        }
        //                        else
        //                        {
        //                            if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
        //                            {
        //                                FieldValues.AppendFormat("{0}0.0 ", Seperator);
        //                            }
        //                        }
        //                        break;
        //                    case "DATETIME":
        //                        DayNode = ChildNode.SelectSingleNode("DAY");
        //                        MonthNode = ChildNode.SelectSingleNode("MONTH");
        //                        YearNode = ChildNode.SelectSingleNode("YEAR");
        //                        HourNode = ChildNode.SelectSingleNode("HOUR");
        //                        MinuteNode = ChildNode.SelectSingleNode("MINUTE");
        //                        SecondNode = ChildNode.SelectSingleNode("SECOND");

        //                        if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
        //                            Date.AppendFormat("{0}/{1}/{2} {3}:{4}:{5}",
        //                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText,
        //                                        HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
        //                        else
        //                            Date.AppendFormat("{0}/{1}/{2}",
        //                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

        //                        FieldValues.AppendFormat("{0}{1} ", Seperator, ConvertDate(Date.ToString()));
        //                        Date.Remove(0, Date.Length);
        //                        break;
        //                }
        //                Seperator = ",";
        //            }
        //        }

        //        QuerySB.AppendFormat("{0}) VALUES ( {1})", FieldList, FieldValues);
        //        //The Execute method takes the Query as parameter and the query is executed.
        //        ErrorObj = Execute(QuerySB.ToString());
        //        if (ErrorObj != null)
        //        {
        //            return ErrorObj;
        //        }
        //        DataReaderObj.Close();
        //        //DataReaderObj = null;
        //        return null;
        //    }
        //    catch (Exception ExceptionObj)
        //    {
        //        DataReaderObj.Close();
        //        ErrorObj = new OMError(OMErrorCodes.CREATE_RECORD_FAILED,
        //                   ExceptionObj.Message, OMErrorObjectType.TABLE,
        //                   null, OMErrorType.EXCEPTION, ExceptionObj, null);
        //        return ErrorObj;
        //    }
        //}
    
    
        /// <summary>This method is used to generate the stored procedures for data population of ODS entities/facets.
        /// This stored procedure resides into MOM database. The stored procedure collects the necessary instances from MOM database
        /// and will then populate into ODS database.
        /// This method also generates the stored procedure to manage the relations of populated objects. The second stored procedure resides
        /// into ODS database. Backoffice service will run these stored procedures sequenctially using StoredProcedures.xml configuration.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="ODSObjectTypeName">Type of ODS object type for which data population stored procedures
        /// will be created into MOM and ODS databases.</param>
        /// <param name="MOMAndODSSpecificInfoXML">This contains the information which requires to generate the stored procedure.
        /// It contains information like ODS and MOM databases names, name of the stored procedures, ODS object type information
        /// and ODS specific target object type information which are used into MOM.</param>
        /// <param name="MOMObjectTypeAndSchemaInfoXML">It contains the schema information for MOM specific object type
        /// which needs to be created into ODS database too with ODS conventions..</param>
        /// <param name="MOMRelationshipCollectionXML">It contains the relevant relationship details which are related to the MOM object type.
        /// The MOM object type is also expected to be created into ODS database with ODS conventions.</param>
        /// <param param name="ODSRelationshipCollectionXML">It contains the relevant relationship details
        /// which are related to the new ODS object type.</param>
        /// <param name="ODSDataPopulationStoredProcedure">The newly created stored procedure which resides into MOM database.
        /// This stored procedure is used to populate the relevant data from MOM to ODS database.</param>
        /// <param name="ODSManageRelationsStoredProcedure">The newly created stored procedure which resides into ODS database.
        /// This stored procedure is used to manage the relations of the populated objects into ODS database.</param>
        /// <returns>On success it will return NULL else an error object will be returned with relevant information.</returns>
        public override OMError PrepareODSPopulationStoredProcedures(String ODSObjectTypeName, 
                String MOMAndODSSpecificInfoXML, String MOMObjectTypeAndSchemaInfoXML,
                String MOMRelationshipCollectionXML, String ODSRelationshipCollectionXML, 
                ref String ODSDataPopulationStoredProcedure, ref String ODSManageRelationsStoredProcedure)
        {
            #region Method Variables

            OMError             omerror_Object              = null;
            XPathDocument       xpathdoc_ObjectTypeInfo     = null;
            XPathDocument       xpathdoc_ODSObjectConfig    = null;
            XPathDocument       xpathdoc_MOMRelationship    = null;
            XPathDocument       xpathdoc_ODSRelationship    = null;
            XPathNavigator      xpathnav_ObjectType         = null;
            XPathNavigator      xpathnav_ODSConfigRoot      = null;
            XPathNavigator      xpathnav_MOMObjectConfig    = null;
            XPathNavigator      xpathnav_ODSObjectConfig    = null;
            XPathNavigator      xpathnav_MOMRelationship    = null;
            XPathNavigator      xpathnav_ODSRelationsColl   = null;
            XPathNavigator      xpathnav_ODSRelationship    = null;
            XPathNavigator      xpathnav_MOMODSObject       = null;
            XPathNavigator      xpathnav_PrimaryFieldNode   = null;
            XPathNodeIterator   xpathiter_Fields            = null;
            XPathNodeIterator   xpathiter_MOMPrimaryFields  = null;
            XPathNodeIterator   xpathiter_PrimaryFieldNodes = null;
            Int32               int_RelatedTypeCount        = 0;
            Int16               int_PrimaryKeyCount         = 0;
            String              str_ObjectType              = null;
            String              str_ObjectTypeID            = null;
            String              str_ObjectTypePrimaryKey    = null;
            String              str_FieldName               = null;
            String              str_FieldType               = null;
            String              str_RelationshipTypeIDs     = null;
            String              str_RelationshipTypeNames   = null;
            String              str_RelationshipName        = null;
            String              str_RelatedObjectAlias      = null;
            String              str_ContainerType           = null;
            String              str_ContainerTypeID         = null;
            String              str_ContainsType            = null;
            String              str_ContainsTypeID          = null;
            String              str_IsParentObject          = null;
            String              str_MOMDatabaseName         = null;
            String              str_ODSDatabaseName         = null;
            String              str_InsertFieldsQuery       = null;
            String              str_SelectedFieldsQuery     = null;
            String              str_FromTableQuery          = null;
            String              str_InnerJoinsQuery         = null;
            String              str_OuterJoinsQuery         = null;
            String              str_ManageRelationsQuery    = null;
            String              str_DeleteOldInstancesQuery = null;
            String              str_DataPopulationSPName    = null;
            String              str_ManageRelationsSPName   = null;
            String              str_AdditinalErrorInfo      = null;
            String              str_ODSTargetTablePrimKey   = null;
            String              str_ODSRelatedTablePrimKey  = null;
            String              str_ODSTargetTablePKVar     = null;
            String              str_ODSRelatedTablePKVar    = null;
            const String        str_MOMRelatedTypeAlias     = "MOMRelatedType";
            String[]            strarr_RelationshipNames    = null;
            Boolean             flag_IsAssociationWithFacet = false;
            Boolean             flag_IsAssociationTableUsed = false;
            Boolean             flag_IsAssemblyWithContainer= false;
            Boolean             flag_IsAssemblyWithContains = false;
            Boolean             flag_IsAssemblyTableUsed    = false; 

            #endregion

            try
            {
                //This is used to display the additional information for an error.
                str_AdditinalErrorInfo = "Error occurred in SQLQueryProcessor.PrepareODSPopulationStoredProcedures() while generating stored procedures.";

                #region Retrieve the necessary information from the passed XMLs

                xpathdoc_ObjectTypeInfo = new XPathDocument(new System.IO.StringReader(MOMObjectTypeAndSchemaInfoXML));
                xpathnav_ObjectType     = xpathdoc_ObjectTypeInfo.CreateNavigator().SelectSingleNode("TYPE");
                xpathiter_Fields        = xpathnav_ObjectType.Select("FIELDCOLLECTION/FIELD");
                if (xpathiter_Fields.Count <= 0)
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "The schema information for MOM object type is NULL or EMPTY.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null,
                            str_AdditinalErrorInfo + " MOM Object's Schema XML: " + MOMObjectTypeAndSchemaInfoXML);
                    return (omerror_Object);
                }

                xpathdoc_ODSObjectConfig    = new XPathDocument(new System.IO.StringReader(MOMAndODSSpecificInfoXML));
                xpathnav_ODSConfigRoot      = xpathdoc_ODSObjectConfig.CreateNavigator();
                xpathnav_MOMObjectConfig    = xpathnav_ODSConfigRoot.SelectSingleNode("ODSINFOXML/MOMOBJECTCONFIGURATION");
                xpathnav_ODSObjectConfig    = xpathnav_ODSConfigRoot.SelectSingleNode(
                        "ODSINFOXML/ODSOBJECTCONFIGURATION/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']");
                str_MOMDatabaseName         = xpathnav_ODSConfigRoot.SelectSingleNode("ODSINFOXML/MOMDATABASENAME").InnerXml;
                str_ODSDatabaseName         = xpathnav_ODSConfigRoot.SelectSingleNode("ODSINFOXML/ODSDATABASENAME").InnerXml;
                str_DataPopulationSPName    = xpathnav_ODSConfigRoot.SelectSingleNode("ODSINFOXML/DATAPOPULATIONSTOREDPROCEDURENAME").InnerXml;
                str_ManageRelationsSPName   = xpathnav_ODSConfigRoot.SelectSingleNode("ODSINFOXML/MANAGERELATIONSSTOREDPROCEDURENAME").InnerXml;

                #endregion

                #region STORED PROCEDURE1: GENERATE STORED PROCEDURE TO MANAGE ODS OBJECT DATA POPULATION

                #region Step01: Generate The INSERT and SELECT queries for the ODS object type table

                //Prepare the INERT query.
                str_InsertFieldsQuery = " INSERT INTO " + str_ODSDatabaseName + ".DBO."
                        + xpathnav_ODSObjectConfig.GetAttribute("ODSObjectTypeTableName", "") + "(";

                //Prepare the SELECT query.
                str_SelectedFieldsQuery = " SELECT ";

                foreach (XPathNavigator xpathnav_Field in xpathiter_Fields)
                {
                    str_FieldType = xpathnav_Field.SelectSingleNode("FIELDTYPE").InnerXml;
                    switch (str_FieldType)
                    {
                        //Ignore the following types of fields. This is as per discussion with Hemant Kirad.
                        case "BINDER":
                        case "FILE":
                        case "XML":
                        case "XMLTEXT":
                            break;

                        case "CURRENCY":
                            str_FieldName = xpathnav_Field.SelectSingleNode("FIELDNAME").InnerXml;

                            //Query will be like- "MOMObjectType.UNALLOCATED_C, MOMObjectType.UNALLOCATED_A, ".
                            str_SelectedFieldsQuery += "MOMObjectType." + str_FieldName + "_C, " + "MOMObjectType." + str_FieldName + "_A, ";

                            //Query will be like- "UNALLOCATED_C, UNALLOCATED_A, ".
                            str_InsertFieldsQuery += str_FieldName + "_C, " + str_FieldName + "_A, ";
                            break;

                        default:
                            str_FieldName = xpathnav_Field.SelectSingleNode("FIELDNAME").InnerXml;

                            //Query will be like- "MOMObjectType.MAPKID, MOMObjectType.NOTES, ".
                            str_SelectedFieldsQuery += "MOMObjectType." + str_FieldName + ", ";

                            //Query will be like- "MAPKID, NOTES, ".
                            str_InsertFieldsQuery += str_FieldName + ", ";

                            break;
                    }                    
                }

                //Query will be like- " FROM AL60R13_25Apr11.DBO.ALFCT_MADETAILS as MOMObjectType".
                str_FromTableQuery = " FROM " + str_MOMDatabaseName + ".DBO." 
                        + xpathnav_ObjectType.SelectSingleNode("TABLENAME").InnerXml + " AS MOMObjectType";

                #endregion

                #region Step02: Generate The Queries By Reading The Relations From ODS Configuration (related object type's primary key fields)

                //Add primary key fields in selection query.
                xpathiter_MOMPrimaryFields = xpathnav_ODSObjectConfig.Select("MOMPrimaryKeyAsODSFields/Field");
                //if (xpathiter_MOMPrimaryFields.Count > 0)
                {
                    xpathdoc_MOMRelationship    = new XPathDocument(new System.IO.StringReader(MOMRelationshipCollectionXML));
                    str_ObjectType              = xpathnav_ODSObjectConfig.GetAttribute("ObjectType", "");
                    str_ObjectTypePrimaryKey    = xpathnav_ObjectType.SelectSingleNode("PRIMARYKEY").InnerXml;
                    str_ObjectTypeID            = xpathnav_ObjectType.SelectSingleNode("TYPEID").InnerXml;

                    switch (str_ObjectType.ToUpper())
                    {
                        //Related object type is ENTITY.
                        case "ENTITY":
                            break;

                        //Related object type is FACET.
                        case "FACET":
                            //Delete the non-required instances (previous entries) from the ODS object type table.
                            str_DeleteOldInstancesQuery = @" TRUNCATE TABLE "
                                    + str_ODSDatabaseName + ".DBO." + xpathnav_ODSObjectConfig.GetAttribute("ODSObjectTypeTableName", "");

                            str_RelationshipTypeNames = "";
                            //Navigate all relations which exist in ODS schema configuration.
                            foreach (XPathNavigator xpathnav_MOMPrimaryKeyField in xpathiter_MOMPrimaryFields)
                            {
                                #region Step02 (a): Add Related Type Fields (Primary Keys) To The INSERT and SELECT Queries

                                //Get the relationship name from configuration and it should not be NULL or EMPTY. 
                                str_RelationshipName = xpathnav_MOMPrimaryKeyField.GetAttribute("MOMRelationshipName", "");
                                if (String.IsNullOrEmpty(str_RelationshipName))
                                    continue;

                                //Check whether the relationship is present in the passed collection XML.
                                //If not then continue with next relationshiptype.
                                xpathnav_MOMRelationship = xpathdoc_MOMRelationship.CreateNavigator().SelectSingleNode(
                                        "RELATIONSHIPCOLLECTION/RELATIONSHIPTYPE[RELATIONSHIPNAME='" + str_RelationshipName + "']");
                                if (xpathnav_MOMRelationship == null)
                                    continue;

                                //Duplicate MOM relationshipnames are avoided.
                                if (str_RelationshipTypeNames.Contains(str_RelationshipName + ", "))
                                    continue;

                                int_RelatedTypeCount++;
                                str_RelatedObjectAlias      = str_MOMRelatedTypeAlias + int_RelatedTypeCount.ToString();
                                str_RelationshipTypeNames   += str_RelationshipName + ", ";

                                //Query will be like- ", MOMRelatedType1.MARKETINGACTIVITYID, ".
                                str_SelectedFieldsQuery += str_RelatedObjectAlias + "."
                                        + xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypePrimaryKey", "") + ", ";

                                //Query will be like- "FIELD1, FIELD2, ".
                                str_InsertFieldsQuery += xpathnav_MOMPrimaryKeyField.InnerXml + ", ";

                                #endregion

                                #region Step02 (b): Generate The OUTER JOIN Queries With The Related Object Type Tables

                                switch (xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectType", "").ToUpper())
                                {
                                    //If the target object type is an entity then the join with ALOM_ASSOENTITYFACET.
                                    case "ENTITY":
                                        //Query will be like- "LEFT OUTER JOIN AL60R13_25Apr11.DBO.ALENT_MARKETINGACTIVITY MOMRelatedType1 ON 
                                        // MOMRelatedType1.MARKETINGACTIVITYID = MOMAssociations.ENTITYID"
                                        str_OuterJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName + ".DBO."
                                                + xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypeTableName", "") + " "
                                                + str_RelatedObjectAlias + " ON "
                                                + str_RelatedObjectAlias + "." + xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypePrimaryKey", "")
                                                + " = MOMAssociations.ENTITYID";
                                        break;

                                    //If the target object type is facet then the join with ALOM_ASSEMBLY.
                                    case "FACET":
                                        //Query will be like- "LEFT OUTER JOIN AL60R13_25Apr11.DBO.ALENT_MARKETINGACTIVITY MOMRelatedType1 ON 
                                        // MOMRelatedType1.MARKETINGACTIVITYID = MOMAssociations.CONTAINERID"
                                        str_OuterJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName + ".DBO."
                                                + xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypeTableName", "") + " "
                                                + str_RelatedObjectAlias + " ON "
                                                + str_RelatedObjectAlias + "." + xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypePrimaryKey", "");

                                        str_IsParentObject = xpathnav_MOMPrimaryKeyField.GetAttribute("IsParentObject", "");
                                        if ((!(String.IsNullOrEmpty(str_IsParentObject))) && (str_IsParentObject.Equals("True", StringComparison.CurrentCultureIgnoreCase)))
                                        {
                                            //If attribute "IsParentObject" contains value "TRUE",
                                            // then the target object is CONTAINER and the ODS object is CONTAINS.
                                            //Here MOMAssemblies is used because relation is in-between FACET-FACET.
                                            str_OuterJoinsQuery += " = MOMAssemblies.CONTAINERID";
                                        }
                                        else
                                        {
                                            //If attribute "IsParentObject" contains value "FALSE",
                                            // then the target object is CONTAINS and the ODS object is CONTAINER.
                                            //Here MOMAssemblies is used because relation is in-between FACET-FACET.
                                            str_OuterJoinsQuery += " = MOMAssemblies.CONTAINSID";
                                        }
                                        break;
                                }

                                #endregion
                            }

                            str_SelectedFieldsQuery = str_SelectedFieldsQuery.Remove(str_SelectedFieldsQuery.LastIndexOf(","));
                            str_InsertFieldsQuery   = str_InsertFieldsQuery.Remove(str_InsertFieldsQuery.LastIndexOf(",")) + ")";

                            #region Step02 (c): Generate The OUTER JOIN Queries With The ASSEMBLY Or ASSOENTITYFACET Tables

                            //Check whether the ODS relationship XML contains the ODS specific relations or not.
                            xpathdoc_ODSRelationship = new XPathDocument(new System.IO.StringReader(ODSRelationshipCollectionXML));
                            xpathnav_ODSRelationsColl = xpathdoc_ODSRelationship.CreateNavigator().SelectSingleNode("RELATIONSHIPCOLLECTION");
                            if (xpathnav_ODSRelationsColl == null)
                            {
                                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                        "The ODS relationship collection is NULL or EMPTY.",
                                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null,
                                        str_AdditinalErrorInfo + " MOM Object's Schema XML: " + MOMObjectTypeAndSchemaInfoXML);
                                return (omerror_Object);
                            }

                            str_ManageRelationsQuery = "DECLARE @CNT BIGINT";

                            //Iterate the relations and generate the related queries.
                            if (!(String.IsNullOrEmpty(str_RelationshipTypeNames)))
                            {
                                str_RelationshipTypeNames   = str_RelationshipTypeNames.Replace(" ", "");
                                str_RelationshipTypeNames   = str_RelationshipTypeNames.Remove(str_RelationshipTypeNames.LastIndexOf(","));
                                strarr_RelationshipNames    = str_RelationshipTypeNames.Split(',');
                                foreach (String str_RelationshipTypeName in strarr_RelationshipNames)
                                {
                                    xpathnav_MOMRelationship = xpathdoc_MOMRelationship.CreateNavigator().SelectSingleNode(
                                            "RELATIONSHIPCOLLECTION/RELATIONSHIPTYPE[RELATIONSHIPNAME='" + str_RelationshipTypeName + "']");
                                    if (xpathnav_MOMRelationship != null)
                                    {
                                        str_RelationshipTypeIDs += xpathnav_MOMRelationship.SelectSingleNode("RELATIONSHIPTYPEID").InnerXml + ", ";
                                        str_ContainerType       = xpathnav_MOMRelationship.SelectSingleNode("CONTAINERTYPE").InnerXml;
                                        str_ContainerTypeID     = xpathnav_MOMRelationship.SelectSingleNode("CONTAINERTYPEID").InnerXml;
                                        str_ContainsType        = xpathnav_MOMRelationship.SelectSingleNode("CONTAINSTYPE").InnerXml;
                                        str_ContainsTypeID      = xpathnav_MOMRelationship.SelectSingleNode("CONTAINSTYPEID").InnerXml;

                                        #region Case1: Container Type is Entity and Contains Type is Facet

                                        //Case1: Container Type is Entity and Contains Type is Facet. Conditions are- 
                                        // (a) The AssoEntityFacet flag value should be FALSE. Only first occurence is considered.
                                        // (b) Container Type should be ENTITY.
                                        // (c) Contains Type should be FACET.
                                        // (d) Contains Type ID should be Object Type ID.
                                        if ((str_ContainerType.Equals("ENTITY", StringComparison.CurrentCultureIgnoreCase))
                                                && (str_ContainsType.Equals("FACET", StringComparison.CurrentCultureIgnoreCase))
                                                && (str_ContainsTypeID.Equals(str_ObjectTypeID)))
                                        {
                                            if (flag_IsAssociationWithFacet == false)
                                            {
                                                //Query will be like- " LEFT OUTER JOIN AL60R13_25Apr11.DBO.ALOM_ASSOENTITYFACET MOMAssociations ON 
                                                // MOMAssociations.FACETID = MOMObjectType.MAPKID"
                                                str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName
                                                        + ".DBO.ALOM_ASSOENTITYFACET MOMAssociations ON "
                                                        + "MOMAssociations.FACETID = MOMObjectType." + str_ObjectTypePrimaryKey;
                                                flag_IsAssociationWithFacet = true;
                                                flag_IsAssociationTableUsed = true;
                                            }

                                            #region STORED PROCEDURE2: GENERATE STORED PROCEDURE TO MANAGE ASSEMBLY/ASSOENTITYFACET IN ODS DATABASE
                                            
                                            xpathiter_PrimaryFieldNodes = xpathnav_ODSObjectConfig.Select(
                                                    "MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + str_RelationshipTypeName + "']");
                                            if (xpathiter_PrimaryFieldNodes.Count == 0)
                                                continue;

                                            foreach (XPathNavigator xpathnav_FieldNode in xpathiter_PrimaryFieldNodes)
                                            {
                                                xpathnav_ODSRelationship = xpathnav_ODSRelationsColl.SelectSingleNode(
                                                        "RELATIONSHIPTYPE[RELATIONSHIPNAME='" + xpathnav_FieldNode.GetAttribute("ODSRelationshipName", "") + "']");
                                                if (xpathnav_ODSRelationship == null)
                                                {
                                                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                                            "The ODS relationship does not exist. Relationship name is "
                                                                + xpathnav_FieldNode.GetAttribute("ODSRelationshipName", ""),
                                                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null,
                                                            str_AdditinalErrorInfo + " MOM Object's Schema XML: " + MOMObjectTypeAndSchemaInfoXML);
                                                    return (omerror_Object);
                                                }

                                                #region SQL Stored Procedure- Manage Relations

                                                //*Note: Following query is for two relations.
                                                //ALTER PROCEDURE [dbo].[ODSMACOSTINFO_ManageRelations_SP] AS 

                                                //DECLARE @CNT BIGINT DECLARE @MA_DETAILSID1 BIGINT, @MACOSTINFOID2 BIGINT 
                                                //SET @CNT = 0
                                                //SELECT @CNT = (SELECT MAX(ASSOCIATIONID) FROM AL60R13_30May11_ODS.DBO.ALOM_ASSOENTITYFACET)
                                                //IF (@CNT IS NULL) SET @CNT = 91000000000

                                                //DECLARE C1 CURSOR FOR 
                                                //    SELECT DISTINCT MA_DETAILSID, MACOSTINFOID FROM AL60R13_30May11_ODS.DBO.ALENT_ODSMARKETINGACTIVITY AS TargetObjectType 
                                                //    INNER JOIN AL60R13_30May11_ODS.DBO.ALFCT_ODSMACOSTINFO ODSObjectType  ON ODSObjectType.FIELD1 = TargetObjectType.MA_ID 
                                                //OPEN C1 FETCH NEXT FROM C1 INTO @MA_DETAILSID1, @MACOSTINFOID2 WHILE @@FETCH_STATUS = 0 
                                                //    BEGIN 
                                                //        SELECT @CNT = @CNT + 1  
                                                //        INSERT INTO ALOM_ASSOENTITYFACET (ASSOCIATIONID, ENTITYID, ENTITYTYPEID, FACETID, FACETTYPEID, PARENTID, RELATIONSHIPTYPEID)
                                                //            VALUES ( @CNT, @MA_DETAILSID1, 269394584766183, @MACOSTINFOID2, 132314398570915, 0, 162020752706083)  
                                                		
                                                //        FETCH NEXT FROM C1 INTO @MA_DETAILSID1, @MACOSTINFOID2 
                                                //    END 
                                                //CLOSE C1 DEALLOCATE C1 

                                                //DECLARE @PLAN_DETAILSID3 BIGINT, @MACOSTINFOID4 BIGINT 
                                                //SET @CNT = 0
                                                //SELECT @CNT = (SELECT MAX(ASSOCIATIONID) FROM AL60R13_30May11_ODS.DBO.ALOM_ASSOENTITYFACET)
                                                //IF (@CNT IS NULL) SET @CNT = 91000000000
                                                //DECLARE C1 CURSOR FOR 
                                                //    SELECT DISTINCT PLAN_DETAILSID, MACOSTINFOID FROM AL60R13_30May11_ODS.DBO.ALENT_ODSPLANDETAILS AS TargetObjectType 
                                                //    INNER JOIN AL60R13_30May11_ODS.DBO.ALFCT_ODSMACOSTINFO ODSObjectType  ON ODSObjectType.FIELD2 = TargetObjectType.PLANID 
                                                //OPEN C1 FETCH NEXT FROM C1 INTO @PLAN_DETAILSID3, @MACOSTINFOID4 WHILE @@FETCH_STATUS = 0 
                                                //    BEGIN 
                                                //        SELECT @CNT = @CNT + 1  
                                                //        INSERT INTO ALOM_ASSOENTITYFACET (ASSOCIATIONID, ENTITYID, ENTITYTYPEID, FACETID, FACETTYPEID, PARENTID, RELATIONSHIPTYPEID)
                                                //            VALUES ( @CNT, @PLAN_DETAILSID3, 183405736402523, @MACOSTINFOID4, 132314398570915, 0, 262039138248721)  
                                                //        FETCH NEXT FROM C1 INTO @PLAN_DETAILSID3, @MACOSTINFOID4 
                                                //    END 
                                                //CLOSE C1 DEALLOCATE C1 
                                                
                                                #endregion

                                                //Get the primary key field names of target and related tables.
                                                //If the entity contains (like ODSWORKFLOW) different types of objects 
                                                //(like PROJECT, DELIVERABLE and TASK) then select the specific type.
                                                if (!String.IsNullOrEmpty(xpathnav_FieldNode.GetAttribute("ODSRelationshipFor", "")))
                                                {
                                                    xpathnav_MOMODSObject = xpathnav_MOMObjectConfig.SelectSingleNode(
                                                            "MOMObject[@MOMObjectTypeName='" + xpathnav_FieldNode.GetAttribute("MOMObjectTypeName", "")
                                                            + "']/ODSObject[@ODSRelationshipFor='"
                                                            + xpathnav_FieldNode.GetAttribute("ODSRelationshipFor", "") + "']");
                                                }
                                                else
                                                {
                                                    xpathnav_MOMODSObject = xpathnav_MOMObjectConfig.SelectSingleNode(
                                                            "MOMObject[@MOMObjectTypeName='" + xpathnav_FieldNode.GetAttribute("MOMObjectTypeName", "") 
                                                            + "']/ODSObject");
                                                }
                                                if (xpathnav_MOMODSObject == null)
                                                    continue;
                                                
                                                str_ODSTargetTablePrimKey   = xpathnav_MOMODSObject.SelectSingleNode("ODSPrimaryKey").InnerXml;
                                                str_ODSRelatedTablePrimKey  = xpathnav_ObjectType.SelectSingleNode("PRIMARYKEY").InnerXml;
                                                str_ODSTargetTablePKVar     = str_ODSTargetTablePrimKey + (++int_PrimaryKeyCount).ToString();
                                                str_ODSRelatedTablePKVar    = str_ODSRelatedTablePrimKey + (++int_PrimaryKeyCount).ToString();

                                                //Generate ASSOENTITYFACET table population query.
                                                str_ManageRelationsQuery += " DECLARE @" + str_ODSTargetTablePKVar
                                                        + " BIGINT, @" + str_ODSRelatedTablePKVar + " BIGINT";

                                                str_ManageRelationsQuery += @" SET @CNT = 0
                                                    SELECT @CNT = (SELECT MAX(ASSOCIATIONID) FROM " + str_ODSDatabaseName + @".DBO.ALOM_ASSOENTITYFACET)
                                                    IF (@CNT IS NULL) SET @CNT = 91000000000
                                                    DECLARE C1 CURSOR FOR 
                                                    SELECT DISTINCT " + str_ODSTargetTablePrimKey + ", " + str_ODSRelatedTablePrimKey
                                                        + " FROM " + str_ODSDatabaseName + ".DBO."
                                                        + xpathnav_MOMODSObject.GetAttribute("ODSObjectTableName", "") + " AS TargetObjectType"
                                                        + " INNER JOIN " + str_ODSDatabaseName + ".DBO."
                                                        + xpathnav_ODSObjectConfig.GetAttribute("ODSObjectTypeTableName", "") + " ODSObjectType "
                                                        + " ON ODSObjectType." + xpathnav_FieldNode.InnerXml
                                                        + " = TargetObjectType." + xpathnav_MOMODSObject.SelectSingleNode("MOMPrimaryKeyAsODSField").InnerXml
                                                        + " OPEN C1 FETCH NEXT FROM C1 INTO @" + str_ODSTargetTablePKVar + ", @" + str_ODSRelatedTablePKVar
                                                        + " WHILE @@FETCH_STATUS = 0 BEGIN SELECT @CNT = @CNT + 1 ";

                                                str_ManageRelationsQuery += @" INSERT INTO ALOM_ASSOENTITYFACET
                                                    (ASSOCIATIONID, ENTITYID, ENTITYTYPEID, FACETID, FACETTYPEID, PARENTID, RELATIONSHIPTYPEID)
                                                    VALUES ( @CNT, @" + str_ODSTargetTablePKVar
                                                        + ", " + xpathnav_ODSRelationship.SelectSingleNode("CONTAINERTYPEID").InnerXml
                                                        + ", @" + str_ODSRelatedTablePKVar
                                                        + ", " + xpathnav_ODSRelationship.SelectSingleNode("CONTAINSTYPEID").InnerXml
                                                        + ", 0, " + xpathnav_ODSRelationship.SelectSingleNode("RELATIONSHIPTYPEID").InnerXml
                                                        + @")  FETCH NEXT FROM C1 INTO @" + str_ODSTargetTablePKVar + ", @" + str_ODSRelatedTablePKVar
                                                        + " END CLOSE C1 DEALLOCATE C1";
                                            }
                                            #endregion
                                        }

                                        #endregion

                                        #region Case2: Container Type is Facet and Contains Type is Facet

                                        //Case2: Container Type is Facet and Contains Type is Facet. Conditions are- 
                                        // (a) Container Type should be FACET.
                                        // (b) Contains Type should be FACET.
                                        // (c) Container Type ID should be Object Type ID.
                                        // OR (c) Contains Type ID should be Object Type ID.
                                        else if ((str_ContainerType.Equals("FACET", StringComparison.CurrentCultureIgnoreCase))
                                                && (str_ContainsType.Equals("FACET", StringComparison.CurrentCultureIgnoreCase)))
                                        {
                                            if ((flag_IsAssemblyWithContainer == false) && (str_ContainerTypeID.Equals(str_ObjectTypeID)))
                                            {
                                                //Query will be like- " LEFT OUTER JOIN AL60R13_25Apr11.DBO.ALOM_ASSEMBLY MOMAssemblies ON 
                                                // MOMAssemblies.CONTAINERID = MOMObjectType.MAPKID"
                                                str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName 
                                                        + ".DBO.ALOM_ASSEMBLY MOMAssemblies ON "
                                                        + "MOMAssemblies.CONTAINERID = MOMObjectType." + str_ObjectTypePrimaryKey;
                                                flag_IsAssemblyWithContainer = true;
                                                flag_IsAssemblyTableUsed = true;
                                            }
                                            else if ((flag_IsAssemblyWithContains == false) && (str_ContainsTypeID.Equals(str_ObjectTypeID)))
                                            {
                                                //Query will be like- " LEFT OUTER JOIN AL60R13_25Apr11.DBO.ALOM_ASSEMBLY MOMAssemblies ON 
                                                // MOMAssemblies.CONTAINSID = MOMObjectType.MAPKID"
                                                str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName 
                                                        + ".DBO.ALOM_ASSEMBLY MOMAssemblies ON "
                                                        + "MOMAssemblies.CONTAINSID = MOMObjectType." + str_ObjectTypePrimaryKey;
                                                flag_IsAssemblyWithContains = true;
                                                flag_IsAssemblyTableUsed = true;
                                            }

                                            #region STORED PROCEDURE2: GENERATE STORED PROCEDURE TO MANAGE ASSEMBLY/ASSOENTITYFACET IN ODS DATABASE

                                            if (flag_IsAssemblyWithContains == false)
                                                continue;

                                            xpathnav_PrimaryFieldNode = xpathnav_ODSObjectConfig.SelectSingleNode(
                                                    "MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + str_RelationshipTypeName + "']");
                                            if (xpathnav_PrimaryFieldNode == null)
                                                continue;

                                            xpathnav_ODSRelationship = xpathnav_ODSRelationsColl.SelectSingleNode(
                                                    "RELATIONSHIPTYPE[RELATIONSHIPNAME='" + xpathnav_PrimaryFieldNode.GetAttribute("ODSRelationshipName", "") + "']");
                                            if (xpathnav_ODSRelationship == null)
                                            {
                                                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                                        "The ODS relationship does not exist. Relationship name is "
                                                            + xpathnav_PrimaryFieldNode.GetAttribute("ODSRelationshipName", ""),
                                                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null,
                                                        str_AdditinalErrorInfo + " MOM Object's Schema XML: " + MOMObjectTypeAndSchemaInfoXML);
                                                return (omerror_Object);
                                            }

                                            #region SQL Stored Procedure- Manage Relations

                                            //*Note: Following query is for two relations and for ASSOENTITYFACET.
                                            //For ASSEMBLY only insert parameters are different.
                                            //ALTER PROCEDURE [dbo].[ODSMACOSTINFO_ManageRelations_SP] AS 

                                            //DECLARE @CNT BIGINT DECLARE @MA_DETAILSID1 BIGINT, @MACOSTINFOID2 BIGINT 
                                            //SET @CNT = 0
                                            //SELECT @CNT = (SELECT MAX(ASSOCIATIONID) FROM AL60R13_30May11_ODS.DBO.ALOM_ASSOENTITYFACET)
                                            //IF (@CNT IS NULL) SET @CNT = 91000000000

                                            //DECLARE C1 CURSOR FOR 
                                            //    SELECT DISTINCT MA_DETAILSID, MACOSTINFOID FROM AL60R13_30May11_ODS.DBO.ALENT_ODSMARKETINGACTIVITY AS TargetObjectType 
                                            //    INNER JOIN AL60R13_30May11_ODS.DBO.ALFCT_ODSMACOSTINFO ODSObjectType  ON ODSObjectType.FIELD1 = TargetObjectType.MA_ID 
                                            //OPEN C1 FETCH NEXT FROM C1 INTO @MA_DETAILSID1, @MACOSTINFOID2 WHILE @@FETCH_STATUS = 0 
                                            //    BEGIN 
                                            //        SELECT @CNT = @CNT + 1  
                                            //        INSERT INTO ALOM_ASSOENTITYFACET (ASSOCIATIONID, ENTITYID, ENTITYTYPEID, FACETID, FACETTYPEID, PARENTID, RELATIONSHIPTYPEID)
                                            //            VALUES ( @CNT, @MA_DETAILSID1, 269394584766183, @MACOSTINFOID2, 132314398570915, 0, 162020752706083)  

                                            //        FETCH NEXT FROM C1 INTO @MA_DETAILSID1, @MACOSTINFOID2 
                                            //    END 
                                            //CLOSE C1 DEALLOCATE C1 

                                            //DECLARE @PLAN_DETAILSID3 BIGINT, @MACOSTINFOID4 BIGINT 
                                            //SET @CNT = 0
                                            //SELECT @CNT = (SELECT MAX(ASSOCIATIONID) FROM AL60R13_30May11_ODS.DBO.ALOM_ASSOENTITYFACET)
                                            //IF (@CNT IS NULL) SET @CNT = 91000000000
                                            //DECLARE C1 CURSOR FOR 
                                            //    SELECT DISTINCT PLAN_DETAILSID, MACOSTINFOID FROM AL60R13_30May11_ODS.DBO.ALENT_ODSPLANDETAILS AS TargetObjectType 
                                            //    INNER JOIN AL60R13_30May11_ODS.DBO.ALFCT_ODSMACOSTINFO ODSObjectType  ON ODSObjectType.FIELD2 = TargetObjectType.PLANID 
                                            //OPEN C1 FETCH NEXT FROM C1 INTO @PLAN_DETAILSID3, @MACOSTINFOID4 WHILE @@FETCH_STATUS = 0 
                                            //    BEGIN 
                                            //        SELECT @CNT = @CNT + 1  
                                            //        INSERT INTO ALOM_ASSOENTITYFACET (ASSOCIATIONID, ENTITYID, ENTITYTYPEID, FACETID, FACETTYPEID, PARENTID, RELATIONSHIPTYPEID)
                                            //            VALUES ( @CNT, @PLAN_DETAILSID3, 183405736402523, @MACOSTINFOID4, 132314398570915, 0, 262039138248721)  
                                            //        FETCH NEXT FROM C1 INTO @PLAN_DETAILSID3, @MACOSTINFOID4 
                                            //    END 
                                            //CLOSE C1 DEALLOCATE C1 

                                            #endregion

                                            //Check the facet object type into ODS Object nodes instead of MOM object nodes.
                                            //This is done so because the relationship is in between FACET-FACET.
                                            xpathnav_MOMODSObject = xpathnav_ODSConfigRoot.SelectSingleNode(
                                                    "ODSINFOXML/ODSOBJECTCONFIGURATION/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeName='"
                                                    + xpathnav_PrimaryFieldNode.GetAttribute("MOMObjectTypeName", "") + "']");
                                            if (xpathnav_MOMODSObject == null)
                                                continue;

                                            str_ODSTargetTablePrimKey   = xpathnav_MOMODSObject.GetAttribute("ODSPrimaryKey", "");
                                            str_ODSRelatedTablePrimKey  = xpathnav_ObjectType.SelectSingleNode("PRIMARYKEY").InnerXml;
                                            str_ODSTargetTablePKVar     = str_ODSTargetTablePrimKey + (++int_PrimaryKeyCount).ToString();
                                            str_ODSRelatedTablePKVar    = str_ODSRelatedTablePrimKey + (++int_PrimaryKeyCount).ToString();

                                            //Generate ASSOENTITYFACET table population query.
                                            str_ManageRelationsQuery += " DECLARE @" + str_ODSTargetTablePKVar
                                                    + " BIGINT, @" + str_ODSRelatedTablePKVar + " BIGINT";

                                            str_ManageRelationsQuery += @" SET @CNT = 0
                                                SELECT @CNT = (SELECT MAX(ASSEMBLYID) FROM " + str_ODSDatabaseName + @".DBO.ALOM_ASSEMBLY)
                                                IF (@CNT IS NULL) SET @CNT = 91000000000
                                                DECLARE C1 CURSOR FOR 
                                                SELECT DISTINCT " + str_ODSTargetTablePrimKey + ", " + str_ODSRelatedTablePrimKey
                                                    + " FROM " + str_ODSDatabaseName + ".DBO."
                                                    + xpathnav_MOMODSObject.GetAttribute("ODSObjectTypeTableName", "") + " AS TargetObjectType"
                                                    + " INNER JOIN " + str_ODSDatabaseName + ".DBO."
                                                    + xpathnav_ODSObjectConfig.GetAttribute("ODSObjectTypeTableName", "") + " ODSObjectType "
                                                    + " ON ODSObjectType." + xpathnav_PrimaryFieldNode.InnerXml
                                                    + " = TargetObjectType." + str_ODSTargetTablePrimKey
                                                    + " OPEN C1 FETCH NEXT FROM C1 INTO @" + str_ODSTargetTablePKVar + ", @" + str_ODSRelatedTablePKVar
                                                    + " WHILE @@FETCH_STATUS = 0 BEGIN SELECT @CNT = @CNT + 1 ";

                                            str_ManageRelationsQuery += @" INSERT INTO ALOM_ASSEMBLY
                                                (ASSEMBLYID, CONTAINERID, CONTAINERTYPEID, CONTAINSID, CONTAINSTYPEID, TYPE, RELATIONSHIPTYPE, RELATIONSHIPTYPEID)
                                                VALUES ( @CNT, @" + str_ODSTargetTablePKVar
                                                    + ", " + xpathnav_ODSRelationship.SelectSingleNode("CONTAINERTYPEID").InnerXml
                                                    + ", @" + str_ODSRelatedTablePKVar
                                                    + ", " + xpathnav_ODSRelationship.SelectSingleNode("CONTAINSTYPEID").InnerXml
                                                    + ", 'FACET', '" + xpathnav_ODSRelationship.SelectSingleNode("RELATIONSHIPTYPE").InnerXml
                                                    + "', " + xpathnav_ODSRelationship.SelectSingleNode("RELATIONSHIPTYPEID").InnerXml
                                                    + @")  FETCH NEXT FROM C1 INTO @" + str_ODSTargetTablePKVar + ", @" + str_ODSRelatedTablePKVar
                                                    + " END CLOSE C1 DEALLOCATE C1";

                                            #endregion
                                        }

                                        #endregion

                                        #region Case3/Case4: Container Type is Entity and Contains Type is Entity

                                        //Case3: Container Type is Entity and Contains Type is Entity.
                                        //This case is not valid here because obbject type is FACET.

                                        //Case4: Container Type is Entity and Contains Type is Entity and same oject is used for both.
                                        //This case is not valid here because obbject type is FACET.

                                        #endregion

                                        #region Case5: Container Type is Facet and Contains Type is Facet and same oject is used for both

                                        //**** CODE PENDING
                                        //Case5: Container Type is Facet and Contains Type is Facet and same oject is used for both.
                                        //Update the IF condition (flag_....) for the current FOR loop with case5 variable.

                                        #endregion
                                    }
                                }
                            }

                            #endregion

                            //Query will be like- "INNER JOIN AL60R13_25Apr11.DBO.ALOM_RELATIONSHIPTYPES MOMRelationshipTypes ON 
                            // MOMAssociations.RELATIONSHIPTYPEID = MOMRelationshipTypes.RELATIONSHIPTYPEID
                            // AND MOMRelationshipTypes.RELATIONSHIPNAME IN ('MARKETINGACTIVITY_MADETAILS')".
                            if (!(String.IsNullOrEmpty(str_RelationshipTypeIDs)))
                            {
                                str_RelationshipTypeIDs = str_RelationshipTypeIDs.Replace(" ", "");
                                str_RelationshipTypeIDs = str_RelationshipTypeIDs.Remove(str_RelationshipTypeIDs.LastIndexOf(","));
                                if ((flag_IsAssociationTableUsed) && (flag_IsAssemblyTableUsed))
                                {
                                    str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName
                                            + ".DBO.ALOM_RELATIONSHIPTYPES MOMRelationshipTypes ON "
                                            + " MOMAssociations.RELATIONSHIPTYPEID = MOMRelationshipTypes.RELATIONSHIPTYPEID"
                                            + " OR MOMAssemblies.RELATIONSHIPTYPEID = MOMRelationshipTypes.RELATIONSHIPTYPEID"
                                            + " AND MOMRelationshipTypes.RELATIONSHIPTYPEID IN (" + str_RelationshipTypeIDs + ")";
                                }
                                else if (flag_IsAssociationTableUsed)
                                {
                                    str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName
                                            + ".DBO.ALOM_RELATIONSHIPTYPES MOMRelationshipTypes ON "
                                            + " MOMAssociations.RELATIONSHIPTYPEID = MOMRelationshipTypes.RELATIONSHIPTYPEID"
                                            + " AND MOMRelationshipTypes.RELATIONSHIPTYPEID IN (" + str_RelationshipTypeIDs + ")";
                                }
                                else if (flag_IsAssemblyTableUsed)
                                {
                                    str_InnerJoinsQuery += " LEFT OUTER JOIN " + str_MOMDatabaseName
                                            + ".DBO.ALOM_RELATIONSHIPTYPES MOMRelationshipTypes ON "
                                            + " MOMAssemblies.RELATIONSHIPTYPEID = MOMRelationshipTypes.RELATIONSHIPTYPEID"
                                            + " AND MOMRelationshipTypes.RELATIONSHIPTYPEID IN (" + str_RelationshipTypeIDs + ")";
                                }
                            }

                            break;
                    }
                }

                #endregion

                //Create stored procedure for data populations from MOM to ODS object types.
                ODSDataPopulationStoredProcedure = "CREATE PROCEDURE [DBO].[" + str_DataPopulationSPName + "] AS "
                        + str_DeleteOldInstancesQuery
                        + str_InsertFieldsQuery + " (" + str_SelectedFieldsQuery + str_FromTableQuery
                        + str_InnerJoinsQuery + str_OuterJoinsQuery + ")";

                //Create stored procedure for managine relations for the populated ODS objects.
                ODSManageRelationsStoredProcedure = "CREATE PROCEDURE [DBO].[" + str_ManageRelationsSPName + "] AS "
                        + str_ManageRelationsQuery;

                #endregion
            }
            catch (Exception ExceptionObject)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED, ExceptionObject.Message, 
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.EXCEPTION, ExceptionObject, str_AdditinalErrorInfo);
            }

            return (omerror_Object);
        }

        /// <summary>
        /// This method is used to generate the queries to drop the stored procedures from MOM and ODS databases.
        /// These stored procedures are used for ODS data population and for the relationship (assembly and assoentityfacet) management.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="MOMAndODSSpecificInfoXML">This contains the name of stored procedures which needs to be deleted from MOM
        /// and ODS databases. It also contains the name of the databases which are used while creating the stored procedures.</param>
        /// <param name="DeleteDataPopulationQuery">This contains the query to delete/drop the stored procedure
        /// which is used for ODS data population.</param>
        /// <param name="DeleteManageRelationsQuery">This contains the query to delete/drop the stored procedure 
        /// which is used for managing the relations (assembly and assoentityfacet) for ODS objects.</param>
        /// <returns>On success it will return NULL else an error object will be returned with relevant information.</returns>
        public override OMError PrepareQueryToDropODSStoredProcedures(String MOMAndODSSpecificInfoXML, 
                ref String DeleteDataPopulationQuery, ref String DeleteManageRelationsQuery)
        {
            #region Method Variables

            OMError omerror_Object              = null;
            String  str_AdditinalErrorInfo      = null;
            String  str_DataPopulationSPName    = null;
            String  str_ManageRelationsSPName   = null;

            #endregion

            try
            {
                str_AdditinalErrorInfo      = "Error occurred in SQLQueryProcessor.PrepareQueryToDropODSStoredProcedures() while generating delete queries.";
                str_DataPopulationSPName    = ALCommon.GetInnerValue(ref MOMAndODSSpecificInfoXML, "<DATAPOPULATIONSTOREDPROCEDURENAME>", "</DATAPOPULATIONSTOREDPROCEDURENAME>");
                str_ManageRelationsSPName   = ALCommon.GetInnerValue(ref MOMAndODSSpecificInfoXML, "<MANAGERELATIONSSTOREDPROCEDURENAME>", "</MANAGERELATIONSSTOREDPROCEDURENAME>");
                DeleteDataPopulationQuery   = "DROP PROCEDURE [DBO].[" + str_DataPopulationSPName + "]";
                DeleteManageRelationsQuery  = "DROP PROCEDURE [DBO].[" + str_ManageRelationsSPName + "]";
            }
            catch (Exception ExceptionObject)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED, ExceptionObject.Message, 
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.EXCEPTION, ExceptionObject, str_AdditinalErrorInfo);
            }

            return (omerror_Object);
        }
    }
}