﻿#region Using Directives
#region System
using System;
using System.Xml;
using System.Xml.XPath;
using System.Text;
using System.Data.SqlClient;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
#endregion
#region Assetlink
using Assetlink;
using Assetlink.DataAccessLayer;
using System.Web.Caching;
using System.Data;
using System.IO;
#endregion
#endregion

//---------------------------------------------------------------------------------------------------//
// File Name: SQLQueryProcessor.cs                                                                   //   
// Author   : Miss. Afsar Khan                                                                       // 
// Class    : QueryProcessor                                                                         //
// Purpose  : Conatins logic SQL Server database access code.                                        //
//---------------------------------------------------------------------------------------------------//

namespace Assetlink.DataAccessLayer
{
	/// <summary>
	/// <CreatedDate>18-04-2005</CreatedDate>
	/// <purpose>
	/// This class contains all the SQL server specific functions for e.g 
	/// Converting the date into SQL specific format.
	/// Initializing the Connection object, Command Object and the Transaction Object.
	/// </purpose>
	/// <Author>Miss Afsar Khan</Author>
	/// </summary>
	public partial class SQLQueryProcessor : QueryProcessor
    {

        private String DataType_SizeMappingXML = @"<DATATYPES>
                                                        <STRING>1024</STRING>
                                                        <BIGINT>8</BIGINT>
                                                        <INTEGER>8</INTEGER>
                                                        <BOOLEAN>1</BOOLEAN>    
                                                        <FLOAT>8</FLOAT>    
                                                        <DECIMAL>8</DECIMAL>    
                                                        <DATETIME>8</DATETIME>
                                                        <CURRENCY>0</CURRENCY>
                                                        <TEXT>0</TEXT>
                                                        <FILE>0</FILE>    
                                                        <BINDER>0</BINDER>
                                                        <XML>0</XML>
                                                   </DATATYPES>";
        #region Properties
        /// <summary>
        /// Specifies whether paging is supported.
        /// </summary>
        public override bool IsPagingSupported
        {
            get
            {
                return true;
            }
        }
        #endregion

        #region Constructor
        public SQLQueryProcessor()
        {
        }
        #endregion

        #region Methods
        /// <summary>
		/// Converts the Date into SQL specific format
		/// </summary>
		/// <param name="Date">Date to be converted</param>
		/// <returns>The Date string which is specific to SQL Server</returns>
		public override string ConvertDate(String Date)
		{
            StringBuilder   SQLDateString       = null;
            SQLDateString                       = new StringBuilder();

            SQLDateString.AppendFormat("CONVERT(DATETIME,'{0}',101)", Date);
            return SQLDateString.ToString();
		}
		
		/// <summary>
		/// This function takes the connection string as input and is used to 
		/// initialize the Connection Object, The command Object specific to SQL Server.
		/// If the TransactionFlag is false then the Transaction will not begin.
		/// This is useful for all the reads where the query execution need not be in a 
		/// Transaction.
		/// </summary>
		/// <param name="ConnectionString">The Connection string used to connect to the database</param>
        /// <param name="TransactionFlag">Whether to open the connection with transaction.</param>
		public override void InitializeDatabaseSetup(string ConnectionString, bool TransactionFlag)
		{
            try
            {
                if (!TransactionFlag)
                    ConnectionString = ConnectionString + "enlist=false";

                ConnectionObj = new SqlConnection(ConnectionString);
                try
                {
                    ConnectionObj.Open();
                }
                catch(SqlException ExcepObj)
                {
                    String strManagedThreadId = System.Threading.Thread.CurrentThread.ManagedThreadId.ToString();
                    if (ConnectionObj.State == ConnectionState.Broken)
                    {
                        // WRITELOG
                        WriteLog(TenantName, "INSTR", strManagedThreadId, strManagedThreadId, @"""" + strManagedThreadId + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""InitializeDatabaseSetup()"",""Exception in InitializeDatabaseSetup() while ConnectionObj.Open():" + ExcepObj.Message.Replace(@"""", @"""""") + " Connection State is broken." + @"""");
                        ConnectionObj.Close();
                        ConnectionObj.Open();
                    }
                    else
                    {
                        // WRITELOG
                        WriteLog(TenantName, "INSTR", strManagedThreadId, strManagedThreadId, @"""" + strManagedThreadId + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""InitializeDatabaseSetup()"",""Exception in InitializeDatabaseSetup() while ConnectionObj.Open():" + ExcepObj.Message.Replace(@"""", @"""""") + @"""");
                        throw ExcepObj;
                    }
                }
                CommandObj = new SqlCommand();
                if (TransactionFlag)
                    TransactionObj = ConnectionObj.BeginTransaction();
            }
            catch (SqlException ExcepObj)
            {
                 throw ExcepObj;
            }
		}
        
        /// <summary>
        /// This function takes the connection string as input and is used to 
        /// initialize the Connection Object, The command Object specific to SQL Server.
        /// </summary>
        /// <param name="ConnectionString">The Connection string used to connect to the database</param>
        public override void InitializeDatabaseSetup(string ConnectionString)
        {
          
            try
            {
                ConnectionString = ConnectionString + "enlist=false";
                ConnectionObj = new SqlConnection(ConnectionString);
                try
                {
                    ConnectionObj.Open();
                }
                catch (SqlException ExcepObj)
                {
                    String strManagedThreadId = System.Threading.Thread.CurrentThread.ManagedThreadId.ToString();
                    if (ConnectionObj.State == ConnectionState.Broken)
                    {
                        // WRITELOG
                        WriteLog(TenantName, "INSTR", strManagedThreadId, strManagedThreadId, @"""" + strManagedThreadId + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""InitializeDatabaseSetup()"",""Exception in InitializeDatabaseSetup() while ConnectionObj.Open():" + ExcepObj.Message.Replace(@"""", @"""""") + " Connection State is broken." + @"""");
                        ConnectionObj.Close();
                        ConnectionObj.Open();
                    }
                    else
                    {
                        // WRITELOG
                        WriteLog(TenantName, "INSTR", strManagedThreadId, strManagedThreadId, @"""" + strManagedThreadId + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""InitializeDatabaseSetup()"",""Exception in InitializeDatabaseSetup() while ConnectionObj.Open():" + ExcepObj.Message.Replace(@"""", @"""""") + @"""");
                        throw ExcepObj;
                    }
                }
                CommandObj = new SqlCommand();
            }
            catch (SqlException ExcepObj)
            {
                throw ExcepObj;
            }
        }

		/// <summary>
		/// <Purpose>
		/// The CreateTable function takes InputXML as parameter.Based on the details of the Fields specified
		/// the query for the creating the table is prepared.Based on the Mandatory node it indicates whether
		/// the field value can be null or no.Based on the type the Field Type will be decided.Based on the 
		/// Primary Key node value the Primary Key of the table will be decided.
		/// Based on all these inputs the Query will be formed and the table will be created
        /// </Purpose>
		/// </summary>
		/// <param name="InputXML">
		/// The structure of this xml is as follows.Please refer to the TypeCollection.XSD
		/// <?xml version="1.0" encoding="UTF-8"?>
		///<TYPE>
	 	///<TABLENAME>AL_ENT_Plan</TABLENAME>
		/// 	<FIELDCOLLECTION>
		///			<FIELD OperationType="NEW">
		///		 		<FIELDNAME>PlanName</FIELDNAME>
		///					<FIELDTYPE>STRING</FIELDTYPE>
		///						<FIELDSIZE>
		///								30</FIELDSIZE>
		///								<ISMANDATORY>TRUE</ISMANDATORY>
		///								<ISPRIMARYKEY>TRUE</ISPRIMARYKEY>
        ///
		///							</FIELD>
		///						<FIELD OperationType="NEW">
		///							<FIELDNAME>Description</FIELDNAME>
		///							<FIELDTYPE>STRING</FIELDTYPE>
		///							<FIELDSIZE>230</FIELDSIZE>
		///					<ISMANDATORY>FALSE</ISMANDATORY>
		///					<ISPRIMARYKEY>FALSE</ISPRIMARYKEY>
		///			</FIELD>
		///	
		///     </FIELDCOLLECTION>
		///
	    ///</TYPE>
		/// </param>
		/// <returns>
		/// This function returns Error object if there is any error occured.
		/// </returns>
		public override OMError CreateTable(string InputXML)
		{
			StringBuilder	Query               = null;	    //The query that is formed for the Create table is stored in this variable
            XmlDocument     InputXMLDocument    = null;
            XmlNode			TableNameNode       = null;	    //The table Name which needs to be created is specified in the input XML
			string			DataType            = null;		//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
			XmlNodeList		FieldNodes          = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
			XmlNode			TypeNameNode        = null;		//The Type node in the InputXML is saved in this node
			XmlNode			FieldCollectionNode = null;		//This has the FieldCollectionNode which holds all the information about the fields
			XmlNode			FieldNameNode       = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			XmlNode			FieldTypeNode       = null;		//This variable holds the Datatype node of the individual field node specified in the FieldCollection
			XmlNode			SizeNode            = null;		//This variable holds the Size node of the individual field node specified in the FieldCollection
			XmlNode			MandatoryNode       = null;		//This variable holds Mandatory node which indicates whether the Field node is mandatory or no.If yes then the FieldValue cannot be blank
			XmlNode			PrimaryKeyNode      = null;		//This variable holds Primary Node which indicates whether the Field node is PrimaryKey or no.
			OMError			ErrorObj            = null;		//In case there is an error in the function this object will be populated

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String          XSDSourceFilePath   = null;
            String          ObjectTypeName      = null;
            IXMLRepository  ObjXMLManager       = null;

            try
			{
                Query               = new StringBuilder();
                ObjXMLManager       = OMDataManager.XMLRepository;//new XMLManager(TenantName);
				InputXMLDocument    = new XmlDocument();
				InputXMLDocument.LoadXml(InputXML);

				TypeNameNode        = InputXMLDocument.SelectSingleNode("TYPE");
				TableNameNode       = TypeNameNode.SelectSingleNode("TABLENAME");
				FieldCollectionNode = TypeNameNode.SelectSingleNode("FIELDCOLLECTION");
				FieldNodes          = FieldCollectionNode.SelectNodes("FIELD");
                ObjectTypeName      = TypeNameNode.SelectSingleNode("TYPENAME").InnerText.Trim().ToUpper();

                Query.AppendFormat("CREATE TABLE {0} (", TableNameNode.InnerText.ToUpper());
				foreach (XmlElement FieldNode in FieldNodes)
				{
					FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
					FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");

                    //Make CreateTable() query with columns other than XML, Binder and File data type columns.
                    if ((FieldTypeNode.InnerText.ToUpper() != "XML") && 
                        (FieldTypeNode.InnerText.ToUpper() != "BINDER") && 
                        (FieldTypeNode.InnerText.ToUpper() != "FILE"))
                    {

                        MandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                        PrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");

                        switch ((FieldTypeNode.InnerText).ToUpper())
                        {
                            case "STRING":
                                DataType = "NVARCHAR";
                                SizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                                break;

                            case "BIGINT":
                            case "INTEGER":
                                DataType = "BIGINT";
                                //SizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                                break;

                            case "BOOLEAN":
                                DataType = "NVARCHAR";
                                SizeNode = InputXMLDocument.CreateElement("FIELDSIZE");
                                SizeNode.InnerText = "1";
                                break;

                            case "FLOAT":
                            case "DECIMAL":
                                DataType = "FLOAT";
                                //Following code is added by Kiran on date 03rd Jan 06.
                                //This is so because the size of FLOAT is taken as 255 if we could not mentioned the following.
                                SizeNode = InputXMLDocument.CreateElement("FIELDSIZE");
                                SizeNode.InnerText = "8";
                                break;

                            case "DATETIME":
                                DataType = "DATETIME";
                                break;
                           
                            //For CURRENCY data type support. Code added on date 13th June 2006 by Pallavi.
                            case "CURRENCY":
                                DataType = "CURRENCY";
                                //1] Currency Type
                                Query.AppendFormat("{0}_C NVARCHAR (25)", FieldNameNode.InnerText.ToUpper());
                                if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                    Query.AppendFormat("NOT NULL,");
                                else
                                    Query.AppendFormat("NULL,");

                                // 2] Currency Value
                                Query.AppendFormat("{0}_A MONEY ", FieldNameNode.InnerText.ToUpper());
                                if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                    Query.AppendFormat("NOT NULL,");
                                else
                                    Query.AppendFormat("NULL,");
                                break;
                            case "TEXT":
                                DataType = "NVARCHAR(MAX)";
                                break;    
                        }

                       /* MandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                        PrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");*/

                        if (DataType != "CURRENCY")
                            Query.AppendFormat("{0} {1} ", FieldNameNode.InnerText.ToUpper(), DataType);

                        if ((DataType != "BIGINT") && (DataType != "DATETIME") && (DataType != "CURRENCY"))
                            Query.AppendFormat("({0}) ", SizeNode.InnerText);


                        if (DataType != "CURRENCY")
                        {
                            if (PrimaryKeyNode != null && PrimaryKeyNode.InnerText == "TRUE")
                                Query.AppendFormat("NOT NULL PRIMARY KEY,");
                            else
                            {
                                if (MandatoryNode != null && MandatoryNode.InnerText == "TRUE")
                                    Query.AppendFormat("NOT NULL,");
                                else
                                    Query.AppendFormat("NULL,");
                            }
                        }
                    }
                    //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                    //else if ((FieldTypeNode.InnerText.ToUpper() == "XML") && (FieldNameNode != null)
                    //            && (FieldNode.SelectSingleNode("XSDPATH") != null))
                    //{
                    //    //XSDSourceFilePath = FieldTypeNode.Attributes["XSDFilePath"].Value;
                    //    XSDSourceFilePath = FieldNode.SelectSingleNode("XSDPATH").InnerText;
                    //    ErrorObj = ObjXMLManager.CopyXSD(ObjectTypeName , FieldNameNode.InnerText.Trim().ToUpper(), XSDSourceFilePath);
                    //    if (ErrorObj != null)
                    //        return (ErrorObj);
                    //}
				}

				Query = Query.Remove((Query.Length - 1), 1);
				Query.AppendFormat(")");
				ErrorObj = Execute(Query.ToString());
			}
			catch (Exception ExceptionObj)
			{
				ErrorObj = new OMError(OMErrorCodes.TABLE_CREATION_FAILED,
						                    ExceptionObj.Message, OMErrorObjectType.TABLE,
						                    null, OMErrorType.EXCEPTION, ExceptionObj, null); 
			}
            return (ErrorObj);
		}

        /// <summary>
        /// <Purpose>
        /// The UpdateTable function takes InputXML as parameter.Based on the details of the Fields specified
        /// the query for the updating the table is prepared.
        /// </Purpose>
         /// <param name="InputXML">
        /// The structure of this xml is as follows.Please refer to the TypeCollection.XSD
        /// <?xml version="1.0" encoding="UTF-8"?>
        ///<TYPE>
        ///<TABLENAME>AL_ENT_Plan</TABLENAME>
        /// 	<FIELDCOLLECTION>
        ///			<FIELD OperationType="MODIFY">
        ///		 		<FIELDNAME>PlanName</FIELDNAME>
        ///					<FIELDTYPE>STRING</FIELDTYPE>
        ///						<FIELDSIZE>
        ///								30</FIELDSIZE>
        ///								<ISMANDATORY>TRUE</ISMANDATORY>
        ///								<ISPRIMARYKEY>TRUE</ISPRIMARYKEY>
        ///
        ///							</FIELD>
        ///						<FIELD OperationType="MODIFY">
        ///							<FIELDNAME>Description</FIELDNAME>
        ///							<FIELDTYPE>STRING</FIELDTYPE>
        ///							<FIELDSIZE>230</FIELDSIZE>
        ///					<ISMANDATORY>FALSE</ISMANDATORY>
        ///					<ISPRIMARYKEY>FALSE</ISPRIMARYKEY>
        ///			</FIELD>
        ///	
        ///     </FIELDCOLLECTION>
        ///
        ///</TYPE>
        /// </param>
        /// <returns></returns>
		public override OMError UpdateTable(string InputXML)
		{
            XmlDocument InputXMLDocument    = null;
			XmlNode     TableNameNode       = null;		  //The table Name which needs to be created is specified in the input XML
  			XmlNode     TypeNameNode        = null;		  //The Type node in the InputXML is saved in this node
			XmlNode     FieldCollectionNode = null;		  //This has the FieldCollectionNode which holds all the information about the fields
			OMError     ErrorObj            = null;		  //In case there is an error in the function this object will be populated

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String      ObjectTypeName      = null;
            

            try
			{
				InputXMLDocument = new XmlDocument();
				InputXMLDocument.LoadXml(InputXML);

				TypeNameNode        = InputXMLDocument.SelectSingleNode("TYPE");
				TableNameNode       = TypeNameNode.SelectSingleNode("TABLENAME");
                FieldCollectionNode = TypeNameNode.SelectSingleNode("FIELDCOLLECTION");
                ObjectTypeName      = TypeNameNode.SelectSingleNode("TYPENAME").InnerText.Trim().ToUpper();

                //Following condition is added on date 04th JAN 06 by Kiran Abhang.
                //This is done so because if the FIELDCOLLECTION node is empty then the code return an error mesage.
                if (FieldCollectionNode != null)
                {
                    //Altering exisiting Fields
                    ErrorObj = modifyExistingFields(ObjectTypeName, FieldCollectionNode, TableNameNode.InnerText.ToUpper());
                    if (ErrorObj != null)
                        return ErrorObj;

                    //Adding new fields
                    ErrorObj = addNewFields(ObjectTypeName, FieldCollectionNode, TableNameNode.InnerText.ToUpper());
                    if (ErrorObj != null)
                        return ErrorObj;

                    //Deleting existing Fields
                    ErrorObj = deleteExistingFields(ObjectTypeName, FieldCollectionNode, TableNameNode.InnerText.ToUpper());

                    //Update the Cache
                    if (ErrorObj == null)
                    {
                        RemoveFromCache(TableNameNode.InnerText.ToUpper());
                    }
                }
			}
			catch (Exception ExceptionObj)
			{
				ErrorObj = new OMError(OMErrorCodes.TABLE_UPDATION_FAILED,
						                    ExceptionObj.Message, OMErrorObjectType.TABLE,
						                    null, OMErrorType.EXCEPTION, ExceptionObj, null);
			}

            

            

            return (ErrorObj);
        }

        /// <summary>
        /// If fieldvalue contains any wildcard characters then these characters are replaced according to SQL Server escape characters
        /// </summary>
        /// <param name="Date">Field value for filter clause</param>
        /// <param name="Date">SearchOperator value for filter</param>
        /// <returns>Returns string with replaced characters</returns>
        /// 
        public override String ReplaceEscapeCharacters(String FilterValue, String SearchOperator)
        {
            switch (SearchOperator)
            {
                case "IN":
                case "NOTIN":
                    if (FilterValue.Contains("'"))
                        FilterValue = FilterValue.Replace("'", "''");
                    break;                
                case "ENDSWITH":
                case "STARTSWITH":
                case "CONTAINS":
                case "NOTLIKE":
                case "LIKE":
                    if (FilterValue.Contains("["))  
                        FilterValue = FilterValue.Replace("[", "[[]");
                    if (FilterValue.Contains("%"))
                        FilterValue = FilterValue.Replace("%", "[%]");
                    if (FilterValue.Contains("_"))
                        FilterValue = FilterValue.Replace("_", "[_]");
                    if (FilterValue.Contains("'"))
                        FilterValue = FilterValue.Replace("'", "''''");
                    break;
                default :
                    if (FilterValue.Contains("'"))
                        FilterValue = FilterValue.Replace("'", "''''");
                    break;
            }
            return FilterValue;
        } 

         /// <summary>
        /// This function will replace 1st occurance of a string by another string.
        /// </summary>
        /// <param name="OldValue">String to be replaced</param>
        /// <param name="NewValue">New String valueto be used for replacing the old value</param>
        /// <param name="TableNameFieldNameCollectionXML">Contains a list of tables and its fields that should be updated</param>
        public override OMError Replace(String OldValue, String NewValue, String TableNameFieldNameCollectionXML)
        {
            XmlDocument     TableNameFieldNameCollectionXMLDoc  = null;
            OMError         ErrorObj                            = null;
            XmlNodeList     TableNameNodes                      = null;
            XmlNodeList     FieldNameNodes                      = null;
            StringBuilder   BatchUpdateQuery                    = null;
            
            try
            {
                TableNameFieldNameCollectionXMLDoc  = new XmlDocument();
                BatchUpdateQuery                    = new StringBuilder();
                TableNameFieldNameCollectionXMLDoc.LoadXml(TableNameFieldNameCollectionXML);
                TableNameNodes                      = TableNameFieldNameCollectionXMLDoc.DocumentElement.SelectNodes("TABLE");

                if (TableNameNodes.Count > 0)
                {
                    foreach (XmlNode TableNameNode in TableNameNodes)
                    {
                        FieldNameNodes = TableNameNode.SelectNodes("FIELD");
                        foreach (XmlNode FieldNameNode in FieldNameNodes)
                        {
                            BatchUpdateQuery.AppendFormat(@"update {0} set {1} = SUBSTRING({1},1,PATINDEX('%{2}%',{1}) - 1)
                                                          + '{3}' + SUBSTRING({1},PATINDEX('%{2}%',{1}) +  LEN('{2}'))
                                                          where PATINDEX('%{2}%',{1}) > 0;", TableNameNode.Attributes.GetNamedItem("Name").Value, FieldNameNode.InnerText, OldValue.Replace("'", "''"), NewValue.Replace("'", "''"));
                        }
                    }
                    return Execute(BatchUpdateQuery.ToString());
                }
                              
            }
            catch (Exception ExceptionObj)
            {
                
                ErrorObj = new OMError(OMErrorCodes.ENTITY_MODIFICATION_FAILED,
                                            ExceptionObj.Message, OMErrorObjectType.TABLE,
                                            null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return ErrorObj;
        }

        /// <summary>
        /// This function will return an FieldName ensuring that it will not be an reserved name.
        /// </summary>
        /// <param name="FieldName">Name of the Entity/Facet field.</param>
        public override String ChangeFieldNameIncaseItIsReserved(String FieldName)
        {
            return FieldName;
        }

        public override String GetDataTypesAndTheirSizes()
        {
            return DataType_SizeMappingXML;
        }

        /// <summary>
        /// This function can be used to update multiple records using a dynamic batch query.
        /// Whether access control check is to be performed is decided based on input parameters. 
        /// </summary>
        /// <param name="BatchUpdateInputParamObj">Function will accept BatchUpdateInputParam class object as input parameter.
        /// BatchUpdateInputParam class has following members
        /// 1. TableName : Table name of the Object type whose records are to be updated.
        /// 2. PrimaryKeyName : PrimaryKey name of the table.
        /// 3. FilterListXML : Filter list XML containing one or more filters.
        ///                    Structure:
        ///                    <FILTERLIST>
        ///                         <FILTER> 1..*
        ///                             <EXPRESSION></EXPRESSION>
        ///                             <OPERATOR></OPERATOR>
        ///                             <EXPRESSIONVALUE></EXPRESSIONVALUE>
        ///                             <ISCASESENSITIVE></ISCASESENSITIVE> 0..1
        ///                         </FILTER>
        ///                     </FILTERLIST>
        /// 4.UserID : ID of the loggedin user.
        /// 5.GroupIDs : Comma separated groups id's in which user belongs directlty of in-directly.
        /// 6.AllowPartialAccess : 0|1 indicating whether subset of objects is to be updated if
        ///                         User does not have access to all the objects resulting from the filters.
        /// 7.AccessCheckRequired : True|False indicating whether access control checking is to be performed.
        /// 8.InputXML: Input xml containing the valuse to be updated.
        ///              Structure:
        ///                    <$OBJECT$>
        ///                         <$ATTRIBUTE$/> 1..*
        ///                    </$OBJECT$>
        /// 9.ObjectTypeInfoXML: Input xml containing object type information
        ///                         Structure: 
        ///                         <TYPE>
        ///                             <TYPEID></TYPEID>
        ///                             <TYPENAME></TYPENAME>
        ///                             <TYPEDESCRIPTION />
        ///                             <OBJECTTYPE></OBJECTTYPE>
        ///                             <PRIMARYKEY></PRIMARYKEY>
        ///                             <FIELDCOLLECTION>
        ///	                               <FIELD>1..*
        ///		                            <SCHEMAID></SCHEMAID>
        ///		                            <OBJECTTYPEID></OBJECTTYPEID>
        ///		                            <OBJECTTYPE></OBJECTTYPE>
        ///		                            <FIELDNAME></FIELDNAME>
        ///			                        <DISPLAYNAME></DISPLAYNAME>
        ///			                        <FIELDTYPE></FIELDTYPE>
        ///			                        <FIELDSIZE></FIELDSIZE>
        ///			                        <FIELDUITYPE></FIELDUITYPE>
        ///			                        <ISPRIMARYKEY></ISPRIMARYKEY>
        ///			                        <ISMANDATORY></ISMANDATORY>
        ///			                        <ISSEARCHFIELD></ISSEARCHFIELD>
        ///			                        <ISNAMEDHIERARCHY></ISNAMEDHIERARCHY>
        ///			                        <DEFAULTVALUE />
        ///			                        <DISPLAYORDER></DISPLAYORDER>
        ///		                           </FIELD>
        ///	                            </FIELDCOLLECTION>
        ///	                        </TYPE>
        /// 10.ObjectTypeName: Object type name.
        /// </param>
        /// <param name="BatchUpdateOutputParamObj">Function will return BatchUpdateOutputParam class object as output parameter.
        /// BatchUpdateOutputParam class has following members
        /// 1.AccessDeniedIDCollection: ID collection containing ID's of object on which user does not have write access.
        /// </param>
        public override OMError UpdateRecords(BatchUpdateInputParam BatchUpdateInputParamObj, out BatchUpdateOutputParam BatchUpdateOutputParamObj)
        {
            OMError                 OMErrorObj                                  = null;
            int                     PrivilegeValue                              = 0;
            String                  WhereClause                                 = null;
            String                  DynamicSQLForAccessControlCheck             = null;
            String                  UpdateQueryWithoutWhereClause               = null;
            String                  DynamicSQLForPopulatingTEObjectTable        = null;
            NameValueCollection     XMLFieldValueCollection                     = null;
            StringBuilder           DynamicSQLForUpdate                         = null;
            StringCollection        AccessAllowedIDs                            = null;
            IXMLRepository          XMLManagerObj                               = null;
            XmlDocument             FilterListXMLDoc                            = null;
            XmlNode                 FilterListNode                              = null;
            XmlNodeList             FilterNodes                                 = null;
            StringBuilder           XPath                                       = null;
            StringBuilder           SBCommaSepratedID                           = null;
            String                  CommaSepratedIDs                            = null;
            String                  Seprator                                    = null;
                         

            BatchUpdateOutputParamObj       = new BatchUpdateOutputParam();

            try
            {                
                DynamicSQLForUpdate         = new StringBuilder();
                AccessAllowedIDs            = new StringCollection();
                XMLManagerObj               = OMDataManager.XMLRepository;//new XMLManager(TenantName);
                SBCommaSepratedID           = new StringBuilder();

                #region Prepare update Query with out where clause
                XMLFieldValueCollection     = new NameValueCollection();
                if ((OMErrorObj             = PrepareUpdateQuery(BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.InputXML, BatchUpdateInputParamObj.ObjectTypeInfoXML,true, ref XMLFieldValueCollection, ref UpdateQueryWithoutWhereClause)) != null)
                {
                    return OMErrorObj;
                }
                #endregion

                #region Generate where clause for TE
                                
                #region Remove IN filters on PrimaryKey and from Select statements
                
                FilterListXMLDoc            = new XmlDocument();
                XPath                       = new StringBuilder(512);

                FilterListXMLDoc.LoadXml(BatchUpdateInputParamObj.FilterListXML);
                FilterListNode              = FilterListXMLDoc.DocumentElement;
                
                XPath.AppendFormat("FILTER[EXPRESSION[translate(text(), 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='{0}'] and OPERATOR[translate(text(), 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='IN']]",BatchUpdateInputParamObj.PrimaryKeyName);
                FilterNodes                 = FilterListNode.SelectNodes(XPath.ToString());
                
                foreach (XmlNode FilterNode in FilterNodes)
                {
                    CommaSepratedIDs    = FilterNode.SelectSingleNode("EXPRESSIONVALUE").InnerText;
                    CommaSepratedIDs    = CommaSepratedIDs.Replace("(","");
                    CommaSepratedIDs    = CommaSepratedIDs.Replace(")", "");
                    SBCommaSepratedID.Append(Seprator + CommaSepratedIDs);
                    if (Seprator == null)
                        Seprator = ",";
                }
                if (FilterNodes.Count > 0)
                {
                    for (int Count = FilterNodes.Count; Count > 0; Count--)
                    {
                        FilterNodes[Count - 1].ParentNode.RemoveChild(FilterNodes[Count - 1]);
                    }
                }

                #endregion

                if (FilterListNode.SelectSingleNode("FILTER") != null)
                {
                    if ((OMErrorObj = PrepareWhereClauseFromFilterList(BatchUpdateInputParamObj.TableName, FilterListNode.OuterXml, BatchUpdateInputParamObj.ObjectTypeInfoXML, ref WhereClause)) != null)
                    {
                        return OMErrorObj;
                    }
                }
                #endregion

                if (BatchUpdateInputParamObj.AccessCheckRequired)
                {
                    #region Generate Dynamic SQL for Access control check
                    PrivilegeValue          = NSearchCommon.GetPrivilegeValueFromName("WRITE");
                    if ((OMErrorObj = PrepareDynamicSQLForAccessControlCheck(BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.PrimaryKeyName, WhereClause, SBCommaSepratedID.ToString(), BatchUpdateInputParamObj.UserID, BatchUpdateInputParamObj.GroupIDs, PrivilegeValue, ref DynamicSQLForAccessControlCheck)) != null)
                    {
                        return OMErrorObj;
                    }
                    #endregion

                    if (BatchUpdateInputParamObj.AllowPartialAccess == 1)
                    {
                        // Append DynamicSQL for access control check
                        DynamicSQLForUpdate.Append(DynamicSQLForAccessControlCheck);

                        #region Populate Access denied entries
                        DynamicSQLForUpdate.Append(@" DECLARE @TEACCESSDENIED TABLE (FILTEREDOBJID NUMERIC) ");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSDENIED (FILTEREDOBJID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEOBJECT  AS TE WHERE TE.FILTEREDOBJID NOT IN (SELECT ACCALLOWED.FILTEREDOBJID FROM @TEAUTHEVERYONEUSER AS ACCALLOWED)");
                        #endregion

                        // Append DynamicSQL for update
                        if (!String.IsNullOrEmpty(UpdateQueryWithoutWhereClause))
                            DynamicSQLForUpdate.AppendFormat(@" {0} FROM {1} JOIN @TEAUTHEVERYONEUSER TEAUTH ON {1}.{2} = TEAUTH.FILTEREDOBJID", UpdateQueryWithoutWhereClause, BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.PrimaryKeyName);

                        // Return Access allowed and denied entries
                        DynamicSQLForUpdate.Append(@" DECLARE @TEACCESSALLOWED_DENIED TABLE (ACCESSALLOWEDID NUMERIC,ACCESSDENIEDID NUMERIC)");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSALLOWED_DENIED (ACCESSALLOWEDID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEAUTHEVERYONEUSER AS TE");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSALLOWED_DENIED (ACCESSDENIEDID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEACCESSDENIED AS TE");

                        DynamicSQLForUpdate.Append(@" SELECT TE.ACCESSALLOWEDID,TE.ACCESSDENIEDID FROM @TEACCESSALLOWED_DENIED AS TE");
                    }
                    else
                    {
                        // Append DynamicSQL for access control check
                        DynamicSQLForUpdate.Append(DynamicSQLForAccessControlCheck);

                        #region Populate Access denied entries
                        DynamicSQLForUpdate.Append(@" DECLARE @TEACCESSDENIED TABLE (FILTEREDOBJID NUMERIC) ");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSDENIED (FILTEREDOBJID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEOBJECT  AS TE WHERE TE.FILTEREDOBJID NOT IN (SELECT ACCALLOWED.FILTEREDOBJID FROM @TEAUTHEVERYONEUSER AS ACCALLOWED)");
                        #endregion

                        #region Check whether Access allowed on all target objects and only then update the Objects
                        if (!String.IsNullOrEmpty(UpdateQueryWithoutWhereClause))
                        {
                            // Append DynamicSQL for update
                            // Filter 0 in (..) is added so that SQL and Teradata code looks same.(In Teradata we cannot have IF condition or declare variable in Dynamic SQL)
                            DynamicSQLForUpdate.AppendFormat(@" {0} FROM {1} JOIN @TEAUTHEVERYONEUSER TEAUTH ON {1}.{2} = TEAUTH.FILTEREDOBJID WHERE 0 IN (SELECT TOP 1 COUNT(TEAD.FILTEREDOBJID) FROM @TEACCESSDENIED AS TEAD)", UpdateQueryWithoutWhereClause, BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.PrimaryKeyName);
                        }
                        #endregion
                                         
                        // Return Access allowed and denied entries
                        DynamicSQLForUpdate.Append(@" DECLARE @TEACCESSALLOWED_DENIED TABLE (ACCESSALLOWEDID NUMERIC,ACCESSDENIEDID NUMERIC)");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSALLOWED_DENIED (ACCESSALLOWEDID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEAUTHEVERYONEUSER AS TE");
                        DynamicSQLForUpdate.Append(@" INSERT INTO @TEACCESSALLOWED_DENIED (ACCESSDENIEDID) ");
                        DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID FROM @TEACCESSDENIED AS TE");

                        DynamicSQLForUpdate.Append(@" SELECT TE.ACCESSALLOWEDID,TE.ACCESSDENIEDID FROM @TEACCESSALLOWED_DENIED AS TE");
                    }
                }
                else
                {
                    // Populate Target object table
                    if ((OMErrorObj = PrepareDynamicSQLForPopulatingTargetObjectTable(BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.PrimaryKeyName, WhereClause, SBCommaSepratedID.ToString(), ref DynamicSQLForPopulatingTEObjectTable)) != null)
                    {
                        return OMErrorObj;
                    }

                    // Append DynamicSQL for populating Target object table
                    DynamicSQLForUpdate.AppendFormat(@" {0}", DynamicSQLForPopulatingTEObjectTable);

                    // Append DynamicSQL for update
                    if (!String.IsNullOrEmpty(UpdateQueryWithoutWhereClause))
                    {
                        DynamicSQLForUpdate.AppendFormat(@" {0} FROM {1} JOIN @TEOBJECT TEAUTH ON {1}.{2} = TEAUTH.FILTEREDOBJID", UpdateQueryWithoutWhereClause, BatchUpdateInputParamObj.TableName, BatchUpdateInputParamObj.PrimaryKeyName);
                    }
                    DynamicSQLForUpdate.Append(@" SELECT TE.FILTEREDOBJID AS ACCESSALLOWEDID,NULL AS ACCESSDENIEDID FROM @TEOBJECT AS TE");
                }

                //DynamicSQLForUpdate.Insert(0, "EXEC SP_EXECUTESQL N'");
                //DynamicSQLForUpdate.Append("'");
                if (BatchUpdateInputParamObj.AccessCheckRequired)
                {
                    try
                    {
                        if ((OMErrorObj = RetrieveResult(DynamicSQLForUpdate.ToString().Replace("''","'"))) != null)
                        {
                            return OMErrorObj;
                        }
                        else
                        {
                            while (DataReaderObj.Read())
                            {
                                if (XMLFieldValueCollection.Count > 0)
                                {
                                    if (!String.IsNullOrEmpty(DataReaderObj[0].ToString()))
                                        AccessAllowedIDs.Add(DataReaderObj[0].ToString());
                                }
                                if (!String.IsNullOrEmpty(DataReaderObj[1].ToString()))
                                    BatchUpdateOutputParamObj.AccessDeniedIDCollection.Add((DataReaderObj[1].ToString()));
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }
                }
                else
                {
                    try
                    {
                        if ((OMErrorObj = RetrieveResult(DynamicSQLForUpdate.ToString().Replace("''", "'"))) != null)
                        {
                            return OMErrorObj;
                        }

                        if (XMLFieldValueCollection.Count > 0)
                        {
                            while (DataReaderObj.Read())
                            {
                                if (!String.IsNullOrEmpty(DataReaderObj[0].ToString()))
                                    AccessAllowedIDs.Add(DataReaderObj[0].ToString());

                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
                        if (DataReaderObj != null && !DataReaderObj.IsClosed)
                            DataReaderObj.Close();
                    }
                }

                #region Update XMLFields
                if (XMLFieldValueCollection.Count > 0)
                {
                    if (BatchUpdateInputParamObj.AllowPartialAccess == 1)
                    {
                        for (int ObjectIDCount = 0; ObjectIDCount < AccessAllowedIDs.Count; ObjectIDCount++)
                        {
                            for (int FieldCount = 0; FieldCount < XMLFieldValueCollection.Count; FieldCount++)
                            {
                                OMErrorObj = XMLManagerObj.Save(BatchUpdateInputParamObj.ObjectTypeName, Convert.ToUInt64(AccessAllowedIDs[ObjectIDCount]), XMLFieldValueCollection.GetKey(FieldCount), XMLFieldValueCollection.Get(FieldCount));
                                if (OMErrorObj != null)
                                    return OMErrorObj;
                            }
                        }
                    }
                    else if (BatchUpdateOutputParamObj.AccessDeniedIDCollection.Count == 0)
                    {
                        for (int ObjectIDCount = 0; ObjectIDCount < AccessAllowedIDs.Count; ObjectIDCount++)
                        {
                            for (int FieldCount = 0; FieldCount < XMLFieldValueCollection.Count; FieldCount++)
                            {
                                OMErrorObj = XMLManagerObj.Save(BatchUpdateInputParamObj.ObjectTypeName, Convert.ToUInt64(AccessAllowedIDs[ObjectIDCount]), XMLFieldValueCollection.GetKey(FieldCount), XMLFieldValueCollection.Get(FieldCount));
                                if (OMErrorObj != null)
                                    return OMErrorObj;
                            }
                        }
                    }
                }

                #endregion

                return null;
            }
            catch (Exception ExceptionObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.ENTITY_MODIFICATION_FAILED,
                                            ExceptionObj.Message, OMErrorObjectType.TABLE,
                                            null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return OMErrorObj;
            }
        }

        public override String ReplaceSingleQuote(string FieldValue, Boolean ToBeUsedWithinDynamicSQL)
		{
            if(ToBeUsedWithinDynamicSQL)
                FieldValue = FieldValue.Replace("'", "''''");
            else
                FieldValue = FieldValue.Replace("'", "''");
           
            return FieldValue;
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// <Purpose>
        ///  Helper method to modify the existing fields. It select all Field nodes with OperationType = 'MODIFY' and 
        ///  generate ALTER command for them.
        /// </Purpose>   
        /// </summary>
        /// <param name="FieldCollectionNode"></param>
        /// <returns>Null if successful or OMError object if there is an error.</returns>
        private OMError modifyExistingFields(String ObjectTypeName, XmlNode FieldCollectionNode, String TableNamePassed)
        {
            //XmlNode       TableNameNode       = null;	    //The table Name which needs to be created is specified in the input XML
            string          DataType            = null;		//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            XmlNodeList     FieldNodes          = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
            StringBuilder   QueryModify         = null;
            XmlNode         FieldNameNode       = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode         FieldTypeNode       = null;		//This variable holds the Datatype node of the individual field node specified in the FieldCollection
            XmlNode         SizeNode            = null;		//This variable holds the Size node of the individual field node specified in the FieldCollection
            XmlNode         MandatoryNode       = null;		//This variable holds Mandatory node which indicates whether the Field node is mandatory or no.If yes then the FieldValue cannot be blank
            XmlNode         PrimaryKeyNode      = null;		//This variable holds Primary Node which indicates whether the Field node is PrimaryKey or no.
            OMError         ErrorObj            = null;		//In case there is an error in the function this object will be populated

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String          XSDSourceFilePath   = null;
            IXMLRepository  ObjXMLManager       = null;

            try
            {
				FieldNodes = FieldCollectionNode.SelectNodes("FIELD[@OperationType='MODIFY']");
				if (FieldNodes.Count > 0)
				{
                    ObjXMLManager = OMDataManager.XMLRepository;//new XMLManager(TenantName);
					foreach (XmlElement FieldNode in FieldNodes)
					{
                        FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
						FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                        QueryModify = new StringBuilder();

                        if ((FieldTypeNode.InnerText.ToUpper() != "XML")
                                && (FieldTypeNode.InnerText.ToUpper() != "BINDER")
                                && (FieldTypeNode.InnerText.ToUpper() != "FILE"))
                        {
                            SizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                            MandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                            PrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");
                            
                            switch ((FieldTypeNode.InnerText).ToUpper())
                            {
                                case "STRING":
                                    DataType = "NVARCHAR";
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    DataType = "BIGINT";
                                    break;

                                case "BOOLEAN":
                                    DataType = "NVARCHAR";
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    DataType = "FLOAT";
                                    break;

                                case "DATETIME":
                                    DataType = "DATETIME";
                                    break;

                                //For CURRENCY data type support. Code added on date 15th June 2006 by Pallavi
                                case "CURRENCY":
                                    DataType = "CURRENCY";
                                    //1] Currency Type
                                    QueryModify.AppendFormat("ALTER TABLE {0} ALTER COLUMN {1}_C NVARCHAR (25)", TableNamePassed, FieldNameNode.InnerText.ToUpper());
                                    if (MandatoryNode.InnerText == "TRUE")
                                        QueryModify.AppendFormat("NOT NULL");
                                    else
                                        QueryModify.AppendFormat("NULL");

                                    // 2] Currency Value
                                    QueryModify.AppendFormat(" ALTER TABLE {0} ALTER COLUMN {1}_A MONEY ", TableNamePassed, FieldNameNode.InnerText.ToUpper());
                                    if (MandatoryNode.InnerText == "TRUE")
                                        QueryModify.AppendFormat("NOT NULL,");
                                    else
                                        QueryModify.AppendFormat("NULL,");

                                    break;

                                case "TEXT":
                                    DataType = "NVARCHAR(MAX)";
                                    break;
                            }

                           /* SizeNode        = FieldNode.SelectSingleNode("FIELDSIZE");
                            MandatoryNode   = FieldNode.SelectSingleNode("ISMANDATORY");
                            PrimaryKeyNode  = FieldNode.SelectSingleNode("ISPRIMARYKEY");*/

                            //QueryModify = new StringBuilder();
                            if (DataType != "CURRENCY")
                            {
                                QueryModify.AppendFormat("ALTER TABLE {0} ", TableNamePassed);
                                QueryModify.AppendFormat("ALTER COLUMN {0} {1} ", FieldNameNode.InnerText, DataType);
                            }

                            if ((DataType != "BIGINT") && (DataType != "DATETIME") && (DataType != "CURRENCY") && (DataType != "NVARCHAR(MAX)"))
                                QueryModify.AppendFormat(" ({0}) ", SizeNode.InnerText);

                            if (DataType != "CURRENCY")
                            {
                                if (MandatoryNode.InnerText == "TRUE")
                                    QueryModify.AppendFormat("NOT NULL,");
                                else
                                    QueryModify.AppendFormat("NULL,");
                            }

                            QueryModify = QueryModify.Remove((QueryModify.Length - 1), 1);
                            ErrorObj = Execute(QueryModify.ToString());
                            QueryModify = null;
                            if (ErrorObj != null)
                                return (ErrorObj);
                        }
                        //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                        //else if ((FieldTypeNode.InnerText.ToUpper() == "XML") && (FieldNameNode != null)
                        //            && (FieldNode.SelectSingleNode("XSDPATH") != null))
                        //{
                        //    //XSDSourceFilePath = FieldTypeNode.Attributes["XSDFilePath"].Value;
                        //    XSDSourceFilePath = FieldNode.SelectSingleNode("XSDPATH").InnerText;
                        //    ErrorObj = ObjXMLManager.CopyXSD(ObjectTypeName, FieldNameNode.InnerText.Trim().ToUpper(), XSDSourceFilePath);
                        //    if (ErrorObj != null)
                        //        return (ErrorObj);
                        //}
					}
				}
            }
            catch(SqlException ExcepObj)
            {
                throw ExcepObj;
            }
            catch(Exception ExcepObj)
            {
                throw ExcepObj;
            }

            return (null);
        }

        /// <summary>
        /// <Purpose>
        ///  Helper method to add new fields. It select all Field nodes with OperationType = 'NEW' and 
        ///  generate ALTER command for them.
        /// </Purpose>   
        /// </summary>
        /// <param name="FieldCollectionNode"></param>
        /// <returns>Null if successful or OMError object if there is an error.</returns>
        private OMError addNewFields(String ObjectTypeName, XmlNode FieldCollectionNode, String TableNamePassed)
        {
            String          DataType            = null;		//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            XmlNodeList     FieldNodes          = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
            StringBuilder   QueryAdd            = null;
            XmlNode         FieldNameNode       = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode         FieldTypeNode       = null;		//This variable holds the Datatype node of the individual field node specified in the FieldCollection
            XmlNode         SizeNode            = null;		//This variable holds the Size node of the individual field node specified in the FieldCollection
            XmlNode         MandatoryNode       = null;		//This variable holds Mandatory node which indicates whether the Field node is mandatory or no.If yes then the FieldValue cannot be blank
            XmlNode         PrimaryKeyNode      = null;		//This variable holds Primary Node which indicates whether the Field node is PrimaryKey or no.
            OMError         ErrorObj            = null;		//In case there is an error in the function this object will be populated

            //For XML data type support. Code added on date 8th March 2006 by Kiran.
            String          XSDSourceFilePath   = null;
            IXMLRepository  ObjXMLManager       = null;
            
            try
            {
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD[@OperationType='NEW']");
				if (FieldNodes.Count > 0)
				{
                    ObjXMLManager = OMDataManager.XMLRepository;//new XMLManager(TenantName);
					foreach (XmlElement Node in FieldNodes)
					{
                        FieldNameNode = Node.SelectSingleNode("FIELDNAME");
						FieldTypeNode = Node.SelectSingleNode("FIELDTYPE");

                        if ((FieldTypeNode.InnerText.ToUpper() != "XML")
                                && (FieldTypeNode.InnerText.ToUpper() != "BINDER")
                                && (FieldTypeNode.InnerText.ToUpper() != "FILE"))
                        {
                            SizeNode = Node.SelectSingleNode("FIELDSIZE");
                            MandatoryNode = Node.SelectSingleNode("ISMANDATORY");
                            PrimaryKeyNode = Node.SelectSingleNode("ISPRIMARYKEY");

                            QueryAdd = new StringBuilder();
                            QueryAdd.AppendFormat("ALTER TABLE {0} ADD ", TableNamePassed);
                           
                            
                            switch ((FieldTypeNode.InnerText).ToUpper())
                            {
                                case "STRING":
                                    DataType = "NVARCHAR";
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    DataType = "BIGINT";
                                    break;

                                case "BOOLEAN":
                                    DataType = "NVARCHAR";
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    DataType = "FLOAT";
                                    break;

                                case "DATETIME":
                                    DataType = "DATETIME";
                                    break;

                                //For CURRENCY data type support. Code added on date 14th June 2006 by Pallavi
                                case "CURRENCY":
                                    DataType = "CURRENCY";
                                    //1] Currency Type
                                    QueryAdd.AppendFormat("{0}_C NVARCHAR (25)", FieldNameNode.InnerText.ToUpper());
                                    if (MandatoryNode.InnerText == "TRUE")
                                        QueryAdd.AppendFormat("NOT NULL,");
                                    else
                                        QueryAdd.AppendFormat("NULL,");

                                    // 2] Currency Value
                                    QueryAdd.AppendFormat("{0}_A MONEY ", FieldNameNode.InnerText.ToUpper());
                                    if (MandatoryNode.InnerText == "TRUE")
                                        QueryAdd.AppendFormat("NOT NULL,");
                                    else
                                        QueryAdd.AppendFormat("NULL,");

                                    break;

                                // For NVarChar(MAX)
                                // This condition will work only when Database is SQL Server 2005.     
                                case "TEXT":
                                    DataType = "NVARCHAR(MAX)";
                                    break;    
      
                            }

                           /* SizeNode        = Node.SelectSingleNode("FIELDSIZE");
                            MandatoryNode   = Node.SelectSingleNode("ISMANDATORY");
                            PrimaryKeyNode  = Node.SelectSingleNode("ISPRIMARYKEY");

                            QueryAdd = new StringBuilder();
                            QueryAdd.AppendFormat("ALTER TABLE {0} ADD ", TableNamePassed);*/
                            if (DataType != "CURRENCY")
                                QueryAdd.AppendFormat("{0} {1} ", FieldNameNode.InnerText, DataType);

                            if ((DataType != "BIGINT") && (DataType != "DATETIME") && (DataType != "CURRENCY") && (DataType != "NVARCHAR(MAX)"))
                                QueryAdd.AppendFormat(" ({0}) ", SizeNode.InnerText);
                            if (DataType != "CURRENCY")
                            {
                                //This is commented because, table alteration does not allow to add columns having non-null values
                                //if (MandatoryNode.InnerText == "TRUE")
                                //    QueryAdd.AppendFormat("NOT NULL,");
                                //else
                                    QueryAdd.AppendFormat("NULL,");
                            }
                            QueryAdd = QueryAdd.Remove((QueryAdd.Length - 1), 1);
                            ErrorObj = Execute(QueryAdd.ToString());
                            QueryAdd = null;
                            if (ErrorObj != null)
                                return (ErrorObj);
                        }
                        //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                        //else if ((FieldTypeNode.InnerText.ToUpper() == "XML") && (FieldNameNode != null)
                        //            && (Node.SelectSingleNode("XSDPATH") != null))
                        //{
                        //    //XSDSourceFilePath = FieldTypeNode.Attributes["XSDFilePath"].Value;
                        //    XSDSourceFilePath = Node.SelectSingleNode("XSDPATH").InnerText;
                        //    ErrorObj = ObjXMLManager.CopyXSD(ObjectTypeName, FieldNameNode.InnerText.Trim().ToUpper(), XSDSourceFilePath);
                        //    if (ErrorObj != null)
                        //        return (ErrorObj);
                        //}
                    }
				}
            }
            catch(SqlException ExcepObj)
            {
                throw ExcepObj;
            }
            catch(Exception ExcepObj)
            {
                throw ExcepObj;
            }

            return (null);
        }

        /// <summary>
        /// <Purpose>
        ///  Helper method to delete existing fields. It select all Field nodes with OperationType = 'DELETE' and 
        ///  generate DROP command for them.
        /// </Purpose>   
        /// </summary>
        /// <param name="FieldCollectionNode"></param>
        /// <returns>Null if successful or OMError object if there is an error.</returns>
        private OMError deleteExistingFields(String ObjectTypeName, XmlNode FieldCollectionNode, String TableNamePassed)
        {
            OMError ErrorObj = null;		//In case there is an error in the function this object will be populated
            IXMLRepository ObjXMLManager = null;
            XmlNodeList FieldNodes = null;		//This hold the list of nodes in the FieldCollection node in the Input XML
            XmlNode         FieldNameNode   = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
            XmlNode         FieldTypeNode   = null;
            StringBuilder QueryDelete = null;

            try
            {
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD[@OperationType='DELETE']");
				if (FieldNodes.Count > 0)
				{
                    ObjXMLManager = OMDataManager.XMLRepository;//new XMLManager(TenantName);
                    foreach (XmlElement FieldNode in FieldNodes)
					{
                        FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
						FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");

                        QueryDelete = new StringBuilder();

                        if (FieldTypeNode.InnerText.ToUpper() == "XML")
                        {
                            //COMMENTED AS DISCUSSED AFTER CODE REVIEW
                            //ObjXMLManager.DeleteXSD(ObjectTypeName, FieldNameNode.InnerText.Trim().ToUpper());
                            ErrorObj = DeleteAllXMLFilesOfXMLField(TableNamePassed, ObjectTypeName, FieldNameNode.InnerText.Trim().ToUpper());
                            if (ErrorObj != null)
                                return (ErrorObj);
                        }


                        else if ((FieldTypeNode.InnerText.ToUpper() == "CURRENCY")
                                  || ((FieldTypeNode.InnerText.ToUpper() != "XML")
                                && (FieldTypeNode.InnerText.ToUpper() != "BINDER")
                                && (FieldTypeNode.InnerText.ToUpper() != "FILE")))
                        {


                            QueryDelete.AppendFormat("ALTER TABLE {0} DROP COLUMN ", TableNamePassed);
                            //For CURRENCY data type support. Code added on date 14th June 2006 by Pallavi
                            if (FieldTypeNode.InnerText.ToUpper() == "CURRENCY")
                            {
                                QueryDelete.AppendFormat("{0}_C,{1}_A,", FieldNameNode.InnerText, FieldNameNode.InnerText);



                            }

                            else if ((FieldTypeNode.InnerText.ToUpper() != "XML")
                                && (FieldTypeNode.InnerText.ToUpper() != "BINDER")
                                && (FieldTypeNode.InnerText.ToUpper() != "FILE")
                                && (FieldTypeNode.InnerText.ToUpper() != "CURRENCY"))  //FOR CURRENCY DATA TYPE , BY PALLAVI 13JUNE 2006
                            {

                                QueryDelete.AppendFormat("{0},", FieldNameNode.InnerText);
                            }

                            QueryDelete = QueryDelete.Remove((QueryDelete.Length - 1), 1);
                            ErrorObj = Execute(QueryDelete.ToString());
                            if (ErrorObj != null)
                                return (ErrorObj);
                            QueryDelete = null;
                        }
					}
				}
            }
            catch(SqlException ExcepObj)
            {
                throw ExcepObj;
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }

            return (ErrorObj);
        }

        private OMError DeleteAllXMLFilesOfXMLField(String TableName, String ObjectTypeName, String FieldName)
        {
            OMError         ErrorObj            = null;
            IXMLRepository  ObjXMLManager       = null;
            String          PrimaryKeyFieldName = null;
            String[]        ArrObjectIDs        = null;
            StringBuilder   ObjectIDsList       = null;

            try
            {
                ErrorObj = GetPrimaryKey(TableName, out PrimaryKeyFieldName);
                if (ErrorObj == null)
                {
                    ErrorObj = GetObjectIDs(TableName, PrimaryKeyFieldName, out ObjectIDsList);
                    if ((ErrorObj == null) && (ObjectIDsList != null) && (ObjectIDsList.Length > 0))
                        ArrObjectIDs = (ObjectIDsList.ToString().Split(','));
                }

                if ((ArrObjectIDs != null) && (ArrObjectIDs.Length > 0))
                {
                    ObjXMLManager = OMDataManager.XMLRepository;//new XMLManager(TenantName);
                    ErrorObj = ObjXMLManager.DeleteXML(ObjectTypeName, ArrObjectIDs, FieldName);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        private OMError GetPrimaryKey(String TableName, out String PrimaryKeyFieldName)
        {
            OMError         ErrorObj    = null;
            StringBuilder   Query       = null;

            //Initialize the 'out' parameter.
            PrimaryKeyFieldName = null;

            try
            {
                Query = new StringBuilder();
                Query.Append("SELECT FIELDNAME FROM ALOM_SCHEMA");
                Query.AppendFormat(" WHERE TABLENAME = '{0}' AND ISPRIMARYKEY = 'TRUE'", TableName);
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj == null)
                    {
                        if (DataReaderObj.Read())
                            PrimaryKeyFieldName = DataReaderObj.GetValue(0).ToString().ToUpper();
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }

        private OMError GetObjectIDs(String TableName, String PrimaryKeyFieldName, out StringBuilder ObjectIDsList)
        {
            OMError         ErrorObj    = null;
            String          Comma       = "";
            StringBuilder   Query       = null;

            //Initialize 'out' paramter.
            ObjectIDsList = null;

            try
            {
                Query = new StringBuilder();
                Query.AppendFormat("SELECT {0} FROM {1}", PrimaryKeyFieldName, TableName);
                try
                {
                    ErrorObj = RetrieveResult(Query.ToString());
                    if (ErrorObj == null)
                    {
                        ObjectIDsList = new StringBuilder();
                        while (DataReaderObj.Read())
                        {
                            ObjectIDsList.AppendFormat("{0}{1}", Comma, DataReaderObj.GetValue(0).ToString());
                            Comma = ",";
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }

        #endregion

        #region Cache Methods
         /// <summary>
        /// Initializes the Cache
        /// </summary>
        public override void InitializeCache(String ConnectionString, String[] TableList)
        {
            SqlCommand          sqlCommand      = null;
            SqlDataReader       sqlReader       = null;
            String              query           = null;
            String              cacheItemName   = null;
            DataTable           cacheTable      = null;

            try
            {
                if (TableList == null)
                    return;

                if (ConnectionObj.State != ConnectionState.Open)
                {
                    InitializeDatabaseSetup(ConnectionString);
                }
                foreach (string tableName in TableList)
                {
                    query = "SELECT * FROM " + tableName;

                    try
                    {
                        sqlCommand = new SqlCommand(query, ConnectionObj as SqlConnection);
                        sqlReader = sqlCommand.ExecuteReader() as SqlDataReader;
                        //Bug fix 5451 : If table does not contain rows, do not add it to cache. Cache should always contain records.
                        if (sqlReader.HasRows)
                        {
                            cacheTable = new DataTable();
                            cacheTable.Load(sqlReader);

                            cacheItemName = TenantName.ToUpper() + "_" + tableName.ToUpper();
                            CacheManager.AddCacheItem(cacheItemName.ToUpper(), cacheTable);
                        }
                    }
                    catch
                    {
                        //Ignore any errors that happen while loading a table into the cache
                    }
                    finally
                    {
                        if(sqlReader != null)
                            sqlReader.Close();
                        if(sqlCommand != null)
                            sqlCommand.Dispose();
                    }
                }
            }
            catch
            {

            }
        }
        #endregion

        public void WriteLog(String TenantName, String SolutionName, String PageName, String SessionID, String ContentToWrite)
        {
            lock (this)
            {
                //if (ReadRegisterykey())
                //{
                PageName = "Thread";
                String InstallationDir = ALCommon.GetInstallationDir();
                int LastIndex = PageName.LastIndexOf('/');
                int ExtIndex = PageName.LastIndexOf('.');

                if (LastIndex > 0 && ExtIndex > 0)
                {
                    PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                    PageName = PageName.Replace(' ', '_');
                }
                String DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                String LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                FileStream LogFileStream = null;
                StreamWriter LogStreamWriter = null;

                try
                {
                    DirectoryInfo Root = new DirectoryInfo(DirName);
                    if (Root.Exists == false)
                        Root.Create();

                    if (File.Exists(LogFileName))
                        LogFileStream = new FileStream(LogFileName, FileMode.Append);
                    else
                        LogFileStream = new FileStream(LogFileName, FileMode.Create);

                    LogStreamWriter = new StreamWriter(LogFileStream);
                    StringBuilder TimeStamp = new StringBuilder();

                    //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                    //LogStreamWriter.WriteLine("=====================================================================");
                    //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                    LogStreamWriter.WriteLine(ContentToWrite);
                    LogStreamWriter.WriteLine();
                }
                catch (Exception ex)
                {
                    PageName = "WriteLogError";
                    InstallationDir = ALCommon.GetInstallationDir();
                    LastIndex = PageName.LastIndexOf('/');
                    ExtIndex = PageName.LastIndexOf('.');

                    if (LastIndex > 0 && ExtIndex > 0)
                    {
                        PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                        PageName = PageName.Replace(' ', '_');
                    }
                    DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                    LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                    FileStream LogFileStreamEx = null;
                    StreamWriter LogStreamWriterEx = null;

                    try
                    {
                        DirectoryInfo Root = new DirectoryInfo(DirName);
                        if (Root.Exists == false)
                            Root.Create();

                        if (File.Exists(LogFileName))
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Append);
                        else
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Create);
                        string strMessageToWrite = SessionID + "," + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + "," + ex.Message;
                        LogStreamWriterEx = new StreamWriter(LogFileStreamEx);
                        StringBuilder TimeStamp = new StringBuilder();

                        //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                        //LogStreamWriter.WriteLine("=====================================================================");
                        //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                        LogStreamWriterEx.WriteLine(strMessageToWrite);
                        LogStreamWriterEx.WriteLine();
                    }
                    catch
                    {

                    }
                    finally
                    {
                        if (LogStreamWriterEx != null)
                            LogStreamWriterEx.Close();

                        if (LogFileStreamEx != null)
                            LogFileStreamEx.Close();

                        LogStreamWriterEx = null;
                        LogFileStreamEx = null;
                    }
                }
                finally
                {
                    if (LogStreamWriter != null)
                        LogStreamWriter.Close();

                    if (LogFileStream != null)
                        LogFileStream.Close();

                    LogStreamWriter = null;
                    LogFileStream = null;
                }
                //}
            }
        }
    }
}