using System;
using System.Collections.Generic;
using System.Threading;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.AccessControl;
using Assetlink.ObjectManager.NamedHierarchy;
using Assetlink.Diagnostic;
namespace Assetlink.ObjectManager.ETFTManagement
{
    /// <summary>
    /// <CreatedDate>03 May 2005</CreatedDate>
    /// <Author>Mr. Hemant Kirad</Author>
    /// <Purpose>
    /// This class represents an EntityType. An EntityType represents an entity involved in the business world.
    /// This is a partial class so it will have some of its functionality divided into
    /// 1. EntityType.cs (This file)
    /// 2. EntityTypeExt1.cs 
    /// </Purpose>
    /// </summary>
    internal partial class EntityType : ObjectManagerBase
    {
        private static Object Lock = new object();
        #region Constructors
        public EntityType()
		{
			
		}

		public EntityType(ulong UserID,string CurrentLanguage,string TenantName)
			: base(UserID,CurrentLanguage,TenantName)
		{

		}

        public EntityType(DataManager DataManagerObj)
            : base(DataManagerObj)
        {

        }

        public EntityType(DataManagerReader OMDataReader, DataManagerWriter OMDataWriter)
            : base(OMDataReader, OMDataWriter)
		{

        }

        public EntityType(ulong UserID, string CurrentLanguage,
                            string TenantName, DataManager DataManagerObj)
            : base(UserID, CurrentLanguage, TenantName, DataManagerObj)
        {
            
        }
        #endregion Constructors

        #region PrivateMethods
        /// <summary>
        /// <CreatedDate>03 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will return a tablename for the given EntityTypeName.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeName">
        /// This contains the EntityTypeName.
        /// </param>
        /// <returns>
        /// TableName.
        /// </returns>
        private String generateTableName(String TypeName)
        {
            StringBuilder TableName = null;    //The StringBuilder object storing the value of TableName. 
            try
            {
                TableName = new StringBuilder();
                TableName.AppendFormat("{0}{1}", "ALENT_", TypeName.ToUpper());
                return TableName.ToString();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>04 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML which is required for adding a row in the ALOM_ENTITYTYPE table.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">This Node represents the Type node.</param>
        /// <param name="TableName">This holds the TableName for the EntityType.</param>
        /// <param name="EntityTypeXMLDocument">This is a reference parameter and will hold the XML which is 
        /// created in this function and returned. The structure of this EntityTypeXML is give below:
        ///      ENTITYTYPE
        ///        |- ENTITYTYPEID
        ///        |- ENTITYTYPENAME
        ///        |- TABLENAME   
        ///        |- PRIMARYKEY
        ///        |- TYPEDESCRIPTION
        /// </param>
        public void generateEntityTypeXML(XmlNode TypeNode, String TableName, ref XmlDocument EntityTypeXMLDocument)
        {
            String TypeID               = null;     //String variable storing the value of TypeID.
            String TypeName             = null;     //String variable storing the value of TypeName.
            String Description          = null;     //String variable storing the value of Description.
            String IsBuiltIn            = null;     //String variable storing the value of IsBuiltIn.
            XmlNode EntityTypeNode      = null;     //DOM Object representing the EntityType node.
            XmlNode EntityTypeIDNode    = null;     //DOM Node representing the EntityTypeID node.
            XmlNode EntityTypeNameNode  = null;     //DOM Node representing the EntityTypeName node.
            XmlNode TableNameNode       = null;     //DOM Node representing the TableName node.
            XmlNode TypeDescriptionNode = null;     //DOM Node representing the TypeDescription node.
            XmlNode TypeIDNode          = null;     //DOM Node representing the TypeID node.
            XmlNode TypeNameNode        = null;     //DOM Node representing the TypeName node.
            XmlNode BuiltInNode         = null;     //DOM Node representing the IsBuiltIn node.
            XmlNode EntityBuiltIntypeNode = null;   //DOM Node representing the EntityBuiltIntypeNode node.

            try
            {
                // Get information from the Type XML.
                TypeIDNode = TypeNode.SelectSingleNode("TYPEID");
                TypeID = TypeIDNode.InnerText;

                TypeNameNode = TypeNode.SelectSingleNode("TYPENAME");
                TypeName = TypeNameNode.InnerText;

                TypeDescriptionNode = TypeNode.SelectSingleNode("TYPEDESCRIPTION");
                if (TypeDescriptionNode == null)
                    Description = "";
                else
                    Description = TypeDescriptionNode.InnerText;

                BuiltInNode = TypeNode.SelectSingleNode("ISBUILTIN");
                if (BuiltInNode == null)
                    IsBuiltIn = "0";
                else
                    IsBuiltIn = BuiltInNode.InnerText;

                // Prepare the EntityType XML.
                EntityTypeXMLDocument = new XmlDocument();

                EntityTypeNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "ENTITYTYPE", "");
                EntityTypeXMLDocument.AppendChild(EntityTypeNode);

                EntityTypeIDNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "ENTITYTYPEID", "");
                EntityTypeIDNode.InnerText = TypeID;
                EntityTypeNode.AppendChild(EntityTypeIDNode);

                EntityTypeNameNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "ENTITYTYPENAME", "");
                EntityTypeNameNode.InnerText = TypeName;
                EntityTypeNode.AppendChild(EntityTypeNameNode);

                TableNameNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                TableNameNode.InnerText = TableName;
                EntityTypeNode.AppendChild(TableNameNode);

                EntityBuiltIntypeNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "ISBUILTIN", "");
                EntityBuiltIntypeNode.InnerText = IsBuiltIn;
                EntityTypeNode.AppendChild(EntityBuiltIntypeNode);

                EntityTypeNode.AppendChild(EntityTypeXMLDocument.ImportNode(TypeNode.SelectSingleNode("PRIMARYKEY"), true));

                TypeDescriptionNode = EntityTypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEDESCRIPTION", "");
                TypeDescriptionNode.InnerText = Description;
                EntityTypeNode.AppendChild(TypeDescriptionNode);
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }


        /// <summary>
        /// <CreatedDate>04 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML which is required for adding a row in the ALOM_SCHEMA table.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXMLDocument">The DOM Object representing the InputXML defined in the caller function.</param>
        /// <param name="FieldNode">The DOM Node representing the Field node. 
        /// The structure of this XML Node is give below:
        /// The XML will be compliant with Field.xsd.
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <param name="SchemaNode">This is a reference parameter and will hold the XML Node which is 
        /// created in this function and returned.
        /// The structure of this XML Node is give below:
        ///             |- SCHEMA
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE   
        ///        (*) Mandatory Element
        /// </param>
        public void generateSchemaXML(XmlDocument InputXMLDocument, XmlNode FieldNode, ref XmlNode SchemaNode)
        {
            int FieldChildCount = 0;    //Integer variable storing the Child count of the Field Node.
            try
            {
                SchemaNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "SCHEMA", "");

                FieldChildCount = FieldNode.ChildNodes.Count;
                for (int index = 0; index < FieldChildCount; index++)
                    SchemaNode.AppendChild(FieldNode.ChildNodes[index].CloneNode(true));
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
       }

        /// <summary>
        /// <CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML Schema (XSD) for all the EntityTypes that are created in the system.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">This XML has all the data depending on which the XSD is to be generated.
        /// The structure of this XML Node is give below:
        /// The XML will be compliant with Type.xsd.
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- NAME          (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        private void generateXSD(XmlNode TypeNode)
        {
            int                     MaxMinOccurs        = 0;    // An Integer variable which stores the Max & Min occurance of an element.
            XmlNode                 FieldCollectionNode = null; //The DOM Node representing the FieldCollection node.
            XmlNode                 NameNode            = null; //The DOM Node representing the Name node.
            XmlNodeList             FieldNodes          = null; //The DOM Node representing the list of Field nodes.
            XmlNode                 FieldNameNode       = null; //The DOM Node representing the Fieldname node.
            XmlNode                 FieldTypeNode       = null; //The DOM Node representing the FieldType node.
            XmlNode                 IsMandatoryNode     = null; //The DOM Node representing the IsMandatory node.
            XmlNode                 IsPrimaryKeyNode    = null; //The DOM Node representing the IsPrimaryKey node.

            String                  FieldName           = null; //The String variable storing the value of FieldName.
            String                  FieldType           = null; //The String variable storing the value of FieldType.
            String                  IsMandatory         = null; //The String variable storing the value of IsMandatory.
            String                  IsPrimaryKey        = null; //The String variable storing the value of IsPrimaryKey.
            StringBuilder           FilePath            = null; //The StringBuilder object storing the XSD file path.

            XmlSchema               SchemaNode          = null; //This contains the definition of a XML schema.
            XmlSchemaInclude        SchemaInclude       = null; //Used to define the includes in an XML schema.
            XmlSchemaElement        elementTypeName     = null; //Used to describe a TypeName element in a XML document.
            
            XmlSchemaSequence       SchemaSequence      = null; //Used to define the sequence of elements within the containing element.
            XmlSchemaElement        elementFieldName    = null; //Used to describe a FieldName element in a XML document.
            XmlNamespaceManager     NameSpaceMgr        = null; //Used to add a namespace to the XSD.
            Stream                  StreamObj           = null; //Stream object used for creating an XSD file.
            StreamWriter            StreamWriterObject  = null; //StreamWriter object used for writing the XNL schema to the file.
            String                  InstalledPath       = null;
            XmlSchemaComplexType    SchemacomplexType   = null; //Used to define a complex type.
            IXMLRepository          ObjXMLManager       = null;
            String                  XSDFilePath         = null;

            try
            {
                NameNode = TypeNode.SelectSingleNode("TYPENAME");

                FieldCollectionNode = TypeNode.SelectSingleNode("FIELDCOLLECTION");
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");

                SchemaNode = new XmlSchema();
                // <Assetlink:include schemaLocation="Common.xsd" />
                SchemaInclude = new XmlSchemaInclude();
                SchemaInclude.SchemaLocation = "../System/Common.xsd";
                SchemaNode.Includes.Add(SchemaInclude);

                //Including XSD file path for XML column.
                //Getting the fields which have XML data type.
                XmlNodeList XMLColumnNodes = null;
                XMLColumnNodes = TypeNode.SelectNodes("//TYPE/FIELDCOLLECTION/FIELD[FIELDTYPE/text()='XML']");
                if ((XMLColumnNodes != null) && (XMLColumnNodes.Count > 0))
                {
                    ObjXMLManager = OMDataManager.XMLRepository; //new XMLManager(TenantName);
                    foreach (XmlNode XMLColumnNode in XMLColumnNodes)
                    {
                        if (XMLColumnNode.SelectSingleNode("XSDPATH") != null)
                        {
                            //if ((XMLColumnNode.SelectSingleNode("FIELDTYPE").Attributes["XSDFilePath"].Value != null)
                            //    && (XMLColumnNode.SelectSingleNode("FIELDTYPE").Attributes["XSDFilePath"].Value.ToString().Trim() != ""))
                            //{
                                //Get the XSD file at application's XSD type location.
                                XSDFilePath = null;
                                ObjXMLManager.RetrieveXSDPath(NameNode.InnerText.Trim().ToUpper(), XMLColumnNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper(), out XSDFilePath);

                                //If XSD file path contains NULL or empty string then the XSD file is not present.
                                if ((XSDFilePath != null) && (XSDFilePath.Trim().Length > 0))
                                {
                                    //Include XSD file like <Assetlink:include schemaLocation="Common.xsd" />
                                    SchemaInclude = new XmlSchemaInclude();
                                    SchemaInclude.SchemaLocation = "../Type/" + NameNode.InnerText.Trim().ToUpper() + "_" + XMLColumnNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper() + ".xsd";
                                    SchemaNode.Includes.Add(SchemaInclude);
                                }
                            //}
                        }
                    }
                }
                //End of including XSD file path for XML column.

                // <Assetlink:element name="<TypeName>"/>
                elementTypeName = new XmlSchemaElement();
                SchemaNode.Items.Add(elementTypeName);
                elementTypeName.Name = NameNode.InnerText.ToUpper();

                // <Assetlink:complexType>
                SchemacomplexType = new XmlSchemaComplexType();
                elementTypeName.SchemaType = SchemacomplexType;

                // <Assetlink:sequence>
                SchemaSequence = new XmlSchemaSequence();
                SchemacomplexType.Particle = SchemaSequence;

                foreach (XmlNode FieldNode in FieldNodes)
                {
                    FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                    FieldName = FieldNameNode.InnerText;

                    FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                    FieldType = FieldTypeNode.InnerText;

                    IsMandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                    if (IsMandatoryNode == null)
                        IsMandatory = "FALSE";
                    else IsMandatory = IsMandatoryNode.InnerText;

                    IsPrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");
                    if (IsPrimaryKeyNode == null)
                        IsPrimaryKey = "FALSE";
                    else IsPrimaryKey = IsPrimaryKeyNode.InnerText;
                    
                    //<Assetlink:element name="<FieldName>" type="<datatype>"/>
                    elementFieldName = new XmlSchemaElement();
                    SchemaSequence.Items.Add(elementFieldName);
                    elementFieldName.Name = FieldName;

                    switch (FieldType.Trim().ToUpper())
                    {
                        case "STRING":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("string", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "BOOLEAN":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("BOOLEAN");//, "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "INTEGER":
                            if (IsPrimaryKey.ToUpper() == "TRUE")
                                elementFieldName.SchemaTypeName = new XmlQualifiedName("IDRULE");
                            else
                                elementFieldName.SchemaTypeName = new XmlQualifiedName("integer", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "DECIMAL":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("decimal", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "DATETIME":

                            elementFieldName.SchemaTypeName = new XmlQualifiedName("DateTimeType");//, "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "XML":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName(Convert.ToString(FieldName.Trim().ToUpper() + "TYPE"));
                            break;
                        case "CURRENCY":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("CURRENCY");//, "http://www.w3.org/2001/XMLSchema");
                            break;
                    }

                    if ((IsMandatory.ToUpper() == "TRUE") || (IsPrimaryKey.ToUpper() == "TRUE"))
                    {
                        MaxMinOccurs = 1;
                        elementFieldName.MaxOccurs = MaxMinOccurs;
                        elementFieldName.MinOccurs = MaxMinOccurs;
                    }
                    else
                    {
                        MaxMinOccurs = 1;
                        elementFieldName.MaxOccurs = MaxMinOccurs;
                        MaxMinOccurs = 0;
                        elementFieldName.MinOccurs = MaxMinOccurs;
                    }
                }

                NameSpaceMgr = new XmlNamespaceManager(new NameTable());
                NameSpaceMgr.AddNamespace("Assetlink", "http://www.w3.org/2001/XMLSchema");

                //Write the XML Schema to a file.
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");
                FilePath = new StringBuilder();
                FilePath.AppendFormat("{0}Configuration\\XSD\\Type\\{1}.xsd", InstalledPath, NameNode.InnerText);
                
                StreamObj = new FileStream(FilePath.ToString(), System.IO.FileMode.Create, System.IO.FileAccess.ReadWrite);
                StreamWriterObject = new StreamWriter(StreamObj);
                SchemaNode.Write(StreamWriterObject, NameSpaceMgr);
                StreamWriterObject.Close();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>04 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will get the AccessorRoleID for the given RoleName.
        /// </Purpose>
        /// </summary>
        /// <param name="RoleName">String variable storing the RoleName for which the RoleID is to be retrieved.</param>
        /// <param name="AccessorRoleID">This is a reference parameter and contains the AccessorRoleID.</param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the Role from the database.
        /// </returns>
        private OMError getAccessorRoleID(String RoleName, ref String AccessorRoleID)
        {
            XmlDocument AccessorRoleXMLDocument = null;     //DOM Object representing the AccessorRole XML Document.
            XmlNode AccessorRoleNode            = null;     //DOM Object representing the AccessorRole Node.
            AccessorRole AccessorRoleObject     = null;     //AccessorRole Object.
            String AccessorRoleXML              = null;     //String variable storing the AccessorRole XML.
            OMError ReadErrorObj                = null;     //OMError object for read record in the database.

            try
            {
                AccessorRoleObject = new AccessorRole(OMDataManager);
                AccessorRoleObject.InternalCall = true;

                ReadErrorObj = AccessorRoleObject.ReadRoleByName("EVERYONE", ref AccessorRoleXML);

                if (ReadErrorObj != null)
                    return ReadErrorObj;

                AccessorRoleXMLDocument = new XmlDocument();
                AccessorRoleXMLDocument.LoadXml(AccessorRoleXML);

                AccessorRoleNode = AccessorRoleXMLDocument.SelectSingleNode("ACCESSORROLECOLLECTION/ACCESSORROLE");

                AccessorRoleID = AccessorRoleNode.SelectSingleNode("ACCESSORROLEID").InnerText;
                
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

     
        /// <summary>
        /// <CreatedDate>04 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will add a clause for the newly created EntityType in the DEFAULT RULE for the EVERYONE 
        /// user role.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeID">String variable storing the EntityTypeID for which the Clause is to be
        /// added in the DEFAULT RULE.</param>
        /// <returns>
        /// NULL: If the creation was successful.
        /// Object of OMError if an error occurred while adding the clause in the DEFAULT RULE.
        /// </returns>
        private OMError addClauseInDefaultRule(String EntityTypeID)
        {
            XmlDocument AccessRuleXMLDocument = null;   //DOM Object representing the AccessRuleXML.
            String AccessRuleXML              = null;   //String variable storing the AccessRule XML.
            String AccessRuleID               = null;   //String variable storing the AccessRuleID.
            String RoleID                     = null;   //String variable storing the RoleID.
            //int ClauseNodeCount               = 0;      //Integer variable storing the Count of the Clause Nodes in the XML.
            ulong AccessRuleClauseID           = 0;      //Long variable storing the AccessRuleClauseID.
            AccessRule AccessRuleObject       = null;   //AccessRule object.
            AccessRuleClause ClauseObj        = null;   //The AccessRuleClauses Object.
            OMError CreateErrorObj            = null;   //OMError object for create record in the database.
            OMError ReadErrorObject           = null;   //OMError object for read record in the database.
            XmlNode AccessRuleCollectionNode  = null;   //DOM Object representing the AccessRuleCollection Node.
            XmlNode AccessRuleNode            = null;   //DOM Object representing the AccessRule Node.
            XmlNode AccessRuleClauseNode      = null;   //DOM Object representing the AccessRuleClause Node.
            //XmlNode ClauseCollectionNode      = null;   //DOM Object representing the ClauseCollection Node.
            XmlNode AccessRuleClauseIDNode    = null;   //DOM Object representing the AccessRuleClauseID Node.
            XmlNode AccessRuleIDNode          = null;   //DOM Object representing the AccessRuleID Node.
            XmlNode AccessorRoleIDNode        = null;   //DOM Object representing the AccessorRoleID Node.
            XmlNode ObjectTypeIDNode          = null;   //DOM Object representing the ObjectTypeID Node.
            XmlNode ObjectTypeNode            = null;   //DOM Object representing the ObjectType Node.
            XmlNode AccessorTypeNode          = null;   //DOM Object representing the AccessorType Node.
            XmlNode PrivilegeNameNode         = null;   //DOM Object representing the PrivilegeName Node.
            XmlNode PrivilegeValueNode        = null;   //DOM Object representing the PrivilegeValue Node.
            XmlNode OperationTypeAttr         = null;   //DOM Object representing the OperationType attribute.
            
            try
            {
                //Get the DEFAULT RULE and RuleID.
                AccessRuleObject = new AccessRule(OMDataManager);
                AccessRuleObject.InternalCall = true;

                ReadErrorObject = AccessRuleObject.ReadRuleByName("DEFAULT RULE", ref AccessRuleXML);

                if (ReadErrorObject != null)
                    return ReadErrorObject;

                AccessRuleXMLDocument = new XmlDocument();
                AccessRuleXMLDocument.LoadXml(AccessRuleXML);

                AccessRuleCollectionNode = AccessRuleXMLDocument.SelectSingleNode("ACCESSRULECOLLECTION");

                AccessRuleNode = AccessRuleXMLDocument.SelectSingleNode("ACCESSRULECOLLECTION/ACCESSRULE");

                AccessRuleID = AccessRuleNode.SelectSingleNode("ACCESSRULEID").InnerText;

                //Get the RoleID for EVERYONE Role.
                ReadErrorObject = getAccessorRoleID("EVERYONE", ref RoleID);

                if (ReadErrorObject != null)
                    return ReadErrorObject;

                /*ClauseCollectionNode = AccessRuleNode.SelectSingleNode("ACCESSRULECLAUSESCOLLECTION");

                if (ClauseCollectionNode != null)
                {

                    ClauseNodeCount = ClauseCollectionNode.ChildNodes.Count;

                    //Remove all the Clause Nodes present in the XML.
                    if (ClauseNodeCount > 0)
                        for (int i = ClauseNodeCount - 1; i >= 0; i--)
                            ClauseCollectionNode.RemoveChild(ClauseCollectionNode.ChildNodes[i]);
                }
                else
                {
                    ClauseCollectionNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSRULECLAUSESCOLLECTION", "");
                    AccessRuleNode.AppendChild(ClauseCollectionNode);
                }*/

                //Prepare the new Clause Node for the given EntityTypeID and Append this node to the ClauseCollectionNode.
                AccessRuleClauseNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSRULECLAUSES", "");
                //ClauseCollectionNode.AppendChild(AccessRuleClauseNode);

                OperationTypeAttr = AccessRuleXMLDocument.CreateNode(XmlNodeType.Attribute, "OperationType", "");
                OperationTypeAttr.Value = "NEW";
                AccessRuleClauseNode.Attributes.SetNamedItem(OperationTypeAttr);

                ReadErrorObject = UniqueID.Read(ref AccessRuleClauseID);

                if (ReadErrorObject != null)
                    return ReadErrorObject;

                AccessRuleClauseIDNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSRULECLAUSEID", "");
                AccessRuleClauseIDNode.InnerText = AccessRuleClauseID.ToString();
                AccessRuleClauseNode.AppendChild(AccessRuleClauseIDNode);

                AccessRuleIDNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSRULEID", "");
                AccessRuleIDNode.InnerText  = AccessRuleID;
                AccessRuleClauseNode.AppendChild(AccessRuleIDNode);

                AccessorRoleIDNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSORID", "");
                AccessorRoleIDNode.InnerText = RoleID;
                AccessRuleClauseNode.AppendChild(AccessorRoleIDNode);

                ObjectTypeIDNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "OBJECTTYPEID", "");
                ObjectTypeIDNode.InnerText = EntityTypeID;
                AccessRuleClauseNode.AppendChild(ObjectTypeIDNode);

                ObjectTypeNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "OBJECTTYPE", "");
                ObjectTypeNode.InnerText = "ENTITY";
                AccessRuleClauseNode.AppendChild(ObjectTypeNode);

                AccessorTypeNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "ACCESSORTYPE", "");
                AccessorTypeNode.InnerText = "ROLE";
                AccessRuleClauseNode.AppendChild(AccessorTypeNode);

                PrivilegeNameNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "PRIVILEGENAME", "");
                PrivilegeNameNode.InnerText = "DELETE";
                AccessRuleClauseNode.AppendChild(PrivilegeNameNode);

                PrivilegeValueNode = AccessRuleXMLDocument.CreateNode(XmlNodeType.Element, "PRIVILEGEVALUE", "");
                PrivilegeValueNode.InnerText = OMPrivileges.DELETE.GetHashCode().ToString();
                AccessRuleClauseNode.AppendChild(PrivilegeValueNode);

                //Add the Clause to the DEFAULT RULE.
                //CreateErrorObj = AccessRuleObject.Update(AccessRuleCollectionNode.InnerXml);
                ClauseObj = new AccessRuleClause(OMDataManager);
                CreateErrorObj = ClauseObj.Create(AccessRuleClauseNode.OuterXml);
                if (CreateErrorObj != null)
                    return CreateErrorObj;

                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>10 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed to outside.
        /// The function checks whether EntityType Exists  in the system or Not.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeName">This contains the EntityTypeName which is to be searched.</param>
        /// <returns>
        /// true: If EntityType exist.
        /// false:If EntityType not exist.
        /// </returns>
        public bool isEntityTypeExists(String EntityTypeName)
        {
            OMError ReadErrorObj               = null;   //The OMError object for reading data from DB.
            String OutputXML                   = null;   //The String variable storing the Output XML.

            try
            {
                ReadErrorObj = OMDataManager.DataProvider.AddClause("ENTITYTYPENAME", OMFieldDataType.STRING,
                                        OMOperator.EQUALTO, EntityTypeName,
                                        OMConjunction.NOTHING);

                if (ReadErrorObj != null)
                    return false;

                ReadErrorObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref OutputXML);

                if (ReadErrorObj != null || OutputXML == null || OutputXML == "")
                    return false;
                else
                    return true;
            }
            catch (Exception ExceptionObj)
            {
               throw ExceptionObj;
            }

        }

        /// <summary>
        /// <CreatedDate>16 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete an entry in the ALOM_SCHEMA table depending on the SchemaID.
        /// </Purpose>
        /// </summary>
        /// <param name="SchemaID">Long variable storing the SchemaID.</param>
        /// <param name="OMErrorCode">OMErrorCodes variable storing the ErrorCode.</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the schema from the ALOM_SCHEMA.
        /// </returns>
        private OMError deleteSchema(ulong SchemaID, OMErrorCodes OMErrorCode)
        {
            OMError DeleteErrObj   = null;    //The OMError object for deleting data from DB.
            OMError ReturnErrorObj = null;    //The OMError object for returning form OM.

            try
            {
                DeleteErrObj = OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER,
                                                    OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING);

                if (DeleteErrObj != null)
                    return DeleteErrObj;

                DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_SCHEMA");

                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCode,
                        "Field Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    return ReturnErrorObj;
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCode,
                        "Field Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                        OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }

        }

        /// <summary>
        /// <CreatedDate>16 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete the clause for the EntityTypeID in the DEFAULT RULE.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeID">Long variable containing the EntityTypeID for which the clause from the 
        /// DEFAULT RULE is to be deleted.</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the clause from the ALOM_ACCESSRULECLAUSES table.
        /// </returns>
        private OMError deleteClauseFromDefaultRule(ulong EntityTypeID)
        {
            OMError DeleteErrorObj            = null;     //OMError object for delete record in the database.
            OMError ReadErrorObject           = null;     //OMError object for read record in the database.
            OMError ReturnErrorObj            = null;     //OMError object for returning from OM.
            String AccessRuleXML              = null;     //String variable storing the AccessRule XML.
            XmlNode ClauseCollectionNode      = null;     //DOM Object representing the ClauseCollection Node.
            XmlNode ClauseNode                = null;     //DOM object representing the Clause Node.
            AccessRule AccessRuleObject       = null;     //AccessRule Object.
            AccessRuleClause ClauseObject     = null;     //AccessRuleClause Object.
            XmlDocument AccessRuleXMLDocument = null;     //DOM Object representing the AccessRuleXML.
            ulong ClauseID                     = 0;        //Long variable storing the CLauseID.
            
            try
            {
                AccessRuleObject = new AccessRule(OMDataManager);
                AccessRuleObject.InternalCall = true;

                ReadErrorObject = AccessRuleObject.ReadRuleByName("DEFAULT RULE", ref AccessRuleXML);

                if (ReadErrorObject != null)
                    return ReadErrorObject;

                AccessRuleXMLDocument = new XmlDocument();
                AccessRuleXMLDocument.LoadXml(AccessRuleXML);

                ClauseCollectionNode = AccessRuleXMLDocument.SelectSingleNode("ACCESSRULECOLLECTION/ACCESSRULE/ACCESSRULECLAUSESCOLLECTION");

                if (ClauseCollectionNode != null)
                {
                    ClauseNode = ClauseCollectionNode.SelectSingleNode("ACCESSRULECLAUSES[OBJECTTYPEID='" + EntityTypeID.ToString() + "']");

                    if (ClauseNode != null)
                    {
                        ClauseID = Convert.ToUInt64(ClauseNode.SelectSingleNode("ACCESSRULECLAUSEID").InnerText);

                        ClauseObject = new AccessRuleClause(OMDataManager);
                        ClauseObject.InternalCall = true;

                        DeleteErrorObj = ClauseObject.Delete(ClauseID);

                        if (DeleteErrorObj != null)
                            return DeleteErrorObj;
                    }
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ACCESSRULECLAUSE_DELETION_FAILED,
                        "Clause Deletion for Default Rule Failed", OMErrorObjectType.ACCESSRULECLAUSE, "AccessRuleClauses",
                        OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>18 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete all the associations from the ALOM_RELATIONSHIPTYPES table for the given EntityTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeID">Long variable containing the EntityTypeID for which the associations from the 
        /// ALOM_RELATIONSHIPTYPES are to be deleted.</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the associations from the ALOM_RELATIONSHIPTYPES table.
        /// </returns>
        private OMError deleteTypeAssociations(ulong EntityTypeID)
        {
            OMError DeleteErrObj               = null;    //OMError object for deleting data from DB.
            OMError ReturnErrorObj             = null;    //OMError object for returning from OM.
            OMError ReadErrorObject            = null;    //OMError object for read record in the database.
            String AssociationXML              = null;    //String variable storing the AssociationXML XML.
            String ContainerTypeID             = null;    //String variable storing the ContainerTypeID.
            String ContainsTypeID              = null;    //String variable storing the ContainsTypeID.
            ulong lContainerTypeID              = 0;       //Long variable storing the ContainerTypeID.
            ulong lContainsTypeID               = 0;       //Long variable storing the ContainsTypeID.
            XmlNode RelationshipTypeCollNode   = null;    //DOM Object representing the ObjectTypesAssoCollection Node.
            XmlNodeList RelationshipTypeNodes  = null;    //DOM Object representing the ObjectTypeAsso Nodes.
            XmlDocument AssociationXMLDocument = null;    //DOM Object representing the EntityTypeXML.

            RelationshipType RelationshipTypeObject = null;    //RelationshipType Object.

            try
            {
                RelationshipTypeObject = new RelationshipType(OMDataManager);
                RelationshipTypeObject.InternalCall = true;

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table where the EntityTypeID maps to the 
                //ContainerTypeID.

                ReadErrorObject = RelationshipTypeObject.ReadAllByContainerType(EntityTypeID, "ENTITY", "",
                                                                              ref AssociationXML);

                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                    return ReadErrorObject;

                if ((AssociationXML != "") && (AssociationXML != null))
                {
                    AssociationXMLDocument = new XmlDocument();
                    AssociationXMLDocument.LoadXml(AssociationXML);

                    RelationshipTypeCollNode = AssociationXMLDocument.SelectSingleNode("RELATIONSHIPTYPECOLLECTION");
                    RelationshipTypeNodes = RelationshipTypeCollNode.SelectNodes("RELATIONSHIPTYPE");

                    if (RelationshipTypeNodes.Count > 0)
                    {
                        foreach (XmlNode RelationshipTypeNode in RelationshipTypeNodes)
                        {
                            ContainerTypeID = null;
                            ContainsTypeID = null;
                            lContainerTypeID = 0;
                            lContainsTypeID = 0;

                            ContainerTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINERTYPEID").InnerText;
                            ContainsTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINSTYPEID").InnerText;

                            lContainerTypeID = Convert.ToUInt64(ContainerTypeID);
                            lContainsTypeID = Convert.ToUInt64(ContainsTypeID);

                            DeleteErrObj = RelationshipTypeObject.Delete(lContainerTypeID, lContainsTypeID);

                            if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                                return DeleteErrObj;
                        }
                    }
                }

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table where the EntityTypeID maps to the 
                //ContainsTypeID.

                AssociationXML = null;

                ReadErrorObject = RelationshipTypeObject.ReadAllByContainsType(EntityTypeID, "ENTITY", "",
                                                                              ref AssociationXML);

                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                    return ReadErrorObject;

                if ((AssociationXML != "") && (AssociationXML != null))
                {
                    AssociationXMLDocument = new XmlDocument();
                    AssociationXMLDocument.LoadXml(AssociationXML);

                    RelationshipTypeCollNode = AssociationXMLDocument.SelectSingleNode("RELATIONSHIPTYPECOLLECTION");
                    RelationshipTypeNodes = RelationshipTypeCollNode.SelectNodes("RELATIONSHIPTYPE");

                    if (RelationshipTypeNodes.Count > 0)
                    {
                        foreach (XmlNode RelationshipTypeNode in RelationshipTypeNodes)
                        {
                            ContainerTypeID = null;
                            ContainsTypeID = null;
                            lContainerTypeID = 0;
                            lContainsTypeID = 0;

                            ContainerTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINERTYPEID").InnerText;
                            ContainsTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINSTYPEID").InnerText;

                            lContainerTypeID = Convert.ToUInt64(ContainerTypeID);
                            lContainsTypeID = Convert.ToUInt64(ContainsTypeID);

                            DeleteErrObj = RelationshipTypeObject.Delete(lContainerTypeID, lContainsTypeID);

                            if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                                return DeleteErrObj;
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ASSOCIATION_DELETION_FAILED,
                        "EntityType FacetType Association Deletion Failed", OMErrorObjectType.ASSOCIATION, 
                        "EntityType FacetType Association", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        #endregion PrivateMethods

        #region Methods
        /// <summary>
        /// <CreatedDate>03 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will create an EntityType based on information provided in the InputXML.
        /// The structure of InputXML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">
        /// The InputXML will be compliant with Type.xsd.
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- TYPENAME      (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If the creation was successful.
        /// Object of OMError if an error occurred while creating the EntityType.
        /// </returns>
        /// Code Change Comments
        /*  Bug Fix : 4380, Date : 19 August 2008, Changed By : Sagar Dasalkar
         *  Description : If Entity Type creation operation (Adding entries in ALOM_SCHEMA, adding entries in ALENT_ENTITYTYPE and creating table) fails, 
         *  before returning OMError object the lock obtained on an object has been released.
         */
        public OMError Create(String InputXML)
        {
            XmlDocument     InputXMLDocument            = null;     //DOM Object representing the InputXML.
            XmlDocument     EntityTypeXMLDocument       = null;     //DOM Object representing the EntityTypeXML.
            XmlNodeList     FieldNodes                  = null;     //DOM Node representing the list of Field nodes.
            XmlNode         NameNode                    = null;     //DOM Node representing the Name node.
            XmlNode         TableNameNode               = null;     //DOM Node representing the TableName node.
            XmlNode         FieldCollectionNode         = null;     //DOM Node representing the FieldCollection node.
            XmlNode         SchemaNode                  = null;     //DOM Node representing the Schema node.
            XmlNode         TypeNode                    = null;     //DOM Node representing the Type node of the InputXML.
            String          TypeName                    = null;     //String variable storing the value of TypeName.
            String          TypeID                      = null;     //String variable storing the value of TypeID.
            String          TableName                   = null;     //String variable storing the value of TableName.
            StringBuilder   Message                     = null;     //StringBuilder object storing the Message that is to be logged.
            String          ModifiedInputXML            = null;     //String variable storing the modified InputXML.
            OMError         CreateErrorObj              = null;     //OMError object for create record in the database.
            OMError         ReturnErrorObj              = null;     //OMError object for returning from OM.
            StringBuilder   QueryBuilder                = null;
            String          TempQuery                   = null;
            Boolean         IsOMDataManagerInitialized  = false;     //Boolean variable storing whether OMDataManagerWriter is initialized or not.
            ulong AssociationID                         = 0;         //Ulong variable storing AssociationID for Field and NamedHierarchy Association
            ulong NamedHierarchyID;                                  //Ulong variable storing NamedHierarchyID for Field and NamedHierarchy Association
            ulong SchemaID;                                          //Ulong variable storing SchemaID for Field and NamedHierarchy Association
            String          CreateTableQuery            = null;
            
            XmlDocument     DataType_SizeMappingXMLDoc  = null;
            XmlNode         FieldSizeNode               = null;
            XmlNode         DefaultFieldSizeNode        = null;
            XmlNode         DataTypeNode                = null;          
            
            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Create API of EntityType class in OM. Input Parameter InputXML: ", InputXML);
                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManagerWriter and OMDataManagerReader is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                DataType_SizeMappingXMLDoc      = new XmlDocument();
                InputXMLDocument                = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                TypeNode = InputXMLDocument.SelectSingleNode("TYPE");
                NameNode = TypeNode.SelectSingleNode("TYPENAME");
                TypeName = NameNode.InnerText;
                TypeID = TypeNode.SelectSingleNode("TYPEID").InnerText;

                if ((TypeName == null) || (TypeName.Length <= 0))
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                            "EntityType Should Contain Meaningful Name", OMErrorObjectType.ENTITYTYPE,
                            "EntityType", OMErrorType.ERROR, null, null);
                    return ReturnErrorObj;
                }
                else if (isEntityTypeExists(TypeName) == true) //Check whether the given EntityType already exists in the system.
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED, 
                            "EntityType Already Exists", OMErrorObjectType.ENTITYTYPE, 
                            "EntityType", OMErrorType.ERROR, null, null);
                   // WriteFailed = true;
                    return ReturnErrorObj;
                }
                
                //Generate the TableName.
                TableName = generateTableName(TypeName);
                //Add record in ALOM_ENTITYTYPE Table.
                generateEntityTypeXML(TypeNode, TableName, ref EntityTypeXMLDocument);
                //Initialiazing the batch insert query builder
                QueryBuilder = new StringBuilder();
                CreateErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_ENTITYTYPE",
                                                                         EntityTypeXMLDocument.InnerXml, 
                                                                         ref TempQuery);
                QueryBuilder.Append(TempQuery);

              
                if (CreateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED, "EntityType Creation Failed",
                        OMErrorObjectType.ENTITYTYPE, "EntityType", CreateErrorObj.ErrorType,
                        CreateErrorObj.ExceptionObject, CreateErrorObj.ErrorXML);
                   // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Add record in ALOM_SCHEMA Table.
                FieldCollectionNode = InputXMLDocument.SelectSingleNode("TYPE/FIELDCOLLECTION");
                if (FieldCollectionNode == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED, "Fields Not Found",
                        OMErrorObjectType.ENTITYTYPE, "EntityType", OMErrorType.ERROR, null, "");
                   // WriteFailed = true;
                    return ReturnErrorObj;
                }

                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                if (FieldNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED, "Fields Not Found", 
                       OMErrorObjectType.ENTITYTYPE, "EntityType", OMErrorType.ERROR, null, "");
                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                DataType_SizeMappingXMLDoc.LoadXml(OMDataManager.DataProvider.GetDataTypesAndTheirSizes());
                foreach (XmlNode FieldNode in FieldNodes)
                {
                    SchemaNode = null;
                    TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                    TableNameNode.InnerText = TableName;
                    FieldNode.AppendChild(TableNameNode);

                    //Anand :  This code is added because FieldSize is not passed for DataTypes other than
                    // String. As DAL code consider's size not to be null we need to fill in appropiate
                    // sizes depending upon the DataType(also this is database dependent).
                    FieldSizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                    if ((FieldSizeNode == null) || (String.IsNullOrEmpty(FieldSizeNode.InnerText)))
                    {
                        DataTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                        if (FieldSizeNode == null)
                        {
                            FieldSizeNode = FieldNode.OwnerDocument.CreateElement("FIELDSIZE");
                            FieldNode.AppendChild(FieldSizeNode);
                        }
                        DefaultFieldSizeNode = DataType_SizeMappingXMLDoc.DocumentElement.SelectSingleNode(DataTypeNode.InnerText.ToUpper());
                        if (DefaultFieldSizeNode != null)
                            FieldSizeNode.InnerText = DefaultFieldSizeNode.InnerText;
                        else
                            FieldSizeNode.InnerText = "0";
                    }

                    generateSchemaXML(InputXMLDocument, FieldNode, ref SchemaNode);

                    CreateErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_SCHEMA", SchemaNode.OuterXml,
                                                                                    ref TempQuery);


                  
                    //Add Field and NamedHierarchy Association in AssoFieldNamedHierarchy, Code added on date 24th August 2006 by Pallavi
                    if ((CreateErrorObj == null) && (FieldNode.SelectSingleNode("ISNAMEDHIERARCHY").InnerText.ToUpper() == "TRUE") && (FieldNode.SelectSingleNode("NAMEDHIERARCHYID") != null))
                    {
                        Assetlink.UniqueID.Read(ref AssociationID);
                        NamedHierarchyID    =   Convert.ToUInt64(FieldNode.SelectSingleNode("NAMEDHIERARCHYID").InnerText);
                        SchemaID             =   Convert.ToUInt64(FieldNode.SelectSingleNode("SCHEMAID").InnerText);
                        AssoFieldNamedHierarchy AssoFieldNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                        CreateErrorObj      =   AssoFieldNHObj.Create(AssociationID, NamedHierarchyID, SchemaID, "ASC");

                        if (CreateErrorObj != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                                "Field and NamedHierarchy  Association Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                                CreateErrorObj.ErrorType, CreateErrorObj.ExceptionObject, CreateErrorObj.ErrorXML);
                            // WriteFailed = true;
                            return ReturnErrorObj;
                        }

                    }
                


                    if (CreateErrorObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                            "Field Creation Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                            CreateErrorObj.ErrorType, CreateErrorObj.ExceptionObject, CreateErrorObj.ErrorXML);
                        // WriteFailed = true;
                        return ReturnErrorObj;
                    }

                    QueryBuilder.Append(TempQuery);
                }

                //Create Table in the Database.
                TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                TableNameNode.InnerText = TableName;
                TypeNode.AppendChild(TableNameNode);

                ModifiedInputXML = InputXMLDocument.InnerXml;
                //Anand : As in some databases(Eg. Teradata)we cannot fire DML and DDL statements together we have bunch them into
                //        two sets and then fire.
                CreateErrorObj = OMDataManager.DataProvider.PrepareCreateTableQuery(ModifiedInputXML, ref CreateTableQuery);
                                
                if (CreateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                        "Table Creation Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", CreateErrorObj.ErrorType, CreateErrorObj.ExceptionObject,
                        CreateErrorObj.ErrorXML);
                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Executing the Query
                Monitor.Enter(Lock);
                
                OMError ErrorObj = OMDataManager.DataProvider.Execute(QueryBuilder.ToString());
                if (ErrorObj != null)
                {
                    Monitor.Exit(Lock);
                    return ErrorObj;
                }

                ErrorObj = OMDataManager.DataProvider.Execute(CreateTableQuery);
                if (ErrorObj != null)
                {
                    Monitor.Exit(Lock);
                    return ErrorObj;
                }

                //Add a clause for the Default Rule. The clause will provide Delete privilege for EveryOne. 
                CreateErrorObj = addClauseInDefaultRule(TypeID);
                
                if (CreateErrorObj != null)
                {
                    Monitor.Exit(Lock);
                    // WriteFailed = true;
                    return CreateErrorObj;
                }
                Monitor.Exit(Lock);

                //Generate XSD.
                generateXSD(TypeNode);

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();
                
                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Create(String InputXML) InputXML: ", InputXML);

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                        "EntityType Creation Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }

        /// <summary>
        /// <CreatedDate>16 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete an EntityType based on EntityTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeID">Long variable containing the EntityTypeID</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the EntityType. 
        /// </returns>
        public OMError Delete(ulong EntityTypeID)
        {
            OMError                 DeleteErrObj                = null;     //OMError object for deleting data from DB.
            OMError                 ReturnErrorObj              = null;     //OMError object for returning from OM.
            OMError                 ReadErrorObject             = null;     //OMError object for read record in the database.
            AccessorRole            AccessorRoleObj             = null;     //Object of AccessorRole class.
            //XmlNodeList           FieldNodes                  = null;     //DOM Node representing the list of Field nodes.
            //XmlNode               SchemaIDNode                = null;     //DOM Node representing the SchemaID node.
            XmlNode                 TableNameNode               = null;     //DOM Node representing the TableName node.
            XmlNode                 TypeNameNode                = null;     //DOM Node representing the TypeName node.
            XmlNode                 EntityTypeNode              = null;     //DOM Node representing the EntityType node.
            StringBuilder           Message                     = null;     //StringBuilder object storing the Message that is to be logged.
            String                  EntityTypeXML               = null;     //String variable storing the EntityType XML.
            //String                SchemaID                    = null;     //String variable storing the SchemaID.
            String                  TableName                   = null;     //String variable storing the TableName.
            String                  TypeName                    = null;     //String variable storing the TypeName.
            StringBuilder           FilePath                    = null;     //StringBuilder pbject storing the Path of the XSD.
            AccessRule              AccessRuleObject            = null;     //AccessRule Object.
            //ulong                 lSchemaID                   = 0;        //Long variable storing the SchemaID.
            String                  DelQry                      = null;
            //Boolean               WriteFailed                 = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            String                  InstalledPath               = null;
            XmlDocument             EntityTypeXMLDocument       = null;     //DOM Object representing the EntityTypeXML.
            Boolean                 IsOMDataManagerInitialized  = false;    //Boolean variable storing whether OMDataManagerWriter is initialized or not.
            AssoFieldNamedHierarchy FieldNamedHierAssoObject    = null;     //AssoFieldNamedHierarchy Object.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Delete API of EntityType class in OM. Input Parameter EntityTypeID: ", EntityTypeID.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }                

                // If Inherited base class member OMDataManagerWriter and OMDataManagerReader is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManagerWriter(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Delete all Accessor Roles associated with the EntityType from ALOM_ACCESSORROLE table.
                AccessorRoleObj = new AccessorRole(OMDataManager);
                AccessorRoleObj.InternalCall = true;

                DeleteErrObj = AccessorRoleObj.DeleteAllForEntityType(EntityTypeID);
                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "Accessor Role Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Delete all Access Rules associated with the EntityType from the ALOM_ACCESSRULE table.
                AccessRuleObject = new AccessRule(OMDataManager);
                AccessRuleObject.InternalCall = true;

                DeleteErrObj = AccessRuleObject.DeleteAllForEntityType(EntityTypeID);
                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "AccessRule Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Delete clause for this EntityType from the DEFAULT RULE.
                DeleteErrObj = deleteClauseFromDefaultRule(EntityTypeID);
                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        DeleteErrObj.ErrorMessage, OMErrorObjectType.ENTITYTYPE, 
                        DeleteErrObj.ObjectTypeName, DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table.
                DeleteErrObj = deleteTypeAssociations(EntityTypeID);
                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                       DeleteErrObj.ErrorMessage, OMErrorObjectType.ENTITYTYPE,
                       DeleteErrObj.ObjectTypeName, DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Delete all the associations from the ALOM_ASSOFILEDNAMEDHIERARCHY table.
                FieldNamedHierAssoObject = new AssoFieldNamedHierarchy(OMDataManager);
                FieldNamedHierAssoObject.InternalCall = true;

                DeleteErrObj = FieldNamedHierAssoObject.DeleteAllUsingObjectTypeID(EntityTypeID);
                if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                {
                   // WriteFailed = true;
                    return DeleteErrObj;
                }

                //Delete all the fields of this EntityType from the ALOM_SCHEMA table.
                ReadErrorObject = Read(EntityTypeID, ref EntityTypeXML);
                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                {
                   // WriteFailed = true;
                    return ReadErrorObject;
                }

                if (EntityTypeXML != null && EntityTypeXML != "")
                {
                    EntityTypeXMLDocument = new XmlDocument();
                    EntityTypeXMLDocument.LoadXml(EntityTypeXML);

                    EntityTypeNode = EntityTypeXMLDocument.SelectSingleNode("TYPE");

                    TableNameNode = EntityTypeNode.SelectSingleNode("TABLENAME");
                    TableName = TableNameNode.InnerText;

                    TypeNameNode = EntityTypeNode.SelectSingleNode("TYPENAME");
                    TypeName = TypeNameNode.InnerText;

                    /* FieldNodes = EntityTypeNode.SelectNodes("FIELDCOLLECTION/FIELD");

                     if (FieldNodes.Count >= 1)
                     {
                         foreach (XmlNode FieldNode in FieldNodes)
                         {
                             SchemaIDNode = FieldNode.SelectSingleNode("SCHEMAID");
                             SchemaID = SchemaIDNode.InnerText;
                             lSchemaID = Convert.ToUInt64(SchemaID);

                             DeleteErrObj = deleteSchema(lSchemaID, OMErrorCodes.ENTITY_TYPE_DELETION_FAILED);

                             if (DeleteErrObj != null)
                             {
                                // WriteFailed = true;
                                 return DeleteErrObj;
                             }
                         }
                     }*/

                    //Following code is commented by Kiran Abhang on date 10th March 2006.
                    //This code should be after DropTable call 
                    //because deletion of XML files and XSDs of object type is depend on ALOM_SCHEMA's XML fields.
                    //DelQry = "Delete from ALOM_SCHEMA where OBJECTTYPEID = " + EntityTypeID;
                    //DeleteErrObj = OMDataManager.DataProvider.Execute(DelQry);
                    //if (DeleteErrObj != null)
                    //    return DeleteErrObj;

                    ////Delete EntityType record from the ALOM_ENTITYTYPE table.
                    //DeleteErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                    //                              OMOperator.EQUALTO, EntityTypeID.ToString(), OMConjunction.NOTHING);
                    //if (DeleteErrObj != null)
                    //{
                    //    // WriteFailed = true;
                    //    return DeleteErrObj;
                    //}

                    //DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_ENTITYTYPE");
                    //if (DeleteErrObj != null)
                    //{
                    //    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                    //        "EntityType Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                    //        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    //    // WriteFailed = true;
                    //    return ReturnErrorObj;
                    //}

                    //Drop the table.
                    DeleteErrObj = OMDataManager.DataProvider.DropTable(TableName);
                    if (DeleteErrObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                            "Table Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                            DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                        // WriteFailed = true;
                        return ReturnErrorObj;
                    }

                    //Delete EntityType record from the ALOM_ENTITYTYPE table.
                    DeleteErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                  OMOperator.EQUALTO, EntityTypeID.ToString(), OMConjunction.NOTHING);
                    if (DeleteErrObj != null)
                    {
                        // WriteFailed = true;
                        return DeleteErrObj;
                    }

                    DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_ENTITYTYPE");
                    if (DeleteErrObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                            "EntityType Deletion Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                            DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                        // WriteFailed = true;
                        return ReturnErrorObj;
                    }

                    DelQry = "DELETE FROM ALOM_SCHEMA WHERE OBJECTTYPEID = " + EntityTypeID;
                    DeleteErrObj = OMDataManager.DataProvider.Execute(DelQry);
                    if (DeleteErrObj != null)
                        return DeleteErrObj;

                    //Delete the <ENTITYTYPENAME>.XSD
                    InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK","Installed Path");
                    FilePath = new StringBuilder();
                    FilePath.AppendFormat("{0}Configuration\\XSD\\Type\\{1}.xsd", InstalledPath, TypeName);
                    File.Delete(FilePath.ToString());
                }

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Delete API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Delete(ulong EntityTypeID) EntityTypeID:", EntityTypeID.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "EntityType Deletion Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                // WriteFailed = true;
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

              
            }
        }

        /// <summary>
        /// <CreatedDate>16 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete an EntityType based on EntityTypeName.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeName">String variable containing the EntityTypeName</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the EntityType. 
        /// </returns>
        public OMError Delete(String EntityTypeName)
        {
            OMError ReadErrObj       = null;   //OMError object for reading data from DB.
            OMError DeleteErrObj     = null;   //OMError object for deleting data from DB.
            OMError ReturnErrorObj   = null;   //OMError object for returning from OM.
            String EntityTypeXML     = null;   //String variable storing the EntityType XML.
            String TypeID            = null;   //String variable storing the value of TypeID.
            StringBuilder Message    = null;   //StringBuilder object storing the Message that is to be logged.
            ulong lTypeID             = 0;      //Long variable storing the value of TypeID.
            XmlNode EntityTypeNode   = null;   //DOM Node representing the EntityType node.
            XmlNode EntityTypeIDNode = null;   //DOM Node representing the EntityTypeID node.
            //Boolean WriteFailed      = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            XmlDocument EntityTypeXMLDocument  = null;    //DOM Object representing the EntityType XML.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManagerWriter is initialized or not.
            
            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Delete API of EntityType class in OM. Input Parameter EntityTypeName: ", EntityTypeName);

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManagerWriter and OMDataManagerReader is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }


                //Get the EntityTypeID from the Database.

                ReadErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPENAME", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, EntityTypeName.ToUpper(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                {
                   // WriteFailed = true;
                    return ReadErrObj;
                }

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref EntityTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                if (EntityTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    // WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Get the EntityTypeID from the EntityType XML.

                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeXML);

                EntityTypeNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPE");

                EntityTypeIDNode = EntityTypeNode.SelectSingleNode("ENTITYTYPEID");
                TypeID = EntityTypeIDNode.InnerText;

                lTypeID = Convert.ToUInt64(TypeID);

                DeleteErrObj = Delete(lTypeID);

                if (DeleteErrObj != null)
                {
                    // WriteFailed = true;
                    return DeleteErrObj;
                }

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Delete API of EntityType class in OM.");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Delete(String EntityTypeName) EntityTypeName: ", EntityTypeName);

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_DELETION_FAILED,
                        "EntityType Deletion Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                // WriteFailed = true;
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

               
            }
        }
        #endregion Methods
    }
}
