using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;
using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.OMCommon;
using Assetlink.Diagnostic;
using Assetlink.ObjectManager.NamedHierarchy;

namespace Assetlink.ObjectManager.ETFTManagement
{
    /// <summary>
    /// <CreatedDate>04 May 2005</CreatedDate>
    /// <Author>Mr. Hemant Kirad</Author>
    /// <Purpose>
    /// This class represents an EntityType. An EntityType represents an entity involved in the business world.
    /// This is a partial class so it will have some of its functionality divided into
    /// 1. EntityType.cs
    /// 2. EntityTypeExt1.cs (This file)
    /// </Purpose>
    /// </summary>
    internal partial class EntityType
    {

        #region PrivateMethods
        /// <summary>
        /// <CreatedDate>04 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML complaint with the Type.xsd
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeNode">This Node represents the EntityType node.</param>
        /// <param name="TypeNode">This is a reference parameter and will hold the XML Node, which is 
        /// created in this function and returned. The structure of this TypeNode is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- NAME          (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- TABLENAME
        ///        (*) Mandatory Element
        /// </param>
        private void generateTypeXML(XmlNode EntityTypeNode, ref XmlDocument TypeXMLDocument)
        {
            String TypeID               = null;     //String variable storing the value of TypeID.
            String TypeName             = null;     //String variable storing the value of TypeName.
            String Description          = null;     //String variable storing the value of Description.
            XmlNode TypeIDNode          = null;     //DOM Node representing the TypeID node.
            XmlNode NameNode            = null;     //DOM Node representing the Name node.
            XmlNode DescriptionNode     = null;     //DOM Node representing the Description node.
            XmlNode TypeNode            = null;     //DOM Node representing the Type node.
            XmlNode ObjectTypeNode      = null;     //DOM Node representing the ObjectType node.
            XmlNode EntityTypeIDNode    = null;     //DOM Node representing the EntityTypeID node.
            XmlNode EntityTypeNameNode  = null;     //DOM Node representing the EntityTypeName node.
            XmlNode TypeDescriptionNode = null;     //DOM Node representing the TypeDescription node.
            XmlNode EntityTypeDisplayNameNode = null;//DOM Node representing the FacetTypeDisplayName node.

            try
            {
                // Get information from the EntityType XML.

                EntityTypeIDNode = EntityTypeNode.SelectSingleNode("ENTITYTYPEID");
                TypeID = EntityTypeIDNode.InnerText;

                EntityTypeNameNode = EntityTypeNode.SelectSingleNode("ENTITYTYPENAME");
                TypeName = EntityTypeNameNode.InnerText;

                TypeDescriptionNode = EntityTypeNode.SelectSingleNode("TYPEDESCRIPTION");
                if (TypeDescriptionNode == null)
                    Description = "";
                else
                    Description = TypeDescriptionNode.InnerText;

                // Prepare the Type XML.

                TypeXMLDocument = new XmlDocument();

                TypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPE", "");
                TypeXMLDocument.AppendChild(TypeNode);

                TypeIDNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEID", "");
                TypeIDNode.InnerText = TypeID;
                TypeNode.AppendChild(TypeIDNode);

                NameNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPENAME", "");
                NameNode.InnerText = TypeName;
                TypeNode.AppendChild(NameNode);

                DescriptionNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEDESCRIPTION", "");
                DescriptionNode.InnerText = Description;
                TypeNode.AppendChild(DescriptionNode);

                ObjectTypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "OBJECTTYPE", "");
                ObjectTypeNode.InnerText = "ENTITY";
                TypeNode.AppendChild(ObjectTypeNode);

                TypeNode.AppendChild(TypeXMLDocument.ImportNode(EntityTypeNode.SelectSingleNode("TABLENAME"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(EntityTypeNode.SelectSingleNode("PRIMARYKEY"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(EntityTypeNode.SelectSingleNode("ISBUILTIN"), true));
                EntityTypeDisplayNameNode = EntityTypeNode.SelectSingleNode("DISPLAYNAME");
                if (EntityTypeDisplayNameNode != null)
                    TypeNode.AppendChild(TypeXMLDocument.ImportNode(EntityTypeNode.SelectSingleNode("DISPLAYNAME"), true));
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method returns the EntityType TableName for the given EntityTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeID">String variable containing the TypeID.</param>
        /// <param name="TableName">String variable which stores the TableName.</param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the TableName from the database.
        /// </returns>
        private OMError getTableNameFromDB(String TypeID, ref String TableName)
        {
            OMError ReadErrorObj               = null;     //OMError object for read record in the database.
            OMError ReturnErrorObj             = null;     //OMError object used to return an error from OM.
            String EntityTypeXML               = null;     //String variable storing the EntityType XML.
            XmlDocument EntityTypeXMLDocument  = null;     //DOM Object representing the EntityTypeXML.

            try
            {
                ReadErrorObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO,
                                                            TypeID, OMConjunction.NOTHING);
                if (ReadErrorObj != null)
                     return ReadErrorObj;

                 ReadErrorObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                        ref EntityTypeXML);
                if (ReadErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrorObj.ErrorType, ReadErrorObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }
                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeXML);
                TableName = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPE/TABLENAME").InnerText;

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method updates the EntityType record in the ALOM_ENTITYTYPE table.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">DOM object representing the TypeNode.</param>
        /// <param name="TypeID">String variable storing the TypeID.</param>
        /// <param name="TableName">String variable storing the TableName.</param>
        /// <returns>
        /// NULL: If the update was successful.
        /// Object of OMError if an error occurred while updating the EntityType in the database.
        /// </returns>
        private OMError updateEntityType(XmlNode TypeNode, String TypeID, String TableName)
        {
            OMError UpdateErrorObj            = null;     //OMError object for create record in the database.
            OMError ReturnErrorObj            = null;     //OMError object used to return an error from OM.
            XmlDocument EntityTypeXMLDocument = null;     //DOM Object representing the EntityTypeXML.
            XmlNode EntityTypeNode            = null;     //DOM Object representing the EntityType Node.

            try
            {
                generateEntityTypeXML(TypeNode, TableName, ref EntityTypeXMLDocument);

                UpdateErrorObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                    OMOperator.EQUALTO, TypeID, OMConjunction.NOTHING);

                if (UpdateErrorObj != null)
                    return UpdateErrorObj;

                EntityTypeNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPE");

                EntityTypeNode.RemoveChild(EntityTypeNode.SelectSingleNode("ENTITYTYPEID"));

                UpdateErrorObj = OMDataManager.DataProvider.UpdateRecord("ALOM_ENTITYTYPE", EntityTypeXMLDocument.InnerXml);

                if (UpdateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                        "EntityType Modification Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                        "EntityType Modification Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is an helper method.
        /// The method update the schema information of an EntityType in the ALOM_SCHEMA table.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXMLDocument">DOM object representing the InputXML.</param>
        /// <param name="TableName">String variable storing the TableName.</param>
        /// <param name="FieldCollectionNode">DOM object representing the FieldCollection Node.</param>
        /// <returns>
        /// NULL: If the update was successful.
        /// Object of OMError if an error occurred while updating the schema in the database.
        /// </returns>
        private OMError updateSchema(XmlDocument InputXMLDocument, String TableName, XmlNode FieldCollectionNode)
        {
            OMError UpdateErrorObj = null;                  //OMError object for create record in the database.
            OMError ReturnErrorObj = null;                  //OMError object for returning from OM.
            XmlNodeList FieldNodes = null;                  //DOM Node representing the list of Field nodes.
            XmlNode SchemaNode     = null;                  //DOM Node representing the Schema node.
            XmlNode TableNameNode  = null;                  //DOM Node representing the TableName node.
            String SchemaID        = null;                  //String variable storing the SchemaID.
            String OperationType   = null;                  //String variable storing the OperationType. 
            XmlDocument     DataType_SizeMappingXMLDoc  = null;
            XmlNode         FieldSizeNode               = null;
            XmlNode         DefaultFieldSizeNode        = null;
            XmlNode         DataTypeNode                = null;   

            try
            {
                DataType_SizeMappingXMLDoc = new XmlDocument();
                //Get all the FieldNodes from the FieldCollectionNode.
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                if (FieldNodes.Count > 0)
                {
                    DataType_SizeMappingXMLDoc.LoadXml(OMDataManager.DataProvider.GetDataTypesAndTheirSizes());
                    foreach (XmlNode FieldNode in FieldNodes)
                    {
                        //For each field read the OperationType attribute and depending on it do the appropriate.
                        SchemaNode = null;
                        SchemaID = null;
                      
                        SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                        OperationType = FieldNode.Attributes.GetNamedItem("OperationType").InnerText;

                        if (OperationType.ToUpper() == "NEW")
                        {
                            //Anand :  This code is added because FieldSize is not passed for DataTypes other than
                            // String. As DAL code consider's size not to be null we need to fill in appropiate
                            // sizes depending upon the DataType(also this is database dependent).
                            FieldSizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                            if ((FieldSizeNode == null) || (String.IsNullOrEmpty(FieldSizeNode.InnerText)))
                            {
                                DataTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                                if (FieldSizeNode == null)
                                {
                                    FieldSizeNode = FieldNode.OwnerDocument.CreateElement("FIELDSIZE");
                                    FieldNode.AppendChild(FieldSizeNode);
                                }
                                DefaultFieldSizeNode = DataType_SizeMappingXMLDoc.DocumentElement.SelectSingleNode(DataTypeNode.InnerText.ToUpper());
                                if (DefaultFieldSizeNode != null)
                                    FieldSizeNode.InnerText = DefaultFieldSizeNode.InnerText;
                                else
                                    FieldSizeNode.InnerText = "0";
                            }
                        }

                        generateSchemaXML(InputXMLDocument, FieldNode, ref SchemaNode);

                        switch (OperationType.ToUpper())
                        {
                            case "NEW":
                                TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                                TableNameNode.InnerText = TableName;
                                SchemaNode.AppendChild(TableNameNode);

                                UpdateErrorObj = OMDataManager.DataProvider.CreateRecord("ALOM_SCHEMA",SchemaNode.OuterXml);

                                if (UpdateErrorObj != null)
                                {
                                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                                        "Field Creation Failed", OMErrorObjectType.ENTITYTYPE,
                                        "EntityType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                                        UpdateErrorObj.ErrorXML);

                                    return ReturnErrorObj;
                                }
                                break;

                            case "MODIFY":
                                UpdateErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID",
                                OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID, OMConjunction.NOTHING);

                                if (UpdateErrorObj != null)
                                    return UpdateErrorObj;
                        
                                SchemaNode.RemoveChild(SchemaNode.SelectSingleNode("SCHEMAID"));

                                if (UpdateErrorObj==null)
                                    UpdateErrorObj = OMDataManager.DataProvider.UpdateRecord("ALOM_SCHEMA",
                                                                                        SchemaNode.OuterXml);

                                if (UpdateErrorObj != null)
                                {
                                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                                        "Field Modification Failed", OMErrorObjectType.ENTITYTYPE,
                                        "EntityType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                                        UpdateErrorObj.ErrorXML);

                                    return ReturnErrorObj;
                                }
                                break;

                            case "DELETE":
                               
                                if (UpdateErrorObj==null)
                                      UpdateErrorObj = deleteSchema(Convert.ToUInt64(SchemaID),
                                                              OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED);

                                if (UpdateErrorObj != null)
                                    return UpdateErrorObj;
                                break;
                        }
                    }
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                        "Field Modification Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }

        }

        /// <summary>
        /// <CreatedDate>18 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will update XSD for the EntityType.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeID">ulong variable storing the TypeID.</param>
        private void updateXSD(ulong TypeID)
        {
            OMError ReadErrObj           = null;     //OMError object for read record in the database.
            String TypeXML               = null;     //String variable storing the TypeXML.
            XmlDocument TypeXMLDocument  = null;     //DOM Object representing the TypeXML.
            XmlNode TypeNode             = null;     //DOM Object representing the Type Node.

            try
            {
                ReadErrObj = Read(TypeID, ref TypeXML);

                if ((ReadErrObj == null) && (TypeXML != null) && (TypeXML != ""))
                {
                    TypeXMLDocument = new XmlDocument();
                    TypeXMLDocument.LoadXml(TypeXML);

                    TypeNode = TypeXMLDocument.SelectSingleNode("TYPE");

                    generateXSD(TypeNode);
                }

            }
            catch (Exception ExceptionObject)
            {
                throw ExceptionObject;
            }

           
        }

        //Following method is added for XML data type support on date 8th March 2006 by Kiran Abhang.
        //This method is used to get the XSD file and XSD file path for fields which are of type XML.
        //XSD file is copied on the application's location and XSD file path is added as an attribute in the
        // field's FieldType node.
        private OMError GetXSDFileAndPathForXMLTypeFields(String TypeID, String ObjectTypeName, 
                                            String ObjectType, ref String OutputXML)
        {
            OMError         ReadErrObj              = null;
            IXMLRepository  ObjXMLManager           = null;
            XmlDocument     FieldXMLDocument        = null;
            XmlDocument     FieldListXMLDocument    = null;
            XmlNodeList     FieldsList              = null;
            XmlNode         XSDFieldNameNode        = null;
            XmlNode         CollectionNode          = null;
            XmlNode         XSDPathNode             = null;
            String          FieldName               = null;
            String          XSDFilePath             = null;
            String          FieldCollectionXML      = null;

            try
            {
                //Selecting fields which are of type XML.
                FieldCollectionXML = null;
                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, TypeID.ToString(),
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "ENTITY",
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("FIELDTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "XML",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                        ref FieldCollectionXML);
                if ((ReadErrObj != null) || (FieldCollectionXML == null) || (FieldCollectionXML == ""))
                    return (null);
                else
                {
                    FieldXMLDocument = new XmlDocument();
                    FieldXMLDocument.LoadXml(FieldCollectionXML);
                    CollectionNode = FieldXMLDocument.DocumentElement;
                    FieldsList = CollectionNode.SelectNodes("FIELD");

                    FieldListXMLDocument = new XmlDocument();
                    FieldListXMLDocument.LoadXml(OutputXML);

                    if ((FieldsList != null) && (FieldsList.Count > 0))
                    {
                        ObjXMLManager = OMDataManager.XMLRepository; //ObjXMLManager = new XMLManager(TenantName);
                        foreach (XmlNode FieldNode in FieldsList)
                        {
                            FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper();
                            //Get the XSD file and XSD file path.
                            ReadErrObj = ObjXMLManager.RetrieveXSDPath(ObjectTypeName, FieldName, out XSDFilePath);
                            if ((XSDFilePath != null) && (XSDFilePath.Length > 0))
                            {
                                //Adding XSDFilePath attribute for XML data type column.
                                XSDFieldNameNode = FieldListXMLDocument.SelectSingleNode("//TYPE/FIELDCOLLECTION/FIELD[FIELDNAME/text()='" + FieldName.Trim().ToUpper() + "']");
                                if (XSDFieldNameNode != null)
                                {
                                    XSDPathNode = FieldListXMLDocument.CreateElement("XSDPATH");
                                    XSDPathNode.InnerText = XSDFilePath;
                                    XSDFieldNameNode.AppendChild(XSDPathNode);
                                }
                            }
                        }
                    }
                    OutputXML = FieldListXMLDocument.OuterXml;
                }
            }
            catch (Exception ExceptionObject)
            {
                throw ExceptionObject;
            }
            return (null);
        }

        #endregion PrivateMethods

        #region Methods
        
        /// <summary>
        /// <CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will update the EntityType based on details in the InputXML.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">String variable which contains all the details of an EntityType 
        /// that are to be updated.</param>
        /// <returns>
        /// NULL: If the update was successful.
        /// Object of OMError if an error occurred while updating the EntityType.
        /// </returns>
        public OMError Update(String InputXML)
        {
            XmlDocument InputXMLDocument = null;     //DOM Object representing the InputXML.
            XmlNode TypeNode             = null;     //DOM Node representing the Type node of the InputXML.
            XmlNode FieldCollectionNode  = null;     //DOM Node representing the FieldCollection node.
            String TypeID                = null;     //String variable storing the value of TypeID.
            String TableName             = null;     //String variable storing the value of TableName.
            StringBuilder Message        = null;     //StringBuilder object storing the Message that is to be logged.
            XmlNode TableNameNode        = null;     //DOM Node representing the TableName node.
            XmlNode TypeNode_Clone       = null;     //DOM Node representing the Clone TypeNode node.
            OMError UpdateErrorObj       = null;     //OMError object for create record in the database.
            OMError ReturnErrorObj       = null;     //OMError object used to return an error from OM.
            OMError ReadErrObj           = null;     //OMError object for read record in the database.
            //Boolean WriteFailed          = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManagerWriter is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Update API of EntityType class in OM. Input Parameter InputXML: ", InputXML);

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManagerWriter and OMDataManagerReader is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

               
                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                TypeNode = InputXMLDocument.SelectSingleNode("TYPE");
                TypeNode_Clone = TypeNode.Clone();

                TypeID = TypeNode.SelectSingleNode("TYPEID").InnerText;

                ReadErrObj = getTableNameFromDB(TypeID, ref TableName);

                if (ReadErrObj != null)
                {
                    //WriteFailed = true;
                    return ReadErrObj;
                }

                TableNameNode = TypeNode.SelectSingleNode("TABLENAME");

                if (TableNameNode == null)
                {
                    TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                    TableNameNode.InnerText = TableName;
                    TypeNode_Clone.AppendChild(TableNameNode);
                }
                
                if (TypeNode.SelectNodes("FIELDCOLLECTION").Count > 0)
                {
                    FieldCollectionNode = TypeNode.SelectSingleNode("FIELDCOLLECTION").Clone();
                    TypeNode.RemoveChild(TypeNode.SelectSingleNode("FIELDCOLLECTION"));
                }

                //Update record in ALOM_ENTITYTYPE Table.
                UpdateErrorObj = updateEntityType(TypeNode, TypeID, TableName);

                if (UpdateErrorObj != null)
                {
                    //WriteFailed = true;
                    return UpdateErrorObj;
                }

                if (FieldCollectionNode != null)
                {
                    //Update record in ALOM_SCHEMA Table.
                    UpdateErrorObj = updateSchema(InputXMLDocument, TableName, FieldCollectionNode);

                    if (UpdateErrorObj != null)
                    {
                        //WriteFailed = true;
                        return UpdateErrorObj;
                    }
                }

                //Update the table structure.
                UpdateErrorObj = OMDataManager.DataProvider.UpdateTable(TypeNode_Clone.OuterXml);

                if (UpdateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                        "Table Modification Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                        UpdateErrorObj.ErrorXML);

                    //WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Update the XML schema definition. 
                if (FieldCollectionNode != null)
                    updateXSD(Convert.ToUInt64(TypeID));

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Update API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Update(String InputXML) InputXML:", InputXML);

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED,
                        "EntityType Modification Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                //WriteFailed = true;
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

              
            }
        }

        /// <summary>
        ///<CreatedDate>05 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read the EntityType based on EntityTypeID. This will return
        /// EntityType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeID">This contains the EntityTypeID.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the information about the EntityType. 
        /// The structure of this XML is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- NAME          (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- TABLENAME
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If read was successful.
        /// Object of OMError if an error occurred while reading the EntityType.
        /// </returns>
        public OMError Read(ulong EntityTypeID, ref String OutputXML)
        {
            OMError ReadErrObj          = null;   //The OMError object for reading data from DB.
            OMError ReturnErrorObj      = null;   //OMError object used to return an error from OM.
            String EntityTypeXML        = null;   //The String variable storing the EntityType XML.
            String FieldCollectionXML   = null;   //The String variable storing the FieldCollection XML.
            String EntityTypeName       = null;
            StringBuilder Message       = null;   //StringBuilder object storing the Message that is to be logged.
            XmlNode FieldCollectionNode = null;   //The DOM Node representing the FieldCollection node.
            XmlNode TypeXMLNode         = null;   //The DOM Node representing the Type node.
            XmlNode EntityTypeNode      = null;   //The DOM Node representing the EntityType node.
            StringCollection TableNameCollection     = null;  //StringCollection object to store the TableNames.
            OMCommon.OMCommon OMCommonObject         = null;  //OMCommon Object.
            XmlDocument TypeXMLDocument              = null;  //The DOM Object representing the Type XML.
            XmlDocument EntityTypeXMLDocument        = null;  //The DOM Object representing the EntityType XML.
            XmlDocument FieldCollectionXMLDocument   = null;  //The DOM Object representing the FieldCollection XML.
            Boolean IsOMDataManagerReaderInitialized = false;     //Boolean variable storing whether OMDataManagerReader is initialized or not.
            AssoFieldNamedHierarchy AssoNHObj        = null;
            
            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Read API of EntityType class in OM. Input Parameter EntityTypeID: ", EntityTypeID.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerReaderInitialized = true;
                }

                //Get the EntityType Info...

                ReadErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, EntityTypeID.ToString(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref EntityTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (EntityTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the EntityType XML to Type XML.
                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeXML);
                
                EntityTypeNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPE");
                EntityTypeName = EntityTypeNode.SelectSingleNode("ENTITYTYPENAME").InnerText.Trim().ToUpper();
                generateTypeXML(EntityTypeNode, ref TypeXMLDocument);

                //Get the fields of EntityType Table...
                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, EntityTypeID.ToString(),
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "ENTITY",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null); 
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                                    ref FieldCollectionXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "Field Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML.
                if (FieldCollectionXML != null)
                {
                    FieldCollectionXMLDocument = new XmlDocument();
                    FieldCollectionXMLDocument.LoadXml(FieldCollectionXML);
                    FieldCollectionNode = FieldCollectionXMLDocument.DocumentElement;

                    TypeXMLNode = TypeXMLDocument.DocumentElement;
                    TypeXMLNode.AppendChild(TypeXMLDocument.ImportNode(FieldCollectionNode, true));
                }

                //Obtaining NamedHierarchy Information associated with Type
                AssoNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                //The following line is commented to improve the performance. The following call gets the Named Hierarchy (including all values) _
                //if the field is associated with Named Hierarchy. In this case instead of returning the entire Named Hierarchy collection, 
                //the code is changed to return only the Named Hierarchy root ID for that field.
                //AssoNHObj.AssociateNamedHierarchyInformation(ref TypeXMLDocument, EntityTypeID);

                AssoNHObj.AssociateNamedHierarchyID(ref TypeXMLDocument, EntityTypeID);
                
                if ((IsOMDataManagerReaderInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_ENTITYTYPE");
                    TableNameCollection.Add("ALOM_SCHEMA");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeXMLNode.OuterXml.ToString(), 
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeXMLNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Read API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                //Get the XSD file and XSD file path for fields which are of type XML.
                ReadErrObj = GetXSDFileAndPathForXMLTypeFields(EntityTypeID.ToString(), EntityTypeName, "ENTITY", ref OutputXML);
                return (ReadErrObj);
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Read(ulong EntityTypeID, ref String OutputXML) EntityTypeID:", EntityTypeID.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerReaderInitialized)
                {
                    OMDataManager.DataProvider.Close();
                }
            }
        }

        /// <summary>
        ///<CreatedDate>10 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read the EntityType based on EntityTypeName. This will return
        /// EntityType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="EntityTypeName">This contains the EntityTypeName.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the information about the EntityType. 
        /// The structure of this XML is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- NAME          (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- TABLENAME
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If read was successful.
        /// Object of OMError if an error occurred while reading the EntityType.
        /// </returns>
        public OMError Read(String EntityTypeName, ref String OutputXML)
        {
            OMError ReadErrObj          = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj      = null;   //OMError object used to return an error from OM.
            String EntityTypeXML        = null;   //String variable storing the EntityType XML.
            String FieldCollectionXML   = null;   //String variable storing the FieldCollection XML.
            String TypeID               = null;   //String variable storing the value of TypeID.
            StringBuilder Message       = null;   //StringBuilder object storing the Message that is to be logged.
            XmlNode EntityTypeIDNode    = null;   //DOM Node representing the EntityTypeID node.
            XmlNode FieldCollectionNode = null;   //DOM Node representing the FieldCollection node.
            XmlNode TypeXMLNode         = null;   //DOM Node representing the Type node.
            XmlNode EntityTypeNode      = null;   //DOM Node representing the EntityType node.

            OMCommon.OMCommon OMCommonObject         = null;  //OMCommon Object.
            StringCollection TableNameCollection     = null;  //StringCollection object to store the TableNames.
            XmlDocument TypeXMLDocument              = null;  //DOM Object representing the Type XML.
            XmlDocument EntityTypeXMLDocument        = null;  //DOM Object representing the EntityType XML.
            XmlDocument FieldCollectionXMLDocument   = null;  //DOM Object representing the FieldCollection XML.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManagerReader is initialized or not.
            AssoFieldNamedHierarchy AssoNHObj = null;
            ulong EntityTypeID = 0;

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Read API of EntityType class in OM. Input Parameter EntityTypeName: ", EntityTypeName);

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the EntityType Info...

                ReadErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPENAME", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, EntityTypeName.ToUpper(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref EntityTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (EntityTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the EntityType XML to Type XML.

                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeXML);

                EntityTypeNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPE");

                EntityTypeIDNode = EntityTypeNode.SelectSingleNode("ENTITYTYPEID");
                TypeID = EntityTypeIDNode.InnerText;

                generateTypeXML(EntityTypeNode, ref TypeXMLDocument);

                //Get the fields of EntityType Table.
                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, TypeID.ToString(),
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "ENTITY",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                                    ref FieldCollectionXML);
                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "Field Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                if (FieldCollectionXML != null)
                {
                    FieldCollectionXMLDocument = new XmlDocument();
                    FieldCollectionXMLDocument.LoadXml(FieldCollectionXML);
                    FieldCollectionNode = FieldCollectionXMLDocument.DocumentElement;

                    TypeXMLNode = TypeXMLDocument.DocumentElement;
                    TypeXMLNode.AppendChild(TypeXMLDocument.ImportNode(FieldCollectionNode, true));
                }
                //Obtaining NamedHierarchy Information associated with Type
                EntityTypeID = Convert.ToUInt64(TypeXMLDocument.SelectSingleNode("TYPE/TYPEID").InnerText);
                AssoNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                //The following line is commented to improve the performance. The following call gets the Named Hierarchy (including all values) _
                //if the field is associated with Named Hierarchy. In this case instead of returning the entire Named Hierarchy collection, 
                //the code is changed to return only the Named Hierarchy root ID for that field.
                //AssoNHObj.AssociateNamedHierarchyInformation(ref TypeXMLDocument, EntityTypeID);

                AssoNHObj.AssociateNamedHierarchyID(ref TypeXMLDocument, EntityTypeID);
                
                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_ENTITYTYPE");
                    TableNameCollection.Add("ALOM_SCHEMA");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeXMLNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeXMLNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Read API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                //Get the XSD file and XSD file path for fields which are of type XML.
                ReadErrObj = GetXSDFileAndPathForXMLTypeFields(TypeID, EntityTypeName, "ENTITY", ref OutputXML);
                return (ReadErrObj);
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Read(String EntityTypeName, ref String OutputXML) EntityTypeName:", EntityTypeName);
                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                                                "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                                                "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return (ReturnErrorObj);
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }

        /// <summary>
        ///<CreatedDate>09 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read all the EntityTypes in the system. This will return a collection of 
        /// EntityType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the collection of 
        /// EntityType information. 
        /// The structure of this XML is give below:
        ///     TYPECOLLECTION
        ///             |- TYPE
        ///                 |- TYPEID        (*)
        ///                 |- NAME          (*)
        ///                 |- DESCRIPTION   
        ///                 |- OBJECTTYPE    (*)    
        ///                 |- PRIMARYKEY
        ///                 |- TABLENAME
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the EntityType.
        /// </returns>
        public OMError ReadAll(ref String OutputXML)
        {
            OMError ReadErrObj                 = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj             = null;   //OMError object used to return an error from OM.
            XmlDocument EntityTypeXMLDocument  = null;   //DOM Object representing the EntityType XML.
            XmlDocument TypeXMLDocument        = null;   //DOM Object representing the Type XML.
            XmlDocument OutputXMLDocument      = null;   //DOM Object representing the Output XML.
            XmlNodeList EntityTypeNodes        = null;   //DOM Node representing the list of EntityType nodes.
            XmlNode EntityTypeCollectionNode   = null;   //DOM Node representing the list of EntityTypeCollection node.
            XmlNode TypeCollectionNode         = null;   //DOM Node representing the TypeCollection node.
            XmlNode TypeNode                   = null;   //DOM Node representing the Type node.
            String EntityTypeCollectionXML     = null;   //String variable storing the EntityTypeCollection XML.
            StringBuilder Message              = null;   //StringBuilder object storing the Message that is to be logged.
          
            OMCommon.OMCommon OMCommonObject     = null;  //OMCommon Object.
            StringCollection TableNameCollection = null;  //StringCollection object to store the TableNames.

            Boolean IsOMDataManagerInitialized = false;   //Boolean variable storing whether OMDataManagerReader is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("Entered the ReadAll API of EntityType class in OM.");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager= new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the EntityType Info...

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref EntityTypeCollectionXML);
                
                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (EntityTypeCollectionXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...
               
                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeCollectionXML);

                EntityTypeCollectionNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPECOLLECTION");
                EntityTypeNodes = EntityTypeCollectionNode.SelectNodes("ENTITYTYPE");

                if (EntityTypeNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the EntityTypeCollection XML to TypeCollection XML.

                OutputXMLDocument = new XmlDocument();
                TypeCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, "TYPECOLLECTION", "");
                OutputXMLDocument.AppendChild(TypeCollectionNode);

                foreach (XmlNode EntityTypeNode in EntityTypeNodes)
                {
                    generateTypeXML(EntityTypeNode, ref TypeXMLDocument);

                    TypeNode = TypeXMLDocument.DocumentElement;
                    TypeCollectionNode.AppendChild(OutputXMLDocument.ImportNode(TypeNode, true));
                }

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_ENTITYTYPE");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeCollectionNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeCollectionNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the ReadAll API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("MethodName: public OMError ReadAll(ref String OutputXML)");

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);

                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                {
                    OMDataManager.DataProvider.Close();
                }
            }
        }

        /// <summary>
        /// <CreatedDate>25 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read EntityTypes that are requested by the consumer. This will return a collection of 
        /// EntityType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="ObjectTypeIDArrayList">ArrayList containing the ObjectTypeIDs.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the collection of 
        /// EntityType information. 
        /// The structure of this XML is give below:
        ///     TYPECOLLECTION
        ///             |- TYPE
        ///                 |- TYPEID        (*)
        ///                 |- NAME          (*)
        ///                 |- DESCRIPTION   
        ///                 |- OBJECTTYPE    (*)    
        ///                 |- PRIMARYKEY
        ///                 |- TABLENAME
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the EntityType.
        /// </returns>
        public OMError ReadAll(ArrayList ObjectTypeIDArrayList, ref String OutputXML)
        {
            OMError ReadErrObj                 = null;    //OMError object for reading data from DB.
            OMError ReturnErrorObj             = null;    //OMError object used to return an error from OM.
            XmlDocument EntityTypeXMLDocument  = null;    //DOM Object representing the EntityType XML.
            XmlDocument TypeXMLDocument        = null;    //DOM Object representing the Type XML.
            XmlDocument OutputXMLDocument      = null;    //DOM Object representing the Output XML.
            XmlNodeList EntityTypeNodes        = null;    //DOM Node representing the list of EntityType nodes.
            XmlNode EntityTypeCollectionNode   = null;    //DOM Node representing the list of EntityTypeCollection node.
            XmlNode TypeCollectionNode         = null;    //DOM Node representing the TypeCollection node.
            XmlNode TypeNode                   = null;    //DOM Node representing the Type node.
            String EntityTypeCollectionXML     = null;    //String variable storing the EntityTypeCollection XML.
            StringBuilder ObjecTypeIDList      = null;    //StringBuilder variable storing the comma separated EntityTypeIDs.
            String Seprator                    = null;    //String storing the Separator.
            StringBuilder Message              = null;    //StringBuilder object storing the Message that is to be logged.
            
            OMCommon.OMCommon OMCommonObject     = null;  //OMCommon Object.
            StringCollection TableNameCollection = null;  //StringCollection object to store the TableNames.

            Boolean IsOMDataManagerInitialized = false;   //Boolean variable storing whether OMDataManagerReader is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the ReadAll API of EntityType class in OM. Input Parameter ObjectTypeIDArrayList: ", ObjectTypeIDArrayList.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the EntityType Info...
                Seprator = "";
                ObjecTypeIDList = new StringBuilder();
                ObjecTypeIDList.AppendFormat("(");
                for (int ArrayListCount = 0; ArrayListCount < ObjectTypeIDArrayList.Count; ArrayListCount++)
                {
                    ObjecTypeIDList.AppendFormat("{0}{1}", Seprator, ObjectTypeIDArrayList[ArrayListCount]);
                    Seprator = ",";
                }
                ObjecTypeIDList.AppendFormat(")");

                ReadErrObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                            OMOperator.IN, ObjecTypeIDList.ToString(),
                                                            OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref EntityTypeCollectionXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (EntityTypeCollectionXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                EntityTypeXMLDocument = new XmlDocument();
                EntityTypeXMLDocument.LoadXml(EntityTypeCollectionXML);

                EntityTypeCollectionNode = EntityTypeXMLDocument.SelectSingleNode("ENTITYTYPECOLLECTION");
                EntityTypeNodes = EntityTypeCollectionNode.SelectNodes("ENTITYTYPE");

                if (EntityTypeNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the EntityTypeCollection XML to TypeCollection XML.

                OutputXMLDocument = new XmlDocument();
                TypeCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, "TYPECOLLECTION", "");
                OutputXMLDocument.AppendChild(TypeCollectionNode);

                foreach (XmlNode EntityTypeNode in EntityTypeNodes)
                {
                    generateTypeXML(EntityTypeNode, ref TypeXMLDocument);

                    TypeNode = TypeXMLDocument.DocumentElement;
                    TypeCollectionNode.AppendChild(OutputXMLDocument.ImportNode(TypeNode, true));
                }

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_ENTITYTYPE");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeCollectionNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeCollectionNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the ReadAll API of EntityType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "ENTITYTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError ReadAll(ArrayList ObjectTypeIDArrayList, ref String OutputXML) ObjectTypeIDArrayList: ", ObjectTypeIDArrayList.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_READ_FAILED,
                        "EntityType Read Failed", OMErrorObjectType.ENTITYTYPE,
                        "EntityType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);

                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }
        #endregion Methods
    }
}
