using System;
using System.Collections.Generic;
using System.Threading;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.NamedHierarchy;
using Assetlink.Diagnostic;
namespace Assetlink.ObjectManager.ETFTManagement
{
    /// <summary>
    /// <CreatedDate>10 May 2005</CreatedDate>
    /// <Author>Mr. Hemant Kirad</Author>
    /// <Purpose>
    /// This class represents an FacetType. An FacetType represents an facet involved in the business world.
    /// This is a partial class so it will have some of its functionality divided into
    /// 1. FacetType.cs (This file)
    /// 2. FacetTypeExt1.cs 
    /// </Purpose>
    /// </summary>
    internal partial class FacetType : ObjectManagerBase
    {
        private static Object Lock = new object();
        #region Constructors
        public FacetType()
		{
			
		}

		public FacetType(ulong UserID,string CurrentLanguage,string TenantName)
			: base(UserID,CurrentLanguage,TenantName)
		{

		}
        public FacetType(DataManager DataManagerObj)
            : base(DataManagerObj)
        {

        }
        public FacetType(ulong UserID, string CurrentLanguage,
                                string TenantName, DataManager DataManagerObj)
            : base(UserID, CurrentLanguage,
            TenantName, DataManagerObj)
        {

        }
        #endregion Constructors

        #region Methods

        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will create a FacetType based on information provided 
        /// in the InputXML. The structure of InputXML is provided below.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">
        /// The InputXML will be compliant with Type.xsd.
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- NAME          (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- TYPEDESCRIPTION   
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME        
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If the creation was successful.
        /// Object of OMError if an error occurred while creating the FacetType.
        /// </returns>
        /// Code Change Comments
        /*  Bug Fix : 4380, Date : 19 August 2008, Changed By : Sagar Dasalkar
         *  Description : If Facet Type creation operation (Adding entries in ALOM_SCHEMA, adding entries in ALFCT_FACETTYPE and creating table) fails, 
         *  before returning OMError object the lock obtained on an object has been released.
         */
        public OMError Create(String InputXML)
        {
            XmlDocument InputXMLDocument       = null;     //DOM Object representing the InputXML.
            XmlDocument FacetTypeXMLDocument   = null;     //DOM Object representing the FacetTypeXML.
            XmlNodeList FieldNodes             = null;     //DOM Node representing the list of Field nodes.
            XmlNode NameNode                   = null;     //DOM Node representing the Name node.
            XmlNode TableNameNode              = null;     //DOM Node representing the TableName node.
            XmlNode FieldCollectionNode        = null;     //DOM Node representing the FieldCollection node.
            XmlNode SchemaNode                 = null;     //DOM Node representing the Schema node.
            XmlNode TypeNode                   = null;     //DOM Node representing the Type node of the InputXML.
            String TypeName                    = null;     //String variable storing the value of TypeName.
            String TableName                   = null;     //String variable storing the value of TableName.
            String ModifiedInputXML            = null;     //String variable storing the modified InputXML.
            StringBuilder Message              = null;     //StringBuilder object storing the Message that is to be logged.
            OMError CreateErrorObj             = null;     //OMError object for create record in the database.
            OMError ReturnErrorObj             = null;     //OMError object for returning from DAL.
            Boolean WriteFailed                = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            String TempQuery = null;
            StringBuilder QueryBuilder = null;
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManager is initialized or not.
            ulong AssociationID                = 0;         //Ulong variable storing AssociationID for Field and NamedHierarchy Association
            ulong NamedHierarchyID;                         //Ulong variable storing NamedHierarchyID for Field and NamedHierarchy Association
            ulong SchemaID;                                  //Ulong variable storing SchemaID for Field and NamedHierarchy Association
            String CreateTableQuery            = null;

            XmlDocument     DataType_SizeMappingXMLDoc  = null;
            XmlNode         FieldSizeNode               = null;
            XmlNode         DefaultFieldSizeNode        = null;
            XmlNode         DataTypeNode                = null;      

            try
            {
               if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Create API of FacetType class in OM. Input Parameter InputXML: ", InputXML);

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManager and OMDataManager is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                DataType_SizeMappingXMLDoc      = new XmlDocument();
                InputXMLDocument                = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                TypeNode = InputXMLDocument.SelectSingleNode("TYPE");
                NameNode = TypeNode.SelectSingleNode("TYPENAME");
                TypeName = NameNode.InnerText;

                if ((TypeName == null) || (TypeName.Length <= 0))
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                        "FacetType Should Contain Meaningful Name", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);
                    return ReturnErrorObj;
                }
                else if (isFacetTypeExists(TypeName) == true) //Check whether the given FacetType already exists in the system.
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                        "FacetType Already Exists", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);
                    return ReturnErrorObj;
                }

                //Generate the TableName.
                TableName = generateTableName(TypeName);
                //Add record in ALOM_FACETTYPE Table.
                generateFacetTypeXML(TypeNode, TableName, ref FacetTypeXMLDocument);
                //Initialiazing the batch insert query builder
                QueryBuilder = new StringBuilder();

                CreateErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_FACETTYPE",
                                                                         FacetTypeXMLDocument.InnerXml,
                                                                         ref TempQuery);
                QueryBuilder.Append(TempQuery);

                //Add record in ALOM_SCHEMA Table.
                FieldCollectionNode = InputXMLDocument.SelectSingleNode("TYPE/FIELDCOLLECTION");
                if (FieldCollectionNode == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                       "Fields Not Found", OMErrorObjectType.FACETTYPE,
                       "FacetType", OMErrorType.ERROR, null, "");
                    WriteFailed = true;
                    return ReturnErrorObj;
                }

                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                if (FieldNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                       "Fields Not Found", OMErrorObjectType.FACETTYPE,
                       "FacetType", OMErrorType.ERROR, null, "");
                    WriteFailed = true;
                    return ReturnErrorObj;
                }

                DataType_SizeMappingXMLDoc.LoadXml(OMDataManager.DataProvider.GetDataTypesAndTheirSizes());
                foreach (XmlNode FieldNode in FieldNodes)
                {
                    SchemaNode = null;
                    TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                    TableNameNode.InnerText = TableName;
                    FieldNode.AppendChild(TableNameNode);

                    //Anand :  This code is added because FieldSize is not passed for DataTypes other than
                    // String. As DAL code consider's size not to be null we need to fill in appropiate
                    // sizes depending upon the DataType(also this is database dependent).
                    FieldSizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                    if ((FieldSizeNode == null) || (String.IsNullOrEmpty(FieldSizeNode.InnerText)))
                    {
                        DataTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                        if (FieldSizeNode == null)
                        {
                            FieldSizeNode = FieldNode.OwnerDocument.CreateElement("FIELDSIZE");
                            FieldNode.AppendChild(FieldSizeNode);
                        }
                        DefaultFieldSizeNode = DataType_SizeMappingXMLDoc.DocumentElement.SelectSingleNode(DataTypeNode.InnerText.ToUpper());
                        if (DefaultFieldSizeNode != null)
                            FieldSizeNode.InnerText = DefaultFieldSizeNode.InnerText;
                        else
                            FieldSizeNode.InnerText = "0";
                    }

                    generateSchemaXML(InputXMLDocument, FieldNode, ref SchemaNode);
                    CreateErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_SCHEMA", SchemaNode.OuterXml,
                                                                                    ref TempQuery);

                 
                    //Add Field and NamedHierarchy Association in AssoFieldNamedHierarchy, Code added on date 25th August 2006 by Pallavi
                    if ((CreateErrorObj == null) && (FieldNode.SelectSingleNode("ISNAMEDHIERARCHY").InnerText.ToUpper() == "TRUE") && (FieldNode.SelectSingleNode("NAMEDHIERARCHYID") != null))
                    {
                        Assetlink.UniqueID.Read(ref AssociationID);
                        NamedHierarchyID = Convert.ToUInt64(FieldNode.SelectSingleNode("NAMEDHIERARCHYID").InnerText);
                        SchemaID = Convert.ToUInt64(FieldNode.SelectSingleNode("SCHEMAID").InnerText);
                        //--Commneted the below line as it was creating another instance of OMDataManager
                        //AssoFieldNamedHierarchy AssoFieldNHObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage, TenantName);
                        AssoFieldNamedHierarchy AssoFieldNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                        CreateErrorObj = AssoFieldNHObj.Create(AssociationID, NamedHierarchyID, SchemaID, "ASC");


                        if (CreateErrorObj != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                                "Field and NamedHierarchy Association Failed", OMErrorObjectType.ENTITYTYPE, "EntityType",
                                CreateErrorObj.ErrorType, CreateErrorObj.ExceptionObject, CreateErrorObj.ErrorXML);
                            // WriteFailed = true;
                            return ReturnErrorObj;
                        }
                    }
                  

                    //CreateErrorObj = OMDataManager.DataProvider.CreateRecord("ALOM_SCHEMA", SchemaNode.OuterXml);
                    QueryBuilder.Append(TempQuery);
                }

                //Create Table in the Database.
                TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                TableNameNode.InnerText = TableName;
                TypeNode.AppendChild(TableNameNode);

                ModifiedInputXML = InputXMLDocument.InnerXml;
                //Anand : As in some databases(Eg. Teradata)we cannot fire DML and DDL statements together we have bunch them into
                //        two sets and then fire.
                CreateErrorObj = OMDataManager.DataProvider.PrepareCreateTableQuery(InputXMLDocument.InnerXml, ref CreateTableQuery);
                                
                //CreateErrorObj = OMDataManager.DataProvider.CreateTable(ModifiedInputXML);
                //Executing the Query
                Monitor.Enter(Lock);
                OMError ErrorObj = OMDataManager.DataProvider.Execute(QueryBuilder.ToString());
                if (ErrorObj != null)
                {
                    Monitor.Exit(Lock);
                    return ErrorObj;
                }
                ErrorObj = OMDataManager.DataProvider.Execute(CreateTableQuery);
                if (ErrorObj != null)
                {
                    Monitor.Exit(Lock);
                    return ErrorObj;
                }
                Monitor.Exit(Lock);

                //Generate XSD.
                generateXSD(TypeNode);
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Create API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Create(String InputXML) InputXML: ", InputXML);
                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                        "FacetType Creation Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                WriteFailed = true;
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

                if (IsOMDataManagerInitialized)
                {
                    if (WriteFailed)
                        OMDataManager.DataProvider.RollBack();
                }
            }
        }

        /// <summary>
        /// <CreatedDate>18 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete a FacetType based on FacetTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeID">Long variable containing the FacetTypeID</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the FacetType. 
        /// </returns>
        public OMError Delete(ulong FacetTypeID)
        {
            OMError                 DeleteErrObj                = null;     //OMError object for deleting data from DB.
            OMError                 ReturnErrorObj              = null;     //OMError object for returning from DAL.
            OMError                 ReadErrorObject             = null;     //OMError object for read record in the database.
            //XmlNodeList           FieldNodes                  = null;     //DOM Node representing the list of Field nodes.
            //XmlNode               SchemaIDNode                = null;     //DOM Node representing the SchemaID node.
            XmlNode                 TableNameNode               = null;     //DOM Node representing the TableName node.
            XmlNode                 TypeNameNode                = null;     //DOM Node representing the TypeName node.
            String                  FacetTypeXML                = null;     //String variable storing the EntityType XML.
            XmlNode                 FacetTypeNode               = null;     //DOM Node representing the FacetType node.
            //String                SchemaID                    = null;     //String variable storing the SchemaID.
            String                  TableName                   = null;     //String variable storing the TableName.
            String                  TypeName                    = null;     //String variable storing the TypeName.
            StringBuilder           FilePath                    = null;     //StringBuilder pbject storing the Path of the XSD.
            StringBuilder           Message                     = null;     //StringBuilder object storing the Message that is to be logged.
            //ulong                 lSchemaID                   = 0;        //Long variable storing the SchemaID.
            String                  DelQry                      = null;
            String                  InstalledPath               = null;
            XmlDocument             FacetTypeXMLDocument        = null;     //DOM Object representing the FacetTypeXML.
            Boolean                 IsOMDataManagerInitialized  = false;    //Boolean variable storing whether OMDataManager is initialized or not.
            AssoFieldNamedHierarchy FieldNamedHierAssoObject    = null;    //AssoFieldNamedHierarchy Object.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Delete API of FacetType class in OM. Input Parameter FacetTypeID: ", FacetTypeID.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManager and OMDataManager is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table.
                DeleteErrObj = deleteTypeAssociations(FacetTypeID);
                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                       DeleteErrObj.ErrorMessage, OMErrorObjectType.FACETTYPE,
                       DeleteErrObj.ObjectTypeName, DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);
                    return ReturnErrorObj;
                }

                //Delete all the associations from the ALOM_ASSOFILEDNAMEDHIERARCHY table.
                FieldNamedHierAssoObject = new AssoFieldNamedHierarchy(OMDataManager);
                FieldNamedHierAssoObject.InternalCall = true;

                DeleteErrObj = FieldNamedHierAssoObject.DeleteAllUsingObjectTypeID(FacetTypeID);
                if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                {
                    return DeleteErrObj;
                }

                //Delete all the fields of this FacetType from the ALOM_SCHEMA table.
                ReadErrorObject = Read(FacetTypeID, ref FacetTypeXML);
                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                {
                    return ReadErrorObject;
                }

                if (FacetTypeXML != null && FacetTypeXML != "")
                {
                    FacetTypeXMLDocument = new XmlDocument();
                    FacetTypeXMLDocument.LoadXml(FacetTypeXML);

                    FacetTypeNode = FacetTypeXMLDocument.SelectSingleNode("TYPE");

                    TableNameNode = FacetTypeNode.SelectSingleNode("TABLENAME");
                    TableName = TableNameNode.InnerText;

                    TypeNameNode = FacetTypeNode.SelectSingleNode("TYPENAME");
                    TypeName = TypeNameNode.InnerText;

                    /* FieldNodes = FacetTypeNode.SelectNodes("FIELDCOLLECTION/FIELD");

                     if (FieldNodes.Count >= 1)
                     {
                         foreach (XmlNode FieldNode in FieldNodes)
                         {
                             SchemaIDNode = FieldNode.SelectSingleNode("SCHEMAID");
                             SchemaID = SchemaIDNode.InnerText;
                             lSchemaID = Convert.ToUInt64(SchemaID);

                             DeleteErrObj = deleteSchema(lSchemaID, OMErrorCodes.FACET_TYPE_DELETION_FAILED);

                             if (DeleteErrObj != null)
                             {
                                 return DeleteErrObj;
                             }
                         }
                     }*/

                    //Following code is commented by Kiran Abhang on date 10th March 2006.
                    //This code should be after DropTable call 
                    //because deletion of XML files and XSDs of object type is depend on ALOM_SCHEMA's XML fields.
                    //DelQry = "Delete from ALOM_SCHEMA where OBJECTTYPEID = " + FacetTypeID;
                    //DeleteErrObj = OMDataManager.DataProvider.Execute(DelQry);
                    //if (DeleteErrObj != null)
                    //    return DeleteErrObj;
                    
                    ////Delete FacetType record from the ALOM_FACETTYPE table.
                    //DeleteErrObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                    //                                         OMOperator.EQUALTO, FacetTypeID.ToString(), OMConjunction.NOTHING);
                    //if (DeleteErrObj != null)
                    //{
                    //    return DeleteErrObj;
                    //}

                    //DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_FACETTYPE");
                    //if (DeleteErrObj != null)
                    //{
                    //    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                    //        "FacetType Deletion Failed", OMErrorObjectType.FACETTYPE, "FacetType",
                    //        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);
                    //    return ReturnErrorObj;
                    //}

                    //Drop the table.
                    DeleteErrObj = OMDataManager.DataProvider.DropTable(TableName);
                    if (DeleteErrObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                            "FacetType Deletion Failed", OMErrorObjectType.FACETTYPE, "FacetType",
                            DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    //Delete FacetType record from the ALOM_FACETTYPE table.
                    DeleteErrObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                                                             OMOperator.EQUALTO, FacetTypeID.ToString(), OMConjunction.NOTHING);
                    if (DeleteErrObj != null)
                    {
                        return DeleteErrObj;
                    }

                    DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_FACETTYPE");
                    if (DeleteErrObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                            "FacetType Deletion Failed", OMErrorObjectType.FACETTYPE, "FacetType",
                            DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    DelQry = "DELETE FROM ALOM_SCHEMA WHERE OBJECTTYPEID = " + FacetTypeID;
                    DeleteErrObj = OMDataManager.DataProvider.Execute(DelQry);
                    if (DeleteErrObj != null)
                        return DeleteErrObj;

                    ////Delete the <FACETTYPENAME>.XSD
                    //FilePath = new StringBuilder();
                    //FilePath.AppendFormat("{0}{1}{2}", "C:\\Assetlink\\Schema\\XSD\\Type\\", TypeName, ".xsd");
                    //File.Delete(FilePath.ToString());

                    //Delete the <FACETTYPENAME>.XSD
                    InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");
                    FilePath = new StringBuilder();
                    FilePath.AppendFormat("{0}Configuration\\XSD\\Type\\{1}.xsd", InstalledPath, TypeName);
                    File.Delete(FilePath.ToString());
                }

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Delete API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Delete(ulong FacetTypeID) FacetTypeID:", FacetTypeID.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                        "FacetType Deletion Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }

        /// <summary>
        /// <CreatedDate>16 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete an FacetType based on FacetTypeName.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeName">String variable containing the FacetTypeName</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the FacetType. 
        /// </returns>
        public OMError Delete(String FacetTypeName)
        {
            OMError ReadErrObj      = null;   //OMError object for reading data from DB.
            OMError DeleteErrObj    = null;   //OMError object for deleting data from DB.
            OMError ReturnErrorObj  = null;   //OMError object for returning from DAL.
            StringBuilder Message   = null;   //StringBuilder object storing the Message that is to be logged.
            String FacetTypeXML     = null;   //String variable storing the FacetType XML.
            String TypeID           = null;   //String variable storing the value of TypeID.
            ulong lTypeID            = 0;      //Long variable storing the value of TypeID.
            XmlNode FacetTypeNode   = null;   //DOM Node representing the FacetType node.
            XmlNode FacetTypeIDNode = null;   //DOM Node representing the FacetTypeID node.
            //Boolean WriteFailed     = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            XmlDocument FacetTypeXMLDocument   = null;     //DOM Object representing the FacetTypeXML.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManager is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Delete API of FacetType class in OM. Input Parameter FacetTypeName: ", FacetTypeName);

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManager and OMDataManager is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

         
                
                //Get the FacetTypeID from the Database.

                ReadErrObj = OMDataManager.DataProvider.AddClause("FACETTYPENAME", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, FacetTypeName.ToUpper(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                {
                    //WriteFailed = true;
                    return ReadErrObj;
                }

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref FacetTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);
                    //WriteFailed = true;
                    return ReturnErrorObj;
                }

                if (FacetTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);
                    //WriteFailed = true;
                    return ReturnErrorObj;
                }

                //Get the FacetTypeID from the FacetType XML.

                FacetTypeXMLDocument = new XmlDocument();
                FacetTypeXMLDocument.LoadXml(FacetTypeXML);

                FacetTypeNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPE");

                FacetTypeIDNode = FacetTypeNode.SelectSingleNode("FACETTYPEID");
                TypeID = FacetTypeIDNode.InnerText;

                lTypeID = Convert.ToUInt64(TypeID);

                DeleteErrObj = Delete(lTypeID);

                if (DeleteErrObj != null)
                {
                    //WriteFailed = true;
                    return DeleteErrObj;
                }

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Delete API of FacetType class in OM.");

                    ExceptionTracing.TraceLog(Message.ToString(), "FacetType", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();

                Message.AppendFormat("{0}{1}", "MethodName: public OMError Delete(String FacetTypeName) FacetTypeName: ", FacetTypeName);

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_DELETION_FAILED,
                        "FacetType Deletion Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                //WriteFailed = true;
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

              
            }
        }

        #endregion Methods

        #region PrivateMethods

        /// <summary>
        /// <CreatedDate>18 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete an entry in the ALOM_SCHEMA table depending on the SchemaID.
        /// </Purpose>
        /// </summary>
        /// <param name="SchemaID">Long variable storing the SchemaID.</param>
        /// <param name="OMErrorCode">OMErrorCodes variable storing the ErrorCode.</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the schema from the ALOM_SCHEMA.
        /// </returns>
        private OMError deleteSchema(ulong SchemaID, OMErrorCodes OMErrorCode)
        {
            OMError DeleteErrObj   = null;    //The OMError object for deleting data from DB.
            OMError ReturnErrorObj = null;    //The OMError object for returning form OM.

            try
            {
                OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER,
                                                    OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING);

                if (DeleteErrObj != null)
                    return DeleteErrObj;

                DeleteErrObj = OMDataManager.DataProvider.DeleteRecord("ALOM_SCHEMA");

                if (DeleteErrObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCode,
                        "Field Deletion Failed", OMErrorObjectType.FACETTYPE, "FacetType",
                        DeleteErrObj.ErrorType, DeleteErrObj.ExceptionObject, DeleteErrObj.ErrorXML);

                    return ReturnErrorObj;
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCode,
                        "Field Deletion Failed", OMErrorObjectType.FACETTYPE, "FacetType",
                        OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }

        }
        
        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML which is required for adding a row in the ALOM_SCHEMA table.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXMLDocument">The DOM Object representing the InputXML defined in the caller function.</param>
        /// <param name="FieldNode">The DOM Node representing the Field node. 
        /// The structure of this XML Node is give below:
        /// The XML will be compliant with FIELD.XSD.
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        /// </param>
        /// <param name="SchemaNode">This is a reference parameter and will hold the XML Node which is 
        /// created in this function and returned. 
        /// The structure of this XML Node is give below:
        ///             |- SCHEMA
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        public void generateSchemaXML(XmlDocument InputXMLDocument, XmlNode FieldNode, ref XmlNode SchemaNode)
        {
            int FieldChildCount = 0;    //Integer variable storing the Child count of the Field Node.
            try
            {
                SchemaNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "SCHEMA", "");

                FieldChildCount = FieldNode.ChildNodes.Count;
                for (int index = 0; index < FieldChildCount; index++)
                    SchemaNode.AppendChild(FieldNode.ChildNodes[index].CloneNode(true));
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }

        }

        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML which is required for adding a row in the ALOM_FACETTYPE table.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">This Node represents the Type node.</param>
        /// <param name="TableName">This holds the TableName for the FacetType.</param>
        /// <param name="FacetTypeXMLDocument">This is a reference parameter and will hold the XML which is 
        /// created in this function and returned. The structure of this FacetTypeXML is give below:
        ///      FACETTYPE
        ///        |- FACETTYPEID
        ///        |- FACETTYPENAME
        ///        |- TYPEDESCRIPTION   
        ///        |- OBJECTTYPE
        ///        |- TABLENAME
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- TYPEDESCRIPTION   
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME        
        /// </param>

        public void generateFacetTypeXML(XmlNode TypeNode, String TableName, ref XmlDocument FacetTypeXMLDocument)
        {
            String TypeID               = null;     //String variable storing the value of TypeID.
            String TypeName             = null;     //String variable storing the value of TypeName.
            String Description          = null;     //String variable storing the value of Description.
            String IsBuiltIn = null;     //String variable storing the value of IsBuiltIn.
            XmlNode TypeIDNode          = null;     //DOM Node representing the TypeID node.
            XmlNode TableNameNode       = null;     //DOM Node representing the TableNameNode node.
            XmlNode FacetTypeNode       = null;     //DOM Node representing the FacetType node.
            XmlNode FacetTypeIDNode     = null;     //DOM Node representing the FacetTypeID node.
            XmlNode FacetTypeNameNode   = null;     //DOM Node representing the FacetTypeName node.
            XmlNode TypeDescriptionNode = null;     //DOM Node representing the TypeDescription node.
            XmlNode TypeNameNode        = null;     //DOM Node representing the TypeName node.
            XmlNode PrimaryKeyNode      = null;     //DOM Node representing the PrimaryKey node.
            XmlNode StorageTypeNode     = null;     //DOM Node representing the StorageType node.
            XmlNode XMLContentTypeNode  = null;     //DOM Node representing the XMLContentType node.
            XmlNode XMLLocationNode     = null;     //DOM Node representing the XMLLocation node.
            XmlNode XMLNameNode         = null;     //DOM Node representing the XMLName node.
            XmlNode XSDNameNode         = null;     //DOM Node representing the XSDName node.
            XmlNode FacetTypeDescriptionNode = null;     //DOM Node representing the FacetTypeDescription node.
            XmlNode BuiltInNode = null;     //DOM Node representing the IsBuiltIn node.
            XmlNode FacetBuiltIntypeNode = null;   //DOM Node representing the EntityBuiltIntypeNode node.

            try
            {
                // Get information from the Type XML.

                TypeIDNode = TypeNode.SelectSingleNode("TYPEID");
                TypeID     = TypeIDNode.InnerText;

                TypeNameNode = TypeNode.SelectSingleNode("TYPENAME");
                TypeName     = TypeNameNode.InnerText;

                TypeDescriptionNode = TypeNode.SelectSingleNode("TYPEDESCRIPTION");
                if (TypeDescriptionNode == null)
                    Description = "";
                else
                    Description = TypeDescriptionNode.InnerText;

                BuiltInNode = TypeNode.SelectSingleNode("ISBUILTIN");
                if (BuiltInNode == null)
                    IsBuiltIn = "0";
                else
                    IsBuiltIn = BuiltInNode.InnerText;

                // Prepare the FacetType XML.

                FacetTypeXMLDocument = new XmlDocument();

                FacetTypeNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "FACETTYPE", "");
                FacetTypeXMLDocument.AppendChild(FacetTypeNode);

                FacetTypeIDNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "FACETTYPEID", "");
                FacetTypeIDNode.InnerText = TypeID;
                FacetTypeNode.AppendChild(FacetTypeIDNode);

                FacetTypeNameNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "FACETTYPENAME", "");
                FacetTypeNameNode.InnerText = TypeName;
                FacetTypeNode.AppendChild(FacetTypeNameNode);

                FacetTypeDescriptionNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEDESCRIPTION", "");
                FacetTypeDescriptionNode.InnerText = Description;
                FacetTypeNode.AppendChild(FacetTypeDescriptionNode);

                TableNameNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                TableNameNode.InnerText = TableName;
                FacetTypeNode.AppendChild(TableNameNode);

                FacetBuiltIntypeNode = FacetTypeXMLDocument.CreateNode(XmlNodeType.Element, "ISBUILTIN", "");
                FacetBuiltIntypeNode.InnerText = IsBuiltIn;
                FacetTypeNode.AppendChild(FacetBuiltIntypeNode);

                PrimaryKeyNode     = TypeNode.SelectSingleNode("PRIMARYKEY");
                StorageTypeNode    = TypeNode.SelectSingleNode("STORAGETYPE");
                XMLContentTypeNode = TypeNode.SelectSingleNode("XMLCONTENTTYPE");
                XMLLocationNode    = TypeNode.SelectSingleNode("XMLLOCATION");
                XMLNameNode        = TypeNode.SelectSingleNode("XMLNAME");
                XSDNameNode        = TypeNode.SelectSingleNode("XSDNAME");

                if (PrimaryKeyNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(PrimaryKeyNode, true));
                if (StorageTypeNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(StorageTypeNode, true));
                if (XMLContentTypeNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(XMLContentTypeNode, true));
                if (XMLLocationNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(XMLLocationNode, true));
                if (XMLNameNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(XMLNameNode, true));
                if (XSDNameNode != null)
                    FacetTypeNode.AppendChild(FacetTypeXMLDocument.ImportNode(XSDNameNode, true));

            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will return a tablename for the given FacetTypeName.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeName">
        /// This contains the FacetTypeName.
        /// </param>
        /// <returns>
        /// TableName.
        /// </returns>
        private String generateTableName(String TypeName)
        {
            StringBuilder TableName = null;    //The StringBuilder object storing the value of TableName. 
            try
            {
                TableName = new StringBuilder();
                TableName.AppendFormat("{0}{1}", "ALFCT_", TypeName.ToUpper());
                return TableName.ToString();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }
        
        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed to outside.
        /// The function checks whether FacetType Exists in the system or Not.
        /// </summary>
        /// <param name="FacetTypeName">This contains the FacetTypeName which is to be searched.</param>
        /// <returns>
        /// true: If FacetType exist.
        /// false:If FacetType not exist.
        /// </returns>
        public bool isFacetTypeExists(String FacetTypeName)
        {
            OMError ReadErrObj                 = null;   //The OMError object for reading data from DB.
            String OutputXML                   = null;   //The String variable storing the Output XML.

            try
            {
                ReadErrObj = OMDataManager.DataProvider.AddClause("FACETTYPENAME", OMFieldDataType.STRING,
                                        OMOperator.EQUALTO, FacetTypeName,
                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return false;

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref OutputXML);

                if (ReadErrObj != null || OutputXML == null || OutputXML == "")
                    return false;
                else
                    return true;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML Schema (XSD) for all the FacetTypes that are created in the system.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">This XML has all the data depending on which the XSD is to be generated.
        /// The structure of this XML Node is give below:
        /// The XML will be compliant with Type.xsd.
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- TYPENAME      (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- TABLENAME
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME
         ///       |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     
        ///                 |- FIELDNAME    
        ///                 |- FIELDTYPE    
        ///                 |- FIELDSIZE
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISPRIMARYKEY
        ///                 |- ISNAMEDLIST
        ///                 |- FIELDUITYPE
        ///                 |- OBJECTTYPEID 
        ///                 |- OBJECTTYPE   
        ///        (*) Mandatory Element
        /// </param>
        private void generateXSD(XmlNode TypeNode)
        {
            int                     MaxMinOccurs        = 0;        //An Integer variable which stores the Max & Min occurance of an element.
            XmlNode                 FieldCollectionNode = null;     //DOM Node representing the FieldCollection node.
            XmlNode                 NameNode            = null;     //DOM Node representing the Name node.
            XmlNodeList             FieldNodes          = null;     //DOM Node representing the list of Field nodes.
            XmlNode                 FieldNameNode       = null;     //DOM Node representing the Fieldname node.
            XmlNode                 FieldTypeNode       = null;     //DOM Node representing the FieldType node.
            XmlNode                 IsMandatoryNode     = null;     //DOM Node representing the IsMandatory node.
            XmlNode                 IsPrimaryKeyNode    = null;     //DOM Node representing the IsPrimaryKey node.

            String                  FieldName           = null;     //String variable storing the value of FieldName.
            String                  FieldType           = null;     //String variable storing the value of FieldType.
            String                  IsMandatory         = null;     //String variable storing the value of IsMandatory.
            String                  IsPrimaryKey        = null;     //String variable storing the value of IsPrimaryKey.
            StringBuilder           FilePath            = null;     //The StringBuilder object storing the XSD file path.

            XmlSchema               SchemaNode          = null;   //This contains the definition of a XML schema.
            XmlSchemaInclude        SchemaInclude       = null;   //Used to define the includes in an XML schema.
            XmlSchemaElement        elementTypeName     = null;   //Used to describe a TypeName element in a XML document.
            XmlSchemaComplexType    complexType         = null;   //Used to define a complex type.
            XmlSchemaComplexType    SchemacomplexType   = null;   //Used to define a complex type.
            XmlSchemaSequence       sequence            = null;   //Used to define the sequence of elements within the containing element.
            XmlSchemaSequence       SchemaSequence      = null;   //Used to define the sequence of elements within the containing element.
            XmlSchemaElement        elementFieldName    = null;   //Used to describe a FieldName element in a XML document.
            XmlSchemaElement        elementDay          = null;   //Used to describe a Day element in a XML document.
            XmlSchemaElement        elementMonth        = null;   //Used to describe a Month element in a XML document.
            XmlSchemaElement        elementYear         = null;   //Used to describe a Year element in a XML document.
            XmlSchemaElement        elementHour         = null;   //Used to describe a Hour element in a XML document.
            XmlSchemaElement        elementMinute       = null;   //Used to describe a Minute element in a XML document.
            XmlSchemaElement        elementSecond       = null;   //Used to describe a Second element in a XML document.
            XmlSchemaAttribute      attributeType       = null;   //Used to describe an attribute for an element.
            XmlNamespaceManager     NameSpaceMgr        = null;   //Used to add a namespace to the XSD.
            Stream                  iStream             = null;   //Stream object used for creating an XSD file.
            StreamWriter            StreamWriterObject  = null;   //StreamWriter object used for writing the XNL schema to the file.
            String                  InstalledPath       = null;
            IXMLRepository          ObjXMLManager       = null;
            String                  XSDFilePath         = null;

            try
            {
                NameNode = TypeNode.SelectSingleNode("TYPENAME");

                FieldCollectionNode = TypeNode.SelectSingleNode("FIELDCOLLECTION");
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");

                SchemaNode = new XmlSchema();

                // <Assetlink:include schemaLocation="Common.xsd" />               
                SchemaInclude = new XmlSchemaInclude();
                SchemaInclude.SchemaLocation = "../System/Common.xsd";
                SchemaNode.Includes.Add(SchemaInclude);

                //Including XSD file path for XML column.
                //Getting the fields which have XML data type.
                XmlNodeList XMLColumnNodes = null;
                XMLColumnNodes = TypeNode.SelectNodes("//TYPE/FIELDCOLLECTION/FIELD[FIELDTYPE/text()='XML']");
                if ((XMLColumnNodes != null) && (XMLColumnNodes.Count > 0))
                {
                    ObjXMLManager = OMDataManager.XMLRepository; //new XMLManager(TenantName);
                    foreach (XmlNode XMLColumnNode in XMLColumnNodes)
                    {
                        if (XMLColumnNode.SelectSingleNode("XSDPATH") != null)
                        {
                            //if ((XMLColumnNode.SelectSingleNode("FIELDTYPE").Attributes["XSDFilePath"].Value != null)
                            //    && (XMLColumnNode.SelectSingleNode("FIELDTYPE").Attributes["XSDFilePath"].Value.ToString().Trim() != ""))
                            //{
                                //Get the XSD file at application's XSD type location.
                                XSDFilePath = null;
                                ObjXMLManager.RetrieveXSDPath(NameNode.InnerText.Trim().ToUpper(), XMLColumnNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper(), out XSDFilePath);

                                //If XSD file path contains NULL or empty string then the XSD file is not present.
                                if ((XSDFilePath != null) && (XSDFilePath.Trim().Length > 0))
                                {
                                    //Include XSD file like <Assetlink:include schemaLocation="Common.xsd" />
                                    SchemaInclude = new XmlSchemaInclude();
                                    SchemaInclude.SchemaLocation = "../Type/" + NameNode.InnerText.Trim().ToUpper() + "_" + XMLColumnNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper() + ".xsd";
                                    SchemaNode.Includes.Add(SchemaInclude);
                                }
                            //}
                        }
                    }
                }
                //End of including XSD file path for XML column.

                // <Assetlink:element name="<TypeName>"/>
                elementTypeName = new XmlSchemaElement();
                SchemaNode.Items.Add(elementTypeName);
                elementTypeName.Name = NameNode.InnerText.ToUpper();

                // <Assetlink:complexType>
                SchemacomplexType = new XmlSchemaComplexType();
                elementTypeName.SchemaType = SchemacomplexType;

                // <Assetlink:sequence>
                SchemaSequence = new XmlSchemaSequence();
                SchemacomplexType.Particle = SchemaSequence;

                foreach (XmlNode FieldNode in FieldNodes)
                {
                    FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                    FieldName = FieldNameNode.InnerText;

                    FieldTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                    FieldType = FieldTypeNode.InnerText;

                    IsMandatoryNode = FieldNode.SelectSingleNode("ISMANDATORY");
                    if (IsMandatoryNode == null)
                        IsMandatory = "FALSE";
                    else IsMandatory = IsMandatoryNode.InnerText;

                    IsPrimaryKeyNode = FieldNode.SelectSingleNode("ISPRIMARYKEY");
                    if (IsPrimaryKeyNode == null)
                        IsPrimaryKey = "FALSE";
                    else IsPrimaryKey = IsPrimaryKeyNode.InnerText;


                    //<Assetlink:element name="<FieldName>" type="<datatype>"/>
                    elementFieldName = new XmlSchemaElement();
                    SchemaSequence.Items.Add(elementFieldName);
                    elementFieldName.Name = FieldName;

                    switch (FieldType.ToUpper())
                    {
                        case "STRING":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("string", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "BOOLEAN":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("string", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "INTEGER":
                            if (IsPrimaryKey.ToUpper() == "TRUE")
                                elementFieldName.SchemaTypeName = new XmlQualifiedName("IDRULE");
                            else
                                elementFieldName.SchemaTypeName = new XmlQualifiedName("integer", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "DECIMAL":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("decimal", "http://www.w3.org/2001/XMLSchema");
                            break;

                        case "DATETIME":
                            // <Assetlink:complexType>
                            complexType = new XmlSchemaComplexType();
                            elementFieldName.SchemaType = complexType;

                            // <Assetlink:sequence>
                            sequence = new XmlSchemaSequence();
                            complexType.Particle = sequence;

                            //<Assetlink:element name="Day" type="Assetlink:short"/>
                            elementDay = new XmlSchemaElement();
                            sequence.Items.Add(elementDay);
                            elementDay.Name = "DAY";
                            elementDay.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            //<Assetlink:element name="Month" type="Assetlink:byte"/>
                            elementMonth = new XmlSchemaElement();
                            sequence.Items.Add(elementMonth);
                            elementMonth.Name = "MONTH";
                            elementMonth.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            //<Assetlink:element name="Year" type="Assetlink:short"/>
                            elementYear = new XmlSchemaElement();
                            sequence.Items.Add(elementYear);
                            elementYear.Name = "YEAR";
                            elementYear.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            //<Assetlink:element name="Hour" type="Assetlink:short"/>
                            elementHour = new XmlSchemaElement();
                            sequence.Items.Add(elementHour);
                            elementHour.Name = "HOUR";
                            elementHour.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            //<Assetlink:element name="Minute" type="Assetlink:byte"/>
                            elementMinute = new XmlSchemaElement();
                            sequence.Items.Add(elementMinute);
                            elementMinute.Name = "MINUTE";
                            elementMinute.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            //<Assetlink:element name="Second" type="Assetlink:short"/>
                            elementSecond = new XmlSchemaElement();
                            sequence.Items.Add(elementSecond);
                            elementSecond.Name = "SECOND";
                            elementSecond.SchemaTypeName = new XmlQualifiedName("short", "http://www.w3.org/2001/XMLSchema");

                            // <Assetlink:attribute name="Type">
                            attributeType = new XmlSchemaAttribute();
                            complexType.Attributes.Add(attributeType);
                            attributeType.Name = "Type";
                            attributeType.SchemaTypeName = new XmlQualifiedName("string", "http://www.w3.org/2001/XMLSchema");
                            attributeType.Use = XmlSchemaUse.Required;
                            break;

                        case "XML":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName(Convert.ToString(FieldName.Trim().ToUpper() + "TYPE"));
                            break;
                        case "CURRENCY":
                            elementFieldName.SchemaTypeName = new XmlQualifiedName("CURRENCY");//, "http://www.w3.org/2001/XMLSchema");
                            break;
                    }

                    if ((IsMandatory.ToUpper() == "TRUE") || (IsPrimaryKey.ToUpper() == "TRUE"))
                    {
                        MaxMinOccurs = 1;
                        elementFieldName.MaxOccurs = MaxMinOccurs;
                        elementFieldName.MinOccurs = MaxMinOccurs;
                    }
                    else
                    {
                        MaxMinOccurs = 1;
                        elementFieldName.MaxOccurs = MaxMinOccurs;
                        MaxMinOccurs = 0;
                        elementFieldName.MinOccurs = MaxMinOccurs;
                    }
                }

                NameSpaceMgr = new XmlNamespaceManager(new NameTable());
                NameSpaceMgr.AddNamespace("Assetlink", "http://www.w3.org/2001/XMLSchema");

                //Write the XML Schema to a file.
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");
                FilePath = new StringBuilder();
                FilePath.AppendFormat("{0}Configuration\\XSD\\Type\\{1}.xsd", InstalledPath, NameNode.InnerText);
                
                iStream = new FileStream(FilePath.ToString(), System.IO.FileMode.Create, System.IO.FileAccess.ReadWrite);
                StreamWriterObject = new StreamWriter(iStream);
                SchemaNode.Write(StreamWriterObject, NameSpaceMgr);
                StreamWriterObject.Close();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>19 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will delete all the associations from the ALOM_RELATIONSHIPTYPES table for the given FacetTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeID">Long variable containing the FacetTypeID for which the associations from the 
        /// ALOM_RELATIONSHIPTYPES are to be deleted.</param>
        /// <returns>
        /// NULL: If the deletion was successful.
        /// Object of OMError if an error occurred while deleting the associations from the ALOM_RELATIONSHIPTYPES table.
        /// </returns>
        private OMError deleteTypeAssociations(ulong FacetTypeID)
        {
            OMError DeleteErrObj              = null;    //OMError object for deleting data from DB.
            OMError ReturnErrorObj            = null;    //OMError object for returning from OM.
            OMError ReadErrorObject           = null;    //OMError object for read record in the database.
            String AssociationXML             = null;    //String variable storing the AssociationXML XML.
            String ContainerTypeID            = null;    //String variable storing the ContainerTypeID.
            String ContainsTypeID             = null;    //String variable storing the ContainsTypeID.
            ulong lContainerTypeID             = 0;       //Long variable storing the ContainerTypeID.
            ulong lContainsTypeID              = 0;       //Long variable storing the ContainsTypeID.
            XmlNode RelationshipTypeCollNode  = null;    //DOM Object representing the ObjectTypesAssoCollection Node.
            XmlNodeList RelationshipTypeNodes = null;    //DOM Object representing the ObjectTypeAsso Nodes.

            XmlDocument AssociationXMLDocument             = null;    //DOM Object representing the EntityTypeXML.
            RelationshipType RelationshipTypeObject        = null;    //RelationshipType Object.

            try
            {
                RelationshipTypeObject = new RelationshipType(OMDataManager);
                RelationshipTypeObject.InternalCall = true;

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table where the FacetTypeID maps to the 
                //ContainerTypeID.

                ReadErrorObject = RelationshipTypeObject.ReadAllByContainerType(FacetTypeID, "FACET", "",
                                                                              ref AssociationXML);

                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                    return ReadErrorObject;

                if ((AssociationXML != "") && (AssociationXML != null))
                {
                    AssociationXMLDocument = new XmlDocument();
                    AssociationXMLDocument.LoadXml(AssociationXML);

                    RelationshipTypeCollNode = AssociationXMLDocument.SelectSingleNode("RELATIONSHIPTYPECOLLECTION");
                    RelationshipTypeNodes = RelationshipTypeCollNode.SelectNodes("RELATIONSHIPTYPE");

                    if (RelationshipTypeNodes.Count > 0)
                    {
                        foreach (XmlNode RelationshipTypeNode in RelationshipTypeNodes)
                        {
                            ContainerTypeID = null;
                            ContainsTypeID = null;
                            lContainerTypeID = 0;
                            lContainsTypeID = 0;

                            ContainerTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINERTYPEID").InnerText;
                            ContainsTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINSTYPEID").InnerText;

                            lContainerTypeID = Convert.ToUInt64(ContainerTypeID);
                            lContainsTypeID = Convert.ToUInt64(ContainsTypeID);

                            DeleteErrObj = RelationshipTypeObject.Delete(lContainerTypeID, lContainsTypeID);

                            if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                                return DeleteErrObj;
                        }
                    }
                }

                //Delete all the associations from the ALOM_RELATIONSHIPTYPES table where the FacetTypeID maps to the 
                //ContainsTypeID.

                AssociationXML = null;

                ReadErrorObject = RelationshipTypeObject.ReadAllByContainsType(FacetTypeID, "FACET", "",
                                                                              ref AssociationXML);

                if (ReadErrorObject != null && ReadErrorObject.ErrorType != OMErrorType.ERROR)
                    return ReadErrorObject;

                if ((AssociationXML != "") && (AssociationXML != null))
                {
                    AssociationXMLDocument = new XmlDocument();
                    AssociationXMLDocument.LoadXml(AssociationXML);

                    RelationshipTypeCollNode = AssociationXMLDocument.SelectSingleNode("RELATIONSHIPTYPECOLLECTION");
                    RelationshipTypeNodes = RelationshipTypeCollNode.SelectNodes("RELATIONSHIPTYPE");

                    if (RelationshipTypeNodes.Count > 0)
                    {
                        foreach (XmlNode RelationshipTypeNode in RelationshipTypeNodes)
                        {
                            ContainerTypeID = null;
                            ContainsTypeID = null;
                            lContainerTypeID = 0;
                            lContainsTypeID = 0;

                            ContainerTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINERTYPEID").InnerText;
                            ContainsTypeID = RelationshipTypeNode.SelectSingleNode("CONTAINSTYPEID").InnerText;

                            lContainerTypeID = Convert.ToUInt64(ContainerTypeID);
                            lContainsTypeID = Convert.ToUInt64(ContainsTypeID);

                            DeleteErrObj = RelationshipTypeObject.Delete(lContainerTypeID, lContainsTypeID);

                            if (DeleteErrObj != null && DeleteErrObj.ErrorType != OMErrorType.ERROR)
                                return DeleteErrObj;
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ASSOCIATION_DELETION_FAILED,
                        "EntityType FacetType Association Deletion Failed", OMErrorObjectType.ASSOCIATION,
                        "EntityType FacetType Association", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        #endregion PrivateMethods
    }
}
