using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Collections;
using System.Collections.Specialized;
using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.OMCommon;
using Assetlink.Diagnostic;
using Assetlink.ObjectManager.NamedHierarchy;

namespace Assetlink.ObjectManager.ETFTManagement
{
    /// <summary>
    /// <CreatedDate>12 May 2005</CreatedDate>
    /// <Author>Mr. Hemant Kirad</Author>
    /// <Purpose>
    /// This class represents an FacetType. An FacetType represents an facet involved in the business world.
    /// This is a partial class so it will have some of its functionality divided into
    /// 1. FacetType.cs
    /// 2. FacetTypeExt1.cs (This file)
    /// </Purpose>
    /// </summary>
    internal partial class FacetType
    {
        #region PrivateMethods
        /// <summary>
        /// <CreatedDate>11 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is a helper function not exposed outside. 
        /// This function will generate a XML complaint with the TYPE.XSD.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeNode">This Node represents the FacetType node.</param>
        /// <param name="TypeNode">This is a reference parameter and will hold the XML Node, which is 
        /// created in this function and returned. The structure of this TypeNode is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- TYPENAME      (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- TABLENAME
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME
        ///        (*) Mandatory Element
        /// </param>
        private void generateTypeXML(XmlNode FacetTypeNode, ref XmlDocument TypeXMLDocument)
        {
            String TypeID               = null;     //String variable storing the value of TypeID.
            String TypeName             = null;     //String variable storing the value of TypeName.
            String Description          = null;     //String variable storing the value of Description.
            XmlNode TypeIDNode          = null;     //DOM Node representing the TypeID node.
            XmlNode NameNode            = null;     //DOM Node representing the Name node.
            XmlNode DescriptionNode     = null;     //DOM Node representing the Description node.
            XmlNode TypeNode            = null;     //DOM Node representing the Type node.
            XmlNode ObjectTypeNode      = null;     //DOM Node representing the ObjectType node.
            XmlNode FacetTypeIDNode     = null;     //DOM Node representing the FacetTypeID node.
            XmlNode FacetTypeNameNode   = null;     //DOM Node representing the FacetTypeName node.
            XmlNode TypeDescriptionNode = null;     //DOM Node representing the TypeDescription node.
            XmlNode FacetTypeDisplayNameNode = null;//DOM Node representing the FacetTypeDisplayName node.

            try
            {
                // Get information from the FacetType XML.

                FacetTypeIDNode = FacetTypeNode.SelectSingleNode("FACETTYPEID");
                TypeID = FacetTypeIDNode.InnerText;

                FacetTypeNameNode = FacetTypeNode.SelectSingleNode("FACETTYPENAME");
                TypeName = FacetTypeNameNode.InnerText;

                TypeDescriptionNode = FacetTypeNode.SelectSingleNode("TYPEDESCRIPTION");
                if (TypeDescriptionNode == null)
                    Description = "";
                else
                    Description = TypeDescriptionNode.InnerText;

                // Prepare the Type XML.

                TypeXMLDocument = new XmlDocument();

                TypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPE", "");
                TypeXMLDocument.AppendChild(TypeNode);

                TypeIDNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPEID", "");
                TypeIDNode.InnerText = TypeID;
                TypeNode.AppendChild(TypeIDNode);

                NameNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "TYPENAME", "");
                NameNode.InnerText = TypeName;
                TypeNode.AppendChild(NameNode);

                DescriptionNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "DESCRIPTION", "");
                DescriptionNode.InnerText = Description;
                TypeNode.AppendChild(DescriptionNode);

                ObjectTypeNode = TypeXMLDocument.CreateNode(XmlNodeType.Element, "OBJECTTYPE", "");
                ObjectTypeNode.InnerText = "FACET";
                TypeNode.AppendChild(ObjectTypeNode);

                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("TABLENAME"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("PRIMARYKEY"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("STORAGETYPE"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("XMLCONTENTTYPE"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("XMLLOCATION"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("XMLNAME"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("XSDNAME"), true));
                TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("ISBUILTIN"), true));
                // check for backward compatability.
                FacetTypeDisplayNameNode = FacetTypeNode.SelectSingleNode("DISPLAYNAME");
                if( FacetTypeDisplayNameNode != null)
                    TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("DISPLAYNAME"), true));
               
                XmlNode isFullTextEnabledNode = FacetTypeNode.SelectSingleNode("FULLTEXTENABLED");
                if(isFullTextEnabledNode != null)
                    TypeNode.AppendChild(TypeXMLDocument.ImportNode(FacetTypeNode.SelectSingleNode("FULLTEXTENABLED"), true));

            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedDate>01 June 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method returns the FacetType TableName for the given FacetTypeID.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeID">String variable containing the TypeID.</param>
        /// <param name="TableName">String variable which stores the TableName.</param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the TableName from the database.
        /// </returns>
        private OMError getTableNameFromDB(String TypeID, ref String TableName)
        {
            OMError ReadErrorObj               = null;    //OMError object for read record in the database.
            OMError ReturnErrorObj             = null;    //OMError object used to return an error from OM.
            String FacetTypeXML                = null;    //String variable storing the FacetType XML.
            XmlDocument FacetTypeXMLDocument   = null;    //DOM Object representing the FacetType XML.

            try
            {
                ReadErrorObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                                                            OMOperator.EQUALTO, TypeID, OMConjunction.NOTHING);
                if (ReadErrorObj != null)
                    return ReadErrorObj;

                ReadErrorObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                        ref FacetTypeXML);
                if (ReadErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrorObj.ErrorType, ReadErrorObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }
                if (FacetTypeXML == null || FacetTypeXML == "")
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }
                else
                {
                    FacetTypeXMLDocument = new XmlDocument();
                    FacetTypeXMLDocument.LoadXml(FacetTypeXML);
                    TableName = FacetTypeXMLDocument.SelectSingleNode("FACETTYPE/TABLENAME").InnerText;
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>01 June 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will update the FacetType record in the ALOM_FACETTYPE table.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeNode">DOM object representing the TypeNode.</param>
        /// <param name="TypeID">String variable storing the TypeID.</param>
        /// <param name="TableName">String variable storing the TableName.</param>
        /// <returns>
        /// NULL: If the update was successful.
        /// Object of OMError if an error occurred while updating the FacetType in the database.
        /// </returns>
        private OMError updateFacetType(XmlNode TypeNode, String TypeID, String TableName)
        {
            OMError UpdateErrorObj           = null;     //OMError object for create record in the database.
            OMError ReturnErrorObj           = null;     //OMError object for returning from OM.
            XmlDocument FacetTypeXMLDocument = null;     //DOM Object representing the FacetTypeXML.
            XmlNode FacetTypeNode            = null;     //DOM Object representing the FacetType Node.

            try
            {
                generateFacetTypeXML(TypeNode, TableName, ref FacetTypeXMLDocument);

                UpdateErrorObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO,
                                                        TypeID, OMConjunction.NOTHING);

                if (UpdateErrorObj != null)
                    return UpdateErrorObj;

                FacetTypeNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPE");

                FacetTypeNode.RemoveChild(FacetTypeNode.SelectSingleNode("FACETTYPEID"));

                UpdateErrorObj = OMDataManager.DataProvider.UpdateRecord("ALOM_FACETTYPE", FacetTypeXMLDocument.InnerXml);

                if (UpdateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                        "FacetType Modification Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                        "FacetType Modification Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>01 June 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This is an helper method.
        /// The method update the schema information of an FacetType in the ALOM_SCHEMA table.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXMLDocument">DOM object representing the InputXML.</param>
        /// <param name="TableName">String variable storing the TableName.</param>
        /// <param name="FieldCollectionNode">DOM object representing the FieldCollection Node.</param>
        /// <returns>
        /// NULL: If the update was successful.
        /// Object of OMError if an error occurred while updating the schema in the database.
        /// </returns>
        private OMError updateSchema(XmlDocument InputXMLDocument, String TableName, XmlNode FieldCollectionNode)
        {
            OMError UpdateErrorObj = null;                  //OMError object for create record in the database.
            OMError ReturnErrorObj = null;                  //OMError object for returning from OM.
            XmlNodeList FieldNodes = null;                  //DOM Node representing the list of Field nodes.
            XmlNode SchemaNode     = null;                  //DOM Node representing the Schema node.
            XmlNode TableNameNode  = null;                  //DOM Node representing the TableName node.
            String SchemaID        = null;                  //String variable storing the SchemaID.
            String OperationType   = null;                  //String variable storing the OperationType. 
            ulong lSchemaID         = 0;                    //Long variable storing the SchemaID.
            XmlDocument     DataType_SizeMappingXMLDoc  = null;
            XmlNode         FieldSizeNode               = null;
            XmlNode         DefaultFieldSizeNode        = null;
            XmlNode         DataTypeNode                = null;  
            
            
            
            
            


            try
            {
                DataType_SizeMappingXMLDoc = new XmlDocument();
                //Get all the FieldNodes from the FieldCollectionNode.
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                if (FieldNodes.Count > 0)
                {
                    DataType_SizeMappingXMLDoc.LoadXml(OMDataManager.DataProvider.GetDataTypesAndTheirSizes());
                    foreach (XmlNode FieldNode in FieldNodes)
                    {
                        //For each field read the OperationType attribute and depending on it do the appropriate.
                        SchemaNode = null;
                        SchemaID = null;
                        
                        SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                        OperationType = FieldNode.Attributes.GetNamedItem("OperationType").InnerText;

                        if (OperationType.ToUpper() == "NEW")
                        {
                            //Anand :  This code is added because FieldSize is not passed for DataTypes other than
                            // String. As DAL code consider's size not to be null we need to fill in appropiate
                            // sizes depending upon the DataType(also this is database dependent).
                            FieldSizeNode = FieldNode.SelectSingleNode("FIELDSIZE");
                            if ((FieldSizeNode == null) || (String.IsNullOrEmpty(FieldSizeNode.InnerText)))
                            {
                                DataTypeNode = FieldNode.SelectSingleNode("FIELDTYPE");
                                if (FieldSizeNode == null)
                                {
                                    FieldSizeNode = FieldNode.OwnerDocument.CreateElement("FIELDSIZE");
                                    FieldNode.AppendChild(FieldSizeNode);
                                }
                                DefaultFieldSizeNode = DataType_SizeMappingXMLDoc.DocumentElement.SelectSingleNode(DataTypeNode.InnerText.ToUpper());
                                if (DefaultFieldSizeNode != null)
                                    FieldSizeNode.InnerText = DefaultFieldSizeNode.InnerText;
                                else
                                    FieldSizeNode.InnerText = "0";
                            }
                        }

                        generateSchemaXML(InputXMLDocument, FieldNode, ref SchemaNode);

                        switch (OperationType.ToUpper())
                        {
                            case "NEW":
                                TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                                TableNameNode.InnerText = TableName;
                                SchemaNode.AppendChild(TableNameNode);

                                UpdateErrorObj = OMDataManager.DataProvider.CreateRecord("ALOM_SCHEMA",
                                                                                         SchemaNode.OuterXml);

                                if (UpdateErrorObj != null)
                                {
                                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                                        "Field Creation Failed", OMErrorObjectType.FACETTYPE,
                                        "FacetType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                                        UpdateErrorObj.ErrorXML);

                                    return ReturnErrorObj;
                                }
                                break;

                            case "MODIFY":
                                UpdateErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID",
                                OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID, OMConjunction.NOTHING);

                                if (UpdateErrorObj != null)
                                    return UpdateErrorObj;

                                SchemaNode.RemoveChild(SchemaNode.SelectSingleNode("SCHEMAID"));

                                if (UpdateErrorObj == null)
                                    UpdateErrorObj = OMDataManager.DataProvider.UpdateRecord("ALOM_SCHEMA",
                                                                                         SchemaNode.OuterXml);

                                if (UpdateErrorObj != null)
                                {
                                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                                        "Field Modification Failed", OMErrorObjectType.FACETTYPE,
                                        "FacetType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                                        UpdateErrorObj.ErrorXML);

                                    return ReturnErrorObj;
                                }
                                break;

                            case "DELETE":
                                lSchemaID = Convert.ToUInt64(SchemaID);
                                if (UpdateErrorObj == null)
                                     UpdateErrorObj = deleteSchema(lSchemaID,
                                                              OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED);

                                if (UpdateErrorObj != null)
                                    return UpdateErrorObj;
                                break;
                        }
                    }
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                        "Field Modification Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>19 July 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will update XSD for the FacetType.
        /// </Purpose>
        /// </summary>
        /// <param name="TypeID">ulong variable storing the TypeID.</param>
        private void updateXSD(ulong TypeID)
        {
            OMError ReadErrObj          = null;     //OMError object for read record in the database.
            String TypeXML              = null;     //String variable storing the TypeXML.
            XmlDocument TypeXMLDocument = null;     //DOM Object representing the TypeXML.
            XmlNode TypeNode            = null;     //DOM Object representing the Type Node.

            try
            {
                ReadErrObj = Read(TypeID, ref TypeXML);

                if ((ReadErrObj == null) && (TypeXML != null) && (TypeXML != ""))
                {
                    TypeXMLDocument = new XmlDocument();
                    TypeXMLDocument.LoadXml(TypeXML);

                    TypeNode = TypeXMLDocument.SelectSingleNode("TYPE");

                    generateXSD(TypeNode);
                }
            }
            catch (Exception ExceptionObject)
            {
                throw ExceptionObject;
            }
        }
        
        //Following method is added for XML data type support on date 8th March 2006 by Kiran Abhang.
        //This method is used to get the XSD file and XSD file path for fields which are of type XML.
        //XSD file is copied on the application's location and XSD file path is added as an attribute in the
        // field's FieldType node.
        private OMError GetXSDFileAndPathForXMLTypeFields(String TypeID, String ObjectTypeName,
                                            String ObjectType, ref String OutputXML)
        {
            OMError         ReadErrObj              = null;
            IXMLRepository  ObjXMLManager           = null;
            XmlDocument     FieldXMLDocument        = null;
            XmlDocument     FieldListXMLDocument    = null;
            XmlNodeList     FieldsList              = null;
            XmlNode         XSDFieldNameNode        = null;
            XmlNode         CollectionNode          = null;
            XmlNode         XSDPathNode             = null;
            String          FieldName               = null;
            String          XSDFilePath             = null;
            String          FieldCollectionXML      = null;

            try
            {
                //Selecting fields which are of type XML.
                FieldCollectionXML = null;
                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, TypeID.ToString(),
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "ENTITY",
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("FIELDTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "XML",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                        ref FieldCollectionXML);
                if ((ReadErrObj != null) || (FieldCollectionXML == null) || (FieldCollectionXML == ""))
                    return (null);
                else
                {
                    FieldXMLDocument = new XmlDocument();
                    FieldXMLDocument.LoadXml(FieldCollectionXML);
                    CollectionNode = FieldXMLDocument.DocumentElement;
                    FieldsList = CollectionNode.SelectNodes("FIELD");

                    FieldListXMLDocument = new XmlDocument();
                    FieldListXMLDocument.LoadXml(OutputXML);

                    if ((FieldsList != null) && (FieldsList.Count > 0))
                    {
                        ObjXMLManager = OMDataManager.XMLRepository; //new XMLManager(TenantName);
                        foreach (XmlNode FieldNode in FieldsList)
                        {
                            FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper();
                            //Get the XSD file and XSD file path.
                            ReadErrObj = ObjXMLManager.RetrieveXSDPath(ObjectTypeName, FieldName, out XSDFilePath);
                            if ((XSDFilePath != null) && (XSDFilePath.Length > 0))
                            {
                                //Adding XSDFilePath attribute for XML data type column.
                                XSDFieldNameNode = FieldListXMLDocument.SelectSingleNode("//TYPE/FIELDCOLLECTION/FIELD[FIELDNAME/text()='" + FieldName.Trim().ToUpper() + "']");
                                if (XSDFieldNameNode != null)
                                {
                                    XSDPathNode = FieldListXMLDocument.CreateElement("XSDPATH");
                                    XSDPathNode.InnerText = XSDFilePath;
                                    XSDFieldNameNode.AppendChild(XSDPathNode);
                                }
                            }
                        }
                    }
                    OutputXML = FieldListXMLDocument.OuterXml;
                }
            }
            catch (Exception ExceptionObject)
            {
                throw ExceptionObject;
            }
            return (null);
        }
        #endregion PrivateMethods

        #region Methods

        /// <summary>
        ///<CreatedDate>11 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read the FacetType based on FacetTypeID. This will return
        /// FacetType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeID">This contains the FacetTypeID.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the information about the FacetType. 
        /// The structure of this XML is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- TYPENAME      (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- TABLENAME
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If read was successful.
        /// Object of OMError if an error occurred while reading the FacetType.
        /// </returns>
        public OMError Read(ulong FacetTypeID, ref String OutputXML)
        {
            OMError ReadErrObj          = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj      = null;   //OMError object for returning from OM.
            String FacetTypeXML         = null;   //String variable storing the FacetType XML.
            String FieldCollectionXML   = null;   //String variable storing the FieldCollection XML.
            String FacetTypeName        = null;
            XmlNode FieldCollectionNode = null;   //DOM Node representing the FieldCollection node.
            XmlNode TypeXMLNode         = null;   //DOM Node representing the Type node.
            XmlNode FacetTypeNode       = null;   //DOM Node representing the FacetType node.

            StringBuilder Message                   = null;  //StringBuilder object storing the Message that is to be logged.
            StringCollection TableNameCollection    = null;  //StringCollection object to store the TableNames.
            OMCommon.OMCommon OMCommonObject        = null;  //OMCommon Object.
            XmlDocument TypeXMLDocument             = null;  //The DOM Object representing the Type XML.
            XmlDocument FacetTypeXMLDocument        = null;  //The DOM Object representing the FacetType XML.
            XmlDocument FieldCollectionXMLDocument  = null;  //The DOM Object representing the FieldCollection XML.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManager is initialized or not.
            AssoFieldNamedHierarchy AssoNHObj = null;

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Read API of FacetType class in OM. Input Parameter FacetTypeID: ", FacetTypeID.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the FacetType Info...

                ReadErrObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, FacetTypeID.ToString(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref FacetTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                     return ReturnErrorObj;
                }

                if (FacetTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the FacetType XML to Type XML.
                FacetTypeXMLDocument = new XmlDocument();
                FacetTypeXMLDocument.LoadXml(FacetTypeXML);

                FacetTypeNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPE");
                FacetTypeName = FacetTypeNode.SelectSingleNode("FACETTYPENAME").InnerText.Trim().ToUpper();
                generateTypeXML(FacetTypeNode, ref TypeXMLDocument);

                //Get the fields of FacetType Table...
                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, FacetTypeID.ToString(),
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "FACET",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null);
                if (ReadErrObj != null)
                    return ReadErrObj;
                
                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                                    ref FieldCollectionXML);
                
                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "Field Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                if (FieldCollectionXML != null)
                {
                    FieldCollectionXMLDocument = new XmlDocument();
                    FieldCollectionXMLDocument.LoadXml(FieldCollectionXML);
                    FieldCollectionNode = FieldCollectionXMLDocument.DocumentElement;

                    TypeXMLNode = TypeXMLDocument.DocumentElement;
                    TypeXMLNode.AppendChild(TypeXMLDocument.ImportNode(FieldCollectionNode, true));
                }

                //Obtaining NamedHierarchy Information associated with Type
                AssoNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                //The following line is commented to improve the performance. The following call gets the Named Hierarchy (including all values) _
                //if the field is associated with Named Hierarchy. In this case instead of returning the entire Named Hierarchy collection, 
                //the code is changed to return only the Named Hierarchy root ID for that field.
                //AssoNHObj.AssociateNamedHierarchyInformation(ref TypeXMLDocument, FacetTypeID);

                AssoNHObj.AssociateNamedHierarchyID(ref TypeXMLDocument, FacetTypeID);

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_FACETTYPE");
                    TableNameCollection.Add("ALOM_SCHEMA");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeXMLNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeXMLNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Read API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                //Get the XSD file and XSD file path for fields which are of type XML.
                ReadErrObj = GetXSDFileAndPathForXMLTypeFields(FacetTypeID.ToString(), FacetTypeName, "FACET", ref OutputXML);
                return (ReadErrObj);
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();
                Message.AppendFormat("{0}{1}", "MethodName: public OMError Read(ulong FacetTypeID, ref String OutputXML) FacetTypeID:", FacetTypeID.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }

        /// <summary>
        ///<CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read the FacetType based on FacetTypeID. This will return
        /// FacetType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="FacetTypeName">This contains the FacetTypeName.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the information about the FacetType. 
        /// The structure of this XML is give below:
        ///      TYPE
        ///        |- TYPEID        (*)
        ///        |- TYPENAME      (*)
        ///        |- DESCRIPTION   
        ///        |- OBJECTTYPE    (*)    
        ///        |- TABLENAME
        ///        |- PRIMARYKEY
        ///        |- STORAGETYPE
        ///        |- XMLCONTENTTYPE
        ///        |- XMLLOCATION
        ///        |- XMLNAME
        ///        |- XSDNAME
        ///        |- FIELDCOLLECTION
        ///             |- FIELD
        ///                 |- SCHEMAID     (*)
        ///                 |- TABLENAME
        ///                 |- OBJECTTYPEID (*)
        ///                 |- OBJECTTYPE   (*)
        ///                 |- FIELDNAME    (*)
        ///                 |- FIELDTYPE    (*)
        ///                 |- FIELDSIZE
        ///                 |- ISPRIMARYKEY
        ///                 |- ISMANDATORY
        ///                 |- ISSEARCHFIELD
        ///                 |- ISNAMEDHIERARCHY
        ///                 |- FIELDUITYPE
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If read was successful.
        /// Object of OMError if an error occurred while reading the FacetType.
        /// </returns>
        public OMError Read(String FacetTypeName, ref String OutputXML)
        {
            OMError ReadErrObj          = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj      = null;   //OMError object for returning from OM.
            String FacetTypeXML         = null;   //String variable storing the FacetType XML.
            String FieldCollectionXML   = null;   //String variable storing the FieldCollection XML.
            String TypeID               = null;   //String variable storing the TypeID.
            XmlNode FieldCollectionNode = null;   //DOM Node representing the FieldCollection node.
            XmlNode TypeXMLNode         = null;   //DOM Node representing the Type node.
            XmlNode FacetTypeNode       = null;   //DOM Node representing the FacetType node.
            XmlNode FacetTypeIDNode     = null;   //DOM Node representing the FacetTypeID node.

            StringBuilder Message                    = null;  //StringBuilder object storing the Message that is to be logged.
            StringCollection TableNameCollection     = null;  //StringCollection object to store the TableNames.
            OMCommon.OMCommon OMCommonObject         = null;  //OMCommon Object.
            XmlDocument TypeXMLDocument              = null;  //DOM Object representing the Type XML.
            XmlDocument FacetTypeXMLDocument         = null;  //DOM Object representing the FacetType XML.
            XmlDocument FieldCollectionXMLDocument   = null;  //DOM Object representing the FieldCollection XML.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManager is initialized or not.
            AssoFieldNamedHierarchy AssoNHObj = null;
            ulong FacetTypeID = 0;
            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Read API of FacetType class in OM. Input Parameter FacetTypeName: ", FacetTypeName);

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the FacetType Info...

                ReadErrObj = OMDataManager.DataProvider.AddClause("FACETTYPENAME", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, FacetTypeName.ToUpper(),
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref FacetTypeXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (FacetTypeXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the FacetType XML to Type XML.

                FacetTypeXMLDocument = new XmlDocument();
                FacetTypeXMLDocument.LoadXml(FacetTypeXML);

                FacetTypeNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPE");

                FacetTypeIDNode = FacetTypeNode.SelectSingleNode("FACETTYPEID");
                TypeID = FacetTypeIDNode.InnerText;

                generateTypeXML(FacetTypeNode, ref TypeXMLDocument);

                //Get the fields of FacetType Table...

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, TypeID,
                                                        OMConjunction.AND);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddClause("OBJECTTYPE", OMFieldDataType.STRING,
                                                        OMOperator.EQUALTO, "FACET",
                                                        OMConjunction.NOTHING);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.AddSortClause("DISPLAYORDER", "ALOM_SCHEMA", "ASC", null);
                if (ReadErrObj != null)
                    return ReadErrObj;

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FIELD", "ALOM_SCHEMA", null,
                                                                    ref FieldCollectionXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "Field Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                if (FieldCollectionXML != null)
                {
                    FieldCollectionXMLDocument = new XmlDocument();
                    FieldCollectionXMLDocument.LoadXml(FieldCollectionXML);
                    FieldCollectionNode = FieldCollectionXMLDocument.DocumentElement;

                    TypeXMLNode = TypeXMLDocument.DocumentElement;
                    TypeXMLNode.AppendChild(TypeXMLDocument.ImportNode(FieldCollectionNode, true));
                }
                //Obtaining NamedHierarchy Information associated with Type
                FacetTypeID = Convert.ToUInt64(TypeXMLDocument.SelectSingleNode("TYPE/TYPEID").InnerText);

                AssoNHObj = new AssoFieldNamedHierarchy(OMDataManager);
                //The following line is commented to improve the performance. The following call gets the Named Hierarchy (including all values) _
                //if the field is associated with Named Hierarchy. In this case instead of returning the entire Named Hierarchy collection, 
                //the code is changed to return only the Named Hierarchy root ID for that field.
                //AssoNHObj.AssociateNamedHierarchyInformation(ref TypeXMLDocument, FacetTypeID);

                AssoNHObj.AssociateNamedHierarchyID(ref TypeXMLDocument, FacetTypeID);

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_FACETTYPE");
                    TableNameCollection.Add("ALOM_SCHEMA");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeXMLNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeXMLNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Read API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                //Get the XSD file and XSD file path for fields which are of type XML.
                ReadErrObj = GetXSDFileAndPathForXMLTypeFields(TypeID, FacetTypeName, "FACET", ref OutputXML);
                return (ReadErrObj);
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();
                Message.AppendFormat("{0}{1}", "MethodName: public OMError Read(String FacetTypeName, ref String OutputXML) FacetTypeName:", FacetTypeName);

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }

        /// <summary>
        ///<CreatedDate>12 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read all the FacetTypes in the system. This will return a collection of 
        /// FacetType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains which contains the collection of
        /// FacetType information. 
        /// The structure of this XML is give below:
        ///     TYPECOLLECTION
        ///             |- TYPE
        ///                 |- TYPEID        (*)
        ///                 |- TYPENAME      (*)
        ///                 |- DESCRIPTION   
        ///                 |- OBJECTTYPE    (*)    
        ///                 |- TABLENAME
        ///                 |- PRIMARYKEY
        ///                 |- STORAGETYPE
        ///                 |- XMLCONTENTTYPE
        ///                 |- XMLLOCATION
        ///                 |- XMLNAME
        ///                 |- XSDNAME
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If read was successful.
        /// Object of OMError if an error occurred while reading the FacetTypes.
        /// </returns>
        public OMError ReadAll(ref String OutputXML)
        {
            OMError ReadErrObj                   = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj               = null;   //OMError object for returning from OM.
            String FacetTypeCollectionXML        = null;   //String variable storing the FacetTypeCollection XML.
            XmlDocument TypeXMLDocument          = null;   //DOM Object representing the Type XML.
            XmlDocument FacetTypeXMLDocument     = null;   //DOM Object representing the FacetType XML.
            XmlDocument OutputXMLDocument        = null;   //DOM Object representing the Output XML.
            XmlNode FacetTypeCollectionNode      = null;   //DOM Node representing the FacetTypeCollection node.
            XmlNodeList FacetTypeNodes           = null;   //DOM Node representing the list of FacetType nodes.
            XmlNode TypeCollectionNode           = null;   //DOM Node representing the TypeCollection node.
            XmlNode TypeNode                     = null;   //DOM Node representing the Type node.
            StringBuilder Message                = null;   //StringBuilder object storing the Message that is to be logged.
            OMCommon.OMCommon OMCommonObject     = null;   //OMCommon Object.
            StringCollection TableNameCollection = null;   //StringCollection object to store the TableNames.
            
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManager is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("Entered the ReadAll API of FacetType class in OM.");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the FacetType Info...

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref FacetTypeCollectionXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (FacetTypeCollectionXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                FacetTypeXMLDocument = new XmlDocument();
                FacetTypeXMLDocument.LoadXml(FacetTypeCollectionXML);

                FacetTypeCollectionNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPECOLLECTION");
                FacetTypeNodes = FacetTypeCollectionNode.SelectNodes("FACETTYPE");

                if (FacetTypeNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the FacetTypeCollection XML to TypeCollection XML.

                OutputXMLDocument = new XmlDocument();
                TypeCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, "TYPECOLLECTION", "");
                OutputXMLDocument.AppendChild(TypeCollectionNode);

                foreach (XmlNode FacetTypeNode in FacetTypeNodes)
                {
                    generateTypeXML(FacetTypeNode, ref TypeXMLDocument);

                    TypeNode = TypeXMLDocument.DocumentElement;
                    TypeCollectionNode.AppendChild(OutputXMLDocument.ImportNode(TypeNode, true));

                }

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_FACETTYPE");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeCollectionNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeCollectionNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the ReadAll API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();
                Message.AppendFormat("MethodName: public OMError ReadAll(ref String OutputXML)");

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }


        /// <summary>
        /// <CreatedDate>25 May 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will read FacetTypes that are requested by the consumer. This will return a collection of 
        /// FacetType information as XML. The structure of XML is given below.
        /// </Purpose>
        /// </summary>
        /// <param name="ObjectTypeIDArrayList">ArrayList containing the ObjectTypeIDs.</param>
        /// <param name="OutputXML">
        /// This is a reference parameter and will hold the XML which contains the collection of 
        /// FacetType information. 
        /// The structure of this XML is give below:
        ///      TYPECOLLECTION
        ///             |- TYPE
        ///                 |- TYPEID        (*)
        ///                 |- TYPENAME      (*)
        ///                 |- DESCRIPTION   
        ///                 |- OBJECTTYPE    (*)    
        ///                 |- TABLENAME
        ///                 |- PRIMARYKEY
        ///                 |- STORAGETYPE
        ///                 |- XMLCONTENTTYPE
        ///                 |- XMLLOCATION
        ///                 |- XMLNAME
        ///                 |- XSDNAME
        ///        (*) Mandatory Element
        /// </param>
        /// <returns>
        /// NULL: If the read was successful.
        /// Object of OMError if an error occurred while reading the FacetTypes.
        /// </returns>
        public OMError ReadAll(ArrayList ObjectTypeIDArrayList, ref String OutputXML)
        {
            OMError ReadErrObj                   = null;   //OMError object for reading data from DB.
            OMError ReturnErrorObj               = null;   //OMError object for returning from OM.
            XmlDocument FacetTypeXMLDocument     = null;   //DOM Object representing the FacetType XML.
            XmlDocument TypeXMLDocument          = null;   //DOM Object representing the Type XML.
            XmlDocument OutputXMLDocument        = null;   //DOM Object representing the Output XML.
            XmlNodeList FacetTypeNodes           = null;   //DOM Node representing the list of FacetType nodes.
            XmlNode FacetTypeCollectionNode      = null;   //DOM Node representing the list of FacetTypeCollection node.
            XmlNode TypeCollectionNode           = null;   //DOM Node representing the TypeCollection node.
            XmlNode TypeNode                     = null;   //DOM Node representing the Type node.
            String FacetTypeCollectionXML        = null;   //String variable storing the FacetTypeCollection XML.
            StringBuilder ObjecTypeIDList        = null;   //StringBuilder variable storing the comma separated FacetTypeIDs.
            String Seprator                      = null;   //String storing the Separator.
            StringBuilder Message                = null;   //StringBuilder object storing the Message that is to be logged.
            StringCollection TableNameCollection = null;   //StringCollection object to store the TableNames.
            OMCommon.OMCommon OMCommonObject     = null;   //OMCommon Object.

            Boolean IsOMDataManagerInitialized = false;   //Boolean variable storing whether OMDataManager is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the ReadAll API of FacetType class in OM. Input Parameter ObjectTypeIDArrayList: ", ObjectTypeIDArrayList.ToString());

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

                //Get the FacetType Info...
                Seprator = "";
                ObjecTypeIDList = new StringBuilder();
                ObjecTypeIDList.AppendFormat("(");
                for (int ArrayListCount = 0; ArrayListCount < ObjectTypeIDArrayList.Count; ArrayListCount++)
                {
                    ObjecTypeIDList.AppendFormat("{0}{1}", Seprator, ObjectTypeIDArrayList[ArrayListCount]);
                    Seprator = ",";
                }
                ObjecTypeIDList.AppendFormat(")");

                ReadErrObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                                                            OMOperator.IN, ObjecTypeIDList.ToString(),
                                                            OMConjunction.NOTHING);
                if (ReadErrObj != null)
                {
                    return ReadErrObj;
                }

                ReadErrObj = OMDataManager.DataProvider.ReadAllRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                    ref FacetTypeCollectionXML);

                if (ReadErrObj != null)    // If DAL returns an error object
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);

                    return ReturnErrorObj;
                }

                if (FacetTypeCollectionXML == null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Prepare the OutputXML...

                FacetTypeXMLDocument = new XmlDocument();
                FacetTypeXMLDocument.LoadXml(FacetTypeCollectionXML);

                FacetTypeCollectionNode = FacetTypeXMLDocument.SelectSingleNode("FACETTYPECOLLECTION");
                FacetTypeNodes = FacetTypeCollectionNode.SelectNodes("FACETTYPE");

                if (FacetTypeNodes.Count < 1)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                        "FacetType Not Found", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.ERROR, null, null);

                    return ReturnErrorObj;
                }

                //Convert the FacetTypeCollection XML to TypeCollection XML.

                OutputXMLDocument = new XmlDocument();
                TypeCollectionNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, "TYPECOLLECTION", "");
                OutputXMLDocument.AppendChild(TypeCollectionNode);

                foreach (XmlNode FacetTypeNode in FacetTypeNodes)
                {
                    generateTypeXML(FacetTypeNode, ref TypeXMLDocument);

                    TypeNode = TypeXMLDocument.DocumentElement;
                    TypeCollectionNode.AppendChild(OutputXMLDocument.ImportNode(TypeNode, true));
                }

                if ((IsOMDataManagerInitialized == true) && (InternalCall == false))
                {
                    OMCommonObject = new OMCommon.OMCommon(OMDataManager);
                    OMCommonObject.InternalCall = true;

                    TableNameCollection = new StringCollection();
                    TableNameCollection.Add("ALOM_FACETTYPE");

                    OMCommonObject.GenerateOutputXML(TableNameCollection, "", TypeCollectionNode.OuterXml.ToString(),
                                                     ref OutputXML);
                }
                else
                    OutputXML = TypeCollectionNode.OuterXml.ToString();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the ReadAll API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();
                Message.AppendFormat("{0}{1}", "MethodName: public OMError ReadAll(ArrayList ObjectTypeIDArrayList, ref String OutputXML) ObjectTypeIDArrayList: ", ObjectTypeIDArrayList.ToString());

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED,
                        "FacetType Read Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                ExceptionTracing.LogError(TenantName, ReturnErrorObj);

                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();
            }
        }
                

        /// <summary>
        /// <CreatedDate>01 June 2005</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// The method will update the FacetType based on details in the InputXML.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">String variable which contains all the details of an FacetType 
        /// that are to be updated.</param>
        /// <returns>
        /// NULL: If update was successful.
        /// Object of OMError if an error occurred while updating the FacetType.
        /// </returns>
        public OMError Update(String InputXML)
        {
            XmlDocument InputXMLDocument = null;     //DOM Object representing the InputXML.
            XmlNode TypeNode             = null;     //DOM Node representing the Type node of the InputXML.
            XmlNode TypeNode_Clone       = null;     //DOM Node representing the Clone TypeNode node.
            XmlNode FieldCollectionNode  = null;     //DOM Node representing the FieldCollection node.
            String TypeID                = null;     //String variable storing the value of TypeID.
            String TableName             = null;     //String variable storing the value of TableName.
            XmlNode TableNameNode        = null;     //DOM Node representing the TableName node.
            OMError UpdateErrorObj       = null;     //OMError object for create record in the database.
            OMError ReadErrObj           = null;     //OMError object for read record in the database.
            OMError ReturnErrorObj       = null;     //OMError object for returning from OM.
            StringBuilder Message        = null;     //StringBuilder object storing the Message that is to be logged.
            //Boolean WriteFailed          = false;    //Boolean variable storing whether the Write operation to the database failed or succeded.
            Boolean IsOMDataManagerInitialized = false;     //Boolean variable storing whether OMDataManagerWriter is initialized or not.

            try
            {
                if (OMDataManager.TraceEnabled == true)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("{0}{1}", "Entered the Update API of FacetType class in OM. Input Parameter InputXML: ", InputXML);

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                // If Inherited base class member OMDataManagerWriter and OMDataManager is not initialized then initalize it.
                if (OMDataManager == null)
                {
                    OMDataManager = new DataManager(TenantName);
                    IsOMDataManagerInitialized = true;
                }

             

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                TypeNode = InputXMLDocument.SelectSingleNode("TYPE");
                TypeNode_Clone = TypeNode.Clone();
                
                TypeID = TypeNode.SelectSingleNode("TYPEID").InnerText;

                ReadErrObj = getTableNameFromDB(TypeID, ref TableName);

                if (ReadErrObj != null)
                {
                    //WriteFailed = true;
                    return ReadErrObj;
                }

                TableNameNode = TypeNode.SelectSingleNode("TABLENAME");

                if (TableNameNode == null)
                {
                    TableNameNode = InputXMLDocument.CreateNode(XmlNodeType.Element, "TABLENAME", "");
                    TableNameNode.InnerText = TableName;
                    TypeNode_Clone.AppendChild(TableNameNode);
                }

                if (TypeNode.SelectNodes("FIELDCOLLECTION").Count > 0)
                {
                    FieldCollectionNode = TypeNode.SelectSingleNode("FIELDCOLLECTION").Clone();
                    TypeNode.RemoveChild(TypeNode.SelectSingleNode("FIELDCOLLECTION"));
                }

                //Update record in ALOM_FACETTYPE Table.
                UpdateErrorObj = updateFacetType(TypeNode, TypeID, TableName);

                if (UpdateErrorObj != null)
                {
                    //WriteFailed = true;
                    return UpdateErrorObj;
                }

                if (FieldCollectionNode != null)
                {
                    //Update record in ALOM_SCHEMA Table.
                    UpdateErrorObj = updateSchema(InputXMLDocument, TableName, FieldCollectionNode);

                    if (UpdateErrorObj != null)
                    {
                        //WriteFailed = true;
                        return UpdateErrorObj;
                    }
                }

                UpdateErrorObj = OMDataManager.DataProvider.UpdateTable(TypeNode_Clone.OuterXml);

                if (UpdateErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                        "Table Modification Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", UpdateErrorObj.ErrorType, UpdateErrorObj.ExceptionObject,
                        UpdateErrorObj.ErrorXML);

                    //WriteFailed = true;
                    return ReturnErrorObj;
                }

                if (FieldCollectionNode != null)
                    updateXSD(Convert.ToUInt64(TypeID));

                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Commit();

                if (OMDataManager.TraceEnabled == true)
                {
                    Message.Remove(0, Message.Length);
                    Message.AppendFormat("Leaving the Update API of FacetType class in OM");

                    ExceptionTracing.TraceLog(Message.ToString(), "FACETTYPE", TenantName,DateTime.Now);
                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                if (Message != null)
                    Message.Remove(0, Message.Length);
                else
                    Message = new StringBuilder();
                Message.AppendFormat("{0}{1}", "MethodName: public OMError Update(String InputXML) InputXML:", InputXML);

                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED,
                        "FacetType Modification Failed", OMErrorObjectType.FACETTYPE,
                        "FacetType", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());

                //WriteFailed = true;
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
            finally
            {
                if (IsOMDataManagerInitialized)
                    OMDataManager.DataProvider.Close();

            }
        }

        #endregion Methods
    }
}
