﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using Assetlink.ObjectManager;
using Assetlink.DataAccessLayer;
using System.Xml;
using System.Xml.XPath;
using System.IO;


namespace Assetlink.ObjectManager.OMCommon
{
    ///<summary>
    /// <CreatedDate>08 June 2007</CreatedDate>
    /// <Author>Miss. Tina C. Jawale</Author>
    /// <purpose>This stucture contains Default value of field. 
    ///     In Create we have to append the nodes which have default values and also the default values 
    ///     to empty nodes, if any. This is used in GetObjectTypeSchemaDetails method
    /// </purpose>
    ///</summary>
    public struct FieldDefaultValue
    {
        public string FieldValue;
        public string FieldType;
    }

    internal class OMCommon : ObjectManagerBase
	{
        public OMCommon(DataManager DataManagerObj)
            : base(DataManagerObj)
        {

        }

        public OMCommon(DataManagerReader OMDataReader, DataManagerWriter OMDataWriter)
            : base(OMDataReader, OMDataWriter)
		{

		}

		public OMError GetDescriptor(string TableName, StringCollection FieldListCollection, ref string OutputXML)
		{
			StringBuilder FieldList = new StringBuilder();
			StringBuilder Query = new StringBuilder();
			string Seperator="";
			string FieldName = "";
			int index = 0;
			OutputXML = "";
			//string OutputXML="";
			XmlElement DescriptorCollectionNode=null;
			XmlElement DescriptorNode = null;
			XmlDocument OutputXMLDocument,XMLDocument;
			OutputXMLDocument = new XmlDocument();
			OMError ErrorObj=null;
			XMLDocument = new XmlDocument();

			DescriptorCollectionNode = OutputXMLDocument.CreateElement("DESCRIPTORCOLLECTION");
			DescriptorNode = OutputXMLDocument.CreateElement("DESCRIPTOR");
			DescriptorNode.SetAttribute("TableName", TableName.ToUpper());
			DescriptorCollectionNode.AppendChild(DescriptorNode);
			Query.AppendFormat("SELECT * FROM ALOM_SCHEMA WHERE TABLENAME='{0}' ", TableName.ToUpper());
			if (FieldListCollection != null)
			{
				for (int count = 0; count < FieldListCollection.Count; count++)
				{
					FieldList.AppendFormat("{0} '{1}'", Seperator, FieldListCollection[count]);
					//Query.AppendFormat("SELECT {0} FROM ALOM_SCHEMA WHERE TABLENAME='{1}' ", FieldList, TableName);
					Seperator = ",";
				}

				StringBuilder QuerySB = new StringBuilder();
				QuerySB.AppendFormat("SELECT FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableName);
                ErrorObj = OMDataManager.DataProvider.RetrieveResult(QuerySB.ToString());
				if (ErrorObj != null)
				{
					return ErrorObj;
				}
                while (OMDataManager.DataProvider.DataReaderObj.Read())
				{
                    FieldName = OMDataManager.DataProvider.DataReaderObj.GetValue(0).ToString();
					index = FieldList.ToString().IndexOf(FieldName.ToUpper());
					// Determine if a valid index is returned. 
					if (index == -1)
					{
						FieldList.AppendFormat("{0} '{1}'", Seperator, FieldName.ToUpper());
						Seperator = ",";
					}
				}
                OMDataManager.DataProvider.DataReaderObj.Close();
				Query.AppendFormat(" AND FIELDNAME IN ({0})", FieldList);
			}
            OMDataManager.DataProvider.ExecuteQuery(Query.ToString(), "FIELD", ref OutputXML);
			if (OutputXML != "")
			{
				XMLDocument.LoadXml(OutputXML);
				DescriptorNode.AppendChild(OutputXMLDocument.ImportNode(XMLDocument.DocumentElement, true));
				OutputXML = DescriptorCollectionNode.OuterXml;
			}
			else
				OutputXML = "";
			
			return null;
		}

		public OMError GenerateOutputXML(StringCollection TableNameCollection, string FieldCollection, string DataXML, ref string OutputXML)
		{
			XmlDocument			OutputXMLDocument=null;
			XmlDocument			DataDocument = null;
			StringBuilder		FieldList=null;
			StringBuilder		Query=null;
			string				Seperator = "";
			XmlElement			DescriptorCollectionNode = null;
			XmlElement			DataNode;
			XmlDocument			FieldCollectionDocument = null, XMLDocument=null;
			XmlElement			DescriptorNode = null;
			XmlNodeList			FieldNodes = null;		
			XmlElement			OutputNode = null;
			OMError				ErrorObj = null;
			string				FieldName = "";
			int					index = 0;

			XMLDocument				= new XmlDocument();
			DataDocument			= new XmlDocument();
			OutputXMLDocument		= new XmlDocument();
			FieldCollectionDocument = new XmlDocument();

			OutputNode = OutputXMLDocument.CreateElement("OUTPUTXML");
			OutputXMLDocument.AppendChild(OutputNode);

			DataDocument.LoadXml(DataXML);
			DataNode = OutputXMLDocument.CreateElement("DATA");
			DataNode.AppendChild(OutputXMLDocument.ImportNode(DataDocument.DocumentElement, true));
			OutputNode.AppendChild(DataNode);
			DescriptorCollectionNode = OutputXMLDocument.CreateElement("DESCRIPTORCOLLECTION");
			OutputNode.AppendChild(DescriptorCollectionNode);
			for (int count = 0; count < TableNameCollection.Count; count++)
			{
				FieldList		= new StringBuilder();
				Query			= new StringBuilder();
				Seperator		= "";
				DescriptorNode	= OutputXMLDocument.CreateElement("DESCRIPTOR");
				DescriptorNode.SetAttribute("TableName", TableNameCollection[count].ToUpper());
				DescriptorCollectionNode.AppendChild(DescriptorNode);
				
				Query.AppendFormat("SELECT * FROM ALOM_SCHEMA WHERE TABLENAME='{0}' ", TableNameCollection[count].ToUpper());
				if ((FieldCollection != "") && (FieldCollection != null))
				{
					FieldCollectionDocument.LoadXml(FieldCollection);
					FieldNodes = FieldCollectionDocument.SelectNodes("FIELDCOLLECTION/FIELD[TABLENAME='" + TableNameCollection[count] + "']");
					for (int NodeCount = 0; NodeCount < FieldNodes.Count; NodeCount++)
					{
						if (FieldNodes[NodeCount].SelectSingleNode("FIELDNAME").HasChildNodes)
						{
							FieldList.AppendFormat("{0} '{1}'", Seperator, FieldNodes[NodeCount].SelectSingleNode("FIELDNAME").InnerXml);
							Seperator = ",";
						}
					}

					StringBuilder QuerySB = new StringBuilder();
					QuerySB.AppendFormat("SELECT FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableNameCollection[count].ToUpper());
                    ErrorObj = OMDataManager.DataProvider.RetrieveResult(QuerySB.ToString());
					if (ErrorObj != null)
					{
						return ErrorObj;
					}
                    while (OMDataManager.DataProvider.DataReaderObj.Read())
					{
                        FieldName = OMDataManager.DataProvider.DataReaderObj.GetValue(0).ToString();
						index = FieldList.ToString().IndexOf(FieldName.ToUpper());
						// Determine if a valid index is returned. 
						if (index == -1)
						{
							FieldList.AppendFormat("{0} '{1}'", Seperator, FieldName.ToUpper());
							Seperator = ",";
						}
					}
                    OMDataManager.DataProvider.DataReaderObj.Close();
					Query.AppendFormat(" AND FIELDNAME IN ({0})", FieldList);
				}
                OMDataManager.DataProvider.ExecuteQuery(Query.ToString(), "FIELD", ref OutputXML);
				if (OutputXML != "")
				{
					XMLDocument.LoadXml(OutputXML);
					DescriptorNode.AppendChild(OutputXMLDocument.ImportNode(XMLDocument.DocumentElement, true));
				}
				OutputXML = OutputXMLDocument.OuterXml;
				FieldList = null;
				Query = null;
			}
			return null;
		}

        //IsDuplicate() method checks whether the passed field value is already present in the database or not.
        public Boolean IsDuplicate(String TableName, String PrimaryKeyFieldName, String PrimaryKeyFieldValue,
            OMFieldDataType PrimaryKeyFieldDataType, String FieldName, String FieldValue, OMFieldDataType FieldDataType)
        {
            OMError     ReadErrorObj        = null;
            XmlDocument OutputXMLDocument   = null;
            XmlNode     FieldNode           = null;
            Boolean     IsOMDMInitialized   = true;
            String      OutputXML           = null;

            try
            {
                if (OMDataManager == null)
                    OMDataManager = new DataManager(TenantName);

                ReadErrorObj = OMDataManager.DataProvider.AddClause(PrimaryKeyFieldName, PrimaryKeyFieldDataType,
                                            OMOperator.NOTEQUALTO, PrimaryKeyFieldValue, OMConjunction.AND);
                if (ReadErrorObj != null)
                    return (false);

                ReadErrorObj = OMDataManager.DataProvider.AddClause(FieldName, FieldDataType,
                                            OMOperator.EQUALTO, FieldValue, OMConjunction.NOTHING);
                if (ReadErrorObj != null)
                    return (false);

                ReadErrorObj = OMDataManager.DataProvider.ReadRecord("FIELD", TableName, null, ref OutputXML);
                if (ReadErrorObj != null || OutputXML == null || OutputXML == "")
                    return (false);
                else
                {
                    OutputXMLDocument = new XmlDocument();
                    OutputXMLDocument.LoadXml(OutputXML);
                    FieldNode = OutputXMLDocument.DocumentElement;
                    if ((FieldNode != null) && (FieldNode.SelectSingleNode(FieldName.ToUpper()).InnerText.ToString().Trim() != ""))
                        return (true);
                }
            }
            catch
            {
                if (IsOMDMInitialized)
                {
                    OMDataManager.DataProvider.Close();
                    OMDataManager = null;
                }
            }
            return (false);
        }


        /// <summary>
        /// <CreatedDate>13 June 2007</CreatedDate>
        /// <Author>Miss. Tina C. Jawale</Author>
        /// <Purpose> This method is invoked while creating an Entity and Facet. It gets the default values
        ///     of the fields of EntityType/FacetType from ALOM_SCHEMA and appends nodes to InputXML accordingly.
        /// </Purpose>
        /// </summary>
        /// <param name="TableName">Name of the EntityType/FacetType for which the instance is created</param>
        /// <param name="InputXMLDocument">InputXML of instance of EntityType/FacetType</param>
        /// <returns>Null if Successful
        ///      otherwise OMError object
        /// </returns>
        public OMError AppendDefaultValueNodesToInputXML(String TableName, ref XmlDocument InputXMLDocument)
        {
            #region Local variables

            Hashtable           FieldNameValueCollection    = null;
            OMError             SchemaDetailsErrorObj       = null;
            String              FieldValue                  = null;
            String              FieldType                   = null;
            int                 FieldCount                  = 0;
            XmlElement          RootNode                    = null;
            XmlNode             FieldNode                   = null;
            XmlNode             DayNode                     = null;
            XmlNode             MonthNode                   = null;
            XmlNode             YearNode                    = null;
            XmlNode             HourNode                    = null;
            XmlNode             MinuteNode                  = null;
            XmlNode             SecondNode                  = null;
            Boolean             FieldFound                  = false;
            String[]            HashKeys_FieldNames         = { null };
            DateTime            DateValue                   = new DateTime();
            FieldDefaultValue   FieldDefaultValueObj        = new FieldDefaultValue();


            #endregion

            SchemaDetailsErrorObj = GetFieldDetails(TableName, out FieldNameValueCollection);

            if (SchemaDetailsErrorObj != null)
            {
                return (SchemaDetailsErrorObj);
            }

            RootNode = InputXMLDocument.DocumentElement;

            /* copy keys, here FieldNames, to string array */
            HashKeys_FieldNames = new String[FieldNameValueCollection.Keys.Count];
            FieldNameValueCollection.Keys.CopyTo((Array)HashKeys_FieldNames, 0);

            for (FieldCount = 0; FieldCount < HashKeys_FieldNames.GetLength(0); FieldCount++)
            {
                /* get the defualt value of the field from hashtable */
                FieldDefaultValueObj = (FieldDefaultValue)FieldNameValueCollection[HashKeys_FieldNames[FieldCount]];
                FieldValue = FieldDefaultValueObj.FieldValue;
                FieldType = FieldDefaultValueObj.FieldType;

                if ((FieldType.ToUpper()).Equals("DATETIME"))
                {
                    if (!String.IsNullOrEmpty(FieldValue))
                    {
                        DateValue = new DateTime();
                        DateValue = Convert.ToDateTime(FieldValue);

                        DayNode = InputXMLDocument.CreateElement("DAY");
                        MonthNode = InputXMLDocument.CreateElement("MONTH");
                        YearNode = InputXMLDocument.CreateElement("YEAR");
                        HourNode = InputXMLDocument.CreateElement("HOUR");
                        MinuteNode = InputXMLDocument.CreateElement("MINUTE");
                        SecondNode = InputXMLDocument.CreateElement("SECOND");

                        DayNode.InnerText = DateValue.Day.ToString();
                        MonthNode.InnerText = DateValue.Month.ToString();
                        YearNode.InnerText = DateValue.Year.ToString();
                        HourNode.InnerText = DateValue.Hour.ToString();
                        MinuteNode.InnerText = DateValue.Minute.ToString();
                        SecondNode.InnerText = DateValue.Second.ToString();
                    }
                }

                /* check whether this field exists in inputxml, if yes and value is not specified, append value 
                 * otherwise create new node and append the node to inputxml 
                 */

                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    FieldFound = false;
                    if (HashKeys_FieldNames[FieldCount].Equals(ChildNode.Name))
                    {
                        FieldFound = true;
                        if ((ChildNode.InnerText.Equals("")) || (ChildNode.InnerText == null))
                        {
                            if ((FieldType.ToUpper()).Equals("DATETIME"))
                            {
                                if (!String.IsNullOrEmpty(FieldValue))
                                {
                                    ChildNode.AppendChild(DayNode);
                                    ChildNode.AppendChild(MonthNode);
                                    ChildNode.AppendChild(YearNode);
                                    ChildNode.AppendChild(HourNode);
                                    ChildNode.AppendChild(MinuteNode);
                                    ChildNode.AppendChild(SecondNode);
                                }
                            }
                            else
                                if ((FieldType.ToUpper() != "XML") && (FieldType.ToUpper() != "BINDER")
                                    && (FieldType.ToUpper() != "FILE") && (FieldType.ToUpper() != "CURRENCY"))
                                {
                                    ChildNode.InnerText = FieldValue;
                                }
                        }
                    }
                    if (FieldFound == true)
                        break;
                }

                if (FieldFound == false && (FieldValue != "" || FieldValue != null))
                {
                    /* Field is not in the Inputxml */
                    /* create new node and append to inputxml */
                    FieldNode = InputXMLDocument.CreateElement(HashKeys_FieldNames[FieldCount]);

                    if ((FieldType.ToUpper()).Equals("DATETIME"))
                    {
                        if (!String.IsNullOrEmpty(FieldValue))
                        {
                            FieldNode.AppendChild(DayNode);
                            FieldNode.AppendChild(MonthNode);
                            FieldNode.AppendChild(YearNode);
                            FieldNode.AppendChild(HourNode);
                            FieldNode.AppendChild(MinuteNode);
                            FieldNode.AppendChild(SecondNode);
                        }
                    }
                    else
                        if ((FieldType.ToUpper() != "XML") && (FieldType.ToUpper() != "BINDER")
                             && (FieldType.ToUpper() != "FILE") && (FieldType.ToUpper() != "CURRENCY"))
                        {
                            FieldNode.InnerText = FieldValue;
                        }

                    RootNode.AppendChild(FieldNode);
                }
            }

            return null;
        }

        /// <summary>
        /// <CreatedDate>13 June 2007</CreatedDate>
        /// <Author>Miss. Tina C. Jawale</Author>
        /// <Purpose> This method is invoked by AppendDefaultValueNodesToInputXML while creating Entity/Facet.
        ///     This method returns the field details like FieldName, FieldType and DefaultValue.
        /// </Purpose>
        /// </summary>
        /// <param name="TableName">Name of EntityType/FacetType</param>
        /// <param name="FieldNameValueCollection">HashTable instance containing FieldName, FieldType and FieldValue</param>
        /// <returns>Null is Successfull
        ///     Otherwise OMError object
        /// </returns>
        /// 
        private OMError GetFieldDetails(String TableName, out  Hashtable FieldNameValueCollection)
        {

            #region Local Variables

            OMError             ErrorObj            = null;
            String              FieldName           = null;
            String              FieldValue          = null;
            String              FieldType           = null;
            String              FieldListXML        = null;
            String              OutputXML           = null;
            String              previousm_Clause    = null;
            XPathDocument       Xp_FieldXMLDocument = null;
            XPathNavigator      XpNav_FieldNav      = null;
            XPathNodeIterator   XpNodeItr           = null;
            FieldDefaultValue   FieldValueStructObj;

            #endregion

            //Initialize 'out' paramter.
            FieldNameValueCollection = null;

            try
            {
                //Prepare the FieldListXML for ReadAllRecord.
                FieldListXML = @" <FIELDCOLLECTION>
                                        <FIELD>
                                            <FIELDNAME>FIELDNAME</FIELDNAME>
                                        </FIELD>
                                        <FIELD>
                                            <FIELDNAME>FIELDTYPE</FIELDNAME>
                                        </FIELD>
                                        <FIELD>
                                            <FIELDNAME>DEFAULTVALUE</FIELDNAME>
                                        </FIELD>
                                    </FIELDCOLLECTION>";
                // m_Clause possibly already has value set by caller. This value may not have been used yet.
                // To avoid problems arising from this, the value is being preserved by storing it before calling AddClause
                // and restoring it after calling ReadRecord/ReadAllRecord.
                previousm_Clause = OMDataManager.DataProvider.m_Clause;
                OMDataManager.DataProvider.m_Clause = null;

                // Adding clauses related to the tables ALOM_SCHEMA
                if ((ErrorObj = OMDataManager.DataProvider.AddClause("TABLENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, TableName, OMConjunction.AND)) == null)
                {
                    if ((ErrorObj = OMDataManager.DataProvider.AddClause("DEFAULTVALUE", OMFieldDataType.STRING, OMOperator.NOTEQUALTO, "null", OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = OMDataManager.DataProvider.ReadAllRecord(TableName, "ALOM_SCHEMA", FieldListXML, ref OutputXML)) != null)
                        {
                            return ErrorObj;
                        }
                    }
                    else
                        return ErrorObj;
                }
                else
                    return ErrorObj;

                OMDataManager.DataProvider.m_Clause = previousm_Clause;    // restore value of m_Clause

                FieldNameValueCollection    = new Hashtable();
                FieldValueStructObj         = new FieldDefaultValue();
                if (OutputXML != null)
                {
                    Xp_FieldXMLDocument = new XPathDocument(new StringReader(OutputXML));
                    XpNav_FieldNav = Xp_FieldXMLDocument.CreateNavigator();
                    XpNodeItr = XpNav_FieldNav.Select(TableName + "COLLECTION" + "/" + TableName);

                    //Here retrieves values of the fiels from the output XML                    
                    while (XpNodeItr.MoveNext())
                    {
                        FieldName = XpNodeItr.Current.SelectSingleNode("FIELDNAME").Value;
                        FieldType = XpNodeItr.Current.SelectSingleNode("FIELDTYPE").Value;
                        FieldValue = XpNodeItr.Current.SelectSingleNode("DEFAULTVALUE").Value;

                        FieldValueStructObj.FieldType = FieldType;
                        FieldValueStructObj.FieldValue = FieldValue;

                        FieldNameValueCollection.Add(FieldName.Trim(), FieldValueStructObj);
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ErrorObj);
        }
    }
}
