using System;
using System.Text;
using System.Xml;
using System.Collections;
using System.Reflection;
using System.Collections.Specialized;
using System.Transactions;
using System.Xml.XPath;
using System.IO;
using System.Data;

using Assetlink;
using Assetlink.ObjectManager;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.EntityFacetManagement;
using Assetlink.ObjectManager.ETFTManagement;
using Assetlink.ObjectManager.AccessControl;
using Assetlink.ObjectManager.Search;
using Assetlink.FileAccessLayer;

namespace Assetlink
{
    public partial class ObjectManagerClass:ObjectManagerBase
    {
        private FileTransferDelegate.OnFileTransfer m_delFileTransfer = null;
        private int RetryCount = 3;

        private int RetrySleepDuration
        {
            get
            {
                return (OMDataManager != null) ? OMDataManager.RetrySleepDuration : 1000;
            }
        }

        public FileTransferDelegate.OnFileTransfer FileTransfer
        {
            set
            {
                m_delFileTransfer = value;
            }
        }
        #region Members Declaration

        #region Constructors
        public ObjectManagerClass()
        {
        }
        public ObjectManagerClass(String TenantName, bool TraceEnabled)
            : base(TenantName)
        {
        }

        public ObjectManagerClass(ulong UserID,String CurrentLanguage,String TenantName, bool TraceEnabled)
            : base(UserID, CurrentLanguage,TenantName)
        {
        }

        public ObjectManagerClass(ulong UserID, string CurrentLanguage,
                            string TenantName, DataManager DataManagerObj)
            : base(UserID, CurrentLanguage, TenantName, DataManagerObj)
        {
            
        }
       
        #endregion

        #endregion

        #region Execute
        /// <summary>
        /// 
        /// <CreatedDate>23 August 2005</CreatedDate>
        /// <Author></Author>
        /// <Purpose>
        ///  This allows executions of multiple operations. 
        ///  It allows consumers to mark whether the operations are within a 
        ///  transaction or not. If they are marked for within a transaction 
        ///  then all operation are executed within a same transaction 
        ///  (database transaction)
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">
        /// 
        /// </param>
        /// <returns>Return Null if successful or return the Error XML.</returns>
        public String Execute(ref String InputXML)
        {
            #region Local Variables

            OMError         ErrorObj = null;
            String          ErrorXML = null;
            StringBuilder   ErrorCollectionXML = null;
            XmlDocument     InputXMLDocument = null;
            XmlNodeList     OperationNodes = null;
            XmlNode         TracingNode = null;
            XmlNode         TransactionNode = null;
            bool            bTracing = false;
            bool            bTransaction = true;
            String          OutputXML = null;
            XmlNodeList     FileSystemNodes = null;
            FileSystemManager FileSystemManagerObj = null;
            XmlNode         FileSystemNodeByRef = null;
            XPathDocument   xpDoc_ErrorXml = null;
            XPathNavigator  xpNav_ErrorXmlRoot = null;
            bool            b_FileSystemOperationCompleted = false;    //Specifies whether File System Operations are completed successfully
            String          str_ErrorType = null;
            String          OrginalInputXml = null;
            int             IterationCount = 0;
            Boolean         DoExecute = true;
            String          IsSchemaUpdateForODS = "FALSE";
            bool            ExecuteOperationOnMomDB = true;
            String          str_ExecuteOperationOnMOMDB = null;
            ulong           TransactionId = 0;
            Boolean         bDBOperationsCompleted = false;

            #endregion            

            OrginalInputXml = InputXML;
            while (DoExecute)
            {
                try
                {
                    InputXML = OrginalInputXml;
                    ErrorXML = null;
                    ErrorCollectionXML = new StringBuilder();

                    // Load the InputXML
                    InputXMLDocument = new XmlDocument();
                    InputXMLDocument.LoadXml(InputXML);

                    #region Parse and validate input XML
		            // This will be false in case of adding ODS support for existing relations. By default the value will be true.i.e. all the operations
                    // will be performed on both the DB's MOMDB and ODSDB
                    if (InputXMLDocument.SelectSingleNode("INPUTXML") != null)
                        str_ExecuteOperationOnMOMDB = ((XmlElement)InputXMLDocument.SelectSingleNode("INPUTXML")).GetAttribute("ExecuteOperationOnMomDB", "");
                    if (String.Compare(str_ExecuteOperationOnMOMDB, "FALSE", true) == 0)
                        ExecuteOperationOnMomDB = false;
                    else
                        ExecuteOperationOnMomDB = true;

                    //Read whether operations are within a transaction or not
                    TracingNode = InputXMLDocument.SelectSingleNode("InputXML/Tracing");

                    if (TracingNode == null) //Case Sensitive
                        TracingNode = InputXMLDocument.SelectSingleNode("INPUTXML/TRACING");

                    if (TracingNode != null)
                        bTracing = Convert.ToBoolean(TracingNode.InnerText);
                    else
                        bTracing = false;

                    //Read the whether operations are within a transaction or not
                    TransactionNode = InputXMLDocument.SelectSingleNode("InputXML/Transaction");

                    if (TransactionNode == null) //Case Sensitive
                        TransactionNode = InputXMLDocument.SelectSingleNode("INPUTXML/TRANSACTION");

                    if (TransactionNode != null)
                        bTransaction = Convert.ToBoolean(TransactionNode.InnerText);
                    else
                        bTransaction = true;

                    // Validation: If transaction is set to true at OperationQueue then it cannot be set to false at Operation level.
                    if (bTransaction)
                    {
                        if ((InputXMLDocument.SelectSingleNode("InputXML/Operation[Transaction[translate(text(), 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='FALSE']]") != null)
                            || (InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION[TRANSACTION[translate(text(), 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='FALSE']]") != null))
                        {
                            ErrorObj = new OMError(OMErrorCodes.OMCLASS_EXECUTE_FAILED, "Cannot set Transaction = False at Operation level when Transaction is set to True at OperationQueue level",
                                            OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                            OMErrorType.ERROR, null, null);
                            ErrorXML = (ErrorCollectionXML.Append("<ERRORCOLLECTION> " + ErrorObj.Serialize() + "</ERRORCOLLECTION>")).ToString();
                            return ErrorXML;
                        }
                    } 
            	    #endregion

                    //Read all the operations nodes from the InputXML
                    OperationNodes = InputXMLDocument.SelectNodes("InputXML/Operation");
                    if (OperationNodes.Count == 0)
                        OperationNodes = InputXMLDocument.SelectNodes("INPUTXML/OPERATION");

                    FileSystemNodes = InputXMLDocument.SelectNodes("descendant::FileSystem");
                    if (FileSystemNodes.Count == 0)
                        FileSystemNodes = InputXMLDocument.SelectNodes("descendant::FILESYSTEM");

                    if (bTransaction == true)
                    {
                        IsOperationInTransaction = bTransaction;

                        //-----------------Transaction Oriented Execution-----------------------
                        using (TransactionScope Scope = new TransactionScope(TransactionScopeOption.RequiresNew, new TimeSpan(1, 0, 0)))
                        {
                            String TempErrorXML = null;

                            // Initialize the DataManager Object
                            OMDataManager = new DataManager(TenantName, true);

                            TransactionId = 0;
                            FileSystemManagerObj = null;
                            // Create File System Manager object to be used for top level processing as well as for implicit file delete operations
                            // This ensures that same file operation transaction id is used for all file operations
                            FileSystemManagerObj = new FileSystemManager(UserID, TenantName, true, OMDataManager);
                            OMDataManager.FileSystemManager = FileSystemManagerObj;
                            //THIS IS DONE PURPOSEFULLY SO THAT XMLRepository GET INITIALIZED WITH LOGDataManager PROPERLY
                            OMDataManager.XMLRepository.Init(OMDataManager);
                            // OMDataManager.SetTracing(bTracing);

                            if (ExecuteOperationOnMomDB)
                            {
                                #region Execute File System Operations

                                    if (FileSystemNodes.Count > 0)
                                    {
                                        try
                                        {
                                            foreach (XmlNode FileSystemNode in FileSystemNodes)
                                            {
                                                FileSystemNodeByRef = FileSystemNode;
                                                ErrorXML = ProcessFileSystemNode(FileSystemManagerObj, ref FileSystemNodeByRef);
                                                if (ErrorXML != null)
                                                {
                                                    ErrorXML = "<ERRORCOLLECTION>" + ErrorXML + "</ERRORCOLLECTION>";
                                                    break;
                                                }
                                            }

                                            // Flag if all file system operations are processed successfully
                                            if (String.IsNullOrEmpty(ErrorXML))
                                            {
                                                // b_FileSystemOperationCompleted may not be set to true if some WARNING is receieved for ProcessFileSystemNode
                                                b_FileSystemOperationCompleted = true;
                                            }
                                        }
                                        catch
                                        {
                                            throw;
                                        }
                                    } 

                                #endregion

                                if (ErrorXML == null)
                                {
                                    ErrorXML = executeTransactionOperation(OMDataManager, ref OperationNodes);
                                }

                                if (!String.IsNullOrEmpty(ErrorXML))
                                    str_ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");

                            } // if ExecuteOperationOnMomDB = TRUE

                            if ((String.IsNullOrEmpty(ErrorXML)) || ((!String.IsNullOrEmpty(str_ErrorType)) && (str_ErrorType.ToUpper().Equals("WARNING"))))
                            {
                                #region Execute ODS Operations
		                        if (InputXMLDocument.SelectSingleNode("INPUTXML") != null)
                                    IsSchemaUpdateForODS = ((XmlElement)InputXMLDocument.SelectSingleNode("INPUTXML")).GetAttribute("IsSchemaUpdateForODS", "");

                                if (IsSchemaUpdateForODS.ToUpper() == "TRUE")
                                {
                                    OMODSDataManager = new DataManager(TenantName, true, true);
                                    ErrorXML = ExecuteOperationsOnODS(InputXML);
                                }
                                else
                                {
                                    if (InputXMLDocument.SelectSingleNode("INPUTXML") != null)
                                    {
                                        if ((InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION/OBJECTNAME") != null) && (InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION/OBJECTNAME").InnerText == "FACETTYPE"))
                                        {
                                            if ((InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION/METHOD") != null) && (InputXMLDocument.SelectSingleNode("INPUTXML/OPERATION/METHOD").InnerText == "DELETE"))
                                            {
                                                OMODSDataManager = new DataManager(TenantName, true, true);
                                                ErrorXML = ExecuteOperationsOnODS(InputXML);
                                            }
                                        }
                                    }
                                } 
	                            #endregion

                                if (!String.IsNullOrEmpty(ErrorXML))
                                    str_ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");

                                #region Commit Database and File Operation Transactions. Catch and handle Zombie exception calling HandleInDoubtTransactions. 
		                        if ((String.IsNullOrEmpty(ErrorXML)) || ((!String.IsNullOrEmpty(str_ErrorType)) && (str_ErrorType.ToUpper().Equals("WARNING"))))
                                {
                                    // Commit File System Operations only when both File System Operations and OM operations are completed
                                    if (FileSystemNodes.Count == 0 || (FileSystemNodes.Count > 0 && b_FileSystemOperationCompleted))
                                    {
                                        try
                                        {
                                            OMDataManager.DataProvider.TransactionObj.Commit();
                                            if (OMODSDataManager != null)
                                            {
                                                OMODSDataManager.DataProvider.TransactionObj.Commit();
                                            }
                                            Scope.Complete();
                                            bDBOperationsCompleted = true;
                                        }
                                        catch (Exception ex)
                                        {                                            
                                            String strManagedThreadId = System.Threading.Thread.CurrentThread.ManagedThreadId.ToString();
                                            WriteLog(TenantName, "INSTR", strManagedThreadId, strManagedThreadId, @"""" + strManagedThreadId + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""Exception in Execute_OM() while committing transaction or completing scope:" + ex.Message.Replace(@"""", @"""""") + @"""");

                                            ErrorObj = new OMError((OMErrorCodes.OMCLASS_EXECUTE_FAILED), "Error in Executing Operation",
                                                                    OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                                                    OMErrorType.EXCEPTION, ex, null);
                                            ErrorXML = ErrorObj.Serialize();

                                            // DO NOT commit FS OPERATIONS in case of ZOMBIE EXCEPTION
                                            // HandleInDoubtTransactions retains the files that would be deleted if the file operations are committed
                                            if (ex.Message.Contains("This SqlTransaction has completed; it is no longer usable"))
                                            {
                                                if (FileSystemManagerObj != null)
                                                {
                                                    ErrorObj = FileSystemManagerObj.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if(ErrorObj !=  null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    ErrorObj = OMDataManager.XMLRepository.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if (ErrorObj != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    TempErrorXML = OMDataManager.DataProvider.DisposeDeferredQueryDataTablesIfAny();
                                                    if(!String.IsNullOrEmpty(TempErrorXML))
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""DisposeDeferredQueryDataTablesIfAny() returned TempErrorXML:" + TempErrorXML.Replace(@"""", @"""""") + @"""");
                                                    return ErrorXML;
                                                }
                                            }
                                            else
                                                throw ex;
                                        }
                                        //OMDataManager.DataProvider.Close();
                                    }

                                    if (bDBOperationsCompleted)
                                    {
                                        ErrorObj = FileSystemManagerObj.CommitFileSystemOperations();
                                        if(ErrorObj !=  null)
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""CommitFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                        ErrorObj = OMDataManager.XMLRepository.CommitXMLOperations();
                                        if (ErrorObj != null)
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""CommitXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                        RemoveCahcedItems();
                                    }
                                    else
                                    {
                                        ErrorObj = FileSystemManagerObj.RollbackFileSystemOperations();
                                        if (ErrorObj != null)
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                        ErrorObj = OMDataManager.XMLRepository.RollbackXMLOperations();
                                        if (ErrorObj != null)
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    }

                                    // If there is any error while committing file system operations, stop doing further operations
                                    if (ErrorObj != null)
                                        ErrorXML = ErrorObj.Serialize(true);

                                    TempErrorXML = OMDataManager.DataProvider.DisposeDeferredQueryDataTablesIfAny();
                                    if (!String.IsNullOrEmpty(TempErrorXML))
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""DisposeDeferredQueryDataTablesIfAny() returned TempErrorXML:" + TempErrorXML.Replace(@"""", @"""""") + @"""");

                                    InputXML = InputXMLDocument.OuterXml;
                                    return ErrorXML;
                                }
	                            #endregion
                            }

                            /* Dispose data tables created for deferred query execution (if applicable)
                             * Deferred query execution will be applicable only for CreateData and UpdateData methods for Entity.
                             * */
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                // In case when operation execution is failed, perform cleanup and without collecting errors for cleanup
                                try
                                {
                                    #region Extract File Operations Transaction Id and Rollback File Operations Transaction unless error type is Warning
		                            // Rollback File System Operation only if there is an ERROR/EXCEPTION, skip it for WARNING.
                                    TransactionId = 0;
                                    xpDoc_ErrorXml = new XPathDocument(new StringReader(ErrorXML));
                                    xpNav_ErrorXmlRoot = xpDoc_ErrorXml.CreateNavigator();
                                    if (xpNav_ErrorXmlRoot.SelectSingleNode("//ERROR/ERRORTYPE[text()='ERROR' or text()='EXCEPTION']") != null)
                                    {
                                        XmlDocument xDocOMError = new XmlDocument();
                                        xDocOMError.LoadXml(ErrorXML);
                                        XmlNode xNodeTranscation = xDocOMError.SelectSingleNode("//ERROR/TRANSACTIONID");
                                        if (xNodeTranscation != null)
                                        {
                                            if (!string.IsNullOrEmpty(xNodeTranscation.InnerText))
                                            {
                                                TransactionId = ulong.Parse(xNodeTranscation.InnerText);
                                            }
                                        }

                                        if (FileSystemManagerObj != null)
                                        {
                                            if (TransactionId != 0)
                                            {
                                                if (FileSystemManagerObj.TranscationId == TransactionId)
                                                {
                                                    //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations called with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    ErrorObj = FileSystemManagerObj.RollbackFileSystemOperations();
                                                    if (ErrorObj != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    ErrorObj = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                    if (ErrorObj != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    //if (b_FileSystemOperationCompleted)
                                                    //    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""FileSysstem Operations successful. RollbackFileSystemOperations called with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    //else
                                                    //    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""FileSysstem Operations not successful. RollbackFileSystemOperations called with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                                }
                                                else
                                                {
                                                    FileSystemManagerObj.RollbackFileSystemOperations();
                                                    if (ErrorObj != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    ErrorObj = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                    if (ErrorObj != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");

                                                    FileSystemManagerObj.TranscationId = TransactionId;
                                                    OMDataManager.XMLRepository.TranscationId = TransactionId;

                                                    FileSystemManagerObj.RollbackFileSystemOperations();
                                                    OMDataManager.XMLRepository.RollbackXMLOperations();
                                                }
                                            }
                                            else
                                            {
                                                ErrorObj = FileSystemManagerObj.RollbackFileSystemOperations();
                                                if (ErrorObj != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                ErrorObj = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                if (ErrorObj != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                            }
                                        } 
                                    }
	                                #endregion
                                }
                                catch
                                {
                                    throw;
                                }
                            }
                            TempErrorXML = OMDataManager.DataProvider.DisposeDeferredQueryDataTablesIfAny();
                            if (!String.IsNullOrEmpty(TempErrorXML))
                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""DisposeDeferredQueryDataTablesIfAny() returned TempErrorXML:" + TempErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                        //-----------------End Of Transaction Oriented Execution-----------------------
                    }
                    else
                    {
                        //------------------Transactionless Operations--------------

                        ErrorXML = executeTransactionlessOperation(ref OperationNodes, bTracing, false, ref OutputXML);

                        //------------------End Of Transactionless Operations--------------
                    }

                    InputXML = InputXMLDocument.OuterXml;
                }
                catch (Exception ExcepObj)
                {
                    ErrorObj = new OMError(OMErrorCodes.OMCLASS_EXECUTE_FAILED, "Error in Executing Operation",
                                            OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                            OMErrorType.EXCEPTION, ExcepObj, null);
                    ErrorXML = ErrorObj.Serialize();

                    #region Extract File Operations Transaction Id and Rollback File Operations Transaction

		                TransactionId = 0;
                        XmlDocument xDocOMError = new XmlDocument();
                        xDocOMError.LoadXml(ErrorXML);
                        XmlNode xNodeTranscation = xDocOMError.SelectSingleNode("//ERROR/TRANSACTIONID");
                        if (xNodeTranscation != null)
                        {
                            if (!string.IsNullOrEmpty(xNodeTranscation.InnerText))
                            {
                                TransactionId = ulong.Parse(xNodeTranscation.InnerText);
                            }
                        }
                        if (FileSystemManagerObj != null)
                        {
                            try
                            {
                                if (bDBOperationsCompleted)
                                {
                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""CommitFileSystemOperations called with in catch block of Execute_OM() with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = FileSystemManagerObj.CommitFileSystemOperations();
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""CommitFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = OMDataManager.XMLRepository.CommitXMLOperations();
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""CommitXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    RemoveCahcedItems();
                                }
                                else
                                {
                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations called with in catch block of Execute_OM() with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = FileSystemManagerObj.RollbackFileSystemOperations();
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackFileSystemOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = OMDataManager.XMLRepository.RollbackXMLOperations();
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""RollbackXMLOperations returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                }
                            }
                            catch
                            {
                                if (bDBOperationsCompleted)
                                {
                                    ErrorObj = FileSystemManagerObj.HandleInDoubtTransactions("ERROR while calling CommitFileSystemOperations()");
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = OMDataManager.XMLRepository.HandleInDoubtTransactions("ERROR while calling CommitFileSystemOperations()");
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                }
                                else
                                {
                                    ErrorObj = FileSystemManagerObj.HandleInDoubtTransactions("ERROR while calling RollbackFileSystemOperations()");
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                    ErrorObj = OMDataManager.XMLRepository.HandleInDoubtTransactions("ERROR while calling RollbackFileSystemOperations()");
                                    if (ErrorObj != null)
                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute_OM()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + ErrorObj.ErrorXML.Replace(@"""", @"""""") + @"""");
                                }
                            }
                        }
                        else
                        {
                            if (TransactionId != 0)
                            {
                                FileSystemManager FileSystemManagerObjEx = new FileSystemManager(UserID, TenantName, true, OMDataManager);
                                FileSystemManagerObjEx.TranscationId = TransactionId;
                                FileSystemManagerObjEx.RollbackFileSystemOperations();
                            }
                        }
     
	                #endregion
                }
                finally
                {
                    if (OMDataManager != null)
                        OMDataManager.DataProvider.Close();

                    if (OMODSDataManager!= null)
                        OMODSDataManager.DataProvider.Close();

                    // Close database connection used by File System Manager
                    if (FileSystemManagerObj != null)
                        FileSystemManagerObj.CloseDatabaseConnection();

                    // Can Retry
                    if (bTransaction)
                        DoExecute = DataManager.CanRetry(ErrorXML, RetryCount, ref IterationCount, RetrySleepDuration);
                    else
                        DoExecute = false;
                }
            }

            return ErrorXML;
        }

        private string ProcessFileSystemNode(FileSystemManager FileSystemManagerObj,ref XmlNode FileSystemNode)
        {
            OMError                 ErrObj                              = null;
            XmlNode                 ObjectNameNode                      = null;
            XmlNode                 OperationNode                       = null;
            XmlNode                 ActualOperationNode                 = null;
            string                  ObjectType                          = null;
            string                  ObjectName                          = null;
            ulong                   ObjectTypeID                        = 0;
            string                  PrimaryKeyName                      = null;
            string                  RelationshipXML                     = null;
            ulong                   PrimaryKeyValue                     = 0;
            XmlNode                 PrimarykeyNode                      = null;
            RelationshipType        RelationshipObj                     = null;
            XPathDocument           RelationXPDoc                       = null;
            XPathNavigator          RelationshipNav                     = null;
            XmlNode                 xmlNode_CopyFiles                   = null;     //Used to select COPYFILES node
            XmlNode                 xmlNode_MoveFiles                   = null;     //Used to select MOVEFILES node
            XmlNode                 xmlNode_OperationNodeForFileSystem  = null;     //Used to select OPERATION node
            bool                    b_CopyFiles                         = false;    //Used to store COPYFILES node node
            bool                    b_MoveFiles                         = false;    //Used to store MOVEFILES node value

            ObjectNameNode = FileSystemNode.ParentNode;
            ObjectName = ObjectNameNode.Name;
            OperationNode = ObjectNameNode.ParentNode;
            ActualOperationNode = OperationNode.ParentNode;

            if (OperationNode.Name.ToUpper() != "PARAM")
            {
                // This is the case of RE's and RF's
                RelationshipObj = new RelationshipType(OMDataManager);
                ErrObj = RelationshipObj.ReadByRelationShipName(OperationNode.Name, ref RelationshipXML);
                if (ErrObj != null)
                    return (ErrObj.ErrorMessage);
                RelationXPDoc = new XPathDocument(new System.IO.StringReader(RelationshipXML));
                RelationshipNav = RelationXPDoc.CreateNavigator();
                RelationshipNav = RelationshipNav.SelectSingleNode("RELATIONSHIPTYPE/CONTAINSTYPE");
                ObjectType = RelationshipNav.Value;
            }
            else
            {
                ObjectType = ActualOperationNode.SelectSingleNode("OBJECTNAME").InnerText;
            }

            if ((ErrObj = getObjectDetails(ObjectName, ObjectType, ref ObjectTypeID, ref PrimaryKeyName)) != null)
                return ErrObj.Serialize();
            
            if (ObjectNameNode.SelectNodes(PrimaryKeyName).Count > 0)
                PrimaryKeyValue = Convert.ToUInt64(ObjectNameNode.SelectSingleNode(PrimaryKeyName).InnerText);
            else
            {
                UniqueID.Read(ref PrimaryKeyValue);
                PrimarykeyNode              = ObjectNameNode.OwnerDocument.CreateElement(PrimaryKeyName);
                PrimarykeyNode.InnerText    = PrimaryKeyValue.ToString();
                ObjectNameNode.AppendChild(PrimarykeyNode);
            }

            //Check whether ActualOperationNode is OPERATION node, it will be used to read COPYFILES attribute
            if (ActualOperationNode != null)
            {
                if (ActualOperationNode.Name.ToUpper().CompareTo("OPERATION") == 0)
                    xmlNode_OperationNodeForFileSystem = ActualOperationNode;
            }

            //If ActualOperationNode is not an OPERATION node, then get OPERATION node to read COPYFILES attribute
            if(xmlNode_OperationNodeForFileSystem == null)
            {
                xmlNode_OperationNodeForFileSystem = GetOperationNodeForFileSystem(FileSystemNode);
            }

            //Identify whether COPYFILES or MOVEFILES attribute is specified and propoget value to File System Manager
            if (xmlNode_OperationNodeForFileSystem != null)
            {
                xmlNode_CopyFiles = xmlNode_OperationNodeForFileSystem.SelectSingleNode("COPYFILES");

                if (xmlNode_CopyFiles != null)
                {
                    Boolean.TryParse(xmlNode_CopyFiles.InnerText, out b_CopyFiles);
                }

                xmlNode_MoveFiles = xmlNode_OperationNodeForFileSystem.SelectSingleNode("MOVEFILES");

                if (xmlNode_MoveFiles != null)
                {
                    Boolean.TryParse(xmlNode_MoveFiles.InnerText, out b_MoveFiles);
                }
            }

            //Propagate MountPoint attribute (given at FILESYSTEM node) if specified.
            if(FileSystemNode.Attributes.GetNamedItem("MountPoint") != null)
                FileSystemManagerObj.UserMountPointForFileSystem = FileSystemNode.Attributes.GetNamedItem("MountPoint").Value;

            FileSystemManagerObj.CopyFiles = b_CopyFiles;
            FileSystemManagerObj.MoveFiles = b_MoveFiles;

            if ((ErrObj = FileSystemManagerObj.ProcessFileSystem(PrimaryKeyValue, ObjectType, ObjectTypeID, FileSystemNode.OuterXml)) != null)
                return ErrObj.Serialize();

            //Reset CopyFiles and MoveFiles flags to false
            FileSystemManagerObj.CopyFiles = false;
            FileSystemManagerObj.MoveFiles = false;
            
            return null;
        }
        private OMError getObjectDetails(String ObjectTypeName,string ObjectType, ref ulong TypeID , ref string PrimaryKeyName)
        {
            OMError ReadErrorObj = null;   //OMError object for read record in the database.
            OMError ReturnErrorObj = null;   //OMError object for returning from OM.
            String ObjectTypeXML = null;   //String variable storing the EntityType XML.
            XmlDocument ObjectTypeXMLDocument = null;

            try
            {
                ObjectTypeXMLDocument = new XmlDocument();
                if (ObjectType.ToUpper() == "ENTITY")
                {
                    ReadErrorObj = OMDataManager.DataProvider.AddClause("ENTITYTYPENAME", OMFieldDataType.STRING, OMOperator.EQUALTO,
                                                                ObjectTypeName, OMConjunction.NOTHING);
                    if (ReadErrorObj != null)
                        return ReadErrorObj;

                    ReadErrorObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                            ref ObjectTypeXML);
                    if (ReadErrorObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                            "EntityType Not Found", OMErrorObjectType.ENTITYTYPE,
                            "EntityType", ReadErrorObj.ErrorType, ReadErrorObj.ExceptionObject, null);

                        return ReturnErrorObj;
                    }
                    ObjectTypeXMLDocument.LoadXml(ObjectTypeXML);

                    TypeID = Convert.ToUInt64(ObjectTypeXMLDocument.SelectSingleNode("ENTITYTYPE/ENTITYTYPEID").InnerText);
                    PrimaryKeyName = ObjectTypeXMLDocument.SelectSingleNode("ENTITYTYPE/PRIMARYKEY").InnerText;

                }
                else
                {
                    ReadErrorObj = OMDataManager.DataProvider.AddClause("FACETTYPENAME", OMFieldDataType.STRING, OMOperator.EQUALTO,
                                                             ObjectTypeName, OMConjunction.NOTHING);
                    if (ReadErrorObj != null)
                        return ReadErrorObj;

                    ReadErrorObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                            ref ObjectTypeXML);
                    if (ReadErrorObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_NOT_FOUND,
                            "FacetType Not Found", OMErrorObjectType.FACETTYPE ,
                            "FacetType", ReadErrorObj.ErrorType, ReadErrorObj.ExceptionObject, null);

                        return ReturnErrorObj;
                    }
                    ObjectTypeXMLDocument.LoadXml(ObjectTypeXML);

                    TypeID = Convert.ToUInt64(ObjectTypeXMLDocument.SelectSingleNode("FACETTYPE/FACETTYPEID").InnerText);
                    PrimaryKeyName = ObjectTypeXMLDocument.SelectSingleNode("FACETTYPE/PRIMARYKEY").InnerText;

                }

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_NOT_FOUND,
                        "Type Not Found", OMErrorObjectType.ENTITYTYPE,
                        "Type", OMErrorType.EXCEPTION, ExceptionObj, null);

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedOn>11 September 2009</CreatedOn>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <purpose>
        /// Fetches OPERATION node from Input Xml.
        /// </purpose>
        /// </summary>
        /// <param name="xmlNodeFileSystemNode">File System node for which OPERATION node is to be searched.</param>
        /// <returns>Returns OPERATION node.</returns>
        private XmlNode GetOperationNodeForFileSystem(XmlNode xmlNodeFileSystemNode)
        {
            //Iterate over parent nodes to get OPERATION node
            if (xmlNodeFileSystemNode != null)
            {
                while (xmlNodeFileSystemNode.ParentNode != null)
                {
                    if (xmlNodeFileSystemNode.ParentNode.Name.ToUpper().CompareTo("OPERATION") == 0)
                    {
                        return xmlNodeFileSystemNode.ParentNode;
                    }

                    xmlNodeFileSystemNode = xmlNodeFileSystemNode.ParentNode;
                }
            }

            return null;
        }

        /// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>24 May 2011</CreatedOn>
        /// </summary>
        /// <param name="InputXML"></param>
        /// <returns>An Error XML if an Error Occurs</returns>
        private String ExecuteOperationsOnODS(String InputXML)
        {
            String                  ErrorXML        = null;
            ObjectManagerODSClass   ODS_OjectMgrObj = null;
            //Call the Execute Method for ODS
            ODS_OjectMgrObj     = new ObjectManagerODSClass(UserID, CurrentLanguage, TenantName, OMDataManager, OMODSDataManager);
            ErrorXML            = ODS_OjectMgrObj.Execute(ref InputXML);
            return ErrorXML;
        }

        #endregion

        public String GetData(String SearchQueryXML, String FieldListXML, 
                                ref String OutputXML)
        {
            Search  OMSearch        = null;
            OMError ErrorObj        = null;
            String  ErrorXML        = null;
            int     IterationCount  = 0;
            Boolean DoExecute       = true;

            while (DoExecute)
            {
                try
                {
                    ErrorXML    = null;
                    OutputXML   = null;

                    //Initialiaze the DataManagerReader Object
                    OMDataManager = new DataManager(TenantName);


                    //Instantiating Search Object
                    OMSearch = new Search(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMSearch.FileTransfer = m_delFileTransfer;
                    ErrorObj = OMSearch.GetData(SearchQueryXML, FieldListXML, ref  OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                    }
                }
                catch (Exception ExcepObj)
                {
                    ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETDATA_FAILED, "Error in Getdata Operation",
                                          OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                          OMErrorType.EXCEPTION, ExcepObj, null);
                    ErrorXML = ErrorObj.Serialize();
                }

                finally
                {
                    if (OMDataManager != null)
                        OMDataManager.DataProvider.Close();
                    //Can Retry
                    DoExecute = DataManager.CanRetry(ErrorXML, RetryCount, ref IterationCount, RetrySleepDuration);
                }
            }

            return ErrorXML;
            
        }

        public String GetFiles(ref String InputXML)
        {
            OMError             ErrorObj                = null;
            String              ErrorXML                = null;
            XmlDocument         InputXMLDoc             = null;
            XmlNodeList         FileNodes               = null;
            XmlNode             FileRequiredAttr        = null;
            StringCollection    ASyncronousFieldList    = null;
            StringCollection    AvoidFileTransferList   = null;
            FileSystemManager   FileSystemManagerObj    = null;
            String              FieldName               = null;
            String              FileRequired            = null;
            XmlNode             TransferModeAttr        = null;
            string              TransferMode            = null;
            XmlNode             SchemaIDNode            = null;

            try
            {
                //Initialize the DataManager Object
                if (OMDataManager == null)
                    OMDataManager = new DataManager(TenantName, false);

                FileSystemManagerObj    = new FileSystemManager(UserID, TenantName, 
                                                                false, OMDataManager);
                InputXMLDoc             = new XmlDocument();
                ASyncronousFieldList    = new StringCollection();
                AvoidFileTransferList   = new StringCollection();

                InputXMLDoc.LoadXml(InputXML);
                
                FileNodes = InputXMLDoc.SelectNodes("//FILE");
                
                foreach (XmlNode FileNode in FileNodes)
                {
                    FileRequiredAttr    = null;
                    FileRequiredAttr    = FileNode.Attributes.GetNamedItem("FileRequired");

                    // Anand : This code is added because if schemaid is provided we use SchemaID else 
                    // FieldName. As Uniqueness is not garanteed incase of field name. i.e. two entities can have
                    // same field name.(Earlier schemaid was not getting passed by getdata/gethierarchy thus we could not 
                    // use this scheme but as getdata/gethierarchy is changed pass schemaid, we can use this scheme).  
                    if ((SchemaIDNode = FileNode.SelectSingleNode("SCHEMAID")) != null)
                       FieldName           = SchemaIDNode.InnerText;
                    else
                       FieldName           = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                                        
                    if (FileRequiredAttr != null)
                    {
                        FileRequired = FileRequiredAttr.InnerText;
                        if (FileRequired.ToUpper() == "NO")
                        {
                            AvoidFileTransferList.Add(FieldName);
                        }
                        else
                        {
                            TransferModeAttr = FileNode.Attributes.GetNamedItem("TransferMode");
                            if (TransferModeAttr != null)
                            {
                                TransferMode = TransferModeAttr.InnerText;
                                if (TransferMode.ToUpper() == "ASYNCRONOUS")
                                {
                                    ASyncronousFieldList.Add(FieldName);
                                }
                            }
                        }
                    }
                    else
                    {
                        AvoidFileTransferList.Add(FieldName);
                    }
                }
                InputXML = InputXMLDoc.OuterXml;
                FileSystemManagerObj.FileTransfer = m_delFileTransfer;
                ErrorObj = FileSystemManagerObj.RetriveFileSystem(ASyncronousFieldList, AvoidFileTransferList, ref InputXML);
                if (ErrorObj != null)
                {
                    ErrorXML = ErrorObj.Serialize();
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETFILES_FAILED, "Error in GetFiles Operation",
                                      OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                      OMErrorType.EXCEPTION, ExcepObj, null);
                ErrorXML = ErrorObj.Serialize();

                return ErrorXML;
            }

            finally
            {
                if (OMDataManager != null)
                    OMDataManager.DataProvider.Close();
                //Close database connections used by File System Manager
                if (FileSystemManagerObj != null)
                    FileSystemManagerObj.CloseDatabaseConnection();
            }

            return null;

        }

        #region GetTypeData
        /// <summary>
        ///<CreatedDate>Aug 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Read configuration data
        /// 1. Type Information
        /// 2. Access Control Information
        /// 3. Named Hierarchy Information
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML"></param>
        /// <param name="OutputXML"></param>
        /// <returns></returns>
        public String GetTypeData(String InputXML, ref String OutputXML)
        {
            OMError         ErrorObj            = null;
            XmlDocument     InputXMLDocument    = null;
            XmlNodeList     OperationNodes      = null;
            //XmlNode         MethodNameNode      = null;
            //XmlNode         OperationNode       = null;
            //XmlNode         ObjectNameNode      = null;
            XmlNode         TracingNode         = null;
            String          ErrorXML            = null;
            //String          MethodName          = null;
            //String          ObjectName          = null;
            bool            bTracing            = false;
            int             IterationCount      = 0;
            Boolean         DoExecute           = true;

            while (DoExecute)
            {
                try
                {
                    OutputXML   = null;
                    ErrorXML    = null;

                    //Load the InputXML
                    InputXMLDocument = new XmlDocument();
                    InputXMLDocument.LoadXml(InputXML);

                    //Set the Tracing attribute of DataManager
                    TracingNode = InputXMLDocument.SelectSingleNode("InputXML/Tracing");

                    if (TracingNode == null)
                        TracingNode = InputXMLDocument.SelectSingleNode("INPUTXML/TRACING");

                    if (TracingNode != null)
                        bTracing = Convert.ToBoolean(TracingNode.InnerText);
                    else
                        bTracing = false;

                    //Process the ParamNodes
                    OperationNodes = InputXMLDocument.SelectNodes("InputXML/Operation");

                    if (OperationNodes.Count == 0)
                        OperationNodes = InputXMLDocument.SelectNodes("INPUTXML/OPERATION");

                    //If OPERATION is not specified or more than one OPERATION is specified, return error
                    if (OperationNodes.Count == 0 || OperationNodes.Count > 1)
                    {
                        //Return error since only one OPERATION node is expected in input xml
                        ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETDATATYPE_FAILED, "Error in GetTypeData Operation",
                                         OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                         OMErrorType.ERROR, null, null);
                        ErrorXML = ErrorObj.Serialize();
                    }

                    ErrorXML = executeReadOperation(OperationNodes.Item(0), bTracing, false, ref OutputXML);
                }
                catch (Exception ExcepObj)
                {
                    ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETDATATYPE_FAILED, "Error in GetTypeData Operation",
                                         OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                         OMErrorType.EXCEPTION, ExcepObj, null);
                    ErrorXML = ErrorObj.Serialize();
                }
                finally
                {
                    //Can Retry
                    DoExecute = DataManager.CanRetry(ErrorXML, RetryCount, ref IterationCount, RetrySleepDuration);
                }
            }
            return ErrorXML;
        }

        #endregion

        #region GetHierarchial Data

        public String GetHierarchialData(String SearchInputXML,
                                ref String OutputXML)
        {
            NSearch NSearchObj  = null;
            //NSearchTeraData NSearchTDObj = null;
            OMError ErrorObj    = null;
            String ErrorXML     = null;
            int IterationCount  = 0;
            Boolean DoExecute   = true;

            while (DoExecute)
            {

                try
                {
                    OutputXML = null;
                    ErrorXML = null;

                    //Initialiaze the DataManagerReader Object
                    OMDataManager = new DataManager(TenantName);

                    //Instantiating NSearch Object
                    NSearchObj = new NSearch(UserID, CurrentLanguage, TenantName, OMDataManager);
                    NSearchObj.FileTransfer = m_delFileTransfer;
                    ErrorObj = NSearchObj.GetHierarchialData(SearchInputXML, ref  OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                    }
                }
                catch (Exception ExcepObj)
                {
                    ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETDATA_FAILED, "Error in GetHierarchialData Operation",
                                          OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                          OMErrorType.EXCEPTION, ExcepObj, null);
                    ErrorXML = ErrorObj.Serialize();
                }
                finally
                {
                    if (OMDataManager != null)
                        OMDataManager.DataProvider.Close();

                    //Can Retry
                    DoExecute = DataManager.CanRetry(ErrorXML, RetryCount, ref IterationCount, RetrySleepDuration);
                }
            }

            return ErrorXML;
        }

        public String GetHierarchialData(String SearchInputXML, ref DataSet DS_Result)
        {
            NSearch     NSearchObj      = null;
            OMError     ErrorObj        = null;
            String      ErrorXML        = null;
            int         IterationCount  = 0;
            Boolean     DoExecute       = true;

            while (DoExecute)
            {

                try
                {
                    ErrorXML        = null;

                    //Initialiaze the DataManagerReader Object
                    OMDataManager   = new DataManager(TenantName);

                    //Instantiating NSearch Object
                    NSearchObj      = new NSearch(UserID, CurrentLanguage, TenantName, OMDataManager);
                    ErrorObj        = NSearchObj.GetHierarchialData(SearchInputXML, ref  DS_Result);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                    }
                }
                catch (Exception ExcepObj)
                {
                    ErrorObj = new OMError(OMErrorCodes.OMCLASS_GETDATA_FAILED, "Error in GetHierarchialData Operation",
                                          OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                          OMErrorType.EXCEPTION, ExcepObj, null);
                    ErrorXML = ErrorObj.Serialize();
                }
                finally
                {
                    if (OMDataManager != null)
                        OMDataManager.DataProvider.Close();

                    //Can Retry
                    DoExecute = DataManager.CanRetry(ErrorXML, RetryCount, ref IterationCount, RetrySleepDuration);
                }
            }

            return ErrorXML;
        }
        #endregion

        public void WriteLog(String TenantName, String SolutionName, String PageName, String SessionID, String ContentToWrite)
        {
            lock (this)
            {
                //if (ReadRegisterykey())
                //{
                PageName = "Thread";
                String InstallationDir = ALCommon.GetInstallationDir();
                int LastIndex = PageName.LastIndexOf('/');
                int ExtIndex = PageName.LastIndexOf('.');

                if (LastIndex > 0 && ExtIndex > 0)
                {
                    PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                    PageName = PageName.Replace(' ', '_');
                }
                String DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                String LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                FileStream LogFileStream = null;
                StreamWriter LogStreamWriter = null;

                try
                {
                    DirectoryInfo Root = new DirectoryInfo(DirName);
                    if (Root.Exists == false)
                        Root.Create();

                    if (File.Exists(LogFileName))
                        LogFileStream = new FileStream(LogFileName, FileMode.Append);
                    else
                        LogFileStream = new FileStream(LogFileName, FileMode.Create);

                    LogStreamWriter = new StreamWriter(LogFileStream);
                    StringBuilder TimeStamp = new StringBuilder();

                    //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                    //LogStreamWriter.WriteLine("=====================================================================");
                    //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                    LogStreamWriter.WriteLine(ContentToWrite);
                    LogStreamWriter.WriteLine();
                }
                catch (Exception ex)
                {
                    PageName = "WriteLogError";
                    InstallationDir = ALCommon.GetInstallationDir();
                    LastIndex = PageName.LastIndexOf('/');
                    ExtIndex = PageName.LastIndexOf('.');

                    if (LastIndex > 0 && ExtIndex > 0)
                    {
                        PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                        PageName = PageName.Replace(' ', '_');
                    }
                    DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                    LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                    FileStream LogFileStreamEx = null;
                    StreamWriter LogStreamWriterEx = null;

                    try
                    {
                        DirectoryInfo Root = new DirectoryInfo(DirName);
                        if (Root.Exists == false)
                            Root.Create();

                        if (File.Exists(LogFileName))
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Append);
                        else
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Create);
                        string strMessageToWrite = SessionID + "," + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + "," + ex.Message;
                        LogStreamWriterEx = new StreamWriter(LogFileStreamEx);
                        StringBuilder TimeStamp = new StringBuilder();

                        //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                        //LogStreamWriter.WriteLine("=====================================================================");
                        //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                        LogStreamWriterEx.WriteLine(strMessageToWrite);
                        LogStreamWriterEx.WriteLine();
                    }
                    catch
                    {

                    }
                    finally
                    {
                        if (LogStreamWriterEx != null)
                            LogStreamWriterEx.Close();

                        if (LogFileStreamEx != null)
                            LogFileStreamEx.Close();

                        LogStreamWriterEx = null;
                        LogFileStreamEx = null;
                    }
                }
                finally
                {
                    if (LogStreamWriter != null)
                        LogStreamWriter.Close();

                    if (LogFileStream != null)
                        LogFileStream.Close();

                    LogStreamWriter = null;
                    LogFileStream = null;
                }
                //}
            }
        }

        private bool TestDBOperationStatus(ulong TransactionId)
        {
            OMError ErrObj = null;
            string DurableLogXml = null;
            string FileListXml = null;

            try
            {
                if ((ErrObj = OMDataManager.DataProvider.AddClause("TRANSACTIONID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TransactionId.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = OMDataManager.DataProvider.ReadAllRecord("DURABLELOG", "ALFS_DURABLELOG", null, ref DurableLogXml)) == null)
                    {
                        if (!String.IsNullOrEmpty(DurableLogXml))
                        {
                            XmlDocument xDocLog = new XmlDocument();
                            xDocLog.LoadXml(DurableLogXml);

                            #region DELETEFILE case
                            XmlNode xNodeDeleteFile = xDocLog.SelectSingleNode("DURABLELOGCOLLECTION/DURABLELOG[OPERATION='DeleteFile']");
                            if (xNodeDeleteFile != null)
                            {
                                XmlDocument xDocParameterInfo = new XmlDocument();
                                xDocParameterInfo.LoadXml(xNodeDeleteFile.SelectSingleNode("PARAMETERINFO").InnerText);
                                XmlNodeList xListParameters = xDocParameterInfo.SelectNodes("PARAMETERS/PARAM");
                                string strFilePath = xListParameters[0].InnerText;
                                string strFileId = strFilePath.Substring(strFilePath.LastIndexOf(@"\") + 1);

                                if ((ErrObj = OMDataManager.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, strFileId, OMConjunction.NOTHING)) == null)
                                {
                                    if ((ErrObj = OMDataManager.DataProvider.ReadAllRecord("FILE", "ALFS_FILE", null, ref FileListXml)) == null)
                                    {
                                        if (string.IsNullOrEmpty(FileListXml))
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" DeleteFile - ALFS_FILE Entry not found for File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                            return true; //DB Entry is not present for Delete file - DB operation commited
                                        }
                                        else
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" DeleteFile - ALFS_FILE Entry found for File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                            return false;
                                        }
                                    }
                                }
                            }
                            #endregion

                            #region CREATEFILE case
                            XmlNode xNodeCreateFile = xDocLog.SelectSingleNode("DURABLELOGCOLLECTION/DURABLELOG[OPERATION='CreateFile']");
                            if (xNodeCreateFile != null)
                            {
                                XmlDocument xDocParameterInfo = new XmlDocument();
                                xDocParameterInfo.LoadXml(xNodeCreateFile.SelectSingleNode("PARAMETERINFO").InnerText);
                                XmlNodeList xListParameters = xDocParameterInfo.SelectNodes("PARAMETERS/PARAM");

                                string strFilePath = xListParameters[1].InnerText;
                                string strFileId = strFilePath.Substring(strFilePath.LastIndexOf(@"\") + 1);

                                if ((ErrObj = OMDataManager.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, strFileId, OMConjunction.NOTHING)) == null)
                                {
                                    if ((ErrObj = OMDataManager.DataProvider.ReadAllRecord("FILE", "ALFS_FILE", null, ref FileListXml)) == null)
                                    {
                                        if (string.IsNullOrEmpty(FileListXml))
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" CreateFile - ALFS_FILE Entry not found for File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                            return false; //DB Entry is not present for Create file - DB operation NOT COMMITED
                                        }
                                        else
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" CreateFile - ALFS_FILE Entry not found for File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                            return true;
                                        }
                                    }
                                }
                            }
                            #endregion

                            #region MOVEFILE case
                            XmlNode xNodeMoveFile = xDocLog.SelectSingleNode("DURABLELOGCOLLECTION/DURABLELOG[OPERATION='MoveFile']");
                            if (xNodeMoveFile != null)
                            {
                                XmlDocument xDocParameterInfo = new XmlDocument();
                                xDocParameterInfo.LoadXml(xNodeMoveFile.SelectSingleNode("PARAMETERINFO").InnerText);
                                XmlNodeList xListParameters = xDocParameterInfo.SelectNodes("PARAMETERS/PARAM");

                                string strFilePath = xListParameters[1].InnerText;
                                string strFileId = strFilePath.Substring(strFilePath.LastIndexOf(@"\") + 1);
                                string strFilePathDb = strFilePath.Substring(0, strFilePath.Substring(0, strFilePath.LastIndexOf(@"\")).Length);

                                if ((ErrObj = OMDataManager.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, strFileId, OMConjunction.AND)) == null)
                                {
                                    if ((ErrObj = OMDataManager.DataProvider.AddClause("PATH", OMFieldDataType.STRING, OMOperator.EQUALTO, strFilePathDb, OMConjunction.NOTHING)) == null)
                                    {
                                        if ((ErrObj = OMDataManager.DataProvider.ReadAllRecord("FILE", "ALFS_FILE", null, ref FileListXml)) == null)
                                        {
                                            if (string.IsNullOrEmpty(FileListXml))
                                            {
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" MoveFile - ALFS_FILE Entry not found for Destination File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                                return false;
                                            }
                                            else
                                            {
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"","" MoveFile - ALFS_FILE Entry found for Destination File ID: " + strFileId + @"""" + @",""" + TransactionId.ToString() + @"""");
                                                return true;
                                            }
                                        }
                                    }
                                }
                            }
                            #endregion
                        }
                        else
                        {
                            return true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""TestDBOperationStatus()"",""Exception in TestDBOperationStatus(): " + ex.Message.Replace(@"""", @"""""") + @"""");
            }
            return true;
        }

        private void RemoveCahcedItems()
        {
            foreach (String sCachedItem in OMDataManager.CacheItemsToRemove)
            {
                CacheManager.RemoveCacheItem(sCachedItem);
            }
        }

    }

    
}



