using System;
using System.Xml;
using System.Xml.XPath;
using System.Collections;
using System.Text;
using System.IO;
using System.Collections.Specialized;
using System.Transactions;
using System.Diagnostics;

using Assetlink;
using Assetlink.ObjectManager;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.EntityFacetManagement;
using Assetlink.ObjectManager.ETFTManagement;
using Assetlink.ObjectManager.AccessControl;
using Assetlink.ObjectManager.NamedHierarchy;
using Assetlink.ObjectManager.GroupHierarchy;
using Assetlink.FileAccessLayer;
/// <summary>
/// Summary description for Class1
/// </summary>
/// 
namespace Assetlink
{
    public partial class ObjectManagerClass : ObjectManagerBase
    {
        #region Transaction Operation
        private String executeTransactionOperation(DataManager DataMangerObj,
                                                   ref XmlNodeList OperationNodes)
        {
            XmlNodeList         ParamNodes                  = null;
            XmlNode             ObjectNameNode              = null;
            XmlNode             MethodNameNode              = null;
            String              MethodName                  = null;
            String              ObjectName                  = null;
            String              ErrorXML                    = null;
            String              OutputXML                   = null;
            StringBuilder       ReturnErrors                = null;
            bool                IsAnyError                  = false;
            XPathDocument       ErrorXMLDoc                 = null;
            XPathNavigator      ErrorXMLDocNav              = null;
            Boolean             ErrorOccured                = false;
            XmlNode             xmlNode_AccessControl       = null; //Used to represent ACCESSCONTROLCHECKREQUIRED node in input xml. This node specifies whether access control check is required for CreateData/UpdateData.
           
            try
            {
                ReturnErrors = new StringBuilder();
                ReturnErrors.Append("<ERRORCOLLECTION>");
                foreach (XmlNode OperationNode in OperationNodes)
                {
                    ErrorOccured    = false;
                    //---Read the ObjectName and Method combination
                    ObjectNameNode  = OperationNode.SelectSingleNode("ObjectName");

                    if (ObjectNameNode == null)
                        ObjectNameNode = OperationNode.SelectSingleNode("OBJECTNAME");
                    
                    if (ObjectNameNode != null)
                        ObjectName = ObjectNameNode.InnerText;
                    else
                        ObjectName = "Unknown";

                    MethodNameNode = OperationNode.SelectSingleNode("Method");
                    if (MethodNameNode == null)
                        MethodNameNode = OperationNode.SelectSingleNode("METHOD");

                    if (MethodNameNode != null)
                        MethodName = MethodNameNode.InnerText;
                    else
                        MethodName = "Unknown";

                    /* Check whether access control check is to be suppressed. This should be specified only for CreateData and UpdateData.
                     * Access control check can be suppressed only for ENTITY object operations like CREATEDATA and UPDATEDATA.*/
                    if (ObjectName.ToUpper().CompareTo("ENTITY") ==0 && 
                        (MethodName.ToUpper().CompareTo("CREATEDATA") == 0 || MethodName.ToUpper().CompareTo("UPDATEDATA") == 0))
                    {
                        xmlNode_AccessControl = OperationNode.SelectSingleNode("ACCESSCONTROLCHECKREQUIRED");
                        if (xmlNode_AccessControl != null)
                            DataMangerObj.DataProvider.IsAccessControlCheckRequired = Convert.ToBoolean(xmlNode_AccessControl.InnerText);
                    }

                    //Read all the Parameter Nodes
                    ParamNodes = OperationNode.SelectNodes("Param");

                    if(ParamNodes.Count == 0)
                        ParamNodes = OperationNode.SelectNodes("PARAM");


                    ErrorXML = executeOperations(DataMangerObj,
                                                 ObjectName,
                                                 MethodName, ref ParamNodes, ref OutputXML);

                    //Anand : This code has been added so that if warning is returned while performing an operation
                    //changes should not be rolled back.
                    if (ErrorXML != null)
                    {
                        ErrorXMLDoc     = new XPathDocument(new StringReader(ErrorXML));
                        ErrorXMLDocNav  = ErrorXMLDoc.CreateNavigator();
                        if (ErrorXMLDocNav.SelectSingleNode("ERROR/ERRORTYPE[text()='ERROR' or text()='EXCEPTION']") != null)
                        {
                            ErrorOccured = true;
                        }
                     }
                    
                    //If there is an error while executing a operation break from the for loop
                    if (ErrorOccured)
                    {
                        ErrorXML        = prepareErrorXML(OperationNode, ErrorXML);
                        ReturnErrors.Append(ErrorXML);
                        IsAnyError      = true;
                        break;
                    }
                    else if (ErrorXML != null)
                    {
                        ErrorXML        = prepareErrorXML(OperationNode, ErrorXML);
                        ReturnErrors.Append(ErrorXML);
                        IsAnyError      = true;
                    }
                    
                    /* It might be the case the ACCESSCONTROLCHECKREQUIRED is not specified to all operations. So set 
                     * IsAccessControlCheckRequired to true to perform access control check for every operation. If the value is specified
                     * in input xml, then it would be overwritten before calling execute operation.
                    * */
                    DataMangerObj.DataProvider.IsAccessControlCheckRequired = true;

                }//End For 
                if (IsAnyError == true)
                {
                    ReturnErrors.Append("</ERRORCOLLECTION>");
                    return ReturnErrors.ToString();
                }
                else
                    return null;

            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }
        #endregion

        #region Transactionless operations
        private String executeTransactionlessOperation(ref XmlNodeList OperationNodes, bool bTracing,
                            bool bTransaction, ref String OutputXML)
        {
            XmlDocument         ErrorXMLDocument            = null;
            XmlNode             ObjectNameNode              = null;
            XmlNode             MethodNameNode              = null;
            XmlNodeList         ParamNodes                  = null;
            XmlNodeList         ErrorNodes                  = null;
            String              ObjectName                  = null;
            String              MethodName                  = null;
            String              ErrorXML                    = null;
            StringBuilder       ReturnErrors                = null;
            XmlNodeList         FileSystemNodes             = null;
            FileSystemManager   FileSystemManagerObj        = null;
            XmlNode             FileSystemNodeByRef         = null;
            XmlNode             TransactionNode             = null;
            XPathDocument       ErrorXMLDoc                 = null;
            XPathNavigator      ErrorXMLDocNav              = null;
            Boolean             ErrorOccured                = false;
            XmlNode             xmlNode_AccessControl       = null; //Used to represent ACCESSCONTROLCHECKREQUIRED node in input xml. This node specifies whether access control check is required for CreateData/UpdateData.
            OMError             obj_OMError                 = null; //Used to prepare error
            StringBuilder       OperationErrorXml           = null;
            ulong               TransactionId               = 0;
            Boolean             bAbortFileSystemTransaction         = false;
            Boolean             bDBOperationsCompleted      = false;
            
            try
            {
                ReturnErrors    = new StringBuilder();
                ReturnErrors.Append("<ERRORCOLLECTION>");
                foreach (XmlNode OperationNode in OperationNodes)
                {
                    Boolean DoExecute                   = true;
                    String  OrginalOperationInputXml    = null;
                    int     IterationCount              = 0;

                    OrginalOperationInputXml            = OperationNode.InnerXml;

                    while (DoExecute)
                    {
                        OperationNode.InnerXml  = OrginalOperationInputXml;
                        OperationErrorXml   = new StringBuilder(1024);
                        ErrorXML            = null;
                        FileSystemNodes     = null;
                        ErrorOccured        = false;
                        obj_OMError         = null;
                        //Read the whether operation is in transaction or not
                        TransactionNode     = OperationNode.SelectSingleNode("Transaction");

                        if (TransactionNode == null) //Case Sensitive
                            TransactionNode = OperationNode.SelectSingleNode("TRANSACTION");

                        if (TransactionNode != null)
                            bTransaction = Convert.ToBoolean(TransactionNode.InnerText);
                        else
                        {
                            //If TRANSACTION is not specified for operation node, then TRANSACTION=TRUE will be considered as default 
                            bTransaction = true;
                        }
                        IsOperationInTransaction = bTransaction;

                        using (TransactionScope Scope = new TransactionScope(TransactionScopeOption.RequiresNew, new TimeSpan(1, 0, 0)))
                        {
                            try
                            {
                                //Initialize the DataManager Object
                                OMDataManager = new DataManager(TenantName, bTransaction);
                                FileSystemManagerObj = null;
                                FileSystemManagerObj = new FileSystemManager(UserID, TenantName, true, OMDataManager);
                                OMDataManager.FileSystemManager = FileSystemManagerObj;
                                OMDataManager.XMLRepository.Init(OMDataManager);
                                //Set the Tracing attribute of DataManager
                                OMDataManager.SetTracing(bTracing);

                                FileSystemNodes = OperationNode.SelectNodes("PARAM/descendant::FILESYSTEM");
                                if (FileSystemNodes == null)
                                    FileSystemNodes = OperationNode.SelectNodes("Param/descendant::FileSystem");
                                if (FileSystemNodes.Count > 0)
                                {
                                    try
                                    {
                                        //A separate instance is not required for each FILESYSTEM node. So single instance is used.
                                        foreach (XmlNode FileSystemNode in FileSystemNodes)
                                        {
                                            //Initialize the FileSystem Object
                                            FileSystemNodeByRef = FileSystemNode;
                                            ErrorXML = ProcessFileSystemNode(FileSystemManagerObj, ref FileSystemNodeByRef);
                                            
                                            if (!String.IsNullOrEmpty(ErrorXML))
                                            {
                                                ErrorXML = prepareErrorXML(OperationNode, ErrorXML);
                                                OperationErrorXml.Append(ErrorXML);
                                                break;
                                            }
                                        }
                                    }
                                    catch
                                    {
                                        //Additional handling is required for exceptions.
                                        throw;
                                    }
                                }
                                if (ErrorXML == null)
                                {
                                    //---Read the ObjectName and Method combination
                                    ObjectNameNode = OperationNode.SelectSingleNode("ObjectName");

                                    if (ObjectNameNode == null)
                                        ObjectNameNode = OperationNode.SelectSingleNode("OBJECTNAME");

                                    if (ObjectNameNode != null)
                                        ObjectName = ObjectNameNode.InnerText;
                                    else
                                        ObjectName = "Unknown";

                                    MethodNameNode = OperationNode.SelectSingleNode("Method");
                                    if (MethodNameNode == null)
                                        MethodNameNode = OperationNode.SelectSingleNode("METHOD");

                                    if (MethodNameNode != null)
                                        MethodName = MethodNameNode.InnerText;
                                    else
                                        MethodName = "Unknown";

                                    //Read all the Parameter Nodes
                                    ParamNodes = OperationNode.SelectNodes("Param");

                                    if (ParamNodes.Count == 0)
                                        ParamNodes = OperationNode.SelectNodes("PARAM");

                                    /* Check whether access control check is to be suppressed. This should be specified only for CreateData and UpdateData.
                                     * Access control check can be suppressed only for ENTITY object operations like CREATEDATA and UPDATEDATA.*/
                                    if (ObjectName.ToUpper().CompareTo("ENTITY") == 0 &&
                                        (MethodName.ToUpper().CompareTo("CREATEDATA") == 0 || MethodName.ToUpper().CompareTo("UPDATEDATA") == 0))
                                    {
                                        xmlNode_AccessControl = OperationNode.SelectSingleNode("ACCESSCONTROLCHECKREQUIRED");
                                        if (xmlNode_AccessControl != null)
                                            OMDataManager.DataProvider.IsAccessControlCheckRequired = Convert.ToBoolean(xmlNode_AccessControl.InnerText);
                                    }

                                    ErrorXML = executeOperations(OMDataManager, ObjectName, MethodName, ref ParamNodes, ref OutputXML);

                                    if (ErrorXML != null)
                                    {
                                        ErrorXML = prepareErrorXML(OperationNode, ErrorXML);
                                        OperationErrorXml.Append(ErrorXML);
                                    }

                                    /* It might be the case the ACCESSCONTROLCHECKREQUIRED is not specified to all operations. So set 
                                     * IsAccessControlCheckRequired to true to perform access control check for every operation. If the value is specified
                                     * in input xml, then it would be overwritten before calling execute operation.
                                     * */
                                    OMDataManager.DataProvider.IsAccessControlCheckRequired = true;
                                }

                                //Anand : This code has been added so that if warning is returned while performing an operation
                                //changes should not be rolled back.
                                if (!String.IsNullOrEmpty(ErrorXML))
                                {
                                    ErrorXMLDoc = new XPathDocument(new StringReader(ErrorXML));
                                    ErrorXMLDocNav = ErrorXMLDoc.CreateNavigator();
                                    if (ErrorXMLDocNav.SelectSingleNode("ERROR/ERRORTYPE[text()='ERROR' or text()='EXCEPTION']") == null)
                                    {
                                        if (IsOperationInTransaction)
                                        {
                                            try
                                            {
                                                //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Calling Database Commit in executeTransactionlessOperation() method with TranscationId :" + TransactionId.ToString() + @"""");
                                                OMDataManager.DataProvider.TransactionObj.Commit();
                                                Scope.Complete();
                                                //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Database Commit Successful in executeTransactionlessOperation() method with TranscationId :" + TransactionId.ToString() + @"""");
                                                bDBOperationsCompleted = true;
                                            }
                                            catch (Exception ex)
                                            {
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in executeTransactionlessOperation() while calling Scope.Complete():" + ex.Message.Replace(@"""", @"""""") + @"""");
                                                // COMMIT FS OPERATIONS IF ZOBIE EXCEPTION
                                                if (ex.Message.Contains("This SqlTransaction has completed; it is no longer usable"))
                                                {
                                                    if (FileSystemManagerObj != null)
                                                    {
                                                        obj_OMError = FileSystemManagerObj.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                        obj_OMError = OMDataManager.XMLRepository.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                        bAbortFileSystemTransaction = true;
                                                    }
                                                }
                                                else
                                                    throw ex;
                                            }
                                        }
                                        else
                                        {
                                            try
                                            {
                                                //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Calling Database Commit (Scope.Complete() only) in executeTransactionlessOperation() method method with TranscationId :" + TransactionId.ToString() + @"""");
                                                Scope.Complete();
                                                //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Database Commit (Scope.Complete() only) Successful in executeTransactionlessOperation() method method with TranscationId :" + TransactionId.ToString() + @"""");
                                                bDBOperationsCompleted = true;
                                            }
                                            catch (Exception ex)
                                            {
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in executeTransactionlessOperation() while calling Scope.Complete() in else:" + ex.Message.Replace(@"""", @"""""") + @"""");
                                                // COMMIT FS OPERATIONS IF ZOBIE EXCEPTION
                                                if (ex.Message.Contains("This SqlTransaction has completed; it is no longer usable"))
                                                {
                                                    if (FileSystemManagerObj != null)
                                                    {
                                                        obj_OMError = FileSystemManagerObj.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                        obj_OMError = OMDataManager.XMLRepository.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                        bAbortFileSystemTransaction = true;
                                                    }
                                                }
                                                else
                                                    throw ex;
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    if (IsOperationInTransaction)
                                    {
                                        try
                                        {
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Calling Database Commit in executeTransactionlessOperation() methodmethod with TranscationId :" + TransactionId.ToString() + @"""");
                                            OMDataManager.DataProvider.TransactionObj.Commit();
                                            Scope.Complete();
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Database Commit Successful in executeTransactionlessOperation() methodmethod with TranscationId :" + TransactionId.ToString() + @"""");
                                            bDBOperationsCompleted = true;
                                        }
                                        catch (Exception ex)
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in executeTransactionlessOperation() while calling Scope.Complete():" + ex.Message.Replace(@"""", @"""""") + @"""");
                                            // COMMIT FS OPERATIONS IF ZOBIE EXCEPTION
                                            if (ex.Message.Contains("This SqlTransaction has completed; it is no longer usable"))
                                            {
                                                if (FileSystemManagerObj != null)
                                                {
                                                    obj_OMError = FileSystemManagerObj.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    obj_OMError = OMDataManager.XMLRepository.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    bAbortFileSystemTransaction = true;
                                                }
                                            }
                                            else
                                                throw ex;
                                        }
                                    }
                                    else
                                    {
                                        try
                                        {
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Calling Database Commit (Scope.Complete() only) in executeTransactionlessOperation() methodmethod with TranscationId :" + TransactionId.ToString() + @"""");
                                            Scope.Complete();
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Database Commit (Scope.Complete() only) Successful in executeTransactionlessOperation() methodmethod with TranscationId :" + TransactionId.ToString() + @"""");
                                            bDBOperationsCompleted = true;
                                        }
                                        catch (Exception ex)
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in executeTransactionlessOperation() while calling Scope.Complete() in else:" + ex.Message.Replace(@"""", @"""""") + @"""");
                                            // COMMIT FS OPERATIONS IF ZOBIE EXCEPTION
                                            if (ex.Message.Contains("This SqlTransaction has completed; it is no longer usable"))
                                            {
                                                if (FileSystemManagerObj != null)
                                                {
                                                    obj_OMError = FileSystemManagerObj.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    obj_OMError = OMDataManager.XMLRepository.HandleInDoubtTransactions("ZOMBIE EXCEPTION");
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""HandleInDoubtTransactions for XML returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    bAbortFileSystemTransaction = true;
                                                }
                                            }
                                            else
                                                throw ex;
                                        }
                                    }
                                }

                                if (!String.IsNullOrEmpty(ErrorXML))
                                {
                                    TransactionId = 0;
                                    ErrorXMLDoc = new XPathDocument(new StringReader(ErrorXML));
                                    ErrorXMLDocNav = ErrorXMLDoc.CreateNavigator();
                                    if (ErrorXMLDocNav.SelectSingleNode("ERROR/ERRORTYPE[text()='ERROR' or text()='EXCEPTION']") != null)
                                    {
                                        ErrorOccured = true;

                                        XmlDocument xDocOMError = new XmlDocument();
                                        xDocOMError.LoadXml(ErrorXML);
                                        XmlNode xNodeTranscation = xDocOMError.SelectSingleNode("//ERROR/TRANSACTIONID");
                                        if (xNodeTranscation != null)
                                        {
                                            if (!string.IsNullOrEmpty(xNodeTranscation.InnerText))
                                            {
                                                TransactionId = ulong.Parse(xNodeTranscation.InnerText);
                                            }
                                        }

                                        try
                                        {
                                            //Rollback File System Operations if there is an error or exception. This will not be done for warnings.
                                            //if (FileSystemNodes.Count > 0 && FileSystemManagerObj != null)
                                            if (FileSystemManagerObj != null)
                                            {
                                                if (TransactionId != 0)
                                                {
                                                    if (FileSystemManagerObj.TranscationId == TransactionId)
                                                    {
                                                        obj_OMError = FileSystemManagerObj.RollbackFileSystemOperations();
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                        obj_OMError = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                        if (obj_OMError != null)
                                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    }
                                                    else
                                                    {
                                                        FileSystemManagerObj.RollbackFileSystemOperations();
                                                        OMDataManager.XMLRepository.RollbackXMLOperations();

                                                        FileSystemManagerObj.TranscationId = TransactionId;
                                                        OMDataManager.XMLRepository.TranscationId = TransactionId;

                                                        FileSystemManagerObj.RollbackFileSystemOperations();
                                                        OMDataManager.XMLRepository.RollbackXMLOperations();
                                                    }
                                                }
                                                else
                                                {
                                                    obj_OMError = FileSystemManagerObj.RollbackFileSystemOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    obj_OMError = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                }
                                                //Handling required for error returned while rolling back operations.
                                            }
                                        }
                                        catch
                                        {
                                            //Additional handling is required for exceptions.
                                            throw;
                                        }
                                    }
                                    else // WARNING after executing DB operations
                                    {
                                        try
                                        {
                                            if (FileSystemManagerObj != null && !bAbortFileSystemTransaction)
                                            {
                                                if (bDBOperationsCompleted)
                                                {
                                                    obj_OMError = FileSystemManagerObj.CommitFileSystemOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    obj_OMError = OMDataManager.XMLRepository.CommitXMLOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    RemoveCahcedItems();
                                                }
                                                else
                                                {
                                                    obj_OMError = FileSystemManagerObj.RollbackFileSystemOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                    obj_OMError = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                    if (obj_OMError != null)
                                                        WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");

                                                }
                                            }
                                        }
                                        catch (Exception ex)
                                        {
                                            //Additional handling is required for exceptions.
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in CommitFileSystemOperations:" + ex.Message.Replace(@"""", @"""""") + @"""");
                                            throw;
                                        }
                                    }
                                }
                                else
                                {
                                    if (FileSystemManagerObj != null && !bAbortFileSystemTransaction)
                                    {
                                        try
                                        {
                                            if (bDBOperationsCompleted)
                                            {
                                                obj_OMError = FileSystemManagerObj.CommitFileSystemOperations();
                                                if (obj_OMError != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                obj_OMError = OMDataManager.XMLRepository.CommitXMLOperations();
                                                if (obj_OMError != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                RemoveCahcedItems();
                                            }
                                            else
                                            {
                                                obj_OMError = FileSystemManagerObj.RollbackFileSystemOperations();
                                                if (obj_OMError != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                                obj_OMError = OMDataManager.XMLRepository.RollbackXMLOperations();
                                                if (obj_OMError != null)
                                                    WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                            }
                                        }
                                        catch (Exception ex)
                                        {
                                            //Additional handling is required for exceptions.
                                            //WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""Exception in FileSystemOperations commit or rollback:" + ex.Message.Replace(@"""", @"""""") + @"""");
                                            throw;
                                        }
                                    }
                                }

                                // If error was encountered in Commit or Rollback of file operations, add it to error collection
                                if (obj_OMError != null)
                                {
                                    String TempErrorXML = null;
                                    TempErrorXML = prepareErrorXML(OperationNode, obj_OMError.Serialize());
                                    OperationErrorXml.Append(TempErrorXML);
                                }

                                //if (ErrorXML != null)
                                //{
                                //    ErrorXML = prepareErrorXML(OperationNode, ErrorXML);
                                //    OperationErrorXml.Append(ErrorXML);
                                //}

                                /* Dispose data tables created for deferred query execution (if applicable)
                                 * Deferred query execution will be applicable only for CreateData and UpdateData methods for Entity.
                                 */
                                ErrorXML = OMDataManager.DataProvider.DisposeDeferredQueryDataTablesIfAny();
                                if (ErrorXML != null)
                                {
                                    ErrorXML = prepareErrorXML(OperationNode, ErrorXML);
                                    OperationErrorXml.Append(ErrorXML);
                                }
                            }
                            catch (Exception ExcepObj)
                            {
                                obj_OMError = new OMError(OMErrorCodes.OMCLASS_EXECUTE_FAILED, "Error in Executing Operation",
                                            OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerClass",
                                            OMErrorType.EXCEPTION, ExcepObj, null);
                                ErrorXML = obj_OMError.Serialize();
                                OperationErrorXml.Append(ErrorXML);

                                if (FileSystemManagerObj != null)
                                {
                                    TransactionId = 0;
                                    XmlDocument xDocOMError = new XmlDocument();
                                    xDocOMError.LoadXml(ErrorXML);
                                    XmlNode xNodeTranscation = xDocOMError.SelectSingleNode("//ERROR/TRANSACTIONID");
                                    if (xNodeTranscation != null)
                                    {
                                        if (!string.IsNullOrEmpty(xNodeTranscation.InnerText))
                                        {
                                            TransactionId = ulong.Parse(xNodeTranscation.InnerText);
                                        }
                                    }

                                    try
                                    {
                                        if (bDBOperationsCompleted)
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitFileSystemOperations called with in catch block of Execute_OM() with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                            obj_OMError = FileSystemManagerObj.CommitFileSystemOperations();
                                            if (obj_OMError != null)
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                            obj_OMError = OMDataManager.XMLRepository.CommitXMLOperations();
                                            if (obj_OMError != null)
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""CommitXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                            RemoveCahcedItems();
                                        }
                                        else
                                        {
                                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations called with in catch block of Execute_OM() with ErrorXML:" + ErrorXML.Replace(@"""", @"""""") + @"""");
                                            obj_OMError = FileSystemManagerObj.RollbackFileSystemOperations();
                                            if (obj_OMError != null)
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackFileSystemOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                            obj_OMError = OMDataManager.XMLRepository.RollbackXMLOperations();
                                            if (obj_OMError != null)
                                                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""executeTransactionlessOperation()"",""RollbackXMLOperations returned ErrorXML:" + obj_OMError.ErrorXML.Replace(@"""", @"""""") + @"""");
                                        }
                                    }
                                    catch
                                    {
                                        if (bDBOperationsCompleted)
                                        {
                                            FileSystemManagerObj.HandleInDoubtTransactions("ERROR while calling CommitFileSystemOperations()");
                                            OMDataManager.XMLRepository.HandleInDoubtTransactions("ERROR while calling CommitFileSystemOperations()");
                                        }
                                        else
                                        {
                                            FileSystemManagerObj.HandleInDoubtTransactions("ERROR while calling RollbackFileSystemOperations()");
                                            OMDataManager.XMLRepository.HandleInDoubtTransactions("ERROR while calling RollbackFileSystemOperations()");
                                        }
                                    }
                                }
                                else
                                {
                                    if (TransactionId != 0)
                                    {
                                        FileSystemManager FileSystemManagerObjEx = new FileSystemManager(UserID, TenantName, true, OMDataManager);
                                        FileSystemManagerObjEx.TranscationId = TransactionId;
                                        FileSystemManagerObjEx.RollbackFileSystemOperations();
                                    }
                                }
                            }
                            finally
                            {
                                //Close database connection
                                if (OMDataManager != null)
                                    OMDataManager.DataProvider.Close();

                                //Close database connection used by File System Manager
                                if (FileSystemManagerObj != null)
                                    FileSystemManagerObj.CloseDatabaseConnection();

                                //Can Retry
                                DoExecute = DataManager.CanRetry(OperationErrorXml.ToString(), RetryCount, ref IterationCount, RetrySleepDuration);
                            }
                        }
                    } // While
                } // End For 

                ReturnErrors.Append(OperationErrorXml);
                ReturnErrors.Append("</ERRORCOLLECTION>");

                //Loading the Error XML to identify whether there is any error or not
                ErrorXMLDocument = new XmlDocument();
                ErrorXMLDocument.LoadXml(ReturnErrors.ToString());

                ErrorNodes = ErrorXMLDocument.SelectNodes("ERRORCOLLECTION/ERROR");
                if (ErrorNodes.Count > 0)
                    return ErrorXMLDocument.InnerXml;
                else
                    return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            finally
            {
                //if (OMDataManager != null)
                //    OMDataManager.DataProvider.Close();
            }
        }

       
        private String executeReadOperation(XmlNode OperationNode, bool bTracing,
                                            bool bTransaction, ref String OutputXML)
        {
            
            XmlNode             ObjectNameNode      = null;
            XmlNode             MethodNameNode      = null;
            XmlNodeList         ParamNodes          = null;
            
            String              ObjectName          = null;
            String              MethodName          = null;
            String              ErrorXML            = null;
            StringBuilder       ReturnErrors        = null;
            
            bool                IsAnyError          = false;

            try
            {
                ReturnErrors = new StringBuilder();
                ReturnErrors.Append("<ERRORCOLLECTION>");
                
                //Initialize the DataManager Object
                OMDataManager = new DataManager(TenantName, false);
                
                //---Read the ObjectName and Method combination
                ObjectNameNode = OperationNode.SelectSingleNode("ObjectName");

                if (ObjectNameNode == null)
                    ObjectNameNode = OperationNode.SelectSingleNode("OBJECTNAME");

                if (ObjectNameNode != null)
                    ObjectName = ObjectNameNode.InnerText;
                else
                    ObjectName = "Unknown";

                MethodNameNode = OperationNode.SelectSingleNode("Method");
                if (MethodNameNode == null)
                    MethodNameNode = OperationNode.SelectSingleNode("METHOD");

                if (MethodNameNode != null)
                    MethodName = MethodNameNode.InnerText;
                else
                    MethodName = "Unknown";

                //Read all the Parameter Nodes
                ParamNodes = OperationNode.SelectNodes("Param");

                if (ParamNodes.Count == 0)
                    ParamNodes = OperationNode.SelectNodes("PARAM");

                ErrorXML = executeOperations(OMDataManager, ObjectName, MethodName, ref ParamNodes, ref OutputXML);

                if (ErrorXML != null)
                {
                    ErrorXML = prepareErrorXML(OperationNode, ErrorXML);
                    ReturnErrors.Append(ErrorXML);
                    IsAnyError = true;
                }

                ReturnErrors.Append("</ERRORCOLLECTION>");


                if (IsAnyError == true)
                    return ReturnErrors.ToString();
                else
                    return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            finally
            {
                if (OMDataManager != null)
                    OMDataManager.DataProvider.Close();
            }
        }
        #endregion
        #region Private Methods
        private String prepareErrorXML(XmlNode OperNode, String ErrorXML)
        {
            XmlDocument     ErrorXMLDoc = null;
            XmlNode         ErrorNode   = null;
            XmlNode         DupOperNode = null;
            
            
            ErrorXMLDoc = new XmlDocument();
            ErrorXMLDoc.LoadXml(ErrorXML);

            ErrorNode   = ErrorXMLDoc.SelectSingleNode("ERROR");
            DupOperNode = ErrorXMLDoc.ImportNode(OperNode, true);
            ErrorNode.AppendChild(DupOperNode);

            return ErrorXMLDoc.OuterXml;
        }
        #endregion

        #region Executing the entity operations
        /// <summary>
        /// <CreatedDate>24 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the Entity Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the Entity object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns></returns>
        private OMError executeEntityOperations(DataManager DataMangerObj, String MethodName, 
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            Entity              EntityObj       = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              EntityTypeName  = null;
            ulong               EntityID        = 0;
            ulong               EntityTypeID    = 0;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj = new Entity(UserID,CurrentLanguage,TenantName, DataMangerObj);
                        ErrorObj  = EntityObj.Create(ref InputXML);
                        break;

                    case "CREATEHIERARCHY":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj = new Entity(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityObj.CreateHierarchy(ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "CREATEDATA":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj = new Entity(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityObj.CreateAndUpdateData(true, ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj = new Entity(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj  = EntityObj.Update(InputXML);
                        break;

                    case "UPDATEDATA":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj = new Entity(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityObj.CreateAndUpdateData(false, ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "BATCHUPDATE":
                        //Extract the parameters
                        InputXML    = ParamArray["INPUTXML"];

                        //Call the method
                        EntityObj   = new Entity(UserID, CurrentLanguage, TenantName,IsOperationInTransaction, DataMangerObj);
                        ErrorObj    = EntityObj.BatchUpdate(ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "DELETE":
                        //Extract the parameters
                        EntityID        = Convert.ToUInt64(ParamArray["ENTITYID"]);
                        EntityTypeID    = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);
                        EntityTypeName  = ParamArray["ENTITYTYPENAME"];
                    
                        //Call the method
                        EntityObj = new Entity(UserID, CurrentLanguage, TenantName, DataMangerObj);

                        if (EntityTypeName != null)
                            ErrorObj = EntityObj.Delete(EntityID, EntityTypeName);
                        else if (EntityTypeID > 0)
                            ErrorObj = EntityObj.Delete(EntityID, EntityTypeID);
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        }

        #endregion

        #region Executing Facet Operations
        /// <summary>
        /// <CreatedDate>24 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the Facet Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the Entity object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns></returns>
        private OMError executeFacetOperations(DataManager DataMangerObj, String MethodName, 
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            Facet               FacetObj        = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              FacetTypeName   = null;
            ulong               FacetID         = 0;
            ulong               FacetTypeID     = 0;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName)
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetObj.Create(ref InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetObj.Update(InputXML);
                        break;

                    case "BATCHUPDATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName,IsOperationInTransaction, DataMangerObj);
                        ErrorObj = FacetObj.BatchUpdate(ref InputXML);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        FacetID         = Convert.ToUInt64(ParamArray["FACETID"]);
                        FacetTypeID     = Convert.ToUInt64(ParamArray["FACETTYPEID"]);
                        FacetTypeName   = ParamArray["FACETTYPENAME"];
                       
                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        if (FacetTypeName != null)
                            ErrorObj = FacetObj.Delete(FacetID, FacetTypeName);
                        else if (FacetTypeID > 0)
                            ErrorObj = FacetObj.Delete(FacetID, FacetTypeID);
                        break;

                    case "CREATEDATA":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetObj.CreateAndUpdateData(true, ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;
                 
                    case "UPDATEDATA":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetObj = new Facet(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetObj.CreateAndUpdateData(false, ref InputXML);
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = InputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        
}
        #endregion

        #region Executing the operations on Assemblies or Facets
        private OMError executeRelationOperations(DataManager DataMangerObj, String MethodName, 
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            
            StringDictionary    ParamArray      = null;
            RelationshipType    ETFTAssocObj    = null;
            EFAssociation       EFAssocObj      = null;
            Assembly            AssmObj         = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              ObjectType      = null;
            String              RelationshipName= null;
            ulong               ContainerTypeID = 0;
            ulong               ContainsTypeID  = 0;
            ulong               ObjectID        = 0;
            ulong               EntityID        = 0;
            ulong               FacetID         = 0;
            ulong               ContainerID     = 0;
            ulong               ContainsID      = 0;
            ulong               ObjectTypeID    = 0;
            String              TypeName        = null;
            String              HierarchyLevel  = null;
            RelationshipType    RelationShipObj = null;
            String              RelationName    = null;
            String              ObjectTypeName  = null;
            bool                IsContainer     = false;
            bool                IsContains      = false;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    //Associate EntityType with a FacetType
                    case "ASSOCIATETYPE":
                        InputXML     = ParamArray["INPUTXML"];
                        
                        ETFTAssocObj = new RelationshipType(UserID,CurrentLanguage, TenantName,OMDataManager);
                        ErrorObj     = ETFTAssocObj.Associate(InputXML);
                        break;

                    //Associate Entity with a Facet
                    case "ASSOCIATEENTITYFACET":
                        InputXML   = ParamArray["INPUTXML"];
                       
                        EFAssocObj = new EFAssociation(UserID, CurrentLanguage, TenantName,OMDataManager);
                        ErrorObj   = EFAssocObj.Associate(InputXML);
                        break;

                    //Associate an instance of entity with an entity or
                    //associate a facet with a facet.
                    case "ASSOCIATEINSTANCE":
                        InputXML = ParamArray["INPUTXML"];
                        
                        AssmObj  = new Assembly(UserID,CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = AssmObj.Associate(InputXML);
                        break;

                    //DisAssociate EntityType and FacetType
                    case "DISASSOCIATETYPE":
                        ContainerTypeID = Convert.ToUInt64(ParamArray["CONTAINERTYPEID"]);
                        ContainsTypeID  = Convert.ToUInt64(ParamArray["CONTAINSTYPEID"]);
                        
                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName,OMDataManager);
                        ErrorObj     = ETFTAssocObj.Delete(ContainerTypeID, ContainsTypeID);
                        break;

                    //Hemant Karkande 17 June 2006
                    //DisAssociate EntityType and FacetType by RelationshipName
                    case "DISASSOCIATETYPEBYRELATIONSHIPNAME":
                        RelationshipName = ParamArray["RELATIONSHIPNAME"];

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.DeleteByRelationShipName(RelationshipName);
                        break;

                    //Disassociate entity and facet
                    case "DISASSOCIATEENTITYFACET":
                        EntityID = Convert.ToUInt64(ParamArray["ENTITYID"]);
                        FacetID  = Convert.ToUInt64(ParamArray["FACETID"]);
                        
                        EFAssocObj = new EFAssociation(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj   = EFAssocObj.Disassociate(EntityID, FacetID);
                        break;

                    //Disassociate instance of entity with entity or facet or facet
                    case "DISASSOCIATEINSTANCE":
                        ContainerID = Convert.ToUInt64(ParamArray["CONTAINERID"]);
                        ContainsID  = Convert.ToUInt64(ParamArray["CONTAINSID"]);
                       
                        AssmObj  = new Assembly(UserID, CurrentLanguage, TenantName,OMDataManager);
                        ErrorObj = AssmObj.Disassociate(ContainerID, ContainsID);
                        break;
                    case "DISASSOCIATEINSTANCEEX":
                        ContainerID = Convert.ToUInt64(ParamArray["CONTAINERID"]);
                        ContainsID = Convert.ToUInt64(ParamArray["CONTAINSID"]);
                        RelationName = ParamArray["RELATIONSHIPNAME"];
 
                        AssmObj = new Assembly(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = AssmObj.DisassociateEx(ContainerID, ContainsID, RelationName);    
                        break;

                    case "DISASSOCIATEALLINSTANCES":
                        ObjectID    = Convert.ToUInt64(ParamArray["OBJECTID"]);
                        ObjectType  = ParamArray["OBJECTTYPE"];
                      
                        AssmObj = new Assembly(UserID, CurrentLanguage, TenantName,OMDataManager);

                        if (ObjectType.ToUpper() == "ENTITY")
                            ErrorObj = AssmObj.DisassociateEntities(ObjectID);
                        else
                            ErrorObj = AssmObj.DisassociateFacets(ObjectID);
                        break;

                    //Sagar Dasalkar 12 Oct 2006
                    case "DISASSOCIATEALLINSTANCESEX":
                        ObjectID = Convert.ToUInt64(ParamArray["OBJECTID"]);
                        ObjectType = ParamArray["OBJECTTYPE"];
                        IsContainer = Convert.ToBoolean(ParamArray["ISCONTAINER"]);
                        IsContains = Convert.ToBoolean(ParamArray["ISCONTAINS"]);
                        RelationshipName = ParamArray["RELATIONSHIPNAME"];

                        AssmObj = new Assembly(UserID, CurrentLanguage, TenantName, OMDataManager);

                        ErrorObj = AssmObj.DisassociateAllInstancesEx(ObjectID,ObjectType,RelationshipName,IsContainer,IsContains);
                        break;

                    case "READASSOCIATEDFACETTYPE":
                        ObjectTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadAllByContainerType(ObjectTypeID, "ENTITY", "FACET", ref OutputXML);
                        break;

                    case "READASSOCIATEDFACETTYPEBYNAME":
                        ObjectTypeName = ParamArray["ENTITYTYPENAME"];

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadAllByContainerType(ObjectTypeName, "ENTITY", "FACET", ref OutputXML);
                        break;

                    case "READASSOCIATEDENTITYTYPE":
                        ObjectTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadAllByContainerType(ObjectTypeID, "ENTITY", "ENTITY", ref OutputXML);
                        break;

                    case "READASSOCIATEDENTITYTYPEEX":
                        ObjectTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadAllAncestorsandChildren(ObjectTypeID,"ENTITY", ref OutputXML);
                        break;

                    case "READRELATIONSHIPBYNAME":
                        RelationshipName = ParamArray["RELATIONSHIPNAME"];

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadByRelationShipName(RelationshipName, ref OutputXML);
                        break;

                    case "READTYPEHIERARCHY":
                        TypeName        = ParamArray["TYPENAME"];
                        ObjectType      = ParamArray["OBJECTTYPE"];
                        HierarchyLevel  = ParamArray["HIERARCHYLEVEL"];
                      
                        RelationShipObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = RelationShipObj.ReadHierarchy(TypeName,ObjectType, HierarchyLevel, ref OutputXML);
                        break;

                    case "READALLENTITYRELATIONSHIP":
                        TypeName    = ParamArray["TYPENAME"];
                        ObjectType  = ParamArray["OBJECTTYPE"];
                        
                        RelationShipObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj        = RelationShipObj.ReadAllByContainerType(TypeName, ObjectType, null, ref OutputXML);
                        break;

                    case "READALLASSOCIATEDFACETTYPE":
                        ObjectTypeID = Convert.ToUInt64(ParamArray["FACETTYPEID"]);

                        ETFTAssocObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = ETFTAssocObj.ReadAllAncestorsandChildren(ObjectTypeID, "FACET", ref OutputXML);
                        break;

                    case "REMOVEASSOCIATIONS":
                        ContainerID = Convert.ToUInt64(ParamArray["CONTAINERID"]);
                        ContainsID = Convert.ToUInt64(ParamArray["CONTAINSID"]);
                        RelationName = ParamArray["RELATIONSHIPNAME"];

                        AssmObj = new Assembly(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = AssmObj.RemoveAssociations(ContainerID, ContainsID, RelationName);
                        break;

                    case "UPDATERELATIONSHIPTYPE":
                        InputXML = ParamArray["INPUTXML"];
                        RelationShipObj = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        ErrorObj = RelationShipObj.UpdateRelationshipType(InputXML);
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                //Prepare the OMError Object
                throw ExcepObj;
            }
            return ErrorObj;
        }
        #endregion


        #region Executing entitytype operations
        /// <summary>
        /// <CreatedDate>24 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the EntityType Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the Entity object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>
        private OMError executeEntityTypeOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            EntityType          EntityTypeObj   = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              EntityTypeName  = null;
            ulong               EntityTypeID    = 0;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];
                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityTypeObj.Create(InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityTypeObj.Update(InputXML);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        EntityTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);
                        EntityTypeName = ParamArray["ENTITYTYPENAME"];

                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        if (EntityTypeName != null)
                            ErrorObj = EntityTypeObj.Delete(EntityTypeName);
                        else if (EntityTypeID > 0)
                            ErrorObj = EntityTypeObj.Delete(EntityTypeID);

                        break;

                    case "READBYID":
                        //Extract the parameters
                        EntityTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);
                        
                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityTypeObj.Read(EntityTypeID,ref OutputXML);
                        break;

                    case "READBYNAME":
                        //Extract the parameters
                        EntityTypeName = ParamArray["ENTITYTYPENAME"];

                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityTypeObj.Read(EntityTypeName, ref OutputXML);
                        break;

                    case "READALL":
                        //Call the method
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = EntityTypeObj.ReadAll(ref OutputXML);
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                //Prepare the OMError Object
                throw ExcepObj;
            }
            return ErrorObj;
        }

        #endregion

        #region Executing FacetType Operations
        /// <summary>
        /// <CreatedDate>24 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the FacetType Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the Entity object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>
        private OMError executeFacetTypeOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            FacetType           FacetTypeObj    = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              FacetTypeName   = null;
            ulong               FacetTypeID     = 0;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetTypeObj.Create(InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetTypeObj.Update(InputXML);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        FacetTypeID = Convert.ToUInt64(ParamArray["FACETTYPEID"]);
                        FacetTypeName = ParamArray["FACETTYPENAME"];

                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);

                        if (FacetTypeName != null)
                            ErrorObj = FacetTypeObj.Delete(FacetTypeName);
                        else if (FacetTypeID > 0)
                            ErrorObj = FacetTypeObj.Delete(FacetTypeID);

                        break;

                    case "READBYID":
                        //Extract the parameters
                        FacetTypeID = Convert.ToUInt64(ParamArray["FACETTYPEID"]);

                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetTypeObj.Read(FacetTypeID, ref OutputXML);
                        break;

                    case "READBYNAME":
                        //Extract the parameters
                        FacetTypeName = ParamArray["FACETTYPENAME"];

                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetTypeObj.Read(FacetTypeName, ref OutputXML);
                        break;

                    case "READALL":
                        //Call the method
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = FacetTypeObj.ReadAll(ref OutputXML);
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        }
        #endregion

        #region Execute Operations on AccessRole
        /// <summary>
        /// <CreatedDate>24 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the AccessRole Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the Entity object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        ///     4. DELETEALL
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>
        private OMError executeAccessRoleOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            AccessorRole        AccessorRoleObj     = null;
            StringDictionary    ParamArray          = null;
            OMError             ErrorObj            = null;
            String              InputXML            = null;
            ulong               AccessorRoleID      = 0;
            String              AccessorRoleName    = null;
            ulong               EntityTypeID        = 0;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML        = ParamArray["INPUTXML"];

                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.Create(InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML        = ParamArray["INPUTXML"];

                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.Update(InputXML);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        AccessorRoleID  = Convert.ToUInt64(ParamArray["ACCESSORROLEID"]);

                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.Delete(AccessorRoleID);
                        break;

                    case "DELETEALL":
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.DeleteAll();
                        break;

                    case "READBYID":
                        //Extract the parameters
                        AccessorRoleID = Convert.ToUInt64(ParamArray["ACCESSORROLEID"]);

                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.Read(AccessorRoleID, ref OutputXML);
                        break;

                    case "READBYNAME":
                        //Extract the parameters
                        AccessorRoleName = ParamArray["ACCESSORROLENAME"];

                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.ReadRoleByName(AccessorRoleName, ref OutputXML);
                        break;
                    case "READALLFORENTITYTYPE":
                        //Extract the parameters
                        EntityTypeID    = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);
                             
                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.ReadAllRolesForEntityType(EntityTypeID,ref OutputXML);
                        break;

                    case "READALL":
                        //Call the method
                        AccessorRoleObj = new AccessorRole(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj        = AccessorRoleObj.ReadAll(ref OutputXML);
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        }
        #endregion

        #region Executing AccessRule Operations
        private OMError executeAccessRuleOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            AccessRule          AccessRuleObj   = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            ulong               AccessRuleID    = 0;
            ulong               EntityTypeID    = 0;
            String              AccessRuleName  = null;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.Create(InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML      = ParamArray["INPUTXML"];

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.Update(InputXML);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        AccessRuleID  = Convert.ToUInt64(ParamArray["ACCESSRULEID"]);

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.Delete(AccessRuleID);
                        break;

                    case "DELETEFORENTITYTYPE":
                        //Extract the parameters
                        EntityTypeID  = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.DeleteAllForEntityType(EntityTypeID);

                        break;

                    case "READBYID":
                        //Extract the parameters
                        AccessRuleID  = Convert.ToUInt64(ParamArray["ACCESSRULEID"]);

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.Read(AccessRuleID, ref OutputXML);
                        break;

                    case "READBYNAME":
                        //Extract the parameters
                        AccessRuleName = ParamArray["ACCESSRULENAME"];

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.ReadRuleByName(AccessRuleName, ref OutputXML);
                        break;

                    case "READALLFORENTITYTYPE":
                        //Extract the parameters
                        EntityTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.ReadRuleForEntityType(EntityTypeID, ref OutputXML);
                        break;

                    case "READALL":
                        //Call the method
                        AccessRuleObj = new AccessRule(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj      = AccessRuleObj.ReadAll(ref OutputXML);
                        break;

                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        }
        #endregion


        #region Executing Authorization Operations
        /// <summary>
        /// <CreatedDate>22 Sept 2006</CreatedDate>
        /// <Author>Miss Pranjali Bhide</Author>
        /// <Purpose>
        /// Execute the CUD operation on the Authorization
        /// </Purpose>
        /// </summary>
        /// <param name="DataMangerObj">
        ///  The DataManager object for performing the operations.
        /// </param>       
        /// <param name="MethodName">
        ///  The name of the method on the Authorization. This can have the following values
        ///     1. CREATE        
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns></returns>
        private OMError executeAuthorizationOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            Authorization       AuthorizationObj = null;
            StringDictionary    ParamArray      = null;
            OMError             ErrorObj        = null;
            String              InputXML        = null;
            String              ObjectType      = null; 
            ulong               ObjectID        = 0;

            
            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                        case "CREATE":
                            //Extract the parameters
                            InputXML = ParamArray["INPUTXML"];

                            //Call the method
                            AuthorizationObj = new Authorization(UserID, CurrentLanguage, TenantName, DataMangerObj);
                            ErrorObj         = AuthorizationObj.AddAuthorization(InputXML);  
                            break;
                    
                        case "UPDATE":
                            //Extract the parameters
                            InputXML = ParamArray["INPUTXML"];

                            //Call the method
                            AuthorizationObj = new Authorization(UserID, CurrentLanguage, TenantName, DataMangerObj);
                            ErrorObj         = AuthorizationObj.UpdateAuthorization(InputXML);
                            break;

                        case "REVOKE":
                            //Extract the parameters
                            InputXML = ParamArray["INPUTXML"];

                            //Call the method
                            AuthorizationObj = new Authorization(UserID, CurrentLanguage, TenantName, DataMangerObj);
                            ErrorObj         = AuthorizationObj.RevokeAuthorization(InputXML);  
                            break;

                        case "READEVERYONEPRIVILEGE":
                            
                            ObjectID = Convert.ToUInt64(ParamArray["OBJECTID"]);
                            ObjectType = ParamArray["OBJECTTYPE"];
                            //Call the method

                            AuthorizationObj = new Authorization(UserID, CurrentLanguage, TenantName, DataMangerObj);
                            // AccessorID is hardcoded as 500. 
                            ErrorObj = AuthorizationObj.ReadObjectPrivilege(500, "ROLE", ObjectID,ObjectType, ref OutputXML);   
                        break;

                    case "READALLPRIVILEGE":
                        //Extract the parameters
                        InputXML = ParamArray["INPUTXML"];

                        //Call the method
                        AuthorizationObj = new Authorization(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = AuthorizationObj.ReadAllPrivilege(InputXML, ref OutputXML);
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj; 
            }
            return ErrorObj; 
        }
        #endregion

        #region Executing NamedHierarchy Operations

        /// <summary>
        /// <CreatedDate>25 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Execute the CUD operation on the NamedHierarchy Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the NamedHierarchy object. This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>
        private OMError executeNamedHierarchyOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            NamedHierarchy      NamedHierarchyObj           = null;
            StringDictionary    ParamArray                  = null;
            OMError             ErrorObj                    = null;
            String              InputXML                    = null;
            String              NamedHierarchyName          = null;
            ulong               NamedHierarchyID            = 0;
            bool                bModifyInstances            = false;
            bool                bObsolete                   = false;
            bool                IsOneLevel                  = false; 
            ulong               EntityTypeID                = 0;
            ulong               FacetTypeID                 = 0;
            ulong               ParentNamedHierarchyID      = 0;
            String              NamedHierarchyDesc          = null;
            String              NamedHierarchyValue         = null;
            String              NHDisplayOrder              = null;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters
                        InputXML          = ParamArray["INPUTXML"];
                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.Create(InputXML);
                        break;

                    case "UPDATE":
                        //Extract the parameters
                        InputXML         = ParamArray["INPUTXML"];
                        bModifyInstances = Convert.ToBoolean(ParamArray["MODIFYINSTANCES"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.Update(InputXML, bModifyInstances);
                        break;

                    case "DELETE":
                        //Extract the parameters
                        NamedHierarchyID  = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.Delete(NamedHierarchyID);
                        break;

                    case "READBYID":
                        //Extract the parameters
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        bObsolete        = Convert.ToBoolean(ParamArray["OBSOLETE"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.Read(NamedHierarchyID,bObsolete, ref OutputXML);
                        break;

                    case "READBYNAME":
                        //Extract the parameters
                        NamedHierarchyName = ParamArray["NamedHierarchyName"];
                        bObsolete = Convert.ToBoolean(ParamArray["OBSOLETE"]);
                        IsOneLevel = Convert.ToBoolean(ParamArray["ISONELEVEL"]);
                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = NamedHierarchyObj.Read(NamedHierarchyName, bObsolete, IsOneLevel, ref OutputXML);     
                        break;

                    case "READALLFORFACETTYPE":
                        //Extract the parameters
                        FacetTypeID       = Convert.ToUInt64(ParamArray["FACETTYPEID"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.ReadByObjectTypeID(FacetTypeID, ref OutputXML);
                        break;

                    case "READALLFORENTITYTYPE":
                        //Extract the parameters
                        EntityTypeID      = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.ReadByObjectTypeID(EntityTypeID, ref OutputXML);
                        break;
                    
                    case "READALL":
                        //Extract the parameters
                        bObsolete         = Convert.ToBoolean(ParamArray["OBSOLETE"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.ReadAll(bObsolete, ref OutputXML);
                        break;
                    
                    // Added by Rahul Pawar in order to support NH retrival of only direct children.
                    case "READBYIDWITHONELEVEL":
                        //Extract the parameters
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        bObsolete = Convert.ToBoolean(ParamArray["OBSOLETE"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj          = NamedHierarchyObj.ReadByIDwithOneLevel(NamedHierarchyID,bObsolete,ref OutputXML);
                        break;
                    
                    case "ADDNAMEDHIERARCHYVALUE":
                        //Extract the parameters
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        NamedHierarchyDesc = ParamArray["NAMEDHIERARCHYDESCRIPTION"];
                        NamedHierarchyValue = ParamArray["NAMEDHIERARCHYVALUE"];
                        ParentNamedHierarchyID = Convert.ToUInt64(ParamArray["PARENTNAMEDHIERARCHYID"]);

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = NamedHierarchyObj.AddNamedHierarchyValue(ParentNamedHierarchyID,NamedHierarchyID, NamedHierarchyValue, NamedHierarchyDesc);
                        break;

                    case "ADDNAMEDHIERARCHYVALUEEX":
                        //Extract the parameters
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        NamedHierarchyDesc = ParamArray["NAMEDHIERARCHYDESCRIPTION"];
                        NamedHierarchyValue = ParamArray["NAMEDHIERARCHYVALUE"];
                        ParentNamedHierarchyID = Convert.ToUInt64(ParamArray["PARENTNAMEDHIERARCHYID"]);
                        NHDisplayOrder = ParamArray["DISPLAYORDER"];

                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = NamedHierarchyObj.AddNamedHierarchyValueEx(ParentNamedHierarchyID, NamedHierarchyID, NamedHierarchyValue, NamedHierarchyDesc,NHDisplayOrder);
                        break;

                    case "UPDATENAMEDHIERARCHYVALUE":
                        //Extract the parameters
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        NamedHierarchyDesc = ParamArray["NAMEDHIERARCHYDESCRIPTION"];
                        NamedHierarchyValue = ParamArray["NAMEDHIERARCHYVALUE"];
                        bObsolete         = Convert.ToBoolean(ParamArray["OBSOLETE"]);
                        //Call the method
                        NamedHierarchyObj = new NamedHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = NamedHierarchyObj.UpdateNamedHierarchyValue(NamedHierarchyID, NamedHierarchyValue, NamedHierarchyDesc, bObsolete);
                        break;


                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;

        }
        #endregion

        #region Executing NamedHierarchy Association Operations

        /// <summary>
        /// <CreatedDate>13 February 2006</CreatedDate>
        /// <Author>Mr. Kiran Abhang</Author>
        /// <Purpose>
        /// Execute the CRUD operation on the AssoFieldNamedHierarchy Object
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="MethodName">
        ///  The name of the method on the AssoFieldNamedHierarchy object. 
        ///  This can have the following values
        ///     1. CREATE
        ///     2. UPDATE
        ///     3. DELETE
        ///     4. DELETEALLUSINGNAMEDHIERARCHYID
        ///     5. DELETEALLUSINGOBJECTTYPEID
        ///     6. DELETEUSINGSCHEMAID
        ///     7. READALLBYNAMEDHIERARCHYID
        ///     8. READBYOBECTTYPEID
        ///     9. READBYSCHEMAID
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>
        private OMError executeAssoFieldNamedHierarchyOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            AssoFieldNamedHierarchy AssoNamedHierarchyObj   = null;
            StringDictionary        ParamArray              = null;
            OMError                 ErrorObj                = null;
            ulong                   AssociationID           = 0;
            ulong                   NamedHierarchyID        = 0;
            ulong                   SchemaID                = 0;
            ulong                   EntityTypeID            = 0;
            ulong                   FacetTypeID             = 0;
            int                     NHLevel                 = 0;
            String                  SortOrder               = null;

            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "CREATE":
                        //Extract the parameters.
                        AssociationID       = Convert.ToUInt64(ParamArray["ASSOCIATIONID"]);
                        NamedHierarchyID    = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        SchemaID            = Convert.ToUInt64(ParamArray["SCHEMAID"]);
                        SortOrder           = ParamArray["SORTORDER"];

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage, 
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.Create(AssociationID, NamedHierarchyID, SchemaID, SortOrder);
                        break;

                    case "UPDATE":
                        //Extract the parameters.
                        AssociationID   = Convert.ToUInt64(ParamArray["ASSOCIATIONID"]);
                        SortOrder       = ParamArray["SORTORDER"];

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.Update(AssociationID, SortOrder);
                        break;

                    case "DELETE":
                        //Extract the parameters.
                        AssociationID = Convert.ToUInt64(ParamArray["ASSOCIATIONID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.Delete(AssociationID);
                        break;

                    case "DELETEALLFORNAMEDHIERARCHY":
                        //Extract the parameters.
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.DeleteAllUsingNamedHierarchyID(NamedHierarchyID);
                        break;

                    case "DELETEALLFORENTITYTYPE":
                        //Extract the parameters.
                        EntityTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.DeleteAllUsingObjectTypeID(EntityTypeID);
                        break;

                    case "DELETEALLFORFACETTYPE":
                        //Extract the parameters.
                        FacetTypeID = Convert.ToUInt64(ParamArray["FACETTYPEID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.DeleteAllUsingObjectTypeID(FacetTypeID);
                        break;

                    case "DELETEFORSCHEMA":
                        //Extract the parameters.
                        SchemaID = Convert.ToUInt64(ParamArray["SCHEMAID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.DeleteUsingSchemaID(SchemaID);
                        break;

                    case "READALLFORNAMEDHIERARCHY":
                        //Extract the parameters.
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        
                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.ReadAllByNamedHierarchyID(NamedHierarchyID, ref OutputXML);
                        break;

                    case "READFORENTITYTYPE":
                        //Extract the parameters.
                        EntityTypeID = Convert.ToUInt64(ParamArray["ENTITYTYPEID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.ReadByObectTypeID(EntityTypeID, ref OutputXML);
                        break;

                    case "READFORFACETTYPE":
                        //Extract the parameters.
                        FacetTypeID = Convert.ToUInt64(ParamArray["FACETTYPEID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.ReadByObectTypeID(FacetTypeID, ref OutputXML);
                        break;

                    case "READFORSCHEMA":
                        //Extract the parameters.
                        SchemaID = Convert.ToUInt64(ParamArray["SCHEMAID"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.ReadBySchemaID(SchemaID, ref OutputXML);
                        break;

                    case "CREATEANDUPDATENHLEVEL":
                        //Extract the parameters.
                        AssociationID = Convert.ToUInt64(ParamArray["ASSOCIATIONID"]);
                        NamedHierarchyID = Convert.ToUInt64(ParamArray["NAMEDHIERARCHYID"]);
                        SchemaID = Convert.ToUInt64(ParamArray["SCHEMAID"]);
                        SortOrder = ParamArray["SORTORDER"];
                        NHLevel = Convert.ToInt32(ParamArray["NHLEVEL"]);

                        //Call the method.
                        AssoNamedHierarchyObj = new AssoFieldNamedHierarchy(UserID, CurrentLanguage,
                                                            TenantName, DataMangerObj);
                        ErrorObj = AssoNamedHierarchyObj.CreateAndUpdateNHLevel(AssociationID, NamedHierarchyID, SchemaID, SortOrder,NHLevel);
                        break; 
                        
                    default:
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;
        }

        #endregion

        #region Executing GroupGierarchy Operation
        /// <summary>
        /// This function will handle the GroupHierarchy methods.
        /// The methods are as follows
        /// 1.ADDMEMBERSHIP
        /// 2.REMOVEMEMBERSHIP
        /// 3.DELETEGROUP
        /// 4.ReadAncestorHierarchy
        /// </summary>
        /// <param name="DataMangerObj"></param>
        /// <param name="MethodName"></param>
        /// <param name="ParamNodes"></param>
        /// <param name="OutputXML"></param>
        /// <returns></returns>
        private OMError executeGroupHierarchyOperations(DataManager DataMangerObj, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {
            #region Local Variables
            StringDictionary ParamArray = null;
            OMError ErrorObj = null;
            GroupHierarchy GroupHierarchyObj = null;
            XmlDocument InputXMLDoc = null;
            XmlNode RootNode = null;
            XmlNode ParentList = null;
            XmlNode ChildList = null;
            string InputXML = null;
            ulong GroupID = 0;
            string ParentIDs = null;
            string ChildIDs = null;
            #endregion
            try
            {
                processMethodParam(ParamNodes, ref ParamArray);
                switch (MethodName.ToUpper())
                {
                    case "ADDMEMBERSHIP":
                        InputXML = ParamArray["INPUTXML"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        InputXMLDoc = new XmlDocument();
                        InputXMLDoc.LoadXml(InputXML);
                        RootNode = InputXMLDoc.SelectSingleNode("//GROUP");
                        if (RootNode != null)
                        {
                            GroupID = Convert.ToUInt64(((XmlElement)RootNode.SelectSingleNode("GROUPID")).InnerText);
                            ParentList = RootNode.SelectSingleNode("PARENTCOLLECTION");///PARENTID");
                            ChildList = RootNode.SelectSingleNode("CHILDCOLLECTION");///CHILDID");
                            if (ParentList != null)
                                ParentIDs = ParentList.OuterXml;
                            if (ChildList != null)
                                ChildIDs = ChildList.OuterXml;
                            ErrorObj=GroupHierarchyObj.AddMembership(GroupID, ParentIDs, ChildIDs);
                        }
                        break;
                    case "REMOVEMEMBERSHIP":
                        InputXML = ParamArray["INPUTXML"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        InputXMLDoc = new XmlDocument();
                        InputXMLDoc.LoadXml(InputXML);
                        RootNode = InputXMLDoc.SelectSingleNode("//GROUP");
                        if (RootNode != null)
                        {
                            GroupID = Convert.ToUInt64(((XmlElement)RootNode.SelectSingleNode("GROUPID")).InnerText);
                            ParentList = RootNode.SelectSingleNode("PARENTCOLLECTION");///PARENTID");
                            ChildList = RootNode.SelectSingleNode("CHILDCOLLECTION");///CHILDID");
                            if (ParentList != null)
                                ParentIDs = ParentList.OuterXml;
                            if (ChildList != null)
                                ChildIDs = ChildList.OuterXml;                                                        ///

                            ErrorObj=GroupHierarchyObj.RemoveMembership(GroupID, ParentIDs, ChildIDs);
                        }
                        break;
                    case "DELETEGROUP":
                        InputXML = ParamArray["INPUTXML"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        InputXMLDoc = new XmlDocument();
                        InputXMLDoc.LoadXml(InputXML);
                        RootNode = InputXMLDoc.SelectSingleNode("//GROUPIDLIST");
                        if (RootNode != null)
                        {
                            ErrorObj=GroupHierarchyObj.DeleteGroup(RootNode.OuterXml);
                        }
                        break;
                    case "READANCESTORHIERARCHY":
                        InputXML = ParamArray["INPUTXML"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        InputXMLDoc = new XmlDocument();
                        InputXMLDoc.LoadXml(InputXML);
                        RootNode = InputXMLDoc.SelectSingleNode("//GROUPIDLIST");
                        if (RootNode != null)
                        {
                            ErrorObj = GroupHierarchyObj.ReadAncestorHierarchy(RootNode.OuterXml, ref OutputXML);
                        }
                        if (ErrorObj == null)
                        {
                            ParamArray["INPUTXML"] = OutputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "READDESCENDENTHIERARCHY":                        
                        InputXML = ParamArray["INPUTXML"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        InputXMLDoc = new XmlDocument();
                        InputXMLDoc.LoadXml(InputXML);
                        RootNode = InputXMLDoc.SelectSingleNode("//GROUPIDLIST");
                        if (RootNode != null)
                        { 
                            ErrorObj = GroupHierarchyObj.ReadDescendentHierarchy(RootNode.OuterXml, ref OutputXML);
                        }
                        if (ErrorObj == null)
                        {
                            /*If the Output Xml is null, an empty value will be set as value for key in ParamArray.
                            Since the deProcessMethodParam methods tries to assign the OutputXml as inner xml for ParamArray key and it fails
                            if Output Xml is null.
                            */
                            if (OutputXML == null)
                                ParamArray["INPUTXML"] = "";
                            else
                                ParamArray["INPUTXML"] = OutputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;

                    case "READFIRSTLEVELGROUP":

                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = GroupHierarchyObj.ReadFirstLevelGroup(ref OutputXML);

                        if(ErrorObj==null)
                        {
                            ParamArray["FIRSTLEVELGROUPS"] = OutputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;
                    case "READHIERARCHYGROUPFORUSER":
                        string cUserID = ParamArray["USERID"];
                        GroupHierarchyObj = new GroupHierarchy(UserID, CurrentLanguage, TenantName, DataMangerObj);
                        ErrorObj = GroupHierarchyObj.ReadHierarchyGroupForUser(cUserID, ref OutputXML);

                        if (ErrorObj == null)
                        {
                            ParamArray["GROUPLISTXML"] = OutputXML;
                            deProcessMethodParam(ref ParamNodes, ParamArray);
                        }
                        break;
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
            return ErrorObj;


        }
        #endregion

        #region Execute Operation
        /// <summary>
        /// <CreatedDate>25 August 2005</CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        ///  Router for executing operations on various Core objects.
        /// </Purpose>
        /// </summary>
        /// <param name="DMReader">
        ///  The DataManagerReader object for performing the Read operations.
        /// </param>
        /// <param name="DMWriter">
        /// The DataManagerReader object for performing the Write operations.
        /// </param>
        /// <param name="ObjectName">
        ///  The name of the object. This can have the following values
        ///     1. ENTITY
        ///     2. FACET
        ///     3. ENTITYTYPE
        ///     4. FACETTYPE
        ///     5. RELATION
        ///     6. NAMEDHIERARCHY
        ///     7. ACCESSROLE
        ///     8. ACCESSRULE
        /// </param>
        /// <param name="MethodName">
        ///  The name of the Method on the Object specified by ObjectName. This can have the following values
        ///     1. ENTITY
        ///     2. FACET
        ///     3. ENTITYTYPE
        ///     4. FACETTYPE
        ///     5. RELATION
        ///     6. NAMEDHIERARCHY
        ///     7. ACCESSROLE
        ///     8. ACCESSRULE
        /// </param>
        /// <param name="ParamNodes">
        ///  This is the list of parameter nodes.
        /// </param>
        /// <returns>OMError object</returns>

        /// <returns></returns>
        private String executeOperations(DataManager DataMangerObj, String ObjectName, String MethodName,
                            ref XmlNodeList ParamNodes, ref String OutputXML)
        {

            String  ErrorXML        = null;
            String  ReturnErrors    = null;
            OMError ErrorObj        = null;

            switch (ObjectName.ToUpper())
            {
                case "ENTITY":
                    ErrorObj = executeEntityOperations(DataMangerObj, MethodName, ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;


                case "FACET":
                    ErrorObj = executeFacetOperations(DataMangerObj, MethodName, ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "RELATION":
                    ErrorObj = executeRelationOperations(DataMangerObj, MethodName, ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "ENTITYTYPE":
                    ErrorObj = executeEntityTypeOperations(DataMangerObj, MethodName, ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "FACETTYPE":
                    ErrorObj = executeFacetTypeOperations(DataMangerObj, MethodName, ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "NAMEDHIERARCHY":
                    ErrorObj = executeNamedHierarchyOperations(DataMangerObj, MethodName, 
                                                    ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "NAMEDHIERARCHYASSOCIATION":
                    ErrorObj = executeAssoFieldNamedHierarchyOperations(DataMangerObj, MethodName,
                                                    ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "ACCESSORROLE":
                    ErrorObj = executeAccessRoleOperations(DataMangerObj, MethodName,
                                                    ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                case "ACCESSRULE":
                    ErrorObj = executeAccessRuleOperations(DataMangerObj, MethodName,
                                                    ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;
                //Added by sagar godbole for GroupHierarchy Operations
                //Date 21 Sep 2006
                case "GROUPHIERARCHY":
                    ErrorObj = executeGroupHierarchyOperations(DataMangerObj, MethodName,
                                                    ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                // Add By Pranjali Bhide on 22 Sept 2006 for authorization operations.                    
                case "AUTHORIZATION":
                    ErrorObj = executeAuthorizationOperations(DataMangerObj, MethodName,
                                                   ref ParamNodes, ref OutputXML);

                    if (ErrorObj != null)
                    {
                        ErrorXML = ErrorObj.Serialize();
                        ReturnErrors += ErrorXML;
                    }
                    break;

                default:
                    break;
            }//End Switch

            return ReturnErrors;
        }

        #endregion

        //#region Executing Read operation for Configuration objects
        //private String executeGetTypeData(XmlNodeList OperationNodes, 
        //                                    bool bTracing,
        //                                    ref String OutputXML)
        //{
        //    XmlDocument     ErrorXMLDocument    = null;
        //    XmlNode         ObjectNameNode      = null;
        //    XmlNode         MethodNameNode      = null;
        //    XmlNodeList     ParamNodes          = null;
        //    XmlNodeList     ErrorNodes          = null;
        //    String          ObjectName          = null;
        //    String          MethodName          = null;
        //    String          ErrorXML            = null;
        //    String          ReturnErrors        = null;
        //    try
        //    {
        //        foreach (XmlNode OperationNode in OperationNodes)
        //        {
        //            //Initialize the DataManager Object
        //            OMDataManager = new DataManager(TenantName);
                                       
        //            //Set the Tracing attribute of DataManager
        //            OMDataManager.SetTracing(bTransaction);
                    
        //            //---Read the ObjectName and Method combination
        //            ObjectNameNode = OperationNode.SelectSingleNode("ObjectName");
        //            if (ObjectNameNode != null)
        //                ObjectName = ObjectNameNode.InnerText;
        //            else
        //                ObjectName = "Unknown";

        //            MethodNameNode = OperationNode.SelectSingleNode("Method");
        //            if (MethodNameNode != null)
        //                MethodName = MethodNameNode.InnerText;
        //            else
        //                MethodName = "Unknown";

        //            //Read all the Parameter Nodes
        //            ParamNodes = OperationNode.SelectNodes("Param");

        //            ErrorXML = executeReadOperations(OMDataManager,
        //                                         ObjectName,
        //                                         MethodName,
        //                                         ParamNodes, ref OutputXML);

        //            //If there is an error while executing a operation break from the for loop
        //            if (ErrorXML != null)
        //                ReturnErrors += ErrorXML;
        //            }
             

        //        }//End For 

        //        ReturnErrors += "</ErrorCollection>";

        //        //Loading the Error XML to identify whether there is any error or not
        //        ErrorXMLDocument = new XmlDocument();
        //        ErrorXMLDocument.LoadXml(ReturnErrors);

        //        ErrorNodes = ErrorXMLDocument.SelectNodes("ErrorCollection/Error");

        //        if (ErrorNodes.Count > 0)
        //            return ErrorXMLDocument.InnerXml;
        //        else
        //            return null;
        //    }
        //    catch (Exception ExceptionObj)
        //    {
        //    }
        //    finally
        //    {
              
        //        if (OMDataManager != null)
        //            OMDataManager.DataProvider.Close();
        //    }
        //    return null;
        //}

        //#endregion
        private void processMethodParam(XmlNodeList ParamNodes, ref StringDictionary ParamArray)
        {
            String ParamName    = null;
            String ParamValue   = null;

            ParamArray = new StringDictionary();
            foreach (XmlNode ParamNode in ParamNodes)
            {
                //Following code is commented by Kiran on date 16/Jan/2006.
                //ParamName = ParamNode.Attributes["Name"].InnerText;
                ParamName = ParamNode.Attributes["Name"].InnerText.Trim().ToUpper();
                ParamValue = ParamNode.InnerXml;
                ParamArray.Add(ParamName, ParamValue);
            }
        }

        private void processMethodParam(XmlNodeList ParamNodes, ref String[] ParamArray)
        {
            String  ParamName   = null;
            String  ParamValue  = null;
            int     Length      = 0;
            int     Index       = 0;

            Length = ParamNodes.Count;
            ParamArray = new String[Length];
            foreach (XmlNode ParamNode in ParamNodes)
            {
                //Following code is commented by Kiran on date 16/Jan/2006.
                //ParamName = ParamNode.Attributes["Name"].InnerText;
                ParamName = ParamNode.Attributes["Name"].InnerText.Trim().ToUpper();
                ParamValue = ParamNode.InnerXml;
                ParamArray[Index++] = ParamValue;
            }
        }

        private void deProcessMethodParam(ref XmlNodeList ParamNodes,  StringDictionary ParamArray)
        {
            String Key = null;
            foreach (XmlNode ParamNode in ParamNodes)
	        {
        		 Key                = ParamNode.Attributes["Name"].InnerText;
                 ParamNode.InnerXml = ParamArray[Key];
	        }
        }

        #region Get Qualified Class Name
        private String getQualifiedClassName(String ClassName)
        {
            String QualifiedClassName = null;

            try
            {
                switch (ClassName)
                {
                    case "ENTITYTYPE":
                        QualifiedClassName = "Assetlink.ObjectManager.ETFTManagement.EntityType";
                        break;

                    case "FACETTYPE":
                        QualifiedClassName = "Assetlink.ObjectManager.ETFTManagement.FacetType";
                        break;

                    case "ACCESSROLE":
                        QualifiedClassName = "Assetlink.ObjectManager.AccessControl.AccessorRole";
                        break;

                    case "ACCESSRULE":
                        QualifiedClassName = "Assetlink.ObjectManager.AccessControl.AccessRule";
                        break;

                    case "NAMEDHIERARCHY":
                        QualifiedClassName = "Assetlink.ObjectManager.NamedHierarchy.NamedHierarchy";
                        break;
                }
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }

            return QualifiedClassName;
        }
        #endregion

      
    }
}

