﻿#region System Includes

using System;
using System.Text;
using System.Xml;
using System.Collections;
using System.Reflection;
using System.Collections.Specialized;
using System.Transactions;
using System.Xml.XPath;
using System.IO;
using System.Threading;
using System.Collections.Generic;

#endregion

#region Assetlink Includes

using Assetlink;
using Assetlink.ObjectManager;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager.EntityFacetManagement;
using Assetlink.ObjectManager.ETFTManagement;
using Assetlink.ObjectManager.AccessControl;
using Assetlink.ObjectManager.Search;
using Assetlink.FileAccessLayer;
using Assetlink.Diagnostic;
using Assetlink.ObjectManager.NamedHierarchy;

#endregion

namespace Assetlink
{
    using ERRORXML = System.String;
    using XML = System.String;
    
	public partial class ObjectManagerODSClass : ObjectManagerBase
    {
        private String ODSConfigXMLPath = null;
                
        #region Constructors

        public ObjectManagerODSClass()
        {
            ODSConfigXMLPath = ALCommon.GetTenantConfigurationPath(TenantName) + "ODSMOMSchemaConfig.xml";
        }
        

        public ObjectManagerODSClass(ulong UserID, string CurrentLanguage,
                            string TenantName, DataManager OMDataManagerObj, DataManager OMODSDataManagerObj)
            : base(UserID, CurrentLanguage, TenantName, OMDataManagerObj, OMODSDataManagerObj)
        {
            ODSConfigXMLPath = ALCommon.GetTenantConfigurationPath(TenantName) + "ODSMOMSchemaConfig.xml";
        }
        
        #endregion

        #region Member Variables

        static  Object  Lock                        = new object();
        const   String  DataPopulationStoredProc    = "_DATASP";
        const   String  ManageRelationsStoredProc   = "_RELNSP";

        #endregion

		#region Execute

        /// <summary>
        /// 
        /// <CreatedDate>29 March 2011</CreatedDate>
        /// <Author>Roopa Mukim</Author>
        /// <Purpose>
        ///  This function does the query execution for ODS.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">
        /// 
        /// </param>
        /// <returns>Return Null if successful or return the Error XML.</returns>
        public String Execute(ref String InputXML)
		{
			#region Member Variables
			XPathDocument		xpDoc				= null;
			OMError				OMErrorObj			= null;
			String				ErrorXML			= null;
			String				MethodName			= null;
            String              Pre_Exec_ConfigXML  = null;
            Boolean             IsContainerParent   = false;
			#endregion

			try
            {
                if (!File.Exists(ODSConfigXMLPath))
                {
                    OMErrorObj = new OMError(OMErrorCodes.ODS_MOM_SCHEMA_FILE_NOT_FOUND, "MOM Schema Config XML does not exist.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, null);
                    ErrorXML = OMErrorObj.Serialize();
                    
                    return ErrorXML;
                    
                }
                
                Pre_Exec_ConfigXML = File.ReadAllText(ODSConfigXMLPath);                
                if (!String.IsNullOrEmpty(InputXML))
                {
                    xpDoc = new XPathDocument(new StringReader(InputXML));
                    if (xpDoc.CreateNavigator().SelectSingleNode("INPUTXML/OPERATION/OBJECTNAME") != null)
                    {
                        if (!String.IsNullOrEmpty(xpDoc.CreateNavigator().SelectSingleNode("INPUTXML").GetAttribute("IsContainerParent", "")))
                            IsContainerParent = Convert.ToBoolean(xpDoc.CreateNavigator().SelectSingleNode("INPUTXML").GetAttribute("IsContainerParent", ""));

                        MethodName = xpDoc.CreateNavigator().SelectSingleNode("INPUTXML/OPERATION/METHOD").Value;

                        if (!String.IsNullOrEmpty(MethodName))
                        {
                            switch (MethodName)
                            {
                                case "ASSOCIATETYPE":
                                    OMErrorObj = ExecuteAssociateType(InputXML, IsContainerParent);
                                    if (OMErrorObj != null)
                                    {
                                        ErrorXML = OMErrorObj.Serialize();                                        
                                    }
                                    break;

                                case "DISASSOCIATETYPEBYRELATIONSHIPNAME":
                                    OMErrorObj = ExecuteDisAssociateType(InputXML);
                                    if (OMErrorObj != null)
                                    {
                                        ErrorXML = OMErrorObj.Serialize();
                                    }
                                    break;
                                case "UPDATE":
                                    OMErrorObj = UpdateObjectType(InputXML);
                                    if (OMErrorObj != null)
                                    {
                                        ErrorXML = OMErrorObj.Serialize();
                                    }
                                    break;
                                case "DELETE":
                                    OMErrorObj = DeleteObjectType(InputXML);
                                    if (OMErrorObj != null)
                                    {
                                        ErrorXML = OMErrorObj.Serialize();
                                    }
                                    break;
                                default:
                                    break;
                            }
                        }
                    }
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        File.WriteAllText(ODSConfigXMLPath, Pre_Exec_ConfigXML);
                    }

                }
                else
                {
                    OMErrorObj = new OMError(OMErrorCodes.OMODSCLASS_EXECUTE_FAILED, "Input XML is null or Empty",
                                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ObjectManagerODSClass",
                                        OMErrorType.EXCEPTION, null, null);
                    ErrorXML = OMErrorObj.Serialize();
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.OMODSCLASS_EXECUTE_FAILED, "Error in Executing Operations for ODS",
                                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ObjectManagerODSClass",
                                        OMErrorType.EXCEPTION, ExcepObj, null);
                ErrorXML = OMErrorObj.Serialize();
            }
            return ErrorXML;
        }


		/// <summary>
		/// This Method creates an relationship between a TARGET ENTITY/FACET to a related ENTITY/FACET
        /// This method is only executed when the target ENTITY/FACET is configured for use in ODS.
        /// The method performs the following steps.
        /// 1. Create a table in ODS DB for the related ENTITY/FACET if it does not exist.
        /// 2. Create a row in the ALOM_FACETTYPE table in MOM DB if table is created in ODS DB.
        /// 3. Create schema information for the table (if table is created in MOM DB) in ALOM_SCHEMA table in MOM DB.
        /// 4. Generate a new relationship in the form TARGET_ENTITY/FACETNAME_ODS_RELATEDENTITY/FACETNAME in ALOM_RELATIONSHIPTYPES in 
        /// MOM DB.
        /// 5. Generate stored procedures in MOM and ODS DB for data population.
		/// </summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>24 May 2011</CreatedOn>
		/// <param name="INPUTXML">The Input XML contains the new relationship details</param>
        /// <param name="IsContainerParent"></param>
		/// <returns>
		/// Returns Error XML if error or exception occures.
		/// </returns>
        private OMError ExecuteAssociateType(String INPUTXML, Boolean IsContainerParent)
		{
			#region Member Variables
			XmlDocument			InputDoc			= null;
			XPathNavigator		xpNav				= null;
			OMError				OMErrorObj			= null;
			String				TypeOutPutXML		= "";
			String				ErrorXML			= null;
			String				TypeId				= null;
			String				RelationshipTypeXML	= null;
			String				TableName			= null;
            String              TypeName            = null;            
            Boolean             TableExists         = false;
            String              ODSTableTypeXML     = null;
            EntityType          EntityTypeObj       = null;
            FacetType           FacetTypeObj        = null;
            OMError             ReturnErrorObj      = null;
            String              ContainerTypeName   = null;
            String              ContainerTypeXML    = null;
            String              ODSRelName          = null;
            //String              IsParent            = null;
            String              FieldName           = null;
            StringBuilder       NewRelCollXML       = null;
            String              NewRelXML           = null;
            String              MomObjectTypeName   = null;
            String              ContainerType       = null;
            String              ContainsType        = null;
            String              MomSchemaConfigXML  = null;
            int                 PrimaryKeyNumber    = 0;
            Boolean             ObjectExists        = false;
            XML                 xml_StoredProcNames = null;
            String              ODSPrimaryKey       = null;
            String              ODSMOMPrimaryKey    = null;
            String              MOMObjectTypeID     = null;
            String              MOM_ODSObjectTypeID = null;
            String              ODS_ForignKeyFieldName = null;
			#endregion

			try
			{                
                InputDoc	    = new XmlDocument();
                NewRelCollXML   = new StringBuilder(100);
                NewRelCollXML.Append("<RELATIONSHIPCOLLECTION>");
				InputDoc.LoadXml(INPUTXML);
				xpNav		= InputDoc.CreateNavigator();

				//TypeId				= xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM/RELATIONSHIPTYPE/CONTAINSTYPEID").Value;
                MOM_ODSObjectTypeID = xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM/RELATIONSHIPTYPE/CONTAINSTYPEID").Value;
				RelationshipTypeXML = xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM/RELATIONSHIPTYPE").OuterXml;
				ContainsType        = xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM/RELATIONSHIPTYPE/CONTAINSTYPE").Value;                
                ContainerType       = xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM/RELATIONSHIPTYPE/CONTAINERTYPE").Value;
                if (!String.IsNullOrEmpty(xpNav.SelectSingleNode("INPUTXML").GetAttribute("ForignKeyNameForODS", "")))
                    FieldName = xpNav.SelectSingleNode("INPUTXML").GetAttribute("ForignKeyNameForODS", "");

                if (ContainsType == "ENTITY")
                {
                    EntityTypeObj   = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj      = EntityTypeObj.Read(Convert.ToUInt64(MOM_ODSObjectTypeID), ref TypeOutPutXML);
                    EntityTypeObj   = null;
                }
                else if (ContainsType == "FACET")
                {
                    FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj   = FacetTypeObj.Read(Convert.ToUInt64(MOM_ODSObjectTypeID), ref TypeOutPutXML);
                    FacetTypeObj = null;
                }

                if ((ContainsType.ToUpper() == "FACET") && (ContainerType.ToUpper() == "ENTITY"))
                    IsContainerParent = true;
               
                if (OMErrorObj != null)
                {
                    return OMErrorObj;
                }

               
				if (String.IsNullOrEmpty(ErrorXML))
				{
                    // Retrieve information about the related entity/facet using Get Type Info
                    // if the related information exists then no need to create a table in ODS DB.
                    // Only add alter the table to include new field and update MOM DB for Schema Update
                    OMErrorObj = CheckIfTableExists(TypeOutPutXML, ContainsType, out TableExists, out ODSTableTypeXML);
                    if (OMErrorObj == null)
                    {
                        MomObjectTypeName = ALCommon.GetInnerValue(ref TypeOutPutXML, "<TYPENAME>", "</TYPENAME>");
                        ODSMOMPrimaryKey = ALCommon.GetInnerValue(ref TypeOutPutXML, "<PRIMARYKEY>", "</PRIMARYKEY>");
                        if (!TableExists)
                        {      
                            //Crete table for Entity or Facet in ODS Database                           
                            OMErrorObj = CreateTable(ref TypeOutPutXML, ref TableName, ref FieldName, out PrimaryKeyNumber);

                            if (OMErrorObj == null)
                            {
                                //Add entry in ALOM_RELATIONSHIPTYPES table in MOM
                                TypeId = ALCommon.GetInnerValue(ref TypeOutPutXML, "<TYPEID>", "</TYPEID>");
                                TypeName = ALCommon.GetInnerValue(ref TypeOutPutXML, "<TYPENAME>", "</TYPENAME>");
                                ODSPrimaryKey = ALCommon.GetInnerValue(ref TypeOutPutXML, "<PRIMARYKEY>", "</PRIMARYKEY>");
                                OMErrorObj = AssociateRelationship(RelationshipTypeXML, TypeId, TypeName, out ContainerTypeName, out ContainerTypeXML, out NewRelXML);
                                
                                if (OMErrorObj == null)
                                {
                                    RelationshipTypeXML = RelationshipTypeXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForMOM='True'>");
                                    NewRelCollXML.Append(RelationshipTypeXML);

                                    NewRelXML = NewRelXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForODS='True'>");
                                    NewRelCollXML.Append(NewRelXML);
                                    NewRelCollXML.Append("</RELATIONSHIPCOLLECTION>");
                                    //Add entry in ALOM_FACETTYPE or ALOM_ENTITYTYPE table in MOM                            
                                    OMErrorObj = AddObjectTypeEntry(TypeOutPutXML, TableName, ContainsType, TypeName);

                                    if (OMErrorObj == null)
                                    {
                                        //Add entry in ALOM_SCHEMA table in MOM
                                        OMErrorObj = AddSchemaEntry(TypeOutPutXML);
                                        if (OMErrorObj == null)
                                        {
                                            // If CreateProcedure Call is successful,UPDATE ODS Config XML
                                            ContainerType       = ALCommon.GetInnerValue(ref RelationshipTypeXML, "<CONTAINERTYPE>", "</CONTAINERTYPE>");
                                            String MOMRelName   = ALCommon.GetInnerValue(ref RelationshipTypeXML, "<RELATIONSHIPNAME>", "</RELATIONSHIPNAME>");
                                            String MOMTable     = ALCommon.GetInnerValue(ref ContainerTypeXML, "<TABLENAME>", "</TABLENAME>");
                                            String MOMPrimaryKey = ALCommon.GetInnerValue(ref ContainerTypeXML, "<PRIMARYKEY>", "</PRIMARYKEY>");
                                            MOMObjectTypeID     = ALCommon.GetInnerValue(ref ContainerTypeXML, "<TYPEID>", "</TYPEID>");

                                            String ODSFieldXML = PrepareODSFieldXML(ContainerType, ContainerTypeName, MOMTable, MOMPrimaryKey, MOMRelName, NewRelCollXML.ToString(), IsContainerParent.ToString(), FieldName, MOMObjectTypeID);
                                            OMErrorObj  =  IsODSObjectExists(TypeName, out ObjectExists);
                                            if (OMErrorObj == null)
                                            {
                                                if (ObjectExists)
                                                {
                                                    UpdateNewFieldInODSConfigurationXML(TypeName, ODSFieldXML, (PrimaryKeyNumber + 1).ToString(), out MomSchemaConfigXML);
                                                }
                                                else
                                                {
                                                    String ODSTypeXML = PrepareODSObjectTypeXML(TypeName, ContainsType, ODSPrimaryKey, ODSMOMPrimaryKey, TableName, MomObjectTypeName, (PrimaryKeyNumber + 1).ToString(), MOM_ODSObjectTypeID, TypeId, ODSFieldXML);
                                                    UpdateNewODSObjectTypeInODSConfigurationXML(ODSTypeXML, out MomSchemaConfigXML);
                                                }
                                            }
                                            if (MomSchemaConfigXML == null)
                                            {
                                                ReturnErrorObj = new OMError(OMErrorCodes.ODS_CONFIGXML_UPDATE_FAILED, "The MOM Schema Config XML for ODS does not Exist", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                                return (ReturnErrorObj);
                                            }

                                            OMErrorObj = createODSPopulationStoredProcedures(TypeName, NewRelCollXML.ToString(),
                                                    MomSchemaConfigXML, out xml_StoredProcNames);
                                            if (OMErrorObj != null)
                                            {
                                                ReturnErrorObj = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED, "Stored Procedure creation failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                                return (ReturnErrorObj);
                                            }

                                            //Update and save the related ODS configuration XMLs.
                                            OMErrorObj = saveODSConfigXMLsForCreateOperations(true, ODSConfigXMLPath, MomSchemaConfigXML, xml_StoredProcNames);
                                            if (OMErrorObj != null)
                                            {
                                                ReturnErrorObj = new OMError(OMErrorCodes.ODS_CONFIGXML_UPDATE_FAILED,
                                                        "Error while updating ODS configuration XML and the stored procedure configuration XML for back office service",
                                                        OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType,
                                                        OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                                return (ReturnErrorObj);
                                            }
                                        }
                                        else
                                        {
                                            if (ContainsType.ToUpper() == "ENTITY")
                                                ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED, "Entity Type Modification in Schema Table Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                            else
                                                ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED, "Entity Type Modification in Schema Table Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        }
                                    }
                                    else
                                    {
                                        //ErrorXML = OMErrorObj.Serialize();
                                        if (ContainsType.ToUpper() == "ENTITY")
                                            ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED, "Entity Type Creation Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        else
                                            ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED, "Entity Type Creation Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                    }
                                }
                                else
                                {
                                    //ErrorXML = OMErrorObj.Serialize();
                                    ReturnErrorObj = new OMError(OMErrorCodes.ASSOCIATION_CREATION_FAILED, "Association Creation failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                }
                            }
                            else
                            {

                                ReturnErrorObj = new OMError(OMErrorCodes.ODS_TABLE_CREATION_FAILED, "Table creation in ODS Failed", OMErrorObjectType.TABLE, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                            }
                        }
                        else // table exists...
                        // only modify the schema in MOM and alter the table in ODS DB.
                        {
                            TypeId      = ALCommon.GetInnerValue(ref ODSTableTypeXML, "<TYPEID>", "</TYPEID>");
                            TypeName    = ALCommon.GetInnerValue(ref ODSTableTypeXML, "<TYPENAME>", "</TYPENAME>");
                            OMErrorObj  = AssociateRelationship(RelationshipTypeXML, TypeId, TypeName, out ContainerTypeName, out ContainerTypeXML, out NewRelXML);
                            if (OMErrorObj == null)
                            {
                                
                                // Alter ODS Table
                                XmlDocument TypeDoc = new XmlDocument();
                                TypeDoc.LoadXml(ODSTableTypeXML);
                                if (String.IsNullOrEmpty(FieldName))
                                {
                                    PrimaryKeyNumber = GetODSPrimaryKeyNumber(TypeName);
                                    FieldName        = "FIELD" + (PrimaryKeyNumber + 1).ToString();
                                }                               

                                XmlNodeList FieldList = TypeDoc.SelectNodes("TYPE/FIELDCOLLECTION/FIELD");
                                int FieldCount = FieldList.Count;

                                TableName = TypeDoc.SelectSingleNode("TYPE/TABLENAME").InnerXml;

                                TypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION").RemoveAll();
                                String strPrimaryField = PrepareTEPrimaryFieldNode(true, PrimaryKeyNumber, TypeId, FieldCount, TableName, ContainsType, FieldName);
                                TypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION").CreateNavigator().AppendChild(strPrimaryField);

                                OMErrorObj = OMODSDataManager.DataProvider.UpdateTable(TypeDoc.OuterXml);
                                if (OMErrorObj == null)
                                {
                                    // ALter MOM Schema
                                    OMErrorObj = AddSchemaEntry(TypeDoc.OuterXml);

                                    if (OMErrorObj != null)
                                    {
                                        if (ContainsType.ToUpper() == "ENTITY")
                                            ReturnErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_MODIFICATION_FAILED, "Entity Type Modification in Schema Table Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        else
                                            ReturnErrorObj = new OMError(OMErrorCodes.FACET_TYPE_MODIFICATION_FAILED, "Entity Type Modification in Schema Table Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return ReturnErrorObj;
                                    }

                                    OMErrorObj = deleteODSPopulationStoredProcedures(TypeName, out xml_StoredProcNames);
                                    if (OMErrorObj != null)
                                    {
                                        ReturnErrorObj = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED, "Procedure Deletion Failed", OMErrorObj.ObjectType, TypeName, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return ReturnErrorObj;
                                    }

                                    //Update and save the related ODS configuration XMLs.
                                    OMErrorObj = saveODSConfigXMLsForDeleteOperations(xml_StoredProcNames);
                                    if (OMErrorObj != null)
                                    {
                                        ReturnErrorObj = new OMError(OMErrorCodes.ODS_CONFIGXML_UPDATE_FAILED,
                                                "Error while updating the stored procedure configuration XML for back office service",
                                                OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return (ReturnErrorObj);
                                    }

                                    
                                    // If CreateProcedure Call is successful,UPDATE ODS Config XML
                                    ContainerType           = ALCommon.GetInnerValue(ref RelationshipTypeXML, "<CONTAINERTYPE>", "</CONTAINERTYPE>");
                                    String MOMRelName       = ALCommon.GetInnerValue(ref RelationshipTypeXML, "<RELATIONSHIPNAME>", "</RELATIONSHIPNAME>");
                                    String MOMTable         = ALCommon.GetInnerValue(ref ContainerTypeXML, "<TABLENAME>", "</TABLENAME>");
                                    String MOMPrimaryKey    = ALCommon.GetInnerValue(ref ContainerTypeXML, "<PRIMARYKEY>", "</PRIMARYKEY>");
                                    MOMObjectTypeID         = ALCommon.GetInnerValue(ref ContainerTypeXML, "<TYPEID>", "</TYPEID>");
                                    RelationshipTypeXML = RelationshipTypeXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForMOM='True'>");
                                    NewRelCollXML.Append(RelationshipTypeXML);

                                    NewRelXML = NewRelXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForODS='True'>");
                                    NewRelCollXML.Append(NewRelXML);
                                    NewRelCollXML.Append("</RELATIONSHIPCOLLECTION>");

                                    String ODSFieldXML = PrepareODSFieldXML(ContainerType, ContainerTypeName, MOMTable, MOMPrimaryKey, MOMRelName, NewRelCollXML.ToString(), IsContainerParent.ToString(), FieldName, MOMObjectTypeID);
                                    OMErrorObj = IsODSObjectExists(TypeName, out ObjectExists);
                                    if (OMErrorObj == null)
                                    {
                                        if (ObjectExists)
                                        {
                                            UpdateNewFieldInODSConfigurationXML( TypeName, ODSFieldXML, (PrimaryKeyNumber + 1).ToString(), out MomSchemaConfigXML);
                                        }
                                        else
                                        {
                                            String ODSTypeXML = PrepareODSObjectTypeXML(TypeName, ContainsType, ODSPrimaryKey, ODSMOMPrimaryKey, TableName, MomObjectTypeName, (PrimaryKeyNumber + 1).ToString(), MOM_ODSObjectTypeID, TypeId, ODSFieldXML);
                                            UpdateNewODSObjectTypeInODSConfigurationXML( ODSTypeXML, out MomSchemaConfigXML);
                                        }
                                    }
                                    if (MomSchemaConfigXML == null)
                                    {
                                        ReturnErrorObj = new OMError(OMErrorCodes.ODS_CONFIGXML_UPDATE_FAILED, "The MOM Schema Config XML for ODS does not Exist", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return (ReturnErrorObj);
                                    }

                                    //RelationshipTypeXML = RelationshipTypeXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForMOM='True'>");
                                    //NewRelCollXML.Append(RelationshipTypeXML);

                                    //NewRelXML = NewRelXML.Replace("<RELATIONSHIPTYPE>", "<RELATIONSHIPTYPE RelationForODS='True'>");
                                    //NewRelCollXML.Append(NewRelXML);
                                    //NewRelCollXML.Append("</RELATIONSHIPCOLLECTION>");

                                    OMErrorObj = createODSPopulationStoredProcedures(TypeName, NewRelCollXML.ToString(),
                                            MomSchemaConfigXML, out xml_StoredProcNames);
                                    if (OMErrorObj != null)
                                    {
                                        ReturnErrorObj = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED, "Stored Procedure creation failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return (ReturnErrorObj);
                                    }

                                    //Update and save the related ODS configuration XMLs.
                                    OMErrorObj = saveODSConfigXMLsForCreateOperations(true, ODSConfigXMLPath, MomSchemaConfigXML, xml_StoredProcNames);
                                    if (OMErrorObj != null)
                                    {
                                        ReturnErrorObj = new OMError(OMErrorCodes.ODS_CONFIGXML_UPDATE_FAILED,
                                                "Error while updating ODS configuration XML and the stored procedure configuration XML for back office service",
                                                OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                        return (ReturnErrorObj);
                                    }
                                }
                                else
                                {
                                    //ErrorXML = OMErrorObj.Serialize();
                                    ReturnErrorObj = new OMError(OMErrorCodes.ODS_UPDATE_TABLE_FAILED, "Update Table in ODS Database Failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                                }
                            }
                            else
                            {
                                //ErrorXML = OMErrorObj.Serialize();
                                ReturnErrorObj = new OMError(OMErrorCodes.ASSOCIATION_CREATION_FAILED, "Association Creation failed", OMErrorObj.ObjectType, ContainsType, OMErrorObj.ErrorType, OMErrorObj.ExceptionObject, OMErrorObj.ErrorXML);
                            }
                        }
                    }                    
				}
			}
			catch(Exception ExcepObj)
			{
                ReturnErrorObj = new OMError(OMErrorCodes.OMODSCLASS_EXECUTE_FAILED, "Error in Executing Associate Operation For ODS",
							OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerODSClass",
							OMErrorType.EXCEPTION, ExcepObj, null);
				ErrorXML = OMErrorObj.Serialize();
			}
            return ReturnErrorObj;
		}

		private OMError ExecuteDisAssociateType(String INPUTXML)
		{
			#region Member Variables
			XPathDocument	    xpDoc				= null;
			XPathNavigator	    xpNav				= null;
            XPathNodeIterator   xpIter_Relns        = null;
			OMError			    OMErrorObj			= null;
			String			    ErrorXML			= null;
			String			    MOMRelationshipName	= null;
            String              MOMObjectName       = null;           
            String              ObjectTypeDetails   = null;
            String              ObjectTypeID        = null;            
            String              ODSTableName        = null;
            String              ODSTypeName         = null;
            String              ObjectType          = null;
            String              DeleteQuery         = null;
            String              RelationshipTypeID  = null;
            XML                 xml_StoredProcNames = null;
            XML                 xml_ODSMOMConfig    = null;
            Boolean             MOMObjectExists     = false;
                       
            String              TargetPKName        = null;
            String              SchemaID            = null;
            String              RelationshipXML     = null;
            RelationshipType    RelationTypeObj     = null;
            //XML                 xml_Relationships   = null;
            
            String              RelatedODSName      = null;
            String              RelatedODSType      = null;
            String              RelatedODSTableName = null;
            String              RelatedMOMObjectName= null;
			#endregion

            /* Steps performed
             * 1. Check if this is the only relationship that exists with the related type.
             *    If YES then following operations will be performed.
             *    -- Remove Stored Procedure from Config XML.
             *    -- Drop Stored Procedures.
             *    -- Remove ODS Config Information from MOM Schema XML.
             *    -- Remove the foll from MOM DB: Schema Info, Relationship details, Object Type Details.
             *    -- Drop Tables from ODS DB.
             *    If more than one relationships exist for the ODS ContainsType object.
             *    -- Only Delete the Relationship.
             *    -- Remove Field Information from MOM Schema XML.
             */

            try
			{
				xpDoc = new XPathDocument(new StringReader(INPUTXML));
				xpNav = xpDoc.CreateNavigator();

				if (xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM [@Name='RELATIONSHIPNAME']") != null)
                {
                    MOMRelationshipName = xpNav.SelectSingleNode("INPUTXML/OPERATION/PARAM [@Name='RELATIONSHIPNAME']").Value;

                    #region Get ODS RelationshipName

                    OMErrorObj = GetODSRelationshipsFromConfig(MOMRelationshipName, out xpIter_Relns);
                    //RelationTypeObj = new RelationshipType(OMDataManager);
                    //OMErrorObj = RelationTypeObj.ReadByRelationShipName("ODS_" + MOMRelationshipName, ref RelationshipXML);
                    //if (OMErrorObj == null)
                    //{
                    //    RelationshipTypeID = ALCommon.GetInnerValue(ref RelationshipXML, "<RELATIONSHIPTYPEID>", "</RELATIONSHIPTYPEID>");
                    //}
                    //else
                    //    return OMErrorObj;
                    #endregion 

                    #region Get the ODS & MOM Object details for the given  MOM relationship name

                    GetODSInfoFromMOMRelationshipName(MOMRelationshipName, out MOMObjectName, out ODSTypeName, out ODSTableName, out ObjectType);

                    // Check if the ODS Object is associated to more than 1 entity IN THE MOM SHCEMA CONFIG XML
                    if ((!String.IsNullOrEmpty(MOMObjectName)) && (!String.IsNullOrEmpty(ODSTypeName)))
                    {
                        MOMObjectExists = CheckIfMOMObjectExistsAsODS(MOMObjectName, MOMRelationshipName, out RelatedODSName, out RelatedODSType, out RelatedODSTableName, out RelatedMOMObjectName);
                        //MOMObjectExists = CheckIfMOMObjectExistsAsODS(MOMObjectName, out RelatedODSName, out RelatedODSType, out RelatedODSTableName, out RelatedMOMObjectName);
                    }
                    #endregion

                    #region Get type details
                    OMErrorObj = GetObjectTypeDetails(ObjectType, ODSTypeName, out ObjectTypeDetails);
                    if (OMErrorObj != null)
                        return OMErrorObj;

                    ObjectTypeID = ALCommon.GetInnerValue(ref ObjectTypeDetails, "<TYPEID>", "</TYPEID>");
                    #endregion
                    // if the ODS Object
                    if (!MOMObjectExists)
                    {
                        #region only Single relationship exists then all the ODS information can be deleted
                        if (OMErrorObj == null)
                        {
                            if (ObjectType.ToUpper() == "FACET")
                            {
                                OMErrorObj = DeleteAllInfo(ODSTableName,"ALOM_FACETTYPE", ObjectTypeID, "FACETTYPEID", ObjectTypeDetails);
                            }
                            else
                            {
                                OMErrorObj = DeleteAllInfo(ODSTableName, "ALOM_ENTITYTYPE" ,ObjectTypeID, "ENTITYTYPEID", ObjectTypeDetails);
                            }                                                          
                            if (OMErrorObj == null)
                            {                                        
                                if (OMErrorObj == null)
                                    OMErrorObj = deleteODSPopulationStoredProcedures(ODSTypeName, out xml_StoredProcNames);

                                if (OMErrorObj == null)
                                {
                                    OMErrorObj = RemoveODSFieldFromConfigXML(ODSTypeName, MOMRelationshipName, MOMObjectExists, out xml_ODSMOMConfig);
                                    OMErrorObj = saveODSConfigXMLsForDeleteOperations(xml_StoredProcNames);                                    
                                }
                            }
                        }
                        #endregion

                        #region Remove Empty ODS node, also remove type info
                        if (!String.IsNullOrEmpty(RelatedODSName) && !String.IsNullOrEmpty(RelatedODSType) && !String.IsNullOrEmpty(RelatedODSTableName) && (RelatedODSName != ODSTypeName))
                        {
                            OMErrorObj = GetObjectTypeDetails(RelatedODSType, RelatedODSName, out ObjectTypeDetails);
                            if (OMErrorObj == null)
                            { 
                                ObjectTypeID = ALCommon.GetInnerValue(ref ObjectTypeDetails, "<TYPEID>", "</TYPEID>");
                                if (RelatedODSType.ToUpper() == "FACET")
                                {
                                    OMErrorObj = DeleteAllInfo(RelatedODSTableName, "ALOM_FACETTYPE", ObjectTypeID, "FACETTYPEID", ObjectTypeDetails);
                                }
                                else
                                {
                                    OMErrorObj = DeleteAllInfo(RelatedODSTableName, "ALOM_ENTITYTYPE", ObjectTypeID, "ENTITYTYPEID", ObjectTypeDetails);
                                }
                                if (OMErrorObj == null)
                                {
                                    //OMErrorObj = RemoveODSObjectFromConfigXML(RelatedMOMObjectName, out xml_ODSMOMConfig, out ODSObjectDeleted);
                                    OMErrorObj = RemoveODSFieldFromConfigXML(RelatedODSName, "", MOMObjectExists, out xml_ODSMOMConfig);
                                }
                            }                            
                        }
                        #endregion
                    }
                    else
                    {
                        #region More than one relationship exists: alter table by removing the field and recreate stored procedures.

                        TargetPKName = GetTargetPrimaryKeyName(ODSTypeName, "ODS_" + MOMRelationshipName);
                        if (!String.IsNullOrEmpty(ObjectTypeDetails))
                        {
                            XmlDocument ObjectTypeDoc = new XmlDocument();
                            ObjectTypeDoc.LoadXml(ObjectTypeDetails);
                            XmlNode FieldNode = ObjectTypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION/FIELD[FIELDNAME='" + TargetPKName + "']");
                            if (FieldNode != null)
                            {
                                ((XmlElement)FieldNode).SetAttribute("OperationType", "DELETE");
                                SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                            }
                            // Alter Table
                            OMErrorObj = OMODSDataManager.DataProvider.UpdateTable(ObjectTypeDoc.OuterXml);
                            if (OMErrorObj == null)
                            {
                                // Update Schema
                                OMErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID, OMConjunction.NOTHING);
                                if (OMErrorObj == null)
                                {
                                    DeleteQuery = String.Empty;
                                    OMErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_SCHEMA", ref DeleteQuery);
                                    if (OMErrorObj == null)
                                    {
                                        OMErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                                        if (OMErrorObj == null)
                                        {
                                            OMErrorObj = deleteODSPopulationStoredProcedures(ODSTypeName, out xml_StoredProcNames);
                                            if (OMErrorObj == null)
                                            {
                                                
                                                OMErrorObj = RemoveODSFieldFromConfigXML(ODSTypeName, MOMRelationshipName, MOMObjectExists, out xml_ODSMOMConfig);
                                                if (OMErrorObj == null)
                                                {
                                                    OMErrorObj = saveODSConfigXMLsForDeleteOperations(xml_StoredProcNames);
                                                    if (OMErrorObj == null)
                                                    {
                                                        OMErrorObj = createODSPopulationStoredProcedures(ODSTypeName, null, xml_ODSMOMConfig, out xml_StoredProcNames);
                                                        if (OMErrorObj == null)
                                                        {
                                                            OMErrorObj = saveODSConfigXMLsForCreateOperations(false, null, null, xml_StoredProcNames);

                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        #endregion
                    }


                    #region Common for all Conditions: Delete relationship
                    if ((xpIter_Relns != null) && (OMErrorObj == null))
                    {
                        while (xpIter_Relns.MoveNext())
                        {
                            OMErrorObj = OMDataManager.DataProvider.AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, xpIter_Relns.Current.GetAttribute("ODSRelationshipName", ""), OMConjunction.NOTHING);
                            if (OMErrorObj != null)
                                return OMErrorObj;
                            OMErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_RELATIONSHIPTYPES", ref DeleteQuery);
                            if (OMErrorObj != null)
                                return OMErrorObj;

                            OMErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                            if (OMErrorObj != null)
                                return OMErrorObj;
                        }
                    }
                    //if (OMErrorObj == null)
                    //{
                    //    OMErrorObj = OMDataManager.DataProvider.AddClause("RELATIONSHIPTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, RelationshipTypeID, OMConjunction.NOTHING);
                    //    if (OMErrorObj != null)
                    //        return OMErrorObj;

                    //    OMErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_RELATIONSHIPTYPES", ref DeleteQuery);
                    //    if (OMErrorObj != null)
                    //        return OMErrorObj;

                    //    OMErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                    //    if (OMErrorObj != null)
                    //        return OMErrorObj;
                    //}
                    #endregion
                    return OMErrorObj;
                }
			}
			catch(Exception ExcepObj)
			{
				OMErrorObj = new OMError(OMErrorCodes.OMODSCLASS_EXECUTE_FAILED, "Error in Executing Disassociate Operation For ODS",
							OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerODSClass",
							OMErrorType.EXCEPTION, ExcepObj, null);
				ErrorXML = OMErrorObj.Serialize();
			}
			
            return (OMErrorObj);
		}
        

		/// <summary>
		/// <CreatedDate>29 April 2011</CreatedDate>
        /// <Author>Mugdha Moolay Gadre</Author>
        /// <Purpose>
        /// This function is used to create table in ODS Database.
        /// </Purpose>
		/// </summary>
		/// <param name="InputXML">
		///		The output XML of GetTypeData function call is passed as input to this function.
        /// </param>
		/// <param name="TableName">
		///		Retrives table name from input XML.
		/// </param>
		/// <returns>
		/// This function returns OMError object if there is any error occured.
		/// </returns>
        private OMError CreateTable(ref String InputXML, ref String TableName, ref String FieldName, out int PrimaryKeyNumber)
        {
            #region member variables
            OMError				OMErrorObj			= null;
			String				CreateTableQuery	= null;
            
            String              TypeName            = null;
            String              strPrimaryField     = null;
            String              ObjectType          = null;            
            ulong               SchemaID            = 0;
            ulong               TypeID              = 0;                            
            int                 FieldCount          = 0;
			XmlDocument		    TypeDoc				= null;

            ulong               AssociationId       = 0;
            ulong               SchemaId            = 0;
            ulong               NamedHierarchyId    = 0;
            OMError             CreateErrorObj      = null;
            #endregion

            //FieldName = String.Empty;
            PrimaryKeyNumber = 0;
            try
            {                
                TypeDoc = new XmlDocument();

                #region Update TypeInputXML for new TE Primary field Node and Name of Object Type
                TypeDoc.LoadXml(InputXML);
                TableName = TypeDoc.CreateNavigator().SelectSingleNode("TYPE/TABLENAME").Value;
                TypeName = TypeDoc.CreateNavigator().SelectSingleNode("TYPE/TYPENAME").Value;
                TableName = TableName.Replace(TypeName, "ODS" + TypeName);
                TypeName = "ODS" + TypeName;
                ObjectType = TypeDoc.CreateNavigator().SelectSingleNode("TYPE/OBJECTTYPE").Value;

                TypeDoc.SelectSingleNode("TYPE/TABLENAME").InnerXml = TableName;
                TypeDoc.SelectSingleNode("TYPE/TYPENAME").InnerXml = TypeName;

                if (ALCommon.GenerateUniqueID(ref TypeID))
                {
                    TypeDoc.SelectSingleNode("TYPE/TYPEID").InnerXml = TypeID.ToString();
                }

                
                XmlNodeList FieldList = TypeDoc.SelectNodes("TYPE/FIELDCOLLECTION/FIELD");
                XmlNode FieldCollNode = TypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION");
                if (FieldList != null)
                {
                    FieldCount = TypeDoc.SelectNodes("TYPE/FIELDCOLLECTION/FIELD[FIELDTYPE!='XML' and FIELDTYPE!='FILE' and FIELDTYPE!='BINDER']").Count;
                    foreach (XmlNode FieldNode in FieldList)
                    {
                        if ((FieldNode.SelectSingleNode("FIELDTYPE").InnerText == "XML") || (FieldNode.SelectSingleNode("FIELDTYPE").InnerText == "FILE") || (FieldNode.SelectSingleNode("FIELDTYPE").InnerText == "BINDER"))
                        {
                            FieldCollNode.RemoveChild(FieldNode);
                        }
                        else
                        {
                            if (ALCommon.GenerateUniqueID(ref SchemaID))
                            {
                                FieldNode.SelectSingleNode("SCHEMAID").InnerXml = SchemaID.ToString();
                            }
                            FieldNode.SelectSingleNode("OBJECTTYPEID").InnerXml = TypeID.ToString();
                            FieldNode.SelectSingleNode("TABLENAME").InnerXml = TableName;
                            if (FieldNode.SelectSingleNode("ISNAMEDHIERARCHY").InnerXml == "FALSE")
                            {
                                XmlNode NHNode = FieldNode.SelectSingleNode("NHLEVEL");
                                FieldNode.RemoveChild(NHNode);
                                NHNode = FieldNode.SelectSingleNode("NHVALUETYPE");
                                FieldNode.RemoveChild(NHNode);
                            }
                        }
                    }
                }

                if (String.IsNullOrEmpty(FieldName))                
                {
                    PrimaryKeyNumber    = GetODSPrimaryKeyNumber(TypeName);
                    FieldName           = "FIELD" + (PrimaryKeyNumber + 1).ToString();
                }
                strPrimaryField = PrepareTEPrimaryFieldNode(false, PrimaryKeyNumber, TypeID.ToString(), FieldCount, TableName, ObjectType, FieldName);
                TypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION").CreateNavigator().AppendChild(strPrimaryField);
                #endregion

                //Check if table already exists
                OMErrorObj = OMODSDataManager.DataProvider.PrepareCreateTableQuery(TypeDoc.OuterXml, ref CreateTableQuery);
                Monitor.Enter(Lock);
                OMErrorObj = OMODSDataManager.DataProvider.Execute(CreateTableQuery);
                Monitor.Exit(Lock);
                    
                //Add Association between the Field and NamedHierarchyId for the ODS table created.
                foreach (XmlNode FieldXMLNode in FieldList)
                {
                    if (FieldXMLNode.SelectSingleNode("ISNAMEDHIERARCHY").InnerText == "TRUE")
                    {
                        Assetlink.UniqueID.Read(ref AssociationId);

                        SchemaId = Convert.ToUInt64(FieldXMLNode.SelectSingleNode("SCHEMAID").InnerText);
                        
                        NamedHierarchyId = Convert.ToUInt64(FieldXMLNode.SelectSingleNode("NAMEDHIERARCHYID").InnerText);

                        AssoFieldNamedHierarchy AssoFieldNHObj = new AssoFieldNamedHierarchy(OMDataManager);

                        CreateErrorObj = AssoFieldNHObj.Create(AssociationId, NamedHierarchyId, SchemaId, "ASC");

                        if (CreateErrorObj != null)
                        {
                            OMErrorObj = new OMError(OMErrorCodes.ODS_NAMEFIELDHIERARCHY_ASSOCIATION_CREATION_FAILED,
                                    "NamedHierarchy Association Failed in ODS", OMErrorObjectType.ODSASSOFIELDNAMEDHIERARCHY, "AssoFieldNamedHierarchy",
                                    CreateErrorObj.ErrorType, null, CreateErrorObj.ErrorXML);
                            // WriteFailed = true;
                            return OMErrorObj;
                        }
                    }
                }

                InputXML = TypeDoc.OuterXml;
                return OMErrorObj;
            }
            catch (Exception ExcepObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.TABLE_CREATION_FAILED, "Error in Executing Operation",
                            OMErrorObjectType.OBJECTMANAGERCLASS, "ObjectManagerODSClass",
                            OMErrorType.EXCEPTION, ExcepObj, null);
                return OMErrorObj;
            }
		}

        /// <summary>
        /// This method prepares the Input XML to create a New field of the form FIELD1 or FIELD2 or FIELD3
        /// and so on ... based on the current field no attribute in MOM Schema config XML
        /// </summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <param name="addNewField">This field is used when a table is only updated with a new field. 
        /// In case a table is newly created pass false.</param>
        /// <param name="PrimaryFieldNumber">The Primary Field no is used to create the new Field, pass directly the mom schema current field no.</param>
        /// <param name="strObjectTypeID">Object Type ID will be the related FACETTYPEID/ENTITYTYPEID</param>
        /// <param name="DisplayOrder">The Display order field</param>
        /// <param name="strTableName">Actual table name in DB</param>
        /// <param name="strObjectType">Type of Object. i.e. ENTITY/FACET</param>
        /// <returns></returns>
        private String PrepareTEPrimaryFieldNode(Boolean addNewField, int PrimaryFieldNumber, String strObjectTypeID, int DisplayOrder, String strTableName, String strObjectType, String ForignKeyFieldNameForODS)
        {
            StringBuilder sbPrimaryFieldNode = null;
            ulong SchemaID = 0;

            if (String.IsNullOrEmpty(ForignKeyFieldNameForODS)) ForignKeyFieldNameForODS = String.Format("FIELD{0}", (PrimaryFieldNumber + 1).ToString());

            if (ALCommon.GenerateUniqueID(ref SchemaID))
            {
                sbPrimaryFieldNode = new StringBuilder(16);
                if (addNewField)
                    sbPrimaryFieldNode.Append("<FIELD OperationType='NEW'>");
                else
                    sbPrimaryFieldNode.Append("<FIELD>");

                sbPrimaryFieldNode.AppendFormat(@"<SCHEMAID>{0}</SCHEMAID>
			                                    <TABLENAME>{1}</TABLENAME>
			                                    <OBJECTTYPEID>{2}</OBJECTTYPEID>
			                                    <OBJECTTYPE>{3}</OBJECTTYPE>
			                                    <FIELDNAME>{4}</FIELDNAME>
			                                    <FIELDTYPE>INTEGER</FIELDTYPE>
			                                    <FIELDSIZE>8</FIELDSIZE>
			                                    <ISPRIMARYKEY>FALSE</ISPRIMARYKEY>
			                                    <ISMANDATORY>FALSE</ISMANDATORY>
			                                    <ISSEARCHFIELD>FALSE</ISSEARCHFIELD>
			                                    <ISNAMEDHIERARCHY>FALSE</ISNAMEDHIERARCHY>
			                                    <FIELDUITYPE>EDITBOX</FIELDUITYPE>
			                                    <DISPLAYNAME>{4}</DISPLAYNAME>
			                                    <DISPLAYORDER>{5}</DISPLAYORDER>
			                                    <DEFAULTVALUE/>			                                    
		                                    </FIELD>", SchemaID.ToString(), strTableName, strObjectTypeID, strObjectType, ForignKeyFieldNameForODS, (DisplayOrder + 1).ToString());
            }
            return sbPrimaryFieldNode.ToString();
        }

        /// <summary>
        /// This method creates new relation betweeen the TARGET AND RELATED ENTITY/FACET for ODS
        /// </summary>
        /// <CreatedOn>24 May 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <param name="InputXML">Relationship Input XML as given for Object Manager</param>
        /// <param name="TypeID">Target Type ID</param>
        /// <param name="strTypeName">Target Type Name</param>
        /// <param name="ContainerTypeName">This in an OUT parameter that contains the container type name used to update MOM schema XML</param>
        /// <param name="ContainerTypeXML">This in an OUT parameter that contains the new container type xml</param>
        /// <param name="ODSRelName">This in an OUT parameter that contains the new relationship name</param>
        /// <param name="NewRelXML">This in an OUT parameter that contains the new relationship details</param>
        /// <returns></returns>
        private OMError AssociateRelationship(String InputXML, String TypeID, String strTypeName, out String ContainerTypeName, out String ContainerTypeXML, out String NewRelXML)
		{
			#region Member Variables
			OMError			OMErrorObj			= null;
			String			RelationshipName	= null;
            String			RelationshipDispName= null;
            String          ContainerTypeId     = null;
            String          ContainerType       = null;            
            String          ODSObjectName       = null;            
            String          TypeOutPutXML       = null;
			XmlDocument	    RelationshipDoc		= null;
            XPathDocument   xpDoc_ODSSchemaXml  = null;
            XPathNavigator  xpNav_ODSSchemaXml  = null;
            XPathNodeIterator xpIter_MultiRelns = null;
            ulong           RelationshipTypeID  = 0;
            FacetType       FacetTypeObj        = null;
            EntityType      EntityTypeObj       = null;
            XmlNode         OdsRelForNode       = null;
			#endregion

            ContainerTypeName = String.Empty;
            ContainerTypeXML  = String.Empty;
            //ODSRelName        = String.Empty;
            NewRelXML         = String.Empty;
            
			try
			{
                RelationshipDoc = new XmlDocument();
                RelationshipDoc.LoadXml(InputXML);				

                #region Get ODSTarget object for adding the relationship in MOM database
                ContainerTypeId = RelationshipDoc.CreateNavigator().SelectSingleNode("RELATIONSHIPTYPE/CONTAINERTYPEID").Value;
                ContainerType = RelationshipDoc.CreateNavigator().SelectSingleNode("RELATIONSHIPTYPE/CONTAINERTYPE").Value;

                

                if (ContainerType == "ENTITY")
                {
                    EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj = EntityTypeObj.Read(Convert.ToUInt64(ContainerTypeId), ref TypeOutPutXML);
                    EntityTypeObj = null;
                }
                else if (ContainerType == "FACET")
                {
                    FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj = FacetTypeObj.Read(Convert.ToUInt64(ContainerTypeId), ref TypeOutPutXML);
                    FacetTypeObj = null;
                }

                if (OMErrorObj != null)
                {
                    return OMErrorObj;
                }

                if (OMErrorObj == null)
                {
                    ContainerTypeName   = ALCommon.GetInnerValue(ref TypeOutPutXML, "<TYPENAME>", "</TYPENAME>");
                    ContainerTypeXML    = TypeOutPutXML;                    
                    xpDoc_ODSSchemaXml = new XPathDocument(ODSConfigXMLPath);
                    if (xpDoc_ODSSchemaXml != null)
                    {
                        xpNav_ODSSchemaXml = xpDoc_ODSSchemaXml.CreateNavigator();
                        if (ContainerType.ToUpper() == "ENTITY")
                        {
                            XPathNavigator xpNav_Temp = xpNav_ODSSchemaXml.SelectSingleNode("ODSMOMSchemaConfiguration/TargetMOMObjectsForODS/MOMObject[@MOMObjectTypeName='" + ContainerTypeName + "']");
                            if (xpNav_Temp != null)
                            {
                                ODSObjectName = xpNav_Temp.SelectSingleNode("ODSObject").GetAttribute("ODSObjectTypeName", "");
                                
                                if (xpNav_Temp.GetAttribute("IsMultipleRelations", "").ToUpper() == "TRUE")
                                {
                                    xpIter_MultiRelns = xpNav_Temp.Select("ODSObject");                                    
                                }
                            }
                        }
                        else
                        {
                            XPathNavigator xpNav_Temp = xpNav_ODSSchemaXml.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeName='" + ContainerTypeName + "']");
                            if (xpNav_Temp != null)
                            {
                                ODSObjectName = xpNav_Temp.GetAttribute("ODSObjectTypeName", "");
                            }
                        }
                    }

                    if (String.IsNullOrEmpty(ODSObjectName))
                    {
                        OMErrorObj = new OMError(OMErrorCodes.ODS_OBJECT_NOT_FOUND, "ODS Object for MOM Object:" + " " + ContainerTypeName + "not found.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ContainerTypeName, OMErrorType.ERROR, null, null);
                        return OMErrorObj;
                    }
                    
                    if (ContainerType == "ENTITY")
                    {
                        EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        OMErrorObj = EntityTypeObj.Read(ODSObjectName, ref TypeOutPutXML);
                    }
                    else if (ContainerType == "FACET")
                    {
                        FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        OMErrorObj = FacetTypeObj.Read(ODSObjectName, ref TypeOutPutXML);
                    }

                    if (OMErrorObj != null)
                    {                        
                        return OMErrorObj;
                    }


                    if (OMErrorObj == null)
                    {
                        ContainerTypeId = ALCommon.GetInnerValue(ref TypeOutPutXML, "<TYPEID>", "</TYPEID>");
                    }
                }
                #endregion

                RelationshipName = RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPNAME").InnerXml;
                RelationshipDispName = RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPDISPLAYNAME").InnerText;
                if (xpIter_MultiRelns != null)
                {
                    while (xpIter_MultiRelns.MoveNext())
                    {
                        RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/CONTAINSTYPEID").InnerXml = TypeID;
                        RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/CONTAINERTYPEID").InnerXml = ContainerTypeId;
                        
                        RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPNAME").InnerXml = "ODS_" + RelationshipName + "_" + xpIter_MultiRelns.Current.GetAttribute("ODSRelationshipFor","");
                        RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPDISPLAYNAME").InnerText = RelationshipDispName + " " + xpIter_MultiRelns.Current.GetAttribute("ODSRelationshipFor", "");
                            
                        if (ALCommon.GenerateUniqueID(ref RelationshipTypeID))
                        {
                            RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPTYPEID").InnerXml = RelationshipTypeID.ToString();
                        }
                        OMErrorObj = CreateRecord("ALOM_RELATIONSHIPTYPES", RelationshipDoc.OuterXml);
                        if (OMErrorObj != null)
                            return OMErrorObj;
                        OdsRelForNode = RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/ODSRELATIONSHIPFOR");
                        if (OdsRelForNode == null)
                        {
                            OdsRelForNode = RelationshipDoc.CreateElement("ODSRELATIONSHIPFOR");
                            RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE").AppendChild(OdsRelForNode);
                        }
                        
                        OdsRelForNode.InnerText = xpIter_MultiRelns.Current.GetAttribute("ODSRelationshipFor", "");                        
                        NewRelXML += RelationshipDoc.OuterXml;
                    }
                }
                else
                {

                    RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/CONTAINSTYPEID").InnerXml = TypeID;
                    RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/CONTAINERTYPEID").InnerXml = ContainerTypeId;
                    //RelationshipName = RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPNAME").InnerXml;
                    RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPNAME").InnerXml = "ODS_" + RelationshipName;
                    //RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPDISPLAYNAME").InnerXml = "ODS_" + RelationshipName;
                    if (ALCommon.GenerateUniqueID(ref RelationshipTypeID))
                    {
                        RelationshipDoc.SelectSingleNode("RELATIONSHIPTYPE/RELATIONSHIPTYPEID").InnerXml = RelationshipTypeID.ToString();
                    }

                    OMErrorObj = CreateRecord("ALOM_RELATIONSHIPTYPES", RelationshipDoc.OuterXml);
                    NewRelXML = RelationshipDoc.OuterXml;
                }
                //ODSRelName = "ODS_" + RelationshipName;
                
                
				return OMErrorObj;
			}
			catch(Exception ExcepObj)
			{
				OMErrorObj = new OMError(OMErrorCodes.ASSOCIATION_CREATION_FAILED, "Association Creation Failed",
							OMErrorObjectType.ASSOCIATION, "ObjectManagerODSClass",
							OMErrorType.EXCEPTION, ExcepObj, null);
				return OMErrorObj;
			}
		}

        /// <summary>
        /// This method adds an entry in the ENTITYTYPE/FACETTYPE table based on the object type parameter
        /// </summary>
        /// <CreatedOn>24 MAY 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <param name="TypeXML">This is used to generate the insert query for the object type</param>
        /// <param name="TableName">The table name of the ENTITYTYPE/FACETTYPE</param>
        /// <param name="ObjectType">The Type of object i.e. ENTITY/FACET</param>
        /// <param name="TypeName">ENTITY or FACET TYPE NAME</param>
        /// <returns></returns>
        private OMError AddObjectTypeEntry(String TypeXML, String TableName, String ObjectType, String TypeName)
        {
            #region Member Variables
            OMError     OMErrorObj = null;  
            XmlDocument xmlDoc_TypeXml = null;
            XmlDocument FacetTypeXmlDoc = null;
            XmlDocument EntityTypeXmlDoc = null;
            String TempQuery = null;
            #endregion

            try
            {
                xmlDoc_TypeXml = new XmlDocument();
                xmlDoc_TypeXml.LoadXml(TypeXML);
                XmlNode TypeNode = xmlDoc_TypeXml.SelectSingleNode("TYPE");

                switch (ObjectType.ToUpper())
                {
                    case "ENTITY":
                        EntityType EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        if (EntityTypeObj.isEntityTypeExists(TypeName) == true) //Check whether the given EntityType already exists in the system.
                        {
                            OMErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                                                    "EntityType Already Exists", OMErrorObjectType.ENTITYTYPE,
                                                    "EntityType", OMErrorType.ERROR, null, null);                            
                            return OMErrorObj;
                        }

                        EntityTypeObj.generateEntityTypeXML(TypeNode, TableName, ref EntityTypeXmlDoc);

                        OMErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_ENTITYTYPE",
                                                                         EntityTypeXmlDoc.InnerXml,
                                                                         ref TempQuery);
                        if (OMErrorObj == null)
                        {
                            OMErrorObj = OMDataManager.DataProvider.Execute(TempQuery);
                            if (OMErrorObj != null)
                                OMErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                                            "Entity Type Creation Failed", OMErrorObjectType.ENTITYTYPE,
                                            "EntityTypeCreation", OMErrorType.ERROR, null, null);
                        }
                        else
                        {
                            OMErrorObj = new OMError(OMErrorCodes.ENTITY_TYPE_CREATION_FAILED,
                                            "Entity Type Creation Failed", OMErrorObjectType.ENTITYTYPE,
                                            "EntityTypeCreation", OMErrorType.ERROR, null, null);
                        }

                        break;
                    case "FACET":
                        FacetType FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        if (FacetTypeObj.isFacetTypeExists(TypeName) == true) //Check whether the given FacetType already exists in the system.
                        {
                            OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                                            "FacetType Already Exists", OMErrorObjectType.FACETTYPE,
                                            "FacetType", OMErrorType.ERROR, null, null);
                            return OMErrorObj;
                        }

                        FacetTypeObj.generateFacetTypeXML(TypeNode, TableName, ref FacetTypeXmlDoc);

                        OMErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_FACETTYPE",
                                                                         FacetTypeXmlDoc.InnerXml,
                                                                         ref TempQuery);
                        if (OMErrorObj == null)
                        {
                            OMErrorObj = OMDataManager.DataProvider.Execute(TempQuery);
                            if(OMErrorObj!=null)
                                OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                                            "Facet Type Creation Failed", OMErrorObjectType.FACETTYPE,
                                            "FacetTypeCreation", OMErrorType.ERROR, null, null);
                        }
                        else
                        {
                            OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED,
                                            "Facet Type Creation Failed", OMErrorObjectType.FACETTYPE,
                                            "FacetTypeCreation", OMErrorType.ERROR, null, null);
                        }
                        break;
                }
                return OMErrorObj;
            }
            catch (Exception ExcepObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED, "Object Type Creation Failed",
                            OMErrorObjectType.FACETTYPE, "ObjectManagerClass",
                            OMErrorType.EXCEPTION, ExcepObj, null);
                return OMErrorObj;
            }
        }

        /// <summary>
        /// This method creates schema in MOM DB 
        /// for the ODS table created in ODS DB
        /// </summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <param name="TypeXML">Type information</param>
        /// <returns>An ERROR Object if an Error Occurs</returns>
        private OMError AddSchemaEntry(String TypeXML)
        {
            #region Member Variables
            OMError         OMErrorObj                  = null;
            XmlDocument     xmlDoc_TypeXml              = null;
            XmlNode         FieldCollectionNode         = null;
            XmlNode         SchemaNode                  = null;
            XmlNodeList     FieldNodes                  = null;
            String          TempQuery                   = null;
            #endregion

            try
            {
                FacetType FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                xmlDoc_TypeXml = new XmlDocument();
                xmlDoc_TypeXml.LoadXml(TypeXML);

                FieldCollectionNode = xmlDoc_TypeXml.SelectSingleNode("TYPE/FIELDCOLLECTION");
                FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
                if (FieldNodes.Count > 0)
                {
                    foreach (XmlNode FieldNode in FieldNodes)
                    {
                        SchemaNode = null;
                        FacetTypeObj.generateSchemaXML(xmlDoc_TypeXml, FieldNode, ref SchemaNode);

                        OMErrorObj = OMDataManager.DataProvider.PrepareInsertQuery("ALOM_SCHEMA", SchemaNode.OuterXml,
                                                                                    ref TempQuery);
                        if (OMErrorObj == null)
                        {
                            OMErrorObj = OMDataManager.DataProvider.Execute(TempQuery);
                            if (OMErrorObj != null)
                                return OMErrorObj;
                        }
                        else
                        {
                            return OMErrorObj;
                        }
                    }
                }
                return OMErrorObj;
            }
            catch (Exception ExcepObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_CREATION_FAILED, "Object Type Creation Failed",
                            OMErrorObjectType.FACETTYPE, "ObjectManagerClass",
                            OMErrorType.EXCEPTION, ExcepObj, null);
                return OMErrorObj;
            }
        }

		private OMError DisAssociateRelationship(String RelationshipName)
		{
			#region Member Variables
			OMError			OMErrorObj			= null;
			XPathDocument	RelationshipDoc		= null;
			XPathNavigator	xpNavRelationship	= null;
			String			OutputXML			= null;
			String			ContainsTypeID		= null;
			String			ContainsType		= null;
			String			TypeOutputXML		= null;
			String			TableName			= null;
			#endregion

			try
			{
				OMErrorObj = OMODSDataManager.DataProvider.AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING,
									OMOperator.EQUALTO, RelationshipName,
									OMConjunction.NOTHING);

				if(OMErrorObj == null)
				{
					OMErrorObj = ReadRecord("RELATIONSHIPTYPE", "ALOM_RELATIONSHIPTYPES", null,
																	ref OutputXML);

					if (OMErrorObj == null && (!String.IsNullOrEmpty(OutputXML)))
					{
						RelationshipDoc		= new XPathDocument(new StringReader(OutputXML));
						xpNavRelationship	= RelationshipDoc.CreateNavigator();

						ContainsTypeID	= xpNavRelationship.SelectSingleNode("RELATIONSHIPTYPE/CONTAINSTYPEID").Value;
						ContainsType	= xpNavRelationship.SelectSingleNode("RELATIONSHIPTYPE/CONTAINSTYPE").Value;

						if (ContainsType == "ENTITY")
						{
							#region ENTITY
							OMErrorObj = OMDataManager.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ContainsTypeID, OMConjunction.NOTHING);

							if (OMErrorObj == null)
							{
								OMErrorObj = OMDataManager.DataProvider.ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,ref TypeOutputXML);

								if (OMErrorObj == null && (!String.IsNullOrEmpty(OutputXML)))
								{
									RelationshipDoc		= new XPathDocument(new StringReader(TypeOutputXML));
									xpNavRelationship	= RelationshipDoc.CreateNavigator();
									TableName			= xpNavRelationship.SelectSingleNode("ENTITYTYPE/TABLENAME").Value;
								}
							}
							#endregion
						}
						else
						{
							#region FACET
							OMErrorObj = OMDataManager.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ContainsTypeID, OMConjunction.NOTHING);

							if(OMErrorObj == null)
							{
								OMErrorObj = OMDataManager.DataProvider.ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
																			ref TypeOutputXML);

								if (OMErrorObj == null && (!String.IsNullOrEmpty(OutputXML)))
								{
									RelationshipDoc		= new XPathDocument(new StringReader(TypeOutputXML));
									xpNavRelationship	= RelationshipDoc.CreateNavigator();
									TableName			= xpNavRelationship.SelectSingleNode("FACETTYPE/TABLENAME").Value;
								}
							}
							#endregion
						}

						#region Delete Record from ALOM_RELATIONSHIPTYPES and Delete ENTITY/FACET table from ODS Database
						if (OMErrorObj == null)
						{
							OMErrorObj = OMODSDataManager.DataProvider.AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, RelationshipName, OMConjunction.NOTHING);

							if (OMErrorObj == null)
							{
								OMErrorObj = DeleteRecord("ALOM_RELATIONSHIPTYPES");

								if (OMErrorObj == null)
								{
									Monitor.Enter(Lock);
									OMErrorObj = DropTable(TableName);
									Monitor.Exit(Lock);
								}
							}
						}
						#endregion
					}
				}
				return OMErrorObj;
			}
			catch(Exception ExcepObj)
			{
				OMErrorObj = new OMError(OMErrorCodes.ASSOCIATION_CREATION_FAILED, "Association Creation Failed",
							OMErrorObjectType.ASSOCIATION, "ObjectManagerODSClass",
							OMErrorType.EXCEPTION, ExcepObj, null);
				return OMErrorObj;
			}
		}

		/// <summary>
		/// This function is used to prepare the Insert Query based on the InputXML.
		/// The Insert query is prepared according to different datatypes used. 
		/// For fields having DateTime as datatype the date is converted to make it 
		/// compatible with SQL and Oracle by calling the ConvertDate function.
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter.This 
		/// specifies which table the record has to be inserted</param>
		/// <param name="InputXML">This function takes InputXML as parameter.
		/// The structure of the XML will be like this
		/// <Plan>
		///		<PlanName>Plan F'or Tes'ting</PlanName>
		///		<Description>This is for Testing</Description>
		///		<Age>25</Age>
		///		<DOB>
		///		<Day>3</Day>
		///		<Month>10</Month>
		///		<Year>1979</Year>
		///		</DOB>
		/// </Plan>
		/// </param>
		/// <returns>
		/// This function returns OMError object if there is any error occured.
		/// </returns>
        public OMError CreateRecord(string TableName, string InputXML)
		{
			#region Member Variables
			XmlNode             DayNode;					//This contains the Day node which specifies the day in the date in InputXML
            XmlNode             MonthNode;					//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode             YearNode;					//This contains the Year node which specifies the Year in the date in InputXML
            string              DataType = "";				//The Mapped Datatype between system and Database (SQL or Oracle) is saved in this.
            string              Seperator = "";				//This contains the seperator value like "," which is used for query formation
            string              FieldSize = "";				//This has the field size of the individual field.This is used for truncation of data
            XmlDocument         InputXMLDocument;			//This Contains the Main Document node of the InputXML
            Hashtable           FieldNameValueCollection;	//HashTable variable contains the FieldName,FieldType pair retrieved form the ALOM_Schema
            XmlElement          RootNode;					//Contains the Root node in the InputXML
            StringBuilder       FieldList;					//The FieldList represents the list of fields.This is used to append in the Insert Query 
            StringBuilder       FieldValues;				//The FieldValues represents the list of field vlaues.This is used to append in the Insert Query 
            StringBuilder       DateString;						//The Value of the Date is saved in this variable
            OMError             ErrorObj = null;			//In case there is an error in the function this object will be populated
            XmlNode             HourNode;					//This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode             MinuteNode;					//This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode             SecondNode;					//This contains the Second node which specifies the Second in the date in InputXML
            StringBuilder       QuerySB = null;
            FieldInformation    FieldDataObj = new FieldInformation(); //This is the Field Object stored in the struct, It contains the value of the FieldName and FieldType

            //For XML data type support. Code added on date 7th March 2006 by Kiran.
            String              PrimaryKeyFieldName         = null;
            String              ObjectTypeName              = null;
            String              ObjectType                  = null;
            ulong               PrimaryKeyValue             = 0;
            ulong               ObjectTypeID                = 0;
            //IXMLRepository      ObjXMLManager               = null;


            //Variables to support Currency As Data Type
            XmlNode TypeNode;						//This contains the Type node which specifies the Currency Type in the InputXML
            XmlNode ValueNode;						//This contains the Value node which specifies the Value in the InputXML
			#endregion

            try
            {
                //StringDictionary FieldNameValueCollection = new StringDictionary(); 
                // Needs to change CaseInsensitiveHashCodeProvider to IEqualityComparer
                //In this code section the records from the ALOM_Schema is retrieved based on the TableName and kept in the HashTable. 
                //Later to determine the DataType of the Field this Hashtable is used.


                ErrorObj = OMDataManager.DataProvider.GetObjectTypeSchemaDetails(TableName, out PrimaryKeyFieldName, out ObjectTypeID,
                            out ObjectType, out ObjectTypeName, out FieldNameValueCollection);

                if (ErrorObj != null)
                    return ErrorObj;

                QuerySB = new StringBuilder();
                FieldList = new StringBuilder();
                FieldValues = new StringBuilder();
                DateString = new StringBuilder();
                //ObjXMLManager = OMDataManager.XMLRepository; //new XMLManager(TenantName);
                FieldDataObj = new FieldInformation();

                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);
                RootNode = InputXMLDocument.DocumentElement;

                //Get the primary key value. This is for XML data type.
                if ((PrimaryKeyFieldName != null) && (PrimaryKeyFieldName.Length > 0))
                {
                    if (RootNode.SelectSingleNode(PrimaryKeyFieldName) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName).InnerText);
                    else if (RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()) != null)
                        PrimaryKeyValue = Convert.ToUInt64(RootNode.SelectSingleNode(PrimaryKeyFieldName.ToUpper()).InnerText);
                }

                QuerySB.AppendFormat("INSERT INTO {0} (", TableName.ToUpper());
                foreach (XmlElement ChildNode in RootNode.ChildNodes)
                {
                    if (FieldNameValueCollection.ContainsKey((ChildNode.Name).Trim()))
                    {
                        FieldDataObj = (FieldInformation)FieldNameValueCollection[(ChildNode.Name).Trim()];
                        DataType = FieldDataObj.FieldType;

                        if ((DataType.ToUpper() != "XML") &&
                            (DataType.ToUpper() != "BINDER") &&
                            (DataType.ToUpper() != "FILE"))
                        {
                            if (DataType.ToUpper() != "CURRENCY")
                                FieldList.AppendFormat("{0} {1}", Seperator, (ChildNode.Name).ToUpper());
                            FieldSize = FieldDataObj.FieldSize;

                            switch (DataType.ToUpper())
                            {
                                case "STRING":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0} N'{1}' ", Seperator, OMDataManager.DataProvider.ReplaceSingleQuote(OMDataManager.DataProvider.TruncateData(ChildNode.InnerText, Convert.ToInt32(FieldSize))));
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}'' ", Seperator);
                                    break;

                                case "XMLTEXT":
                                    if (!(ChildNode.InnerXml == null) && !(ChildNode.InnerXml == ""))
                                        FieldValues.AppendFormat("{0} N'{1}' ", Seperator, OMDataManager.DataProvider.TruncateData(OMDataManager.DataProvider.ReplaceSingleQuote(ChildNode.InnerXml), Convert.ToInt32(FieldSize)));
                                    break;

                                case "BIGINT":
                                case "INTEGER":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}{1} ", Seperator, Convert.ToString(ChildNode.InnerText));
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "BOOLEAN":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0} N'{1}' ", Seperator, ChildNode.InnerText);
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "FLOAT":
                                case "DECIMAL":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}{1} ", Seperator, Convert.ToString(ChildNode.InnerText));
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}NULL ", Seperator);
                                    break;

                                case "DATETIME":
                                    DayNode = ChildNode.SelectSingleNode("DAY");
                                    MonthNode = ChildNode.SelectSingleNode("MONTH");
                                    YearNode = ChildNode.SelectSingleNode("YEAR");
                                    HourNode = ChildNode.SelectSingleNode("HOUR");
                                    MinuteNode = ChildNode.SelectSingleNode("MINUTE");
                                    SecondNode = ChildNode.SelectSingleNode("SECOND");

                                    if (MonthNode != null && DayNode != null && YearNode != null)
                                    {
                                        if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                                            DateString.AppendFormat("{0}/{1}/{2} {3}:{4}:{5}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText,
                                                        HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                                        else
                                            DateString.AppendFormat("{0}/{1}/{2}",
                                                        MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

                                        if ((MonthNode.InnerText != null && MonthNode.InnerText != "") && (DayNode.InnerText != null && DayNode.InnerText != "") && (YearNode.InnerText != null && YearNode.InnerText != ""))
                                            FieldValues.AppendFormat("{0}{1} ", Seperator, OMDataManager.DataProvider.ConvertDate(DateString.ToString()));
                                        else
                                            FieldValues.AppendFormat("{0}{1} ", Seperator, "null");
                                        DateString.Remove(0, DateString.Length);
                                    }
                                    else
                                        FieldValues.AppendFormat("{0}{1}", Seperator, "null");

                                    break;

                                //For CURRENCY data type support. Code added on date 14th June 2006 by Pallavi
                                case "CURRENCY":
                                    TypeNode = ChildNode.SelectSingleNode("CURRENCY");
                                    ValueNode = ChildNode.SelectSingleNode("AMOUNT");
                                    FieldList.AppendFormat("{0} {1}_C , {2}_A", Seperator, (ChildNode.Name).ToUpper(), (ChildNode.Name).ToUpper());

                                    if (TypeNode != null && ValueNode != null)
                                    {
                                        FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, TypeNode.InnerText == "" ? "NULL" : TypeNode.InnerText, ValueNode.InnerText == "" ? "NULL" : ValueNode.InnerText);
                                    }
                                    else if (TypeNode != null && ValueNode == null)
                                    {
                                        FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, TypeNode.InnerText == "" ? "NULL" : TypeNode.InnerText, "NULL");
                                    }
                                    else
                                    {
                                        FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, "NULL", "NULL");
                                    }
                                    //CurrencyString.AppendFormat("{0}_T =N'{1}'{2}{3}_V={4}", ChildNode.Name, TypeNode.InnerText, Seperator, ChildNode.Name, ValueNode.InnerText);
                                    //FieldList.AppendFormat("{0} {1}_C , {2}_A", Seperator, (ChildNode.Name).ToUpper(),  (ChildNode.Name).ToUpper());
                                    //FieldValues.AppendFormat("{0}N'{1}',{2}", Seperator, TypeNode.InnerText, ValueNode.InnerText);
                                    //Query.AppendFormat("{0}{1} ", Seperator, CurrencyString);
                                    //CurrencyString.Remove(0, CurrencyString.Length);
                                    break;
                                case "TEXT":
                                    if (!(ChildNode.InnerText == null) && !(ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0} N'{1}' ", Seperator, OMDataManager.DataProvider.ReplaceSingleQuote(ChildNode.InnerText));
                                    else if ((ChildNode.InnerText == null) || (ChildNode.InnerText == ""))
                                        FieldValues.AppendFormat("{0}'' ", Seperator);
                                    break;

                            }
                            Seperator = ",";
                        }
                        //If column data type is XML then create an XML file and add XML data into the
                        //newly created file by using XML manager. This is for XML data type.
                        //else if ((DataType.ToUpper() == "XML") && (ObjectTypeName != null) && (PrimaryKeyValue != 0))
                        //    ErrorObj = ObjXMLManager.Save(ObjectTypeName, PrimaryKeyValue, ChildNode.Name.Trim().ToUpper(), ChildNode.InnerXml);
                    }
                }

                QuerySB.AppendFormat("{0}) VALUES ({1})", FieldList, FieldValues);

                Monitor.Enter(Lock);
				//The Execute method takes the Query as parameter and the query is executed.
                ErrorObj = OMDataManager.DataProvider.Execute(QuerySB.ToString());
                OMDataManager.DataProvider.RemoveFromCache(TableName);
                OMODSDataManager.DataProvider.RemoveFromCache(TableName);
				Monitor.Exit(Lock);
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.CREATE_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
            }
            return (ErrorObj);
		}

		/// <summary>
		/// This function is used to select records based on the criteria.
		/// This function takes TypeName as parameter for e.g. Plan,Item etc.
		/// This function also takes FieldListXML as parameter which will indicate the 
		/// Fields to retrieve in the select statement.If the parameter is blank then all 
		/// the fields will be fetched.
		/// This function also takes the OutputXML as parameter. If the number of records 
		/// is greater than 1 then the FieldCollection is returned else only detail of one 
		/// record is given.
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter.This 
		/// specifies from which table to fetch the records.</param>
		/// <param name="FieldListXML">This function takes FieldListXML as parameter.
		/// For the structure of the XML refer to the FieldCollection.XSD
		/// </param>
		/// <returns>
		/// This function returns OMError object if there is any error occured.
		/// </returns>
		public OMError ReadRecord(string TypeName, string TableName, string FieldListXML, ref string OutputXML)
		{
			string			FieldName="";				//This has the field name of the individual field.This field name is used for formation of OutputXML to return the data back
			string			FieldValue="";				//This has the field Value of the individual field.This field value is used for formation of OutputXML to return the data back
			string			Seperator= "";				//This contains the seperator value like "," which is used for query formation
			string			SortType="";				//This decides whether the data will be fetched in ascending or descending manner
			XmlNodeList		FieldNodes=null;			//This hold the list of nodes in the FieldCollection node in the Input XML
			XmlNode			FieldCollectionNode = null;	//This has the FieldCollectionNode which holds all the information about the fields
			XmlNode			FieldNameNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			XmlNode			ObjectNode = null;			//This is the Type name node
			XmlNode			SortTypeNode = null;		//The records are selected based on the sortType (Ascending/Descending) specified in the InputXML.This contains the SortType node in the InputXML
			XmlNodeList		SortOrderNodes = null;		//The records are selected based on the sortOrder specified in the InputXML.This contains the SortOrder node in the InputXML
			XmlElement		OutputFieldNode = null;		//This variable holds the FieldName Node of the individual field node specified in the FieldCollection
			int				index=0;					//If the String is found in another string then Index will be 0 else it will be -1
			StringBuilder	Query = null;				//The select query formed is saved in this variable
			StringBuilder	FieldList = null;			//The FieldList represents the list of fields.This is used to append in the Insert Query 
			XmlDocument		InputXMLDocument = null;	//This Contains the Main Document node of the InputXML
			OMError			ErrorObj = null;			//In case there is an error in the function this object will be populated
			StringBuilder	SortClause = new StringBuilder();
			XmlNode			HourNode;					//This contains the Hour node which specifies the Hour in the date
			XmlNode			MinuteNode;					//This contains the Minute node which specifies the Minute in the date 
			XmlNode			SecondNode;					//This contains the Second node which specifies the Second in the date 
			XmlNode			DayNode;					//This contains the Day node which specifies the day in the date 
			XmlNode			MonthNode;					//This contains the Month node which specifies the Month in the date 
			XmlNode			YearNode;					//This contains the Year node which specifies the Year in the date 
			DateTime		DateValue;
			string			FieldType = "";				//This has the field Data Type of the individual field.This field type is used for formation of OutputXML to return the data back
            bool            CacheEnabled =  false;
            String          cachedItemName  = null;
			try
            {
                #region Read From Cache
                CacheEnabled    = true;//Convert.ToBoolean(ConfigurationManager.AppSettings["OMCacheEnabled"]);
                cachedItemName  = TenantName.ToUpper() + "_" + TableName.ToUpper();

                if (CacheEnabled == true &&
                    CacheManager.IsCached(cachedItemName) == true)
                {
                    ErrorObj = OMDataManager.DataProvider.ReadFromCache(false, cachedItemName, TypeName, FieldListXML, ref  OutputXML);
                    if (ErrorObj == null)
                        return null;

                }
                #endregion Read From Cache

                /*If the FieldListXML is blank then select all the fields of the given TypeName 
				else select the fields specified in the InputXML*/
				Query = new StringBuilder();
				if ((FieldListXML == "") || (FieldListXML==null))
				{
					Query.AppendFormat("SELECT TOP 1 * FROM {0}", TableName.ToUpper());
				}
				else
				{
					FieldList           = new StringBuilder();
					InputXMLDocument    = new XmlDocument();
					InputXMLDocument.LoadXml(FieldListXML);
					Query.AppendFormat("SELECT TOP 1 ");
					
					FieldCollectionNode = InputXMLDocument.SelectSingleNode("FIELDCOLLECTION");
					FieldNodes = FieldCollectionNode.SelectNodes("FIELD");
					Seperator = "";
					foreach (XmlElement FieldNode in FieldNodes)
					{
						FieldNameNode = FieldNode.SelectSingleNode("FIELDNAME");
                        FieldList.AppendFormat("{0}{1} ", Seperator, OMODSDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
						Query.AppendFormat("{0}{1} ", Seperator, OMODSDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved((FieldNameNode.InnerText).ToUpper()));
						Seperator = ",";
					}

					/* While selecting the records the consumer can demand for all the fields or selected fields. 
					The mandatory fields also needs to be returned along with the Fields asked for, otherwise the Solution 
					layer will have problem.Hence the mandatory fields are selected from the ALOM_Schema table*/

					StringBuilder QuerySB = new StringBuilder();
					QuerySB.AppendFormat("Select FIELDNAME FROM ALOM_SCHEMA WHERE TABLENAME ='{0}' AND Upper(ISMANDATORY) = 'TRUE'", TableName);

                    try
                    {
                        ErrorObj = OMDataManager.DataProvider.RetrieveResult(QuerySB.ToString());
                        if (ErrorObj != null)
                        {
							OMDataManager.DataProvider.m_Clause = "";
							OMDataManager.DataProvider.m_SortClause = "";
							OMDataManager.DataProvider.m_ClauseForCache = "";
							OMDataManager.DataProvider.m_SortClauseForCache = "";
                            return ErrorObj;
                        }
                        while (OMDataManager.DataProvider.DataReaderObj.Read())
                        {
							FieldName = OMDataManager.DataProvider.DataReaderObj.GetValue(0).ToString();
                            index = FieldList.ToString().IndexOf(FieldName.ToUpper());
                            // Determine if a valid index is returned. 
                            if (index == -1)
                            {
								Query.AppendFormat("{0}{1} ", Seperator, OMDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved(FieldName.ToUpper()));
                            }
                        }
                    }
                    catch
                    {
                        throw;
                    }
                    finally
                    {
						if (OMDataManager.DataProvider.DataReaderObj != null && !OMDataManager.DataProvider.DataReaderObj.IsClosed)
							OMDataManager.DataProvider.DataReaderObj.Close();
                    }

					/* The consumer might specify the Sort order and SortType while selecting the records
					this part of the code prepares the order by Clause which will be used in the select Query*/
					
					SortOrderNodes = FieldCollectionNode.SelectNodes("FIELD/SORTORDER");
					Seperator = "";
					for (int NodeCount = 0; NodeCount  < SortOrderNodes.Count; NodeCount ++)
					{
						FieldNodes = FieldCollectionNode.SelectNodes("FIELD[SORTORDER='" + NodeCount + "']");
						if (FieldNodes.Count > 0)
						{
							FieldNameNode = FieldNodes[0].SelectSingleNode("FIELDNAME");
							SortTypeNode  = FieldNodes[0].SelectSingleNode("SORTTYPE");
							if ((SortTypeNode.InnerXml).ToUpper() == "ASCENDING")
							{
								SortType = "ASC";
							}
							else
							{
								SortType = "DESC";
							}
							SortClause.AppendFormat("{0} {1} {2}", Seperator, OMODSDataManager.DataProvider.ChangeFieldNameIncaseItIsReserved(FieldNameNode.InnerXml.ToUpper()), SortType);
							Seperator = ",";
						}
					}
					Query.AppendFormat("FROM {0}", TableName.ToUpper());
				}
				if ((OMODSDataManager.DataProvider.m_Clause != "") && (OMODSDataManager.DataProvider.m_Clause != null))
				{
					Query.AppendFormat(" WHERE {0}", OMODSDataManager.DataProvider.m_Clause);
				}
				if ((SortClause.ToString() != "") && (SortClause.ToString() != null))
				{
					Query.AppendFormat(" ORDER BY {0}", SortClause.ToString());
				}

                /* Based on the Query the Records are fetched. If the number of records is greater than one 
				 * the collection of type nodes is formed and returned else only a sicgle Type node is returned*/
                try
                {
                    ErrorObj = OMODSDataManager.DataProvider.RetrieveResult(Query.ToString());
                    if (ErrorObj != null)
                    {
						OMODSDataManager.DataProvider.m_Clause = "";
						OMODSDataManager.DataProvider.m_SortClause = "";
						OMODSDataManager.DataProvider.m_ClauseForCache = "";
						OMODSDataManager.DataProvider.m_SortClauseForCache = "";
                        return ErrorObj;
                    }


                    int RowCount = 0;
                    XmlDocument OutputXMLDocument = new XmlDocument();
                    ObjectNode = OutputXMLDocument.CreateNode(XmlNodeType.Element, TypeName.ToUpper(), "");
                    OutputXMLDocument.AppendChild(ObjectNode);
					while (OMODSDataManager.DataProvider.DataReaderObj.Read())
                    {
						for (int FieldCount = 0; FieldCount < OMODSDataManager.DataProvider.DataReaderObj.FieldCount; FieldCount++)
                        {
							FieldType = OMODSDataManager.DataProvider.DataReaderObj.GetFieldType(FieldCount).ToString();
							FieldName = OMODSDataManager.DataProvider.DataReaderObj.GetName(FieldCount);
							FieldValue = OMODSDataManager.DataProvider.DataReaderObj.GetValue(FieldCount).ToString();

                            OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                            OutputFieldNode.InnerText = "";

                            if ((FieldValue != null) && (FieldValue != ""))
                            {
                                switch (FieldType)
                                {
                                    case "System.DateTime":
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        DayNode = OutputXMLDocument.CreateElement("DAY");
                                        DateValue = Convert.ToDateTime(FieldValue);
                                        DayNode.InnerText = DateValue.Day.ToString();
                                        OutputFieldNode.AppendChild(DayNode);
                                        MonthNode = OutputXMLDocument.CreateElement("MONTH");
                                        MonthNode.InnerText = DateValue.Month.ToString();
                                        OutputFieldNode.AppendChild(MonthNode);
                                        YearNode = OutputXMLDocument.CreateElement("YEAR");
                                        YearNode.InnerText = DateValue.Year.ToString();
                                        OutputFieldNode.AppendChild(YearNode);
                                        if (DateValue.Hour != 0)
                                        {
                                            HourNode = OutputXMLDocument.CreateElement("HOUR");
                                            HourNode.InnerText = DateValue.Hour.ToString();
                                            OutputFieldNode.AppendChild(HourNode);
                                            MinuteNode = OutputXMLDocument.CreateElement("MINUTE");
                                            MinuteNode.InnerText = DateValue.Minute.ToString();
                                            OutputFieldNode.AppendChild(MinuteNode);
                                            SecondNode = OutputXMLDocument.CreateElement("SECOND");
                                            SecondNode.InnerText = DateValue.Second.ToString();
                                            OutputFieldNode.AppendChild(SecondNode);
                                        }
                                        break;
                                    default:
                                        //OutputFieldNode = OutputXMLDocument.CreateElement(FieldName.ToUpper());
                                        OutputFieldNode.InnerText = FieldValue;
                                        break;
                                }
                            }
                            ObjectNode.AppendChild(OutputFieldNode);
                        }
                        RowCount = RowCount + 1;
                    }

                    if (RowCount == 0)
                        OutputXML = null;
                    else
                        OutputXML = ObjectNode.OuterXml;

					OMODSDataManager.DataProvider.m_Clause = "";
					OMODSDataManager.DataProvider.m_SortClause = "";
					OMODSDataManager.DataProvider.m_ClauseForCache = "";
					OMODSDataManager.DataProvider.m_SortClauseForCache = "";
					OMDataManager.DataProvider.m_Clause = "";
					OMDataManager.DataProvider.m_SortClause = "";
					OMDataManager.DataProvider.m_ClauseForCache = "";
					OMDataManager.DataProvider.m_SortClauseForCache = "";
                }
                catch
                {
                    throw;
                }
                finally
                {
					if (OMODSDataManager.DataProvider.DataReaderObj != null && !OMODSDataManager.DataProvider.DataReaderObj.IsClosed)
						OMODSDataManager.DataProvider.DataReaderObj.Close();

					if (OMDataManager.DataProvider.DataReaderObj != null && !OMDataManager.DataProvider.DataReaderObj.IsClosed)
						OMDataManager.DataProvider.DataReaderObj.Close();
                }
				return null;
			}
			catch (Exception ExceptionObj)
			{
				OMODSDataManager.DataProvider.m_Clause = "";
				OMODSDataManager.DataProvider.m_SortClause = "";
				OMODSDataManager.DataProvider.m_ClauseForCache = "";
				OMODSDataManager.DataProvider.m_SortClauseForCache = "";
				OMDataManager.DataProvider.m_Clause = "";
				OMDataManager.DataProvider.m_SortClause = "";
				OMDataManager.DataProvider.m_ClauseForCache = "";
				OMDataManager.DataProvider.m_SortClauseForCache = "";
				ErrorObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null);
				return ErrorObj;
			}
		}

		/// <summary>
		/// This function is used to delete the records from a given table.
		/// </summary>
		/// <param name="TableName">This specifies the table name from which 
		/// the records have to be deleted
		/// </param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		public OMError DeleteRecord(string TableName)
		{
			OMError         ErrorObj        = null;
            StringBuilder   Query           = null;
            long            RowsAffected    = 0;

			try
			{
                TableName = TableName.ToUpper();

                //Delete the records from table.
                Query = new StringBuilder();
				Query.AppendFormat("DELETE FROM {0}", TableName);
				if ((OMODSDataManager.DataProvider.m_Clause != "") && (OMODSDataManager.DataProvider.m_Clause != null))
					Query.AppendFormat(" WHERE {0}", OMODSDataManager.DataProvider.m_Clause);

				ErrorObj = OMODSDataManager.DataProvider.Execute(Query.ToString(), ref RowsAffected);
                if (ErrorObj != null)
                {
                    ErrorObj = new OMError(OMErrorCodes.UPDATE_RECORD_FAILED,
                           "Error while deletion of records", OMErrorObjectType.TABLE,
                           null, OMErrorType.ERROR, null, null);
                }

				OMODSDataManager.DataProvider.m_Clause = "";
				OMODSDataManager.DataProvider.m_SortClause = "";
				OMODSDataManager.DataProvider.m_ClauseForCache = "";
				OMODSDataManager.DataProvider.m_SortClauseForCache = "";

				OMDataManager.DataProvider.RemoveFromCache(TableName);
				OMODSDataManager.DataProvider.RemoveFromCache(TableName);

			}
			catch (Exception ExceptionObj)
			{
				OMODSDataManager.DataProvider.m_Clause = "";
				OMODSDataManager.DataProvider.m_SortClause = "";
				OMODSDataManager.DataProvider.m_ClauseForCache = "";
				OMODSDataManager.DataProvider.m_SortClauseForCache = "";
				ErrorObj = new OMError(OMErrorCodes.DELETE_RECORD_FAILED,
						   ExceptionObj.Message, OMErrorObjectType.TABLE,
						   null, OMErrorType.EXCEPTION, ExceptionObj, null); 
			}
            return (ErrorObj);
		}
		 
        #endregion

        #region Methods for ODS Data Population Stored Procedures

        /// <summary>
        /// This method is used to generate the stored procedures to populate data into ODS database (from MOM database)
        /// and to manage the relations (assembly/assoentityfacet) into ODS database.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="ODSObjectTypeName">Type of ODS object type for which data population stored procedures
        /// will be created into MOM and ODS databases.</param>
        /// <param name="NewRelationshipsCollectionXML">It contains the details of new relationships which are created for MOM and ODS applications.
        /// The parameter is passed with the RELATIONSHIPTYPE node to differentiate MOM and ODS relations.</param>
        /// <param name="ODSMOMSchemaConfigurationXML">It contains the ODS configuration XMLs with the updated ODS schema details.
        /// This is used while generating stored procedures for the new ODS object types and the relations.</param>
        /// <param name="StoredProcedureNamesXML">This is used as out parameter. It stores the name of stored procedures
        /// which are created newly on MOM and ODS databases.</param>
        /// <returns>On success NULL will be returned else an error object will be returned with relevant information.</returns>
        private OMError createODSPopulationStoredProcedures(String ODSObjectTypeName, XML NewRelationshipsCollectionXML, 
                XML ODSMOMSchemaConfigurationXML, out XML StoredProcedureNamesXML)
        {
            #region Method Variables

            EntityType          entitytype_Object               = null;
            FacetType           facettype_Object                = null;
            RelationshipType    relationshiptype_Object         = null;
            OMError             omerror_Object                  = null;
            XPathDocument       xpathdoc_NewRelationsCollection = null;
            XPathDocument       xpathdoc_ODSObjectConfig        = null;
            XPathNodeIterator   xpathiter_MOMPrimaryFields      = null;
            XPathNavigator      xpathnav_MOMODSCOnfigRoot       = null;
            XPathNavigator      xpathnav_NewRelationship        = null;
            XPathNavigator      xpathnav_MOMObject              = null;
            XPathNavigator      xpathnav_MOMObjectConfig        = null;
            XPathNavigator      xpathnav_ODSObjectConfig        = null;
            String              str_ObjectType                  = null;
            String              str_ObjectTypeName              = null;
            String              str_RelationshipName            = null;
            String              str_MOMDatabaseName             = null;
            String              str_ODSDatabaseName             = null;
            String              str_MOMStoredProcedureName      = null;
            String              str_MOMStoredProcedure          = null;
            String              str_ODSStoredProcedureName      = null;
            String              str_ODSStoredProcedure          = null;
            String              str_AdditinalErrorInfo          = null;
            String              str_NewMOMRelationshipName      = null;
            String              str_NewODSRelationshipName      = null;
            String              str_RelationshipTypeNames       = null;
            String              str_MOMObjectTypeName           = null;
            String              str_MOMObjectTypeNames          = null;
            XML                 xml_MOMAndODSRelatedInfo        = null;
            XML                 xml_MOMObjectTypeInfo           = null;
            XML                 xml_MOMObjectConfiguration      = null;
            XML                 xml_MOMRelationshipCollection   = null;
            XML                 xml_ODSRelationshipCollection   = null;
            XML                 xml_RelationshipDetails         = null;

            #endregion

            StoredProcedureNamesXML = null;

            try
            {                
                //This is used as additional information for an error.
                str_AdditinalErrorInfo = "Error occurred in ObjectManagerODSClass.createODSPopulationStoredProcedures() for " + ODSObjectTypeName;

                #region Retrieve the object type information

                //The ODS configuration XML (ODSMOMSchemaConfig.xml) maintains at web service level
                // which contains the basic MOM + ODS schema information.
                xpathdoc_ODSObjectConfig = new XPathDocument(new StringReader(ODSMOMSchemaConfigurationXML));
                xpathnav_MOMODSCOnfigRoot = xpathdoc_ODSObjectConfig.CreateNavigator();
                xpathnav_ODSObjectConfig = xpathnav_MOMODSCOnfigRoot.SelectSingleNode(
                        "ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']");
                if (xpathnav_ODSObjectConfig == null)
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "The ODS object type does not present in ODS schema configuration XML.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                    
                    return (omerror_Object);
                }

                //Get the parent node of target MOM object types which are configured for ODS. 
                xpathnav_MOMObjectConfig = xpathnav_MOMODSCOnfigRoot.SelectSingleNode("ODSMOMSchemaConfiguration/TargetMOMObjectsForODS");
                if (xpathnav_MOMObjectConfig == null)
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "The MOM object types are not configured in ODS schema configuration XML.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                    
                    return (omerror_Object);
                }

                //Get the type of object. For MOM and ODS it will be same which can be either ENTITY or FACET.
                str_ObjectType = xpathnav_ODSObjectConfig.GetAttribute("ObjectType", "");
                if (String.IsNullOrEmpty(str_ObjectType))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "The type of object does not set in ODS schema configuration XML.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                    
                    return (omerror_Object);
                }

                //Get the object type name related to MOM.
                str_ObjectTypeName = xpathnav_ODSObjectConfig.GetAttribute("MOMObjectTypeName", "");
                if (String.IsNullOrEmpty(str_ObjectTypeName))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "The MOM object type name does not set for the related ODS object type in ODS schema configuration XML.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                    
                    return (omerror_Object);
                }

                //**** CONSIDER CASE LIKE CREATE ENTITY/FACET FOR WHICH DATA DOES NOT PRESENT IN THE DATABASE.
                //**** FOR SUCH CASE FOLLOWING DATA RETRIEVAL IS NOT REQUIRED. COSIDER THIS CASE AND WRITE THE CODE FOR IT.
                switch (str_ObjectType.ToUpper())
                {
                    case "ENTITY":
                        entitytype_Object = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        omerror_Object = entitytype_Object.Read(str_ObjectTypeName, ref xml_MOMObjectTypeInfo);
                        break;

                    case "FACET":
                        facettype_Object = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                        omerror_Object = facettype_Object.Read(str_ObjectTypeName, ref xml_MOMObjectTypeInfo);
                        break;
                }

                if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                {                    
                    return (omerror_Object);
                }

                #endregion

                xpathiter_MOMPrimaryFields = xpathnav_ODSObjectConfig.Select("MOMPrimaryKeyAsODSFields/Field");
                //if (xpathiter_MOMPrimaryFields.Count > 0)
                {
                    #region Generate MOM and ODS Specific Relationship Collection

                    xml_MOMRelationshipCollection   = "<RELATIONSHIPCOLLECTION>";
                    xml_ODSRelationshipCollection   = "<RELATIONSHIPCOLLECTION>";
                    xml_MOMObjectConfiguration      = "<MOMOBJECTCONFIGURATION>";

                    #region Process the new relationships which are created on ODS and MOM

                    //Add newly generated MOM relationship details which are passed from OM.
                    if (!(String.IsNullOrEmpty(NewRelationshipsCollectionXML)))
                    {
                        xpathdoc_NewRelationsCollection = new XPathDocument(new StringReader(NewRelationshipsCollectionXML));
                        xpathnav_NewRelationship = xpathdoc_NewRelationsCollection.CreateNavigator().SelectSingleNode(
                                "RELATIONSHIPCOLLECTION/RELATIONSHIPTYPE[@RelationForMOM='True']");
                        if (xpathnav_NewRelationship != null)
                        {
                            xml_MOMRelationshipCollection += xpathnav_NewRelationship.OuterXml;
                            if (xpathnav_NewRelationship.SelectSingleNode("RELATIONSHIPNAME") != null)
                                str_NewMOMRelationshipName = xpathnav_NewRelationship.SelectSingleNode("RELATIONSHIPNAME").InnerXml;
                        }

                        //Add newly generated ODS relationship details which are passed from OM.
                        xpathnav_NewRelationship = xpathdoc_NewRelationsCollection.CreateNavigator().SelectSingleNode(
                                "RELATIONSHIPCOLLECTION/RELATIONSHIPTYPE[@RelationForODS='True']");
                        if (xpathnav_NewRelationship != null)
                        {
                            xml_ODSRelationshipCollection += xpathnav_NewRelationship.OuterXml;
                            if (xpathnav_NewRelationship.SelectSingleNode("RELATIONSHIPNAME") != null)
                                str_NewODSRelationshipName = xpathnav_NewRelationship.SelectSingleNode("RELATIONSHIPNAME").InnerXml;
                        }
                    }

                    #endregion

                    str_RelationshipTypeNames   = "";
                    str_MOMObjectTypeNames      = "";
                    relationshiptype_Object     = new RelationshipType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    foreach (XPathNavigator xpathnav_MOMPrimaryKeyField in xpathiter_MOMPrimaryFields)
                    {
                        #region Get MOM specific relationship details

                        str_RelationshipName = xpathnav_MOMPrimaryKeyField.GetAttribute("MOMRelationshipName", "");
                        if (!(str_RelationshipTypeNames.Contains(str_RelationshipName + ", ")))
                        {
                            str_RelationshipTypeNames += str_RelationshipName + ", ";
                            if (str_NewMOMRelationshipName != str_RelationshipName)
                            {
                                omerror_Object = relationshiptype_Object.ReadByRelationShipName(str_RelationshipName, ref xml_RelationshipDetails);
                                if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                                {                                    
                                    return (omerror_Object);
                                }
                                else if (String.IsNullOrEmpty(xml_RelationshipDetails))
                                {
                                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                            "The relationship details are NULL or EMPTY for relationship name " + str_RelationshipName,
                                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                                    
                                    return (omerror_Object);
                                }
                                xml_MOMRelationshipCollection += xml_RelationshipDetails;
                            }
                        }

                        #endregion

                        #region Get ODS specific relationship details

                        xml_RelationshipDetails = null;
                        str_RelationshipName = xpathnav_MOMPrimaryKeyField.GetAttribute("ODSRelationshipName", "");
                        if (str_NewODSRelationshipName != str_RelationshipName)
                        {
                            omerror_Object = relationshiptype_Object.ReadByRelationShipName(str_RelationshipName, ref xml_RelationshipDetails);
                            if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                            {                                
                                return (omerror_Object);
                            }
                            else if (String.IsNullOrEmpty(xml_RelationshipDetails))
                            {
                                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                        "The relationship details are NULL or EMPTY for relationship name " + str_RelationshipName,
                                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                                
                                return (omerror_Object);
                            }
                            xml_ODSRelationshipCollection += xml_RelationshipDetails;
                        }

                        #endregion

                        #region Get the matching details of MOM object types from ODS configuration

                        //If ODSRelationshipFor this attrobute not present then the MOM target entity is used only for one reference.
                        //Else the case can be like WORKFLOWPROCESS which is used for multiple entities like PROJECT, DELIVERABLE, TASK, etc.
                        str_MOMObjectTypeName = xpathnav_MOMPrimaryKeyField.GetAttribute("MOMObjectTypeName", "");
                        if (!(str_MOMObjectTypeNames.Contains(str_MOMObjectTypeName + ", ")))
                        {
                            str_MOMObjectTypeNames += str_MOMObjectTypeName + ", ";
                            xpathnav_MOMObject = xpathnav_MOMObjectConfig.SelectSingleNode(
                                    "MOMObject[@MOMObjectTypeName='" + str_MOMObjectTypeName + "']");
                            if (xpathnav_MOMObject != null)
                            {
                                xml_MOMObjectConfiguration += xpathnav_MOMObject.OuterXml;
                            }
                        }

                        #endregion
                    }

                    xml_MOMRelationshipCollection   += "</RELATIONSHIPCOLLECTION>";
                    xml_ODSRelationshipCollection   += "</RELATIONSHIPCOLLECTION>";
                    xml_MOMObjectConfiguration      += "</MOMOBJECTCONFIGURATION>";

                    #endregion

                    #region Set the required MOM + ODS configuration details

                    //Generate the name of the stored procedure.
                    str_MOMStoredProcedureName = ODSObjectTypeName + DataPopulationStoredProc;
                    str_ODSStoredProcedureName = ODSObjectTypeName + ManageRelationsStoredProc;

                    ALCommon.GetDatabaseName(TenantName, out str_MOMDatabaseName, out str_ODSDatabaseName);
                    if (String.IsNullOrEmpty(str_MOMDatabaseName))
                    {
                        omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                "The MOM specific database name is NULL or EMPTY.",
                                OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                        
                        return (omerror_Object);
                    }
                    else if (String.IsNullOrEmpty(str_ODSDatabaseName))
                    {
                        omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                "The ODS specific database name is NULL or EMPTY.",
                                OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                        
                        return (omerror_Object);
                    }

                    //Generate the ODS configuration.
                    xml_MOMAndODSRelatedInfo = "<ODSINFOXML><MOMDATABASENAME>"
                            + str_MOMDatabaseName + "</MOMDATABASENAME><ODSDATABASENAME>"
                            + str_ODSDatabaseName + "</ODSDATABASENAME><DATAPOPULATIONSTOREDPROCEDURENAME>"
                            + str_MOMStoredProcedureName + "</DATAPOPULATIONSTOREDPROCEDURENAME><MANAGERELATIONSSTOREDPROCEDURENAME>"
                            + str_ODSStoredProcedureName + "</MANAGERELATIONSSTOREDPROCEDURENAME><ODSOBJECTCONFIGURATION>"
                            + xpathnav_MOMODSCOnfigRoot.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS").OuterXml
                            + "</ODSOBJECTCONFIGURATION>" + xml_MOMObjectConfiguration + "</ODSINFOXML>";

                    #endregion

                    #region Generate stored procedures

                    //Generate the ODS data population stored procedure and manage relations stored procedure.
                    omerror_Object = OMDataManager.DataProvider.PrepareODSPopulationStoredProcedures(ODSObjectTypeName,
                            xml_MOMAndODSRelatedInfo, xml_MOMObjectTypeInfo, xml_MOMRelationshipCollection, xml_ODSRelationshipCollection,
                            ref str_MOMStoredProcedure, ref str_ODSStoredProcedure);
                    if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                    {                        
                        return (omerror_Object);
                    }
                    else if ((!(String.IsNullOrEmpty(str_MOMStoredProcedure))) && (!(String.IsNullOrEmpty(str_ODSStoredProcedure))))
                    {
                        #region Create stored procedures on MOM and ODS databases
                            
                        //Generate the stored procedure on MOM database.
                        omerror_Object = OMDataManager.DataProvider.Execute(str_MOMStoredProcedure);
                        if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                        {                            
                            return (omerror_Object);
                        }

                        //Generate the stored procedure on ODS database.
                        omerror_Object = OMODSDataManager.DataProvider.Execute(str_ODSStoredProcedure);
                        if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                        {                            
                            return (omerror_Object);
                        }

                        //Stored proceuders are created successfully without an error.
                        //Now send the stored procedure names outside for further processing like updation of "StoredProcedures.xml".
                        StoredProcedureNamesXML = "<STOREDPROCEDURES><DATAPOPULATIONSTOREDPROCEDURENAME>"
                                + str_MOMStoredProcedureName + "</DATAPOPULATIONSTOREDPROCEDURENAME><MANAGERELATIONSSTOREDPROCEDURENAME>"
                                + str_ODSStoredProcedureName + "</MANAGERELATIONSSTOREDPROCEDURENAME></STOREDPROCEDURES>";

                        #endregion
                    }
                    else
                    {
                        if (String.IsNullOrEmpty(str_MOMStoredProcedure))
                        {
                            omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                    "The stored procedure is failed to generate which manages the data population."
                                    + " This stored procedure is expected to be created on MOM database.",
                                    OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                            
                            return (omerror_Object);
                        }
                        else if (String.IsNullOrEmpty(str_ODSStoredProcedure))
                        {
                            omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                                    "The stored procedure is failed to generate which manages the relations." 
                                    + " This stored procedure is expected to be created on ODS database.",
                                    OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);                            
                            return (omerror_Object);
                        }
                    }

                    #endregion
                }
                //else
                //{
                //    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                //            "ODS configuration does not contain the field and relationship details which are referred as primary keys.",
                //            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                //    return (omerror_Object);
                //}                
            }
            catch (Exception ExceptionObject)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED, ExceptionObject.Message, 
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.EXCEPTION, ExceptionObject, str_AdditinalErrorInfo);                
            }

            return (omerror_Object);
        }

        /// <summary>
        /// This method is used to delete the stored procedures which are created for ODS object type for data population 
        /// and for managing the relations between ODS objects.
        /// </summary>
        /// <Created By>Kiran Abhang</Created>
        /// <Creaded On>16th May 2011</Creaded>
        /// <param name="ODSObjectTypeName">It contains the name of the ODS object type for which the stored procedures are to be deleted.</param>
        /// <param name="StoredProcedureNamesXML">This is used as out parameter. It stores the name of stored procedures
        /// which are created newly on MOM and ODS databases.</param>
        /// <returns>On success NULL will be returned else an error object will be returned with relevant information.</returns>
        private OMError deleteODSPopulationStoredProcedures(String ODSObjectTypeName, out XML StoredProcedureNamesXML)
        {
            #region Method Variables

            OMError omerror_Object                  = null;
            String  str_MOMDatabaseName             = null;
            String  str_ODSDatabaseName             = null;
            String  str_MOMStoredProcedureName      = null;
            String  str_ODSStoredProcedureName      = null;
            String  str_AdditinalErrorInfo          = null;
            String  str_MOMDeleteStoredProcQuery    = null;
            String  str_ODSDeleteStoredProcQuery    = null;
            XML     xml_MOMAndODSRelatedInfo        = null;

            #endregion

            StoredProcedureNamesXML = null;

            try
            {
                //This is used as additional information for an error.
                str_AdditinalErrorInfo = "Error occurred in ObjectManagerODSClass.DeleteStoredProcedures() for " + ODSObjectTypeName;

                ALCommon.GetDatabaseName(TenantName, out str_MOMDatabaseName, out str_ODSDatabaseName);
                if (String.IsNullOrEmpty(str_MOMDatabaseName))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED,
                            "The MOM specific database name is NULL or EMPTY.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                    return (omerror_Object);
                }
                else if (String.IsNullOrEmpty(str_ODSDatabaseName))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED,
                            "The ODS specific database name is NULL or EMPTY.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                    return (omerror_Object);
                }

                //Generate the name of the stored procedure.
                str_MOMStoredProcedureName = ODSObjectTypeName + DataPopulationStoredProc;
                str_ODSStoredProcedureName = ODSObjectTypeName + ManageRelationsStoredProc;

                //Generate the ODS configuration.
                xml_MOMAndODSRelatedInfo = "<ODSINFOXML><DATAPOPULATIONSTOREDPROCEDURENAME>"
                        + str_MOMStoredProcedureName + "</DATAPOPULATIONSTOREDPROCEDURENAME><MANAGERELATIONSSTOREDPROCEDURENAME>"
                        + str_ODSStoredProcedureName + "</MANAGERELATIONSSTOREDPROCEDURENAME></ODSINFOXML>";

                //Drop the stored procedures from 
                omerror_Object = OMDataManager.DataProvider.PrepareQueryToDropODSStoredProcedures(xml_MOMAndODSRelatedInfo,
                        ref str_MOMDeleteStoredProcQuery, ref str_ODSDeleteStoredProcQuery);
                if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                {
                    return (omerror_Object);
                }
                else if ((!(String.IsNullOrEmpty(str_MOMDeleteStoredProcQuery))) && (!(String.IsNullOrEmpty(str_ODSDeleteStoredProcQuery))))
                {
                    //Delete stored procedure from MOM database.
                    omerror_Object = OMDataManager.DataProvider.Execute(str_MOMDeleteStoredProcQuery);
                    if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                    {
                        return (omerror_Object);
                    }

                    //Delete stored procedure from ODS database.
                    omerror_Object = OMODSDataManager.DataProvider.Execute(str_ODSDeleteStoredProcQuery);
                    if ((omerror_Object != null) && ((omerror_Object.ErrorType == OMErrorType.ERROR) || (omerror_Object.ErrorType == OMErrorType.EXCEPTION)))
                    {
                        return (omerror_Object);
                    }

                    //Stored proceuders are deleted successfully without an error.
                    //Now send the stored procedure names outside for further processing like updation of "StoredProcedures.xml".
                    StoredProcedureNamesXML = "<STOREDPROCEDURES><DATAPOPULATIONSTOREDPROCEDURENAME>"
                            + str_MOMStoredProcedureName + "</DATAPOPULATIONSTOREDPROCEDURENAME><MANAGERELATIONSSTOREDPROCEDURENAME>"
                            + str_ODSStoredProcedureName + "</MANAGERELATIONSSTOREDPROCEDURENAME></STOREDPROCEDURES>";
                }
                else
                {
                    if (String.IsNullOrEmpty(str_MOMDeleteStoredProcQuery))
                    {
                        omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED,
                                "The stored procedure is failed to generate which manages the data population." 
                                + " This stored procedure is expected to be created on MOM database.",
                                OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                        return (omerror_Object);
                    }
                    else if (String.IsNullOrEmpty(str_ODSDeleteStoredProcQuery))
                    {
                        omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED,
                                "The stored procedure is failed to generate which manages the relations." 
                                + " This stored procedure is expected to be created on ODS database.",
                                OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                        return (omerror_Object);
                    }
                }
            }
            catch (Exception ExceptionObject)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DROPSTOREDPROCEDURE_FAILED, ExceptionObject.Message, 
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.EXCEPTION, ExceptionObject, str_AdditinalErrorInfo);
            }

            return (omerror_Object);
        }

        #endregion

        #region Helper Methods (Private)

        private OMError CheckIfTableExists(String TypeOutPutXML, String ObjectName, out Boolean TableExists, out String ODSTableTypeXML)
        {
            String          ODSTableName    = null;
            String          TypeName        = null;
            XPathDocument   xpDoc_Type      = null;            
            String          ErrorXml        = null;
            EntityType      EntityTypeObj   = null;
            FacetType       FacetTypeObj    = null;
            OMError         OMErrorObj      = null;

            TableExists = false;
            ODSTableTypeXML = String.Empty;
            if (!String.IsNullOrEmpty(TypeOutPutXML))
            {
                
                xpDoc_Type = new XPathDocument(new StringReader(TypeOutPutXML));
                //ODSTableName = xpDoc_Type.CreateNavigator().SelectSingleNode("TYPE/TABLENAME").Value;
                TypeName = xpDoc_Type.CreateNavigator().SelectSingleNode("TYPE/TYPENAME").Value;

                ODSTableName = "ODS" + TypeName;
                

                if (ObjectName == "ENTITY")
                {
                    EntityTypeObj = new EntityType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj = EntityTypeObj.Read(ODSTableName, ref ODSTableTypeXML);
                    EntityTypeObj = null;
                }
                else if (ObjectName == "FACET")
                {
                    FacetTypeObj = new FacetType(UserID, CurrentLanguage, TenantName, OMDataManager);
                    OMErrorObj =  FacetTypeObj.Read(ODSTableName, ref ODSTableTypeXML);
                }
                

                if (OMErrorObj != null)
                {
                    ErrorXml = OMErrorObj.Serialize();                                       
                }

                if (String.IsNullOrEmpty(ErrorXml))
                {
                    if (!String.IsNullOrEmpty(TypeOutPutXML))
                    {
                       // ODSTableTypeXML = TypeOutPutXML;
                        xpDoc_Type = new XPathDocument(new StringReader(ODSTableTypeXML));                        
                        TypeName   = xpDoc_Type.CreateNavigator().SelectSingleNode("TYPE/TYPENAME").Value;

                        if (TypeName.ToUpper() == ODSTableName.ToUpper())
                            TableExists = true;
                        return OMErrorObj;
                    }
                    TableExists = false;
                    return OMErrorObj;
                }
                else
                { 
                    // check if the Error XML contains Facet Type Not Found Message.
                    // if yes, set ErrorXML to null, return false.
                    if ((ErrorXml.Contains("FacetType Not Found")) || (ErrorXml.Contains("EntityType Not Found")))
                    {
                        ErrorXml = String.Empty;
                        OMErrorObj = null;
                        TableExists = false;
                    }
                }
            }
            return OMErrorObj;
        }
        
        private int GetODSPrimaryKeyNumber(String ODSObjectTypeName)
        {
            XPathDocument       xpDoc_ODSSchemaXml  = null;
            XPathNavigator      xpNav_ODSSchemaXml  = null;
            

            //Read Primary key of target entity from ODSMOMSchemaConfig.xml
            
            xpDoc_ODSSchemaXml = new XPathDocument(ODSConfigXMLPath);
            if (xpDoc_ODSSchemaXml != null)
            {
                xpNav_ODSSchemaXml = xpDoc_ODSSchemaXml.CreateNavigator();
                XPathNavigator xpNav_Temp = xpNav_ODSSchemaXml.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']");
                if (xpNav_Temp != null)
                {
                    return Convert.ToInt32(xpNav_Temp.GetAttribute("CurrentFieldNumber", ""));
                }
            }
            return 0;
        }

        private void UpdateNewFieldInODSConfigurationXML(String ODSObjectTypeName, String FieldXML, String CurrentFieldNo, out XML MomSchemaConfigXML)
        {
            XmlDocument ODSConfigDoc            = null;
            XmlNode     ODSRelatedObjectNode    = null;
            MomSchemaConfigXML                  = null;

            
            ODSConfigDoc = new XmlDocument();
            ODSConfigDoc.Load(ODSConfigXMLPath);
            ODSRelatedObjectNode = ODSConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']");
            if (ODSRelatedObjectNode != null)
            {
                if (ODSRelatedObjectNode.SelectSingleNode("MOMPrimaryKeyAsODSFields") == null)
                    ODSRelatedObjectNode.InnerXml = "<MOMPrimaryKeyAsODSFields>" + FieldXML + "</MOMPrimaryKeyAsODSFields>";
                else
                    ODSRelatedObjectNode.SelectSingleNode("MOMPrimaryKeyAsODSFields").InnerXml += FieldXML;

                ((XmlElement)ODSRelatedObjectNode).SetAttribute("CurrentFieldNumber", CurrentFieldNo);

                //ODSConfigDoc.Save(ODSConfigXMLPath);
            }
            MomSchemaConfigXML = ODSConfigDoc.OuterXml;
        }

        private void UpdateNewODSObjectTypeInODSConfigurationXML(String ODSObjectTypeXML,out XML MomSchemaConfigXML)
        {
            XmlDocument ODSConfigDoc        = null;
            XmlNode     RelatedMomObjNode   = null;
            MomSchemaConfigXML              = null;

            
            ODSConfigDoc = new XmlDocument();
            ODSConfigDoc.Load(ODSConfigXMLPath);
            RelatedMomObjNode = ODSConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS");
            if (RelatedMomObjNode != null)
            {
                RelatedMomObjNode.InnerXml += ODSObjectTypeXML;
            }
            //ODSConfigDoc.Save(ODSConfigXMLPath);
            MomSchemaConfigXML = ODSConfigDoc.OuterXml;
        }

        //saveODSConfigXMLsForCreateOperations() method is used to update and save the configuration XMLs required for ODS schema operations.
        //It adds the details of newly created stored procedures into "StoredProcedures.xml".
        //This method also updates the "ODSMOMSchemaConfig.xml" with the newly configured ODS schema info.
        private OMError saveODSConfigXMLsForCreateOperations(Boolean FlagUpdateODSConfig, String XMLPath, XML ConfigXML, 
                XML StoredProcedureNamesXML)
        {
            #region Method Variables

            OMError         omerror_Object              = null;
            XPathDocument   xpathdoc_StoredProcConfig   = null;
            XPathNavigator  xpathnav_StoredProcRoot     = null;
            XPathNavigator  xpathnav_StoredProcedure    = null;
            XPathExpression xpathexprn_SequenceNumber   = null;
            Int32           int_SequenceNumber          = 0;
            String          str_AdditinalErrorInfo      = null;
            String          str_StoredProceduresXMLPath = null;
            String          str_MOMStoredProcedureName  = null;
            String          str_ODSStoredProcedureName  = null;
            XML             xml_StoredProcConfig        = null;
            XML             xml_StoredProcNode          = null;

            #endregion

            //This is used as additional information for an error.
            str_AdditinalErrorInfo = "Error occurred in ObjectManagerODSClass.saveODSConfigXMLsForCreateOperations()"
                    + " while saving ODS configuration XML and the stored procedure configuration XML for back office service.";

            #region Handle errors for all mandatory variables

            if (FlagUpdateODSConfig)
            {
                //ODS configuration XML file path is null then return an error.
                if ((String.IsNullOrEmpty(XMLPath)) || (!(File.Exists(XMLPath))))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "ODS configuration XML file path does not exist.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                    return (omerror_Object);
                }

                //ODS configuration XML is null then return an error.
                if (String.IsNullOrEmpty(ConfigXML))
                {
                    omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                            "ODS configuration XML is NULL or EMPTY.",
                            OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                    return (omerror_Object);
                }
            }

            //ODS new stored procedure's list XML is null then return an error.
            if (String.IsNullOrEmpty(StoredProcedureNamesXML))
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "Stored procedures list is NULL or EMPTY.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            //Back office's stored procedures XML does not exist then return an error.
            str_StoredProceduresXMLPath = ALCommon.GetTenantConfigurationPath(TenantName) + "StoredProcedures.xml";
            if (!(File.Exists(str_StoredProceduresXMLPath)))
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "Stored Procedures configuration XML does not exist into tenant configuration.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            #endregion

            #region Update configuration XMLs

            //Step01: Create nodes for the new stored procedures in "StoredProcedures.xml" and save it.
            //Step02: Update ODS configuration XML and save it.

            str_MOMStoredProcedureName = ALCommon.GetInnerValue(ref StoredProcedureNamesXML, "<DATAPOPULATIONSTOREDPROCEDURENAME>", "</DATAPOPULATIONSTOREDPROCEDURENAME>");
            str_ODSStoredProcedureName = ALCommon.GetInnerValue(ref StoredProcedureNamesXML, "<MANAGERELATIONSSTOREDPROCEDURENAME>", "</MANAGERELATIONSSTOREDPROCEDURENAME>");

            xpathdoc_StoredProcConfig = new XPathDocument(str_StoredProceduresXMLPath);
            xpathnav_StoredProcRoot = xpathdoc_StoredProcConfig.CreateNavigator();
            if (xpathnav_StoredProcRoot.SelectSingleNode("STOREDPROCEDURES/STOREDPROCEDURE[text()='" + str_MOMStoredProcedureName + "']") != null)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "The name of the MOM stored procedure ('" + str_MOMStoredProcedureName + "') is already exist in 'StoredProcedures.xml'.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }
            else if (xpathnav_StoredProcRoot.SelectSingleNode("STOREDPROCEDURES/STOREDPROCEDURE[text()='" + str_ODSStoredProcedureName + "']") != null)
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "The name of the ODS stored procedure ('" + str_ODSStoredProcedureName + "') is already exist in 'StoredProcedures.xml'.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            //Get the last node (which has maximum sequence number) from the list of stored procedures.
            //Get maximum sequence number from the list of stored procedures.
            xpathexprn_SequenceNumber = XPathExpression.Compile("STOREDPROCEDURES/STOREDPROCEDURE");
            xpathexprn_SequenceNumber.AddSort("@SeqNo", XmlSortOrder.Descending, XmlCaseOrder.None, "", XmlDataType.Number);
            xpathnav_StoredProcedure = xpathnav_StoredProcRoot.SelectSingleNode(xpathexprn_SequenceNumber);
            if (xpathnav_StoredProcedure != null)
            {
                int_SequenceNumber = Convert.ToInt32(xpathnav_StoredProcedure.GetAttribute("SeqNo", ""));
                xml_StoredProcConfig = File.ReadAllText(str_StoredProceduresXMLPath);

                //Add the node like <STOREDPROCEDURE SeqNo='17'>LOAD_DAM_DATA_TO_ODS_SP</STOREDPROCEDURE>.
                xml_StoredProcNode = @"<STOREDPROCEDURE SeqNo='" + (++int_SequenceNumber).ToString()
                        + @"'>" + str_MOMStoredProcedureName + @"</STOREDPROCEDURE><STOREDPROCEDURE SeqNo='"
                        + (++int_SequenceNumber).ToString() + @"' FromODS='True'>"
                        + str_ODSStoredProcedureName + @"</STOREDPROCEDURE>";
                xml_StoredProcConfig = ALCommon.InsertBefore(ref xml_StoredProcConfig, "</STOREDPROCEDURES>", ref xml_StoredProcNode);

                //Update stored procedures configuration XML.
                File.WriteAllText(str_StoredProceduresXMLPath, xml_StoredProcConfig);

                //Update ODS configuration XML.
                if (FlagUpdateODSConfig)
                    File.WriteAllText(XMLPath, ConfigXML);
            }
            else
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "Stored Procedure configuration XML is NULL or EMPTY.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            #endregion

            return (null);
        }

        //saveODSConfigXMLsForDeleteOperations() method is used to update and save the configuration XMLs required for ODS schema operations.
        //It deletes the stored procedure details from "StoredProcedures.xml" for those stored procedures which are removed from the system..
        private OMError saveODSConfigXMLsForDeleteOperations(XML StoredProcedureNamesXML)
        {
            #region Method Variables

            OMError     omerror_Object              = null;
            XmlDocument xmldoc_StoredProcedure      = null;
            XmlNode     xmlnode_StoredProcedure     = null;
            XmlElement  xmlelem_StoredProcedures    = null;
            String      str_AdditinalErrorInfo      = null;
            String      str_StoredProceduresXMLPath = null;
            String      str_MOMStoredProcedureName  = null;
            String      str_ODSStoredProcedureName  = null;
            Boolean     bool_IsStoredProcRemoved    = false;

            #endregion

            //This is used as additional information for an error.
            str_AdditinalErrorInfo = "Error occurred in ObjectManagerODSClass.saveODSConfigXMLsForDeleteOperations()"
                    + " while updating the stored procedure configuration XML for back office service.";

            #region Handle errors for all mandatory variables

            //ODS new stored procedure's list XML is null then return an error.
            if (String.IsNullOrEmpty(StoredProcedureNamesXML))
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "Stored procedures list is NULL or EMPTY.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            //Back office's stored procedures XML does not exist then return an error.
            str_StoredProceduresXMLPath = ALCommon.GetTenantConfigurationPath(TenantName) + "StoredProcedures.xml";
            if (!(File.Exists(str_StoredProceduresXMLPath)))
            {
                omerror_Object = new OMError(OMErrorCodes.ODS_DATAPOPUATIONSTOREDPROCEDURE_FAILED,
                        "Stored Procedures configuration XML does not exist into tenant configuration.",
                        OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, str_AdditinalErrorInfo);
                return (omerror_Object);
            }

            #endregion

            #region Update configuration XML

            str_MOMStoredProcedureName = ALCommon.GetInnerValue(ref StoredProcedureNamesXML, "<DATAPOPULATIONSTOREDPROCEDURENAME>", "</DATAPOPULATIONSTOREDPROCEDURENAME>");
            str_ODSStoredProcedureName = ALCommon.GetInnerValue(ref StoredProcedureNamesXML, "<MANAGERELATIONSSTOREDPROCEDURENAME>", "</MANAGERELATIONSSTOREDPROCEDURENAME>");

            xmldoc_StoredProcedure = new XmlDocument();
            xmldoc_StoredProcedure.Load(str_StoredProceduresXMLPath);
            xmlelem_StoredProcedures = xmldoc_StoredProcedure.DocumentElement;

            //Remove stored procedure (which is on MOM database) details
            // which is used for managing data population from MOM to ODS database.
            xmlnode_StoredProcedure = xmlelem_StoredProcedures.SelectSingleNode("STOREDPROCEDURE[text()='" + str_MOMStoredProcedureName + "']");
            if (xmlnode_StoredProcedure != null)
            {
                ((XmlNode)xmlelem_StoredProcedures).RemoveChild(xmlnode_StoredProcedure);
                bool_IsStoredProcRemoved = true;
            }

            //Remove stored procedure (which is on ODS database) details
            // which is used for managing relations in ODS database for the populated instances.
            xmlnode_StoredProcedure = xmlelem_StoredProcedures.SelectSingleNode("STOREDPROCEDURE[text()='" + str_ODSStoredProcedureName + "']");
            if (xmlnode_StoredProcedure != null)
            {
                ((XmlNode)xmlelem_StoredProcedures).RemoveChild(xmlnode_StoredProcedure);
                bool_IsStoredProcRemoved = true;
            }

            //If any of the stored procedure is removed then save the updated stored procedure XML.
            if (bool_IsStoredProcRemoved)
            {
                xmldoc_StoredProcedure.Save(str_StoredProceduresXMLPath);
            }

            #endregion

            return (null);
        }

        private OMError RemoveODSFieldFromConfigXML(String ODSObjectName, String MomRelationshipName, Boolean MOMObjectExists, out XML ODSMOMConfigXML)
        {
            XmlDocument OdsConfigDoc        = null;
            XmlNode     OdsObjNode          = null;
            XmlNodeList FieldNodeList       = null;
            XmlNode     MomPkNode           = null;
            XmlNode     RelatedMOMObjNode   = null;
            OMError     OmErrorObj          = null;
            String      MOMObjectName       = null;

            ODSMOMConfigXML = null;            
            if (!File.Exists(ODSConfigXMLPath))
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_MOM_SCHEMA_FILE_NOT_FOUND, "MOM Schema Config File at path :" + ODSConfigXMLPath + " not found.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectName, OMErrorType.ERROR, null, null);
                return OmErrorObj;
            }
            OdsConfigDoc = new XmlDocument();
            OdsConfigDoc.Load(ODSConfigXMLPath);
            OdsObjNode = OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectName + "']");
            if (OdsObjNode == null)
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_OBJECT_NOT_FOUND, "ODS Object:" + ODSObjectName + " not found in MOM Schema config XML.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectName, OMErrorType.ERROR, null, null);
                return OmErrorObj;
            }
            RelatedMOMObjNode = OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS");
            MOMObjectName = ((XmlElement)OdsObjNode).GetAttribute("MOMObjectTypeName","");

            MomPkNode =  OdsObjNode.SelectSingleNode("MOMPrimaryKeyAsODSFields");
            if (MomPkNode != null)
            {
                if (!String.IsNullOrEmpty(MomRelationshipName))
                {
                    FieldNodeList = MomPkNode.SelectNodes("Field[@MOMRelationshipName='" + MomRelationshipName + "']");
                    if (FieldNodeList == null)
                    {
                        OmErrorObj = new OMError(OMErrorCodes.ODS_FIELDINFO_NOT_FOUND, "Field Node with MOM Relationship :" + MomRelationshipName + " for ODS Object: " + ODSObjectName + " not found in MOM Schema config XML.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectName, OMErrorType.ERROR, null, null);
                        return OmErrorObj;
                    }
                    foreach (XmlNode FNode in FieldNodeList)
                    {
                        MomPkNode.RemoveChild(FNode);
                    }
                    //MomPkNode.RemoveChild(FieldNode);
                }
                if ((MomPkNode.SelectNodes("Field").Count == 0) && (!MOMObjectExists))
                {
                    OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS").RemoveChild(OdsObjNode);
                }
            }
            OdsConfigDoc.Save(ODSConfigXMLPath);
            ODSMOMConfigXML = OdsConfigDoc.OuterXml;

            return OmErrorObj;
        }

        private OMError IsODSObjectExists(String ODSObjectTypeName, out Boolean IsObjectExists)
        {
            OMError OmErrorObj = null;
            IsObjectExists = false;
            if (!File.Exists(ODSConfigXMLPath))
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_MOM_SCHEMA_FILE_NOT_FOUND, "MOM Schema Config File at path :" + ODSConfigXMLPath + " not found.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectTypeName, OMErrorType.ERROR, null, null);
                
            }
            XPathDocument xpDoc_ODS = new XPathDocument(ODSConfigXMLPath);
            if (xpDoc_ODS.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']") != null)
                IsObjectExists = true;
            else
                IsObjectExists = false;
            return OmErrorObj;
        }

        String PrepareODSObjectTypeXML(String ODSObjectTypeName, String ObjectType, String ODSPrimaryKey, String ODSMOMPrimaryKey,
            String ODSTableName, String MOMObjectTypeName, String CurrentFieldNo, String MOMObjectTypeID,String ODSObjectTypeID,  String FieldXML)
        {
            StringBuilder sb_ODSTypeXML = new StringBuilder(100);
            sb_ODSTypeXML.AppendFormat(@"<ODSObject ObjectType='{0}' ODSObjectTypeName='{1}' ODSPrimaryKey='{2}'
                    MOMPrimaryKey='{3}' ODSObjectTypeTableName='{4}' MOMObjectTypeName='{5}' CurrentFieldNumber='{6}' MOMObjectTypeID='{7}' ODSObjectTypeID='{8}'>",
                    ObjectType, ODSObjectTypeName, ODSPrimaryKey, ODSMOMPrimaryKey, ODSTableName, MOMObjectTypeName, CurrentFieldNo, MOMObjectTypeID, ODSObjectTypeID);
            sb_ODSTypeXML.AppendFormat("<MOMPrimaryKeyAsODSFields>{0}</MOMPrimaryKeyAsODSFields>", FieldXML);
            sb_ODSTypeXML.Append("</ODSObject>");

            return sb_ODSTypeXML.ToString();
        }

        String PrepareODSFieldXML(String MOMObjectType, String MOMObjectTypeName, String MOMTableName, String MOMPrimaryKey, 
            String MOMRelName, XML ODSRelationship, String IsParent, String FieldName, String MOMObjectTypeID)
        {
            String ODSRelationShipName = null;

            StringBuilder sb_FieldXML = new StringBuilder(100);
            XPathDocument xpDoc_ODSRel = new XPathDocument(new StringReader(ODSRelationship));
            XPathNodeIterator xpIter_RelForODS =  xpDoc_ODSRel.CreateNavigator().Select("RELATIONSHIPCOLLECTION/RELATIONSHIPTYPE[@RelationForODS='True']");
            while(xpIter_RelForODS.MoveNext())
            {
                ODSRelationShipName = xpIter_RelForODS.Current.SelectSingleNode("RELATIONSHIPNAME").Value;
                if (xpIter_RelForODS.Current.SelectSingleNode("ODSRELATIONSHIPFOR") != null)
                {
                    sb_FieldXML.AppendFormat(@"<Field MOMObjectType='{0}' MOMObjectTypeName='{1}' MOMObjectTypeTableName='{2}' MOMObjectTypePrimaryKey='{3}' 
                                              MOMRelationshipName='{4}' ODSRelationshipName='{5}' IsParentObject='{6}' MOMObjectTypeID='{8}' ODSRelationshipFor='{9}'>{7}</Field>", MOMObjectType, MOMObjectTypeName, MOMTableName,
                                                                                                                                                                                    MOMPrimaryKey, MOMRelName, ODSRelationShipName, IsParent, FieldName, MOMObjectTypeID, xpIter_RelForODS.Current.SelectSingleNode("ODSRELATIONSHIPFOR").Value);
                }
                else
                {
                    sb_FieldXML.AppendFormat(@"<Field MOMObjectType='{0}' MOMObjectTypeName='{1}' MOMObjectTypeTableName='{2}' MOMObjectTypePrimaryKey='{3}' 
                                              MOMRelationshipName='{4}' ODSRelationshipName='{5}' IsParentObject='{6}' MOMObjectTypeID='{8}'>{7}</Field>", MOMObjectType, MOMObjectTypeName, MOMTableName, MOMPrimaryKey, MOMRelName, ODSRelationShipName, IsParent, FieldName, MOMObjectTypeID);
                }
            }
            return sb_FieldXML.ToString();
        }        

        private String GetTargetPrimaryKeyName(String ODSObjectTypeName,String ODSRelationshipName)
        {            
            if (File.Exists(ODSConfigXMLPath))
            {
                XPathDocument xpDoc_MomSchema = new XPathDocument(ODSConfigXMLPath);
                XPathNavigator xpNav_ODS       = null;
                XPathNavigator  xpNav_Field    = null;
                xpNav_ODS = xpDoc_MomSchema.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectTypeName + "']");
                if (xpNav_ODS != null)
                {
                    xpNav_Field = xpNav_ODS.SelectSingleNode("MOMPrimaryKeyAsODSFields/Field[@ODSRelationshipName='" + ODSRelationshipName + "']");
                    if (xpNav_Field != null)
                        return xpNav_Field.Value;
                }                    
            }
            return String.Empty;
        }

        /// <summary>
        /// <CreatedBy>Mugdha Moolay</CreatedBy>
        /// <CreatedOn>8 June 2011</CreatedOn>
        /// <Purpose>This method checks if the given MOM Object is related to one or more ODS Objects</Purpose>
        /// </summary>
        /// <param name="MOMObjectName"></param>
        /// <returns>True if MOM Object exists more than once for ODS Objects</returns>
        private Boolean CheckIfMOMObjectExistsAsODS(String MOMObjectName, String MOMRelationshipName, out String RelatedODSObjectName, out String RelatedODSObjectType, out String RelatedODSObjectTypeTableName, out String RelatedMomObjectType)
        {

            RelatedODSObjectName = String.Empty;
            RelatedODSObjectType = String.Empty;
            RelatedODSObjectTypeTableName = String.Empty;
            RelatedMomObjectType = String.Empty;
            if (File.Exists(ODSConfigXMLPath))
            {
                XPathDocument xpDoc_MomSchema = new XPathDocument(ODSConfigXMLPath);
                XPathNavigator xpNav_ODS = null;
                xpNav_ODS = xpDoc_MomSchema.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[MOMPrimaryKeyAsODSFields/Field[@MOMObjectTypeName='" + MOMObjectName + "']]");
                if (xpNav_ODS != null)
                {
                    return true;
                }
                xpNav_ODS = xpDoc_MomSchema.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeName='" + MOMObjectName + "']");
                if (xpNav_ODS != null)
                {
                    if (xpNav_ODS.Select("MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName!='" + MOMRelationshipName + "']").Count > 1)
                        return true;
                    else if (xpNav_ODS.Select("MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + MOMRelationshipName + "']").Count >= 1)
                    {
                        RelatedMomObjectType = xpNav_ODS.SelectSingleNode("MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + MOMRelationshipName + "']").GetAttribute("MOMObjectTypeName", "");
                        xpNav_ODS = xpDoc_MomSchema.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeName='" + RelatedMomObjectType + "']");
                        if (xpNav_ODS != null)
                        {
                            RelatedODSObjectName = xpNav_ODS.GetAttribute("ODSObjectTypeName", "");
                            RelatedODSObjectType = xpNav_ODS.GetAttribute("ObjectType", "");
                            RelatedODSObjectTypeTableName = xpNav_ODS.GetAttribute("ODSObjectTypeTableName", "");
                        }
                    }
                }
            }
            return false;
        }

        /// <summary>
        /// <CreatedBy>Mugdha Moolay</CreatedBy>
        /// <CreatedOn>8 June 2011</CreatedOn>
        /// <Purpose>This method returns the ODS & MOM Object Details for the given MOM relationship</Purpose>
        /// </summary>
        /// <param name="MOMRelName">The MOM relationshipname</param>
        /// <param name="MOMObjectName">The MOM Object Name: OUT PARAMETER</param>
        /// <param name="ODSObjectName">THE ODS Object Name : OUT PARAMETER</param>
        /// <param name="ODSTableName">ODS table name in the ODS DB</param>
        /// <param name="ObjectType">The type of MOM Object: ENTITY/FACET</param>
        private void GetODSInfoFromMOMRelationshipName(String MOMRelName, out String MOMObjectName, out String ODSObjectName, out String ODSTableName, out String ObjectType)
        {
            MOMObjectName = String.Empty;
            ODSObjectName = String.Empty;
            ODSTableName = String.Empty;
            ObjectType   = String.Empty;
            
            if (File.Exists(ODSConfigXMLPath))
            {
                XPathDocument xpDoc_MomSchema = new XPathDocument(ODSConfigXMLPath);
                XPathNavigator xpNav_ODS = null;                
                xpNav_ODS = xpDoc_MomSchema.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + MOMRelName + "']]");
                if (xpNav_ODS != null)
                {
                    MOMObjectName = xpNav_ODS.GetAttribute("MOMObjectTypeName", "");
                    ODSObjectName = xpNav_ODS.GetAttribute("ODSObjectTypeName", "");
                    ODSTableName = xpNav_ODS.GetAttribute("ODSObjectTypeTableName", "");
                    ObjectType  = xpNav_ODS.GetAttribute("ObjectType", "");
                }
            }
        }
               

        /// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>7 June 2011</CreatedOn>
        /// <Purpose>This method Reads the ODS Object Type Details from the MOM DB</Purpose>
        /// </summary>
        /// <param name="ObjectType">The Type of Object: ENTITY/FACET</param>
        /// <param name="ODSTypeName">Name of the ODS Object to Read from the DB</param>
        /// <param name="ObjectTypeDetails">OUT PARAMETER: an XML that returns the ODS Object Type details</param>
        /// <returns></returns>
        private OMError GetObjectTypeDetails(String ObjectType, String ODSTypeName, out String ObjectTypeDetails)
        {
            OMError OMErrorObj  = null;
            
            FacetType           FacetTypeObj        = null;
            EntityType          EntityTypeObj       = null; 
            ObjectTypeDetails   = String.Empty;
            if (ObjectType.ToUpper() == "FACET")
            {
                FacetTypeObj = new FacetType(OMDataManager);
                OMErrorObj = FacetTypeObj.Read(ODSTypeName, ref ObjectTypeDetails);
                if (OMErrorObj != null)
                    return OMErrorObj;
            }
            else if (ObjectType.ToUpper() == "ENTITY")
            {
                EntityTypeObj = new EntityType(OMDataManager);
                OMErrorObj = EntityTypeObj.Read(ODSTypeName, ref ObjectTypeDetails);
                if (OMErrorObj != null)
                    return OMErrorObj;
            }

            if (String.IsNullOrEmpty(ObjectTypeDetails))
            {
                if (ObjectType.ToUpper() == "FACET")
                    OMErrorObj = new OMError(OMErrorCodes.FACET_TYPE_READ_FAILED, "Error Reading Facet Type Details.", OMErrorObjectType.FACETTYPE, "", OMErrorType.ERROR, null, null);
                else
                    OMErrorObj = new OMError(OMErrorCodes.ENTITY_READ_FAILED, "Error Reading Entity Type Details.", OMErrorObjectType.FACETTYPE, "", OMErrorType.ERROR, null, null);
                return OMErrorObj;
            }
            return OMErrorObj;
        }

        /// <summary>
        /// <CreatedDate>29 April 2011</CreatedDate>
        /// <Author>Bhavana Patil</Author>
        /// <Purpose>
        /// This is a helper function not exposed to outside.
        /// The function checks whether the RelationshipName Exists in the system or Not.
        /// </Purpose>
        /// </summary>
        /// <param name="RelationshipName">This contains the RelationshipName which is to be searched.</param>
        /// <returns>
        /// true: If RelationshipName exist.
        /// false:If RelationshipName not exist.
        /// </returns>
        private bool IsRelationshipNameExists(String RelationshipName)
        {
            OMError ReadErrorObj               = null;   //The OMError object for reading data from DAL.
            String OutputXML                   = null;   //The String variable storing the Output XML.

            try
            {
                ReadErrorObj = OMODSDataManager.DataProvider.AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING,
                                        OMOperator.EQUALTO, RelationshipName,
                                        OMConjunction.NOTHING);

                if (ReadErrorObj != null)
                    return false;

                ReadErrorObj = ReadRecord("RELATIONSHIPTYPE", "ALOM_RELATIONSHIPTYPES", null,
                                                                    ref OutputXML);

                if (ReadErrorObj != null || OutputXML == null || OutputXML == "")
                    return false;
                else
                    return true;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

		/// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>1 June 2011</CreatedOn>
		/// <purpose>This function is used to delete a table from the ODS DB</purpose>
		/// </summary>
		/// <param name="TableName">This function takes TableName as parameter,
		/// this specifies which table has to be dropped.</param>
		/// <returns>This function returns OMError object if there is any error occured.</returns>
		private OMError DropTable(string TableName)
		{
			OMError         ErrorObj    = null;
            StringBuilder   Query       = null;
			
            try
			{
                TableName = TableName.ToUpper();

				Query = new StringBuilder();
				Query.AppendFormat("DROP Table {0}", TableName);

				ErrorObj = OMODSDataManager.DataProvider.Execute(Query.ToString());

				OMDataManager.DataProvider.RemoveFromCache(TableName);
				OMODSDataManager.DataProvider.RemoveFromCache(TableName);

				return (ErrorObj);
			}
			catch (Exception ExceptionObj)
			{
                throw ExceptionObj;
			}
		}
        
        /// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>8 June 2011</CreatedOn>
        /// <Purpose>This methods deletes the following from the MOM DB
        /// 1. Object Type entry from ALOM_FACETTYPE/ALOM_ENTITYTYPE
        /// 2. SCHEMA information from 
        /// 3. Drops the Table</Purpose>
        /// </summary>
        /// <param name="TableName">Name of Table to be deleted</param>
        /// <param name="ObjectTypeID">The TypeID of FACET or ENTITY</param>
        /// <param name="FieldName">Name of the field to be used in where clause for delete</param>
        /// <param name="ObjectTypeDetails">The Type Details XML</param>
        /// <returns>An Error Object if an error occurs.</returns>
        private OMError DeleteAllInfo(String TableName,String ObjectTypeTableName, String ObjectTypeID,String FieldName, String ObjectTypeDetails)
        {
            OMError             OmErrorObj      = null;            
            XPathDocument       xpDoc_ObjType   = null;
            XPathNodeIterator   xpIterField     = null;
            String              DeleteQuery     = String.Empty;
            StringBuilder       SchemaIDList    = null;

            OmErrorObj =  DropTable(TableName);
            if (OmErrorObj == null)
            {
                OmErrorObj = OMDataManager.DataProvider.AddClause(FieldName, OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID, OMConjunction.NOTHING);
                if (OmErrorObj == null)
                {
                    OmErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery(ObjectTypeTableName, ref DeleteQuery);
                    if (OmErrorObj == null)
                    {
                        OmErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                        if ((OmErrorObj == null) && (!String.IsNullOrEmpty(ObjectTypeDetails)))
                        {                            
                            xpDoc_ObjType = new XPathDocument(new StringReader(ObjectTypeDetails));
                            xpIterField = xpDoc_ObjType.CreateNavigator().Select("TYPE/FIELDCOLLECTION/FIELD");
                            SchemaIDList = new StringBuilder(10);
                            SchemaIDList.Append("(");
                            while (xpIterField.MoveNext())
                            {
                                SchemaIDList.AppendFormat("{0},", xpIterField.Current.SelectSingleNode("SCHEMAID").Value);
                            }
                            SchemaIDList.Replace(",", ")", SchemaIDList.Length - 1, 1);
                            OmErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.IN, SchemaIDList.ToString(), OMConjunction.NOTHING);

                            if (OmErrorObj == null)
                            {
                                DeleteQuery = String.Empty;
                                OmErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_SCHEMA", ref DeleteQuery);

                                if (OmErrorObj == null)
                                {
                                    OmErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                                }
                            }
                        }
                    }
                }
            }
            return OmErrorObj;
        }

        /// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>10 June 2011</CreatedOn>
        /// <Purpose>This method  Update the ODS Table in ODS DB for the following cases:
        /// 1. A new field is added.
        /// 2. An existing field is removed.
        /// 3. An existing field is updated.
        /// This method will also update the Schema changes for the ODS table in MOM DB</Purpose>
        /// </summary>
        /// <param name="InputXML">The Input XML contains the update information given to the MOM DB</param>
        /// <returns>An Error Object if an error occurs</returns>
        private OMError UpdateObjectType(String MOMInputXML)
        {
            OMError             OmErrorObj          = null;
            XPathDocument       xpDoc_MOMInput      = null;
            XPathNavigator      xpNav_MOMInput      = null;
            String              ODSTypeName         = null;
            String              xml_StoredProcNames = null;
            XPathNodeIterator   xpIter_NewFields    = null;
            String              MomTypeName         = null;
            String              ODSConfigXML        = null;
            String              ODSTableName        = null;
            XmlDocument         ODSTypeDoc          = null;
            XmlNode             TypeNode            = null;
            XmlNode             TableNameNode       = null;
            XmlNode             FieldCollNode       = null;
            XmlNodeList         FieldList           = null;
            XmlNode             SchemaIDNode        = null;
            XmlNode             FieldNode           = null;
            XmlNode             SchemaNode          = null;
            String              ODSSchemaID         = null;
            String              ODSType             = null;
            FacetType           FacetTypeObj        = null;
            EntityType          EntityTypeObj       = null;
            String              ODSObjectTypeDetails= null;
            String              OperationType       = null;
            String              ObjectTypeID        = null;
            StringBuilder       sb_SchemaID         = null;
            String              DeleteQuery         = null;
            String              FieldType           = null;
            
            ulong SchemaID = 0;
            xpDoc_MOMInput = new XPathDocument(new StringReader(MOMInputXML));
            xpNav_MOMInput = xpDoc_MOMInput.CreateNavigator().SelectSingleNode("INPUTXML");
            MomTypeName = xpNav_MOMInput.SelectSingleNode("OPERATION/PARAM/TYPE/TYPENAME").Value;
            OmErrorObj = GetODSObjectName(MomTypeName, out ODSTypeName, out ODSTableName, out ODSType, out ODSConfigXML);

            #region Read the ODS Type Informaton
            if (ODSType.ToUpper() == "FACET")
            {
                FacetTypeObj = new FacetType(OMDataManager);
                OmErrorObj = FacetTypeObj.Read(ODSTypeName, ref ODSObjectTypeDetails);
            }
            else
            {
                EntityTypeObj = new EntityType(OMDataManager);
                OmErrorObj = EntityTypeObj.Read(ODSTypeName, ref ODSObjectTypeDetails);
            }

            #endregion

            if (OmErrorObj == null)
            {
                ODSTypeDoc = new XmlDocument();
                ODSTypeDoc.LoadXml(ODSObjectTypeDetails);
                TypeNode = ODSTypeDoc.SelectSingleNode("TYPE");

                FieldCollNode = ODSTypeDoc.SelectSingleNode("TYPE/FIELDCOLLECTION");
                //FieldCollNode.RemoveAll();
               
                #region Add/Remove New Fields & Update Schema Table for Deletion & Modify cases

                if (xpNav_MOMInput.Select("OPERATION/PARAM/TYPE/FIELDCOLLECTION/FIELD[@OperationType='NEW' or @OperationType='DELETE' or @OperationType='MODIFY' ]").Count > 0)
                {
                    xpIter_NewFields = xpNav_MOMInput.Select("OPERATION/PARAM/TYPE/FIELDCOLLECTION/FIELD[@OperationType='NEW' or @OperationType='DELETE' or @OperationType='MODIFY']");
                    while (xpIter_NewFields.MoveNext())
                    {
                        
                        OperationType = xpIter_NewFields.Current.GetAttribute("OperationType", "") ;
                        switch(OperationType.ToUpper())
                        {
                            case "NEW":
                                    FieldType = xpIter_NewFields.Current.SelectSingleNode("FIELDTYPE").Value;
                                    if (FieldType.ToUpper() != "XML" && FieldType.ToUpper() != "FILE" && FieldType.ToUpper() != "BINDER")
                                    {
                                        TableNameNode           = ODSTypeDoc.CreateElement("TABLENAME");
                                        TableNameNode.InnerText = ODSTableName;
                                        FieldCollNode.InnerXml += xpIter_NewFields.Current.OuterXml;
                                        
                                        FieldNode = FieldCollNode.SelectSingleNode("FIELD[FIELDNAME='" + xpIter_NewFields.Current.SelectSingleNode("FIELDNAME").Value + "']");
                                        if (FieldNode != null)
                                        {
                                            FieldNode.AppendChild(TableNameNode);
                                            SchemaIDNode = FieldNode.SelectSingleNode("SCHEMAID");
                                            ALCommon.GenerateUniqueID(ref SchemaID);
                                            SchemaIDNode.InnerText = SchemaID.ToString();
                                            ObjectTypeID = ALCommon.GetInnerValue(ref ODSObjectTypeDetails, "<TYPEID>", "</TYPEID>");
                                            FieldNode.SelectSingleNode("OBJECTTYPEID").InnerText = ObjectTypeID;
                                            ((XmlElement)FieldNode).SetAttribute("OperationType", "NEW");
                                        }
                                    }
                                    break;
                            case "MODIFY":
                                    FieldNode = FieldCollNode.SelectSingleNode("FIELD[FIELDNAME='" + xpIter_NewFields.Current.SelectSingleNode("FIELDNAME").Value + "']");
                                    if (FieldNode !=null)
                                    {
                                        ODSSchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                        ObjectTypeID = FieldNode.SelectSingleNode("OBJECTTYPEID").InnerText;
                                        FieldNode.InnerXml = xpIter_NewFields.Current.InnerXml;
                                        FieldNode.SelectSingleNode("SCHEMAID").InnerText = ODSSchemaID;
                                        FieldNode.SelectSingleNode("OBJECTTYPEID").InnerText = ObjectTypeID;
                                        ((XmlElement)FieldNode).SetAttribute("OperationType", "MODIFY");
                                    }
                                    break;
                            case "DELETE":
                                    FieldNode = FieldCollNode.SelectSingleNode("FIELD[FIELDNAME='" + xpIter_NewFields.Current.SelectSingleNode("FIELDNAME").Value + "']");
                                    if (FieldNode != null)
                                    {
                                        ((XmlElement)FieldNode).SetAttribute("OperationType", "DELETE");                                        
                                    }
                                    break;
                        }                                              
                    }
                    OmErrorObj = OMODSDataManager.DataProvider.UpdateTable(ODSTypeDoc.OuterXml);                    
                }

                if ((OmErrorObj == null) && (FieldCollNode.SelectNodes("FIELD[@OperationType='DELETE']").Count > 0))
                {
                    sb_SchemaID = new StringBuilder(100);
                    sb_SchemaID.Append("(");
                    //xpIter_NewFields = xpNav_Input.Select("OPERATION/PARAM/TYPE/FIELDCOLLECTION/FIELD[@OperationType='DELETE']");
                    FieldList = FieldCollNode.SelectNodes("FIELD[@OperationType='DELETE']");
                    if (FieldList.Count > 0)
                    {
                        foreach (XmlNode FNode in FieldList)
                        {
                            //FieldCollNode.InnerXml += xpIter_NewFields.Current.OuterXml;
                            sb_SchemaID.AppendFormat("{0},", FNode.SelectSingleNode("SCHEMAID").InnerText);
                        }
                        sb_SchemaID = sb_SchemaID.Replace(",", ")", sb_SchemaID.Length - 1, 1);

                        if (OmErrorObj == null)
                        {
                            // Delete from Schema Table
                            OmErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.IN, sb_SchemaID.ToString(), OMConjunction.NOTHING);
                            if (OmErrorObj == null)
                            {
                                OmErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_SCHEMA", ref DeleteQuery);
                                if (OmErrorObj == null)
                                {
                                    OmErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                                }
                            }
                        }
                    }
                }

                if ((OmErrorObj == null) &&(FieldCollNode.SelectNodes("FIELD[@OperationType='MODIFY']").Count > 0))
                {
                    // Modify The Schema Entries for ODS Table
                    FieldList = FieldCollNode.SelectNodes("FIELD[@OperationType='MODIFY']");
                    foreach (XmlNode FNode in FieldList)
                    {

                        if (ODSType.ToUpper() == "FACET")
                        {
                            FacetTypeObj.generateSchemaXML(ODSTypeDoc, FNode, ref SchemaNode);
                        }
                        else
                        {
                            EntityTypeObj.generateSchemaXML(ODSTypeDoc, FNode, ref SchemaNode);
                        }
                        ODSSchemaID = FNode.SelectSingleNode("SCHEMAID").InnerText;

                        OmErrorObj = OMDataManager.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ODSSchemaID, OMConjunction.NOTHING);
                       
                        SchemaNode.RemoveChild(SchemaNode.SelectSingleNode("SCHEMAID"));

                        if (OmErrorObj != null)
                            return OmErrorObj;

                        if (OmErrorObj == null)
                            OmErrorObj = OMDataManager.DataProvider.UpdateRecord("ALOM_SCHEMA", SchemaNode.OuterXml);

                        if (OmErrorObj != null)
                            return OmErrorObj;
                    }                    
                }
                #endregion

                #region Update Schema for New Fields 
                if (OmErrorObj == null)
                {
                    FieldList = FieldCollNode.SelectNodes("FIELD[not(@OperationType='NEW')]");
                   
                    foreach (XmlNode FNode in FieldList)
                    {
                        FieldCollNode.RemoveChild(FNode);
                    }
                    FieldList = FieldCollNode.SelectNodes("FIELD[@OperationType='NEW']");
                    if (FieldList.Count > 0)
                    {
                        OmErrorObj = AddSchemaEntry(ODSTypeDoc.OuterXml);
                    }
                }
                #endregion

                #region Delete Stored Procedures & Recreate them
                if (OmErrorObj == null)
                {
                    OmErrorObj = deleteODSPopulationStoredProcedures(ODSTypeName, out xml_StoredProcNames);

                    if (OmErrorObj == null)
                    {
                        OmErrorObj = saveODSConfigXMLsForDeleteOperations(xml_StoredProcNames);
                        if (OmErrorObj == null)
                        {
                            OmErrorObj = createODSPopulationStoredProcedures(ODSTypeName, null, ODSConfigXML, out xml_StoredProcNames);
                            if (OmErrorObj == null)
                            {
                                OmErrorObj = saveODSConfigXMLsForCreateOperations(false, null, null, xml_StoredProcNames);
                            }
                        }
                    }
                }
                #endregion
            }
            return OmErrorObj;
        }

        /// <summary>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <CreatedOn>10 June 2011</CreatedOn>
        /// <Purpose>This method returns the ODS Object Type Name related to the MOM Object Type Name</Purpose>
        /// </summary>
        /// <param name="MOMObjectName">MOM Object Name</param>
        /// <param name="ODSObjectName">Related ODS Object Type Name</param>
        /// <returns>An Error Object if the related ODS Object Type is not found.</returns>
        private OMError GetODSObjectName(String MOMObjectName, out String ODSObjectName, out String ODSTypeTableName,out String ODSType, out String ODSConfigXML)
        {
            
            OMError         OmErrorObj          = null;        
            XPathNavigator  xpNav_ODS           = null;

            ODSObjectName       = String.Empty;
            ODSConfigXML        = String.Empty;
            ODSTypeTableName    = String.Empty;
            ODSType             = String.Empty;

            if (File.Exists(ODSConfigXMLPath))
            {
                XPathDocument xpDoc_Config = new XPathDocument(ODSConfigXMLPath);
                

                ODSConfigXML = xpDoc_Config.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration").OuterXml;
                xpNav_ODS = xpDoc_Config.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeName='" + MOMObjectName + "']");
                if (xpNav_ODS != null)
                {
                    ODSObjectName = xpNav_ODS.GetAttribute("ODSObjectTypeName", "");
                    ODSTypeTableName = xpNav_ODS.GetAttribute("ODSObjectTypeTableName", "");
                    ODSType = xpNav_ODS.GetAttribute("ObjectType", "");
                }
                else
                {
                    OmErrorObj = new OMError(OMErrorCodes.ODS_OBJECT_NOT_FOUND, "ODS Object:" + ODSObjectName + " Not Found", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, MOMObjectName, OMErrorType.ERROR, null, null);
                }
            }
            return OmErrorObj;
        }

        /// <summary>
        /// <CreatedOn>14 June 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <Purpose>This Method Deletes an Object Type (used for ODS)</Purpose>
        /// </summary>
        /// <param name="InputXML">The Input XML contains MOM Object Name which will be 
        /// used to fetch the ODS Object Type which can then be deleted</param>
        /// <returns>An Error Object if an error occurs when performing delete operations</returns>
        private OMError DeleteObjectType(String MOMInputXML)
        {
            XPathDocument       xpDoc_MOMInput      = null;
            OMError             OmErrorObj          = null;
            XPathNavigator      xpNav_MOMInput      = null;
            String              ODSTypeName         = null;            
            String              ODSTableName        = null;            
            String              ODSType             = null;
            String              ObjectName          = null;
            String              TypeID              = null;
            String              ObjectTypeDetails   = null;
            EntityType          EntityTypeObj       = null;
            FacetType           FacetTypeObj        = null;
            String              ObjectTypeID        = null;
            String              AssociationXML      = null;
            String              StoredProcXML       = null;
            Hashtable           ListObjectstoBeDeleted  = null;
            String              UpdatedConfigXML    = null;

            xpDoc_MOMInput = new XPathDocument(new StringReader(MOMInputXML));
            xpNav_MOMInput = xpDoc_MOMInput.CreateNavigator().SelectSingleNode("INPUTXML");
            
            ObjectName  = xpNav_MOMInput.SelectSingleNode("OPERATION/OBJECTNAME").Value;
            TypeID      = xpNav_MOMInput.SelectSingleNode("OPERATION/PARAM").Value;

            OmErrorObj = GetODSObjectName_Ex(TypeID, out ODSTypeName, out ODSTableName, out ODSType, out ObjectTypeID);

            #region Delete From ALOM_FACETTYPE/ENTITYTYPE, SCHEMA, DROP ODS TABLE AND ALL ASSOCIATED RELATIONS
            if (!String.IsNullOrEmpty(ObjectTypeID) && OmErrorObj == null)
            {
                switch (ObjectName.ToUpper())
                {
                    case "ENTITYTYPE":
                        EntityTypeObj = new EntityType(OMDataManager);
                        OmErrorObj = EntityTypeObj.Read(Convert.ToUInt64(ObjectTypeID), ref ObjectTypeDetails);
                        if (OmErrorObj == null)
                        {
                            OmErrorObj = DeleteAllInfo(ODSTableName, "ALOM_ENTITYTYPE", ObjectTypeID, "ENTITYTYPEID", ObjectTypeDetails);
                            if (OmErrorObj == null)
                            {
                                RelationshipType RelationshipTypeObj = null;
                                RelationshipTypeObj = new RelationshipType(OMDataManager);
                                OmErrorObj = RelationshipTypeObj.ReadAllByContainerType(Convert.ToUInt64(ObjectTypeID), "ENTITY", "", ref AssociationXML);
                                if (OmErrorObj == null)
                                {
                                    OmErrorObj = DeleteRelations(AssociationXML);
                                    if (OmErrorObj == null)
                                    {

                                        OmErrorObj = RelationshipTypeObj.ReadAllByContainsType(Convert.ToUInt64(ObjectTypeID), "ENTITY", "", ref AssociationXML);
                                        if (OmErrorObj == null)
                                        {
                                            OmErrorObj = DeleteRelations(AssociationXML);
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case "FACETTYPE":
                        FacetTypeObj = new FacetType(OMDataManager);
                        OmErrorObj = FacetTypeObj.Read(Convert.ToUInt64(ObjectTypeID), ref ObjectTypeDetails);
                        if (OmErrorObj == null)
                        {                            
                            RelationshipType RelationshipTypeObj = null;
                            RelationshipTypeObj = new RelationshipType(OMDataManager);
                            OmErrorObj = RelationshipTypeObj.ReadAllByContainerType(Convert.ToUInt64(ObjectTypeID), "FACET", "", ref AssociationXML);
                            if (OmErrorObj == null)
                            {
                                OmErrorObj = DeleteRelations(AssociationXML);
                                if (OmErrorObj == null)
                                {
                                    OmErrorObj = RelationshipTypeObj.ReadAllByContainsType(Convert.ToUInt64(ObjectTypeID), "FACET", "", ref AssociationXML);

                                    if ((OmErrorObj != null) && (OmErrorObj.ErrorCode == OMErrorCodes.ASSOCIATION_NOT_FOUND))
                                        OmErrorObj = null;

                                    if (OmErrorObj == null)
                                    {
                                        OmErrorObj = DeleteRelations(AssociationXML);
                                        if (OmErrorObj == null)
                                        {
                                            OmErrorObj = DeleteAllInfo(ODSTableName, "ALOM_FACETTYPE", ObjectTypeID, "FACETTYPEID", ObjectTypeDetails);
                                        }
                                    }
                                }
                            }                            
                        }
                        break;
                }
            }
            #endregion

            #region Update Config XML's
            if ((!String.IsNullOrEmpty(ODSTypeName)) && (OmErrorObj == null))
            {
                OmErrorObj = DeleteODSObject(ODSTypeName, out UpdatedConfigXML, out ListObjectstoBeDeleted);
                if (OmErrorObj == null)
                {

                    OmErrorObj = deleteODSPopulationStoredProcedures(ODSTypeName, out StoredProcXML);
                    if (OmErrorObj == null)
                    {
                        OmErrorObj = saveODSConfigXMLsForDeleteOperations(StoredProcXML);
                        
                    }
                    if (ListObjectstoBeDeleted.Count > 0)
                    {
                        IDictionaryEnumerator IEnum =  ListObjectstoBeDeleted.GetEnumerator();
                        while (IEnum.MoveNext())
                        {
                            if (IEnum.Value.ToString() == "FACET")
                            {
                                FacetTypeObj = new FacetType(OMDataManager);
                                OmErrorObj = FacetTypeObj.Read(Convert.ToUInt64(IEnum.Key), ref ObjectTypeDetails);
                                if (OmErrorObj != null)
                                    return OmErrorObj;
                                ODSTableName = ALCommon.GetInnerValue(ref ObjectTypeDetails, "<TABLENAME>", "</TABLENAME>");
                                OmErrorObj = DeleteAllInfo(ODSTableName, "ALOM_FACETTYPE", IEnum.Key.ToString(), "FACETTYPEID", ObjectTypeDetails);
                                if (OmErrorObj != null)
                                    return OmErrorObj;
                            }
                            else
                            {
                                EntityTypeObj = new EntityType(OMDataManager);
                                OmErrorObj = EntityTypeObj.Read(Convert.ToUInt64(IEnum.Key), ref ObjectTypeDetails);
                                if (OmErrorObj != null)
                                    return OmErrorObj;
                                ODSTableName = ALCommon.GetInnerValue(ref ObjectTypeDetails, "<TABLENAME>", "</TABLENAME>");
                                OmErrorObj = DeleteAllInfo(ODSTableName, "ALOM_ENTITYTYPE", IEnum.Key.ToString(), "ENTITYTYPEID", ObjectTypeDetails);
                                if (OmErrorObj != null)
                                    return OmErrorObj;
                            }
                            ODSTypeName = ALCommon.GetInnerValue(ref ObjectTypeDetails, "<TYPENAME>", "</TYPENAME>");
                            if (!String.IsNullOrEmpty(ODSTypeName))
                            {
                                OmErrorObj = deleteODSPopulationStoredProcedures(ODSTypeName, out StoredProcXML);
                                if (OmErrorObj == null)
                                {
                                    OmErrorObj = saveODSConfigXMLsForDeleteOperations(StoredProcXML);
                                    if (OmErrorObj != null)
                                        return OmErrorObj;
                                }
                            }
                        }
                    }

                    if ((OmErrorObj == null) && (!String.IsNullOrEmpty(UpdatedConfigXML)))
                    {
                        File.WriteAllText(ODSConfigXMLPath, UpdatedConfigXML);
                    }
                    
                }
            }
            #endregion
            return OmErrorObj;
        }

        /// <summary>
        /// <CreatedOn>14 June 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// </summary>
        /// <param name="AssociationXML">This method Deletes All relationships from the ALOM_RELATIONSHIPTYPE table</param>
        /// <returns></returns>
        private OMError DeleteRelations(String AssociationXML)
        {
            XPathDocument       xpDoc_Input     = null;            
            OMError             OmErrorObj      = null;
            String              DeleteQuery     = null;
            StringBuilder       sb_TypeID       = null;
            
            if ((AssociationXML != "") && (AssociationXML != null))
            {
                xpDoc_Input = new XPathDocument(new StringReader(AssociationXML));
                XPathNodeIterator xpIter_Rel = xpDoc_Input.CreateNavigator().Select("RELATIONSHIPTYPECOLLECTION/RELATIONSHIPTYPE");
                if (xpIter_Rel.Count > 0)
                {
                    sb_TypeID = new StringBuilder(10);
                    sb_TypeID.Append("(");
                    
                    while (xpIter_Rel.MoveNext())
                    {
                        sb_TypeID.AppendFormat("{0},", xpIter_Rel.Current.SelectSingleNode("RELATIONSHIPTYPEID").Value);                        
                    }
                    sb_TypeID = sb_TypeID.Replace(",", ")", sb_TypeID.Length - 1, 1);
                    OmErrorObj = OMDataManager.DataProvider.AddClause("RELATIONSHIPTYPEID", OMFieldDataType.INTEGER, OMOperator.IN, sb_TypeID.ToString(), OMConjunction.NOTHING);
                    if (OmErrorObj == null)
                    {
                        OmErrorObj = OMDataManager.DataProvider.PrepareDeleteQuery("ALOM_RELATIONSHIPTYPES", ref DeleteQuery);
                        if (OmErrorObj != null)
                            return OmErrorObj;
                        OmErrorObj = OMDataManager.DataProvider.Execute(DeleteQuery);
                    }
                }
            }
            return OmErrorObj;
        }


        /// <summary>
        /// <CreatedOn>14 June 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <Purpose></Purpose>
        /// </summary>
        /// <param name="MOMObjectTypeID">This variable will store the Object Type ID of the MOM Object</param>
        /// <param name="ODSObjectName">Based on the MOM Object Type ID the ODS Object Name will returned in this out parameter</param>
        /// <param name="ODSTypeTableName">Based on the MOM Object Type ID the ODS Object Table Name will returned in this out parameter</param>
        /// <param name="ODSType">Based on the MOM Object Type ID the ODS Type will returned in this out parameter</param>
        /// <param name="ODSObjectTypeID">Based on the MOM Object Type ID the ODS Object Type ID will returned in this out parameter</param>
        /// <returns></returns>
        private OMError GetODSObjectName_Ex(String MOMObjectTypeID, out String ODSObjectName, out String ODSTypeTableName, out String ODSType, out String ODSObjectTypeID)
        {
            
            OMError         OmErrorObj = null;
            XPathNavigator  xpNav_ODS = null;

            
            XPathDocument xpDoc_Config = new XPathDocument(ODSConfigXMLPath);
            ODSObjectName       = String.Empty;
            ODSTypeTableName    = String.Empty;
            ODSType             = String.Empty;
            ODSObjectTypeID     = String.Empty;
            xpNav_ODS = xpDoc_Config.CreateNavigator().SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@MOMObjectTypeID='" + MOMObjectTypeID + "']");
            if (xpNav_ODS != null)
            {
                ODSObjectName       = xpNav_ODS.GetAttribute("ODSObjectTypeName", "");
                ODSTypeTableName    = xpNav_ODS.GetAttribute("ODSObjectTypeTableName", "");
                ODSType             = xpNav_ODS.GetAttribute("ObjectType", "");
                ODSObjectTypeID     = xpNav_ODS.GetAttribute("ODSObjectTypeID", "");
            }            
            return OmErrorObj;
        }

        
        /// <summary>
        /// <CreatedOn>15 June 2011</CreatedOn>
        /// <CreatedBy>Mugdha Moolay Gadre</CreatedBy>
        /// <Purpose>This method deletes an ODS Object from the MOM config XML</Purpose>
        /// </summary>
        /// <param name="ODSObjectName">The Name of the ODS Object to be deleted</param>
        /// <returns>An Error Object if an ODS Object is not found.</returns>
        private OMError DeleteODSObject(String ODSObjectName, out String UpdateConfigXML, out  Hashtable ObjectsTobeDeleted)
        { 
            
            XmlDocument OdsConfigDoc        = null;
            XmlNode     OdsObjNode          = null;
            XmlNode     RelatedMOMObjNode   = null;            
            XmlNode     FieldNode           = null;
            OMError     OmErrorObj          = null;
            String      MOMObjectName       = null;
            XmlNodeList ODSNodeList         = null;

            UpdateConfigXML = String.Empty;
            ObjectsTobeDeleted = new Hashtable();
            
            if (!File.Exists(ODSConfigXMLPath))
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_MOM_SCHEMA_FILE_NOT_FOUND, "MOM Schema Config File at path :" + ODSConfigXMLPath + " not found.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectName, OMErrorType.ERROR, null, null);
                return OmErrorObj;
            }
            OdsConfigDoc = new XmlDocument();
            OdsConfigDoc.Load(ODSConfigXMLPath);
            OdsObjNode = OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject[@ODSObjectTypeName='" + ODSObjectName + "']");
            if (OdsObjNode == null)
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_OBJECT_NOT_FOUND, "ODS Object:" + ODSObjectName + " not found in MOM Schema config XML.", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, ODSObjectName, OMErrorType.ERROR, null, null);
                return OmErrorObj;
            }
            RelatedMOMObjNode   = OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS");
            MOMObjectName       = ((XmlElement)OdsObjNode).GetAttribute("MOMObjectTypeName");
            RelatedMOMObjNode.RemoveChild(OdsObjNode);
            
            // Also check if the ODS Object is used as a MOM Object.
            if (!String.IsNullOrEmpty(MOMObjectName))
            {
                ODSNodeList = RelatedMOMObjNode.SelectNodes("ODSObject[MOMPrimaryKeyAsODSFields/Field[@MOMObjectTypeName='" + MOMObjectName + "']]");
                foreach (XmlNode ODSNode in ODSNodeList)
                {
                    FieldNode = OdsConfigDoc.SelectSingleNode("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject/MOMPrimaryKeyAsODSFields/Field[@MOMObjectTypeName='" + ((XmlElement)ODSNode).GetAttribute("MOMObjectTypeName") + "']");
                    if (FieldNode == null)
                    {
                        //ObjectsTobeDeleted.Add(((XmlElement)ODSNode).GetAttribute("ODSObjectTypeName"));
                        ObjectsTobeDeleted.Add(((XmlElement)ODSNode).GetAttribute("ODSObjectTypeID"), ((XmlElement)ODSNode).GetAttribute("ObjectType"));
                        RelatedMOMObjNode.RemoveChild(ODSNode);
                    }
                }
            }
            UpdateConfigXML = OdsConfigDoc.OuterXml;
            //OdsConfigDoc.Save(ODSConfigXMLPath);
            return OmErrorObj;
        }

        private OMError GetODSRelationshipsFromConfig(String MOMRelName, out XPathNodeIterator xpIter_ODSRelns)
        {
            OMError             OmErrorObj      = null;
            xpIter_ODSRelns = null;
            if (File.Exists(ODSConfigXMLPath))
            {
                XPathDocument xpDoc_ODSConfig = new XPathDocument(ODSConfigXMLPath);
                xpIter_ODSRelns = xpDoc_ODSConfig.CreateNavigator().Select("ODSMOMSchemaConfiguration/RelatedMOMObjectsForODS/ODSObject/MOMPrimaryKeyAsODSFields/Field[@MOMRelationshipName='" + MOMRelName + "']");
            }
            else
            {
                OmErrorObj = new OMError(OMErrorCodes.ODS_MOM_SCHEMA_FILE_NOT_FOUND, "ODS Schema Configuration XML not found", OMErrorObjectType.OBJECTMANAGER_ODS_CLASS, "ODS", OMErrorType.ERROR, null, null);
            }
            return OmErrorObj;
        }
        #endregion
    }
}
