#region Using Directives

#region System

using System;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Text;
using System.Reflection;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text.RegularExpressions;
using System.Data.SqlClient;
using System.Threading;
using System.Data;
using Microsoft.Data;
using Microsoft.Data.SqlXml;
using System.Transactions;
using Teradata.Client.Provider;

#endregion

#region Assetlink

using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.ObjectManager;
using Assetlink.ObjectManager.ETFTManagement;
using Assetlink.ObjectManager.EntityFacetManagement;  
using Assetlink.Diagnostic;
using Assetlink.FileAccessLayer;
using Assetlink.ObjectManager.GroupHierarchy;
using Assetlink.ObjectManager.Search;

#endregion

#endregion

namespace Assetlink.ObjectManager.Search
{
    public class NSearch : ObjectManagerBase
    {
        #region FileTransfer Delegate
        private FileTransferDelegate.OnFileTransfer m_delFileTransfer = null;
        public FileTransferDelegate.OnFileTransfer FileTransfer
        {
            set
            {
                m_delFileTransfer = value;
            }
        }
        #endregion

        #region Private Members
        
        NSearchCommon NSearchCommonObj = null;
        //Following int_UniqueIDCount number does not initialize to ZERO because one ParentID attribute from search hierarchy object is used 
        // while defining the XML structure. The ParentID number is started with ZERO. 
        //Few cases (IF conditions) are handled with this parent number and the unique number.
        //If the numbers are same then some operations are avoided.
        Int64 int_UniqueIDCount = 1000;

        #endregion

        #region Constructors

        public NSearch()
        {
        }

        public NSearch(ulong UserID, string CurrentLanguage, string TenantName)
            : base(UserID, CurrentLanguage, TenantName)
        {
        }

        public NSearch(DataManager DataManagerObj)
            : base(DataManagerObj)
        {
        }

        public NSearch(ulong UserID, string CurrentLanguage,
                            string TenantName, DataManager DataManagerObj)
            : base(UserID, CurrentLanguage, TenantName, DataManagerObj)
        {
        }

        #endregion Constructors

        #region Public Methods

        /// <summary>
        /// Entry point for NLevel Search.
        /// <FeatureList>
        /// 1. <feature>N Level Search</feature>
        /// 2. <feature>Support for Aggregate Functions</feature>
        /// 3. <feature>Sorting on Fields with Sequence Number</feature>
        /// 4. <feature>Support for Brackets and Conjuntions like AND/OR for Filters</feature>
        /// 5. <feature>Specifying Privilege Levels for Target Entity/Related Entity/Related Facets</feature>
        /// 6. <feature>Sorting on Facets with Fieldlist</feature>
        /// </FeatureList>
        /// </summary>
        /// <param name="InputXML"></param>
        /// <param name="OutputSearchResult"></param>
        /// <returns></returns>
        public OMError GetHierarchialData(string InputXML, ref string OutputSearchResult)
        {
            DataSet DS_SearchResult = null;
            return GetHierarchialData(InputXML, false, ref OutputSearchResult, ref DS_SearchResult);
        }

        /// <summary>
        /// Entry point for NLevel Search.
        /// <FeatureList>
        /// 1. <feature>N Level Search</feature>
        /// 2. <feature>Sorting on Fields with Sequence Number</feature>
        /// 3. <feature>Support for Brackets and Conjuntions like AND/OR for Filters</feature>
        /// 4. <feature>Specifying Privilege Levels for Target Entity/Related Entity/Related Facets</feature>
        /// 5. <feature>Sorting on Facets with Fieldlist</feature>
        /// </FeatureList>
        /// </summary>
        /// <param name="InputXML"></param>
        /// <param name="DS_SearchResult"></param>
        /// <returns></returns>
        public OMError GetHierarchialData(string InputXML, ref DataSet DS_SearchResult)
        {
            String OutputSearchResult = null;
            return GetHierarchialData(InputXML, true, ref OutputSearchResult, ref DS_SearchResult);
        }

        #endregion

        #region Private Methods

        #region Data Structure Methods.

        private OMError GetHierarchialData(String InputXML, Boolean FetchDataAsScalar, ref String OutputSearchResult, ref DataSet DS_SearchResult)
        {
            OMError             ErrorObj                    = null;
            OMError             ReturnErrorObj              = null;
            XmlNode             RootNode                    = null;
            XmlElement          TargetEntityNode            = null;
            XmlNode             MaxResultSizeNode           = null;
            XmlNode             SimpleSearchNode            = null;
            XmlNode             DocumentSearchNode          = null;
            XmlNodeList         DocumentSearchPathList      = null;
            XmlNode             SearchAllFacetsNode         = null;
            XmlNode             Search_In_All_FacetsNode    = null;            
            XmlNode             ThresholdCountNode          = null;
            XmlNodeList         TESiblingNodeList           = null;
            long                TagID                       = 10;
            long                ParentID                    = 0;
            long                UniqueSortID                = 0;
            int                 SortValueCounter            = 0;
            int                 BinderFieldCount            = 0;         //For storing binder field count
            HierarchyObject     HierarchyObj                = null;
            SortPartition       SortPartitionObject         = null;
            Boolean             IsFileSystemRequested       = false;
            Boolean             SearchAllFacetsFlag         = false;
            XmlNode             FileSystemNode              = null;
            XmlNode             NewRelationshipNode         = null;
            XmlNode             AuthorizationRequired       = null;
            XmlNode             IsSimpleSearchCaseSensitive = null;
            XmlAttribute        FileSystemName              = null;
            StringBuilder       DynamicSQL                  = null;
            XmlAttribute        xmlAttr_PageNumber          = null;
            XmlAttribute        xmlAttr_RecordsPerPage      = null;
			String				FromODS						= null;
            String              ConnectionString            = null;
            XmlNode             ObjectAliasNameNode         = null;

            try
            {

                if (OMDataManager == null)
                    OMDataManager = new DataManager(TenantName);

                NSearchCommonObj = new NSearchCommon(this.UserID,this.CurrentLanguage,this.TenantName);  

                // Process TargetEntity Node.
                // Prepare a List of HierarchyObject

                NSearchCommonObj.HierarchyObjectList = new List<HierarchyObject>();
                NSearchCommonObj.SortPartitionList = new List<SortPartition>();
                List<Predecessor> TEPredecessorList = new List<Predecessor>();
                List<Sibling> SiblingList = new List<Sibling>();
                NSearchCommonObj.m_SPColTagIDList = new List<SortPartitionColumnTagIDPair>();


                NSearchCommonObj.m_UniqueIDList = new ArrayList();

                // NSearchCommonObj datastructure will hold the sort partition columns.

                NSearchCommonObj.m_SortPartitionColumnsList = new SortedList();

                HierarchyObj = new HierarchyObject();

                NSearchCommonObj.InputXMLDocument = new XmlDocument();
                NSearchCommonObj.InputXMLDocument.LoadXml(InputXML);

                RootNode = NSearchCommonObj.InputXMLDocument.DocumentElement;

                HierarchyObj.m_ObjectTypeName       = RootNode.SelectSingleNode("TARGETOBJECT").InnerText;
                if ((ObjectAliasNameNode = RootNode.SelectSingleNode("OBJECTALIAS")) != null)
                    HierarchyObj.m_ObjectAliasName  = ObjectAliasNameNode.InnerText;
                else
                    HierarchyObj.m_ObjectAliasName  = HierarchyObj.m_ObjectTypeName;

                HierarchyObj.m_ObjectType           = RootNode.SelectSingleNode("TARGETOBJECTTYPE").InnerText;
                HierarchyObj.m_CollectionName       = HierarchyObj.m_ObjectTypeName + "COLLECTION";
                HierarchyObj.m_RelationshipName     = "TARGETENTITY";

				if (RootNode.Attributes["FromODS"] != null)
					FromODS = RootNode.Attributes["FromODS"].Value;

                TargetEntityNode = (XmlElement)RootNode.SelectSingleNode("TARGETENTITY");

                AuthorizationRequired = RootNode.SelectSingleNode("AUTHORIZATIONREQUIRED");
                if (AuthorizationRequired != null)
                    NSearchCommonObj.IsAuthorizationRequired = Convert.ToBoolean(AuthorizationRequired.InnerText);

                //check for MaxResultCount 
                MaxResultSizeNode = RootNode.SelectSingleNode("MAXRESULTSIZE");
                if (MaxResultSizeNode != null)
                {
                    if (OMDataManager.DataProvider.IsPagingSupported && MaxResultSizeNode.InnerText.CompareTo("0") != 0)
                    {
                        //Set RecordsPerPage=MAXRESULTSIZE and PageNumber=1 so that SORTING and MAXRESULTSIZE will work together properly
                        //Create PageNumber attribute
                        xmlAttr_PageNumber = NSearchCommonObj.InputXMLDocument.CreateAttribute(ConfigConstantAttributes.ATTRIB_PAGE_NUMBER);
                        xmlAttr_PageNumber.Value = "1";
                        TargetEntityNode.Attributes.Append(xmlAttr_PageNumber);
                        xmlAttr_PageNumber = null;

                        //Create RecordsPerPage attribute
                        xmlAttr_RecordsPerPage = NSearchCommonObj.InputXMLDocument.CreateAttribute(ConfigConstantAttributes.ATTRIB_RECORDS_PER_PAGE);
                        xmlAttr_RecordsPerPage.Value = MaxResultSizeNode.InnerText;
                        TargetEntityNode.Attributes.Append(xmlAttr_RecordsPerPage);
                        xmlAttr_RecordsPerPage = null;

                        //Putting this to show that MaxResults is there. Because of the above changes, it will get converted to paging and this will
                        //be ignored
                    }

                    HierarchyObj.m_MaxResultSize = Convert.ToUInt64(MaxResultSizeNode.InnerText);

                }

                // check for simple search string
                SimpleSearchNode = RootNode.SelectSingleNode("SEARCHSTRING");
                if (SimpleSearchNode != null)
                {
                    HierarchyObj.m_SimpleSearchString = SimpleSearchNode.InnerText;
                    IsSimpleSearchCaseSensitive = RootNode.SelectSingleNode("ISCASESENSITIVE");
                    if (IsSimpleSearchCaseSensitive != null)
                        HierarchyObj.m_IsSimpleSearchCaseSensitive = Convert.ToBoolean(IsSimpleSearchCaseSensitive.InnerText);
                }

                //Check for search in documents
                DocumentSearchNode = RootNode.SelectSingleNode("SEARCHINFILES");
                if (DocumentSearchNode != null)
                {
                    try
                    {
                        HierarchyObj.m_IsSearchInDocuments = Convert.ToBoolean(DocumentSearchNode.InnerText.ToString());
                        DocumentSearchPathList = RootNode.SelectNodes("SEARCHPATHS/SEARCHPATH/PATH");

                        if (DocumentSearchPathList != null)
                        {
                            foreach (XmlNode SearchPath in DocumentSearchPathList)
                            {
                                HierarchyObj.m_FileRelationships += "$ALSEP$" + SearchPath.InnerText.ToString();
                            }
                        }
                    }
                    catch
                    {
                        HierarchyObj.m_IsSearchInDocuments = false;
                    }
                    
                }

                //check for ThresholdCount 
                ThresholdCountNode = RootNode.SelectSingleNode("THRESHOLD_COUNT");
                if (ThresholdCountNode != null)
                    HierarchyObj.m_ThresholdResultCount = Convert.ToInt64(ThresholdCountNode.InnerText);

                //TargetEntityNode = (XmlElement)RootNode.SelectSingleNode("TARGETENTITY");


                Search_In_All_FacetsNode = RootNode.SelectSingleNode("SEARCH_IN_ALL_FACETS");
                if (Search_In_All_FacetsNode != null)
                    HierarchyObj.m_IsSearch_In_All_Facets = Convert.ToBoolean(Search_In_All_FacetsNode.InnerText.ToString());                
                
                SearchAllFacetsNode = RootNode.SelectSingleNode("SEARCHALLFACETS");
                if (SearchAllFacetsNode != null)
                {
                    SearchAllFacetsFlag = Convert.ToBoolean(SearchAllFacetsNode.InnerText.ToString());
                    if (SearchAllFacetsFlag)
                        HierarchyObj.m_IsSearchAllFacets = SearchAllFacetsFlag;
                    
                }

                ErrorObj = PrepareHierarchyObject(TargetEntityNode, ref HierarchyObj, ParentID, TagID, ref IsFileSystemRequested);
                if (ErrorObj != null)
                    return ErrorObj;
                NSearchCommonObj.HierarchyObjectList.Add(HierarchyObj);

                // Prepare Predecessor Object
                Predecessor PredecessorObjforTE = new Predecessor();
                PredecessorObjforTE.m_PredecessorID = TagID;
                PredecessorObjforTE.m_RelationshipName = HierarchyObj.m_ObjectTypeName;
                PredecessorObjforTE.m_StartColumnNumber = 5;
                PredecessorObjforTE.m_NumberofFields = HierarchyObj.FieldCollectionList.Count;
                PredecessorObjforTE.m_RelatedObjectLevel = HierarchyObj.m_RelatedObjectLevel;
                TEPredecessorList.Add(PredecessorObjforTE);

                // 4 because it first 2 columns are for Parent and Tag in Universal table and next 2 are for TECollection and TEData for SQLXML.
                NSearchCommonObj.m_TotalNumberOfFields = 4 + HierarchyObj.FieldCollectionList.Count + HierarchyObj.AggregateAttrList.Count;

                // Preparing a sort partition object for Target Entity.
                SortPartitionObject = new SortPartition();
                SortPartitionObject.m_NodeID = TagID; // Assigned a unique id for Target Entity NodeID.
                SortPartitionObject.m_ParentID = 0;
                SortPartitionObject.m_RelationshipName = HierarchyObj.m_ObjectTypeName;
                SortPartitionObject.m_SortPartitionColumnName = "SPCol0";

                if (IsFileSystemRequested)
                {
                    FileSystemNode = NSearchCommonObj.InputXMLDocument.CreateElement("RELATIONSHIP");
                    FileSystemName = NSearchCommonObj.InputXMLDocument.CreateAttribute("NAME");
                    FileSystemName.InnerText = "AL$FILESYSTEM_AL$" + TagID.ToString();
                    FileSystemNode.Attributes.Append(FileSystemName);
                    // Get Binder and FileFields.
                    ErrorObj = GetBinderandFileFields();
                    NSearchCommonObj.IsFileBinderFieldsFetched = true; 
                }

                // Check for SEARCHALLFACETS flag
                if (SearchAllFacetsFlag)
                {
                    ErrorObj = GetAssociatedFacetTypes(HierarchyObj, TargetEntityNode, ref NewRelationshipNode);
                    if (ErrorObj != null)
                        return ErrorObj;
                    else
                        TargetEntityNode = (XmlElement)NewRelationshipNode;
                }

                // Add Siblings for TE
                TESiblingNodeList = TargetEntityNode.SelectNodes("RELATIONSHIP");

                // NSearchCommonObj code ensures that FileSystemRelation Node is always first node in XML below TE.
                if (TESiblingNodeList.Count > 0)
                {
                    if (IsFileSystemRequested)
                    {
                        foreach (XmlNode TESiblingNode in TESiblingNodeList)
                        {
                            TargetEntityNode.InsertBefore(FileSystemNode, TESiblingNode);
                            break;
                        }
                    }
                }
                else
                {
                    if (IsFileSystemRequested)
                    {
                        TargetEntityNode.AppendChild(FileSystemNode);
                    }
                }

                TESiblingNodeList = TargetEntityNode.SelectNodes("RELATIONSHIP");
                // NSearchCommonObj condition says that in TE there exists a field of type BINDER/FILE.
                SortValueCounter++;
                if (TESiblingNodeList.Count > 0)
                {
                    //Random RandomObject = new Random();
                    foreach (XmlNode TESiblingNode in TESiblingNodeList)
                    {
                        Sibling TESiblingObj = new Sibling();
                        TESiblingObj.m_RelationshipName = TESiblingNode.Attributes.GetNamedItem("NAME").InnerText;
                        BinderFieldCount = 0;
                        UniqueSortID = 0;
                        ErrorObj = GetUniqueId(ref UniqueSortID);
                        TESiblingObj.m_PredecessorID = TagID;
                        //TESiblingObj.m_SortPartitionID = RandomObject.Next(11, 50); 
                        TESiblingObj.m_SortPartitionID = UniqueSortID;
                        TESiblingObj.m_SortValue = (SortValueCounter * 2);
                        SortPartitionObject.SiblingList.Add(TESiblingObj);

                        /* If the relationship node is for FILESYSTEM, the Sort (column) Value count needs to be incremented by number of binder fields
                         * requested. This is to support fetching multiple binder fields.
                         */
                        if (TESiblingObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                        {
                            ErrorObj = GetBinderFieldCount(HierarchyObj, out BinderFieldCount);
                            if (ErrorObj != null)
                                return ErrorObj;
                            
                            /* Note: Here 1 is added to the counter so that it will be used for fetching files when multiple binder and file fields are requested.
                             * This is required to fetch all binder fields first and then file fields.
                             * */
                            SortValueCounter = SortValueCounter + BinderFieldCount + 1;
                        }
                        else
                            SortValueCounter++;
                    }
                }

                NSearchCommonObj.SortPartitionList.Add(SortPartitionObject);

                // Give a recursive call which will populate the datastructure.
                ErrorObj = ProcessRelationships(TargetEntityNode, TagID, ref NSearchCommonObj.SortPartitionList, TEPredecessorList);
                if (ErrorObj != null)
                    return ErrorObj;

                // Process HierarchyObjectList to form Stored Procedure and format search results.  
                //TEXMLStructQuery = new StringBuilder();

                if (NSearchCommonObj.IsAuthorizationRequired)
                    ErrorObj = GetGroupIDList(ref NSearchCommonObj.GroupIDList, this.UserID);     

                List<String> ExecBeforeDynamicQueries = null;
                List<String> CleanupAfterDynamicQueries = null;
                                

                if (!FetchDataAsScalar)
                {
                    ErrorObj = OMDataManager.DataProvider.PrepareDynamicSQL(ref NSearchCommonObj, ref DynamicSQL,
                                                                            ref ExecBeforeDynamicQueries, ref CleanupAfterDynamicQueries);


                    if (ErrorObj != null)
                        return ErrorObj;

                    List<String> DynamicSQLs    = new List<string>();
                    List<String> OutputResults  = new List<string>();
                    Dictionary<String, String> ConnectionStrings = new Dictionary<string, string>();

                    //Add the ExecBeforeDynamicQueries to the DynamicSQLs list
                    if (ExecBeforeDynamicQueries != null)
                    {
                        foreach (String DynQuery in ExecBeforeDynamicQueries)
                        {
                            DynamicSQLs.Add(DynQuery);
                        }
                    }

                    //Add the DynamicSQL to the DynamicSQLs list
                    DynamicSQLs.Add(DynamicSQL.ToString());

                    //Add the CleanupAfterDynamicQueries to the DynamicSQLs list
                    if (CleanupAfterDynamicQueries != null)
                    {
                        foreach (string DynQuery in CleanupAfterDynamicQueries)
                        {
                            DynamicSQLs.Add(DynQuery);
                        }
                    }

                    //Since paging is supported or not is checked in QueryProcessor and paging query requires connectionString.
                    if ((!String.IsNullOrEmpty(FromODS)) && (FromODS.ToUpper() == "TRUE"))
                    {
                        ConnectionStrings.Add("ConnectionString", OMDataManager.ODSConnectionString);
                        ConnectionStrings.Add("ConnectionStringForRead", OMDataManager.ODSConnectionStringForRead);
                    }
                    else
                    {
                        ConnectionStrings.Add("ConnectionStringForRead", OMDataManager.ConnectionStringForRead);
                        ConnectionStrings.Add("ConnectionString", OMDataManager.ConnectionString);
                    }
                    //Executes DynamicSQL's

                    ErrorObj = ExecuteDynamicSQLsEx(DynamicSQLs, ConnectionStrings, NSearchCommonObj, ref OutputResults);
                    if (ErrorObj != null)
                        return ErrorObj;

                    foreach (String OutputResult in OutputResults)
                    {
                        if (!String.IsNullOrEmpty(OutputResult))
                            OutputSearchResult = OutputResult;
                    }
                }
                else
                {
                    ErrorObj = OMDataManager.DataProvider.PrepareDynamicSQL_Scalar(ref NSearchCommonObj, ref DynamicSQL);
                    if (ErrorObj != null)
                        return ErrorObj;

                    if ((!String.IsNullOrEmpty(FromODS)) && (FromODS.ToUpper() == "TRUE"))
                    {
                        ConnectionString = OMDataManager.ODSConnectionString;
                    }
                    else
                    {
                        ConnectionString = OMDataManager.ConnectionString;
                    }

                    //Executes DynamicSQL's
                    ErrorObj = ExecuteDynamicSQLsEx_Scalar(DynamicSQL.ToString(), ConnectionString, NSearchCommonObj, ref DS_SearchResult);
                    if (ErrorObj != null)
                        return ErrorObj;
                }
                return ErrorObj;
 
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in Get N Level Data",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        private OMError ProcessRelationships(XmlElement RelationshipNode, long TagID, ref List<SortPartition> SortPartitionList, List<Predecessor> PredecessorListNodes)
        {
            OMError                     ErrorObj                        = null;
            OMError                     ReturnErrorObj                  = null;
            XmlNodeList                 RelationshipNodeList            = null;
            XmlNodeList                 SiblingNodeList                 = null;
            XmlElement                  ExistingRelationshipNode        = null;
            XmlNode                     RelationInfoNode                = null;
            long                        ParentID                        = 0;
            long                        SortPartitionID                 = 0;
            ulong                       ParentObjectTypeID              = 0;
            ulong                       ContainerTypeID                 = 0;
            ulong                       ContainsTypeID                  = 0;
            ulong                       RelationshipTypeID              = 0;
            long                        UniqueTagID                     = 0;
            long                        UniqueSortID                    = 0;
            int                         SortValueCounter                = 0;
            int                         BinderFieldCount                = 0;        //For storing binder field count
            String                      ObjectTypeName                  = null;
            String                      ParentObjectType                = null;
            SearchUtils                 SearchUtilsObj                  = null;
            bool                        IsRelationshipValid             = false;
            Predecessor                 PredecessorObject               = null;
            List<Predecessor>           TempPredecessorList             = new List<Predecessor>();
            List<Sibling>               SiblingList                     = new List<Sibling>();
            Boolean                     IsFileSystemRequested           = false;
            XmlNode                     FileSystemNode                  = null;
            XmlAttribute                FileSystemName                  = null;
            String                      ParentObjectTypeName            = null;
            HierarchyObject             ParentHierarchyObject           = null;
            Boolean                     IsBinderRequested               = false;
            Boolean                     IsFileRequested                 = false;
            Boolean                     SearchAllFacetsFlag             = false;
            XmlNode                     NewRelationshipNode             = null;
            XmlNode                     SearchAllFacetsNode             = null;
            XmlNode                     HierarchyNode                   = null;

            try
            {
                // Variable for storing sort partition information.
                SortPartition SortPartitionObj = new SortPartition();
                HierarchyObject HierarchyObj = new HierarchyObject();
                //RandomObjectforTag = new Random(); ;
                //RandomObjectforSortID = new Random(); 

                // Relationship is resolved by looking at the ParentObjectTypeID 
                foreach (HierarchyObject TempHierarchyObj in NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObj.m_TagID == TagID)
                    {
                        ParentObjectTypeID = TempHierarchyObj.m_ObjectTypeID;
                        ParentObjectType = TempHierarchyObj.m_ObjectType;
                        ParentObjectTypeName = TempHierarchyObj.m_ObjectTypeName;
                        ObjectTypeName = ParentObjectTypeName;
                        ParentHierarchyObject = TempHierarchyObj;
                        break;
                    }
                }

                // Check whether its a Related Entity or Facet. NSearchCommonObj checks for Relationship Name.
                if (RelationshipNode.HasAttribute("NAME"))
                {
                    // Generating a New TagID for Relationship node which is processed.
                    ParentID = TagID;
                    UniqueTagID = 0;
                    ErrorObj = GetUniqueId(ref UniqueTagID);
                    if (ErrorObj != null)
                        return ErrorObj;

                    TagID = UniqueTagID;

                    HierarchyObj.m_RelationshipName = RelationshipNode.Attributes.GetNamedItem("NAME").InnerText;
                    HierarchyObj.m_CollectionName = RelationshipNode.Attributes.GetNamedItem("NAME").InnerText;
                    SortPartitionObj.m_RelationshipName = RelationshipNode.Attributes.GetNamedItem("NAME").InnerText;

                }

                // Get RelationshipInfo based on RelationshipName
                if (HierarchyObj.m_RelationshipName != null)
                {
                    // Preparing PredecessorList.
                    TempPredecessorList.Clear();
                    foreach (Predecessor TempPredecessorObj in PredecessorListNodes)
                    {
                        Predecessor NewPredecessorObj = new Predecessor();
                        NewPredecessorObj.m_PredecessorID = TempPredecessorObj.m_PredecessorID;
                        NewPredecessorObj.m_RelationshipName = TempPredecessorObj.m_RelationshipName;
                        NewPredecessorObj.m_StartColumnNumber = TempPredecessorObj.m_StartColumnNumber;
                        NewPredecessorObj.m_NumberofFields = TempPredecessorObj.m_NumberofFields;
                        NewPredecessorObj.m_RelatedObjectLevel = TempPredecessorObj.m_RelatedObjectLevel;
                        if (TempPredecessorObj.m_RelationshipName == HierarchyObj.m_RelationshipName)
                        {
                            SortPartitionID = TempPredecessorObj.m_SortPartitionID;
                        }
                        TempPredecessorList.Add(NewPredecessorObj);
                    }
                    
                    if (!HierarchyObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                    {
                        // Reading relationship details based on its name.
                        SearchUtilsObj = new SearchUtils(OMDataManager);
                        SearchUtilsObj.ReadByRelationShipNameEx(HierarchyObj.m_RelationshipName, ref RelationInfoNode);
                        ContainsTypeID = Convert.ToUInt64(RelationInfoNode.SelectSingleNode("//CONTAINSTYPEID").InnerText);
                        ContainerTypeID = Convert.ToUInt64(RelationInfoNode.SelectSingleNode("//CONTAINERTYPEID").InnerText);
                        RelationshipTypeID = Convert.ToUInt64(RelationInfoNode.SelectSingleNode("//RELATIONSHIPTYPEID").InnerText);

                        // Check for relationship on Same Entity/Facet
                        if ((ParentObjectTypeID == ContainerTypeID) && (ParentObjectTypeID == ContainsTypeID))
                        {
                            // Check if Hierarchy Attribute is specified.
                            HierarchyNode = RelationshipNode.Attributes.GetNamedItem("HIERARCHY");
                            if (HierarchyNode == null)
                            {
                                IsRelationshipValid = true;

                                HierarchyObj.m_ObjectTypeID = ContainsTypeID;
                                HierarchyObj.m_ObjectType = RelationInfoNode.SelectSingleNode("//CONTAINSTYPE").InnerText;

                                if (ParentObjectType.ToUpper() == "ENTITY" && HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                                    HierarchyObj.m_IsRelatedEntity = true;
                                else
                                    HierarchyObj.m_IsRelatedEntity = false;

                                HierarchyObj.m_ContainerName = "CONTAINSID";
                                SearchUtilsObj.GetObjectTypeName(HierarchyObj.m_ObjectTypeID, HierarchyObj.m_ObjectType, ref ObjectTypeName);
                                HierarchyObj.m_ObjectTypeName = ObjectTypeName;
                            }
                            else
                            {
                                if (HierarchyNode.InnerText == "TOPDOWN")
                                {
                                    IsRelationshipValid = true;

                                    HierarchyObj.m_ObjectTypeID = ContainsTypeID;
                                    HierarchyObj.m_ObjectType = RelationInfoNode.SelectSingleNode("//CONTAINSTYPE").InnerText;

                                    if (ParentObjectType.ToUpper() == "ENTITY" && HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                                        HierarchyObj.m_IsRelatedEntity = true;
                                    else
                                        HierarchyObj.m_IsRelatedEntity = false;

                                    HierarchyObj.m_ContainerName = "CONTAINSID";
                                    SearchUtilsObj.GetObjectTypeName(HierarchyObj.m_ObjectTypeID, HierarchyObj.m_ObjectType, ref ObjectTypeName);
                                    HierarchyObj.m_ObjectTypeName = ObjectTypeName;
                                }
                                else
                                {
                                    if (HierarchyNode.InnerText == "BOTTOMUP")
                                    {
                                        IsRelationshipValid = true;
                                        HierarchyObj.m_ObjectTypeID = ContainerTypeID;
                                        HierarchyObj.m_ObjectType = RelationInfoNode.SelectSingleNode("//CONTAINERTYPE").InnerText;

                                        if (ParentObjectType.ToUpper() == "ENTITY" && HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                                            HierarchyObj.m_IsRelatedEntity = true;
                                        else
                                            HierarchyObj.m_IsRelatedEntity = false;

                                        HierarchyObj.m_ContainerName = "CONTAINERID";
                                        SearchUtilsObj.GetObjectTypeName(HierarchyObj.m_ObjectTypeID, HierarchyObj.m_ObjectType, ref ObjectTypeName);
                                        HierarchyObj.m_ObjectTypeName = ObjectTypeName;
                                    }
                                }
                            }
                        }
                        else
                        {
                            // Check for Parent ObjectTypeID or ObjectTypeName.
                            if (ParentObjectTypeID == ContainerTypeID)
                            {
                                IsRelationshipValid = true;

                                HierarchyObj.m_ObjectTypeID = ContainsTypeID;
                                HierarchyObj.m_ObjectType = RelationInfoNode.SelectSingleNode("//CONTAINSTYPE").InnerText;

                                if (ParentObjectType.ToUpper() == "ENTITY" && HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                                    HierarchyObj.m_IsRelatedEntity = true;
                                else
                                    HierarchyObj.m_IsRelatedEntity = false;

                                HierarchyObj.m_ContainerName = "CONTAINSID";
                                SearchUtilsObj.GetObjectTypeName(HierarchyObj.m_ObjectTypeID, HierarchyObj.m_ObjectType, ref ObjectTypeName);
                                HierarchyObj.m_ObjectTypeName = ObjectTypeName;

                            }
                            else if (ParentObjectTypeID == ContainsTypeID)
                            {
                                IsRelationshipValid = true;
                                HierarchyObj.m_ObjectTypeID = ContainerTypeID;
                                HierarchyObj.m_ObjectType = RelationInfoNode.SelectSingleNode("//CONTAINERTYPE").InnerText;

                                if (ParentObjectType.ToUpper() == "ENTITY" && HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                                    HierarchyObj.m_IsRelatedEntity = true;
                                else
                                    HierarchyObj.m_IsRelatedEntity = false;

                                HierarchyObj.m_ContainerName = "CONTAINERID";
                                SearchUtilsObj.GetObjectTypeName(HierarchyObj.m_ObjectTypeID, HierarchyObj.m_ObjectType, ref ObjectTypeName);
                                HierarchyObj.m_ObjectTypeName = ObjectTypeName;

                            }
                        }
                        HierarchyObj.m_RelationshipTypeID = RelationshipTypeID;

                        // If Relationship is valid between Parent and its child then add processed data to its datastructure.
                        if (IsRelationshipValid)
                        {
                            ErrorObj = PrepareHierarchyObject(RelationshipNode, ref HierarchyObj, ParentID, TagID, ref IsFileSystemRequested);
                            // Set the level of Hierarchy Object from InputXML.
                            HierarchyObj.m_RelatedObjectLevel = PredecessorListNodes.Count;
                            //HierarchyObj.m_RelatedObjectNumber = SiblingCounter;
                            NSearchCommonObj.RelatedObjectNumber[PredecessorListNodes.Count] +=1;
                            HierarchyObj.m_RelatedObjectNumber = NSearchCommonObj.RelatedObjectNumber[PredecessorListNodes.Count];
                            
                            if (IsFileSystemRequested)
                            {
                                FileSystemNode = NSearchCommonObj.InputXMLDocument.CreateElement("RELATIONSHIP");
                                FileSystemName = NSearchCommonObj.InputXMLDocument.CreateAttribute("NAME");
                                FileSystemName.InnerText = "AL$FILESYSTEM_AL$" + TagID.ToString();
                                FileSystemNode.Attributes.Append(FileSystemName);
                                //RelationshipNode.AppendChild(FileSystemNode);  
                            }

                            //Code for SiblingList
                            // Iterate the SortPartitionObject List for getting SortPartitionID
                            foreach (SortPartition TempSortPartitionObj in NSearchCommonObj.SortPartitionList)
                            {
                                if (TempSortPartitionObj.m_NodeID == ParentID)
                                {
                                    // iterate sibling list
                                    foreach (Sibling TempSibling in TempSortPartitionObj.SiblingList)
                                    {
                                        if (TempSibling.m_RelationshipName == HierarchyObj.m_RelationshipName)
                                        {
                                            SortPartitionID = TempSibling.m_SortPartitionID;
                                            break;
                                        }
                                    }
                                    break;
                                }
                            }

                            // Prepare sibling List.
                            SiblingNodeList = RelationshipNode.SelectNodes("RELATIONSHIP");
                            if (SiblingNodeList.Count > 0)
                            {
                                if (IsFileSystemRequested)
                                {
                                    foreach (XmlNode SiblingNode in SiblingNodeList)
                                    {
                                        RelationshipNode.InsertBefore(FileSystemNode, SiblingNode);
                                        break;
                                    }
                                }
                            }
                            else
                            {
                                if (IsFileSystemRequested)
                                {
                                    RelationshipNode.AppendChild(FileSystemNode);
                                }
                            }

                            // Check for SearchAllFacets

                            // Check for SEARCHALLFACETS flag
                            SearchAllFacetsNode = RelationshipNode.SelectSingleNode("SEARCHALLFACETS");
                            if (SearchAllFacetsNode != null)
                            {
                                SearchAllFacetsFlag = Convert.ToBoolean(SearchAllFacetsNode.InnerText.ToString());
                                if (SearchAllFacetsFlag)
                                {
                                    ErrorObj = GetAssociatedFacetTypes(HierarchyObj, RelationshipNode, ref NewRelationshipNode);
                                    if (ErrorObj != null)
                                        return ErrorObj;
                                    else
                                        RelationshipNode = (XmlElement)NewRelationshipNode;
                                }
                            }                            

                            SiblingNodeList = RelationshipNode.SelectNodes("RELATIONSHIP");

                            SortValueCounter = 0;
                            SortValueCounter++;
                            //Thread.Sleep(10);  
                            foreach (XmlNode SiblingNode in SiblingNodeList)
                            {
                                Sibling SiblingObj = new Sibling();
                                SiblingObj.m_PredecessorID = TagID;

                                BinderFieldCount = 0;
                                UniqueSortID = 0;
                                ErrorObj = GetUniqueId(ref UniqueSortID);
                                if (ErrorObj != null)
                                    return ErrorObj;
                                
                                SiblingObj.m_SortPartitionID = UniqueSortID;

                                SiblingObj.m_RelationshipName = SiblingNode.Attributes.GetNamedItem("NAME").InnerText;
                                SiblingObj.m_SortValue = SortValueCounter * 2;
                                SiblingList.Add(SiblingObj);

                                /* If the relationship node is for FILESYSTEM, the Sort (column) Value count needs to be incremented by number of binder fields
                                 * requested. This is to support fetching multiple binder fields.
                                 */
                                if (SiblingObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                                {
                                    ErrorObj = GetBinderFieldCount(HierarchyObj, out BinderFieldCount);
                                    if (ErrorObj != null)
                                        return ErrorObj;

                                    /* Note: Here 1 is added to the counter so that it will be used for fetching files when multiple binder and file fields are requested.
                                     * This is required to fetch all binder fields first and then file fields.
                                     * */
                                    SortValueCounter = SortValueCounter + BinderFieldCount + 1;
                                }
                                else
                                    SortValueCounter++;
                            }

                            // Code for SiblingList ends
                            SortPartitionObj.m_NodeID = TagID;
                            SortPartitionObj.m_ParentID = ParentID;
                            SortPartitionObj.SiblingList = SiblingList;
                            SortPartitionObj.m_SortPartitionColumnName = "SP" + ParentID;

                            // Add a predecessor Object for Next Relationship Node.
                            PredecessorObject = new Predecessor();
                            PredecessorObject.m_PredecessorID = TagID;
                            PredecessorObject.m_RelationshipName = HierarchyObj.m_RelationshipName;
                            PredecessorObject.m_StartColumnNumber = NSearchCommonObj.m_TotalNumberOfFields + 2;
                            PredecessorObject.m_NumberofFields = HierarchyObj.FieldCollectionList.Count;
                            PredecessorObject.m_RelatedObjectLevel = HierarchyObj.m_RelatedObjectLevel;
                            TempPredecessorList.Add(PredecessorObject);
                            SortPartitionObj.m_StartColumnNumber = NSearchCommonObj.m_TotalNumberOfFields + 2;
                            SortPartitionObj.PredecessorList = PredecessorListNodes;

                            // NSearchCommonObj looping is done for getting the correct SortPartitionID for a relationship Node.
                            SortPartitionObj.m_SortPartitionID = SortPartitionID;

                            NSearchCommonObj.HierarchyObjectList.Add(HierarchyObj);
                            NSearchCommonObj.SortPartitionList.Add(SortPartitionObj);

                            NSearchCommonObj.m_TotalNumberOfFields = NSearchCommonObj.m_TotalNumberOfFields + HierarchyObj.FieldCollectionList.Count + 1 + HierarchyObj.AggregateAttrList.Count;
                        }
                        else
                        {
                            ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Invalid Relationship specified" + HierarchyObj.m_RelationshipName,
                                                    OMErrorObjectType.GETDATAQUERYBUILDER, null, OMErrorType.ERROR, null, RelationshipNode.OuterXml);
                        }
                    }
                    else
                    {
                        ErrorObj = PrepareHierarchyObject(RelationshipNode, ref HierarchyObj, ParentID, TagID, ref IsFileSystemRequested);
                        SortPartitionObj.m_NodeID = TagID;
                        SortPartitionObj.m_ParentID = ParentID;
                        SortPartitionObj.SiblingList = SiblingList;
                        SortPartitionObj.m_SortPartitionColumnName = "SP" + ParentID;

                        HierarchyObj.m_ObjectTypeName = ParentObjectTypeName;
                        HierarchyObj.m_ObjectTypeID = ParentObjectTypeID;
                        HierarchyObj.m_ObjectType = ParentObjectType;

                        // Add a predecessor Object for Next Relationship Node.
                        PredecessorObject = new Predecessor();
                        PredecessorObject.m_PredecessorID = TagID;
                        PredecessorObject.m_RelationshipName = HierarchyObj.m_RelationshipName;
                        PredecessorObject.m_StartColumnNumber = NSearchCommonObj.m_TotalNumberOfFields + 2;
                        PredecessorObject.m_NumberofFields = HierarchyObj.FieldCollectionList.Count - 1;
                        PredecessorObject.m_RelatedObjectLevel = HierarchyObj.m_RelatedObjectLevel;
                        TempPredecessorList.Add(PredecessorObject);
                        SortPartitionObj.m_StartColumnNumber = NSearchCommonObj.m_TotalNumberOfFields + 2;

                        // Incremement the total number of fields depending on what is selected.
                        // Iterate through Custom fields of parent hierarchyobject and add it to total number of fields.
                        foreach (FieldCollection Field in ParentHierarchyObject.CustomFieldCollectionList)
                        {
                            // check for datatypes.
                            if (Field.m_FieldDataType.ToUpper() == "BINDER")
                                IsBinderRequested = true;
                            if (Field.m_FieldDataType.ToUpper() == "FILE")
                                IsFileRequested = true;
                        }
                        // both file and binder are requested.
                        if ((IsBinderRequested) && (IsFileRequested))
                            NSearchCommonObj.m_TotalNumberOfFields = NSearchCommonObj.m_TotalNumberOfFields + NSearchCommon.m_BinderandFileFieldsCount;

                        // only binder is requested.
                        if ((IsBinderRequested) && (!IsFileRequested))
                            NSearchCommonObj.m_TotalNumberOfFields = NSearchCommonObj.m_TotalNumberOfFields + NSearchCommon.m_OnlyBinderFields;

                        // only file is requested.
                        if ((!IsBinderRequested) && (IsFileRequested))
                            NSearchCommonObj.m_TotalNumberOfFields = NSearchCommonObj.m_TotalNumberOfFields + NSearchCommon.m_OnlyFileFields;

                        SortPartitionObj.PredecessorList = PredecessorListNodes;

                        //Code for SiblingList
                        // Iterate the SortPartitionObject List for getting SortPartitionID
                        foreach (SortPartition TempSortPartitionObj in NSearchCommonObj.SortPartitionList)
                        {
                            if (TempSortPartitionObj.m_NodeID == ParentID)
                            {
                                // iterate sibling list
                                foreach (Sibling TempSibling in TempSortPartitionObj.SiblingList)
                                {
                                    if (TempSibling.m_RelationshipName == HierarchyObj.m_RelationshipName)
                                    {
                                        SortPartitionID = TempSibling.m_SortPartitionID;
                                        break;
                                    }
                                }
                                break;
                            }
                        }
                        // NSearchCommonObj looping is done for getting the correct SortPartitionID for a relationship Node.
                        SortPartitionObj.m_SortPartitionID = SortPartitionID;

                        NSearchCommonObj.HierarchyObjectList.Add(HierarchyObj);
                        NSearchCommonObj.SortPartitionList.Add(SortPartitionObj);

                        /*
                        //In case of binders, add a further sort partition because of the binder-file relationship
                        SortPartition BinderSortPartition = new SortPartition();
                        BinderSortPartition.m_NodeID = TagID;
                        BinderSortPartition.m_ParentID = ParentID;
                        BinderSortPartition.SiblingList = SiblingList;
                        BinderSortPartition.m_SortPartitionColumnName = "SP" + ParentID;
                        */

                        //NSearchCommonObj.m_TotalNumberOfFields = NSearchCommonObj.m_TotalNumberOfFields + HierarchyObj.FieldCollectionList.Count + 1;
                    }
                }

                if (RelationshipNode.HasAttribute("OBJECTALIAS"))
                {
                    HierarchyObj.m_ObjectAliasName      = RelationshipNode.Attributes.GetNamedItem("OBJECTALIAS").InnerText;
                }
                else
                {
                    if (String.IsNullOrEmpty(HierarchyObj.m_ObjectAliasName))
                        HierarchyObj.m_ObjectAliasName  = ObjectTypeName;
                }

                // Get all the RelationshipNodes.
                RelationshipNodeList = RelationshipNode.SelectNodes("RELATIONSHIP");
                //Int32 ChildCounter = 1;
                foreach (XmlNode RelationNode in RelationshipNodeList)
                {
                    String SiblingRelationshipName = null;
                    ExistingRelationshipNode = (XmlElement)RelationNode;

                    // Give a recursive call.
                    if (HierarchyObj.m_RelationshipName != null)
                        //ErrorObj = ProcessRelationships(ExistingRelationshipNode, TagID, ref NSearchCommonObj.SortPartitionList, TempPredecessorList, ChildCounter);
                        ErrorObj = ProcessRelationships(ExistingRelationshipNode, TagID, ref NSearchCommonObj.SortPartitionList, TempPredecessorList);
                    else
                        //ErrorObj = ProcessRelationships(ExistingRelationshipNode, TagID, ref NSearchCommonObj.SortPartitionList, PredecessorListNodes, ChildCounter);
                        ErrorObj = ProcessRelationships(ExistingRelationshipNode, TagID, ref NSearchCommonObj.SortPartitionList, PredecessorListNodes);
                    SiblingRelationshipName = ExistingRelationshipNode.GetAttribute("NAME");
                    //if (!SiblingRelationshipName.Contains("AL$FILESYSTEM_AL$"))
                    //ChildCounter++;
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ProcessRelationships",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// NSearchCommonObj function builds a datastructure for Hierarchy Object
        /// </summary>
        /// <param name="ObjectTypeNode">XMLNode which contains SearchInfo</param>
        /// <param name="HierarchyObj">Datastructure Object which contains all the processed information from XML</param>
        /// <param name="ParentID">ParentID for identifying parent. NSearchCommonObj is required for forming XML structure</param>
        /// <returns>OMError object which contains Errors from Object Manager</returns>
        private OMError PrepareHierarchyObject(XmlElement ObjectTypeNode, ref HierarchyObject HierarchyObj, long ParentID, long TagID, ref Boolean IsFileSystemRequested)
        {
            // Process FilterList.
            // Process SortList.
            // Process SELECTLIST.
            // Process GROUPBYLIST.
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNode PrivilegeNode = null;
            XmlNode FilterListNode = null;
            XmlNode GroupListNode = null;
            XmlNode SelectListNode = null;
            XmlNode SortListNode = null;
            XmlNode EFTypeNode = null;
            XmlNode SearchAllFacetsNode = null;
            Boolean SearchAllFacetsFlag = false;

            try
            {

                if (ObjectTypeNode.HasAttribute("FileRequired"))
                {
                    if (ObjectTypeNode.Attributes.GetNamedItem("FileRequired").InnerText.ToUpper() == "YES")
                    {
                        HierarchyObj.m_IsFileRequired = true;
                        NSearchCommonObj.IsFileSystemRequired = true;
                    }
                    else
                    {
                        HierarchyObj.m_IsFileRequired = false;
                    }
                }

                //Parameters for paging
                if (ObjectTypeNode.HasAttribute(ConfigConstantAttributes.ATTRIB_RECORDS_PER_PAGE) &&
                    ObjectTypeNode.HasAttribute(ConfigConstantAttributes.ATTRIB_PAGE_NUMBER))
                {
                    try
                    {
                        //Paging can only be done at one place right now
                        if (NSearchCommonObj.m_Paging == true)
                        {
                            ErrorObj = new OMError(OMErrorCodes.OMI_GETDATA_FAILED,
                                                        "Only one set of paging parameters can be specified in a request",
                                                        OMErrorObjectType.OBJECTMANAGERCLASS, "Entity/Facet", OMErrorType.ERROR, null, "");
                            return ErrorObj;
                        }
                        else
                        {
                            NSearchCommonObj.m_Paging = true;
                        }

                        HierarchyObj.m_RecordsPerPage = Convert.ToInt32(ObjectTypeNode.Attributes.GetNamedItem(ConfigConstantAttributes.ATTRIB_RECORDS_PER_PAGE).InnerText);
                        HierarchyObj.m_PageNumber = Convert.ToInt32(ObjectTypeNode.Attributes.GetNamedItem(ConfigConstantAttributes.ATTRIB_PAGE_NUMBER).InnerText);
                        HierarchyObj.m_DoPaging = true;

                        //If paging, then the default sort is ascending and the default sort field is the primary key
                        HierarchyObj.m_PagingSortFieldOrder = "ASC";
                        HierarchyObj.m_PagingSortField = "";//Use the primary key
                    }
                    catch
                    {
                        HierarchyObj.m_DoPaging = false;
                    }
                }
                else
                {
                    HierarchyObj.m_DoPaging = false;
                }

                // Check whether privilege node is present. If present assign a Privilege level else assign default privlege as READ.
                PrivilegeNode = ObjectTypeNode.SelectSingleNode("PRIVILEGENAME");
                if (PrivilegeNode != null)
                    HierarchyObj.m_PrivilegeLevel = PrivilegeNode.InnerText;
                else
                    HierarchyObj.m_PrivilegeLevel = "READ";

                // Set member variables for HierarchyObject
                // Generate a unique number for NSearchCommonObj tag ID

                HierarchyObj.m_TagID = TagID;

                //HierarchyObj.m_TagID = 0;
                HierarchyObj.m_ParentID = ParentID;


                SearchAllFacetsNode = ObjectTypeNode.SelectSingleNode("SEARCHALLFACETS");
                if (SearchAllFacetsNode != null)
                {
                    SearchAllFacetsFlag = Convert.ToBoolean(SearchAllFacetsNode.InnerText.ToString());
                    if (SearchAllFacetsFlag)
                        HierarchyObj.m_IsSearchAllFacets = SearchAllFacetsFlag;

                }

                //// Get RelationshipName if present.
                //if (ObjectTypeNode.HasAttribute("NAME"))
                //{
                //    HierarchyObj.m_RelationshipName = ObjectTypeNode.Attributes.GetNamedItem("NAME").InnerText;
                //}

                //Process FilterList.
                FilterListNode = ObjectTypeNode.SelectSingleNode("FILTERLIST");
                if (FilterListNode != null)
                {
                    ErrorObj = PrepareFilterList(FilterListNode, ref HierarchyObj);
                    if (ErrorObj != null)
                        return ErrorObj;
                }

                // Process SelectList
                SelectListNode = ObjectTypeNode.SelectSingleNode("SELECTLIST");
                ErrorObj = PrepareSelectList(SelectListNode, ref HierarchyObj, ref IsFileSystemRequested, ref EFTypeNode);
                if (ErrorObj != null)
                    return ErrorObj;

                // Process Sortlist
                SortListNode = ObjectTypeNode.SelectSingleNode("SORTLIST");
                if (SortListNode != null)
                {
                    ErrorObj = PrepareSortList(SortListNode, ref HierarchyObj, ref EFTypeNode);
                    if (ErrorObj != null)
                        return ErrorObj;
                }

                // Process GroupByList
                GroupListNode = ObjectTypeNode.SelectSingleNode("GROUPBYLIST");
                if (GroupListNode != null)
                {
                    ErrorObj = PrepareGroupByList(GroupListNode, ref HierarchyObj);
                    if (ErrorObj != null)
                        return ErrorObj;
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareHierarchyObject",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        private OMError PrepareFilterList(XmlNode FilterListNode, ref HierarchyObject HierarchyObj)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNode FilterCombinationNode = null;
            XmlNode RelationshipNode = null;
            XmlNode IsCaseSensitiveNode = null;
            XmlNode FilterIDNode = null;
            XmlNode ExprValueNode = null;
            XmlNode ValueNode = null;
            XmlNode IsParent = null;
            bool IsFilterCombinationValid = false;
            String FilterCombinationString = null;
            XmlNodeList FilterNodeList = null;
            bool IsFilterCombinationAccepted = false;
            XmlNode ObjectTypeNode = null;
            XmlNode PrimaryKeyNode = null;
            string PrimaryKeyFieldName = null;
            bool IsPrimaryKeyPresent = false;
            List<int> AssignedFilterIds = new List<int>();

            try
            {
                // Process FilterCombination.
                FilterCombinationNode = FilterListNode.SelectSingleNode("FILTERCOMBINATION");
                if (FilterCombinationNode != null)
                {
                    FilterCombinationString = FilterCombinationNode.InnerText;
                    IsFilterCombinationValid = NSearchCommon.ValidateFilterCombination(FilterCombinationString);
                    if (!IsFilterCombinationValid)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                        "Filter Combination is Invalid", OMErrorObjectType.GETDATAQUERYBUILDER,
                        "NSearch", OMErrorType.ERROR, null, null);
                    }
                    HierarchyObj.m_FilterCombination = FilterCombinationString;
                    IsFilterCombinationAccepted = true;
                }

                ErrorObj = OMDataManager.DataProvider.ReadTypeDetails(HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeName, ref ObjectTypeNode);
                if (ErrorObj != null)
                    return ErrorObj;

                PrimaryKeyNode = ObjectTypeNode.SelectSingleNode("PRIMARYKEY");
                if (PrimaryKeyNode != null)
                {
                    PrimaryKeyFieldName = PrimaryKeyNode.InnerText.ToString();
                }

                int CountFilterNodes = 0;
                int CountFilterIdsPresent = 0;
                bool IsINClausePresent = false;
                Dictionary<string, List<int>> FieldNamesWithCounts = new Dictionary<string, List<int>>();
                bool IsDuplicateFieldNamePresent = false;
                string RelationAndFieldName = null;

                // Process Individual filter.
                FilterNodeList = FilterListNode.SelectNodes("FILTER");
                foreach (XmlNode FilterNode in FilterNodeList)
                {
                    FilterCollection FilterObj = new FilterCollection();
                    FilterObj.m_FieldName = FilterNode.SelectSingleNode("EXPRESSION").InnerText;

                    if (string.Compare(FilterObj.m_FieldName, PrimaryKeyFieldName, true) == 0)
                    {
                        IsPrimaryKeyPresent = true;
                    }

                    // Special processing for DateNode.
                    ExprValueNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE");
                    // Special Processing for Date Filter
                    ValueNode = ExprValueNode.SelectSingleNode("VALUE");
                    if (ValueNode != null)
                    {
                        FilterObj.m_XMLNode = ExprValueNode.ParentNode;
                    }
                    else
                    {
                        FilterObj.m_FieldValue = ExprValueNode.InnerText;
                    }
                    FilterObj.m_Operator = FilterNode.SelectSingleNode("OPERATOR").InnerText;

                    // check for relationship name if present.
                    RelationshipNode = FilterNode.SelectSingleNode("RELATIONSHIPNAME");
                    if (RelationshipNode != null)
                    {
                        FilterObj.m_RelationshipName = RelationshipNode.InnerText;
                        IsParent = RelationshipNode.Attributes.GetNamedItem("IsParent");
                        if (IsParent != null)
                            FilterObj.m_IsParent = IsParent.InnerText;
                    }

                    if (string.IsNullOrEmpty(FilterObj.m_RelationshipName))
                    {
                        if (FieldNamesWithCounts.ContainsKey(FilterObj.m_FieldName))
                        {
                            FieldNamesWithCounts[FilterObj.m_FieldName].Add(CountFilterNodes);
                        }
                        else
                        {
                            List<int> TheList = new List<int>();
                            TheList.Add(CountFilterNodes);
                            FieldNamesWithCounts.Add(FilterObj.m_FieldName, TheList);
                        }
                    }
                    else
                    {
                        //To add uniqueness in case two object types have same field name
                        RelationAndFieldName = FilterObj.m_FieldName +"$AL$" +FilterObj.m_RelationshipName;

                        if (FieldNamesWithCounts.ContainsKey(RelationAndFieldName))
                        {
                            FieldNamesWithCounts[RelationAndFieldName].Add(CountFilterNodes);
                        }
                        else
                        {
                            List<int> TheList = new List<int>();
                            TheList.Add(CountFilterNodes);
                            FieldNamesWithCounts.Add(RelationAndFieldName, TheList);
                        }
                    }

                    // check for IsCaseSensitive node.
                    IsCaseSensitiveNode = FilterNode.SelectSingleNode("ISCASESENSITIVE");
                    if (IsCaseSensitiveNode != null)
                        FilterObj.m_IsCaseSensitive = Convert.ToBoolean(IsCaseSensitiveNode.InnerText);

                    // check for FilterID for filter combination
                    FilterIDNode = FilterNode.SelectSingleNode("FILTERID");
                    if (FilterIDNode != null)
                    {
                        FilterObj.m_FilterID = Convert.ToInt32(FilterIDNode.InnerText);
                        AssignedFilterIds.Add(FilterObj.m_FilterID);

                        CountFilterIdsPresent++;
                    }
                    else
                    {
                    }

                    if (FilterObj.m_Operator == "IN")
                    {
                        IsINClausePresent = true;
                    }

                    HierarchyObj.FilterCollectionList.Add(FilterObj);
                    
                    CountFilterNodes++;
                }

                //Need to do this only if the filter combination is not present, and there is an IN query only for TE
                if (IsPrimaryKeyPresent && !IsFilterCombinationAccepted && IsINClausePresent && CountFilterNodes > 1 && HierarchyObj.m_ParentID == 0)
                {
                    foreach (KeyValuePair<string, List<int>> FieldNameWithCount in FieldNamesWithCounts)
                    {
                        if (FieldNameWithCount.Value.Count > 1)
                        {
                            IsDuplicateFieldNamePresent = true;
                            break;
                        }
                    }

                    if (CountFilterIdsPresent == CountFilterNodes)
                    {
                        //All are present
                    }
                    else
                    {
                        //Not all are present
                        if (CountFilterIdsPresent == 0)
                        {
                            //If none are present
                            //add all of them
                            for (int Counter = 0; Counter < CountFilterNodes; Counter++)
                            {
                                HierarchyObj.FilterCollectionList[Counter].m_FilterID = Counter + 1;
                            }
                        }
                        else
                        {
                            //If some are present
                            //add the ones that are not present
                            int CurrentFilterId = 1;
                            for (int Counter = 0; Counter < CountFilterNodes; Counter++)
                            {
                                if (HierarchyObj.FilterCollectionList[Counter].m_FilterID == 0)
                                {
                                    Random r = new Random();
                                    while (AssignedFilterIds.Contains(CurrentFilterId))
                                    {
                                        CurrentFilterId = r.Next(200);
                                    }

                                    HierarchyObj.FilterCollectionList[Counter].m_FilterID = CurrentFilterId;
                                    AssignedFilterIds.Add(CurrentFilterId);
                                }
                                else
                                {
                                    CurrentFilterId = HierarchyObj.FilterCollectionList[Counter].m_FilterID;
                                }
                                CurrentFilterId++;
                            }
                        }
                    }

                    //If FilterCombination is not present, then create it
                    StringBuilder sbFilterCombinationString = new StringBuilder();
                    if (!IsDuplicateFieldNamePresent)
                    {
                        sbFilterCombinationString.Append("(");
                        for (int Counter = 0; Counter < CountFilterNodes; Counter++)
                        {
                            sbFilterCombinationString.Append("(");
                            sbFilterCombinationString.Append(HierarchyObj.FilterCollectionList[Counter].m_FilterID);
                            sbFilterCombinationString.Append(") AND ");
                        }

                        sbFilterCombinationString.Remove(sbFilterCombinationString.Length - " AND ".Length, " AND ".Length);
                        sbFilterCombinationString.Append(")");
                    }
                    else
                    {
                        sbFilterCombinationString.Append("(");
                        //int Counter = 0;
                        foreach (KeyValuePair<string, List<int>> FieldNameWithCount in FieldNamesWithCounts)
                        {
                            if (FieldNameWithCount.Value.Count > 1)
                            {
                                //This is a duplicate
                                sbFilterCombinationString.Append("(");

                                foreach (int PositionElement in FieldNameWithCount.Value)
                                {
                                    sbFilterCombinationString.Append("(");
                                    sbFilterCombinationString.Append(
                                        HierarchyObj.FilterCollectionList[PositionElement].m_FilterID);

                                    sbFilterCombinationString.Append(") OR ");
                                    HierarchyObj.FilterCollectionList[PositionElement].m_IsProcessedForFilterCombination = true;
                                }
                                sbFilterCombinationString.Remove(sbFilterCombinationString.Length - " OR ".Length, " OR ".Length);
                                
                                sbFilterCombinationString.Append(") AND ");
                            }
                            else if (!HierarchyObj.FilterCollectionList[FieldNameWithCount.Value[0]].m_IsProcessedForFilterCombination)
                            {
                                sbFilterCombinationString.Append("(");
                                sbFilterCombinationString.Append(HierarchyObj.FilterCollectionList[FieldNameWithCount.Value[0]].m_FilterID);
                                sbFilterCombinationString.Append(") AND ");

                                HierarchyObj.FilterCollectionList[FieldNameWithCount.Value[0]].m_IsProcessedForFilterCombination = true;
                            }
                            //Counter++;
                        }
                        sbFilterCombinationString.Remove(sbFilterCombinationString.Length - " AND ".Length, " AND ".Length);
                        sbFilterCombinationString.Append(")");
                    }

                    //Check whether the filter combination string is valid
                    HierarchyObj.m_FilterCombination = sbFilterCombinationString.ToString();
                    HierarchyObj.m_IsFilterCombinationAdded = true;
                }

                return ErrorObj;
            }

            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareFilterList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }
        
        /// <summary>
        /// NSearchCommonObj function prepares a sort list based on inputxml and populates a datastructure.
        /// </summary>
        /// <param name="SortListNode">XML Node which contains information about Sorting Expressions</param>
        /// <param name="HierarchyObj"></param>
        /// <param name="EFTypeNode"></param>
        /// <returns>OMError Object</returns>
        private OMError PrepareSortList(XmlNode SortListNode, ref HierarchyObject HierarchyObj, ref XmlNode EFTypeNode)
        {
            OMError             ReturnErrorObj              = null;
            OMError             ErrorObj                    = null;
            XmlNodeList         SortExpressionList          = null;
            XmlNode             RelationshipNode            = null;
            XmlNode             SortOrderNode               = null;
            XmlNode             SortSequence                = null;
            XmlNode             FieldNode                   = null;
            int                 SortSequenceCounter         = 0;
            Boolean             IsSortColumninSelectList    = false;
            bool                IsCurrencyField             = false;    //Used to identify whether field data type is CURRENCY

            try
            {
                SortExpressionList = SortListNode.SelectNodes("SORTEXPRESSION");
                foreach (XmlNode SortExpression in SortExpressionList)
                {
                    IsSortColumninSelectList = false;
                    IsCurrencyField = false;
                    SortSequenceCounter++;
                    SortSpecification SortSpecificationObj = new SortSpecification();
                    SortSpecificationObj.m_SortFieldName = SortExpression.SelectSingleNode("COLUMNNAME").InnerText;

                    // check for SortOrder. if no sortorder is specified then add by default sort order as ascending.
                    SortOrderNode = SortExpression.SelectSingleNode("SORTORDER");
                    if (SortOrderNode == null)
                        SortSpecificationObj.m_SortOrder = "DESC";
                    else
                        SortSpecificationObj.m_SortOrder = SortOrderNode.InnerText;

                    // Check whether that field is there in select list
                    foreach (FieldCollection SelectedField in HierarchyObj.FieldCollectionList)
                    {
                        if (SortSpecificationObj.m_SortFieldName == SelectedField.m_FieldName)
                        {
                            //Changes for : Support for sorting on currency field
                            //Since CURRENCY field is displayed as an XML in search output, sorting cannot be done on that.
                            //So an additional hidden column would be added for sorting which would hold currency amount.
                            //Set sort requested = false for currency field. 
                            if (SelectedField.m_FieldDataType.ToUpper().CompareTo("CURRENCY") == 0)
                            {
                                IsCurrencyField = true;
                                IsSortColumninSelectList = true;
                                SelectedField.m_IsSortRequested = false;
                            }
                            else
                            {
                                IsSortColumninSelectList = true;
                                if (!SelectedField.m_IsSortRequested)
                                {
                                    SelectedField.m_IsSortRequested = true;
                                    SelectedField.m_SortType = SortSpecificationObj.m_SortOrder;
                                }
                            }
                            break;
                        }
                    }

                    //Changes for : Support for sorting on currency field
                    //Add column to field collection list if it is present in sort list and not present in select list.
                    //Special case for currency field : Add a hidden column which would hold currency amount and use it for sorting.
                    //Actual currency field would not be used for sorting.
                    if (!IsSortColumninSelectList || (IsSortColumninSelectList && IsCurrencyField))
                    {
                        FieldNode = EFTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDNAME/text()='" + SortSpecificationObj.m_SortFieldName + "']");
                        if (FieldNode != null)
                        {
                            if ((FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper() != "XML") && 
                                (FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper() != "FILE") && 
                                (FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper() != "BINDER"))
                            {
                                // Add a column in FieldCollectionList of Hierarchy Object.
                                FieldCollection SortField = new FieldCollection();
                                SortField.m_FieldName = SortSpecificationObj.m_SortFieldName;

                                if (FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper() == "CURRENCY")
                                {
                                    SortSpecificationObj.m_SortFieldName = SortField.m_FieldName = SortSpecificationObj.m_SortFieldName + "_A";
                                    SortField.m_FieldDataType = OMInternalDataTypeForCurrency.CURRENY_AMOUNT;
                                }
                                else
                                {
                                    SortField.m_FieldName = SortSpecificationObj.m_SortFieldName;
                                    SortField.m_FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText;
                                }

                                if (SortField.m_FieldDataType.ToUpper() == "STRING")
                                    SortField.m_FieldSize = Convert.ToInt32(FieldNode.SelectSingleNode("FIELDSIZE").InnerText); 
                                SortField.m_SortType = SortSpecificationObj.m_SortOrder;
                                SortField.m_IsHidden = true;
                                SortField.m_IsSortRequested = true;
                                SortField.m_Alias = SortSpecificationObj.m_SortFieldName;
                                HierarchyObj.FieldCollectionList.Add(SortField);
                            }
                        }
                    }

                    // check for relationshipname node.
                    RelationshipNode = SortExpression.SelectSingleNode("RELATIONSHIPNAME");
                    if (RelationshipNode != null)
                        SortSpecificationObj.m_RelationshipName = RelationshipNode.InnerText;

                    // check for SortSequence
                    // If no sortseqence is given then default sort sequence will be applied.
                    SortSequence = SortExpression.SelectSingleNode("SORTSEQUENCE");
                    if (SortSequence == null)
                        SortSpecificationObj.m_SortSequence = SortSequenceCounter;
                    else
                        SortSpecificationObj.m_SortSequence = Convert.ToInt32(SortSequence.InnerText);

                    HierarchyObj.SortSpecificationList.Add(SortSpecificationObj);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareSortList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// NSearchCommonObj function prepares fieldlist and aggregate function list.
        /// </summary>
        /// <param name="SelectListNode">SelectList node which contains information for fields to be selected and aggregate functions which are to be applied</param>
        /// <param name="HierarchyObj">HierarchyObject which contains processed information for individual node</param>
        /// <returns></returns>
        private OMError PrepareSelectList(XmlNode SelectListNode, ref HierarchyObject HierarchyObj, ref Boolean IsFileSystemRequested, ref XmlNode EFTypeNode)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            SearchUtils SearchUtilsObj = null;
            XmlNode ObjectTypeNode = null;

            try
            {
                //Processing 
                //1. Check if any selectlist is given.
                //2. If no select list is given then get all fields of that Entitytype/FacetType. 
                //3. Do the processing of Aggregate Functions.

                // Get fields for Entity/Facet type.

                if ((HierarchyObj.m_RelationshipName != null) && (!HierarchyObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$")))
                {
                    SearchUtilsObj = new SearchUtils(OMDataManager);
                    if (HierarchyObj.m_ObjectTypeName != null)
                    {
                        if ((ErrorObj = SearchUtilsObj.ReadObjectType(HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeName, ref ObjectTypeNode)) != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                    ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType, null, ErrorObj.ErrorXML);
                            return ReturnErrorObj;
                        }
                    }
                    else
                    {
                        if ((ErrorObj = SearchUtilsObj.ReadObjectTypeByID(HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeID, ref ObjectTypeNode)) != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                    ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType, null, ErrorObj.ErrorXML);
                            return ReturnErrorObj;
                        }
                    }

                    EFTypeNode = ObjectTypeNode;

                    // set tablename and primary key for Entity/facet type.
                    if (ObjectTypeNode != null)
                    {
                        HierarchyObj.m_ObjectTypeID = Convert.ToUInt64(ObjectTypeNode.SelectSingleNode("TYPEID").InnerText);
                        HierarchyObj.m_TableName = ObjectTypeNode.SelectSingleNode("TABLENAME").InnerText;
                        HierarchyObj.m_PrimaryKeyName = ObjectTypeNode.SelectSingleNode("PRIMARYKEY").InnerText;
                    }

                    // prepare a list of all fields and add it to hierarchy object.
                    if ((ErrorObj = PrepareFieldList(ObjectTypeNode, ref HierarchyObj, SelectListNode, ref IsFileSystemRequested)) != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType, null, ErrorObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    // prepare aggergate function list
                    if ((ErrorObj = PrepareAggregateFunctionList(SelectListNode, ref HierarchyObj)) != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType, null, ErrorObj.ErrorXML);
                        return ReturnErrorObj;
                    }
                }
                else
                {
                    if (HierarchyObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                    {
                        if ((ErrorObj = PrepareFieldList(null, ref HierarchyObj, null, ref IsFileSystemRequested)) != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                    ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType, null, ErrorObj.ErrorXML);
                            return ReturnErrorObj;
                        }
                    }
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareSelectList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// NSearchCommonObj function iterates through ObjectTypeNode and prepares a list of fields from objecttypenode(NSearchCommonObj node has Entity/facet type details).
        /// If SelectListNode is null then all the fields are selected for that Entity or facet type.
        /// If SelectListNode has list of fields to select then only those fields are added.
        /// If any custom datatypes are requested then it adds field CustomFieldList.
        /// </summary>
        /// <param name="ObjectTypeNode">Entity/Facet type details</param>
        /// <param name="HierarchyObj">hierarchy object at that level.</param>
        /// <param name="SelectListNode">XMLNode which contains select list node. if selectlistnode is null then inputxml doesnt contain selectlist</param>
        /// <returns>OMERRROR if any error is encountered</returns>
        private OMError PrepareFieldList(XmlNode ObjectTypeNode, ref HierarchyObject HierarchyObj, XmlNode SelectListNode, ref Boolean IsFileSystemRequested)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNodeList FieldList = null;
            XmlNodeList SelectFieldList = null;
            String FieldName = null;
            XmlNode FieldAliasNode = null;
            XmlNode PrimaryFieldNode = null;
            XmlNode IsFileRequired = null;
            XmlNode TransferMode = null;
            String FieldAliasName = null;
            Boolean IsPrimaryFieldAdded = false;
            String FieldDataType = null;
            HierarchyObject ParentHierarchyObject = null;

            try
            {
                if (!HierarchyObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                {
                    if (HierarchyObj.m_IsSearchAllFacets || HierarchyObj.m_IsSearch_In_All_Facets)
                    {
                        HierarchyObj.m_ObjectTypeID = Convert.ToUInt64(ObjectTypeNode.SelectSingleNode("TYPEID").InnerText);
                        ErrorObj = GetAssociatedFacetTypes(HierarchyObj);
                    }

                    if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                    {
                        HierarchyObj.SimpleSearchQueryBuilder = new StringBuilder();
                        ErrorObj = PrepareSimpleSearchQuery(HierarchyObj, ref ObjectTypeNode, ref HierarchyObj.SimpleSearchQueryBuilder);
                        if (ErrorObj != null)
                             return ErrorObj;

                         if (HierarchyObj.m_IsSearchInDocuments)
                         {
                             //Call the Full Text Search Query preparing function
                             ErrorObj = PrepareDocumentSearchQuery(HierarchyObj, ref ObjectTypeNode, ref HierarchyObj.DocumentSearchQueryBuilder);
                             if (ErrorObj != null)
                                 return ErrorObj;
                         }
                    }

                    // NSearchCommonObj implies no selectlist is given. 
                    FieldList = ObjectTypeNode.SelectNodes("FIELDCOLLECTION/FIELD");
                    if (SelectListNode == null)
                    {
                        foreach (XmlNode FieldNode in FieldList)
                        {
                            FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper();
                            // Check whether filesystem is requested or not.
                            if ((FieldDataType == "BINDER") || (FieldDataType == "FILE"))
                            {
                                IsFileSystemRequested = true;
                            }

                            if ((FieldDataType.ToUpper() != "BINDER") && (FieldDataType.ToUpper() != "FILE") && (FieldDataType.ToUpper() != "XML"))
                            {
                                FieldCollection FieldObj = new FieldCollection();
                                FieldObj.m_SchemaID      = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                FieldObj.m_FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                FieldObj.m_FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText;
                                if( FieldObj.m_FieldDataType.ToUpper() == "STRING")
                                    FieldObj.m_FieldSize = Convert.ToInt32(FieldNode.SelectSingleNode("FIELDSIZE").InnerText); 
                                FieldObj.m_Alias = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                FieldObj.m_IsHidden = false;
                                if ((FieldNode.SelectSingleNode("ISPRIMARYKEY").InnerText.ToUpper()) == "TRUE")
                                {
                                    FieldObj.m_IsSortRequested = true;
                                    FieldObj.m_SortType = "ASC";
                                    FieldObj.m_IsPrimaryKey = true;
                                    HierarchyObj.FieldCollectionList.Insert(0, FieldObj);
                                }
                                else
                                {
                                    HierarchyObj.FieldCollectionList.Add(FieldObj);
                                }
                                //HierarchyObj.FieldCollectionList.Add(FieldObj);
                            }
                            else
                            {
                                // Add Fields to custom field collection list
                                if (!NSearchCommonObj.IsFileBinderFieldsFetched)
                                {
                                    ErrorObj = GetBinderandFileFields();
                                    NSearchCommonObj.IsFileBinderFieldsFetched = true;
                                }
                                FieldCollection FieldObj = new FieldCollection();
                                FieldObj.m_SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                FieldObj.m_FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                FieldObj.m_FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText;
                                FieldObj.m_Alias = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                if (FieldObj.m_FieldDataType.ToUpper() == "STRING")
                                    FieldObj.m_FieldSize = Convert.ToInt32(FieldNode.SelectSingleNode("FIELDSIZE").InnerText); 

                                if ((FieldNode.SelectSingleNode("ISPRIMARYKEY").InnerText.ToUpper()) == "TRUE")
                                {
                                    FieldObj.m_IsSortRequested = true;
                                    FieldObj.m_SortType = "ASC";
                                    FieldObj.m_IsPrimaryKey = true;
                                    HierarchyObj.CustomFieldCollectionList.Insert(0, FieldObj);
                                }
                                else
                                {
                                    FieldObj.m_IsFileRequired = "No";
                                    if (HierarchyObj.m_IsFileRequired == true)
                                    {
                                        FieldObj.m_IsFileRequired = "Yes";
                                    }
                                    FieldObj.m_TransferMode = "Synchronous";

                                    FieldObj.m_IsHidden = false;
                                    HierarchyObj.CustomFieldCollectionList.Add(FieldObj);
                                }
                            }
                        }
                    }
                    // NSearchCommonObj implies selectlist is given.
                    else
                    {
                        SelectFieldList = SelectListNode.SelectNodes("EXPRESSION");
                        foreach (XmlNode FieldExpression in SelectFieldList)
                        {
                            FieldName = FieldExpression.SelectSingleNode("FIELDNAME").InnerText;
                            FieldAliasNode = FieldExpression.SelectSingleNode("ALIAS");
                            if (FieldAliasNode != null)
                                FieldAliasName = FieldAliasNode.InnerText;
                            foreach (XmlNode FieldNode in FieldList)
                            {
                                if (FieldNode.SelectSingleNode("FIELDNAME").InnerText == FieldName)
                                {
                                    FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText.ToUpper();

                                    // Check whether filesystem is requested or not.
                                    if ((FieldDataType == "BINDER") || (FieldDataType == "FILE"))
                                    {
                                        IsFileSystemRequested = true;
                                    }

                                    if ((FieldDataType.ToUpper() != "BINDER") && (FieldDataType.ToUpper() != "FILE") && (FieldDataType.ToUpper() != "XML"))
                                    {
                                        FieldCollection FieldObj = new FieldCollection();
                                        FieldObj.m_SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                        FieldObj.m_FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                        FieldObj.m_FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText;
                                        if (FieldObj.m_FieldDataType.ToUpper() == "STRING")
                                            FieldObj.m_FieldSize = Convert.ToInt32(FieldNode.SelectSingleNode("FIELDSIZE").InnerText); 
                                        if (FieldAliasName == null)
                                            FieldObj.m_Alias = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                        else
                                            FieldObj.m_Alias = FieldAliasName;

                                        if ((FieldNode.SelectSingleNode("ISPRIMARYKEY").InnerText.ToUpper()) == "TRUE")
                                        {
                                            IsPrimaryFieldAdded = true;
                                            FieldObj.m_IsSortRequested = true;
                                            FieldObj.m_SortType = "ASC";
                                            FieldObj.m_IsPrimaryKey = true;
                                            // NSearchCommonObj ensures that Primary key is selected at first index.
                                            HierarchyObj.FieldCollectionList.Insert(0, FieldObj);
                                        }
                                        else
                                        {
                                            HierarchyObj.FieldCollectionList.Add(FieldObj);
                                        }
                                    }
                                    else
                                    {
                                        // Add Fields to custom field collection list
                                        if (!NSearchCommonObj.IsFileBinderFieldsFetched)
                                        {
                                            ErrorObj = GetBinderandFileFields();
                                            NSearchCommonObj.IsFileBinderFieldsFetched = true;
                                        }
                                        FieldCollection FieldObj = new FieldCollection();
                                        FieldObj.m_SchemaID = FieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                        FieldObj.m_FieldName = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                        FieldObj.m_FieldDataType = FieldNode.SelectSingleNode("FIELDTYPE").InnerText;

                                        if ((FieldNode.SelectSingleNode("ISPRIMARYKEY").InnerText.ToUpper()) == "TRUE")
                                        {
                                            FieldObj.m_IsSortRequested = true;
                                            FieldObj.m_SortType = "ASC";
                                            FieldObj.m_IsPrimaryKey = true;
                                            IsPrimaryFieldAdded = true;
                                        }
                                        if (FieldDataType.ToUpper() == "FILE" || FieldDataType.ToUpper() == "BINDER")
                                        {
                                            IsFileRequired = FieldExpression.SelectSingleNode("FIELDNAME").Attributes.GetNamedItem("FileRequired");
                                            if (IsFileRequired != null)
                                            {
                                                if (IsFileRequired.InnerText.ToUpper() == "YES")
                                                    NSearchCommonObj.IsFileSystemRequired = true;
                                                FieldObj.m_IsFileRequired = IsFileRequired.InnerText;
                                                TransferMode = FieldExpression.SelectSingleNode("FIELDNAME").Attributes.GetNamedItem("TransferMode");
                                                if (TransferMode != null)
                                                    FieldObj.m_TransferMode = TransferMode.InnerText;
                                                else
                                                    FieldObj.m_TransferMode = "Synchronous";
                                            }
                                            else
                                            {
                                                FieldObj.m_IsFileRequired = "No";
                                                FieldObj.m_TransferMode = "Synchronous";
                                            }
                                        }
                                        FieldObj.m_Alias = FieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                        FieldObj.m_IsHidden = false;
                                        HierarchyObj.CustomFieldCollectionList.Add(FieldObj);
                                    }
                                }
                            }
                        }
                        // NSearchCommonObj code adds Primary key if not given in Select List for Entity/Facet
                        if (!IsPrimaryFieldAdded)
                        {
                            PrimaryFieldNode = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[ISPRIMARYKEY/text()='TRUE']");
                            if (PrimaryFieldNode != null)
                            {
                                FieldCollection FieldObj = new FieldCollection();
                                FieldObj.m_SchemaID     = PrimaryFieldNode.SelectSingleNode("SCHEMAID").InnerText;
                                FieldObj.m_FieldName    = PrimaryFieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                FieldObj.m_FieldDataType = PrimaryFieldNode.SelectSingleNode("FIELDTYPE").InnerText;
                                FieldObj.m_IsPrimaryKey = true;
                                FieldObj.m_IsSortRequested = true;
                                FieldObj.m_SortType = "ASC";

                                FieldObj.m_Alias = PrimaryFieldNode.SelectSingleNode("FIELDNAME").InnerText;
                                FieldObj.m_IsHidden = false;
                                // NSearchCommonObj ensures that Primary key is selected at first index.
                                HierarchyObj.FieldCollectionList.Insert(0, FieldObj);
                            }
                        }
                    }
                }
                else
                {
                    // NSearchCommonObj implies that relationship node is added as a result of entity type having custom Datatype.
                    // Copy the custom field list of its parent entity to NSearchCommonObj hierarchy object.
                    // get Parent hierarchy object from NSearchCommonObj hierarchyObject.
                    foreach (HierarchyObject TempHierarchyObj in NSearchCommonObj.HierarchyObjectList)
                    {
                        if (TempHierarchyObj.m_TagID == HierarchyObj.m_ParentID)
                        {
                            ParentHierarchyObject = TempHierarchyObj;
                            break;
                        }
                    }

                    if (ParentHierarchyObject != null)
                    {
                        if (ParentHierarchyObject.CustomFieldCollectionList.Count > 0)
                            HierarchyObj.CustomFieldCollectionList = ParentHierarchyObject.CustomFieldCollectionList;

                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareFieldList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// NSearchCommonObj function prepares AggregateFunction List
        /// </summary>
        /// <param name="SelectListNode">XMLNode which contains select list node. if selectlistnode is null then inputxml doesnt contain selectlist and aggregate function list</param>
        /// <param name="HierarchyObj">Hierarchy Object</param>
        /// <returns>OMError Object</returns>
        private OMError PrepareAggregateFunctionList(XmlNode SelectListNode, ref HierarchyObject HierarchyObj)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNodeList AggregateFunctionList = null;
            XmlNode AliasNode = null;
            XmlNode DisinctNode = null;
            XmlNode RelationshipNameNode = null;
            try
            {
                if (SelectListNode != null)
                {
                    AggregateFunctionList = SelectListNode.SelectNodes("AGGREGATEFUNCTION");
                    if (AggregateFunctionList != null)
                    {
                        foreach (XmlNode AggregateFunctionNode in AggregateFunctionList)
                        {
                            AggregateAttributes AggregateAttribObj = new AggregateAttributes();
                            AggregateAttribObj.m_FunctionName = AggregateFunctionNode.SelectSingleNode("FUNCTIONNAME").InnerText;
                            AggregateAttribObj.m_FieldName = AggregateFunctionNode.SelectSingleNode("EXPRESSION").InnerText;

                            // check for alias name
                            AliasNode = AggregateFunctionNode.SelectSingleNode("ALIAS");
                            if (AliasNode == null)
                                AggregateAttribObj.m_Alias = AggregateFunctionNode.SelectSingleNode("EXPRESSION").InnerText;
                            else
                                AggregateAttribObj.m_Alias = AliasNode.InnerText;

                            // check for DISTINCT flag.
                            // If distinct flag is not given the by default make it TRUE.
                            DisinctNode = AggregateFunctionNode.SelectSingleNode("DISTINCT");
                            if (DisinctNode != null)
                                AggregateAttribObj.m_IsDistinct = Convert.ToBoolean(DisinctNode.InnerText);

                            // check for relationshipname.

                            RelationshipNameNode = AggregateFunctionNode.SelectSingleNode("REALTIONSHIPNAME");
                            if (RelationshipNameNode != null)
                                AggregateAttribObj.m_RelationshipName = RelationshipNameNode.InnerText;

                            HierarchyObj.AggregateAttrList.Add(AggregateAttribObj);
                        }
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareAggregateFunctionList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }

        }

        /// <summary>
        /// NSearchCommonObj function fills in all the information required for Grouping.
        /// </summary>
        /// <param name="GroupListNode">XML node which contains groupby expressions</param>
        /// <param name="HierarchyObj">HIerarchy Object</param>
        /// <returns>OMError object</returns>
        private OMError PrepareGroupByList(XmlNode GroupListNode, ref HierarchyObject HierarchyObj)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNodeList ExpressionListNode = null;

            try
            {
                // Process Groupbylist.
                ExpressionListNode = GroupListNode.SelectNodes("EXPRESSIONLIST/EXPRESSION");
                foreach (XmlNode ExpressionNode in ExpressionListNode)
                {
                    GroupByExpression Expression = new GroupByExpression();
                    Expression.Expression = ExpressionNode.InnerText;
                    HierarchyObj.GroupByExpressionList.Add(Expression);
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareGroupByList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// NSearchCommonObj function adds Associated Facets to a Relationship node.
        /// </summary>
        /// <param name="Hierarchyobj">Hierarchy ObjectNode which has all the information for Target Entity</param>
        /// <param name="RelationshipNode">All relationships are appended to NSearchCommonObj node</param>
        /// <param name="NewRelationshipNode">RelationshipNode appended to target entity</param>
        /// <returns></returns>
        private OMError GetAssociatedFacetTypes(HierarchyObject Hierarchyobj, XmlNode RelationshipNode, ref XmlNode NewRelationshipNode)
        {
            OMError ReturnErrorObj = null;
            XmlElement NewRelationNode = null;
            XmlAttribute RelName = null;
            XmlAttribute AttributeFileRequired = null;
            XmlElement SearchAllFacetNode = null;
            int SearchFacetLevel = 1;
            String FacetLevel = null;
            try
            {
                if (Hierarchyobj.m_RelatedFacetTypes != null)
                {
                    NewRelationshipNode = RelationshipNode;
                    FacetLevel = ((XmlElement)NewRelationshipNode).GetAttribute("SEARCHEDFACETLEVEL");
                    if (!String.IsNullOrEmpty(FacetLevel))                    
                        SearchFacetLevel = Int32.Parse(FacetLevel) + 1;

                    foreach (DictionaryEntry RelationEntry in Hierarchyobj.m_RelatedFacetTypes)
                    {
                        NewRelationNode = NSearchCommonObj.InputXMLDocument.CreateElement("RELATIONSHIP");
                        RelName = NSearchCommonObj.InputXMLDocument.CreateAttribute("NAME");
                        RelName.InnerText = RelationEntry.Key.ToString();
                        NewRelationNode.Attributes.Append(RelName);

                        //If the parent has FileRequired, then make FileRequired for these facets also
                        if (Hierarchyobj.m_IsFileRequired == true && Hierarchyobj.m_IsSearchAllFacets)
                        {
                            AttributeFileRequired = NSearchCommonObj.InputXMLDocument.CreateAttribute("FileRequired");
                            AttributeFileRequired.InnerText = "YES";
                            NewRelationNode.Attributes.Append(AttributeFileRequired);
                        }
                        //For SearchAllFacet, we need to get N level facets. 
                        //For this while adding facets it also add SearchAllFacet attribute for it. Due to this N level facets will get retrieved.                        
                        NewRelationNode.SetAttribute("SEARCHEDFACETLEVEL", SearchFacetLevel.ToString());
                        if (SearchFacetLevel < 2)
                        {
                            if (Hierarchyobj.m_IsSearchAllFacets)
                            {
                                SearchAllFacetNode = NSearchCommonObj.InputXMLDocument.CreateElement("SEARCHALLFACETS");
                                SearchAllFacetNode.InnerText = "TRUE";
                                NewRelationNode.AppendChild(SearchAllFacetNode);
                            }
                        }
                        NewRelationshipNode.AppendChild(NewRelationNode);
                    }
                }
                else
                {
                    NewRelationshipNode = RelationshipNode;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetAssociatedFacetTypes",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }


        }

        private OMError GetAssociatedFacetTypes(HierarchyObject Hierarchyobj)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            RelationshipType EFAssoObj = null;
            String RelatedFacetTypeXML = null;
            String FacetTypeID = null;
            XPathDocument XPathDoc = null;
            XPathNavigator XPathNav = null;
            XPathNodeIterator XPathNodeItr = null;
            String RelationshipName = null;

            try
            {
                EFAssoObj = new RelationshipType(OMDataManager);
                
                //Previously it was getting only Facets related to Entity. Now it is enhanced to Facet to Facet also.
                if (String.IsNullOrEmpty(Hierarchyobj.m_ObjectType))
	                ErrorObj = EFAssoObj.ReadAllByContainerType(Hierarchyobj.m_ObjectTypeID, "ENTITY", "FACET", ref RelatedFacetTypeXML);
                else
                    ErrorObj = EFAssoObj.ReadAllByContainerType(Hierarchyobj.m_ObjectTypeID, Hierarchyobj.m_ObjectType.ToUpper(), "FACET", ref RelatedFacetTypeXML);

                if (ErrorObj != null)
                    return ErrorObj;

                if (RelatedFacetTypeXML != null && RelatedFacetTypeXML != "")
                {
                    Hierarchyobj.m_RelatedFacetTypes = new SortedList();
                    XPathDoc = new XPathDocument(new StringReader(RelatedFacetTypeXML));
                    XPathNav = XPathDoc.CreateNavigator();
                    XPathNodeItr = XPathNav.Select("RELATIONSHIPTYPECOLLECTION/RELATIONSHIPTYPE");
                    while (XPathNodeItr.MoveNext())
                    {
                        RelationshipName = XPathNodeItr.Current.SelectSingleNode("RELATIONSHIPNAME").Value;
                        FacetTypeID = XPathNodeItr.Current.SelectSingleNode("CONTAINSTYPEID").Value;
                        Hierarchyobj.m_RelatedFacetTypes.Add(RelationshipName, FacetTypeID);
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetAssociatedFacetTypes",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }


        }
        
        #endregion

        private OMError ExecuteDynamicSQL(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String OutputSearchResult)
        {
            String SPOutputXML = null;
            OMError ErrorObj = null;
            StringCollection AvoidFileTransferList = null;
            StringCollection ASyncronousFields = null;
            FileSystemManager FSManagerObj = null;

            try
            {
                ErrorObj = OMDataManager.DataProvider.ExecuteDynamicSQL(DynamicSQL, ConnectionString, NSearchCommonObj, ref SPOutputXML);
                if (ErrorObj != null)
                {
                    OutputSearchResult = null;
                    return ErrorObj;
                }

                if (!String.IsNullOrEmpty(SPOutputXML))
                    FormResultWithXMLData(SPOutputXML, ref OutputSearchResult);

                if (!String.IsNullOrEmpty(SPOutputXML))
                {
                    ASyncronousFields = new StringCollection();
                    AvoidFileTransferList = new StringCollection();

                    foreach (HierarchyObject Hierarchyobject in NSearchCommonObj.HierarchyObjectList)
                    {
                        foreach (FieldCollection Field in Hierarchyobject.CustomFieldCollectionList)
                        {
                            if ((Field.m_FieldDataType.ToUpper() == "BINDER") || (Field.m_FieldDataType.ToUpper() == "FILE"))
                            {
                                if (Field.m_IsFileRequired.ToUpper().Equals("NO"))
                                {
                                    AvoidFileTransferList.Add(Field.m_SchemaID);
                                }
                                if (Field.m_TransferMode.ToUpper().Equals("ASYNCHRONOUS"))
                                    ASyncronousFields.Add(Field.m_SchemaID);
                            }
                        }
                    }

                    // Anand : NSearchCommonObj condition will be true only if FileRequired is set to true for any 
                    // of the Entity or Facet Attribute. 
                    if (NSearchCommonObj.IsFileSystemRequired)
                    {
                        try
                        {
                            FSManagerObj = new FileSystemManager(NSearchCommonObj.UserID, NSearchCommonObj.TenantName, false, OMDataManager);
                            FSManagerObj.FileTransfer = m_delFileTransfer;
                            ErrorObj = FSManagerObj.RetriveFileSystem(ASyncronousFields, AvoidFileTransferList, ref OutputSearchResult);
                        }
                        catch
                        {
                            throw;
                        }
                        finally
                        {
                            //Close database connections used by File System Manager
                            if (FSManagerObj != null)
                                FSManagerObj.CloseDatabaseConnection();
                        }
                    }
                    if (ErrorObj != null)
                    {
                        OutputSearchResult = null;
                        return ErrorObj;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ErrorObj);
                return ErrorObj;
            }
        }


        /// <summary>
        /// Executes list of DynamicSQLs on the same connection for TD.
        /// </summary>
        /// <param name="DynamicSQLs">List of DynamicSQLs</param>
        /// <param name="ConnectionStrings">Contains ConnectionStrings -1.ConnectionStringForRead and 2.ConnectionString</param>
        /// <param name="NSearchCommonObj">NSearchCommonObj</param>
        /// <param name="OutputResults">Contains results of dynamicSQLs</param>
        /// <returns>OMError if any error/exception occurs, otherwise null.</returns>
        private OMError ExecuteDynamicSQLs(List<String> DynamicSQLs, Dictionary<String,String> ConnectionStrings, NSearchCommon NSearchCommonObj, ref List<String> OutputResults)
        {
            //String SPOutputXML = null;
            OMError ErrorObj = null;
            StringCollection AvoidFileTransferList = null;
            StringCollection ASyncronousFields = null;
            FileSystemManager FSManagerObj = null;
            String OutputSearchResult = null;
            List<String> TempOutputResult = new List<String>();


            try
            {
                ErrorObj = OMDataManager.DataProvider.ExecuteDynamicSQLs(DynamicSQLs, ConnectionStrings, NSearchCommonObj, ref TempOutputResult);
                if (ErrorObj != null)
                {
                    OutputSearchResult = null;
                    return ErrorObj;
                }

                foreach (String  SPOutputXML in TempOutputResult)
                {
                    if (!String.IsNullOrEmpty(SPOutputXML))
                    {
                        FormResultWithXMLData(SPOutputXML, ref OutputSearchResult);
                        
                    }

                    if (!String.IsNullOrEmpty(SPOutputXML))
                    {
                        ASyncronousFields = new StringCollection();
                        AvoidFileTransferList = new StringCollection();

                        foreach (HierarchyObject Hierarchyobject in NSearchCommonObj.HierarchyObjectList)
                        {
                            foreach (FieldCollection Field in Hierarchyobject.CustomFieldCollectionList)
                            {
                                if ((Field.m_FieldDataType.ToUpper() == "BINDER") || (Field.m_FieldDataType.ToUpper() == "FILE"))
                                {
                                    if (Field.m_IsFileRequired.ToUpper().Equals("NO"))
                                    {
                                        AvoidFileTransferList.Add(Field.m_SchemaID);
                                    }
                                    if (Field.m_TransferMode.ToUpper().Equals("ASYNCHRONOUS"))
                                        ASyncronousFields.Add(Field.m_SchemaID);
                                }
                            }
                        }

                        // Anand : NSearchCommonObj condition will be true only if FileRequired is set to true for any 
                        // of the Entity or Facet Attribute. 
                        if (NSearchCommonObj.IsFileSystemRequired)
                        {
                            try
                            {
                                FSManagerObj = new FileSystemManager(NSearchCommonObj.UserID, NSearchCommonObj.TenantName, false, OMDataManager);
                                FSManagerObj.FileTransfer = m_delFileTransfer;
                                ErrorObj = FSManagerObj.RetriveFileSystem(ASyncronousFields, AvoidFileTransferList, ref OutputSearchResult);
                            }
                            catch
                            {
                                throw;
                            }
                            finally
                            {
                                //Close database connections used by File System Manager
                                if (FSManagerObj != null)
                                    FSManagerObj.CloseDatabaseConnection();
                            }
                        }
                        if (ErrorObj != null)
                        {
                            OutputSearchResult = null;
                            return ErrorObj;
                        }
                        OutputResults.Add(OutputSearchResult);
                        OutputSearchResult = null;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQLs",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ErrorObj);
                return ErrorObj;
            }
        }

        private OMError ExecuteDynamicSQLsEx(List<String> DynamicSQLs, Dictionary<String, String> ConnectionStrings, NSearchCommon NSearchCommonObj, ref List<String> OutputResults)
        {
            //String SPOutputXML = null;
            OMError ErrorObj = null;
            StringCollection AvoidFileTransferList = null;
            StringCollection ASyncronousFields = null;
            FileSystemManager FSManagerObj = null;
            String OutputSearchResult = null;
            List<String> TempOutputResult = new List<String>();


            try
            {
                ErrorObj = OMDataManager.DataProvider.ExecuteDynamicSQLsEx(DynamicSQLs, ConnectionStrings, NSearchCommonObj, ref TempOutputResult);
                if (ErrorObj != null)
                {
                    OutputSearchResult = null;
                    return ErrorObj;
                }

                foreach (String SPOutputXML in TempOutputResult)
                {
                    if (!String.IsNullOrEmpty(SPOutputXML))
                    {
                        FormResultWithXMLData(SPOutputXML, ref OutputSearchResult);

                    }

                    if (!String.IsNullOrEmpty(SPOutputXML))
                    {
                        ASyncronousFields = new StringCollection();
                        AvoidFileTransferList = new StringCollection();

                        foreach (HierarchyObject Hierarchyobject in NSearchCommonObj.HierarchyObjectList)
                        {
                            foreach (FieldCollection Field in Hierarchyobject.CustomFieldCollectionList)
                            {
                                if ((Field.m_FieldDataType.ToUpper() == "BINDER") || (Field.m_FieldDataType.ToUpper() == "FILE"))
                                {
                                    if (Field.m_IsFileRequired.ToUpper().Equals("NO"))
                                    {
                                        AvoidFileTransferList.Add(Field.m_SchemaID);
                                    }
                                    if (Field.m_TransferMode.ToUpper().Equals("ASYNCHRONOUS"))
                                        ASyncronousFields.Add(Field.m_SchemaID);
                                }
                            }
                        }

                        // Anand : NSearchCommonObj condition will be true only if FileRequired is set to true for any 
                        // of the Entity or Facet Attribute. 
                        if (NSearchCommonObj.IsFileSystemRequired)
                        {
                            try
                            {
                                FSManagerObj = new FileSystemManager(NSearchCommonObj.UserID, NSearchCommonObj.TenantName, false, OMDataManager);
                                FSManagerObj.FileTransfer = m_delFileTransfer;
                                ErrorObj = FSManagerObj.RetriveFileSystem(ASyncronousFields, AvoidFileTransferList, ref OutputSearchResult);
                            }
                            catch
                            {
                                throw;
                            }
                            finally
                            {
                                //Close database connections used by File System Manager
                                if (FSManagerObj != null)
                                    FSManagerObj.CloseDatabaseConnection();
                            }
                        }
                        if (ErrorObj != null)
                        {
                            OutputSearchResult = null;
                            return ErrorObj;
                        }
                        OutputResults.Add(OutputSearchResult);
                        OutputSearchResult = null;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQLs",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ErrorObj);
                return ErrorObj;
            }
        }

        private OMError ExecuteDynamicSQLsEx_Scalar(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref DataSet DS_Result)
        {
            OMError ErrorObj            = null;

            try
            {
                ErrorObj = OMDataManager.DataProvider.ExecuteDynamicSQL_Scalar(DynamicSQL, ConnectionString, ref DS_Result);
                if (ErrorObj != null)
                {
                    return ErrorObj;
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQLsEx_Scalar",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ErrorObj);
                return ErrorObj;
            }
        }

        private OMError ExecutePagingDynamicSQL(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String OutputSearchResult)
        {
            String              SPOutputXML             = null;
            OMError             ErrorObj                = null;
            StringCollection    AvoidFileTransferList   = null;
            StringCollection    ASyncronousFields       = null;
            FileSystemManager   FSManagerObj            = null;

            try
            {
                ErrorObj = OMDataManager.DataProvider.ExecutePagingDynamicSQL(DynamicSQL, ConnectionString, NSearchCommonObj, ref SPOutputXML);
                if (ErrorObj != null)
                {
                    OutputSearchResult = null;
                    return ErrorObj;
                }

                if (!String.IsNullOrEmpty(SPOutputXML))
                    FormResultWithXMLData(SPOutputXML, ref OutputSearchResult);

                if (!String.IsNullOrEmpty(SPOutputXML))
                {
                    ASyncronousFields = new StringCollection();
                    AvoidFileTransferList = new StringCollection();

                    foreach (HierarchyObject Hierarchyobject in NSearchCommonObj.HierarchyObjectList)
                    {
                        foreach (FieldCollection Field in Hierarchyobject.CustomFieldCollectionList)
                        {
                            if ((Field.m_FieldDataType.ToUpper() == "BINDER") || (Field.m_FieldDataType.ToUpper() == "FILE"))
                            {
                                if (Field.m_IsFileRequired.ToUpper().Equals("NO"))
                                {
                                    AvoidFileTransferList.Add(Field.m_SchemaID);
                                }
                                if (Field.m_TransferMode.ToUpper().Equals("ASYNCHRONOUS"))
                                    ASyncronousFields.Add(Field.m_SchemaID);
                            }
                        }
                    }

                    // Anand : NSearchCommonObj condition will be true only if FileRequired is set to true for any 
                    // of the Entity or Facet Attribute. 
                    if (NSearchCommonObj.IsFileSystemRequired)
                    {
                        try
                        {
                            FSManagerObj = new FileSystemManager(NSearchCommonObj.UserID, NSearchCommonObj.TenantName, false, OMDataManager);
                            FSManagerObj.FileTransfer = m_delFileTransfer;
                            ErrorObj = FSManagerObj.RetriveFileSystem(ASyncronousFields, AvoidFileTransferList, ref OutputSearchResult);
                        }
                        catch
                        {
                            throw;
                        }
                        finally
                        {
                            //Close database connections used by File System Manager
                            if (FSManagerObj != null)
                                FSManagerObj.CloseDatabaseConnection();
                        }
                    }
                    if (ErrorObj != null)
                    {
                        OutputSearchResult = null;
                        return ErrorObj;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecutePagingDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ErrorObj);
                return ErrorObj;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="HierarchyObj">Hierarchy Object which has all information about TargetEntity</param>
        /// <param name="SimpleSearchQuery">SimpleSearch Query with where clause appended</param>
        /// <returns></returns>
        private OMError PrepareSimpleSearchQuery(HierarchyObject HierarchyObj, ref XmlNode ObjectTypeNode, ref StringBuilder SimpleSearchQueryBuilder)
        {
            QueryBuilder QueryBuilderTeradataObj = null;
            SearchUtils SearchUtilsObj = null;
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;

            string Query = null;
            StringCollection PostFixString = null;
            Parser ParserObj = null;
            XmlNode FacetTypeNode = null;

            try
            {
                PostFixString = new StringCollection();
                ParserObj = new Parser();

                if ((ErrorObj = ParserObj.ParseString(HierarchyObj.m_SimpleSearchString)) != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType,
                                ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                    return ReturnErrorObj;
                }
                if (ParserObj.IsValid)
                {
                    if ((ErrorObj = ParserObj.PostFix()) != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                        ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType,
                                        ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    QueryBuilderTeradataObj = new QueryBuilder(this.UserID, this.CurrentLanguage, this.TenantName, OMDataManager);
                    QueryBuilderTeradataObj.m_PostFixString = ParserObj.PostFixString;
                    QueryBuilderTeradataObj.m_IsCaseSensitive = HierarchyObj.m_IsSimpleSearchCaseSensitive.ToString();

                    if ((ErrorObj = QueryBuilderTeradataObj.PrepareDummyQuery(ParserObj.PostFixString, "")) != null)
                        return ErrorObj;

                    if ((ErrorObj = OMDataManager.DataProvider.SearchTargetObjectForQueryString(ObjectTypeNode, QueryBuilderTeradataObj.m_DummyQuery,QueryBuilderTeradataObj.m_DummyFieldname, ref Query)) != null)
                        return ErrorObj;

                    if (!String.IsNullOrEmpty(Query))
                        SimpleSearchQueryBuilder.Append(Query);

                    if (HierarchyObj.m_IsSearch_In_All_Facets)
                    {
                        SearchUtilsObj = new SearchUtils(OMDataManager);
                        if ((HierarchyObj.m_RelatedFacetTypes != null) && (HierarchyObj.m_RelatedFacetTypes.Count > 0))
                        {
                            foreach (DictionaryEntry RelationEntry in HierarchyObj.m_RelatedFacetTypes)
                            {
                                if ((ErrorObj = SearchUtilsObj.ReadObjectTypeByID("FACET", Convert.ToUInt64(RelationEntry.Value.ToString()), ref FacetTypeNode)) != null)
                                    return ErrorObj;
                                Query = null;

                                //11-Feb-2009, used the changed signature with the searchstring passed
                                if ((ErrorObj = OMDataManager.DataProvider.SearchSubFacetsForQueryString(FacetTypeNode,
                                    HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeID,
                                    QueryBuilderTeradataObj.m_DummyQuery,
                                    QueryBuilderTeradataObj.m_DummyFieldname,
                                    HierarchyObj.m_SimpleSearchString,
                                    ref Query)) != null)
                                    return ErrorObj;

                                if (!String.IsNullOrEmpty(Query))
                                    SimpleSearchQueryBuilder.AppendFormat(" {0} ", Query);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in processing search query",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }

        }

        private OMError GetBinderandFileFields()
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            QueryBuilder QueryBuilderTeradataObj = null;

            try
            {
                QueryBuilderTeradataObj = new QueryBuilder(OMDataManager);
                NSearchCommonObj.m_BinderFields = new SortedList();
                NSearchCommonObj.m_FileFields = new SortedList();
                NSearchCommonObj.m_AssoFileBinderFields = new SortedList();

                NSearchCommonObj.m_BinderFields = QueryBuilderTeradataObj.GetListOfFieldsFromSchema("ALFS_BINDER");
                // NSearchCommonObj will add FIELDNAME as default 
                NSearchCommonObj.m_BinderFields.Add("FIELDNAME", "STRING");
                NSearchCommonObj.m_FileFields = QueryBuilderTeradataObj.GetListOfFieldsFromSchema("ALFS_FILE");
                NSearchCommonObj.m_FileFields.Add("FIELDNAME", "STRING");
                NSearchCommonObj.m_AssoFileBinderFields = QueryBuilderTeradataObj.GetListOfFieldsFromSchema("ALFS_ASSOFILEBINDER");

                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetBinderandFileFields",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }
        /// <summary>
        /// <Purpose>
        /// Generate uniqueid.
        /// It uses GUID to generate the nextid.
        /// It creates a GUId and remove the first 3 bytes and
        /// convert the last byte into integer        ///
        /// </Purpose>
        /// </summary>
        /// <param name="UniqueID">Returns a long UnqiueID</param>
        /// <returns></returns>
        private OMError GetUniqueId(ref long UniqueID)
        {
            #region Original Code
            //****This code is commented by Kiran Abhang on 30th September 2011.
            //Instead of generating unique ID from GUID, we are now using a sequential ID.
            //This unique ID is used for temporary table variable names and for the field names from such tables.
            //OMError ErrorObj = null;
            //OMError ReturnErrorObj = null;
            //String GUID = null;
            //OMError ReturnErrObj = null;
            //Object Lock = new Object();

            //try
            //{
            //    lock (Lock)
            //    {
            //        GUID = Guid.NewGuid().ToString();
            //        GUID = GUID.Remove(0, 24);
            //        UniqueID = Convert.ToInt64(GUID, 16);
            //    }
            //    return ErrorObj;
            //}
            //catch (Exception ExceptionObj)
            //{
            //    ReturnErrObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetUniqueId",
            //        OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
            //    ExceptionTracing.LogError(TenantName, ReturnErrorObj);
            //    return ReturnErrorObj;
            //}
            #endregion

            //New Code.
            UniqueID = ++int_UniqueIDCount;
            return (null);
        }

        private OMError GetGroupIDList(ref String GroupIDList, ulong LoggedInUserID)
        {
            OMError ErrorObj = null;
            String AssociationXML = null;
            XmlDocument AssociationXMLDoc = null;
            XmlNodeList AssociationNodeList = null;
            String DirectGroupIDList = null;
            StringBuilder Message = null;
            XmlDocument RelationXMLDoc = null;
            String RelationshipOutXML = null;
            String RelationshipTypeID = null;

            try
            {
                GroupIDList = null;

                if (OMDataManager.TraceEnabled)
                {
                    Message = new StringBuilder();
                    Message.AppendFormat("Entered GetGroupIDList Method of Search ");
                    ExceptionTracing.TraceLog(Message.ToString(), "SEARCH", TenantName, DateTime.Now);
                }

                // Get RelationshipId for relationshipname "GROUP_MEMBERUSER"
                if ((ErrorObj = OMDataManager.DataProvider.AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, "GROUP_MEMBERUSER", OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = OMDataManager.DataProvider.ReadRecord("RELATION", "ALOM_RELATIONSHIPTYPES", null, ref RelationshipOutXML)) == null)
                    {
                        if (RelationshipOutXML != null && RelationshipOutXML != "")
                        {
                            RelationXMLDoc = new XmlDocument();
                            RelationXMLDoc.LoadXml(RelationshipOutXML);
                            RelationshipTypeID = RelationXMLDoc.SelectSingleNode("RELATION/RELATIONSHIPTYPEID").InnerText;

                        }
                    }
                }

                if (ErrorObj != null)
                    return ErrorObj;


                if ((ErrorObj = OMDataManager.DataProvider.AddClause("CONTAINSID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, LoggedInUserID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrorObj = OMDataManager.DataProvider.AddClause("RELATIONSHIPTYPEID", OMFieldDataType.STRING, OMOperator.EQUALTO, RelationshipTypeID, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = OMDataManager.DataProvider.ReadAllRecord("ASSEMBLY", "ALOM_ASSEMBLY", null, ref AssociationXML)) == null)
                        {
                            if (ErrorObj != null)
                            {
                                return ErrorObj;
                            }

                            if (AssociationXML != null && AssociationXML != "")
                            {
                                AssociationXMLDoc = new XmlDocument();
                                AssociationXMLDoc.LoadXml(AssociationXML);
                                AssociationNodeList = AssociationXMLDoc.SelectNodes("ASSEMBLYCOLLECTION/ASSEMBLY");
                                if (AssociationNodeList.Count == 0)
                                    return null;
                                else
                                {
                                    // CreateXML Node for GroupList
                                    XmlNode RootGroupNode = AssociationXMLDoc.CreateElement("GROUPIDLIST");
                                    foreach (XmlNode AssemblyNode in AssociationNodeList)
                                    {
                                        XmlNode GroupNode = AssociationXMLDoc.CreateElement("GROUPID");
                                        GroupNode.InnerText = AssemblyNode.SelectSingleNode("CONTAINERID").InnerText;
                                        RootGroupNode.AppendChild(GroupNode);
                                    }
                                    DirectGroupIDList = RootGroupNode.OuterXml;
                                }
                            }
                        }
                    }
                }

                if (ErrorObj != null)
                {
                    return ErrorObj;
                }

                if (DirectGroupIDList != null && DirectGroupIDList != "")
                {
                    // call a GroupHierarchy function which will return all the Groups directly or indirectly for that User
                    Assetlink.ObjectManager.GroupHierarchy.GroupHierarchy GroupObj = new Assetlink.ObjectManager.GroupHierarchy.GroupHierarchy(OMDataManager);
                    ErrorObj = GroupObj.ReadAncestorHierarchy(DirectGroupIDList, ref GroupIDList);
                    if (ErrorObj != null)
                        return ErrorObj;

                }
                
                if (String.IsNullOrEmpty(GroupIDList))
                    GroupIDList = this.UserID.ToString();
                else
                    GroupIDList = GroupIDList + "," + this.UserID.ToString();
                return null;

            }
            catch (Exception ExceptionObj)
            {
                Message = new StringBuilder();
                Message.AppendFormat("private OMError GetGroupIDList(ref String GroupIDList");
                ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Association Read Failed", OMErrorObjectType.GETDATAQUERYBUILDER,
                    "NamedHierarchy", OMErrorType.EXCEPTION, ExceptionObj, Message.ToString());
                return ErrorObj;
            }
        }

        private OMError FormResultWithXMLData(String SPOutputXML, ref String OutputSearchResult)
        {
            OMError                     ErrorObj                        = null;
            OMError                     ReturnErrorObj                  = null;
            XmlDocument                 XMLProcessDoc                   = null;
            XmlNodeList                 InstanceNodeList                = null;
            StringBuilder               XPath                           = null;
            ulong                       ObjectID                        = 0;
            String                      XMLFieldData                    = null;
            IXMLRepository              ObjXMLManager                   = null;
            XmlElement                  XMLDataNode                     = null;

            try
            {
                if (SPOutputXML != null && SPOutputXML != "")
                {
                    //Replace xmlns:xsi namespace and xsi:nil attributes with empty string (so that solutions need not declare xsi namespace in xml)
                    if (SPOutputXML.IndexOf("xmlns:xsi=") >= 0)
                    {
                        SPOutputXML = SPOutputXML.Replace("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"", "");

                        /* In case of Paging (i.e. MAXRESULTSIZE specified), xsi:nil="true" gets returned and in case when paging is not
                         * specified, xsi:nil="1" gets returned for integer and float field nodes having value as NULL. So xsi:nil="1" and
                         * xsi:nil="true" will be searched and removed.
                         * */
                        if (SPOutputXML.IndexOf("xsi:nil=\"1\"") >= 0)
                        {
                            SPOutputXML = SPOutputXML.Replace("xsi:nil=\"1\"", "");
                        }

                        if (SPOutputXML.IndexOf("xsi:nil=\"true\"") >= 0)
                        {
                            SPOutputXML = SPOutputXML.Replace("xsi:nil=\"true\"", "");
                        }
                    }

                    ObjXMLManager = OMDataManager.XMLRepository; //new XmlDocument();
                    XMLProcessDoc = new XmlDocument();
                    XMLProcessDoc.LoadXml(SPOutputXML);
                    XPath = new StringBuilder();
                    ObjXMLManager = new XMLManager(TenantName);
                }
                else
                {
                    ErrorObj = new OMError(OMErrorCodes.OMI_GETDATA_FAILED, " Invalid Stored Procedure output XML", OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.ERROR, null, "");
                    return ErrorObj;
                }
                // Process XML Fields.
                foreach (HierarchyObject TempHierarchyObject in NSearchCommonObj.HierarchyObjectList)
                {
                    if (!TempHierarchyObject.m_RelationshipName.Contains("FILESYSTEM"))
                    {
                        if (TempHierarchyObject.m_ParentID == 0)
                        {
                            // Processing for TE.
                            foreach (FieldCollection Field in TempHierarchyObject.CustomFieldCollectionList)
                            {
                                if (Field.m_FieldDataType.ToUpper() == "XML")
                                {
                                    //XPath = TempHierarchyObject.m_CollectionName  
                                    XPath.Remove(0, XPath.Length);
                                    XPath.AppendFormat("{0}/{1}", TempHierarchyObject.m_CollectionName, TempHierarchyObject.m_ObjectTypeName);
                                    InstanceNodeList = XMLProcessDoc.SelectNodes(XPath.ToString());
                                    foreach (XmlNode Instance in InstanceNodeList)
                                    {
                                        ObjectID = Convert.ToUInt64(Instance.SelectSingleNode(TempHierarchyObject.m_PrimaryKeyName).InnerText);
                                        ObjXMLManager.Load(TempHierarchyObject.m_ObjectTypeName, ObjectID, Field.m_FieldName, out XMLFieldData);
                                        if (Field.m_Alias != null && Field.m_Alias != "")
                                            XMLDataNode = XMLProcessDoc.CreateElement(Field.m_Alias);
                                        else
                                            XMLDataNode = XMLProcessDoc.CreateElement(Field.m_FieldName);
                                        if (XMLFieldData != null)
                                            XMLDataNode.InnerXml = XMLFieldData;
                                        Instance.AppendChild(XMLDataNode);
                                    }
                                }
                            }
                        }
                        else
                        {
                            // Processing for Related Entity and Related Facets.
                            foreach (FieldCollection Field in TempHierarchyObject.CustomFieldCollectionList)
                            {
                                if (Field.m_FieldDataType.ToUpper() == "XML")
                                {

                                    //Bug 6621 Fix : To get the path for individual xml fields iterate through the HierarchyObjectList
                                    //and compose the entire path
                                    string ParentPath = "";
                                    long CurrentParentID = 0;
                                    CurrentParentID = TempHierarchyObject.m_ParentID;

                                    do
                                    {
                                        foreach (HierarchyObject SearchHierarchyObj in NSearchCommonObj.HierarchyObjectList)
                                        {
                                            if (CurrentParentID == SearchHierarchyObj.m_TagID)
                                            {
                                                if (SearchHierarchyObj.m_ParentID != 0)
                                                {
                                                    // In case parent is not the root node
                                                    ParentPath = SearchHierarchyObj.m_RelationshipName + "/" + SearchHierarchyObj.m_ObjectTypeName + "/" + ParentPath;
                                                    CurrentParentID = SearchHierarchyObj.m_ParentID;
                                                    break;
                                                }
                                                else
                                                {
                                                    // In case parent is the root node
                                                    ParentPath = SearchHierarchyObj.m_CollectionName + "/" + SearchHierarchyObj.m_ObjectTypeName + "/" + ParentPath;
                                                    CurrentParentID = SearchHierarchyObj.m_ParentID;
                                                    break;
                                                }
                                            }
                                        }
                                    } while (CurrentParentID != 0);

                                    //XPath = TempHierarchyObject.m_CollectionName  
                                    XPath.Remove(0, XPath.Length);

                                    //The following lines have been commented to fix Bug 6621
                                    //XPath.AppendFormat("//*/{0}/{1}/{2}", TempHierarchyObject.m_RelationshipName,
                                    //                                TempHierarchyObject.m_ObjectTypeName,
                                    //                                TempHierarchyObject.m_PrimaryKeyName);

                                    XPath.AppendFormat("{0}{1}/{2}/{3}", ParentPath, TempHierarchyObject.m_RelationshipName,
                                                                                TempHierarchyObject.m_ObjectTypeName,
                                                                                    TempHierarchyObject.m_PrimaryKeyName);

                                    InstanceNodeList = XMLProcessDoc.SelectNodes(XPath.ToString());
                                    foreach (XmlNode Instance in InstanceNodeList)
                                    {
                                        ObjectID = Convert.ToUInt64(Instance.ParentNode.SelectSingleNode(TempHierarchyObject.m_PrimaryKeyName).InnerText);
                                        ObjXMLManager.Load(TempHierarchyObject.m_ObjectTypeName, ObjectID, Field.m_FieldName, out XMLFieldData);
                                        XMLDataNode = XMLProcessDoc.CreateElement(Field.m_FieldName);
                                        if (XMLFieldData != null)
                                            XMLDataNode.InnerXml = XMLFieldData;

                                        Instance.ParentNode.AppendChild(XMLDataNode);
                                    }
                                }
                            }
                        }
                    }
                }

                if (XMLProcessDoc.InnerXml == null)
                    OutputSearchResult = SPOutputXML;
                else
                    OutputSearchResult = XMLProcessDoc.InnerXml;
                
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormResultWithXMLData",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// Returns count of BINDER fields from custom field collection list.
        /// </summary>
        /// <param name="HierarchyObj">Hierarchy object.</param>
        /// <param name="BinderFieldCount">Output parameter, representing number of binder fields present.</param>
        /// <returns>OMError if any error/exception occurs, otherwise null.</returns>
        private OMError GetBinderFieldCount(HierarchyObject HierarchyObj, out int BinderFieldCount)
        {
            OMError             obj_OMError             = null;     //Used to prepare error

            BinderFieldCount = 0;

            try
            {
                //Iterate over custom field collection list and return count of BINDER fields
                if (HierarchyObj.CustomFieldCollectionList.Count > 0)
                {
                    foreach (FieldCollection obj_CustomField in HierarchyObj.CustomFieldCollectionList)
                    {
                        if (obj_CustomField.m_FieldDataType.ToUpper() == "BINDER")
                            BinderFieldCount++;    
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                obj_OMError = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetBinderFieldCount",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");

                ExceptionTracing.LogError(TenantName, obj_OMError);
            }

            return obj_OMError;
        }

        /// <summary>
        /// The method generates a Full Text query over the ALFCT_SEARCHABLE_FILE_TEXT table.
        /// </summary>
        /// <param name="HierarchyObj">Hierarchy Object which has all information about TargetEntity</param>
        /// <param name="ObjectTypeNode">Entity/Facet type data</param>
        /// <param name="DocumentSearchQueryBuilder">Document Search Query</param>
        /// <returns></returns>
        private OMError PrepareDocumentSearchQuery(HierarchyObject HierarchyObj, ref XmlNode ObjectTypeNode, ref StringBuilder DocumentSearchQueryBuilder)
        {
            QueryBuilder QueryBuilderTeradataObj = null;
            SearchUtils SearchUtilsObj = null;            
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            string Query = null;
            StringCollection PostFixString = null;
            Parser ParserObj = null;
            XmlNode FacetTypeNode = null;

            try 
            {
                PostFixString = new StringCollection();
                ParserObj = new Parser();
                DocumentSearchQueryBuilder = new StringBuilder();

                if ((ErrorObj = ParserObj.ParseString(HierarchyObj.m_SimpleSearchString)) != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType,
                                ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                    return ReturnErrorObj;
                }
                if (ParserObj.IsValid)
                {
                    if ((ErrorObj = ParserObj.PostFix()) != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, ErrorObj.ErrorMessage,
                                        ErrorObj.ObjectType, ErrorObj.ObjectTypeName, ErrorObj.ErrorType,
                                        ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    QueryBuilderTeradataObj = new QueryBuilder(this.UserID, this.CurrentLanguage, this.TenantName, OMDataManager);
                    QueryBuilderTeradataObj.m_PostFixString = ParserObj.PostFixString;
                    QueryBuilderTeradataObj.m_IsCaseSensitive = HierarchyObj.m_IsSimpleSearchCaseSensitive.ToString();

                    if ((ErrorObj = QueryBuilderTeradataObj.PrepareDummyQuery(ParserObj.PostFixString, "")) != null)
                        return ErrorObj;

                    //Prepare search query for the entity type
                    if ((ErrorObj = OMDataManager.DataProvider.SearchTargetObjectForQueryStringInFile(ObjectTypeNode, HierarchyObj.m_SimpleSearchString, HierarchyObj.m_FileRelationships, ref Query)) != null)
                        return ErrorObj;

                    if (!String.IsNullOrEmpty(Query))
                        DocumentSearchQueryBuilder.Append(Query);

                    //Prepare search query for the associated facet types
                    if (HierarchyObj.m_IsSearch_In_All_Facets)
                    {
                        SearchUtilsObj = new SearchUtils(OMDataManager);
                        if ((HierarchyObj.m_RelatedFacetTypes != null) && (HierarchyObj.m_RelatedFacetTypes.Count > 0))
                        {
                            foreach (DictionaryEntry RelationEntry in HierarchyObj.m_RelatedFacetTypes)
                            {
                                if ((ErrorObj = SearchUtilsObj.ReadObjectTypeByID("FACET", Convert.ToUInt64(RelationEntry.Value.ToString()), ref FacetTypeNode)) != null)
                                    return ErrorObj;
                                Query = null;

                                //11-Feb-2009, used the changed signature with the searchstring passed
                                if ((ErrorObj = OMDataManager.DataProvider.SearchSubFacetsForQueryStringInFile(FacetTypeNode,
                                    HierarchyObj.m_ObjectTypeID,
                                    HierarchyObj.m_SimpleSearchString,
                                    ref Query)) != null)
                                    return ErrorObj;

                                if (!String.IsNullOrEmpty(Query))
                                    DocumentSearchQueryBuilder.AppendFormat(" {0} ", Query);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in processing search query",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                ExceptionTracing.LogError(TenantName, ReturnErrorObj);
                return ReturnErrorObj;
            }
        }

        #endregion

    }
}
