using System;
using Assetlink;
using Assetlink.ObjectManager;
using System.IO;
using System.Text;
using System.Data;

namespace Assetlink.Framework
{
    public class OMInterface:AssetlinkBase
    {
        private FileTransferDelegate.OnFileTransfer m_delFileTransfer = null;
        public FileTransferDelegate.OnFileTransfer FileTransfer
        {
            set
            {
                m_delFileTransfer = new FileTransferDelegate.OnFileTransfer(value);
            }
        }
        #region Constructor
        public OMInterface()
        {
        }

        public OMInterface(ulong UserID, string CurrentLanguage, string TenantName)
            : base(UserID, CurrentLanguage, TenantName)
        {
        }
        #endregion

        /// <summary>
        /// This allows consumers to execute multiple operations.
        /// </summary>
        /// <remarks>
        /// NULL or Empty String if successful execution. 
        /// Otherwise returns a XML having detail of the  errors.
        /// The XML will follow the Error.xsd.
        /// </remarks>
        /// <param name="InputXML">
        /// Operations are represented using the input xml. 
        /// The XML follows InputXML.xsd.
        /// </param>
        public String Execute(ref String InputXML)
        {
            String  ErrorXML        = null;   //String variable to hold ErrorXML
            OMError ExcepErrObj     = null;  //Storing the Error object when Exception occurs
            bool    TraceEnabled    = false;
            try
            {

                //Validating the InputXML
                //if ((ErrorXML = DataValidator.IsDataValid(InputXML)) != null)
                //    return ErrorXML;


                //Read the Configuration file for Tracing
                //<Code need to put>
                
                //Call the OM Execute Method
                ObjectManagerClass OM = new ObjectManagerClass(UserID,null,TenantName, TraceEnabled);
                if ((ErrorXML = OM.Execute(ref InputXML)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
              
                ExcepErrObj = new OMError(OMErrorCodes.OMI_EXECUTE_FAILED,
                        "OMI Execute operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                        "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);

                ErrorXML = ExcepErrObj.Serialize();
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""Execute()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML ;
            }

            return null;
        }

        /// <summary>
        /// This method provides consumer to retrieve information related to entities and facets. It also provides way to obtain information about the related entities and  facets.
        /// </summary>
        /// <param name="InputXML">This represent the query or filters provided by consumer for retrieval of data. This should follow SearchQueryXML.xsd.</param>
        /// <param name="FieldListXML">This represent the list of columns that is expected as part of output. The consumer can also request for columns from related objects i.e. objects related to Target objects. The XML should follow FieldList.xsd.</param>
        /// <param name="OutputXML">The return XML which contains the result.</param>
        /// <returns>NULL or Empty String if successful execution. Otherwise returns a XML having detail of the errors. The XML will follow the Error.xsd.</returns>
        [Obsolete("GetData method is Depricated. Please use GetHierarchialData for future developments",false)] 
        public String GetData(String InputXML, string FieldListXML, ref string OutputXML)
        {
            String  ErrorXML        = null; //String variable to hold ErrorXML
            OMError ExcepErrObj     = null; //Storing the Error object when Exception occurs
            bool    TraceEnabled    = false;

            try
            {
                //Validating the InputXML.
                //if ((ErrorXML = DataValidator.IsDataValid(InputXML)) != null)
                //    return ErrorXML;

                //Validating the FieldListXML.
                if ((ErrorXML = DataValidator.ValidateSearchFieldListXML(FieldListXML)) != null)
                    return ErrorXML;

                //Call the OM GetData Method.
                ObjectManagerClass OM = new ObjectManagerClass(UserID, null, TenantName, TraceEnabled);
                OM.FileTransfer = m_delFileTransfer;

                if ((ErrorXML = OM.GetData(InputXML, FieldListXML, ref OutputXML)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ExcepErrObj = new OMError(OMErrorCodes.OMI_GETDATA_FAILED,
                                    "OMI GetData operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                                    "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);
                ErrorXML = ExcepObj.ToString(); //Has to Change
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML;
            }

            return null;
        }

        /// <summary>
        /// This method provides consumer to retrieve specific Files.
        /// </summary>
        /// <param name="InputXML">This should follow FileRequest.xsd.</param>
        /// <returns>NULL or Empty String if successful execution.Also on successful exeicution path nodes will be appended to the File nodes,which specify from where file should be picked up. Otherwise returns a XML having detail of the errors. The XML will follow the Error.xsd.</returns>
        public String GetFiles(ref String InputXML)
        {
            String ErrorXML = null;   //String variable to hold ErrorXML
            OMError ExcepErrObj = null; //Storing the Error object when Exception occurs
            bool TraceEnabled = false;
            try
            {
                //Validating the InputXML
                //if ((ErrorXML = DataValidator.IsDataValid(InputXML)) != null)
                //    return ErrorXML;
                //Call the OM Execute Method
                ObjectManagerClass OM = new ObjectManagerClass(UserID, null, TenantName, TraceEnabled);
                OM.FileTransfer = m_delFileTransfer;
                if ((ErrorXML = OM.GetFiles(ref InputXML)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetFiles()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExcepObj)
            {
                ExcepErrObj = new OMError(OMErrorCodes.OMI_GETFILES_FAILED,
                       "OMI GetFiles operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                       "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);

                ErrorXML = ExcepObj.ToString(); //Has to Change
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetFiles()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML;
            }

           
        }



        /// <summary>
        /// This provides consumer to retrieve information about various Object Manager configuration object like EntityType, FacetType and Access Control objects.
        /// </summary>
        /// <param name="InputXML">The input xml that provides the information configuration object to be retrieved.  The XML will follow InputXML.xsd.</param>
        /// <param name="OutputXML">The return XML.</param>
        /// <returns>NULL or Empty String if successful execution. Otherwise returns a XML having detail of the   errors. The XML will follow the Error.xsd.</returns>
        public String GetTypeData(string InputXML, ref string OutputXML)
        {
            String  ErrorXML     = null;   //String variable to hold ErrorXML
            OMError ExcepErrObj = null; //Storing the Error object when Exception occurs
            bool    TraceEnabled = false;
            try
            {
                //Validating the InputXML
                //if ((ErrorXML = DataValidator.IsDataValid(InputXML)) != null)
                //    return ErrorXML;

                //Call the OM Execute Method
                ObjectManagerClass OM = new ObjectManagerClass(UserID, null, TenantName, TraceEnabled);
                if ((ErrorXML = OM.GetTypeData(InputXML, ref OutputXML)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetTypeData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ExcepErrObj = new OMError(OMErrorCodes.OMI_GETTYPEDATA_FAILED,
                       "OMI GetTypeData operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                       "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);

                ErrorXML = ExcepObj.ToString(); //Has to Change
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetTypeData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML;
            }

            return null;

        }

        /// <summary>
        /// This method provides consumer to retrieve information related to entities and facets upto NLevel. 
        /// </summary>
        /// <param name="SearchInputXML">Search Input XML</param>
        /// <param name="OutputXML">OutputXML for Search Result</param>
        /// <returns>NULL or Empty String if successful execution. Otherwise returns a XML having detail of the   errors.</returns>
        public String GetHierarchialData(String SearchInputXML, ref String OutputXML)
        {
            String ErrorXML = null;     //String variable to hold ErrorXML
            OMError ExcepErrObj = null;     //Storing the Error object when Exception occurs
            bool TraceEnabled = false;

            try
            {

                ObjectManagerClass OM = new ObjectManagerClass(UserID, null, TenantName, TraceEnabled);

                if ((ErrorXML = OM.GetHierarchialData(SearchInputXML, ref OutputXML)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetHierarchialData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                ExcepErrObj = new OMError(OMErrorCodes.OMI_GETTYPEDATA_FAILED,
                       "OMI GetHierarchialData operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                       "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);

                ErrorXML = ExcepObj.ToString(); //Has to Change
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetHierarchialData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML;
            }
        }

         /// <summary>
        /// This method provides consumer to retrieve information related to entities and facets upto NLevel. 
        /// </summary>
        /// <param name="SearchInputXML">Search Input XML</param>
        /// <param name="DS_Result">DataSet containing Search Result</param>
        /// <returns>NULL or Empty String if successful execution. Otherwise returns a XML having detail of the   errors.</returns>
        public String GetHierarchialData(String SearchInputXML, ref DataSet DS_Result)
        {
            String  ErrorXML        = null;     //String variable to hold ErrorXML
            OMError ExcepErrObj     = null;     //Storing the Error object when Exception occurs
            bool    TraceEnabled    = false;

            try
            {

                ObjectManagerClass OM = new ObjectManagerClass(UserID, null, TenantName, TraceEnabled);

                if ((ErrorXML = OM.GetHierarchialData(SearchInputXML, ref DS_Result)) != null)
                {
                    if (ErrorXML != null)
                    {
                        if (ErrorXML.Contains("<EXCEPTION>"))
                        {
                            WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetHierarchialData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                        }
                    }
                    return ErrorXML;
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                ExcepErrObj = new OMError(OMErrorCodes.OMI_GETDATA_FAILED,
                       "OMI GetHierarchialData operation failed", OMErrorObjectType.OBJECTMANAGERINTERFACE,
                       "ObjectManagerInterface", OMErrorType.EXCEPTION, ExcepObj, null);

                ErrorXML = ExcepObj.ToString(); //Has to Change
                WriteLog(TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetHierarchialData()"",""" + ErrorXML.Replace(@"""", @"""""") + @"""");
                return ErrorXML;
            }
        }

        public void WriteLog(String TenantName, String SolutionName, String PageName, String SessionID, String ContentToWrite)
        {
            lock (this)
            {
                //if (ReadRegisterykey())
                //{
                PageName = "Thread";
                String InstallationDir = ALCommon.GetInstallationDir();
                int LastIndex = PageName.LastIndexOf('/');
                int ExtIndex = PageName.LastIndexOf('.');

                if (LastIndex > 0 && ExtIndex > 0)
                {
                    PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                    PageName = PageName.Replace(' ', '_');
                }
                String DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                String LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                FileStream LogFileStream = null;
                StreamWriter LogStreamWriter = null;

                try
                {
                    DirectoryInfo Root = new DirectoryInfo(DirName);
                    if (Root.Exists == false)
                        Root.Create();

                    if (File.Exists(LogFileName))
                        LogFileStream = new FileStream(LogFileName, FileMode.Append);
                    else
                        LogFileStream = new FileStream(LogFileName, FileMode.Create);

                    LogStreamWriter = new StreamWriter(LogFileStream);
                    StringBuilder TimeStamp = new StringBuilder();

                    //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                    //LogStreamWriter.WriteLine("=====================================================================");
                    //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                    LogStreamWriter.WriteLine(ContentToWrite);
                    LogStreamWriter.WriteLine();
                }
                catch (Exception ex)
                {
                    PageName = "WriteLogError";
                    InstallationDir = ALCommon.GetInstallationDir();
                    LastIndex = PageName.LastIndexOf('/');
                    ExtIndex = PageName.LastIndexOf('.');

                    if (LastIndex > 0 && ExtIndex > 0)
                    {
                        PageName = PageName.Substring(LastIndex + 1, ExtIndex - (LastIndex + 1));
                        PageName = PageName.Replace(' ', '_');
                    }
                    DirName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName;
                    LogFileName = InstallationDir + "\\" + TenantName + "\\Logs\\" + SolutionName + " \\" + PageName + "_" + SessionID + ".txt";

                    FileStream LogFileStreamEx = null;
                    StreamWriter LogStreamWriterEx = null;

                    try
                    {
                        DirectoryInfo Root = new DirectoryInfo(DirName);
                        if (Root.Exists == false)
                            Root.Create();

                        if (File.Exists(LogFileName))
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Append);
                        else
                            LogFileStreamEx = new FileStream(LogFileName, FileMode.Create);
                        string strMessageToWrite = SessionID + "," + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + "," + ex.Message;
                        LogStreamWriterEx = new StreamWriter(LogFileStreamEx);
                        StringBuilder TimeStamp = new StringBuilder();

                        //TimeStamp.AppendFormat(" Date: {0}/{1}/{2} Time: {3}:{4}:{5}:{6}", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second, DateTime.Now.Millisecond);
                        //LogStreamWriter.WriteLine("=====================================================================");
                        //LogStreamWriter.WriteLine(ContentToWrite + TimeStamp.ToString());
                        LogStreamWriterEx.WriteLine(strMessageToWrite);
                        LogStreamWriterEx.WriteLine();
                    }
                    catch
                    {

                    }
                    finally
                    {
                        if (LogStreamWriterEx != null)
                            LogStreamWriterEx.Close();

                        if (LogFileStreamEx != null)
                            LogFileStreamEx.Close();

                        LogStreamWriterEx = null;
                        LogFileStreamEx = null;
                    }
                }
                finally
                {
                    if (LogStreamWriter != null)
                        LogStreamWriter.Close();

                    if (LogFileStream != null)
                        LogFileStream.Close();

                    LogStreamWriter = null;
                    LogFileStream = null;
                }
                //}
            }
        }
    }
}
