using System;
using System.Collections.Generic;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.IO;
using System.Linq;
using System.Xml.Linq;

#region AssetlinkInclude
using Assetlink;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.Services.Engines;
using Assetlink.Services.Engines.DAMEngine;
using Assetlink.Services.Engines.MessageBoardEngine;
#endregion

/*************************************************************
 * Author: Mugdha Moolay
 * Date Create: 24 July 08
 * Purpose: This class is an extension of Process Manager class.
 * This class contains methods for data transfer.
*************************************************************/
namespace Assetlink.ProcessManager
{
    using XML       = String;
    using ERRORXML  = String;

	public partial class ProcessManagerClass
    {
        #region Public Methods
            /// <alsummary>
			/// <createddate>11 Aug 08</createddate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method Initialises the given Process Relevant Data Fields when the process is marked as active</purpose>			
			/// </alsummary>			
			/// <param name="SubflowActivityID">The Activity ID of the Activity that refers to the given process. 
            /// <param name="str_Status">The new changed status of the process</param>
			/// The Actual parameters of this Activity and the Formal parameters of the given process will be used for Data Transfer</param>
			/// <returns>An Error XML on Error, null otherwise.</returns>
			public ERRORXML TransferDataOnProcessInitialisation(String SubflowActivityID, String str_Status)
			{
				ERRORXML					ErrorXML				= null;
				XML							OutputXML				= null;
				XPathDocument				xpDocParameter			= null;				
				XPathNavigator				xpNavActivity			= null;
				XPathNavigator				xpNavActualParams		= null;
				XPathNavigator				xpNavFormalParams		= null;
				XPathNavigator				xpNavActualParam		= null;
				XPathNavigator				xpNavFormalParam		= null;
				XPathNavigator				xpNavRDF				= null;
				XPathNavigator				xpNavSubProcessRDF		= null;
				XPathNavigator				xpNavDataField			= null;
				XPathNavigator				xpNav_NavPath			= null;
				XPathNodeIterator			xpNodeIterFormalParam	= null;
				ParameterMap				PmMap					= null;				
				String						strSeqNo				= null;				
				String						FormalParameterID		= null;
				String						ActualParameterID		= null;
				String						strDataFieldName		= null;
				List<EntityFacetCollection> lstEFObj				= null;
				EntityFacetCollection		EFObj					= null;
				String						strPrimaryKeyName		= null;
				String						strPrimaryKeyValue		= null;
				String						strType					= null;
				String						strTypeName				= null;
				String						strDataType				= null;	                
                String                      str_WorkflowCategory    = null;
				XPathNavigator				xpNavTypeName			= null;
				int							Index					= -1;				
				ALEngineError				ALEngineErrorObj        = null;

                StringDictionary            strdict_AdditionalInfo  = null;

				try
				{
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Start TransferDataOnProcessInitialization() function.");
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Before GetFormalParametersAndActualParametersForSubflow() function.");
					ErrorXML = GetFormalParametersAndActualParametersForSubflow(SubflowActivityID, out OutputXML);
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "After GetFormalParametersAndActualParametersForSubflow() function.");
					if((ErrorXML == null)&&(OutputXML != null))
					{
						xpDocParameter		= new XPathDocument(new StringReader(OutputXML));
						xpNavActivity		= xpDocParameter.CreateNavigator();
						xpNavActivity		= xpNavActivity.SelectSingleNode("ACTIVITYCOLLECTION/ACTIVITY");
						xpNavActualParams	= xpNavActivity.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTUALPARAMETERS");

						xpNavRDF			= xpNavActivity.SelectSingleNode("PROCESS_ACTIVITY/WORKFLOWPROCESS/PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS");
						xpNavFormalParams	= xpNavActivity.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/PROCESS_INFORMATION/WORKFLOWPROCESSINFO/FORMALPARAMETERS");
						xpNavSubProcessRDF	= xpNavActivity.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS");                        
                        str_WorkflowCategory = xpNavActivity.SelectSingleNode("SUBFLOWACTIVITY/WORKFLOWPROCESS/WORKFLOWPROCESS_WORKFLOWINFORMATION/WORKFLOWINFORMATION/WORKFLOWSUBCATEGORY").Value;
						if((xpNavFormalParams != null) && (!String.IsNullOrEmpty(xpNavFormalParams.InnerXml)))
						{
							lstEFObj = new List<EntityFacetCollection>();
							// Select all the formal parameters with mode = OUT/INOUT
							xpNodeIterFormalParam = xpNavFormalParams.Select("PROCESS_FORMALPARAMETERS/FORMALPARAMETER[MODE='IN' or MODE='INOUT']");
							while(xpNodeIterFormalParam.MoveNext())
							{
								xpNavFormalParam	= xpNodeIterFormalParam.Current;								
								FormalParameterID	= xpNavFormalParam.SelectSingleNode("FORMALPARAMETERID").Value;
								strSeqNo			= xpNavFormalParam.SelectSingleNode("SEQUENCENUMBER").Value;
								// Select the Actual parameter with the same sequence number as that of Formal parameter
								xpNavActualParam = xpNavActualParams.SelectSingleNode("ACTUALPARAMETERS/ACTUALPARAMETER[SEQUENCENUMBER='" + strSeqNo + "']");
								if((xpNavActualParam != null) && ((xpNavRDF != null)))
								{
									ActualParameterID = xpNavActualParam.SelectSingleNode("ACTUALPARAMETERID").Value;
									xpNavDataField	  = xpNavRDF.SelectSingleNode("PROCESS_DATAFIELDS/DATAFIELD[DATAFIELDID='" + ActualParameterID + "']");
									if(xpNavDataField != null)
									{
										strDataFieldName	= xpNavDataField.SelectSingleNode("DATAFIELDNAME").Value;
										if (xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE")!=null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE").Value;
										}
										else if(xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE") != null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE").Value;
										}                                        
										PmMap				= new ParameterMap(ActualParameterID, FormalParameterID, strDataFieldName);
										PmMap.DataType		= strDataType;                                       
										xpNav_NavPath		= xpNavDataField.SelectSingleNode("STORAGE/NAVIGATIONPATH");
										if(xpNav_NavPath != null)
										{
											// In this Xpath query the first child::node() will refer to the WORKFLOWPROCESS/ACTIVITY node
											// the second child::node will select the node where the type = relation attribute is found
											// the third child:: node expression returns the facet/entity node having the primary key name and value attributes.
											xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
											if(xpNavTypeName != null)
											{
												// check if there exists one more type='relation' node , if yes then its a series data transfer at the second level.
                                                xpNavTypeName = xpNavTypeName.SelectSingleNode("child::node()[@Type='RELATION']");
                                                if (xpNavTypeName != null)
                                                {
                                                    PmMap.TypeName              = xpNavTypeName.GetAttribute("TypeName", "");
                                                    PmMap.TypePrimaryKeyName    = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    xpNavTypeName.MoveToParent();
                                                    strPrimaryKeyName           = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    strPrimaryKeyValue          = xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
                                                    strType                     = xpNavTypeName.GetAttribute("Type", "");
                                                    strTypeName                 = xpNavTypeName.Name;
                                                    EFObj                       = new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
                                                }
                                                else
                                                {
                                                    xpNavTypeName       = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
                                                    strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName","");
												    strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
												    strType				= xpNavTypeName.GetAttribute("Type","");
												    strTypeName			= xpNavTypeName.Name;
												    EFObj				= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
                                                }
											}
											else // here only the type = relation nodes exist.
											{
												xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']");
												if(xpNavTypeName != null)
												{
													PmMap.TypeName				= xpNavTypeName.GetAttribute("TypeName","");
													PmMap.TypePrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");

													xpNavTypeName				= xpNav_NavPath.SelectSingleNode("child::node()");

													strPrimaryKeyName			= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
													strPrimaryKeyValue			= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
													strType						= xpNavTypeName.GetAttribute("Type","");
													strTypeName					= xpNavTypeName.Name;
													EFObj						= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
												}
												else
												{
													xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()");
													if(xpNavTypeName != null)
													{
														strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName","");
														strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
														strType				= xpNavTypeName.GetAttribute("Type","");
														strTypeName			= xpNavTypeName.Name;
														EFObj				= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
													}
												}
											}
											if(!lstEFObj.Contains(EFObj))
											{
												EFObj.AddDataField(PmMap);
												lstEFObj.Add(EFObj);
											}
											else
											{
												Index = lstEFObj.IndexOf(EFObj);
												lstEFObj[Index].AddDataField(PmMap);
											}
										}
									}
								}
							}
							ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Before GetDataFieldValues() function.");
							ErrorXML = GetDataFieldValues(ref lstEFObj);
							if(ErrorXML == null)
							{

								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "After GetDataFieldValues() function.");
								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Before UpdateProcessDataFieldsOnProcessInitialization() function.");
								if((xpNavSubProcessRDF != null) && (lstEFObj.Count > 0))
                                    ErrorXML = UpdateProcessDataFieldsOnProcessInitialization(lstEFObj, xpNavSubProcessRDF, str_WorkflowCategory, str_Status);
								if (ErrorXML!=null)
								{
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while updating process data fields on process initialization.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("UPDATE_DATAFIELDS_FOR_PROCESS_FAILED", this.GetType().ToString(), "UpdateProcessDataFieldsOnProcessInitialization()", null, ErrorXML);
                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
								}
								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "After UpdateProcessDataFieldsOnProcessInitialization() function.");
							}
							else
							{
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while getting data field values.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("GET_DATAFIELDS_FAILED", this.GetType().ToString(), "TransferDataOnProcessInitialisation()", null, ErrorXML);
                                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
							}
						}

					}
					else if(ErrorXML != null)
					{
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                        strdict_AdditionalInfo.Add("MOREINFO", "Error while getting formal and actual parameters for subflow activity.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_FORMAL_ACTUAL_PARAMETERS_FOR_ACTIVITY_FAILED", this.GetType().ToString(), "TransferDataOnProcessInitialisation()", null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
					}
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "End TransferDataOnProcessInitialisation() function.");
					return ErrorXML;
				}
				catch(Exception ExcepObj)
				{
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                    strdict_AdditionalInfo.Add("MOREINFO", "Exception while transfering data on process initialisation.");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("PROCESS_DATA_TRANSFER_FAILED", this.GetType().ToString(), "TransferDataOnProcessInitialisation()", ExcepObj, null);
                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    ExceptionTracing.LogError(TenantName, ErrorXML);
                    return ErrorXML;
				}
			}

            /// <alsummary>
			/// <createddate>4 Aug 08</createddate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This purpose of this method is to replace the RDF XML with the instance ID's of the process,
			/// Activity and their related facets</purpose>
			/// </alsummary>
			/// <param name="ProcessInstanceNode">The process Instance node will contain 
			/// all the associated instances which will be used to update the RDF XML</param>
			/// <returns>Returns the RDF XML replaces with the instance ID's of process, activity and/or related facets.
			///	If No RDF are present returns null.</returns>
            public XML GetRDFXMLWithUpdatedPrimaryKeyValues(XmlNode ProcessInstanceNode)
			{				
				XmlNode				RdfNode						= null;
				XmlNode				TypeNameNode				= null;				
				XmlNodeList			DatFieldNodeList			= null;
				XmlNodeList			PrimarykeyList				= null;
				XmlNodeList			ChildPrimaryKeyList			= null;
				String				PrimarykeyValue				= null;
				String				DefID						= null;
                ALError             ExcepErrObj                 = null;
                StringBuilder       sb_InputData                = null;
                ERRORXML            ErrorXml                    = null; 

				try
				{
					RdfNode = ProcessInstanceNode.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS[PROCESS_DATAFIELDS]");
					if(RdfNode != null)
					{
						// select all the data fields node where storage is in database.
						DatFieldNodeList = RdfNode.SelectNodes("PROCESS_DATAFIELDS/DATAFIELD[STORAGE/INDATABASE='True']");
						foreach(XmlNode DataFieldNode in DatFieldNodeList)
						{
							PrimarykeyList = DataFieldNode.SelectNodes("STORAGE/NAVIGATIONPATH/child::node()[@PrimaryKeyName]");
							foreach(XmlNode PKNode in PrimarykeyList)
							{
								// Check if the Primary Key node is WORKFLOWPROCESS or ACTIVITY

								switch(PKNode.Name.ToUpper())
								{
									case "ACTIVITY":
										// read the the Activity definition ID contained in the attribute PrimaryKeyvalue and 
										// replace it with InstanceID by reading the correspnding definition from the instance XML
										DefID			= ((XmlElement)PKNode).GetAttribute("PrimaryKeyValue","");
										TypeNameNode	=  ProcessInstanceNode.SelectSingleNode("PROCESS_ACTIVITY/ACTIVITY[ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTIVITYDEFINITIONID='" + DefID + "']");
										if(TypeNameNode != null)
										{
											PrimarykeyValue		= TypeNameNode.SelectSingleNode("ACTIVITYID").InnerText;
											((XmlElement)PKNode).SetAttribute("PrimaryKeyValue", PrimarykeyValue);
											// Select the further child nodes to set the primary key.
											ChildPrimaryKeyList = PKNode.SelectNodes("child::node()[@Type='RELATION']/child::node()[@PrimaryKeyName]");											
											UpdateChildNodePrimaryKeys(ChildPrimaryKeyList, ref TypeNameNode);
										}

										
										break;

									case "WORKFLOWPROCESS":
										// read the the Workflowprocess definition ID contained in the attribute PrimaryKeyvalue and 
										// replace it with InstanceID by reading the correspnding definition from the instance XML
										DefID			= ((XmlElement)PKNode).GetAttribute("PrimaryKeyValue","");
																	
										PrimarykeyValue	= ProcessInstanceNode.SelectSingleNode("WORKFLOWPROCESSID").InnerText;
										((XmlElement)PKNode).SetAttribute("PrimaryKeyValue", PrimarykeyValue);
										// Select the further child nodes to set the primary key.
										ChildPrimaryKeyList = PKNode.SelectNodes("child::node()[@Type='RELATION']/child::node()[@PrimaryKeyName]");
										UpdateChildNodePrimaryKeys(ChildPrimaryKeyList, ref ProcessInstanceNode);
										break;									

									default:
										// replace directly the instance ID by searching in the instance XML
										if(ProcessInstanceNode.SelectSingleNode("descendant::" + PKNode.Name)  != null)
										{
											TypeNameNode	= ProcessInstanceNode.SelectSingleNode("descendant::" +  ((XmlElement)PKNode).GetAttribute("PrimaryKeyName"));
											if(TypeNameNode != null)
											{
												PrimarykeyValue = TypeNameNode.InnerText;
												((XmlElement)PKNode).SetAttribute("PrimaryKeyValue", PrimarykeyValue);	
											}
										}
										break;
								}
							}
						}
						return RdfNode.SelectSingleNode("PROCESS_DATAFIELDS").OuterXml;
					}
					return null;
				}
				catch(Exception ExcepObj)
				{
                    sb_InputData = new StringBuilder(10);
                    sb_InputData.Append("Process XML = " + ProcessInstanceNode.OuterXml + "");
                    ExcepErrObj = new ALError(ErrorCodes.ACTIVITY_MODIFICATION_FAILED,
                                                 "Exception in GetRDFXMLWithUpdatedPrimaryKeyValues().", ErrorObjectType.ACTIVITY,
                                                 "Entity", ErrorType.EXCEPTION, ExcepObj, sb_InputData.ToString());
                    ErrorXml = ExcepErrObj.Serialize();
                    ExceptionTracing.LogError(TenantName, ErrorXml);
                    return ErrorXml;
				}
			}

            /// <summary>
            /// <Author>Ms. Radha A</Author>
            /// <CreatedOnDate>14-05-2009</CreatedOnDate>
            /// <Purpose>This API is used to get the predefined section path for the process.</Purpose>
            /// </summary>
            /// <param name="str_ProcessID">The process id </param>
            /// <param name="str_PackageDefID">Package definition id</param>
            /// <param name="str_ProcessDefID">Process definition id</param>
            /// <param name="str_FixedSectionPath">Fixed section path</param>
            /// <param name="str_FixedSectionID">Fixed section path id </param>
            /// <param name="str_FinalSectionPath">Final section path</param>
            /// <param name="str_FinalSectionID">Final predefined section path id</param>
            /// <returns>On success method will return NULL else returns an error XML.</returns>
            public ERRORXML GetPredefinedSectionForProcess(String str_ProcessID, String str_PackageDefID, String str_ProcessDefID, out String str_FixedSectionPath, out String str_FixedSectionID, out String str_FinalSectionPath, out String str_FinalSectionID)
            {
                ERRORXML            ErrorXml                 = null;
                String              str_ProcessDefinitionXml = null;
                String              str_VariableSectionPath  = null;
                ulong               ul_SectionID             = 0;
                String              str_ProcessDataXml       = null;
                String              str_NavigationPath       = null;
                String              str_VaultName            = null;
                XPathDocument       xpathDoc_ProcessDefXml   = null;
                XPathNodeIterator   xpathNodeIter_Sections   = null;
                XPathDocument       xpathDoc_ProcessXml      = null;
                XPathNavigator      xpathNav_Section         = null;
                StringBuilder       sb_SearchXML             = null;
                OMInterface         OmIObj                   = null;
                DigitalAsset	    DigitalAssetObj			 = null;
                Boolean             b_VaultExists            = false;
                Section             SectionObj               = null;
                XPathNavigator      xpathNav_Process         = null;
                String              str_SectionValue         = null;
                XPathNavigator      nav_SectionID            = null;
                ALEngineError       ALEngineErrorObj         = null;
                String              str_SolutionName         = null;
                
                StringDictionary    strdict_AdditionalInfo   = null;

                try
                {
                    str_FixedSectionPath    = null;
                    str_FixedSectionID      = null;
                    str_FinalSectionPath    = null;
                    str_FinalSectionID      = null;
                    sb_SearchXML            = new StringBuilder(500);
                    OmIObj			        = new OMInterface(UserID, CurrentLanguage, TenantName);
                    DigitalAssetObj         = new DigitalAsset(UserID, CurrentLanguage, TenantName);
                    SectionObj              = new Section(UserID, CurrentLanguage, TenantName);

                    if ((String.IsNullOrEmpty(str_PackageDefID)) && (String.IsNullOrEmpty(str_ProcessDefID)))
                    {
                        ErrorXml = GetProcessInformation(str_ProcessID, out str_PackageDefID, out str_ProcessDefID, out str_SolutionName);                        
                    }

                    if ((!String.IsNullOrEmpty(str_PackageDefID)) && (!String.IsNullOrEmpty(str_ProcessDefID)))
                    {
                        ErrorXml    = GetProcessDefinitionByID(Convert.ToUInt64(str_PackageDefID), str_ProcessDefID, out str_ProcessDefinitionXml);
                        if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_ProcessDefinitionXml)))
                        {
                            xpathDoc_ProcessDefXml = new XPathDocument(new StringReader(str_ProcessDefinitionXml));
                            if (xpathDoc_ProcessDefXml.CreateNavigator().SelectSingleNode("WORKFLOWPROCESS/TARGETVAULT/FIXEDPATH/SECTIONPATH") != null)
                            {
                                str_FixedSectionPath = xpathDoc_ProcessDefXml.CreateNavigator().SelectSingleNode("WORKFLOWPROCESS/TARGETVAULT/FIXEDPATH/SECTIONPATH").Value;
                                if (!String.IsNullOrEmpty(str_FixedSectionPath))
                                {
                                    if (str_FixedSectionPath.IndexOf(" > ") > 0)
                                    {
                                        str_VaultName = str_FixedSectionPath.Substring(0, str_FixedSectionPath.IndexOf(" > "));
                                    }
                                    else
                                    {
                                        str_VaultName = str_FixedSectionPath.Trim();
                                    }
                                    str_VaultName = ALCommon.TrimAndReplaceSpecialChars(str_VaultName);
                                    b_VaultExists = IsVaultExists(str_VaultName);

                                    if (b_VaultExists)
                                    {
                                        nav_SectionID = xpathDoc_ProcessDefXml.CreateNavigator().SelectSingleNode("WORKFLOWPROCESS/TARGETVAULT/FIXEDPATH/SECTIONID");
                                        if (nav_SectionID != null)
                                        {
                                            str_FixedSectionID = nav_SectionID.Value;
                                            str_FinalSectionID = str_FixedSectionID;
                                        }
                                        str_FixedSectionPath = str_FixedSectionPath.Replace(" > ", "\\");
                                        str_FinalSectionPath = str_FixedSectionPath;
                                        //For Variable path
                                        xpathNodeIter_Sections = xpathDoc_ProcessDefXml.CreateNavigator().Select("WORKFLOWPROCESS/TARGETVAULT/VARIABLEPATH/SECTIONPATH/SECTION");
                                        if (xpathNodeIter_Sections.Count > 0)
                                        {
                                            if (!String.IsNullOrEmpty(str_FixedSectionPath))
                                            {
                                                str_VariableSectionPath = str_FixedSectionPath + "\\";
                                            }
                                            ErrorXml    = GetProcessData(str_ProcessID, str_PackageDefID, str_ProcessDefID, str_ProcessDefinitionXml, out str_ProcessDataXml);
                                            if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_ProcessDataXml)))
                                            {
                                                xpathDoc_ProcessXml = new XPathDocument(new StringReader(str_ProcessDataXml));
                                                xpathNav_Process = xpathDoc_ProcessXml.CreateNavigator().SelectSingleNode("WORKFLOWPROCESS");
                                                while (xpathNodeIter_Sections.MoveNext())
                                                {
                                                    str_NavigationPath = xpathNodeIter_Sections.Current.SelectSingleNode("NAVIGATIONPATH").Value;
                                                    if (!String.IsNullOrEmpty(str_NavigationPath))
                                                    {
                                                        xpathNav_Section = xpathNav_Process.SelectSingleNode(str_NavigationPath);
                                                        if ((xpathNav_Section != null) && (!String.IsNullOrEmpty(xpathNav_Section.Value)))
                                                        {
                                                            str_SectionValue = xpathNav_Section.Value;
                                                            str_VariableSectionPath += str_SectionValue + "\\";
                                                        }
                                                    }
                                                }
                                                str_VariableSectionPath = str_VariableSectionPath.Remove(str_VariableSectionPath.LastIndexOf("\\"));
                                                ErrorXml = SectionObj.CreateSectionsOnPath(str_VariableSectionPath, out ul_SectionID);
                                                if (String.IsNullOrEmpty(ErrorXml))
                                                {
                                                    str_FinalSectionID      = ul_SectionID.ToString();
                                                    str_FinalSectionPath    = str_VariableSectionPath;
                                                }
                                                else if (!String.IsNullOrEmpty(ErrorXml))
                                                {
                                                    strdict_AdditionalInfo = new StringDictionary();
                                                    strdict_AdditionalInfo.Add("VARIABLESECTIONPATH", str_VariableSectionPath);
                                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while creating section on given path.");

                                                    ALEngineErrorObj = new ALEngineError();
                                                    ALEngineErrorObj.AppendError("CREATE_SECTIONS_ON_PATH_FAILED", this.GetType().ToString(), "GetPredefinedSectionForProcess()", null, ErrorXml);
                                                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                                    return ErrorXml;
                                                }
                                            }
                                            else if (!String.IsNullOrEmpty(ErrorXml))
                                            {
                                                strdict_AdditionalInfo = new StringDictionary();
                                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                                                strdict_AdditionalInfo.Add("MOREINFO", "Error while getting process data.");

                                                ALEngineErrorObj = new ALEngineError();
                                                ALEngineErrorObj.AppendError("GET_PROCESSDATA_FAILED", this.GetType().ToString(), "GetPredefinedSectionForProcess()", null, ErrorXml);
                                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                                return ErrorXml;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else if (!String.IsNullOrEmpty(ErrorXml))
                        {
                            strdict_AdditionalInfo = new StringDictionary();
                            strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                            strdict_AdditionalInfo.Add("PROCESSDEFINITIONID", str_ProcessDefID);
                            strdict_AdditionalInfo.Add("PACKAGEDEFINITIONID", str_PackageDefID);
                            strdict_AdditionalInfo.Add("MOREINFO", "Error while getting process definition by id.");

                            ALEngineErrorObj = new ALEngineError();
                            ALEngineErrorObj.AppendError("GET_PROCESSDEFINITIONBYID_FAILED", this.GetType().ToString(), "GetPredefinedSectionForProcess()", null, ErrorXml);
                            ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                            return ErrorXml;
                        }
                    }
                    return ErrorXml;
                }
                catch (Exception ExceptionObj)
                {
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                    strdict_AdditionalInfo.Add("PROCESSDEFINITIONID", str_ProcessDefID);
                    strdict_AdditionalInfo.Add("PACKAGEDEFINITIONID", str_PackageDefID);
                    strdict_AdditionalInfo.Add("MOREINFO", "Exception while getting predefined section for process.");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("GET_PROCESSDEFINITIONBYID_FAILED", this.GetType().ToString(), "GetPredefinedSectionForProcess()", ExceptionObj, null);
                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                    ExceptionTracing.LogError(TenantName, ErrorXml);
                    str_FixedSectionPath    = null;
                    str_FixedSectionID      = null;
                    str_FinalSectionPath    = null;
                    str_FinalSectionID      = null;
                    return ErrorXml;
                }
            }
            
            /// <alsummary>
            /// <Author>Mrs. Mugdha Moolay Gadre</Author>
            /// <Purpose>This method creates a job using Message Q to process files for Video Approvals</Purpose>
            /// </alsummary>
            /// <param name="str_ProcessID"></param>
            /// <returns></returns>
            public ERRORXML CreateMediaProcessorJob(String str_ProcessID)
            {
                const String            SolutionName                    = "Approvals";
                ulong                   ul_JobID                        = 0;
                ulong                   ul_RequestID                    = 0;
                String                  str_FileName                    = null;
                String                  str_FileDisplayName             = null;
                String                  str_FileNameWithoutExt          = null;
                String                  str_FileExtension               = null;
                String                  ErrorXML                        = null;
                String                  str_JobPath                     = null;
                String                  str_ApplicationType             = String.Empty;
                String                  str_ProcessXML                  = null;
                String                  str_FileFormatDetials           = null;
                StringBuilder           sb_OutputFilePath               = null;
                StringBuilder           sb_InputXML                     = null;
                StringBuilder           sb_ExtraInformation             = null;
                StringBuilder           sb_SourceFilePath               = null ;
                StringBuilder           sb_AttributesXML                = null;
                StringBuilder           sb_AssociationXML               = null;
                MediaRequestRouter      obj_MediaRequestRouter          = null;
                XPathDocument           xpDoc_Process                   = null;
                XPathNavigator          xpNav_Process                   = null;
                XPathNavigator          xpNav_FileName                  = null;
                XPathNavigator          xpNav_TopicID                   = null;

                StringDictionary        strdict_AdditionalInfo          = null;
                ALEngineError           ALEngineErrorObj                = null;

                try
                {   
                    sb_OutputFilePath       = new StringBuilder(100);
                    sb_SourceFilePath       = new StringBuilder(100);
                    obj_MediaRequestRouter  = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), CurrentLanguage, SolutionName);
                    sb_AttributesXML        = new StringBuilder(210);
                    sb_AttributesXML.Append(@"<WORKFLOWPROCESS>
	                                        <WORKFLOWPROCESSID/>
                                                <PROCESSFILE/>
                                            </WORKFLOWPROCESS>");
                    sb_AssociationXML = new StringBuilder(320);
                    sb_AssociationXML.Append(@"<ASSOCIATION>
                                                    <WORKFLOWPROCESS_MEDIATOPIC Type='RELATION'>
                                                        <TOPICID/>
                                                    </WORKFLOWPROCESS_MEDIATOPIC>
                                                </ASSOCIATION>");
                    ErrorXML = GetProcess(Convert.ToUInt64(str_ProcessID), sb_AssociationXML.ToString(), sb_AttributesXML.ToString(), out str_ProcessXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                        strdict_AdditionalInfo.Add("ASSOCIATIONXML", sb_AssociationXML.ToString());
                        strdict_AdditionalInfo.Add("ATTRIBUTESXML", sb_AttributesXML.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error while getting process to create media processor job.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("PROCESS_READ_FAILED", this.GetType().ToString(),
                                                        "CreateMediaProcessorJob()", null, strdict_AdditionalInfo,
                                                        null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXML;
                    }
                    xpDoc_Process = new XPathDocument(new StringReader(str_ProcessXML));
                    xpNav_Process = xpDoc_Process.CreateNavigator();

                    //Check file type
                    xpNav_FileName = xpNav_Process.SelectSingleNode("WORKFLOWPROCESS/FILESYSTEM/FILE/NAME");
                    if (xpNav_FileName != null)
                    {
                        str_FileName = xpNav_FileName.Value;
                        str_FileExtension = str_FileName.Substring(str_FileName.LastIndexOf(".") + 1);
                        if (str_FileExtension.ToUpper() == "JPE" || str_FileExtension.ToUpper() == "JPG")
                            str_FileExtension = "JPEG";
                        if (str_FileExtension.ToUpper() == "TIF")
                            str_FileExtension = "TIFF";

                        str_FileDisplayName = xpNav_Process.SelectSingleNode("WORKFLOWPROCESS/FILESYSTEM/FILE/DISPLAYNAME").Value;
                    }
                    else
                        return null;

                    try
                    {   
                        if (str_FileExtension.ToUpper() != "PDF")
                        {
	                        if (!obj_MediaRequestRouter.IsMediaProcessorRequired(str_FileExtension, out str_ApplicationType, out str_FileFormatDetials, out str_JobPath))
	                        {   
	                            //if topic is already created and file of approval is not of type media file then delete topic
	                            xpNav_TopicID = xpNav_Process.SelectSingleNode("WORKFLOWPROCESS/WORKFLOWPROCESS_MEDIATOPIC/TOPIC/TOPICID");
	                            if (xpNav_TopicID != null)
	                            {
	                                String str_TopicXML             = null;
	                                MessageBoard obj_MessageBoard   = new MessageBoard(UserID, CurrentLanguage, TenantName);
	                                StringBuilder sb_TopicXML       = new StringBuilder(50);
	                                sb_TopicXML.AppendFormat("<TOPIC><TOPICID>{0}</TOPICID></TOPIC>", xpNav_TopicID.Value);
	                                str_TopicXML                    = sb_TopicXML.ToString();
	                                ErrorXML                        = obj_MessageBoard.DeleteTopic(str_TopicXML,null);
	                                if (!String.IsNullOrEmpty(ErrorXML))
	                                {
	                                    strdict_AdditionalInfo = new StringDictionary();
	                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
	                                    strdict_AdditionalInfo.Add("TOPICID", xpNav_TopicID.Value);
	                                    strdict_AdditionalInfo.Add("INPUTXML", str_TopicXML);
	                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while deleting topic.");
	
	                                    ALEngineErrorObj = new ALEngineError();
	                                    ALEngineErrorObj.AppendError("TOPIC_DELETE_FAILED", this.GetType().ToString(),
	                                                                    "CreateMediaProcessorJob()", null, strdict_AdditionalInfo,
	                                                                    null, ErrorXML);
	                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
	                                    return ErrorXML;
	                                }
	
	                            }
	                            return null;
	                        }
	                     }

                        if (str_FileExtension.ToUpper() == "PDF")
                            str_ApplicationType = "pdf2swf";
                    }
                    catch(ApplicationException InvalidFormat)
                    {
                        InvalidFormat.ToString();
                        xpNav_TopicID = xpNav_Process.SelectSingleNode("WORKFLOWPROCESS/WORKFLOWPROCESS_MEDIATOPIC/TOPIC/TOPICID");
                        if (xpNav_TopicID != null)
                        {
                            String str_TopicXML = null;
                            MessageBoard obj_MessageBoard   = new MessageBoard(UserID, CurrentLanguage, TenantName);
                            StringBuilder sb_TopicXML       = new StringBuilder(50);
                            sb_TopicXML.AppendFormat("<TOPIC><TOPICID>{0}</TOPICID></TOPIC>", xpNav_TopicID.Value);
                            str_TopicXML                    = sb_TopicXML.ToString();
                            ErrorXML                        = obj_MessageBoard.DeleteTopic(str_TopicXML,null);
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                                strdict_AdditionalInfo.Add("TOPICID", xpNav_TopicID.Value);
                                strdict_AdditionalInfo.Add("INPUTXML", str_TopicXML);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while deleting topic.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("TOPIC_DELETE_FAILED", this.GetType().ToString(),
                                                                "CreateMediaProcessorJob()", null, strdict_AdditionalInfo,
                                                                null, ErrorXML);
                                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXML;
                            }
                        }
                        return null;
                    }

                    obj_MediaRequestRouter.GetJobPath(str_ApplicationType, out str_JobPath);
                    ALCommon.GenerateUniqueID(ref  ul_JobID);
                    sb_OutputFilePath.AppendFormat("{0}\\{1}_{2}\\{3}", str_JobPath,SessionID.ToString(),UserID.ToString(),ul_JobID);
                    Directory.CreateDirectory(sb_OutputFilePath.ToString());
                    str_JobPath = sb_OutputFilePath.ToString();
                    //HemantK: Defect ID S0801897
                    //This change has been done to handle special characters in file name
                    sb_SourceFilePath.AppendFormat("{0}\\{1}.{2}", sb_OutputFilePath, str_ProcessID, str_FileExtension);

                    xpNav_FileName.MoveToParent(); //FILE
                    xpNav_FileName.MoveToParent();//FILESYSTEM

                    ErrorXML = GetFile(xpNav_FileName.OuterXml, "PROCESSFILE", sb_SourceFilePath.ToString(), "");
                    if (ErrorXML != null)
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                        strdict_AdditionalInfo.Add("FILEXML", xpNav_FileName.OuterXml);
                        strdict_AdditionalInfo.Add("DOWNLOADFILEPATH", sb_SourceFilePath.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error while getting process file.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_FILE_FAILED", this.GetType().ToString(),
                                                        "CreateMediaProcessorJob()", null, strdict_AdditionalInfo,
                                                        null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXML;
                    }
                    
                    sb_InputXML = new StringBuilder();

                    //HemantK: Defect ID S0801897
                    //This change has been done to handle special characters in file name
                    str_FileNameWithoutExt = str_ProcessID;
                    if (str_ApplicationType.ToUpper() != "IMAGEPROCESSOR")
                    {
                        if (str_ApplicationType.ToUpper() == "PDF2SWF")
                        {
                            sb_OutputFilePath.AppendFormat("\\{0}.swf", str_FileNameWithoutExt);
                            sb_InputXML.AppendFormat(@"<FILECONVERTER>
	                                            <INPUT FILENAME='{0}' MEDIAGENERATOR='pdf2swf.exe'  EVENT='CONVERTPDFTOSWF'/>
                                                <INPUTFILENAME>{3}</INPUTFILENAME>
                                                <INPUTFILDISPLAYENAME>{4}</INPUTFILDISPLAYENAME>
	                                            <OUTPUTLIST><OUTPUT>
	                                            <FILEFORMAT FILENAME='{2}' DESTINATIONFILENAME='{2}' MEDIAGENERATORPARAM='-T9 -f -t' MEDIAGENERATOR='pdf2swf.exe' EVENT='CONVERTPDFTOSWF'>SWF</FILEFORMAT>
                                                </OUTPUT></OUTPUTLIST>
                                          </FILECONVERTER>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()), ALCommon.TrimAndReplaceSpecialChars(str_FileName), ALCommon.TrimAndReplaceSpecialChars(str_FileDisplayName));
                        }
                        else
                        {
                            sb_OutputFilePath.AppendFormat("\\{0}.flv", str_FileNameWithoutExt);
                            sb_InputXML.AppendFormat(@"<REQUEST>
		                                            <SOURCEFILE>{0}</SOURCEFILE>
		                                            <TENANTNAME>{1}</TENANTNAME>
		                                            <OUTPUTFILE>{2}</OUTPUTFILE>
		                                            <JOBPATH>{3}</JOBPATH>
		                                            <VIDEOWIDTH>0</VIDEOWIDTH>
		                                            <VIDEOHEIGHT>0</VIDEOHEIGHT>
		                                            <FLVGENERATOR>MEncoder.exe</FLVGENERATOR>
		                                            <LOGPATH>{4}</LOGPATH>
		                                            <FLVGENERATORPARAM>-mc 0 -af volnorm -ofps 25 -srate 44100 -oac mp3lame -lameopts abr:br=64 -ovc lavc -of lavf -lavfopts format=flv -lavcopts vcodec=flv:vbitrate=512:mbd=2:mv0:trell:v4mv:cbp:last_pred=3 -vf harddup</FLVGENERATORPARAM>
	                                          </REQUEST>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()), ALCommon.TrimAndReplaceSpecialChars(str_JobPath), ALCommon.TrimAndReplaceSpecialChars(str_JobPath));
                        }
                        
                    }
                    else
                    {  
                        sb_OutputFilePath.AppendFormat("\\{0}.jpg", str_FileNameWithoutExt);
                        sb_InputXML.AppendFormat(@"<FILECONVERTER>
	                                            <INPUT FILENAME='{0}' EVENT='GENERATEPREVIEWFILE'/>
	                                            <OUTPUTLIST><OUTPUT>
	                                            <FILEFORMAT FILENAME='{2}' FILEFORMAT='JPG' QUALITY='0' DPI='72' DEPTH='24' DESTINATIONFILENAME='{2}' EVENT='GeneratePreviewFile'>JPG</FILEFORMAT>
                                                </OUTPUT></OUTPUTLIST>
                                          </FILECONVERTER>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()));//, ALCommon.TrimAndReplaceSpecialChars(str_JobPath), ALCommon.TrimAndReplaceSpecialChars(str_JobPath));
                    }

                    sb_ExtraInformation = new StringBuilder();
                    sb_ExtraInformation.AppendFormat(@"<JOBDETAILS TYPE=""MEDIA TOPIC CREATION"">
		                                                        <WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>
		                                                        <JOBINITATORID>{1}</JOBINITATORID>
                                                                <JOBID>{2}</JOBID>
		                                                        <JOBTYPE>MEDIA TOPIC CREATION</JOBTYPE>                                                                
	                                                        </JOBDETAILS>", str_ProcessID, UserID.ToString(), ul_JobID.ToString());
                    ALCommon.GenerateUniqueID(ref ul_RequestID);
                    ErrorXML = obj_MediaRequestRouter.SendRequestToMediaProcessor(sb_InputXML.ToString(), str_ApplicationType, ul_RequestID.ToString(), sb_ExtraInformation.ToString());
                    if (ErrorXML != null)
                    {
                        return ErrorXML;
                    }
                }
                catch (Exception obj_Exception)
                {
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                    strdict_AdditionalInfo.Add("MOREINFO", "Exception while creating media processor job.");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                    "CreateMediaProcessorJob()", null, strdict_AdditionalInfo,
                                                    obj_Exception, null);
                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    ExceptionTracing.LogError(TenantName, ErrorXML);
                    return ErrorXML;
                }
                return null;
            }

            /// <summary>
            /// <Author>Parita Harniya</Author>
            /// <Purpose>This API is used to generate FLVs for video files.</Purpose>
            /// <CreatedOn>18th June 2010</CreatedOn>
            /// <param name="str_ProcessID">Workflow Process ID</param>
            /// <param name="str_ProcessReviewDetails">Process Review Details containing files for approvals</param>
            /// </summary>        
            /// <returns>Error XML if error occurs else null</returns>
            public ERRORXML GenerateFLVs(String str_ProcessID, String str_ProcessReviewDetails)
            {
                ERRORXML            ErrorXML                    = null;
                XElement            xEle_ProcessReviewDetails   = null;
                ALError             ExcepErrObj                 = null;
                StringBuilder       sb_InputData                = null;
                StringBuilder       sb_TempFilePath             = null; 
                ALEngineError       ALEngineErrorObj            = null; 
                StringDictionary    strdict_AdditionalInfo      = null;

                try
                {
                    //Check whether process review details are empty
                    //if yes then get the media files for the Multi-File
                    if (String.IsNullOrEmpty(str_ProcessReviewDetails))
                        ErrorXML = GetMediaFiles(str_ProcessID, out str_ProcessReviewDetails);

                    if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(str_ProcessReviewDetails))
                    {
                        //Save this ProcessReviewDetails in temp location so that it will be used by AL_ApprovalsComponent
                        sb_TempFilePath = new StringBuilder(50);
                        sb_TempFilePath.Append(ALCommon.GetTempPath() + "\\" + str_ProcessID + "\\MF_MediaFiles_" + str_ProcessID + ".xml");
                        if (File.Exists(sb_TempFilePath.ToString()))
                            File.Delete(sb_TempFilePath.ToString());

                        Directory.CreateDirectory(ALCommon.GetTempPath() + "\\" + str_ProcessID);
                        File.WriteAllText(sb_TempFilePath.ToString(), str_ProcessReviewDetails);

                        xEle_ProcessReviewDetails = XElement.Load(XmlReader.Create(new StringReader(str_ProcessReviewDetails)));
                        foreach (XElement xEle_ReviewDetails in xEle_ProcessReviewDetails.XPathSelectElements("PROCESSREVIEWDETAILS"))
                        {
                            ErrorXML = CreateMediaProcessorJob_MultiFile(str_ProcessID, xEle_ReviewDetails, sb_TempFilePath.ToString());

                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while creating media processor job for multi-file.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("GET_MEDIAFILES_FAILED", this.GetType().ToString(), "GenerateFLVs()", null, ErrorXML);
                                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                            }
                        }
                    }
                    else if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                        strdict_AdditionalInfo.Add("MOREINFO", "Error while getting media files.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_MEDIAFILES_FAILED", this.GetType().ToString(), "GenerateFLVs()", null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    }
                    return ErrorXML;
                }
                catch (Exception obj_Exception)
                {
                    sb_InputData = new StringBuilder(10);
                    sb_InputData.Append("ProcessID = " + str_ProcessID + "");
                    ExcepErrObj = new ALError(ErrorCodes.UPDATE_FAILED,
                                                 "Exception while generating FLVs.", ErrorObjectType.PROCESS,
                                                 "Entity", ErrorType.EXCEPTION, obj_Exception, sb_InputData.ToString());
                    ErrorXML = ExcepErrObj.Serialize();
                    ExceptionTracing.LogError(TenantName, ErrorXML);
                    return ErrorXML;
                }
            }

            /// <summary>
            /// <CreatedOn>13 January 2012</CreatedOn>
            /// <Author>Mr. Hemant Karkande</Author>
            /// <Purpose>To fetch media files for the multi-file route</Purpose>
            /// </summary>
            /// <param name="str_ProcessID">Workflow Process id</param>
            /// <param name="str_ProcessReviewDetails">Media files xml going out</param>
            /// <returns>ErrorXML if any</returns>
            private ERRORXML GetMediaFiles(String str_ProcessID, out String str_ProcessReviewDetails)
            {
                ERRORXML        ErrorXML            = null;
                StringBuilder   sb_SearchInputXML   = null; 
                String          str_OutputXML       = null; 
                OMInterface     OMIObj              = null;
                XElement        xEle_Process        = null;
                XElement        xEle_File           = null;
                XElement        xEle_ReviewDetails  = null;
                String          str_MediaType       = null; 

                str_ProcessReviewDetails = null;

                sb_SearchInputXML = new StringBuilder(200);
 
                //Get the ProcessReviewDetails for the given processid
                sb_SearchInputXML.Append(@"<SEARCH>
	                                            <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                            <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <SELECTLIST>
													    <EXPRESSION>
			                                                <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
			                                                <ALIAS>WORKFLOWPROCESSID</ALIAS>
		                                                </EXPRESSION>
	                                                </SELECTLIST>
													<RELATIONSHIP NAME='WORKFLOWPROCESS_PROCESSREVIEWDETAILS'>
	                                                    <SELECTLIST>
															<EXPRESSION>
			                                                    <FIELDNAME>FILEFORAPPROVAL</FIELDNAME>
			                                                    <ALIAS>FILEFORAPPROVAL</ALIAS>
		                                                    </EXPRESSION>	
														    <EXPRESSION>
			                                                    <FIELDNAME>STATUS</FIELDNAME>
			                                                    <ALIAS>STATUS</ALIAS>
		                                                    </EXPRESSION>
                                                            <EXPRESSION>
			                                                    <FIELDNAME>ANNOTATIONMETHOD</FIELDNAME>
			                                                    <ALIAS>ANNOTATIONMETHOD</ALIAS>
		                                                    </EXPRESSION>
	                                                    </SELECTLIST>
		                                            </RELATIONSHIP>													
		                                            <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
				                                            <OPERATOR>EQUALTO</OPERATOR>
				                                            <EXPRESSIONVALUE>" + str_ProcessID + @"</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            <FILTERID>1</FILTERID>
			                                            </FILTER>
		                                            </FILTERLIST>
	                                            </TARGETENTITY>
                                            </SEARCH>");

                OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sb_SearchInputXML.ToString(), ref str_OutputXML);
                if (String.IsNullOrEmpty(ErrorXML) && !String.IsNullOrEmpty(str_OutputXML))
                {
                    xEle_Process = XElement.Load(XmlReader.Create(new StringReader(str_OutputXML)));
                    xEle_ReviewDetails = new XElement("WORKFLOWPROCESS_PROCESSREVIEWDETAILS", new XAttribute("TotalFileCount", xEle_Process.XPathSelectElements("WORKFLOWPROCESS/WORKFLOWPROCESS_PROCESSREVIEWDETAILS/PROCESSREVIEWDETAILS").Count()));
                    foreach (XElement xEle_ProcessReviewDetails in xEle_Process.XPathSelectElements("WORKFLOWPROCESS/WORKFLOWPROCESS_PROCESSREVIEWDETAILS/PROCESSREVIEWDETAILS"))
                    {
                        xEle_File = xEle_ProcessReviewDetails.XPathSelectElement("FILESYSTEM/FILE[FIELDNAME='FILEFORAPPROVAL']/NAME");

                        if (ALCommon.IsMediaFile(TenantName, xEle_File.Value, "Approvals", ref str_MediaType))
                            xEle_ReviewDetails.Add(xEle_ProcessReviewDetails);
                    }
                    str_ProcessReviewDetails = xEle_ReviewDetails.ToString(); 
                }
                return ErrorXML;
            }

            /// <summary>
            /// <Author>Ms. Parita Harniya</Author>
            /// <Purpose>This method creates a job using Message Q to process multiple files for Video Approvals</Purpose>
            /// <CreatedOn>18th June 2010</CreatedOn>
            /// </summary>
            /// <param name="str_ProcessID">Workflow Process ID</param>
            /// <param name="xEle_ProcessDetails">Workflow process review details</param>
            /// <param name="str_TempFilePath">Temp file path which has details about all the media files</param>
            /// <returns>ErrorXML</returns>
            public ERRORXML CreateMediaProcessorJob_MultiFile(String str_ProcessID, XElement xEle_ProcessDetails, String str_TempFilePath)
            {
                const String            SolutionName                    = "Approvals";
                ulong                   ul_JobID                        = 0;
                ulong                   ul_RequestID                    = 0;
                String                  str_FileName                    = null;
                String                  str_FileDisplayName             = null;
                String                  str_FileExtension               = null;                
                String                  str_FileNameWithoutExt          = null;
                String                  ErrorXML                        = null;
                String                  str_JobPath                     = null;
                String                  str_ApplicationType             = String.Empty;
                String                  str_FileFormatDetials           = null;   
                StringBuilder           sb_OutputFilePath               = null;
                StringBuilder           sb_InputXML                     = null;
                StringBuilder           sb_ExtraInformation             = null;
                StringBuilder           sb_SourceFilePath               = null ;
                MediaRequestRouter      obj_MediaRequestRouter          = null;
                ALError                 ExcepErrObj                     = null;
                StringBuilder           sb_InputData                    = null;

                XElement                xEle_FileName                   = null;

                try
                {
                    obj_MediaRequestRouter  = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), CurrentLanguage, SolutionName);
                        
                    xEle_FileName = xEle_ProcessDetails.XPathSelectElement("FILESYSTEM/FILE/NAME");
                    if (xEle_FileName != null)
                    {
                        str_FileName = xEle_FileName.Value;
                        str_FileExtension = str_FileName.Substring(str_FileName.LastIndexOf(".") + 1);
                        if (str_FileExtension.ToUpper() == "JPE" || str_FileExtension.ToUpper() == "JPG")
                            str_FileExtension = "JPEG";
                        if (str_FileExtension.ToUpper() == "TIF")
                            str_FileExtension = "TIFF";

                        str_FileDisplayName = xEle_ProcessDetails.XPathSelectElement("FILESYSTEM/FILE/DISPLAYNAME").Value;
                    }
                    else
                        return null;
                    
                    obj_MediaRequestRouter.IsMediaProcessorRequired(str_FileExtension, out str_ApplicationType, out str_FileFormatDetials, out str_JobPath);

                    if (str_FileExtension.ToUpper() == "PDF")
                        str_ApplicationType = "pdf2swf";

                    obj_MediaRequestRouter.GetJobPath(str_ApplicationType, out str_JobPath);
                    ALCommon.GenerateUniqueID(ref ul_JobID);

                    sb_OutputFilePath = new StringBuilder(50);
                    sb_OutputFilePath.AppendFormat("{0}\\{1}_{2}\\{3}", str_JobPath,SessionID.ToString(),UserID.ToString(),ul_JobID);
                    Directory.CreateDirectory(sb_OutputFilePath.ToString());
                    str_JobPath = sb_OutputFilePath.ToString();

                    sb_SourceFilePath = new StringBuilder(50);
                    //HemantK: Defect ID S0801897
                    //This change has been done to handle special characters in file name
                    sb_SourceFilePath.AppendFormat("{0}\\{1}.{2}", sb_OutputFilePath, xEle_ProcessDetails.Element("REVIEWFILEID").Value, str_FileExtension);

                    ErrorXML = GetFile(xEle_ProcessDetails.Element("FILESYSTEM").ToString(), "PROCESSFILE", sb_SourceFilePath.ToString(), "");
                    if (ErrorXML != null)
                    {
                        return ErrorXML;
                    }
                    
                    sb_InputXML = new StringBuilder();
                    //HemantK: Defect ID S0801897
                    //This change has been done to handle special characters in file name
                    str_FileNameWithoutExt = xEle_ProcessDetails.Element("REVIEWFILEID").Value;
                    if (str_ApplicationType.ToUpper() != "IMAGEPROCESSOR")
                    {
                        if (str_ApplicationType.ToUpper() == "PDF2SWF")
                        {
                            sb_OutputFilePath.AppendFormat("\\{0}.swf", str_FileNameWithoutExt);
                            sb_InputXML.AppendFormat(@"<FILECONVERTER>
	                                            <INPUT FILENAME='{0}' MEDIAGENERATOR='pdf2swf.exe' EVENT='CONVERTPDFTOSWF'/>
                                                <INPUTFILENAME>{3}</INPUTFILENAME>
                                                <INPUTFILDISPLAYENAME>{4}</INPUTFILDISPLAYENAME>                                                
	                                            <OUTPUTLIST><OUTPUT>
	                                            <FILEFORMAT FILENAME='{2}' DESTINATIONFILENAME='{2}' MEDIAGENERATORPARAM='-T9 -f -t' MEDIAGENERATOR='pdf2swf.exe' EVENT='CONVERTPDFTOSWF'>SWF</FILEFORMAT>
                                                </OUTPUT></OUTPUTLIST>
                                          </FILECONVERTER>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()), ALCommon.TrimAndReplaceSpecialChars(str_FileName), ALCommon.TrimAndReplaceSpecialChars(str_FileDisplayName));
                        }
                        else
                        {
                            sb_OutputFilePath.AppendFormat("\\{0}.flv", str_FileNameWithoutExt);
                            sb_InputXML.AppendFormat(@"<REQUEST>
		                                            <SOURCEFILE>{0}</SOURCEFILE>
		                                            <TENANTNAME>{1}</TENANTNAME>
		                                            <OUTPUTFILE>{2}</OUTPUTFILE>
		                                            <JOBPATH>{3}</JOBPATH>
		                                            <VIDEOWIDTH>0</VIDEOWIDTH>
		                                            <VIDEOHEIGHT>0</VIDEOHEIGHT>
		                                            <FLVGENERATOR>MEncoder.exe</FLVGENERATOR>
		                                            <LOGPATH>{4}</LOGPATH>
		                                            <FLVGENERATORPARAM>-mc 0 -af volnorm -ofps 25 -srate 44100 -oac mp3lame -lameopts abr:br=64 -ovc lavc -of lavf -lavfopts format=flv -lavcopts vcodec=flv:vbitrate=512:mbd=2:mv0:trell:v4mv:cbp:last_pred=3 -vf harddup</FLVGENERATORPARAM>
	                                          </REQUEST>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()), ALCommon.TrimAndReplaceSpecialChars(str_JobPath), ALCommon.TrimAndReplaceSpecialChars(str_JobPath));
                        }

                    }
                    else
                    {
                        sb_OutputFilePath.AppendFormat("\\{0}.jpg", str_FileNameWithoutExt);
                        sb_InputXML.AppendFormat(@"<FILECONVERTER>
	                                            <INPUT FILENAME='{0}' EVENT='GENERATEPREVIEWFILE'/>
	                                            <OUTPUTLIST><OUTPUT>
	                                            <FILEFORMAT FILENAME='{2}' FILEFORMAT='JPG' QUALITY='0' DPI='72' DEPTH='24' DESTINATIONFILENAME='{2}' EVENT='GeneratePreviewFile'>JPG</FILEFORMAT>
                                                </OUTPUT></OUTPUTLIST>
                                          </FILECONVERTER>", ALCommon.TrimAndReplaceSpecialChars(sb_SourceFilePath.ToString()), TenantName, ALCommon.TrimAndReplaceSpecialChars(sb_OutputFilePath.ToString()));//, ALCommon.TrimAndReplaceSpecialChars(str_JobPath), ALCommon.TrimAndReplaceSpecialChars(str_JobPath));
                    }
                    
                    sb_ExtraInformation = new StringBuilder();
                    sb_ExtraInformation.AppendFormat(@"<JOBDETAILS TYPE=""MEDIA TOPIC CREATION MULTIFILE"">
		                                                        <WORKFLOWPROCESSID>{0}</WORKFLOWPROCESSID>
		                                                        <REVIEWFILEID>{1}</REVIEWFILEID>
		                                                        <JOBINITATORID>{2}</JOBINITATORID>
                                                                <JOBID>{3}</JOBID>
		                                                        <JOBTYPE>MEDIA TOPIC CREATION MULTIFILE</JOBTYPE>
                                                                <FILEPATH>{4}</FILEPATH>
	                                                        </JOBDETAILS>", str_ProcessID, xEle_ProcessDetails.Element("REVIEWFILEID").Value, UserID.ToString(), ul_JobID.ToString(), str_TempFilePath);
                    ALCommon.GenerateUniqueID(ref ul_RequestID);
                    ErrorXML = obj_MediaRequestRouter.SendRequestToMediaProcessor(sb_InputXML.ToString(), str_ApplicationType, ul_RequestID.ToString(), sb_ExtraInformation.ToString());
                    if (ErrorXML != null)
                    {
                        return ErrorXML;
                    }
                }
                catch (Exception obj_Exception)
                {
                    sb_InputData = new StringBuilder(10);
                    sb_InputData.Append("ProcessID = " + xEle_ProcessDetails.Element("REVIEWFILEID").Value + "");
                    ExcepErrObj = new ALError(ErrorCodes.PROCESS_MODIFICATION_FAILED,
                                                 "Exception while creating Media Processor Job.", ErrorObjectType.PROCESS,
                                                 "Entity", ErrorType.EXCEPTION, obj_Exception, sb_InputData.ToString());
                    ErrorXML = ExcepErrObj.Serialize();
                    ExceptionTracing.LogError(TenantName, ErrorXML);
                    return ErrorXML;
                }
                return null;
            }
        #endregion

        #region Private Methods

            /// <alsummary>
			/// <Author>Mugdha Moolay</Author>
			/// <DateCreated>22 July 08</DateCreated>
			/// <Purpose>This method transfers data from a process (of type 
			/// subflow) that is completed to the parent process of the activity that refers to this process</Purpose>
			/// </alsummary>
			/// <returns></returns>
			private ERRORXML TransferDataOnProcessCompletion(String ProcessID,String SubflowActivityID)
			{
				XML							OutputXML				= null;
				ERRORXML					ErrorXML				= null;
				XPathDocument				xpDocProcess			= null;
				XPathNavigator				xPNavProcess			= null;
				XPathNavigator				xpNavFormalParam		= null;
				XPathNavigator				xpNavActualParam		= null;
				XPathNavigator				xpNavFormalParams   	= null;
				XPathNavigator				xpNavActivity			= null;
				XPathNavigator				xpNavActualParams		= null;
				XPathNavigator				xpRelevantDataFields	= null;
				XPathNavigator				xpNavDataField			= null;
				XPathNavigator				xpNav_NavPath			= null;
				XPathNavigator				xpNavTypeName			= null;
				XPathNodeIterator			xpNodeIterFormalParam	= null;				
				String						strSeqNo				= null;				
				ParameterMap				PmMap					= null;
				String						ActualParamID			= null;
				String						FormalParameterID		= null;
				String						strDataFieldName		= null;				
				EntityFacetCollection		EFObj					= null;
				List<EntityFacetCollection> lstEFObj				= null;
				String						strPrimaryKeyName		= null;
				String						strPrimaryKeyValue		= null;
				String						strType					= null;
				String						strTypeName				= null;				
				String						strDataType				= null;
                String                      strRelationshipName     = null;
                String                      str_TransferType        = null;
				int							Index					= -1;				
				ALEngineError				ALEngineErrorObj        = null;

                StringDictionary            strdict_AdditionalInfo  = null;
				
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Start TransferDataOnProcessCompletion() function.");					
					lstEFObj		 = new List<EntityFacetCollection>();
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Before GetFormalParametersAndActualParametersForProcess() function.");
					ErrorXML		 = GetFormalParametersAndActualParametersForProcess(ProcessID,SubflowActivityID, out OutputXML);
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "After GetFormalParametersAndActualParametersForProcess() function.");
					if((ErrorXML==null)&&(OutputXML != null))
					{
						xpDocProcess			= new XPathDocument(new StringReader(OutputXML));
						xPNavProcess			= xpDocProcess.CreateNavigator();
						xPNavProcess			= xPNavProcess.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
						// get the formal parameters
						xpNavFormalParams		= xPNavProcess.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/FORMALPARAMETERS");
						xpRelevantDataFields	= xPNavProcess.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS");
						xpNavActivity			= xPNavProcess.SelectSingleNode("SUBFLOWACTIVITY/ACTIVITY");
						xpNavActualParams		= xpNavActivity.SelectSingleNode("ACTIVITY_INFORMATION/ACTIVITYINFORMATION/ACTUALPARAMETERS");
						if((xpNavFormalParams != null) && (!String.IsNullOrEmpty(xpNavFormalParams.InnerXml)))
						{
							// Select all the formal parameters with mode = OUT/INOUT
							xpNodeIterFormalParam = xpNavFormalParams.Select("PROCESS_FORMALPARAMETERS/FORMALPARAMETER[MODE='OUT' or MODE='INOUT']");
							
							while (xpNodeIterFormalParam.MoveNext())
							{
								//select all the DataFields where DataFieldID = FormalParameterID
                                strRelationshipName     = null;								
								xpNavFormalParam		= xpNodeIterFormalParam.Current;
								FormalParameterID		= xpNavFormalParam.SelectSingleNode("FORMALPARAMETERID").Value;
								strSeqNo				= xpNavFormalParam.SelectSingleNode("SEQUENCENUMBER").Value;

								// Select the Actual parameter with the same sequence number as that of Formal parameter
								xpNavActualParam		= xpNavActualParams.SelectSingleNode("ACTUALPARAMETERS/ACTUALPARAMETER[SEQUENCENUMBER='" + strSeqNo + "']");
								if(xpNavActualParam != null)
								{
									ActualParamID	= xpNavActualParam.SelectSingleNode("ACTUALPARAMETERID").Value;
									
									xpNavDataField = xpRelevantDataFields.SelectSingleNode("PROCESS_DATAFIELDS/DATAFIELD[DATAFIELDID='" + FormalParameterID + "']");
									if(xpNavDataField != null)
									{
										strDataFieldName	= xpNavDataField.SelectSingleNode("DATAFIELDNAME").Value;
										if (xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE")!=null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE").Value;
										}
										else if(xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE") != null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE").Value;
										}
                                        if (xpNavDataField.SelectSingleNode("TRANSFERTYPE") != null)
                                        {
                                            str_TransferType = xpNavDataField.SelectSingleNode("TRANSFERTYPE").Value;
                                        }

										PmMap				= new ParameterMap(ActualParamID, FormalParameterID, strDataFieldName);
										PmMap.DataType		= strDataType;
										xpNav_NavPath		= xpNavDataField.SelectSingleNode("STORAGE/NAVIGATIONPATH");
										if(xpNav_NavPath != null)
										{
											// In this Xpath query the first child::node() will refer to the WORKFLOWPROCESS/ACTIVITY node
											// the second child::node will select the node where the type = relation attribute is found
											// the third child:: node expression returns the facet/entity node having the primary key name and value attributes.
											xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
											if(xpNavTypeName != null)
											{
											    // check if there exists one more type='relation' node , if yes then its a series data transfer at the second level.
                                                xpNavTypeName = xpNavTypeName.SelectSingleNode("child::node()[@Type='RELATION']");
                                                if (xpNavTypeName != null)
                                                {
                                                    PmMap.TypeName              = xpNavTypeName.GetAttribute("TypeName", "");
                                                    PmMap.TypePrimaryKeyName    = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    xpNavTypeName.MoveToParent();
                                                    strPrimaryKeyName           = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    strPrimaryKeyValue          = xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
                                                    strType                     = xpNavTypeName.GetAttribute("Type", "");
                                                    strTypeName                 = xpNavTypeName.Name;
                                                    EFObj                       = new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
                                                }
                                                else
                                                {
                                                    xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
                                                    //if the RESPONSE entity is selected for data transfer then treat it like the case of relationship, 
                                                    // i.e do not create an EntityFacetCollection object of type RESPONSE but use the parent (ACTIVITY/RESPONSE)
                                                    if (xpNavTypeName.Name.ToUpper() == "RESPONSE")
                                                    {
                                                        xpNavTypeName               = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']");
                                                        strRelationshipName         = xpNavTypeName.Name.ToUpper();
                                                        xpNavTypeName.MoveToParent();

                                                        strPrimaryKeyName           = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                        strPrimaryKeyValue          = xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
                                                        strType                     = xpNavTypeName.GetAttribute("Type", "");
                                                        strTypeName                 = xpNavTypeName.Name;
													    EFObj						= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
                                                        PmMap.RelationshipName      = strRelationshipName;
                                                    }
                                                    else
                                                    {
                                                        strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName","");
											            strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
											            strType				= xpNavTypeName.GetAttribute("Type","");
											            strTypeName			= xpNavTypeName.Name;
											            EFObj				= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
                                                    }
                                                }
											}
											else // here only the type = relation node exists
											{
												xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']");
												if(xpNavTypeName != null)
												{													
													PmMap.TypeName				= xpNavTypeName.GetAttribute("TypeName","");
													PmMap.TypePrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");                                                    
                                                    if (str_TransferType != null)
                                                    {
                                                        PmMap.TransferType = str_TransferType;
                                                    }
													xpNavTypeName				= xpNav_NavPath.SelectSingleNode("child::node()");

													strPrimaryKeyName			= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
													strPrimaryKeyValue			= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
													strType						= xpNavTypeName.GetAttribute("Type","");
													strTypeName					= xpNavTypeName.Name;
													EFObj						= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
												}
												else // here the data field is part of the target entity.
												{
													xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()");
													if(xpNavTypeName != null)
													{
														strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName","");
														strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
														strType				= xpNavTypeName.GetAttribute("Type","");
														strTypeName			= xpNavTypeName.Name;
														EFObj				= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
													}
												}
											}

											if(EFObj != null)
											{
												if(!lstEFObj.Contains(EFObj))
												{
													EFObj.AddDataField(PmMap);
													lstEFObj.Add(EFObj);
												}
												else
												{
													Index = lstEFObj.IndexOf(EFObj);
													lstEFObj[Index].AddDataField(PmMap);
												}
											}
										}
									}
								}
							}
							ErrorXML = GetDataFieldValues(ref lstEFObj);
							if(ErrorXML != null)
							{
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while getting data field values.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("GET_DATAFIELDS_FAILED", this.GetType().ToString(), "TransferDataOnProcessCompletion()", null, ErrorXML);
                                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXML;
							}
							else
							{
								ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "Before UpdateProcessDataFieldsOnCompletion() function.");
								//Update the formal parameters to the parent process of the subflow Activity								
								ErrorXML = UpdateProcessDataFieldsOnCompletion(lstEFObj, SubflowActivityID);
								if(ErrorXML != null)
								{
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while updating data fields on process completion.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("UPDATE_DATAFIELDS_FOR_PROCESS_FAILED", this.GetType().ToString(), "TransferDataOnProcessCompletion()", null, ErrorXML);
                                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                    
									return ErrorXML;
								}
							}
							ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "After UpdateProcessDataFieldsOnCompletion() function.");
						}
					}
					else if(ErrorXML != null)
					{
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", ProcessID);
                        strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                        strdict_AdditionalInfo.Add("MOREINFO", "Error while getting formal and actual parameters for process.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("GET_FORMAL_ACTUAL_PARAMETERS_FOR_PROCESS_FAILED", this.GetType().ToString(), "TransferDataOnProcessCompletion()", null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
						return ErrorXML;
					}
					ALCommon.WriteLog(TenantName, "Process Manager", "ProcessManager", SessionID.ToString(), "End TransferDataOnProcessCompletion() function.");
					return ErrorXML;
			}

			/// <alsummary>
			/// <author>Mugdha Moolay</author>
			/// <datecreated>2 Aug 08</datecreated>
			/// <purpose>The purpose of this API is to retrieve the process formal and Activity actual parameters</purpose>
			/// </alsummary>
			/// <param name="ProcessID">The ID of the Workflowprocess whose formal parameters are to be retrieved</param>
			/// <param name="SubflowActivityID">The ID of the Subflow Activity( that refers to the process with ID "ProcessID")
			/// whose actual parameters are to be retrieved.
			///	</param>
			/// <param name="OutputXML">The resulting XML that consists of the formal and actual parameters</param>
			/// <returns>An ErrorXML on Error, null otherwise</returns>
			private ERRORXML GetFormalParametersAndActualParametersForProcess(String ProcessID,String SubflowActivityID, out XML OutputXML)
			{
				StringBuilder SearchXML = null;
				OMInterface   OMIObj    = null;
				ERRORXML	  ErrorXML  = null;

                StringDictionary    strdict_AdditionalInfo  = null;
				ALEngineError       ALEngineErrorObj        = null;
				
					OutputXML = null;
					SearchXML = new StringBuilder(1000);
                    SearchXML.Append(@"<SEARCH>
	                                            <TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                            <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <SELECTLIST>
													    <EXPRESSION>
			                                                <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
			                                                <ALIAS>WORKFLOWPROCESSID</ALIAS>
		                                                </EXPRESSION>
	                                                </SELECTLIST>
													<RELATIONSHIP NAME='PROCESS_INFORMATION'>
		                                                    <SELECTLIST>
																<EXPRESSION>
				                                                    <FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
				                                                    <ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
			                                                    </EXPRESSION>
																<EXPRESSION>
																	<FIELDNAME>FORMALPARAMETERS</FIELDNAME>
																	<ALIAS>FORMALPARAMETERS</ALIAS>
															</EXPRESSION>
															<EXPRESSION>
																	<FIELDNAME>RELEVANTDATAFIELDS</FIELDNAME>
																	<ALIAS>RELEVANTDATAFIELDS</ALIAS>
															</EXPRESSION>	
		                                                    </SELECTLIST>
		                                            </RELATIONSHIP>
													<RELATIONSHIP NAME='SUBFLOWACTIVITY'>
	                                                    <SELECTLIST>
															<EXPRESSION>
			                                                    <FIELDNAME>ACTIVITYID</FIELDNAME>
			                                                    <ALIAS>ACTIVITYID</ALIAS>
		                                                    </EXPRESSION>															
														</SELECTLIST>														
														<RELATIONSHIP NAME='ACTIVITY_INFORMATION'>
															<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>ACTIVITYINFOID</FIELDNAME>
																	<ALIAS>ACTIVITYINFOID</ALIAS>
																</EXPRESSION>
																<EXPRESSION>
																	<FIELDNAME>ACTUALPARAMETERS</FIELDNAME>
																	<ALIAS>ACTUALPARAMETERS</ALIAS>
															</EXPRESSION>	
															</SELECTLIST>
														</RELATIONSHIP>
														<FILTERLIST>
															<FILTER>
																<EXPRESSION>ACTIVITYID</EXPRESSION>
																<OPERATOR>EQUALTO</OPERATOR>
																<EXPRESSIONVALUE>" + SubflowActivityID + @"</EXPRESSIONVALUE>
																<ISCASESENSITIVE>False</ISCASESENSITIVE>
																<FILTERID>1</FILTERID>
															</FILTER>
														</FILTERLIST>
		                                            </RELATIONSHIP>
		                                            <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
				                                            <OPERATOR>EQUALTO</OPERATOR>
				                                            <EXPRESSIONVALUE>" + ProcessID + @"</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            <FILTERID>1</FILTERID>
			                                            </FILTER>
		                                            </FILTERLIST>
	                                            </TARGETENTITY>
                                            </SEARCH>");
					OMIObj		= new OMInterface(UserID, CurrentLanguage, TenantName);
					ErrorXML	= OMIObj.GetHierarchialData(SearchXML.ToString(),ref OutputXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", ProcessID);
                        strdict_AdditionalInfo.Add("ACTIVITYID", SubflowActivityID);
                        strdict_AdditionalInfo.Add("INPUTXML", SearchXML.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData().");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("PROCESS_READ_FAILED", this.GetType().ToString(),
                                                        "GetFormalParametersAndActualParametersForProcess()", null, strdict_AdditionalInfo,
                                                        null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    }
					return ErrorXML;
			}

			/// <summary>
			/// <createdate>12 Aug 08</createdate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method gets the RDF and Formal of the process referred by SubflowActivity, the Actual parameters of the Activity
			/// and also the parent process RDF</purpose>
			/// </summary>			
			/// <param name="SubflowActivityID">The ID of the Activity whose Actual parameters will be fetched.</param>
			/// <param name="OutputXML">The return XML that contains the requested information</param>
			/// <returns>An ErrorXML if an Error occurs when fetching requested information</returns>
			private ERRORXML GetFormalParametersAndActualParametersForSubflow(String SubflowActivityID, out XML OutputXML)
			{
				StringBuilder SearchXML = null;
				OMInterface   OMIObj    = null;
				ERRORXML	  ErrorXML  = null;

                StringDictionary    strdict_AdditionalInfo  = null;
				ALEngineError       ALEngineErrorObj        = null;
				
					OutputXML = null;
					SearchXML = new StringBuilder(1000);
                    SearchXML.Append(@"<SEARCH>
	                                            <TARGETOBJECT>ACTIVITY</TARGETOBJECT>
	                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                                            <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
	                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                            <TARGETENTITY>
		                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <SELECTLIST>
													    <EXPRESSION>
			                                                <FIELDNAME>ACTIVITYID</FIELDNAME>
			                                                <ALIAS>ACTIVITYID</ALIAS>
		                                                </EXPRESSION>
	                                                </SELECTLIST>
													<RELATIONSHIP NAME='ACTIVITY_INFORMATION'>
														<SELECTLIST>
															<EXPRESSION>
																<FIELDNAME>ACTIVITYINFOID</FIELDNAME>
																<ALIAS>ACTIVITYINFOID</ALIAS>
															</EXPRESSION>
															<EXPRESSION>
																<FIELDNAME>ACTUALPARAMETERS</FIELDNAME>
																<ALIAS>ACTUALPARAMETERS</ALIAS>
														</EXPRESSION>	
														</SELECTLIST>
													</RELATIONSHIP>
													<RELATIONSHIP NAME='PROCESS_ACTIVITY'>
										                    <SELECTLIST>
																<EXPRESSION>
				                                                    <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
				                                                    <ALIAS>WORKFLOWPROCESSID</ALIAS>
			                                                    </EXPRESSION>																
		                                                    </SELECTLIST>
															<RELATIONSHIP NAME='PROCESS_INFORMATION'>
																<SELECTLIST>
																	<EXPRESSION>
																		<FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
																		<ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
																	</EXPRESSION>																
																	<EXPRESSION>
																		<FIELDNAME>RELEVANTDATAFIELDS</FIELDNAME>
																		<ALIAS>RELEVANTDATAFIELDS</ALIAS>
																	</EXPRESSION>	
																</SELECTLIST>
															</RELATIONSHIP>														
		                                            </RELATIONSHIP>
													<RELATIONSHIP NAME='SUBFLOWACTIVITY'>
	                                                    <SELECTLIST>
															<EXPRESSION>
			                                                    <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
			                                                    <ALIAS>WORKFLOWPROCESSID</ALIAS>
		                                                    </EXPRESSION>															
														</SELECTLIST>														
														<RELATIONSHIP NAME='PROCESS_INFORMATION'>
															<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
																	<ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
																</EXPRESSION>
																<EXPRESSION>
																	<FIELDNAME>FORMALPARAMETERS</FIELDNAME>
																	<ALIAS>FORMALPARAMETERS</ALIAS>
																</EXPRESSION>
																<EXPRESSION>
																	<FIELDNAME>RELEVANTDATAFIELDS</FIELDNAME>
																	<ALIAS>RELEVANTDATAFIELDS</ALIAS>
															</EXPRESSION>
															</SELECTLIST>
														</RELATIONSHIP>
														<RELATIONSHIP NAME='WORKFLOWPROCESS_WORKFLOWINFORMATION'>
															<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>WORKFLOWSUBCATEGORY</FIELDNAME>
																	<ALIAS>WORKFLOWSUBCATEGORY</ALIAS>
																</EXPRESSION>																
															</SELECTLIST>
														</RELATIONSHIP>
		                                            </RELATIONSHIP>
		                                            <FILTERLIST>
															<FILTER>
																<EXPRESSION>ACTIVITYID</EXPRESSION>
																<OPERATOR>EQUALTO</OPERATOR>
																<EXPRESSIONVALUE>" + SubflowActivityID + @"</EXPRESSIONVALUE>
																<ISCASESENSITIVE>False</ISCASESENSITIVE>
																<FILTERID>1</FILTERID>
															</FILTER>
														</FILTERLIST>
	                                            </TARGETENTITY>
                                            </SEARCH>");
					OMIObj		= new OMInterface(UserID, CurrentLanguage, TenantName);
					ErrorXML	= OMIObj.GetHierarchialData(SearchXML.ToString(),ref OutputXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        //HemantK ErrorHandlingCodeChanges
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("ACTIVITYID", SubflowActivityID);
                        strdict_AdditionalInfo.Add("INPUTXML", SearchXML.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData().");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("ACTIVITY_READ_FAILED", this.GetType().ToString(),
                                                        "GetFormalParametersAndActualParametersForSubflow()", null, strdict_AdditionalInfo,
                                                        null, ErrorXML);
                        ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    }
					return ErrorXML;
			}

			/// <alsummary>
			/// <createddate>8 Aug 08</createddate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method Updates the RDF XML with the primary key values for the 
			/// entity/facet in the which the Data Field belongs. 
			/// This is a recursive function which will replace the primary key for the entire navigation path wherever required</purpose>
			/// </alsummary>
			/// <param name="ChildPrimaryKeyList"> The List of nodes that the have the primarykeyname attribute</param>
			/// <param name="InstanceNode">The Instance node of process that contain the primary key values</param>			
			void UpdateChildNodePrimaryKeys(XmlNodeList ChildPrimaryKeyList, ref XmlNode InstanceNode)
			{
				XmlNode		    TypeNameNode			= null;
				XmlNodeList     SubChildPrimaryKeyList	= null;
				String		    PrimarykeyValue			= null;                
				
					foreach(XmlNode ChildPKNode in ChildPrimaryKeyList)
					{
						if(InstanceNode.SelectSingleNode("descendant::" + ChildPKNode.Name) != null)
						{
                            TypeNameNode = InstanceNode.SelectSingleNode("descendant::" + ChildPKNode.Name + "/" + ((XmlElement)ChildPKNode).GetAttribute("PrimaryKeyName"));
							if(TypeNameNode != null)
							{
								PrimarykeyValue = TypeNameNode.InnerText;
								((XmlElement)ChildPKNode).SetAttribute("PrimaryKeyValue", PrimarykeyValue);
                                SubChildPrimaryKeyList = ChildPKNode.SelectNodes("child::node()[@Type='RELATION']/child::node()[@PrimaryKeyName]");
								UpdateChildNodePrimaryKeys(SubChildPrimaryKeyList, ref TypeNameNode);
							}
						}						
					}
			}

			/// <summary>
			/// <createddate>12 Aug 08</createddate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method updates the value of a single data field</purpose>
			/// </summary>
			/// <param name="DataFieldXML">The Relevant Data Field XML that describes the navigation path of the field</param>
			/// <param name="TargetEntityType">The Type of Target (Entity/Facet)</param>
			/// <param name="DataFieldName">The name of the data field to be updated</param>
			/// <param name="DataFieldValue">The value of the Data Field to be updated</param>
			/// <param name="DataType">The Data Type of the Field which will be used to handle data transfer</param>
			/// <returns>An Error XML if an error occurs on updating the Data Field, null otherwise.</returns>
			private ERRORXML UpdateDataField(XML DataFieldXML,String TargetEntityType,String DataFieldName, String DataType, String DataFieldValue)
			{
				String					InstalledPath	= null;
				StringBuilder			XSLTFilePath	= null;
				MemoryStream			Memory_Stream	= null;
				StreamReader			Stream_Reader	= null;
				XPathDocument			TempDoc			= null;
				XML						UpdateXML		= null;
				XML						OperationXML	= null;
				XmlDocument				RDFDoc			= null;
				XmlNode					DateFieldNode	= null;
				StringBuilder			OperationInputXML= null;
				XslCompiledTransform	transformObj	= null;
				OMInterface				OMIObj			= null;
				XPathNavigator			xpNavTemp		= null;
				ERRORXML				ErrorXml		= null;
				
					InstalledPath	= ALCommon.GetInstallationDir();
					XSLTFilePath	= new StringBuilder();
					OMIObj			= new OMInterface(UserID, CurrentLanguage, TenantName);
						
					XSLTFilePath.AppendFormat("{0}\\XSLT\\PROCESSDESIGNER\\UpdateRDF.xslt", InstalledPath);
					transformObj	= new XslCompiledTransform();
					transformObj.Load(XSLTFilePath.ToString());
					TempDoc			= new XPathDocument(new StringReader(DataFieldXML));
					Memory_Stream	= new MemoryStream();
					xpNavTemp		= TempDoc.CreateNavigator();
					transformObj.Transform(xpNavTemp, null, Memory_Stream);
					Memory_Stream.Position = 0;
					Stream_Reader	= new StreamReader(Memory_Stream);
					UpdateXML		= Stream_Reader.ReadToEnd();
					Memory_Stream.Flush();
					Stream_Reader.Close();

					switch(DataType.ToUpper())
					{
						case "FILE":
							break;
						case "BINDER":
							break;
						case "RELATIONSHIP":
							break;
						case "BOOLEAN":
							break;

						case "INTEGER":
						case "STRING":
						case "DATETIME":
						case "CURRENCY":
						case "TEXT":
						case "XML":
						case "FLOAT":
							// Load the XML document to update the value of the RDF.
							RDFDoc = new XmlDocument();
							RDFDoc.LoadXml(UpdateXML);
							DateFieldNode = RDFDoc.SelectSingleNode("descendant::" + DataFieldName);
							DateFieldNode.InnerXml = DataFieldValue;
							OperationInputXML = new StringBuilder(500);
                            OperationInputXML.Append(@"<INPUTXML>
												        <OPERATION>
											                <OBJECTNAME>" + TargetEntityType.ToUpper() + @"</OBJECTNAME>
							                                <METHOD>UPDATEDATA</METHOD>
													        <PARAM Name='INPUTXML'>" + RDFDoc.DocumentElement.OuterXml + @"</PARAM>
													    </OPERATION>
													<TRANSACTION>TRUE</TRANSACTION>
												</INPUTXML>");
							OperationXML = OperationInputXML.ToString();
							ErrorXml = OMIObj.Execute(ref OperationXML);
							break;
					}
					return ErrorXml;
			}

			/// <summary>
			/// <createon>19 Aug 08</createon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>The purpose of this method is to retrieve the 
			/// values of the fields specified in the Entity facet coll list</purpose>
			/// </summary>
			/// <param name="lstEFObj">The List of Objects of Class EntityFacetCollection which will be used to retrieve data field values</param>
			/// <returns>An ErrorXML if an error occurs while reading Data Fields.</returns>
			private ERRORXML GetDataFieldValues(ref List<EntityFacetCollection>	lstEFObj)
			{								
				XPathDocument	    xpDocDataField		= null;
				XPathNavigator	    xpNavDoc			= null;
				XPathNavigator	    xpNavDataField		= null;
                XPathNavigator      xpNavTargetObject   = null;
                XPathNavigator      xpathNav_File       = null;
                XPathNodeIterator   xpNodeIter_ChildNodes = null;
				XmlDocument		RelatedEntityDoc	= null;
				XmlNodeList		TypeNodeList		= null;
				XmlNode			PkNode				= null;
				StringBuilder	sbSearchXML			= null;
				StringBuilder	sbSelectList		= null;
				String			strTargetObjName	= null;				
				ERRORXML		ErrorXml			= null;
				XML				outputXML			= null;
				OMInterface		OmiObj				= null;				
				XML				strRelationShipXML	= null;
				StringBuilder	sbFileXML			= null;
                StringBuilder   sb_Query            = null;
                StringBuilder   sb_RelationshipFieldList = null;
                //Boolean         bool_RelationshipNameExists = false;
                String          AssetRelationShips  = null;
				ALEngineError	ALEngineErrorObj	= null;
                
                Hashtable       VisitedRelationships = null;
                StringDictionary strdict_AdditionalInfo = null;

					if(lstEFObj.Count > 0)
					{
						OmiObj = new OMInterface(UserID, CurrentLanguage, TenantName);                        
						for(int Counter = 0; Counter < lstEFObj.Count; Counter++)
						{
							sbSearchXML		            = new StringBuilder(1000);
							sbSelectList	            = new StringBuilder(1000);
                            VisitedRelationships        = new Hashtable();
                            AssetRelationShips          = String.Empty;
                            //bool_RelationshipNameExists = false;
							strTargetObjName = lstEFObj[Counter].TypeName;
							sbSearchXML.AppendFormat(@"<SEARCH>
														<TARGETOBJECT>{0}</TARGETOBJECT>
														<TARGETOBJECTTYPE>{1}</TARGETOBJECTTYPE>
														<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
														<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
														<MAXRESULTSIZE>1000</MAXRESULTSIZE>
														<TARGETENTITY>
														<PRIVILEGENAME>READ</PRIVILEGENAME>
														<FILTERLIST>
															<FILTER>
																<EXPRESSION>{2}</EXPRESSION>
																<OPERATOR>EQUALTO</OPERATOR>
																<EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
															</FILTER>
														</FILTERLIST>														
													", strTargetObjName, lstEFObj[Counter].Type, lstEFObj[Counter].PrimaryKeyName, lstEFObj[Counter].PrimaryKeyValue);
							sbSelectList.Append("<SELECTLIST>");
                            
							foreach(ParameterMap PmObj in lstEFObj[Counter].ParameterMapList)
							{
                                //bool_RelationshipNameExists = false;
                                if (!String.IsNullOrEmpty(PmObj.RelationshipName))
                                {
                                    if (!VisitedRelationships.ContainsKey(PmObj.RelationshipName))
                                    {
                                        sb_RelationshipFieldList = new StringBuilder();
                                        sb_RelationshipFieldList.AppendFormat("<RELATIONSHIP NAME='{0}'><SELECTLIST>", PmObj.RelationshipName);
                                        VisitedRelationships.Add(PmObj.RelationshipName, sb_RelationshipFieldList.ToString());
                                        sb_RelationshipFieldList = null;
                                    }
                                    // update the field List
                                    sb_RelationshipFieldList = new StringBuilder();
                                    sb_RelationshipFieldList.Append(VisitedRelationships[PmObj.RelationshipName]);
                                    if ((PmObj.DataType.ToUpper() == "FILE") || (PmObj.DataType.ToUpper() == "BINDER"))
                                    {
                                        sb_RelationshipFieldList.AppendFormat(@"<EXPRESSION>
																<FIELDNAME FileRequired='YES'>{0}</FIELDNAME>
																<ALIAS>{0}</ALIAS>
															   </EXPRESSION>", PmObj.ParameterName);
                                    }
                                    else
                                    {
                                        sb_RelationshipFieldList.AppendFormat(@"<EXPRESSION>
																<FIELDNAME>{0}</FIELDNAME>
																<ALIAS>{0}</ALIAS>
															   </EXPRESSION>", PmObj.ParameterName);
                                    }
                                    VisitedRelationships[PmObj.RelationshipName] = sb_RelationshipFieldList.ToString();
                                }
                                else
                                { 
                                    if(PmObj.DataType.ToUpper() == "RELATIONSHIP")
								    {
									    if(PmObj.TypeName.ToUpper() == "DIGITALASSET")
									    {
                                            if (!AssetRelationShips.Contains(PmObj.ParameterName)) 
                                            //if (!bool_RelationshipNameExists)
                                            {
                                                sbSearchXML.AppendFormat("<RELATIONSHIP NAME='{0}'><SEARCHALLFACETS>FALSE</SEARCHALLFACETS></RELATIONSHIP>", PmObj.ParameterName, strRelationShipXML);
                                                //bool_RelationshipNameExists = true;
                                                AssetRelationShips += PmObj.ParameterName + ";";
                                            }
									    }
									    else
									    {
                                            sbSearchXML.AppendFormat("<RELATIONSHIP NAME='{0}' FileRequired='YES'><SEARCHALLFACETS>TRUE</SEARCHALLFACETS></RELATIONSHIP>", PmObj.ParameterName);
									    }
								    }
								    else
								    {
									    if((PmObj.DataType.ToUpper() == "FILE") || (PmObj.DataType.ToUpper() == "BINDER"))
									    {
										    sbSelectList.AppendFormat(@"<EXPRESSION>
																    <FIELDNAME FileRequired='YES'>{0}</FIELDNAME>
																    <ALIAS>{0}</ALIAS>
															       </EXPRESSION>", PmObj.ParameterName);
									    }
									    else
									    {
										    sbSelectList.AppendFormat(@"<EXPRESSION>
																    <FIELDNAME>{0}</FIELDNAME>
																    <ALIAS>{0}</ALIAS>
															       </EXPRESSION>", PmObj.ParameterName);
									    }
								    }
                                }
							}
							sbSelectList.Append("</SELECTLIST>");
                            IDictionaryEnumerator Ienum_visitedRel = null;
                            Ienum_visitedRel = VisitedRelationships.GetEnumerator();
                            while (Ienum_visitedRel.MoveNext())
                            {
                                sbSearchXML.AppendFormat("{0}</SELECTLIST></RELATIONSHIP>", Ienum_visitedRel.Value);
                            }                                                
                            sbSearchXML.AppendFormat("{0}</TARGETENTITY></SEARCH>", sbSelectList.ToString());
                            
							ErrorXml = OmiObj.GetHierarchialData(sbSearchXML.ToString(), ref outputXML);
                            if (ErrorXml != null)
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("INPUTXML", sbSearchXML.ToString());
                                strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData().");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("READ_FAILED", this.GetType().ToString(),
                                                                "GetDataFieldValues()", null, strdict_AdditionalInfo,
                                                                null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }
							// Update the DataFieldList with values of data fields.
							xpDocDataField	= new XPathDocument(new StringReader(outputXML));
							xpNavDoc		= xpDocDataField.CreateNavigator();
                            xpNavTargetObject = xpNavDoc.SelectSingleNode(strTargetObjName + "COLLECTION/" + strTargetObjName);                             

                            foreach(ParameterMap PmObj in lstEFObj[Counter].ParameterMapList)
						    {
							    if (!String.IsNullOrEmpty(PmObj.RelationshipName))
                                {
                                    sb_Query = new StringBuilder();
                                    if(PmObj.DataType.ToUpper() == "BINDER")
							        {
                                        sb_Query.AppendFormat("{0}/child::node()/FILESYSTEM/BINDER/FILE",PmObj.RelationshipName);
                                        xpNodeIter_ChildNodes = xpNavTargetObject.Select(sb_Query.ToString());
                                        while (xpNodeIter_ChildNodes.MoveNext())
                                        {
                                            PmObj.ParameterValue = PmObj.ParameterValue + xpNodeIter_ChildNodes.Current.OuterXml;
                                        }
                                        sb_Query.Remove(0, sb_Query.Length);
							        }
							        else
							        {
                                        sb_Query.AppendFormat("{0}/child::node()/{1}", PmObj.RelationshipName, PmObj.ParameterName);
                                        xpNodeIter_ChildNodes = xpNavTargetObject.Select(sb_Query.ToString());
                                        while (xpNodeIter_ChildNodes.MoveNext())
                                        {
                                            if (PmObj.ParameterValue != null)
                                                PmObj.ParameterValue = PmObj.ParameterValue + "\n" + DateTime.Today.ToLongDateString() + "\n" + xpNodeIter_ChildNodes.Current.InnerXml;
                                            else
                                                PmObj.ParameterValue = xpNodeIter_ChildNodes.Current.InnerXml;
                                        }
							        }
                                }
                                else
                                {
                                    if(PmObj.DataType.ToUpper() == "FILE")
							        {
                                        xpathNav_File = xpNavTargetObject.SelectSingleNode("FILESYSTEM/FILE[FIELDNAME='" + PmObj.ParameterName + "']");
                                        if (xpathNav_File != null)
                                        {
                                            sbFileXML = new StringBuilder(500);
                                            sbFileXML.AppendFormat("<NAME>{0}</NAME>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpathNav_File.SelectSingleNode("NAME").Value));
                                            sbFileXML.AppendFormat("<DISPLAYNAME>{0}</DISPLAYNAME>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpathNav_File.SelectSingleNode("DISPLAYNAME").Value));
                                            sbFileXML.AppendFormat("<PATH>{0}</PATH>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpathNav_File.SelectSingleNode("PATH").Value));
                                            PmObj.ParameterValue = sbFileXML.ToString();
                                        }
							        }
							        else if(PmObj.DataType.ToUpper() == "BINDER")
							        {
                                        if (xpNavTargetObject.SelectSingleNode("FILESYSTEM/BINDER[FIELDNAME='" + PmObj.ParameterName + "']") != null)
                                        {
                                            if (xpNavTargetObject.Select("FILESYSTEM/BINDER[FIELDNAME='" + PmObj.ParameterName + "']/FILE").Count > 0)
                                                PmObj.ParameterValue = xpNavTargetObject.SelectSingleNode("FILESYSTEM/BINDER[FIELDNAME='" + PmObj.ParameterName + "']").InnerXml;
                                        }
							        }
							        else
							        {
                                        xpNavDataField = xpNavTargetObject.SelectSingleNode(PmObj.ParameterName);
								        if(xpNavDataField != null)
								        {
									        if(!String.IsNullOrEmpty(PmObj.TypeName)) // type = 'relationship'
									        {
										        if(PmObj.TypeName.ToUpper() == "DIGITALASSET")
										        {
											        PmObj.ParameterValue = xpNavDataField.OuterXml;
										        }
										        else
										        {
											        RelatedEntityDoc = new XmlDocument();
											        RelatedEntityDoc.LoadXml(xpNavDataField.OuterXml);
											        TypeNodeList = RelatedEntityDoc.DocumentElement.ChildNodes;
											        foreach(XmlNode ChildNode in TypeNodeList)
											        {
												        PkNode = ChildNode.SelectSingleNode(PmObj.TypePrimaryKeyName);
												        if(PkNode != null)
												        {
													        ChildNode.RemoveChild(PkNode);
												        }
												        // Check if there is any field of type binder, if yes then prepare appropriate XML for binder / file creation

												        // TO DO: Yet to be added.
											        }
											        PmObj.ParameterValue = RelatedEntityDoc.DocumentElement.InnerXml;
										        }
									        }
									        else
                                            {
                                                switch (PmObj.DataType.ToUpper())
                                                {
                                                    case "XML":
                                                    case "DATETIME":
                                                        PmObj.ParameterValue = xpNavDataField.InnerXml;
                                                        break;
                                                    case "CURRENCY":
                                                        if (xpNavDataField.SelectSingleNode("AMOUNT") != null)
                                                        {
                                                            PmObj.ParameterValue = xpNavDataField.InnerXml;
                                                        }
                                                        break;
                                                    default:
                                                        PmObj.ParameterValue = xpNavDataField.Value;
                                                        break;
                                                }
                                            }
								        }
							        }
                                }
						    }
						}
					}
					return ErrorXml;
			}

			/// <alsummary>
			/// <createdon>1 Aug 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method updates the process data fields on completion</purpose>
			/// </alsummary>
			/// <param name="lstEFObj">List of EntityFacetCollection Objects</param>
			/// <param name="SubflowActivityID">The ID of the SubflowActivity</param>
			/// <returns>an ErrorXML on Error, else null</returns>
			private ERRORXML UpdateProcessDataFieldsOnCompletion(List<EntityFacetCollection> lstEFObj,String SubflowActivityID)
			{
				XML							ResultXml			= null;
				ERRORXML					ErrorXml			= null;
				StringBuilder				SearchXml			= null;
				OMInterface					OMIObj				= null;
				XPathDocument				xpDocProcess		= null;
				XPathNavigator				xpNavProcess		= null;
				XPathNavigator				xpNavRDF			= null;
				XPathNavigator				xpNavDataField		= null;
				XPathNavigator				xpNav_NavPath		= null;
				XPathNavigator				xpNavTypeName		= null;
				String						strType				= null;
				String						DataFieldName		= null;				
				String						strPrimaryKeyName	= null;
				String						strPrimaryKeyValue	= null;
				String						strTypeName			= null;
				String						strDataType			= null;                
				List<EntityFacetCollection> lstProcessEFObj		= null;
				EntityFacetCollection		EFObj				= null;
				ParameterMap				PmMap				= null;				
				int							Index				= -1;
                String                      str_OverwriteBinderFiles = null; 				
                StringDictionary    strdict_AdditionalInfo = null;
                ALEngineError       ALEngineErrorObj        = null;

					// get the parent process.
					if (lstEFObj.Count > 0)
					{
						SearchXml = new StringBuilder(1000);
						SearchXml.AppendFormat(@"<SEARCH>
													<TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
													<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
													<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
													<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
													<TARGETENTITY>
														<PRIVILEGENAME>READ</PRIVILEGENAME>
														<SELECTLIST>
															<EXPRESSION>
																<FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
																<ALIAS>WORKFLOWPROCESSID</ALIAS>
															</EXPRESSION>
														</SELECTLIST>													
														<FILTERLIST>
															<FILTER>
																<EXPRESSION>ACTIVITYID</EXPRESSION>
																<OPERATOR>EQUALTO</OPERATOR>
																<EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
																<ISCASESENSITIVE>False</ISCASESENSITIVE>
																<RELATIONSHIPNAME>PROCESS_ACTIVITY</RELATIONSHIPNAME>
															</FILTER>
														</FILTERLIST>
														<RELATIONSHIP NAME='PROCESS_INFORMATION'>
															<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>WORKFLOWPROCESSINFOID</FIELDNAME>
																	<ALIAS>WORKFLOWPROCESSINFOID</ALIAS>
																</EXPRESSION>
																<EXPRESSION>
																	<FIELDNAME>RELEVANTDATAFIELDS</FIELDNAME>
																	<ALIAS>RELEVANTDATAFIELDS</ALIAS>
																</EXPRESSION>	
															</SELECTLIST>
														</RELATIONSHIP>		                                          
													</TARGETENTITY>
												</SEARCH>", SubflowActivityID);

						OMIObj		= new OMInterface(UserID, CurrentLanguage, TenantName);
						ErrorXml	= OMIObj.GetHierarchialData(SearchXml.ToString(), ref ResultXml);
						if(ErrorXml == null)
						{
							lstProcessEFObj = new List<EntityFacetCollection>();
							xpDocProcess = new XPathDocument(new StringReader(ResultXml));
							xpNavProcess = xpDocProcess.CreateNavigator();
							xpNavProcess = xpNavProcess.SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
							xpNavRDF	 = xpNavProcess.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO/RELEVANTDATAFIELDS");
							
							foreach(EntityFacetCollection DFObj in lstEFObj)
							{
								foreach(ParameterMap PmObj in DFObj.ParameterMapList)
								{
									xpNavDataField = xpNavRDF.SelectSingleNode("PROCESS_DATAFIELDS/DATAFIELD[DATAFIELDID='" + PmObj.ActualParameterID + "']");
									if(xpNavDataField != null)
									{
										DataFieldName = xpNavDataField.SelectSingleNode("DATAFIELDNAME").Value;
										if(xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE") != null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE").Value;
										}
										else if(xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE") != null)
										{
											strDataType = xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE").Value;
										}
                                        if (xpNavDataField.SelectSingleNode("OVERWRITEBINDERFILES") != null)
                                        {
                                            str_OverwriteBinderFiles = xpNavDataField.SelectSingleNode("OVERWRITEBINDERFILES").Value;
                                        }
                                       
										xpNav_NavPath = xpNavDataField.SelectSingleNode("STORAGE/NAVIGATIONPATH");
										if(xpNav_NavPath != null)
										{
											xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
											if(xpNavTypeName != null)
											{
												xpNavTypeName = xpNavTypeName.SelectSingleNode("child::node()[@Type='RELATION']");
                                                if (xpNavTypeName != null)
                                                {
                                                    PmMap				        = new ParameterMap();
                                                    PmMap.TypeName              = xpNavTypeName.GetAttribute("TypeName", "");
                                                    PmMap.TypePrimaryKeyName    = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    xpNavTypeName.MoveToParent();
                                                    strPrimaryKeyName           = xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    strPrimaryKeyValue          = xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
                                                    strType                     = xpNavTypeName.GetAttribute("Type", "");
                                                    strTypeName                 = xpNavTypeName.Name;
                                                    PmMap.ParameterName         = DataFieldName;
												    PmMap.ParameterValue        = PmObj.ParameterValue;
												    PmMap.DataType		        = strDataType;
                                                    EFObj                       = new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
                                                }
                                                else
                                                {
                                                    xpNavTypeName       = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
                                                    strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
												    strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
												    strType				= xpNavTypeName.GetAttribute("Type", "");
												    strTypeName			= xpNavTypeName.Name;
												    PmMap				= new ParameterMap();
												    PmMap.ParameterName = DataFieldName;
												    PmMap.ParameterValue= PmObj.ParameterValue;
												    PmMap.DataType		= strDataType;
                                                    if (strDataType.ToUpper() == "BINDER")
                                                    {
                                                        if (!String.IsNullOrEmpty(str_OverwriteBinderFiles))
                                                            PmMap.OverwriteBinderFiles = str_OverwriteBinderFiles;
                                                        else
                                                            PmMap.OverwriteBinderFiles = "FALSE";
                                                    }
												    EFObj				= new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
                                                }                                                
											}
											else
											{
												// here only the type = relation node exists
												xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']");
												if(xpNavTypeName != null)
												{
													PmMap						= new ParameterMap();
													PmMap.ParameterName			= DataFieldName;
													PmMap.ParameterValue		= PmObj.ParameterValue;
													PmMap.DataType				= strDataType;
													PmMap.TypeName				= xpNavTypeName.GetAttribute("TypeName","");
													PmMap.TypePrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                                    PmMap.RelationshipName      = xpNavTypeName.Name;
                                                    PmMap.TransferType          = PmObj.TransferType;
                                                    
													xpNavTypeName				= xpNav_NavPath.SelectSingleNode("child::node()");

													strPrimaryKeyName			= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
													strPrimaryKeyValue			= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
													strType						= xpNavTypeName.GetAttribute("Type","");
													strTypeName					= xpNavTypeName.Name;
													EFObj						= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
												}
												else // here the fields selected are from the WORKFLOWPROCESS/ACTIVITY or related entity fields.
												{
													xpNavTypeName		= xpNav_NavPath.SelectSingleNode("child::node()");
													if (xpNavTypeName!=null)
													{
														strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
														strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
														strType				= xpNavTypeName.GetAttribute("Type", "");
														strTypeName			= xpNavTypeName.Name;
														PmMap				= new ParameterMap();
														PmMap.ParameterName = DataFieldName;
														PmMap.ParameterValue= PmObj.ParameterValue;
														PmMap.DataType		= strDataType;
                                                        if (strDataType.ToUpper() == "BINDER")
                                                        {
                                                            if (!String.IsNullOrEmpty(str_OverwriteBinderFiles))
                                                                PmMap.OverwriteBinderFiles = str_OverwriteBinderFiles;
                                                            else
                                                                PmMap.OverwriteBinderFiles = "FALSE";
                                                        }
														EFObj				= new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
													}
												}
											}
										}
										if(!lstProcessEFObj.Contains(EFObj))
										{
											EFObj.AddDataField(PmMap);
											lstProcessEFObj.Add(EFObj);
										}
										else
										{
											Index = lstProcessEFObj.IndexOf(EFObj);
											if(Index >= 0)
												lstProcessEFObj[Index].AddDataField(PmMap);
										}
									}
								}
							}
							ErrorXml = UpdateProcessDataFields("COMPLETED",lstProcessEFObj);
                            if (!String.IsNullOrEmpty(ErrorXml))
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while updating process data fields.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                "UpdateProcessDataFieldsOnCompletion()", null, strdict_AdditionalInfo,
                                                                null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }
						}
                        else
                        {
                            strdict_AdditionalInfo = new StringDictionary();
                            strdict_AdditionalInfo.Add("SUBFLOWACTIVITYID", SubflowActivityID);
                            strdict_AdditionalInfo.Add("INPUTXML", SearchXml.ToString());
                            strdict_AdditionalInfo.Add("MOREINFO", "Error in  GetHierarchialData(), while getting parent process.");

                            ALEngineErrorObj = new ALEngineError();
                            ALEngineErrorObj.AppendError("PROCESS_READ_FAILED", this.GetType().ToString(),
                                                            "UpdateProcessDataFieldsOnCompletion()", null, strdict_AdditionalInfo,
                                                            null, ErrorXml);
                            ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                            return ErrorXml;
                        }
					}                    
					return ErrorXml;
			}

			/// <alsummary>
			/// <createdon>20-Aug-08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>The purpose of this method is to update the Relevant data fields of
			/// the process that is marked as Active or completed</purpose>
			/// </alsummary>
			/// <param name="lstEFColl">This is a list of objects of class EntityFacetCollection</param>
			/// <param name="XpNavRDF">This is the xpathNavigator the refers to the relevant 
			/// data fields of the process that is marked as active</param>
            /// <param name="str_Workflowcategory">The Workflowcategory is used to find whether the Approvals job or task file is passed as a video approvals file</param>
            /// /// <param name="str_ProcessStatus">The new changed status of the process</param>
			/// <returns>An error XML </returns>
            private ERRORXML UpdateProcessDataFieldsOnProcessInitialization(List<EntityFacetCollection> lstEFColl, XPathNavigator XpNavRDF, String str_Workflowcategory, String str_ProcessStatus)
			{
				XPathNavigator				xpNavDataField		= null;
				XPathNavigator				xpNav_NavPath		= null;
				XPathNavigator				xpNavTypeName		= null;
				String						strType				= null;
				String						DataFieldName		= null;				
				String						strPrimaryKeyName	= null;
				String						strPrimaryKeyValue	= null;
				String						strTypeName			= null;
				String						strDataType			= null;
				List<EntityFacetCollection> lstProcessEFColl	= null;
				EntityFacetCollection		EFObj				= null;
				ParameterMap				PmMap				= null;
				ERRORXML					ErrorXml			= null;                
                String                      str_TransferType    = null;
				int							Index				= -1;
                String                      str_OverwriteBinderFiles = null; 

                StringDictionary    strdict_AdditionalInfo  = null;
                ALEngineError       ALEngineErrorObj        = null;
				
					lstProcessEFColl        = new List<EntityFacetCollection>();
                                    
					foreach(EntityFacetCollection DFObj in lstEFColl)
					{
						foreach(ParameterMap PmObj in DFObj.ParameterMapList)
						{
							xpNavDataField = XpNavRDF.SelectSingleNode("PROCESS_DATAFIELDS/DATAFIELD[DATAFIELDID='" + PmObj.FormalParameterID + "']");
							if(xpNavDataField != null)
							{
								DataFieldName = xpNavDataField.SelectSingleNode("DATAFIELDNAME").Value;
								if(xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE") != null)
								{
									strDataType = xpNavDataField.SelectSingleNode("DATATYPE/BASICTYPE").Value;
								}
								else if(xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE") != null)
								{
									strDataType = xpNavDataField.SelectSingleNode("DATATYPE/DECLAREDTYPE").Value;
								}
                                if (xpNavDataField.SelectSingleNode("TRANSFERTYPE") != null)
                                {
                                    str_TransferType = xpNavDataField.SelectSingleNode("TRANSFERTYPE").Value;
                                }
                                if (xpNavDataField.SelectSingleNode("OVERWRITEBINDERFILES") != null)
                                {
                                    str_OverwriteBinderFiles = xpNavDataField.SelectSingleNode("OVERWRITEBINDERFILES").Value;
                                }

								xpNav_NavPath = xpNavDataField.SelectSingleNode("STORAGE/NAVIGATIONPATH");
								if(xpNav_NavPath != null)
								{
									xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']/child::node()[@Type]");
									if(xpNavTypeName != null)
									{
										strPrimaryKeyName		= xpNavTypeName.GetAttribute("PrimaryKeyName","");
										strPrimaryKeyValue		= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
										strType					= xpNavTypeName.GetAttribute("Type","");
										strTypeName				= xpNavTypeName.Name;
										PmMap					= new ParameterMap();
										PmMap.ParameterName		= DataFieldName;
										PmMap.ParameterValue	= PmObj.ParameterValue;
										PmMap.DataType			= strDataType;
                                        if (strDataType.ToUpper() == "BINDER")
                                        {
                                            if (!String.IsNullOrEmpty(str_OverwriteBinderFiles))
                                                PmMap.OverwriteBinderFiles = str_OverwriteBinderFiles;
                                            else
                                                PmMap.OverwriteBinderFiles = "FALSE";
                                        }
										EFObj					= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);										
									}
									else
									{
										// here only the type = relation node exists
										xpNavTypeName = xpNav_NavPath.SelectSingleNode("child::node()/child::node()[@Type='RELATION']");
										if(xpNavTypeName != null)
										{
											PmMap						= new ParameterMap();
											PmMap.ParameterName			= DataFieldName;
											PmMap.ParameterValue		= PmObj.ParameterValue;
											PmMap.DataType				= strDataType;
											PmMap.TypeName				= xpNavTypeName.GetAttribute("TypeName","");
											PmMap.TypePrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
                                            PmMap.RelationshipName      = xpNavTypeName.Name;
                                            if (str_TransferType != null)
                                            {
                                                PmMap.TransferType = str_TransferType;
                                            }

											xpNavTypeName				= xpNav_NavPath.SelectSingleNode("child::node()");

											strPrimaryKeyName			= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
											strPrimaryKeyValue			= xpNavTypeName.GetAttribute("PrimaryKeyValue","");
											strType						= xpNavTypeName.GetAttribute("Type","");
											strTypeName					= xpNavTypeName.Name;
											EFObj						= new EntityFacetCollection(strTypeName,strPrimaryKeyName,strPrimaryKeyValue,strType);
										}
										else // here the fields selected are from the WORKFLOWPROCESS/ACTIVITY or related entity fields.
										{
											xpNavTypeName		= xpNav_NavPath.SelectSingleNode("child::node()");
											if (xpNavTypeName!=null)
											{
												strPrimaryKeyName	= xpNavTypeName.GetAttribute("PrimaryKeyName", "");
												strPrimaryKeyValue	= xpNavTypeName.GetAttribute("PrimaryKeyValue", "");
												strType				= xpNavTypeName.GetAttribute("Type", "");
												strTypeName			= xpNavTypeName.Name;
												PmMap				= new ParameterMap();
												PmMap.ParameterName = DataFieldName;
												PmMap.ParameterValue= PmObj.ParameterValue;
												PmMap.DataType		= strDataType;
                                                if (strDataType.ToUpper() == "BINDER")
                                                {
                                                    if (!String.IsNullOrEmpty(str_OverwriteBinderFiles))
                                                        PmMap.OverwriteBinderFiles = str_OverwriteBinderFiles;
                                                    else
                                                        PmMap.OverwriteBinderFiles = "FALSE";
                                                }
												EFObj				= new EntityFacetCollection(strTypeName, strPrimaryKeyName, strPrimaryKeyValue, strType);
											}
										}
									}

									if(!lstProcessEFColl.Contains(EFObj))
									{
										EFObj.AddDataField(PmMap);
										lstProcessEFColl.Add(EFObj);
									}
									else
									{
										Index = lstProcessEFColl.IndexOf(EFObj);
										if (Index>=0)
											lstProcessEFColl[Index].AddDataField(PmMap);
									}
								}
							}
						}
					}
                    // Update the Process DataFieldList
                    if (lstProcessEFColl.Count > 0)
                    {
                        ErrorXml = UpdateProcessDataFields(str_ProcessStatus, lstProcessEFColl);
                        if (!String.IsNullOrEmpty(ErrorXml))
                        {
                            strdict_AdditionalInfo = new StringDictionary();
                            strdict_AdditionalInfo.Add("MOREINFO", "Error while updating process data fields.");

                            ALEngineErrorObj = new ALEngineError();
                            ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                            "UpdateProcessDataFieldsOnProcessInitialization()", null, strdict_AdditionalInfo,
                                                            null, ErrorXml);
                            ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                        }
                    }
	                return ErrorXml;
			}

			/// <alsummary>
			/// <createdon>20-Aug-08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method updates the process data fields.</purpose>
			/// </alsummary>
			/// <param name="lstProcessEFColl">The List of Entity facet collection objects</param>
            /// <param name="str_ProcessStatus"></param>
			/// <returns>An ErrorXML if an error occurs, else returns null</returns>
			private ERRORXML UpdateProcessDataFields(String str_ProcessStatus, List<EntityFacetCollection> lstProcessEFColl)
			{
				OMInterface				OMIObj				= null;
				StringBuilder			UpdateXML			= null;
				StringBuilder			OperationInputXML	= null;
                XPathDocument           xpDoc_UpdataData    = null;
                XPathNavigator          xpNav_TargetObject  = null;
                XPathNodeIterator       xpNodeIter_Child    = null;
				XML						OperationXML		= null;
				XML						OutputXML			= null;
                XML                     DisAssociateXML     = null;
				ERRORXML				ErrorXml			= null;
				String					strDataType			= null;
				StringBuilder			sbFileXML			= null;
				XPathDocument			xpDocBinder			= null;
				XPathNavigator			xpNavBinder			= null;
				XPathNavigator			xpNavFileSystem		= null;
				XPathNodeIterator		xpIterFile			= null;
				String					strFileName			= null;
				String					strDisplayName		= null;
				String					strDescription		= null;
				String					strPath				= null;
				String					strBinderName		= null;
				String					strBinderID			= null;	
				String					strFieldName		= null;
                String                  str_FileSize        = null; 
				Boolean					BinderExists		= false;
				Boolean					GetBinderField		= false;				
				Boolean					UpdateData			= false;
				String					MethodName			= "UPDATE";                
				ALEngineError			ALEngineErrorObj	= null;
                String			        strSectionID		= null;
                StringBuilder           sb_File             = null;
                Boolean                 bool_CloseBider     = false;
                List<EntityFacetCollection> EFC_DigitalAssets   = null;
				
                StringDictionary    strdict_AdditionalInfo  = null;
                Boolean             b_AppendFiles           = false;
                XElement            xElt_Binder             = null;

					OMIObj = new OMInterface(UserID, CurrentLanguage, TenantName);
                    EFC_DigitalAssets = new List<EntityFacetCollection>();
                    
					// Update the Process DataFieldList
					foreach(EntityFacetCollection DFObj in lstProcessEFColl)
					{
						GetBinderField  = false;
						UpdateXML = new StringBuilder(500);
                        sbFileXML = new StringBuilder(500);
                        sb_File   = new StringBuilder(100);
                        DisAssociateXML = null;
						UpdateXML.AppendFormat("<{0} MODIFIED='TRUE'>", DFObj.TypeName);
						UpdateXML.AppendFormat("<{0}>{1}</{0}>",DFObj.PrimaryKeyName,DFObj.PrimaryKeyValue);
						foreach(ParameterMap PmObj in DFObj.ParameterMapList)
						{
                            BinderExists    = false;
                            strBinderID     = null;
                            strBinderName   = null;
                            strFieldName    = null;
                            str_FileSize    = null; 
							strDataType     = PmObj.DataType;

                            // 3 November 09: When mode is initialisation and the Value to be updated is empty then do not Update it.

                            if ((String.IsNullOrEmpty(PmObj.ParameterValue)) && (str_ProcessStatus.ToUpper() != "COMPLETED"))
                                continue;

							switch(strDataType.ToUpper())
							{
								case "FILE":
									if(!String.IsNullOrEmpty(PmObj.ParameterValue))
									{
                                        if (!GetBinderField)
                                        {
                                            ErrorXml = GetBinder(DFObj, out OutputXML);
                                            GetBinderField = true;
                                        }
                                        if (ErrorXml == null)
                                        {
                                            xpDocBinder = new XPathDocument(new StringReader(OutputXML));
                                            xpNavBinder = xpDocBinder.CreateNavigator();
                                            xpNavBinder = xpNavBinder.SelectSingleNode(DFObj.TypeName + "COLLECTION" + "/" + DFObj.TypeName);
                                            xpNavFileSystem = xpNavBinder.SelectSingleNode("FILESYSTEM");
                                            if ((xpNavFileSystem != null) && (xpNavFileSystem.HasChildren))
                                            {
                                                xpNavBinder =  xpNavFileSystem.SelectSingleNode("FILE[FIELDNAME='" + PmObj.ParameterName + "']");
                                                if (xpNavBinder != null)
                                                {
                                                    //delete old file.
                                                    sb_File.Append(@"<FILE Operation='DELETE'>"
                                                                        + xpNavBinder.InnerXml +
															        "</FILE>");
                                                }
                                            }
                                            MethodName = "UPDATEDATA";
                                            sb_File.AppendFormat(@"<FILE Operation='CREATE'>{0}
															<FIELDNAME>{1}</FIELDNAME>
															</FILE>", PmObj.ParameterValue, PmObj.ParameterName);

                                            xpDocBinder     = null;
                                            xpNavBinder     = null;
                                            xpNavFileSystem = null;
                                        }
                                        else
                                        {
                                            strdict_AdditionalInfo = new StringDictionary();
                                            strdict_AdditionalInfo.Add("DATATYPE", strDataType);
                                            strdict_AdditionalInfo.Add("TYPENAME", DFObj.TypeName);
                                            strdict_AdditionalInfo.Add("PRIMARYKEY", DFObj.PrimaryKeyValue);
                                            strdict_AdditionalInfo.Add("PARAMETERVALUE", PmObj.ParameterValue);
                                            strdict_AdditionalInfo.Add("MOREINFO", "Error while getting binder.");

                                            ALEngineErrorObj = new ALEngineError();
                                            ALEngineErrorObj.AppendError("GET_BINDER_FAILED", this.GetType().ToString(),
                                                                            "UpdateProcessDataFields()", null, strdict_AdditionalInfo,
                                                                            null, ErrorXml);
                                            ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                            return ErrorXml;
                                        }
									}
									break;
								case "BINDER":
                                    if (!String.IsNullOrEmpty(PmObj.ParameterValue))
                                    {
                                        // Get the field first, check whether it already exists, if yes then the Binder will be updated with the new files, else the binder will be newly created.
                                        if (!GetBinderField)
                                        {
                                            ErrorXml        = GetBinder(DFObj, out OutputXML);
                                            GetBinderField  = true;
                                        }
                                        if (ErrorXml == null)
                                        {
                                            xpDocBinder = new XPathDocument(new StringReader(OutputXML));
                                            xpNavBinder = xpDocBinder.CreateNavigator();
                                            xpNavBinder = xpNavBinder.SelectSingleNode(DFObj.TypeName + "COLLECTION" + "/" + DFObj.TypeName);
                                            xpNavFileSystem = xpNavBinder.SelectSingleNode("FILESYSTEM/BINDER[FIELDNAME='" + PmObj.ParameterName + "']");
                                            if ((xpNavFileSystem != null) && (xpNavFileSystem.HasChildren))
                                            {
                                                BinderExists  = true;
                                                strBinderID   = xpNavFileSystem.SelectSingleNode("BINDERID").Value;
                                                strBinderName = xpNavFileSystem.SelectSingleNode("NAME").Value;
                                                strFieldName  = xpNavFileSystem.SelectSingleNode("FIELDNAME").Value;
                                                if ((str_ProcessStatus.ToUpper() != "COMPLETED") || ((!String.IsNullOrEmpty(PmObj.OverwriteBinderFiles)) && PmObj.OverwriteBinderFiles.ToUpper() == "TRUE"))   
                                                {
                                                    //Check whether there exist any files already getting deleted
                                                    if (sbFileXML.Length > 0)
                                                        CheckForDuplicateFiles(sbFileXML.ToString(), strBinderID, ref xpNavFileSystem);

                                                    if ((xpNavFileSystem != null) && (xpNavFileSystem.HasChildren))
                                                    {
                                                        sbFileXML.AppendFormat("<BINDER Operation='Update'><BINDERID>{0}</BINDERID>", strBinderID);
                                                        sbFileXML.AppendFormat("<NAME>{0}</NAME>", strBinderName);
                                                        sbFileXML.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", strFieldName);
                                                        xpIterFile = xpNavFileSystem.Select("FILE");
                                                        while (xpIterFile.MoveNext())
                                                        {
                                                            sbFileXML.Append("<FILE Operation='DELETE'>"
                                                                                + xpIterFile.Current.InnerXml +
                                                                            "</FILE>");
                                                        }
                                                        sbFileXML.Append("</BINDER>");
                                                    }
                                                }
                                            }
                                            xpDocBinder     = null;
                                            xpNavBinder     = null;
                                            xpNavFileSystem = null;
                                            MethodName      = "UPDATEDATA";

                                            xpDocBinder = new XPathDocument(new StringReader("<BINDER>" + PmObj.ParameterValue + "</BINDER>"));
                                            xpNavBinder = xpDocBinder.CreateNavigator();
                                            xpNavBinder = xpNavBinder.SelectSingleNode("BINDER");
                                            xpIterFile = xpNavBinder.Select("FILE");

                                            if (xpIterFile.Count > 0)
                                            {
                                                if (BinderExists)
                                                {
                                                    sbFileXML.AppendFormat("<BINDER Operation='Update'><BINDERID>{0}</BINDERID>", strBinderID);
                                                    sbFileXML.AppendFormat("<NAME>{0}</NAME>", strBinderName);
                                                    sbFileXML.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", strFieldName);
                                                    bool_CloseBider = true; 
                                                }
                                                if (!BinderExists)
                                                {
                                                    //Fix for S0785700
                                                    //Two binders mapped to one binder in data transfer, seperate binder creation nodes were created previously, hence was causing issue. 
                                                    if (sbFileXML.Length > 0 && sbFileXML.ToString().Contains("<FIELDNAME>" + PmObj.ParameterName + "</FIELDNAME>"))
                                                    {
                                                        b_AppendFiles   = true;
                                                        bool_CloseBider = false;
                                                        xElt_Binder     = XElement.Load(XmlReader.Create(new StringReader(sbFileXML.ToString())));
                                                    }
                                                    else
                                                    {
                                                        sbFileXML.Append("<BINDER Operation='Create'>");
                                                        sbFileXML.AppendFormat("<NAME>{0}</NAME>", PmObj.ParameterName);
                                                        sbFileXML.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", PmObj.ParameterName);
                                                        bool_CloseBider = true;
                                                    }
                                                }


                                                while (xpIterFile.MoveNext())
                                                {
                                                    strFileName = ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpIterFile.Current.SelectSingleNode("NAME").Value);
                                                    strDisplayName = ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpIterFile.Current.SelectSingleNode("DISPLAYNAME").Value);
                                                    strDescription = ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpIterFile.Current.SelectSingleNode("DESCRIPTION").Value);
                                                    strPath = ALSpecialCharHandling.TrimAndReplaceSpecialChars(xpIterFile.Current.SelectSingleNode("PATH").Value);
                                                    str_FileSize = xpIterFile.Current.SelectSingleNode("FILESIZE").Value;

                                                    //Fix for S0785700     
                                                    if (b_AppendFiles && xElt_Binder != null)
                                                    {
                                                        String str_File = @"<FILE Operation='CREATE'>
																				<DISPLAYNAME>" + strDisplayName + @"</DISPLAYNAME>
																				<NAME>" + strFileName + @"</NAME>
																				<PATH>" + strPath + @"</PATH>
																				<DESCRIPTION>" + strDescription + @"</DESCRIPTION>
																				<FIELDNAME>" + PmObj.ParameterName + @"</FIELDNAME>
                                                                                <FILESIZE>" + str_FileSize + @"</FILESIZE> 
																		    </FILE>";
                                                        XElement xElt_File = null;
                                                        xElt_File          = XElement.Load(XmlReader.Create(new StringReader(str_File)));

                                                        xElt_Binder.Add(xElt_File);
                                                    }
                                                    else
                                                    {
                                                        sbFileXML.AppendFormat(@"<FILE Operation='CREATE'>
																				        <DISPLAYNAME>{0}</DISPLAYNAME>
																				        <NAME>{1}</NAME>
																				        <PATH>{2}</PATH>
																				        <DESCRIPTION>{3}</DESCRIPTION>
																				        <FIELDNAME>{4}</FIELDNAME>
                                                                                        <FILESIZE>" + str_FileSize + @"</FILESIZE> 
																		        </FILE>", strDisplayName, strFileName, strPath, strDescription, PmObj.ParameterName);
                                                    }
                                                }

                                                //Fix for S0785700     
                                                if (b_AppendFiles && xElt_Binder != null)
                                                {
                                                    sbFileXML.Remove(0, sbFileXML.Length);
                                                    sbFileXML.Append(xElt_Binder.ToString());
                                                    b_AppendFiles = false;
                                                }
                                                if (bool_CloseBider)
                                                {
                                                    sbFileXML.Append("</BINDER>");
                                                    bool_CloseBider = false; 
                                                }
                                            }
                                        }
                                        else
                                        {
                                            strdict_AdditionalInfo = new StringDictionary();
                                            strdict_AdditionalInfo.Add("DATATYPE", strDataType);
                                            strdict_AdditionalInfo.Add("TYPENAME", DFObj.TypeName);
                                            strdict_AdditionalInfo.Add("PRIMARYKEY", DFObj.PrimaryKeyValue);
                                            strdict_AdditionalInfo.Add("PARAMETERVALUE", PmObj.ParameterValue);
                                            strdict_AdditionalInfo.Add("MOREINFO", "Error while getting binder.");

                                            ALEngineErrorObj = new ALEngineError();
                                            ALEngineErrorObj.AppendError("GET_BINDER_FAILED", this.GetType().ToString(),
                                                                            "UpdateProcessDataFields()", null, strdict_AdditionalInfo,
                                                                            null, ErrorXml);
                                            ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                            return ErrorXml;
                                        }
                                    }
									break;
								case "RELATIONSHIP":
									if(!String.IsNullOrEmpty(PmObj.ParameterValue))
									{
                                        if (PmObj.TypeName.ToUpper() == "DIGITALASSET" && PmObj.TransferType.ToUpper() != "ASSOCIATE")
                                        {
                                            UpdateData = false;
                                            if ((str_ProcessStatus.ToUpper() == "ACTIVE") || (str_ProcessStatus.ToUpper() == "COMPLETED"))
                                            {
                                                if (!EFC_DigitalAssets.Contains(DFObj))
                                                {
                                                    EFC_DigitalAssets.Add(DFObj);
                                                }
                                            }
                                        }
                                        else
                                        {
                                            if (PmObj.TransferType.ToUpper() == "ASSOCIATE")
                                            {
                                                MethodName = "UPDATEDATA";
                                                ErrorXml = CreateAssociations(DFObj, PmObj.ParameterValue, PmObj, str_ProcessStatus, ref UpdateXML);
                                                if (ErrorXml != null)
                                                {
                                                    strdict_AdditionalInfo = new StringDictionary();
                                                    strdict_AdditionalInfo.Add("DATATYPE", strDataType);
                                                    strdict_AdditionalInfo.Add("TYPENAME", DFObj.TypeName);
                                                    strdict_AdditionalInfo.Add("PRIMARYKEY", DFObj.PrimaryKeyValue);
                                                    strdict_AdditionalInfo.Add("PARAMETERVALUE", PmObj.ParameterValue);
                                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while creating associations.");

                                                    ALEngineErrorObj = new ALEngineError();
                                                    ALEngineErrorObj.AppendError("DIGITALASSET_ASSOCIATION_FAILED", this.GetType().ToString(), "UpdateProcessDataFields()", null, ErrorXml);
                                                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                                    return ErrorXml;
                                                }
                                            }
                                            else
                                            {
                                                if (str_ProcessStatus.ToUpper() != "COMPLETED")
                                                {
                                                    DisAssociateXML += @"<OPERATION>
                                                                        <METHOD>DISASSOCIATEALLINSTANCESEX</METHOD>
                                                                        <OBJECTNAME>RELATION</OBJECTNAME>"
                                                                                + "<PARAM Name='OBJECTID'>" + DFObj.PrimaryKeyValue + "</PARAM>"
                                                                                + "<PARAM Name='OBJECTTYPE'>" + DFObj.Type + "</PARAM>"
                                                                                + "<PARAM Name='ISCONTAINER'>TRUE</PARAM>"
                                                                                + "<PARAM Name='ISCONTAINS'>FALSE</PARAM>"
                                                                                + "<PARAM Name='RELATIONSHIPNAME'>" + PmObj.ParameterName + "</PARAM>"
                                                                        + "</OPERATION>";
                                                }
                                                MethodName = "UPDATEDATA";
                                                UpdateXML.AppendFormat("<{0} TYPE='RELATION'>", PmObj.ParameterName);
                                                UpdateXML.Append(PmObj.ParameterValue);
                                                UpdateXML.AppendFormat("</{0}>", PmObj.ParameterName);
                                            }
                                        }
									}
									break;

                                case "XML":
                                case "DATETIME":
                                case "CURRENCY":
                                    MethodName = "UPDATEDATA";
                                    UpdateXML.AppendFormat("<{0}>", PmObj.ParameterName);
                                    UpdateXML.Append(PmObj.ParameterValue);
                                    UpdateXML.AppendFormat("</{0}>", PmObj.ParameterName);
                                    break;

								case "BOOLEAN":
								case "INTEGER":
								case "STRING":
								case "TEXT":
								case "FLOAT":									
									MethodName = "UPDATEDATA";
									UpdateXML.AppendFormat("<{0}>", PmObj.ParameterName);
									UpdateXML.Append( ALSpecialCharHandling.TrimAndReplaceSpecialChars(PmObj.ParameterValue));
									UpdateXML.AppendFormat("</{0}>", PmObj.ParameterName);
									break;
							}
						}
                        //if File/Binder is present.
                        if (sb_File.Length > 0 || sbFileXML.Length > 0)
                        {
                            UpdateXML.Append("<FILESYSTEM>");
                            if ((sbFileXML != null) && (sbFileXML.Length > 0))
                            {
                                //sbFileXML.Append("</BINDER>");
                                UpdateXML.Append(sbFileXML.ToString());
                                sbFileXML = null;
                            }
                            if (sb_File.Length > 0)
                            {
                                UpdateXML.Append(sb_File.ToString());
                            }
                            UpdateXML.Append("</FILESYSTEM>");
                        }

						UpdateXML.AppendFormat("</{0}>", DFObj.TypeName);

                        // Load the update Data XML and identify whether only the primary key node exists.
                        // if yes then no need to update, else update the data fields
                        xpDoc_UpdataData = new XPathDocument(new StringReader(UpdateXML.ToString()));
                        xpNav_TargetObject = xpDoc_UpdataData.CreateNavigator();
                        xpNav_TargetObject = xpNav_TargetObject.SelectSingleNode(DFObj.TypeName);
                        xpNodeIter_Child = xpNav_TargetObject.Select("child::node()");
                        if (xpNodeIter_Child.Count > 1)
                        {
                            UpdateData = true;
                        }
                        else if ((xpNodeIter_Child.Count == 1) && (String.Compare(xpNodeIter_Child.Current.SelectSingleNode("child::node()").Name, DFObj.PrimaryKeyName, true)==0))
                        {
                            UpdateData = false;
                        }
						if(UpdateData)
						{
							OperationInputXML = new StringBuilder(500);
							OperationInputXML.Append("<INPUTXML>");
                            if (!String.IsNullOrEmpty(DisAssociateXML))
                                OperationInputXML.Append(DisAssociateXML);
								OperationInputXML.Append(@"<OPERATION>
										               <OBJECTNAME>");
							OperationInputXML.AppendFormat("{0}</OBJECTNAME>", DFObj.Type);
							OperationInputXML.AppendFormat(@"<METHOD>{1}</METHOD>
												<PARAM Name='INPUTXML'>{0}</PARAM>
												</OPERATION>
												<TRANSACTION>TRUE</TRANSACTION>", UpdateXML.ToString(), MethodName);
                            
							OperationInputXML.Append("</INPUTXML>");
							OperationXML = OperationInputXML.ToString();
							ErrorXml = OMIObj.Execute(ref OperationXML);
                            if (ErrorXml != null)
                            {
                                if (ErrorXml.Contains("<OBJECTTYPE>DEFERREDQUERY</OBJECTTYPE>"))
                                {
                                    ErrorXml = null;
                                }
                            }
                            if (ErrorXml != null)
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("DATATYPE", strDataType);
                                strdict_AdditionalInfo.Add("TYPENAME", DFObj.TypeName);
                                strdict_AdditionalInfo.Add("PRIMARYKEY", DFObj.PrimaryKeyValue);
                                strdict_AdditionalInfo.Add("INPUTXML", OperationXML);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error in OM Execute.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("UPDATE_FAILED", this.GetType().ToString(), "UpdateProcessDataFields()", null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }
						}
					}


                    foreach (EntityFacetCollection DFObj in EFC_DigitalAssets)
                    {
                        foreach (ParameterMap PmObj in DFObj.ParameterMapList)
                        {
                            strDataType = PmObj.DataType;

                            switch (strDataType.ToUpper())
                            {
                                case "RELATIONSHIP":
                                    if (!String.IsNullOrEmpty(PmObj.ParameterValue))
                                    {
                                        if (PmObj.TypeName.ToUpper() == "DIGITALASSET" && PmObj.TransferType.ToUpper() != "ASSOCIATE")
                                        {
                                            UpdateData = false;
                                            if ((str_ProcessStatus.ToUpper() == "ACTIVE") || (str_ProcessStatus.ToUpper() == "COMPLETED"))
                                            {
                                                if (DFObj.TypeName.ToUpper() == "ACTIVITY")
                                                {
                                                    strSectionID = null;
                                                }
                                                ErrorXml = CopyDigitalAssets(DFObj, PmObj.ParameterValue, ref strSectionID);
                                                if (ErrorXml != null)
                                                {
                                                    strdict_AdditionalInfo = new StringDictionary();
                                                    strdict_AdditionalInfo.Add("DATATYPE", strDataType);
                                                    strdict_AdditionalInfo.Add("TYPENAME", DFObj.TypeName);
                                                    strdict_AdditionalInfo.Add("PRIMARYKEY", DFObj.PrimaryKeyValue);
                                                    strdict_AdditionalInfo.Add("PARAMETERVALUE", PmObj.ParameterValue);
                                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while copying digital assets.");

                                                    ALEngineErrorObj = new ALEngineError();
                                                    ALEngineErrorObj.AppendError("COPY_DIGITALASSET_FAILED", this.GetType().ToString(), "CopyDigitalAssets()", null, ErrorXml);
                                                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                                    return ErrorXml;
                                                }
                                            }
                                        }
                                    }
                                    break;
                            }
                        }
                    }
					return ErrorXml;
			}

            /// <summary>
            /// <CreatedOn>25 May 2011</CreatedOn>
            /// <Author>Mr. Hemant Karkande</Author>
            /// <Purpose>To check same file is not going for deletion more than once</Purpose>
            /// </summary>
            /// <param name="str_ExistingBinderXML">Existing binder xml</param>
            /// <param name="str_BinderID">Binder id</param>
            /// <param name="xpNavFileSystem">File system navigator</param>
            private void CheckForDuplicateFiles(String str_ExistingBinderXML, String str_BinderID, ref XPathNavigator xpNavFileSystem)
            {
                XPathDocument       xp_ExistingBinderXML= null; 
                XPathNodeIterator   iter_Binder         = null; 
                XPathNavigator      nav_File            = null; 
                XmlDocument         doc_FilesToDelete   = null; 
                XmlNodeList         nodeList_Files      = null; 

                str_ExistingBinderXML = "<FILESYSTEM>" + str_ExistingBinderXML + "</FILESYSTEM>";

                xp_ExistingBinderXML = new XPathDocument(new StringReader(str_ExistingBinderXML));
                iter_Binder = xp_ExistingBinderXML.CreateNavigator().Select("FILESYSTEM/BINDER[BINDERID='" + str_BinderID + "']");
                if (iter_Binder.Count > 0)
                {
                    //Load the files that are to be deleted
                    doc_FilesToDelete = new XmlDocument();
                    doc_FilesToDelete.LoadXml(xpNavFileSystem.OuterXml);
                    nodeList_Files = doc_FilesToDelete.SelectNodes("BINDER/FILE");  
                    foreach (XPathNavigator nav_Binder in iter_Binder)
                    {
                        foreach (XmlNode node_file in nodeList_Files)
                        {
                            String str_FileID = node_file.SelectSingleNode("FILEID").InnerText;
                            nav_File = nav_Binder.SelectSingleNode("FILE[FILEID='" + str_FileID + "']");
                            if ((nav_File != null) && (nav_File.GetAttribute("Operation", "").ToUpper() == "DELETE"))
                                node_file.ParentNode.RemoveChild(node_file);
                        }
                    }
                    nodeList_Files = null;
                    nodeList_Files = doc_FilesToDelete.SelectNodes("BINDER/FILE");
                    if (nodeList_Files.Count > 0)
                        xpNavFileSystem = doc_FilesToDelete.CreateNavigator();
                    else
                        xpNavFileSystem = null; 
                }
            }

			/// <alsummary>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method retrieves the binder field</purpose>
			/// <createdon>8 Sept 08</createdon>
			/// </alsummary>
			/// <param name="EfObj">The entity or facet object that contains the binder field</param>			
			/// <param name="OutputXML">The return XML contains the details of the binder field </param>
			/// <returns>On Error this method returns an error XML, else returns null</returns>
			private ERRORXML GetBinder(EntityFacetCollection EfObj, out XML OutputXML)
			{
				StringBuilder	SearchXML	= null;
				OMInterface		OMIObj		= null;
				ERRORXML		ErrorXml	= null;
                List<ParameterMap> PmObjList = null;

                StringDictionary    strdict_AdditionalInfo  = null;
                ALEngineError       ALEngineErrorObj        = null;
				
					OMIObj		= new OMInterface(UserID,CurrentLanguage,TenantName);
					OutputXML = null;
					SearchXML	= new StringBuilder(1000);
					SearchXML.AppendFormat(@"<SEARCH>
												<TARGETOBJECT>{0}</TARGETOBJECT>
												<TARGETOBJECTTYPE>{1}</TARGETOBJECTTYPE>
												<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
												<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
												<MAXRESULTSIZE>1000</MAXRESULTSIZE>
												<TARGETENTITY>
												<PRIVILEGENAME>READ</PRIVILEGENAME>
												<FILTERLIST>
													<FILTER>
														<EXPRESSION>{2}</EXPRESSION>
														<OPERATOR>EQUALTO</OPERATOR>
														<EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
													</FILTER>
												</FILTERLIST><SELECTLIST>", EfObj.TypeName, EfObj.Type, EfObj.PrimaryKeyName, EfObj.PrimaryKeyValue);

                    PmObjList = EfObj.ParameterMapList;
                    foreach (ParameterMap obj_ParameterMap in PmObjList)
                    {
                        if ((String.Compare(obj_ParameterMap.DataType, "File", true) == 0) || (String.Compare(obj_ParameterMap.DataType, "Binder", true) == 0))
                        {
                            SearchXML.Append(@"<EXPRESSION>
												    <FIELDNAME>" + obj_ParameterMap.ParameterName + @"</FIELDNAME>
												    <ALIAS>" + obj_ParameterMap.ParameterName + @"</ALIAS>
											 </EXPRESSION>");
                        }
                    }

                    SearchXML.Append(@"     </SELECTLIST>
                                        </TARGETENTITY>
									</SEARCH>");
					ErrorXml = OMIObj.GetHierarchialData(SearchXML.ToString(), ref OutputXML);
                    if (!String.IsNullOrEmpty(ErrorXml))
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("INPUTXML", SearchXML.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData().");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("READ_FAILED", this.GetType().ToString(),
                                                        "GetBinder()", null, strdict_AdditionalInfo,
                                                        null, ErrorXml);
                        ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                    }
					return ErrorXml;
			}

			/// <alsummary>
			/// <createdon>10 Sept 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This API handles the special case of relationship, where the related entity is DIGITALASSET</purpose>
			/// </alsummary>
			/// <param name="EfObj">This is the EntityFacetCollection Object, the instance ID of object represented will be used to associate with the newly created Digital Asset</param>
			/// <param name="DigitalAssetsXML">This consists of the Digital Assets that will be copied and associated to the entity represented in the EFObj</param>
            /// <param name="str_FinalSectionID">The section id for cataloging digital assets for the process</param>
			/// <returns>An ErrorXML if an error occurs when copying a digitalAsset, else returns null</returns>
			private ERRORXML CopyDigitalAssets(EntityFacetCollection EfObj,XML DigitalAssetsXML, ref String str_FinalSectionID)
			{
				XmlDocument		DigitalAssetDoc			= null;				
				XPathDocument	xpDocTemp				= null;
				XPathNavigator	xpNavVault				= null;
				XPathNavigator	xpNavSec				= null;				
				XmlNodeList		DigitalAssetNodeList	= null;				
				ERRORXML		ErrorXml				= null;				
				XML				OutputXML				= null;
				XML				SectionXML				= null;				
				DigitalAsset	DigitalAssetObj			= null;
				String			strSectionPath			= null;
				String			strVaultID				= null;
				String			strRootSectionID		= null;
				String			strSecurityPolicyID		= null;
				StringBuilder	sbSearchXML				= null;
				StringBuilder	sbCreateSection			= null;
				OMInterface		OmIObj					= null;
				Section			SectionObj				= null;				
				Boolean			SectionExists			= false;				
                String          str_PackageDefID        = null;
                String          str_ProcessDefID        = null;
                String          str_FixedSectionPath    = null;
                String          str_FixedSectionID      = null;
                String          str_FinalSectionPath    = null;
                String          str_DataTransferredXml  = null;

                StringDictionary    strdict_AdditionalInfo  = null;
                ALEngineError       ALEngineErrorObj        = null;
                
					DigitalAssetDoc = new XmlDocument();
					sbSearchXML		= new StringBuilder();
                    OmIObj			= new OMInterface(UserID,CurrentLanguage,TenantName);
                    SectionObj      = new Section(UserID, CurrentLanguage, TenantName);

                    if (String.IsNullOrEmpty(str_FinalSectionID))
                    {
                        if (EfObj.TypeName.ToUpper() != "ACTIVITY")
                        {
                            //Radha : To get default section path for the process
                            ErrorXml = GetPredefinedSectionForProcess(EfObj.PrimaryKeyValue, str_PackageDefID, str_ProcessDefID, out str_FixedSectionPath, out str_FixedSectionID, out str_FinalSectionPath, out str_FinalSectionID);
                            if (!String.IsNullOrEmpty(ErrorXml))
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", EfObj.PrimaryKeyValue);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while getting predefined section for the process.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("PROCESS_MODIFICATION_FAILED", this.GetType().ToString(),
                                                                "CopyDigitalAssets()", null, strdict_AdditionalInfo,
                                                                null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }
                        }
                        if ((String.IsNullOrEmpty(ErrorXml)) && (String.IsNullOrEmpty(str_FinalSectionID)))
                        {
                            // Get the VaultID of the MWB system Vault
                            sbSearchXML.Append(@"<SEARCH>
												<TARGETOBJECT>VAULT</TARGETOBJECT>
												<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
												<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
												<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
												<MAXRESULTSIZE>1000</MAXRESULTSIZE>
												<TARGETENTITY>
												<PRIVILEGENAME>READ</PRIVILEGENAME>
												<FILTERLIST>
													<FILTER>
														<EXPRESSION>NAME</EXPRESSION>
														<OPERATOR>EQUALTO</OPERATOR>
														<EXPRESSIONVALUE>MWB VAULT</EXPRESSIONVALUE>
													</FILTER>
												</FILTERLIST>
												<SELECTLIST>
												<EXPRESSION>
													<FIELDNAME>VAULTID</FIELDNAME>
													<ALIAS>VAULTID</ALIAS>
												</EXPRESSION>
												</SELECTLIST>
												<RELATIONSHIP NAME='VAULT_ROOTSECTION'>
													<SELECTLIST>
														<EXPRESSION>
															<FIELDNAME>SECTIONID</FIELDNAME>
															<ALIAS>SECTIONID</ALIAS>
														</EXPRESSION>
													</SELECTLIST>
	
														<RELATIONSHIP NAME='SECTION_OWNER'>
															<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>USERID</FIELDNAME>
																	<ALIAS>USERID</ALIAS>
																</EXPRESSION>
															</SELECTLIST>
														</RELATIONSHIP>
													<RELATIONSHIP NAME='SECTION_SECURITYPOLICY'>
														<SELECTLIST>
																<EXPRESSION>
																	<FIELDNAME>SECURITYPOLICYID</FIELDNAME>
																	<ALIAS>SECURITYPOLICYID</ALIAS>
																</EXPRESSION>
															</SELECTLIST>												
													</RELATIONSHIP>
												</RELATIONSHIP>
												</TARGETENTITY>
												</SEARCH>");

                            ErrorXml = OmIObj.GetHierarchialData(sbSearchXML.ToString(), ref OutputXML);

                            if (ErrorXml == null)
                            {
                                xpDocTemp           = new XPathDocument(new StringReader(OutputXML));
                                xpNavVault          = xpDocTemp.CreateNavigator();
                                strVaultID          = xpNavVault.SelectSingleNode("VAULTCOLLECTION/VAULT/VAULTID").Value;
                                strRootSectionID    = xpNavVault.SelectSingleNode("VAULTCOLLECTION/VAULT/VAULT_ROOTSECTION/SECTION/SECTIONID").Value;
                                strSecurityPolicyID = xpNavVault.SelectSingleNode("VAULTCOLLECTION/VAULT/VAULT_ROOTSECTION/SECTION/SECTION_SECURITYPOLICY/SECURITYPOLICY/SECURITYPOLICYID").Value;
                                xpDocTemp = null;
                                // check whether the section with the name EfObj.PrimaryKeyValue already exists, if no then create it.
                                SectionExists = CheckIfSectionExists(EfObj.PrimaryKeyValue, out str_FinalSectionID);
                                if (!SectionExists)
                                {
                                    // Now Create a section in this Vault
                                    sbCreateSection     = new StringBuilder();                                    
                                    strSectionPath      = "MWB Vault\\" + EfObj.PrimaryKeyValue;
                                    sbCreateSection.AppendFormat(@"<SECTION>
																<NAME>{0}</NAME>
																<PATH>{1}</PATH>												  
																<SECTION_VAULT TYPE='RELATION' PARENTID='TRUE'>{2}</SECTION_VAULT>
																<SECTION_SUBSECTION TYPE='RELATION' PARENTID='TRUE'>{3}</SECTION_SUBSECTION>
																<SECTION_SECURITYPOLICY TYPE='RELATION'>{4}</SECTION_SECURITYPOLICY>
																<SECTION_OWNER TYPE='RELATION'>{5}</SECTION_OWNER>
													 </SECTION>", EfObj.PrimaryKeyValue, strSectionPath, strVaultID, strRootSectionID, strSecurityPolicyID, UserID.ToString());
                                    SectionXML  = sbCreateSection.ToString();
                                    ErrorXml    = SectionObj.CreateSection(ref SectionXML);
                                    if (ErrorXml != null)
                                    {
                                        strdict_AdditionalInfo = new StringDictionary();
                                        strdict_AdditionalInfo.Add("SECTIONXML", SectionXML);
                                        strdict_AdditionalInfo.Add("MOREINFO", "Error while creating section.");

                                        ALEngineErrorObj = new ALEngineError();
                                        ALEngineErrorObj.AppendError("SECTION_CREATION_FAILED", this.GetType().ToString(),
                                                                        "CopyDigitalAssets()", null, strdict_AdditionalInfo,
                                                                        null, ErrorXml);
                                        ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                        return ErrorXml;
                                    }
                                    // Read the SectionID
                                    xpDocTemp           = new XPathDocument(new StringReader(SectionXML));
                                    xpNavSec            = xpDocTemp.CreateNavigator();
                                    str_FinalSectionID  = xpNavSec.SelectSingleNode("SECTION/SECTIONID").Value;
                                }
                            }
                            else
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("INPUTXML", sbSearchXML.ToString());
                                strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData() while getting vault information.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("VAULT_READ_FAILED", this.GetType().ToString(),
                                                                "CopyDigitalAssets()", null, strdict_AdditionalInfo,
                                                                null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }
                        }
                    }

                    if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_FinalSectionID)))
                    {
                        DigitalAssetObj = new DigitalAsset(UserID, CurrentLanguage, TenantName);

                        DigitalAssetDoc.LoadXml(DigitalAssetsXML);
                        DigitalAssetNodeList = DigitalAssetDoc.DocumentElement.SelectNodes("DIGITALASSET");
                        if (DigitalAssetNodeList.Count > 0)
                        {
                            if (EfObj.TypeName.ToUpper() != "ACTIVITY")
                            {
                                ErrorXml = GetDigitalAssetTransferredDataXml(EfObj.PrimaryKeyValue, out str_DataTransferredXml);                                
                            }
                            if (ErrorXml == null)
                            {
                                DigitalAssetsXML = @"<INPUTXML>
                                                        <DIGITALASSETCOLLECTION>                                                            
	                                                        <ISCOPYCOMMENT>TRUE</ISCOPYCOMMENT>";


                                foreach (XmlNode DigitalAssetNode in DigitalAssetNodeList)
                                {
                                    DigitalAssetsXML += "<DIGITALASSET><DIGITALASSETID>" + DigitalAssetNode.SelectSingleNode("DIGITALASSETID").InnerText + "</DIGITALASSETID>";
                                    if (!String.IsNullOrEmpty(str_DataTransferredXml))
                                        DigitalAssetsXML += str_DataTransferredXml;
                                    if (EfObj.TypeName.ToUpper() == "ACTIVITY")
                                    {
                                        DigitalAssetsXML += "<ACTIVITY_ITEM TYPE='RELATION'>" + EfObj.PrimaryKeyValue + "</ACTIVITY_ITEM>";
                                    }
                                    else
                                    {
                                        DigitalAssetsXML += "<PROCESS_ITEM TYPE='RELATION'>" + EfObj.PrimaryKeyValue + "</PROCESS_ITEM>";
                                    }
                                    DigitalAssetsXML += "</DIGITALASSET>";
                                }                                
                                DigitalAssetsXML += @"</DIGITALASSETCOLLECTION>		                                              
                                            </INPUTXML>";
                                ErrorXml = SectionObj.Copy(Convert.ToUInt64(str_FinalSectionID), null, DigitalAssetsXML);
                                if (!String.IsNullOrEmpty(ErrorXml))
                                {
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", EfObj.PrimaryKeyValue);
                                    strdict_AdditionalInfo.Add("FINALSECTIONID", str_FinalSectionID);
                                    strdict_AdditionalInfo.Add("DIGITALASSETSXML", DigitalAssetsXML);
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while copying digital assets.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("DIGITALASSET_COPY_FAILED", this.GetType().ToString(),
                                                                    "CopyDigitalAssets()", null, strdict_AdditionalInfo,
                                                                    null, ErrorXml);
                                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                    return ErrorXml;
                                }
                            }
                            else if (!String.IsNullOrEmpty(ErrorXml))
                            {
                                strdict_AdditionalInfo = new StringDictionary();
                                strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", EfObj.PrimaryKeyValue);
                                strdict_AdditionalInfo.Add("MOREINFO", "Error while getting data transfer xml for digital assets.");

                                ALEngineErrorObj = new ALEngineError();
                                ALEngineErrorObj.AppendError("DATATRANSFER_CONFIG_READ_FAILED", this.GetType().ToString(),
                                                                "CopyDigitalAssets()", null, strdict_AdditionalInfo,
                                                                null, ErrorXml);
                                ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                return ErrorXml;
                            }                           
                        }                                             
                    }
					return ErrorXml;
			}

            private ERRORXML CreateAssociations(EntityFacetCollection EfObj, XML UpdateDataXML,ParameterMap ObjPM, String ProcessStatus, ref StringBuilder UpdateXML)
			{
				XmlDocument		UpdateXMLDoc			= null;											
				XmlNodeList		UpdateDataNodeList	    = null;				
				ERRORXML		ErrorXml				= null;												
				DigitalAsset	DigitalAssetObj			= null;				
				OMInterface		OmIObj					= null;
				Section			SectionObj				= null;	
				StringBuilder	sbSearchXML				= null;
                String          OutputXML               = null;                
                StringBuilder   sbIDList                = null;
                XmlDocument     xmlDoc_OutputXML        = null;
                XmlNodeList     Dis_AssetNodeList       = null;
                XPathNavigator  xpNav_OutputXML         = null;
                StringBuilder   sbXpath                 = null;

                StringDictionary    strdict_AdditionalInfo  = null;
                ALEngineError       ALEngineErrorObj        = null;

                UpdateXMLDoc = new XmlDocument();				
                OmIObj			= new OMInterface(UserID,CurrentLanguage,TenantName);
                SectionObj      = new Section(UserID, CurrentLanguage, TenantName);                    
                sbSearchXML		= new StringBuilder();
                sbXpath         = new StringBuilder(16);
                sbIDList        = new StringBuilder(16);
                
                DigitalAssetObj = new DigitalAsset(UserID, CurrentLanguage, TenantName);

                UpdateXMLDoc.LoadXml(UpdateDataXML);
                UpdateDataNodeList = UpdateXMLDoc.DocumentElement.SelectNodes(ObjPM.TypeName);
                
                #region Get old association                
                sbSearchXML.AppendFormat(@"<SEARCH>
												<TARGETOBJECT>{0}</TARGETOBJECT>
												<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
												<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
												<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>												
												<TARGETENTITY>
												<PRIVILEGENAME>READ</PRIVILEGENAME>
												<FILTERLIST>
													<FILTER>
														<EXPRESSION>{1}</EXPRESSION>
														<OPERATOR>EQUALTO</OPERATOR>
														<EXPRESSIONVALUE>{2}</EXPRESSIONVALUE>
													</FILTER>
												</FILTERLIST>
												<SELECTLIST>
												<EXPRESSION>
													<FIELDNAME>{1}</FIELDNAME>
													<ALIAS>{1}</ALIAS>
												</EXPRESSION>
												</SELECTLIST>
												<RELATIONSHIP NAME='{3}'>
													<SELECTLIST>
														<EXPRESSION>
															<FIELDNAME>{4}</FIELDNAME>
															<ALIAS>{4}</ALIAS>
														</EXPRESSION>
													</SELECTLIST>                                                    
												</RELATIONSHIP>
												</TARGETENTITY>
												</SEARCH>", EfObj.TypeName, EfObj.PrimaryKeyName, EfObj.PrimaryKeyValue, ObjPM.RelationshipName, ObjPM.TypePrimaryKeyName);

                ErrorXml = OmIObj.GetHierarchialData(sbSearchXML.ToString(), ref OutputXML);
                #endregion

                if (ErrorXml == null)
                {
                    xmlDoc_OutputXML = new XmlDocument();
                    xmlDoc_OutputXML.LoadXml(OutputXML);
                    xpNav_OutputXML = xmlDoc_OutputXML.CreateNavigator();

                    #region Check the ID's in new XML and compare it with old to make association
                    UpdateDataNodeList = UpdateXMLDoc.DocumentElement.SelectNodes(ObjPM.TypeName);
                    foreach (XmlNode UpdateNode in UpdateDataNodeList)
                    {
                        XmlNode AssetIDNode = xmlDoc_OutputXML.SelectSingleNode("//" + ObjPM.TypePrimaryKeyName + "[text()='" + UpdateNode.SelectSingleNode(ObjPM.TypePrimaryKeyName).InnerText + "']");
                        if (AssetIDNode == null)
                        {
                            UpdateXML.AppendFormat("<{0} TYPE='RELATION'>{1}</{0}>", ObjPM.RelationshipName, UpdateNode.SelectSingleNode(ObjPM.TypePrimaryKeyName).InnerText);
                        }
                        else
                        {
                            AssetIDNode.ParentNode.RemoveChild(AssetIDNode);
                        }
                    }
                    #endregion

                    #region Disassociate the old ID's
                    if (ProcessStatus != "COMPLETED")
                    {
                        Dis_AssetNodeList = xmlDoc_OutputXML.DocumentElement.SelectNodes("//" + ObjPM.TypePrimaryKeyName + "");
                        foreach (XmlNode UpdateNode in Dis_AssetNodeList)
                        {
                            UpdateXML.AppendFormat("<{0} TYPE='RELATION' DISASSOCIATE='TRUE'>{1}</{0}>", ObjPM.RelationshipName, UpdateNode.InnerText);
                        }
                    }
                    #endregion
                }
                else
                {
                    strdict_AdditionalInfo = new StringDictionary();
                    strdict_AdditionalInfo.Add("INPUTXML", sbSearchXML.ToString());
                    strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData().");

                    ALEngineErrorObj = new ALEngineError();
                    ALEngineErrorObj.AppendError("READ_FAILED", this.GetType().ToString(),
                                                    "CreateAssociations()", null, strdict_AdditionalInfo,
                                                    null, ErrorXml);
                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                }
                return ErrorXml;
			}

			/// <alsummary>
			/// <createdon>11 Sept 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method retrieves the entity type detials for an enity.</purpose>
			/// </alsummary>
			/// <param name="EntityTypeName">The name of the entity type for which details are to be retrieved</param>
			/// <param name="EntityTypeXML">The XML that contains the entity type details</param>
			/// <returns>An Error XML if an error occurs, else null</returns>
			private ERRORXML GetEntityTypeDetails(String EntityTypeName, ref String EntityTypeXML)
			{
				XML             InputXML                = null;				
				ERRORXML        ErrorXML                = null;
				OperationQueue  QMQueueObj              = null;
				Operation       OperObj                 = null;
				OMInterface     OMIObj                  = null;
				
					OMIObj					= new OMInterface(UserID,CurrentLanguage,TenantName);
					QMQueueObj              = new OperationQueue(false);
					OperObj                 = new Operation("ENTITYTYPE", "READBYNAME");
					OperObj.AddParameter("ENTITYTYPENAME", EntityTypeName);
					QMQueueObj.AddOperation(OperObj);
					InputXML                = QMQueueObj.Xml;
					ErrorXML                = OMIObj.GetTypeData(InputXML, ref EntityTypeXML);
					return ErrorXML;
			}

			/// <summary>
			/// <createdon>11 Sept 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method returns the Select List XML of fields for a relationship as required by GetHierarchicalData.</purpose>
			/// </summary>
			/// <param name="RelationshipName">The Name of relationship that will be returned as the root node of XML</param>
			/// <param name="EntityTypeXML">The entity type XML consists of the Field Collection of the Entity</param>
			/// <param name="StrRelationshipNodes">The return SelectList XML</param>
			/// <returns>an ErrorXML if an error occurs.</returns>
			private void GetSelectListXML(String RelationshipName, XML EntityTypeXML, ref String StrRelationshipNodes)
			{
				XPathDocument	xpDocEntityType			= null;
				XPathNavigator	xpNavEntityType			= null;				
				
					xpDocEntityType = new XPathDocument(new StringReader(EntityTypeXML));
					xpNavEntityType = xpDocEntityType.CreateNavigator();

					GetRelationshipNodesBasedOnFieldInformation(RelationshipName, xpNavEntityType, ref StrRelationshipNodes);
			}

			/// <summary>
			/// <createdon>11 Sept 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This method prepares the Select List XML of fields for a relationship as required by GetHierarchicalData.
			/// If the Field List contains a field of type File/Binder then the FileRequired attribute is aset</purpose>
			/// </summary>
			/// <param name="RelationshipName">The name of the relationship for which select List will be prepared</param>
			/// <param name="XPathNav">The XPath Navigator points to the </param>
			/// <param name="StrRelationshipNodes">The return SelectList XML</param>
			/// <returns>an ErrorXML if an error occurs.</returns>
			private void GetRelationshipNodesBasedOnFieldInformation(String RelationshipName,XPathNavigator XPathNav,ref String StrRelationshipNodes)
			{
				StringBuilder       SBRelationshipNodes     = null;
				XPathNodeIterator   XPathNodeItr            = null;
					            
					SBRelationshipNodes     = new StringBuilder(100);
	                
					if (XPathNav.SelectSingleNode("//FIELDCOLLECTION/FIELD[FIELDTYPE='FILE' or FIELDTYPE='BINDER']") == null)
						SBRelationshipNodes.AppendFormat(@"<RELATIONSHIP NAME='{0}'/>", RelationshipName);
					else
					{
						SBRelationshipNodes.AppendFormat(@"<RELATIONSHIP NAME='{0}'>", RelationshipName);
						//Explicity mention all fields(This is required as we need files in outbox)
						SBRelationshipNodes.Append("<SELECTLIST>");
						XPathNodeItr    = XPathNav.Select("//FIELDCOLLECTION/FIELD");
						while (XPathNodeItr.MoveNext())
						{
							if (XPathNodeItr.Current.SelectSingleNode("FIELDTYPE[text()='FILE' or text()='BINDER']") == null)
							{
								SBRelationshipNodes.AppendFormat(@"<EXPRESSION>
																<FIELDNAME>{0}</FIELDNAME>
																<ALIAS>{0}</ALIAS>
														  </EXPRESSION>",XPathNodeItr.Current.SelectSingleNode("FIELDNAME").Value);
							}
							else
							{
								SBRelationshipNodes.AppendFormat(@"<EXPRESSION>
																<FIELDNAME FileRequired='YES'>{0}</FIELDNAME>
																<ALIAS>{0}</ALIAS>
														  </EXPRESSION>", XPathNodeItr.Current.SelectSingleNode("FIELDNAME").Value);

							}
						}
						SBRelationshipNodes.Append("</SELECTLIST>");
						SBRelationshipNodes.Append(@"</RELATIONSHIP>");
					}

					StrRelationshipNodes = SBRelationshipNodes.ToString();
			}

			/// <summary>
			/// <createdon>15 Sept 08</createdon>
			/// <author>Mugdha Moolay</author>
			/// <purpose>This API tests whether A Section already exists, if yes returns true and the SectionID as an OUT parameter, else false.</purpose>
			/// </summary>
			/// <param name="strSectionName">The name of the section</param>
			/// <param name="strSectionID">The section ID will be set if the section alreay exists</param>
			/// <returns>True if the section exists else false.</returns>
			private Boolean CheckIfSectionExists(String strSectionName,out String strSectionID)
			{
				StringBuilder	sbSearchXML = null;
				OMInterface		OmIObj		= null;
				XML				OutputXML	= null;
				ERRORXML		ErrorXml	= null;
				XPathDocument	xpDocSection= null;
				XPathNavigator	xpNavSection= null;
				
					strSectionID = null;
					sbSearchXML = new StringBuilder();
                    sbSearchXML.Append(@"<SEARCH>
												<TARGETOBJECT>SECTION</TARGETOBJECT>
												<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
												<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
												<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
												<MAXRESULTSIZE>1000</MAXRESULTSIZE>
												<TARGETENTITY>
												<PRIVILEGENAME>READ</PRIVILEGENAME>
												<FILTERLIST>
													<FILTER>
														<EXPRESSION>NAME</EXPRESSION>
														<OPERATOR>EQUALTO</OPERATOR>
														<EXPRESSIONVALUE>" + strSectionName + @"</EXPRESSIONVALUE>
													</FILTER>
												</FILTERLIST>
												<SELECTLIST>
												<EXPRESSION>
													<FIELDNAME>SECTIONID</FIELDNAME>
													<ALIAS>SECTIONID</ALIAS>
												</EXPRESSION>
												</SELECTLIST>												
												</TARGETENTITY>
												</SEARCH>");
					OmIObj			= new OMInterface(UserID,CurrentLanguage,TenantName);
					ErrorXml		= OmIObj.GetHierarchialData(sbSearchXML.ToString(),ref OutputXML);

					if(ErrorXml == null)
					{
						xpDocSection = new XPathDocument(new StringReader(OutputXML));
						xpNavSection = xpDocSection.CreateNavigator();
						xpNavSection = xpNavSection.SelectSingleNode("SECTIONCOLLECTION/SECTION");
						if(xpNavSection != null)
						{
							strSectionID = xpNavSection.SelectSingleNode("SECTIONID").Value;
							return true;
						}
						return false;
					}
					return false;
			}

			/// <summary>
			/// <CreatedBy>Anand Mahadik</CreatedBy>
			/// <Purpose>This method is used to retrieve facet type hierarchy that can be associated with an given Entity type.
			/// </Purpose>
			/// <CreatedOn>14 October 2008</CreatedOn>
			/// </summary>
			/// <param name="EntityTypeName">Entity type name</param>
			/// <param name="AssociatedObjectXML">XML containing facet types that can be associated with the Entity type</param>
			/// <returns>On success method will return NULL else returns an error XML.</returns>
            private ERRORXML GetAssociatedFacetTypeHierarchy(String EntityTypeName, ref String AssociatedObjectXML)
			{
				ERRORXML        ErrorXML                = null;
				OperationQueue  QMQueueObj              = null;
				Operation       OperObj                 = null;
				OMInterface     OMIObj                  = null;
                XML             OperationXML            = null;

				   QMQueueObj       = new OperationQueue(false);
                   OperObj          = new Operation("RELATION", "READTYPEHIERARCHY");
                   OperObj.AddParameter("TYPENAME", EntityTypeName);
                   OperObj.AddParameter("OBJECTTYPE", "ENTITY");
                   OperObj.AddParameter("HIERARCHYLEVEL", "FACET");
                   QMQueueObj.AddOperation(OperObj);
                   OperationXML     = QMQueueObj.Xml;

                   OMIObj           = new OMInterface(UserID,CurrentLanguage, TenantName);
                   ErrorXML         = OMIObj.GetTypeData(OperationXML, ref AssociatedObjectXML);
                   return ErrorXML;
			}

			/// <alsummary>
			/// <createddate>7 OCT 08</createddate>
			/// <author>Mugdha Moolay</author>
			/// <purpose>The purpose of this API is to return the primary key names of facets associated with an entity</purpose>
			/// </alsummary>
			/// <param name="xpIterRelation">The Associated type XML that contains a list of facets associated with and entity</param>
			/// <param name="FacetTypeKeys">The return value contains the name of the facet and its primary key name</param>
			/// <returns>an error XML if an error occurs, null otherwise.</returns>
			private ERRORXML GetPrimaryKeyForAssociatedFacetTypes(XPathNodeIterator xpIterRelation,out String[] FacetTypeKeys)
			{
				XPathDocument	xpDocFacetType		= null;				
				XPathNavigator	xpNavFacetType		= null;				
				String			strContainsTypeID	= null;
				XML				ContainsTypeXML		= null;
				String			strContainsTypeName	= null;
				String			strPrimaryKeyName	= null;
				ERRORXML		ErrorXml			= null;
				int				Index				= 0;
				
					FacetTypeKeys		= new String[xpIterRelation.Count];
					while(xpIterRelation.MoveNext())
					{
						strContainsTypeName = xpIterRelation.Current.SelectSingleNode("CONTAINSTYPENAME").Value;
						strContainsTypeID	= xpIterRelation.Current.SelectSingleNode("CONTAINSTYPEID").Value;
						ErrorXml			= GetTypeDetails("FACETTYPE", "READBYID", "FACETTYPEID", strContainsTypeID, out ContainsTypeXML);
						if(ErrorXml == null)
						{
							xpDocFacetType = new XPathDocument(new StringReader(ContainsTypeXML));
							xpNavFacetType = xpDocFacetType.CreateNavigator();
							strPrimaryKeyName = xpNavFacetType.SelectSingleNode("TYPE/PRIMARYKEY").Value;
							FacetTypeKeys[Index] = strPrimaryKeyName;
							Index++;
						}
						else
							return ErrorXml;
					}
					return ErrorXml;
			}

            /// <summary>
        /// <CreatedBy>Mr. Anand Mahadik </CreatedBy>
        /// <Purpose>This method is used to remove primary key nodes from the Entity/Facet collection XML.
        /// </Purpose>
        /// <CreatedOn>14 Oct 2008</CreatedOn>
        /// </summary>
        /// <param name="PrimaryKeyName">Name of the primary key field of the Entity/Facet</param>
        /// <param name="RelationshipName">Relationship name</param>
        /// <param name="ObjectCollectionXMLDoc">XMLDocument object containing Entity(s)/Facet(s) nodes.
        /// </param>
        /// <returns>On success method will return NULL else returns an error XML.</returns>
            internal static void RemovePrimaryKeyNodes(String PrimaryKeyName, String RelationshipName, XmlDocument ObjectCollectionXMLDoc)
            {
                StringBuilder   XPath                       = null;
                XmlNodeList     TempRelationshipNodes       = null;
                XmlNodeList     PrimaryKeyNodes             = null;
                
                    XPath                       = new StringBuilder(15);
                    XPath.AppendFormat("//{0}", RelationshipName);
                    TempRelationshipNodes       = ObjectCollectionXMLDoc.SelectNodes(XPath.ToString());
                    
                    // Remove ID columns
                    foreach(XmlNode TempRelationshipNode in TempRelationshipNodes)
                    {
                        ((XmlElement)TempRelationshipNode).SetAttribute("TYPE","RELATION");
                        XPath.Remove(0, XPath.Length);
                        XPath.AppendFormat("*/{0}", PrimaryKeyName);
                        PrimaryKeyNodes = TempRelationshipNode.SelectNodes(XPath.ToString());
                        foreach (XmlNode PrimaryKeyNode in PrimaryKeyNodes)
                        {
                            PrimaryKeyNode.ParentNode.RemoveChild(PrimaryKeyNode);
                        }
                    }
            }

            /// <summary>
            /// <Author>Radha A</Author>
            /// <CreatedOn>14-05-2009</CreatedOn>
            /// <Purpose>This API is used to check whether the vault is exists or not.</Purpose>
            /// </summary>
            /// <param name="str_VaultName">The name of the vault to be checked.</param>
            /// <returns>if vault is exits returns true otherwise false.</returns>
            private Boolean IsVaultExists(String str_VaultName)
            {
                StringBuilder           sb_SearchInputXml   = null;
                String                  str_ReturnXML       = null;
                ERRORXML                ErrorXml            = null;
                OMInterface             OmIObj              = null;
                
                    sb_SearchInputXml = new StringBuilder(300);
                    OmIObj			  = new OMInterface(UserID, CurrentLanguage, TenantName);

                    sb_SearchInputXml.Append(@"<SEARCH>
                                                    <SESSIONINFO TenantName='" + TenantName+ "' Language='" + CurrentLanguage + "' UserID='" + UserID.ToString() + @"' />
                                                    <TARGETOBJECT>VAULT</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
                                                    <TARGETENTITY>
	                                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                        <FILTERLIST>
                                                            <FILTER>
                                                                <FILTERID>1</FILTERID>
                                                                <EXPRESSION>NAME</EXPRESSION>
                                                                <OPERATOR>EQUALTO</OPERATOR>
                                                                <EXPRESSIONVALUE>" + str_VaultName + "</EXPRESSIONVALUE>"
                                                            + @"<ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>VAULTID</FIELDNAME>
                                                                <ALIAS>VAULTID</ALIAS>
                                                            </EXPRESSION>
                                                        </SELECTLIST>
                                                    </TARGETENTITY>
                                                </SEARCH>");
                    
                    ErrorXml        =  OmIObj.GetHierarchialData(sb_SearchInputXml.ToString(), ref str_ReturnXML);
                    if (String.IsNullOrEmpty(ErrorXml))
                    {
                        if (str_ReturnXML.IndexOf("<VAULTID>") > 0)
                        {
                            return true;
                        }
                    }
                    return false;
            }

            /// <summary>
            /// <Author>Radha A</Author>
            /// <CreatedOn>14-05-2009</CreatedOn>
            /// <Purpose>This API is used to get the data xml of process.</Purpose>
            /// </summary>
            /// <param name="str_ProcessID">The Process ID</param>
            /// <param name="str_PackageDefID">Package definition id </param>
            /// <param name="str_ProcessDefID">Process definition id</param>
            /// <param name="str_ProcessDefinitionXml">Definition xml of the process </param>
            /// <param name="str_ProcessDataXml">The data xml of the process</param>
            /// <returns>On success method will return NULL else returns an error XML.</returns>
            private ERRORXML GetProcessData(String str_ProcessID, String str_PackageDefID, String str_ProcessDefID, String str_ProcessDefinitionXml, out String str_ProcessDataXml)
            {
                String                      str_ProcessAssocatedFacets      = null;
                StringBuilder               sb_AssociationXml               = null;
                StringBuilder               sb_AttributeXml                 = null;
                ERRORXML                    ErrorXml                        = null;
                
                    str_ProcessDataXml   = null;
                    sb_AssociationXml    = new StringBuilder(200);
                    sb_AttributeXml      = new StringBuilder(100);

                    ErrorXml            = GetAssociatedAdditionalFacets(str_PackageDefID, str_ProcessDefID, str_ProcessDefinitionXml, false, out str_ProcessAssocatedFacets);
                    if (!String.IsNullOrEmpty(str_ProcessAssocatedFacets))
                    {
                        sb_AssociationXml.Append("<ASSOCIATION>" + str_ProcessAssocatedFacets);
                    }
                    else
                    {
                        sb_AssociationXml.Append("<ASSOCIATION GetAllFacets='TRUE'>");
                    }
                    sb_AssociationXml.Append(@"</ASSOCIATION>");
                    
                    sb_AttributeXml.Append(@"<WORKFLOWPROCESS>
                                                <WORKFLOWPROCESSID/>
                                             </WORKFLOWPROCESS>");

                    ErrorXml        = GetProcess(Convert.ToUInt64(str_ProcessID), sb_AssociationXml.ToString(), sb_AttributeXml.ToString(), out str_ProcessDataXml);
                    return ErrorXml;
            }

            /// <summary>
            /// <Author>Radha A</Author>
            /// <CreatedOn>14-05-2009</CreatedOn>
            /// <Purpose>This API is used to get the associated facets with the process.</Purpose>
            /// </summary>
            /// <param name="str_PackageDefID">Package definition id</param>
            /// <param name="str_ProcessDefID">Process definition id</param>
            /// <param name="str_ProcessDefinitionXml">The definititon xml of the process</param>
            /// <param name="str_ProcessAssocatedFacets">The associated facets xml</param>
            /// <param name="b_StdFacetsReq">This boolean value indicates whether standard facets which is associated with the process is required or not</param>
            /// <returns>On success method will return NULL else returns an error XML.</returns>
            private ERRORXML GetAssociatedAdditionalFacets(String str_PackageDefID, String str_ProcessDefID, String str_ProcessDefinitionXml, Boolean b_StdFacetsReq, out String str_ProcessAssocatedFacets)
            {
                ERRORXML            ErrorXMLString              = null;            
                XPathDocument       xp_ProcessDefinition        = null; 
                XPathNavigator      nav_WFProcess               = null;
                XPathNodeIterator   iter_ProcessAssoFacets      = null;
                StringBuilder       sb_ProcessAssociatedFacets  = null; 
                                                
                    str_ProcessAssocatedFacets      = null;
                    sb_ProcessAssociatedFacets      = new StringBuilder(100);

                    if (String.IsNullOrEmpty(str_ProcessDefinitionXml))
                    {
                        //Get the Process Definition
                        ErrorXMLString = GetProcessDefinitionByID(Convert.ToUInt64(str_PackageDefID), str_ProcessDefID, out str_ProcessDefinitionXml);
                    }
                    if ((String.IsNullOrEmpty(ErrorXMLString)) && (!String.IsNullOrEmpty(str_ProcessDefinitionXml)))
                    {
                        xp_ProcessDefinition    = new XPathDocument(new StringReader(str_ProcessDefinitionXml));
                        nav_WFProcess           = xp_ProcessDefinition.CreateNavigator().SelectSingleNode("WORKFLOWPROCESS");
                        if (nav_WFProcess != null)
                        {
                            if (b_StdFacetsReq)
                            {
                                //put process related standard facets
                                sb_ProcessAssociatedFacets.Append(@"<PROCESS_INFORMATION Type='RELATION' />
                                                                <WORKFLOWPROCESS_WORKFLOWINFORMATION Type='RELATION' />");
                            }
                            iter_ProcessAssoFacets = nav_WFProcess.Select("PROCESS_RELATEDFACETS/FACET");
                            while (iter_ProcessAssoFacets.MoveNext())
                            {
                                if ((iter_ProcessAssoFacets.Current.SelectSingleNode("RELATIONSHIPNAME") != null) && (!String.IsNullOrEmpty(iter_ProcessAssoFacets.Current.SelectSingleNode("RELATIONSHIPNAME").Value)))
                                {
                                    sb_ProcessAssociatedFacets.Append("<" + iter_ProcessAssoFacets.Current.SelectSingleNode("RELATIONSHIPNAME").Value + " Type='RELATION' GetAllFacets='TRUE' />");
                                }
                            }
                            str_ProcessAssocatedFacets = sb_ProcessAssociatedFacets.ToString();  
                        }
                    }
                    return ErrorXMLString;
            }

            /// <summary>
            /// <Author>Radha A</Author>
            /// <CreatedOn>01 June 2009</CreatedOn>
            /// </summary>
            /// <param name="str_ProcessID">Process ID</param>
            /// <param name="str_OutputXml">The digital assets transfer data xml.</param>
            /// <returns></returns>
            private ERRORXML GetDigitalAssetTransferredDataXml(String str_ProcessID, out String str_OutputXml)
            {
                DataTransfer                obj_DataTransfer                = null;
                StringBuilder               sb_FilterXml                    = null;
                String                      str_TransferredXml              = null;
                XPathDocument               xpathDoc_AssetDataTransferXml   = null;
                ERRORXML                    ErrorXml                        = null;
                String                      str_SolutionName                = null;
                String                      str_PackageDefID                = null;
                String                      str_ProcessDefID                = null;

                StringDictionary            strdict_AdditionalInfo          = null;
                ALEngineError               ALEngineErrorObj                = null;  
                                
                    str_OutputXml       = null;
                    //Get the process definition id.
                    ErrorXml            = GetProcessInformation(str_ProcessID, out str_PackageDefID, out str_ProcessDefID, out str_SolutionName);

                    if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_ProcessDefID)))
                    {
                        sb_FilterXml = new StringBuilder(100);
                        sb_FilterXml.Append(@"<FILTERLIST>
                                            <FILTER>
                                                <FILTERID>1</FILTERID>
                                                <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>" + str_ProcessID + @"</EXPRESSIONVALUE>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                            </FILTER>			                                                
                                        </FILTERLIST>");

                        if ((!String.IsNullOrEmpty(str_SolutionName)) && (str_SolutionName.ToUpper() == "MARKETING WORKBENCH"))
                        {
                            str_SolutionName    = "MarketingWorkBench";
                            obj_DataTransfer = new DataTransfer(SessionID.ToString(), UserID.ToString(), TenantName, CurrentLanguage, str_SolutionName);
                            obj_DataTransfer.GetFilesToSolutionMachine = false;
                            obj_DataTransfer.ConfigurationName = str_ProcessDefID + "_DIGITALASSET";
                            if (obj_DataTransfer.IsConfigurationExist())
                            {
                                obj_DataTransfer.AddSourceFilter("SRC_TARGET_FILTER", sb_FilterXml.ToString());
                                obj_DataTransfer.SolutionName = str_SolutionName;

                                //get the asset data transfer xml.
                                ErrorXml = obj_DataTransfer.GetTransferredDataXML("CREATE", out str_TransferredXml);
                                if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_TransferredXml)))
                                {
                                    xpathDoc_AssetDataTransferXml = new XPathDocument(new StringReader(str_TransferredXml));
                                    str_OutputXml = xpathDoc_AssetDataTransferXml.CreateNavigator().SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET").InnerXml;
                                }
                                else if (!String.IsNullOrEmpty(ErrorXml))
                                {
                                    strdict_AdditionalInfo = new StringDictionary();
                                    strdict_AdditionalInfo.Add("CONFIGURATIONNAME", str_ProcessDefID + "_DIGITALASSET");
                                    strdict_AdditionalInfo.Add("MOREINFO", "Error while getting asset data transfer xml.");

                                    ALEngineErrorObj = new ALEngineError();
                                    ALEngineErrorObj.AppendError("DATATRANSFER_CONFIG_READ_FAILED", this.GetType().ToString(),
                                                                    "GetDigitalAssetTransferredDataXml()", null, strdict_AdditionalInfo,
                                                                    null, ErrorXml);
                                    ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                                    return ErrorXml;
                                }
                            }
                        }
                    }
                    return ErrorXml;
            }

            private ERRORXML GetProcessInformation(String str_ProcessID, out String str_PackageDefID, out String str_ProcessDefID, out String str_SolutionName)
            {
                StringBuilder   sb_SearchXML            = null;
                ERRORXML        ErrorXml                = null;
                XPathDocument   xp_OutputXML            = null;
                String          str_OutputXML           = null;
                XPathNavigator  nav_ProcessInfo         = null;
                OMInterface     OmIObj                  = null;
                XPathNavigator  nav_Process             = null;

                StringDictionary    strdict_AdditionalInfo  = null;
                ALEngineError       ALEngineErrorObj        = null;
                
                
                    str_PackageDefID    = null;
                    str_ProcessDefID    = null;
                    str_SolutionName    = null;
                    sb_SearchXML        = new StringBuilder(500);
                    OmIObj              = new OMInterface(UserID, CurrentLanguage, TenantName);
                    sb_SearchXML.Append(@"<SEARCH>
                                                <SESSIONINFO UserID='" + UserID.ToString() + "' TenantName='" + TenantName + "' Language='" + CurrentLanguage + " '/>"
                                                + @"<TARGETOBJECT>WORKFLOWPROCESS</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
                                                <TARGETENTITY>
                                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
	                                                    <FILTER>
		                                                    <EXPRESSION>WORKFLOWPROCESSID</EXPRESSION>
		                                                    <OPERATOR>EQUALTO</OPERATOR>
		                                                    <EXPRESSIONVALUE>" + str_ProcessID + "</EXPRESSIONVALUE>"
                                                        + @"<FILTERID>1</FILTERID>
		                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                    </FILTER>
                                                    </FILTERLIST>
                                                    <SELECTLIST>
	                                                    <EXPRESSION>
		                                                    <FIELDNAME>WORKFLOWPROCESSID</FIELDNAME>
		                                                    <ALIAS>WORKFLOWPROCESSID</ALIAS>
	                                                    </EXPRESSION>                                                  
	                                                    <EXPRESSION>
		                                                    <FIELDNAME>SOLUTIONNAME</FIELDNAME>
		                                                    <ALIAS>SOLUTIONNAME</ALIAS>
	                                                    </EXPRESSION>
                                                    </SELECTLIST>
                                                    <RELATIONSHIP NAME='PROCESS_INFORMATION'>
                                                        <SELECTLIST>
			                                                 <EXPRESSION>
				                                                 <FIELDNAME>PACKAGEDEFINITIONID</FIELDNAME>
				                                                 <ALIAS>PACKAGEDEFINITIONID</ALIAS>
			                                                 </EXPRESSION>
			                                                 <EXPRESSION>
				                                                 <FIELDNAME>PROCESSDEFINITIONID</FIELDNAME>
				                                                 <ALIAS>PROCESSDEFINITIONID</ALIAS>
			                                                  </EXPRESSION>
                                                         </SELECTLIST>
                                                    </RELATIONSHIP>
                                                </TARGETENTITY>
                                            </SEARCH>");
                    ErrorXml = OmIObj.GetHierarchialData(sb_SearchXML.ToString(), ref str_OutputXML);

                    if ((String.IsNullOrEmpty(ErrorXml)) && (!String.IsNullOrEmpty(str_OutputXML)))
                    {
                        xp_OutputXML        = new XPathDocument(new StringReader(str_OutputXML));
                        nav_Process         = xp_OutputXML.CreateNavigator().SelectSingleNode("WORKFLOWPROCESSCOLLECTION/WORKFLOWPROCESS");
                        str_SolutionName    = nav_Process.SelectSingleNode("SOLUTIONNAME").Value;
                        nav_ProcessInfo     = nav_Process.SelectSingleNode("PROCESS_INFORMATION/WORKFLOWPROCESSINFO");
                        if (nav_ProcessInfo != null)
                        {
                            str_PackageDefID = nav_ProcessInfo.SelectSingleNode("PACKAGEDEFINITIONID").Value;
                            str_ProcessDefID = nav_ProcessInfo.SelectSingleNode("PROCESSDEFINITIONID").Value;
                        }
                    }
                    else if (!String.IsNullOrEmpty(ErrorXml))
                    {
                        strdict_AdditionalInfo = new StringDictionary();
                        strdict_AdditionalInfo.Add("WORKFLOWPROCESSID", str_ProcessID);
                        strdict_AdditionalInfo.Add("INPUTXML", sb_SearchXML.ToString());
                        strdict_AdditionalInfo.Add("MOREINFO", "Error in GetHierarchialData() while getting process information.");

                        ALEngineErrorObj = new ALEngineError();
                        ALEngineErrorObj.AppendError("PROCESS_READ_FAILED", this.GetType().ToString(),
                                                        "GetProcessInformation()", null, strdict_AdditionalInfo,
                                                        null, ErrorXml);
                        ErrorXml = ALEngineErrorObj.ErrorCollectionXML;
                        return ErrorXml;
                    }
                    return ErrorXml;
        }

            private ERRORXML GetFile(String FileXML, String FieldName, String DownloadPath, String OrgDisplayFileName)
            {
                
                StringBuilder   sb_FileSystemXML    = null;
                String          ErrorXML            = null;
                String          FileSystemXML       = null;
                XPathDocument   xpDoc_File          = null;
                XPathNavigator  xpNav_File          = null;
                ALFile          Obj_ALFile          = null;

                Obj_ALFile          = new ALFile(UserID,CurrentLanguage,TenantName);
                sb_FileSystemXML    = new StringBuilder(100);
                xpDoc_File          = new XPathDocument(new StringReader(FileXML));
                xpNav_File          = xpDoc_File.CreateNavigator();

                if (xpNav_File != null)
                {
                    sb_FileSystemXML.AppendFormat("<FILESYSTEM ObjectID=\"{0}\">", xpNav_File.SelectSingleNode("FILESYSTEM/FILE/OBJECTID").Value);
                    sb_FileSystemXML.AppendFormat("<FILE FileRequired=\"Yes\">{0}</FILE></FILESYSTEM>", xpNav_File.SelectSingleNode("FILESYSTEM/FILE").InnerXml);

                    FileSystemXML = sb_FileSystemXML.ToString();
                    ErrorXML = Obj_ALFile.GetFile(ref FileSystemXML);
                    if (ErrorXML != null)
                        return ErrorXML;
                    
                    xpDoc_File = new XPathDocument(new StringReader(FileSystemXML));
                    xpNav_File = xpDoc_File.CreateNavigator();

                    File.Copy(xpNav_File.SelectSingleNode("FILESYSTEM/FILE/PATH").Value, DownloadPath);
                }
                return null;
            }

            /// <summary>
            /// <CreatedOn>3 Nov 09</CreatedOn>
            /// <Author>Mugdha Moolay Gadre</Author>
            /// <Purpose>This method gets all the digital assets associated with the Target entity</Purpose>
            /// </summary>
            /// <param name="EfObj">This object represents an entity or facet</param>
            /// <param name="outputXML">return output xml</param>
            /// <param name="TotalRecords"></param>
            /// <returns>An Error XML if an error occurs.</returns>
            private ERRORXML GetDigitalAssets(EntityFacetCollection EfObj, out XML outputXML,out Int32 TotalRecords)
            {
                StringBuilder   sb_Search       = null;
                String          str_RelName     = null;
                OMInterface     obj_Omi         = null;            
                ERRORXML        ErrorXml        = null;
                XPathDocument   xpDoc_DAssets   = null;
                XPathNavigator  xpNav_Assets    = null;
                TotalRecords                    = 0;
                
                    outputXML   = null;
                    sb_Search   = new StringBuilder(100);
                    obj_Omi     = new OMInterface(UserID, CurrentLanguage, TenantName);
                    if (EfObj.TypeName.ToUpper() == "ACTIVITY")
                        str_RelName = "ACTIVITY_ITEM";
                    else if (EfObj.TypeName.ToUpper() == "WORKFLOWPROCESS")
                        str_RelName = "PROCESS_ITEM";

                    sb_Search.AppendFormat(@"<SEARCH>
                                                <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
											    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
											    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
											    <SEARCHALLFACETS>FALSE</SEARCHALLFACETS>
											    <MAXRESULTSIZE>1000</MAXRESULTSIZE>
											    <TARGETENTITY>
											    <PRIVILEGENAME>READ</PRIVILEGENAME>
											    <FILTERLIST>
												    <FILTER>
													    <EXPRESSION>{0}</EXPRESSION>
													    <OPERATOR>EQUALTO</OPERATOR>
													    <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>
                                                        <RELATIONSHIPNAME>{2}</RELATIONSHIPNAME>
												    </FILTER>
											    </FILTERLIST>
                                                <SELECTLIST>
                                                    <EXPRESSION>
										                <FIELDNAME>DIGITALASSETID</FIELDNAME>
										                <ALIAS>DIGITALASSETID</ALIAS>
										            </EXPRESSION>
                                                </SELECTLIST>                                            
                                                </TARGETENTITY>
                                            </SEARCH>", EfObj.PrimaryKeyName, EfObj.PrimaryKeyValue, str_RelName);
                    ErrorXml = obj_Omi.GetHierarchialData(sb_Search.ToString(), ref outputXML);
                    if (ErrorXml == null)
                    {
                        xpDoc_DAssets = new XPathDocument(new StringReader(outputXML));
                        xpNav_Assets  = xpDoc_DAssets.CreateNavigator().SelectSingleNode("DIGITALASSETCOLLECTION");
                        TotalRecords  = Convert.ToInt32(xpNav_Assets.GetAttribute("TotalRecordCount", ""));
                    }
                    return ErrorXml;
            }
		#endregion
	}
}
