using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Collections.Specialized;

using Assetlink;
using Assetlink.DataAccessLayer;
using Assetlink.FileAccessLayer;

namespace Assetlink
{
    //----------------------------------------------------------------------------------------------------- //
    // File Name: XMLManager.cs                                                                             //   
    // Author   : Mr. Kiran S. Abhang                                                                       // 
    // Class    : XMLManager                                                                                //
    // Date     : 08th March 2006 And 09th March 2006                                                       //
    // Purpose  : Core class to support XML files and XSD files for XML data type in Object Manager.        //
    //----------------------------------------------------------------------------------------------------- //
    
    public class XMLManager : AssetlinkBase, IXMLRepository
    {
        #region Member variables
        private DataManager     m_DataManager       = null;
        DataManager             LOGDMObj            = null;
        ulong                   TransactionID       = 0;
        String                  SHADOW_FOLDER       = "SHADOW_FOLDER";
        WindowsFileSystem       WAManager          = null;
        #endregion

        #region Properties 

        //public DataManager OMDataManager
        //{
        //    get { return m_DataManager; }
        //    set 
        //    { 
        //        m_DataManager = value;
        //        LOGDMObj = ((FileAccessLayer.FileSystemManager)m_DataManager.FileSystemManager).LogDataManager;
        //        TransactionID = ((FileAccessLayer.FileSystemManager)m_DataManager.FileSystemManager).TranscationId;
        //    }
        //}

        public ulong TranscationId
        {
            get { return TransactionID; }
            set { TransactionID = value; }
        }

        #endregion

        #region Constructor

        public XMLManager()
        {
            WAManager = new WindowsFileSystem();
        }

        public XMLManager(ulong UserID, string CurrentLanguage, string TenantName)
            : base(UserID, CurrentLanguage, TenantName)
        {
            WAManager = new WindowsFileSystem();
        }

        public XMLManager(string TenantName)
            : base(TenantName)
        {
            WAManager = new WindowsFileSystem();
        }

        #endregion

        public void Init(DataManager dataManager)
        {
            m_DataManager = dataManager;
            LOGDMObj = ((FileAccessLayer.FileSystemManager)m_DataManager.FileSystemManager).LogDataManager;
            TransactionID = ((FileAccessLayer.FileSystemManager)m_DataManager.FileSystemManager).TranscationId;
        }

        /// <summary>
        /// This method is used to create XML file and write XML data into the new created file.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// File path is created with the combination of root path, TypeName, ObjectId and FieldName.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ObjectID">ID of an object. Object can be an Entity or Facet.</param>
        /// <param name="FieldName">This is the name of field which is declared as XML type.</param>
        /// <param name="FileData">Contents of file.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it creates XML file at specified location and writes the passed data (contents) into the file.
        /// </returns>
        public OMError Save(String TypeName, ulong ObjectID, String FieldName, String FileData)
        {
            OMError         ObjOMError      = null;
            StreamWriter    ObjStreamWriter = null;
            StringBuilder   FilePath        = null;
            StringBuilder   DirectoryPath   = null;

            StringBuilder   DirectoryPathShadow = null;
            StringCollection Parameters     = new StringCollection();
            StringBuilder   FilePathShadow  = null;
            String          ParameterXML    = null;
            String          DurableLogXML   = null;
            OMError         ErrObj          = null;
            OMError         ReturnErrObj    = null;

            try
            {
                if (ObjectID < 0)
                {
                    return new OMError(OMErrorCodes.XMLMANAGER_SAVE_FAILED, "XML File Save Failed. Invalid ObjectId : " + Convert.ToString(ObjectID),
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.ERROR, null, null);
                }
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName, out DirectoryPath);
                

                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    //Create a directory at specified location.
                    //Check whether the directory is already present or not at the specified location.
                    DirectoryPath.AppendFormat("\\{0}", ObjectID);
                    if (WAManager.IsFolderExists(DirectoryPath.ToString()) == false)
                    {
                        ObjOMError = WAManager.CreateFolder(DirectoryPath.ToString());
                        if (ObjOMError != null)
                            return ObjOMError;
                    }

                    FieldName = FieldName.Trim().ToUpper();
                    FilePath = new StringBuilder();
                    FilePath.AppendFormat("{0}\\{1}.xml", DirectoryPath.ToString(), FieldName);

                    //Check the contents of file contains NULL or empty value. If it doesnot contain NULL or empty value
                    // then write the the data into the file. At the same time file is created.
                    //If file is already present then the cotents are overwritten into the file.
                    //If file contents is NULL or empty then delete the already created XML file.
                    if ((FileData != null) && (FileData.Trim().Length > 0))
                    {
                        //if File Exists then UpdateFile case
                        //else CreateFile case
                        if (WAManager.IsFileExists(FilePath.ToString()))
                        {
                            GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);
                            //UPDATEFILE
                            FilePathShadow = new StringBuilder();
                            FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, ObjectID);
                            if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                            {
                                ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            FilePathShadow.AppendFormat("\\{0}.xml", FieldName.Trim());
                            ObjOMError = WAManager.CopyFile(FilePath.ToString(), FilePathShadow.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                            Parameters.Clear();
                            Parameters.Add(FilePath.ToString());
                            Parameters.Add(FilePathShadow.ToString());

                            FormPamareterXML(Parameters, ref ParameterXML);
                            FormDurableLogXML(TransactionID, "UpdateFile", ParameterXML, "PREPARED", ref DurableLogXML);
                            if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                            "Writing into ALFS_DURABLELOG Failed",
                                                            OMErrorObjectType.FILESYSTEM,
                                                            "FileSystemManager", ErrObj.ErrorType,
                                                            ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }

                        }
                        else
                        {
                            GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), false, out DirectoryPathShadow);
                            //CREATEFILE
                            Parameters.Clear();
                            Parameters.Add(FilePath.ToString());
                            FormPamareterXML(Parameters, ref ParameterXML);
                            FormDurableLogXML(TransactionID, "CreateFile", ParameterXML, "PREPARED", ref DurableLogXML);
                            if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                            "Writing into ALFS_DURABLELOG Failed",
                                                            OMErrorObjectType.FILESYSTEM,
                                                            "FileSystemManager", ErrObj.ErrorType,
                                                            ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                        }
                        
                        ObjStreamWriter = File.CreateText(FilePath.ToString());
                        ObjStreamWriter.WriteLine(FileData);
                        ObjStreamWriter.Close();
                    }
                    else if (WAManager.IsFileExists(FilePath.ToString()) == true)
                    {
                        GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);

                        FilePathShadow = new StringBuilder();
                        FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, ObjectID);
                        if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                        {
                            ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }

                        FilePathShadow.AppendFormat("\\{0}.xml", FieldName.Trim());
                        ObjOMError = WAManager.MoveFile(FilePath.ToString(), FilePathShadow.ToString());
                        if (ObjOMError != null)
                            return ObjOMError;
                        Parameters.Clear();
                        Parameters.Add(FilePath.ToString());
                        Parameters.Add(FilePathShadow.ToString());

                        FormPamareterXML(Parameters, ref ParameterXML);
                        FormDurableLogXML(TransactionID, "DeleteFile", ParameterXML, "PREPARED", ref DurableLogXML);
                        if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                        "Writing into ALFS_DURABLELOG Failed",
                                                        OMErrorObjectType.FILESYSTEM,
                                                        "FileSystemManager", ErrObj.ErrorType,
                                                        ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }

                        //DeleteXML File case
                        //File.Delete(FilePath.ToString());
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_SAVE_FAILED, "XML File Save Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager", 
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to get (read) the contents from XML file.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// File path is created with the combination of root path, TypeName, ObjectId and FieldName.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ObjectID">ID of an object. Object can be an Entity or Facet.</param>
        /// <param name="FieldName">This is the name of field which is declared as XML type.</param>
        /// <param name="FileData">Contents of file. It is an 'out' paramteter.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it reads the contents from XML file and assigned it to the 'out' parameter that is FileData.
        /// </returns>
        public OMError Load(String TypeName, ulong ObjectID, String FieldName, out String FileData)
        {
            OMError         ObjOMError      = null;
            StreamReader    ObjStreamReader = null;
            StringBuilder   DirectoryPath   = null;
            StringBuilder   FilePath        = null;

            //Initialize 'out' parameter.
            FileData = null;

            try
            {
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName, out DirectoryPath);

                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    FieldName = FieldName.Trim().ToUpper();
                    FilePath = new StringBuilder();
                    FilePath.AppendFormat("{0}\\{1}\\{2}.xml", DirectoryPath.ToString(), ObjectID, FieldName);

                    //Check whether the file is exist at the given path or not.
                    //If exists then read the contents of the file.
                    if (WAManager.IsFileExists(FilePath.ToString()) == true)
                    {
                        ObjStreamReader = new StreamReader(FilePath.ToString());
                        FileData = ObjStreamReader.ReadToEnd();
                        ObjStreamReader.Close();
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_SAVE_FAILED, "XML File Load Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete the XML file.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// File path is created with the combination of root path, TypeName, ObjectId and FieldName.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ObjectID">ID of an object. Object can be an Entity or Facet.</param>
        /// <param name="FieldName">This is the name of field which is declared as XML type.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes the XML file from the specified location.
        /// </returns>
        public OMError DeleteXML(String TypeName, ulong ObjectID, String FieldName)
        {
            OMError         ObjOMError      = null;
            StringBuilder   DirectoryPath   = null;
            StringBuilder   FilePath        = null;
            StringBuilder   DirectoryPathShadow = null;
            StringCollection Parameters     = new StringCollection();
            StringBuilder   FilePathShadow  = null;
            String          ParameterXML    = null;
            String          DurableLogXML   = null;
            OMError         ErrObj          = null;
            OMError         ReturnErrObj    = null;
            try
            {
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName.Trim().ToUpper(), out DirectoryPath);
                
                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    FilePath = new StringBuilder();
                    FilePath.AppendFormat("{0}\\{1}\\{2}.xml", DirectoryPath.ToString(), ObjectID, FieldName.Trim().ToUpper());

                    //Check whther the XML file is present at the specified location. If file is present then delete it.
                    if (WAManager.IsFileExists(FilePath.ToString()) == true)
                    {
                        GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);
                        FilePathShadow = new StringBuilder();
                        FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, ObjectID);
                        if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                        {
                            ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }

                        FilePathShadow.AppendFormat("\\{0}.xml", FieldName.Trim());
                        ObjOMError = WAManager.MoveFile(FilePath.ToString(), FilePathShadow.ToString());
                        if (ObjOMError != null)
                            return ObjOMError;
                        Parameters.Add(FilePath.ToString());
                        Parameters.Add(FilePathShadow.ToString());

                        FormPamareterXML(Parameters, ref ParameterXML);
                        FormDurableLogXML(TransactionID, "DeleteFile", ParameterXML, "PREPARED", ref DurableLogXML);
                        if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                        "Writing into ALFS_DURABLELOG Failed",
                                                        OMErrorObjectType.FILESYSTEM,
                                                        "FileSystemManager", ErrObj.ErrorType,
                                                        ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }

                        //File.Delete(FilePath.ToString());
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XML Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete the XML file.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// File path is created with the combination of root path, TypeName, ObjectId and FieldName.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ArrObjectID">This is the list of IDs of object. Object can be an Entity or Facet.</param>
        /// <param name="FieldName">This is the name of field which is declared as XML type.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes the XML file from the specified location.
        /// </returns>
        public OMError DeleteXML(String TypeName, String[] ArrObjectID, String FieldName)
        {
            OMError         ObjOMError      = null;
            StringBuilder   DirectoryPath   = null;
            StringBuilder   FilePath        = null;
            StringBuilder   DirectoryPathShadow = null;
            StringCollection Parameters     = new StringCollection();
            StringBuilder   FilePathShadow  = null;
            String          ParameterXML    = null;
            String          DurableLogXML   = null;
            OMError         ErrObj          = null;
            OMError         ReturnErrObj    = null;

            try
            {
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName.Trim().ToUpper(), out DirectoryPath);
                

                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    FieldName = FieldName.Trim().ToUpper();
                    if(ArrObjectID.Length >0)
                        GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);
                    for (int ArrCount = 0; ArrCount < ArrObjectID.Length; ArrCount++)
                    {
                        FilePath = new StringBuilder();
                        FilePath.AppendFormat("{0}\\{1}\\{2}.xml", DirectoryPath.ToString(), ArrObjectID[ArrCount], FieldName);
                        //Check whther the XML file is present at the specified location. If file is present then delete it.
                        if (WAManager.IsFileExists(FilePath.ToString()) == true)
                        {
                            FilePathShadow = new StringBuilder();
                            FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, ArrObjectID[ArrCount]);
                            if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                            {
                                ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            FilePathShadow.AppendFormat("\\{0}.xml", FieldName.Trim());
                            ObjOMError = WAManager.MoveFile(FilePath.ToString(), FilePathShadow.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                            Parameters.Add(FilePath.ToString());
                            Parameters.Add(FilePathShadow.ToString());

                            FormPamareterXML(Parameters, ref ParameterXML);
                            FormDurableLogXML(TransactionID, "DeleteFile", ParameterXML, "PREPARED", ref DurableLogXML);
                            if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                            "Writing into ALFS_DURABLELOG Failed",
                                                            OMErrorObjectType.FILESYSTEM,
                                                            "FileSystemManager", ErrObj.ErrorType,
                                                            ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                            //File.Delete(FilePath.ToString());
                        }
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XML Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete all XML files of an object.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// File path is created with the combination of root path, TypeName and ObjectId.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ObjectID">This is the ID of an object. Object can be an Entity or Facet.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes all XML files of an object from the specified location.
        /// </returns>
        public OMError DeleteXML(String TypeName, ulong ObjectID)
        {
            OMError         ObjOMError      = null;
            StringBuilder   DirectoryPath   = null;
            StringBuilder   FilePath        = null;
            StringBuilder   DirectoryPathShadow = null;
            StringCollection Parameters     = new StringCollection();
            StringBuilder   FilePathShadow  = null;
            String          ParameterXML    = null;
            String          DurableLogXML   = null;
            OMError         ErrObj          = null;
            OMError         ReturnErrObj    = null;

            try
            {
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName.Trim().ToUpper(), out DirectoryPath);

                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    DirectoryPath.AppendFormat("\\{0}", ObjectID);
                    //If directory of an object is exist then delete the directory.
                    //It deletes all files and sub directories included in the object's directory.
                    if (WAManager.IsFolderExists(DirectoryPath.ToString()) == true)
                    {
                        DirectoryInfo dInfo = new DirectoryInfo(DirectoryPath.ToString());
                        FileInfo[] arrFiles = dInfo.GetFiles();
                        if(arrFiles.Length > 0)
                            GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);

                        for (int iFileCount = 0; iFileCount < arrFiles.Length; iFileCount++)
                        {
                            FilePath = new StringBuilder();
                            FilePath.AppendFormat("{0}\\{1}", DirectoryPath.ToString(), arrFiles[iFileCount].Name);

                            FilePathShadow = new StringBuilder();
                            FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, ObjectID);
                            if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                            {
                                ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            FilePathShadow.AppendFormat("\\{0}", arrFiles[iFileCount].Name);
                            ObjOMError = WAManager.MoveFile(FilePath.ToString(), FilePathShadow.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                            
                            Parameters.Clear();
                            Parameters.Add(FilePath.ToString());
                            Parameters.Add(FilePathShadow.ToString());
                            FormPamareterXML(Parameters, ref ParameterXML);
                            FormDurableLogXML(TransactionID, "DeleteFile", ParameterXML, "PREPARED", ref DurableLogXML);

                            if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                            "Writing into ALFS_DURABLELOG Failed",
                                                            OMErrorObjectType.FILESYSTEM,
                                                            "FileSystemManager", ErrObj.ErrorType,
                                                            ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }

                        }
                        ObjOMError = WAManager.DeleteFolder(DirectoryPath.ToString(), true);
                        if (ObjOMError != null)
                            return ObjOMError;
                        //Directory.Delete(DirectoryPath.ToString(), true);
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XML Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete all XML files of an object type.
        /// The root path of XML is taken from the client XML (XML mount point) where the application is installed.
        /// Directory path is created with the combination of root path and TypeName.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes all XML files of an object type from the specified location.
        /// </returns>
        public OMError DeleteXML(String TypeName)
        {
            OMError         ObjOMError      = null;
            StringBuilder   DirectoryPath   = null;
            StringBuilder   FilePath        = null;
            StringBuilder   DirectoryPathShadow = null;
            StringCollection Parameters     = new StringCollection();
            StringBuilder   FilePathShadow  = null;
            String          ParameterXML    = null;
            String          DurableLogXML   = null;
            OMError         ErrObj          = null;
            OMError         ReturnErrObj    = null;

            try
            {
                DirectoryPath = new StringBuilder();
                //Get the root path of XML directory from client XML (where the application is installaed).
                //This is XML mount point.
                GetXMLFileDirectoryPath(TypeName.Trim().ToUpper(), out DirectoryPath);
                

                if ((DirectoryPath != null) && (DirectoryPath.Length > 0))
                {
                    //If directory of an object type is exist then delete the directory.
                    //It deletes all files and sub directories included in the object type's directory.
                    if (WAManager.IsFolderExists(DirectoryPath.ToString()) == true)
                    {
                        DirectoryInfo dInfo = new DirectoryInfo(DirectoryPath.ToString());
                        DirectoryInfo[] dInfoSubDir = dInfo.GetDirectories();
                        if(dInfoSubDir.Length > 0)
                            GetXMLFileDirectoryPathForShadowFolder(TypeName.Trim().ToUpper(), true, out DirectoryPathShadow);
                        
                        for (int iDirCount = 0; iDirCount < dInfoSubDir.Length; iDirCount++)
                        {
                            FileInfo[] arrFiles = dInfoSubDir[iDirCount].GetFiles(); //Directory.GetFiles(arrDirectories[iDirCount]);

                            for (int iFileCount = 0; iFileCount < arrFiles.Length; iFileCount++)
                            {
                                FilePath = new StringBuilder();
                                FilePath.AppendFormat("{0}\\{1}\\{2}", DirectoryPath.ToString(), dInfoSubDir[iDirCount].Name, arrFiles[iFileCount].Name);

                                FilePathShadow = new StringBuilder();
                                FilePathShadow.AppendFormat("{0}\\{1}", DirectoryPathShadow, dInfoSubDir[iDirCount].Name);
                                if (!WAManager.IsFolderExists(FilePathShadow.ToString()))
                                {
                                    ObjOMError = WAManager.CreateFolder(FilePathShadow.ToString());
                                    if (ObjOMError != null)
                                        return ObjOMError;
                                }

                                FilePathShadow.AppendFormat("\\{0}", arrFiles[iFileCount].Name);
                                ObjOMError = WAManager.MoveFile(FilePath.ToString(), FilePathShadow.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;

                                Parameters.Clear();
                                Parameters.Add(FilePath.ToString());
                                Parameters.Add(FilePathShadow.ToString());
                                FormPamareterXML(Parameters, ref ParameterXML);
                                FormDurableLogXML(TransactionID, "DeleteFile", ParameterXML, "PREPARED", ref DurableLogXML);

                                if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                                {
                                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                                "Writing into ALFS_DURABLELOG Failed",
                                                                OMErrorObjectType.FILESYSTEM,
                                                                "FileSystemManager", ErrObj.ErrorType,
                                                                ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                    ReturnErrObj.TransactionId = TransactionID.ToString();
                                    return ReturnErrObj;
                                }
                            }
                        }
                        ObjOMError = WAManager.DeleteFolder(DirectoryPath.ToString(), true);
                        if (ObjOMError != null)
                            return ObjOMError;
                        //Directory.Delete(DirectoryPath.ToString(), true);
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XML Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to copy XSD file of an XML type field for an object type.
        /// The root path of XSD is taken from the client XML (XSD mount point) where the application is installed.
        /// Directory path is created with the combination of root path, TypeName and FieldName.
        /// XSD file is defined for each field of an object type, which has data type XML.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="FieldName">Name of field. This field is of type XML.</param>
        /// <param name="SourceXSDPath">This is the source path of XSD file.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it copies XSD file from source XSD path to the XSD mount point path.
        /// </returns>
        public OMError CopyXSD(String TypeName, String FieldName, String SourceXSDPath)
        {
            OMError         ObjOMError                  = null;
            StringBuilder   DestinationDirectoryPath    = null;
            StringBuilder   DestinationFilePath         = null;

            try
            {
                if (WAManager.IsFileExists(SourceXSDPath) == true)
                {
                    DestinationDirectoryPath = new StringBuilder();
                    //Get the root path of XSD directory from client XML (where the application is installaed).
                    //This is XSD mount point.
                    //GetXSDFileDirectoryPath(TypeName.Trim().ToUpper(), out DestinationDirectoryPath);
                    DestinationDirectoryPath = GetApplicationPath();

                    if ((DestinationDirectoryPath != null) && (DestinationDirectoryPath.Length > 0))
                    {
                        DestinationFilePath = new StringBuilder();
                        DestinationFilePath.AppendFormat("{0}\\{1}_{2}.xsd", DestinationDirectoryPath.ToString(), TypeName.Trim().ToUpper(), FieldName.Trim().ToUpper());

                        if ((SourceXSDPath != null) && (SourceXSDPath.Length > 0))
                        {
                            //If XSD directory not present then create it.
                            if (WAManager.IsFolderExists(DestinationDirectoryPath.ToString()) == false)
                            {
                                ObjOMError = WAManager.CreateFolder(DestinationDirectoryPath.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            //Delete the XSD file from destination if file already present on the destination path.
                            if (WAManager.IsFileExists(DestinationFilePath.ToString()) == true)
                            {
                                ObjOMError = WAManager.DeleteFile(DestinationFilePath.ToString());
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            ObjOMError = WAManager.CopyFile(SourceXSDPath, DestinationFilePath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                        else if (WAManager.IsFileExists(DestinationFilePath.ToString()) == true)
                        {
                            ObjOMError = WAManager.DeleteFile(DestinationFilePath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                    }
                }
                else
                {
                    ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_COPYXSDFILE_FAILED, "Invalid Source XSD File Path",
                                                    OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                    OMErrorType.EXCEPTION, null, null);
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_COPYXSDFILE_FAILED, "Copy XSD File Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to copy XSD file to the application's direcotry (../Configuration/XSD/Type) from XSD mount point.
        /// The source path of XSD is taken from the client XML (XSD mount point) where the application is installed.
        /// Directory path is created with the combination of root path, TypeName and FieldName.
        /// XSD file is defined for each field of an object type, which has data type XML.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="FieldName">Name of field. This field is of type XML.</param>
        /// <param name="XSDFilePath">This is the destination path of XSD file. This is an 'out' paramter.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it copies XSD file from XSD mount point path to the application XSD path.
        /// </returns>
        public OMError RetrieveXSDPath(String TypeName, String FieldName, out String XSDFilePath)
        {
            OMError         ObjOMError              = null;
            StringBuilder   XSDSourceDirectoryPath  = null;

            //Initialize 'out' parameter.
            XSDFilePath = null;

            try
            {
                XSDSourceDirectoryPath = new StringBuilder();
                TypeName = TypeName.Trim().ToUpper();
                FieldName = FieldName.Trim().ToUpper();
                XSDSourceDirectoryPath = GetApplicationPath();

                if ((XSDSourceDirectoryPath != null) && (XSDSourceDirectoryPath.Length > 0))
                {
                    XSDSourceDirectoryPath.AppendFormat("\\{0}_{1}.xsd", TypeName, FieldName);
                    if (WAManager.IsFileExists(XSDSourceDirectoryPath.ToString()) == true)
                        XSDFilePath = XSDSourceDirectoryPath.ToString();
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_RETRIEVEXSDFILE_FAILED, "Retrieve XSD File Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete XSD file of an XML field of object type.
        /// This deletes the XSD file from application direcotry (../Configuration/XSD/Type) and from XSD mount point.
        /// The root path of XSD is taken from the client XML (XSD mount point) where the application is installed.
        /// Directory path is created with the combination of root path, TypeName and FieldName.
        /// XSD file is defined for each field of an object type, which has data type XML.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="FieldName">Name of field. This field is of type XML.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it copies XSD file from XSD mount point path and from application XSD path.
        /// </returns>
        public OMError DeleteXSD(String TypeName, String FieldName)
        {
            OMError         ObjOMError          = null;
            StringBuilder   XSDApplicationPath  = null;

            try
            {
                TypeName = TypeName.Trim().ToUpper();
                FieldName = FieldName.Trim().ToUpper();

                //Delete XSD file from application directory.
                XSDApplicationPath = new StringBuilder();
                XSDApplicationPath = GetApplicationPath();
                if ((XSDApplicationPath != null) && (XSDApplicationPath.Length > 0))
                {
                    XSDApplicationPath.AppendFormat("\\{0}_{1}.xsd", TypeName, FieldName);
                    if (WAManager.IsFileExists(XSDApplicationPath.ToString()) == true)
                    {
                        ObjOMError = WAManager.DeleteFile(XSDApplicationPath.ToString());
                        if (ObjOMError != null)
                            return ObjOMError;
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XSD Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete XSD files of passed XML fields of object type.
        /// This deletes all XSD files from application direcotry (../Configuration/XSD/Type) and from XSD mount point.
        /// The root path of XSD is taken from the client XML (XSD mount point) where the application is installed.
        /// Directory path is created with the combination of root path, TypeName and FieldName.
        /// XSD file is defined for each field of an object type, which has data type XML.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <param name="ArrFieldName">This is list of names of field. This field is of type XML.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes XSD files from XSD mount point path and from application XSD path.
        /// </returns>
        public OMError DeleteXSD(String TypeName, String[] ArrFieldName)
        {
            OMError         ObjOMError          = null;
            StringBuilder   FilePath            = null;
            StringBuilder   XSDApplicationPath  = null;

            try
            {
                XSDApplicationPath = new StringBuilder();
                XSDApplicationPath = GetApplicationPath();

                if ((XSDApplicationPath != null) && (XSDApplicationPath.Length > 0))
                {
                    TypeName = TypeName.Trim().ToUpper();
                    for (int ArrCount = 0; ArrCount < ArrFieldName.Length; ArrCount++)
                    {
                        FilePath = new StringBuilder();
                        FilePath.AppendFormat("{0}\\{1}_{2}.xsd", XSDApplicationPath, TypeName, ArrFieldName[ArrCount].Trim().ToUpper());
                        if (WAManager.IsFileExists(FilePath.ToString()) == true)
                        {
                            ObjOMError = WAManager.DeleteFile(FilePath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                    }
                }
            }
            catch (Exception ObjException)
            {
                ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XSD Deletion Failed",
                                                OMErrorObjectType.XMLMANAGER, "XML Manager",
                                                OMErrorType.EXCEPTION, ObjException, null);
            }
            return (ObjOMError);
        }

        /// <summary>
        /// This method is used to delete all XSD directories of an object type.
        /// This deletes all XSD directories from application direcotry (../Configuration/XSD/Type) and from XSD mount point.
        /// The root path of XSD is taken from the client XML (XSD mount point) where the application is installed.
        /// Directory path is created with the combination of root path, TypeName and FieldName.
        /// XSD file is defined for each field of an object type, which has data type XML.
        /// </summary>
        /// <param name="TypeName">Name of an object type. Object type can be an Entity Type or Facet Type.</param>
        /// <returns>On success it returns NULL else returns an error object.
        /// On success it deletes all XSD files of an object type from XSD mount point path and from application XSD path.
        /// </returns>
        //public OMError DeleteXSD(String TypeName)
        //{
        //    OMError         ObjOMError              = null;
        //    StringBuilder   XSDSourceDirectoryPath  = null;
        //    StringBuilder   XSDApplicationPath      = null;

        //    try
        //    {
        //        XSDSourceDirectoryPath = new StringBuilder();
        //        //Get the root path of XSD directory from client XML (where the application is installaed).
        //        //This is XSD mount point.
        //        GetXSDFileDirectoryPath(TypeName.Trim().ToUpper(), out XSDSourceDirectoryPath);

        //        if ((XSDSourceDirectoryPath != null) && (XSDSourceDirectoryPath.Length > 0))
        //        {
        //            //Delete XSD directory from XSD mount point of an object type.
        //            //This deletes all subdirectories and files present in the XSD directory.
        //            if (Directory.Exists(XSDSourceDirectoryPath.ToString()) == true)
        //                Directory.Delete(XSDSourceDirectoryPath.ToString(), true);

        //            //Delete XSD directory from application path.
        //            //This deletes all subdirectories and files present in the XSD directory.
        //            XSDApplicationPath = GetApplicationPath();
        //            if ((XSDApplicationPath != null) && (XSDApplicationPath.Length > 0))
        //            {
        //                XSDApplicationPath.AppendFormat("\\{0}", TypeName);
        //                if (Directory.Exists(XSDApplicationPath.ToString()) == true)
        //                    Directory.Delete(XSDApplicationPath.ToString(), true);
        //            }

        //        }
        //    }
        //    catch (Exception ObjException)
        //    {
        //        ObjOMError = new OMError(OMErrorCodes.XMLMANAGER_DELETE_FAILED, "XSD Deletion Failed",
        //                                        OMErrorObjectType.XMLMANAGER, "XML Manager",
        //                                        OMErrorType.EXCEPTION, ObjException, null);
        //    }
        //    return (ObjOMError);
        //}

        # region Helper Functions (private methods)
        //This method is used to get the XML mount point.
        //The path is retrieved from the configuration XML which is present at the application's installation directory.
        private void GetXMLFileDirectoryPath(String TypeName, out StringBuilder DirectoryPath)
        {
            XmlDocument     TenantXMLDocument   = null;
            XmlNode         XMLMountPointNode   = null;
            StringBuilder   AppXMLPath          = null;
            String          InstalledPath       = null;
            String          XMLMountPoint       = null;

            DirectoryPath = new StringBuilder();
            try
            {
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");

                AppXMLPath = new StringBuilder();
                AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(TenantName));

                TenantXMLDocument = new XmlDocument();
                TenantXMLDocument.Load(AppXMLPath.ToString());

                XMLMountPointNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/XMLFILESYSTEM/XMLMOUNTPOINT");
                if (XMLMountPointNode != null)
                    XMLMountPoint = XMLMountPointNode.InnerText;
                XMLMountPointNode = null;

                if ((XMLMountPoint != null) && (XMLMountPoint.Length > 0))
                {
                    XMLMountPoint = XMLMountPoint.Trim().ToUpper();
                    TypeName = TypeName.Trim().ToUpper();
                    DirectoryPath.AppendFormat("{0}\\{1}", XMLMountPoint, TypeName);
                }
            }
            catch (Exception ObjException)
            {
                throw (ObjException);
            }
        }

        private OMError GetXMLFileDirectoryPathForShadowFolder(String TypeName, bool bCreateFolder, out StringBuilder DirectoryPath)
        {
            OMError ObjOMError = null;
            XmlDocument TenantXMLDocument = null;
            XmlNode XMLMountPointNode = null;
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            String XMLMountPoint = null;
            TypeName = TypeName.Trim().ToUpper();
            DirectoryPath = new StringBuilder();
            try
            {
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");

                AppXMLPath = new StringBuilder();
                AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(TenantName));

                TenantXMLDocument = new XmlDocument();
                TenantXMLDocument.Load(AppXMLPath.ToString());

                XMLMountPointNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/XMLFILESYSTEM/XMLMOUNTPOINT");
                if (XMLMountPointNode != null)
                    XMLMountPoint = XMLMountPointNode.InnerText;
                XMLMountPointNode = null;

                if ((XMLMountPoint != null) && (XMLMountPoint.Length > 0))
                {
                    XMLMountPoint = XMLMountPoint.Trim().ToUpper();
                    TypeName = TypeName.Trim().ToUpper();
                    DirectoryPath.AppendFormat("{0}\\{1}", XMLMountPoint, SHADOW_FOLDER);

                    if (bCreateFolder)
                    {
                        //Create SHADOW_FOLDER 
                        if (!WAManager.IsFolderExists(DirectoryPath.ToString()))
                        {
                            ObjOMError = WAManager.CreateFolder(DirectoryPath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                    }

                    DirectoryPath.AppendFormat("\\{0}", TransactionID);

                    if (bCreateFolder)
                    {
                        //Create Transaction Id folder
                        if (!WAManager.IsFolderExists(DirectoryPath.ToString()))
                        {
                            ObjOMError = WAManager.CreateFolder(DirectoryPath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                    }

                    DirectoryPath.AppendFormat("\\{0}", TypeName);

                    if (bCreateFolder)
                    {
                        //Create Type Name folder
                        if (!WAManager.IsFolderExists(DirectoryPath.ToString()))
                        {
                            ObjOMError = WAManager.CreateFolder(DirectoryPath.ToString());
                            if (ObjOMError != null)
                                return ObjOMError;
                        }
                    }
                }
                return ObjOMError;
            }
            catch (Exception ObjException)
            {
                throw (ObjException);
            }
        }

        //This method is used to get the XSD mount point.
        //The path is retrieved from the configuration XML which is present at the application's installation directory.
        //private void GetXSDFileDirectoryPath(String TypeName, out StringBuilder DirectoryPath)
        //{
        //    XmlDocument     TenantXMLDocument   = null;
        //    XmlNode         XSDMountPointNode   = null;
        //    StringBuilder   AppXMLPath          = null;
        //    String          InstalledPath       = null;
        //    String          XSDMountPoint       = null;

        //    //Initialize 'out' parameter.
        //    DirectoryPath = new StringBuilder();

        //    try
        //    {
        //        InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");

        //        AppXMLPath = new StringBuilder();
        //        AppXMLPath.AppendFormat("{0}CONFIGURATION\\XML\\{1}.xml", InstalledPath, TenantName);

        //        TenantXMLDocument = new XmlDocument();
        //        TenantXMLDocument.Load(AppXMLPath.ToString());

        //        XSDMountPointNode = TenantXMLDocument.SelectSingleNode("Configuration/XMLFileSystem/XSDMountPoint");
        //        if (XSDMountPointNode == null)
        //            XSDMountPointNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/XMLFILESYSTEM/XSDMOUNTPOINT");

        //        if (XSDMountPointNode != null)
        //            XSDMountPoint = XSDMountPointNode.InnerText;

        //        XSDMountPointNode = null;

        //        if ((XSDMountPoint != null) && (XSDMountPoint.Length > 0))
        //        {
        //            XSDMountPoint = XSDMountPoint.Trim().ToUpper();
        //            DirectoryPath.AppendFormat("{0}\\{1}", XSDMountPoint, TypeName);
        //        }
        //    }
        //    catch (Exception ObjException)
        //    {
        //        throw (ObjException);
        //    }
        //}

        //This method is used to get the application path.
        //The path is retrieved from the configuration XML which is present at the application's installation directory.
        private StringBuilder GetApplicationPath()
        {
            String          InstalledPath   = null;
            StringBuilder   ApplicationPath = null;

            try
            {
                ApplicationPath = new StringBuilder();
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK", "Installed Path");
                ApplicationPath.AppendFormat("{0}CONFIGURATION\\XSD\\TYPE", InstalledPath);
                return (ApplicationPath);
            }
            catch (Exception ObjException)
            {
                throw (ObjException);
            }
        }
        # endregion Helper Functions (private methods)

        #region NEW CODE

        private OMError FormDurableLogXML(ulong TransactionID, string Operation, string ParameterXML, string Status, ref string DurableLogXML)
        {
            StringBuilder LogXML = null;

            try
            {
                LogXML = new StringBuilder();

                LogXML.Append("<INPUTXML>");
                LogXML.AppendFormat("<TRANSACTIONID>{0}</TRANSACTIONID>", TransactionID);
                LogXML.AppendFormat("<OPERATION>{0}</OPERATION>", Operation);
                LogXML.AppendFormat("<PARAMETERINFO>{0}</PARAMETERINFO>", ParameterXML);
                LogXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", "FALSE");
                LogXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
                LogXML.Append("<ISXMLOPERATION>1</ISXMLOPERATION>");

                LogXML.AppendFormat(@"<CREATEDON> 
                                                <YEAR>{0}</YEAR> 
                                                <MONTH>{1}</MONTH> 
                                                <DAY>{2}</DAY> 
                                                <HOUR>{3}</HOUR> 
                                                <MINUTE>{4}</MINUTE> 
                                                <SECOND>{5}</SECOND> 
                                                </CREATEDON>", DateTime.Now.Year, DateTime.Now.Month,
                                                DateTime.Now.Day, DateTime.Now.Hour,
                                                DateTime.Now.Minute, DateTime.Now.Second);


                LogXML.Append("</INPUTXML>");
                DurableLogXML = LogXML.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError FormPamareterXML(StringCollection Parameters, ref string ParameterXML)
        {
            StringBuilder OutputXML = null;
            String value = null;

            try
            {
                OutputXML = new StringBuilder();
                OutputXML.Append("<PARAMETERS>");
                foreach (string Parameter in Parameters)
                {
                    value = Assetlink.ALCommon.TrimAndReplaceSpecialChars(Parameter);
                    OutputXML.AppendFormat("<PARAM>{0}</PARAM>", value);
                }
                OutputXML.Append("</PARAMETERS>");
                ParameterXML = OutputXML.ToString();
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        public OMError CommitXMLOperations()
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ErrorObj = ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null && ErrorObj == null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = CommitOperations(LogNodes);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        public OMError RollbackXMLOperations()
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ErrorObj = ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null && ErrorObj == null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = RollBackOperations(LogNodes);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        public OMError HandleInDoubtTransactions(String strInfo)
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""HandleInDoubtTransactions()"",""HandleInDoubtTransactions() called for TransactionID:" + TransactionID.ToString() + @"""");
                DurableLogDoc = new XmlDocument();
                ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = ExecuteInDoubtTransactions(LogNodes, strInfo);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private OMError ReadDurableLog(ulong TransactionID, ref string DurableLogXML)
        {
            OMError ErrorObj = null;
            try
            {
                if ((ErrorObj = m_DataManager.DataProvider.AddClause("TRANSACTIONID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TransactionID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrorObj = m_DataManager.DataProvider.AddClause("ISXMLOPERATION", OMFieldDataType.INTEGER, OMOperator.EQUALTO, "1", OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = m_DataManager.DataProvider.ReadAllRecordEx("LOG", "ALFS_DURABLELOG", null, ref DurableLogXML)) != null)
                        {
                            return ErrorObj;
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private OMError CommitOperations(XmlNodeList LogNodes)
        {
            OMError ObjOMError = null;
            string Operation = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;

            try
            {
                foreach (XmlNode LogNode in LogNodes)
                {
                    bCaseHit = false;
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                    
                    switch (Operation.ToUpper())
                    {
                        case "DELETEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                            if (ObjOMError != null)
                                return ObjOMError;
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            
                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                            
                            break;
                        case "UPDATEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                            if (ObjOMError != null)
                                return ObjOMError;
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            
                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                            
                            break;

                        case "CREATEFILE":
                            bCaseHit = true;
                            break;

                        case "MOVEFILE":
                            bCaseHit = true;
                            break;

                        case "COPYFILE":
                            bCaseHit = true;
                            break;

                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO COMMITED
                        if ((ObjOMError = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, "COMMITED", string.Empty)) != null)
                        {
                            return ObjOMError;
                        }
                    }
                }
                
                foreach (string strFolder in LocalFolders)
                {
                    ObjOMError = WAManager.DeleteFolder(strFolder, true);
                    if (ObjOMError != null)
                        return ObjOMError;
                }
                
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError RollBackOperations(XmlNodeList LogNodes)
        {
            OMError ObjOMError = null;
            string Operation = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;
            try
            {
                foreach (XmlNode LogNode in LogNodes)
                {
                    bCaseHit = false;
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                   
                    switch (Operation.ToUpper())
                    {
                        case "CREATEFILE": 
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            if (WAManager.IsFileExists(ParamsNodes[0].InnerText))
                            {
                                ObjOMError = WAManager.DeleteFile(ParamsNodes[0].InnerText);
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }
                            bCaseHit = true;
                            break;

                        case "UPDATEFILE": 
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM"); //DELETE FILE FROM DESTINATION LOCATION
                            if (WAManager.IsFileExists(ParamsNodes[1].InnerText))
                            {
                                ObjOMError = WAManager.CopyFile(ParamsNodes[1].InnerText, ParamsNodes[0].InnerText);
                                if (ObjOMError != null)
                                    return ObjOMError;
                                ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }

                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                            
                            break;

                        case "DELETEFILE": 
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM"); //DELETE FILE FROM DESTINATION LOCATION
                            if (WAManager.IsFileExists(ParamsNodes[1].InnerText))
                            {
                                #region CREATE SOURCE FOLDER IF NOT EXISTS
                                String strSourceFolder = ParamsNodes[0].InnerText;
                                strSourceFolder = strSourceFolder.Substring(0, strSourceFolder.LastIndexOf(@"\"));
                                String sObjectIdFolder = strSourceFolder;
                                strSourceFolder = strSourceFolder.Substring(0, strSourceFolder.LastIndexOf(@"\"));
                                String sObjectTypeFolder = strSourceFolder;
                                
                                if (!WAManager.IsFolderExists(sObjectTypeFolder))
                                    ObjOMError = WAManager.CreateFolder(sObjectTypeFolder);
                                if (ObjOMError != null)
                                    return ObjOMError;
                                if (!WAManager.IsFolderExists(sObjectIdFolder))
                                    ObjOMError = WAManager.CreateFolder(sObjectIdFolder);
                                if (ObjOMError != null)
                                    return ObjOMError;
                                #endregion

                                ObjOMError = WAManager.CopyFile(ParamsNodes[1].InnerText, ParamsNodes[0].InnerText);
                                if (ObjOMError != null)
                                    return ObjOMError;
                                ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                                if (ObjOMError != null)
                                    return ObjOMError;
                            }
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                           
                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                           
                            break;
                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO ROLLEDBACK
                        if ((ObjOMError = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, "ROLLEDBACK", string.Empty)) != null)
                        {
                            return ObjOMError;
                        }
                    }
                }
                
                foreach (string strFolder in LocalFolders)
                {
                    ObjOMError = WAManager.DeleteFolder(strFolder, true);
                    if (ObjOMError != null)
                        return ObjOMError;
                }
                
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError ExecuteInDoubtTransactions(XmlNodeList LogNodes, String strInfo)
        {
            OMError ObjOMError = null;
            string Operation = null;
            StringCollection Parameters = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;
            string sOperationStatus = string.Empty;

            try
            {
                Parameters = new StringCollection();
                foreach (XmlNode LogNode in LogNodes)
                {
                    bCaseHit = false;
                    Parameters.Clear();
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                    
                    switch (Operation.ToUpper())
                    {
                        case "DELETEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "ROLLEDBACK";
                            if (!WAManager.IsFileExists(ParamsNodes[0].InnerText))
                            {
                                if (WAManager.IsFileExists(ParamsNodes[1].InnerText))
                                {
                                    #region CREATE SOURCE FOLDER IF NOT EXISTS
                                    String strSourceFolder = ParamsNodes[0].InnerText;
                                    strSourceFolder = strSourceFolder.Substring(0, strSourceFolder.LastIndexOf(@"\"));
                                    String sObjectIdFolder = strSourceFolder;
                                    strSourceFolder = strSourceFolder.Substring(0, strSourceFolder.LastIndexOf(@"\"));
                                    String sObjectTypeFolder = strSourceFolder;

                                    if (!WAManager.IsFolderExists(sObjectTypeFolder))
                                        ObjOMError = WAManager.CreateFolder(sObjectTypeFolder);
                                    if (ObjOMError != null)
                                        return ObjOMError;
                                    if (!WAManager.IsFolderExists(sObjectIdFolder))
                                        ObjOMError = WAManager.CreateFolder(sObjectIdFolder);
                                    if (ObjOMError != null)
                                        return ObjOMError;
                                    #endregion

                                    ObjOMError = WAManager.CopyFile(ParamsNodes[1].InnerText, ParamsNodes[0].InnerText);
                                    if (ObjOMError != null)
                                        return ObjOMError;
                                    ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                                    if (ObjOMError != null)
                                        return ObjOMError;
                                }
                            }

                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));

                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                            
                            break;
                        case "UPDATEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            ObjOMError = WAManager.DeleteFile(ParamsNodes[1].InnerText);
                            if (ObjOMError != null)
                                return ObjOMError;
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));

                            if (!LocalFolders.Contains(FolderPath))
                                LocalFolders.Add(FolderPath);
                            sOperationStatus = "COMMITED";
                            bCaseHit = true;
                            break;

                        case "CREATEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "COMMITED";
                            bCaseHit = true;
                            break;
                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO COMMITED
                        if ((ObjOMError = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, sOperationStatus, strInfo)) != null)
                        {
                            return ObjOMError;
                        }
                    }
                }
                foreach (string strFolder in LocalFolders)
                {
                    ObjOMError = WAManager.DeleteFolder(strFolder, true);
                    if (ObjOMError != null)
                        return ObjOMError;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError UpdateDurableLogStatus(string LogId, string Status, string strInfo)
        {
            OMError ErrorObj = null;
            try
            {
                String strUpdateXML = string.Empty;
                FormDurableLogXMLForUpdate(Status, strInfo, ref strUpdateXML);
                if ((ErrorObj = LOGDMObj.DataProvider.AddClause("LOGID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, LogId, OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = LOGDMObj.DataProvider.UpdateRecord("ALFS_DURABLELOG", strUpdateXML)) != null)
                    {
                        ErrorObj.TransactionId = TransactionID.ToString();
                        return ErrorObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""UpdateDurableLogStatus()"",""Exception in UpdateDurableLogStatus() for LogId:" + LogId + " and Status:" + Status + " with Exception:" + ExceptionObj.Message.Replace(@"""", @"""""") + @"""");
                throw ExceptionObj;
            }

        }

        private OMError FormDurableLogXMLForUpdate(string Status, string strInfo, ref string DurableLogXML)
        {
            StringBuilder LogXML = null;

            try
            {
                LogXML = new StringBuilder();

                LogXML.Append("<INPUTXML>");
                LogXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
                LogXML.AppendFormat("<MOREINFO>{0}</MOREINFO>", strInfo);

                LogXML.AppendFormat(@"<MODIFIEDON> 
                                                <YEAR>{0}</YEAR> 
                                                <MONTH>{1}</MONTH> 
                                                <DAY>{2}</DAY> 
                                                <HOUR>{3}</HOUR> 
                                                <MINUTE>{4}</MINUTE> 
                                                <SECOND>{5}</SECOND> 
                                                </MODIFIEDON>", DateTime.Now.Year, DateTime.Now.Month,
                                                DateTime.Now.Day, DateTime.Now.Hour,
                                                DateTime.Now.Minute, DateTime.Now.Second);

                LogXML.Append("</INPUTXML>");
                DurableLogXML = LogXML.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        #endregion
    }
}