using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Data;
using Assetlink.Framework;

namespace Assetlink.Services.Engines
{
    public class AssetlinkSearch : ALBaseEngine
    {
        #region DataMembers

      

        #endregion DataMembers
            
        #region Constructors
        public AssetlinkSearch():base()
        {
        }

        public AssetlinkSearch(UInt64 UserID, String Language,
                              String TenantName):base(UserID,Language,TenantName)
        {
            
        }
        #endregion Constructors

        #region Public_Functions
        /// <summary>
        /// <CreatedDate>16 February 2006</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This API invokes the Search. This API is used to search the Target Entities and its related entities and related facets.
        /// This API will be used for Advanced as well as Simple Search.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">DoSearch accepts an InputXML as a parameter. It contents all the parameters required for the 
        /// search. The structure of this XML is as follows:
        ///     INPUTXML
        ///        |-SESSIONINFO (Attributes: TenantName, Language, UserID, SessionID)
        ///        |-SEARCHCOLLECTION
        ///             |-SEARCH  (1..*)
        ///                 |-TARGETOBJECT
        ///                 |-TARGETOBJECTTYPE
        ///                 |-MAXRESULTSIZE 
        ///                 |-SORTORDER 
        ///                 |-SORTCOLUMN
        ///                 |-ISCASESENSITIVE 
        ///                 |-SEARCHSTRING
        ///                 |-SEARCHALLFACETS
        ///                 |-FILTERLIST (0..1)
        ///                     |-FILTER (1..*)
        ///                         |-FIELDNAME
        ///                         |-OPERATOR
        ///                         |-FIELDVALUE
        ///                         |-NAVIGATIONPATH
        ///                         |-ISCASESENSITIVE
        ///                 |-FIELDCOLLECTION
        ///                     |-TARGETENTITYFIELDS  (0..1)
        ///                         |-<$TARGETENTITY$ Fields> 
        ///                     |-RELATEDBUSINESSINFOFIELDS  (0..*)(Attribute: NavigationPath)
        ///                         |-<$RELATEDBUSINESSINFO$ Fields> 
        ///                     |-RELATEDENTITYFIELDS (0..*)(Attribute: NavigationPath)
        ///                         |-<$RELATEDENTITY$ Fields> 
        /// </param>
        /// <param name="OutputXML">OutputXML contains the result set. The structure of this XML is as follows:
        ///     SEARCHOUTPUTXML
        ///         |-<$TARGETOBJECTCOLLECTION$>
        ///                 |-<$TARGETOBJECT$> 
        ///                        |-<$TARGETOBJECT$ Fields> 
        ///                        |-<$RELATEDENTITY-RELATIONSHIPNAME$>
        ///                             |-<$RELATEDENTITY$>
        ///                                  |-<$RELATEDENTITY$ Fields>
        ///                        |-<$RELATEDFACET-RELATIONSHIPNAME$>
        ///                             |-<$RELATEDFACET$>
        ///                                  |-<$RELATEDFACET$ Fields>
        /// </param>
        /// <returns>
        /// NULL, If the call was successful.
        /// ErrorXML, if error occurred.
        /// </returns>
        public String DoSearch(String InputXML, ref String OutputXML)
        {
            #region Local_Variables
            XmlDocument InputXMLDocument    = null;
            XmlNode InputXMLNode            = null;
            XmlNode SessionInfoNode         = null;
            XmlNode SearchCollectionNode    = null;
            XmlNode SearchNode              = null;
            XmlNode FieldCollectionNode     = null;
            XmlNode FilterModeNode              = null;
            XmlNodeList  RelatedFacetFields     = null;
            XmlNodeList  RelatedEntityFields    = null;
            String SearchOutputXML   = null;
            StringBuilder SearchOutputXMLBuilder = null;
            String ErrorXML                 = null;  //String variable storing the ErrorXML.
            String OMErrorXML               = null;  //String variable storing the ErrorXML.
            OMInterface OMIObject           = null;  //OMInterface Object.
            ServiceError ServiceErrObject   = null;  //ServiceError Object.

            #endregion Local_Variables

            try
            {
                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                InputXMLNode         = InputXMLDocument.SelectSingleNode("INPUTXML");
                SearchCollectionNode = InputXMLNode.SelectSingleNode("SEARCHCOLLECTION");
                SessionInfoNode      = InputXMLNode.SelectSingleNode("SESSIONINFO");
                FilterModeNode       = InputXMLNode.SelectSingleNode("FILTERMODE");

                Language            = SessionInfoNode.Attributes["Language"].InnerText;
                TenantName          = SessionInfoNode.Attributes["TenantName"].InnerText;
                UserID              = ulong.Parse(SessionInfoNode.Attributes["UserID"].InnerText);
                

                if ((SearchCollectionNode != null) && (SearchCollectionNode.HasChildNodes))
                {
                    SearchOutputXMLBuilder = new StringBuilder();
                    SearchOutputXMLBuilder.Append("<SEARCHOUTPUTXML>");
                    foreach (XmlElement SearchChildNode in SearchCollectionNode.ChildNodes)
                    {
                        GetParametersForSearch(InputXMLDocument, SearchChildNode, ref SearchNode, ref FieldCollectionNode);
                       if (FilterModeNode != null)
                       {
                           if (((XmlElement)FilterModeNode).GetAttribute("ApplyFilter").ToUpper() == "YES")
                           {

                               RelatedFacetFields = FieldCollectionNode.SelectNodes("RELATEDFACETFIELDS");
                               RelatedEntityFields = FieldCollectionNode.SelectNodes("RELATEDENTITYFIELDS");
                               foreach (XmlNode RelatedFacetField in RelatedFacetFields)
                               {
                                   ((XmlElement)RelatedFacetField).SetAttribute("ApplyFilter", "Yes");

                               }

                               foreach (XmlNode RelatedEntityField in RelatedEntityFields)
                               {
                                   ((XmlElement)RelatedEntityField).SetAttribute("ApplyFilter", "Yes");
                               }
                           }
                       }

                        //Call the OMI function.
                        OMIObject = new OMInterface(UserID, Language,TenantName);

                        if (FieldCollectionNode != null)
                            OMErrorXML = OMIObject.GetData(SearchNode.OuterXml, FieldCollectionNode.OuterXml, ref SearchOutputXML);
                        else
                            OMErrorXML = OMIObject.GetData(SearchNode.OuterXml, null, ref SearchOutputXML);

                        if (OMErrorXML != null)
                        {
                            ServiceErrObject = new ServiceError();
                            ErrorXML = ServiceErrObject.TransformOMErrorXML(OMErrorXML);
                            return ErrorXML;
                        }

                        SearchOutputXMLBuilder.Append(SearchOutputXML);
                    }
                    SearchOutputXMLBuilder.Append("</SEARCHOUTPUTXML>");
                }

                OutputXML = SearchOutputXMLBuilder.ToString();

                return null;
            }
            catch(Exception ExceptionObject)
            {
                ServiceErrObject = new ServiceError(OMErrorType.EXCEPTION, "Exception occurred", "Search",
                                      ExceptionObject, "DoSearch", InputXML);
                ErrorXML = ServiceErrObject.Serialize();
                return ErrorXML;
            }
        }

        /// <summary>
        /// <CreatedDate>20 December 2006</CreatedDate>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <Purpose>
        /// This API invokes the NSearch. 
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">NSearch accepts an InputXML as a parameter. It contents all the parameters required for the 
        /// </param>
        /// <param name="OutputXML">OutputXML contains the result set.
        /// </param>
        /// <returns>
        /// NULL, If the call was successful.
        /// ErrorXML, if error occurred.
        /// </returns>
        public String GetHierarchialData(string InputXML, ref string OutputSearchResult)
        {
            #region Local_Variables
            OMInterface OMIObject = null;  //OMInterface Object.
            ServiceError ServiceErrObject = null;  //ServiceError Object.
            String OMErrorXML = null;  //String variable storing the ErrorXML.
            String ErrorXML = null;  //String variable storing the ErrorXML.
            XmlDocument InputXMLDocument = null;
            XmlNode SearchNode = null;
            XmlNode SessionInfoNode = null;
            #endregion Local_Variables

            try
            {
                InputXMLDocument = new XmlDocument();
                InputXMLDocument.LoadXml(InputXML);

                SearchNode = InputXMLDocument.SelectSingleNode("SEARCH");
                SessionInfoNode = SearchNode.SelectSingleNode("SESSIONINFO");

                Language = SessionInfoNode.Attributes["Language"].InnerText;
                TenantName = SessionInfoNode.Attributes["TenantName"].InnerText;
                UserID = ulong.Parse(SessionInfoNode.Attributes["UserID"].InnerText);

                //Call the OMI function.
                OMIObject = new OMInterface(UserID, Language, TenantName);
                OMErrorXML = OMIObject.GetHierarchialData(InputXML, ref OutputSearchResult);

                if (OMErrorXML != null)
                {
                    ServiceErrObject = new ServiceError();
                    ErrorXML = ServiceErrObject.TransformOMErrorXML(OMErrorXML);
                    return ErrorXML;
                }
                return null;
            }
            catch (Exception ExceptionObject)
            {
                ServiceErrObject = new ServiceError(OMErrorType.EXCEPTION, "Exception occurred", "Search",
                                      ExceptionObject, "NSearch", InputXML);
                ErrorXML = ServiceErrObject.Serialize();
                return ErrorXML;
            }
        }

        /// <summary>
        /// <CreatedDate>19 February 2008</CreatedDate>
        /// <Author>Meenal Joshi</Author>
        /// <Purpose>
        /// This API accepts multiple Search nodes in a collection and invoke NSearch for each search node.  
        /// </Purpose>
        /// </summary>
        /// <param name="InputXML">NSearch accepts an InputXML as a parameter. It contents all the parameters required for the 
        /// </param>
        /// <param name="OutputXML">OutputXML contains the result set.
        /// </param>
        /// <returns>
        /// NULL, If the call was successful.
        /// ErrorXML, if error occurred.
        /// </returns>
        public String GetHierarchialDataEx(string SearchCollectionXML, ref string OutputSearchResult)
        {
            #region Local_Variables
            OMInterface                 OMIObject           = null;  //OMInterface Object.
            ServiceError                ServiceErrObject    = null;  //ServiceError Object.
            String                      OMErrorXML          = null;  //String variable storing the ErrorXML.
            String                      ErrorXML            = null;  //String variable storing the ErrorXML.
            XmlDocument                 InputXMLDocument    = null;            
            XmlNode                     SessionInfoNode     = null;
            XmlNodeList                 SearchNodelist      = null;
            XmlNode                     SearchCollectionNode    = null;
            StringBuilder               SearchOutputXMLBuilder  = null;
            #endregion Local_Variables

            try
            {
                SearchOutputXMLBuilder  = new StringBuilder();
                InputXMLDocument        = new XmlDocument();
                InputXMLDocument.LoadXml(SearchCollectionXML);

                SearchCollectionNode    = InputXMLDocument.SelectSingleNode("SEARCHCOLLECTION");                
                SessionInfoNode         = SearchCollectionNode.SelectSingleNode("SESSIONINFO");

                Language                = SessionInfoNode.Attributes["Language"].InnerText;
                TenantName              = SessionInfoNode.Attributes["TenantName"].InnerText;
                UserID                  = ulong.Parse(SessionInfoNode.Attributes["UserID"].InnerText);

                SearchOutputXMLBuilder.Append("<SEARCHOUTPUTXML>");
                SearchNodelist          = SearchCollectionNode.SelectNodes("SEARCH");
                OMIObject               = new OMInterface(UserID, Language, TenantName);
                ServiceErrObject        = new ServiceError();

                foreach (XmlNode NSearchNode in SearchNodelist)
                {
                    //Call the OMI function.
                    OMErrorXML          = OMIObject.GetHierarchialData(NSearchNode.OuterXml, ref OutputSearchResult);
                    if (OMErrorXML != null)
                    {
                        ErrorXML            = ServiceErrObject.TransformOMErrorXML(OMErrorXML);
                        return ErrorXML;
                    }                    
                    SearchOutputXMLBuilder.Append(OutputSearchResult);
                }
                SearchOutputXMLBuilder.Append("</SEARCHOUTPUTXML>");
                OutputSearchResult = SearchOutputXMLBuilder.ToString();
                return null;
            }
            catch (Exception ExceptionObject)
            {
                ServiceErrObject = new ServiceError(OMErrorType.EXCEPTION, "Exception occurred", "Search",
                                      ExceptionObject, "NSearch", SearchCollectionXML);
                ErrorXML = ServiceErrObject.Serialize();
                return ErrorXML;
            }
        }

        public String GetHierarchialData(string SearchInputXML, ref DataSet DS_Result)
        {
            #region Local_Variables
            OMInterface                 OMIObject               = null;  //OMInterface Object.
            ServiceError                ServiceErrObject        = null;  //ServiceError Object.
            String                      OMErrorXML              = null;  //String variable storing the ErrorXML.
            String                      ErrorXML                = null;  //String variable storing the ErrorXML.
            #endregion Local_Variables

            try
            {
                OMIObject               = new OMInterface(UserID, Language, TenantName);
                ServiceErrObject        = new ServiceError();

               
                //Call the OMI function.
                OMErrorXML = OMIObject.GetHierarchialData(SearchInputXML, ref DS_Result);
                if (OMErrorXML != null)
                {
                    ErrorXML            = ServiceErrObject.TransformOMErrorXML(OMErrorXML);
                    return ErrorXML;
                }                    
                return null;
            }
            catch (Exception ExceptionObject)
            {
                ServiceErrObject = new ServiceError(OMErrorType.EXCEPTION, "Exception occurred", "Search",
                                      ExceptionObject, "NSearch", null);
                ErrorXML = ServiceErrObject.Serialize();
                return ErrorXML;
            }
        }

        #endregion Public_Functions

        #region Private_Function

        
        /// <summary>
        /// <CreatedDate>20 February 2006</CreatedDate>
        /// <Author>Mr. Hemant Kirad</Author>
        /// <Purpose>
        /// This helper function takes the SearchNode as a parameter and modifies the SearchNode and creates a FieldCollectionNode
        /// as required by OM and returns them.
        /// </Purpose>
        /// </summary>
        /// <param name="InputXMLDocument">XML Document object representing the InputXML.</param>
        /// <param name="SearchChildNode">XML Element object representing the SearchNode.</param>
        /// <param name="SearchNode">This is a XMLNode Object. It contains the modified XML structure of the SearchNode.</param>
        /// <param name="FieldCollectionNode">This is a XMLNode Object. It contains the XML structure of the FieldCollectionNode.</param>
        /// <returns>
        /// NULL, If the call was successful.
        /// ErrorXML, if error occurred.
        /// </returns>
        private String GetParametersForSearch(XmlDocument InputXMLDocument, XmlElement SearchChildNode, ref XmlNode SearchNode, ref XmlNode FieldCollectionNode)
        {
            XmlNodeList RelatedBINodes = null;
            XmlNode RelatedFacetFields = null;
            XmlNode NavigationPathNode = null;  //DOM Object representing the NavigationPath Node.

            try
            {
                SearchNode = SearchChildNode.CloneNode(true);

                FieldCollectionNode = SearchNode.SelectSingleNode("FIELDCOLLECTION");
                if ((FieldCollectionNode != null) && (FieldCollectionNode.HasChildNodes))
                {
                    RelatedBINodes = FieldCollectionNode.SelectNodes("RELATEDBUSINESSINFOFIELDS");
                    if (RelatedBINodes.Count > 0)
                    {
                        for (int i = 0; i < RelatedBINodes.Count; i++)
                        {
                            String NavigationPath = RelatedBINodes.Item(i).Attributes["NavigationPath"].InnerText;

                            RelatedFacetFields = InputXMLDocument.CreateNode(XmlNodeType.Element, "RELATEDFACETFIELDS", "");

                            NavigationPathNode = InputXMLDocument.CreateNode(XmlNodeType.Attribute, "NavigationPath", "");
                            NavigationPathNode.Value = NavigationPath;

                            RelatedFacetFields.Attributes.SetNamedItem(NavigationPathNode);
                            
                            foreach (XmlElement ChildNode in RelatedBINodes.Item(i).ChildNodes)
                                RelatedFacetFields.AppendChild(ChildNode);

                            FieldCollectionNode.AppendChild(RelatedFacetFields);
                        }

                        for (int i = 0; i < RelatedBINodes.Count; i++)
                            FieldCollectionNode.RemoveChild(RelatedBINodes.Item(i));
                    }

                    SearchNode.RemoveChild(FieldCollectionNode);
                }

                return null;
            }
            catch(Exception ExceptionObject)
            {
                throw ExceptionObject;
            }
    
        }

        #endregion Private_Function
    }
}
