﻿#region System Includes

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.IO;
using System.Data;
using System.Xml;
using System.Xml.XPath;
using System.Globalization;
using Microsoft.Office.Interop.Word;
using Microsoft.Office.Interop.Excel;

#endregion

namespace Assetlink.MediaProcessors
{
    #region Directives

    using XML       = System.String;
    using ERRORXML  = System.String;

    #endregion

    public class MSOffice_MediaPlugin
    {
        #region Private Classes
        private class HeaderInfo
        {
            public String Name;
            public String UIType;
            public String DataType;
            public Boolean IsPrimaryKey;
            public Boolean IsVisible;
            public int ExcelCellIndex;
            public Boolean IsReadOnly;
            public Boolean DoAutoGenerate;
            public String SeriesID;
            public String[] SeriesIDList = null;
            public int SeriesIDListIndex = 0;            

            public HeaderInfo(String Name, String UIType, String DataType, String IsVisible, String PrimaryKey, String IsReadOnly, String DoAutoGenerate, String SeriesID, String[] SeriesIDList)
            {
                this.Name = Name;
                this.UIType = UIType;
                this.DataType = DataType;

                if (this.Name.Equals(PrimaryKey, StringComparison.OrdinalIgnoreCase))
                    IsPrimaryKey = true;

                if (!String.IsNullOrEmpty(IsVisible) && IsVisible.ToUpper() == "NO")
                    this.IsVisible = false;
                else
                    this.IsVisible = true;

                if (String.IsNullOrEmpty(IsReadOnly))
                    this.IsReadOnly = false;
                else
                    this.IsReadOnly = Boolean.Parse(IsReadOnly);

                if (String.IsNullOrEmpty(DoAutoGenerate))
                    this.DoAutoGenerate = false;
                else
                    this.DoAutoGenerate = Boolean.Parse(DoAutoGenerate);

                this.SeriesID = SeriesID;
                this.SeriesIDList = SeriesIDList;
                this.ExcelCellIndex = -1;
            }
        }
        #endregion

        #region Private Constants
        private const double PointtoPixelFactor = 7.0; //Used for Ad hoc reports to set width of column in points
        #endregion

        #region Private Member Variables
        String str_FormatXMLPath = null;
        String str_CultureInfo = null;
        CultureInfo customculture_Object = null;
        String TenantName = null; //Used in FormatValueforDataType function
        #endregion

        #region Public Methods

        //ExportCrystalReportDataToExcel() method is used to generate an excel file using given Data XML.
        public ERRORXML ExportCrystalReportDataToExcel(String DataFilePath, String ExcelFilePath)
        {
            Microsoft.Office.Interop.Excel.ApplicationClass ExcelApplicationObj = null;
            Microsoft.Office.Interop.Excel.WorkbookClass ExcelWorkbookObj = null;
            Worksheet ExcelWorksheetObj = null;
            Microsoft.Office.Interop.Excel.Range CellRangeObj = null;
            Int32 RowNumber = 0;
            Int32 ColumnNumber = 0;
            Int32 ColumnStartIndexNumber = -1;
            Int32 ColumnEndIndexNumber = 0;
            Int32 DivReminderResult = 0;
            const Int32 TotalArrCharsCount = 25;
            Int32 FieldCount = 0;
            StringBuilder ColumnIndex = null;
            Char[] ArrOfAlphabets = {'A','B','C','D','E','F','G','H','I','J','K','L','M',
                                                                                            'N','O','P','Q','R','S','T','U','V','W','X','Y','Z'};
            ERRORXML ErrorXML = null;
            try
            {
                ExcelApplicationObj = new Microsoft.Office.Interop.Excel.ApplicationClass();

                ExcelWorkbookObj = ExcelApplicationObj.Workbooks.Add(ALCommon.GetInstallationDir() + "ALOffice2007Template.xls") as WorkbookClass;
                ExcelWorksheetObj = ExcelWorkbookObj.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                ExcelWorksheetObj.Name = "ExportedData";

                System.Data.DataTable dt = new System.Data.DataTable("ExportedData");
                dt.ReadXml(DataFilePath);
                DataTableReader ObjDataTableReader = dt.CreateDataReader();

                //Creating header row.
                RowNumber++;
                ColumnIndex = new StringBuilder();
                for (FieldCount = 0; FieldCount < ObjDataTableReader.FieldCount; FieldCount++)
                {
                    ColumnIndex.Remove(0, ColumnIndex.Length);
                    if (ColumnNumber <= TotalArrCharsCount)
                        ColumnIndex.AppendFormat("{0}", ArrOfAlphabets[ColumnNumber]);
                    else
                    {
                        Math.DivRem(ColumnNumber, TotalArrCharsCount, out DivReminderResult);
                        if (DivReminderResult == 1)
                        {
                            ColumnStartIndexNumber++;
                            ColumnEndIndexNumber = 0;
                        }
                        ColumnIndex.AppendFormat("{0}{1}", ArrOfAlphabets[ColumnStartIndexNumber], ArrOfAlphabets[ColumnEndIndexNumber]);
                        ColumnEndIndexNumber++;
                    }
                    CellRangeObj = ExcelWorksheetObj.Cells[RowNumber, ColumnIndex.ToString()] as Microsoft.Office.Interop.Excel.Range;
                    CellRangeObj.Value2 = ObjDataTableReader.GetName(FieldCount);
                    CellRangeObj.Font.Bold = true;
                    //CellRangeObj.WrapText = true;
                    ColumnNumber++;
                }

                while (ObjDataTableReader.Read())
                {
                    RowNumber++;
                    ColumnNumber = 0;
                    ColumnStartIndexNumber = -1;
                    for (FieldCount = 0; FieldCount < ObjDataTableReader.FieldCount; FieldCount++)
                    {
                        ColumnIndex.Remove(0, ColumnIndex.Length);
                        if (ColumnNumber <= TotalArrCharsCount)
                            ColumnIndex.AppendFormat("{0}", ArrOfAlphabets[ColumnNumber]);
                        else
                        {
                            Math.DivRem(ColumnNumber, TotalArrCharsCount, out DivReminderResult);
                            if (DivReminderResult == 1)
                            {
                                ColumnStartIndexNumber++;
                                ColumnEndIndexNumber = 0;
                            }
                            ColumnIndex.AppendFormat("{0}{1}", ArrOfAlphabets[ColumnStartIndexNumber], ArrOfAlphabets[ColumnEndIndexNumber]);
                            ColumnEndIndexNumber++;
                        }
                        CellRangeObj = ExcelWorksheetObj.Cells[RowNumber, ColumnIndex.ToString()] as Microsoft.Office.Interop.Excel.Range;
                        CellRangeObj.Value2 = ObjDataTableReader[FieldCount].ToString();
                        ColumnNumber++;
                    }
                }

                if (File.Exists(ExcelFilePath))
                    File.Delete(ExcelFilePath);

                ExcelWorkbookObj.SaveCopyAs(ExcelFilePath);
                if (ExcelWorksheetObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelWorksheetObj);
                    ExcelWorksheetObj = null;
                }

                if (ExcelWorkbookObj != null)
                {
                    ExcelWorkbookObj.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelWorkbookObj);
                    ExcelWorkbookObj = null;
                }

                if (ExcelApplicationObj != null)
                {
                    ExcelApplicationObj.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelApplicationObj);
                    ExcelApplicationObj = null;
                }
            }
            catch (Exception ExceptionObject)
            {                
                ErrorXML = ExceptionObject.Message;
            }
            return ErrorXML;
        }

        /// <summary>
        /// This method converts MHtml file to PDF
        /// </summary>
        /// <param name="SourcePath">Source MHTML file Path</param>
        /// <param name="DestinationPath">Target Path for PDF file</param>
        /// <returns>Returns null on success else returns error xml</returns>
        public ERRORXML GeneratePDFFromMHtml(String SourcePath, String DestinationPath)
        {
            # region Local Variables
            Microsoft.Office.Interop.Word.ApplicationClass          wordApplication         = null;
            Microsoft.Office.Interop.Word.Document                  wordDocument            = null;
            Microsoft.Office.Interop.Word.WdExportFormat            paramExportFormat       = WdExportFormat.wdExportFormatPDF;
            Microsoft.Office.Interop.Word.WdExportOptimizeFor       paramExportOptimizeFor  = WdExportOptimizeFor.wdExportOptimizeForPrint;
            Microsoft.Office.Interop.Word.WdExportRange             paramExportRange        = WdExportRange.wdExportAllDocument;
            Microsoft.Office.Interop.Word.WdExportItem              paramExportItem         = WdExportItem.wdExportDocumentContent;
            Microsoft.Office.Interop.Word.WdExportCreateBookmarks   paramCreateBookmarks    = WdExportCreateBookmarks.wdExportCreateWordBookmarks;

            object  paramSourceDocPath          = null;
            object  paramMissing                = Type.Missing;
            object  paramSaveChanges            = false;
            object  ConfirmConversions          = false;
            object  ConfirmConversions1         = true;
            object  SaveChanges                 = false;
            String  ErrorString                 = null;
            int     paramStartPage              = 0;
            int     paramEndPage                = 0;
            bool    paramOpenAfterExport        = false;
            bool    paramIncludeDocProps        = true;
            bool    paramKeepIRM                = true;
            bool    paramDocStructureTags       = true;
            bool    paramBitmapMissingFonts     = true;
            bool    paramUseISO19005_1          = false;
            # endregion

            try
            {
                wordApplication = new Microsoft.Office.Interop.Word.ApplicationClass();
                paramSourceDocPath = SourcePath;

                // Open the source document.
                wordDocument = wordApplication.Documents.Open(
                    ref paramSourceDocPath, ref ConfirmConversions, ref ConfirmConversions1, ref paramMissing, ref paramMissing,
                    ref paramMissing, ref paramMissing, ref paramMissing, ref paramMissing, ref paramMissing, ref paramMissing,
                    ref paramMissing, ref paramMissing, ref paramMissing, ref paramMissing, ref paramMissing);

                if (wordDocument != null)
                {
                    System.Threading.Thread.Sleep(5000);
                    // Page Setup for PDF
                    wordDocument.PageSetup.PaperSize = WdPaperSize.wdPaperLegal;
                    wordDocument.PageSetup.Orientation = WdOrientation.wdOrientLandscape;
                    wordDocument.PageSetup.LeftMargin = 25.0F;
                    wordDocument.PageSetup.RightMargin = 25.0F;
                    wordDocument.PageSetup.TopMargin = 25.0F;
                    wordDocument.PageSetup.BottomMargin = 25.0F;

                    // Export to PDF
                    wordDocument.ExportAsFixedFormat(
                        DestinationPath, paramExportFormat, paramOpenAfterExport, paramExportOptimizeFor, paramExportRange,
                        paramStartPage, paramEndPage, paramExportItem, paramIncludeDocProps, paramKeepIRM, paramCreateBookmarks, 
                        paramDocStructureTags, paramBitmapMissingFonts, paramUseISO19005_1, ref paramMissing);
                }
            }
            catch (Exception ExcepObj)
            {
                //ErrorString = HelperFunctions.prepareExceptionXML(ExcepObj);
                ErrorString = ExcepObj.Message;
            }

            # region finally
            finally
            {

                // Close and release the Document object.
                if (wordDocument != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(wordDocument);
                    wordDocument = null;
                }

                // Quit Word and release the ApplicationClass object.
                if (wordApplication != null)
                {
                    wordApplication.Documents.Close(ref paramSaveChanges, ref paramMissing, ref paramMissing);
                    wordApplication.Quit(ref paramMissing, ref paramMissing, ref paramMissing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(wordApplication);
                    wordApplication = null;
                }
            }
            # endregion

            return ErrorString;
        }

        /// <summary>
        /// This function converts MHtm File to Excel
        /// </summary>
        /// <param name="SourcePath">Source MHTML file Path</param>
        /// <param name="DestinationPath">Target Path for Excel file</param>
        /// <returns>Returns null on success else returns error xml</returns>
        public ERRORXML GenerateExcelFromMHtml(String SourcePath, String DestinationPath)
        {
            # region Local variables
            Microsoft.Office.Interop.Excel.ApplicationClass     xlAppClass              = null;
            Microsoft.Office.Interop.Excel.Workbook             xlWB                    = null;
            object                                              TargetFormat            = XlFileFormat.xlExcel7;
            object                                              ExportFilePathObject    = null;
            String                                              ErrorString             = null;
            # endregion

            try
            {
                ExportFilePathObject = DestinationPath;
                xlAppClass = new Microsoft.Office.Interop.Excel.ApplicationClass();

                // Open the source document.
                xlWB = xlAppClass.Workbooks.Open(
                    SourcePath, Type.Missing, false, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing,
                    Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                // Export/Save it in the specified format.
                if (xlWB != null)
                {
                    xlWB.SaveAs(DestinationPath, TargetFormat, Type.Missing, Type.Missing, false, Type.Missing, XlSaveAsAccessMode.xlExclusive, 
                        Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                }
            }
            catch (Exception ExcepObj)
            {
                //ErrorString = HelperFunctions.prepareExceptionXML(ExcepObj);
                ErrorString = ExcepObj.Message;
            }
            finally
            {
                if (xlWB != null)
                {
                    xlWB.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(xlWB);
                    xlWB = null;
                }
                if (xlAppClass != null)
                {
                    xlAppClass.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(xlAppClass);
                    xlAppClass = null;
                }
            }

            return ErrorString;
        }

        /// <summary>
        /// <CreatedBy>Ankit Agrawal</CreatedBy>
        /// <CreatedDate>7th February 2011</CreatedDate>
        /// <Purpose>This method is used to read the Excel for facet series using the configuration and data provided.</Purpose>
        /// </summary>
        /// <param name="FilePathForFacetSeriesConfig">This contains the list data XML with configuration which is used for Excel generation.</param>        
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        public ERRORXML ImportExcelForFacetSeries(XML FacetSeriesConfigXML, XML SeriesIDListXML, ref XML xml_FileCollection)
        {
            #region Local Variables
            String                              ErrorXML                    = null;
            String                              TargetObjectName            = null;
            String                              TargetObjectType            = null;
            String                              PrimaryKey                  = null;            
            String                              FieldType                   = null;
            String                              ColumnName                  = null;
            String                              DisplayState                = null;
            String                              ExcelFile                   = null;
            String                              FieldValue                  = null;
            String                              OldFieldValue               = null;
            String                              UIType                      = null;
            String                              IsReadOnly                  = null;
            String                              DoAutoGenerate              = null;
            String                              SeriesID                    = null;
            XmlDocument                         ListDataDoc                 = null;
            XmlDocument                         SearchResultXmlDoc          = null;
            XmlNode                             ListNode                    = null;
            XPathDocument                       xpDoc_ObjectTypeInfo        = null;
            XPathNavigator                      xpNav_ObjectTypeInfo        = null;
            XPathNavigator                      FieldNav                    = null;
            XmlNode                             ColumnNode                  = null;            
            Dictionary<String, HeaderInfo>      HeaderInfoDictionary        = new Dictionary<string, HeaderInfo>();
            HeaderInfo                          HeaderInfoObj               = null;
            
            Microsoft.Office.Interop.Excel.Application  ObjApplication      = null;
            Workbook                                    ObjWorkBook         = null;
            Sheets                                      ObjSheet            = null;
            Boolean                                     IsNewRecord         = false;
            Boolean                                     IsEmptyRecord       = true;
            Boolean                                     IsCanceled          = false;
            StringBuilder                               sb_ImportErrorXML   = null;
            XmlNode                                     TargetObjectNode    = null;
            HeaderInfo                                  PrimaryColumnInfo   = null;
            StringBuilder                               TargetAttributeXml  = null;
            String                                      str_ExcelColumnName = null;
            XmlNodeList                                 HeaderColumns       = null; 
            
            XML                                         xml_ExcelGenerationStatusLog    = null;
            String                                      str_StatusCompleteFilePath      = null;
            String                                      TotalNumberOfRecords            = null;
            String                                      Recordsprocessed                = null;
            XML                                         xml_ExcelGenerationProcessedLog = null;
            String                                      str_MultiSelectDelimiter        = null;
            XmlDocument                                 xmlDoc_SeriesIDList             = null;
            XmlNode                                     xmlNode_SeriesIDNodes           = null;
            StringBuilder                               sb_FileCollectionXML            = new StringBuilder("<FILECOLLECTION>");
            ulong                                       PrimaryKeyID                    = 0;
            XML                                         xml_FacetSeriesConfig           = null; 
            XML                                         xml_FacetSeriesListData         = null; 
            XML                                         xml_SearchResultData            = null; 
            XML                                         ObjectTypeInfoXML               = null;
            String                                      str_Err_RECORDNOTEXISTING       = null;
            String                                      str_Err_RECORDIMPROPER          = null;
            String                                      IDList                          = null;
            #endregion

            try
            {
                //Read details stored into the file at shared location.
                xml_FacetSeriesConfig = FacetSeriesConfigXML;

                //Retrieve the necessary details from the config XML.
                xml_FacetSeriesListData = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FACETSERIESLISTDATAXML>", "</FACETSERIESLISTDATAXML>");
                xml_SearchResultData = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FACETSERIESSEARCHXML>", "</FACETSERIESSEARCHXML>");
                ExcelFile = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<EXCELCOMPLETEFILEPATH>", "</EXCELCOMPLETEFILEPATH>");
                ObjectTypeInfoXML = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<OBJECTTYPEINFOXML>", "</OBJECTTYPEINFOXML>");
                str_StatusCompleteFilePath = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<STATUSCOMPLETEFILEPATH>", "</STATUSCOMPLETEFILEPATH>");
                str_MultiSelectDelimiter = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<MULTISELECETDELIMITER>", "</MULTISELECETDELIMITER>");                
                str_Err_RECORDNOTEXISTING = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<ERR_FSIMPORT_RECORDNOTEXISTING>", "</ERR_FSIMPORT_RECORDNOTEXISTING>");
                str_Err_RECORDIMPROPER = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<ERR_FSIMPORT_RECORDIMPROPER>", "</ERR_FSIMPORT_RECORDIMPROPER>");

                if (!String.IsNullOrEmpty(SeriesIDListXML))
                {
                    xmlDoc_SeriesIDList = new XmlDocument();
                    xmlDoc_SeriesIDList.LoadXml(SeriesIDListXML);                    
                    xmlNode_SeriesIDNodes = xmlDoc_SeriesIDList.SelectSingleNode("COLUMNS");
                }

                ListDataDoc = new XmlDocument();
                ListDataDoc.LoadXml(xml_FacetSeriesListData);

                SearchResultXmlDoc = new XmlDocument();
                SearchResultXmlDoc.LoadXml(xml_SearchResultData);

                ListNode = ListDataDoc.SelectSingleNode("List");
                TargetObjectName = ((XmlElement)ListNode).GetAttribute("TARGETOBJECTNAME", "");
                TargetObjectType = ((XmlElement)ListNode).GetAttribute("TARGETOBJECTTYPE", "");
                PrimaryKey = ((XmlElement)ListNode).GetAttribute("TARGETOBJECTPRIMARYKEY", "");

                #region initialize search result doc
                if (SearchResultXmlDoc.FirstChild == null)
                    SearchResultXmlDoc.InnerXml = "<SEARCHRESULT><" + TargetObjectName + "COLLECTION/></SEARCHRESULT>";
                else
                    if (SearchResultXmlDoc.FirstChild.FirstChild == null)
                        SearchResultXmlDoc.FirstChild.InnerXml = "<" + TargetObjectName + "COLLECTION/>";
                #endregion

                HeaderColumns = ListNode.SelectNodes("Header/Column");                
                if (!String.IsNullOrEmpty(TargetObjectName) && !String.IsNullOrEmpty(TargetObjectType))
                {                    
                    if (!String.IsNullOrEmpty(ObjectTypeInfoXML))
                    {
                        xpDoc_ObjectTypeInfo = new XPathDocument(new StringReader(ObjectTypeInfoXML));
                        xpNav_ObjectTypeInfo = xpDoc_ObjectTypeInfo.CreateNavigator();
                    }
                                        
                    if (HeaderColumns != null)
                    {
                        for (int Index = 0; Index < HeaderColumns.Count; Index++)
                        {
                            ColumnName = ((XmlElement)HeaderColumns[Index]).GetAttribute("Name");
                            UIType = ((XmlElement)HeaderColumns[Index]).GetAttribute("UIType");
                            DisplayState = ((XmlElement)HeaderColumns[Index]).GetAttribute("DisplayState");

                            if (HeaderColumns[Index].Attributes["IsReadOnly"] != null)
                                IsReadOnly = HeaderColumns[Index].Attributes["IsReadOnly"].Value;

                            if (HeaderColumns[Index].Attributes["DoAutoGenerate"] != null)
                                DoAutoGenerate = HeaderColumns[Index].Attributes["DoAutoGenerate"].Value;

                            if (HeaderColumns[Index].Attributes["SeriesID"] != null)
                                SeriesID = HeaderColumns[Index].Attributes["SeriesID"].Value;

                            FieldNav = xpNav_ObjectTypeInfo.SelectSingleNode("/TYPE/FIELDCOLLECTION/FIELD[FIELDNAME/text()='" + ColumnName + "']");

                            if (FieldNav != null)
                                FieldType = FieldNav.SelectSingleNode("FIELDTYPE").Value;
                            else
                                FieldType = "";

                            HeaderInfoDictionary.Add(ColumnName, new HeaderInfo(ColumnName, UIType, FieldType, DisplayState, PrimaryKey, IsReadOnly, DoAutoGenerate, SeriesID, null));
                        }

                        #region Read Excel File                        

                        int ExcelUsedRowcount = 0;
                        int ExcelUsedColumnCount = 0;

                        ObjApplication = new Microsoft.Office.Interop.Excel.Application();
                        ObjWorkBook = ObjApplication.Workbooks.Open(ExcelFile, 0, true, Type.Missing, "", "", false, Type.Missing, "", true, false, 0, false, Type.Missing, Type.Missing);
                        ObjSheet = ObjWorkBook.Worksheets;

                        Microsoft.Office.Interop.Excel.Worksheet ObjWorkSheet = (Microsoft.Office.Interop.Excel.Worksheet)ObjSheet.get_Item(1);
                        ObjWorkSheet.Activate();

                        #region Column Count in excel (used columns)
                        int index = 1;
                        Microsoft.Office.Interop.Excel.Range ObjRangeC = (Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.UsedRange;
                        Microsoft.Office.Interop.Excel.Range UColumns = ObjRangeC.Columns;
                        ExcelUsedColumnCount = UColumns.Count;
                        #endregion

                        #region Identify excel cell index for columns in List config
                        while (index <= ExcelUsedColumnCount)
                        {
                            if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Comment != null)
                            {
                                String CommentText = ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Comment.Shape.AlternativeText;
                                ColumnName = CommentText.Substring(CommentText.IndexOf(":") + 2);
                                if (HeaderInfoDictionary.TryGetValue(ColumnName, out HeaderInfoObj))
                                    HeaderInfoObj.ExcelCellIndex = index;
                            }
                            else
                            {
                                if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Value2 != null)
                                {
                                    ColumnNode = ListNode.SelectSingleNode("Header/Column[@DisplayValue='" + ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Value2.ToString() + "']");
                                    if (ColumnNode != null)
                                    {
                                        ColumnName = ((XmlElement)ColumnNode).GetAttribute("FIELDNAME");
                                        if (HeaderInfoDictionary.TryGetValue(ColumnName, out HeaderInfoObj))
                                            HeaderInfoObj.ExcelCellIndex = index;

                                    }
                                }
                            }

                            if (HeaderInfoObj.ExcelCellIndex > -1)
                            {
                                if (HeaderInfoObj.DoAutoGenerate && HeaderInfoObj.IsVisible)
                                {
                                    if (xmlNode_SeriesIDNodes != null)
                                    {
                                        IDList = xmlNode_SeriesIDNodes.SelectSingleNode("COLUMN[@ExcelCellIndex='"+index.ToString()+"']").InnerText;
                                        HeaderInfoObj.SeriesIDList = IDList.Split(new String[] { "$AL$" }, StringSplitOptions.None);
                                    }
                                }
                            }

                            index++;
                        }
                        #endregion

                        #region Row Count in excel (used rows)
                        Microsoft.Office.Interop.Excel.Range ObjRangeR = (Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.UsedRange;
                        Microsoft.Office.Interop.Excel.Range URows = ObjRangeR.Rows;
                        ExcelUsedRowcount = URows.Count;
                        #endregion

                        PrimaryColumnInfo = HeaderInfoDictionary[PrimaryKey];
                        TotalNumberOfRecords = Convert.ToString(ExcelUsedRowcount - 1);

                        //Add total record count into the progress status XML.
                        xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><TOTALNUMBEROFRECORDS>"
                                + TotalNumberOfRecords.ToString()
                                + "</TOTALNUMBEROFRECORDS><EXPORTSTATUS>$AL$StatusOfExport$AL$</EXPORTSTATUS>"
                                + "<RECORDSPROCESSED>$AL$ProcessedRecords$AL$</RECORDSPROCESSED></PROGRESSSTATUSUSEDBYTHREAD>";

                        //str_ErrorInfoFilePath = ALSolutionCommon.GetTempPath(ALParamObj.SessionID) + "ImportRecordsError.xml";
                        sb_ImportErrorXML = new StringBuilder();
                        sb_ImportErrorXML.Append("<ERRORCOLLECTION>");

                        for (int RowItr = 2; RowItr <= ExcelUsedRowcount; RowItr++)
                        {
                            lock (typeof(MSOffice_MediaPlugin))
                            {
                                IsCanceled = checkIsExcelGenerationCancelledEvent(str_StatusCompleteFilePath);                               
                            }
                            if (IsCanceled)
                            {
                                break;
                            }

                            IsNewRecord = false;
                            IsEmptyRecord = true;
                            TargetObjectNode = null;
                            TargetAttributeXml = new StringBuilder(215);

                            #region Check if data for primary column exist
                            if (PrimaryColumnInfo.ExcelCellIndex > 0 && ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[RowItr, PrimaryColumnInfo.ExcelCellIndex]).get_Value(Type.Missing) != null)
                            {
                                FieldValue = ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[RowItr, PrimaryColumnInfo.ExcelCellIndex]).get_Value(Type.Missing).ToString();
                                if (!String.IsNullOrEmpty(FieldValue))
                                {
                                    TargetObjectNode = SearchResultXmlDoc.SelectSingleNode("*/*/" + TargetObjectName + "[" + PrimaryKey + "/text()='" + FieldValue + "']");
                                    if (TargetObjectNode == null) //this record is deleted from database
                                    {
                                        sb_ImportErrorXML.Append("<ERROR>");
                                        sb_ImportErrorXML.Append("<ERRORCODE>ERR_FSIMPORT_RECORDNOTEXISTING</ERRORCODE>");
                                        sb_ImportErrorXML.Append("<ERRORINFORMATION>" + str_Err_RECORDNOTEXISTING + "</ERRORINFORMATION>");
                                        sb_ImportErrorXML.AppendFormat("<ERRORROWNUMBER>{0}</ERRORROWNUMBER>", RowItr.ToString());
                                        sb_ImportErrorXML.Append("</ERROR>");
                                        continue;
                                    }
                                    else
                                    {
                                        //Record need to be updated.
                                        TargetAttributeXml.Append("<" + PrimaryKey + ">" + FieldValue + "</" + PrimaryKey + ">");
                                        ((XmlElement)TargetObjectNode).SetAttribute("Operation", "Update");
                                        ((XmlElement)TargetObjectNode).SetAttribute("Visited", "True");
                                    }
                                }
                            }

                            #region Primary key coumn data is not found or is Empty                            
                            if (TargetObjectNode == null)
                            {
                                IsNewRecord = true;
                                TargetObjectNode = SearchResultXmlDoc.CreateElement(TargetObjectName);
                                ALCommon.GenerateUniqueID(ref PrimaryKeyID);
                                TargetAttributeXml.Append("<" + PrimaryKey + ">" + PrimaryKeyID.ToString() + "</" + PrimaryKey + ">");
                                ((XmlElement)TargetObjectNode).SetAttribute("Operation", "Create");
                                ((XmlElement)TargetObjectNode).SetAttribute("Visited", "True");
                            }
                            #endregion
                            #endregion

                            foreach (HeaderInfo ListHeader in HeaderInfoDictionary.Values)
                            {
                                FieldValue = "";
                                OldFieldValue = "";

                                if (ListHeader.IsPrimaryKey || ListHeader.ExcelCellIndex == -1)
                                    continue;

                                if (TargetObjectNode.SelectSingleNode(ListHeader.Name) != null)
                                    OldFieldValue = TargetObjectNode.SelectSingleNode(ListHeader.Name).InnerXml;

                                if (!ListHeader.IsVisible)
                                {
                                    TargetAttributeXml.Append("<" + ListHeader.Name + ">" + OldFieldValue + "</" + ListHeader.Name + ">");
                                }
                                else
                                {
                                    if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[RowItr, ListHeader.ExcelCellIndex]).get_Value(Type.Missing) != null)
                                    {
                                        FieldValue = ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[RowItr, ListHeader.ExcelCellIndex]).get_Value(Type.Missing).ToString();
                                        str_ExcelColumnName = ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, ListHeader.ExcelCellIndex]).get_Value(Type.Missing).ToString();
                                        if (ValidateFieldValuewithType(ListHeader, FieldValue, str_ExcelColumnName, RowItr.ToString(), ref sb_ImportErrorXML, str_Err_RECORDIMPROPER))
                                            TargetAttributeXml.Append(PrepareTargetAttibuteNode(ListHeader, ListHeader.Name, OldFieldValue, FieldValue, IsNewRecord, str_MultiSelectDelimiter));
                                        else
                                            TargetAttributeXml.Append(PrepareTargetAttibuteNode(ListHeader, ListHeader.Name, OldFieldValue, "", IsNewRecord, str_MultiSelectDelimiter));
                                    }
                                    else
                                    {
                                        TargetAttributeXml.Append(PrepareTargetAttibuteNode(ListHeader, ListHeader.Name, OldFieldValue, "", IsNewRecord, str_MultiSelectDelimiter));
                                    }
                                }

                                if (!String.IsNullOrEmpty(FieldValue))
                                    IsEmptyRecord = false;
                            }
                            TargetObjectNode.InnerXml = TargetAttributeXml.ToString();

                            if (IsNewRecord && (!IsEmptyRecord))
                                SearchResultXmlDoc.FirstChild.FirstChild.AppendChild(TargetObjectNode);


                            Recordsprocessed = Convert.ToString(RowItr - 1);
                            lock (typeof(MSOffice_MediaPlugin))
                            {
                                IsCanceled = checkIsExcelGenerationCancelledEvent(str_StatusCompleteFilePath);

                                xml_ExcelGenerationProcessedLog = xml_ExcelGenerationStatusLog.Replace("$AL$StatusOfExport$AL$", "NEW");
                                xml_ExcelGenerationProcessedLog = xml_ExcelGenerationProcessedLog.Replace("$AL$ProcessedRecords$AL$",
                                        Recordsprocessed);
                                if (!IsCanceled)
                                {                                    
                                    File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationProcessedLog);
                                }
                            }
                        }

                        sb_ImportErrorXML.Append("</ERRORCOLLECTION>");
                        sb_FileCollectionXML.Append(sb_ImportErrorXML.ToString());

                        #endregion

                        lock (typeof(MSOffice_MediaPlugin))
                        {
                            IsCanceled = checkIsExcelGenerationCancelledEvent(str_StatusCompleteFilePath);
                        }
                        if (!IsCanceled)
                        {
                            #region Save Xmls
                            #region Save Operations Xml                                                        
                            XmlDocument OperationXmlDoc = new XmlDocument();
                            OperationXmlDoc.LoadXml("<OPERATION/>");
                            OperationXmlDoc.FirstChild.InnerXml = SearchResultXmlDoc.FirstChild.FirstChild.InnerXml;

                            foreach (XmlNode OperationTargetObjectNode in OperationXmlDoc.SelectNodes("//" + TargetObjectName + "[not (@Visited)]"))
                                OperationTargetObjectNode.ParentNode.RemoveChild(OperationTargetObjectNode);

                            foreach (XmlNode OperationTargetObjectNode in OperationXmlDoc.SelectNodes("//" + TargetObjectName + "[(@Visited)]"))
                                ((XmlElement)OperationTargetObjectNode).RemoveAttribute("Visited");
                                                       
                            sb_FileCollectionXML.Append(OperationXmlDoc.InnerXml);
                            #endregion

                            #region save Search result xml

                            foreach (XmlNode OperationTargetObjectNode in SearchResultXmlDoc.SelectNodes("//" + TargetObjectName + "[(@Visited)]"))
                                ((XmlElement)OperationTargetObjectNode).RemoveAttribute("Visited");

                            foreach (XmlNode OperationTargetObjectNode in SearchResultXmlDoc.SelectNodes("//" + TargetObjectName + "[(@Operation)]"))
                                ((XmlElement)OperationTargetObjectNode).RemoveAttribute("Operation");

                            sb_FileCollectionXML.Append(SearchResultXmlDoc.InnerXml);
                            #endregion
                            #endregion
                        }
                        else
                        {
                            sb_FileCollectionXML.Append("</FILECOLLECTION>");
                            xml_FileCollection = sb_FileCollectionXML.ToString();
                            if (!(String.IsNullOrEmpty(str_StatusCompleteFilePath)))
                            {
                                xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>CANCELED</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                                lock (typeof(MSOffice_MediaPlugin))
                                {
                                    File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationStatusLog);
                                }
                            }
                            return ErrorXML;
                            //return "CANCELED";
                        }
                    }
                }
                else
                {
                    sb_FileCollectionXML.Append("</FILECOLLECTION>");
                    xml_FileCollection = sb_FileCollectionXML.ToString();
                    if (!(String.IsNullOrEmpty(str_StatusCompleteFilePath)))
                    {
                        xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>FAIL</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                        lock (typeof(MSOffice_MediaPlugin))
                        {
                            File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationStatusLog);
                        }
                    }
                    return ErrorXML;
                    //return "FAIL";
                }

                sb_FileCollectionXML.Append("</FILECOLLECTION>");
                xml_FileCollection = sb_FileCollectionXML.ToString();

                return ErrorXML;
                //return "SUCCESS";
            }
            catch (Exception ExceptionObject)
            {
                ErrorXML = ExceptionObject.Message;
                if (!(String.IsNullOrEmpty(str_StatusCompleteFilePath)))
                {
                    xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>FAIL</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                    lock (typeof(MSOffice_MediaPlugin))
                    {
                        File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationStatusLog);
                    }
                }
            }
            finally
            {
                if (ObjApplication != null)
                    ObjApplication.Quit();
            }
            sb_FileCollectionXML.Append("</FILECOLLECTION>");
            xml_FileCollection = sb_FileCollectionXML.ToString();
            return ErrorXML;
        }

        public ERRORXML GetExcelRecordCountForFacetSeries(XML FacetSeriesConfigXML, ref String ColListXML)
        {
            #region Member Variables
            Microsoft.Office.Interop.Excel.Application  ObjApplication              = null;
            Workbook                                    ObjWorkBook                 = null;
            Sheets                                      ObjSheet                    = null;
            String                                      ExcelFile                   = null;
            int                                         ExcelUsedRowcount           = 0;
            ERRORXML                                    ErrorXML                    = null;
            String                                      str_StatusCompleteFilePath  = null;
            XmlDocument                                 ListDataDoc                 = null;
            XmlNode                                     ListNode                    = null;
            XmlNodeList                                 HeaderColumns               = null;
            String                                      ColumnName                  = null;
            String                                      DisplayState                = null;
            String                                      DoAutoGenerate              = null;
            String                                      SeriesID                    = null;
            Dictionary<String, HeaderInfo>              HeaderInfoDictionary        = new Dictionary<string, HeaderInfo>();
            HeaderInfo                                  HeaderInfoObj               = null;
            XmlNode                                     ColumnNode                  = null;
            int                                         RecordCount                 = 0;
            XML                                         xml_ExcelGenerationStatusLog = null; 
            #endregion

            try
            {
                str_StatusCompleteFilePath = ALCommon.GetInnerValue(ref FacetSeriesConfigXML, "<STATUSCOMPLETEFILEPATH>", "</STATUSCOMPLETEFILEPATH>");
                ExcelFile = ALCommon.GetInnerValue(ref FacetSeriesConfigXML, "<EXCELCOMPLETEFILEPATH>", "</EXCELCOMPLETEFILEPATH>");
                ObjApplication = new Microsoft.Office.Interop.Excel.Application();
                ObjWorkBook = ObjApplication.Workbooks.Open(ExcelFile, 0, true, Type.Missing, "", "", false, Type.Missing, "", true, false, 0, false, Type.Missing, Type.Missing);
                ObjSheet = ObjWorkBook.Worksheets;

                Microsoft.Office.Interop.Excel.Worksheet ObjWorkSheet = (Microsoft.Office.Interop.Excel.Worksheet)ObjSheet.get_Item(1);
                ObjWorkSheet.Activate();

                #region Row Count in excel (used rows)
                Microsoft.Office.Interop.Excel.Range ObjRangeR = (Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.UsedRange;
                Microsoft.Office.Interop.Excel.Range URows = ObjRangeR.Rows;
                ExcelUsedRowcount = URows.Count;
                #endregion
                                
                XML xml_FacetSeriesListData = ALCommon.GetInnerValue(ref FacetSeriesConfigXML, "<FACETSERIESLISTDATAXML>", "</FACETSERIESLISTDATAXML>");

                ListDataDoc = new XmlDocument();
                ListDataDoc.LoadXml(xml_FacetSeriesListData);
                ListNode = ListDataDoc.SelectSingleNode("List");

                HeaderColumns = ListNode.SelectNodes("Header/Column");                

                if (HeaderColumns != null)
                {
                    for (int Index = 0; Index < HeaderColumns.Count; Index++)
                    {
                        ColumnName = ((XmlElement)HeaderColumns[Index]).GetAttribute("Name");                        
                        DisplayState = ((XmlElement)HeaderColumns[Index]).GetAttribute("DisplayState");

                        if (HeaderColumns[Index].Attributes["DoAutoGenerate"] != null)
                            DoAutoGenerate = HeaderColumns[Index].Attributes["DoAutoGenerate"].Value;

                        if (HeaderColumns[Index].Attributes["SeriesID"] != null)
                            SeriesID = HeaderColumns[Index].Attributes["SeriesID"].Value;

                        HeaderInfoDictionary.Add(ColumnName, new HeaderInfo(ColumnName, null, null, DisplayState, null, null, DoAutoGenerate, SeriesID, null));
                    }
                }

                #region Column Count in excel (used columns)
                int index = 1;
                Microsoft.Office.Interop.Excel.Range ObjRangeC = (Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.UsedRange;
                Microsoft.Office.Interop.Excel.Range UColumns = ObjRangeC.Columns;
                int ExcelUsedColumnCount = UColumns.Count;                
                #endregion

                #region Identify excel cell index for columns in List config
                while (index <= ExcelUsedColumnCount)
                {
                    if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Comment != null)
                    {
                        String CommentText = ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Comment.Shape.AlternativeText;
                        ColumnName = CommentText.Substring(CommentText.IndexOf(":") + 2);
                        if (HeaderInfoDictionary.TryGetValue(ColumnName, out HeaderInfoObj))
                            HeaderInfoObj.ExcelCellIndex = index;
                    }
                    else
                    {
                        if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Value2 != null)
                        {
                            ColumnNode = ListNode.SelectSingleNode("Header/Column[@DisplayValue='" + ((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[1, index]).Value2.ToString() + "']");
                            if (ColumnNode != null)
                            {
                                ColumnName = ((XmlElement)ColumnNode).GetAttribute("FIELDNAME");
                                if (HeaderInfoDictionary.TryGetValue(ColumnName, out HeaderInfoObj))
                                    HeaderInfoObj.ExcelCellIndex = index;

                            }
                        }
                    }
                    index++;
                }
                #endregion
                                
                ColListXML = "<COLUMNS>";
                foreach (HeaderInfo ListHeader in HeaderInfoDictionary.Values)
                {
                    if (ListHeader.ExcelCellIndex > -1)
                    {
                        if (ListHeader.DoAutoGenerate && ListHeader.IsVisible)
                        {
                            ColListXML += "<COLUMN ExcelCellIndex='" + ListHeader.ExcelCellIndex.ToString() + "' SeriesID='" + ListHeader.SeriesID + "'>";
                            RecordCount = 0;
                            for (int RowItr = 2; RowItr <= ExcelUsedRowcount; RowItr++)
                            {
                                if (((Microsoft.Office.Interop.Excel.Range)ObjWorkSheet.Cells[RowItr, ListHeader.ExcelCellIndex]).get_Value(Type.Missing) == null)
                                    RecordCount++;
                            }
                            ColListXML += RecordCount.ToString() + "</COLUMN>";
                        }
                    }
                }
                ColListXML += "</COLUMNS>";                
            }
            catch (Exception ExceptionObject)
            {
                ErrorXML = ExceptionObject.Message;
                if (!(String.IsNullOrEmpty(str_StatusCompleteFilePath)))
                {
                    xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>FAIL</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                    lock (typeof(MSOffice_MediaPlugin))
                    {
                        File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationStatusLog);
                    }
                }
            }
            return ErrorXML;
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>11th February 2011</CreatedDate>
        /// <Purpose>This method is used to generate the Excel template for facet series using the configuration provided.</Purpose>
        /// </summary>
        /// <param name="FilePathForFacetSeriesConfig">This contains the list data XML with configuration which is used for Excel generation.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        public ERRORXML GenerateImportExcelTemplateForFacetSeries(String FilePathForFacetSeriesConfig, out XML FileCollectionXML)
        {
            #region Method Variables

            Microsoft.Office.Interop.Excel.ApplicationClass excelapplication_Object         = null;
            Microsoft.Office.Interop.Excel.WorkbookClass    excelworkbook_Object            = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_Object           = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_LookUpObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_CellObject           = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_LookUpCellObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_StartCellObject      = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_EndCellObject        = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_DataValidationObject = null;

            ALErrorCommon       errorcommon_Object          = null;
            XPathDocument       xpathdoc_FormConfig         = null;
            XPathDocument       xpathdoc_NamedHierarchy     = null;
            XPathNodeIterator   xpathiter_TargetColumns     = null;
            XPathNodeIterator   xpathiter_ColumnsCloneCopy  = null;
            XPathNodeIterator   xpathiter_NamedHierarchy    = null;
            XPathNavigator      xpathnav_FormConfigRoot     = null;
            XPathNavigator      xpathnav_NamedHierarchy     = null;
            Int32               int_ColumnIterator          = 1;
            Int32               int_LookUpColumnIndex       = 1;
            Int32               int_LookUpRowIndex          = 2;
            String              str_XPath                   = null;
            String              str_TargetObjectName        = null;
            String              str_PrimaryKey              = null;
            String              str_ColumnName              = null;
            String              str_DisplayValue            = null;
            String              str_UIType                  = null;
            String              str_NamedHierarchyID        = null;
            String              str_NamedHierarchyFilePath  = null;
            String              str_NameList                = null;
            String              str_ExcelCompleteFilePath   = null;
            String              str_ExcelFileName           = null;
            String              str_SectionOrder            = null;
            String              str_ColumnOrder             = null;
            String              str_FieldOrder              = null;
            String              str_MSOfficeProcessorNetworkPath = null;
            Boolean             bool_FlagUseAllColumns      = false;
            StringBuilder       strbldr_NameList            = null;
            XML                 xml_FacetSeriesConfig       = null;
            XML                 xml_FacetSeriesFormConfig   = null;
            ERRORXML            errorxml_Output             = null;

            #endregion

            FileCollectionXML = null;
            try
            {
                #region STEP00: Mandatory checks

                errorcommon_Object = new ALErrorCommon();
                if (!(File.Exists(FilePathForFacetSeriesConfig)))
                {
                    errorcommon_Object.AppendError("ERR_FS_CONFIGFILEPATHNOTFOUND", this.GetType().ToString(),
                            "GenerateExcelForFacetSeries", null, errorxml_Output);
                    return (errorcommon_Object.ErrorCollectionXML);
                }

                #endregion

                #region STEP01: Retrieve mandatory configuration values

                //Read details stored into the file at shared location.
                xml_FacetSeriesConfig = File.ReadAllText(FilePathForFacetSeriesConfig);

                //Retrieve the necessary details from the config XML.
                xml_FacetSeriesFormConfig   = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FACETSERIESFORMXML>", "</FACETSERIESFORMXML>");
                str_ExcelCompleteFilePath   = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<EXCELCOMPLETEFILEPATH>", "</EXCELCOMPLETEFILEPATH>");
                str_ExcelFileName           = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<EXCELFILENAME>", "</EXCELFILENAME>");
                str_SectionOrder            = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<SECTIONORDER>", "</SECTIONORDER>");
                str_ColumnOrder             = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<COLUMNORDER>", "</COLUMNORDER>");
                str_FieldOrder              = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FIELDORDER>", "</FIELDORDER>");
                str_MSOfficeProcessorNetworkPath = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<MSOFFICEPROCESSORNETWORKPATH>", "</MSOFFICEPROCESSORNETWORKPATH>");

                #endregion

                #region STEP02: Retrieve the required parameters and attributes from the form configuration XML

                xpathdoc_FormConfig     = new XPathDocument(new StringReader(xml_FacetSeriesFormConfig));
                xpathnav_FormConfigRoot = xpathdoc_FormConfig.CreateNavigator();
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode("Form");
                str_XPath = "Section [@Order='" + str_SectionOrder+ "'] / Column [@Order='" + str_ColumnOrder 
                        + "'] / Field [@Order='" + str_FieldOrder + "']";
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode(str_XPath);

                bool_FlagUseAllColumns  = Boolean.Parse(xpathnav_FormConfigRoot.GetAttribute("UseAllColumnsForImportExport", ""));
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode("Configuration/DisplayLayout/List");
                str_TargetObjectName    = xpathnav_FormConfigRoot.GetAttribute("TARGETOBJECTNAME", "");
                str_PrimaryKey          = xpathnav_FormConfigRoot.GetAttribute("TARGETOBJECTPRIMARYKEY", "");

                if (bool_FlagUseAllColumns)
                    xpathiter_TargetColumns = xpathnav_FormConfigRoot.Select("DisplayColumns/TargetObject/Column");
                else
                    xpathiter_TargetColumns = xpathnav_FormConfigRoot.Select("DisplayColumns/TargetObject/Column [translate(@DisplayState,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'YES']");

                xpathiter_ColumnsCloneCopy = xpathiter_TargetColumns.Clone();

                #endregion

                #region STEP03: Process data and generate the Excel

                excelapplication_Object = new Microsoft.Office.Interop.Excel.ApplicationClass();
                excelworkbook_Object = (WorkbookClass)excelapplication_Object.Workbooks.Add(ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls");
                excelworksheet_LookUpObject = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                excelworksheet_LookUpObject.Name = "LookUp Sheet";

                #region STEP04: Prepare look-up sheet using named-hierarchy values

                strbldr_NameList = new StringBuilder();
                if (xpathiter_ColumnsCloneCopy != null)
                {
                    while (xpathiter_ColumnsCloneCopy.MoveNext())
                    {
                        str_UIType = xpathiter_ColumnsCloneCopy.Current.GetAttribute("UIType", "");
                        str_NamedHierarchyID = xpathiter_ColumnsCloneCopy.Current.GetAttribute("NAMEDHIERARCHYID", "");
                        if (!String.IsNullOrEmpty(str_NamedHierarchyID))
                        {
                            int_LookUpRowIndex = 2;

                            str_NamedHierarchyFilePath = str_MSOfficeProcessorNetworkPath + @"\" + str_NamedHierarchyID + "_NHHierarchy.xml";
                            if (File.Exists(str_NamedHierarchyFilePath))
                            {
                                str_ColumnName = xpathiter_ColumnsCloneCopy.Current.GetAttribute("FIELDNAME", "");
                                xpathdoc_NamedHierarchy = new XPathDocument(str_NamedHierarchyFilePath);
                                xpathnav_NamedHierarchy = xpathdoc_NamedHierarchy.CreateNavigator();
                                xpathnav_NamedHierarchy = xpathnav_NamedHierarchy.SelectSingleNode("ROOT/NAMEDHIERARCHYVALUECOLLECTION");
                                if (xpathnav_NamedHierarchy != null)
                                {
                                    xpathiter_NamedHierarchy = xpathnav_NamedHierarchy.Select("NAMEDHIERARCHYVALUE/NAME");
                                    if (xpathiter_NamedHierarchy.Count > 0)
                                    {
                                        excelrange_LookUpCellObject = excelworksheet_LookUpObject.Cells[1, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                        excelrange_LookUpCellObject.Value2 = str_ColumnName;

                                        while (xpathiter_NamedHierarchy.MoveNext())
                                        {
                                            if (xpathiter_NamedHierarchy.CurrentPosition == 1)
                                                excelrange_StartCellObject = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;

                                            excelrange_LookUpCellObject = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                            excelrange_LookUpCellObject.Value2 = xpathiter_NamedHierarchy.Current.Value;
                                            int_LookUpRowIndex++;
                                        }
                                        excelworkbook_Object.Names.Add(str_ColumnName, excelworksheet_LookUpObject.get_Range(excelrange_StartCellObject, excelrange_LookUpCellObject), true, 3, Type.Missing, Type.Missing, Type.Missing,
                                                                    Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                                        if (str_UIType.Equals("DROPDOWNLIST", StringComparison.OrdinalIgnoreCase))
                                            strbldr_NameList.AppendFormat(" $;{0};$ ", str_ColumnName);

                                        int_LookUpColumnIndex++;
                                    }
                                }
                            }
                        }
                    }
                    str_NameList = strbldr_NameList.ToString();
                }

                #endregion

                #region STEP05: Add header columns

                excelworksheet_Object = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                excelworksheet_Object.Name = str_TargetObjectName;
                if (xpathiter_TargetColumns != null)
                {
                    while (xpathiter_TargetColumns.MoveNext())
                    {
                        str_ColumnName      = xpathiter_TargetColumns.Current.GetAttribute("FIELDNAME", "");
                        str_DisplayValue    = xpathiter_TargetColumns.Current.GetAttribute("DisplayValue", "");
                        if (str_ColumnName.ToUpper() != str_PrimaryKey.ToUpper())
                        {
                            excelrange_CellObject = excelworksheet_Object.Cells[1, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_CellObject.Value2 = str_DisplayValue;
                            excelrange_CellObject.AddComment(str_ColumnName);

                            if (str_NameList.Contains(" $;" + str_ColumnName + ";$ "))
                            {
                                excelrange_StartCellObject = excelworksheet_Object.Cells[2, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                                excelrange_EndCellObject = excelworksheet_Object.Cells[10001, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                                excelrange_DataValidationObject = excelworksheet_Object.get_Range(excelrange_StartCellObject, excelrange_EndCellObject);
                                excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateList, XlDVAlertStyle.xlValidAlertStop, XlFormatConditionOperator.xlEqual, "=" + str_ColumnName, Type.Missing);
                            }

                            int_ColumnIterator++;
                        }
                    }
                }

                #endregion

                #endregion

                #region STEP06: Save Excel

                if (File.Exists(str_ExcelCompleteFilePath))
                    File.Delete(str_ExcelCompleteFilePath);

                excelworkbook_Object.SaveAs(str_ExcelCompleteFilePath, Microsoft.Office.Interop.Excel.XlFileFormat.xlExcel8, Type.Missing,
                        Type.Missing, Type.Missing, Type.Missing, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange, 
                        Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                //Prepare file collection details for Excel view.
                FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_ExcelCompleteFilePath + "</PATH><NAME>"
                        + str_ExcelFileName + "</NAME></FILE></FILECOLLECTION>";

                #endregion
            }
            catch (Exception ExceptionObject)
            {
                errorxml_Output = ExceptionObject.Message;
            }
            finally
            {
                #region STEP07: Cleanups- Close the Excel objects

                if (excelrange_CellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_CellObject);
                    excelrange_CellObject = null;
                }

                if (excelrange_LookUpCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_LookUpCellObject);
                    excelrange_LookUpCellObject = null;
                }

                if (excelrange_StartCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_StartCellObject);
                    excelrange_StartCellObject = null;
                }

                if (excelrange_EndCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_EndCellObject);
                    excelrange_EndCellObject = null;
                }

                if (excelrange_DataValidationObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_DataValidationObject);
                    excelrange_DataValidationObject = null;
                }

                if (excelworksheet_Object != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_Object);
                    excelworksheet_Object = null;
                }

                if (excelworksheet_LookUpObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_LookUpObject);
                    excelworksheet_LookUpObject = null;
                }

                if (excelworkbook_Object != null)
                {
                    excelworkbook_Object.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworkbook_Object);
                    excelworkbook_Object = null;
                }

                if (excelapplication_Object != null)
                {
                    excelapplication_Object.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelapplication_Object);
                    excelapplication_Object = null;
                }

                #endregion            
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>31st January 2011</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for facet series using the configuration and data provided.</Purpose>
        /// </summary>
        /// <param name="FilePathForFacetSeriesConfig">This contains the list data XML with configuration which is used for Excel generation.</param>
        /// <param name="FilePathForFacetSeriesResultantData">This contains the resultant data XML which is used for Excel generation.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        public ERRORXML GenerateExcelForFacetSeries(String FilePathForFacetSeriesConfig, String FilePathForFacetSeriesResultantData, 
                out XML FileCollectionXML)
        {
            #region Method Variables

            Microsoft.Office.Interop.Excel.ApplicationClass excelapplication_Object         = null;
            Microsoft.Office.Interop.Excel.WorkbookClass    excelworkbook_Object            = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_Object           = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_LookUpObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_CellObject           = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_HeaderCellObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_LookUpCellObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_StartCellObject      = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_EndCellObject        = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_DataValidationObject = null;

            ALErrorCommon       errorcommon_Object              = null;
            XPathDocument       xpathdoc_ListData               = null;
            XPathDocument       xpathdoc_NamedHierarchy         = null;
            XPathDocument       xpathdoc_ResultantData          = null;
            XPathNodeIterator   xpathiter_NamedHierarchy        = null;
            XPathNodeIterator   xpathiter_ResultantRecord       = null;
            XPathNodeIterator   xpathiter_HeaderColumn          = null;
            XPathNodeIterator   xpathiter_CopyOfHeaderColumn    = null;
            XPathNavigator      xpathnav_ListData               = null;
            XPathNavigator      xpathnav_Column                 = null;
            XPathNavigator      xpathnav_NamedHierarchy         = null;
            XPathNavigator      xpathnav_ResultantData          = null;
            XPathExpression     xpathexpr_ResultantRecord       = null;
            Int32               int_RowIterator                 = 2;
            Int32               int_ColumnIterator              = 1;
            Int32               int_LookUpColumnIndex           = 1;
            Int32               int_LookUpRowIndex              = 2;
            String              str_TargetObjectName            = null;
            String              str_PrimaryKey                  = null;
            String              str_ColumnName                  = null;
            String              str_DisplayValue                = null;
            String              str_SelectedRowsForExport       = null;
            String              str_UIType                      = null;
            String              str_NamedHierarchyID            = null;
            String              str_NameList                    = null;
            String              str_FlagUseAllColumns           = null;
            String              str_MultiSelectDelimiter        = null;
            String              str_ExcelFileName               = null;
            String              str_ExcelCompleteFilePath       = null;
            String              str_StatusCompleteFilePath      = null;
            String              str_MSOfficeProcessorNetworkPath = null;
            String              str_NamedHierarchyFilePath      = null;
            String[]            arrstr_ValuesToExport           = null;
            StringBuilder       strbldr_XPathQuery              = null;
            StringBuilder       strbldr_NameList                = null;
            Boolean             bool_IsCanceled                 = false;
            XML                 xml_FacetSeriesConfig           = null;
            XML                 xml_FacetSeriesListData         = null;
            XML                 xml_ExcelGenerationStatusLog    = null;
            XML                 xml_ExcelGenerationProcessedLog = null;
            ERRORXML            errorxml_Output                 = null;

            #endregion

            FileCollectionXML = null;
            try
            {
                #region STEP00: Mandatory checks

                errorcommon_Object = new ALErrorCommon();
                if (!(File.Exists(FilePathForFacetSeriesConfig)))
                {
                    errorcommon_Object.AppendError("ERR_FS_CONFIGFILEPATHNOTFOUND", this.GetType().ToString(),
                            "GenerateExcelForFacetSeries", null, errorxml_Output);
                    return (errorcommon_Object.ErrorCollectionXML);
                }
                else if (!(File.Exists(FilePathForFacetSeriesResultantData)))
                {
                    errorcommon_Object.AppendError("ERR_FS_DATAFILEPATHNOTFOUND", this.GetType().ToString(),
                            "GenerateExcelForFacetSeries", null, errorxml_Output);
                    return (errorcommon_Object.ErrorCollectionXML);
                }

                #endregion

                #region STEP01: Retrieve mandatory configuration values

                //Read details stored into the file at shared location.
                xml_FacetSeriesConfig = File.ReadAllText(FilePathForFacetSeriesConfig);

                //Retrieve the necessary details from the config XML.
                xml_FacetSeriesListData     = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FACETSERIESLISTDATAXML>", "</FACETSERIESLISTDATAXML>");
                str_ExcelCompleteFilePath   = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<EXCELCOMPLETEFILEPATH>", "</EXCELCOMPLETEFILEPATH>");
                str_ExcelFileName           = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<EXCELFILENAME>", "</EXCELFILENAME>");
                str_StatusCompleteFilePath  = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<STATUSCOMPLETEFILEPATH>", "</STATUSCOMPLETEFILEPATH>");
                str_FlagUseAllColumns       = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<FLAGUSEALLCOLUMNS>", "</FLAGUSEALLCOLUMNS>");
                str_MultiSelectDelimiter    = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<MULTISELECETDELIMITER>", "</MULTISELECETDELIMITER>");
                str_MSOfficeProcessorNetworkPath = ALCommon.GetInnerValue(ref xml_FacetSeriesConfig, "<MSOFFICEPROCESSORNETWORKPATH>", "</MSOFFICEPROCESSORNETWORKPATH>");

                #endregion

                #region STEP02: Retrieve the required parameters and attributes from the list data XML

                //Here the ListDataXml is used only for the refering the List configuration.
                //Actual data is refered from Search Result xml document.
                xpathdoc_ListData = new XPathDocument(new StringReader(xml_FacetSeriesListData));
                xpathnav_ListData = xpathdoc_ListData.CreateNavigator();
                xpathnav_ListData = xpathnav_ListData.SelectSingleNode("List");
                if (xpathnav_ListData != null)
                {
                    str_TargetObjectName    = xpathnav_ListData.GetAttribute("TARGETOBJECTNAME", "");
                    str_PrimaryKey          = xpathnav_ListData.GetAttribute("TARGETOBJECTPRIMARYKEY", "");
                }

                //if target object or primary key is empty then return status as FAIL.
                if (String.IsNullOrEmpty(str_TargetObjectName) || String.IsNullOrEmpty(str_PrimaryKey))
                {
                    xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>FAIL</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                    lock (typeof(MSOffice_MediaPlugin))
                    {
                        File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationProcessedLog);
                    }
                    return (null);
                }

                if (String.Compare(str_FlagUseAllColumns, "TRUE", true) == 0)
                    xpathiter_HeaderColumn = xpathnav_ListData.Select("Header/Column [@NavigationPath='" + str_TargetObjectName + "']");
                else
                {
                    xpathiter_HeaderColumn = xpathnav_ListData.Select(
                            "Header/Column [(translate(@DisplayState, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')='YES' or @Name='"
                            + str_PrimaryKey + "') and @NavigationPath='" + str_TargetObjectName + "']");
                }
                xpathiter_CopyOfHeaderColumn = xpathiter_HeaderColumn.Clone();

                #endregion

                #region STEP03: Generate the xpath for target nodes from resultant data

                strbldr_XPathQuery = new StringBuilder(16);
                strbldr_XPathQuery = strbldr_XPathQuery.Append("*/*/" + str_TargetObjectName);
                str_SelectedRowsForExport = xpathnav_ListData.GetAttribute("SelectedRowsForExport", "");
                if (!String.IsNullOrEmpty(str_SelectedRowsForExport))
                {
                    arrstr_ValuesToExport = str_SelectedRowsForExport.Split(',');
                    if (arrstr_ValuesToExport != null && arrstr_ValuesToExport.Length > 0)
                    {
                        for (int RowIndex = 0; RowIndex < arrstr_ValuesToExport.Length; RowIndex++)
                        {
                            if (RowIndex == 0)
                                strbldr_XPathQuery.AppendFormat("[" + str_PrimaryKey + "/text()='{0}'", arrstr_ValuesToExport[RowIndex]);
                            else
                                strbldr_XPathQuery.AppendFormat(" or " + str_PrimaryKey +"/text()='{0}'", arrstr_ValuesToExport[RowIndex]);
                        }
                        strbldr_XPathQuery.Append("]");
                    }
                }

                #endregion

                #region STEP04: Process data and generate the Excel

                xpathdoc_ResultantData      = new XPathDocument(FilePathForFacetSeriesResultantData);
                xpathnav_ResultantData      = xpathdoc_ResultantData.CreateNavigator();
                xpathexpr_ResultantRecord   = XPathExpression.Compile(strbldr_XPathQuery.ToString());
                xpathiter_ResultantRecord   = xpathnav_ResultantData.Select(xpathexpr_ResultantRecord);
                if (xpathiter_HeaderColumn != null)
                {
                    excelapplication_Object = new Microsoft.Office.Interop.Excel.ApplicationClass();
                    excelworkbook_Object = (WorkbookClass)excelapplication_Object.Workbooks.Add(ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls");

                    #region STEP05: Prepare look-up sheet using named-hierarchy values

                    excelworksheet_LookUpObject = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                    excelworksheet_LookUpObject.Name = "LookUp Sheet";

                    strbldr_NameList = new StringBuilder();
                    while (xpathiter_CopyOfHeaderColumn.MoveNext())
                    {
                        str_UIType = xpathiter_CopyOfHeaderColumn.Current.GetAttribute("UIType", "");
                        str_NamedHierarchyID = xpathiter_CopyOfHeaderColumn.Current.GetAttribute("NAMEDHIERARCHYID", "");
                        if (!(String.IsNullOrEmpty(str_NamedHierarchyID)))
                        {
                            int_LookUpRowIndex = 2;
                            str_NamedHierarchyFilePath = str_MSOfficeProcessorNetworkPath + @"\" + str_NamedHierarchyID + "_NHHierarchy.xml";
                            if (File.Exists(str_NamedHierarchyFilePath))
                            {
                                str_ColumnName = xpathiter_CopyOfHeaderColumn.Current.GetAttribute("FIELDNAME", "");
                                xpathdoc_NamedHierarchy = new XPathDocument(str_NamedHierarchyFilePath);
                                xpathnav_NamedHierarchy = xpathdoc_NamedHierarchy.CreateNavigator();
                                xpathnav_NamedHierarchy = xpathnav_NamedHierarchy.SelectSingleNode("ROOT/NAMEDHIERARCHYVALUECOLLECTION");
                                if (xpathnav_NamedHierarchy != null)
                                {
                                    xpathiter_NamedHierarchy = xpathnav_NamedHierarchy.Select("NAMEDHIERARCHYVALUE/NAME");
                                    if (xpathiter_NamedHierarchy.Count > 0)
                                    {
                                        excelrange_LookUpCellObject = excelworksheet_LookUpObject.Cells[1, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                        excelrange_LookUpCellObject.Value2 = str_ColumnName;

                                        while (xpathiter_NamedHierarchy.MoveNext())
                                        {
                                            if (xpathiter_NamedHierarchy.CurrentPosition == 1)
                                                excelrange_StartCellObject = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;

                                            excelrange_LookUpCellObject = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                            excelrange_LookUpCellObject.Value2 = xpathiter_NamedHierarchy.Current.Value;
                                            int_LookUpRowIndex++;
                                        }

                                        excelworkbook_Object.Names.Add(str_ColumnName, 
                                                excelworksheet_LookUpObject.get_Range(excelrange_StartCellObject, excelrange_LookUpCellObject), 
                                                true, 3, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                                        if (str_UIType.Equals("DROPDOWNLIST", StringComparison.OrdinalIgnoreCase))
                                            strbldr_NameList.AppendFormat(" $;{0};$ ", str_ColumnName);

                                        int_LookUpColumnIndex++;
                                    }
                                }
                            }
                        }
                    }
                    str_NameList = strbldr_NameList.ToString();

                    excelworksheet_Object = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                    excelworksheet_Object.Name = str_TargetObjectName;
                    while (xpathiter_HeaderColumn.MoveNext())
                    {
                        str_ColumnName                  = xpathiter_HeaderColumn.Current.GetAttribute("FIELDNAME", "");
                        str_DisplayValue                = xpathiter_HeaderColumn.Current.GetAttribute("DisplayValue", "");
                        excelrange_CellObject           = excelworksheet_Object.Cells[1, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                        excelrange_CellObject.Value2    = str_DisplayValue;
                        excelrange_CellObject.AddComment(str_ColumnName);

                        if (str_NameList.Contains(" $;" + str_ColumnName + ";$ "))
                        {
                            excelrange_StartCellObject      = excelworksheet_Object.Cells[2, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_EndCellObject        = excelworksheet_Object.Cells[10001, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_DataValidationObject = excelworksheet_Object.get_Range(excelrange_StartCellObject, excelrange_EndCellObject);
                            excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateList, XlDVAlertStyle.xlValidAlertStop, 
                                    XlFormatConditionOperator.xlEqual, "=" + str_ColumnName, Type.Missing);
                        }
                        int_ColumnIterator++;
                    }

                    #endregion

                    //Add total record count into the progress status XML.
                    xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><TOTALNUMBEROFRECORDS>"
                            + xpathiter_ResultantRecord.Count.ToString() 
                            + "</TOTALNUMBEROFRECORDS><EXPORTSTATUS>$AL$StatusOfExport$AL$</EXPORTSTATUS>"
                            + "<RECORDSPROCESSED>$AL$ProcessedRecords$AL$</RECORDSPROCESSED></PROGRESSSTATUSUSEDBYTHREAD>";

                    #region STEP06: Interate the resultant records and generate Excel (rows and columns)

                    if (String.Compare(str_FlagUseAllColumns, "TRUE", true) == 0)
                        xpathiter_HeaderColumn = xpathnav_ListData.Select("Header/Column");
                    else
                    {
                        xpathiter_HeaderColumn = xpathnav_ListData.Select("Header/Column [translate(@DisplayState, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')='YES' or @Name='"
                                + str_PrimaryKey + "']");
                    }
                    if (xpathiter_ResultantRecord != null)
                    {
                        ////while (xpathiter_ResultantRecord.MoveNext() && (!ExcelThreadParametersObj.Stop))
                        while (xpathiter_ResultantRecord.MoveNext())
                        {
                            xpathiter_HeaderColumn = xpathnav_ListData.Select("Header/Column [translate(@DisplayState, 'abcdefghijklmnopqrstuvwxyz', 'ABCDEFGHIJKLMNOPQRSTUVWXYZ')='YES' or @Name='" + str_PrimaryKey + "']");
                            int_ColumnIterator = 1;
                            while (xpathiter_HeaderColumn.MoveNext())
                            {
                                str_ColumnName              = xpathiter_HeaderColumn.Current.GetAttribute("Name", "");
                                excelrange_CellObject       = excelworksheet_Object.Cells[int_RowIterator, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                                excelrange_HeaderCellObject = excelworksheet_Object.Cells[1, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                                xpathnav_Column             = xpathiter_ResultantRecord.Current.SelectSingleNode(str_ColumnName);

                                if (str_ColumnName.Equals(str_PrimaryKey, StringComparison.OrdinalIgnoreCase) && xpathnav_Column.Value.Contains("-"))
                                    excelrange_CellObject.Value2 = "";
                                else
                                {
                                    excelrange_CellObject.Value2 = formatCellData(xpathiter_HeaderColumn.Current.GetAttribute("UIType", ""),
                                            str_MultiSelectDelimiter, xpathnav_Column);
                                }
                                int_ColumnIterator++;
                            }
                            int_RowIterator++;

                            //Check whether the Excel generation is cancelled from solution. 
                            //Event generated at solution level which updates the status using web service. 
                            bool_IsCanceled = checkIsExcelGenerationCancelledEvent(str_StatusCompleteFilePath);
                            if (bool_IsCanceled)
                                break;

                            //Add total processed records into the progress status XML.
                            xml_ExcelGenerationProcessedLog = xml_ExcelGenerationStatusLog.Replace("$AL$StatusOfExport$AL$", "NEW");
                            xml_ExcelGenerationProcessedLog = xml_ExcelGenerationProcessedLog.Replace("$AL$ProcessedRecords$AL$", 
                                    Convert.ToString(int_RowIterator - 2));
                            lock (typeof(MSOffice_MediaPlugin))
                            {
                                File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationProcessedLog);
                            }
                        }
                    }

                    #endregion

                    #region STEP07: Save Excel

                    //Check whether the Excel generation is cancelled from solution. 
                    //Event generated at solution level which updates the status using web service. 
                    bool_IsCanceled = checkIsExcelGenerationCancelledEvent(str_StatusCompleteFilePath);
                    if (!(bool_IsCanceled))
                    {
                        excelworkbook_Object.SaveAs(str_ExcelCompleteFilePath, Microsoft.Office.Interop.Excel.XlFileFormat.xlExcel8, Type.Missing,
                                Type.Missing, Type.Missing, Type.Missing, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange,
                                Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                        xml_ExcelGenerationProcessedLog = xml_ExcelGenerationStatusLog.Replace("$AL$StatusOfExport$AL$", "SUCCESS");
                        xml_ExcelGenerationProcessedLog = xml_ExcelGenerationProcessedLog.Replace("$AL$ProcessedRecords$AL$",
                                Convert.ToString(int_RowIterator - 2));
                        lock (typeof(MSOffice_MediaPlugin))
                        {
                            File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationProcessedLog);
                        }

                        //Prepare file collection details for Excel view.
                        FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_ExcelCompleteFilePath + "</PATH><NAME>"
                                + str_ExcelFileName + "</NAME></FILE></FILECOLLECTION>";
                    }

                    #endregion
                }

                #endregion
            }
            catch (Exception ExceptionObject)
            {
                errorxml_Output = ExceptionObject.Message;
                if (!(String.IsNullOrEmpty(str_StatusCompleteFilePath)))
                {
                    xml_ExcelGenerationStatusLog = "<PROGRESSSTATUSUSEDBYTHREAD><EXPORTSTATUS>FAIL</EXPORTSTATUS></PROGRESSSTATUSUSEDBYTHREAD>";
                    lock (typeof(MSOffice_MediaPlugin))
                    {
                        File.WriteAllText(str_StatusCompleteFilePath, xml_ExcelGenerationStatusLog);
                    }
                }
            }
            finally
            {
                #region STEP08: Cleanups- Close the Excel objects

                if (excelrange_CellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_CellObject);
                    excelrange_CellObject = null;
                }

                if (excelrange_HeaderCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_HeaderCellObject);
                    excelrange_HeaderCellObject = null;
                }

                if (excelrange_LookUpCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_LookUpCellObject);
                    excelrange_LookUpCellObject = null;
                }

                if (excelrange_StartCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_StartCellObject);
                    excelrange_StartCellObject = null;
                }

                if (excelrange_EndCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_EndCellObject);
                    excelrange_EndCellObject = null;
                }

                if (excelrange_DataValidationObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_DataValidationObject);
                    excelrange_DataValidationObject = null;
                }

                if (excelworksheet_Object != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_Object);
                    excelworksheet_Object = null;
                }

                if (excelworksheet_LookUpObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_LookUpObject);
                    excelworksheet_LookUpObject = null;
                }

                if (excelworkbook_Object != null)
                {
                    excelworkbook_Object.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworkbook_Object);
                    excelworkbook_Object = null;
                }

                if (excelapplication_Object != null)
                {
                    excelapplication_Object.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelapplication_Object);
                    excelapplication_Object = null;
                }

                #endregion
            }

            return (errorxml_Output);
        }
       
        /// <summary>
        /// <CreatedBy>Meenal Joshi</CreatedBy>
        /// <CreatedDate>6 June 2011</CreatedDate>
        /// <Purpose>This method is used to generate the Excel template for Entity Import using the configuration provided.</Purpose>
        /// </summary>
        /// <param name="FilePathForEntityConfig">This contains the list data XML with configuration which is used for Excel generation.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        public ERRORXML GenerateImportExcelTemplateForEntity(String FilePathForEntityConfig, out XML FileCollectionXML)
        {
            #region Method Variables

            Microsoft.Office.Interop.Excel.ApplicationClass excelapplication_Object         = null;
            Microsoft.Office.Interop.Excel.WorkbookClass    excelworkbook_Object            = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_Object           = null;
            Microsoft.Office.Interop.Excel.Worksheet        excelworksheet_LookUpObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_CellObject           = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_LookUpCellObject     = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_StartCellObject      = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_EndCellObject        = null;
            Microsoft.Office.Interop.Excel.Range            excelrange_DataValidationObject = null;

            ALErrorCommon       errorcommon_Object          = null;
            Int32               int_LookUpColumnIndex       = 1;                        
            String              str_NameList                = null;
            String              str_ExcelCompleteFilePath   = null;
            String              str_ExcelFileName           = null;            
            String              str_MSOfficeProcessorNetworkPath = null;            
            StringBuilder       strbldr_NameList            = null;
            XML                 xml_EntityConfig            = null;            
            ERRORXML            errorxml_Output             = null;
            XPathDocument       xp_EntityConfigXml          = null;
            XPathNavigator      xpNav_EntityConfigRoot      = null;
            XPathNodeIterator   xpItr_FieldList             = null;
            XPathNodeIterator   xpItr_RelatedFacetList      = null;
            XPathNodeIterator   xpItr_RelatedEntityList     = null;
            XPathNodeIterator   xpItr_EntityNHFieldList     = null;
            XPathNodeIterator   xpItr_RelEntityNHFieldList  = null;
            XPathNodeIterator   xpItr_RelFacetNHFieldList   = null;
            String              str_TargetObjectName        = null;
            XPathNodeIterator   XpItr_OmittedRelations      = null;
            String              str_IsImportWithoutIDs      = null;
            XPathNavigator      xpNav_EntityNode            = null;
            #endregion

            FileCollectionXML = null;
            try
            {
                #region STEP00: Mandatory checks

                errorcommon_Object = new ALErrorCommon();
                if (!(File.Exists(FilePathForEntityConfig)))
                {
                    errorcommon_Object.AppendError("ERR_FS_CONFIGFILEPATHNOTFOUND", this.GetType().ToString(),
                            "GenerateImportExcelTemplateForEntity", null, errorxml_Output);
                    return (errorcommon_Object.ErrorCollectionXML);
                }

                #endregion

                #region STEP01: Retrieve mandatory configuration values

                //Read details stored into the file at shared location.
                xml_EntityConfig                    = File.ReadAllText(FilePathForEntityConfig);

                //Retrieve the necessary details from the config XML.                
                str_ExcelCompleteFilePath           = ALCommon.GetInnerValue(ref xml_EntityConfig, "<EXCELCOMPLETEFILEPATH>", "</EXCELCOMPLETEFILEPATH>");
                str_ExcelFileName                   = ALCommon.GetInnerValue(ref xml_EntityConfig, "<EXCELFILENAME>", "</EXCELFILENAME>");                
                str_MSOfficeProcessorNetworkPath    = ALCommon.GetInnerValue(ref xml_EntityConfig, "<MSOFFICEPROCESSORNETWORKPATH>", "</MSOFFICEPROCESSORNETWORKPATH>");
                str_TargetObjectName                = ALCommon.GetInnerValue(ref xml_EntityConfig, "<ENTITYNAME>", "</ENTITYNAME>");
                #endregion

                #region STEP02: Retrieve the required parameters and attributes from the form configuration XML
                /*
                xpathdoc_FormConfig     = new XPathDocument(new StringReader(xml_EntitySchemaConfig));
                xpathnav_FormConfigRoot = xpathdoc_FormConfig.CreateNavigator();
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode("Form");
                str_XPath = "Section [@Order='" + str_SectionOrder+ "'] / Column [@Order='" + str_ColumnOrder 
                        + "'] / Field [@Order='" + str_FieldOrder + "']";
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode(str_XPath);

                bool_FlagUseAllColumns  = Boolean.Parse(xpathnav_FormConfigRoot.GetAttribute("UseAllColumnsForImportExport", ""));
                xpathnav_FormConfigRoot = xpathnav_FormConfigRoot.SelectSingleNode("Configuration/DisplayLayout/List");
                str_TargetObjectName    = xpathnav_FormConfigRoot.GetAttribute("TARGETOBJECTNAME", "");
                str_PrimaryKey          = xpathnav_FormConfigRoot.GetAttribute("TARGETOBJECTPRIMARYKEY", "");

                if (bool_FlagUseAllColumns)
                    xpathiter_TargetColumns = xpathnav_FormConfigRoot.Select("DisplayColumns/TargetObject/Column");
                else
                    xpathiter_TargetColumns = xpathnav_FormConfigRoot.Select("DisplayColumns/TargetObject/Column [translate(@DisplayState,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'YES']");

                xpathiter_ColumnsCloneCopy = xpathiter_TargetColumns.Clone();
                */
                #endregion
                
                #region STEP02
                xp_EntityConfigXml      = new XPathDocument(new StringReader(xml_EntityConfig));
                xpNav_EntityConfigRoot  = xp_EntityConfigXml.CreateNavigator();

                xpItr_RelatedEntityList = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/SCHEMAXML/TYPEINFOCOLLECTION/TYPE[OBJECTTYPE='ENTITY']/FIELDCOLLECTION/FIELD");
                xpItr_RelatedFacetList  = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/SCHEMAXML/TYPEINFOCOLLECTION/TYPE[OBJECTTYPE='FACET']/FIELDCOLLECTION/FIELD");
                xpItr_FieldList         = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/SCHEMAXML/ENTITYINFO/TYPE/FIELDCOLLECTION/FIELD");
                xpNav_EntityNode        = xpNav_EntityConfigRoot.SelectSingleNode("ENTITYCONFIGFOREXCEL/IMPORTDETAILS");
                if (xpNav_EntityNode != null)
                    str_IsImportWithoutIDs = xpNav_EntityNode.GetAttribute("IsImportWithoutIDs", "");
                #endregion

                #region STEP03: Process data and generate the Excel

                excelapplication_Object             = new Microsoft.Office.Interop.Excel.ApplicationClass();
                excelworkbook_Object                = (WorkbookClass)excelapplication_Object.Workbooks.Add(ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls");
                excelworkbook_Object.Activate();
                excelworksheet_LookUpObject         = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                excelworksheet_LookUpObject.Name    = "LookUp Sheet";

                #region STEP04: Prepare look-up sheet using named-hierarchy values
                xpItr_EntityNHFieldList             = xpItr_FieldList.Clone();
                xpItr_RelEntityNHFieldList          = xpItr_RelatedEntityList.Clone();
                xpItr_RelFacetNHFieldList           = xpItr_RelatedFacetList.Clone();
                strbldr_NameList                    = new StringBuilder();
                int_LookUpColumnIndex               = 1;

                //Fill Entity Lookup Columns data
                FillUpLookUpCloumnData(xpItr_EntityNHFieldList, str_MSOfficeProcessorNetworkPath, ref int_LookUpColumnIndex, ref strbldr_NameList, ref excelworksheet_LookUpObject, ref excelworkbook_Object);
                
                //NO NEED TO FILL UP RELATED ENTITY NH LOOKUPS
                //Fill related Entity Lookup Columns data                
                //-FillUpLookUpCloumnData(xpItr_RelEntityNHFieldList, str_MSOfficeProcessorNetworkPath, ref int_LookUpColumnIndex, ref strbldr_NameList, ref excelworksheet_LookUpObject, ref excelworkbook_Object);
                
                //Fill related Facet Lookup Columns data
                FillUpLookUpCloumnData(xpItr_RelFacetNHFieldList, str_MSOfficeProcessorNetworkPath, ref int_LookUpColumnIndex, ref strbldr_NameList, ref excelworksheet_LookUpObject, ref excelworkbook_Object);

                str_NameList        = strbldr_NameList.ToString();
                #endregion

                #region STEP05: Add header columns              
                XpItr_OmittedRelations      = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/IMPORTDETAILS/OMITTEDRELATIONS/RELATIONSHIP");

                //Add sheet for Related Entities
                //Related Entity sheets are added only when str_IsImportWithoutIDs is not true
                if(String.Compare(str_IsImportWithoutIDs,"true",true) != 0)
                {
                    str_TargetObjectName                = null;
                    xpItr_RelatedEntityList             = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/SCHEMAXML/TYPEINFOCOLLECTION/TYPE[OBJECTTYPE='ENTITY']");
                    foreach (XPathNavigator xpNav_RelEntity in xpItr_RelatedEntityList)
                    {
                        XPathNodeIterator xpItr_EFields = xpNav_RelEntity.Select("FIELDCOLLECTION/FIELD[ISPRIMARYKEY='TRUE']");
                        str_TargetObjectName            = xpNav_RelEntity.GetAttribute("RelationshipName", "");

                        if(IsAddSheetForRelation(XpItr_OmittedRelations,str_TargetObjectName))
                            CreateWorkSheet(ref excelworkbook_Object, "RELATEDENTITY", xpNav_EntityConfigRoot, xpItr_EFields, str_TargetObjectName, str_NameList);
                    }
                }

                //Add sheet for Related Facet
                str_TargetObjectName                = null;
                xpItr_RelatedFacetList              = xpNav_EntityConfigRoot.Select("ENTITYCONFIGFOREXCEL/SCHEMAXML/TYPEINFOCOLLECTION/TYPE[OBJECTTYPE='FACET']");
                foreach (XPathNavigator xpNav_RelFacet in xpItr_RelatedFacetList)
                {
                    XPathNodeIterator xpItr_FFields = xpNav_RelFacet.Select("FIELDCOLLECTION/FIELD");
                    str_TargetObjectName            = xpNav_RelFacet.GetAttribute("RelationshipName", "");

                    if (IsAddSheetForRelation(XpItr_OmittedRelations, str_TargetObjectName))
                        CreateWorkSheet(ref excelworkbook_Object, "RELATEDFACET", xpNav_EntityConfigRoot, xpItr_FFields, str_TargetObjectName, str_NameList);
                }

                //Add sheet for entity
                str_TargetObjectName                = ALCommon.GetInnerValue(ref xml_EntityConfig, "<ENTITYNAME>", "</ENTITYNAME>");
                CreateWorkSheet(ref excelworkbook_Object, "TARGETENTITY", xpNav_EntityConfigRoot, xpItr_FieldList, str_TargetObjectName, str_NameList);

                RemoveExtraSheets(ref excelworkbook_Object);               
                #endregion

                #endregion

                #region STEP06: Save Excel

                if (File.Exists(str_ExcelCompleteFilePath))
                    File.Delete(str_ExcelCompleteFilePath);

                try
                {
                    excelworkbook_Object.SaveAs(str_ExcelCompleteFilePath, Microsoft.Office.Interop.Excel.XlFileFormat.xlExcel8,Type.Missing,
                            Type.Missing, Type.Missing, Type.Missing, Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange,
                            Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                }
                catch (Exception ObjExcep)
                {
                    errorxml_Output = ObjExcep.Message;
                    //File.WriteAllText("c:\\Error.txt", ObjExcep.Message);
                    //File.AppendAllText("c:\\Error.txt", ObjExcep.StackTrace); ;
                }

                //Prepare file collection details for Excel view.
                FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_ExcelCompleteFilePath + "</PATH><NAME>"
                        + str_ExcelFileName + "</NAME></FILE></FILECOLLECTION>";

                #endregion
            }
            catch (Exception ExceptionObject)
            {
                errorxml_Output = ExceptionObject.Message;
                //File.AppendAllText("c:\\Error.txt", ExceptionObject.StackTrace); ;
            }
            finally
            {
                #region STEP07: Cleanups- Close the Excel objects

                if (excelrange_CellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_CellObject);
                    excelrange_CellObject = null;
                }

                if (excelrange_LookUpCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_LookUpCellObject);
                    excelrange_LookUpCellObject = null;
                }

                if (excelrange_StartCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_StartCellObject);
                    excelrange_StartCellObject = null;
                }

                if (excelrange_EndCellObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_EndCellObject);
                    excelrange_EndCellObject = null;
                }

                if (excelrange_DataValidationObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelrange_DataValidationObject);
                    excelrange_DataValidationObject = null;
                }

                if (excelworksheet_Object != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_Object);
                    excelworksheet_Object = null;
                }

                if (excelworksheet_LookUpObject != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworksheet_LookUpObject);
                    excelworksheet_LookUpObject = null;
                }

                if (excelworkbook_Object != null)
                {
                    excelworkbook_Object.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworkbook_Object);
                    excelworkbook_Object = null;
                }

                if (excelapplication_Object != null)
                {
                    excelapplication_Object.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelapplication_Object);
                    excelapplication_Object = null;
                }

                #endregion            
            }

            return (errorxml_Output);
        }

        public ERRORXML ExportSubmissionDataToExcel(XML ExportSubmissionXML)
        {
            ERRORXML ErrorXML = null;
            XPathDocument xpSubmissionDocObj = null;
            XPathNavigator xpSubmissionNav = null;
            XPathNodeIterator xpVehicleIter = null;
            String OfficeTemplateFilePath = null;
            Microsoft.Office.Interop.Excel.ApplicationClass ExcelAppObj = null;
            WorkbookClass ExcelWorkbookObj = null;
            Worksheet[] ExcelWorksheetObj = null;
            Worksheet LookUpExcelWorksheetObj = null;
            Microsoft.Office.Interop.Excel.Range CellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range LookUpCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range EndDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range[] DataValidationRangeObj = null;
            XPathNodeIterator xpSpaceIter = null;
            XPathNodeIterator xpHeaderColumnIter = null;
            XPathNavigator xpColumnNav = null;
            XPathExpression xpExp = null;
            XPathExpression xpHeaderExp = null;
            int SpaceLevel = 0;
            int CurrentSpaceLevel = 0;
            StringBuilder sbxPath = null;            
            String xpath = null;
            String SheetName = null;
            const int DisplayHeaderRowNo = 3;
            const int InternalHeaderRowNo = 4;
            int RowPosition = 5;
            String LookupSheetName = null;
            int LookUpRowIndex = 2;
            String LookupName = null;
            XPathDocument xpLookupDocObj = null;
            XPathNavigator xpLookupNav = null;
            XPathNodeIterator xpLookupIter = null;
            XPathNodeIterator xpLookupValueIter = null;
            StringBuilder sbNameList = null;
            String NameList = null;
            Boolean ApplyDataValidation = false;
            String ValidationErrorMessage = null;
            String SubmissionDetailsXML = null;
            String LookupXML = null;
            String ExcelFilePath = null;
            String StatusXML = null;
            String FieldType = null;
            ALErrorCommon ObjErrorCommon = null;
            XPathDocument xpInputDoc = null;
            XPathNavigator xpInputNav = null;
            String Formula = null;
            String InnerFormula = null;
            String DataType = null;
            XmlDocument StatusDocObj = null;
            XmlNode NodeObj = null;
            Boolean LengthValidation = false;
            String MaxLength = null;
            Boolean RangeValidation = false;
            String MinValue = null;
            String MaxValue = null;
            String ColumnName = null;
            String MultiSelectSeparator = null;
            String FacetName = null;
            String RelationshipName = null;
            try
            {
                xpInputDoc = new XPathDocument(new StringReader(ExportSubmissionXML));
                xpInputNav = xpInputDoc.CreateNavigator().SelectSingleNode("EXPORTSUBMISSION/DATAXMLPATH");
                if (xpInputNav != null)
                    SubmissionDetailsXML = xpInputNav.Value;

                xpInputNav = xpInputDoc.CreateNavigator().SelectSingleNode("EXPORTSUBMISSION/LOOKUPXMLPATH");
                if (xpInputNav != null)
                    LookupXML = xpInputNav.Value;

                xpInputNav = xpInputDoc.CreateNavigator().SelectSingleNode("EXPORTSUBMISSION/EXCELFILEPATH");
                if (xpInputNav != null)
                    ExcelFilePath = xpInputNav.Value;

                xpInputNav = xpInputDoc.CreateNavigator().SelectSingleNode("EXPORTSUBMISSION/STATUSFILEPATH");
                if (xpInputNav != null)
                    StatusXML = xpInputNav.Value;

                xpInputNav = xpInputDoc.CreateNavigator().SelectSingleNode("EXPORTSUBMISSION/MULTISELECTSEPARATOR");
                if (xpInputNav != null)
                    MultiSelectSeparator = xpInputNav.Value;

                sbNameList = new StringBuilder(30);
                xpLookupDocObj = new XPathDocument(LookupXML);
                xpLookupNav = xpLookupDocObj.CreateNavigator();
                xpLookupNav = xpLookupNav.SelectSingleNode("LOOKUPXML");
                xpLookupIter = xpLookupNav.Select("*");

                xpSubmissionDocObj = new XPathDocument(SubmissionDetailsXML);
                xpSubmissionNav = xpSubmissionDocObj.CreateNavigator();
                xpSubmissionNav = xpSubmissionNav.SelectSingleNode("EXTRAINFORMATION/SUBMISSION");
                xpVehicleIter = xpSubmissionNav.Select("VEHICLECOLLECTION/VEHICLE");                

                ExcelAppObj = new Microsoft.Office.Interop.Excel.ApplicationClass();
                OfficeTemplateFilePath = ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls";
                ExcelWorkbookObj = ExcelAppObj.Workbooks.Add(OfficeTemplateFilePath) as WorkbookClass;

                //if (!IsExportSubmissionCancelled(StatusXML))
                {
                    #region Prepare Lookup Sheet
                    LookupSheetName = "Lookup_" + xpSubmissionNav.SelectSingleNode("SUBMISSIONID").Value;
                    LookUpExcelWorksheetObj = ExcelWorkbookObj.Sheets.Add(Type.Missing,
                                                           Type.Missing, Type.Missing, Type.Missing) as Worksheet;

                    LookUpExcelWorksheetObj.Columns.ColumnWidth = 30;
                    LookUpExcelWorksheetObj.Columns.WrapText = true;
                    LookUpExcelWorksheetObj.Name = LookupSheetName;
                    LookUpExcelWorksheetObj.Visible = XlSheetVisibility.xlSheetHidden;

                    while (xpLookupIter.MoveNext())
                    {
                        LookUpRowIndex = 2;
                        LookupName = xpLookupIter.Current.Name;
                        LookUpCellRangeObj = LookUpExcelWorksheetObj.Cells[1, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                        LookUpCellRangeObj.Value2 = LookupName;

                        xpLookupValueIter = xpLookupIter.Current.Select("VALUE");
                        while (xpLookupValueIter.MoveNext())
                        {
                            if (xpLookupValueIter.CurrentPosition == 1)
                                StartCellRangeObj = LookUpExcelWorksheetObj.Cells[LookUpRowIndex, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;

                            LookUpCellRangeObj = LookUpExcelWorksheetObj.Cells[LookUpRowIndex, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                            LookUpCellRangeObj.Value2 = xpLookupValueIter.Current.Value;

                            LookUpRowIndex++;
                        }

                        ExcelWorkbookObj.Names.Add(LookupName, LookUpExcelWorksheetObj.get_Range(StartCellRangeObj, LookUpCellRangeObj),
                                                                true, 3, Type.Missing, Type.Missing, Type.Missing,
                                                                Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                        sbNameList.AppendFormat(" $;{0};$ ", LookupName);
                    }
                    NameList = sbNameList.ToString();
                    #endregion

                    if (xpVehicleIter.Count > 0)
                    {
                        ExcelWorksheetObj = new Worksheet[xpVehicleIter.Count];
                        DataValidationRangeObj = new Microsoft.Office.Interop.Excel.Range[xpVehicleIter.Count];
                    }
                    sbxPath = new StringBuilder();
                    while (xpVehicleIter.MoveNext())
                    {
                        //if (!IsExportSubmissionCancelled(StatusXML))
                        {
                            #region Prepare Vehicle Sheet
                            RowPosition = 5;
                            //SheetName = xpVehicleIter.Current.SelectSingleNode("NAME").Value.Replace(" ", "_") + "_" + xpVehicleIter.Current.SelectSingleNode("VEHICLEID").Value;
                            SheetName = "Vehicle_" + xpVehicleIter.Current.SelectSingleNode("VEHICLEID").Value;
                            ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1] = ExcelWorkbookObj.Sheets.Add(Type.Missing,
                                                                Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                            ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Name = SheetName;
                            ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Columns.WrapText = true;
                            ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells.VerticalAlignment = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignTop;

                            //Add Note in first row
                            CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].get_Range(ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[1, 1], ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[1, 4]);
                            CellRangeObj.Merge(true);
                            CellRangeObj.Value2 = "IMPORTANT: Please do not delete or modify the columns in this excel sheet.\n";
                            CellRangeObj.Font.Color = System.Drawing.ColorTranslator.ToOle(System.Drawing.Color.Red);
                            CellRangeObj.Font.Bold = true;
                            CellRangeObj.Cells.RowHeight = 30;

                            //Add Format details in second row
                            CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[2, 1] as Microsoft.Office.Interop.Excel.Range;
                            CellRangeObj.Value2 = "Vehicle Name: \nStart Date: \nEnd Date: ";
                            CellRangeObj.Font.Bold = true;

                            CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].get_Range(ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[2, 2], ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[2, 3]);
                            CellRangeObj.Merge(true);
                            CellRangeObj.Value2 = xpVehicleIter.Current.SelectSingleNode("NAME").Value + "\n" +
                                                xpVehicleIter.Current.SelectSingleNode("INMKTSTARTDATE").Value + "\n" +
                                                xpVehicleIter.Current.SelectSingleNode("INMKTENDDATE").Value;

                            CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].get_Range(ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[2, 1], ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[2, 3]);
                            CellRangeObj.EntireColumn.AutoFit();

                            // Hide Row containing TE and GE elements for internal use
                            CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].get_Range(ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[InternalHeaderRowNo, 1], ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[InternalHeaderRowNo, 2]);
                            CellRangeObj.EntireRow.Hidden = true;

                            xpHeaderExp = XPathExpression.Compile("HEADERXML/headercolumn");
                            xpHeaderExp.AddSort("@order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);

                            #region Header columns
                            xpHeaderColumnIter = xpVehicleIter.Current.Select(xpHeaderExp);
                            while (xpHeaderColumnIter.MoveNext())
                            {
                                ApplyDataValidation = false;
                                CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[DisplayHeaderRowNo, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                                CellRangeObj.EntireRow.Font.Bold = true;
                                CellRangeObj.Value2 = xpHeaderColumnIter.Current.GetAttribute("displaytext", "");
                                CellRangeObj.ColumnWidth = 20;
                                CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[InternalHeaderRowNo, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                                if (String.IsNullOrEmpty(xpHeaderColumnIter.Current.GetAttribute("actualname", "")))
                                    CellRangeObj.Value2 = xpHeaderColumnIter.Current.GetAttribute("name", "");
                                else
                                    CellRangeObj.Value2 = xpHeaderColumnIter.Current.GetAttribute("actualname", "");

                                StartDataCellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[RowPosition, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                                EndDataCellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[RowPosition + 1000, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                                DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1] = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].get_Range(StartDataCellRangeObj, EndDataCellRangeObj);
                                //ColumnName = String.Empty;
                                ////ColumnName = SheetName + "!" + GetStandardExcelColumnName(StartDataCellRangeObj.Column);
                                //ColumnName = GetStandardExcelColumnName(StartDataCellRangeObj.Column);
                                //ColumnName = ColumnName + StartDataCellRangeObj.Row.ToString();

                                if (!String.IsNullOrEmpty(NameList) && NameList.Contains(" $;" + xpHeaderColumnIter.Current.GetAttribute("lookupname", "") + ";$ "))
                                {
                                    FieldType = xpHeaderColumnIter.Current.GetAttribute("fieldtype", "");
                                    if (String.Compare(FieldType, "MULTISELECT", true) == 0)
                                        ApplyDataValidation = false;
                                    else
                                        ApplyDataValidation = true;

                                    DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.Add(XlDVType.xlValidateList, XlDVAlertStyle.xlValidAlertStop,
                                            XlFormatConditionOperator.xlEqual, "=" + xpHeaderColumnIter.Current.GetAttribute("lookupname", ""), Type.Missing);

                                    DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.ShowError = false;
                                    ValidationErrorMessage = "Value should be selected from the available values and should not be modified.";
                                }
                                /*else
                                {
                                    Formula = String.Empty;
                                    InnerFormula = String.Empty;
                                    if (!String.IsNullOrEmpty(ColumnName))
                                    {
                                        #region Data Type Validation
                                        
                                        DataType = xpHeaderColumnIter.Current.GetAttribute("type", "");
                                        if (String.Compare(DataType, "NUMBER", true) == 0)
                                        {
                                            ApplyDataValidation = true;
                                            Formula = String.Format("=IF(ISNUMBER({0}),1,0)", ColumnName);                                            
                                            //Formula = "=IF(ISNUMBER($COL$),IF($COL$=ROUNDUP($COL$,0),$NESTEDCONDITION$,0),0)";                                            
                                            ValidationErrorMessage = "Please enter Numeric data without floating point.";
                                        }
                                        else if (String.Compare(DataType, "FLOAT", true) == 0)
                                        {
                                            ApplyDataValidation = true;
                                            //Formula = "=IF(ISNUMBER($COL$),$NESTEDCONDITION$,0)";                                            
                                            Formula = String.Format("=IF(ISNUMBER({0}),1,0)", ColumnName);
                                            ValidationErrorMessage = "Please enter float data.";
                                        }
                                        else
                                        {
                                            Formula = "=IF(1,$NESTEDCONDITION$,0)";                                            
                                        }
                                        #endregion
                                        
                                        #region Max Length Validation
                                        try
                                        {
                                            LengthValidation = Boolean.Parse(xpHeaderColumnIter.Current.GetAttribute("lengthValidation", ""));
                                        }
                                        catch (Exception)
                                        {
                                            LengthValidation = false;
                                        }
                                        if (LengthValidation)
                                        {
                                            ApplyDataValidation = true;
                                            MaxLength = xpHeaderColumnIter.Current.GetAttribute("maxLength", "");
                                            InnerFormula = "IF(LEN($COL$)<= " + MaxLength + ",$NESTEDCONDITION$,0)";
                                            Formula = Formula.Replace("$NESTEDCONDITION$", InnerFormula);
                                            ValidationErrorMessage = "Length should not exceed beyond " + MaxLength + " characters.";
                                        }
                                        #endregion

                                        #region Range Validation
                                        try
                                        {
                                            RangeValidation = Boolean.Parse(xpHeaderColumnIter.Current.GetAttribute("rangeValidation", ""));
                                        }
                                        catch (Exception)
                                        {
                                            RangeValidation = false;
                                        }
                                        if (RangeValidation)
                                        {
                                            ApplyDataValidation = true;
                                            MinValue = xpHeaderColumnIter.Current.GetAttribute("minValue", "");
                                            MaxValue = xpHeaderColumnIter.Current.GetAttribute("maxValue", "");
                                            InnerFormula = "AND($COL$ >= " + MinValue + ",$COL$ <= " + MaxValue + ")";
                                            Formula = Formula.Replace("$NESTEDCONDITION$", InnerFormula);
                                            ValidationErrorMessage = "Value should be between " + MinValue + " and " + MaxValue + ".";
                                        }
                                        #endregion
                                        
                                        if (ApplyDataValidation)
                                        {
                                            Formula = Formula.Replace("$NESTEDCONDITION$", "1");
                                            Formula = Formula.Replace("$COL$", ColumnName);

                                            try
                                            {
                                                DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.Add(XlDVType.xlValidateCustom, XlDVAlertStyle.xlValidAlertStop,
                                                    Type.Missing, Formula, Type.Missing);
                                            }
                                            catch (Exception)
                                            {
                                                //Incase exception is thrown while specifing validation criteria still excel should be created.
                                            }

                                        }
                                    }
                                }*/

                                if (ApplyDataValidation)
                                {
                                    DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.ShowError = true;
                                    DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.ErrorMessage = ValidationErrorMessage;
                                    DataValidationRangeObj[xpVehicleIter.CurrentPosition - 1].Validation.ErrorTitle = "Validation Failed";
                                }
                            }
                            #endregion

                            try
                            {
                                SpaceLevel = int.Parse(xpVehicleIter.Current.SelectSingleNode("SPACELEVEL").Value);
                            }
                            catch (Exception)
                            {
                                SpaceLevel = 1;
                            }
                            FacetName = xpVehicleIter.Current.SelectSingleNode("FACETNAME").Value;
                            RelationshipName = xpVehicleIter.Current.SelectSingleNode("RELATIONSHIPNAME").Value;
                            xpExp = null;
                            xpExp = XPathExpression.Compile("VEHICLE_SPACE/SPACE");
                            
                            SpaceLevel--;
                            sbxPath.Remove(0, sbxPath.Length);                            

                            while (SpaceLevel > 0)
                            {                                                            
                                sbxPath.Append("SPACE_SPACE/SPACE/");
                                SpaceLevel--;
                            }
                            Boolean IsSpace = false;
                            xpSpaceIter = xpVehicleIter.Current.Select(xpExp);
                            while (xpSpaceIter.MoveNext())
                            {
                                //Rows of Excel   
                                //if (!IsExportSubmissionCancelled(StatusXML))                                
                                {
                                    xpHeaderColumnIter = xpVehicleIter.Current.Select(xpHeaderExp);
                                    while (xpHeaderColumnIter.MoveNext())
                                    {                                        
                                        try
                                        {
                                            IsSpace = Boolean.Parse(xpHeaderColumnIter.Current.GetAttribute("isspace", ""));
                                        }
                                        catch (Exception)
                                        {
                                            IsSpace = false;
                                        }
                                        try
                                        {
                                            CurrentSpaceLevel = int.Parse(xpHeaderColumnIter.Current.GetAttribute("spacelevel", ""));
                                        }
                                        catch (Exception)
                                        {
                                            CurrentSpaceLevel = 1;
                                        }
                                        xpath = String.Empty;
                                        if (IsSpace)
                                        {
                                            if (CurrentSpaceLevel == 1)
                                                xpath = xpHeaderColumnIter.Current.GetAttribute("name", "");
                                            else
                                            {
                                                CurrentSpaceLevel--;
                                                while (CurrentSpaceLevel > 0)
                                                {                                                    
                                                    xpath += "SPACE_SPACE/SPACE/";
                                                    CurrentSpaceLevel--;
                                                }
                                                xpath += xpHeaderColumnIter.Current.GetAttribute("name", "");
                                            }
                                        }
                                        else
                                            xpath = sbxPath.ToString() +  RelationshipName + "/" + FacetName + "/" + xpHeaderColumnIter.Current.GetAttribute("name", "");

                                        if (!String.IsNullOrEmpty(xpath))
                                        {
                                            xpColumnNav = xpSpaceIter.Current.SelectSingleNode(xpath);
                                            if (xpColumnNav != null)
                                            {
                                                CellRangeObj = ExcelWorksheetObj[xpVehicleIter.CurrentPosition - 1].Cells[RowPosition, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                                                if (String.Compare(xpHeaderColumnIter.Current.GetAttribute("fieldtype", ""), "MULTISELECT", true) == 0)
                                                    CellRangeObj.Value2 = formatCellData("MultiSelect", MultiSelectSeparator, xpColumnNav);
                                                else
                                                    CellRangeObj.Value2 = xpColumnNav.Value;
                                                CellRangeObj.ColumnWidth = 20;
                                            }
                                        }
                                    }
                                    RowPosition++;
                                }
                                //else
                                //    break;
                            }
                            #endregion

                            //if (!IsExportSubmissionCancelled(StatusXML))
                            {
                                #region Update completed Vehicle Count
                                StatusDocObj = new XmlDocument();
                                StatusDocObj.Load(StatusXML);
                                NodeObj = StatusDocObj.SelectSingleNode("PROGRESSSTATUS/VEHICLESPROCESSED");
                                if (NodeObj != null)
                                    NodeObj.InnerText = xpVehicleIter.CurrentPosition.ToString();
                                StatusDocObj.Save(StatusXML);
                                StatusDocObj = null;
                                #endregion
                            }
                        }
                        //else 
                        //    break;
                    }

                    //if (!IsExportSubmissionCancelled(StatusXML))
                    {
                        if (File.Exists(ExcelFilePath))
                            File.Delete(ExcelFilePath);

                        ExcelWorkbookObj.SaveAs(ExcelFilePath, Microsoft.Office.Interop.Excel.XlFileFormat.xlExcel8, Type.Missing,
                                                                       Type.Missing, Type.Missing, Type.Missing,
                                                                       Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange, Type.Missing,
                                                                       Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                        #region Mark Status as Done
                        StatusDocObj = new XmlDocument();
                        StatusDocObj.Load(StatusXML);
                        NodeObj = StatusDocObj.SelectSingleNode("PROGRESSSTATUS/EXPORTSTATUS");
                        if (NodeObj != null)
                            NodeObj.InnerText = "COMPLETED";
                        StatusDocObj.Save(StatusXML);
                        StatusDocObj = null;
                        #endregion
                    }
                }
            }
            catch (Exception ExcepObj)
            {
                ObjErrorCommon = new ALErrorCommon();
                ObjErrorCommon.AppendError("MSOFFICEMEDIAPLUGIN_ExportSubmissionDataToExcel", this.GetType().ToString(), "ExportSubmissionDataToExcel", ExcepObj, null);
                #region Mark Status as Error
                XmlNode ErrorNode = null;

                if (File.Exists(StatusXML))
                {
                    StatusDocObj = new XmlDocument();
                    StatusDocObj.Load(StatusXML);
                    NodeObj = StatusDocObj.SelectSingleNode("PROGRESSSTATUS/EXPORTSTATUS");
                    if (NodeObj != null)
                        NodeObj.InnerText = "ERROR";

                    ErrorNode = (XmlNode)StatusDocObj.CreateElement("ERRORXML");
                    ErrorNode.InnerXml = ObjErrorCommon.ErrorCollectionXML;
                    StatusDocObj.SelectSingleNode("PROGRESSSTATUS").AppendChild(ErrorNode);
                    StatusDocObj.Save(StatusXML);
                    StatusDocObj = null;
                }
                #endregion
                return ObjErrorCommon.ErrorCollectionXML;
            }
            finally
            {
                if (StartDataCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(StartDataCellRangeObj);
                    StartDataCellRangeObj = null;
                }
                if (EndDataCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(EndDataCellRangeObj);
                    EndDataCellRangeObj = null;
                }
                if (DataValidationRangeObj != null)
                {
                    for (int index = 0; index < DataValidationRangeObj.Length; index++)
                    {
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(DataValidationRangeObj[index]);
                        DataValidationRangeObj[index] = null;
                    }
                }
                if (StartCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(StartCellRangeObj);
                    StartCellRangeObj = null;
                }
                if (LookUpCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(LookUpCellRangeObj);
                    LookUpCellRangeObj = null;
                }
                if (CellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(CellRangeObj);
                    CellRangeObj = null;
                }
                if (LookUpExcelWorksheetObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(LookUpExcelWorksheetObj);
                    LookUpExcelWorksheetObj = null;
                }
                if (ExcelWorksheetObj != null)
                {
                    for (int index = 0; index < ExcelWorksheetObj.Length; index++)
                    {
                        System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelWorksheetObj[index]);
                        ExcelWorksheetObj[index] = null;
                    }
                }
                if (ExcelWorkbookObj != null)
                {
                    ExcelWorkbookObj.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelWorkbookObj);
                    ExcelWorkbookObj = null;
                }

                if (ExcelAppObj != null)
                {
                    ExcelAppObj.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelAppObj);
                    ExcelAppObj = null;
                }
            }
            return ErrorXML;
        }

        public ERRORXML ExportVehicleDataToExcel(String JobInfoXML, String ExcelLayoutNode)
        {
            ERRORXML ErrorXml = null;
            XPathDocument xpDoc_JobInfo = null;
            XPathNavigator xpNav_JobInfo = null;
            string DataXmlPath = null;
            string LookUpXMLPath = null;
            string ExcelFilePath = null;
            string StatusFilePath = null;
            string MultiSelectSeparator = null;
            String OfficeTemplateFilePath = null;
            Microsoft.Office.Interop.Excel.ApplicationClass ExcelAppObj = null;
            WorkbookClass ExcelWorkbookObj = null;
            Worksheet DataWorksheetObj = null;
            Worksheet LookUpExcelWorksheetObj = null;
            Microsoft.Office.Interop.Excel.Range CellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range LookUpCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range EndDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range DataValidationRangeObj = null;
            String LookupSheetName = null;
            int LookUpRowIndex = 2;
            String LookupName = null;
            XPathDocument xpLookupDocObj = null;
            XPathNavigator xpLookupNav = null;
            XPathNodeIterator xpLookupIter = null;
            XPathNodeIterator xpLookupValueIter = null;
            StringBuilder sbNameList = null;
            String NameList = null;
            int RowPosition = 0;
            string SheetName = null;
            XPathDocument LayoutDoc = null;
            XPathNavigator xpNav_ParentLayoutDoc = null;
            XPathNavigator xpNav_LayoutDoc = null;
            XPathDocument DataDoc = null;
            XPathNavigator xpNav_SubData = null;
            XPathNodeIterator xpIter_VehicleCollection = null;
            XPathNavigator xpNav_DataDoc = null;
            XPathNavigator xpColumnNav = null;
            XPathNavigator xpNav_Date = null;
            XPathNodeIterator xpHeaderColumnIter = null;
            XPathNodeIterator xpIter_Instance = null;
            XPathNodeIterator xpIter_SelectionField = null;
            XPathExpression xpExp_Instance = null;
            Boolean ApplyDataValidation = false;
            String ValidationErrorMessage = null;
            const int DisplayHeaderRowNo = 3;
            const int InternalHeaderRowNo = 4;            
            string FieldType = null;
            string StartDate = null;
            string EndDate = null;
            string ValidationLookUp = null;
            string CurrentColumnDataType = null; 
            object EmptySheet = null;
            string TemplateType = null;

            try
            {
                #region Get all parameters
                xpDoc_JobInfo = new XPathDocument(new StringReader(JobInfoXML));
                xpNav_JobInfo = xpDoc_JobInfo.CreateNavigator();
                DataXmlPath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/DATAXMLPATH").Value;
                LookUpXMLPath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/LOOKUPXMLPATH").Value;
                ExcelFilePath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/EXCELFILEPATH").Value;
                StatusFilePath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/STATUSFILEPATH").Value;
                MultiSelectSeparator = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/MULTISELECTSEPARATOR").Value;
                #endregion

                ExcelAppObj = new Microsoft.Office.Interop.Excel.ApplicationClass();
                OfficeTemplateFilePath = ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls";
                ExcelWorkbookObj = ExcelAppObj.Workbooks.Add(OfficeTemplateFilePath) as WorkbookClass;

                if (ExcelWorkbookObj.Sheets.Count > 0)
                {

                    int TotalSheets = ExcelWorkbookObj.Sheets.Count;

                    for (int cnt = TotalSheets; cnt >= 1; cnt--)
                    {
                        EmptySheet = ExcelWorkbookObj.Sheets.get_Item(cnt);
                        if ((EmptySheet != null) && (cnt != 1))
                            ((Microsoft.Office.Interop.Excel.Worksheet)EmptySheet).Delete();
                    }
                }

                #region Prepare Lookup Sheet
                sbNameList = new StringBuilder(30);
                xpLookupDocObj = new XPathDocument(LookUpXMLPath);
                xpLookupNav = xpLookupDocObj.CreateNavigator();
                xpLookupNav = xpLookupNav.SelectSingleNode("LOOKUPXML");
                xpLookupIter = xpLookupNav.Select("*");

                LookupSheetName = "Lookup Sheet";
                LookUpExcelWorksheetObj = ExcelWorkbookObj.Sheets.Add(Type.Missing,
                                                       Type.Missing, Type.Missing, Type.Missing) as Worksheet;

                LookUpExcelWorksheetObj.Columns.ColumnWidth = 30;
                LookUpExcelWorksheetObj.Columns.WrapText = true;
                LookUpExcelWorksheetObj.Name = LookupSheetName;
                LookUpExcelWorksheetObj.Visible = XlSheetVisibility.xlSheetHidden;                

                while (xpLookupIter.MoveNext())
                {
                    LookUpRowIndex = 2;
                    LookupName = xpLookupIter.Current.Name;
                    LookUpCellRangeObj = LookUpExcelWorksheetObj.Cells[1, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                    LookUpCellRangeObj.Value2 = LookupName;

                    xpLookupValueIter = xpLookupIter.Current.Select("VALUE");
                    while (xpLookupValueIter.MoveNext())
                    {
                        if (xpLookupValueIter.CurrentPosition == 1)
                            StartCellRangeObj = LookUpExcelWorksheetObj.Cells[LookUpRowIndex, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;

                        LookUpCellRangeObj = LookUpExcelWorksheetObj.Cells[LookUpRowIndex, xpLookupIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                        LookUpCellRangeObj.Value2 = xpLookupValueIter.Current.Value;

                        LookUpRowIndex++;
                    }

                    ExcelWorkbookObj.Names.Add(LookupName, LookUpExcelWorksheetObj.get_Range(StartCellRangeObj, LookUpCellRangeObj),
                                                            true, 3, Type.Missing, Type.Missing, Type.Missing,
                                                            Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                    sbNameList.AppendFormat(" $;{0};$ ", LookupName);
                }
                NameList = sbNameList.ToString();
                #endregion

                DataDoc = new XPathDocument(DataXmlPath);
                xpNav_SubData = DataDoc.CreateNavigator();
                xpIter_VehicleCollection = xpNav_SubData.Select("SUBMISSION/VEHICLE");

                LayoutDoc = new XPathDocument(new StringReader(ExcelLayoutNode));
                xpNav_ParentLayoutDoc = LayoutDoc.CreateNavigator();

                while (xpIter_VehicleCollection.MoveNext())
                {
                    #region Create data sheet
                    
                    xpNav_DataDoc = xpIter_VehicleCollection.Current;

                    RowPosition = 5;
                    
                    SheetName = "Vehicle_" + xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/VEHICLEID").Value;
                    if (SheetName.Length > 30)
                        SheetName = SheetName.Substring(0, 30);

                    TemplateType = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/TEMPLATETYPE").Value;
                    xpNav_LayoutDoc = xpNav_ParentLayoutDoc.SelectSingleNode("ROOT/VEHICLETYPE[@Name = " + ALXmlUtil.GetXpathValueString(TemplateType) + "]");

                    DataWorksheetObj = new Worksheet();

                    if (xpIter_VehicleCollection.CurrentPosition == 1)
                    {
                        DataWorksheetObj = ExcelWorkbookObj.Sheets.Add(EmptySheet,
                                                            Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                        //((Microsoft.Office.Interop.Excel.Worksheet)EmptySheet).Delete();
                    }
                    else
                    {
                        DataWorksheetObj = ExcelWorkbookObj.Sheets.Add(ExcelWorkbookObj.Sheets.get_Item(ExcelWorkbookObj.Sheets.Count),
                                                            Type.Missing, Type.Missing, Type.Missing) as Worksheet;
                    }
                DataWorksheetObj.Name = SheetName;
                DataWorksheetObj.Columns.WrapText = true;
                DataWorksheetObj.Cells.VerticalAlignment = Microsoft.Office.Interop.Excel.XlVAlign.xlVAlignTop;

                //Add Note in first row
                CellRangeObj = DataWorksheetObj.get_Range(DataWorksheetObj.Cells[1, 1], DataWorksheetObj.Cells[1, 4]);
                CellRangeObj.Merge(true);
                CellRangeObj.Value2 = "IMPORTANT: Please do not delete or modify the columns in this excel sheet.\n";
                CellRangeObj.Font.Color = System.Drawing.ColorTranslator.ToOle(System.Drawing.Color.Red);
                CellRangeObj.Font.Bold = true;
                CellRangeObj.Cells.RowHeight = 30;

                //Add Vehicle details in second row
                CellRangeObj = DataWorksheetObj.Cells[2, 1] as Microsoft.Office.Interop.Excel.Range;
                CellRangeObj.Value2 = "Vehicle Name: \nStart Date: \nEnd Date: ";
                CellRangeObj.Font.Bold = true;

                CellRangeObj = DataWorksheetObj.get_Range(DataWorksheetObj.Cells[2, 2], DataWorksheetObj.Cells[2, 3]);
                CellRangeObj.Merge(true);

                xpNav_Date = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/INMKTSTARTDATE/YEAR");
                if (xpNav_Date != null)
                {
                    xpNav_Date.MoveToParent();
                    StartDate = ALCommon.FormatDateField(str_FormatXMLPath,
                                    xpNav_Date.SelectSingleNode("DAY").Value,
                                    xpNav_Date.SelectSingleNode("MONTH").Value,
                                    xpNav_Date.SelectSingleNode("YEAR").Value,
                                    customculture_Object, TenantName);
                }

                xpNav_Date = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/INMKTENDDATE/YEAR");
                if (xpNav_Date != null)
                {
                    xpNav_Date.MoveToParent();
                    EndDate = ALCommon.FormatDateField(str_FormatXMLPath,
                                    xpNav_Date.SelectSingleNode("DAY").Value,
                                    xpNav_Date.SelectSingleNode("MONTH").Value,
                                    xpNav_Date.SelectSingleNode("YEAR").Value,
                                    customculture_Object, TenantName);
                }

                CellRangeObj.Value2 = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/NAME").Value + "\n" +
                                    StartDate + "\n" +
                                    EndDate;

                CellRangeObj = DataWorksheetObj.get_Range(DataWorksheetObj.Cells[2, 1], DataWorksheetObj.Cells[2, 3]);
                CellRangeObj.EntireColumn.AutoFit();

                // Hide Row containing sample elements for internal use
                CellRangeObj = DataWorksheetObj.get_Range(DataWorksheetObj.Cells[InternalHeaderRowNo, 1], DataWorksheetObj.Cells[InternalHeaderRowNo, 2]);
                CellRangeObj.EntireRow.Hidden = true;

                #region Header columns
                xpHeaderColumnIter = xpNav_LayoutDoc.Select("EXCELLAYOUTXML/DATALAYOUT/FIELDCOLLECTION/FIELD");
                while (xpHeaderColumnIter.MoveNext())
                {
                    ApplyDataValidation = false;
                    CellRangeObj = DataWorksheetObj.Cells[DisplayHeaderRowNo, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                    CellRangeObj.EntireRow.Font.Bold = true;
                    CellRangeObj.Value2 = xpHeaderColumnIter.Current.SelectSingleNode("COLUMNNAME").Value;
                    CellRangeObj.ColumnWidth = 20;
                    CellRangeObj = DataWorksheetObj.Cells[InternalHeaderRowNo, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                    CurrentColumnDataType = xpHeaderColumnIter.Current.GetAttribute("DataType", "");
                    switch (CurrentColumnDataType)
                    {
                        case "INTEGER":
                            CellRangeObj.Value2 = "1";
                            break;
                        case "FLOAT":
                            CellRangeObj.Value2 = "1.00";
                            break;
                        default:
                            CellRangeObj.Value2 = "a";
                            break;
                    }                   

                    StartDataCellRangeObj = DataWorksheetObj.Cells[RowPosition, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                    EndDataCellRangeObj = DataWorksheetObj.Cells[RowPosition + 1000, xpHeaderColumnIter.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                    DataValidationRangeObj = DataWorksheetObj.get_Range(StartDataCellRangeObj, EndDataCellRangeObj);

                    ValidationLookUp = "";
                    ValidationLookUp = xpHeaderColumnIter.Current.GetAttribute("NHID", "");
                    if (!String.IsNullOrEmpty(ValidationLookUp))
                    {
                        ValidationLookUp = "NH_" + ValidationLookUp;
                        if (!String.IsNullOrEmpty(xpHeaderColumnIter.Current.GetAttribute("NHLEVEL", "")))
                        {
                            ValidationLookUp = ValidationLookUp + "_" + xpHeaderColumnIter.Current.GetAttribute("NHLEVEL", "");
                        }
                    }

                    if (!String.IsNullOrEmpty(NameList) && NameList.Contains(" $;" + ValidationLookUp + ";$ "))
                    {
                        FieldType = xpHeaderColumnIter.Current.GetAttribute("fieldtype", "");
                        if (String.Compare(FieldType, "MULTISELECT", true) == 0)
                            ApplyDataValidation = false;
                        else
                            ApplyDataValidation = true;

                        DataValidationRangeObj.Validation.Add(XlDVType.xlValidateList, XlDVAlertStyle.xlValidAlertStop,
                                XlFormatConditionOperator.xlEqual, "=" + ValidationLookUp, Type.Missing);

                        DataValidationRangeObj.Validation.ShowError = false;
                        ValidationErrorMessage = "Value should be selected from the available values and should not be modified.";
                    }                   

                    if (ApplyDataValidation)
                    {
                        DataValidationRangeObj.Validation.ShowError = true;
                        DataValidationRangeObj.Validation.ErrorMessage = ValidationErrorMessage;
                        DataValidationRangeObj.Validation.ErrorTitle = "Validation Failed";
                    }
                }                
                #endregion

                xpExp_Instance = XPathExpression.Compile("INSTANCES/INSTANCE");
                xpExp_Instance.AddSort("Order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);
                xpIter_Instance = xpNav_DataDoc.Select(xpExp_Instance);

                while (xpIter_Instance.MoveNext())
                {
                    xpIter_SelectionField = xpNav_LayoutDoc.Select("EXCELLAYOUTXML/DATALAYOUT/FIELDCOLLECTION/FIELD");

                    while (xpIter_SelectionField.MoveNext())
                    {
                        xpColumnNav = xpIter_Instance.Current.SelectSingleNode(xpIter_SelectionField.Current.SelectSingleNode("FIELDNAME").Value);
                        if (xpColumnNav != null)
                        {
                            CellRangeObj = DataWorksheetObj.Cells[RowPosition, xpIter_SelectionField.CurrentPosition] as Microsoft.Office.Interop.Excel.Range;
                            if (String.Compare(xpIter_SelectionField.Current.GetAttribute("fieldtype", ""), "MULTISELECT", true) == 0)
                                CellRangeObj.Value2 = formatCellData("MultiSelect", MultiSelectSeparator, xpColumnNav);
                            else
                                CellRangeObj.Value2 = xpColumnNav.Value;
                            CellRangeObj.ColumnWidth = 20;
                        }
                    }
                    RowPosition++;
                }
                #endregion
                }

                #region Save Excel Sheet
                if (File.Exists(ExcelFilePath))
                    File.Delete(ExcelFilePath);

                ExcelWorkbookObj.SaveAs(ExcelFilePath, Microsoft.Office.Interop.Excel.XlFileFormat.xlExcel8, Type.Missing,
                                                               Type.Missing, Type.Missing, Type.Missing,
                                                               Microsoft.Office.Interop.Excel.XlSaveAsAccessMode.xlNoChange, Type.Missing,
                                                               Type.Missing, Type.Missing, Type.Missing, Type.Missing);
                #endregion

                
            }
            catch
            {

            }
            finally
            {
                if (StartDataCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(StartDataCellRangeObj);
                    StartDataCellRangeObj = null;
                }
                if (EndDataCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(EndDataCellRangeObj);
                    EndDataCellRangeObj = null;
                }
                if (DataValidationRangeObj != null)
                {                    
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(DataValidationRangeObj);
                    DataValidationRangeObj = null;                    
                }
                if (StartCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(StartCellRangeObj);
                    StartCellRangeObj = null;
                }
                if (LookUpCellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(LookUpCellRangeObj);
                    LookUpCellRangeObj = null;
                }
                if (CellRangeObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(CellRangeObj);
                    CellRangeObj = null;
                }
                if (LookUpExcelWorksheetObj != null)
                {
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(LookUpExcelWorksheetObj);
                    LookUpExcelWorksheetObj = null;
                }
                if (DataWorksheetObj != null)
                {                    
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(DataWorksheetObj);
                    DataWorksheetObj = null;                    
                }
                if (ExcelWorkbookObj != null)
                {
                    ExcelWorkbookObj.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelWorkbookObj);
                    ExcelWorkbookObj = null;
                }

                if (ExcelAppObj != null)
                {
                    ExcelAppObj.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(ExcelAppObj);
                    ExcelAppObj = null;
                }
            }
            return ErrorXml;
        }
        public ERRORXML PrepareConsolidatedSubmissionReport(String JobInfoXML, String ExcelLayoutNode)
        {
            ERRORXML ErrorXml = null;
            XPathDocument xpDoc_JobInfo = null;
            XPathNavigator xpNav_JobInfo = null;
            string DataXmlPath = null;
            string LookUpXMLPath = null;
            string ExcelFilePath = null;
            string StatusFilePath = null;
            string MultiSelectSeparator = null;
            String OfficeTemplateFilePath = null;
            Microsoft.Office.Interop.Excel.ApplicationClass ExcelAppObj = null;
            WorkbookClass ExcelWorkbookObj = null;
            Worksheet DataWorksheetObj = null;
            Worksheet LookUpExcelWorksheetObj = null;
            Microsoft.Office.Interop.Excel.Range CellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range LookUpCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range StartDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range EndDataCellRangeObj = null;
            Microsoft.Office.Interop.Excel.Range DataValidationRangeObj = null;
            String LookupSheetName = null;
            int LookUpRowIndex = 2;
            String LookupName = null;
            XPathDocument xpLookupDocObj = null;
            XPathNavigator xpLookupNav = null;
            XPathNodeIterator xpLookupIter = null;
            XPathNodeIterator xpLookupValueIter = null;
            StringBuilder sbNameList = null;
            String NameList = null;
            int RowPosition = 0;
            string SheetName = null;
            XPathDocument LayoutDoc = null;
            XPathNavigator xpNav_LayoutDocCollection = null;
            XPathNavigator xpNav_LayoutDoc = null;
            XPathDocument DataDoc = null;
            XPathNavigator xpNav_SubData = null;
            XPathNodeIterator xpIter_VehicleCollection = null;
            XPathNavigator xpNav_DataDoc = null;
            XPathNavigator xpColumnNav = null;
            XPathNavigator xpNav_Date = null;
            XPathNodeIterator xpHeaderColumnIter = null;
            XPathNodeIterator xpIter_Instance = null;
            XPathNodeIterator xpIter_SelectionField = null;
            XPathExpression xpExp_Instance = null;
            Boolean ApplyDataValidation = false;
            String ValidationErrorMessage = null;
            const int DisplayHeaderRowNo = 3;
            const int InternalHeaderRowNo = 4;
            string FieldType = null;
            string StartDate = null;
            string EndDate = null;
            string ValidationLookUp = null;
            string CurrentColumnDataType = null;
            string TemplateType = null;
            object EmptySheet = null;            

            try
            {
                #region Get all parameters
                xpDoc_JobInfo = new XPathDocument(new StringReader(JobInfoXML));
                xpNav_JobInfo = xpDoc_JobInfo.CreateNavigator();
                DataXmlPath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/DATAXMLPATH").Value;                
                ExcelFilePath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/EXCELFILEPATH").Value;
                StatusFilePath = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/STATUSFILEPATH").Value;
                MultiSelectSeparator = xpNav_JobInfo.SelectSingleNode("EXPORTVEHICLE/MULTISELECTSEPARATOR").Value;
                #endregion 

                DataDoc = new XPathDocument(DataXmlPath);
                xpNav_SubData = DataDoc.CreateNavigator();
                xpIter_VehicleCollection = xpNav_SubData.Select("SUBMISSION/VEHICLE");

                LayoutDoc = new XPathDocument(new StringReader(ExcelLayoutNode));
                xpNav_LayoutDocCollection = LayoutDoc.CreateNavigator();

                PrepareSubmissionSheetXML(ExcelFilePath, xpNav_SubData.OuterXml, xpNav_LayoutDocCollection.OuterXml, MultiSelectSeparator);

            }
            catch
            {
                throw;
            }
            
            return ErrorXml;
        }
        #endregion

        #region Private Methods
        private String FormatValueforDataType(String str_DataFieldType, XElement xEle_TempNode)
        {
            String str_FieldValue = "";
            try
            {                
                switch (str_DataFieldType)
                {
                    case "DATE":
                        if (xEle_TempNode.Element("YEAR") != null)
                        {
                            str_FieldValue = ALCommon.FormatDateField(str_FormatXMLPath,
                                    xEle_TempNode.Element("DAY").Value,
                                    xEle_TempNode.Element("MONTH").Value,
                                    xEle_TempNode.Element("YEAR").Value,
                                    customculture_Object, TenantName);
                        }
                        break;
                    case "DATETIME":
                        if (xEle_TempNode.Element("YEAR") != null)
                        {
                            String currHour = "0";
                            String currMin = "0";
                            String currSec = "0";
                            if (xEle_TempNode.Element("HOUR") != null)
                                currHour = xEle_TempNode.Element("HOUR").Value;
                            if (xEle_TempNode.Element("MINUTE") != null)
                                currMin = xEle_TempNode.Element("MINUTE").Value;
                            if (xEle_TempNode.Element("SECOND") != null)
                                currSec = xEle_TempNode.Element("SECOND").Value;

                            int currHourNum = Convert.ToInt32(currHour);
                            int currMinNum = Convert.ToInt32(currMin);
                            int currSecNum = Convert.ToInt32(currSec);

                            if (currHourNum == 0 && currMinNum == 0 && currSecNum == 0)
                            {
                                str_FieldValue = ALCommon.FormatDateField(str_FormatXMLPath,
                                        xEle_TempNode.Element("DAY").Value,
                                        xEle_TempNode.Element("MONTH").Value,
                                        xEle_TempNode.Element("YEAR").Value,
                                        customculture_Object, TenantName);
                            }
                            else
                            {
                                str_FieldValue = ALCommon.FormatDateTimeField(str_FormatXMLPath,
                                        xEle_TempNode.Element("DAY").Value,
                                        xEle_TempNode.Element("MONTH").Value,
                                        xEle_TempNode.Element("YEAR").Value,
                                        currHour,
                                        currMin,
                                        currSec,
                                        customculture_Object, null, TenantName);
                            }
                        }
                        break;
                    case "FLOAT":
                        str_FieldValue = xEle_TempNode.Value;
                        str_FieldValue = ALCommon.FormatFloatField(str_FormatXMLPath,
                                str_FieldValue, customculture_Object);
                        break;

                    case "CURRENCY":
                        if (xEle_TempNode.Element("AMOUNT") != null)
                        {
                            String str_Currency = "";
                            if (xEle_TempNode.Element("CURRENCY") != null)
                                str_Currency = xEle_TempNode.Element("CURRENCY").Value;
                            str_FieldValue =
                                    str_Currency + " "
                                    + ALCommon.FormatCurrencyField(str_FormatXMLPath,
                                    xEle_TempNode.Element("AMOUNT").Value,
                                    customculture_Object);
                        }
                        break;

                    default:
                        str_FieldValue = xEle_TempNode.Value;
                        ALSpecialCharHandling.DecodeSpecialCharacters(ref str_FieldValue);
                        str_FieldValue = str_FieldValue.Replace("$ALP$", "-").Replace("$AL$", "");
                        break;
                }
            }
            catch
            {
                //Ignore Exception for formatting of data value based on field data type.
            }
            return str_FieldValue;
        }

        private String ConvertToExcelWidth(String width)
        {            
            int widthNum = Convert.ToInt32(width);
            double widthDouble = widthNum / PointtoPixelFactor;
            widthNum = Convert.ToInt32(Math.Ceiling(widthDouble));
            return widthNum.ToString();
        }

        //formatCellData() method is used to format the cell data based on the UI Type.
        private String formatCellData(String UIType, String MultiSelectDelimiter, XPathNavigator TargetObjectAttribute)
        {
            String      str_MultiSelectvalue    = null;
            String[]    arrstr_MultiSelectvalue = null;

            if (TargetObjectAttribute != null)
            {
                switch (UIType.ToUpper())
                {
                    case "MULTISELECT":
                    case "TREEVIEW":
                        convertDelimiterSeparatorToSeparator(TargetObjectAttribute.Value, ",", MultiSelectDelimiter, 
                                out str_MultiSelectvalue, out arrstr_MultiSelectvalue, false);
                        break;

                    case "CURRENCY":
                        if ((TargetObjectAttribute.SelectSingleNode("CURRENCY") != null) 
                                && (TargetObjectAttribute.SelectSingleNode("CURRENCY").Value.ToUpper() != "NULL"))
                        {
                            if (TargetObjectAttribute.SelectSingleNode("AMOUNT") != null)
                            {
                                str_MultiSelectvalue = TargetObjectAttribute.SelectSingleNode("CURRENCY").Value + " "
                                        + TargetObjectAttribute.SelectSingleNode("AMOUNT").Value;
                            }
                            else
                                str_MultiSelectvalue = TargetObjectAttribute.SelectSingleNode("CURRENCY").Value + " 0.0";
                        }
                        break;

                    case "JCALENDAR":
                    case "ALDATE":
                        if (TargetObjectAttribute.SelectSingleNode("DAY") != null && TargetObjectAttribute.SelectSingleNode("MONTH") != null
                                && TargetObjectAttribute.SelectSingleNode("YEAR") != null && TargetObjectAttribute.SelectSingleNode("HOUR") != null
                                && TargetObjectAttribute.SelectSingleNode("MINUTE") != null && TargetObjectAttribute.SelectSingleNode("SECOND") != null)
                        {
                            str_MultiSelectvalue = TargetObjectAttribute.SelectSingleNode("MONTH") + "/" 
                                    + TargetObjectAttribute.SelectSingleNode("DAY") + "/" 
                                    + TargetObjectAttribute.SelectSingleNode("YEAR") + " " 
                                    + TargetObjectAttribute.SelectSingleNode("HOUR") + ":" 
                                    + TargetObjectAttribute.SelectSingleNode("MINUTE") + ":" 
                                    + TargetObjectAttribute.SelectSingleNode("SECOND");
                        }
                        break;

                    default:
                        ////case "TEXT":
                        ////case "EDITBOX":
                        ////case "TEXTAREA":
                        ////case "DROPDOWNLIST":
                        str_MultiSelectvalue = TargetObjectAttribute.Value;
                        break;

                }
            }

            return (str_MultiSelectvalue);
        }

        //ConvertDelimiterSeparatorToSeparator() method is used to converts the Multiselect Delimiter separator string to separator (,) string, 
        // to display the data on screen.
        private void convertDelimiterSeparatorToSeparator(String InputString, String Separator, String MultiSelectDelimiter,
                out String OutPutString, out String[] OutPutStringArray, bool PrepareOutputArray)
        {
            String[] arrstr_Separator = { Separator };

            OutPutString        = null;
            OutPutStringArray   = null;
            if (InputString != null)
            {
                OutPutString = InputString.Replace(MultiSelectDelimiter, Separator);
                if (OutPutString.StartsWith(Separator))
                {
                    OutPutString = OutPutString.Remove(0, Separator.Length);
                }

                if (OutPutString.EndsWith(Separator))
                {
                    OutPutString = OutPutString.Remove(OutPutString.Length - Separator.Length);
                }

                if (PrepareOutputArray && OutPutString != "")
                    OutPutStringArray = OutPutString.Split(arrstr_Separator, StringSplitOptions.RemoveEmptyEntries);
            }
        }

        /// <summary>
        /// Converts the separator (,) String to Multiselect Delimiter separator string, which can be store in Database.
        /// </summary>
        /// <param name="InputString">Input Separator string</param>
        /// <param name="Separator">Separator value</param>
        /// <param name="OutPutString">Output string</param>
        /// <param name="OutPutStringArray">Output String Array</param>
        /// <param name="PrepareOutputArray">Flag to specify whether to generate the Output string Array or not.</param>
        /// <param name="str_MultiSelectDelimiter">Multi-select Delimiter used</param>
        /// <returns></returns>
        private ERRORXML ConvertSeparatorToDelimiterSeparator(String InputString, String Separator, out String OutPutString, out String[] OutPutStringArray, bool PrepareOutputArray, String str_MultiSelectDelimiter)
        {
            ERRORXML ErrorMessage = null;
            OutPutString = null;
            OutPutStringArray = null;

            String[] arrSeparator = { str_MultiSelectDelimiter };

            try
            {
                if (InputString != null)
                {
                    OutPutString = InputString.Replace(Separator, str_MultiSelectDelimiter);
                    if (PrepareOutputArray && OutPutString != "")
                        OutPutStringArray = OutPutString.Split(arrSeparator, StringSplitOptions.None);

                    OutPutString = str_MultiSelectDelimiter + OutPutString + str_MultiSelectDelimiter;
                }
            }
            catch (Exception Excep)
            {
                ErrorMessage = Excep.Message;
            }

            return ErrorMessage;
        }

        //checkIsExcelGenerationCancelledEvent() method is used to verify whether the CANCEL event is generated from solution
        // while Excel processing is in-progress.
        //Solution will set the processing status to CANCELED using web service to the file StatusCompleteFilePath.  
        private Boolean checkIsExcelGenerationCancelledEvent(String StatusCompleteFilePath)
        {
            String  str_ExcelExportEventStatus      = null;
            XML     xml_ExcelGenerationProcessedLog = null;

            if (File.Exists(StatusCompleteFilePath))
            {
                xml_ExcelGenerationProcessedLog = File.ReadAllText(StatusCompleteFilePath);
                str_ExcelExportEventStatus = ALCommon.GetInnerValue(ref xml_ExcelGenerationProcessedLog, "<EXPORTSTATUS>", "</EXPORTSTATUS>");
                if ((!(String.IsNullOrEmpty(str_ExcelExportEventStatus))) && (str_ExcelExportEventStatus.Equals("CANCELED")))
                    return (true);
            }

            return (false);
        }

        private bool ValidateFieldValuewithType(HeaderInfo HeaderInfoObj, String FieldValue, String ExcelColumnName, String RowItr, ref StringBuilder sb_ImportErrorXML, String str_Err_RECORDIMPROPER)
        {
            #region Local variables
            decimal DecimalValue = 0;
            Boolean b_ReturnValue = true;
            #endregion

            switch (HeaderInfoObj.DataType.ToUpper())
            {
                case "STRING":
                case "TEXT":
                    b_ReturnValue = true;
                    break;
                case "INTEGER":
                    long IntValue = 0;
                    b_ReturnValue = System.Int64.TryParse(FieldValue, out IntValue);
                    break;
                case "DATETIME":
                    DateTime objDateTime;
                    b_ReturnValue = System.DateTime.TryParse(FieldValue, out objDateTime);
                    break;
                case "FLOAT":
                    b_ReturnValue = System.Decimal.TryParse(FieldValue, out DecimalValue);
                    break;
                case "CURRENCY":
                    String[] CurrencySeparator = { " " };
                    String[] arrCurrencyValues = FieldValue.Split(CurrencySeparator, StringSplitOptions.None);
                    if (arrCurrencyValues.Length > 1)
                        b_ReturnValue = System.Decimal.TryParse(arrCurrencyValues[1], out DecimalValue);
                    else
                        b_ReturnValue = false;
                    break;
                default:
                    b_ReturnValue = true;
                    break;
            }
            if (!b_ReturnValue)
            {
                sb_ImportErrorXML.Append("<ERROR>");
                sb_ImportErrorXML.Append("<ERRORCODE>ERR_FSIMPORT_RECORDIMPROPER</ERRORCODE>");
                //sb_ImportErrorXML.Append("<ERRORINFORMATION>" + Localization.GetString("ALErrorResource", TenantName, "ERR_FSIMPORT_RECORDIMPROPER").Replace("#COLUMNNAME#", ExcelColumnName) + " " + HeaderInfoObj.DataType + "</ERRORINFORMATION>");
                sb_ImportErrorXML.Append("<ERRORINFORMATION>" + str_Err_RECORDIMPROPER.Replace("#COLUMNNAME#", ExcelColumnName) + " " + HeaderInfoObj.DataType + "</ERRORINFORMATION>");
                sb_ImportErrorXML.AppendFormat("<ERRORROWNUMBER>{0}</ERRORROWNUMBER>", RowItr.ToString());
                sb_ImportErrorXML.Append("</ERROR>");
            }
            return b_ReturnValue;
        }

        private StringBuilder PrepareTargetAttibuteNode(HeaderInfo HeaderInfoObj, String ColumnName, String OldColumnValue, String NewColumnValue, Boolean IsNewRecord, String str_MultiSelectDelimiter)
        {
            StringBuilder TargetAttibuteNodeXml = new StringBuilder(125);
            String MultiSelectvalue = null;
            String[] MultiSelectvalueArray = null;
            String SeriesNextID = null;

            TargetAttibuteNodeXml.Append("<" + ColumnName + ">");

            if (HeaderInfoObj.IsReadOnly && HeaderInfoObj.DoAutoGenerate)
                NewColumnValue = null;

            if (HeaderInfoObj.IsReadOnly && (!HeaderInfoObj.DoAutoGenerate))
            {
                TargetAttibuteNodeXml.Append(OldColumnValue);
                return TargetAttibuteNodeXml.Append("</" + ColumnName + ">");
            }
            else if (HeaderInfoObj.DoAutoGenerate && String.IsNullOrEmpty(NewColumnValue) && (!String.IsNullOrEmpty(HeaderInfoObj.SeriesID)) && IsNewRecord)
            {
                //if (GetInfrastructureFacadeInstance().GetNextIDInSeriesByID(HeaderInfoObj.SeriesID, out SeriesNextID) == null)
                //TargetAttibuteNodeXml.Append(SeriesNextID);

                SeriesNextID = HeaderInfoObj.SeriesIDList[HeaderInfoObj.SeriesIDListIndex];
                HeaderInfoObj.SeriesIDListIndex++;
                TargetAttibuteNodeXml.Append(SeriesNextID);
                return TargetAttibuteNodeXml.Append("</" + ColumnName + ">");
            }


            switch (HeaderInfoObj.UIType.ToUpper())
            {
                default:
                case "TEXT":
                case "EDITBOX":
                case "TEXTAREA":
                case "DROPDOWNLIST":
                    TargetAttibuteNodeXml.Append(ALSpecialCharHandling.TrimAndReplaceSpecialChars(NewColumnValue));
                    break;

                case "MULTISELECT":
                case "TREEVIEW":
                    ConvertSeparatorToDelimiterSeparator(ALSpecialCharHandling.TrimAndReplaceSpecialChars(NewColumnValue), ",", out MultiSelectvalue, out MultiSelectvalueArray, false, str_MultiSelectDelimiter);
                    TargetAttibuteNodeXml.Append(MultiSelectvalue);
                    break;

                case "CURRENCY":
                    String[] arrCurrencyValues = NewColumnValue.Split(' ');
                    if (!String.IsNullOrEmpty(arrCurrencyValues[0]))
                    {
                        TargetAttibuteNodeXml.Append("<CURRENCY>" + arrCurrencyValues[0] + "</CURRENCY>");
                    }
                    if (!String.IsNullOrEmpty(arrCurrencyValues[1]))
                    {
                        TargetAttibuteNodeXml.Append("<AMOUNT>" + arrCurrencyValues[1] + "</AMOUNT>");
                    }
                    break;

                case "JCALENDAR":
                case "ALDATE":
                    String[] arrDateTimeValues = NewColumnValue.Split(' ');
                    if (arrDateTimeValues != null && arrDateTimeValues.Length >= 2)
                    {
                        String[] arrDateValues = arrDateTimeValues[0].Split('/');

                        if (arrDateValues.Length == 3)
                        {
                            TargetAttibuteNodeXml.Append("<DAY>" + arrDateValues[1] + "</DAY>");
                            TargetAttibuteNodeXml.Append("<MONTH>" + arrDateValues[0] + "</MONTH>");
                            if (arrDateValues[2].IndexOf(" ") != -1)
                            {
                                arrDateValues[2] = arrDateValues[2].Substring(0, arrDateValues[2].IndexOf(" "));
                            }
                            TargetAttibuteNodeXml.Append("<YEAR>" + arrDateValues[2] + "</YEAR>");
                        }

                        String[] arrTimeValues = arrDateTimeValues[1].Split(':');
                        if (arrTimeValues.Length == 3)
                        {
                            TargetAttibuteNodeXml.Append("<HOUR>" + arrTimeValues[0] + "</HOUR>");
                            TargetAttibuteNodeXml.Append("<MINUTE>" + arrTimeValues[1] + "</MINUTE>");
                            TargetAttibuteNodeXml.Append("<SECOND>" + arrTimeValues[2] + "</SECOND>");
                        }
                    }
                    else
                    {
                        if (arrDateTimeValues != null && arrDateTimeValues.Length >= 1)
                        {
                            String[] arrDateValues = arrDateTimeValues[0].Split('/');

                            if (arrDateValues.Length == 3)
                            {
                                TargetAttibuteNodeXml.Append("<DAY>" + arrDateValues[1] + "</DAY>");
                                TargetAttibuteNodeXml.Append("<MONTH>" + arrDateValues[0] + "</MONTH>");
                                if (arrDateValues[2].IndexOf(" ") != -1)
                                {
                                    arrDateValues[2] = arrDateValues[2].Substring(0, arrDateValues[2].IndexOf(" "));
                                }
                                TargetAttibuteNodeXml.Append("<YEAR>" + arrDateValues[2] + "</YEAR>");

                                TargetAttibuteNodeXml.Append("<HOUR>" + DateTime.Now.Hour + "</HOUR>");
                                TargetAttibuteNodeXml.Append("<MINUTE>" + DateTime.Now.Minute + "</MINUTE>");
                                TargetAttibuteNodeXml.Append("<SECOND>" + DateTime.Now.Second + "</SECOND>");
                            }
                        }
                    }
                    break;
            }
            return TargetAttibuteNodeXml.Append("</" + ColumnName + ">");
        }

        private void FillUpLookUpCloumnData(XPathNodeIterator xpItr_ColumnsCloneCopy, String str_MSOfficeProcessorNetworkPath, ref int int_LookUpColumnIndex, ref StringBuilder strbldr_NameList,
            ref Worksheet excelworksheet_LookUpObject, ref WorkbookClass excelworkbook_Object)
        {
            #region Member Variable
            String                                  str_UIType                  = null;
            String                                  str_NamedHierarchyID        = null;
            String                                  str_NamedHierarchyFilePath  = null;
            String                                  str_ColumnName              = null;
            XPathDocument                           xpathdoc_NamedHierarchy     = null;
            XPathNavigator                          xpathnav_NamedHierarchy     = null;
            XPathNodeIterator                       xpItr_NamedHierarchy        = null;
            int                                     int_LookUpRowIndex          = 2;

            Microsoft.Office.Interop.Excel.Range    excelrange_LookUpCellObject     = null;
            Microsoft.Office.Interop.Excel.Range    excelrange_StartCellObject      = null;            
            #endregion

            while (xpItr_ColumnsCloneCopy.MoveNext())
            {
                str_NamedHierarchyID    = null;
                str_UIType              = xpItr_ColumnsCloneCopy.Current.SelectSingleNode("FIELDUITYPE").Value;

                if (xpItr_ColumnsCloneCopy.Current.SelectSingleNode("NAMEDHIERARCHYID") != null)
                    str_NamedHierarchyID    = xpItr_ColumnsCloneCopy.Current.SelectSingleNode("NAMEDHIERARCHYID").Value;

                if (!String.IsNullOrEmpty(str_NamedHierarchyID))
                {
                    int_LookUpRowIndex = 2;
                    str_NamedHierarchyFilePath = str_MSOfficeProcessorNetworkPath + @"\" + str_NamedHierarchyID + "_NHHierarchy.xml";

                    if (File.Exists(str_NamedHierarchyFilePath))
                    {
                        str_ColumnName              = xpItr_ColumnsCloneCopy.Current.SelectSingleNode("FIELDNAME").Value;
                        xpathdoc_NamedHierarchy     = new XPathDocument(str_NamedHierarchyFilePath);
                        xpathnav_NamedHierarchy     = xpathdoc_NamedHierarchy.CreateNavigator();
                        xpathnav_NamedHierarchy     = xpathnav_NamedHierarchy.SelectSingleNode("ROOT/NAMEDHIERARCHYVALUECOLLECTION");

                        if (xpathnav_NamedHierarchy != null)
                        {
                            xpItr_NamedHierarchy = xpathnav_NamedHierarchy.Select("NAMEDHIERARCHYVALUE/NAME");
                            if (xpItr_NamedHierarchy.Count > 0)
                            {
                                excelrange_LookUpCellObject         = excelworksheet_LookUpObject.Cells[1, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                excelrange_LookUpCellObject.Value2  = str_ColumnName;

                                while (xpItr_NamedHierarchy.MoveNext())
                                {
                                    if (xpItr_NamedHierarchy.CurrentPosition == 1)
                                        excelrange_StartCellObject = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;

                                    excelrange_LookUpCellObject         = excelworksheet_LookUpObject.Cells[int_LookUpRowIndex, int_LookUpColumnIndex] as Microsoft.Office.Interop.Excel.Range;
                                    excelrange_LookUpCellObject.Value2  = xpItr_NamedHierarchy.Current.Value;
                                    int_LookUpRowIndex++;
                                }
                                excelworkbook_Object.Names.Add(str_ColumnName, excelworksheet_LookUpObject.get_Range(excelrange_StartCellObject, excelrange_LookUpCellObject), true, 3, Type.Missing, Type.Missing, Type.Missing,
                                                            Type.Missing, Type.Missing, Type.Missing, Type.Missing);

                                if (str_UIType.Equals("DROPDOWNLIST", StringComparison.OrdinalIgnoreCase))
                                    strbldr_NameList.AppendFormat(" $;{0};$ ", str_ColumnName);

                                int_LookUpColumnIndex++;
                            }
                        }
                    }// If NH Value Xml present
                }
            }            
        }

        private void CreateWorkSheet(ref WorkbookClass excelworkbook_Object, String AddSheetFor, XPathNavigator xpNav_EnittyConfig, XPathNodeIterator xpItr_FieldList, String SheetName,
            String str_NameList)
        {
            #region Member Variable
            Microsoft.Office.Interop.Excel.Range    excelrange_StartCellObject      = null;
            Microsoft.Office.Interop.Excel.Range    excelrange_EndCellObject        = null;
            Microsoft.Office.Interop.Excel.Worksheet excelworksheet_Object          = null;
            Microsoft.Office.Interop.Excel.Range    excelrange_DataValidationObject = null;
            Microsoft.Office.Interop.Excel.Range    excelrange_CellObject           = null;
            String                                  str_PrimaryKey                  = "";
            String                                  str_ColumnName                  = null;
            String                                  str_DisplayValue                = null;
            Int32                                   int_ColumnIterator              = 1;
            XPathNodeIterator                       xpItr_HiddenFields              = null;
            XPathNodeIterator                       xpItr_AutoPopulateFields        = null;
            String                                  str_TableName                   = null;
            String                                  strDataType                     = null;
            //-String                                  Formula                         = null;
            //-String                                  ValidationErrorMessage          = null;            
            String                                  excelColumnName                 = null;
            XPathNavigator                          xpNav_EntityNode                = null;
            String                                  str_IsImportWithoutIDs          = null;
            String                                  str_DummyPKFieldName            = null;
            #endregion

            excelworksheet_Object       = excelworkbook_Object.Sheets.Add(Type.Missing, Type.Missing, Type.Missing, Type.Missing) as Worksheet;
            excelworksheet_Object.Name  = SheetName;

            
            xpItr_HiddenFields          = xpNav_EnittyConfig.Select("ENTITYCONFIGFOREXCEL/IMPORTDETAILS/HIDEENFIELDS/FIELD");
            xpItr_AutoPopulateFields    = xpNav_EnittyConfig.Select("ENTITYCONFIGFOREXCEL/IMPORTDETAILS/AUTOPOPULATEDFIELDS/FIELD");
            str_PrimaryKey              = "";

            xpNav_EntityNode = xpNav_EnittyConfig.SelectSingleNode("ENTITYCONFIGFOREXCEL/IMPORTDETAILS");
            if (xpNav_EntityNode != null)
            {
                str_IsImportWithoutIDs  = xpNav_EntityNode.GetAttribute("IsImportWithoutIDs", "");
                str_DummyPKFieldName    = xpNav_EntityNode.GetAttribute("DummyPKFieldName", "");
            }

            if (xpItr_FieldList != null)
            {
                excelrange_CellObject                   = excelworksheet_Object.get_Range(excelworksheet_Object.Cells[1, 1], excelworksheet_Object.Cells[1, 8]);                
                excelrange_CellObject.Merge(true);
                excelrange_CellObject.Value2            = "\nIMPORTANT: 1)Please do not delete or modify the columns in this excel sheet.\n";
                excelrange_CellObject.Font.Color        = System.Drawing.ColorTranslator.ToOle(System.Drawing.Color.Red);
                excelrange_CellObject.Font.Bold         = true;
                excelrange_CellObject.Cells.RowHeight   = 30;
                
                switch (AddSheetFor.ToUpper())
                { 
                    case "TARGETENTITY":
                        int_ColumnIterator = 1;
                        break;

                    case "RELATEDENTITY":
                    case "RELATEDFACET":
                        //Get Primary Key For TargetEntity
                        if(String.Compare(str_IsImportWithoutIDs, "true",true) != 0)
                        {
                            str_ColumnName      = xpNav_EnittyConfig.SelectSingleNode("ENTITYCONFIGFOREXCEL/SCHEMAXML/ENTITYINFO/TYPE/FIELDCOLLECTION/FIELD[ISPRIMARYKEY='TRUE']/FIELDNAME").Value;
                            str_DisplayValue    = xpNav_EnittyConfig.SelectSingleNode("ENTITYCONFIGFOREXCEL/SCHEMAXML/ENTITYINFO/TYPE/FIELDCOLLECTION/FIELD[ISPRIMARYKEY='TRUE']/DISPLAYNAME").Value;
                        }
                        else
                        {
                            str_ColumnName      = str_DummyPKFieldName;
                            str_DisplayValue    = str_DummyPKFieldName;
                        }

                        excelrange_CellObject           = excelworksheet_Object.Cells[2, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                        excelrange_CellObject.Value2    = str_DisplayValue;
                        excelrange_CellObject.Font.Bold = true;

                        excelrange_CellObject           = excelworksheet_Object.Cells[3, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                        excelrange_CellObject.Value2    = str_ColumnName;
                        excelrange_CellObject.Font.Bold = true;
                        int_ColumnIterator++;
                        break;

                }

                while (xpItr_FieldList.MoveNext())
                {
                    str_ColumnName      = xpItr_FieldList.Current.SelectSingleNode("FIELDNAME").Value;
                    str_DisplayValue    = xpItr_FieldList.Current.SelectSingleNode("DISPLAYNAME").Value;
                    str_TableName       = xpItr_FieldList.Current.SelectSingleNode("TABLENAME").Value;
                    str_PrimaryKey      = xpItr_FieldList.Current.SelectSingleNode("ISPRIMARYKEY").Value;
                    strDataType         = xpItr_FieldList.Current.SelectSingleNode("FIELDTYPE").Value;
                    //-ValidationErrorMessage = null;
                   
                    if(IsAddFieldInSheet(xpItr_HiddenFields,xpItr_AutoPopulateFields, str_ColumnName, str_TableName))
                    {
                        if (String.Compare(str_IsImportWithoutIDs, "true", true) == 0)
                        {
                            if (String.Compare(str_PrimaryKey, "TRUE", true) == 0 && String.Compare(AddSheetFor, "TARGETENTITY", true) == 0)
                            {
                                str_DisplayValue    = str_DummyPKFieldName;
                                str_ColumnName      = str_DummyPKFieldName;
                            }
                        }

                        //For Related entity - Association is done hence only primary key of Target Entity and Related Entity are required
                        if ((String.Compare(str_PrimaryKey, "TRUE",true) == 0 && String.Compare(AddSheetFor, "RELATEDENTITY",true) == 0) 
                            || String.Compare(AddSheetFor, "RELATEDENTITY",true) != 0)
                        {
                            excelrange_CellObject           = excelworksheet_Object.Cells[2, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_CellObject.Value2    = str_DisplayValue;
                            excelrange_CellObject.Font.Bold = true;
                            
                            excelrange_CellObject           = excelworksheet_Object.Cells[3, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_CellObject.Value2    = str_ColumnName;
                            excelrange_CellObject.Font.Bold = true;
                            //excelrange_CellObject.AddComment(str_ColumnName);

                            //Add Validation for DataType and Lookup values
                            excelrange_StartCellObject      = excelworksheet_Object.Cells[3, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_EndCellObject        = excelworksheet_Object.Cells[10001, int_ColumnIterator] as Microsoft.Office.Interop.Excel.Range;
                            excelrange_DataValidationObject = excelworksheet_Object.get_Range(excelrange_StartCellObject, excelrange_EndCellObject);

                            excelColumnName                 = GetStandardExcelColumnName(excelrange_StartCellObject.Column);
                            excelColumnName                 = excelColumnName + excelrange_StartCellObject.Row.ToString();
                            if(String.Compare(AddSheetFor, "RELATEDENTITY",true) != 0)
                            {
                                //TO DO: NESTED VALIDATION FOR NH + RANGE & LENGTH
                                if (str_NameList.Contains(" $;" + str_ColumnName + ";$ "))
                                {                                
                                    excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateList, XlDVAlertStyle.xlValidAlertStop, XlFormatConditionOperator.xlEqual, "=" + str_ColumnName, Type.Missing);
                                }
                                else
                                {
                                    /*
                                    switch (strDataType)
                                    {
                                        case "DATETIME":
                                            excelrange_DataValidationObject.Validation.Add(Microsoft.Office.Interop.Excel.XlDVType.xlValidateDate, Microsoft.Office.Interop.Excel.XlDVAlertStyle.xlValidAlertStop, XlFormatConditionOperator.xlGreaterEqual, DateTime.Parse("1/1/2001"), Type.Missing);
                                            ValidationErrorMessage = ValidationErrorMessage + "Please enter valid Date value.\n";
                                            break;
                                        case "INTEGER":
                                            Formula                 = String.Format("=IF(ISNUMBER({0}),1,0)", excelColumnName); ;
                                            ValidationErrorMessage  = ValidationErrorMessage + "Please enter numeric value.\n";
                                            excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateCustom, XlDVAlertStyle.xlValidAlertStop, Type.Missing, Formula, Type.Missing);                                    
                                            break;
                                        case "FLOAT":
                                            Formula                 = String.Format("=IF(ISNUMBER({0}),1,0)", excelColumnName); ;
                                            ValidationErrorMessage  = ValidationErrorMessage + "Please enter float value.\n";
                                            excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateCustom, XlDVAlertStyle.xlValidAlertStop, Type.Missing, Formula, Type.Missing);                                    
                                            break;
                                            
                                        //case "STRING":
                                        //    String strFieldSize     = xpItr_FieldList.Current.SelectSingleNode("FIELDSIZE").Value;
                                        //    Formula                 = "IF(LEN($COL$)<= " + strFieldSize + ",$NESTEDCONDITION$,0)";
                                        //    ValidationErrorMessage  = ValidationErrorMessage + ":- Length should not exceed beyond " + strFieldSize + " characters.\n";
                                        //    excelrange_DataValidationObject.Validation.Add(XlDVType.xlValidateCustom, XlDVAlertStyle.xlValidAlertStop, Type.Missing, Formula, Type.Missing);
                                        //    break;
                                    }
                                    if (!String.IsNullOrEmpty(ValidationErrorMessage))
                                    {
                                        excelrange_DataValidationObject.Validation.ErrorMessage = ValidationErrorMessage;
                                        excelrange_DataValidationObject.Validation.ErrorTitle   = "Validation Failed";
                                    }
                                    */
                                 }
                              }
                            int_ColumnIterator++;
                        }
                    }
                }
            }
        }

        private bool IsAddSheetForRelation(XPathNodeIterator XpItr_OmittedRelationList, String RelationShipName)
        {
            bool IsAddRelation = true;

            foreach (XPathNavigator xpNav_omittedRel in XpItr_OmittedRelationList)
            {
                if (String.Compare(xpNav_omittedRel.GetAttribute("Name", ""), RelationShipName, true) == 0)
                {
                    IsAddRelation = false;
                    break;
                }
            }
            return IsAddRelation;
        }

        private bool IsAddFieldInSheet(XPathNodeIterator xpItr_HiddenFields,XPathNodeIterator xpItr_AutoPopulateFields, String FieldName, String TableName)
        {
            bool        AddField            = true;
            String      str_ConfigFieldName = null;
            String      str_ConfigTableName = null;

            foreach(XPathNavigator xpNav_HiddenField in xpItr_HiddenFields)
            {
                str_ConfigFieldName = xpNav_HiddenField.GetAttribute("Name", "");
                str_ConfigTableName = xpNav_HiddenField.GetAttribute("TargetObject", "");

                if (String.Compare(str_ConfigFieldName, FieldName, true) == 0 && String.Compare(str_ConfigTableName, TableName, true) == 0)
                {
                    AddField = false;
                    break;
                }
            }
            if (!AddField) return false;

            foreach(XPathNavigator xpNav_AutoField in xpItr_AutoPopulateFields)
            {
                str_ConfigFieldName = xpNav_AutoField.GetAttribute("Name", "");
                str_ConfigTableName = xpNav_AutoField.GetAttribute("TargetObject", "");

                if (String.Compare(str_ConfigFieldName, FieldName, true) == 0 && String.Compare(str_ConfigTableName, TableName, true) == 0)
                {
                    AddField = false;
                    break;
                }
            }
            return AddField;        
        }

        private static string GetStandardExcelColumnName(int columnNumberOneBased)
        {
            int baseValue = Convert.ToInt32('A') - 1;
            string ret = "";

            if (columnNumberOneBased > 26)
            {
                ret = GetStandardExcelColumnName(columnNumberOneBased / 26);
            }

            return ret + Convert.ToChar(baseValue + (columnNumberOneBased % 26));
        }

        private void RemoveExtraSheets(ref WorkbookClass excelworkbook_Object)
        {
            //Detele Extra sheets present in Template
            //((Excel.Worksheet)this.Application.ActiveWorkbook.Sheets[4]).Delete();
            StringCollection ExtraWorksheets = new StringCollection();
            ExtraWorksheets.Add("Sheet1");
            ExtraWorksheets.Add("Sheet2");
            ExtraWorksheets.Add("Sheet3");
            foreach (Worksheet ObjWorkSheet in excelworkbook_Object.Sheets)
            {
                if (ExtraWorksheets.Contains(ObjWorkSheet.Name))
                {
                    ObjWorkSheet.Delete();
                }
            }
        }

        #region XML SpreadSheet related Functions

        private string createXLSFile(string XMLFile, string ExcelFilePath)
        {
            #region Member Variables
            Microsoft.Office.Interop.Excel.ApplicationClass excelapplication_Object = null;
            Workbook excelworkbook_Object = null;
            string strErrorXml = null;           
            #endregion

            try
            {
                excelapplication_Object = new Microsoft.Office.Interop.Excel.ApplicationClass();                
                excelworkbook_Object = excelapplication_Object.Workbooks.Add(ALCommon.GetInstallationDir() + "\\ALOffice2007Template.xls") as WorkbookClass;                

                excelworkbook_Object = excelapplication_Object.Workbooks._Open(XMLFile,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                false,
                Type.Missing,
                Type.Missing,
                false);

                excelworkbook_Object.SaveAs(ExcelFilePath,
                XlFileFormat.xlWorkbookNormal,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                Type.Missing,
                XlSaveAsAccessMode.xlExclusive,
                Type.Missing,
                false,
                Type.Missing,
                Type.Missing,
                Type.Missing);
                excelworkbook_Object.Saved = true;
                File.Delete(XMLFile);                
            }
            catch (Exception ObjException)
            { 
            }
            finally
            {
                #region STEP05: Cleanups- Close the Excel objects
                if (excelworkbook_Object != null)
                {
                    excelworkbook_Object.Close(false, Type.Missing, Type.Missing);
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelworkbook_Object);
                    excelworkbook_Object = null;
                }

                if (excelapplication_Object != null)
                {
                    excelapplication_Object.Quit();
                    System.Runtime.InteropServices.Marshal.ReleaseComObject(excelapplication_Object);
                    excelapplication_Object = null;
                }

                #endregion
            }
            return strErrorXml;
        }

        private ERRORXML printCellData_Xml(String row, String column, String data, String DataType, string StyleID, bool bIsDataHtml, ref StringBuilder sb_OutputXml, XElement xEle_TempNode)
        {
            return printCellData_Xml(row, column, data, DataType, StyleID, bIsDataHtml, ref sb_OutputXml, xEle_TempNode, true);
        }

        private ERRORXML printCellData_Xml(String row, String column, String data, String DataType, string StyleID, bool bIsDataHtml, ref StringBuilder sb_OutputXml, XElement xEle_TempNode, bool OverWriteStyle)
        {
            #region Member Varibales
            String day = null;
            String month = null;
            String year = null;
            String hour = "00";
            String minute = "00";
            String second = "00";
            String Time = null;
            #endregion

            if (String.IsNullOrEmpty(DataType)) DataType = "String";

            switch (DataType.ToUpper())
            {
                case "DATETIME":
                    #region DATETIME
                    if (!string.IsNullOrEmpty(data) || xEle_TempNode != null)
                    {
                        if (xEle_TempNode.Element("DAY") != null)
                            day = xEle_TempNode.Element("DAY").Value;
                        if (xEle_TempNode.Element("MONTH") != null)
                            month = xEle_TempNode.Element("MONTH").Value;
                        if (xEle_TempNode.Element("YEAR") != null)
                            year = xEle_TempNode.Element("YEAR").Value;


                        if (xEle_TempNode.Element("HOUR") != null)
                            hour = xEle_TempNode.Element("HOUR").Value;
                        if (xEle_TempNode.Element("MINUTE") != null)
                            minute = xEle_TempNode.Element("MINUTE").Value;
                        if (xEle_TempNode.Element("SECOND") != null)
                            second = xEle_TempNode.Element("SECOND").Value;

                        Time = hour + ":" + minute + ":" + second + ".000";

                        if (!String.IsNullOrEmpty(day) && !String.IsNullOrEmpty(month) && !String.IsNullOrEmpty(year))
                            data = year + "-" + month + "-" + day + "T" + Time;
                        else
                            data = "";

                        DataType = "DateTime";

                        if (OverWriteStyle)
                            StyleID = "s64";
                    }
                    //-DataType = "String";
                    #endregion
                    break;

                case "BOOLEAN":
                    #region BOOLEAN
                    switch (data)
                    {
                        case "1": data = "Yes";
                            break;
                        case "0": data = "No";
                            break;
                        default:
                            break;
                    }
                    #endregion
                    break;

                case "INTEGER":
                case "FLOAT":
                case "NUMBER":
                    #region INTEGER or FLOAT or NUMBER
                    DataType = "Number";
                    if(OverWriteStyle)
                        StyleID = "s65";
                    #endregion
                    break;

                case "CURRENCY":
                    #region CURRENCY
                    DataType = "Number";
                    if(OverWriteStyle)
                        StyleID = "s65";
                    #endregion
                    break;

                default:
                    DataType = "String";
                    break;
            }

            #region prepare cell node structure
            sb_OutputXml.Append("<Cell ss:Index=\"" + column + "\" ss:StyleID=\"" + StyleID + "\" >");
            if (bIsDataHtml == false)
            {
                data = ALCommon.TrimAndReplaceSpecialChars(data);
                //data = data.Replace("\r\n", "&#10;");
                data = data.Replace("\n", "&#10;");
                if (!string.IsNullOrEmpty(data))
                {
                    if (String.Compare(DataType, "DATETIME", true) == 0 || String.Compare(DataType, "NUMBER", true) == 0)
                    {
                        sb_OutputXml.Append("<Data ss:Type=\"" + DataType + "\">" + data + "</Data>");
                    }
                    else
                    {
                        sb_OutputXml.Append("<Data ss:Type=\"String\">" + data + "</Data>");
                    }
                }
                sb_OutputXml.Append("</Cell>");

            }
            else
            {
                data = data.Replace("\n", "&#10;");
                if (String.Compare(DataType, "DATETIME", true) == 0 || String.Compare(DataType, "NUMBER", true) == 0)
                {
                    sb_OutputXml.Append("<ss:Data ss:Type=\"" + DataType + "\" xmlns=\"http://www.w3.org/TR/REC-html40\">" + data + "</ss:Data></Cell>");
                }
                else
                {
                    sb_OutputXml.Append("<ss:Data ss:Type=\"String\" xmlns=\"http://www.w3.org/TR/REC-html40\">" + data + "</ss:Data></Cell>");
                }
            }
            #endregion
            return null;
        }


        #endregion

        #region Spreadsheet for Consolidated Submission Report

        private String PrepareSubmissionSheetXML(String str_WorkbookPath, String DataXml, String ExcelLayoutNode, String MultiSelectSeparator)
        {
            #region member variables
            StringBuilder sb_SpreadsheetXML = null;
            XPathDocument xpDoc_Data = null;
            XPathNavigator xpNav_Data = null;
            XPathNodeIterator xpIter_VehicleCollection = null;
            XPathNavigator xpNav_DataDoc = null;
            XPathNavigator xpNav_Date = null;
            XPathExpression xpExp_Instance = null;
            XPathNodeIterator xpIter_Instance = null;
            XPathNodeIterator xpIter_SelectionField = null;
            XPathNavigator xpColumnNav = null;
            String StartDate = null;
            String EndDate = null;
            XElement xEle_GroupColl = null;
            Int32 rowStartIdx = 0;
            Int32 rowEndIdx = 0;
            Int32 int_RowNumber = 1;
            IOrderedEnumerable<XElement> GroupByColumnList = null;
            IOrderedEnumerable<XElement> NonGroupByColumnList = null;
            IEnumerable<XElement> totalColumnsList = null;
            XElement xEle_FieldTitle = null;
            XElement xEle_ColWidth = null;
            String str_IntermidiateXMLPath = null;
            Int32 TotalColumnCnt = 0;
            Int32 int_ColumnNumber = 0;
            String str_Width = null;
            String str_DataFieldType = null;
            StringBuilder sb_TmpColumnNode = null;
            String ErrorXml = null;
            Int32 RowEndIndex = 0;
            int colIdx = 1;
            StringBuilder GroupDataTemplate = null;
            String WorkSheetName = null;
            XPathDocument LayoutDoc = null;
            XPathNavigator xpNav_LayoutDocCollection = null;
            XPathNavigator xpNav_LayoutDoc = null;
            XPathNodeIterator xpIter_LayoutNodes = null;
            string SheetName = null;
            string ColumnNumber = null;
            string TemplateType = null;
            int RowPosition = 0;
            String CellDataValue = null;
            string DataType = null;
            #endregion

            try
            {
                sb_SpreadsheetXML = new StringBuilder(1000);
                sb_TmpColumnNode = new StringBuilder(1000);
                GroupDataTemplate = new StringBuilder(1000);
                xpDoc_Data = new XPathDocument(new StringReader(DataXml));
                xpNav_Data = xpDoc_Data.CreateNavigator();                
                rowStartIdx = int_RowNumber;
                rowEndIdx = int_RowNumber;
                //customculture_Object = new CultureInfo(str_CultureInfo);
                str_IntermidiateXMLPath = str_WorkbookPath.Replace(".xls", ".xml");                

                #region Prepare SpreadSheet XML
                sb_SpreadsheetXML.Append(@"<?xml version='1.0'?>
                                                <?mso-application progid='Excel.Sheet'?>
                                                <Workbook xmlns='urn:schemas-microsoft-com:office:spreadsheet'
                                                 xmlns:o='urn:schemas-microsoft-com:office:office'
                                                 xmlns:x='urn:schemas-microsoft-com:office:excel'
                                                 xmlns:ss='urn:schemas-microsoft-com:office:spreadsheet'
                                                 xmlns:html='http://www.w3.org/TR/REC-html40'>
                                                     <DocumentProperties xmlns='urn:schemas-microsoft-com:office:office'>   
                                                        <Author>ASPNET</Author>
		                                                <LastAuthor>ASPNET</LastAuthor>                                                   
                                                     </DocumentProperties>
                                                     <OfficeDocumentSettings xmlns='urn:schemas-microsoft-com:office:office'>
                                                      <AllowPNG/>
                                                      <RemovePersonalInformation/>
                                                     </OfficeDocumentSettings>
                                                     <ExcelWorkbook xmlns='urn:schemas-microsoft-com:office:excel'>
                                                      <WindowHeight>8010</WindowHeight>
                                                      <WindowWidth>14805</WindowWidth>
                                                      <WindowTopX>240</WindowTopX>
                                                      <WindowTopY>105</WindowTopY>
                                                      <ActiveSheet>2</ActiveSheet>
                                                      <ProtectStructure>False</ProtectStructure>
                                                      <ProtectWindows>False</ProtectWindows>
                                                     </ExcelWorkbook>
                                                     <Styles>
                                                          <Style ss:ID='Default' ss:Name='Normal'>
                                                           <Alignment ss:Vertical='Bottom'/>
                                                           <Borders/>
                                                           <Font ss:FontName='Calibri' x:Family='Swiss' ss:Size='11' ss:Color='#000000'/>
                                                           <Interior/>
                                                           <NumberFormat/>
                                                           <Protection/>
                                                          </Style>
                                                          <Style ss:ID='s62'>
                                                           <Alignment ss:Vertical='Top' ss:WrapText='1'/>
                                                          </Style>
                                                          <Style ss:ID='s65'>
                                                           <Alignment ss:Vertical='Top' ss:WrapText='1'/>
                                                           <Font ss:FontName='Calibri' x:Family='Swiss' ss:Size='11' ss:Color='#000000'
                                                            ss:Bold='1'/>
                                                          </Style>
                                                          <Style ss:ID='s66'>
                                                           <Alignment ss:Vertical='Top' ss:WrapText='1'/>
                                                           <NumberFormat ss:Format='Medium Date'/>
                                                          </Style>
                                                          <Style ss:ID='s67'>
                                                           <Alignment ss:Vertical='Top' ss:WrapText='1'/>
                                                           <Font ss:FontName='Calibri' x:Family='Swiss' ss:Size='11' ss:Color='#FF0000'
                                                            ss:Bold='1'/>
                                                          </Style>
                                                     </Styles>");
                //<ss:Names>
                //    <ss:NamedRange/>
                //</ss:Names>");

                xpIter_VehicleCollection = xpNav_Data.Select("SUBMISSION/VEHICLE");

                LayoutDoc = new XPathDocument(new StringReader(ExcelLayoutNode));
                xpNav_LayoutDocCollection = LayoutDoc.CreateNavigator();

                while (xpIter_VehicleCollection.MoveNext())
                {
                    #region Prepare individual sheets for each vehicle

                    xpNav_DataDoc = xpIter_VehicleCollection.Current;

                    SheetName = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/NAME").Value;
                    SheetName = ALCommon.TrimAndReplaceSpecialCharactersForNTFSFileName(SheetName);
                    SheetName += "_" + xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/VEHICLEID").Value;
                    if (SheetName.Length > 30)
                        SheetName = SheetName.Substring(0, 30);

                    RowPosition = 3;
                    sb_SpreadsheetXML.Append("<Worksheet ss:Name=\"" + SheetName + "\">");
                    sb_SpreadsheetXML.Append(@"<Table ss:ExpandedColumnCount='$TOTALCOLUMNCOUNT$' ss:ExpandedRowCount='$TOTALROWCOUNT$' ss:DefaultRowHeight='15'>");

                    #region Prepare Column Header
                    sb_SpreadsheetXML.Append(@"<Column ss:StyleID='s62' ss:AutoFitWidth='0' ss:Width='108.75' ss:Span='32'/>");

                    sb_SpreadsheetXML.Append(@"<Row ss:AutoFitHeight='0' ss:Height='30'>
                                                <Cell ss:MergeAcross='3' ss:StyleID='s67'><Data ss:Type='String'>IMPORTANT: Please do not delete or modify the columns in this excel sheet.&#10;</Data></Cell>
                                               </Row>");


                    xpNav_Date = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/INMKTSTARTDATE/YEAR");
                    if (xpNav_Date != null)
                    {
                        xpNav_Date.MoveToParent();
                        StartDate = ALCommon.FormatDateField(str_FormatXMLPath,
                                        xpNav_Date.SelectSingleNode("DAY").Value,
                                        xpNav_Date.SelectSingleNode("MONTH").Value,
                                        xpNav_Date.SelectSingleNode("YEAR").Value,
                                        customculture_Object, TenantName);
                    }
                    else
                    {
                        StartDate = "";
                    }

                    xpNav_Date = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/INMKTENDDATE/YEAR");
                    if (xpNav_Date != null)
                    {
                        xpNav_Date.MoveToParent();
                        EndDate = ALCommon.FormatDateField(str_FormatXMLPath,
                                        xpNav_Date.SelectSingleNode("DAY").Value,
                                        xpNav_Date.SelectSingleNode("MONTH").Value,
                                        xpNav_Date.SelectSingleNode("YEAR").Value,
                                        customculture_Object, TenantName);
                    }
                    else
                    {
                        EndDate = "";
                    }

                    sb_SpreadsheetXML.AppendFormat(@"<Row ss:Height='45'>
                                                        <Cell ss:StyleID='s65'><Data ss:Type='String'>Vehicle Name: &#10;Start Date: &#10;End Date: </Data></Cell>
                                                        <Cell ss:MergeAcross='1'><Data ss:Type='String'>{0}&#10;{1}&#10;{2}</Data></Cell>
                                                       </Row>", xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/NAME").Value, StartDate, EndDate);
                    sb_SpreadsheetXML.Append("<Row>");

                    TemplateType = xpNav_DataDoc.SelectSingleNode("INSTANCES/VEHICLEINFO/TEMPLATETYPE").Value;
                    xpNav_LayoutDoc = xpNav_LayoutDocCollection.SelectSingleNode("EXCELCONFIG/EXCELLAYOUTXML[@Type = '" + TemplateType + "']");

                    xpIter_LayoutNodes = xpNav_LayoutDoc.Select("DATALAYOUT/FIELDCOLLECTION/FIELD");
                    while (xpIter_LayoutNodes.MoveNext())
                    {
                        ColumnNumber = xpIter_LayoutNodes.Current.GetAttribute("DisplayOrder", "");
                        
                        str_Width = "200";
                        ErrorXml += printCellData_Xml("3", ColumnNumber, xpIter_LayoutNodes.Current.SelectSingleNode("COLUMNNAME").Value, "String", "s65", true, ref sb_SpreadsheetXML, null, false);
                        
                    }

                    sb_SpreadsheetXML.Append("</Row>");                                     
                    #endregion

                    #region Prepare Column Data
                    //CreateExcelDataRows(NonGroupByColumnList, xEle_GroupColl, str_TargetObjectTypeName, colIdx, int_RowNumber, false, GroupDataTemplate, ref RowEndIndex, ref sb_SpreadsheetXML);

                    xpExp_Instance = XPathExpression.Compile("INSTANCES/INSTANCE");
                    xpExp_Instance.AddSort("Order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);
                    xpIter_Instance = xpNav_DataDoc.Select(xpExp_Instance);

                    while (xpIter_Instance.MoveNext())
                    {
                        sb_SpreadsheetXML.Append("<Row>");

                        xpIter_SelectionField = xpNav_LayoutDoc.Select("DATALAYOUT/FIELDCOLLECTION/FIELD");
                        RowPosition++;
                        
                        while (xpIter_SelectionField.MoveNext())
                        {
                            xpColumnNav = xpIter_Instance.Current.SelectSingleNode(xpIter_SelectionField.Current.SelectSingleNode("FIELDNAME").Value);
                            if (xpColumnNav != null)
                            {
                                ColumnNumber = xpIter_SelectionField.Current.GetAttribute("DisplayOrder", "");
                                DataType = xpIter_SelectionField.Current.GetAttribute("DataType", "");
                                if (String.Compare(xpIter_SelectionField.Current.GetAttribute("fieldtype", ""), "MULTISELECT", true) == 0)
                                    CellDataValue = formatCellData("MultiSelect", MultiSelectSeparator, xpColumnNav);
                                else
                                    CellDataValue = xpColumnNav.Value;

                                
                                ErrorXml += printCellData_Xml(RowPosition.ToString(), ColumnNumber, CellDataValue, DataType, "s62", true, ref sb_SpreadsheetXML, null, false);
                                
                            }
                        }
                        sb_SpreadsheetXML.Append("</Row>"); 
                    }
                    #endregion

                    sb_SpreadsheetXML.Append("</Table>");
                    sb_SpreadsheetXML.Append("</Worksheet>");
                    
                    sb_SpreadsheetXML = sb_SpreadsheetXML.Replace("$TOTALCOLUMNCOUNT$", xpIter_LayoutNodes.Count.ToString());
                    sb_SpreadsheetXML = sb_SpreadsheetXML.Replace("$TOTALROWCOUNT$", RowPosition.ToString());
                    #endregion
                }
                sb_SpreadsheetXML.Append("</Workbook>");
                #endregion

                File.WriteAllText(str_IntermidiateXMLPath, sb_SpreadsheetXML.ToString());
                createXLSFile(str_IntermidiateXMLPath, str_WorkbookPath);

            }
            catch (Exception ObjException)
            {
                throw ObjException;
            }
            finally
            {
            }
            return null;
        }

        #endregion
        #endregion
    }
}
