using System;
using System.ServiceProcess;
using System.Security.Cryptography.X509Certificates;
using System.Net.Security;
using System.Net;
using System.Configuration;

using log4net;
using log4net.Config;

namespace ALReportScheduler
{
    public partial class ALReportSchedulerService : ServiceBase
    {
        private static readonly ILog log = LogManager.GetLogger("ReportSchedulerService");
        System.Timers.Timer testTimer = new System.Timers.Timer();
        bool RunModeDebug = false;
        ReportScheduler rs = null;

        public ALReportSchedulerService()
        {
            InitializeComponent();
            log.Info("Report Scheduler started");
        }

        protected override void OnStart(string[] args)
        {
            try
            {
                System.Net.ServicePointManager.ServerCertificateValidationCallback += ServerValidateCertificate;

                testTimer.Enabled = true;
                testTimer.Interval = (int)(1 * 60 * 1000);//1 minute

                try
                {
                    if (ConfigurationManager.AppSettings["RUNMODE"] == "DEBUG")
                        RunModeDebug = true;
                    else
                        RunModeDebug = false;
                }
                catch
                {
                    RunModeDebug = false;
                }

                log.Debug("DEBUG mode = " + RunModeDebug);

                if (RunModeDebug)
                    testTimer.Elapsed += new System.Timers.ElapsedEventHandler(DebugEventTimer_Elapsed);
                else
                    testTimer.Elapsed += new System.Timers.ElapsedEventHandler(NormalEventTimer_Elapsed);
            }
            catch (Exception e)
            {
                log.Error("Could not start Report Scheduler Service. Error: " + e.Message);
            }
        }

        private void NormalEventTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            string NowDateTime;
            NowDateTime = DateTime.Now.ToString("s").Substring(0, 17) + "00";//Get the complete time, but remove seconds, and add seconds as 00

            //Do all our processing here
            try
            {
                //If valid interval, proceed, else return
                if (!IsValidTimerInterval(NowDateTime))
                {
                    log.Debug("Timer Event not processed for " + NowDateTime);
                    return;
                }

                DateTime AdjustedTime = DateTime.Parse(NowDateTime);

                log.Info("Processing Started for Timer Event for interval: " + NowDateTime);

                if(rs == null)
                    rs = new ReportScheduler();
                rs.ProcessReportsForScheduling(AdjustedTime);

                log.Info("Processing Ended for Timer Event for interval: " + NowDateTime);
            }
            catch (Exception ex)
            {
                log.Error("Processing Ended with error " + ex.Message);
            }
            finally
            {
            }
        }

        private void DebugEventTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            string NowDateTime;
            NowDateTime = DateTime.Now.ToString("s").Substring(0, 17) + "00";//Get the complete time, but remove seconds, and add seconds as 00

            //Do all our processing here
            try
            {
                //If valid interval, proceed, else return
                if (RunModeDebug)
                {
                    testTimer.Enabled = false;//for debugging
                }

                DateTime AdjustedTime = DateTime.Parse(NowDateTime);

                log.Info("Processing Started for Timer Event for interval: " + NowDateTime);

                if (rs == null)
                    rs = new ReportScheduler();
                rs.ProcessReportsForScheduling(AdjustedTime);

                log.Info("Processing Ended for Timer Event for interval: " + NowDateTime);
            }
            catch (Exception ex)
            {
                log.Error("Processing Ended with error " + ex.Message);
            }
            finally
            {
                if (RunModeDebug)
                    testTimer.Enabled = true;//for debugging
            }
        }

        /// <summary>
        /// Only process every half an hour at the 30 mins and 00 mins count
        /// </summary>
        /// <param name="NowDateTime"></param>
        /// <returns></returns>
        private bool IsValidTimerInterval(string NowDateTime)
        {
            if (NowDateTime.Substring(14, 2) == "30" || NowDateTime.Substring(14, 2) == "00")
                return true;
            else
                return false;
        }

        bool ServerValidateCertificate(object sender, X509Certificate cert, X509Chain chain, SslPolicyErrors sslError)
        {
            return true;
        }

        protected override void OnStop()
        {
            log.Info("Report Scheduler stopped");
        }

        public class MyPolicy : ICertificatePolicy
        {
            public bool CheckValidationResult(ServicePoint srvPoint, X509Certificate certificate, WebRequest request, int certificateProblem)
            {
                return true;
            } // end CheckValidationResult
        }
    }
}
