using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Reflection;
using System.Data;
using System.Data.SqlClient;
using Teradata.Client.Provider;
using Oracle.DataAccess.Client; 

using System.Configuration;

using Assetlink;
using Assetlink.Framework;
using Assetlink.Services.WindowsService;
using Assetlink.Reporting;

//using Assetlink.WebServices.ALNotificationWS;

using ALReportScheduler.SystemConnectInfrastructure;
//using Assetlink.WebServices.ALSearchWS;

using CrystalDecisions.CrystalReports.Engine;
using CrystalDecisions.Shared;

using log4net;
using log4net.Config;

namespace ALReportScheduler
{
    using XML = System.String;
    using ERRORXML = System.String;

    public class ReportScheduler
    {
        private DateTime _ReportScheduleTimeToCheck;
        private static readonly ILog log = LogManager.GetLogger("ReportScheduler");
        private SystemConnect sc;
        private string InstalledPath;
        private string m_DatabaseType;
        private string m_ConnectionString;
        private string m_ConnectionStringSP;
        ReportDocument oRpt = null;
        private List<String> DelayedSchedulProccesedTenantList;

        public ReportScheduler()
        {
            DelayedSchedulProccesedTenantList = new List<String>();
        }

        public void ProcessReportsForScheduling(DateTime ReportScheduleTimeToCheck)
        {

            InstalledPath = ALCommon.GetInstallationDir();
            _ReportScheduleTimeToCheck = ReportScheduleTimeToCheck;

            try
            {
                string[] AllTenants = getAllTenants();
                if (AllTenants.Length > 0)
                {
                    foreach (string TenantName in AllTenants)
                    {
                        try
                        {
                            string TenantConfigPath = ALCommon.GetTenantXMLPath(TenantName);
                            log.Debug("TenantXML Path = " + TenantConfigPath);

                            XPathDocument XpDoc_TenantXML = null;
                            XPathNavigator XpNav_Tenant = null;

                            XpDoc_TenantXML = new XPathDocument(TenantConfigPath);
                            XpNav_Tenant = XpDoc_TenantXML.CreateNavigator();

                            // Set Class Variables
                            this.m_DatabaseType = XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/DATABASETYPE").Value.ToUpper();
                            this.m_ConnectionString = Assetlink.ALCommon.Decrypt(XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING").Value);
                            this.m_ConnectionStringSP = Assetlink.ALCommon.Decrypt(XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").Value);

                            sc = new SystemConnect();

                            sc.DoSystemConnectForTenant(TenantName);

                            // Steps: 
                            //1. Process the reports to be scheduled at this time
                            //2. check the status of Stampped Email Notification and accordingly change status of Scheduled Instance
                            //3. Delete the Scheduled Instances marked for Deletion

                            ProcessTenantReportsForSchedulesNotExecuted(TenantName);
                            ProcessTenantReportsForScheduling(TenantName);
                            UpdateInstanceStatusForNotification(TenantName);
                            DeleteScehduleInstancesPermanently(TenantName);

                            sc = null;
                        }
                        catch (Exception ex)
                        {
                            log.ErrorFormat("An error occurred while processing tenant {3}. {0} {1} {2}", ex.Message, ex.StackTrace, ex.InnerException, TenantName);
                        }
                    }
                }
                else
                {
                    log.Warn("No tenants found");
                }
            }
            catch (Exception ex)
            {
                log.ErrorFormat("An error occurred while processing tenants. {0} {1} {2}", ex.Message, ex.StackTrace, ex.InnerException);
            }
        }

        

        private void ProcessTenantReportsForScheduling(string TenantName)
        {
            try
            {
                ERRORXML ErrorXml = null;

                //Get all reports that are to be sent out at this time
                XML ReportInstancesToSendNowXML = null;
                log.Info(string.Format("Fetching report instances to be sent at {1} for Tenant {0}", TenantName, _ReportScheduleTimeToCheck.ToString("s")));
                ErrorXml = FetchReportInstancesToBeSentNow(TenantName, ref ReportInstancesToSendNowXML);
                log.Info(string.Format("Fetched report instances for Tenant {0}", TenantName));

                //If there are reports, load the report and all its data
                if (!string.IsNullOrEmpty(ReportInstancesToSendNowXML))
                {
                    log.Debug("ReportInstances Found : " + ReportInstancesToSendNowXML);
                    XmlDocument ScheduledReportDoc = new XmlDocument();

                    ScheduledReportDoc.LoadXml(ReportInstancesToSendNowXML);
                    XmlNodeList ScheduledReportDetails_NodeList = ScheduledReportDoc.SelectNodes("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS");
                    String ScheduledReportId            = null;
                    String ScheduledReportInstanceID    = null;
                    foreach (XmlNode ScheduledReportDetailsNode in ScheduledReportDetails_NodeList)
                    {
                        try
                        {
                            //Get the report details and configuration
                            XmlNode TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("SCHEDREPORTID");
                            ScheduledReportId = TempNode_Details.InnerText;
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("SCHEDREPORTINSTANCENAME");
                            string ScheduledReportName = TempNode_Details.InnerText;
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("EXPORTFORMAT");
                            string ScheduledReportExportFormat = TempNode_Details.InnerText;
                            
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("REPORTDATETYPE");
                            string ReportDateType = TempNode_Details.InnerText.ToUpper();
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("SCHEDTYPE"); // Run Once & Run Between
                            string ScheduleType = TempNode_Details.InnerText.ToUpper();
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("RUNCYCLE"); // daily/monthly/weekly/quarterly
                            string RunCycle = TempNode_Details.InnerText.ToUpper();

                            string ScheduleReportType = null;
                            TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("SCHEDULEREPORTTYPE");
                            if (TempNode_Details != null)
                                ScheduleReportType = TempNode_Details.InnerText.ToUpper();
                            else
                                ScheduleReportType = "ANALYTICALREPORT";


                            string TempFileName             = null;
                            string TempFilePath             = null;
                            string NotificationSubject      = null;
                            string NotificationMessage      = null;
                            ScheduledReportInstanceID       = ScheduledReportDetailsNode.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID").InnerText;

                            switch (ScheduleReportType)
                            {
                                case "ANALYTICALREPORT":
                                    try
                                    {
                                        Dictionary<string, string> ReportInfo = new Dictionary<string, string>();
                                        string WhereClause = null;

                                        ExportFormatType exportFormatType;
                                        string ExportFileExtension = null;

                                        if (ScheduledReportExportFormat.ToUpper() == "EXCEL")
                                        {
                                            exportFormatType = ExportFormatType.Excel;
                                            ExportFileExtension = ".xls";
                                        }
                                        else if (ScheduledReportExportFormat.ToUpper() == "PDF")
                                        {
                                            exportFormatType = ExportFormatType.PortableDocFormat;
                                            ExportFileExtension = ".pdf";
                                        }
                                        else
                                        {
                                            throw new Exception("Export format not supported for ScheduledReport ID " + ScheduledReportId);
                                        }

                                        //Load the report parameters
                                        GetReportParameters(TenantName, ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_ANALYTICALRPT/ANALYTICALREPORT"), ReportInfo);

                                        GetReportFileNameFromFilterValues(ScheduledReportDetailsNode.SelectSingleNode("FILTERVALUES/REPORT"), ReportInfo);

                                        //Create the interface for report data
                                        AbstractReportData AReportDataObject = null;
                                        AReportDataObject = CreateBaseReportDataObject(ReportInfo, TenantName);

                                        if (AReportDataObject == null)
                                            throw new Exception("Report Data object could not be loaded for " + ScheduledReportInstanceID);

                                        string FromDate, ToDate, DisplayDate;
                                        XML StoredProcParams;
                                        GetFromAndToDates(out FromDate, out ToDate, out DisplayDate, ScheduleType, RunCycle, ReportDateType, _ReportScheduleTimeToCheck.ToString("s"));

                                        //log.Debug("\"" + ScheduledReportDetailsNode.SelectSingleNode("FILTERVALUES").InnerXml + "\"");
                                        AReportDataObject.CreateStoredProcParams(ScheduledReportDetailsNode.SelectSingleNode("FILTERVALUES").InnerXml,
                                            FromDate, ToDate, Boolean.Parse(ReportInfo["IncludeDate"]), out StoredProcParams);

                                        //Fill the default report parameters
                                        oRpt = new ReportDocument();

                                        string ReportFilePath = ReportInfo["ReportPath"] + ReportInfo["ReportFileName"];
                                        oRpt.Load(ReportFilePath);
                                        //oRpt.SetDatabaseLogon("", "", "", "");

                                        //Run the stored proc
                                        DataSet ds = ExecuteStoredProcedure(ReportInfo["StoredProcedureName"], WhereClause, ReportInfo["StoredProcedureName"] + "_R", StoredProcParams);

                                        //Create the report
                                        oRpt.SetDataSource(ds.Tables[0]);

                                        AReportDataObject.AddReportParameters(ScheduledReportDetailsNode.SelectSingleNode("FILTERVALUES").InnerXml, ref oRpt);

                                        if (Boolean.Parse(ReportInfo["IncludeDate"]))
                                            oRpt.SetParameterValue("TimePeriod", DisplayDate);
                                        //This needs to become a parameter of AddReportParameters based on config. For now, 
                                        //placing it here

                                        TempFileName = ReportInfo["ReportName"] +
                                                       _ReportScheduleTimeToCheck.ToString("s").Replace(":", "") +
                                                       ExportFileExtension;

                                        TempFilePath = ALCommon.GetTempPath() + TempFileName;

                                        oRpt.ExportToDisk(exportFormatType, TempFilePath);

                                        NotificationSubject = string.Format("<ReportName>{0}</ReportName><TenantName>{1}</TenantName><DisplayDate>{2}</DisplayDate>",
                                                                ScheduledReportName, TenantName, DisplayDate);
                                        NotificationMessage = "MessageInXSLT";
                                    }
                                    catch (Exception ExceptionObj)
                                    {
                                        ErrorXml = "<ERRORCOLLECTION><ERROR><ERRORCODE>ERR_REPORTSCHEDULER_EXPORTFAILED</ERRORCODE><MESSAGE>" + ExceptionObj.Message + "</MESSAGE></ERROR></ERRORCOLLECTION>";
                                        UpdateReportScheduleInstanceForNextRun(ScheduledReportId, ScheduledReportInstanceID, "Error", ErrorXml, TenantName, 0);

                                        log.Info("Error while generating report generated ");
                                        continue;
                                    }

                                    break;

                                case "ADHOCREPORT":
                                        TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("ADHOCREPORT_SCHRPTDETAILS/ADHOCREPORT/REPORTSTATUS");

                                        if (TempNode_Details != null && TempNode_Details.InnerText.ToUpper() != "PUBLISH")
                                            continue;

                                        String AdhocReportID            = null;
                                        String OutPutFileLocation       = ALCommon.GetTempPath();
                                        
                                        String ExportOutPutXml          = null;

                                        TempNode_Details    = ScheduledReportDetailsNode.SelectSingleNode("ADHOCREPORT_SCHRPTDETAILS/ADHOCREPORT/REPORTID");
                                        AdhocReportID       = TempNode_Details.InnerText;
                                        ErrorXml            = ExportAdhocReport(AdhocReportID, ScheduledReportInstanceID, ScheduledReportExportFormat, TenantName, out ExportOutPutXml);

                                        if (ErrorXml != null)
                                        {
                                            ErrorXml = UpdateReportScheduleInstanceForNextRun(ScheduledReportId, ScheduledReportInstanceID, "Error", ErrorXml, TenantName, 0);

                                            StampNotificationErrorReportExecutionFalied(ScheduledReportDetailsNode.SelectSingleNode("ADHOCREPORT_SCHRPTDETAILS/ADHOCREPORT/ADHOCREPORT_CREATEDBY"), 
                                                    ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_OWNER"),
                                                    ScheduledReportDetailsNode.SelectSingleNode("SCHEDREPORTINSTANCENAME"),
                                                    ScheduledReportDetailsNode.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTINSTANCES/INSTANCEDATETIME"), TenantName);
                                            

                                            log.Info("Error while generating report generated ");
                                            continue;
                                        }
                                        else
                                        {
                                            XmlDocument ExportOutPutXmlDoc = new XmlDocument();
                                            ExportOutPutXmlDoc.LoadXml(ExportOutPutXml);
                                            TempFileName = ExportOutPutXmlDoc.SelectSingleNode("FILECOLLECTION/FILE/NAME").InnerText;
                                            TempFilePath = ExportOutPutXmlDoc.SelectSingleNode("FILECOLLECTION/FILE/PATH").InnerText;
                                        }

                                        TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("SUBJECT");
                                        if(TempNode_Details != null && TempNode_Details.InnerText != "")
                                            NotificationSubject = "<ReportSubject>" + TempNode_Details.InnerText + "</ReportSubject>";
                                        else
                                            NotificationSubject = "<ReportName>" + ScheduledReportName + "</ReportName>";

                                        TempNode_Details = ScheduledReportDetailsNode.SelectSingleNode("MESSAGE");
                                        if (TempNode_Details != null && TempNode_Details.InnerText != "")
                                            NotificationMessage = "<ReportMessage>" + TempNode_Details.InnerText + "</ReportMessage>";
                                        else
                                            NotificationMessage = "MessageInXSLT";
                                    break;
                            }


                            log.Info("Report generated successfully at: " + TempFilePath);

                            PushReportFileToServicesMachine(ref TempFilePath, TempFileName, TenantName);
                            log.Info("Report transferred to: " + TempFilePath);

                            //Stamp the notification with the report file as an attachment

                            bool NotifyMe = false;
                            if (ScheduledReportDetailsNode.SelectSingleNode("NOTIFYME")!= null)
                            {
                                if (ScheduledReportDetailsNode.SelectSingleNode("NOTIFYME").InnerText.Equals("1"))
                                    NotifyMe = true;
                            }

                            ulong NotificationID = 0;
                            NotificationID = StampNotification(ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_TOUSER"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_TOGROUP"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_TOEMAILRECIPIENT"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_CCUSER"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_CCGROUP"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_CCEMAILRECIPIENT"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_BCCUSER"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_BCCGROUP"),
                                                ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_BCCEMAILRECIPIENT "),
                                                NotifyMe, ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_OWNER"),
                                                TempFilePath, "6308", NotificationSubject, NotificationMessage, TenantName);

                            log.Info("Notification stamped successfully");

                            #region Handling Last Run & Next Run
                            ErrorXml = UpdateReportScheduleInstanceForNextRun(ScheduledReportId, ScheduledReportInstanceID, "Sending Notification", null, TenantName, NotificationID);
                            #endregion
                        }
                        catch (Exception ExceptionObj)
                        {
                            log.ErrorFormat("ProcessTenantReportsForScheduling: {0} {1} {2} {3}", TenantName, ExceptionObj.Message, ExceptionObj.StackTrace, ExceptionObj.InnerException);
                            if (ScheduledReportId != null && ScheduledReportInstanceID != null)
                            {
                                ErrorXml = "<ERRORCOLLECTION><ERROR><ERRORCODE>ERR_REPORTSCHEDULER_EXPORTFAILED</ERRORCODE><MESSAGE>" + ExceptionObj.Message + "</MESSAGE></ERROR></ERRORCOLLECTION>";
                                UpdateReportScheduleInstanceForNextRun(ScheduledReportId, ScheduledReportInstanceID, "Error", ErrorXml, TenantName, 0);
                            }
                            log.Info("Error while generating report generated ");
                        }
                        finally
                        {
                            if (oRpt != null)
                            {
                                oRpt.Dispose();
                                oRpt.Close();
                                
                                oRpt = null;
                            }

                        }
                    }
                }
            }
            catch (Exception e)
            {
                log.Error(string.Format("ProcessTenantReportsForScheduling for Tenant {0}. Error: {1}.{2}.{3}", TenantName, e.Message, e.StackTrace, e.InnerException));
            }
            finally
            {
            }
        }
        
        private void ProcessTenantReportsForSchedulesNotExecuted(string TenantName)
        {
            try
            {
                if (DelayedSchedulProccesedTenantList.Contains(TenantName))
                    return;

                #region Prepare Search Xml
                StringBuilder sbReportInstancesSearchXML = new StringBuilder(2500);
                sbReportInstancesSearchXML.Append("<SEARCH>");
                sbReportInstancesSearchXML.AppendFormat("<SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />", sc.curSessionID, sc.curUserID, TenantName, sc.curLanguage);
                sbReportInstancesSearchXML.Append("<TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>");
                sbReportInstancesSearchXML.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                sbReportInstancesSearchXML.Append("<TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>");
                sbReportInstancesSearchXML.Append("    <SELECTLIST>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDREPORTID</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDREPORTID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("    </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    <FILTERLIST>");
                sbReportInstancesSearchXML.Append("    <FILTER>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>INSTANCEDATETIME</EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>");
                sbReportInstancesSearchXML.Append("      <OPERATOR>LESSTHAN</OPERATOR>");
                sbReportInstancesSearchXML.Append("      <EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("        <VALUE>");

                sbReportInstancesSearchXML.Append(GetDateInXML(_ReportScheduleTimeToCheck));

                sbReportInstancesSearchXML.Append("        </VALUE>");
                sbReportInstancesSearchXML.Append("      </EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("      <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("    </FILTER>");
                sbReportInstancesSearchXML.Append("    <FILTER>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>");
                sbReportInstancesSearchXML.Append("      <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("      <EXPRESSIONVALUE>PENDING</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("      <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("    </FILTER>");
                sbReportInstancesSearchXML.Append(@"    <FILTER>
									                    <EXPRESSION>MARKEDASDELETED</EXPRESSION>
									                    <OPERATOR>EQUALTO</OPERATOR>
									                    <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
									                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>
								                    </FILTER>");
                sbReportInstancesSearchXML.Append("    </FILTERLIST>");
                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHEDULEDREPORT_INSTANCES\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>INSTANCEDATETIME</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>LESSTHAN</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("            <VALUE>");

                sbReportInstancesSearchXML.Append(GetDateInXML(_ReportScheduleTimeToCheck));

                sbReportInstancesSearchXML.Append("            </VALUE>");
                sbReportInstancesSearchXML.Append("          </EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("         <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("         <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("         <EXPRESSIONVALUE>PENDING</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("         <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append(@"       <FILTER>
									                    <EXPRESSION>MARKEDASDELETED</EXPRESSION>
									                    <OPERATOR>EQUALTO</OPERATOR>
									                    <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
									                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
								                       </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("       <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("       </EXPRESSION>");
                sbReportInstancesSearchXML.Append("       <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>INSTANCEDATETIME</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>INSTANCEDATETIME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("       </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");
                sbReportInstancesSearchXML.Append("</TARGETENTITY></SEARCH>");
                #endregion

                #region Execute Search
                OMInterface omi = new OMInterface();
                omi.CurrentLanguage = sc.curLanguage;
                omi.SessionID = sc.curSessionID;
                omi.UserID = sc.curUserID;
                omi.TenantName = TenantName;

                String ReportInstancesToSendNowXML = null;
                String ErrorXML = omi.GetHierarchialData(sbReportInstancesSearchXML.ToString(), ref ReportInstancesToSendNowXML);
                #endregion

                #region Process Delayed Instances

                if (!string.IsNullOrEmpty(ReportInstancesToSendNowXML))
                {
                    log.Error(string.Format("ProcessTenantReportsForSchedulesNotExecuted for Tenant {0}. Processing Delayed Instances Started.", TenantName));

                    XmlDocument ScheduledReportDoc = new XmlDocument();
                    ScheduledReportDoc.LoadXml(ReportInstancesToSendNowXML);
                    foreach (XmlNode DelayedScheduledReportDetailsNode in ScheduledReportDoc.SelectNodes("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS"))
                    {
                        String ScheduledReportId = null;
                        String ScheduledReportInstanceID = null;
                        String ErrorXml = null;
                        try
                        {
                            ScheduledReportId = DelayedScheduledReportDetailsNode.SelectSingleNode("SCHEDREPORTID").InnerText;
                            ScheduledReportInstanceID = DelayedScheduledReportDetailsNode.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID").InnerText;

                            #region Handling Last Run & Next Run
                            ErrorXml = "<ERRORCOLLECTION><ERROR><ERRORCODE>ERR_REPORTSCHEDULER_NOTEXECUTED</ERRORCODE><MESSAGE>Schedule was not processed due to unknown reason.</MESSAGE></ERROR></ERRORCOLLECTION>";
                            UpdateReportScheduleInstanceForNextRun(ScheduledReportId, ScheduledReportInstanceID, "Error", ErrorXml, TenantName, 0);
                            #endregion
                        }
                        catch (Exception ExceptionObj)
                        {
                            log.Error(string.Format("ProcessTenantReportsForSchedulesNotExecuted for Tenant {0}. Error while Process Schedule {4} Schedule Instance {5} : {1}. {2}. {3}", TenantName, ExceptionObj.Message, ExceptionObj.StackTrace, ExceptionObj.InnerException, ScheduledReportId, ScheduledReportInstanceID));
                        }
                    }
                    log.Error(string.Format("ProcessTenantReportsForSchedulesNotExecuted for Tenant {0}. Processing Delayed Instances Completed.", TenantName));
                }

                DelayedSchedulProccesedTenantList.Add(TenantName);
                #endregion
            }
            catch (Exception e)
            {
                log.Error(string.Format("ProcessTenantReportsForSchedulesNotExecuted for Tenant {0}. Error: {1}.{2}.{3}", TenantName, e.Message, e.StackTrace, e.InnerException));
            }
        }

        private ERRORXML UpdateReportScheduleInstanceForNextRun(String ScheduleDetailsID, String LastExecutedScheduleInstanceID, String ScheduleInstanceStatus, String ErrorXml, String TenantName, ulong NotificationID)
        {
            StringBuilder       ErrorMessage            = null;
            ERRORXML            ErrorXML                = null;
            Int32               WebServiceTimeout       = 3600000;
            String              str_WebServiceTimeout   = null;

            Assetlink.WebServices.ALReportWS.ALReportWS       ReportServiceObj = null;
            Assetlink.WebServices.ALReportWS.ALServiceHeader  ReportReqHeader  = null;

            try
            {
                str_WebServiceTimeout = ConfigurationManager.AppSettings["WebServiceTimeout"];
                if (!String.IsNullOrEmpty(str_WebServiceTimeout))
                    Int32.TryParse(str_WebServiceTimeout, out WebServiceTimeout);

                ReportReqHeader                          = new Assetlink.WebServices.ALReportWS.ALServiceHeader();
                ReportServiceObj                         = new Assetlink.WebServices.ALReportWS.ALReportWS();
                ReportServiceObj.Url                     = ALCommon.GetWebServiceURL("ALReportWS");
                ReportServiceObj.Timeout                 = WebServiceTimeout;
                ReportReqHeader.SessionID                = sc.curSessionID;
                ReportReqHeader.UserID                   = sc.curUserID;
                ReportReqHeader.TenantName               = TenantName;
                ReportReqHeader.Language                 = sc.curLanguage;
                ReportServiceObj.ALServiceHeaderValue    = ReportReqHeader;

                ErrorXML = ReportServiceObj.UpdateReportScheduleInstanceForNextRun(ScheduleDetailsID, LastExecutedScheduleInstanceID, ScheduleInstanceStatus, ErrorXml, NotificationID);

                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    log.Info("An error occured while executing function UpdateReportScheduleInstanceForNextRun. ErrorXml : " + ErrorXml);
                }
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Append("Error occured while executing function UpdateReportScheduleInstanceForNextRun. Exception:" + ExceptionObj.Message);
            }

            return ErrorXML;
        }


        /// <summary>
        /// UpdateInstanceStatusForNotification: this function is used to change the satus of instance either Delivered or Error depending on stampped notification
        /// </summary>
        /// <param name="TenantName"></param>
        private void UpdateInstanceStatusForNotification(String TenantName)
        { 
            StringBuilder       SearchInputXML              = null;
            ERRORXML            ErrorXML                    = null;
            StringBuilder       ErrorMessage                = null;
            XPathDocument       ScheduleInstanceXMLDoc      = null;
            XPathNavigator      RootNodeNav                 = null;
            XPathNodeIterator   InstanceIDNodes             = null;
            XML                 InstanceDetailsXML          = null;
            Int32               WebServiceTimeout           = 3600000;
            String              str_WebServiceTimeout       = null;
            String              SchRptInstanceID            = null;
            XPathNodeIterator   NotificationDiagnosticsItr  = null;
            StringBuilder       UpdateInputXML              = null;
            StringBuilder       ErrorCollectionXML          = null;
            bool                IsError                     = false;

            Assetlink.WebServices.ALSearchWS.Search                         SearchObj       = null;
            Assetlink.WebServices.ALReportWS.ALReportWS                     ReportServiceObj = null;
            Assetlink.WebServices.ALReportWS.ALServiceHeader                ReportReqHeader  = null;


            try
            {
                str_WebServiceTimeout = ConfigurationManager.AppSettings["WebServiceTimeout"];
                if (!String.IsNullOrEmpty(str_WebServiceTimeout))
                    Int32.TryParse(str_WebServiceTimeout, out WebServiceTimeout);

                SearchObj           = new Assetlink.WebServices.ALSearchWS.Search();
                SearchObj.Url       = ConfigurationManager.AppSettings["ALSearchWS"];
                SearchObj.Timeout   = WebServiceTimeout;











                SearchInputXML      = new StringBuilder(1024);
                ErrorMessage        = new StringBuilder(100);

                #region Get  SCHEDULEDREPORTINSTANCES marked for Sending Notification
                SearchInputXML.AppendFormat(@"<SEARCH>
	                                                <TARGETOBJECT>SCHEDULEDREPORTINSTANCES</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}' SessionID='{3}'/> 
	                                                <TARGETENTITY>
                                                        <FILTERLIST>
	                                                        <FILTER>
		                                                        <EXPRESSION>STATUS</EXPRESSION>
		                                                        <OPERATOR>EQUALTO</OPERATOR>
		                                                        <EXPRESSIONVALUE>Sending Notification</EXPRESSIONVALUE>
		                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                        </FILTER>
                                                        </FILTERLIST>
                                                        <SELECTLIST>
			                                                <EXPRESSION>
				                                                <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
			                                                </EXPRESSION>
			                                                <EXPRESSION>
				                                                <FIELDNAME>INSTANCEDATETIME</FIELDNAME>
			                                                </EXPRESSION>
		                                                </SELECTLIST>
                                                        <RELATIONSHIP NAME=""SCHEDULEDREPORT_INSTANCES"">
                                                          <SELECTLIST>
                                                            <EXPRESSION>
                                                              <FIELDNAME>SCHEDREPORTINSTANCENAME</FIELDNAME>
                                                            </EXPRESSION>
                                                          </SELECTLIST>
                                                            <RELATIONSHIP NAME=""SCHRPT_OWNER"">
                                                              <FILTERLIST>
                                                                <FILTER>
                                                                  <FILTERID>1</FILTERID>
                                                                  <EXPRESSION>STATUS</EXPRESSION>
                                                                  <OPERATOR>EQUALTO</OPERATOR>
                                                                  <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>
                                                                  <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                </FILTER>
                                                              </FILTERLIST>
                                                              <SELECTLIST>
                                                                <EXPRESSION>
                                                                  <ALIAS>USERID</ALIAS>
                                                                  <FIELDNAME>USERID</FIELDNAME>
                                                                </EXPRESSION>
                                                              </SELECTLIST>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME=""SCHRPTINSTANCE_NOTIFICATION"">
                                                            <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>NOTIFICATIONID</FIELDNAME>
                                                                </EXPRESSION>   
                                                            </SELECTLIST>
                                                             <RELATIONSHIP NAME=""NOTIFICATION_NOTIFICATIONRECIPIENT"">
                                                                <FILTERLIST>
                                                                    <FILTER>
                                                                      <FILTERID>1</FILTERID>
                                                                      <EXPRESSION>STATUS</EXPRESSION>
                                                                      <OPERATOR>NOTEQUALTO</OPERATOR>
                                                                      <EXPRESSIONVALUE>SENT</EXPRESSIONVALUE>
                                                                      <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                    </FILTER>
                                                               </FILTERLIST>
                                                                <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>NOTIFICATIONRECIPIENTID</FIELDNAME>
                                                                    </EXPRESSION>   
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>STATUS</FIELDNAME>
                                                                    </EXPRESSION>   
                                                                </SELECTLIST>
                                                                <RELATIONSHIP NAME=""NOTIFICATION_DIAGNOSTICS"">
                                                                    <SELECTLIST>
                                                                        <EXPRESSION>
                                                                            <FIELDNAME>NOTIFICATIONDIAGNOSTICS</FIELDNAME>
                                                                        </EXPRESSION>   
                                                                    </SELECTLIST>
                                                            </RELATIONSHIP>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                    </TARGETENTITY>
                                                </SEARCH>", sc.curUserID, TenantName, sc.curLanguage, sc.curSessionID);

                ErrorXML = SearchObj.GetHierarchialData(SearchInputXML.ToString(), ref InstanceDetailsXML);
                if (ErrorXML != null)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.AppendFormat("Retrieving Schedule Instances for stampping notification for Email. Tenant: {0}", TenantName);                    
                    //LogDetails(ErrorMessage.ToString(), ErrorXML, true);
                }

                ScheduleInstanceXMLDoc  = new XPathDocument(new StringReader(InstanceDetailsXML));
                RootNodeNav             = ScheduleInstanceXMLDoc.CreateNavigator();
                if ((RootNodeNav.SelectSingleNode("SCHEDULEDREPORTINSTANCESCOLLECTION/SCHEDULEDREPORTINSTANCES") == null))
                {
                    return;
                }
                #endregion

                #region Check for Error while stampping

                str_WebServiceTimeout = ConfigurationManager.AppSettings["WebServiceTimeout"];
                if (!String.IsNullOrEmpty(str_WebServiceTimeout))
                    Int32.TryParse(str_WebServiceTimeout, out WebServiceTimeout);

                ReportReqHeader                          = new Assetlink.WebServices.ALReportWS.ALServiceHeader();
                ReportServiceObj                         = new Assetlink.WebServices.ALReportWS.ALReportWS();
                ReportServiceObj.Url                     = ALCommon.GetWebServiceURL("ALReportWS");
                ReportServiceObj.Timeout                 = WebServiceTimeout;
                ReportReqHeader.SessionID                = sc.curSessionID;
                ReportReqHeader.UserID                   = sc.curUserID;
                ReportReqHeader.TenantName               = TenantName;
                ReportReqHeader.Language                 = sc.curLanguage;
                ReportServiceObj.ALServiceHeaderValue    = ReportReqHeader;

                InstanceIDNodes     = RootNodeNav.Select("SCHEDULEDREPORTINSTANCESCOLLECTION/SCHEDULEDREPORTINSTANCES");
                UpdateInputXML      = new StringBuilder(100);
                ErrorCollectionXML  = new StringBuilder(100);
                while (InstanceIDNodes.MoveNext()) 
                {
                    IsError          = false;
                    SchRptInstanceID = InstanceIDNodes.Current.SelectSingleNode("SCHEDREPORTINSTANCEID").InnerXml;
                    UpdateInputXML.Remove(0, UpdateInputXML.Length);
                    ErrorCollectionXML.Remove(0, ErrorCollectionXML.Length);
                    // all are SENT
                    if (InstanceIDNodes.Current.Select("SCHRPTINSTANCE_NOTIFICATION/NOTIFICATION/NOTIFICATION_NOTIFICATIONRECIPIENT/NOTIFICATIONRECIPIENT/STATUS").Count == 0)
                    { // STATUS of SCHEDULEDREPORTINSTANCES = Delivered
                        UpdateInputXML.Append("<SCHEDULEDREPORTINSTANCES>");
                        UpdateInputXML.AppendFormat("<SCHEDREPORTINSTANCEID>{0}</SCHEDREPORTINSTANCEID><STATUS>Delivered</STATUS>", SchRptInstanceID);
                        UpdateInputXML.Append("</SCHEDULEDREPORTINSTANCES>");
                    }
                    else
                    {  // either NEW or NOTSENT
                        if (InstanceIDNodes.Current.Select("SCHRPTINSTANCE_NOTIFICATION/NOTIFICATION/NOTIFICATION_NOTIFICATIONRECIPIENT/NOTIFICATIONRECIPIENT/STATUS[text()='NEW']").Count == 0)
                        {
                            NotificationDiagnosticsItr = InstanceIDNodes.Current.Select("SCHRPTINSTANCE_NOTIFICATION/NOTIFICATION/NOTIFICATION_NOTIFICATIONRECIPIENT/NOTIFICATIONRECIPIENT[STATUS[text()='NOTSENT']]/NOTIFICATION_DIAGNOSTICS/NOTIFICATIONDIAGNOSTICS");

                            if (NotificationDiagnosticsItr.Count > 0)
                            {// STATUS of SCHEDULEDREPORTINSTANCES = Error and populate ERRORXML for the same
                                UpdateInputXML.Append("<SCHEDULEDREPORTINSTANCES>");
                                UpdateInputXML.AppendFormat("<SCHEDREPORTINSTANCEID>{0}</SCHEDREPORTINSTANCEID><STATUS>Error</STATUS>", SchRptInstanceID);
                                IsError = true;
                                if (NotificationDiagnosticsItr.Count > 0)
                                {
                                    ErrorCollectionXML.Append("<ERRORCOLLECTION>");
                                    foreach (XPathNavigator NotificationDiagnostic in NotificationDiagnosticsItr)
                                    {
                                        ErrorCollectionXML.AppendFormat("<ERROR><MESSAGE>{0}</MESSAGE></ERROR>", NotificationDiagnostic.SelectSingleNode("NOTIFICATIONDIAGNOSTICS").InnerXml);
                                    }
                                    ErrorCollectionXML.Append("</ERRORCOLLECTION>");
                                    UpdateInputXML.AppendFormat("<ERRORCONFIGURATIONXML>{0}</ERRORCONFIGURATIONXML>", ErrorCollectionXML);
                                }
                                UpdateInputXML.Append("</SCHEDULEDREPORTINSTANCES>");
                            }
                        }
                        else
                        { 
                            // do nothing as there are still instances having STATUS as NEW
                            continue;
                        }
                    }
                    
                    ErrorXML = ReportServiceObj.UpdateReportSchedule(UpdateInputXML.ToString(), false);
                    if (ErrorXML != null)
                    {
                        ErrorMessage.Remove(0, ErrorMessage.Length);
                        ErrorMessage.Append("Error occured while updating STATUS of Schedule instances.");
                        //LogDetails(ErrorMessage.ToString(), ErrorXML, true);
                    }
                    
                    #region Stamp to owner in case of Error
                    if (IsError == true)
                    {
                        StampNotificationErrorReportNotSend(InstanceIDNodes.Current.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTDETAILS/SCHRPT_OWNER"),
                            InstanceIDNodes.Current.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTDETAILS/SCHEDREPORTINSTANCENAME"),
                            InstanceIDNodes.Current.SelectSingleNode("INSTANCEDATETIME"), TenantName);
                    }
                    #endregion
                }
                #endregion
            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.Append("Error occured while deleting Schedule instances. Exception:" + ExceptionObj.Message);
                //LogDetails(ErrorMessage.ToString(), null, true);
            }
        }

        /// <summary>
        /// This function is used to send Notification to Owner as there is error while generating
        /// </summary>
        /// <param name="SchReportOwnerNode"></param>
        /// <param name="SchReportInstanceNode"></param>
        /// <param name="SchReportInstanceTimeNode"></param>
        /// <param name="TenantName"></param>
        private void StampNotificationErrorReportNotSend(XPathNavigator SchReportOwnerNode, XPathNavigator SchReportInstanceNode, XPathNavigator SchReportInstanceTimeNode, String TenantName)
        {
            EmailChannel        oEmailChannel       = null;
            NotificationHelper  oNotificationHelper = null;
            String              ErrorXML            = null;
            Notification        NotificationObj     = null;  
            string Id;

            oEmailChannel = new EmailChannel(PriorityType.NORMAL, false);
            if (SchReportOwnerNode != null)
            {
                Id = SchReportOwnerNode.SelectSingleNode("USER/USERID").InnerXml;
                oEmailChannel.AddRecipient("TO", Id, false);
            }

            oNotificationHelper = new NotificationHelper(sc.curUserID, sc.curSessionID, sc.curLanguage, TenantName);
            oNotificationHelper.Init("6309", sc.curUserID.ToString());
            String Subject = string.Format("<ReportName>{0}</ReportName><TenantName>{1}</TenantName><InstanceTime>{2}</InstanceTime>",
                                     SchReportInstanceNode.InnerXml, TenantName, SchReportInstanceTimeNode.InnerXml);

            String Message = string.Format("<ReportName>{0}</ReportName><TenantName>{1}</TenantName><InstanceTime>{2}</InstanceTime>",
                         SchReportInstanceNode.InnerXml, TenantName, SchReportInstanceTimeNode.InnerXml);

            String NotificationXML = null;
            oNotificationHelper.AddSubject(AddTag(Subject, "SUBJECT"));
            oNotificationHelper.AddMessage(AddTag(Message, "MESSAGE"));

            //add link to notification
            oNotificationHelper.AddLink("SchedulerError", GetErrorPageUrl(TenantName));

            oNotificationHelper.AddChannel(oEmailChannel);
            oNotificationHelper.GetNotificationXML(ref NotificationXML);
            NotificationObj = new Notification(TenantName);
            ErrorXML = NotificationObj.StampNotification(NotificationXML);
            if (!String.IsNullOrEmpty(ErrorXML))
            {
                log.Info("An error occured while executing function StampNotificationErrorReportNotSend. ErrorXml : " + ErrorXML);
            }
        }

        private void StampNotificationErrorReportExecutionFalied(XmlNode SchReportOwnerNode, XmlNode SchReportSecheduleOwnerNode, XmlNode SchReportInstanceNameNode, XmlNode SchReportInstanceTimeNode, String TenantName)
        {
            EmailChannel        oEmailChannel       = null;
            NotificationHelper  oNotificationHelper = null;
            String              ErrorXML            = null;
            Notification        NotificationObj     = null;  
            string Id;

            oEmailChannel = new EmailChannel(PriorityType.NORMAL, false);
            if (SchReportOwnerNode != null)
            {
                Id = SchReportOwnerNode.SelectSingleNode("USER/USERID").InnerXml;
                oEmailChannel.AddRecipient("TO", Id, false);
            }

            if (SchReportSecheduleOwnerNode != null)
            {
                Id = SchReportSecheduleOwnerNode.SelectSingleNode("USER/USERID").InnerXml;
                oEmailChannel.AddRecipient("TO", Id, false);
            }

            oNotificationHelper = new NotificationHelper(sc.curUserID, sc.curSessionID, sc.curLanguage, TenantName);
            oNotificationHelper.Init("6309", sc.curUserID.ToString());
            String Subject = string.Format("<ReportName>{0}</ReportName><TenantName>{1}</TenantName><InstanceTime>{2}</InstanceTime>",
                                     SchReportInstanceNameNode.InnerXml, TenantName, SchReportInstanceTimeNode.InnerXml);

            String Message = string.Format("<ReportName>{0}</ReportName><TenantName>{1}</TenantName><InstanceTime>{2}</InstanceTime>",
                         SchReportInstanceNameNode.InnerXml, TenantName, SchReportInstanceTimeNode.InnerXml);

            String NotificationXML = null;
            oNotificationHelper.AddSubject(AddTag(Subject, "SUBJECT"));
            oNotificationHelper.AddMessage(AddTag(Message, "MESSAGE"));

            //add link to notification
            oNotificationHelper.AddLink("SchedulerError", GetErrorPageUrl(TenantName));

            oNotificationHelper.AddChannel(oEmailChannel);
            oNotificationHelper.GetNotificationXML(ref NotificationXML);
            NotificationObj = new Notification(TenantName);
            ErrorXML        = NotificationObj.StampNotification(NotificationXML);

            if (!String.IsNullOrEmpty(ErrorXML))
            {
                log.Info("An error occured while executing function StampNotificationErrorReportExecutionFalied. ErrorXml : " + ErrorXML);
            }
        }


        private String GetErrorPageUrl(String TenantName)
        {
            # region Local VAriables
            StringBuilder               ErrorPageURL            = null;
            String                      DefaultPageMode         = "EMAIL";
            String                      CommunicationProtocol   = null;
            String                      InternetAddress         = null;        
            StringBuilder               EmailPageParams         = new StringBuilder();
            StringBuilder               SolutionPageUrl         = null;
            #endregion

            CommunicationProtocol   = ALCommon.GetCommunicationProtocol();
            InternetAddress         = ALCommon.GetInternetAddress();
            ErrorPageURL            = new StringBuilder();
            SolutionPageUrl         = new StringBuilder();

            SolutionPageUrl.AppendFormat("{0}{1}/{2}/Reports/Aspx/Report_View.aspx", CommunicationProtocol, InternetAddress, TenantName);
            EmailPageParams.Append("&ReportCategory=ALL&ReportSection=Reports - All");

            ErrorPageURL.AppendFormat("{0}{1}/{2}/default.aspx?", CommunicationProtocol, InternetAddress, TenantName);
            ErrorPageURL.AppendFormat("Language={0}", sc.curLanguage);
            ErrorPageURL.AppendFormat("&Mode={0}", DefaultPageMode);
            ErrorPageURL.AppendFormat("&Imp=N");

            ErrorPageURL.AppendFormat("&SolutionPage={0}", SolutionPageUrl.ToString());
            ErrorPageURL.AppendFormat("&Param={0}", ALCommon.Encrypt(EmailPageParams.ToString()));
            ErrorPageURL = ErrorPageURL.Replace("&", "%26");

            return ErrorPageURL.ToString();
        }

        /// <summary>
        /// DeleteScehduleInstancesPermanently : this is used to delete all the instances marked for scheduled 
        /// </summary>
        /// <param name="TenantName"></param>
        private void DeleteScehduleInstancesPermanently(String TenantName)
        {
            StringBuilder       SearchInputXML          = null;
            ERRORXML            ErrorXML                = null;
            StringBuilder       ErrorMessage            = null;
            Int32               WebServiceTimeout       = 3600000;
            String              str_WebServiceTimeout   = null;

            Assetlink.WebServices.ALReportWS.ALReportWS         ReportServiceObj = null;
            Assetlink.WebServices.ALReportWS.ALServiceHeader    ReportReqHeader  = null;

            try
            {
                str_WebServiceTimeout = ConfigurationManager.AppSettings["WebServiceTimeout"];
                if (!String.IsNullOrEmpty(str_WebServiceTimeout))
                    Int32.TryParse(str_WebServiceTimeout, out WebServiceTimeout);

                ReportReqHeader                         = new Assetlink.WebServices.ALReportWS.ALServiceHeader();
                ReportServiceObj                        = new Assetlink.WebServices.ALReportWS.ALReportWS();
                ReportServiceObj.Url                    = ALCommon.GetWebServiceURL("ALReportWS");
                ReportServiceObj.Timeout                = WebServiceTimeout;
                ReportReqHeader.SessionID               = sc.curSessionID;
                ReportReqHeader.UserID                  = sc.curUserID;
                ReportReqHeader.TenantName              = TenantName;
                ReportReqHeader.Language                = sc.curLanguage;
                ReportServiceObj.ALServiceHeaderValue   = ReportReqHeader;

                SearchInputXML      = new StringBuilder(1024);
                ErrorMessage        = new StringBuilder(100);
               
                ErrorXML = ReportServiceObj.DeleteScheduleReportInstanceDetails();
                if (ErrorXML != null)
                {
                    ErrorMessage.Remove(0, ErrorMessage.Length);
                    ErrorMessage.Append("Error occured while deleting Schedule instances.");
                    //LogDetails(ErrorMessage.ToString(), ErrorXML, true);
                }


            }
            catch (Exception ExceptionObj)
            {
                ErrorMessage.Remove(0, ErrorMessage.Length);
                ErrorMessage.Append("Error occured while deleting Schedule instances. Exception:" + ExceptionObj.Message);
                //LogDetails(ErrorMessage.ToString(), null, true);
            }
        }

        private void PushReportFileToServicesMachine(ref string TempFilePath, string TempFileName, string TenantName)
        {
            StringBuilder FileCollectionXML = new StringBuilder(500);
            String FileListXML = null;

            FileCollectionXML.Append("<FILECOLLECTION>");

            FileCollectionXML.Append("<FILE>");

            FileCollectionXML.Append("<PATH>");
            FileCollectionXML.Append(TempFilePath);
            FileCollectionXML.Append("</PATH>");

            FileCollectionXML.Append("<NAME>");
            FileCollectionXML.Append(TempFileName);
            FileCollectionXML.Append("</NAME>");

            FileCollectionXML.Append("</FILE>");
            
            FileCollectionXML.AppendFormat("</FILECOLLECTION>");
            FileListXML = FileCollectionXML.ToString();

            FileTransferInfrastructure fti = new FileTransferInfrastructure(sc.curSessionID.ToString(), sc.curUserID.ToString(), TenantName, sc.curLanguage);
            string Error;
            try
            {

                fti.FileTransferServiceName = ConfigurationManager.AppSettings["ALMTOMWS"].ToString();

                Error = fti.SaveFile(ref FileListXML);

                if (!string.IsNullOrEmpty(Error))
                {
                    throw new Exception(Error);
                }
                else
                {
                    XmlDocument FileDoc = new XmlDocument();
                    FileDoc.LoadXml(FileListXML);

                    XmlNode xnode = FileDoc.SelectSingleNode("FILECOLLECTION/FILE/NEWPATH");

                    TempFilePath = xnode.InnerText;

                    //log.Debug("File Transfer XML " + FileListXML);
                }
            }
            catch
            {
                throw;
            }
        }

        #region Database related methods
        private DataSet ExecuteStoredProcedure(string spName, string WhereClause, string GlobalTempTableName, XML StoredProcParams)
        {
            #region Local Variables
            SqlConnection SQLConnObj = null;
            SqlCommand SQLCmdObj = null;
            SqlDataAdapter SQLDataAdapterObj = null;
            TdConnection TdConnection = null;
            TdCommand TdCCommandObj = null;
            TdDataAdapter TdDataAdapter = null;
            TdCommand TdCCommandforRead = null;
            OracleConnection OracleConnObj = null;
            OracleCommand OracleCmdObj = null;
            OracleCommand OracleCmdforRead = null;
            OracleDataAdapter OrclDataAdapter = null;
            DataSet dsProject = null;
            #endregion

            try
            {
                switch (this.m_DatabaseType)
                {
                    case "SQLEXPRESS":
                    case "SQLSERVER":
                        SQLConnObj = new System.Data.SqlClient.SqlConnection(this.m_ConnectionString);
                        SQLCmdObj = new System.Data.SqlClient.SqlCommand();
                        dsProject = new DataSet();
                        SQLCmdObj.Connection = SQLConnObj;
                        SQLCmdObj.CommandType = CommandType.StoredProcedure;
                        SQLCmdObj.CommandText = spName;
                        SQLCmdObj.CommandTimeout = 300;

                        // Assign Dynamic Parameters

                        if (!string.IsNullOrEmpty(WhereClause))
                        {
                            SQLCmdObj.Parameters.AddWithValue("@WHERE", WhereClause);
                        }

                        AssignDynamicParamsforSQL(StoredProcParams, ref SQLCmdObj);

                        SQLDataAdapterObj = new SqlDataAdapter((SqlCommand)SQLCmdObj);
                        SQLDataAdapterObj.Fill(dsProject);

                        SQLCmdObj.Dispose();
                        SQLConnObj.Close();
                        SQLConnObj.Dispose();
                        break;
                    case "TERADATA":
                        TdConnection = new TdConnection(this.m_ConnectionStringSP);
                        TdConnection.Open();
                        TdCCommandObj = new TdCommand();
                        dsProject = new DataSet();
                        TdCCommandObj.Connection = TdConnection;
                        TdCCommandObj.CommandType = CommandType.StoredProcedure;
                        TdCCommandObj.CommandText = spName;
                        TdCCommandObj.CommandTimeout = 300;
                        
                        // Assign Dynamic Parameters
                        if (!string.IsNullOrEmpty(WhereClause))
                        {
                            TdParameter ParamObj = TdCCommandObj.CreateParameter();

                            string strName = "WHERE";
                            ParamObj.ParameterName = strName.ToUpper();
                            ParamObj.Value = WhereClause;
                            ParamObj.TdType = TdType.VarChar;
                            ParamObj.Direction = ParameterDirection.Input;
                            TdCCommandObj.Parameters.Add(ParamObj);
                        }

                        AssignDynamicParamsforTD(StoredProcParams, ref TdCCommandObj);

                        TdCCommandObj.ExecuteNonQuery();

                        TdCCommandforRead = new TdCommand();
                        TdCCommandforRead.Connection = TdConnection;
                        TdCCommandforRead.CommandType = CommandType.Text;
                        TdCCommandforRead.CommandText = " SELECT * FROM " + GlobalTempTableName;
                        TdCCommandforRead.CommandTimeout = 300;
                        TdDataAdapter = new TdDataAdapter((TdCommand)TdCCommandforRead);
                        TdDataAdapter.Fill(dsProject);

                        TdCCommandObj.Dispose();
                        TdCCommandforRead.Dispose();
                        TdConnection.Close();
                        TdConnection.Dispose();
                        break;
                    case "ORACLE":
                    case "ORACLEEXPRESS":
                        OracleConnObj = new OracleConnection(this.m_ConnectionStringSP);
                        OracleConnObj.Open();
                        OracleCmdObj = new OracleCommand();
                        dsProject = new DataSet();
                        OracleCmdObj.Connection = OracleConnObj;
                        OracleCmdObj.CommandType = CommandType.StoredProcedure;
                        OracleCmdObj.CommandText = spName;
                        OracleCmdObj.CommandTimeout = 300;

                        // Assign Dynamic Parameters
                        if (!string.IsNullOrEmpty(WhereClause))
                        {
                            OracleParameter ParamObj = OracleCmdObj.CreateParameter();

                            string strName = "WHERE";
                            ParamObj.ParameterName = strName.ToUpper();
                            //check for to_timestamp function
                            if (WhereClause.Contains("to_timestamp"))
                            {
                                WhereClause = WhereClause.Replace("'to_timestamp", "to_timestamp");
                                WhereClause = WhereClause.Replace("DD-MON-YY HH24.MI.SSXFF')'", "DD-MON-YY HH24.MI.SSXFF')");
                            }
                            ParamObj.Value = WhereClause;
                            ParamObj.OracleDbType = OracleDbType.NVarchar2;
                            ParamObj.Direction = ParameterDirection.Input;
                            OracleCmdObj.Parameters.Add(ParamObj);
                        }

                        AssignDynamicParamsforOracle(StoredProcParams, ref OracleCmdObj);

                        OracleCmdObj.ExecuteNonQuery();

                        OracleCmdforRead = new OracleCommand();
                        OracleCmdforRead.Connection = OracleConnObj;
                        OracleCmdforRead.CommandType = CommandType.Text;
                        OracleCmdforRead.CommandText = " SELECT * FROM " + GlobalTempTableName;
                        OracleCmdforRead.CommandTimeout = 300;
                        OrclDataAdapter = new OracleDataAdapter((OracleCommand)OracleCmdforRead);
                        OrclDataAdapter.Fill(dsProject);

                        OracleCmdObj.Dispose();
                        OracleCmdforRead.Dispose();
                        OracleConnObj.Close();
                        OracleConnObj.Dispose();
                        break;
                }

                return dsProject;
            }
            catch
            {
                throw;
            }
        }

        private void AssignDynamicParamsforTD(XML SPDynamicParams, ref TdCommand TdCCommandObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;
                        if (!String.IsNullOrEmpty(strName))
                        {
                            TdParameter ParamObj = TdCCommandObj.CreateParameter();
                            if (strName.Contains("@"))
                                strName = strName.Replace("@", "");
                            ParamObj.ParameterName = strName.ToUpper();
                            ParamObj.Value = strVal;
                            ParamObj.TdType = TdType.VarChar;
                            ParamObj.Direction = ParameterDirection.Input;
                            TdCCommandObj.Parameters.Add(ParamObj);
                        }
                    }
                }
            }
            catch
            {
                throw;
            }
        }

        private void AssignDynamicParamsforOracle(XML SPDynamicParams, ref OracleCommand OracleCmdObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;
                        if (!String.IsNullOrEmpty(strName))
                        {
                            OracleParameter ParamObj = OracleCmdObj.CreateParameter();
                            if (strName.Contains("@"))
                                strName = strName.Replace("@", "");
                            ParamObj.ParameterName = strName.ToUpper();
                            if (strVal.Contains("to_timestamp"))
                            {
                                strVal = strVal.Replace("'to_timestamp", "to_timestamp");
                                strVal = strVal.Replace("DD-MON-YY HH24.MI.SSXFF')'", "DD-MON-YY HH24.MI.SSXFF')");
                            }
                            ParamObj.Value = strVal;
                            ParamObj.OracleDbType = OracleDbType.NVarchar2;
                            ParamObj.Direction = ParameterDirection.Input;
                            OracleCmdObj.Parameters.Add(ParamObj);
                        }
                    }
                }
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="SPDynamicParams"></param>
        /// <param name="SQLCmdObj"></param>
        private void AssignDynamicParamsforSQL(XML SPDynamicParams, ref SqlCommand SQLCmdObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;
                        SQLCmdObj.Parameters.AddWithValue(strName, strVal);
                    }
                }
            }
            catch
            {
                throw;
            }
        }
        #endregion

        private void GetReportParameters(string TenantName, XmlNode xmlNode, Dictionary<string, string> ReportInfo)
        {
            bool IncludeDate = false;
            try
            {
                ReportInfo.Add("Tenant", TenantName);
                ReportInfo.Add("ReportId", xmlNode.SelectSingleNode("REPORTID").InnerText);
                ReportInfo.Add("ReportName", xmlNode.SelectSingleNode("REPORTNAME").InnerText);
                ReportInfo.Add("ReportFileName", xmlNode.SelectSingleNode("REPORTFILENAME").InnerText);

                //Open the report configuration and get the addl. details
                string ReportPath = GetReportPath(TenantName, ReportInfo["ReportName"]);
                ReportInfo.Add("ReportPath", ReportPath);

                ReportInfo.Add("StoredProcedureName", xmlNode.SelectSingleNode("STOREDPROCEDURENAME").InnerText);
                ReportInfo.Add("ReportDataAssemblyName", xmlNode.SelectSingleNode("REPORTDATAASSEMBLYNAME").InnerText);
                ReportInfo.Add("ReportDataClassName", xmlNode.SelectSingleNode("REPORTDATACLASSNAME").InnerText);

                if (xmlNode.SelectSingleNode("INCLUDEDATE") != null)
                {
                    if (xmlNode.SelectSingleNode("INCLUDEDATE").InnerText == "1")
                        IncludeDate = true;
                }

                ReportInfo.Add("IncludeDate", IncludeDate.ToString());
            }
            catch (Exception e)
            {
                ReportInfo = null;

                throw new Exception("Report could not be configured " + e.Message + e.StackTrace + e.InnerException);
            }
            finally
            {
            }
        }


        private ulong StampNotification(XmlNode ReceivingToUsersNode, XmlNode ReceivingToGroupsNode, XmlNode EmailToRecipientsNode,
                                       XmlNode ReceivingCcUsersNode, XmlNode ReceivingCcGroupsNode, XmlNode EmailCcRecipientsNode,
                                       XmlNode ReceivingBccUsersNode, XmlNode ReceivingBccGroupsNode, XmlNode EmailBccRecipientsNode, 
                                       bool NotifyMe, XmlNode SchReportOwnerNode,
                                       string ReportOutputFile, string NotificationTypeId, string MailSubject, string MailMessage, string TenantName)
        {
            EmailChannel        oEmailChannel           = null;
            NotificationHelper  oNotificationHelper     = null;
            XML                 NotificationXML         = null;
            ulong               NotificationID          = 0;
            ERRORXML            ErrorXML                = null;
            StringBuilder       sbErrorXML              = new StringBuilder(500);
            try
            {
                CreateEmailChannel(ref oEmailChannel, ReceivingToUsersNode, ReceivingToGroupsNode, EmailToRecipientsNode,
                                       ReceivingCcUsersNode, ReceivingCcGroupsNode, EmailCcRecipientsNode,
                                       ReceivingBccUsersNode, ReceivingBccGroupsNode, EmailBccRecipientsNode,
                                       NotifyMe, SchReportOwnerNode);

                oNotificationHelper = new NotificationHelper(sc.curUserID, sc.curSessionID, sc.curLanguage, TenantName);
                oNotificationHelper.Init(NotificationTypeId, sc.curUserID.ToString());
                ErrorXML = oNotificationHelper.AddSubject(AddTag(MailSubject, "SUBJECT"));

                if (!string.IsNullOrEmpty(ErrorXML))
                {
                    sbErrorXML.Append(ErrorXML);
                    log.Debug(ErrorXML);
                }
                
                ErrorXML = oNotificationHelper.AddMessage(AddTag(MailMessage, "MESSAGE"));
                if (!string.IsNullOrEmpty(ErrorXML))
                {
                    sbErrorXML.Append(ErrorXML);
                    log.Debug(ErrorXML);
                }

                ErrorXML = oNotificationHelper.AddChannel(oEmailChannel);
                if (!string.IsNullOrEmpty(ErrorXML))
                {
                    sbErrorXML.Append(ErrorXML);
                    log.Debug(ErrorXML);
                }

                //The below code does not work correctly, hence doing it the MANUAL way
                //ErrorXML = oNotificationHelper.AddAttachment(ReportOutputFile);
                //if (!string.IsNullOrEmpty(ErrorXML))
                //{
                //    sbErrorXML.Append(ErrorXML);
                //    log.Debug("AddAttachment Error: " + ErrorXML);
                //}

                ErrorXML = oNotificationHelper.GetNotificationXML(ref NotificationXML);
                if (!string.IsNullOrEmpty(ErrorXML))
                {
                    sbErrorXML.Append(ErrorXML);
                    log.Debug(ErrorXML);
                }
                log.Debug(NotificationXML);

                StringBuilder sb = new StringBuilder(NotificationXML.Length + 200);
                sb.Append(NotificationXML);
                sb = sb.Replace("</NOTIFICATION>", "");
                sb.Append("<FILECOLLECTION>");
                sb.Append("<FILE>");
                sb.Append(ReportOutputFile);
                sb.Append("</FILE>");
                sb.Append("</FILECOLLECTION>");
                UniqueID.Read(ref NotificationID);
                sb.AppendFormat("<NOTIFICATIONID>{0}</NOTIFICATIONID>", NotificationID);
                sb.Append("</NOTIFICATION>");

                

                Notification n = new Notification(TenantName);

                NotificationXML = sb.ToString();
                ErrorXML = n.StampNotification(NotificationXML);
                if (!string.IsNullOrEmpty(ErrorXML))
                {
                    sbErrorXML.Append(ErrorXML);
                    log.Debug(ErrorXML);
                }

                if (!string.IsNullOrEmpty(sbErrorXML.ToString()))
                {
                    throw new Exception(sbErrorXML.ToString());
                }

                log.Debug("Notification Details: " + NotificationXML);

                NotificationXML = null;
                sb.Remove(0, sb.Length);
                sb = null;
            }
            catch
            {
                throw;
            }
            return NotificationID;
        }

        /// <summary>
        /// Handle the notification sender and receivers. Receivers are users, groups and open email addresses
        /// </summary>
        /// <param name="obj_EmailChannel"></param>
        /// <param name="ReceivingUsersNode"></param>
        /// <param name="ReceivingGroupsNode"></param>
        /// <param name="EmailRecipientsNode"></param>
        private void CreateEmailChannel(ref EmailChannel obj_EmailChannel, XmlNode ReceivingToUsersNode,XmlNode ReceivingToGroupsNode, XmlNode EmailToRecipientsNode,
                                            XmlNode ReceivingCcUsersNode, XmlNode ReceivingCcGroupsNode, XmlNode EmailCcRecipientsNode,
                                            XmlNode ReceivingBccUsersNode, XmlNode ReceivingBccGroupsNode, XmlNode EmailBccRecipientsNode,
                                            bool NotifyMe, XmlNode SchReportOwnerNode)
        {
            obj_EmailChannel = new EmailChannel(PriorityType.NORMAL, false);
            //obj_EmailChannel.AddRecipient("FROM", sc.curUserID.ToString(), false);
            string Id;
            string EmailAddress;
            XmlNodeList nl;

            #region Add "TO"
            if (ReceivingToUsersNode != null)
            {
                nl = ReceivingToUsersNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("USERID").InnerText;
                    obj_EmailChannel.AddRecipient("TO", Id, false);
                }
            }

            if (ReceivingToGroupsNode != null)
            {
                nl = ReceivingToGroupsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("GROUPID").InnerText;
                    obj_EmailChannel.AddRecipient("TO", Id, true);
                }
            }

            if (EmailToRecipientsNode != null)
            {
                nl = EmailToRecipientsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    EmailAddress = n.SelectSingleNode("EMAILID").InnerText;
                    obj_EmailChannel.AddRecipient("TO", EmailAddress);


                }
            }
            #endregion

            #region Add "CC"
            if (ReceivingCcUsersNode != null)
            {
                nl = ReceivingCcUsersNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("USERID").InnerText;
                    obj_EmailChannel.AddRecipient("Cc", Id, false);
                }
            }

            if (ReceivingCcGroupsNode != null)
            {
                nl = ReceivingCcGroupsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("GROUPID").InnerText;
                    obj_EmailChannel.AddRecipient("Cc", Id, true);
                }
            }

            if (EmailCcRecipientsNode != null)
            {
                nl = EmailCcRecipientsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    EmailAddress = n.SelectSingleNode("EMAILID").InnerText;
                    obj_EmailChannel.AddRecipient("Cc", EmailAddress);


                }
            }
            
            if (NotifyMe) // if NotifyMe is true then CC to the Report Owner
            {
                Id = SchReportOwnerNode.SelectSingleNode("USER/USERID").InnerText;
                obj_EmailChannel.AddRecipient("Cc", Id, false);
            }
            #endregion

            #region Add "BCC"
            if (ReceivingBccUsersNode != null)
            {
                nl = ReceivingBccUsersNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("USERID").InnerText;
                    obj_EmailChannel.AddRecipient("Bcc", Id, false);
                }
            }

            if (ReceivingBccGroupsNode != null)
            {
                nl = ReceivingBccGroupsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    Id = n.SelectSingleNode("GROUPID").InnerText;
                    obj_EmailChannel.AddRecipient("Bcc", Id, true);
                }
            }

            if (EmailBccRecipientsNode != null)
            {
                nl = EmailBccRecipientsNode.ChildNodes;
                foreach (XmlNode n in nl)
                {
                    EmailAddress = n.SelectSingleNode("EMAILID").InnerText;
                    obj_EmailChannel.AddRecipient("Bcc", EmailAddress);


                }
            }
            #endregion
        }


        private ERRORXML FetchReportInstancesToBeSentNow(string TenantName, ref string ReportInstancesToSendNowXML)
        {
            ERRORXML ErrorXML;
            string ReportInstancesSearchXML;

            ReportInstancesSearchXML = CreateReportInstancesSearchXML(TenantName);

            OMInterface omi = new OMInterface();

            omi.CurrentLanguage = sc.curLanguage;
            omi.SessionID = sc.curSessionID;
            omi.UserID = sc.curUserID;
            omi.TenantName = TenantName;
            
            ErrorXML = omi.GetHierarchialData(ReportInstancesSearchXML, ref ReportInstancesToSendNowXML);
            if (!(String.IsNullOrEmpty(ErrorXML)))
                throw new Exception(string.Format("An error occurred while getting report instances for tenant {0}. Error {1}", TenantName, ErrorXML));

            if (!ReportInstancesToSendNowXML.Contains("SCHEDULEDREPORTINSTANCES"))
                ReportInstancesToSendNowXML = null;

            return null;
        }

        private string CreateReportInstancesSearchXML(string TenantName)
        {
            StringBuilder sbReportInstancesSearchXML = new StringBuilder(2500);

            try
            {
                sbReportInstancesSearchXML.Append("<SEARCH>");
                sbReportInstancesSearchXML.AppendFormat("<SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />", sc.curSessionID, sc.curUserID, TenantName, sc.curLanguage);
                sbReportInstancesSearchXML.Append("<TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>");
                sbReportInstancesSearchXML.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                sbReportInstancesSearchXML.Append("<TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>");

                sbReportInstancesSearchXML.Append("    <SELECTLIST>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDREPORTID</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDREPORTID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDREPORTINSTANCENAME</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDREPORTINSTANCENAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>FILTERVALUES</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>FILTERVALUES</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>EXPORTFORMAT</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>EXPORTFORMAT</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>REPORTDATETYPE</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>REPORTDATETYPE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDTYPE</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDTYPE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>RUNCYCLE</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>RUNCYCLE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");

                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>MESSAGE</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>MESSAGE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SUBJECT</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SUBJECT</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>SCHEDULEREPORTTYPE</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>SCHEDULEREPORTTYPE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");

                sbReportInstancesSearchXML.Append("      <EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <ALIAS>NOTIFYME</ALIAS>");
                sbReportInstancesSearchXML.Append("        <FIELDNAME>NOTIFYME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("      </EXPRESSION>");
                sbReportInstancesSearchXML.Append("    </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    <FILTERLIST>");
                sbReportInstancesSearchXML.Append("    <FILTER>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>INSTANCEDATETIME</EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>");
                sbReportInstancesSearchXML.Append("      <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("      <EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("        <VALUE>");

                sbReportInstancesSearchXML.Append(GetDateInXML(_ReportScheduleTimeToCheck));

                sbReportInstancesSearchXML.Append("        </VALUE>");
                sbReportInstancesSearchXML.Append("      </EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("      <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("    </FILTER>");
                sbReportInstancesSearchXML.Append("    <FILTER>");
                sbReportInstancesSearchXML.Append("      <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("      <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>");
                sbReportInstancesSearchXML.Append("      <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("      <EXPRESSIONVALUE>PENDING</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("      <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("    </FILTER>");
                sbReportInstancesSearchXML.Append(@"    <FILTER>
										                    <EXPRESSION>MARKEDASDELETED</EXPRESSION>
										                    <OPERATOR>EQUALTO</OPERATOR>
										                    <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
										                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>
									                    </FILTER>");
                sbReportInstancesSearchXML.Append("    </FILTERLIST>");
                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHEDULEDREPORT_INSTANCES\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>INSTANCEDATETIME</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("            <VALUE>");

                sbReportInstancesSearchXML.Append(GetDateInXML(_ReportScheduleTimeToCheck));

                sbReportInstancesSearchXML.Append("            </VALUE>");
                sbReportInstancesSearchXML.Append("          </EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("         <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("         <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("         <EXPRESSIONVALUE>PENDING</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("         <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append(@"       <FILTER>
										                    <EXPRESSION>MARKEDASDELETED</EXPRESSION>
										                    <OPERATOR>EQUALTO</OPERATOR>
										                    <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
										                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
									                      </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                /*sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_NEXTRUN\">");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_LASTRUN\">");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");*/

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_OWNER\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>USERID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>USERID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>EMAILID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>EMAILID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_TOUSER\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>USERID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>USERID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>EMAILID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>EMAILID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_CCUSER\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>USERID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>USERID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>EMAILID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>EMAILID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_BCCUSER\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>USERID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>USERID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>EMAILID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>EMAILID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_TOGROUP\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>ISDELETED</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>0</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>GROUPID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>GROUPID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_CCGROUP\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>ISDELETED</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>0</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>GROUPID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>GROUPID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_BCCGROUP\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>ISDELETED</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>0</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>GROUPID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>GROUPID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");


                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_ANALYTICALRPT\">");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTNAME</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTNAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTNAME_DISPLAYTEXTID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTNAME_DISPLAYTEXTID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTFILENAME</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTFILENAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>STOREDPROCEDURENAME</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>STOREDPROCEDURENAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTDATAASSEMBLYNAME</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTDATAASSEMBLYNAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTDATACLASSNAME</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTDATACLASSNAME</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>INCLUDEDATE</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>INCLUDEDATE</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"ADHOCREPORT_SCHRPTDETAILS\">");
                sbReportInstancesSearchXML.Append("     <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>REPORTSTATUS</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>REPORTSTATUS</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("     </SELECTLIST>");
                sbReportInstancesSearchXML.Append("     <RELATIONSHIP NAME=\"ADHOCREPORT_CREATEDBY\">");
                sbReportInstancesSearchXML.Append("      <FILTERLIST>");
                sbReportInstancesSearchXML.Append("        <FILTER>");
                sbReportInstancesSearchXML.Append("          <FILTERID>1</FILTERID>");
                sbReportInstancesSearchXML.Append("          <EXPRESSION>STATUS</EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <OPERATOR>EQUALTO</OPERATOR>");
                sbReportInstancesSearchXML.Append("          <EXPRESSIONVALUE>ACTIVE</EXPRESSIONVALUE>");
                sbReportInstancesSearchXML.Append("          <ISCASESENSITIVE>False</ISCASESENSITIVE>");
                sbReportInstancesSearchXML.Append("        </FILTER>");
                sbReportInstancesSearchXML.Append("      </FILTERLIST>");
                sbReportInstancesSearchXML.Append("      <SELECTLIST>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>USERID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>USERID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("        <EXPRESSION>");
                sbReportInstancesSearchXML.Append("          <ALIAS>EMAILID</ALIAS>");
                sbReportInstancesSearchXML.Append("          <FIELDNAME>EMAILID</FIELDNAME>");
                sbReportInstancesSearchXML.Append("        </EXPRESSION>");
                sbReportInstancesSearchXML.Append("      </SELECTLIST>");
                sbReportInstancesSearchXML.Append("     </RELATIONSHIP>");
     
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_TOEMAILRECIPIENT\">");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");
                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_CCEMAILRECIPIENT\">");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");
                sbReportInstancesSearchXML.Append("    <RELATIONSHIP NAME=\"SCHRPT_BCCEMAILRECIPIENT\">");
                sbReportInstancesSearchXML.Append("    </RELATIONSHIP>");

                sbReportInstancesSearchXML.Append("</TARGETENTITY></SEARCH>");

                return sbReportInstancesSearchXML.ToString();
            }

            finally
            {
                sbReportInstancesSearchXML.Remove(0, sbReportInstancesSearchXML.Length);
                sbReportInstancesSearchXML = null;
            }
        }

        private void GetFromAndToDates(out string FromDate, out string ToDate, out string DisplayDate,
                                        string ScheduleType, string RunCycle, string ReportDateType, string InstanceDate)
        {
            //Count to add.
            //Unit of Days in case of DAILY, WEEKLY, FORTNIGHTLY
            //Unit of Months in case of MONTHLY, QUARTERLY
            //Unit of Years in case of YEARLY
            int AddToFromDate = 0, AddToToDate = 0;
            FromDate = ""; ToDate = ""; DisplayDate = "";
            switch (ScheduleType)
            { 
                case "Run Once":
                    RunCycle = "DAILY";
                    // Same like RunCycle = "DAILY"
                    if (ReportDateType == "FUTUREPERIOD")
                    {
                        AddToFromDate = 0;
                        AddToToDate = 1;
                    }
                    else if (ReportDateType == "PASTPERIOD")
                    {
                        AddToFromDate = -1;
                        AddToToDate = 0;
                    }
                    else if (ReportDateType == "NOTAPPLICABLE")
                    {
                        AddToFromDate = 0;
                        AddToToDate = 0;
                    }

                    break;
                case "Run Between":
                    switch (RunCycle)
                    {
                        case "DAILY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 1;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -1;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;
                        case "WEEKLY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 7;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -7;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;
                        case "FORTNIGHTLY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 15;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -15;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;
                        case "MONTHLY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 1;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -1;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;
                        case "QUARTERLY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 3;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -3;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;
                        case "YEARLY":
                            if (ReportDateType == "FUTUREPERIOD")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 1;
                            }
                            else if (ReportDateType == "PASTPERIOD")
                            {
                                AddToFromDate = -1;
                                AddToToDate = 0;
                            }
                            else if (ReportDateType == "NOTAPPLICABLE")
                            {
                                AddToFromDate = 0;
                                AddToToDate = 0;
                            }
                            break;

                        default:
                            throw new Exception("Run type is incorrect: " + RunCycle);
                    }
                    break;
            }
            

            if (RunCycle == "DAILY" || RunCycle == "WEEKLY" || RunCycle == "FORTNIGHTLY")
            {
                FromDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddDays(AddToFromDate).ToString("s"));
                ToDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddDays(AddToToDate).ToString("s"));
            }
            else if (RunCycle == "MONTHLY" || RunCycle == "QUARTERLY")
            {
                FromDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddMonths(AddToFromDate).ToString("s"));
                ToDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddMonths(AddToToDate).ToString("s"));
            }
            else if (RunCycle == "YEARLY")
            {
                FromDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddYears(AddToFromDate).ToString("s"));
                ToDate = ConvertToMMDDYYYY(DateTime.Parse(InstanceDate).AddYears(AddToToDate).ToString("s"));
            }

            if (ReportDateType == "FUTUREPERIOD" || ReportDateType == "PASTPERIOD")
            {
                DisplayDate = string.Format("{0} to {1}", FromDate, ToDate);
            }
            else if (ReportDateType == "NOTAPPLICABLE")
            {
                DisplayDate = string.Format("{0}", FromDate);
            }
        }

        #region Helper Functions
        private string AddTag(string str, string TagName)
        {
            return string.Format("<{0}>{1}</{0}>", TagName, str);
        }
        
        /// <summary>
        /// Converts DateTime's "s" format to MMDDYYYY
        /// </summary>
        /// <param name="InstanceDate">Will be string of format YYYY-MM-DDTHH:MM:SS</param>
        /// <returns></returns>
        private string ConvertToMMDDYYYY(string InstanceDate)
        {
            return string.Format("{0}/{1}/{2}", InstanceDate.Substring(5, 2),
                                                InstanceDate.Substring(8, 2),
                                                InstanceDate.Substring(0, 4));
        }

        private AbstractReportData CreateBaseReportDataObject(Dictionary<string, string> ReportInfo, String TenantName)
        {
            string AssemblyPath = null;
            Type ClassType = null;
            Assembly LocalAssembly = null;

            AssemblyPath = string.Format("{0}Binaries\\{1}", InstalledPath, ReportInfo["ReportDataAssemblyName"]);
            log.InfoFormat("ReportDataAssemblyName AssemblyPath: {0}", AssemblyPath);
            LocalAssembly = Assembly.LoadFrom(AssemblyPath);

            ClassType = GetAssemblyType(LocalAssembly, ReportInfo["ReportDataClassName"]);

            if (ClassType == null)
            {
                return null;
            }

            return (AbstractReportData)Activator.CreateInstance(ClassType, sc.curUserID, sc.curSessionID, TenantName, sc.curLanguage);
        }

        private Type GetAssemblyType(Assembly Assembly, string TextProcessorName)
        {

            foreach (Type ClassType in Assembly.GetTypes())
            {
                if (ClassType.Name == TextProcessorName)
                    return ClassType;
            }
            return null;
        }

        private string GetDateInXML(DateTime dt)
        {
            return string.Format("<YEAR>{0}</YEAR><MONTH>{1}</MONTH><DAY>{2}</DAY><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND>",
                                    dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
        }

        private String [] getAllTenants()
        {
            StringBuilder TenantList = new StringBuilder(100);
            try
            {
                String AssetlinkXML = ALCommon.GetConfigurationXMLPath() + "Assetlink.xml";

                if (File.Exists(AssetlinkXML))
                {
                    log.Info("Opened " + AssetlinkXML);

                    XmlDocument mainDoc = new XmlDocument();
                    mainDoc.Load(AssetlinkXML);
                    XmlNodeList tenantNodes = mainDoc.SelectNodes("//Tenant");

                    foreach (XmlNode tNode in tenantNodes)
                    {
                        String tName = ((XmlElement)tNode).GetAttribute("Name");
                        TenantList.Append(tName + ",");
                    }
                }
                else
                {
                    throw new Exception("File not found: " + AssetlinkXML);
                }
                return TenantList.ToString().Split(new char[]  { ',' }, StringSplitOptions.RemoveEmptyEntries);
            }
            catch (Exception e)
            {
                log.Error("Error occurred. getAlltenants() " + e.Message + e.StackTrace + e.InnerException);

                throw;
            }
        }//getAllTenants

        private String GetReportPath(String TenantName, String ReportName)
        {
            return String.Format("{0}Reports\\{1}\\", ALCommon.GetTenantPath(TenantName), ReportName);
        }

        private void GetReportFileNameFromFilterValues(XmlNode xmlNodeFilterValues, Dictionary<string, string> ReportInfo)
        {
            String  strReportFileName = null;

            if (xmlNodeFilterValues != null)
            {
                strReportFileName = ((XmlElement)xmlNodeFilterValues).GetAttribute("ReportFileName", "");

                if (!String.IsNullOrEmpty(strReportFileName))
                {
                    if (ReportInfo.ContainsKey("ReportFileName"))
                        ReportInfo["ReportFileName"] = strReportFileName;
                    else
                        ReportInfo.Add("ReportFileName", strReportFileName);
                }
            }
        }

        private ERRORXML ExportAdhocReport(String AdhocReportID, String ScheduledReportInstanceID, String ScheduledReportExportFormat, String TenantName, out String ExportOutPutXml)
        {
            Int32   WebServiceTimeout                                           = 3600000;
            String  str_WebServiceTimeout                                       = null;
            Assetlink.WebServices.ALReportWS.ALReportWS    ReportServiceObj     = null;
            Assetlink.WebServices.ALReportWS.ALServiceHeader ReportReqHeader    = null;

            ExportOutPutXml = null;
            

            try
            {

                str_WebServiceTimeout = ConfigurationManager.AppSettings["WebServiceTimeout"];
                if (!String.IsNullOrEmpty(str_WebServiceTimeout))
                    Int32.TryParse(str_WebServiceTimeout, out WebServiceTimeout);


                ReportReqHeader                          = new Assetlink.WebServices.ALReportWS.ALServiceHeader();
                ReportServiceObj                         = new Assetlink.WebServices.ALReportWS.ALReportWS();
                ReportServiceObj.Url                     = ALCommon.GetWebServiceURL("ALReportWS");
                ReportServiceObj.Timeout                 = WebServiceTimeout;
                ReportReqHeader.SessionID                = sc.curSessionID;
                ReportReqHeader.UserID                   = sc.curUserID;
                ReportReqHeader.TenantName               = TenantName;
                ReportReqHeader.Language                 = sc.curLanguage;
                ReportServiceObj.ALServiceHeaderValue    = ReportReqHeader;

                return ReportServiceObj.ExportAdhocReport(AdhocReportID, ScheduledReportInstanceID, ScheduledReportExportFormat, out ExportOutPutXml);
            }
            catch (Exception ExceptionObj)
            {
                return "<ERROR><MESSAGE>" + ExceptionObj.Message + "</MESSAGE></ERROR>";
            }
        }
        
        #endregion
    }
}
