﻿#region System Includes

using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.Threading;
using System.Reflection;
using System.Collections.Generic;
using System.IO;

#endregion

#region Assetlink Inlcudes

using Assetlink;
using Assetlink.Solutions;
using Assetlink.Solutions.Pages;
using Assetlink.CustomControls;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.ReportFacade;

#endregion

namespace Assetlink.Solutions.Reports
{
    #region Directives

    using XML       = String;
    using ERRORXML  = String;

    #endregion

    public partial class AdHocReportConfiguration : ALAjaxPage
    {
        #region Member Variables

        ALSolutionCommon SolutionCommonObj = null;
        ALParam ALParamObj = null;
        UInt64 ErrorID = 0;
        String ScreenMode = null;
        Boolean IsTabClickEvent = false;
        List<ObjectInfo> ObjectInfoCollection = new List<ObjectInfo>();
        FormFacade FormFacadeObject = null;
        String ReportID = null;
        UserControl ALUC_AdvQryBldr = null;
        String InstallationDir = null;
        String EventGenerator = null;
        String UserRoleName = null;
        #endregion

        #region Private Properties
        String m_ReportName = null;
        private String ReportName
        {
            set
            {
                ViewState["REPORTNAME"] = value;
            }

            get
            {
                return (String)ViewState["REPORTNAME"];
            }
        }

        private String CurrentPageURL
        {
            get
            {
                if (ViewState["CurrentPageURL"] == null)
                    return Page.Request.RawUrl;
                else
                    return (String)ViewState["CurrentPageURL"];
            }
            set
            {
                ViewState["CurrentPageURL"] = value;
            }
        }
        #endregion

        #region Protected Page Methods

        protected void Page_Init(object sender, EventArgs e)
        {
            try
            {
                initializeURLParameters();      // set the parameters from URL
                initializeControls();           // get the controls and set properties of controls
                initializeServerSideEvents();   //set the event handlers assigned to controls.
            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "Page_Init", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        protected override void CreateChildControls()
        {
            try
            {
                if (ViewState["ChannelType"] != null)
                    setAdvancedQueryControl(ViewState["ChannelType"].ToString(), "CreateChildControls");
                else
                    setAdvancedQueryControl(null, "CreateChildControls");

                
                #region Assign User Group Controls
                ALUC_AdhocReportAuthor.Delete_User_Click += new ALUC_ALUserSelection.UserDeleteEventHandler(ALUC_AdhocReportAuthor_Delete_User_Click);
                ALUC_AdhocReportAuthor.DoneClick         += new EventHandler(ALUC_AdhocReportAuthor_DoneClick);
                ALUC_AdhocReportReader.Delete_User_Click += new ALUC_ALUserSelection.UserDeleteEventHandler(ALUC_AdhocReportReader_Delete_User_Click);
                ALUC_AdhocReportReader.DoneClick         += new EventHandler(ALUC_AdhocReportReader_DoneClick);
                ALGC_AdhocReportAuthorGroup.DeleteClick  += new EventHandler(ALGC_AdhocReportAuthorGroup_DeleteClick);
                ALGC_AdhocReportAuthorGroup.DoneClick    += new EventHandler(ALGC_AdhocReportAuthorGroup_DoneClick);
                ALGC_AdhocReportReaderGroup.DeleteClick  += new EventHandler(ALGC_AdhocReportReaderGroup_DeleteClick);
                ALGC_AdhocReportReaderGroup.DoneClick    += new EventHandler(ALGC_AdhocReportReaderGroup_DoneClick);
                #endregion

                base.CreateChildControls();

            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "CreateChildControls", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                Ajax.AsynchronousRequest = true;
                if (!IsPostBack)
                {
                    Page.ClientScript.RegisterClientScriptInclude("SpecialCharJS", "/" + ALParamObj.TenantName + "/Common/JS/SpecialCharHandling.js");
                    ViewState["PageURL"] = Page.Request.RawUrl;

                    //Update ViewSates
                    ReportName = m_ReportName;
                    CurrentPageURL = Page.Request.RawUrl;
                }

                SetControls();
            }
            catch (ThreadAbortException)
            { }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "Page_Load", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            Page.ClientScript.RegisterHiddenField("__DeletedFieldValues", "");

            #region Schedule Popup 
            if (!IsPostBack)
            {
                StringBuilder CreateScheduleURL = new StringBuilder();
                CreateScheduleURL.AppendFormat("AdHocReportSchedule_Create_Edit.aspx?TenantName={0}&SessionID={1}&UserID={2}&Language={3}&SolutionName={4}&ClosePopupWindowFun={5}&AddScheduleBtnUniqueID={6}&ReportID={7}&ReportName={8}",
                    ALParamObj.TenantName, ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.Language, ALParamObj.SolutionName, ALSpecialCharHandling.EncodeSpecialCharactersForJS(CreateEdit_SchedulePopup.ClosePopupJavascript), ALB_AddSchedule.UniqueID, ReportID, Server.UrlEncode(ReportName));

                Page.RegisterHiddenField("CreateEdit_PopupCreateCaption", ALSpecialCharHandling.EncodeSpecialCharactersForJS(CreateEdit_SchedulePopup.ChangeCaptionJavascript(Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_CreateSchedule"))));
                Page.RegisterHiddenField("CreateEdit_PopupUpdateCaption", ALSpecialCharHandling.EncodeSpecialCharactersForJS(CreateEdit_SchedulePopup.ChangeCaptionJavascript(Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_UpdateSchedule"))));

                Page.RegisterHiddenField("CreateEdit_SchedulePopupURL", SolutionCommonObj.EncodeURLForResponseRedirects(CreateScheduleURL.ToString()));
                Page.RegisterHiddenField("CreateEdit_ShowPopupWindowCall", ALSpecialCharHandling.EncodeSpecialCharactersForJS(CreateEdit_SchedulePopup.ShowPopupWindowJavascript));
            }
            #endregion

            if (!Page.ClientScript.IsClientScriptIncludeRegistered("ReportHelperFunctionsScript"))
                Page.ClientScript.RegisterClientScriptInclude("ReportHelperFunctionsScript", "/" + ALParamObj.TenantName + "/" + ALParamObj.SolutionName + "/JS/ReportHelperFunctions.js");

            if (!Page.ClientScript.IsClientScriptIncludeRegistered("LocalizationScript2"))
                Page.ClientScript.RegisterClientScriptInclude("LocalizationScript2", "/" + ALParamObj.TenantName + "/Common/JS/Localization.js");

            base.OnPreRender(e);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            Control control_SubObject       = null;
            String  str_JavaScriptToExecute = null;
            String  str_RedirectURL         = null;

            try
            {
                if ((IsPostBack) && (!(Ajax.IsNonAjaxPostback)))
                {
                    if (Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportAuthorList_List_UpperNext") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportAuthorList_List_UpperPrev") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportAuthorList_List_UpperLast") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportAuthorList_List_UpperFirst"))
                        Ajax.AddControlsToRender(ADHOCA);

                    if (Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportAuthorGroupGroup_List_List_UpperNext") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportAuthorGroupGroup_List_List_UpperFirst") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportAuthorGroupGroup_List_List_UpperPrev") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportAuthorGroupGroup_List_List_UpperLast"))
                        Ajax.AddControlsToRender(ADHOCAG);

                    if (Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportReaderList_List_UpperNext") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportReaderList_List_UpperFirst") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportReaderList_List_UpperLast") || Ajax.EventGeneratorUniqueID.Contains("ALUC_AdhocReportReaderList_List_UpperPrev"))
                        Ajax.AddControlsToRender(ADHOCR);

                    if (Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportReaderGroupGroup_List_List_UpperNext") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportReaderGroupGroup_List_List_UpperFirst") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportReaderGroupGroup_List_List_UpperPrev") || Ajax.EventGeneratorUniqueID.Contains("ALGC_AdhocReportReaderGroupGroup_List_List_UpperLast"))
                        Ajax.AddControlsToRender(ADHOCRG);

                    if ((ViewState["ChannelType"] != null) && (!(String.IsNullOrEmpty(ViewState["ChannelType"].ToString()))))
                    {
                        if (ALUC_AdvQryBldr.GetType().GetProperty("JavaScriptToExecute") != null)
                        {
                            str_JavaScriptToExecute = (String)ALUC_AdvQryBldr.GetType().GetProperty("JavaScriptToExecute").GetValue(ALUC_AdvQryBldr, null);
                            if(!String.IsNullOrEmpty(str_JavaScriptToExecute))
                                Ajax.WriteExecuteJavaScript("Javascript:" + str_JavaScriptToExecute);
                        }

                        control_SubObject = (Control)ALUC_AdvQryBldr.GetType().GetProperty("ControlToRender").GetValue(ALUC_AdvQryBldr, null);
                        if (control_SubObject != null)
                            Ajax.AddControlsToRender(control_SubObject);
                        else
                        {
                            if (Ajax.EventGeneratorUniqueID.Contains("SelectTargetEntity"))
                                Ajax.AddControlsToRender(Div_UserControl);
                        }

                        if (EventGenerator == null)
                        {
                            if (Ajax.EventGeneratorUniqueID.Contains("ReportScheduleList"))
                                Ajax.AddControlsToRender(ReportScheduleList);
                        }
                        else
                        if (EventGenerator.Equals("ADHOCCONFIGTABCLICK"))
                            Ajax.AddControlsToRender(Tab_AdHocConfigs.SelectedTab);
                        else if (EventGenerator.Equals("SCHEDULEADDED") || EventGenerator.Equals("SCHEDULERESUBMITED"))
                        {
                            Ajax.AddControlsToRender(ReportScheduleList);
                        }
                        else if (!EventGenerator.Equals("VIEWREPORT"))
                            Ajax.AddControlsToRender(Div_UserControl);
                            
                    }
                    else
                    {
                        Ajax.AddControlsToRender(Div_UserControl);
                        if (ALUC_AdvQryBldr != null && ALUC_AdvQryBldr.GetType().GetProperty("JavaScriptToExecute") != null)
                        {
                            str_JavaScriptToExecute = (String)ALUC_AdvQryBldr.GetType().GetProperty("JavaScriptToExecute").GetValue(ALUC_AdvQryBldr, null);
                            Ajax.WriteExecuteJavaScript("Javascript:" + str_JavaScriptToExecute);
                        }
                    }


                    if (IsTabClickEvent)
                    {
                        Ajax.AddControlsToRender(Tab_AdHocConfigs.SelectedTab);
                        Ajax.WriteExecuteJavaScript("javascript:" + Tab_AdHocConfigs.HighlightTabJavaScript);
                        IsTabClickEvent = false;
                    }

                    //If the View Report option is selected then save the report (which is handled in action bar event)
                    // and then open the preview page.
                    if ((!(String.IsNullOrEmpty(EventGenerator))) && (EventGenerator.Equals("VIEWREPORT")))
                    {
                        str_RedirectURL = "AdHocReportPreview.aspx?SessionID=" + ALParamObj.SessionID + "&UserID=" + ALParamObj.UserID
                                + "&Language=" + ALParamObj.Language + "&TenantName=" + ALParamObj.TenantName
                                + "&SolutionName=" + ALParamObj.SolutionName + "&REPORTID=" + ReportID;
                        str_RedirectURL += "&ParentPageName=AdHocReportConfiguration&UserRoleName=" + UserRoleName;

                        str_RedirectURL = SolutionCommonObj.EncodeURLForResponseRedirects(str_RedirectURL);
                        Ajax.WriteExecuteJavaScript("Javascript:window.open('" + str_RedirectURL + "' ,'_blank', '');");
                    }

                    Ajax.WriteExecuteJavaScript("ResizeDropDisplayColumnArea()");
                    Ajax.WriteControlHTMl(Page, writer);
                }
                else
                {
                    base.Render(writer);
                }
            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "Render", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        #endregion

        #region Event Handlers

        protected void Form_AdHocConfig_FormSelectionChanged(object sender, EventArgs e)
        {
            try
            {
                setAdvancedQueryControl(((DropDownList)sender).SelectedValue, "FormSelectionChanged");
            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "Add_Edit_Frm_FormSelectionChanged", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        protected void Tab_AdHocConfigs_onTabClick(object Sender, TabClickEventarg TabclickEventarg)
        {
            try
            {
                EventGenerator  = "ADHOCCONFIGTABCLICK";
                IsTabClickEvent = true;

                switch (TabclickEventarg.SelectedTab.Name.ToString().ToUpper())
                {
                    case "TAB_ADHOCCONFIGS":
                        break;

                    case "SCHEDULE":
                        PrepareScheduleListDataXml();
                        break;

                    case "USERS":
                        GetAdhocReport_AllRoleData(ReportID);
                        break;
                }
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }          
        }

        protected void ActionBar_AdvQryConfig_Click(object sender, EventArgs e)
        {
            String      str_SenderButtonText    = null;
            String      str_RedirectURL         = null;
            String      str_RetrievedReportID   = null;
            XML         xml_AdHocReportInstance = null;
            ERRORXML    errorxml_ErrorResult    = null;
            
            try
            {
                ObjectInfoCollection = (List<ObjectInfo>)ViewState["ObjectInfoList"];
                errorxml_ErrorResult = FormFacadeObject.GetFormDataXML("READONLY", "DATAFIELD", Form_AdHocConfig, ObjectInfoCollection, out xml_AdHocReportInstance);
                if (!(String.IsNullOrEmpty(errorxml_ErrorResult)))
                {
                    redirectToErrorPage("ERR_AdhocReport_RetrieveFormData", true);
                    return;
                }

                str_SenderButtonText = sender.ToString().Remove(0, sender.ToString().IndexOf("$AL$") + 4);
                str_SenderButtonText = str_SenderButtonText.Replace("$AL$", "");
                switch (str_SenderButtonText)
                {
                    case "Save":
                        saveAdHocReportConfigDetails(xml_AdHocReportInstance, out xml_AdHocReportInstance);
                        str_RedirectURL = "AdHocReportConfigurationList.aspx?SessionID=" + ALParamObj.SessionID + "&UserID=" + ALParamObj.UserID
                                    + "&Language=" + ALParamObj.Language + "&TenantName=" + ALParamObj.TenantName 
                                    + "&SolutionName=" + ALParamObj.SolutionName;
                        str_RedirectURL = SolutionCommonObj.EncodeURLForResponseRedirects(str_RedirectURL);
                        //Ajax.WriteExecuteJavaScript("Javascript:window.open('" + str_RedirectURL + "','_self');");
                        Ajax.Redirect(this.Page, str_RedirectURL);
                        break;

                    case "ViewReport":
                        if (UserRoleName.ToUpper() == "READER")
                        {
                            EventGenerator = "VIEWREPORT";
                        }
                        else
                        {
                            Ajax.WriteExecuteJavaScript("ShowAlert('Alert_AdhocReport_ReportConfigurationSaved');");

                            saveAdHocReportConfigDetails(xml_AdHocReportInstance, out xml_AdHocReportInstance);
                            if (!(String.IsNullOrEmpty(xml_AdHocReportInstance)))
                                EventGenerator = "VIEWREPORT";
                            else
                                redirectToErrorPage("ERR_AdhocReport_SaveConfigDetails", true);
                        }
                        break;
                }
            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "ActionBar_AdvQryConfig_Click", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }
      
        void ALB_DeleteSelectedField_Click(object sender, EventArgs e)
        {
            String ErrorXML = null;

            if (Request["__EVENTTARGET"] != null && Request["__EVENTTARGET"].Contains("ALB_DeleteSelectedField"))
            {
                String SelectedValues = Request["__DeletedFieldValues"];

                if (!String.IsNullOrEmpty(SelectedValues))
                {
                    ScheduleReport objSCHRptFacade = new ScheduleReport(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    ErrorXML = objSCHRptFacade.DeleteScheduleReport(SelectedValues);

                    if (ErrorXML == null)
                    {
                        PrepareScheduleListDataXml();
                        Ajax.AddControlsToRender(ReportScheduleList);
                    }
                    else
                    {
                        //String ErrorPageURL = null;
                        //SolutionCommonObj.PrepareErrorPageUrl(ALParamObj.SolutionName, null, "SCHEDULEREPORT", ErrorXML, null, null, ViewState["PageURL"].ToString(), ref ErrorPageURL);
                        //if (!IsPostBack)
                        //    Response.Redirect(ErrorPageURL);
                        //else
                        //    Ajax.Redirect(Page, ErrorPageURL);
                    }
                }
            }
        }

        void Hidden_ReSubmitBtn_Click(object sender, EventArgs e)
        {
            String                  ErrorXML            = null;
            ScheduleReport          objRPTFacade        = null;
            String                  SCHRPTDetailsID     = null;
            bool                    IsInstanceExists    = false;
            String                  ErrorPageURL        = null;
            try
            {
                SCHRPTDetailsID     = Request["__EVENTARGUMENT"].ToString();
                objRPTFacade        = new ScheduleReport(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);

                ErrorXML = objRPTFacade.ResendErrorInstance(SCHRPTDetailsID, ref IsInstanceExists);
                if (ErrorXML != null)
                {
                    SolutionCommonObj.PrepareErrorPageUrl(ALParamObj.SolutionName, null, "SCHEDULEREPORT", ErrorXML, null, null, null, ref ErrorPageURL);
                    Ajax.Redirect(this.Page, ErrorPageURL);
                    return;
                }

                if (IsInstanceExists)
                    Ajax.WriteExecuteJavaScript("ShowAlert('Alert_Rpt_InstanceConflict')");
                else
                {
                    PrepareScheduleListDataXml();
                    EventGenerator = "SCHEDULERESUBMITED";
                    Ajax.WriteExecuteJavaScript("ShowAlert('Alert_Rpt_InstanceResubmitted')");
                }
            }
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ExceptionObj)
            {
                ErrorID = ALSolutionCommon.DumpExceptionInfo(this.Page, "AdHocReportConfiguration.aspx", "ActionBar_AdvQryConfig_Click", ExceptionObj, null, null);
                redirectToErrorPage(ExceptionObj.Message, false);
            }
        }

        #region User/Group Section Control Event Handlers
        void ALUC_AdhocReportAuthor_Delete_User_Click(object Sender, string UserIDList)
        {
            try
            {
                HandleDeleteUserRole("AdhocReport_Authors", "ADHOCREPORT_AUTHOR");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_DeleteAuthor_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALUC_AdhocReportAuthor_DoneClick(object sender, EventArgs e)
        {

            try
            {
                HandleAddUserRole("AdhocReport_Authors", "ADHOCREPORT_AUTHOR");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_AddAuthor_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALUC_AdhocReportReader_Delete_User_Click(object Sender, string UserIDList)
        {
            try
            {
                HandleDeleteUserRole("AdhocReport_Readers", "ADHOCREPORT_READER");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_DeleteReader_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALUC_AdhocReportReader_DoneClick(object sender, EventArgs e)
        {
            try
            {
                HandleAddUserRole("AdhocReport_Readers", "ADHOCREPORT_READER");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_AddReader_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALGC_AdhocReportAuthorGroup_DeleteClick(object sender, EventArgs e)
        {
            try
            {
                HandleDeleteGroupRole("AdhocReportAuthor_Group", "ADHOCREPORT_AUTHORGROUP");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_DeleteAuthorGroup_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALGC_AdhocReportAuthorGroup_DoneClick(object sender, EventArgs e)
        {
            try
            {
                HandleAddGroupRole("AdhocReportAuthor_Group", "ADHOCREPORT_AUTHORGROUP");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_AddAuthorGroup_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALGC_AdhocReportReaderGroup_DeleteClick(object sender, EventArgs e)
        {
            try
            {
                HandleDeleteGroupRole("AdhocReportReader_Group", "ADHOCREPORT_READERGROUP");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_DeleteReaderGroup_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }

        void ALGC_AdhocReportReaderGroup_DoneClick(object sender, EventArgs e)
        {
            try
            {
                HandleAddGroupRole("AdhocReportReader_Group", "ADHOCREPORT_READERGROUP");
            }
            catch (ThreadAbortException) { }
            catch (Exception ExcepObj)
            {
                String AdditionalParams = "&ResourceID=Err_AdhocReport_ReportDetails_AddReaderGroup_Exception";
                ALSolutionCommonExtension.DumpExceptionAndRedirectToErrorPage(ExcepObj, null, null, true, null, AdditionalParams, GetBackURL());
            }
        }
        #endregion

        #region AddSchedule Event Handler
        void ALB_AddSchedule_Click(object sender, EventArgs e)
        {
            Report_Facade Report_FacadeObj = null;
            String StrReportUpdateXML = null;

            Report_FacadeObj = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);

            StrReportUpdateXML = "<ADHOCREPORT><REPORTID>" + ReportID + "</REPORTID></ADHOCREPORT>";
            Report_FacadeObj.UpdateAdHocReport(ref StrReportUpdateXML); //Update Modifief by
            PrepareScheduleListDataXml();
            EventGenerator = "SCHEDULEADDED";
        }
        #endregion

        #endregion

        #region Helper Methods

        //initializeURLParameters() method is used to set parameters from URL and to assigned required default values to parameters.
        private void initializeURLParameters()
        {
            SolutionCommonObj = new ALSolutionCommon();
            ALParamObj = new ALParam();

            SolutionCommonObj.httpContext = HttpContext.Current;
            SolutionCommonObj.InitALParam(ref ALParamObj);

            if (String.IsNullOrEmpty(ALParamObj.SolutionName))
                ALParamObj.SolutionName = "Reports";

            this.Master.GetType().GetProperty("SessionID").SetValue(this.Master, ALParamObj.SessionID, null);
            this.Master.GetType().GetProperty("UserID").SetValue(this.Master, ALParamObj.UserID, null);
            this.Master.GetType().GetProperty("Language").SetValue(this.Master, ALParamObj.Language, null);
            this.Master.GetType().GetProperty("TenantName").SetValue(this.Master, ALParamObj.TenantName, null);
            this.Master.GetType().GetProperty("SolutionName").SetValue(this.Master, ALParamObj.SolutionName, null);

            ReportID        = SolutionCommonObj.GetParamValue("REPORTID");
            ScreenMode      = SolutionCommonObj.GetParamValue("SCREENMODE");
            UserRoleName    = SolutionCommonObj.GetParamValue("UserRoleName");

            if (UserRoleName == null)
                UserRoleName = "";
        }

        //initializeControls() method is used to get the controls and set properties of controls.
        private void initializeControls()
        {
            LeftPaneFacade facade_LeftPane = null;

            facade_LeftPane = new LeftPaneFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, 
                    ALParamObj.Language, ALParamObj.SolutionName);
            facade_LeftPane.GetTaskPaneNode(Page, null);

            ABC_AdHocReportConfig.TenantName     = ALParamObj.TenantName;
            ABC_AdHocReportConfig.CaptionText    = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_ReportDetails");
            ABC_AdHocReportConfig.cssCaptionText = "MainTitleText";
            Tree_AdHoc.TenantName = ALParamObj.TenantName;

            if (UserRoleName.ToUpper() == "READER")
                EditAdhocReportPanel.Visible = false;
            else
            {
                Btn_EditAdhocReport.Text = Btn_EditAdhocReport.ButtonToolTip = Localization.GetString("ALResource", ALParamObj.TenantName, "Edit_Shared");
                Btn_EditAdhocReport.Attributes.Add("href", PrepareEditPageURL());
            }

            if (!String.IsNullOrEmpty(System.Configuration.ConfigurationManager.AppSettings["SHAREDDRIVEROOT"]))
                InstallationDir = System.Configuration.ConfigurationManager.AppSettings["SHAREDDRIVEROOT"];
            else
                InstallationDir = ALSolutionCommon.GetInstallationDir();
            
            //Set the properties to the form control used for Adhoc reporting.
            setAdHocReportConfigFormControl();
            
            //ViewState["ChannelID"] = ChannelID;
            //ViewState["ScreenMode"] = ScreenMode;
            //if (!IsPostBack)
            //{
            //    Page.ClientScript.RegisterClientScriptInclude("SpecialCharJS", "/" + ALParamObj.TenantName + "/Common/JS/SpecialCharHandling.js");
            //}

            //Set the required properties for tab control.
            setTabDetails();

            //Set the required properties for the Advanced Query Builder control.
            //setAdvancedQueryControl(null);

            ActionBar_AdvQryConfig.TenantName = ALParamObj.TenantName;
            if (UserRoleName.ToUpper() == "READER")
            {
                XmlDocument ActionBar_AdvQryConfigXmlDoc = new XmlDocument();
                ActionBar_AdvQryConfigXmlDoc.Load(ALSolutionCommon.GetConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "", "AdHoc_AdvanceQryBldrConfig_ActionBar_Save"));
                ActionBar_AdvQryConfig.InputXMLType = ActionBar.XMLType.InnerXML;

                ((XmlElement)ActionBar_AdvQryConfigXmlDoc.SelectSingleNode("ActionControlList/ActionControl[@Name='Save']")).SetAttribute("Visible", "false");
                ActionBar_AdvQryConfig.InputXML = ActionBar_AdvQryConfigXmlDoc.OuterXml;
            }
            else
            {
                ActionBar_AdvQryConfig.InputXMLType = ActionBar.XMLType.XMLPath;
                ActionBar_AdvQryConfig.InputXML     = ALSolutionCommon.GetConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "", "AdHoc_AdvanceQryBldrConfig_ActionBar_Save");
                ActionBar_AdvQryConfig.AddAttributes("Save", "Visible", "false");
                ActionBar_AdvQryConfig.AddAttributes("Save", "href", "Javascript:__doPostBack('" + ActionBar_AdvQryConfig.UniqueID + "$leftSave0','')");
            }

            #region Schedule List
            RSL_Container.TenantName      = ALParamObj.TenantName;
            RSL_Container.CaptionText     = Localization.GetString("ALResource", ALParamObj.TenantName, "ScheduleList_Shared");
            RSL_Container.cssCaptionText  = "MainTitleText";
            ReportScheduleList.TenantName = ALParamObj.TenantName;
            ReportScheduleList.InputType  = Assetlink.CustomControls.ListControl.SourceType.InnerXML;
            ReportScheduleList.XSLTPath   = ALSolutionCommon.GetInstallationDir() + ALParamObj.TenantName + "\\XSLT\\SortListXML.xslt";
            #endregion

            #region Delete Schedule Button
            ALB_DeleteSelectedField.TenantName = ALParamObj.TenantName;
            ALB_DeleteSelectedField.Text = ALB_DeleteSelectedField.ButtonToolTip = Localization.GetString("ALResource", ALParamObj.TenantName, "Delete_Shared");
            ALB_DeleteSelectedField.Attributes.Add("href", "javascript:DeleteField('" + ALB_DeleteSelectedField.UniqueID + "');");
            ALB_DeleteSelectedField.Click += new EventHandler(ALB_DeleteSelectedField_Click);
            #endregion

            #region Schedule Action Bar
            AB_ScheduleReport.TenantName    = ALParamObj.TenantName;
            AB_ScheduleReport.InputXMLType  = ActionBar.XMLType.XMLPath;
            AB_ScheduleReport.InputXML      = ALSolutionCommon.GetConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "", "AdhocReportUserRole_ActionBar");
            AB_ScheduleReport.AddAttributes("ScheduleReport", "href", "Javascript:if(ValidateForm_RSF()){if(Validate_Date_Range_Custom()){__doPostBack('" + AB_ScheduleReport.UniqueID + "$leftScheduleReport0',''); }}");
            #endregion

            #region Schedule Action Bar
            AB_UserRole.TenantName      = ALParamObj.TenantName;
            String UserRoleActionBarXml = ALSolutionCommon.GetConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "", "AdhocReportUserRole_ActionBar");

            AB_UserRole.InputXMLType    = ActionBar.XMLType.XMLPath;
            AB_UserRole.InputXML        = UserRoleActionBarXml;
            #endregion

            #region Show Error Popup Control
            StringBuilder sbNavigateUrl_Popup = new StringBuilder();
            sbNavigateUrl_Popup.AppendFormat("ScheduledReport_ErrorInfo.aspx?TenantName={0}&SessionID={1}&UserID={2}&Language={3}&SolutionName={4}&PopupWindowID={5}", ALParamObj.TenantName, ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.Language, ALParamObj.SolutionName, Popup_ErrorInfo.ClientID);
            Popup_ErrorInfo.TenantName          = ALParamObj.TenantName;
            Popup_ErrorInfo.NavigateURL         = SolutionCommonObj.EncodeURLForResponseRedirects(sbNavigateUrl_Popup.ToString());
            Popup_ErrorInfo.PopupDisplayMode    = Assetlink.CustomControls.ePopupDisplayMode.Simple;
            Popup_ErrorInfo.PopupWidth          = 500;
            Popup_ErrorInfo.PopupHeight         = 300;
            Popup_ErrorInfo.ShowDimmer          = true;
            #endregion

            #region Schedule popup
            ALB_AddSchedule.Text = ALB_AddSchedule.ButtonToolTip = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_AddSchedule");
            ALB_AddSchedule.Attributes.Add("href", "Javascript:CreateSchedule()");
            ALB_AddSchedule.Click += new EventHandler(ALB_AddSchedule_Click);

            CreateEdit_SchedulePopup.TenantName          = ALParamObj.TenantName;
            CreateEdit_SchedulePopup.NavigateURL         = "$SCHEDULEPOPUPURL$";
            CreateEdit_SchedulePopup.PopupDisplayMode    = Assetlink.CustomControls.ePopupDisplayMode.Simple;
            CreateEdit_SchedulePopup.PopupWidth          = 900;
            CreateEdit_SchedulePopup.PopupHeight         = 700;
            CreateEdit_SchedulePopup.ShowDimmer          = true;
            #endregion

            #region Resubmit od the task
            Hidden_ReSubmitBtn.Click += new EventHandler(Hidden_ReSubmitBtn_Click);
            #endregion
        }


        //initializeServerSideEvents() method sets the event handlers assigned to controls.
        private void initializeServerSideEvents()
        {
            String str_RedirectURL = null;

            Tab_AdHocConfigs.onTabClick             += new ALTab.TabEventHandler(Tab_AdHocConfigs_onTabClick);
            ActionBar_AdvQryConfig.Click            += new EventHandler(ActionBar_AdvQryConfig_Click);
            Form_AdHocConfig.FormSelectionChanged   += new EventHandler(Form_AdHocConfig_FormSelectionChanged);

            str_RedirectURL = "AdHocReportConfigurationList.aspx?SessionID=" + ALParamObj.SessionID + "&UserID=" + ALParamObj.UserID
                    + "&Language=" + ALParamObj.Language + "&TenantName=" + ALParamObj.TenantName + "&SolutionName=" + ALParamObj.SolutionName;
            str_RedirectURL = SolutionCommonObj.EncodeURLForResponseRedirects(str_RedirectURL);
            ActionBar_AdvQryConfig.AddAttributes("Cancel", "href", "Javascript:window.open('" + str_RedirectURL + "','_self');");
            AB_ScheduleReport.AddAttributes("Cancel", "href", "Javascript:window.open('" + str_RedirectURL + "','_self');");
            AB_UserRole.AddAttributes("Cancel", "href", "Javascript:window.open('" + str_RedirectURL + "','_self');");
        }

        //redirectToErrorPage() method is used to redirect current page to an error page if any error occurred while working with system.
        private void redirectToErrorPage(String ErrorMessageString, Boolean IsLocalized)
        {
            String          str_RedirectionPage = null;
            ERRORXML        errorxml_Output     = null;
            StringBuilder   sb_ErrorPageURL     = null;
            String          str_ErrorPageURL    = null;

            if (IsLocalized)
                ErrorMessageString = Localization.GetString("ALErrorResource", ALParamObj.TenantName, ErrorMessageString);

            errorxml_Output = SolutionCommonObj.PrepareErrorPageUrl(ALParamObj.SolutionName, null, "REPORTS", null, ErrorMessageString,
                    null, Page.Request.Url.ToString(), ref str_ErrorPageURL);
            if (String.IsNullOrEmpty(errorxml_Output))
            {
                sb_ErrorPageURL = new StringBuilder(200);
                sb_ErrorPageURL.Append("/" + ALParamObj.TenantName + "/Common/ASPX/ShowError.aspx?" + ALSolutionCommon.AppendStandardParamforURL(ALParamObj));
                sb_ErrorPageURL.Append("&ShowOKButton=FALSE&ErrorMessage=" + ErrorMessageString);
                str_ErrorPageURL = sb_ErrorPageURL.ToString();
            }

            str_RedirectionPage = SolutionCommonObj.EncodeURLForResponseRedirects(str_ErrorPageURL);
            if (Page.IsPostBack)
                Ajax.Redirect(this.Page, str_RedirectionPage);
            else
                Response.Redirect(str_RedirectionPage);
        }

        private String getVirtualPath(String FileName, bool IsCommonControl)
        {
            StringBuilder WidgetVirtualPath = null;
            String TempSolutionName = null;

            if ((FileName == null) || (FileName == ""))
                return null;

            WidgetVirtualPath = new StringBuilder();

            if ((ALParamObj.SolutionName != null) && (ALParamObj.SolutionName != ""))
            {
                if (IsCommonControl)
                    TempSolutionName = "Common";
                else
                    TempSolutionName = ALParamObj.SolutionName;

                WidgetVirtualPath.AppendFormat("{0}{1}\\{2}\\ASCX\\{3}", ALSolutionCommon.GetInstallationDir(), ALParamObj.TenantName, TempSolutionName, FileName);
                bool FileExists = System.IO.File.Exists(WidgetVirtualPath.ToString());
                if (FileExists)
                {
                    WidgetVirtualPath.Remove(0, WidgetVirtualPath.Length);
                    WidgetVirtualPath.AppendFormat("/{0}/{1}/ASCX/{2}", ALParamObj.TenantName, TempSolutionName, FileName);
                    return WidgetVirtualPath.ToString();
                }
            }
            return null;
        }

        //setTabDetails() method is used to set the required properties for tab control.
        private void setTabDetails()
        {
            TabFacade facade_TabObject = null;

            facade_TabObject = new TabFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            facade_TabObject.SolutionName   = ALParamObj.SolutionName;
            Tab_AdHocConfigs.TenantName     = ALParamObj.TenantName;
            facade_TabObject.TabName        = "Tab_AdHocConfigs";
            facade_TabObject.ScreenName     = "AdHocReportConfiguration";
            facade_TabObject.InitializeTab(ref Tab_AdHocConfigs);

            //Tab_AdHocConfigs.SelectedTabName = "UserControl";
        }

        //setAdvancedQueryControl() method is used to set the required properties for the Advanced Query Builder control.
        private void setAdvancedQueryControl(String SelectedReportType, String SelectedEventOption) //String Channeltype, String SearchOutputXML, String Mode
        {
            MethodInfo  methodinfo_Object           = null;
            Object[]    objarr_Params               = null;
            XmlDocument xmldoc_SearchOutput         = null;
            XmlNode     xmlnode_ChannelField        = null;
            XmlNode     xmlnode_ChannelConfig       = null;
            String      str_TargetObject            = null;
            String      str_UserControlName         = null;
            String      str_ConfigurationXmlPath    = null;
            String      str_TemplateDirectoryPath   = null;
            Boolean     bool_FlagCommonControl      = false;
            XML         xml_SearchOutput            = null;
            ERRORXML    errorxml_ErrorResult        = null;

            if (ViewState["AdHocConfigSearchOutputXML"] != null)
                xml_SearchOutput = ViewState["AdHocConfigSearchOutputXML"].ToString();

            if (!(String.IsNullOrEmpty(xml_SearchOutput)))
            {
                xmldoc_SearchOutput = new XmlDocument();
                xmldoc_SearchOutput.LoadXml(xml_SearchOutput);

                //Retrieve the channel configuration node.
                xmlnode_ChannelConfig = xmldoc_SearchOutput.SelectSingleNode("ADHOCREPORTCOLLECTION/ADHOCREPORT/REPORTCONFIGXML");

                //Retrieve the report category.
                if (String.IsNullOrEmpty(SelectedReportType))
                    SelectedReportType = ALCommon.GetInnerValue(ref xml_SearchOutput, "<REPORTCATEGORY>", "</REPORTCATEGORY>");
            }

            ALUC_AdvQryBldr = Page.LoadControl(getVirtualPath("ALUC_AdvancedQueryBuilder/AdvancedQueryBuilder.ascx", true)) as UserControl;
            ALUC_AdvQryBldr.ID = "ALUC_AdvQryBldr";
            ALUC_AdvQryBldr.GetType().GetProperty("SessionID").SetValue(ALUC_AdvQryBldr, ALParamObj.SessionID, null);
            ALUC_AdvQryBldr.GetType().GetProperty("UserID").SetValue(ALUC_AdvQryBldr, ALParamObj.UserID, null);
            ALUC_AdvQryBldr.GetType().GetProperty("Language").SetValue(ALUC_AdvQryBldr, ALParamObj.Language, null);
            ALUC_AdvQryBldr.GetType().GetProperty("TenantName").SetValue(ALUC_AdvQryBldr, ALParamObj.TenantName, null);
            ALUC_AdvQryBldr.GetType().GetProperty("TempPath").SetValue(ALUC_AdvQryBldr, ALSolutionCommon.GetTempPath(ALParamObj.SessionID), null);
            ALUC_AdvQryBldr.GetType().GetProperty("ConfigurationType").SetValue(ALUC_AdvQryBldr, "AdHocReportConfig", null);
            ALUC_AdvQryBldr.GetType().GetProperty("FieldsSectionVisible").SetValue(ALUC_AdvQryBldr, true, null);
            ALUC_AdvQryBldr.GetType().GetProperty("CalendarFieldsSectionVisible").SetValue(ALUC_AdvQryBldr, false, null);

            str_ConfigurationXmlPath = ALSolutionCommon.GetConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "", "EntityTypeConfiguration");
            str_TemplateDirectoryPath = InstallationDir + "\\" + ALParamObj.TenantName + "\\" + ALParamObj.SolutionName + "\\Base Setup\\Channels\\Xml";
            
            ALUC_AdvQryBldr.GetType().GetProperty("EntityConfigurationXmlPath").SetValue(ALUC_AdvQryBldr, str_ConfigurationXmlPath, null);
            ALUC_AdvQryBldr.GetType().GetProperty("TemplateDirectoryPath").SetValue(ALUC_AdvQryBldr, str_TemplateDirectoryPath, null);

            //This is commented. In the initial code this was not present but added when CalendarAdmin introduced.
            //Due to this the issue 8634 is raised. So commenting it. Code is tested after this commented.
            //Its working properly for both DashboardAdmin and CalendarAdmin.
            //if (!(String.IsNullOrEmpty(str_TargetObject)))
            //    ALUC_AdvQryBldr.GetType().GetProperty("EntityName").SetValue(ALUC_AdvQryBldr, str_TargetObject, null);

            if ((!(String.IsNullOrEmpty(SelectedEventOption))) && (SelectedEventOption.Equals("FormSelectionChanged")))
            {
                methodinfo_Object = ALUC_AdvQryBldr.GetType().GetMethod("ResetControlEx");
                if (methodinfo_Object != null)
                    errorxml_ErrorResult = (String)methodinfo_Object.Invoke(ALUC_AdvQryBldr, objarr_Params);
            }

            if (!(String.IsNullOrEmpty(SelectedReportType)))
                ALUC_AdvQryBldr.GetType().GetProperty("ChannelType").SetValue(ALUC_AdvQryBldr, SelectedReportType, null);
            else
                ALUC_AdvQryBldr.GetType().GetProperty("ChannelType").SetValue(ALUC_AdvQryBldr, null, null);

            if (ALUC_AdvQryBldr.GetType().GetProperty("AjaxDataObject") != null)
                ALUC_AdvQryBldr.GetType().GetProperty("AjaxDataObject").SetValue(ALUC_AdvQryBldr, Ajax, null);
            
            switch (ScreenMode)
            {
                case "EDIT":
                    if (xmlnode_ChannelConfig != null)
                    {
                        ALUC_AdvQryBldr.GetType().GetProperty("EntityInputConfigurationNode").SetValue(
                                ALUC_AdvQryBldr, xmlnode_ChannelConfig.FirstChild, null);

                        //Set the READONLY property value based on the user role.
                        if ((!(String.IsNullOrEmpty(UserRoleName))) && (UserRoleName.ToUpper() == "READER"))
                            ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, true, null);
                        else
                            ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, false, null);
                    }
                    else
                    {
                        ALUC_AdvQryBldr.GetType().GetProperty("EntityInputConfigurationNode").SetValue(ALUC_AdvQryBldr, null, null);

                        //Set the READONLY property value based on the user role.
                        if ((!(String.IsNullOrEmpty(UserRoleName))) && (UserRoleName.ToUpper() == "READER"))
                            ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, true, null);
                        else
                            ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, false, null);
                    }

                    break;

                default:
                    ALUC_AdvQryBldr.GetType().GetProperty("EntityInputConfigurationNode").SetValue(ALUC_AdvQryBldr, null, null);

                    //Set the READONLY property value based on the user role.
                    if ((!(String.IsNullOrEmpty(UserRoleName))) && (UserRoleName.ToUpper() == "READER"))
                        ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, true, null);
                    else
                        ALUC_AdvQryBldr.GetType().GetProperty("IsReadOnly").SetValue(ALUC_AdvQryBldr, false, null);

                    break;
            }

            ViewState["ChannelType"] = SelectedReportType;

            Div_AdvQryBldr.Controls.Clear();
            Div_AdvQryBldr.Controls.Add(ALUC_AdvQryBldr);
            if (Div_AdvQryBldr.Controls.Count > 2)
                Div_AdvQryBldr.Controls.RemoveAt(1);
        }

        //setAdHocReportConfigFormControl() method is used to set the properties to the form control used for Adhoc reporting.
        private void setAdHocReportConfigFormControl()
        {
            XML         xml_FormControlInput    = null;
            XML         xml_SearchOutput        = null;
            ERRORXML    errorxml_Output         = null;

            FormFacadeObject = new FormFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            FormFacadeObject.ShowConfiguredFields   = true;
            FormFacadeObject.SolutionName           = ALParamObj.SolutionName;
            FormFacadeObject.FormName               = "AdHocReport";

            Form_AdHocConfig.TenantName             = ALParamObj.TenantName;
            Form_AdHocConfig.ConfigurationXMLType   = CustomControls.Form.InputType.XMLPath;
            Form_AdHocConfig.ConfigurationXML       = ALSolutionCommon.GetFormConfigurationXMLPath(ALParamObj.TenantName, ALParamObj.SolutionName, "AdHocReport_FormConfiguration");
            Form_AdHocConfig.SectionHeading         = Localization.GetString("ALResource", ALParamObj.TenantName, "AdhocReport_FormSectionHeading");

            if (!IsPostBack)
            {
                switch (ScreenMode)
                {
                    case "CREATE":
                        errorxml_Output = FormFacadeObject.GetFormInputXML(ScreenMode, "", out xml_FormControlInput, out ObjectInfoCollection);
                        if (!(String.IsNullOrEmpty(errorxml_Output)))
                        {
                            redirectToErrorPage("ERR_AdhocReport_RetrieveFormInput", true);
                        }
                        break;

                    case "EDIT":
                        xml_SearchOutput = getFormDataSearchResultSetXML();
                        if (String.IsNullOrEmpty(xml_SearchOutput))
                        {
                            redirectToErrorPage("ERR_AdhocReport_RetrieveData", true);
                        }

                        m_ReportName = ALCommon.GetInnerValue(ref xml_SearchOutput, "<REPORTNAME>", "</REPORTNAME>");
                        ALSolutionCommon.DecodeSpecialCharacters(ref m_ReportName);
                        ReportName = m_ReportName;

                        errorxml_Output = FormFacadeObject.GetFormInputXML("READONLY", xml_SearchOutput, out xml_FormControlInput, out ObjectInfoCollection);
                        if (!(String.IsNullOrEmpty(errorxml_Output)))
                        {
                            redirectToErrorPage("ERR_AdhocReport_RetrieveFormInput", true);
                        }

                        break;

                    //case "READONLY":
                    //    PrepareSearchInputXML(null);
                    //    DashboardFacade.DoSearch(ref SearchInputXML, ref SearchOutputXML);

                    //    //Check whether calendar is task calendar
                    //    if (SearchOutputXML.Contains("CalendarID=\"WorkbenchTasksCalendar\"") || SearchOutputXML.Contains("CalendarID='WorkbenchTasksCalendar'"))
                    //    {
                    //        bool_ISTaskCalendar = true;
                    //    }

                        //SearchOutputXML = SearchOutputXML.Replace("<SEARCHOUTPUTXML>", "");
                        //SearchOutputXML = SearchOutputXML.Replace("</SEARCHOUTPUTXML>", "");
                        //FormFacadeObject.GetFormInputXML(ScreenMode, SearchOutputXML, out xml_FormControlInput, out ObjectInfoCollection);

                    //    break;
                }
                if (ObjectInfoCollection != null)
                    ViewState["ObjectInfoList"] = ObjectInfoCollection;
                Form_AdHocConfig.InputXMLType = Assetlink.CustomControls.Form.InputType.InnerXML;
                Form_AdHocConfig.InputXML = xml_FormControlInput;
            }
        }

        //getFormDataSearchResultSetXML() method is used to retrieve the data results for Adhoc report instance which is required for form control.
        private XML getFormDataSearchResultSetXML()
        {
            SearchConfiguration facade_SearchConfiguration  = null;
            String              str_InsertString            = null;
            XML                 xml_SearchInput             = null;
            XML                 xml_SearchOutput            = null;
            ERRORXML            errorxml_Output             = null;

            errorxml_Output = FormFacadeObject.GetHierarchicalSearchXML(ScreenMode, out xml_SearchInput);
            if ((!(String.IsNullOrEmpty(errorxml_Output))) || (String.IsNullOrEmpty(xml_SearchInput)))
            {
                redirectToErrorPage("ERR_AdhocReport_RetrieveSearchInput", true);
            }

            str_InsertString = @"<FILTERLIST>
			            <FILTER>
				            <FILTERID>1</FILTERID>
				            <EXPRESSION>REPORTID</EXPRESSION>
				            <OPERATOR>EQUALTO</OPERATOR>
				            <EXPRESSIONVALUE>" + ReportID + @"</EXPRESSIONVALUE>
				            <ISCASESENSITIVE>False</ISCASESENSITIVE>
			            </FILTER>
                    </FILTERLIST>";
            xml_SearchInput = ALCommon.InsertAfter(ref xml_SearchInput, "<TARGETENTITY>", ref str_InsertString);
            if (!(xml_SearchInput.Contains("<AUTHORIZATIONREQUIRED>")))
            {
                str_InsertString = "<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>";
                xml_SearchInput = ALCommon.InsertBefore(ref xml_SearchInput, "<TARGETENTITY>", ref str_InsertString);
            }

            str_InsertString = "<SESSIONINFO UserID='" + ALParamObj.UserID + "' Language='" + ALParamObj.Language
                    + "' TenantName='" + ALParamObj.TenantName + "' SessionID='" + ALParamObj.SessionID + "'/>";
            xml_SearchInput = ALCommon.InsertBefore(ref xml_SearchInput, "<TARGETENTITY>", ref str_InsertString);

            if (!(xml_SearchInput.Contains("REPORTCONFIGXML")))
            {
                str_InsertString = "<EXPRESSION><FIELDNAME>REPORTCONFIGXML</FIELDNAME><ALIAS>REPORTCONFIGXML</ALIAS></EXPRESSION>";
                xml_SearchInput = ALCommon.InsertBefore(ref xml_SearchInput, "</SELECTLIST>", ref str_InsertString);
            }

            facade_SearchConfiguration = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            errorxml_Output = facade_SearchConfiguration.GetHierarchialData(xml_SearchInput, ref xml_SearchOutput);
            if (!String.IsNullOrEmpty(errorxml_Output))
            {
                redirectToErrorPage("ERR_AdhocReport_RetrieveData", true);
            }

            ViewState["AdHocConfigSearchOutputXML"] = xml_SearchOutput;
            return (xml_SearchOutput);
        }

        private void PrepareSearchInputXML(String CurrentChannelType)
        {
            //StringBuilder InputXML = null;
            //int           Index    = -1;

            //InputXML = new StringBuilder();
            //XmlNode PageListConfigNode  = ListconfigurationDoc.SelectSingleNode("//Configuration[@Name='Channel_Selected_by_Pages']");
            //PageListConfigNode.InnerXml = PageListConfigNode.InnerXml.Replace("$CHANNELID$", ChannelID);
            //ConfigurationInpuputXML     = PageListConfigNode.OuterXml;

            //ListControlFacade   = new ListFacade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            //SearchInputXML      = ListControlFacade.PrepareSearchInputXml(PageListConfigNode.OuterXml);
            //SearchInputXML      = PrepareSearchInputXmlString(SearchInputXML);
            
            //#region attaching additional search fields

            //Index = SearchInputXML.IndexOf("</RELATEDENTITYFIELDS>");
            //InputXML.AppendFormat("{0}",SearchInputXML.Substring(0,(Index +"</RELATEDENTITYFIELDS>".Length)));

            ////Prepare XML to retrieve the Calendar ID for the current Channel in case of CalendarAdmin or if the type is Calendar.
            //if ((!(String.IsNullOrEmpty(CurrentChannelType))) && (CurrentChannelType.Equals("Calendar")))
            //{
            //    InputXML.AppendFormat("<RELATEDENTITYFIELDS NavigationPath=\"CHANNEL.CALENDAR_CHANNEL\">");
            //    InputXML.AppendFormat("<FIELD>CALENDARID</FIELD>");
            //    InputXML.AppendFormat("</RELATEDENTITYFIELDS>");
            //}

            //// preapare XML for static FACETS
            //InputXML.AppendFormat("<RELATEDFACETFIELDS NavigationPath=\"STATICCHANNEL.CHANNEL_STATIC\">");
            //InputXML.AppendFormat("<FIELD>STATICCHANNELID</FIELD>");
            //InputXML.AppendFormat("<FIELD>STATICCHANNELTYPE</FIELD>");
            //InputXML.AppendFormat("<FIELD FileRequired=\"Yes\" TransferMode=\"SYNCRONOUS\">STATICCHANNELFILE</FIELD>");
            //InputXML.AppendFormat("<FIELD FileRequired=\"Yes\" TransferMode=\"SYNCRONOUS\">XSLFILE</FIELD>");
            //InputXML.AppendFormat("<FIELD>ATTACHMENTS</FIELD>");
            //InputXML.AppendFormat("<FIELD>HEIGHT</FIELD>");
            //InputXML.AppendFormat("</RELATEDFACETFIELDS>");

            //// preapare XML for Weblink FACETS
            //InputXML.AppendFormat("<RELATEDFACETFIELDS NavigationPath=\"WEBLINKCHANNEL.CHANNEL_WEBLINKCREATION\">");
            //InputXML.AppendFormat("<FIELD>WEBLINKCHANNELID</FIELD>");
            //InputXML.AppendFormat("<FIELD>DISPLAYTEXT</FIELD>");
            //InputXML.AppendFormat("<FIELD>DESCRIPTION</FIELD>");
            //InputXML.AppendFormat("<FIELD>LINKURL</FIELD>");
            //InputXML.AppendFormat("<FIELD>TARGET</FIELD>");
            //InputXML.AppendFormat("<FIELD>ISSYSTEMURL</FIELD>");
            //InputXML.AppendFormat("<FIELD>GROUPNAME</FIELD>");
            //InputXML.AppendFormat("<FIELD>LAYOUT</FIELD>");
            //InputXML.AppendFormat("<FIELD FileRequired=\"Yes\" TransferMode=\"SYNCRONOUS\">WEBLINKFILE</FIELD>");
            //InputXML.AppendFormat("</RELATEDFACETFIELDS>");

            //InputXML.AppendFormat("{0}",SearchInputXML.Substring((Index +"</RELATEDENTITYFIELDS>".Length)));
            //SearchInputXML = InputXML.ToString();

            //#endregion
        }

        //saveAdHocReportConfigDetails() method is used to save the AdHoc Report Configuration.
        private void saveAdHocReportConfigDetails(XML AdHocReportInstanceXML, out String RetrievedReportID)
        {
            MethodInfo      methodinfo_Object       = null;
            Object[]        objarr_Params           = null;
            Report_Facade   facade_AdHocReport      = null;
            String          str_ConfigDetails       = null;
            String          str_InsertString        = null;
            ERRORXML        errorxml_ErrorResult    = null;

            RetrievedReportID = null;
            methodinfo_Object = ALUC_AdvQryBldr.GetType().GetMethod("GetOutputConfigurationXML");
            if (methodinfo_Object != null)
            {
                objarr_Params           = new Object[1];
                objarr_Params[0]        = str_ConfigDetails;
                errorxml_ErrorResult    = (String)methodinfo_Object.Invoke(ALUC_AdvQryBldr, objarr_Params);
                if ((String.IsNullOrEmpty(errorxml_ErrorResult)) && objarr_Params[0] != null && (!(String.IsNullOrEmpty(objarr_Params[0].ToString()))))
                {
                    facade_AdHocReport = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                    AdHocReportInstanceXML = AdHocReportInstanceXML.Replace("</ADHOCREPORT>", "<REPORTCONFIGXML>" + objarr_Params[0].ToString() + "</REPORTCONFIGXML></ADHOCREPORT>");
                    switch (ScreenMode)
                    {
                        case "CREATE":
                            {
                                errorxml_ErrorResult = facade_AdHocReport.CreateAdHocReport(ref AdHocReportInstanceXML);
                                if (!(String.IsNullOrEmpty(errorxml_ErrorResult)))
                                {
                                    redirectToErrorPage("ERR_AdhocReport_SaveConfigDetails", true);
                                    return;
                                }

                                ReportID = RetrievedReportID = ALCommon.GetInnerValue(ref AdHocReportInstanceXML, "<REPORTID>", "</REPORTID>");
                            }
                            break;

                        case "EDIT":
                            {
                                RetrievedReportID = ReportID;
                                if (!(AdHocReportInstanceXML.Contains("<REPORTID>")))
                                {
                                    str_InsertString = "<REPORTID>" + ReportID + "</REPORTID>"; ;
                                    AdHocReportInstanceXML = ALCommon.InsertAfter(ref AdHocReportInstanceXML, "<ADHOCREPORT>", ref str_InsertString);
                                }

                                errorxml_ErrorResult = facade_AdHocReport.UpdateAdHocReport(ref AdHocReportInstanceXML);
                                if (!(String.IsNullOrEmpty(errorxml_ErrorResult)))
                                {
                                    redirectToErrorPage("ERR_AdhocReport_SaveConfigDetails", true);
                                    return;
                                }
                            }
                            break;
                    }
                }
            }
        }

        #region Schedule List Xml functions
        private void PrepareScheduleListDataXml()
        {
            String      ErrorXML                        = null;
            String      ScheduleListInputXML            = null;
            ListFacade  ListFacadeObj                   = null;

            String AdhocReportScheduledConfigXml        = null;
            String SearchInputXml                       = null;
            String SearchOutputXml                      = null;
            SearchConfiguration facade_SearchConfiguration = null;
            XmlDocument SearchInputXmlDoc               = null;
            
            ListFacadeObj                   = new ListFacade(ALParamObj.SessionID.ToString(), ALParamObj.UserID.ToString(), ALParamObj.TenantName, ALParamObj.Language);
            AdhocReportScheduledConfigXml   = ListFacadeObj.GetConfigurationXML(ALParamObj.SolutionName, null, "AdhocReportScheduled_List");
            SearchInputXml                  = ListFacadeObj.GetHierarchicalSearchXML(AdhocReportScheduledConfigXml);
            SearchInputXml                  = SearchInputXml.Replace("$REPORTID$", ReportID);
            SearchInputXml                  = SearchInputXml.Replace("<SEARCH>", "<SEARCH><SESSIONINFO Language='" + ALParamObj.Language + "' TenantName='" + ALParamObj.TenantName + "' UserID='" + ALParamObj.UserID + "' SessionID='" + ALParamObj.SessionID + "'/>");
            SearchInputXmlDoc               = new XmlDocument();
            SearchInputXmlDoc.LoadXml(SearchInputXml);
            SearchInputXmlDoc.SelectSingleNode("SEARCH/TARGETENTITY/RELATIONSHIP[@ NAME='SCHRPT_LASTRUN']/SELECTLIST/EXPRESSION/FIELDNAME[text()='LASTRUNINSTANCEDATETIME']").InnerText = "INSTANCEDATETIME";
            SearchInputXmlDoc.SelectSingleNode("SEARCH/TARGETENTITY/RELATIONSHIP[@ NAME='SCHRPT_NEXTRUN']/SELECTLIST/EXPRESSION/FIELDNAME[text()='NEXTRUNINSTANCEDATETIME']").InnerText = "INSTANCEDATETIME";

            facade_SearchConfiguration      = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            ErrorXML                        = facade_SearchConfiguration.GetHierarchialData(SearchInputXmlDoc.OuterXml, ref SearchOutputXml);

            if (ErrorXML == null)
            {
                SearchOutputXml = "<SEARCHOUTPUT>" + SearchOutputXml + "</SEARCHOUTPUT>";
                ListFacadeObj.PrepareListXml(AdhocReportScheduledConfigXml, SearchOutputXml, out ScheduleListInputXML);
            }

            if (ErrorXML ==null && !String.IsNullOrEmpty(ScheduleListInputXML))
            {
                #region Check for the Access Roles
                XmlDocument ScheduleListInputXMLDoc = new XmlDocument();
                ScheduleListInputXMLDoc.LoadXml(ScheduleListInputXML);
                foreach (XmlNode ScheduleRow in ScheduleListInputXMLDoc.SelectNodes("//Row[Column[@Name='USERID']/text() != '" + ALParamObj.UserID + "']"))
                {
                    ((XmlElement)ScheduleRow).SetAttribute("IsDisabled", "TRUE");
                    ((XmlElement)ScheduleRow.SelectSingleNode("Column[@Name='SCHEDREPORTINSTANCENAME']")).SetAttribute("UIType", "text");
                }
                #endregion

                ProcessListXMLForStatus(ref ScheduleListInputXMLDoc);
                ReportScheduleList.Source = ScheduleListInputXMLDoc.OuterXml;
            }
            else
            {
                RedirectToErrorPage("Err_AdhocReport_Configuration_PrepareSchedulelist_Error", ErrorXML);   
            }
        }

        private void ProcessListXMLForStatus(ref XmlDocument xmlDoc_ScheduleList)
        {
            String              strErrorHTML            = null;            
            XmlNodeList         xmlItr_Row              = null;            

            xmlItr_Row = xmlDoc_ScheduleList.SelectNodes("//Row");
            if (xmlItr_Row != null && xmlItr_Row.Count > 0)
            {
                foreach (XmlNode RowNode in xmlItr_Row)
                {
                    XmlNode StatusNode = RowNode.SelectSingleNode("Column[@Name='STATUS']");
                    if (StatusNode != null)
                    {
                        if (StatusNode.InnerXml.ToUpper() == "ERROR")
                        {
                            XmlNode InstanceIDNode = RowNode.SelectSingleNode("Column[@Name='SCHEDREPORTINSTANCEID']");
                            XmlNode SCHRptIDNode = RowNode.SelectSingleNode("Column[@Name='SCHEDREPORTID']");
                            strErrorHTML = "<table width='100%' cellpadding='0' cellspacing='0'><tr><td><a class='AgTextLinkRed' href=\"javascript:OnErrorClick('" + ALSpecialCharHandling.EncodeSpecialCharactersForJS(Popup_ErrorInfo.ShowPopupWindowJavascript) + "','" + InstanceIDNode.InnerXml + "','" + SCHRptIDNode.InnerXml + "')\">" + Localization.GetString("ALResource", ALParamObj.TenantName, "Error_Shared") + "</a></td><td>|</td><td><a href=\"javascript:__doPostBack('" + Hidden_ReSubmitBtn.UniqueID + "','" + SCHRptIDNode.InnerXml + "')\" class=\"SmallTextLink\"> " + Localization.GetString("ALResource", ALParamObj.TenantName, "Resubmit_Shared") + "</a></td></tr></table>";
                            StatusNode.InnerText = strErrorHTML;
                            ((XmlElement)StatusNode).SetAttribute("UIType", "HTML");
                            ((XmlElement)StatusNode).SetAttribute("HTMLDecoding", "TRUE");
                            //((XmlElement)StatusNode).SetAttribute("DisplayTextID", "NH_237976665446031_48899695803543");
                        }
                        else if (StatusNode.InnerXml.ToUpper() == "RESUBMITTED" || StatusNode.InnerXml.ToUpper() == "SENDING NOTIFICATION")
                        {
                            StatusNode.InnerXml = "In Progress";
                            ((XmlElement)RowNode).SetAttribute("IsDisabled", "TRUE");
                            ((XmlElement)StatusNode).SetAttribute("DisplayTextID", "NH_237976665446031_94488602666089");
                        }
                    }
                    else
                    {
                        String strStatus = "<Column Name='STATUS' DisplayTextID='NH_237976665446031_3866595030773'>Scheduled</Column>";
                        RowNode.CreateNavigator().AppendChild(strStatus);
                    }
                    if (RowNode.SelectSingleNode("Column[@Name='SCHEDTYPE']") != null && RowNode.SelectSingleNode("Column[@Name='SCHEDTYPE']").InnerXml == "Run Once")
                    {
                        XmlNode RunCycleColumn = RowNode.SelectSingleNode("Column[@Name='RUNCYCLE']");
                        RunCycleColumn.InnerXml = "Run Once";
                        ((XmlElement)RunCycleColumn).SetAttribute("DisplayTextID", "NH_89625371561772_120550645377979");
                    }                    
                }
            }
        }
        #endregion

        #region Functions to Setups User / Group Selection Controls
        private void SetControls()
        {
           if (!IsPostBack)
            {
                SetAdhocReportAuthorSelectionControl(null);

                SetAdhocReportReaderSelectionControl(null);

                SetAdhocReportAuthorGroupSelectionControl(null);

                SetAdhocReportReaderGroupSelectionControl(null);
            }
        }

        private void SetAdhocReportAuthorSelectionControl(XPathNavigator AuthorsNode)
        {
            StringBuilder sb_SelectedUserXML = null;

            sb_SelectedUserXML = new StringBuilder(100);
            if (AuthorsNode != null)
                sb_SelectedUserXML.AppendFormat("<UserSelectionXML><SelectedUsersList>{0}</SelectedUsersList>", AuthorsNode.InnerXml);
            else
                sb_SelectedUserXML.AppendFormat("<UserSelectionXML><SelectedUsersList></SelectedUsersList>");

            sb_SelectedUserXML.Append("</UserSelectionXML>");

            ALUC_AdhocReportAuthor.TenantName             = ALParamObj.TenantName;
            ALUC_AdhocReportAuthor.SessionID              = ALParamObj.SessionID;
            ALUC_AdhocReportAuthor.UserID                 = ALParamObj.UserID;
            ALUC_AdhocReportAuthor.Language               = ALParamObj.Language;
            ALUC_AdhocReportAuthor.InstanceName           = "AdhocReport_Authors";
            ALUC_AdhocReportAuthor.Layout                 = "LIST";
            ALUC_AdhocReportAuthor.SolutionName           = ALParamObj.SolutionName;
            ALUC_AdhocReportAuthor.UserSelection_Mode     = "BUTTON";
            ALUC_AdhocReportAuthor.OnlyUserIDs            = "FALSE";
            ALUC_AdhocReportAuthor.SelectionMode          = "MultiSelect";
            ALUC_AdhocReportAuthor.ConfigListName         = "AdhocReportAuthorList";
            ALUC_AdhocReportAuthor.FirstTime              = "TRUE";
            ALUC_AdhocReportAuthor.Mode                   = (UserRoleName.ToUpper() == "READER")? "READONLY" : "EDIT";;
            ALUC_AdhocReportAuthor.PrevSelectedUserXML = sb_SelectedUserXML.ToString();
        }
        
        private void SetAdhocReportReaderSelectionControl(XPathNavigator ReadersNode)
        {
            StringBuilder sb_SelectedUserXML = null;

            sb_SelectedUserXML = new StringBuilder(100);
            if (ReadersNode != null)
                sb_SelectedUserXML.AppendFormat("<UserSelectionXML><SelectedUsersList>{0}</SelectedUsersList>", ReadersNode.InnerXml);
            else
                sb_SelectedUserXML.AppendFormat("<UserSelectionXML><SelectedUsersList></SelectedUsersList>");

            sb_SelectedUserXML.Append("</UserSelectionXML>");

            ALUC_AdhocReportReader.TenantName             = ALParamObj.TenantName;
            ALUC_AdhocReportReader.SessionID              = ALParamObj.SessionID;
            ALUC_AdhocReportReader.UserID                 = ALParamObj.UserID;
            ALUC_AdhocReportReader.Language               = ALParamObj.Language;
            ALUC_AdhocReportReader.InstanceName           = "AdhocReport_Readers";
            ALUC_AdhocReportReader.Layout                 = "LIST";
            ALUC_AdhocReportReader.SolutionName           = ALParamObj.SolutionName;
            ALUC_AdhocReportReader.UserSelection_Mode     = "BUTTON";
            ALUC_AdhocReportReader.OnlyUserIDs            = "FALSE";
            ALUC_AdhocReportReader.SelectionMode          = "MultiSelect";
            ALUC_AdhocReportReader.ConfigListName         = "AdhocReportReaderList";
            ALUC_AdhocReportReader.FirstTime              = "TRUE";
            ALUC_AdhocReportReader.Mode                   = (UserRoleName.ToUpper() == "READER")? "READONLY" : "EDIT";; 
            ALUC_AdhocReportReader.PrevSelectedUserXML = sb_SelectedUserXML.ToString();
        }

        private void SetAdhocReportAuthorGroupSelectionControl(XPathNavigator AuthorGroupNode)
        {
            StringBuilder sb_SelectedGroupXML = new StringBuilder();

            if (AuthorGroupNode != null)
                sb_SelectedGroupXML.AppendFormat("<GroupSelectionXML><SelectedGroupsList>{0}</SelectedGroupsList>", AuthorGroupNode.InnerXml);
            else
                sb_SelectedGroupXML.AppendFormat("<GroupSelectionXML><SelectedGroupsList></SelectedGroupsList>");

            sb_SelectedGroupXML.Append("</GroupSelectionXML>");

            ALGC_AdhocReportAuthorGroup.InstanceName          = "AdhocReportAuthor_Group";
            ALGC_AdhocReportAuthorGroup.TenantName            = ALParamObj.TenantName;
            ALGC_AdhocReportAuthorGroup.SessionID             = ALParamObj.SessionID;
            ALGC_AdhocReportAuthorGroup.UserID                = ALParamObj.UserID;
            ALGC_AdhocReportAuthorGroup.Language              = ALParamObj.Language;
            ALGC_AdhocReportAuthorGroup.Layout                = "LIST";
            ALGC_AdhocReportAuthorGroup.SolutionName          = ALParamObj.SolutionName;
            ALGC_AdhocReportAuthorGroup.ConfigListName        = "AdhocReportAuthorGrouplist";
            ALGC_AdhocReportAuthorGroup.GroupSelection_Mode   = "BUTTON";
            ALGC_AdhocReportAuthorGroup.OnlyGroupIDs          = "FALSE";
            ALGC_AdhocReportAuthorGroup.SelectionMode         = "MultiSelect";
            ALGC_AdhocReportAuthorGroup.Mode                  = (UserRoleName.ToUpper() == "READER")? "READONLY" : "EDIT";;
            ALGC_AdhocReportAuthorGroup.FirstTime             = "TRUE";
            ALGC_AdhocReportAuthorGroup.PrevSelectedGroupXML  = sb_SelectedGroupXML.ToString();
        }

        private void SetAdhocReportReaderGroupSelectionControl(XPathNavigator ReaderGroupNode)
        {
            StringBuilder sb_SelectedGroupXML = new StringBuilder();

            if (ReaderGroupNode != null)
                sb_SelectedGroupXML.AppendFormat("<GroupSelectionXML><SelectedGroupsList>{0}</SelectedGroupsList>", ReaderGroupNode.InnerXml);
            else
                sb_SelectedGroupXML.AppendFormat("<GroupSelectionXML><SelectedGroupsList></SelectedGroupsList>");

            sb_SelectedGroupXML.Append("</GroupSelectionXML>");

            ALGC_AdhocReportReaderGroup.InstanceName          = "AdhocReportReader_Group";
            ALGC_AdhocReportReaderGroup.TenantName            = ALParamObj.TenantName;
            ALGC_AdhocReportReaderGroup.SessionID             = ALParamObj.SessionID;
            ALGC_AdhocReportReaderGroup.UserID                = ALParamObj.UserID;
            ALGC_AdhocReportReaderGroup.Language              = ALParamObj.Language;
            ALGC_AdhocReportReaderGroup.Layout                = "LIST";
            ALGC_AdhocReportReaderGroup.SolutionName          = ALParamObj.SolutionName;
            ALGC_AdhocReportReaderGroup.ConfigListName        = "AdhocReportReaderGrouplist";
            ALGC_AdhocReportReaderGroup.GroupSelection_Mode   = "BUTTON";
            ALGC_AdhocReportReaderGroup.OnlyGroupIDs          = "FALSE";
            ALGC_AdhocReportReaderGroup.SelectionMode         = "MultiSelect";
            ALGC_AdhocReportReaderGroup.Mode                  =  (UserRoleName.ToUpper() == "READER")? "READONLY" : "EDIT";
            ALGC_AdhocReportReaderGroup.FirstTime             = "TRUE";
            ALGC_AdhocReportReaderGroup.PrevSelectedGroupXML  = sb_SelectedGroupXML.ToString();
        }
        #endregion

        private String GetBackURL()
        {
            String BackURL = null;
            BackURL = "/" + ALParamObj.TenantName + "/" + ALParamObj.SolutionName + "/ASPX/AdHocReportConfigurationList.aspx?SessionID=" + ALParamObj.SessionID + "&UserID=" + ALParamObj.UserID + "&Language=" + ALParamObj.Language + "&TenantName=" + ALParamObj.TenantName + "&SolutionName=" + ALParamObj.SolutionName;
            return BackURL;
        }

        private String PrepareEditPageURL()
        {
            String EditPageURL = null;
            EditPageURL = "/" + ALParamObj.TenantName + "/" + ALParamObj.SolutionName + "/ASPX/AdHocReport_Create_Edit.aspx?SessionID=" + ALParamObj.SessionID + "&UserID=" + ALParamObj.UserID + "&Language=" + ALParamObj.Language + "&TenantName=" + ALParamObj.TenantName + "&SolutionName=" + ALParamObj.SolutionName;
            EditPageURL += "&AdhocReportID=" + ReportID + "&ScreenMode=EDIT&FromScreen=AdHocReportConfiguration";
            return SolutionCommonObj.EncodeURLForResponseRedirects(EditPageURL);
        }

        #region Group User Hndler Function
        private void HandleAddUserRole(String ListInstanceName, String RelationShipName)
        {
            XML                 UserSelectionXML        = null;
            XPathDocument       UserDoc                 = null;
            XPathNavigator      UserNav                 = null;
            XPathNodeIterator   UserList                = null;
            StringBuilder       AdhocReportUpdateXML    = null;
            Boolean             IsUserPresent           = false;
            XML                 ErrorXML                = null;
            StringBuilder       sb_SelectedUserXML      = null;
            Report_Facade       Report_FacadeObj        = null;

            #region Preprea User List
            AdhocReportUpdateXML    = new StringBuilder(200);
            sb_SelectedUserXML      = new StringBuilder(200);

            switch (ListInstanceName.ToUpper())
            {
                case "ADHOCREPORT_AUTHORS":
                    UserSelectionXML = ALUC_AdhocReportAuthor.GetOutputUserList(ListInstanceName);
                    break;
                case "ADHOCREPORT_READERS":
                    UserSelectionXML = ALUC_AdhocReportReader.GetOutputUserList(ListInstanceName);
                    break;
            }
            
            UserDoc                 = new XPathDocument(new StringReader(UserSelectionXML));
            UserNav                 = UserDoc.CreateNavigator();

            AdhocReportUpdateXML.AppendFormat(@"<ADHOCREPORT>
                                             <REPORTID>{0}</REPORTID>", ReportID);
            UserList = UserNav.Select("UserSelectionXML/SelectedUsersList/USER[@DISASSOCIATE='TRUE']/USERID");
            if (UserList.Count > 0)
            {
                IsUserPresent = true;
            }
            while (UserList.MoveNext())
            {
                AdhocReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' MODIFIED='TRUE' DISASSOCIATE='TRUE'>{0}</" + RelationShipName + ">", UserList.Current.InnerXml);
            }
            UserList = UserNav.Select("UserSelectionXML/SelectedUsersList/USER[@NEW='TRUE']/USERID");
            if (UserList.Count > 0)
            {
                IsUserPresent = true;
            }
            while (UserList.MoveNext())
            {
                AdhocReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' MODIFIED='TRUE'>{0}</" + RelationShipName + ">", UserList.Current.InnerXml);
            }
            #endregion

            #region Update Users Roles
            if (IsUserPresent)
            {
                AdhocReportUpdateXML.AppendFormat(@"</ADHOCREPORT>");

                Report_FacadeObj                = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                String StrReportUpdateXML       = AdhocReportUpdateXML.ToString();
                ErrorXML                        = Report_FacadeObj.UpdateAdHocReport(ref StrReportUpdateXML);

                if (ErrorXML == null)
                {
                    GetAdhocReport_RoleSpecificData(ReportID, RelationShipName, "USER");
                    EventGenerator = "ADHOCCONFIGTABCLICK";
                }
                else
                {
                    RedirectToErrorPage("Err_AdhocReport_Configuration_AddUser_Error", ErrorXML);
                }
            }
            #endregion
        }

        private void HandleDeleteUserRole(String ListInstanceName, String RelationShipName)
        {
            XML                     UserSelectionXML    = null;
            XPathDocument           UserDoc             = null;
            XPathNavigator          UserNav             = null;
            XPathNodeIterator       UserList            = null;
            StringBuilder           ReportUpdateXML      = null;
            XML                     UpdateXML           = null;
            XML                     ErrorXML            = null;
            Boolean                 IsUserPresent       = false;
            Report_Facade           Report_FacadeObj    = null;

            #region Preprea User List
            ReportUpdateXML         = new StringBuilder(100);

            switch (ListInstanceName.ToUpper())
            {
                case "ADHOCREPORT_AUTHORS":
                    UserSelectionXML = ALUC_AdhocReportAuthor.GetOutputUserList(ListInstanceName);
                    break;
                case "ADHOCREPORT_READERS":
                    UserSelectionXML = ALUC_AdhocReportReader.GetOutputUserList(ListInstanceName);
                    break;
            }

            UserDoc                 = new XPathDocument(new StringReader(UserSelectionXML));
            UserNav                 = UserDoc.CreateNavigator();

            ReportUpdateXML.AppendFormat("<ADHOCREPORT><REPORTID>{0}</REPORTID>", ReportID);

            UserList = UserNav.Select("UserSelectionXML/SelectedUsersList/USER[@DISASSOCIATE='TRUE']/USERID");
            if (UserList.Count > 0)
            {
                IsUserPresent = true;
            }

            while (UserList.MoveNext())
            {
                ReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</" + RelationShipName + ">", UserList.Current.InnerXml);
            }
            #endregion

            #region Update Users Roles
            if (IsUserPresent == true)
            {
                ReportUpdateXML.AppendFormat("</ADHOCREPORT>");
                UpdateXML   = ReportUpdateXML.ToString();

                Report_FacadeObj                = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                String StrReportUpdateXML       = ReportUpdateXML.ToString();
                ErrorXML                        = Report_FacadeObj.UpdateAdHocReport(ref StrReportUpdateXML);
                if (ErrorXML == null)
                {
                    GetAdhocReport_RoleSpecificData(ReportID, RelationShipName, "USER");
                    EventGenerator = "ADHOCCONFIGTABCLICK";
                }
                else
                {
                    RedirectToErrorPage("Err_AdhocReport_Configuration_DeleteUser_Error", ErrorXML);
                }
            }
            #endregion
        }

        private void HandleAddGroupRole(String ListInstanceName, String RelationShipName)
        {
            XML                 GroupSelectionXML       = null;
            XPathDocument       GroupDoc                = null;
            XPathNavigator      GroupNav                = null;
            XPathNodeIterator   GroupList               = null;
            StringBuilder       AdhocReportUpdateXML    = null;
            Boolean             IsGroupPresent          = false;
            XML                 ErrorXML                = null;
            StringBuilder       sb_SelectedGroupXML     = null;
            Report_Facade       Report_FacadeObj        = null;

            #region Preprea Group List
            AdhocReportUpdateXML    = new StringBuilder(200);
            sb_SelectedGroupXML      = new StringBuilder(200);

            switch (ListInstanceName.ToUpper())
            {
                case "ADHOCREPORTAUTHOR_GROUP":
                    GroupSelectionXML = ALGC_AdhocReportAuthorGroup.GetOutputGroupList(ListInstanceName);
                    break;
                case "ADHOCREPORTREADER_GROUP":
                    GroupSelectionXML = ALGC_AdhocReportReaderGroup.GetOutputGroupList(ListInstanceName);
                    break;
            }
            
            GroupDoc                 = new XPathDocument(new StringReader(GroupSelectionXML));
            GroupNav                 = GroupDoc.CreateNavigator();

            AdhocReportUpdateXML.AppendFormat(@"<ADHOCREPORT>
                                             <REPORTID>{0}</REPORTID>", ReportID);
            GroupList = GroupNav.Select("GroupSelectionXML/SelectedGroupsList/GROUP[@DISASSOCIATE='TRUE']/GROUPID");
            if (GroupList.Count > 0)
            {
                IsGroupPresent = true;
            }
            while (GroupList.MoveNext())
            {
                AdhocReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' MODIFIED='TRUE' DISASSOCIATE='TRUE'>{0}</" + RelationShipName + ">", GroupList.Current.InnerXml);
            }
            GroupList = GroupNav.Select("GroupSelectionXML/SelectedGroupsList/GROUP[@NEW='TRUE']/GROUPID");
            if (GroupList.Count > 0)
            {
                IsGroupPresent = true;
            }
            while (GroupList.MoveNext())
            {
                AdhocReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' MODIFIED='TRUE'>{0}</" + RelationShipName + ">", GroupList.Current.InnerXml);
            }
            #endregion

            #region Update Group Users Roles
            if (IsGroupPresent)
            {
                AdhocReportUpdateXML.AppendFormat(@"</ADHOCREPORT>");

                Report_FacadeObj                = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                String StrReportUpdateXML       = AdhocReportUpdateXML.ToString();
                ErrorXML                        = Report_FacadeObj.UpdateAdHocReport(ref StrReportUpdateXML);

                if (ErrorXML == null)
                {
                    GetAdhocReport_RoleSpecificData(ReportID, RelationShipName, "GROUP");
                    EventGenerator = "ADHOCCONFIGTABCLICK";
                }
                else
                {
                    RedirectToErrorPage("Err_AdhocReport_Configuration_AddGroup_Error", ErrorXML);
                }
            }
            #endregion
        }

        private void HandleDeleteGroupRole(String ListInstanceName, String RelationShipName)
        {
            XML                     GroupSelectionXML   = null;
            XPathDocument           GroupDoc            = null;
            XPathNavigator          GroupNav            = null;
            XPathNodeIterator       GroupList           = null;
            StringBuilder           ReportUpdateXML     = null;
            XML                     UpdateXML           = null;
            XML                     ErrorXML            = null;
            Boolean                 IsGroupPresent      = false;
            Report_Facade           Report_FacadeObj    = null;

            #region Preprea Group List
            ReportUpdateXML         = new StringBuilder(100);

            switch (ListInstanceName.ToUpper())
            {
                case "ADHOCREPORTAUTHOR_GROUP":
                    GroupSelectionXML = ALGC_AdhocReportAuthorGroup.GetOutputGroupList(ListInstanceName);
                    break;
                case "ADHOCREPORTREADER_GROUP":
                    GroupSelectionXML = ALGC_AdhocReportReaderGroup.GetOutputGroupList(ListInstanceName);
                    break;
            }

            GroupDoc                 = new XPathDocument(new StringReader(GroupSelectionXML));
            GroupNav                 = GroupDoc.CreateNavigator();

            ReportUpdateXML.AppendFormat("<ADHOCREPORT><REPORTID>{0}</REPORTID>", ReportID);

            GroupList = GroupNav.Select("GroupSelectionXML/SelectedGroupsList/GROUP[@DISASSOCIATE='TRUE']/GROUPID");
            if (GroupList.Count > 0)
            {
                IsGroupPresent = true;
            }

            while (GroupList.MoveNext())
            {
                ReportUpdateXML.AppendFormat("<" + RelationShipName + " TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</" + RelationShipName + ">", GroupList.Current.InnerXml);
            }
            #endregion

            #region Update Group Users Roles
            if (IsGroupPresent == true)
            {
                ReportUpdateXML.AppendFormat("</ADHOCREPORT>");
                UpdateXML   = ReportUpdateXML.ToString();

                Report_FacadeObj                = new Report_Facade(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
                String StrReportUpdateXML       = ReportUpdateXML.ToString();
                ErrorXML                        = Report_FacadeObj.UpdateAdHocReport(ref StrReportUpdateXML);
                if (ErrorXML == null)
                {
                    GetAdhocReport_RoleSpecificData(ReportID, RelationShipName, "GROUP");
                    EventGenerator = "ADHOCCONFIGTABCLICK";
                }
                else
                {
                    RedirectToErrorPage("Err_AdhocReport_Configuration_DeleteGroup_Error", ErrorXML);
                }
            }
            #endregion
        }
        #endregion

        private void GetAdhocReport_RoleSpecificData(String AdhocReortID, String RoleToFetch, String RoleType)
        {
            #region Member Variables
            StringBuilder       FilterXML                       = null;
            ERRORXML            ErrorXML                        = null;
            SearchConfiguration SearchObj                       = null;
            XML                 AdhocReportXml                  = null;
            XPathDocument       AdhocReportDoc                  = null;
            XPathNavigator      AdhocReportNav                  = null;
            #endregion

            SearchObj                       = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            FilterXML                       = new StringBuilder(100);

            #region Search Input Xml
            FilterXML.AppendFormat(@"<SEARCH>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETOBJECT>ADHOCREPORT</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'/>
                                        <TARGETENTITY>
                                            <FILTERLIST>
                                                <FILTER>
	                                                <FILTERID>1</FILTERID>
	                                                <EXPRESSION>REPORTID</EXPRESSION>
	                                                <OPERATOR>EQUALTO</OPERATOR>
	                                                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
	                                                <FIELDNAME>REPORTID</FIELDNAME>
	                                                <ALIAS>REPORTID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>", ALParamObj.Language, ALParamObj.TenantName, ALParamObj.UserID, ALParamObj.SessionID, ReportID);
                
            
                FilterXML.Append("<RELATIONSHIP NAME='" + RoleToFetch + "'>");
                if (RoleType.Equals("GROUP"))
                {
                    FilterXML.AppendFormat(@"<SELECTLIST>
	                                            <EXPRESSION>
	                                                <FIELDNAME>GROUPID</FIELDNAME>
	                                                <ALIAS>GROUPID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
	                                                <FIELDNAME>GROUPNAME</FIELDNAME>
	                                                <ALIAS>GROUPNAME</ALIAS>
                                                </EXPRESSION>
                                                </SELECTLIST>
                                              </RELATIONSHIP>"); 
                }
                else
                {
                    FilterXML.AppendFormat(@"<SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>USERID</FIELDNAME>
                                                    <ALIAS>USERID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>LOGINNAME</FIELDNAME>
                                                    <ALIAS>LOGINNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>FIRSTNAME</FIELDNAME>
                                                    <ALIAS>FIRSTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>LASTNAME</FIELDNAME>
                                                    <ALIAS>LASTNAME</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>EMAILID</FIELDNAME>
                                                    <ALIAS>EMAILID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>");
                
                }
            
            FilterXML.Append("</TARGETENTITY></SEARCH>");
            #endregion

            ErrorXML    = SearchObj.GetHierarchialData(FilterXML.ToString(), ref AdhocReportXml);
            if (ErrorXML == null)
            {
                AdhocReportDoc            = new XPathDocument(new System.IO.StringReader(AdhocReportXml));
                AdhocReportNav            = AdhocReportDoc.CreateNavigator();
            }

            SetAdhocReportRole(AdhocReportNav, RoleToFetch);
        }

        private void GetAdhocReport_AllRoleData(String AdhocReortID)
        {
            #region Member Variables
            
            StringBuilder       FilterXML                       = null;
            ERRORXML            ErrorXML                        = null;
            SearchConfiguration SearchObj                       = null;
            XML                 AdhocReportXml                  = null;
            XPathDocument       AdhocReportDoc                  = null;
            XPathNavigator      AdhocReportNav                  = null;
            #endregion

            SearchObj                       = new SearchConfiguration(ALParamObj.SessionID, ALParamObj.UserID, ALParamObj.TenantName, ALParamObj.Language);
            FilterXML                       = new StringBuilder(100);

            #region Search Input Xml
            FilterXML.AppendFormat(@"<SEARCH>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                        <TARGETOBJECT>ADHOCREPORT</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'/>
                                        <TARGETENTITY>
                                            <FILTERLIST>
                                                <FILTER>
	                                                <FILTERID>1</FILTERID>
	                                                <EXPRESSION>REPORTID</EXPRESSION>
	                                                <OPERATOR>EQUALTO</OPERATOR>
	                                                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
	                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
	                                                <FIELDNAME>REPORTID</FIELDNAME>
	                                                <ALIAS>REPORTID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>", ALParamObj.Language, ALParamObj.TenantName, ALParamObj.UserID, ALParamObj.SessionID, ReportID);
                    
                        FilterXML.Append(@"<RELATIONSHIP NAME='ADHOCREPORT_AUTHOR'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>USERID</FIELDNAME>
                                                        <ALIAS>USERID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>LOGINNAME</FIELDNAME>
                                                        <ALIAS>LOGINNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>FIRSTNAME</FIELDNAME>
                                                        <ALIAS>FIRSTNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>LASTNAME</FIELDNAME>
                                                        <ALIAS>LASTNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>EMAILID</FIELDNAME>
                                                        <ALIAS>EMAILID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                          </RELATIONSHIP>");

                        FilterXML.Append(@"<RELATIONSHIP NAME='ADHOCREPORT_READER'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>USERID</FIELDNAME>
                                                        <ALIAS>USERID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>LOGINNAME</FIELDNAME>
                                                        <ALIAS>LOGINNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>FIRSTNAME</FIELDNAME>
                                                        <ALIAS>FIRSTNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>LASTNAME</FIELDNAME>
                                                        <ALIAS>LASTNAME</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>EMAILID</FIELDNAME>
                                                        <ALIAS>EMAILID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                          </RELATIONSHIP>");

                        FilterXML.Append(@"<RELATIONSHIP NAME='ADHOCREPORT_AUTHORGROUP'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GROUPID</FIELDNAME>
                                                        <ALIAS>GROUPID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GROUPNAME</FIELDNAME>
                                                        <ALIAS>GROUPNAME</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                          </RELATIONSHIP>");

                        FilterXML.Append(@"<RELATIONSHIP NAME='ADHOCREPORT_READERGROUP'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GROUPID</FIELDNAME>
                                                        <ALIAS>GROUPID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GROUPNAME</FIELDNAME>
                                                        <ALIAS>GROUPNAME</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                          </RELATIONSHIP>");
            
                FilterXML.Append("</TARGETENTITY></SEARCH>");
            #endregion

            ErrorXML    = SearchObj.GetHierarchialData(FilterXML.ToString(), ref AdhocReportXml);
            if (ErrorXML == null)
            {
                AdhocReportDoc            = new XPathDocument(new System.IO.StringReader(AdhocReportXml));
                AdhocReportNav            = AdhocReportDoc.CreateNavigator();
                SetAdhocReportRole(AdhocReportNav, "ALL");
            }
        }

        private void SetAdhocReportRole(XPathNavigator AdhocReportUsers, String SetRole)
        {
            #region Variables
            XPathNavigator      AuthorsNode         = null;
            XPathNavigator      ReadersNode         = null;
            XPathNavigator      AuthorGroupNode     = null;
            XPathNavigator      ReaderGroupNode     = null;
            Object              Obj                 = null;

            #endregion
            if (AdhocReportUsers != null)
            {
                switch(SetRole.ToUpper()) 
                {
                    case "ADHOCREPORT_AUTHOR":
                        AuthorsNode     = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_AUTHOR"); 
                        SetAdhocReportAuthorSelectionControl(AuthorsNode);
                        break;
                    case "ADHOCREPORT_READER":
                        ReadersNode     = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_READER"); 
                        SetAdhocReportReaderSelectionControl(ReadersNode);
                        break;
                    case "ADHOCREPORT_AUTHORGROUP":
                        AuthorGroupNode = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_AUTHORGROUP"); 
                        SetAdhocReportAuthorGroupSelectionControl(AuthorGroupNode);
                        break;
                    case "ADHOCREPORT_READERGROUP":
                        ReaderGroupNode = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_READERGROUP"); 
                        SetAdhocReportReaderGroupSelectionControl(ReaderGroupNode);
                        break;
                    case "ALL":
                        AuthorsNode         = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_AUTHOR");
                        SetAdhocReportAuthorSelectionControl(AuthorsNode);
                    
                        ReadersNode         = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_READER");
                        SetAdhocReportReaderSelectionControl(ReadersNode);
                    
                        AuthorGroupNode     = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_AUTHORGROUP");
                        SetAdhocReportAuthorGroupSelectionControl(AuthorGroupNode);
                    
                        ReaderGroupNode     = AdhocReportUsers.SelectSingleNode("//ADHOCREPORT_READERGROUP");
                        SetAdhocReportReaderGroupSelectionControl(ReaderGroupNode);
                        break;
                }

                #region Access Control for User Roles Tab
                /*Obj = this.ViewState["AccessString"];
                if (Obj != null)
                {
                    AccessString = (AccessCode)Obj;
                }
                if (AdhocReportFacadeObj.isAccessFound(AccessString, AdhocReportAccessConstant.CanWrite) && !IsAdhocReportLocked)
                {
                    AdhocReportOwner.Mode         = "EDIT";
                    AdhocReportAuthors.Mode       = "EDIT";
                    AdhocReportAuthorGroups.Mode  = "EDIT";
                    AdhocReportReaders.Mode       = "EDIT";
                    AdhocReportReaderGroups.Mode  = "EDIT";
                }
                else
                {
                    AdhocReportOwner.Mode         = "READONLY";
                    AdhocReportAuthors.Mode       = "READONLY";
                    AdhocReportAuthorGroups.Mode  = "READONLY";
                    AdhocReportReaders.Mode       = "READONLY";
                    AdhocReportReaderGroups.Mode  = "READONLY";
                }*/
                #endregion

            }
        }

        private String GetBackUrl()
        {
            if (SolutionCommonObj == null)
                SolutionCommonObj = new ALSolutionCommon();

            return CurrentPageURL.Substring(0, CurrentPageURL.IndexOf("?") + 1) + SolutionCommonObj.DecryptURL(CurrentPageURL.Substring(CurrentPageURL.IndexOf("=") + 1));
        }

        private void RedirectToErrorPage(String ErrorResoirceKey,String ErrorXml)
        {
            String ErrorPageURL = null;
            SolutionCommonObj.PrepareErrorPageUrl(ALParamObj.SolutionName, null, null, ErrorXml, Localization.GetString("ALErrorResource", ALParamObj.TenantName, ErrorResoirceKey), null, GetBackUrl(), ref ErrorPageURL);
            if (!IsPostBack)
                Response.Redirect(ErrorPageURL);
            else
                Ajax.Redirect(Page, ErrorPageURL);
        }

        #endregion
    }
}
