<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                xmlns:msxml="urn:schemas-microsoft-com:xslt"
                xmlns:myUtils="arpv:MyUtils"
                extension-element-prefixes="msxml">

  <xsl:param name="ParamTargetObjectName"/>
  <xsl:param name="ParamArrayGridColumns" select="/.."/>
  <xsl:param name="ParamGroupDataTypeMap" select="/.."/>
  <xsl:param name="LocalizedTotalFor"/>
  <xsl:param name="IsAggregationRequired"/>
  <xsl:output method="html" indent="yes"/>
  
  <xsl:template match="/">    
      <table cellpadding="0" cellspacing="0" class="AdHocReportPrintViewBorder">
        <xsl:variable name="ROOTNAME">        
          <xsl:text>GROUPDATACOLLECTION</xsl:text>
        </xsl:variable>

        <xsl:call-template name="CreateHeader">
        </xsl:call-template>
        
        <xsl:for-each select="child::*/child::*[msxml:node-set($ROOTNAME)]">
          <xsl:call-template name="CreateGridNodes">
            <xsl:with-param name="CurrLevel" select="0"></xsl:with-param>          
          </xsl:call-template>
        </xsl:for-each>
      </table>
  </xsl:template>

  <xsl:template name="CreateHeader">
    <tr>
      <xsl:for-each select="$ParamArrayGridColumns/GRIDCOLUMNS/COLUMN/FIELDNAME">
        
        <xsl:variable name="var_Width">
          <xsl:value-of select="parent::*/WIDTH/text()"/>
          <xsl:text>px</xsl:text>
        </xsl:variable>

        <xsl:variable name="varxpath_GridObjectField" select="text()"></xsl:variable>

        <td width="{$var_Width}" class="AGHeaderWithWrapAdHocReportPrintView AGHeadTextWithWrapAdHocReportPreview">          
          <xsl:value-of select="parent::*/FIELDTITLE/text()"/>
        </td>
        
      </xsl:for-each>
    </tr>
  </xsl:template>
  
  <xsl:template name="CreateGridNodes">
    <xsl:param name="CurrLevel"></xsl:param>
    
    <xsl:variable name="varxpath_DataTargetNodeObject">
      <xsl:copy-of select="."/>
    </xsl:variable>
    
    <tr>      
      <xsl:for-each select="$ParamArrayGridColumns/GRIDCOLUMNS/COLUMN/FIELDNAME">

        <xsl:variable name="varxpath_GridObjectField" select="text()"></xsl:variable>
        <xsl:variable name="var_ColumnObjectName">
          <xsl:value-of select="parent::*/OBJECTNAME/text()"/>
        </xsl:variable>
        <xsl:variable name="var_ColumnRelationName">
          <xsl:value-of select="parent::*/RELATIONSHIPNAME/text()"/>
        </xsl:variable>
        <xsl:variable name="var_Aggregation">
          <xsl:value-of select="parent::*/AGGREGATION/text()"/>
        </xsl:variable>
        <xsl:variable name="var_FieldDataType">
          <xsl:value-of select="parent::*/FIELDDATATYPE/text()"/>
        </xsl:variable>
        <xsl:variable name="var_Width">
          <xsl:value-of select="parent::*/WIDTH/text()"/>
          <xsl:text>px</xsl:text>
        </xsl:variable>
        
        <xsl:variable name="varxpath_DataTempObjectForGroup" select="msxml:node-set($varxpath_DataTargetNodeObject)/*">
        </xsl:variable>

        <xsl:variable name="varxpath_DataTempObject">
          <xsl:choose>
            <xsl:when test="($var_ColumnObjectName = $var_ColumnRelationName) or (name($varxpath_DataTempObjectForGroup) = 'GROUPDATA')">
              <xsl:copy-of select="msxml:node-set($varxpath_DataTargetNodeObject)/*"></xsl:copy-of>
            </xsl:when>
            <xsl:otherwise>
              <xsl:copy-of select="msxml:node-set($varxpath_DataTargetNodeObject)/*/child::*[name() = $var_ColumnRelationName]/child::*[name() = $var_ColumnObjectName]"></xsl:copy-of>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="varxpath_DataTargetObject" select="msxml:node-set($varxpath_DataTempObject)/*">
        </xsl:variable>

        <xsl:variable name="varxpath_FieldNameCombination">
          <xsl:value-of select="$var_ColumnRelationName"/>
          <xsl:value-of select ="'.'"/>
          <xsl:value-of select ="$varxpath_GridObjectField"/>
        </xsl:variable>

        <xsl:variable name="GroupType">
          <xsl:choose>
            <xsl:when test="name($varxpath_DataTargetObject) = 'GROUPDATA'">
              <xsl:choose>
                <xsl:when test="$CurrLevel = '0'">
                  <xsl:text>Group</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>SubGroup</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:when>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="DataType">
          <xsl:choose>
            <xsl:when test="($var_FieldDataType = 'CURRENCY') or ($var_FieldDataType = 'NUMBER') or ($var_FieldDataType = 'FLOAT')">
              <xsl:text>Number</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>Text</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>

        <xsl:variable name="GridNodeFieldValue">
          <xsl:choose>
            <xsl:when test="name($varxpath_DataTargetObject) = 'GROUPDATA'">
              <xsl:choose>
                <xsl:when test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                  <xsl:value-of select="$varxpath_DataTargetObject/@VALUE"/>
                </xsl:when>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
              <xsl:if test="$varxpath_GridObjectField != 'GROUPDATAVALUE'">
                <xsl:choose>
                  <xsl:when test="($var_FieldDataType = 'CURRENCY') or ($var_FieldDataType = 'DATETIME') or ($var_FieldDataType = 'STRING')">
                    <xsl:if test="$var_FieldDataType = 'CURRENCY'">
                      <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/AMOUNT/text()"/>
                    </xsl:if>
                    <xsl:if test="($var_FieldDataType = 'DATETIME') or ($var_FieldDataType = 'DATE')">
                      <xsl:if test="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text() != ''">
                        <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/MONTH/text()"/>
                        <xsl:text>/</xsl:text>
                        <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/DAY/text()"/>
                        <xsl:text>/</xsl:text>
                        <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/YEAR/text()"/>
                      </xsl:if>
                      <xsl:if test="$var_FieldDataType = 'DATETIME'">
                        <xsl:if test="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/HOUR/text() != ''">
                          <xsl:text> </xsl:text>
                          <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/HOUR/text()"/>
                          <xsl:text>:</xsl:text>
                          <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/MINUTE/text()"/>
                          <xsl:text>:</xsl:text>
                          <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/SECOND/text()"/>
                        </xsl:if>
                      </xsl:if>
                    </xsl:if>
                    <xsl:if test="$var_FieldDataType = 'STRING'">
                      <xsl:variable name="RawStringValue">
                        <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/text()"/>
                      </xsl:variable>
                      <xsl:choose>
                        <xsl:when test="contains($RawStringValue,'$AL$')">
                          <xsl:value-of select="myUtils:FormatValueforDataType($var_FieldDataType, $RawStringValue)"/>
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:value-of select="$RawStringValue"/>
                        </xsl:otherwise>
                      </xsl:choose>
                    </xsl:if>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$varxpath_DataTargetObject/child::*[name() = $varxpath_GridObjectField]/text()"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
      
        <!--Added code for formatting the group value as per its datatype-->
        <xsl:variable name="FormattingDataType">
          <xsl:choose>
            <xsl:when test="name($varxpath_DataTargetObject) = 'GROUPDATA'">              
              <xsl:if test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                <xsl:value-of select="$ParamGroupDataTypeMap/GROUPS/GROUP[@level=number($CurrLevel)+1]/@DataType"/>
              </xsl:if>              
            </xsl:when>
            <xsl:otherwise>
              <xsl:if test="$varxpath_GridObjectField != 'GROUPDATAVALUE'">
                <xsl:value-of select="$var_FieldDataType"/>
              </xsl:if>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <xsl:variable name="FormattedDataValue">          
          <xsl:value-of select="myUtils:FormatValueforDataType($FormattingDataType, $GridNodeFieldValue)"/>
        </xsl:variable>

        <xsl:variable name="var_Class">
          <xsl:text>AdHocReportPreviewAGCell </xsl:text>
          <xsl:text>AdHocReportPreview</xsl:text>
          <xsl:value-of select="$GroupType"/>
          <xsl:value-of select="$DataType"/>
        </xsl:variable>
        
        <td width="{$var_Width}" class="{$var_Class}">
          
          <xsl:if test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
            <xsl:if test="$GroupType = 'SubGroup'">
              <xsl:call-template name="GetIndentationForLevel">
                <xsl:with-param name="currLevel" select="0"></xsl:with-param>
                <xsl:with-param name="GivenLevel" select="$CurrLevel"></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:if>

          <xsl:choose>
            <xsl:when test="$FormattedDataValue = ''">
              <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$FormattedDataValue"/>
            </xsl:otherwise>
          </xsl:choose>           
          
        </td>
        
      </xsl:for-each>
    </tr>
    
    <xsl:choose>      
      <xsl:when test="count(child::GROUPDATA) > 0">
        <xsl:for-each select="child::GROUPDATA">
          <xsl:call-template name="CreateGridNodes">
            <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
          </xsl:call-template>            
        </xsl:for-each>
      </xsl:when>      
      <xsl:otherwise>
        <xsl:for-each select="child::*[local-name() = $ParamTargetObjectName]">
          <xsl:call-template name="CreateGridNodes">
            <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
          </xsl:call-template>            
        </xsl:for-each>          
      </xsl:otherwise>
    </xsl:choose>
  
    <xsl:call-template name="CreateAggregationFooter">
      <xsl:with-param name="CurrLevel" select="number($CurrLevel)+1"></xsl:with-param>
      <xsl:with-param name="varxpath_DataTargetObject" select="$varxpath_DataTargetNodeObject"></xsl:with-param>
    </xsl:call-template>  
  </xsl:template>

  <xsl:template name="CreateAggregationFooter">
    <xsl:param name="CurrLevel"></xsl:param>
    <xsl:param name="varxpath_DataTargetObject"></xsl:param>

    <xsl:variable name="varxpath_DataTempObject" select="msxml:node-set($varxpath_DataTargetObject)/*">
    </xsl:variable>

    <xsl:if test="name($varxpath_DataTempObject) = 'GROUPDATA' and $IsAggregationRequired = 'TRUE'">

      <tr>
        <xsl:for-each select="$ParamArrayGridColumns/GRIDCOLUMNS/COLUMN/FIELDNAME">

          <xsl:variable name="varxpath_GridObjectField" select="text()"></xsl:variable>
          <xsl:variable name="var_ColumnObjectName">
            <xsl:value-of select="parent::*/OBJECTNAME/text()"/>
          </xsl:variable>
          <xsl:variable name="var_ColumnRelationName">
            <xsl:value-of select="parent::*/RELATIONSHIPNAME/text()"/>
          </xsl:variable>
          <xsl:variable name="var_Aggregation">
            <xsl:value-of select="parent::*/AGGREGATION/text()"/>
          </xsl:variable>
          <xsl:variable name="var_FieldDataType">
            <xsl:value-of select="parent::*/FIELDDATATYPE/text()"/>
          </xsl:variable>
          <xsl:variable name="var_Width">
            <xsl:value-of select="parent::*/WIDTH/text()"/>
            <xsl:text>px</xsl:text>
          </xsl:variable>

          <xsl:variable name="varxpath_FieldNameCombination">
            <xsl:value-of select="$var_ColumnRelationName"/>
            <xsl:value-of select ="'.'"/>
            <xsl:value-of select ="$varxpath_GridObjectField"/>
          </xsl:variable>

          <xsl:variable name="GridNodeFieldValue">
            <xsl:choose>
              <xsl:when test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
                <xsl:value-of select="$LocalizedTotalFor"/>
                <xsl:text> </xsl:text>
                <xsl:value-of select="$varxpath_DataTempObject/@VALUE"/>
                <!--<xsl:value-of select="''"/>-->
              </xsl:when>
              <xsl:otherwise>
                <xsl:choose>
                  <xsl:when test="$var_Aggregation = 'NONE'">
                    <xsl:value-of select="''"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="$varxpath_DataTempObject/attribute::*[name()=$varxpath_FieldNameCombination]"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>

          <xsl:variable name="GroupType">
            <xsl:choose>
              <xsl:when test="$CurrLevel = '1'">
                <xsl:text>Group</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>SubGroup</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>

          <xsl:variable name="var_Class">
            <xsl:text>AdHocReportPreviewAggregationFooterCell </xsl:text>
            <xsl:text>AdHocReportPreview</xsl:text>
            <xsl:value-of select="$GroupType"/>
            <xsl:text>AggregationFooter</xsl:text>
            <xsl:if test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
              <xsl:text>Title</xsl:text>
            </xsl:if>
          </xsl:variable>

          <xsl:variable name="FormattedDataValue">
            <xsl:value-of select="myUtils:FormatValueforDataType($var_FieldDataType, $GridNodeFieldValue)"/>
          </xsl:variable>

          <td width="{$var_Width}" class="{$var_Class}">

            <xsl:if test="$varxpath_GridObjectField = 'GROUPDATAVALUE'">
              <xsl:call-template name="GetIndentationForLevel">
                <xsl:with-param name="currLevel" select="1"></xsl:with-param>
                <xsl:with-param name="GivenLevel" select="$CurrLevel"></xsl:with-param>
              </xsl:call-template>
            </xsl:if>

            <xsl:choose>
              <xsl:when test="$FormattedDataValue = ''">
                <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$FormattedDataValue"/>
              </xsl:otherwise>
            </xsl:choose>

          </td>

        </xsl:for-each>        
      </tr>
    </xsl:if>
  </xsl:template>

  <xsl:template name="GetIndentationForLevel">
    <xsl:param name="currLevel"></xsl:param>
    <xsl:param name="GivenLevel"></xsl:param>

    <xsl:if test="number($currLevel) &lt; number($GivenLevel)">
      <xsl:text disable-output-escaping="yes">&amp;nbsp;&amp;nbsp;&amp;nbsp;&amp;nbsp;</xsl:text>
      <xsl:call-template name="GetIndentationForLevel">
        <xsl:with-param name="currLevel" select="number($currLevel) + 1"></xsl:with-param>
        <xsl:with-param name="GivenLevel" select="$GivenLevel"></xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>
  
</xsl:stylesheet>