# region System Includes
using System;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using System.Xml.XPath;
using System.Xml.Xsl;
using System.Configuration;
# endregion

# region Assetlink Includes
using Assetlink.Solutions;
using Assetlink.Solutions.Facade;
using Assetlink.Solutions.Facade.CommonFacade;
using Assetlink.Solutions.Facade.InfrastructureFacade;
using Assetlink.Solutions.Facade.TMSFacade;
using Assetlink.WebServices.ALReportWS;
using Assetlink.Solutions.Facade.DAMFacade;
using Assetlink.Solutions.Facade.UGMFacade;
# endregion

namespace Assetlink.Solutions.Facade.ReportFacade
{
    using XML = String;
    using ErrorXML = String;    

    public class Report_Facade : BaseFacade
    {
        # region Member Variables
        private AdvancedSearchStateManagement       AdvSearchStateObject        = null;
        private const String                        m_TypeConfigurationName     = "TYPECONFIGURATION";

        private String[]    NHValueSeprator         = {"$AL$"};
        private String[]    NHIDValueSeprator       = {"$COLON$"};
        private String[]    OldNHValueSeprator      = {","};
        private String[]    OldNHIDValueSeprator    = {":"};
        private String      m_SortOn                = null;
        private String      m_SortOrder             = null;
        private XmlNode     OperatorsXMLRootNode    = null;
        private XmlNode     SourceConfigRootNode    = null;
        # endregion

        #region Constructors

        public Report_Facade(String SessionID, String UserID, String TenantName, String CurrentLanguage)
            : base(SessionID, UserID, TenantName, CurrentLanguage)
        {

        }

        public Report_Facade(String SessionID, String UserID, String TenantName, String CurrentLanguage, String SolutionName, String ThemeName)
            : base(SessionID, UserID, TenantName, CurrentLanguage, SolutionName, ThemeName)
        {
        }

        #endregion

        #region Property
        public String TypeConfigurationName
        {
            get
            {
                return (m_TypeConfigurationName);
            }
        }

        public String SortOn
        {
            set
            {
                m_SortOn = value;
            }
            get
            {
                return (m_SortOn);
            }
        }

        public String SortOrder
        {
            set
            {
                m_SortOrder = value;
            }
            get
            {
                return (m_SortOrder);
            }
        }
        #endregion

        #region Private Methods

        private ALServiceHeader getRequestHeader()
        {
            ALServiceHeader RequestHeader = null;
            RequestHeader               = new ALServiceHeader();
            RequestHeader.SessionID     = Convert.ToUInt64(SessionID);
            RequestHeader.UserID        = Convert.ToUInt64(UserID);
            RequestHeader.Language      = CurrentLanguage;
            RequestHeader.TenantName    = this.TenantName;

            return RequestHeader;
        }

        /// <summary>
        /// This function is used to get list of assetID
        /// </summary>
        /// <param name="InputDocument"></param>
        /// <returns>Comma separated assetd ID list</returns>
        private String getIDs(XmlDocument InputDocument)
        {
            # region Local Variables
            XmlNodeList     FiledIDList = null;
            String          TempStr     = null;
            # endregion

            FiledIDList = InputDocument.SelectNodes("descendant::DIGITALASSETID");
            foreach (XmlNode IDNode in FiledIDList)
            {
                TempStr = TempStr + IDNode.InnerText + ",";
            }

            //remove last comma.
            if (!String.IsNullOrEmpty(TempStr))
                TempStr = TempStr.Substring(0, TempStr.LastIndexOf(","));
            else
                TempStr = "";

            return TempStr;
        }

        /// <summary>
        /// This function will return Search Operation XML.
        /// </summary>
        /// <param name="AssetIDList">String containing the ID list of Asset</param>        
        /// <returns>SearchOpeartionXML, null otherwise</returns>
        private String prepareSearchXML(String AssetIDList, String AssetDescrPath)
        {
            # region Local Variables
            StringBuilder       SearchInputXml      = null;
            # endregion

            #region Get AssetId list and prepare Search operation Node.
            SearchInputXml = new StringBuilder();
            SearchInputXml.Append("<SEARCH>");
            SearchInputXml.Append(getSessionInfoNode());
            SearchInputXml.Append(prepareSearchHeader());
            SearchInputXml.Append("<TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>");
            SearchInputXml.Append(prepareSortNode());
            SearchInputXml.Append(prepareFilterList("DIGITALASSETID", AssetIDList, "1"));
            SearchInputXml.Append(prepareDisplayFieldList(AssetDescrPath));
            SearchInputXml.Append("</TARGETENTITY></SEARCH>");
            #endregion

            return SearchInputXml.ToString();
        }

        /// <summary>
        /// This function returns required session information
        /// </summary>
        /// <returns>Required session information string</returns>
        private String getSessionInfoNode()
        {
            return "<SESSIONINFO TenantName=\"" + TenantName + "\" Language=\"" + CurrentLanguage + "\" UserID=\"" + UserID + "\" SessionID=\"" + SessionID + "\" />";
        }

        /// <summary>
        /// This function prepares Sorting node
        /// </summary>
        /// <returns>Sort node</returns>
        private String prepareSortNode()
        {
            # region Local Variables
            StringBuilder   SortNode            = null;
            # endregion

            # region Get sorting arguments
            SortNode        = new StringBuilder(30);
            if (!String.IsNullOrEmpty(m_SortOn))
            {
                SortNode.Append("<SORTLIST><SORTEXPRESSION>");
                SortNode.AppendFormat("<COLUMNNAME>{0}</COLUMNNAME>", m_SortOn);
                if (!String.IsNullOrEmpty(m_SortOrder))
                    SortNode.AppendFormat("<SORTORDER>{0}</SORTORDER>", SortOrder);
                SortNode.Append("</SORTEXPRESSION></SORTLIST>");
            }
            # endregion

            return SortNode.ToString();
        }


        /// <summary>
        /// This function prepares SearchHeader
        /// </summary>
        /// <returns>Search Header Node</returns>       
        private String prepareSearchHeader()
        {
            # region Local Variables
            StringBuilder   SearchHeader        = null;
            # endregion

            SearchHeader = new StringBuilder();
            SearchHeader.Append("<TARGETOBJECT>DIGITALASSET</TARGETOBJECT><TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
            SearchHeader.Append("<MAXRESULTSIZE>1000</MAXRESULTSIZE><AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>");
            SearchHeader.Append("<SEARCHSTRING/><SEARCHALLFACETS>false</SEARCHALLFACETS>");

            return SearchHeader.ToString();
        }

        /// <summary>
        /// This function prepares FilterList Node.
        /// </summary>
        /// <param name="FieldName">Filter Field Name</param>        
        /// <param name="FilterValue">Filter Value</param>
        /// <param name="FilterID">Filter ID</param>
        /// <returns>Filter List Node</returns>       
        private String prepareFilterList(String FieldName, String FilterValue, String FilterID)
        {
            # region Local Variables
            StringBuilder SearchFilter = null;
            # endregion

            SearchFilter = new StringBuilder();
            SearchFilter.Append(("<FILTERLIST>"));
            SearchFilter.AppendFormat(@"<FILTER><FILTERID>{0}</FILTERID><EXPRESSION>{1}</EXPRESSION><OPERATOR>IN</OPERATOR>
                                        <EXPRESSIONVALUE>({2})</EXPRESSIONVALUE><ISCASESENSITIVE>False</ISCASESENSITIVE>", FilterID, FieldName, FilterValue);
            SearchFilter.Append("</FILTER></FILTERLIST>");
            return SearchFilter.ToString ();
        }

        /// <summary>
        /// Prepares list of required display field
        /// </summary>
        /// <returns>List of display field</returns>
        private String prepareDisplayFieldList(String AssetDescrPath)
        {
            # region Local Variables
            StringBuilder       SeachDataList       = null;
            XmlDocument         AssetDescrXmlDoc    = null;
            XPathNavigator      AssetDescrRoot      = null;
            XPathNavigator      AssetNavigator      = null;
            XPathNavigator      EntityNavigator     = null;
            XPathNavigator      FacetNavigator      = null;
            //String              AssetDescrPath      = null;
            # endregion

            try
            {
                # region Load DAM descriptor
                if (String.IsNullOrEmpty(AssetDescrPath) == true)
                {
                    AssetDescrPath  = ALSolutionCommon.GetConfigurationXMLPath(TenantName, "DAM", "", "FacetFieldSelection");
                }
                AssetDescrXmlDoc    = new XmlDocument();
                AssetDescrXmlDoc.Load(AssetDescrPath);
                # endregion

                if (AssetDescrXmlDoc != null)
                {
                    SeachDataList   = new StringBuilder();
                    AssetDescrRoot  = AssetDescrXmlDoc.CreateNavigator();

                    # region Prepare Asset Node
                    AssetNavigator  = AssetDescrRoot.SelectSingleNode("SECTIONCONFIGURATION/DIGITALASSET");
                    SeachDataList.Append(prepareAssetNode(AssetNavigator));
                    # endregion

                    # region Prepare Related Entity Node
                    EntityNavigator = AssetNavigator.SelectSingleNode("RELATEDENTITIES");
                    SeachDataList.Append(prepareRelatedEntityNode(EntityNavigator));
                    # endregion

                    # region Prepare Related Facet Node
                    FacetNavigator  = AssetNavigator.SelectSingleNode("RELATEDFACETS");
                    SeachDataList.Append(prepareRelatedfacetNode(FacetNavigator));
                    # endregion

                    # region Prepare other required data nodes
                    SeachDataList.Append(prepareOtherInfoNode());
                    # endregion
                }
                else
                    return null;
            }
            catch (Exception)
            {
                return null;
            }
            return SeachDataList.ToString();
        }

        /// <summary>
        /// Prepare list of asset node as display fields
        /// </summary>
        /// <param name="AssetNode"></param>
        /// <returns></returns>
        private String prepareAssetNode(XPathNavigator AssetNode)
        {
            # region Local Variables
            StringBuilder   AssetData           = null;
            Boolean         IsAssetNodePresent  = false;
            # endregion

            if (AssetNode != null)
            {
                AssetData = new StringBuilder();
                AssetData.Append("<SELECTLIST>");

                # region Append Asset related fields
                foreach (XPathNavigator EntityFieldNode in AssetNode.SelectChildren(XPathNodeType.Element))
                {
                    if (!String.IsNullOrEmpty(EntityFieldNode.GetAttribute("DisplayText", "")))
                    {
                        AssetData.Append("<EXPRESSION><FIELDNAME>");
                        AssetData.Append(EntityFieldNode.LocalName);
                        AssetData.Append("</FIELDNAME><ALIAS>");
                        AssetData.Append(EntityFieldNode.LocalName);
                        AssetData.Append("</ALIAS></EXPRESSION>");

                        if (String.Compare(EntityFieldNode.LocalName, "Name", true) == 0)
                            IsAssetNodePresent = true;
                    }
                }

                if (IsAssetNodePresent == false)
                {
                    AssetData.Append("<EXPRESSION><FIELDNAME>NAME</FIELDNAME><ALIAS>NAME</ALIAS></EXPRESSION>");
                }

                //ASSET EXPIRED
                if (AssetNode.SelectChildren("ASSETEXPIRED", "").Count <= 0)
                {
                    AssetData.Append("<EXPRESSION><FIELDNAME>ASSETEXPIRED</FIELDNAME><ALIAS>ASSETEXPIRED</ALIAS></EXPRESSION>");
                }

                # endregion

                AssetData.Append("</SELECTLIST>");
            }
            else
                return null;

            return AssetData.ToString();
        }

        /// <summary>
        /// Prepare list of related entity nodes
        /// </summary>
        /// <param name="EntityNode">Entity node list from asset descriptor</param>
        /// <returns>Collection of related entity nodes</returns>
        private String prepareRelatedEntityNode(XPathNavigator EntityNode)
        {
            # region Local Variables
            StringBuilder   EntityData    = null;
            # endregion

            if (EntityNode != null)
            {
                EntityData = new StringBuilder();

                foreach (XPathNavigator RelatedEntityNode in EntityNode.SelectChildren(XPathNodeType.Element))
                {
                    if (RelatedEntityNode.LocalName.Equals("DIRECTRELATION", StringComparison.OrdinalIgnoreCase))
                    {
                        # region Append Direct Relation Node
                        foreach (XPathNavigator RelationNode in RelatedEntityNode.SelectChildren(XPathNodeType.Element))
                        {
                            EntityData.AppendFormat("<RELATIONSHIP NAME=\"{0}\">", RelationNode.LocalName);
                            EntityData.Append("<SELECTLIST>");
                            RelationNode.MoveToFirstChild();

                            foreach (XPathNavigator EntityFieldNode in RelationNode.SelectChildren(XPathNodeType.Element))
                            {
                                EntityData.Append("<EXPRESSION><FIELDNAME>");
                                EntityData.Append(EntityFieldNode.LocalName);
                                EntityData.Append("</FIELDNAME><ALIAS>");
                                EntityData.Append(EntityFieldNode.LocalName);
                                EntityData.Append("</ALIAS></EXPRESSION>");
                            }
                            EntityData.Append("</SELECTLIST>");
                            EntityData.Append("</RELATIONSHIP>");
                        }
                        # endregion
                    }
                    else if (RelatedEntityNode.LocalName.Equals("INDIRECTRELATION", StringComparison.OrdinalIgnoreCase))
                    {
                        # region Append Indirect Relation Node
                        foreach (XPathNavigator RelationNode in RelatedEntityNode.SelectChildren(XPathNodeType.Element))
                        {
                            EntityData.AppendFormat("<RELATIONSHIP NAME=\"{0}\">", RelationNode.GetAttribute("RelationWithSourceEntity", ""));
                            EntityData.AppendFormat("<RELATIONSHIP NAME=\"{0}\">", RelationNode.LocalName);
                            EntityData.Append("<SELECTLIST>"); 
                            RelationNode.MoveToFirstChild();

                            foreach (XPathNavigator EntityFieldNode in RelationNode.SelectChildren(XPathNodeType.Element))
                            {
                                EntityData.Append("<EXPRESSION><FIELDNAME>");
                                EntityData.Append(EntityFieldNode.LocalName);
                                EntityData.Append("</FIELDNAME><ALIAS>");
                                EntityData.Append(EntityFieldNode.LocalName);
                                EntityData.Append("</ALIAS></EXPRESSION>");
                            }
                            EntityData.Append("</SELECTLIST>");
                            EntityData.Append("</RELATIONSHIP>");
                            EntityData.Append("</RELATIONSHIP>");
                        }
                        # endregion
                    }
                }
            }
            else
                return null;

            return EntityData.ToString();
        }

        /// <summary>
        /// Prepare list of related facet nodes
        /// </summary>
        /// <param name="FacetNode">Facet node list from asset descriptor</param>
        /// <returns>Collection of related facet nodes</returns>
        private String prepareRelatedfacetNode(XPathNavigator FacetNode)
        {
            # region Local Variables
            StringBuilder   FacetData     = null;
            # endregion

            if (FacetNode != null)
            {
                FacetData = new StringBuilder();

                # region Append All Related Facets
                foreach (XPathNavigator RelatedFacetNode in FacetNode.SelectChildren(XPathNodeType.Element))
                {
                    FacetData.AppendFormat("<RELATIONSHIP NAME=\"{0}\">", RelatedFacetNode.LocalName);
                    FacetData.Append("<SELECTLIST>");
                    RelatedFacetNode.MoveToFirstChild();

                    foreach (XPathNavigator FacetFieldNode in RelatedFacetNode.SelectChildren(XPathNodeType.Element))
                    {
                        FacetData.Append("<EXPRESSION><FIELDNAME>");
                        FacetData.Append(FacetFieldNode.LocalName);
                        FacetData.Append("</FIELDNAME><ALIAS>");
                        FacetData.Append(FacetFieldNode.LocalName);
                        FacetData.Append("</ALIAS></EXPRESSION>");
                    }
                    FacetData.Append("</SELECTLIST>");
                    FacetData.Append("</RELATIONSHIP>");
                }
                # endregion
            }
            else
                return null;

            return FacetData.ToString();
        }

        /// <summary>
        /// Prepare list of other information nodes
        /// </summary>
        /// <returns>Collection of other information nodes</returns>
        private String prepareOtherInfoNode()
        {
            # region Local Variables
            StringBuilder OtherInfo = null;
            # endregion

            OtherInfo = new StringBuilder();

            # region Append Thumbnail Detail Node
            OtherInfo.Append("<RELATIONSHIP NAME=\"DIGITALASSET_CURRENTTHUMBNAILRENDITION\">");
            OtherInfo.Append("<SELECTLIST>");
            OtherInfo.Append("<EXPRESSION><FIELDNAME>NAME</FIELDNAME><ALIAS>THUMBNAIL_NAME</ALIAS></EXPRESSION>");
            OtherInfo.Append("<EXPRESSION><FIELDNAME FileRequired=\"NO\">ACTUALFILE</FIELDNAME><ALIAS>ACTUALFILE</ALIAS></EXPRESSION>");
            //(FileRequired = "NO"); // File will not be downloaded to outbox if value is NO
            OtherInfo.Append("<EXPRESSION><FIELDNAME>HEIGHT</FIELDNAME><ALIAS>HEIGHT</ALIAS></EXPRESSION>");
            OtherInfo.Append("<EXPRESSION><FIELDNAME>WIDTH</FIELDNAME><ALIAS>WIDTH</ALIAS></EXPRESSION>");
            OtherInfo.Append("<EXPRESSION><FIELDNAME>FILEFORMAT</FIELDNAME><ALIAS>FILEFORMAT</ALIAS></EXPRESSION>");
            OtherInfo.Append("</SELECTLIST>");
            OtherInfo.Append("</RELATIONSHIP>");
            # endregion

            return OtherInfo.ToString();
        }

        /// <summary>
        /// This function downloads thumbnails to out-box and from out box to Dowonload folder, iff not present in Dowonload folder
        /// </summary>
        /// <param name="AssetDetailsXml">Asset detail xml</param>
        /// <returns>Error xml on error else null</returns>
        private ErrorXML getThumbnails(XPathNavigator AssetDetailsXml)
        {
            # region Local Variables
            FileTransfer FTObj = null;
            FileFacade ObjFileFacade = null;
            XPathNodeIterator ThumbNodeList = null;
            XmlDocument FSCollectionDoc = null;
            XmlNodeList FSNodeList = null;
            XmlNode FSCollectionNode = null;
            XmlNode FSNode = null;
            XmlNode FileNode = null;
            XmlNode FSAttribNode = null;
            XML InputXml = null;
            ErrorXML ErrorXML = null;
            StringBuilder FilePath = null;
            StringBuilder FormatedFileName = null;
            String RenditionID = null;
            String FileName = null;
            String DownloadPath = null;
            String FileID = null;
            String SchemaID = null;
            String OnMediaServer = null;
            String MediaServerUrl = null;
            # endregion

            try
            {
                # region Get list of required thumbnails
                FilePath = new StringBuilder();
                FSCollectionDoc = new XmlDocument();
                DownloadPath = ALSolutionCommon.GetDownloadFilePath(TenantName);
                ThumbNodeList = AssetDetailsXml.Select("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTTHUMBNAILRENDITION/ASSETRENDITION/FILESYSTEM");
                FSCollectionNode = FSCollectionDoc.CreateElement("FILESYSTEMCOLLECTION");
                FSCollectionDoc.AppendChild(FSCollectionNode);

                foreach (XPathNavigator ThumbNode in ThumbNodeList) //.SelectChildren(XPathNodeType.Element))
                {
                    RenditionID = ThumbNode.SelectSingleNode("FILE/OBJECTID").Value;
                    FileName = ThumbNode.SelectSingleNode("FILE/NAME").Value;
                    FileID = ThumbNode.SelectSingleNode("FILE/FILEID").Value;
                    FilePath.AppendFormat("{0}{1}_{2}", DownloadPath, RenditionID, FileName);
                    SchemaID = ThumbNode.SelectSingleNode("FILE/SCHEMAID").Value;

                    if (ThumbNode.SelectSingleNode("FILE/ONMEDIASERVER") != null)
                        OnMediaServer = ThumbNode.SelectSingleNode("FILE/ONMEDIASERVER").Value;
                    else OnMediaServer = "FALSE";

                    if (ThumbNode.SelectSingleNode("FILE/MEDIASERVERURL") != null)
                        MediaServerUrl = ThumbNode.SelectSingleNode("FILE/MEDIASERVERURL").Value;
                    else MediaServerUrl = "";


                    if (!File.Exists(FilePath.ToString()))
                    {
                        FSNode = FSCollectionDoc.CreateElement("FILESYSTEM");
                        FSCollectionNode.AppendChild(FSNode);

                        FileNode = FSCollectionDoc.CreateElement("FILE");
                        ((XmlElement)FileNode).SetAttribute("FileRequired", "YES");
                        FSNode.AppendChild(FileNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("FIELDNAME");
                        FSAttribNode.InnerText = "ACTUALFILE";
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("FILEID");
                        FSAttribNode.InnerText = FileID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("OBJECTID");
                        FSAttribNode.InnerText = RenditionID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("OBJECTTYPE");
                        FSAttribNode.InnerText = "ENTITY";
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("SCHEMAID");
                        FSAttribNode.InnerText = SchemaID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("NAME");
                        FSAttribNode.InnerText = FileName;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("ONMEDIASERVER");
                        FSAttribNode.InnerText = OnMediaServer;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode = FSCollectionDoc.CreateElement("MEDIASERVERURL");
                        FSAttribNode.InnerText = MediaServerUrl;
                        FileNode.AppendChild(FSAttribNode);
                    }
                    FilePath.Remove(0, FilePath.Length);
                }
                # endregion

                if (FSCollectionNode.SelectNodes("FILESYSTEM").Count > 0)
                {
                    # region Get Thumbnail files to outbox
                    ObjFileFacade = new FileFacade(SessionID, UserID, TenantName, CurrentLanguage);
                    InputXml = FSCollectionDoc.OuterXml;
                    ErrorXML = ObjFileFacade.GetFile(ref InputXml);

                    if (String.IsNullOrEmpty(ErrorXML) == false) // != null && ErrorXML != "")
                    {
                        return "Get thumbnail failed";
                    }
                    # endregion

                    # region Get Thumbnail files to download folder
                    FSCollectionDoc.LoadXml(InputXml);
                    FSNodeList = FSCollectionDoc.SelectNodes("FILESYSTEMCOLLECTION/FILESYSTEM/FILE");
                    FTObj = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
                    FTObj.GetFilePath = DownloadPath;
                    FormatedFileName = new StringBuilder();

                    foreach (XmlNode FNode in FSNodeList)
                    {
                        FSAttribNode = FSCollectionDoc.CreateElement("SOURCEPATH");
                        FSAttribNode.InnerText = FNode.SelectSingleNode("PATH").InnerText;
                        FNode.AppendChild(FSAttribNode);
                        OnMediaServer = FNode.SelectSingleNode("ONMEDIASERVER").InnerText;

                        InputXml = FNode.ParentNode.OuterXml;
                        RenditionID = FNode.SelectSingleNode("OBJECTID").InnerText;
                        FileName = FNode.SelectSingleNode("NAME").InnerText;
                        FormatedFileName.AppendFormat("{0}_{1}", RenditionID, FileName);
                        if (String.Compare(OnMediaServer, "TRUE", true) == 0)
                            FTObj.UseMediaServer = true;
                        else
                            FTObj.UseMediaServer = false;
                        FTObj.DownloadFileName = FormatedFileName.ToString();
                        ErrorXML = FTObj.GetFile(ref InputXml);
                        FormatedFileName.Remove(0, FormatedFileName.Length);
                    }
                    # endregion
                }
            }
            catch (Exception)
            {
                return "Get thumbnail failed";
            }
            return null;
        }

        /// <summary>
        /// This method is used to populate the drop-down in the list control
        /// </summary>
        /// <returns>XML containing items in drop-down list</returns>
        private XML getTypeAttributeName()
        {
            #region Local Variables
            XmlNode         ConfigurationNode   = null;
            XmlNodeList     TypeNodeList        = null;
            String          ParentName          = null;
            String          RelationShipName    = null;
            String          TypeDisplayName     = null;
            String          TypeName            = null;
            StringBuilder   TypeFieldName       = null;
            #endregion

            if (AdvSearchStateObject == null)
            {
                getAdvSearchStateObject();
            }
            TypeFieldName       = new StringBuilder();
            ConfigurationNode   = AdvSearchStateObject.StateXMLDoc.SelectSingleNode("STATE/DATA[@Name='" + m_TypeConfigurationName + "']");

            if (ConfigurationNode != null)
            {
                TypeNodeList    = ConfigurationNode.SelectNodes("//Types/Type");
                foreach (XmlNode TypeNode in TypeNodeList)
                {
                    TypeName            = ((XmlElement)TypeNode).GetAttribute("Value");
                    TypeDisplayName     = ((XmlElement)TypeNode).GetAttribute("DisplayText");
                    ParentName          = ((XmlElement)TypeNode).GetAttribute("ParentObjectType");
                    RelationShipName    = ((XmlElement)TypeNode).GetAttribute("RelationshipName");

                    if (TypeDisplayName.Contains("======") == true)
                        break;

                    TypeFieldName.Append("<Item Text='" + ALSpecialCharHandling.TrimAndReplaceSpecialChars(TypeDisplayName)  + "'");
                    TypeFieldName.Append(" Value='" + RelationShipName + "'/>");
                    
                }
            }
            return TypeFieldName.ToString();
        }

        /// <summary>
        /// This method creates new row and appends it to the list XML
        /// </summary>
        /// <param name="RowNumber">Position where row to be added in list</param>
        /// <returns>All columns data for given position of row</returns>
        private String addRow(int RowNumber, String ListType)
        {
            # region Local Variables
            StringBuilder   ListData    = null;
            StringBuilder   DivHTML     = null;
            StringBuilder   AttributeValue  = null;
            # endregion

            AttributeValue = new StringBuilder();
            AttributeValue.AppendFormat("---{0}---", Localization.GetString("ALResource", TenantName, "Select_Shared"));

            ListData    = new StringBuilder();
            DivHTML     = new StringBuilder();

            ListData.AppendFormat("<Column Name=\"SequenceNo\">{0}</Column>", RowNumber);
            ListData.AppendFormat("<Column Name=\"Source\" SelectedText=\"{0}\" SelectedValue=\"\"/>", AttributeValue.ToString());
            ListData.AppendFormat("<Column Name=\"Attribute\" SelectedText=\"{0}\" SelectedValue=\"\">", AttributeValue.ToString());
            ListData.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", AttributeValue.ToString());
            ListData.Append("</Column>");
            if (String.Compare(ListType, "FILTERLIST", true) == 0)
            {
                ListData.AppendFormat("<Column Name=\"Operator\" SelectedText=\"{0}\" SelectedValue=\"\"/>", AttributeValue.ToString());
                ListData.Append("<Column Name=\"AttributeValue\">");

                #region Single Text Box
                DivHTML.AppendFormat("<div id=\"SingleTextDiv_{0}\" style=\"display: block;\">", RowNumber);
                DivHTML.AppendFormat("<input name=\"SingleText_{0}\" type=\"text\" size=\"20\" id=\"SingleText_{0}\" class=\"FormInputArea\"/>", RowNumber);
                DivHTML.Append("</div>");
                ListData.Append(ALSolutionCommon.TrimAndReplaceSpecialChars(DivHTML.ToString()));
                #endregion

                ListData.Append("</Column>");
            }

            ListData.Append("<Column Name=\"AttributeDataType\" SelectedText=\"\" SelectedValue=\"\"/>");
            ListData.Append("<Column Name=\"ObjectType\" SelectedText=\"\" SelectedValue=\"\"/>");
            ListData.Append("<Column Name=\"PrimaryKey\" SelectedText=\"\" SelectedValue=\"\"/>");
            ListData.Append("<Column Name=\"ObjectName\" SelectedText=\"\" SelectedValue=\"\"/>");
            ListData.Append("<Column Name=\"ParentEntityTypeName\" SelectedText=\"\" SelectedValue=\"\"/>");
            ListData.Append("<Column Name=\"FieldUIType\" SelectedText=\"\" SelectedValue=\"\"/>");

            return ListData.ToString();
        }

        /// <summary>
        /// This method is use to add types to state XML
        /// </summary>
        private void addTypesToStateXML(ObjectType ObjectTypeObj, String TypeName, String DisplayName, String ParentEntityTypeName, String RelationshipName, ref XmlElement TypesCollectionElement)
        {
            # region Local Variables
            TMFacade        TMSFacadeObject         = null;
            XmlDocument     AssoXMLDoc              = null;
            XmlElement      TypeElement             = null;
            String          OutputXMLResult         = null;
            ErrorXML        ErrorXML                = null;
            String          TempRelationshipName    = null;
            String          RelationshipDisplayName = null;
            StringBuilder   XPath = null;
            # endregion

            if (AdvSearchStateObject == null)
            {
                getAdvSearchStateObject();
            }
            AssoXMLDoc  = new XmlDocument();
            TypeElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Type");
            TypeElement.SetAttribute("DisplayText",DisplayName);
            TypeElement.SetAttribute("Value", RelationshipName);
            TypeElement.SetAttribute("ObjectName", TypeName);
            TypeElement.SetAttribute("RelationshipName", RelationshipName);
            TypeElement.SetAttribute("ParentEntityTypeName", ParentEntityTypeName);

            if (ObjectTypeObj == ObjectType.ENTITY)
                TypeElement.SetAttribute("ObjectType", "ENTITY");
            else
                TypeElement.SetAttribute("ObjectType", "FACET");
                        
            TypesCollectionElement.AppendChild(TypeElement);

            # region Get associated facets only for DigitalAsset and Collection
            if (TypeName == ParentEntityTypeName)
            {
                XPath = new StringBuilder();
                TMSFacadeObject = new TMFacade(SessionID, UserID, TenantName, CurrentLanguage);
                ErrorXML = TMSFacadeObject.GetAssociatedObjectTypes("ENTITY", TypeName, ref OutputXMLResult);
                if (OutputXMLResult != null)
                {
                    if (SourceConfigRootNode == null)
                    {
                        getSourceConfigRootNode();
                    }
                    AssoXMLDoc.LoadXml(OutputXMLResult);
                    foreach (XmlNode RelationTypeNode in AssoXMLDoc.SelectNodes("//FACETCOLLECTION/RELATIONSHIPTYPE"))
                    {
                        TempRelationshipName = RelationTypeNode.SelectSingleNode("RELATIONSHIPNAME").InnerText;
                        XPath.Remove(0, XPath.Length);
                        XPath.AppendFormat("//FACETEXCLUSIONLIST[RELATIONSHIPNAME='{0}']", TempRelationshipName);

                        if (SourceConfigRootNode.SelectSingleNode(XPath.ToString()) == null)
                        {
                            TypeElement             = AdvSearchStateObject.StateXMLDoc.CreateElement("Type");
                            RelationshipDisplayName = RelationTypeNode.SelectSingleNode("RELATIONSHIPDISPLAYNAME").InnerText;
                            if (!String.IsNullOrEmpty(RelationshipDisplayName))
                                TypeElement.SetAttribute("DisplayText", RelationshipDisplayName);
                            else
                                TypeElement.SetAttribute("DisplayText", TempRelationshipName);

                            TypeElement.SetAttribute("Value", TempRelationshipName);
                            TypeElement.SetAttribute("ObjectName", RelationTypeNode.SelectSingleNode("CONTAINSTYPENAME").InnerText);
                            TypeElement.SetAttribute("RelationshipName", RelationTypeNode.SelectSingleNode("RELATIONSHIPNAME").InnerText);
                            TypeElement.SetAttribute("ParentEntityTypeName",TypeName);
                            TypeElement.SetAttribute("ObjectType", "FACET");
                            TypesCollectionElement.AppendChild(TypeElement);
                        }
                    }
                }
            }
            # endregion
        }

        /// <summary>
        /// This method is used to get user defined advanced search query node
        /// </summary>
        private ErrorXML getPredefinedSearchQueryListXML(String  FacetRelationShipName, Boolean GetOnlyIfNotAlreadyPresent)
        {
            # region Local Variables
            String          UserXml                         = null;
            String          UserAttrXml                     = null;
            StringBuilder   AssoXml                         = null;
            ErrorXML        ErrorXML                        = null;
            StringBuilder   PredefinedQueryListFilePath     = null;
            UserFacade      ObjUserFacade                   = null;
            XmlDocument     PredefinedQueryListDoc          = null;
            XmlNodeList     PredefinedSearchQueryNodes      = null;
            # endregion

            AssoXml                         = new StringBuilder();
            PredefinedQueryListFilePath     = new StringBuilder();
            PredefinedQueryListDoc          = new XmlDocument();
            UserAttrXml                     = "<USER><USERID></USERID></USER>";
            PredefinedQueryListFilePath.AppendFormat("{0}{1}\\PredefinedSearchQueryList.xml", ALSolutionCommon.GetTempPath(SessionID), UserID);

            if ((!File.Exists(PredefinedQueryListFilePath.ToString())) || (!GetOnlyIfNotAlreadyPresent))
            {
                if (!String.IsNullOrEmpty(FacetRelationShipName))
                {
                    AssoXml.AppendFormat("<ASSOCIATION><{0} Type=\"RELATION\"/></ASSOCIATION>", FacetRelationShipName);
                    ErrorXML        = null;

                    ObjUserFacade   = new UserFacade(SessionID, UserID, TenantName, CurrentLanguage);

                    ErrorXML        = ObjUserFacade.GetUser(Convert.ToUInt64(UserID), UserAttrXml, AssoXml.ToString(), out UserXml);

                    if (!String.IsNullOrEmpty(UserXml))
                    {
                        PredefinedQueryListDoc.LoadXml(UserXml);
                        PredefinedQueryListDoc.Save(PredefinedQueryListFilePath.ToString());
                        PredefinedSearchQueryNodes = PredefinedQueryListDoc.SelectNodes("//DAM_ADVANCEDSEARCHQUERY");
                    }
                }
            }
            else
            {
                PredefinedQueryListDoc.Load(PredefinedQueryListFilePath.ToString());
                PredefinedSearchQueryNodes = PredefinedQueryListDoc.SelectNodes("//DAM_ADVANCEDSEARCHQUERY");
            }
            return ErrorXML;
        }

        /// <summary>
        /// This method method is used to add attributes of the selected TYPE into the ATTRIBUTE's dropdown list
        /// </summary>
        private void getObjectAttributes(String RelationshipName, String DisplayName)
        {
            # region Local Variables
            XPathDocument       TypeInfoResultXMLDoc            = null;
            XPathNavigator      XPathNavigatorObj               = null;
            XPathNodeIterator   TypeInfoFieldsNodeList          = null;
            XPathExpression     XPathExpressionObj              = null;
            XmlNode             TypeCollectionNode              = null;
            XmlNode             AttributeNode                   = null;
            XPathNavigator      PrimaryKeyNode                  = null;
            XmlNode             ConfigurationNode               = null;
            XmlElement          AttributesCollectionElement     = null;
            XmlElement          AttributeElement                = null;
            String              PrimaryKeyValue                 = null;
            String              OutputXMLResult                 = null;
            String              ObjectType                      = null;
            String              FieldName                       = null;
            ErrorXML            ErrorXML                        = null;
            String              FieldDisplayName                = null;
            String              ObjectTypeName                  = null;
            XmlNode             RelationshipNode                = null;
            XmlNodeList         FieldNameNodes                  = null;
            StringCollection    FieldNameCollection             = null;
            StringBuilder       XPath                           = null;
            TMFacade TMSFacadeObject = null;
            # endregion

            try
            {
                //Getting the object type details from the configuration XML file.
                XPath = new StringBuilder();
                FieldNameCollection = new StringCollection();

                if (SourceConfigRootNode == null)
                {
                    getSourceConfigRootNode();
                }

                //Following code ensures that only fields specified in config file are inserted into 
                //state file.
                XPath.Remove(0, XPath.Length);
                XPath.AppendFormat("//RELATIONSHIPNAME[text()={0}]",ALXmlUtil.GetXpathValueString(RelationshipName));
                RelationshipNode    = SourceConfigRootNode.SelectSingleNode(XPath.ToString());
                if (RelationshipNode != null)
                {
                    FieldNameNodes  = RelationshipNode.ParentNode.SelectNodes("FIELDCOLLECTION/FIELD/NAME");
                    foreach (XmlNode FieldNameNode in FieldNameNodes)
                        FieldNameCollection.Add(FieldNameNode.InnerText);
                }
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                ConfigurationNode = AdvSearchStateObject.StateXMLDoc.SelectSingleNode("STATE/DATA[@Name='" + m_TypeConfigurationName + "']");
                if (ConfigurationNode != null)
                {
                    //If the attribute list for the selected type is present in the XML file 
                    // then show those attributes into the ATTRIBUTE dropdown list. 
                    // Else get the list of attributes from the database 
                    // and add those attributes into the ATTRIBUTE dropdown list.
                   
                    AttributeNode   = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(
                                                "//Types/Type[@Value=" + ALXmlUtil.GetXpathValueString(RelationshipName) + "]/Attributes");
                    if (AttributeNode == null)
                    {
                        TypeCollectionNode = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(
                                                            "//Types/Type[@Value=" + ALXmlUtil.GetXpathValueString(RelationshipName) + "]");
                        if (TypeCollectionNode != null)
                        {

                            ObjectType      = ((XmlElement)TypeCollectionNode).GetAttribute("ObjectType");
                            ObjectTypeName  = ((XmlElement)TypeCollectionNode).GetAttribute("ObjectName");

                            TMSFacadeObject = new TMFacade(SessionID, UserID, TenantName, CurrentLanguage);
                            ErrorXML        = TMSFacadeObject.GetTypeInfobyName(ObjectType, ObjectTypeName,
                                                                    ref OutputXMLResult);


                            if ((ErrorXML == null) && (OutputXMLResult != null))
                            {
                                TypeInfoResultXMLDoc    = new XPathDocument(new StringReader(OutputXMLResult));
                                XPathNavigatorObj       = TypeInfoResultXMLDoc.CreateNavigator();
                                XPathExpressionObj      = XPathExpression.Compile("//FIELDCOLLECTION/FIELD");
                                XPathExpressionObj.AddSort("FIELDNAME/text()", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Text);
                                TypeInfoFieldsNodeList  = XPathNavigatorObj.Select(XPathExpressionObj);

                                if (TypeInfoFieldsNodeList.Count > 0)
                                {
                                    PrimaryKeyNode      = XPathNavigatorObj.SelectSingleNode(
                                                            "//FIELDCOLLECTION/FIELD/ISPRIMARYKEY[text()='TRUE']");
                                    if (PrimaryKeyNode != null)
                                    {
                                        PrimaryKeyNode.MoveToParent();
                                        PrimaryKeyValue = PrimaryKeyNode.SelectSingleNode("FIELDNAME").Value;
                                    }
                            
                                    AttributesCollectionElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Attributes");
                                    TypeCollectionNode.AppendChild(AttributesCollectionElement);

                                    foreach (XPathNavigator TypeInfoFieldNode in TypeInfoFieldsNodeList)
                                    {
                                        FieldName       = TypeInfoFieldNode.SelectSingleNode("FIELDNAME").Value;
                                        // This condition is added to ensure that only field specified in config file 
                                        // are shown in dropdown list in case of entity.
                                        if (FieldNameCollection.Count > 0)
                                        {
                                            if (!FieldNameCollection.Contains(FieldName))
                                                continue;
                                        }

                                        getFieldAttributes(ObjectTypeName, ObjectType, FieldName, RelationshipName, out FieldDisplayName);

                                        if ((TypeInfoFieldNode.SelectSingleNode("ISPRIMARYKEY").Value == "FALSE")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value != "BINDER")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value != "FILE")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value != "XML")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value != "XMLTEXT")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value != "CURRENCY")
                                                && (TypeInfoFieldNode.SelectSingleNode("FIELDUITYPE").Value != "HIDDEN"))
                                                //&& (TypeInfoFieldNode.SelectSingleNode("ISSEARCHFIELD").InnerText == "TRUE"))
                                        {
                                            AttributeElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Attribute");
                                            if (String.IsNullOrEmpty(FieldDisplayName))
                                            {
                                                AttributeElement.SetAttribute("DisplayText",
                                                            TypeInfoFieldNode.SelectSingleNode("DISPLAYNAME").Value);
                                            }
                                            else
                                            {
                                                AttributeElement.SetAttribute("DisplayText", FieldDisplayName);
                                            }

                                            AttributeElement.SetAttribute("Value",
                                                        TypeInfoFieldNode.SelectSingleNode("FIELDNAME").Value);
                                            AttributeElement.SetAttribute("FieldDataType",
                                                        TypeInfoFieldNode.SelectSingleNode("FIELDTYPE").Value);
                                            AttributeElement.SetAttribute("ObjectType",
                                                        TypeInfoFieldNode.SelectSingleNode("OBJECTTYPE").Value);
                                            AttributeElement.SetAttribute("PrimaryKey", PrimaryKeyValue);
                                            AttributeElement.SetAttribute("IsNamedHierarchy",
                                                        TypeInfoFieldNode.SelectSingleNode("ISNAMEDHIERARCHY").Value);
                                            AttributeElement.SetAttribute("TypeTableName",
                                                        TypeInfoFieldNode.SelectSingleNode("TABLENAME").Value);
                                            AttributeElement.SetAttribute("FieldUIType",
                                                        TypeInfoFieldNode.SelectSingleNode("FIELDUITYPE").Value);
                                           
                                            AttributesCollectionElement.AppendChild(AttributeElement);
                                        }
                                    }
                                    AdvSearchStateObject.SetStateData(m_TypeConfigurationName,
                                                                                ConfigurationNode.InnerXml);
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        /// <summary>
        /// This function is used get field attributes
        /// </summary>
        private void getFieldAttributes(String TypeName, String ObjectType, String FieldName, String RelationshipName, out String DisplayName)
        {
            XmlNode         TempNode                = null;
            XmlNode         DisplayNameNode         = null;
            
             DisplayName = null;
            
            if (SourceConfigRootNode != null)
            {
                if (TypeName.Equals(RelationshipName))
                {
                    TempNode = SourceConfigRootNode.SelectSingleNode("//TARGETENTITY/FIELDCOLLECTION/FIELD/NAME[text()='" + FieldName + "']");
                }
                else
                    if (ObjectType.Equals("ENTITY"))
                    {
                        TempNode    = SourceConfigRootNode.SelectSingleNode("//RELATEDENTITY/NAME[text()='" + TypeName + "']");
                        TempNode    = TempNode.ParentNode.SelectSingleNode("FIELDCOLLECTION/FIELD/NAME[text()='" + FieldName + "']");
                    }


                if (TempNode != null)
                {
                    DisplayNameNode = TempNode.ParentNode.SelectSingleNode("DISPLAYNAME");
                    if (DisplayNameNode != null)
                    {
                        if(DisplayNameNode.Attributes["DisplayTextID"] != null)
                            DisplayName = Localization.GetString("ALResource",TenantName,DisplayNameNode.Attributes["DisplayTextID"].Value);
                        else
                            DisplayName = DisplayNameNode.InnerText;
                    }
                    
                }
            }
        }

        /// <summary>
        /// This method is used to set the value to controls in the attribute value column  depending upon the Operator selected
        /// </summary>
        private XML setAttributeValues(XML AttributeNodeXml, String AttributeActualValue, String OperatorValue, String RowNo, String ObjectType, String TypeName, String TypeDisplayName, String AttributeName, String AttributeDataType, String FieldUIType, String AttributeDispValue)
        {
            #region Local Variables
            String[]        Separator                   = {" AND "};
            String[]        Values                      = null;
            StringBuilder   ListData                    = null;
            StringBuilder   PatameterList               = null;
            StringBuilder   NamedHierarchyFilePath      = null;
            String          NamedHierarchySelectionMode = null;
            String[]        NamedHierarchyIDValuePairs  = null;      
            StringBuilder   NamedHierarchyValue         = null;
            String          CommaSeprator               = null;
            String[]        LocalNHValueSeprator        = NHValueSeprator;
            String[]        LocalNHIDValueSeprator      = NHIDValueSeprator;
            StringBuilder   Str_Select                  = null;
            ALSolutionCommon ALSolutionCommonObj = null;
            #endregion

            Str_Select = new StringBuilder();
            Str_Select.AppendFormat("---{0}---", Localization.GetString("ALResource", TenantName, "Select_Shared"));

            ListData        = new StringBuilder();
            PatameterList   = new StringBuilder();
            ALSolutionCommonObj = new ALSolutionCommon();
            
            if (String.IsNullOrEmpty(AttributeDispValue))
                AttributeDispValue = AttributeActualValue;

            if ((isNamedHierarchyField(TypeDisplayName, AttributeName)) && ((OperatorValue.ToUpper()=="EQUALTO") || (OperatorValue.ToUpper()=="NOTEQUALTO")))
            {
                if (FieldUIType.ToUpper() == "DROPDOWNLIST")
                    NamedHierarchySelectionMode = "SINGLESELECT";
                else
                    NamedHierarchySelectionMode = "MULTISELECT";

                NamedHierarchyFilePath  = new StringBuilder();
                NamedHierarchyFilePath.AppendFormat(@"{0}\{1}\NamedHeirarchyData",ALSolutionCommon.GetTempPath(SessionID), UserID);
                NamedHierarchyFilePath.AppendFormat(@"\ReportSrch_{0}_{1}.xml",AttributeName,RowNo);

                PatameterList.Remove(0, PatameterList.Length);
                ListData.AppendFormat("<div id=\"NamedHierarchyDiv_{0}\" style=\"display: block;\">", RowNo);
                ListData.AppendFormat("<input type=\"hidden\" name=\"HiddenInput_{0}\" id=\"HiddenInput_{0}\" value=\"{1}\" />", RowNo,ALSolutionCommon.TrimAndReplaceSpecialChars( AttributeActualValue));
                ListData.Append("<table>");
                ListData.Append("<tr>");

                //Remove NamedHierarchy ID's from the AttributeActualValue
                if (!String.IsNullOrEmpty(AttributeActualValue))
                {
                    NamedHierarchyValue         = new StringBuilder();

                    // Anand : This code has been added so that old prededined queries created with "," and ":"
                    // separator also work.(As no migration of predefined query xml have been done.)
                    if (!AttributeActualValue.Contains(NHIDValueSeprator[0]))
                    {
                        LocalNHValueSeprator    = OldNHValueSeprator;
                        LocalNHIDValueSeprator  = OldNHIDValueSeprator;
                    }

                    NamedHierarchyIDValuePairs = AttributeActualValue.Split(LocalNHValueSeprator, StringSplitOptions.RemoveEmptyEntries);
                    for (int cnt = 0; cnt < NamedHierarchyIDValuePairs.Length; cnt++)
                    {
                        NamedHierarchyValue.AppendFormat("{0}{1}", CommaSeprator, ((NamedHierarchyIDValuePairs[cnt].Split(LocalNHIDValueSeprator, StringSplitOptions.RemoveEmptyEntries))[1]));
                        if (CommaSeprator == null)
                            CommaSeprator       = ", ";
                    }
                    AttributeActualValue        = NamedHierarchyValue.ToString();
                }

                ListData.Append("<td align=\"left\">");
                ListData.AppendFormat("<span id=\"Hidden_NHBtn_{0}\" class=\"Text\">{1}</span>", RowNo,ALCommon.TrimAndReplaceSpecialChars(AttributeActualValue));
                ListData.Append("</td>");
                
                ListData.Append("<td>");

                PatameterList.AppendFormat("UserID={0}&SessionID={1}&Language={2}&TenantName={3}&SolutionName={4}&HTMLContainer=Hidden_NHBtn_{5}&Hiddenip=HiddenInput_{5}&ObjectType={6}&ObjectName={7}&FieldName={8}&SelectedValueDisplayLayout=ROW&SelectedValueListXML={9}&SelectionMode={10}&ThemeName={11}",
                    UserID, SessionID, CurrentLanguage, TenantName, SolutionName, RowNo, ObjectType, TypeName, AttributeName,NamedHierarchyFilePath.ToString(),NamedHierarchySelectionMode,ThemeName);
                ListData.AppendFormat("<a href=\"Javascript:ShowNamedPopup('/{0}/common/ASPX/Tree.aspx?Param={1}');\">",
                                           TenantName, ALSolutionCommonObj.EncryptURL(PatameterList.ToString()));

                ListData.AppendFormat("<img id=\"ImgNH_{0}\" src=\"/{1}/App_Themes/{2}/Images/icon_Large_Add_Values.gif\" style=\"border-style: None; border-width: 0px;\" />", RowNo, TenantName,ThemeName);
                ListData.Append("</a>");
                ListData.Append("</td>");

                ListData.Append("</tr>");
                ListData.Append("</table>");
                ListData.Append("</div>");
            }
            else if (String.Equals(AttributeDataType, "BOOLEAN", StringComparison.CurrentCultureIgnoreCase))
            {
                #region Show Dropdown List
                ListData.AppendFormat("<div id=\"SingleDropDownDiv_{0}\" style=\"display: block;\">", RowNo);
                if (!String.IsNullOrEmpty(AttributeActualValue))
                {
                    Values = AttributeActualValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                    if (Values.Length > 0)
                    {
                        if (Values[0] != null)
                        {
                            ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"  id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                            ListData.AppendFormat("<option value=\"{0}\">{1}</option>", Str_Select.ToString(), Str_Select.ToString());
                            ListData.Append(getBooleanOptions(Values[0]));
                        }
                        else
                        {
                            ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"   id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                            ListData.AppendFormat("<option value=\"{0}\">{1}</option>",Str_Select.ToString(),Str_Select.ToString());
                            ListData.Append(getBooleanOptions(null));
                        }
                    }
                }
                else
                {
                    ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"   id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                    ListData.AppendFormat("<option value=\"{0}\">{1}</option>",Str_Select.ToString(),Str_Select.ToString());
                    ListData.Append(getBooleanOptions(null));
                }
                ListData.Append("</select>");
                ListData.Append("</div>");
                #endregion
            }
            else if (OperatorValue == "DATEMACRO")
            {
                #region Show Dropdown List
                ListData.AppendFormat("<div id=\"SingleDropDownDiv_{0}\" style=\"display: block;\">", RowNo);
                if (!String.IsNullOrEmpty(AttributeActualValue))
                {
                    Values = AttributeActualValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                    if (Values.Length > 0)
                    {
                        if (Values[0] != null)
                        {
                            ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"  id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                            ListData.AppendFormat("<option value=\"{0}\">{1}</option>",Str_Select.ToString(),Str_Select.ToString());
                            ListData.Append(getDateMacros(Values[0]));
                        }
                        else
                        {
                            ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"   id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                            ListData.AppendFormat("<option value=\"{0}\">{1}</option>",Str_Select.ToString(),Str_Select.ToString());
                            ListData.Append(getDateMacros(null));
                        }
                    }
                }
                else
                {
                    ListData.AppendFormat("<select name=\"SingleDropDown_{0}\"   id=\"SingleDropDown_{0}\" class=\"FormInputArea\">", RowNo);
                    ListData.AppendFormat("<option value=\"{0}\">{1}</option>",Str_Select.ToString(),Str_Select.ToString());
                    ListData.Append(getDateMacros(null));
                }
                ListData.Append("</select>");
                ListData.Append("</div>");
                #endregion
            }
            else if (OperatorValue != "INRANGE")//show Single control
            {
                if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))//If the Atrribute is of Date type show Date Control
                {
                    #region Show Single Calendar Control
                    //Show single Calendar Control
                    ListData.AppendFormat("<div id=\"SingleCalendarDiv_{0}\" style=\"display: block;\">", RowNo);
                    ListData.Append("<table cellspacing=\"0\" cellpadding=\"0\" >");
                    ListData.Append("<tr>");
                    ListData.Append("<td align=\"left\">");
                    if (!String.IsNullOrEmpty(AttributeDispValue))
                    {
                        Values = AttributeDispValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                        if (Values.Length > 0)
                        {
                            if (Values[0] != null)
                            {
                                ListData.AppendFormat("<input name=\"SingleCalendar_{0}\" type=\"text\" readonly=\"readonly\" id=\"SingleCalendar_{0}\" class=\"FormInputArea\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[0]));
                            }
                            else
                            {
                                ListData.AppendFormat("<input name=\"SingleCalendar_{0}\" type=\"text\" readonly=\"readonly\" id=\"SingleCalendar_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                            }
                        }
                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"SingleCalendar_{0}\" type=\"text\" readonly=\"readonly\" id=\"SingleCalendar_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                    }

                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        Values = AttributeActualValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                        if (Values.Length > 0 && Values[0] != null)
                        {
                            ListData.AppendFormat("<input type=\"hidden\" name=\"SingleHiddenText_{0}\" id=\"SingleHiddenText_{0}\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[0]));                            
                        }
                        else
                            ListData.AppendFormat("<input type=\"hidden\" name=\"SingleHiddenText_{0}\" id=\"SingleHiddenText_{0}\" />", RowNo);
                    }
                    else
                    {
                        ListData.AppendFormat("<input type=\"hidden\" name=\"SingleHiddenText_{0}\" id=\"SingleHiddenText_{0}\" />", RowNo);
                    }
                    ListData.AppendFormat("<input type=\"hidden\" name=\"__TENANTNAME_SingleCalendar_{0}\" id=\"__TENANTNAME_SingleCalendar_{0}\" value=\"{1}\" />", RowNo, TenantName);
                    ListData.AppendFormat("<input type=\"hidden\" name =\"__THEMENAME_SingleCalendar_{0}\" id=\"__THEMENAME_SingleCalendar_{0}\" value=\"{1}\" />", RowNo,ThemeName);
                    ListData.AppendFormat("<input type=\"hidden\" name=\"__CalendarTitle_SingleCalendar_{0}\" id=\"__CalendarTitle_SingleCalendar_{0}\" value=\"CalendarTitle\" />", RowNo);

                    //Add Hidden fields for Month Name, Weekdays Localized values                   
                    AddLocalizedValueForCalendar(ref ListData, "SingleCalendar", RowNo); 

                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<img height=\"1\" alt=\"\" width=\"2\" src=\"/{0}/App_Themes/{1}/images/spacer.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<a href=\"Javascript:show_calendar('SingleCalendar_{0}','SingleHiddenText_{0}','forms[0].SingleCalendar_{0}')\">", RowNo);
                    ListData.AppendFormat("<img style=\"border:0;\" alt=\"\" class=\"btn\" src=\"/{0}/App_Themes/{1}/images/cal.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</a>");
                    ListData.Append("</td>");
                    ListData.Append("</tr>");
                    ListData.Append("</table>");
                    ListData.Append("</div>");
                    #endregion
                }
                else //Show the Text Control
                {
                    #region Show single Text Box
                    //Single Text Box Control
                    ListData.AppendFormat("<div id=\"SingleTextDiv_{0}\" style=\"display: block;\">", RowNo);
                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        Values = AttributeActualValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                        if (Values.Length > 0)
                        {
                            if (Values[0] != null)
                            {
                                ListData.AppendFormat("<input name=\"SingleText_{0}\" type=\"text\" size=\"20\" id=\"SingleText_{0}\" class=\"FormInputArea\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[0]));
                            }
                            else
                            {
                                ListData.AppendFormat("<input name=\"SingleText_{0}\" type=\"text\" size=\"20\" id=\"SingleText_{0}\" class=\"FormInputArea\"  Value=\"\"/>", RowNo);
                            }
                        }
                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"SingleText_{0}\" type=\"text\" size=\"20\" id=\"SingleText_{0}\" class=\"FormInputArea\"  Value=\"\"/>", RowNo);
                    }

                    ListData.Append("</div>");
                    #endregion
                }

            }
            else
            {
                if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))//Show Multiple Date Controls
                {

                    #region Multi Calendar Control

                    //Multi Calendar
                    ListData.AppendFormat("<div id=\"MultiCalendarDiv_{0}\" style=\"display: block;\">", RowNo);
                    ListData.Append("<table cellspacing=\"0\" cellpadding=\"0\" >");
                    ListData.Append("<tr>");
                    ListData.Append("<td>");
                    if (!String.IsNullOrEmpty(AttributeDispValue))
                    {
                        Values = AttributeDispValue.Split(Separator, StringSplitOptions.None);
                        if (Values.Length > 0)
                        {
                            if (Values[0] != null)
                                ListData.AppendFormat("<input name=\"MultiCalendarValue1_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue1_{0}\" class=\"FormInputArea\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[0]));
                            else
                                ListData.AppendFormat("<input name=\"MultiCalendarValue1_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue1_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                        }
                        else
                        {
                            // This is required because incase of user does'nt select any date the " AND " comes as a result
                            // thus this code is required. 
                            ListData.AppendFormat("<input name=\"MultiCalendarValue1_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue1_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                        }

                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"MultiCalendarValue1_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue1_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                    }

                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        Values = AttributeActualValue.Split(Separator, StringSplitOptions.None);
                        if (Values.Length > 0)
                        {
                            if (Values[0] != null)
                                ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue1_{0}\" id=\"MultiHiddenTextValue1_{0}\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[0]));
                            else
                                ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue1_{0}\" id=\"MultiHiddenTextValue1_{0}\"/>", RowNo);
                        }
                        else
                            ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue1_{0}\" id=\"MultiHiddenTextValue1_{0}\"/>", RowNo);
                    }
                    else
                        ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue1_{0}\" id=\"MultiHiddenTextValue1_{0}\"/>", RowNo);

                    ListData.AppendFormat("<input type=\"hidden\" name=\"__TENANTNAME_MultiCalendarValue1_{0}\" id=\"__TENANTNAME_MultiCalendarValue1_{0}\" value=\"{1}\" />", RowNo, TenantName);
                    ListData.AppendFormat("<input type=\"hidden\" name =\"__THEMENAME_MultiCalendarValue1_{0}\" id=\"__THEMENAME_MultiCalendarValue1_{0}\" value=\"{1}\" />", RowNo,ThemeName);
                    ListData.AppendFormat("<input type=\"hidden\" name=\"__CalendarTitle_MultiCalendarValue1_{0}\" id=\"__CalendarTitle_MultiCalendarValue1_{0}\" value=\"CalendarTitle\" />", RowNo);
                    
                    //Add Hidden fields for Month Name, Weekdays Localized values                 
                    AddLocalizedValueForCalendar(ref ListData, "MultiCalendarValue1", RowNo);   

                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<img height=\"1\" alt=\"\" width=\"2\" src=\"/{0}/App_Themes/{1}/images/spacer.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<a href=\"Javascript:show_calendar('MultiCalendarValue1_{0}','MultiHiddenTextValue1_{0}','forms[0].MultiCalendarValue1_{0}')\">", RowNo);
                    ListData.AppendFormat("<img style=\"border:0;\" alt=\"\" class=\"btn\" src=\"/{0}/App_Themes/{1}/images/cal.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</a>");
                    ListData.Append("</td>");
                    ListData.Append("<td style=\"width: 5px;\" />");
                    ListData.Append("<td>");
                    ListData.Append("<span class=\"LabelText\">AND</span>");
                    ListData.Append("</td>");

                    ListData.Append("<td>");

                    if (!String.IsNullOrEmpty(AttributeDispValue))
                    {
                        Values = AttributeDispValue.Split(Separator, StringSplitOptions.None);
                        if (Values.Length > 1)
                        {
                            if (Values[1] != null)
                            {
                                ListData.AppendFormat("<input name=\"MultiCalendarValue2_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue2_{0}\" class=\"FormInputArea\" Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[1]));
                            }
                            else
                            {
                                ListData.AppendFormat("<input name=\"MultiCalendarValue2_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue2_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                            }
                        }
                        else
                        {
                            // This is required because incase of user does'nt select any date the " AND " comes as a result
                            // thus this code is required. 
                            ListData.AppendFormat("<input name=\"MultiCalendarValue2_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue2_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                        }
                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"MultiCalendarValue2_{0}\" type=\"text\" readonly=\"readonly\" id=\"MultiCalendarValue2_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                    }

                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        Values = AttributeActualValue.Split(Separator, StringSplitOptions.None);
                        if (Values.Length > 1 && Values[1] != null)
                            ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue2_{0}\" id=\"MultiHiddenTextValue2_{0}\" Value=\"{1}\"/>", RowNo, Values[1]);
                        else
                            ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue2_{0}\" id=\"MultiHiddenTextValue2_{0}\" />", RowNo);
                    }
                    else
                        ListData.AppendFormat("<input type=\"hidden\" name=\"MultiHiddenTextValue2_{0}\" id=\"MultiHiddenTextValue2_{0}\" />", RowNo);
    
                    ListData.AppendFormat("<input type=\"hidden\" name=\"__TENANTNAME_MultiCalendarValue2_{0}\" id=\"__TENANTNAME_MultiCalendarValue2_{0}\" value=\"{1}\" />", RowNo, TenantName);
                    ListData.AppendFormat("<input type=\"hidden\" name =\"__THEMENAME_MultiCalendarValue2_{0}\" id=\"__THEMENAME_MultiCalendarValue2_{0}\" value=\"{1}\" />", RowNo,ThemeName);
                    ListData.AppendFormat("<input type=\"hidden\" name=\"__CalendarTitle_MultiCalendarValue2_{0}\" id=\"__CalendarTitle_MultiCalendarValue2_{0}\" value=\"CalendarTitle\" />", RowNo);

                    //Add Hidden fields for Month Name, Weekdays Localized values
                    AddLocalizedValueForCalendar(ref ListData, "MultiCalendarValue2", RowNo); 

                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<img height=\"1\" alt=\"\" width=\"2\" src=\"/{0}/App_Themes/{1}/images/spacer.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</td>");
                    ListData.Append("<td>");
                    ListData.AppendFormat("<a href=\"Javascript:show_calendar('MultiCalendarValue2_{0}','MultiHiddenTextValue2_{0}','forms[0].MultiCalendarValue2_{0}')\">", RowNo);
                    ListData.AppendFormat("<img style=\"border:0;\" alt=\"\" class=\"btn\" src=\"/{0}/App_Themes/{1}/images/cal.gif\"/>", TenantName,ThemeName);
                    ListData.Append("</a>");
                    ListData.Append("</td>");
                    ListData.Append("</tr>");
                    ListData.Append("</table>");
                    ListData.Append("</div>");
                    #endregion
                }
                else //Show Multiple Text Controls
                {
                    #region MultiTextBox
                    //Multi Text Box
                    ListData.AppendFormat("<div id=\"MultiTextDiv_{0}\" style=\"display: block;\">", RowNo);
                    ListData.Append("<table cellspacing=\"0\" cellpadding=\"0\" >");
                    ListData.Append("<tr>");
                    ListData.Append("<td>");

                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        Values = AttributeActualValue.Split(Separator, StringSplitOptions.RemoveEmptyEntries);
                        if (Values.Length > 0)
                        {
                            if (Values[0] != null)
                                ListData.AppendFormat("<input name=\"Value1\" type=\"text\" size=\"15\" id=\"MultiTextValue1_{0}\" class=\"FormInputArea\"  Value=\"{1}\"/>", RowNo,ALSolutionCommon.TrimAndReplaceSpecialChars( Values[0]));
                            else
                                ListData.AppendFormat("<input name=\"Value1\" type=\"text\" size=\"15\" id=\"MultiTextValue1_{0}\" class=\"FormInputArea\" Value=\"\"/>", RowNo);
                        }
                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"Value1\" type=\"text\" size=\"15\" id=\"MultiTextValue1_{0}\" class=\"FormInputArea\"  Value=\"\"/>", RowNo);
                    }

                    ListData.Append("</td>");
                    ListData.Append("<td style=\"width: 5px;\" />");
                    ListData.Append("<td>");
                    ListData.Append("<span class=\"LabelText\">AND</span>");
                    ListData.Append("</td>");
                    ListData.Append("<td>");

                    if (!String.IsNullOrEmpty(AttributeActualValue))
                    {
                        if (Values.Length > 0)
                        {
                            if (Values[1] != null)
                                ListData.AppendFormat("<input name=\"Value2\" type=\"text\" size=\"15\" id=\"MultiTextValue2_{0}\" class=\"FormInputArea\"  Value=\"{1}\"/>", RowNo, ALSolutionCommon.TrimAndReplaceSpecialChars(Values[1]));
                            else
                                ListData.AppendFormat("<input name=\"Value2\" type=\"text\" size=\"15\" id=\"MultiTextValue2_{0}\" class=\"FormInputArea\"  Value=\"\"/>", RowNo);
                        }
                    }
                    else
                    {
                        ListData.AppendFormat("<input name=\"Value2\" type=\"text\" size=\"15\" id=\"MultiTextValue2_{0}\" class=\"FormInputArea\"  Value=\"\"/>", RowNo);
                    }
                    ListData.Append("</td>");

                    ListData.Append("</tr>");
                    ListData.Append("</table>");
                    ListData.Append("</div>");
                    #endregion
                }
            }
                return ListData.ToString();
        }

        /// <summary>
        /// This function verifies whether the given value is belong to named hierarchy 
        /// </summary>
        /// <returns>true if field is named hierarchy else false</returns>
        private Boolean isNamedHierarchyField(String ObjectTypeName, String TypeAttributeName)
        {
            # region Local Variables
            XmlNode         ConfigNode      = null;
            StringBuilder   XPath           = null;
            # endregion

            try
            {
                XPath = new StringBuilder();
                XPath.AppendFormat("STATE/DATA[@Name='{0}']/Configuration/Types/", m_TypeConfigurationName);
                XPath.AppendFormat("Type[@DisplayText='{0}']/Attributes/", ALSolutionCommon.TrimAndReplaceSpecialChars(ObjectTypeName));
                XPath.AppendFormat("Attribute[@Value='{0}'][@IsNamedHierarchy='TRUE']", TypeAttributeName);
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                ConfigNode = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(XPath.ToString());
                if (ConfigNode != null)
                    return (true);
                else
                    return (false);
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        /// <summary>
        /// This method returns date macro xml
        /// </summary>
        /// <returns>Date macro xml</returns>
        private XML getDateMacros(String SelectedOption)
        {
            #region Local Variables
            XmlNode         DateMacroNode               = null;
            XmlNodeList     OperatorNodes               = null;
            StringBuilder   OptionsValues               = null;
            String          DropdownListValue           = null;
            String          DropdownListDisplayValue    = null;            
            String          DisplayTextID               = null;
            #endregion
           
            try
            {
                OptionsValues   = new StringBuilder();
                if (OperatorsXMLRootNode == null)
                {
                    getOperatorsXml();
                }
                DateMacroNode   = OperatorsXMLRootNode.SelectSingleNode("DATEMACRO");
                OperatorNodes   = DateMacroNode.SelectNodes("OPERATORS/OPERATOR");
                foreach (XmlNode OperatorNode in OperatorNodes)
                {
                    DropdownListValue           = OperatorNode.Attributes.GetNamedItem("Name").Value;
                    DisplayTextID               = OperatorNode.Attributes.GetNamedItem("DisplayTextID").Value;
                    DropdownListDisplayValue    = Localization.GetString("ALResource", TenantName, DisplayTextID, OperatorNode.Attributes.GetNamedItem("DisplayText").Value);

                    if(DropdownListValue == SelectedOption)
                        OptionsValues.AppendFormat("<option value=\"{0}\" selected=\"true\">{1}</option>", DropdownListValue, DropdownListDisplayValue);
                    else
                        OptionsValues.AppendFormat("<option value=\"{0}\">{1}</option>", DropdownListValue, DropdownListDisplayValue);
                }
            }
            catch (Exception)
            {
            }
            return OptionsValues.ToString();
        }

        /// <summary>
        /// This methos is used to get option xml for boolean type
        /// </summary>
        /// <returns>Option-xml for boolean type</returns>
        private XML getBooleanOptions(String SelectedOption)
        {
            #region Local Variables
            StringBuilder   OptionsValues               = null;
            #endregion
           
            try
            {
                OptionsValues   = new StringBuilder();

                if (SelectedOption == "1")
                    OptionsValues.AppendFormat("<option value=\"1\" selected=\"selected\">{0}</option>", Localization.GetString("ALResource",TenantName,"Yes_Shared","Yes"));
                else
                    OptionsValues.AppendFormat("<option value=\"1\">{0}</option>", Localization.GetString("ALResource", TenantName, "Yes_Shared", "Yes"));

                if (SelectedOption == "0")
                    OptionsValues.AppendFormat("<option value=\"0\" selected=\"selected\">{0}</option>", Localization.GetString("ALResource", TenantName, "No_Shared", "No"));
                else
                    OptionsValues.AppendFormat("<option value=\"0\">{0}</option>", Localization.GetString("ALResource", TenantName, "No_Shared", "No"));
            }
            catch (Exception)
            {
            }
            return OptionsValues.ToString();
        }

        /// <summary>
        /// This method is used to delete the all clauses from the selection list
        /// </summary>
        /// <returns>true on success; false otherwise</returns>
        private Boolean DeleteAllFilters(String ListType)
        {
            # region Local Variables
            XmlNode         ListTypeNode    = null;
            XmlNode         GroupNode       = null;
            XmlElement      GroupElement    = null;
            Boolean         FlagRowDeleted  = false;
            StringBuilder   XPath           = null;
            # endregion

            try
            {
                XPath = new StringBuilder();
                XPath.AppendFormat("STATE/DATA[@Name='{0}']", ListType);
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                ListTypeNode = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(XPath.ToString());
                if (ListTypeNode != null)
                {
                    GroupNode = ListTypeNode.SelectSingleNode("List/Data/Group");
                    if (GroupNode != null)
                    {
                        FlagRowDeleted = true;
                        ListTypeNode.SelectSingleNode("List/Data").RemoveChild(GroupNode);
                        ListTypeNode.SelectSingleNode("List").Attributes["NumberOfRows"].Value = "0";

                        GroupElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Group");
                        GroupElement.SetAttribute("Value", "None");
                        GroupElement.SetAttribute("EndRow", "0");
                        ListTypeNode.SelectSingleNode("List/Data").AppendChild(GroupElement);

                        AdvSearchStateObject.SetStateData(ListType, ListTypeNode.InnerXml);
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (FlagRowDeleted);
        }

        //getAttributeValue() method will  return the value of the attribute
        private String getAttributeValue(String AttributeName, String AttributeDataType, XML FieldValueXml, Int32 Row, String Operator, ref String Value1, ref String Value2, bool IsNHValue, ref String AttrDisplayValue)
        {
            #region Variables
            XmlDocument     XmlDoc              = null;
            XmlNode         FieldNode           = null;
            XmlNode         Value1Node          = null;
            XmlNode         Value2Node          = null;
            String          AttributeValue      = null;
            String          RowSelectionChanged = null;
            XmlNode         TempNode            = null;
            StringBuilder   FilePath            = null;
            XmlNode         DispValue1Node      = null;
            XmlNode         DispValue2Node      = null;
            #endregion

            XmlDoc = new XmlDocument();
            if (!String.IsNullOrEmpty(FieldValueXml))
            {
                XmlDoc.LoadXml(FieldValueXml);
                FieldNode           = XmlDoc.SelectSingleNode("//FIELD[@RowNumber ='" + Row + "']");
                
                if (FieldNode != null)
                {
                    if ((TempNode = FieldNode.Attributes.GetNamedItem("RowSelectionChanged")) != null)
                        RowSelectionChanged = FieldNode.Attributes.GetNamedItem("RowSelectionChanged").Value;
                    else
                        RowSelectionChanged = "FALSE";

                    if (RowSelectionChanged.ToUpper()!="TRUE")
                    {
                        if (Operator == "DATEMACRO")
                        {
                            Value1Node      = FieldNode.SelectSingleNode("descendant::DDFIELDVALUE1");
                           
                            if (Value1Node != null)
                            {
                                AttributeValue = Value1Node.InnerText;
                                Value1 = Value1Node.InnerText;
                            }
                        }
                        if (String.Equals(AttributeDataType, "BOOLEAN", StringComparison.CurrentCultureIgnoreCase))
                        {
                            if (IsNHValue)
                                Value1Node = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                            else
                                Value1Node = FieldNode.SelectSingleNode("descendant::DDFIELDVALUE1");
                            
                            if (Value1Node != null)
                            {
                                AttributeValue  = Value1Node.InnerText;
                                Value1          = Value1Node.InnerText;
                            }
                        }
                        else if (String.Equals(AttributeDataType, "CURRENCY", StringComparison.CurrentCultureIgnoreCase))
                        {
                            Value1Node      = FieldNode.SelectSingleNode("descendant::DDCURRENCYFIELDVALUE1");
                            if (Value1Node != null)
                                Value1 = Value1Node.InnerText;
                            Value2Node      = FieldNode.SelectSingleNode("descendant::EBCURRENCYFIELDVALUE1");
                            if (Value2Node != null)
                                Value2 = Value2Node.InnerText;
                            AttributeValue  = Value1 + "$AL$" + Value2;
                        }
                        else if (Operator != "INRANGE")
                        {
                            if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))
                            {
                                Value1Node      = FieldNode.SelectSingleNode("descendant::DFIELDVALUE1");
                                DispValue1Node  = FieldNode.SelectSingleNode("descendant::DFIELDDISPLAYVALUE1");
                            }
                            else
                            {
                                Value1Node = FieldNode.SelectSingleNode("descendant::FIELDVALUE1");
                                if (Value1Node == null)
                                {
                                    Value1Node      = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                                    //Here it is added as it is used in common code below
                                    DispValue1Node  = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                                }
                            }

                            if (Value1Node != null)
                            {
                                AttributeValue  = Value1Node.InnerText;
                                Value1          = Value1Node.InnerText;
                                if (DispValue1Node != null)
                                    AttrDisplayValue = DispValue1Node.InnerText;
                            }
                        }
                        else
                        {
                            if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))
                            {
                                Value1Node      = FieldNode.SelectSingleNode("descendant::DFIELDVALUE2");
                                DispValue1Node  = FieldNode.SelectSingleNode("descendant::DFIELDDISPLAYVALUE2");

                                if (Value1Node != null)
                                {
                                    AttributeValue  = Value1Node.InnerText;
                                    Value1          = Value1Node.InnerText;
                                    if (DispValue1Node != null)
                                        AttrDisplayValue = DispValue1Node.InnerText;
                                }

                                Value2Node      = FieldNode.SelectSingleNode("descendant::DFIELDVALUE3");
                                DispValue2Node  = FieldNode.SelectSingleNode("descendant::DFIELDDISPLAYVALUE3");
                            }
                            else
                            {
                                Value1Node = FieldNode.SelectSingleNode("descendant::FIELDVALUE2");
                                if (Value1Node != null)
                                {
                                    AttributeValue = Value1Node.InnerText;
                                    Value1 = Value1Node.InnerText;
                                }

                                Value2Node      = FieldNode.SelectSingleNode("descendant::FIELDVALUE3");
                                //Here it is added as it is used in common code below
                                DispValue2Node  = FieldNode.SelectSingleNode("descendant::FIELDVALUE3");
                            }

                            if (Value2Node != null)
                            {
                                AttributeValue += " AND " + Value2Node.InnerText;
                                Value2          = Value2Node.InnerText;
                                if (DispValue2Node != null)
                                    AttrDisplayValue += " AND " + DispValue2Node.InnerText;
                            }
                        }
                    }
                    else
                    {
                        // Do not do state management for the currently changed row because the control for Values
                        // may change.I.e from TextBox to DateControl.
                        AttributeValue = "";

                        // Incase of NamedHierarchy delete the previous XML.OW previous value will be shown selected if user
                        //clicks on it.
                        Value1Node = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                        if (Value1Node != null)
                        {
                            FilePath = new StringBuilder();
                            FilePath.AppendFormat(@"{0}\{1}\NamedHeirarchyData\ReportSrch_{2}_{3}.xml", ALSolutionCommon.GetTempPath(SessionID), UserID, AttributeName, Row);
                            if (File.Exists(FilePath.ToString()))
                            {
                                File.Delete(FilePath.ToString());
                            }
                        }

                    }
                }
            }

            //SingleTextDivNode = XmlDoc.SelectSingleNode("//div[@id='SingleTextDiv_" + Row + "']");
            //DisplayValue = ((XmlElement)SingleTextDivNode).GetAttribute("style");
            //if (DisplayValue == "display: block;")
            //{
            //    InputNode = SingleTextDivNode.SelectSingleNode("descendant::input[@name='SingleText_"+Row+"']");
            //    if (InputNode != null)
            //        AttributeValue = InputNode.InnerText;

            //}
            return AttributeValue;
        }
        /// <summary>
        /// This method will  return the value of the attribute
        /// </summary>
        /// <returns>Value of the selected attribute</returns>
        private String getAttributeValue_old(String AttributeName, String AttributeDataType, XML FieldValueXml, Int32 Row, String Operator, ref String Value1, ref String Value2, bool IsNHValue)
        {
            #region Local Variables
            XmlDocument     XmlDoc              = null;
            XmlNode         FieldNode           = null;
            XmlNode         Value1Node          = null;
            XmlNode         Value2Node          = null;
            String          AttributeValue      = null;
            String          RowSelectionChanged = null;
            XmlNode         TempNode            = null;
            StringBuilder   FilePath            = null; 
            #endregion

            XmlDoc = new XmlDocument();
            if (!String.IsNullOrEmpty(FieldValueXml))
            {
                XmlDoc.LoadXml(FieldValueXml);
                FieldNode           = XmlDoc.SelectSingleNode("//FIELD[@RowNumber ='" + Row + "']");
                
                if (FieldNode != null)
                {
                    if ((TempNode = FieldNode.Attributes.GetNamedItem("RowSelectionChanged")) != null)
                        RowSelectionChanged = FieldNode.Attributes.GetNamedItem("RowSelectionChanged").Value;
                    else
                        RowSelectionChanged = "FALSE";

                    if (RowSelectionChanged.ToUpper()!="TRUE")
                    {
                        if (Operator == "DATEMACRO")
                        {
                            Value1Node = FieldNode.SelectSingleNode("descendant::DDFIELDVALUE1");
                            
                            if (Value1Node != null)
                            {
                                AttributeValue = Value1Node.InnerText;
                                Value1 = Value1Node.InnerText;
                            }
                        }
                        if (String.Equals(AttributeDataType, "BOOLEAN", StringComparison.CurrentCultureIgnoreCase))
                        {
                            if (IsNHValue)
                                Value1Node = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                            else
                                Value1Node = FieldNode.SelectSingleNode("descendant::DDFIELDVALUE1");
                            
                            if (Value1Node != null)
                            {
                                AttributeValue  = Value1Node.InnerText;
                                Value1          = Value1Node.InnerText;
                            }
                        }
                        else if (Operator != "INRANGE")
                        {
                            if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))
                                Value1Node = FieldNode.SelectSingleNode("descendant::DFIELDVALUE1");
                            else
                            {
                                Value1Node = FieldNode.SelectSingleNode("descendant::FIELDVALUE1");
                                if (Value1Node == null)
                                    Value1Node = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                            }

                            if (Value1Node != null)
                            {
                                AttributeValue = Value1Node.InnerText;
                                Value1 = Value1Node.InnerText;
                            }
                        }
                        else
                        {
                            if (String.Equals(AttributeDataType, "DATETIME", StringComparison.CurrentCultureIgnoreCase))
                            {
                                Value1Node = FieldNode.SelectSingleNode("descendant::DFIELDVALUE2");
                                if (Value1Node != null)
                                {
                                    AttributeValue = Value1Node.InnerText;
                                    Value1 = Value1Node.InnerText;
                                }

                                Value2Node = FieldNode.SelectSingleNode("descendant::DFIELDVALUE3");
                            }
                            else
                            {
                                Value1Node = FieldNode.SelectSingleNode("descendant::FIELDVALUE2");
                                if (Value1Node != null)
                                {
                                    AttributeValue = Value1Node.InnerText;
                                    Value1 = Value1Node.InnerText;
                                }

                                Value2Node = FieldNode.SelectSingleNode("descendant::FIELDVALUE3");
                            }
                            if (Value2Node != null)
                            {
                                AttributeValue += " AND " + Value2Node.InnerText;
                                Value2 = Value2Node.InnerText; ;
                            }
                        }
                    }
                    else
                    {
                        // Do not do state management for the currently changed row because the control for Values
                        // may change.I.e from TextBox to DateControl.
                        AttributeValue = "";

                        // Incase of NamedHierarchy delete the previous XML.OW previous value will be shown selected if user clicks on it.
                        Value1Node = FieldNode.SelectSingleNode("descendant::NHFIELDVALUE1");
                        if (Value1Node != null)
                        {
                            FilePath = new StringBuilder();
                            FilePath.AppendFormat(@"{0}\{1}\NamedHeirarchyData\ReportSrch_{2}_{3}.xml", ALSolutionCommon.GetTempPath(SessionID), UserID, AttributeName, Row);
                            if (File.Exists(FilePath.ToString()))
                            {
                                File.Delete(FilePath.ToString());
                            }
                        }
                    }
                }
            }
            return AttributeValue;
        }

        /// <summary>
        /// This method is used to get operators xml
        /// </summary>
        private void getOperatorsXml()
        {
            # region Local Variables
            StringBuilder   OperatorsXMLFilePath    = null;
            XmlDocument     OperatorsXMLDoc         = null;
            # endregion

            try
            {
                if (OperatorsXMLRootNode == null)
                {
                    # region Get Operators xml
                    OperatorsXMLDoc         = new XmlDocument();
                    OperatorsXMLFilePath    = new StringBuilder();
                    OperatorsXMLFilePath.Append(ALSolutionCommon.GetConfigurationXMLPath(TenantName, "DAM", "", "OperatorsXML"));
                    OperatorsXMLDoc.Load(OperatorsXMLFilePath.ToString());
                    OperatorsXMLRootNode    = OperatorsXMLDoc.DocumentElement;
                    # endregion
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        /// <summary>
        /// This method is to get object type details from the configuration XML file
        /// </summary>
        private void getSourceConfigRootNode()
        {
            # region Local Variables
            StringBuilder   ADSConfigFilePath   = null;
            XmlDocument     ADSConfigDoc        = null;
            # endregion

            try
            {
                if (SourceConfigRootNode == null)
                {
                    # region Get object type details from the configuration XML file
                    ADSConfigDoc            = new XmlDocument();
                    ADSConfigFilePath       = new StringBuilder();
                    ADSConfigFilePath.Append(ALSolutionCommon.GetConfigurationXMLPath(TenantName, "DAM", "", "AdvancedSearchSourceConfig"));
                    ADSConfigDoc.Load(ADSConfigFilePath.ToString());
                    SourceConfigRootNode    = ADSConfigDoc.DocumentElement;
                    # endregion
                }
            }
            catch (Exception e)
            {
                throw e;
            }
        }

        private void getAdvSearchStateObject()
        {
            # region Local Variables
            StringBuilder StateDocFilePath = null;
            # endregion

            StateDocFilePath        = new StringBuilder();
            StateDocFilePath.AppendFormat(@"{0}\{1}\{2}.xml", ALSolutionCommon.GetTempPath(SessionID), UserID, "DAMReport_AdvancedSearch_State");
            AdvSearchStateObject    = new AdvancedSearchStateManagement(StateDocFilePath.ToString());
        }

        //Add Hidden fields for Month Name, Weekdays Localized values
        private void AddLocalizedValueForCalendar(ref StringBuilder ListData, String ControlID, String RowNo)
        {
            ListData.AppendFormat("<input type='hidden' name='__TMonthNameArray_{1}_{0}'  id='__TMonthNameArray_{1}_{0}' value=\"", RowNo, ControlID);

            DateTime DateObjForMonthName = new DateTime(1, 1, 1);

            for (int i = 0; i < 12; i++)
            {
                if (i != 11)
                    ListData.AppendFormat("{0}$AL$", DateObjForMonthName.ToString("MMMM", System.Threading.Thread.CurrentThread.CurrentUICulture));
                else
                    ListData.Append(DateObjForMonthName.ToString("MMMM", System.Threading.Thread.CurrentThread.CurrentUICulture));

                DateObjForMonthName = DateObjForMonthName.AddMonths(1);
            }
            ListData.Append("\" />");

            ListData.AppendFormat("<input type=\"hidden\" name=\"__TShortMonthNameArray_{1}_{0}\" id=\"__TShortMonthNameArray_{1}_{0}\" value=\"", RowNo, ControlID);

            DateObjForMonthName = new DateTime(1, 1, 1);

            for (int i = 0; i < 12; i++)
            {
                if (i != 11)
                    ListData.AppendFormat("{0}$AL$", DateObjForMonthName.ToString("MMM", System.Threading.Thread.CurrentThread.CurrentUICulture));
                else
                    ListData.Append(DateObjForMonthName.ToString("MMM", System.Threading.Thread.CurrentThread.CurrentUICulture));

                DateObjForMonthName = DateObjForMonthName.AddMonths(1);
            }
            ListData.Append("\" />");

            ListData.AppendFormat("<input type=\"hidden\" name=\"__TWeekDayArray_{1}_{0}\" id=\"__TWeekDayArray_{1}_{0}\" value=\"", RowNo, ControlID);

            ListData.Append(Localization.GetString("ALResource", TenantName, "Sunday_Su_Shared", "Su"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Monday_Mo_Shared", "Mo"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Tuesday_Tu_Shared", "Tu"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Wednesday_We_Shared", "We"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Thursday_Th_Shared", "Th"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Friday_Fr_Shared", "Fr"));
            ListData.Append("$AL$");
            ListData.Append(Localization.GetString("ALResource", TenantName, "Saturday_Sa_Shared", "Sa"));
            ListData.Append("\" />");

        }
        #endregion

        #region Public Methods

        public ErrorXML CreateReport(XML InputXML)
        {
            # region Local Variable

            ErrorXML            ErrorXML = null;
            ALReportWS          ReportWSObj = null;
           
            # endregion

            try
            {
                ReportWSObj                         = new ALReportWS();
                ReportWSObj.Url                     = GetWebServiceURL("ALReportWS");
                ReportWSObj.Timeout                 = HelperClass.GetWebServiceTimeout();
                ReportWSObj.ALServiceHeaderValue    = getRequestHeader();
                ErrorXML                            = ReportWSObj.CreateReport(InputXML);
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
                return ErrorXML;
            }

            return ErrorXML;
        }

        public ErrorXML UpdateReport(XML InputXML)
        {
           
            # region Local Variable

            ErrorXML            ErrorXML        = null;
            ALReportWS          ReportWSObj     = null;
            
            # endregion

            try
            {
                ReportWSObj                         = new ALReportWS();
                ReportWSObj.Url                     = GetWebServiceURL("ALReportWS");
                ReportWSObj.Timeout                 = HelperClass.GetWebServiceTimeout();
                ReportWSObj.ALServiceHeaderValue    = getRequestHeader();
                ErrorXML                            = ReportWSObj.UpdateReport(InputXML);
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
                return ErrorXML;
            }

            return ErrorXML;
        }

        public ErrorXML DeleteReport(String ReportID)
        {
           
            # region Local Variable

            ErrorXML            ErrorXML = null;
            ALReportWS          ReportWSObj = null;
            
            # endregion

            try
            {
                ReportWSObj                         = new ALReportWS();
                ReportWSObj.Url                     = GetWebServiceURL("ALReportWS");
                ReportWSObj.Timeout                 = HelperClass.GetWebServiceTimeout();
                ReportWSObj.ALServiceHeaderValue    = getRequestHeader();

                ErrorXML = ReportWSObj.DeleteReport(ReportID);
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
                return ErrorXML;
            }

            return ErrorXML;
        }

        public String IsSessionValid()
        {
            # region Local variables
            bool                        bIsSessionValid     = false;
            InfrastructureFacadeClass   InfFCObj            = null;
            String                      ErrorXML            = null;
            String                      RedirectUrl         = null;
            ALSolutionCommon            ObjCommon           = null;
            # endregion

            InfFCObj = new InfrastructureFacadeClass(SessionID, UserID, TenantName, CurrentLanguage);
            ErrorXML = InfFCObj.IsSessionValid(ref bIsSessionValid);

            if (bIsSessionValid == false)
            {
                RedirectUrl = "/" + TenantName + "/Common/ASPX/ShowError.aspx?SessionID=" + SessionID + "&UserID=" + UserID;
                RedirectUrl = RedirectUrl + "&TenantName=" + TenantName + "&Language=" + CurrentLanguage;
                RedirectUrl = RedirectUrl + "&ErrorCode=SESSION_INVALID&ErrorMessage=Session Invalid! Please login again";
                ObjCommon   = new ALSolutionCommon();
                RedirectUrl = ObjCommon.EncodeURLForResponseRedirects(RedirectUrl);
            }
            return RedirectUrl;
        }

        public String GetBypassUrl(String ReportID, String SolutionName)
        {
            # region Local variables
            StringBuilder           SearchInputXml          = null;
            SearchConfiguration     SearchConfigurationObj  = null;
            String                  OutputXml               = null;
            XmlDocument             XmlDocObj               = null;
            XmlNode                 ReportInputXmlNode      = null;
            XmlNodeList             FieldNodeList           = null;
            String                  ReportInputXml          = null;
            String                  ReportNTFSPath          = null;
            String                  ReportName              = null;
            String                  ReportFileName          = null;
            String                  FilePath                = null;
            String                  DirectoryPath           = null;
            String                  url_to_Redirect         = null;
            String                  EncodedUrl              = null;
            String                  url_Param               = null;
            XmlDocument             RptIpDocumnet           = null;
            XmlElement              DocRootNode             = null;
            XmlElement              UserInfoNode            = null;
            String                  UserName                = null;
            ALSolutionCommon        SolnCommonObj           = null;
            Boolean                 IsStdConfigXml          = false;
            StringBuilder           ReportHandlerFilePath   = null;
            # endregion

            try
            {

                #region Search report
                SearchInputXml = new StringBuilder();
                SearchInputXml.Append("<INPUTXML>");
                SearchInputXml.AppendFormat("<SESSIONINFO TenantName=\"{0}\" Language=\"{1}\" UserID=\"{2}\" SessionID=\"{3}\"/>", TenantName, CurrentLanguage, UserID, SessionID);
                SearchInputXml.Append("<SEARCHCOLLECTION><SEARCH><TARGETOBJECT>ANALYTICALREPORT</TARGETOBJECT>");
                SearchInputXml.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
                SearchInputXml.Append("<ISCASESENSITIVE>FALSE</ISCASESENSITIVE>");
                SearchInputXml.Append("<MAXRESULTSIZE>100</MAXRESULTSIZE><SEARCHSTRING/>");
                SearchInputXml.Append("<SEARCHALLFACETS>FALSE</SEARCHALLFACETS>");
                SearchInputXml.AppendFormat("<FILTERMODE ApplyFilter=\"\"/><FILTERLIST><FILTER><FIELDNAME>REPORTID</FIELDNAME>");
                SearchInputXml.Append("<OPERATOR>EQUALTO</OPERATOR>");
                SearchInputXml.AppendFormat("<FIELDVALUE>{0}</FIELDVALUE>", ReportID);
                SearchInputXml.Append("<ISCASESENSITIVE>FALSE</ISCASESENSITIVE>");
                SearchInputXml.Append("<NAVIGATIONPATH>ANALYTICALREPORT</NAVIGATIONPATH></FILTER></FILTERLIST>");
                SearchInputXml.Append("<FIELDCOLLECTION><TARGETENTITYFIELDS><FIELD>REPORTNAME</FIELD>");
                SearchInputXml.Append("<FIELD>REPORTCATEGORY</FIELD><FIELD>REPORTDESCRIPTION</FIELD><FIELD>INPUTSCREENLAYOUTXML</FIELD><FIELD>REPORTCATEGORY</FIELD><FIELD>REPORTFILENAME</FIELD>");
                SearchInputXml.Append("</TARGETENTITYFIELDS></FIELDCOLLECTION>");
                SearchInputXml.Append("</SEARCH></SEARCHCOLLECTION>");
                SearchInputXml.Append("</INPUTXML>");

                SearchConfigurationObj = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                SearchConfigurationObj.DoSearch(SearchInputXml.ToString(), ref OutputXml);
                #endregion

                # region Get required data from search output xml
                XmlDocObj = new XmlDocument();
                XmlDocObj.LoadXml(OutputXml);

                ReportName          = XmlDocObj.DocumentElement.SelectSingleNode("/SEARCHOUTPUTXML/ANALYTICALREPORTCOLLECTION/ANALYTICALREPORT/REPORTNAME").InnerText;
                ReportFileName      = XmlDocObj.DocumentElement.SelectSingleNode("/SEARCHOUTPUTXML/ANALYTICALREPORTCOLLECTION/ANALYTICALREPORT/REPORTFILENAME").InnerText;
                ReportInputXmlNode  = XmlDocObj.DocumentElement.SelectSingleNode("/SEARCHOUTPUTXML/ANALYTICALREPORTCOLLECTION/ANALYTICALREPORT/INPUTSCREENLAYOUTXML");
                ReportInputXml      = ReportInputXmlNode.InnerXml;
                # endregion

                # region Load report input filter xml
                if (ReportInputXml.Length == 0)
                {
                    ReportNTFSPath = ReportCommon.GetReportXMLPath(TenantName, SolutionName, ReportName, ReportName);
                    if (File.Exists(ReportNTFSPath) == false)
                    {
                        return null;
                    }

                    XmlDocObj.Load(ReportNTFSPath);

                    // Verify Standard report xml format
                    IsStdConfigXml = IsStdReportConfigXml(XmlDocObj.CreateNavigator());
                    if (IsStdConfigXml == false)
                    {
                        return null;
                    }

                    ReportInputXmlNode  = XmlDocObj.DocumentElement.SelectSingleNode("/Form");
                    ReportInputXml      = ReportInputXmlNode.InnerXml;
                }
                # endregion

                if (ReportInputXml.Length > 0)
                {
                    if (IsStdConfigXml == false)
                        IsStdConfigXml = IsStdReportConfigXml(ReportInputXmlNode.CreateNavigator());

                    if (IsStdConfigXml == false)
                    {
                        return null;
                    }
                    FieldNodeList = ReportInputXmlNode.SelectNodes("descendant::Field");

                    if (FieldNodeList.Count <= 0)
                    {
                        
                        ReportHandlerFilePath = new StringBuilder(50);

                        // Check if report handler file exist
                        ReportHandlerFilePath.Append(ALCommon.GetInstallationDir());
                        ReportHandlerFilePath.AppendFormat("{0}\\{1}\\{2}\\{2}.aspx", TenantName, SolutionName, ReportName);
                        if (File.Exists(ReportHandlerFilePath.ToString()) == false)
                        {
                            return null;
                        }

                        # region Get bypass url
                        SolnCommonObj       = new ALSolutionCommon();
                        DirectoryPath       = ALSolutionCommon.GetTempPath(SessionID) + "XMLS\\";
                        FilePath            = DirectoryPath + ReportID.ToString() + "_ReportFiltersXml.xml";
                        url_Param           = "SessionID=" + SessionID + "&UserID=" + UserID + "&Language=" + CurrentLanguage + "&TenantName=" + TenantName;
                        url_to_Redirect     = "/" + TenantName + "/" + SolutionName + "/" + ReportName + "/" + ReportName + ".aspx?";
                        url_to_Redirect     = url_to_Redirect + url_Param + "&paramReportName=" + ReportName + "&paramReportID=" + ReportID;
                        url_to_Redirect     = url_to_Redirect + "&paramFileName=" + ReportFileName + "&paramFilePath=" + FilePath;
                        EncodedUrl          = SolnCommonObj.EncodeURLForResponseRedirects(url_to_Redirect);

                        // --- Form report configuration (filter) xml for report handler ---
                        RptIpDocumnet       = new XmlDocument();
                        DocRootNode         = RptIpDocumnet.CreateElement("REPORT");

                        DocRootNode.SetAttribute("Name", ReportName);
                        RptIpDocumnet.AppendChild(DocRootNode);

                        UserName                = GetUserDetails();
                        UserInfoNode            = RptIpDocumnet.CreateElement("USERNAME");
                        UserInfoNode.InnerText  = UserName;
                        DocRootNode.AppendChild(UserInfoNode);

                        Directory.CreateDirectory(DirectoryPath);
                        RptIpDocumnet.Save(FilePath);

                        # endregion

                        return "Javascript:ShowReport('" + EncodedUrl + "');";
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }

            return null;
        }

        public String GetUserDetails()
        {
            #region Local Variables
            StringBuilder           UserName                = null;
            String                  FirstName               = null;
            String                  LastName                = null;
            String                  LoginName               = null;
            #endregion

            try
            {
                ALSolutionCommon.GetFormattedUserName(UserID, TenantName, ref LoginName, ref FirstName, ref LastName);

                if ((String.IsNullOrEmpty(FirstName)) && (String.IsNullOrEmpty(LastName)) && (!String.IsNullOrEmpty(LoginName)))
                    return LoginName;

                UserName = new StringBuilder();
                UserName.Append(FirstName);
                if (!(String.IsNullOrEmpty(FirstName)) && !(String.IsNullOrEmpty(LastName)))
                    UserName.Append(" ");

                UserName.Append(LastName + " ");
                UserName.Append("(" + LoginName + ")");

                return UserName.ToString();
            }
            catch(Exception ExcepObj)
            {
                throw ExcepObj;
            }
        }
        
        /// <summary>
        /// This function takes list of asssetId and returns details of those assets
        /// </summary>
        /// <param name="AssetIDList">List of assetID</param>
        /// <param name="SearchResultXml">Reference parameter, gives asset detail xml</param>
        /// <returns>On Error returns error else null</returns>
        public ErrorXML GetAssetDetails(String AssetIDList, String AssetDescrPath, ref XML SearchResultXml)
        {
            # region Local Variables
            SearchConfiguration     SearchConfigObj         = null;
            StringBuilder           SearchInputXml          = null;
            XmlDocument             AssetDetailsDoc         = null;
            XML                     AssetDetailsXml         = null;
            XPathNavigator          AssetDetailsNav         = null;
            XslCompiledTransform    AssetDetailsXslt        = null;
            XsltArgumentList        ArgListObj              = null;
            HelperClass             HelperClassObj          = null;
            Stream                  StreamObj               = null;
            StreamReader            AssetDetailsSR          = null;
            String                  AssetDetailsXsltPath    = null;
            String                  ErrorXML                = null;
            ALCommon                ALCommonObj             = null;
            # endregion

            try
            {
                if (String.IsNullOrEmpty(AssetIDList) == false)
                {
                    # region Search digital assets
                    SearchInputXml          = new StringBuilder();
                    SearchConfigObj         = new SearchConfiguration(SessionID, UserID, TenantName, CurrentLanguage);
                    SearchInputXml.Append(prepareSearchXML(AssetIDList, AssetDescrPath));
                    ErrorXML                = SearchConfigObj.GetHierarchialData(SearchInputXml.ToString(), ref AssetDetailsXml);
                    SearchInputXml.Remove(0, SearchInputXml.Length);
                    # endregion

                    # region Load Search result xml and apply XSLT for data representation
                    AssetDetailsDoc         = new XmlDocument();
                    AssetDetailsXslt        = new XslCompiledTransform();
                    ArgListObj              = new XsltArgumentList();
                    StreamObj               = new MemoryStream();
                    AssetDetailsXsltPath    = ALCommon.GetInstallationDir() + TenantName + "\\Reports\\XSLT\\DAMReport.xslt";


                    AssetDetailsDoc.LoadXml(AssetDetailsXml);
                    AssetDetailsNav          = AssetDetailsDoc.CreateNavigator();

                    // HelperClass object for formatting dates
                    HelperClassObj      = new HelperClass(ALSolutionCommon.GetInstallationDir(), TenantName);
                    ArgListObj.AddExtensionObject("urn:HelperClass", HelperClassObj);

                    // Parameter to XSLT for Multiselect field delimeter
                    ALCommonObj = new ALCommon();
                    ArgListObj.AddParam("MVSeparator", "", ALCommonObj.MultiSelectSeparator(TenantName));
                    ArgListObj.AddParam("YesDispValue", "", Localization.GetString("ALResource",TenantName,"Yes_Shared","Yes"));
                    ArgListObj.AddParam("NoDispValue", "", Localization.GetString("ALResource", TenantName, "No_Shared", "No"));
                    AssetDetailsXslt.Load(AssetDetailsXsltPath);
                    AssetDetailsXslt.Transform(AssetDetailsNav, ArgListObj, StreamObj);

                    StreamObj.Position  = 0;
                    AssetDetailsSR      = new StreamReader(StreamObj);
                    SearchResultXml     = AssetDetailsSR.ReadToEnd();

                    AssetDetailsSR.Close();
                    AssetDetailsSR.Dispose();
                    StreamObj.Close();
                    StreamObj.Dispose();
                    # endregion

                    # region Download thumbnails to download folder
                    getThumbnails(AssetDetailsNav);
                    # endregion
                }
                else
                {
                    ErrorXML = "Error: Asset-id list is empty";
                }
                return ErrorXML;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// This function takes file path of asssetId list and returns details of those assets
        /// </summary>
        /// <param name="AssetIDListXMLPath">Path of xml file containing assetID list</param>
        /// <param name="SearchResultXml">Reference parameter, gives asset detail xml</param>
        /// <returns>On Error returns error else null</returns>
        public ErrorXML GetAssetDetailsByPath(String AssetIDListXMLPath, String AssetDescrPath, ref XML SearchResultXml)
        {
            # region Local Variables
            XmlDocument     AssetIDXmlDoc   = null;
            String          AssetIDList     = null;
            ErrorXML        ErrorXML        = null;
            # endregion

            try
            {
                if (String.IsNullOrEmpty(AssetIDListXMLPath) == false)
                {
                    if (File.Exists(AssetIDListXMLPath))
                    {
                        AssetIDXmlDoc   = new XmlDocument();
                        AssetIDXmlDoc.Load(AssetIDListXMLPath);
                        AssetIDList     = getIDs(AssetIDXmlDoc);
                        ErrorXML        = GetAssetDetails(AssetIDList, AssetDescrPath, ref SearchResultXml);
                    }
                    else
                    {
                        ErrorXML = "Error: File path does not exist";
                    }
                }
                else
                {
                    ErrorXML = "Error: File path is empty";
                }
                return ErrorXML;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// This method is used to generate the list without rows (data).
        /// </summary>
        /// <param name="AddDefaultEmptyRows"></param>
        /// <returns></returns>
        public String GenerateListXMLWithoutData(Boolean AddDefaultEmptyRows, String ListType, String ListHeading)
        {
            # region Local Variables
            StringBuilder   ListData = null;
            String          ListXml  = null; 
            String          AttributeValue = null;
            # endregion

            try
            {
                AttributeValue = Localization.GetString("ALResource", TenantName, "Select_Shared");
                ListData = new StringBuilder();
                if (String.IsNullOrEmpty(ListHeading)) // == null) || (ListHeading.Trim() == ""))
                    ListData.AppendFormat("<List Name=\"{0}\" NumberOfRows=\"0\" ErrorMessageID=\"ERR_DAM_AdvancedSearch_NoFiltersPresent\" GridLines=\"Yes\" EnableDragDrop=\"False\" EnableColumnDragDrop=\"False\" EnableIterators=\"False\">", ListType);
                else
                    ListData.AppendFormat("<List Name=\"{0}\" NumberOfRows=\"0\" ErrorMessageID=\"ERR_DAM_AdvancedSearch_NoFiltersPresent\" GridLines=\"Yes\" EnableDragDrop=\"False\" EnableColumnDragDrop=\"False\" EnableIterators=\"False\">", ListHeading);

                ListData.AppendFormat("<Header SelectionType=\"CheckBox\" SelectAll=\"Yes\" SelectionName=\"CB_{0}\">", ListType);

                ListData.Append("<Column DisplayState=\"No\" DisplayValue=\"\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"1\" Name=\"SequenceNo\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\" />");

                if (String.Compare(ListType, "FILTERLIST", true) == 0)
                {
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"Source\" DisplayTextID=\"DAM_AdvancedSearch_Source\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"2\" Name=\"Source\" Width=\"20%\" ApplySort=\"False\" UIType=\"DROPDOWNLIST\" COLUMNCONTENTTYPE=\"UserDefined\" EventType=\"onchange\" EventHandler=\"javascript:SetAttributesBasedOnSource(this);\">");
                    ListData.Append(getTypeAttributeName());
                    ListData.Append("</Column>");
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"Attribute\" DisplayTextID=\"DAM_AdvancedSearch_Attribute\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"3\" Name=\"Attribute\" Width=\"20%\" ApplySort=\"False\" UIType=\"DROPDOWNLIST\" COLUMNCONTENTTYPE=\"UserDefined\" EventType=\"onchange\" EventHandler=\"javascript:SetOperatorsBasedOnAttribute(this);\">");
                    ListData.Append("</Column>");
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"Operator\" DisplayTextID=\"DAM_AdvancedSearch_Operator\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"4\" Name=\"Operator\" Width=\"20%\" ControlWidth=\"22\" ApplySort=\"False\" UIType=\"DROPDOWNLIST\" COLUMNCONTENTTYPE=\"UserDefined\" EventType=\"onchange\" EventHandler=\"javascript:SetValueControlsBasedOnOperator(this);\">");
                    ListData.AppendFormat("<Item Text=\"---{0}---\" Value=\"\" />", AttributeValue);
                    ListData.Append("</Column>");
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"AttributeValue\" DisplayTextID=\"DAM_AdvancedSearch_AttributeValue\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"5\" Name=\"AttributeValue\" Width=\"40%\" ControlWidth=\"24\" ApplySort=\"False\" UIType=\"HTML\" COLUMNCONTENTTYPE=\"UserDefined\" HTMLDecoding=\"True\"/>");
                }
                else  //if (String.Compare(ListType, "FILTERLIST", true) != 0)
                {
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"Source\" DisplayTextID=\"DAM_AdvancedSearch_Source\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"2\" Name=\"Source\" Width=\"20%\" ApplySort=\"False\" UIType=\"DROPDOWNLIST\" COLUMNCONTENTTYPE=\"UserDefined\" EventType=\"onchange\" EventHandler=\"javascript:SetAttributesBasedOnSource(this);\">");
                    ListData.Append(getTypeAttributeName());
                    ListData.Append("</Column>");
                    ListData.AppendFormat("<Column DisplayState=\"Yes\" DisplayValue=\"Attribute\" DisplayTextID=\"DAM_AdvancedSearch_Attribute\" Align=\"left\" Valign=\"Middle\" RowAlign=\"left\" RowValign=\"middle\" Order=\"3\" Name=\"Attribute\" Width=\"80%\" ApplySort=\"False\" UIType=\"DROPDOWNLIST\" COLUMNCONTENTTYPE=\"UserDefined\" EventType=\"onchange\" EventHandler=\"javascript:SetOperatorsBasedOnAttribute(this);\">");
                    ListData.Append("</Column>");
                }

                // Coloums not to be shown but are used for processing.
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Field Data Type\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"6\" Name=\"AttributeDataType\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Object Type\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"7\" Name=\"ObjectType\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Primary Key\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"8\" Name=\"PrimaryKey\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Object Name\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"9\" Name=\"ObjectName\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Parent Object Name\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"10\" Name=\"ParentEntityTypeName\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                ListData.Append("<Column  DisplayState=\"No\" DisplayValue=\"Field UI Type\" Align=\"middle\" Valign=\"top\" RowAlign=\"left\" RowValign=\"top\" Order=\"11\" Name=\"FieldUIType\" Width=\"1%\" ApplySort=\"False\" UIType=\"TEXT\"/>");
                     
                ListData.Append("</Header>");
                ListData.Append("<Data GroupLevel=\"1\">");
                ListData.Append("<Group Value=\"None\" EndRow=\"0\">");              
                ListData.Append("</Group>");
                ListData.Append("</Data>");
                ListData.Append("</List>");
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                AdvSearchStateObject.SetStateData(ListType, ListData.ToString());
                if (AddDefaultEmptyRows)
                    AddNewRow(3, ListType, ListHeading);
                ListXml = GetListXML(ListType, ListHeading);
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ListXml);
        }

        //To add new Row to the List
        /// <summary>
        /// This method adds new Row to the List
        /// </summary>
        /// <param name="RowCount">Number of row(s) to be added</param>
        public void AddNewRow(int RowCount, String ListType, String ListHeading)
        {
            #region Local Variables
            XmlDocument     ListDoc         = null;
            XmlNode         GroupNode       = null;
            XmlNode         ListNode        = null;
            XmlNodeList     RowList         = null;
            XmlNode         RowNode         = null;
            #endregion
            
            try
            {
                ListDoc     = new XmlDocument();
                ListDoc.LoadXml(GetListXML(ListType, ListHeading));
                ListNode    = ListDoc.SelectSingleNode("//List");

                if (ListNode != null)
                {
                    GroupNode   = ListNode.SelectSingleNode("//Group");
                    RowList     = GroupNode.SelectNodes("descendant::Row");
                    RowCount    = RowList.Count + RowCount;
                    ((XmlElement)ListNode).SetAttribute("NumberOfRows", RowCount.ToString());
                    ((XmlElement)GroupNode).SetAttribute("EndRow", RowCount.ToString());                    

                    for (int i = RowList.Count +1 ; i <= RowCount; i++)
                    {
                        RowNode             = ListDoc.CreateElement("Row");
                        ((XmlElement)RowNode).SetAttribute("Order", i.ToString());
                        ((XmlElement)RowNode).SetAttribute("SelectionValue", i.ToString());
                        RowNode.InnerXml    = addRow(i, ListType);
                        GroupNode.AppendChild(RowNode);
                    }
                }
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                AdvSearchStateObject.SetStateData(ListType, ListDoc.OuterXml);
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
        }

        /// <summary>
        /// This method is used to retrieve the list XML with data.
        /// </summary>
        /// <returns>List XML with data if exist else without data</returns>
        public String GetListXML(String ListType, String ListHeading)
        {
            String ListDataXML = null;
            StringBuilder NamedHierarchyFilePath = null;

            try
            {
                NamedHierarchyFilePath = new StringBuilder(50);
                NamedHierarchyFilePath.AppendFormat(@"{0}\{1}\NamedHeirarchyData", ALSolutionCommon.GetTempPath(SessionID), UserID);
                if (!Directory.Exists(NamedHierarchyFilePath.ToString()))
                    Directory.CreateDirectory(NamedHierarchyFilePath.ToString());

                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                AdvSearchStateObject.GetStateData(ListType, ref ListDataXML);
                if ((ListDataXML == null) || (ListDataXML.Trim().Length == 0))
                    ListDataXML = GenerateListXMLWithoutData(true, ListType, ListHeading);
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (ListDataXML);
        }

        /// <summary>
        /// This methos sets state data
        /// </summary>
        /// <returns>State data xml</returns>
        public XML GetObjectTypes()
        {
            # region Local Variables
            XmlElement      ConfigRootElement           = null;
            XmlElement      TypesCollectionElement      = null;
            XmlElement      AttributesCollectionElement = null;
            XmlElement      TypeElement                 = null;
            XmlElement      AttributeElement            = null;
            XML             ObjectAttribXml             = null;
            XmlNodeList     TargetEntityNodes           = null;
            XmlNodeList     RelatedEntityNodes          = null;
            XmlNode         xmlnode_DisplayName         = null;
            StringBuilder   AttributeValue              = null;
            StringBuilder   XPath                       = null;
            String          TypeConfigXML               = null;
            String          EntityTypeName              = null;
            String          RelationshipName            = null;
            String          EntityDisplayName           = null;
            String          ParentEntityTypeName        = null;
            int             TargetEntityCount           = 0;
            
            
            
            # endregion

            try
            {
                AttributeValue = new StringBuilder();
                XPath = new StringBuilder();
                AttributeValue.AppendFormat("---{0}---", Localization.GetString("ALResource", TenantName, "Select_Shared"));

                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                AdvSearchStateObject.GetStateData(m_TypeConfigurationName, ref TypeConfigXML);
                if ((TypeConfigXML == null) || (TypeConfigXML.Trim().Length == 0))
                {
                    ConfigRootElement           = AdvSearchStateObject.StateXMLDoc.CreateElement("Configuration");

                    TypesCollectionElement      = AdvSearchStateObject.StateXMLDoc.CreateElement("Types");
                    ConfigRootElement.AppendChild(TypesCollectionElement);

                    TypeElement                 = AdvSearchStateObject.StateXMLDoc.CreateElement("Type");
                    TypeElement.SetAttribute("DisplayText", AttributeValue.ToString());
                    TypeElement.SetAttribute("Value", "SELECT");
                    TypesCollectionElement.AppendChild(TypeElement);

                    AttributesCollectionElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Attributes");
                    TypeElement.AppendChild(AttributesCollectionElement);

                    AttributeElement            = AdvSearchStateObject.StateXMLDoc.CreateElement("Attribute");
                    AttributeElement.SetAttribute("DisplayText", AttributeValue.ToString());
                    AttributeElement.SetAttribute("Value", "SELECT");
                    AttributeElement.SetAttribute("FieldDataType", "TEXT");
                    AttributeElement.SetAttribute("ObjectType", "");
                    AttributeElement.SetAttribute("PrimaryKey", "");
                    AttributeElement.SetAttribute("IsNamedHierarchy", "FALSE");
                    AttributesCollectionElement.AppendChild(AttributeElement);

                    // Add DigitalAsset and Collection 
                    if (SourceConfigRootNode == null)
                    {
                        getSourceConfigRootNode();
                    }
                    TargetEntityNodes           = SourceConfigRootNode.SelectNodes("//TARGETENTITYCOLLECTION/TARGETENTITY");
                    TargetEntityCount           = TargetEntityNodes.Count;
                    foreach (XmlNode TargetEntityNode in TargetEntityNodes)
                    {
                        TargetEntityCount--;
                        EntityTypeName          = TargetEntityNode.SelectSingleNode("NAME").InnerText;
                        xmlnode_DisplayName = TargetEntityNode.SelectSingleNode("DISPLAYNAME");
                        if (xmlnode_DisplayName != null)
                        {
                            if(xmlnode_DisplayName.Attributes["DisplayTextID"] != null)
                                EntityDisplayName = Localization.GetString("ALResource",TenantName,xmlnode_DisplayName.Attributes["DisplayTextID"].Value);
                            else
                                EntityDisplayName = TargetEntityNode.SelectSingleNode("DISPLAYNAME").InnerText;
                        }
                        
                        addTypesToStateXML(ObjectType.ENTITY, EntityTypeName, EntityDisplayName, EntityTypeName, EntityTypeName, ref TypesCollectionElement);

                        // Add Related entities 
                        XPath.Remove(0, XPath.Length);
                        XPath.AppendFormat("//RELATEDENTITYCOLLECTION/RELATEDENTITY[PARENTENTITYTYPENAME='{0}']",ALSolutionCommon.TrimAndReplaceSpecialChars(EntityTypeName));
                        RelatedEntityNodes          = SourceConfigRootNode.SelectNodes(XPath.ToString());
                        foreach (XmlNode RelatedEntityNode in RelatedEntityNodes)
                        {
                            EntityTypeName          = RelatedEntityNode.SelectSingleNode("NAME").InnerText;
                            RelationshipName        = RelatedEntityNode.SelectSingleNode("RELATIONSHIPNAME").InnerText;
                            xmlnode_DisplayName     = RelatedEntityNode.SelectSingleNode("DISPLAYNAME");
                            if (xmlnode_DisplayName != null)
                            {
                                if (xmlnode_DisplayName.Attributes["DisplayTextID"] != null)
                                    EntityDisplayName = Localization.GetString("ALResource",TenantName,xmlnode_DisplayName.Attributes["DisplayTextID"].Value);
                                else
                                    EntityDisplayName = xmlnode_DisplayName.InnerText;
                            }
                            ParentEntityTypeName    = RelatedEntityNode.SelectSingleNode("PARENTENTITYTYPENAME").InnerText;
                            addTypesToStateXML(ObjectType.ENTITY, EntityTypeName, EntityDisplayName, ParentEntityTypeName, RelationshipName, ref TypesCollectionElement);
                        }

                        if (TargetEntityCount > 0)
                        {
                            TypeElement                 = AdvSearchStateObject.StateXMLDoc.CreateElement("Type");
                            TypeElement.SetAttribute("DisplayText", "============");
                            TypeElement.SetAttribute("Value", "SELECT");
                            TypesCollectionElement.AppendChild(TypeElement);

                            AttributesCollectionElement = AdvSearchStateObject.StateXMLDoc.CreateElement("Attributes");
                            TypeElement.AppendChild(AttributesCollectionElement);

                            AttributeElement            = AdvSearchStateObject.StateXMLDoc.CreateElement("Attribute");
                            AttributeElement.SetAttribute("DisplayText", "============");
                            AttributeElement.SetAttribute("Value", "SELECT");
                            AttributeElement.SetAttribute("FieldDataType", "TEXT");
                            AttributeElement.SetAttribute("ObjectType", "");
                            AttributeElement.SetAttribute("PrimaryKey", "");
                            AttributeElement.SetAttribute("IsNamedHierarchy", "FALSE");
                            AttributesCollectionElement.AppendChild(AttributeElement);
                        }
                    }
                    ObjectAttribXml = ConfigRootElement.OuterXml;    
                }
                
                if (!String.IsNullOrEmpty(ObjectAttribXml))
                    AdvSearchStateObject.SetStateData(m_TypeConfigurationName, ObjectAttribXml);
                return (ObjectAttribXml);
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        /// <summary>
        /// /// This method is used to get user defined advanced search query node
        /// </summary>
        /// <param name="FacetRelationShipName">Relationship name</param>
        /// <returns></returns>
        public ErrorXML GetPredefinedSearchQueryListXML(String FacetRelationShipName)
        {
            String ErrorXML = null;
            try
            {
                ErrorXML = getPredefinedSearchQueryListXML(FacetRelationShipName, true);
            }
            catch (Exception)
            {
            }
            return ErrorXML;
        }

        public String SetAttributesBasedOnSource(UInt64 RowNumber, ref XML ListXml, String ListType)
        {
            # region Local Variables
            XmlDocument     ListXMLDoc              = null;
            XmlNode         ListNode                = null;
            XmlNode         RowNode                 = null;
            String          RelationshipName        = null;
            String          ObjectDisplayName       = null;
            XmlNode         ColumnNode              = null;
            XmlNode         TypeNode                = null;
            XmlNodeList     AttributeNodes          = null;
            StringBuilder   Attributes              = null;
            String          ObjectTypeString        = null;
            String          ObjectTypeName          = null;
            String          ParentEntityTypeName    = null;
            StringBuilder   Str_Select = null;
            StringBuilder   XPath = null;
            # endregion

            try
            {
                ListXMLDoc  = new XmlDocument();
                Attributes  = new StringBuilder();
                Str_Select = new StringBuilder();
                XPath = new StringBuilder();
                Str_Select.AppendFormat("---{0}---", Localization.GetString("ALResource", TenantName, "Select_Shared"));
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }

                ListXMLDoc.LoadXml(ListXml);
                ListNode    = ListXMLDoc.DocumentElement;

                XPath.Remove(0, XPath.Length);
                XPath.AppendFormat("Data/Group/Row[@Order={0}]", RowNumber);
                RowNode     =  ListNode.SelectSingleNode(XPath.ToString());

                if (RowNode != null)
                {
                    ColumnNode          = RowNode.SelectSingleNode("Column[@Name='Source']");
                    ObjectDisplayName   = ColumnNode.Attributes.GetNamedItem("SelectedText").Value;
                    RelationshipName    = ColumnNode.Attributes.GetNamedItem("SelectedValue").Value;
                }

                if (RelationshipName != "")
                {
                    getObjectAttributes(RelationshipName, ObjectDisplayName);

                    // Filling up the attributes into the drop down list
                   ColumnNode  = RowNode.SelectSingleNode("Column[@Name='Attribute']");

                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("STATE/DATA[@Name='TYPECONFIGURATION']/Configuration/Types/Type[@RelationshipName={0}]",ALXmlUtil.GetXpathValueString(RelationshipName));
                    TypeNode    = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(XPath.ToString());
                    if (TypeNode != null)
                    {
                        ObjectTypeString        = TypeNode.Attributes.GetNamedItem("ObjectType").Value;
                        ObjectTypeName          = TypeNode.Attributes.GetNamedItem("ObjectName").Value;
                        ParentEntityTypeName    = TypeNode.Attributes.GetNamedItem("ParentEntityTypeName").Value;
                        Attributes.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", Str_Select.ToString());
                        AttributeNodes          = TypeNode.SelectNodes("Attributes/Attribute");
                        foreach (XmlNode AttributeNode in AttributeNodes)
                        {
                            Attributes.AppendFormat("<Item Text=\"{0}\" Value=\"{1}\" />",ALCommon.TrimAndReplaceSpecialChars(AttributeNode.Attributes.GetNamedItem("DisplayText").Value), AttributeNode.Attributes.GetNamedItem("Value").Value);
                        }
                        ColumnNode.InnerXml     = Attributes.ToString();
                        ((XmlElement)ColumnNode).SetAttribute("SelectedText",Str_Select.ToString());
                        ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");
                        ((XmlElement)ColumnNode).SetAttribute("Selected", "true");
                    }
                    // Setting the ObjectType value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ObjectType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", ObjectTypeString);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", ObjectTypeString);

                    // Setting the ObjectName value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ObjectName']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", ObjectTypeName);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", ObjectTypeName);

                    // Setting the ParentEntityTypeName value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ParentEntityTypeName']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", ParentEntityTypeName);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", ParentEntityTypeName);

                    if (String.Compare(ListType, "FILTERLIST", true) == 0)
                    {
                        // Filling up the default operator into the dropdown list
                        ColumnNode              = RowNode.SelectSingleNode("Column[@Name='Operator']");
                        Attributes.Remove(0, Attributes.Length);
                        Attributes.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", Str_Select.ToString());
                        ColumnNode.InnerXml     = Attributes.ToString();
                        ((XmlElement)ColumnNode).SetAttribute("SelectedText", Str_Select.ToString());
                        ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");
                        ((XmlElement)ColumnNode).SetAttribute("Selected", "true");
                    }
                }
                else
                {
                    // Setting the ObjectType value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ObjectType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Setting the ObjectName value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ObjectName']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Setting the ParentEntityTypeName value
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='ParentEntityTypeName']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Filling up the default Attribute into the drop down list
                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("Column[@Name='Attribute']", RowNumber);
                    ColumnNode              = RowNode.SelectSingleNode(XPath.ToString());
                    Attributes.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", Str_Select.ToString());
                    ColumnNode.InnerXml     = Attributes.ToString();

                    if (String.Compare(ListType, "FILTERLIST", true) == 0)
                    {
                        // Filling up the default operator into the dropdown list
                        XPath.Remove(0, XPath.Length);
                        XPath.AppendFormat("Column[@Name='Operator']", RowNumber);
                        ColumnNode              = RowNode.SelectSingleNode(XPath.ToString());
                        ColumnNode.InnerXml     = Attributes.ToString();
                    }
                }

                ListXml     = ListXMLDoc.InnerXml;
                AdvSearchStateObject.SetStateData(ListType, ListXml);
            }
            catch (Exception ExceptionObj)
            {
                return ExceptionObj.Message;
            }
            return null;
        }

        public String SetOperatorsBasedOnAttribute(UInt64 RowNumber, ref XML ListXml, String ListType)
        {
            # region Local Variables
            XmlDocument     ListXMLDoc              = null;
            XmlNode         ListNode                = null;
            XmlNode         RowNode                 = null;
            String          RelationshipName        = null;
            String          ObjectDisplayName       = null;
            String          AttributeName           = null;
            XmlNode         ColumnNode              = null;
            XmlNodeList     OperatorNodes           = null;
            StringBuilder   OperatorList            = null;
            String          AttributeDataType       = null;
            String          PrimaryKey              = null;
            String          FieldUIType             = null;
            XmlNode         AttributeNode           = null;
            XmlNode         ItemNode                = null;
            String          IsNamedHierarchy        = null;
            StringBuilder   NamedHierarchyFilePath  = null;
            String          TempAttributeDataType   = null;
            StringBuilder   Str_Select              = null;
            String          DisplayTextID           = null;
            StringBuilder   XPath                   = null;
            # endregion

            try
            {
                XPath = new StringBuilder();
                Str_Select = new StringBuilder();
                Str_Select.AppendFormat("---{0}---", Localization.GetString("ALResource", TenantName, "Select_Shared"));
                                
                ListXMLDoc      = new XmlDocument();
                OperatorList    = new StringBuilder();
                ListXMLDoc.LoadXml(ListXml);
                ListNode        = ListXMLDoc.DocumentElement;
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }

                XPath.Remove(0, XPath.Length);
                XPath.AppendFormat("Data/Group/Row[@Order={0}]", RowNumber);
                RowNode     =  ListNode.SelectSingleNode(XPath.ToString());

                if (RowNode != null)
                {
                    ColumnNode          = RowNode.SelectSingleNode("Column[@Name='Source']");
                    ObjectDisplayName   = ColumnNode.Attributes.GetNamedItem("SelectedText").Value;
                    RelationshipName    = ColumnNode.Attributes.GetNamedItem("SelectedValue").Value;
                    ColumnNode          = RowNode.SelectSingleNode("Column[@Name='Attribute']");
                    AttributeName       = ColumnNode.Attributes.GetNamedItem("SelectedValue").Value;
                           
                    ((XmlElement)ColumnNode).SetAttribute("Selected", "true");

                    // Deselect the previosly selected Attribute
                    XPath.Remove(0, XPath.Length);
                    XPath.Append("Item[@Selected='true']");
                    ItemNode            = ColumnNode.SelectSingleNode(XPath.ToString());
                    if (ItemNode != null)
                        ((XmlElement)ItemNode).RemoveAttribute("Selected");

                    // Set the selected attribute
                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("Item[@Value='{0}']", AttributeName);
                    ItemNode            = ColumnNode.SelectSingleNode(XPath.ToString());
                    ((XmlElement)ItemNode).SetAttribute("Selected", "true");
                }

                if ((RelationshipName.Trim().Length > 0) && (AttributeName.Trim().Length > 0))
                {
                    // Filling up the Operators into the drop down list
                    XPath.Remove(0, XPath.Length);
                    XPath.Append("Column[@Name='Operator']");
                    ColumnNode          = RowNode.SelectSingleNode(XPath.ToString());

                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("STATE/DATA[@Name='TYPECONFIGURATION']/Configuration/Types/Type[@RelationshipName={0}]/Attributes/Attribute[@Value='{1}']", ALXmlUtil.GetXpathValueString(RelationshipName), AttributeName);
                    AttributeNode       = AdvSearchStateObject.StateXMLDoc.SelectSingleNode(XPath.ToString());
                    if (AttributeNode != null)
                    {
                        AttributeDataType   = AttributeNode.Attributes.GetNamedItem("FieldDataType").Value;
                        PrimaryKey          = AttributeNode.Attributes.GetNamedItem("PrimaryKey").Value;
                        FieldUIType         = AttributeNode.Attributes.GetNamedItem("FieldUIType").Value;
                        IsNamedHierarchy    = AttributeNode.Attributes.GetNamedItem("IsNamedHierarchy").Value;

                        if ((IsNamedHierarchy.ToUpper() == "TRUE") && (FieldUIType.ToUpper() == "TREEVIEW"))
                            TempAttributeDataType = "NAMEDHIERARCHY";
                        else
                            TempAttributeDataType = AttributeDataType.ToUpper();

                        OperatorList.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", Str_Select.ToString());

                        XPath.Remove(0, XPath.Length);
                        XPath.AppendFormat("{0}/OPERATORS/OPERATOR", TempAttributeDataType);

                        if (OperatorsXMLRootNode == null)
                        {
                            getOperatorsXml();
                        }
                        OperatorNodes       = OperatorsXMLRootNode.SelectNodes(XPath.ToString());
                        foreach (XmlNode OperatorNode in OperatorNodes)
                        {
                            if (OperatorNode.Attributes.GetNamedItem("DisplayTextID") != null)
                            {
                                DisplayTextID = OperatorNode.Attributes.GetNamedItem("DisplayTextID").Value;
                                if (!String.IsNullOrEmpty(DisplayTextID))
                                    OperatorList.AppendFormat("<Item Text=\"{0}\" Value=\"{1}\" />", Localization.GetString("ALResource", TenantName, DisplayTextID), ALSolutionCommon.TrimAndReplaceSpecialChars(OperatorNode.Attributes.GetNamedItem("Value").Value));
                                else
                                    OperatorList.AppendFormat("<Item Text=\"{0}\" Value=\"{1}\" />", ALSolutionCommon.TrimAndReplaceSpecialChars(OperatorNode.Attributes.GetNamedItem("DisplayText").Value), ALSolutionCommon.TrimAndReplaceSpecialChars(OperatorNode.Attributes.GetNamedItem("Value").Value));
                            }
                            else
                                OperatorList.AppendFormat("<Item Text=\"{0}\" Value=\"{1}\" />", ALSolutionCommon.TrimAndReplaceSpecialChars(OperatorNode.Attributes.GetNamedItem("DisplayText").Value), ALSolutionCommon.TrimAndReplaceSpecialChars(OperatorNode.Attributes.GetNamedItem("Value").Value));
                        }
                        ColumnNode.InnerXml = OperatorList.ToString();
                        ((XmlElement)ColumnNode).SetAttribute("SelectedText", Str_Select.ToString());
                        ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");
                    }
                    // Setting DataType of the Attribute
                    ColumnNode  = RowNode.SelectSingleNode("Column[@Name='AttributeDataType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", AttributeDataType);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", AttributeDataType);

                    // Setting PrimaryKey 
                    ColumnNode  = RowNode.SelectSingleNode("Column[@Name='PrimaryKey']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", PrimaryKey);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", PrimaryKey);

                    // Setting PrimaryKey 
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='FieldUIType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", FieldUIType);
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", FieldUIType);
                }
                else
                {
                    // Setting DataType of the Attribute
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='AttributeDataType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Setting PrimaryKey 
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='PrimaryKey']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Setting PrimaryKey 
                    ColumnNode = RowNode.SelectSingleNode("Column[@Name='FieldUIType']");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedText", "");
                    ((XmlElement)ColumnNode).SetAttribute("SelectedValue", "");

                    // Filling up the default Operator into the drop down list
                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("Column[@Name='Operator']", RowNumber);
                    ColumnNode = RowNode.SelectSingleNode(XPath.ToString());
                    OperatorList.AppendFormat("<Item Text=\"{0}\" Value=\"\" />", Str_Select.ToString());
                    ColumnNode.InnerXml = OperatorList.ToString();
                }
                ListXml        = ListXMLDoc.InnerXml;
                AdvSearchStateObject.SetStateData(ListType, ListXml);

                //If the Field is NamedHierarchy then check if the NamedHierarchy values xml exists
                // on NTFS if then delete it(This code to delete the temp file is also present in getAttributeValue api but
                // still this code is required here too becasue when advance search url is clicked that time previously
                // selected value(i.e previous visit to advanced search in same session) should not be shown)
                if (IsNamedHierarchy.ToUpper() == "TRUE")
                {
                    NamedHierarchyFilePath = new StringBuilder();
                    NamedHierarchyFilePath.AppendFormat(@"{0}\{1}\NamedHeirarchyData\ReportSrch_{2}_{3}.xml", ALSolutionCommon.GetTempPath(SessionID), UserID, AttributeName, RowNumber);
                    if (File.Exists(NamedHierarchyFilePath.ToString()))
                    {
                        File.Delete(NamedHierarchyFilePath.ToString());
                    }
                }

            }
            catch (Exception ExceptionObj)
            {
                return ExceptionObj.Message;
            }
            return null;
        }

        public String SetSelectedOperator(UInt64 RowNumber, ref XML ListXml, String ListType)
        {
            # region Local Variables
            XmlDocument     ListXMLDoc          = null;
            XmlNode         ListNode            = null;
            XmlNode         RowNode             = null;
            XmlNode         ColumnNode          = null;
            StringBuilder   OperatorList        = null;
            XmlNode         ItemNode            = null;
            String          OperatorValue       = null;
            StringBuilder   XPath               = null;
            # endregion
            try
            {
                XPath           = new StringBuilder();
                ListXMLDoc      = new XmlDocument();
                OperatorList    = new StringBuilder();
                ListXMLDoc.LoadXml(ListXml);
                ListNode        = ListXMLDoc.DocumentElement;
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }

                XPath.Remove(0, XPath.Length);
                XPath.AppendFormat("Data/Group/Row[@Order={0}]", RowNumber);
                RowNode         =  ListNode.SelectSingleNode(XPath.ToString());

                if (RowNode != null)
                {
                    ColumnNode          = RowNode.SelectSingleNode("Column[@Name='Operator']");
                    OperatorValue       = ColumnNode.Attributes.GetNamedItem("SelectedValue").Value;
                    ((XmlElement)ColumnNode).SetAttribute("Selected", "true");

                    // Deselect the previosly selected Operator
                    XPath.Remove(0, XPath.Length);
                    XPath.Append("Item[@Selected='true']");
                    ItemNode            = ColumnNode.SelectSingleNode(XPath.ToString());
                    if (ItemNode != null)
                        ((XmlElement)ItemNode).RemoveAttribute("Selected");

                    // Set the selected Operator
                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("Item[@Value='{0}']", OperatorValue);
                    ItemNode            = ColumnNode.SelectSingleNode(XPath.ToString());
                    ((XmlElement)ItemNode).SetAttribute("Selected", "true");
                }

                ListXml         = ListXMLDoc.InnerXml;
                AdvSearchStateObject.SetStateData(ListType, ListXml);
            }
            catch (Exception ExceptionObj)
            {
                return ExceptionObj.Message;
            }
            return null;
        }

        /// <summary>
        /// This method is used to generate the list with rows (data)
        /// </summary>
        /// <param name="ListQueryNode"></param>
        /// <param name="StateFlag"></param>
        public void GenerateListXMLWithData(XmlNode ListQueryNode, Boolean StateFlag, String ListType, String ListHeading)
        {
            #region Variables
            XmlDocument     ListDoc                     = null;
            XmlElement      ListElement                 = null;
            String          OrderNumberValue            = null;
            String          AttributeNameValue          = null;
            String          OperatorValue               = null;
            String          AttributeActualValue        = null;
            String          RelationshipNameValue       = null;
            String          AttributeDisplayNameValue   = null;
            String          AttributeDataType           = null;
            String          ObjectTypeVal               = null;
            String          PrimaryKey                  = null;
            StringBuilder   ClauseValue                 = null;
            XmlNode         RowNode                     = null;
            XmlNode         GroupNode                   = null;
            XmlNode         TempNode                    = null;
            XmlNode         TempRowNode                 = null;
            Int32           DiffCount                   = 0;
            String          SourceDisplayName           = null;
            String          OperatorDisplayText         = null;
            String          ObjectName                  = null;
            String          ParentEntityTypeName        = null;
            String          FieldUIType                 = null;
            String          AttributeDispValue          = null;
            #endregion

            try
            {
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }

                ListDoc = new XmlDocument();
                if(StateFlag==false)
                    ListDoc.LoadXml(GetListXML(ListType, ListHeading));
                else
                    ListDoc.LoadXml(GenerateListXMLWithoutData(true, ListType, ListHeading));

                if (ListQueryNode != null)
                {
                    ClauseValue = new StringBuilder();
                    ListElement = (XmlElement)ListDoc.SelectSingleNode("//List");
                    if (ListElement != null)
                    {
                        DiffCount = ListQueryNode.SelectNodes("//FILTER").Count - Convert.ToInt32(ListElement.Attributes["NumberOfRows"].Value);
                        if (DiffCount > 0)
                        {
                            AddNewRow(DiffCount, ListType, ListHeading);
                            ListDoc.LoadXml(GetListXML(ListType, ListHeading));
                        }
                        foreach (XmlNode ClauseNode in ListQueryNode.SelectNodes("//FILTER"))
                        {
                            AttributeDispValue          = null;
                            OrderNumberValue            = ClauseNode.SelectSingleNode("ORDER").InnerText;
                            GroupNode                   = ListDoc.SelectSingleNode("//List/Data/Group");
                            RowNode                     = GroupNode.SelectSingleNode("//Row[@Order='" + OrderNumberValue + "']");
                            TempRowNode                 = ListDoc.CreateElement("Row");
                            TempRowNode.InnerXml        = RowNode.InnerXml;
                            if (RowNode != null)
                            {
                                ((XmlElement)TempRowNode).SetAttribute("Order", OrderNumberValue.ToString());
                                ((XmlElement)TempRowNode).SetAttribute("SelectionValue", OrderNumberValue.ToString());
                               
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='Source']");
                                RelationshipNameValue   = TempNode.Attributes.GetNamedItem("SelectedValue").Value;

                                SourceDisplayName       = TempNode.Attributes.GetNamedItem("SelectedText").Value;
                                
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='Attribute']");
                                AttributeNameValue      = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                AttributeDisplayNameValue = TempNode.Attributes.GetNamedItem("SelectedText").Value;
                               
                                if (String.Compare(ListType, "FILTERLIST", true) == 0)
                                {
                                    TempNode                = RowNode.SelectSingleNode("Column[@Name='Operator']");
                                    OperatorValue           = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                    OperatorDisplayText     = TempNode.Attributes.GetNamedItem("SelectedText").Value;
                                }

                                TempNode                = RowNode.SelectSingleNode("Column[@Name='AttributeDataType']");
                                AttributeDataType       = TempNode.Attributes.GetNamedItem("SelectedValue").Value;

                                TempNode                = RowNode.SelectSingleNode("Column[@Name='FieldUIType']");
                                FieldUIType             = TempNode.Attributes.GetNamedItem("SelectedValue").Value;

                                TempNode                = RowNode.SelectSingleNode("Column[@Name='ObjectType']");
                                ObjectTypeVal           = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                                       
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='PrimaryKey']");
                                PrimaryKey              = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='ObjectName']");
                                ObjectName              = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='ParentEntityTypeName']");
                                ParentEntityTypeName    = TempNode.Attributes.GetNamedItem("SelectedValue").Value;

                                AttributeActualValue    = ClauseNode.SelectSingleNode("ATTRIBUTEVALUE").InnerText;
                                if (ClauseNode.SelectSingleNode("ATTRIBUTEDISPLAYVALUE") != null)
                                    AttributeDispValue = ClauseNode.SelectSingleNode("ATTRIBUTEDISPLAYVALUE").InnerText;

                                TempNode            = TempRowNode.SelectSingleNode("//Column[@Name='AttributeValue']");
                                TempNode.InnerText = setAttributeValues(TempNode.OuterXml, AttributeActualValue, OperatorValue, OrderNumberValue, ObjectTypeVal, ObjectName, SourceDisplayName, AttributeNameValue, AttributeDataType, FieldUIType, AttributeDispValue);
                                
                                (RowNode.ParentNode).ReplaceChild(TempRowNode, RowNode);

                            }
                        }
                    }
                }
                AdvSearchStateObject.SetStateData(ListType, ListDoc.OuterXml);
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        /// <summary>
        /// This method is used to delete the filter row from the list XML
        /// </summary>
        /// <param name="SelectedFilters">Filter rows to be deleted from the list XML</param>
        /// <returns>true if deleted successfully else false</returns>
        public Boolean DeleteFilter(String SelectedFilters, String ListType)
        {
            #region Variable
            XmlNodeList RowNodeList                 = null;
            XmlNode     ListTypeNode                = null;
            XmlNode     GroupNode                   = null;
            XmlNode     RemoveRowNode               = null;
            XmlNode     RowNode                     = null;
            Int32       RowDeletedCount             = 0;
            Int32       SelRowNumber                = 0;
            Int32       CurrentRowNumber            = 0;
            Int32       TotalRemovedCount           = 0;
            String[]    ArrSelectedClauses          = null;
            Boolean     FlagRowDeleted              = false;
            XmlNode     SequenceNode                = null;
            XmlNode     AttributeColumnNode         = null;
            XmlNode     SingleDivNode               = null;
            XmlNode     MultiDivNode                = null;
            XmlNode     InputNode                   = null;
            XmlNode     JavaScriptNode              = null;
            XmlNode     SingleCalendarDivNode       = null;
            XmlNode     MultiCalendarDivNode        = null;
            XmlNode     InputCalendarTextNode       = null;
            XmlNode     InputHiddenTenantNode       = null;
            XmlNode     InputThemeNode              = null;
            XmlNode     InputCalendarTitleNode      = null;
            XmlNode     InputHiddenCalendarTextNode = null;
            XmlNode     SingleNamedHierarchyDivNode = null;
            XmlNode     SingleDropDownDivNode       = null;
            XmlNode     TempNodeForUpdatingControlID = null;
            XmlNode     InputCalendarMonthNode      = null;
            XmlNode     InputCalendarSMonthNode     = null;
            XmlNode     InputCalendarWeekDayNode    = null;
            #endregion

            try
            {
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                ListTypeNode                    = AdvSearchStateObject.StateXMLDoc.SelectSingleNode("STATE/DATA[@Name='"
                                        + ListType + "']");
                if (ListTypeNode != null)
                {
                    GroupNode = ListTypeNode.SelectSingleNode("List/Data/Group");
                    if (GroupNode != null)
                    {
                        ArrSelectedClauses = SelectedFilters.Split(',');
                        if (ArrSelectedClauses.Length == GroupNode.SelectNodes("Row").Count)
                            FlagRowDeleted = DeleteAllFilters(ListType);
                        else
                        {
                            TempNodeForUpdatingControlID = GroupNode.OwnerDocument.CreateElement("CONTROLTOUPDATE");
                            for (Int32 ArrCount = 0; ArrCount < ArrSelectedClauses.Length; ArrCount++)
                            {
                                if (ArrSelectedClauses[ArrCount].Trim().Length > 0)
                                {
                                    SelRowNumber = Convert.ToInt32(ArrSelectedClauses[ArrCount]) - RowDeletedCount;
                                    RowDeletedCount++;
                                    RemoveRowNode = GroupNode.SelectSingleNode("Row[@Order='"
                                                            + SelRowNumber + "']");
                                    if (RemoveRowNode != null)
                                    {
                                        TotalRemovedCount++;
                                        FlagRowDeleted  = true;
                                        GroupNode.RemoveChild(RemoveRowNode);

                                        RowNodeList = GroupNode.SelectNodes("Row");
                                        for (Int32 RowCount = 0; RowCount < RowNodeList.Count; RowCount++)
                                        {
                                            RowNode = RowNodeList[RowCount];
                                            CurrentRowNumber = Convert.ToInt32(RowNode.Attributes["Order"].Value);
                                            if (CurrentRowNumber > SelRowNumber)
                                            {
                                                RowNode.Attributes["Order"].Value =
                                                                    Convert.ToString(CurrentRowNumber - 1);
                                                RowNode.Attributes["SelectionValue"].Value =
                                                                    RowNode.Attributes["Order"].Value;
                                                
                                                SequenceNode = RowNode.SelectSingleNode("descendant::Column[@Name='SequenceNo']");
                                                if (SequenceNode != null)
                                                    SequenceNode.InnerText = RowNode.Attributes["Order"].Value;

                                                
                                                //Seting the Control Id's
                                                AttributeColumnNode = RowNode.SelectSingleNode("descendant::Column[@Name='AttributeValue']");
                                                if (AttributeColumnNode != null)
                                                {
                                                    TempNodeForUpdatingControlID.InnerXml = AttributeColumnNode.InnerText;
                                                    //Single Text
                                                    SingleDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='SingleTextDiv_" + CurrentRowNumber + "']");
                                                    if (SingleDivNode != null)
                                                    {
                                                        ((XmlElement)SingleDivNode).SetAttribute("id", "SingleTextDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode = SingleDivNode.SelectSingleNode("descendant::input[@id='SingleText_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "SingleText_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputNode).SetAttribute("name", "SingleText_" + Convert.ToString(CurrentRowNumber - 1));
                                                        AttributeColumnNode.InnerText = SingleDivNode.OuterXml;
                                                    }
                                                    //Multi Text 
                                                    MultiDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='MultiTextDiv_" + CurrentRowNumber + "']");
                                                    if (MultiDivNode != null)
                                                    {
                                                        ((XmlElement)MultiDivNode).SetAttribute("id", "MultiTextDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode = MultiDivNode.SelectSingleNode("descendant::input[@id='MultiTextValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "MultiTextValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputNode).SetAttribute("name", "MultiTextValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode = MultiDivNode.SelectSingleNode("descendant::input[@id='MultiTextValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "MultiTextValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputNode).SetAttribute("name", "MultiTextValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        AttributeColumnNode.InnerText = MultiDivNode.OuterXml;
                                                    }

                                                    //single DropDown 
                                                    SingleDropDownDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='SingleDropDownDiv_" + CurrentRowNumber + "']");
                                                    if (SingleDropDownDivNode != null)
                                                    {
                                                        ((XmlElement)SingleDropDownDivNode).SetAttribute("id", "SingleDropDownDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode   = SingleDropDownDivNode.SelectSingleNode("descendant::select[@id='SingleDropDown_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "SingleDropDown_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputNode).SetAttribute("name", "SingleDropDown_" + Convert.ToString(CurrentRowNumber - 1));
                                                        AttributeColumnNode.InnerText = SingleDropDownDivNode.OuterXml;
                                                    }

                                                    //single NamedHierarchy  
                                                    SingleNamedHierarchyDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='NamedHierarchyDiv_" + CurrentRowNumber + "']");
                                                    if (SingleNamedHierarchyDivNode != null)
                                                    {
                                                        ((XmlElement)SingleNamedHierarchyDivNode).SetAttribute("id", "NamedHierarchyDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode   = SingleNamedHierarchyDivNode.SelectSingleNode("descendant::input[@id='HiddenInput_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "HiddenInput_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputNode).SetAttribute("name", "HiddenInput_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputNode   = SingleNamedHierarchyDivNode.SelectSingleNode("descendant::span[@id='Hidden_NHBtn_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputNode).SetAttribute("id", "Hidden_NHBtn_" + Convert.ToString(CurrentRowNumber - 1));
                                                        AttributeColumnNode.InnerText = SingleNamedHierarchyDivNode.OuterXml;
                                                    }

                                                    //Single Calendar Control                                                
                                                    SingleCalendarDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='SingleCalendarDiv_" + CurrentRowNumber + "']");
                                                    if (SingleCalendarDivNode != null)
                                                    {
                                                        ((XmlElement)SingleCalendarDivNode).SetAttribute("id", "SingleCalendarDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarTextNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("id", "SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("name", "SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenCalendarTextNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='SingleHiddenText_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("id", "SingleHiddenText_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("name", "SingleHiddenText_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenTenantNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__TENANTNAME_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("id", "__TENANTNAME_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("name", "__TENANTNAME_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputThemeNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__THEMENAME_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputThemeNode).SetAttribute("id", "__THEMENAME_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputThemeNode).SetAttribute("name", "__THEMENAME_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarTitleNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__CalendarTitle_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("id", "__CalendarTitle_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("name", "__CalendarTitle_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarMonthNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__TMonthNameArray_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("id", "__TMonthNameArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("name", "__TMonthNameArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarSMonthNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__TShortMonthNameArray_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("id", "__TShortMonthNameArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("name", "__TShortMonthNameArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarWeekDayNode = SingleCalendarDivNode.SelectSingleNode("descendant::input[@id='__TWeekDayArray_SingleCalendar_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("id", "__TWeekDayArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("name", "__TWeekDayArray_SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1));

                                                        JavaScriptNode = SingleCalendarDivNode.SelectSingleNode("descendant::a[@href=\"Javascript:show_calendar('SingleCalendar_" + CurrentRowNumber + "','SingleHiddenText_"+CurrentRowNumber+"','forms[0].SingleCalendar_"+CurrentRowNumber+"')\"]");
                                                        ((XmlElement)JavaScriptNode).SetAttribute("href", "Javascript:show_calendar('SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1) + "','SingleHiddenText_" + Convert.ToString(CurrentRowNumber - 1) + "','forms[0].SingleCalendar_" + Convert.ToString(CurrentRowNumber - 1) + "')");
                                                        AttributeColumnNode.InnerText = SingleCalendarDivNode.OuterXml;
                                                    }

                                                    //Multi Calendar Control
                                                    MultiCalendarDivNode = TempNodeForUpdatingControlID.SelectSingleNode("descendant::div[@id='MultiCalendarDiv_" + CurrentRowNumber + "']");
                                                    if (MultiCalendarDivNode != null)
                                                    {
                                                        ((XmlElement)MultiCalendarDivNode).SetAttribute("id", "MultiCalendarDiv_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarTextNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("id", "MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("name", "MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenCalendarTextNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='MultiHiddenTextValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("id", "MultiHiddenTextValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("name", "MultiHiddenTextValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenTenantNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TENANTNAME_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("id", "__TENANTNAME_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("name", "__TENANTNAME_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputThemeNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__THEMENAME_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputThemeNode).SetAttribute("id", "__THEMENAME_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputThemeNode).SetAttribute("name", "__THEMENAME_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarTitleNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__CalendarTitle_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("id", "__CalendarTitle_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("name", "__CalendarTitle_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarMonthNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TMonthNameArray_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("id", "__TMonthNameArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("name", "__TMonthNameArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarSMonthNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TShortMonthNameArray_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("id", "__TShortMonthNameArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("name", "__TShortMonthNameArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarWeekDayNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TWeekDayArray_MultiCalendarValue1_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("id", "__TWeekDayArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("name", "__TWeekDayArray_MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1));
                                                       
                                                        JavaScriptNode = MultiCalendarDivNode.SelectSingleNode("descendant::a[@href=\"Javascript:show_calendar('MultiCalendarValue1_" + CurrentRowNumber + "','MultiHiddenTextValue1_" + CurrentRowNumber + "','forms[0].MultiCalendarValue1_" + CurrentRowNumber + "')\"]");
                                                        ((XmlElement)JavaScriptNode).SetAttribute("href", "Javascript:show_calendar('MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1) + "','MultiHiddenTextValue1_" + Convert.ToString(CurrentRowNumber - 1) + "','forms[0].MultiCalendarValue1_" + Convert.ToString(CurrentRowNumber - 1) + "')");

                                                        //2nd Calendar Control
                                                        InputCalendarTextNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("id", "MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTextNode).SetAttribute("name", "MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenCalendarTextNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='MultiHiddenTextValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("id", "MultiHiddenTextValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenCalendarTextNode).SetAttribute("name", "MultiHiddenTextValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputHiddenTenantNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TENANTNAME_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("id", "__TENANTNAME_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputHiddenTenantNode).SetAttribute("name", "__TENANTNAME_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputThemeNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__THEMENAME_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputThemeNode).SetAttribute("id", "__THEMENAME_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputThemeNode).SetAttribute("name", "__THEMENAME_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarTitleNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__CalendarTitle_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("id", "__CalendarTitle_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarTitleNode).SetAttribute("name", "__CalendarTitle_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarMonthNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TMonthNameArray_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("id", "__TMonthNameArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarMonthNode).SetAttribute("name", "__TMonthNameArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarSMonthNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TShortMonthNameArray_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("id", "__TShortMonthNameArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarSMonthNode).SetAttribute("name", "__TShortMonthNameArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));

                                                        InputCalendarWeekDayNode = MultiCalendarDivNode.SelectSingleNode("descendant::input[@id='__TWeekDayArray_MultiCalendarValue2_" + CurrentRowNumber + "']");
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("id", "__TWeekDayArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        ((XmlElement)InputCalendarWeekDayNode).SetAttribute("name", "__TWeekDayArray_MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1));
                                                        
                                                        JavaScriptNode = MultiCalendarDivNode.SelectSingleNode("descendant::a[@href=\"Javascript:show_calendar('MultiCalendarValue2_" + CurrentRowNumber + "','MultiHiddenTextValue2_" + CurrentRowNumber + "','forms[0].MultiCalendarValue2_" + CurrentRowNumber + "')\"]");
                                                        ((XmlElement)JavaScriptNode).SetAttribute("href", "Javascript:show_calendar('MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1) + "','MultiHiddenTextValue2_" + Convert.ToString(CurrentRowNumber - 1) + "','forms[0].MultiCalendarValue2_" + Convert.ToString(CurrentRowNumber - 1) + "')");
                                                        AttributeColumnNode.InnerText = MultiCalendarDivNode.OuterXml;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            if (FlagRowDeleted == true)
                            {
                                GroupNode.Attributes["EndRow"].Value = Convert.ToString(
                                        Convert.ToInt32(GroupNode.Attributes["EndRow"].Value) - TotalRemovedCount);
                                ListTypeNode.SelectSingleNode("List").Attributes["NumberOfRows"].Value =
                                        GroupNode.Attributes["EndRow"].Value;

                                AdvSearchStateObject.SetStateData(ListType, ListTypeNode.InnerXml);
                            }
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (FlagRowDeleted);
        }

        /// <summary>
        /// This method is used to extract the data from list control and make the XML structure
        /// </summary>
        /// <param name="ListXml"></param>
        /// <param name="FieldValueXml"></param>
        /// <param name="ClauseDoc"></param>
        /// <param name="ErrorMessage"></param>
        /// <returns></returns>
        public Boolean GetDataFromListXML(XML ListXml, XML FieldValueXml,ref XmlDocument ClauseDoc, ref String ErrorMessage, String ListType)
        {
            #region Local Variables
            XmlNode         ListTypeNode            = null;
            XmlNodeList     RowNodeList             = null;
            XmlElement      FilterListNode          = null;
            XmlElement      FilterNode              = null;
            XmlElement      ColumnElement           = null;
            XmlNode         TempNode                = null;
            String          RelationshipName        = null;
            String          AttributeName           = null;
            String          OperatorValue           = null; 
            String          SourceDisplayText       = null; 
            String          AttributeStringValue    = null;
            String          AttributeDataType       = null;
            XmlNode         RootElement             = null;
            Int32           RowCount                = 0;
            String          Value1                  = null;
            String          Value2                  = null;
            bool            IsNHValue               = false;
            String          AttributeDispValue      = null;
            #endregion

            try
            {
                if (AdvSearchStateObject == null)
                {
                    getAdvSearchStateObject();
                }
                ErrorMessage = null;
                AdvSearchStateObject.SetStateData(ListType, ListXml);

                ListTypeNode = AdvSearchStateObject.StateXMLDoc.SelectSingleNode("STATE/DATA[@Name='" + ListType + "']");
                if (ListTypeNode != null)
                {
                    RowNodeList             = ListTypeNode.SelectNodes("List/Data/Group/Row");
                    if (RowNodeList.Count > 0)
                    {
                        RootElement         = ClauseDoc.DocumentElement;
                        FilterListNode      = ClauseDoc.CreateElement(ListType);
                        RootElement.AppendChild(FilterListNode);

                        RowCount            = 0;
                        foreach (XmlNode RowNode in RowNodeList)
                        {
                            RowCount++;
                            TempNode                    = RowNode.SelectSingleNode("Column[@Name='Source']");
                            RelationshipName            = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                            AttributeDispValue          = null;

                            if (!String.IsNullOrEmpty(RelationshipName))
                            {
                                FilterNode              = ClauseDoc.CreateElement("FILTER");
                                ColumnElement           = ClauseDoc.CreateElement("ORDER");
                                ColumnElement.InnerText = RowNode.Attributes["Order"].Value;
                                FilterNode.AppendChild(ColumnElement);

                                SourceDisplayText       = TempNode.Attributes.GetNamedItem("SelectedText").Value;
                                
                                TempNode                = RowNode.SelectSingleNode("Column[@Name='Attribute']");
                                AttributeName           = TempNode.Attributes.GetNamedItem("SelectedValue").Value;


                                if (String.Compare(ListType, "FILTERLIST", true) == 0)
                                {
                                    TempNode                = RowNode.SelectSingleNode("Column[@Name='Operator']");
                                    OperatorValue           = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                    
                                    TempNode                = RowNode.SelectSingleNode("Column[@Name='AttributeDataType']");
                                    AttributeDataType       = TempNode.Attributes.GetNamedItem("SelectedValue").Value;
                                
                                    TempNode                = RowNode.SelectSingleNode("Column[@Name='AttributeValue']");
                                    if (TempNode != null)
                                    {
                                       IsNHValue            = isNamedHierarchyField(SourceDisplayText, AttributeName);
                                       AttributeStringValue = getAttributeValue(AttributeName, AttributeDataType,FieldValueXml, RowCount, OperatorValue, ref Value1, ref Value2,IsNHValue, ref AttributeDispValue);
                                    }

                                    ColumnElement           = ClauseDoc.CreateElement("ATTRIBUTEVALUE");
                                    ColumnElement.InnerText = AttributeStringValue;
                                    FilterNode.AppendChild(ColumnElement);

                                    ColumnElement = ClauseDoc.CreateElement("ATTRIBUTEDISPLAYVALUE");
                                    ColumnElement.InnerText = AttributeDispValue;
                                    FilterNode.AppendChild(ColumnElement);

                                    FilterListNode.AppendChild(FilterNode);
                                }
                                else
                                {
                                    
                                    TempNode = RowNode.SelectSingleNode("Column[@Name='AttributeDataType']");
                                    //if (TempNode != null)
                                    //{
                                    //    ColumnElement = ClauseDoc.CreateElement("AttributeDataType");
                                    //    AttributeDataType = TempNode.Attributes.GetNamedItem("AttributeDataType").Value;
                                    //    FilterNode.AppendChild(ColumnElement);
                                    //}
                                }
                             }
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
            return (true);
        }

        /// <summary>
        /// GetFilterXML() method is used to retrieve the clause list data from list control and make the XML structure
        /// </summary>
        /// <param name="ListXml">List control source xml</param>
        /// <param name="FieldValueXml">Selected values in list control</param>
        /// <returns>Xml string containing clause list data</returns>
        public XML GetFilterXML(XML ListXml, XML FieldValueXml, String ListType, String ListHeading)
        {
            # region Local Variables
            XmlDocument     ListDataDoc             = null;
            XmlElement      RootElement             = null;
            Boolean         FlagSuccess             = false;
            String          ErrorMessage            = null;
            AdvancedSearch  AdvancedSearchFacadeObj = null;
            # endregion

            try
            {
                ListDataDoc = new XmlDocument();
                RootElement = ListDataDoc.CreateElement("SELECTIONQUERY");
                ListDataDoc.AppendChild(RootElement);

                AdvancedSearchFacadeObj = new AdvancedSearch(SessionID, UserID, TenantName, CurrentLanguage, SolutionName, "DAM_AdvancedSearch_State", "Default");
                AdvancedSearchFacadeObj.ListType = ListType;
                AdvancedSearchFacadeObj.ListHeading = ListHeading;

                FlagSuccess = AdvancedSearchFacadeObj.GetDataFromListXML(ListXml, FieldValueXml, ref ListDataDoc, ref ErrorMessage);


                if (FlagSuccess == true)
                {
                    return (ListDataDoc.OuterXml);
                }
                else
                {
                    return (null);
                }
            }
            catch (Exception ExceptionObj)
            {
                throw (ExceptionObj);
            }
        }

        /// <summary>
        /// To verify standard report input xml format
        /// </summary>
        /// <param name="ReportInputXml">User specified input xml of report</param>
        /// <returns>
        /// Returns true, if input xml is in standard report input xml format;
        /// otherwise false
        /// </returns>
        public Boolean IsStdReportConfigXml(XPathNavigator ReportInputXml)
        {
            XPathNodeIterator XmlIterator = null;

            XmlIterator = ReportInputXml.Select("Form");

            if (XmlIterator.Count == 0)
            {
                return false;
            }

            XmlIterator = ReportInputXml.Select("Form/*");
            if (XmlIterator.Count == 0)
            {
                return false;
            }

            foreach (XPathNavigator XNavSection in ReportInputXml.Select("Form/*"))
            {
                if (String.Compare(XNavSection.LocalName, "Section") != 0)
                {
                    //Currently not checking too much in scheduling section. 
                    //If the section is found, check no further and continue checking other sections
                    if (String.Compare(XNavSection.LocalName, "Scheduling") == 0)
                        continue;
                    else
                        return false;
                }
                XmlIterator = XNavSection.Select("*");
                if (XmlIterator.Count == 0)
                {
                    return false;
                }

                foreach (XPathNavigator XNavColumn in XmlIterator)
                {
                    if (String.Compare(XNavColumn.LocalName, "Column") != 0)
                    {
                        return false;
                    }
                    foreach (XPathNavigator XNavField in XNavColumn.Select("*"))
                    {
                        if (String.Compare(XNavField.LocalName, "Field") != 0)
                        {
                            return false;
                        }
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// This function downloads all thumbnail versions to out-box and from out box to Dowonload folder, iff not present in Dowonload folder
        /// </summary>
        /// <param name="AssetDetailsXml">Asset detail xml</param>
        /// <returns>Error xml on error else null</returns>
        public ErrorXML GetAllRenditionThumbnails(XPathNavigator AssetDetailsXml)
        {
            # region Local Variables
            FileTransfer        FTObj               = null;
            FileFacade          ObjFileFacade       = null;
            XPathNodeIterator   ThumbNodeList       = null;
            XmlDocument         FSCollectionDoc     = null;
            XmlNodeList         FSNodeList          = null;
            XmlNode             FSCollectionNode    = null;
            XmlNode             FSNode              = null;
            XmlNode             FileNode            = null;
            XmlNode             FSAttribNode        = null;
            XML                 InputXml            = null;
            ErrorXML            ErrorXML            = null;
            StringBuilder       FilePath            = null;
            StringBuilder       FormatedFileName    = null;
            String              RenditionID         = null;
            String              FileName            = null;
            String              DownloadPath        = null;
            String              FileID              = null;
            String              SchemaID            = null;
            # endregion

            try
            {
                # region Get list of required thumbnails
                FilePath            = new StringBuilder();
                FSCollectionDoc     = new XmlDocument();
                DownloadPath		= ALSolutionCommon.GetDownloadFilePath(TenantName);
                //ThumbNodeList       = AssetDetailsXml.Select("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTTHUMBNAILRENDITION/ASSETRENDITION/FILESYSTEM");

                ThumbNodeList = AssetDetailsXml.Select("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION/FILESYSTEM");
                FSCollectionNode    = FSCollectionDoc.CreateElement("FILESYSTEMCOLLECTION");
                FSCollectionDoc.AppendChild(FSCollectionNode);

                foreach (XPathNavigator ThumbNode in ThumbNodeList) //.SelectChildren(XPathNodeType.Element))
                {
                    RenditionID     = ThumbNode.SelectSingleNode("FILE/OBJECTID").Value;
                    FileName        = ThumbNode.SelectSingleNode("FILE/NAME").Value;
                    FileID          = ThumbNode.SelectSingleNode("FILE/FILEID").Value;
                    FilePath.AppendFormat("{0}{1}_{2}",DownloadPath, RenditionID, FileName);
                    SchemaID        = ThumbNode.SelectSingleNode("FILE/SCHEMAID").Value;

                    if (!File.Exists(FilePath.ToString()))
                    {
                        FSNode      = FSCollectionDoc.CreateElement("FILESYSTEM");
                        FSCollectionNode.AppendChild(FSNode);

                        FileNode    = FSCollectionDoc.CreateElement("FILE");
                        ((XmlElement)FileNode).SetAttribute("FileRequired", "YES");
                        FSNode.AppendChild(FileNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("FIELDNAME");
                        FSAttribNode.InnerText  = "ACTUALFILE";
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("FILEID");
                        FSAttribNode.InnerText  = FileID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("OBJECTID");
                        FSAttribNode.InnerText  = RenditionID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("OBJECTTYPE");
                        FSAttribNode.InnerText  = "ENTITY";
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("SCHEMAID");
                        FSAttribNode.InnerText  = SchemaID;
                        FileNode.AppendChild(FSAttribNode);

                        FSAttribNode            = FSCollectionDoc.CreateElement("NAME");
                        FSAttribNode.InnerText  = FileName;
                        FileNode.AppendChild(FSAttribNode);
                    }
                    FilePath.Remove(0, FilePath.Length);
                }
                # endregion

                if (FSCollectionNode.SelectNodes("FILESYSTEM").Count > 0)
                {
                    # region Get Thumbnail files to outbox
                    ObjFileFacade   = new FileFacade(SessionID, UserID, TenantName, CurrentLanguage);
                    InputXml        = FSCollectionDoc.OuterXml;
                    ErrorXML        = ObjFileFacade.GetFile(ref InputXml);

                    if (String.IsNullOrEmpty(ErrorXML) == false) // != null && ErrorXML != "")
                    {
                        return "Get thumbnail failed";
                    }
                    # endregion

                    # region Get Thumbnail files to download folder
                    FSCollectionDoc.LoadXml(InputXml);
                    FSNodeList          = FSCollectionDoc.SelectNodes("FILESYSTEMCOLLECTION/FILESYSTEM/FILE");
                    FTObj               = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
                    FTObj.GetFilePath   = DownloadPath;
                    FormatedFileName    = new StringBuilder();

                    foreach (XmlNode FNode in FSNodeList)
                    {
                        FSAttribNode            = FSCollectionDoc.CreateElement("SOURCEPATH");
                        FSAttribNode.InnerText  = FNode.SelectSingleNode("PATH").InnerText;
                        FNode.AppendChild(FSAttribNode);

                        InputXml                = FNode.ParentNode.OuterXml;
                        RenditionID             = FNode.SelectSingleNode("OBJECTID").InnerText;
                        FileName                = FNode.SelectSingleNode("NAME").InnerText;
                        FormatedFileName.AppendFormat("{0}_{1}", RenditionID, FileName);

                        FTObj.DownloadFileName  = FormatedFileName.ToString();
                        ErrorXML                = FTObj.GetFile(ref InputXml);
                        FormatedFileName.Remove(0, FormatedFileName.Length);
                    }
                    # endregion 
                }
            }
            catch (Exception)
            {
                return "Get thumbnail failed";
            }
            return null;
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to create the new instance for AdHocReport Entity.</Purpose>
        /// <returns>On Success it will create a record into the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ErrorXML CreateAdHocReport(ref XML AdHocReportInputXML)
        {
            ErrorXML    errorxml_Output         = null;
            ALReportWS  reportwebservice_Object = null;

            try
            {
                reportwebservice_Object = new ALReportWS();
                reportwebservice_Object.Url = GetWebServiceURL("ALReportWS");
                reportwebservice_Object.ALServiceHeaderValue = getRequestHeader();
                reportwebservice_Object.Timeout = HelperClass.GetWebServiceTimeout();
                errorxml_Output = reportwebservice_Object.CreateAdHocReport(ref AdHocReportInputXML);
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to update the existing instance for AdHocReport Entity.</Purpose>
        /// <returns>On Success it will update a record into the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ErrorXML UpdateAdHocReport(ref XML AdHocReportInputXML)
        {

            ErrorXML    errorxml_Output         = null;
            ALReportWS  reportwebservice_Object = null;

            try
            {
                reportwebservice_Object = new ALReportWS();
                reportwebservice_Object.Url = GetWebServiceURL("ALReportWS");
                reportwebservice_Object.Timeout = HelperClass.GetWebServiceTimeout();
                reportwebservice_Object.ALServiceHeaderValue = getRequestHeader();
                errorxml_Output = reportwebservice_Object.UpdateAdHocReport(ref AdHocReportInputXML);
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to delete an instance of AdHocReport Entity.</Purpose>
        /// <returns>On Success it will delete a record from the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ErrorXML DeleteAdHocReports(String CommaSeparatedReportIDList)
        {

            ErrorXML    errorxml_Output         = null;
            ALReportWS  reportwebservice_Object = null;

            try
            {
                reportwebservice_Object = new ALReportWS();
                reportwebservice_Object.Url = GetWebServiceURL("ALReportWS");
                reportwebservice_Object.Timeout = HelperClass.GetWebServiceTimeout();
                reportwebservice_Object.ALServiceHeaderValue = getRequestHeader();
                errorxml_Output = reportwebservice_Object.DeleteAdHocReports(CommaSeparatedReportIDList);
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>06th December 2010</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for the ad-hoc report. Internally it reads the configured info
        /// for ad-hoc reporting and retrieves the entity data. Then using it it generates the Excel.
        /// THIS API WILL BE MOSTLY USED FROM SOLUTION LEVEL TO VIEW EXCEL.
        /// </Purpose>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        /// <param name="AdHocReportID">Ad-hoc report instance ID to get the configuration for Excel generation.</param>
        public ErrorXML AdHocReportExportToExcelForView(String AdHocReportID, out String FileCollectionXML)
        {
            ALReportWS      reportwebservice_Object = null;
            FileTransfer    commomfacade_FileObject = null;
            XmlDocument     xmldoc_FileCollection   = null;
            XmlNode         xmlnode_FileNode        = null;
            String          str_AdHocReportName     = null;
            String          str_AdHocExcelFilePath  = null;
            String          str_AdHocExcelFileName  = null;
            String          str_DownloadFilePath    = null;
            ErrorXML        errorxml_Output         = null;

            FileCollectionXML = null;

            try
            {
                reportwebservice_Object = new ALReportWS();
                reportwebservice_Object.Url = GetWebServiceURL("ALReportWS");
                reportwebservice_Object.ALServiceHeaderValue = getRequestHeader();
                reportwebservice_Object.Timeout = HelperClass.GetWebServiceTimeout();
                errorxml_Output = reportwebservice_Object.AdHocReportExportToExcelForView(AdHocReportID, out FileCollectionXML);
                if (!(String.IsNullOrEmpty(FileCollectionXML)))
                {
                    //Get the required details from the generated output FileCollection's XML.
                    xmldoc_FileCollection = new XmlDocument();
                    xmldoc_FileCollection.LoadXml(FileCollectionXML);

                    xmlnode_FileNode = xmldoc_FileCollection.SelectSingleNode("FILECOLLECTION/FILE/PATH");
                    if (xmlnode_FileNode != null)
                        str_AdHocExcelFilePath = xmlnode_FileNode.InnerText;

                    xmlnode_FileNode = xmldoc_FileCollection.SelectSingleNode("FILECOLLECTION/FILE/NAME");
                    if (xmlnode_FileNode != null)
                        str_AdHocExcelFileName = xmlnode_FileNode.InnerText;

                    xmlnode_FileNode = xmldoc_FileCollection.SelectSingleNode("FILECOLLECTION/FILE/REPORTNAME");
                    if (xmlnode_FileNode != null)
                    {
                        str_AdHocReportName = xmlnode_FileNode.InnerText;
                        str_AdHocReportName = ALCommon.TrimAndReplaceSpecialCharactersForNTFSFileName(str_AdHocReportName) + ".xlsx";
                        str_AdHocReportName = str_AdHocReportName.Replace("#", "_");
                        str_AdHocReportName = str_AdHocReportName.Replace("%", "_");
                        ALSpecialCharHandling.EncodeSpecialCharacters(ref str_AdHocReportName);
                    }

                    str_DownloadFilePath = ALSolutionCommon.GetDownloadFilePath(TenantName, SessionID);
                    if (!(Directory.Exists(str_DownloadFilePath)))
                        Directory.CreateDirectory(str_DownloadFilePath);

                    //Create the new collection to download file from service to solution.
                    FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_AdHocExcelFilePath
                            + "</PATH><NAME>" + str_AdHocExcelFileName + "</NAME><DESTINATIONPATH>" + str_DownloadFilePath + str_AdHocReportName
                            + "</DESTINATIONPATH></FILE></FILECOLLECTION>";
                    commomfacade_FileObject = new FileTransfer(SessionID, UserID, TenantName, CurrentLanguage);
                    errorxml_Output = commomfacade_FileObject.GetFile(ref FileCollectionXML);
                    if (!(String.IsNullOrEmpty(FileCollectionXML)))
                    {
                        str_AdHocReportName = "<REPORTFILENAME>" + str_AdHocReportName + "</REPORTFILENAME>";
                        FileCollectionXML = ALCommon.InsertBefore(ref FileCollectionXML, "</FILE>", ref str_AdHocReportName);
                    }
                }
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>06th December 2010</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for the ad-hoc report. Internally it reads the configured info
        /// for ad-hoc reporting and retrieves the entity data. Then using it it generates the Excel.
        /// THIS API WILL BE MOSTLY USED FROM REPORT SCHEDULER TO GENERATE AND ATTACH AN EXCEL.
        /// </Purpose>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        /// <param name="AdHocReportID">Ad-hoc report instance ID to get the configuration for Excel generation.</param>
        /// <param name="ReportSchedulerID">Scheduler ID which is used while generating the Excel file path.</param>
        public ErrorXML AdHocReportExportToExcelForReportScheduler(String AdHocReportID, String ReportSchedulerID, out String FileCollectionXML)
        {
            ALReportWS  reportwebservice_Object = null;
            ErrorXML    errorxml_Output         = null;

            FileCollectionXML = null;

            try
            {
                reportwebservice_Object = new ALReportWS();
                reportwebservice_Object.Url = GetWebServiceURL("ALReportWS");
                reportwebservice_Object.Timeout = HelperClass.GetWebServiceTimeout();
                reportwebservice_Object.ALServiceHeaderValue = getRequestHeader();
                errorxml_Output = reportwebservice_Object.AdHocReportExportToExcelForReportScheduler(AdHocReportID, ReportSchedulerID, out FileCollectionXML);
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }

            return (errorxml_Output);
        }

        /// <summary>
        /// <CreatedBy>Ankit Agrawal</CreatedBy>
        /// <CreatedDate>19th January 2011</CreatedDate>
        /// <Purpose>This method is used to delete an instance of AdHocReport Entity.</Purpose>
        /// <returns>On Success it will pass by reference the Ad hoc Report Grouped Data XML; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ErrorXML GetAdHocReportGroupedDataXML(XML ReportConfiguration, XML xml_SearchOutput, ref XML GroupCollectionXML)
        {
            # region Local Variable
            ErrorXML        ErrorXML        = null;
            ALReportWS      ReportWSObj     = null;
            # endregion

            try
            {
                ReportWSObj = new ALReportWS();
                ReportWSObj.Url = GetWebServiceURL("ALReportWS");
                ReportWSObj.Timeout = HelperClass.GetWebServiceTimeout();
                ReportWSObj.ALServiceHeaderValue = getRequestHeader();

                ErrorXML = ReportWSObj.GetAdHocReportGroupedDataXML(ReportConfiguration, xml_SearchOutput, ref GroupCollectionXML);
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
                return ErrorXML;
            }
            return ErrorXML;
        }

        public ErrorXML ExportAssetMetadataReport(String ExportType, XML UserInputXml, XML SearchInputXml, XML SortFieldListXml, XML FormConfigXml, int SortType, out String ResultExportFile)
        {
            # region Local Variable
            ErrorXML ErrorXML = null;
            ALReportWS ReportWSObj = null;
            # endregion

            ResultExportFile = null;
            try
            {
                ReportWSObj = new ALReportWS();
                ReportWSObj.Url = GetWebServiceURL("ALReportWS");
                ReportWSObj.ALServiceHeaderValue = getRequestHeader();
                ReportWSObj.Timeout = 900000;
                ErrorXML = ReportWSObj.ExportAssetMetadataReport(ExportType, UserInputXml, SearchInputXml, SortFieldListXml, FormConfigXml, SortType, out ResultExportFile);
            }
            catch (Exception obj_Exception)
            {
                ErrorXML = obj_Exception.Message;
                return ErrorXML;
            }

            return ErrorXML;

        }
      
        # endregion        
    }
}
