using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Data.OleDb;
using Assetlink.DataAccessLayer;
using System.Globalization;
using System.Xml;
using System.IO;
using System.Xml.XPath;
using System.Data.SqlClient;
using Microsoft.Data;
using Microsoft.Data.SqlXml;
using System.Data.SqlTypes;
using System.Text.RegularExpressions;

using Assetlink;

namespace Assetlink.DataAccessLayer
{
    public partial class SQLQueryProcessor: QueryProcessor
    {
        private const int CONST_MAX_IDS_FOR_IN = 500;
        NSearchCommon m_NSearchCommonObj = null;
        private String PagingCountField = null;
        private String PagingCountFromClause = null;
        //Following int_UniqueIDCount number does not initialize to ZERO because one ParentID attribute from search hierarchy object is used 
        // while defining the XML structure. The ParentID number is started with ZERO. 
        //Few cases (IF conditions) are handled with this parent number and the unique number.
        //If the numbers are same then some operations are avoided.
        Int64 int_UniqueIDCount = 1000;

        #region Building Dynamic SQL(Stored Procedure based on HierarchyObjectList) Methods

        public override OMError PrepareDynamicSQL(ref NSearchCommon NSearchCommonObj, ref StringBuilder DynamicSQL,
                                                    ref List<String> ExecBeforeDynamicQueries, 
                                                    ref List<String> CleanupAfterDynamicQueries)
        {
            OMError                     ErrorObj                    = null;
            OMError                     ReturnErrorObj              = null;
            String                      ReturnFilterClause          = null;
            StringBuilder               TEInsertQuery               = null;
            StringBuilder               TESelectQuery               = null;
            StringBuilder               REandRFSelectQuery          = null;
            StringBuilder               REandRFInsertQuery          = null;
            int                         PrivilegeValue              = -1;
            String                      Query                       = null;
            StringBuilder               TEXMLStructQuery            = null;

            try
            {
                PagingCountField = null;
                PagingCountFromClause = null;

                m_NSearchCommonObj = NSearchCommonObj;
                // Iterate HierarchyObjectList to Process TargetEntity.
                foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    // If parentId is 0 then its TargetEntity.
                    if (HierarchyObj.m_ParentID == 0)
                    {
                        // Prepare Structure Table for Target Entity.
                        TEXMLStructQuery = new StringBuilder(); 
                        ErrorObj = FormTEXMLStructureQuery(HierarchyObj, ref TEXMLStructQuery);

                        if (ErrorObj != null)
                            return ErrorObj;

                        // Give a recursive call which will form a structure query for all HierarchyObjects in HierarchyObjectList
                        ErrorObj = FormXMLStructureQueryforREandRF(HierarchyObj, ref TEXMLStructQuery, HierarchyObj.m_TagID);
                        if (ErrorObj != null)
                            return ErrorObj;

                        // This is for processing SortColumns to support Sorting.
                        ErrorObj = AddSortColumnstoStructureQuery(ref m_NSearchCommonObj.SortPartitionList, ref TEXMLStructQuery);
                        if (ErrorObj != null)
                            return ErrorObj;


                        // Get filter clause for TargetEntity.
                        //ErrorObj = GetSelectandFilterClause(HierarchyObj, ref SelectandFilterClauseString);
                        ErrorObj = GetFilterClause(HierarchyObj, ref ReturnFilterClause);

                        if (ErrorObj != null)
                            return ErrorObj;

                        //Check whether the FILTERCOMBINATION has been specified, if yes, dont' include select query
                        if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                        {
                            ErrorObj = PrepareQuerys(HierarchyObj, ref Query);
                            if (!string.IsNullOrEmpty(ReturnFilterClause))
                            {
                                Query = Query + " " + ReturnFilterClause;

                                PagingCountFromClause = PagingCountFromClause + ReturnFilterClause;
                            }
                        }
                        else
                        {
                            if (HierarchyObj.FilterCollectionList.Count == 1 && HierarchyObj.m_FilterCombination != null)
                            {
                                ErrorObj = PrepareQuerys(HierarchyObj, ref Query);
                                if (!string.IsNullOrEmpty(ReturnFilterClause))
                                {
                                    Query = Query + " " + ReturnFilterClause;

                                    PagingCountFromClause = PagingCountFromClause + ReturnFilterClause;
                                }
                            }
                            else
                            {
                                Query = " " + ReturnFilterClause;
                                
                            }
                        }

                        PrivilegeValue = NSearchCommon.GetPrivilegeValueFromName(HierarchyObj.m_PrivilegeLevel);
                        if (ErrorObj != null)
                            return ErrorObj;

                        // Check Access Control for Target Entity.
                        TEInsertQuery = new StringBuilder();
                        HierarchyObj.m_IsHierarchyObjectProcessed = true;
                        ErrorObj = FormTEInsertQuriesWithAccessControl(HierarchyObj, Query, PrivilegeValue, ref TEInsertQuery);

                        if (ErrorObj != null)
                            return ErrorObj;

                        //In case of threshold/count being specified, find out the number of records,
                        //see whether they are greater than the specified limit, and stop if greater
                        if (HierarchyObj.m_ThresholdResultCount >= 0)
                        {
                            StringBuilder TECountQuery = new StringBuilder(TEInsertQuery.Capacity + 200);
                            TECountQuery.Append(TEInsertQuery.ToString());

                            if (m_NSearchCommonObj.IsAuthorizationRequired)
                            {
                                TECountQuery.Append("select count(*) as TECount from @TEAUTHEVERYONEUSER");
                            }
                            else
                            {
                                TECountQuery.Append("select count(*) as TECount from @TEOBJECT");
                            }

                            object ScalarResult = null;
                            StringBuilder strTECountQuery = new StringBuilder();
                            ParameterizeDynamicSQL(NSearchCommonObj, TECountQuery, ref strTECountQuery);
                            TECountQuery = strTECountQuery;
                            ErrorObj = ExecuteDynamicSQLForScalar(TECountQuery.ToString(),
                                        null, ref ScalarResult);
                            if (ErrorObj != null)
                            {
                                return ErrorObj;
                            }

                            long TECountOut = (long)((int)ScalarResult);

                            if (TECountOut > HierarchyObj.m_ThresholdResultCount)
                            {
                                ErrorObj = new OMError();
                                ErrorObj.ErrorMessage = "Result count has exceeded the threshold limit of " + HierarchyObj.m_ThresholdResultCount;
                                ErrorObj.ErrorType = OMErrorType.WARNING;
                                ErrorObj.ErrorXML = "<SEARCHRESULT_COUNT>" + TECountOut + "</SEARCHRESULT_COUNT>";
                                ErrorObj.ErrorCode = OMErrorCodes.SEARCH_THRESHOLD_RESULTS_EXCEEDED;

                                return ErrorObj;
                            }
                        }

                        // Form TESelect Query.
                        TESelectQuery = new StringBuilder();
                        ErrorObj = FormTESelectQuery(HierarchyObj, ref TESelectQuery);

                        REandRFInsertQuery = new StringBuilder();

                        ErrorObj = FormREandRFInsertQueriesWithAccessControl(HierarchyObj, ref REandRFInsertQuery);

                        if (ErrorObj != null)
                            return ErrorObj;

                        REandRFSelectQuery = new StringBuilder();
                        ErrorObj = FormREandRFSelectQueries(HierarchyObj, ref m_NSearchCommonObj.SortPartitionList, ref REandRFSelectQuery);

                        if (ErrorObj != null)
                            return ErrorObj;

                        // Prepare Sort Clause

                        ErrorObj = FormSortClause(ref REandRFSelectQuery);
                        if (ErrorObj != null)
                            return ErrorObj;
                        
                        DynamicSQL = new StringBuilder();

                        if (!String.IsNullOrEmpty(TEXMLStructQuery.ToString()))
                        {
                            DynamicSQL.AppendFormat("{0} {1} ", TEInsertQuery, REandRFInsertQuery);

                            DynamicSQL.Append(TEXMLStructQuery);

                            DynamicSQL.Append(TESelectQuery);
                            if (REandRFSelectQuery.Length > 0)
                                DynamicSQL.Append(REandRFSelectQuery);

                            DynamicSQL.Append(" FOR XML EXPLICIT ");

                            if (m_NSearchCommonObj.m_Paging)
                            {
                                if (HierarchyObj.m_MaxResultSize == 0)
                                {
                                    PagingCountField = string.Format(" select 1 as tag, null as parent, count(*) as [Paging!1!TotalRecordCount] ");
                                }
                                else
                                {
                                    if (m_NSearchCommonObj.IsAuthorizationRequired)
                                        PagingCountField = string.Format(" select 1 as tag, null as parent, count(distinct(objectid)) as [Paging!1!TotalRecordCount] ", HierarchyObj.m_PrimaryKeyName);
                                    else
                                        PagingCountField = string.Format(" select 1 as tag, null as parent, count(distinct({0})) as [Paging!1!TotalRecordCount] ", HierarchyObj.m_PrimaryKeyName);
                                }

                                DynamicSQL.Append(PagingCountField);

                                DynamicSQL.Append(PagingCountFromClause);

                                DynamicSQL.Append(" FOR XML EXPLICIT ");
                            }

                        }
                        if (ErrorObj != null)
                            return ErrorObj;

                        break;
                    }
                }
                StringBuilder strSQLBatch = new StringBuilder();
                ParameterizeDynamicSQL(NSearchCommonObj, DynamicSQL, ref strSQLBatch);

                DynamicSQL = strSQLBatch;

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        public override OMError PrepareDynamicSQL_Scalar(ref NSearchCommon NSearchCommonObj, ref StringBuilder DynamicSQL)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            String ReturnFilterClause = null;
            StringBuilder TEInsertQuery = null;
            StringBuilder REandRFInsertQuery = null;
            int PrivilegeValue = -1;
            String Query = null;
            StringBuilder SB_DynamicQuery = null;

            try
            {
                PagingCountFromClause = null;

                m_NSearchCommonObj = NSearchCommonObj;
                // Iterate HierarchyObjectList to Process TargetEntity.
                foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    // If parentId is 0 then its TargetEntity.
                    if (HierarchyObj.m_ParentID == 0)
                    {
                        ErrorObj = GetFilterClause(HierarchyObj, ref ReturnFilterClause);

                        if (ErrorObj != null)
                            return ErrorObj;

                        //Check whether the FILTERCOMBINATION has been specified, if yes, dont' include select query
                        if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                        {
                            ErrorObj = PrepareQuerys(HierarchyObj, ref Query);
                            if (!string.IsNullOrEmpty(ReturnFilterClause))
                            {
                                Query = Query + " " + ReturnFilterClause;

                                PagingCountFromClause = PagingCountFromClause + ReturnFilterClause;
                            }
                        }
                        else
                        {
                            if (HierarchyObj.FilterCollectionList.Count == 1 && HierarchyObj.m_FilterCombination != null)
                            {
                                ErrorObj = PrepareQuerys(HierarchyObj, ref Query);
                                if (!string.IsNullOrEmpty(ReturnFilterClause))
                                {
                                    Query = Query + " " + ReturnFilterClause;

                                    PagingCountFromClause = PagingCountFromClause + ReturnFilterClause;
                                }
                            }
                            else
                            {
                                Query = " " + ReturnFilterClause;

                            }
                        }

                        PrivilegeValue = NSearchCommon.GetPrivilegeValueFromName(HierarchyObj.m_PrivilegeLevel);
                        if (ErrorObj != null)
                            return ErrorObj;

                        // Check Access Control for Target Entity.
                        TEInsertQuery = new StringBuilder();
                        HierarchyObj.m_IsHierarchyObjectProcessed = true;
                        ErrorObj = FormTEInsertQuriesWithAccessControl(HierarchyObj, Query, PrivilegeValue, ref TEInsertQuery);

                        if (ErrorObj != null)
                            return ErrorObj;

                        //In case of threshold/count being specified, find out the number of records,
                        //see whether they are greater than the specified limit, and stop if greater
                        if (HierarchyObj.m_ThresholdResultCount >= 0)
                        {
                            StringBuilder TECountQuery = new StringBuilder(TEInsertQuery.Capacity + 200);
                            TECountQuery.Append(TEInsertQuery.ToString());

                            if (m_NSearchCommonObj.IsAuthorizationRequired)
                            {
                                TECountQuery.Append("select count(*) as TECount from @TEAUTHEVERYONEUSER");
                            }
                            else
                            {
                                TECountQuery.Append("select count(*) as TECount from @TEOBJECT");
                            }

                            object ScalarResult = null;
                            StringBuilder strTECountQuery = new StringBuilder();
                            ParameterizeDynamicSQL(NSearchCommonObj, TECountQuery, ref strTECountQuery);
                            TECountQuery = strTECountQuery;
                            ErrorObj = ExecuteDynamicSQLForScalar(TECountQuery.ToString(),
                                        null, ref ScalarResult);
                            if (ErrorObj != null)
                            {
                                return ErrorObj;
                            }

                            long TECountOut = (long)((int)ScalarResult);

                            if (TECountOut > HierarchyObj.m_ThresholdResultCount)
                            {
                                ErrorObj = new OMError();
                                ErrorObj.ErrorMessage = "Result count has exceeded the threshold limit of " + HierarchyObj.m_ThresholdResultCount;
                                ErrorObj.ErrorType = OMErrorType.WARNING;
                                ErrorObj.ErrorXML = "<SEARCHRESULT_COUNT>" + TECountOut + "</SEARCHRESULT_COUNT>";
                                ErrorObj.ErrorCode = OMErrorCodes.SEARCH_THRESHOLD_RESULTS_EXCEEDED;

                                return ErrorObj;
                            }
                        }


                        #region Add select list for TE, RE's and RF's
                        SB_DynamicQuery = new StringBuilder(256);
                        FormSelectQuery_Scalar(ref SB_DynamicQuery);
                        #endregion


                        REandRFInsertQuery = new StringBuilder();
                        ErrorObj = FormREandRFInsertQueriesWithAccessControl(HierarchyObj, ref REandRFInsertQuery);
                        if (ErrorObj != null)
                            return ErrorObj;

                        PrepareJoinQueries(ref SB_DynamicQuery);

                        // Append Sort Clauses
                        ErrorObj = AppendSortClause(ref SB_DynamicQuery);
                        if (ErrorObj != null)
                            return ErrorObj;

                        DynamicSQL = new StringBuilder();
                        DynamicSQL.AppendFormat("{0} {1} ", TEInsertQuery, REandRFInsertQuery);
                        DynamicSQL.Append(SB_DynamicQuery.ToString());

                        ConvertTableVartoTempTableQuery(ref DynamicSQL);

                        StringBuilder strSQLBatch = new StringBuilder(512);
                        ParameterizeDynamicSQL(NSearchCommonObj, DynamicSQL, ref strSQLBatch);

                        DynamicSQL = strSQLBatch;
                        break;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareDynamicSQL_Scalar",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        private String PrepareJoinQueries(ref StringBuilder SB_DynamicQuery)
        {
            foreach (HierarchyObject ParentHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
            {
                if (ParentHierarchyObj.m_ParentID == 0)
                    SB_DynamicQuery.AppendFormat(" FROM  @{2} {2}_ALIAS LEFT OUTER JOIN {0} AS {1} ON {2}_ALIAS.{4} = {1}.{3} ", ParentHierarchyObj.m_TableName, ParentHierarchyObj.m_ObjectAliasName, ParentHierarchyObj.m_EveryOneUserTableName, ParentHierarchyObj.m_PrimaryKeyName, ParentHierarchyObj.m_PrimaryKeyforTableVariable);

                foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (HierarchyObj.m_ParentID == ParentHierarchyObj.m_TagID)
                    {
                        SB_DynamicQuery.AppendFormat(@" LEFT OUTER JOIN @{0} {0}_ALIAS ON {1}.{2} = {0}_ALIAS.FILTEREDOBJID 
                                                    LEFT OUTER JOIN {3} AS {4} ON {4}.{5} = {0}_ALIAS.{6}
                                                     ", HierarchyObj.m_EveryOneUserTableName, ParentHierarchyObj.m_ObjectAliasName,
                                                           ParentHierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName,
                                                           HierarchyObj.m_ObjectAliasName, HierarchyObj.m_PrimaryKeyName,
                                                           HierarchyObj.m_PrimaryKeyforTableVariable);

                    }
                }
            }
            return null;
        }

        private OMError AppendSortClause(ref StringBuilder SB_DynamicQuery)
        {
            String Seprator = String.Empty;
            SortedList SortList = null;
            int SortSequence = 0;
            String Sortclause = null;
            Boolean b_SkipSortingForField = false;

            SortList = new SortedList();
            foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
            {
                if (HierarchyObj.SortSpecificationList.Count > 0)
                {
                    foreach (SortSpecification SortSpecificationObj in HierarchyObj.SortSpecificationList)
                    {
                        //if (!String.IsNullOrEmpty(SortSpecificationObj.m_RelationshipName))
                        //{
                        //    foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                        //    {
                        //        if (TempHierarchyObj.m_RelationshipName == SortSpecificationObj.m_RelationshipName)
                        //        {
                        //            foreach (FieldCollection Field in TempHierarchyObj.FieldCollectionList)
                        //            {
                        //                if (Field.m_FieldName == SortSpecificationObj.m_SortFieldName)
                        //                {
                        //                    SB_DynamicQuery.AppendFormat("{0}{1}${2} {3}", Seprator, HierarchyObj.m_ObjectAliasName, SortSpecificationObj.m_SortFieldName, SortSpecificationObj.m_SortOrder);
                        //                    break;
                        //                }
                        //            }
                        //            break;
                        //        }
                        //    }
                        //}
                        //else
                        //{

                        b_SkipSortingForField = false;
                        foreach (FieldCollection Field in HierarchyObj.FieldCollectionList)
                        {
                            if (Field.m_FieldName == SortSpecificationObj.m_SortFieldName)
                            {
                                if (Field.m_FieldDataType.ToUpper() == "CURRENCY")
                                {
                                    b_SkipSortingForField = true;
                                }
                                break;
                            }
                        }

                        if (b_SkipSortingForField)
                            continue;

                        if (SortSpecificationObj.m_SortSequence != 0)
                        {
                            SortSequence = SortSpecificationObj.m_SortSequence;
                            if (SortList.ContainsKey(SortSequence))
                            {
                                SortSequence = SortList.Count + 1;
                            }
                        }
                        else
                        {
                            SortSequence = SortList.Count + 1;
                        }

                        Sortclause = String.Format("{0}${1} {2}", HierarchyObj.m_ObjectAliasName, SortSpecificationObj.m_SortFieldName, SortSpecificationObj.m_SortOrder);
                        SortList.Add(SortSequence, Sortclause);
                    }
                }
            }

            if (SortList.Count > 0)
            {
                SB_DynamicQuery.Append(" ORDER BY ");
                for (int Index = 0; Index < SortList.Count; Index++)
                {
                    SB_DynamicQuery.AppendFormat("{0}{1}", Seprator, (String)SortList.GetByIndex(Index));
                    if (Seprator == String.Empty)
                        Seprator = ",";
                }
            }
            return null;
        }

        private OMError ConvertTableVartoTempTableQuery(ref StringBuilder DynamicSQL)
        {
            int StartIndex = 0;
            String VariableName = null;
            String ActualVariableName = null;

            #region Convert Table variables to Temp Tables
            StartIndex = DynamicSQL.ToString().IndexOf("@EVERYONEID");
            if (StartIndex != -1)
            {
                DynamicSQL = DynamicSQL.Replace(" @", " #");
                DynamicSQL = DynamicSQL.Replace("#EVERYONEID", "@EVERYONEID");
                #region Replace parameter variables
                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    foreach (FilterCollection filter in TempHierarchyObj.FilterCollectionList)
                    {
                        for (int i = 0; i < filter.m_ParameterizedVariables.Count; i++)
                        {
                            VariableName = " #" + filter.m_ParameterizedVariables[i].Substring(1) + " ";
                            ActualVariableName = " " + filter.m_ParameterizedVariables[i] + " ";
                            DynamicSQL = DynamicSQL.Replace(VariableName, ActualVariableName);
                        }
                    }
                }
                #endregion
                DynamicSQL = DynamicSQL.Replace(" TABLE ", "");
                DynamicSQL = DynamicSQL.Replace(" DECLARE ", " CREATE TABLE ", StartIndex, (DynamicSQL.Length - StartIndex));
            }
            #endregion

            return null;
        }

        /// <summary>
        /// This function executes Dynamic SQL and returns the result of a scalar. 
        /// </summary>
        /// <param name="DynamicSQL">Dynamic SQL which is teradata Database specific</param>
        /// <param name="ConnectionString">Connection String. This is being passed because OM has the connection String information.</param>
        /// <param name="ScalarResult">An object, the result of the scalar</param>
        /// <returns>OMError object if an error is encountered else NULL</returns>
        public override OMError ExecuteDynamicSQLForScalar(String DynamicSQL, String ConnectionString, ref object ScalarResult)
        {
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            XmlDocument TenantXMLDocument = null;
            OMError ReturnErrorObj = null;

            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    AppXMLPath = new StringBuilder();
                    //Reading the connection String for Stored Procedure.
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING").InnerText);
                }

                string DatabaseQuery = DynamicSQL.ToString();

                using (SqlConnection cn = new SqlConnection(ConnectionString))
                {
                    using (SqlCommand cm = new SqlCommand(DatabaseQuery, cn))
                    {
                        cn.Open();
                        ScalarResult = cm.ExecuteScalar();
                    }
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        public override OMError ExecuteDynamicSQL_Scalar(String DynamicSQL, String ConnectionString, ref DataSet DS_Result)
        {
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            XmlDocument TenantXMLDocument = null;
            OMError ReturnErrorObj = null;

            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    //Reading the connection String for Stored Procedure.
                    AppXMLPath = new StringBuilder();
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING").InnerText);
                }

                using (SqlConnection cn = new SqlConnection(ConnectionString))
                {
                    using (SqlCommand cm = new SqlCommand(DynamicSQL.ToString(), cn))
                    {
                        cm.CommandTimeout = 180;
                        DS_Result = new DataSet();
                        SqlDataAdapter SqlDataAdapterObj = new SqlDataAdapter(cm);
                        SqlDataAdapterObj.Fill(DS_Result);
                    }
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL_Scalar",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        public override string GetUnicodeCharacterSpecifier()
        {
            return "N";
        }

        /// <summary>
        /// This function executes Dynamic SQL and returns OutputXML. 
        /// </summary>
        /// <param name="DynamicSQL">Dynamic SQL which is teradata Database specific</param>
        /// <param name="ConnectionString">Connection String. This is being passed because OM has the connection String information.</param>
        /// <param name="NSearchCommonObj">NSearchCommon object for generating input XML</param>
        /// <param name="SPOutputXML">Output XML which is generated using Datareader object</param>
        /// <returns>OMError object if an error is encountered else NULL</returns>
        public override OMError ExecuteDynamicSQL(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            StringBuilder               AppXMLPath              = null;
            String                      InstalledPath           = null;
            XmlDocument                 TenantXMLDocument       = null;
            OMError                     ReturnErrorObj          = null;
            SqlXmlCommand               ObjXMLCommand           = null;
            XmlDocument                 ObjXMLDoc               = null;
            XmlReader                   ObjXMLReader            = null;
            

            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    AppXMLPath = new StringBuilder();
                    //Reading the connection String for Stored Procedure.
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").InnerText);
                }

                ObjXMLCommand = new SqlXmlCommand(ConnectionString);
                ObjXMLCommand.CommandText = "EXEC SP_EXECUTESQL N'" + DynamicSQL.ToString() + "'";
                try
                {
                    ObjXMLReader = (XmlReader)ObjXMLCommand.ExecuteXmlReader();
                    ObjXMLDoc = new XmlDocument();
                    ObjXMLDoc.Load(ObjXMLReader);
                    SPOutputXML = ObjXMLDoc.OuterXml;
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (ObjXMLReader != null && ObjXMLReader.ReadState != ReadState.Closed)
                        ObjXMLReader.Close();
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        public override OMError ExecuteDynamicSQLEx(String DynamicSQL, String ConnectionString, NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            XmlDocument TenantXMLDocument = null;
            OMError ReturnErrorObj = null;
            SqlXmlCommand ObjXMLCommand = null;
            XmlDocument ObjXMLDoc = null;
            XmlReader ObjXMLReader = null;


            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    AppXMLPath = new StringBuilder();
                    //Reading the connection String for Stored Procedure.
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").InnerText);
                }

                ObjXMLCommand = new SqlXmlCommand(ConnectionString);
                ObjXMLCommand.CommandText = DynamicSQL.ToString();
                try
                {
                    ObjXMLReader = (XmlReader)ObjXMLCommand.ExecuteXmlReader();
                    ObjXMLDoc = new XmlDocument();
                    ObjXMLDoc.Load(ObjXMLReader);
                    SPOutputXML = ObjXMLDoc.OuterXml;
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (ObjXMLReader != null && ObjXMLReader.ReadState != ReadState.Closed)
                        ObjXMLReader.Close();
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        /// <summary>
        /// This function executes Dynamic SQL and returns OutputXML. 
        /// </summary>
        /// <param name="DynamicSQL">Dynamic SQL which is teradata Database specific</param>
        /// <param name="ConnectionString">Connection String. This is being passed because OM has the connection String information.</param>
        /// <param name="NSearchCommonObj">NSearchCommon object for generating input XML</param>
        /// <param name="SPOutputXML">Output XML which is generated using Datareader object</param>
        /// <returns>OMError object if an error is encountered else NULL</returns>
        public override OMError ExecutePagingDynamicSQL(String DynamicSQL, String ConnectionString, 
                                                        NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            XmlDocument TenantXMLDocument = null;
            OMError ReturnErrorObj = null;
            XmlReader ObjXMLReader = null;

            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    AppXMLPath = new StringBuilder();
                    //Reading the connection String for Stored Procedure.
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").InnerText);
                }

                try
                {
                    using (SqlConnection cn = new SqlConnection(ConnectionString))
                    {
                        using (SqlCommand cm = new SqlCommand(DynamicSQL.Replace("''", "'"), cn))
                        {
                            cn.Open();

                            cm.CommandTimeout = 180;
                            ObjXMLReader = cm.ExecuteXmlReader();
                            XPathDocument xpathDoc = new XPathDocument(ObjXMLReader, XmlSpace.Preserve);
                            string xsltDocumentPath = ALCommon.GetInstallationDir() + "Binaries\\PlainXML.xslt";
                            System.Xml.Xsl.XslTransform docStyle = new System.Xml.Xsl.XslTransform();

                            docStyle.Load(xsltDocumentPath);

                            StringWriter stringWriter = new StringWriter();

                            docStyle.Transform(xpathDoc, null, stringWriter);
                            SPOutputXML = stringWriter.ToString();

                            stringWriter.Close();
                            stringWriter.Dispose();
                            stringWriter = null;

                            ProcessPagingXML(ref SPOutputXML);
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (ObjXMLReader != null && ObjXMLReader.ReadState != ReadState.Closed)
                        ObjXMLReader.Close();
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        public override OMError ExecutePagingDynamicSQLEx(String DynamicSQL, String ConnectionString,
                                                        NSearchCommon NSearchCommonObj, ref String SPOutputXML)
        {
            StringBuilder AppXMLPath = null;
            String InstalledPath = null;
            XmlDocument TenantXMLDocument = null;
            OMError ReturnErrorObj = null;
            XmlReader ObjXMLReader = null;

            try
            {
                if (String.IsNullOrEmpty(ConnectionString))
                {
                    AppXMLPath = new StringBuilder();
                    //Reading the connection String for Stored Procedure.
                    InstalledPath = ALCommon.GetInstallationDir();
                    AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(this.TenantName));

                    TenantXMLDocument = new XmlDocument();
                    TenantXMLDocument.Load(AppXMLPath.ToString());
                    if (TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP") != null)
                        ConnectionString = Assetlink.ALCommon.Decrypt(TenantXMLDocument.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").InnerText);
                }

                try
                {
                    using (SqlConnection cn = new SqlConnection(ConnectionString))
                    {
                        using (SqlCommand cm = new SqlCommand(DynamicSQL, cn))
                        {
                            cn.Open();

                            ObjXMLReader = cm.ExecuteXmlReader();
                            XPathDocument xpathDoc = new XPathDocument(ObjXMLReader, XmlSpace.Preserve);
                            string xsltDocumentPath = ALCommon.GetInstallationDir() + "Binaries\\PlainXML.xslt";
                            System.Xml.Xsl.XslTransform docStyle = new System.Xml.Xsl.XslTransform();

                            docStyle.Load(xsltDocumentPath);

                            StringWriter stringWriter = new StringWriter();

                            docStyle.Transform(xpathDoc, null, stringWriter);
                            SPOutputXML = stringWriter.ToString();

                            stringWriter.Close();
                            stringWriter.Dispose();
                            stringWriter = null;

                            ProcessPagingXML(ref SPOutputXML);
                        }
                    }
                }
                catch
                {
                    throw;
                }
                finally
                {
                    if (ObjXMLReader != null && ObjXMLReader.ReadState != ReadState.Closed)
                        ObjXMLReader.Close();
                }

                return ReturnErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ExecuteDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        private void ProcessPagingXML(ref string xml)
        {
            string XmlWithoutPaging = xml.Substring(0, xml.LastIndexOf("<Paging"));

            int LengthOfXmlTag = "<?".Length + xml.IndexOf("?>");

            string NormalXmlToReturn = xml.Substring(LengthOfXmlTag, xml.LastIndexOf("<Paging") - LengthOfXmlTag);

            string PagingString = xml.Substring(xml.LastIndexOf("<Paging"));

            int TotalRecordCountLength = "TotalRecordCount=\"".Length + PagingString.LastIndexOf("\"") - PagingString.IndexOf("\"");
            string TotalRecordCount = PagingString.Substring(PagingString.IndexOf("TotalRecordCount="), TotalRecordCountLength);

            int LengthOfTopCollection = "<".Length + NormalXmlToReturn.IndexOf(">") - 1;

            string Collection = NormalXmlToReturn.Substring(0, LengthOfTopCollection);
            if (Collection.EndsWith("/"))
                Collection = NormalXmlToReturn.Substring(0, LengthOfTopCollection - 1);

            NormalXmlToReturn = NormalXmlToReturn.Replace(Collection, Collection + " " + TotalRecordCount);

            xml = NormalXmlToReturn;

        }

        /// <summary>
        /// 
        /// This function executes Dynamic SQL and returns OutputXML. 
        /// </summary>
        /// <param name="DynamicSQL">Dynamic SQL which is teradata Database specific</param>
        /// <param name="ConnectionString">Connection String. This is being passed because OM has the connection String information.</param>
        /// <param name="NSearchCommonObj">NSearchCommon object for generating input XML</param>
        /// <param name="SPOutputXML">Output XML which is generated using Datareader object</param>
        /// <returns>OMError object if an error is encountered else NULL</returns>
        private OMError PrepareDynamicSQLForAccessControlCheck(String TETablename, String PrimaryKey, String WhereClause, String CommaSeparatedTargetObjectIDs, String UserID, String GroupIDs, int RequestedAccessRightValue, ref String DynamicSQLForAccessControlCheck)
        {
            OMError         OMErrorObj                                  = null;
            StringBuilder   SBDynamicSQL                                = null;
            StringBuilder   AccessorIDs                                 = null;
            String          DynamicSQLForPopulatingTargetObjectTable    = null;

            try
            {
                SBDynamicSQL    = new StringBuilder();
                AccessorIDs     = new StringBuilder();

                if(!String.IsNullOrEmpty(GroupIDs))
                   AccessorIDs.AppendFormat("{0},{1}",UserID,GroupIDs);
                else
                   AccessorIDs.AppendFormat("{0}",UserID);
                

                // Get Everyone ID
                SBDynamicSQL.Append(@" DECLARE @EVERYONEID NUMERIC SELECT @EVERYONEID = (SELECT ACCESSORROLEID FROM ALOM_ACCESSORROLE 
                                      WHERE ACCESSORROLENAME = ''EveryOne'')");

                //Populate TargetObject ID's in Temp table
                if ((OMErrorObj = PrepareDynamicSQLForPopulatingTargetObjectTable(TETablename, PrimaryKey, WhereClause, CommaSeparatedTargetObjectIDs, ref DynamicSQLForPopulatingTargetObjectTable)) == null)
                    SBDynamicSQL.Append(DynamicSQLForPopulatingTargetObjectTable);
                else
                    return OMErrorObj;


                //Access Control checking algorithm
                SBDynamicSQL.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER1 TABLE (OBJECTID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                            INSERT INTO @TEAUTHEVERYONEUSER1 (OBJECTID, PRIVILEGEVALUE, ACCESSORID)
                                            SELECT TEO.FILTEREDOBJID, MAX(AUTH.PRIVILEGEVALUE) AS PRIVILEGEVALUE,{0}
                                            FROM @TEOBJECT TEO INNER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = TEO.FILTEREDOBJID
                                            AND AUTH.ACCESSORID IN ({1})
                                            GROUP BY TEO.FILTEREDOBJID HAVING MAX(AUTH.PRIVILEGEVALUE) >= {2} AND MAX(AUTH.PRIVILEGEVALUE) <> 5", UserID,AccessorIDs,RequestedAccessRightValue);

                SBDynamicSQL.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER2 TABLE (OBJECTID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                            INSERT INTO @TEAUTHEVERYONEUSER2 (OBJECTID, PRIVILEGEVALUE, ACCESSORID)
                                            SELECT TEO.FILTEREDOBJID, MAX(AUTH.PRIVILEGEVALUE) AS PRIVILEGEVALUE,AUTH.ACCESSORID
                                            FROM @TEOBJECT TEO
                                            LEFT OUTER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = TEO.FILTEREDOBJID
                                            AND AUTH.ACCESSORID IN ({0})
                                            GROUP BY TEO.FILTEREDOBJID, AUTH.ACCESSORID HAVING MAX(AUTH.PRIVILEGEVALUE) IS NULL", AccessorIDs);

                SBDynamicSQL.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER TABLE (FILTEREDOBJID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                            INSERT INTO @TEAUTHEVERYONEUSER (FILTEREDOBJID, PRIVILEGEVALUE, ACCESSORID)
                                            SELECT TEAG1.OBJECTID,TEAG1.PRIVILEGEVALUE,TEAG1.ACCESSORID FROM @TEAUTHEVERYONEUSER1 TEAG1
                                            UNION
                                            SELECT TEG2.OBJECTID, MAX(AUTH.PRIVILEGEVALUE), @EVERYONEID 
                                            FROM @TEAUTHEVERYONEUSER2 TEG2
                                            INNER JOIN ALOM_AUTHORIZATION AUTH ON TEG2.OBJECTID = AUTH.OBJECTID
                                            AND AUTH.ACCESSORID IN (@EVERYONEID)
                                            GROUP BY TEG2.OBJECTID
                                            HAVING MAX(AUTH.PRIVILEGEVALUE) >= {0} and MAX(AUTH.PRIVILEGEVALUE) <> 5", RequestedAccessRightValue);

                DynamicSQLForAccessControlCheck = SBDynamicSQL.ToString();
            }
            catch (Exception ExceptionObj)
            {
                OMErrorObj = new OMError(OMErrorCodes.EXCEPTION_OCCURED, "Exception in PrepareDynamicSQLForAccessControlCheck",
                    OMErrorObjectType.DATAACCESSLAYER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return OMErrorObj;
            }
            return null;
        }

        private OMError PrepareDynamicSQLForPopulatingTargetObjectTable(String TETablename, String PrimaryKey, String WhereClause,String CommaSeparatedTargetObjectIDs, ref String DynamicSQLForPopulatingTargetObjectTable)
        {
            OMError         OMError                 = null;
            StringBuilder   SBDynamicSQL            = null;
            String[]        IDCollection            = null;
            String[]        Seprator                = {","};
            Int64           MaxIDCountPerInsert     = 1000;
            Int64           Index                   = 0;
            String          ObjectIDAtIndexPos      = null;
            String          CommaSeparatedIDs       = null;
            
            try
            {
                SBDynamicSQL                = new StringBuilder();
                //Populate TargetObject ID's in Temp table

                if (!String.IsNullOrEmpty(CommaSeparatedTargetObjectIDs))
                {
                    #region Break the ID's into batches of MaxIDCountPerInsert
                    IDCollection = CommaSeparatedTargetObjectIDs.Split(Seprator, StringSplitOptions.RemoveEmptyEntries);
                    if (IDCollection.LongLength >= MaxIDCountPerInsert)
                    {
                        if (!String.IsNullOrEmpty(WhereClause))
                            SBDynamicSQL.Append(@"DECLARE @TEMPTEOBJECT TABLE (FILTEREDOBJID NUMERIC)");
                        else
                            SBDynamicSQL.Append(@"DECLARE @TEOBJECT TABLE (FILTEREDOBJID NUMERIC)");
                        do
                        {
                            if (!String.IsNullOrEmpty(WhereClause))
                                SBDynamicSQL.Append(@" INSERT INTO @TEMPTEOBJECT (FILTEREDOBJID)");
                            else
                                SBDynamicSQL.Append(" INSERT INTO @TEOBJECT (FILTEREDOBJID)");

                            Index                   = Index + MaxIDCountPerInsert;
                            if (IDCollection.LongLength >= Index)
                            {
                                ObjectIDAtIndexPos  = IDCollection[Index - 1];
                            }
                            else
                            {
                                Index               = IDCollection.LongLength;
                                ObjectIDAtIndexPos  = IDCollection[Index - 1];
                            }

                            CommaSeparatedIDs               = CommaSeparatedTargetObjectIDs.Substring(0, CommaSeparatedTargetObjectIDs.IndexOf(ObjectIDAtIndexPos) + ObjectIDAtIndexPos.Length);
                            CommaSeparatedTargetObjectIDs   = CommaSeparatedTargetObjectIDs.Remove(0, CommaSeparatedTargetObjectIDs.IndexOf(ObjectIDAtIndexPos) + ObjectIDAtIndexPos.Length);
                            if (CommaSeparatedTargetObjectIDs.Trim().StartsWith(","))
                                CommaSeparatedTargetObjectIDs = CommaSeparatedTargetObjectIDs.Remove(0,1);

                            CommaSeparatedIDs = CommaSeparatedIDs.Replace(",", " UNION ALL SELECT ");
                            CommaSeparatedIDs = CommaSeparatedIDs.Insert(0, " SELECT ");
                            SBDynamicSQL.Append(" " + CommaSeparatedIDs);                       

                        }while (IDCollection.LongLength > Index);

                        if (!String.IsNullOrEmpty(WhereClause))
                            SBDynamicSQL.AppendFormat(@" DECLARE @TEOBJECT TABLE (FILTEREDOBJID NUMERIC)
                                                         INSERT INTO @TEOBJECT (FILTEREDOBJID)
                                                         SELECT TE.FILTEREDOBJID FROM @TEMPTEOBJECT AS TE INTERSECT SELECT {0}.{1} FROM {0} {2}", TETablename, PrimaryKey, WhereClause);
                    }
                    else
                    {
                        SBDynamicSQL.Append(@"DECLARE @TEOBJECT TABLE (FILTEREDOBJID NUMERIC)");
                        SBDynamicSQL.Append(" INSERT INTO @TEOBJECT (FILTEREDOBJID)");
                        CommaSeparatedTargetObjectIDs = CommaSeparatedTargetObjectIDs.Replace(",", " UNION ALL SELECT ");
                        CommaSeparatedTargetObjectIDs = CommaSeparatedTargetObjectIDs.Insert(0, " SELECT ");
                        SBDynamicSQL.Append(" (" + CommaSeparatedTargetObjectIDs + ")");

                        if (!String.IsNullOrEmpty(WhereClause))
                            SBDynamicSQL.AppendFormat(@" INTERSECT SELECT {0}.{1} FROM {0} {2}", TETablename, PrimaryKey, WhereClause);
     
                    }
                     #endregion
                }
                else if (!String.IsNullOrEmpty(WhereClause))
                {
                    SBDynamicSQL.Append(@"DECLARE @TEOBJECT TABLE (FILTEREDOBJID NUMERIC)");
                    SBDynamicSQL.Append(" INSERT INTO @TEOBJECT (FILTEREDOBJID)");
                    SBDynamicSQL.AppendFormat(@" SELECT {0}.{1} FROM {0} {2}", TETablename, PrimaryKey, WhereClause);
                }
                DynamicSQLForPopulatingTargetObjectTable = SBDynamicSQL.ToString();
            }
            catch (Exception ExceptionObj)
            {
                OMError = new OMError(OMErrorCodes.EXCEPTION_OCCURED, "Exception in PrepareDynamicSQLForPopulatingTargetObjectTable",
                    OMErrorObjectType.DATAACCESSLAYER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return OMError;
            }
            return null;
        }
        
        private OMError FormSortClause(ref StringBuilder REandRFSelectQuery)
        {
            OMError                     ReturnErrorObj                  = null;
            OMError                     ErrorObj                        = null;
            Boolean                     IsOrderbyClauseAdded            = false;
            ArrayList                   ProcessedIDList                 = new ArrayList();
            ArrayList                   IDListtoProcess                 = new ArrayList();
            String                      StructureColforTECollection     = null;
            Dictionary<String, String>  ColumnstoSort                   = new Dictionary<String, String>();
            long                        UniqueTagID                     = 0;
            long                        SortPartitionID                 = 0;

            try
            {

                // Processing for TE
                foreach (HierarchyObject TempHierarchObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchObj.m_ParentID == 0)
                    {
                        UniqueTagID = 0;
                        UniqueTagID = TempHierarchObj.m_TagID;

                        // By Default Sort on TE Collection Node
                        StructureColforTECollection = "[" + TempHierarchObj.m_ObjectTypeName + "!2" + "]";
                        ColumnstoSort.Add(StructureColforTECollection, " DESC");
                        //Add fields to sort column list as per order specified in sort list and not as per select list.
                        ErrorObj = AddSortColumnsToList(TempHierarchObj, true, ref ColumnstoSort);
                        if (ErrorObj != null)
                            return ErrorObj;
                    }
                }

                // Processing for RE and RF
                foreach (SortPartitionColumnTagIDPair TempColTagIDObj in m_NSearchCommonObj.m_SPColTagIDList)
                {
                    // Add a Sort Partition Column 
                    if (!ColumnstoSort.ContainsKey(TempColTagIDObj.SortPartitionStructColumnName))
                        ColumnstoSort.Add(TempColTagIDObj.SortPartitionStructColumnName, "ASC");

                    IDListtoProcess.Clear();

                    // Get all HierarchyObjects whose ParentID is TE's TagID
                    foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                    {
                        if (TempHierarchyObj.m_ParentID == TempColTagIDObj.TagID)
                        {
                            // This adds up all the TagIDs of whose siblings are to be checked.

                            IDListtoProcess.Add(TempHierarchyObj.m_TagID);

                            //Add fields to sort column list as per order specified in sort list and not as per select list.
                            ErrorObj = AddSortColumnsToList(TempHierarchyObj, true, ref ColumnstoSort);
                            if (ErrorObj != null)
                                return ErrorObj;

                            // Get SortPartitionObject for this TagID
                            // This is done for Sorting on HierarchyObject which doesnot have any Sort Partition Column.

                            foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                            {
                                if (TempSortPartitionObj.m_NodeID == TempColTagIDObj.TagID)
                                {
                                    // Get All Siblings 
                                    foreach (Sibling TempSibling in TempSortPartitionObj.SiblingList)
                                    {
                                        SortPartitionID = 0;
                                        SortPartitionID = TempSibling.m_SortPartitionID;
                                        // Now Get Sort Partition Object this for this SortPartitionID
                                        foreach (SortPartition SiblingSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                                        {
                                            if (SiblingSortPartitionObj.m_SortPartitionID == SortPartitionID)
                                            {
                                                // Check for Sibling Count
                                                if (SiblingSortPartitionObj.SiblingList.Count == 0)
                                                {
                                                    // Get TagID and ParentID and Get Hierarchy Object for this TagID.
                                                    foreach (HierarchyObject SiblingHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                                                    {
                                                        if (SiblingSortPartitionObj.m_NodeID == SiblingHierarchyObj.m_TagID)
                                                        {
                                                            //Add fields to sort column list as per order specified in sort list and not as per select list.
                                                            ErrorObj = AddSortColumnsToList(SiblingHierarchyObj, true, ref ColumnstoSort);
                                                            if (ErrorObj != null)
                                                                return ErrorObj;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }

                        }
                    }

                    // This Processing is done for those hierarchy objects who dont have any siblings 
                    // This will add primary keys of those HierarchyObjects whose sibling count is = 0
                    /*foreach (long TempTagID in IDListtoProcess)
                    {
                        // Get SortPartitionObject for this TagID
                       foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                        {
                            if (TempSortPartitionObj.m_NodeID == TempTagID)
                            {
                                // Get All Siblings 
                                foreach (Sibling TempSibling in TempSortPartitionObj.SiblingList)
                                {
                                    SortPartitionID = 0;
                                    SortPartitionID = TempSibling.m_SortPartitionID;
                                    // Now Get Sort Partition Object this for this SortPartitionID
                                    foreach (SortPartition SiblingSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                                    {
                                        if (SiblingSortPartitionObj.m_SortPartitionID == SortPartitionID)
                                        {
                                            // Check for Sibling Count
                                            if (SiblingSortPartitionObj.SiblingList.Count == 0)
                                            {
                                                // Get TagID and ParentID and Get Hierarchy Object for this TagID.
                                                foreach (HierarchyObject SiblingHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                                                {
                                                    if (SiblingSortPartitionObj.m_NodeID == SiblingHierarchyObj.m_TagID)
                                                    {
                                                        // Add Sort Columns for that HierarchyObject.
                                                        foreach (FieldCollection TempField in SiblingHierarchyObj.FieldCollectionList)
                                                        {
                                                            if (TempField.m_IsSortRequested)
                                                            {
                                                                // Check whether the Field is Primary Key or Not.
                                                                // This is because Primary Key should always come at the end
                                                                if (TempField.m_IsPrimaryKey)
                                                                {
                                                                    PrimaryKeyforObject = TempField.m_StructureColumnName;
                                                                    PrimaryKeySortType = TempField.m_SortType;
                                                                }
                                                                else
                                                                {
                                                                    if (!ColumnstoSort.ContainsKey(TempField.m_StructureColumnName))
                                                                        ColumnstoSort.Add(TempField.m_StructureColumnName, TempField.m_SortType);
                                                                }
                                                            }
                                                        }

                                                        // Now Add a Primary Key for that Hierarchy Object as Primary is to be sorted at last
                                                        if ((PrimaryKeyforObject != null && PrimaryKeyforObject != "") && (PrimaryKeySortType != null && PrimaryKeySortType != ""))
                                                        {
                                                            if (!ColumnstoSort.ContainsKey(PrimaryKeyforObject))
                                                                ColumnstoSort.Add(PrimaryKeyforObject, PrimaryKeySortType);
                                                            PrimaryKeyforObject = null;
                                                            PrimaryKeySortType = null;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }*/
                }


                IsOrderbyClauseAdded = false;

                foreach (KeyValuePair<String, String> SortColumn in ColumnstoSort)
                {
                    if (!IsOrderbyClauseAdded)
                    {
                        REandRFSelectQuery.AppendFormat(" ORDER BY {0} {1} ", SortColumn.Key, SortColumn.Value);
                        //REandRFSelectQuery.AppendFormat(" {0} {1} ", Field.m_StructureColumnName, Field.m_SortType);
                        IsOrderbyClauseAdded = true;
                    }
                    else
                        REandRFSelectQuery.AppendFormat(" , {0} {1} ", SortColumn.Key, SortColumn.Value);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormSortClause",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <CreatedDate>11 May 09</CreatedDate>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <purpose>
        /// Add colums to sort list dicationary according to the order specified in input xml.
        /// </purpose>
        /// </summary>
        /// <param name="obj_HierarchyObject">Specifies Hierarchy Object for which sort list is to be prepared.</param>
        /// <param name="b_AddPrimaryKeyAtTheEnd">Specifies whether primary key is to be added at the end. NSearch implementation always require primary key to be added at the end.</param>
        /// <param name="ColumnstoSort">Dictionary structure to store sort column with sort order.</param>
        /// <returns>OMError if any error occurs, otherwise returns null.</returns>
        private OMError AddSortColumnsToList(HierarchyObject obj_HierarchyObject, bool b_AddPrimaryKeyAtTheEnd, ref Dictionary<String, String> ColumnstoSort)
        {
            #region Variables

            OMError                     obj_OMError                     = null; //Used to prepare error
            String                      str_PrimaryKeyforObject         = null; //Used to store primary key for an object
            String                      str_PrimaryKeySortType          = null; //Used to store sort type for primary key
            Dictionary<String, String>  dict_SortFieldName              = null; //Used to store field names from sort specification list.
            bool                        b_SkipSortingForField           = false;//Used to identify whether field data type is CURRENCY

            #endregion Variables

            #region Add columns to sort list (as per the order specified in input xml)

            try
            {
                /* Iterate over field collection list and identify fields for which sorting is requested (other than primary key).
                 * Because primary key will always be added at the end. This function maintains order of the fields given in sort list 
                 * regerdless of the sequence of fields in select list.
                 */

                foreach (FieldCollection Field in obj_HierarchyObject.FieldCollectionList)
                {
                    b_SkipSortingForField = false;
                    if (Field.m_IsSortRequested)
                    {
                        // Check whether the Field is Primary Key or Not.
                        // This is because Primary Key should always come at the end
                        if (Field.m_IsPrimaryKey)
                        {
                            str_PrimaryKeyforObject = Field.m_StructureColumnName;
                            str_PrimaryKeySortType  = Field.m_SortType;
                        }
                        else
                        {
                            //CURRENCY field will not be considered for sorting, instead additional field containing amount will be used for sorting  
                            if (!String.IsNullOrEmpty(Field.m_FieldDataType) && Field.m_FieldDataType.ToUpper().CompareTo("CURRENCY") == 0)
                                b_SkipSortingForField = true;

                            //Skip CURRENCY field
                            if (!b_SkipSortingForField)
                            {
                                if (dict_SortFieldName == null)
                                    dict_SortFieldName = new Dictionary<String, String>();

                                dict_SortFieldName.Add(Field.m_FieldName, Field.m_StructureColumnName);
                            }
                        }
                    }
                }

                /* Bug fix: 6401
                 * Sorting can be specified on multiple columns. In that case the order should be considered as specified in sort list.
                 * Iterating over field collection will add columns to ColumntoSort list, but the order doesn't get persisted.
                 * So the sorting happens according to the sequence of field in field collection.
                 * The behaviour is changed to iterate over sort specification list and columns are added maintaining sort sequence.
                 */

                if (dict_SortFieldName != null && dict_SortFieldName.Count > 0)
                {
                    foreach (SortSpecification obj_SortSpecification in obj_HierarchyObject.SortSpecificationList)
                    {
                        if (dict_SortFieldName.ContainsKey(obj_SortSpecification.m_SortFieldName))
                        {
                            if (ColumnstoSort == null)
                            {
                                ColumnstoSort = new Dictionary<String, String>();
                                ColumnstoSort.Add(dict_SortFieldName[obj_SortSpecification.m_SortFieldName], obj_SortSpecification.m_SortOrder);
                            }
                            else
                            {
                                if (!ColumnstoSort.ContainsKey(dict_SortFieldName[obj_SortSpecification.m_SortFieldName]))
                                    ColumnstoSort.Add(dict_SortFieldName[obj_SortSpecification.m_SortFieldName], obj_SortSpecification.m_SortOrder);
                            }
                        }
                    }
                }

                // Add a Primary Key for TE
                if (b_AddPrimaryKeyAtTheEnd)
                {
                    if (!String.IsNullOrEmpty(str_PrimaryKeyforObject) && !String.IsNullOrEmpty(str_PrimaryKeySortType) && (obj_HierarchyObject.m_RelationshipName != "FILESYSTEM"))
                    {
                        if (!ColumnstoSort.ContainsKey(str_PrimaryKeyforObject))
                            ColumnstoSort.Add(str_PrimaryKeyforObject, str_PrimaryKeySortType);
                    }
                }

            }
            catch (Exception obj_Exception)
            {
                obj_OMError = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in AddSortColumnsToList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, obj_Exception, "");

            }

            return obj_OMError;

            #endregion Add columns to sort list (as per the order specified in input xml)
        }

        private OMError FormTEXMLStructureQuery(HierarchyObject HierarchyObj, ref StringBuilder TEXMLStructQuery)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;

            try
            {
                TEXMLStructQuery.Append(" SELECT 1 AS TAG, NULL AS PARENT");
                m_NSearchCommonObj.m_ParentNodeNumber++;
                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}]", HierarchyObj.m_CollectionName, m_NSearchCommonObj.m_ParentNodeNumber);

                m_NSearchCommonObj.m_ParentNodeNumber++;
                TEXMLStructQuery.AppendFormat(", ''zzz'' AS [{0}!{1}]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber);

                HierarchyObj.m_ActualTagID = m_NSearchCommonObj.m_ParentNodeNumber;

                // Add Fields which are selected for TargetEntity.
                if (HierarchyObj.FieldCollectionList.Count > 0)
                {
                    foreach (FieldCollection Field in HierarchyObj.FieldCollectionList)
                    {
                        if (!Field.m_IsHidden)
                        {
                            // check for field datatype
                            switch (Field.m_FieldDataType)
                            {
                                case "DATETIME":
                                case "CURRENCY":
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "XML" + "]";
                                    }
                                    break;

                                case "INTEGER":
                                case "BIGINT":
                                case "FLOAT":
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENTXSINIL]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENTXSINIL" + "]";
                                    }
                                    break;

                                default:
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                    }
                                    break;
                            }
                        }
                        else
                        {
                            switch (Field.m_FieldDataType)
                            {
                                case "DATETIME":
                                case "CURRENCY":
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "XML" + "!" + "HIDE" + "]";
                                    }
                                    break;

                                case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!ELEMENT!HIDE]";
                                    }
                                    break;

                                case "INTEGER":
                                case "BIGINT":
                                case "FLOAT":
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENTXSINIL!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENTXSINIL" + "!" + "HIDE" + "]";
                                    }
                                    break;

                                default:
                                    {
                                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                        Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "!" + "HIDE" + "]";
                                    }
                                    break;
                            }
                        }
                    }
                }
                // Check for Aggregate Function List

                if (HierarchyObj.AggregateAttrList.Count > 0)
                {
                    foreach (AggregateAttributes AggrFunction in HierarchyObj.AggregateAttrList)
                    {
                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, AggrFunction.m_Alias);
                        //Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + AggrFunction.m_Alias + "!" + "ELEMENT" + "]";
                    }
                }
                //m_NSearchCommonObj.m_ParentNodeNumber--;
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormTEXMLStructureQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError FormXMLStructureQueryforREandRF(HierarchyObject HierarchyObj, ref StringBuilder TEXMLStructQuery, long TagID)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            Boolean OnlyFileRequested = false;
            Boolean OnlyBinderRequested = false;

            try
            {
                if (HierarchyObj.m_ParentID != 0)
                {
                    if (!HierarchyObj.m_RelationshipName.Contains("AL$FILESYSTEM_AL$"))
                    {
                        m_NSearchCommonObj.m_ParentNodeNumber++;
                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", HierarchyObj.m_RelationshipName, m_NSearchCommonObj.m_ParentNodeNumber);

                        m_NSearchCommonObj.m_ParentNodeNumber++;
                        HierarchyObj.m_ActualTagID = m_NSearchCommonObj.m_ParentNodeNumber;

                        if (HierarchyObj.FieldCollectionList.Count > 0)
                        {
                            foreach (FieldCollection Field in HierarchyObj.FieldCollectionList)
                            {
                                if (!Field.m_IsHidden)
                                {
                                    // check for field datatype

                                    switch (Field.m_FieldDataType)
                                    {
                                        case "DATETIME":
                                        case "CURRENCY":
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "XML" + "]";
                                            }
                                            break;

                                        case "INTEGER":
                                        case "BIGINT":
                                        case "FLOAT":
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENTXSINIL]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENTXSINIL" + "]";
                                            }
                                            break;

                                        default:
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                            }
                                            break;

                                    }
                                }
                                else
                                {

                                    switch (Field.m_FieldDataType)
                                    {
                                        case "DATETIME":
                                        case "CURRENCY":
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "XML" + "!HIDE" + "]";
                                            }
                                            break;

                                        case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!ELEMENT!HIDE]";
                                            }
                                            break;

                                        case "INTEGER":
                                        case "BIGINT":
                                        case "FLOAT":
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENTXSINIL!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENTXSINIL" + "!HIDE" + "]";
                                            }
                                            break;

                                        default:
                                            {
                                                TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT!HIDE]", HierarchyObj.m_ObjectTypeName, m_NSearchCommonObj.m_ParentNodeNumber, Field.m_Alias);
                                                Field.m_StructureColumnName = "[" + HierarchyObj.m_ObjectTypeName + "!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "!HIDE" + "]";
                                            }
                                            break;

                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        // This is a FileSystemNode.
                        m_NSearchCommonObj.m_ParentNodeNumber++;
                        HierarchyObj.m_RelationshipName = "FILESYSTEM";

                        TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", HierarchyObj.m_RelationshipName, m_NSearchCommonObj.m_ParentNodeNumber);
                        m_NSearchCommonObj.m_ParentNodeNumber++;

                        HierarchyObj.m_ActualTagID = m_NSearchCommonObj.m_ParentNodeNumber;

                        // Check what is requested in Select list.
                        foreach (FieldCollection Field in HierarchyObj.CustomFieldCollectionList)
                        {
                            if (Field.m_FieldDataType.ToUpper() == "BINDER")
                                OnlyBinderRequested = true;
                            if (Field.m_FieldDataType.ToUpper() == "FILE")
                                OnlyFileRequested = true;
                        }

                        //Requires for sorting files within the binder accorading to sequenceorder 
                        SortSpecification   objSortSpecification    =   null;
                        FieldCollection     objFieldCollection      =   null;
                        String              Structure_FieldName     =   null;

                        if ((OnlyBinderRequested) && (OnlyFileRequested))
                        {
                            // both file and binder is requested.
                            // process binder.
                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;

                            foreach (DictionaryEntry ActualField in m_NSearchCommonObj.m_BinderFields)
                            {
                                if ((ActualField.Value.ToString() == "DATETIME") || (ActualField.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber, ActualField.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber, ActualField.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[BINDER!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }

                            m_NSearchCommonObj.m_ParentNodeNumber++;
                            // Add AssoFileBinder Fields

                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;

                            foreach (DictionaryEntry AssoFileBinderFields in m_NSearchCommonObj.m_AssoFileBinderFields)
                            {
                                if ((AssoFileBinderFields.Value.ToString() == "DATETIME") || (AssoFileBinderFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, AssoFileBinderFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, AssoFileBinderFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }

                                if ((AssoFileBinderFields.Key.ToString().ToUpper()).Equals("SEQUENCEORDER"))
                                {
                                    objSortSpecification    = new SortSpecification();
                                    objFieldCollection      = new FieldCollection();
                                    if ((AssoFileBinderFields.Value.ToString() == "DATETIME") || (AssoFileBinderFields.Value.ToString() == "CURRENCY"))
                                        Structure_FieldName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + AssoFileBinderFields.Key.ToString() + "!XML]";
                                    else
                                        Structure_FieldName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + AssoFileBinderFields.Key.ToString() + "!ELEMENT]";

                                    objSortSpecification.m_SortFieldName = AssoFileBinderFields.Key.ToString();
                                    objSortSpecification.m_SortOrder = "ASC";

                                    //Add SEQUENCEORDER to the SortSpecificationList
                                    HierarchyObj.SortSpecificationList.Add(objSortSpecification);

                                    objFieldCollection.m_IsSortRequested = true;
                                    objFieldCollection.m_FieldName = AssoFileBinderFields.Key.ToString();
                                    objFieldCollection.m_StructureColumnName = Structure_FieldName;

                                    //Add SEQUENCEORDER to the FieldCollectionList
                                    HierarchyObj.FieldCollectionList.Add(objFieldCollection);
                                }

                            }

                            // Add File Fields for BINDER.
                            foreach (DictionaryEntry FileFields in m_NSearchCommonObj.m_FileFields)
                            {
                                if ((FileFields.Value.ToString() == "DATETIME") || (FileFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }

                            // Add Explicit File Fields.
                            m_NSearchCommonObj.m_ParentNodeNumber++;

                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;
                            foreach (DictionaryEntry FileFields in m_NSearchCommonObj.m_FileFields)
                            {
                                if ((FileFields.Value.ToString() == "DATETIME") || (FileFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }

                        }
                        if ((OnlyFileRequested) && (!(OnlyBinderRequested)))
                        {
                            // Add Explicit File Fields.
                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;
                            foreach (DictionaryEntry FileFields in m_NSearchCommonObj.m_FileFields)
                            {
                                if ((FileFields.Value.ToString() == "DATETIME") || (FileFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }

                        }
                        if ((OnlyBinderRequested) && (!(OnlyFileRequested)))
                        {
                            // Only Binder is requested.
                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;

                            foreach (DictionaryEntry ActualField in m_NSearchCommonObj.m_BinderFields)
                            {
                                if ((ActualField.Value.ToString() == "DATETIME") || (ActualField.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber, ActualField.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "BINDER", m_NSearchCommonObj.m_ParentNodeNumber, ActualField.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[BINDER!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }

                            // Add AssoFileBinder Fields
                            m_NSearchCommonObj.m_ParentNodeNumber++;

                            TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber);
                            //m_NSearchCommonObj.m_ParentNodeNumber++;

                            foreach (DictionaryEntry AssoFileBinderFields in m_NSearchCommonObj.m_AssoFileBinderFields)
                            {
                                if ((AssoFileBinderFields.Value.ToString() == "DATETIME") || (AssoFileBinderFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, AssoFileBinderFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, AssoFileBinderFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                                
                                //This code adds SEQUENCEORDER column to sort
                                if ((AssoFileBinderFields.Key.ToString().ToUpper()).Equals("SEQUENCEORDER"))
                                {
                                    objSortSpecification = new SortSpecification();
                                    objFieldCollection = new FieldCollection();
                                    if ((AssoFileBinderFields.Value.ToString() == "DATETIME") || (AssoFileBinderFields.Value.ToString() == "CURRENCY"))
                                        Structure_FieldName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + AssoFileBinderFields.Key.ToString() + "!XML]";
                                    else
                                        Structure_FieldName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + AssoFileBinderFields.Key.ToString() + "!ELEMENT]";

                                    objSortSpecification.m_SortFieldName = AssoFileBinderFields.Key.ToString();
                                    objSortSpecification.m_SortOrder = "ASC";

                                    //Add SEQUENCEORDER to the SortSpecificationList
                                    HierarchyObj.SortSpecificationList.Add(objSortSpecification);

                                    objFieldCollection.m_IsSortRequested = true;
                                    objFieldCollection.m_FieldName = AssoFileBinderFields.Key.ToString();
                                    objFieldCollection.m_StructureColumnName = Structure_FieldName;

                                    //Add SEQUENCEORDER to the FieldCollectionList
                                    HierarchyObj.FieldCollectionList.Add(objFieldCollection);
                                }

                            }

                            // Add File Fields for BINDER.
                            foreach (DictionaryEntry FileFields in m_NSearchCommonObj.m_FileFields)
                            {
                                if ((FileFields.Value.ToString() == "DATETIME") || (FileFields.Value.ToString() == "CURRENCY"))
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!XML]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                }
                                else
                                {
                                    TEXMLStructQuery.AppendFormat(", NULL AS [{0}!{1}!{2}!ELEMENT]", "FILE", m_NSearchCommonObj.m_ParentNodeNumber, FileFields.Key.ToString());
                                    //ActualField.m_StructureColumnName = "[FILE!" + m_NSearchCommonObj.m_ParentNodeNumber + "!" + Field.m_Alias + "!" + "ELEMENT" + "]";
                                }
                            }
                        }
                    }
                }


                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObj.m_ParentID == TagID)
                    {
                        ErrorObj = FormXMLStructureQueryforREandRF(TempHierarchyObj, ref TEXMLStructQuery, TempHierarchyObj.m_TagID);
                        if (ErrorObj != null)
                            return ErrorObj;
                    }

                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormTEXMLStructureQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError FormTESelectQuery(HierarchyObject HierarchyObj, ref StringBuilder TESelectQuery)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            int TESelectCounter = 0;
            int StartCounter = 0;
            Boolean GroupByFlag = false;
            StringBuilder GroupByList = null;

            try
            {
                TESelectQuery.Append(" UNION");
                TESelectQuery.Append(" SELECT 2 AS TAG, 1 AS PARENT");
                TESelectQuery.Append(", NULL, NULL");
                TESelectCounter = 4;
                if (HierarchyObj.AggregateAttrList.Count > 0)
                {
                    GroupByFlag = true;
                    GroupByList = new StringBuilder();
                }
                foreach (FieldCollection Field in HierarchyObj.FieldCollectionList)
                {
                    TESelectCounter++;
                    //Switch - case has been used instead of if..else
                    switch (Field.m_FieldDataType)
                    {
                        case "DATETIME":
                            TESelectQuery.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, TE.{0})) + ''</YEAR>''", Field.m_FieldName);
                            TESelectQuery.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), TE.{0},101),1,2) + ''</MONTH>''", Field.m_FieldName);
                            TESelectQuery.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), TE.{0},101),4,2) + ''</DAY>''", Field.m_FieldName);
                            TESelectQuery.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), TE.{0},121),12,2) + ''</HOUR>''", Field.m_FieldName);
                            TESelectQuery.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), TE.{0},121),15,2) + ''</MINUTE>''", Field.m_FieldName);
                            TESelectQuery.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), TE.{0},121),18,2) + ''</SECOND>'','''')", Field.m_FieldName);
                            if (GroupByFlag)
                                GroupByList.AppendFormat(", TE.{0}", Field.m_FieldName);
                            break;
                        case "CURRENCY":
                            TESelectQuery.AppendFormat(",ISNULL(''<CURRENCY>'' + TE.{0} + ''</CURRENCY>'','''')", Field.m_FieldName + "_C");
                            TESelectQuery.AppendFormat(" + ISNULL(''<AMOUNT>'' + RTRIM(CAST(TE.{0} AS DECIMAL(30,4))) + ''</AMOUNT>'','''')", Field.m_FieldName + "_A");
                            if (GroupByFlag)
                            {
                                GroupByList.AppendFormat(", TE.{0}", Field.m_FieldName + "_C");
                                GroupByList.AppendFormat(", TE.{0}", Field.m_FieldName + "_A");
                            }
                            break;

                        case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                            TESelectQuery.AppendFormat(", CAST(TE.{0} AS DECIMAL(30,4))", Field.m_FieldName);
                            break;
                        case "FLOAT":
                            TESelectQuery.AppendFormat(", CAST(TE.{0} AS DECIMAL(30,4))", Field.m_FieldName);
                            break;
                        case "BIGINT":
                        case "INTEGER":
                            TESelectQuery.AppendFormat(", TE.{0}", Field.m_FieldName);
                            break;
                        default:
                            TESelectQuery.AppendFormat(", ISNULL(TE.{0},'''')", Field.m_FieldName);
                            if (GroupByFlag)
                                GroupByList.AppendFormat(", TE.{0}", Field.m_FieldName);
                            break;
                    }
                }
                foreach (AggregateAttributes AggrFunction in HierarchyObj.AggregateAttrList)
                {
                    TESelectCounter++;
                    if (AggrFunction.m_RelationshipName == null || AggrFunction.m_RelationshipName == "")
                    {
                        if (AggrFunction.m_IsDistinct)
                            TESelectQuery.AppendFormat(", {0}(DISTINCT(ISNULL(TE.{1},'''')))", AggrFunction.m_FunctionName, AggrFunction.m_FieldName);
                        else
                            TESelectQuery.AppendFormat(", {0}(ISNULL(TE.{1},''''))", AggrFunction.m_FunctionName, AggrFunction.m_FieldName);
                    }
                    // do processing for related entity/facets 
                }
                //for RE Collection if it exists
                TESelectCounter++;
                for (StartCounter = TESelectCounter; StartCounter <= m_NSearchCommonObj.m_TotalNumberOfFields; StartCounter++)
                {
                    TESelectQuery.AppendFormat(" ,NULL");
                }

                if (m_NSearchCommonObj.IsAuthorizationRequired)
                    TESelectQuery.Append(" FROM @TEAUTHEVERYONEUSER AS TEEU");
                else
                    TESelectQuery.Append(" FROM @TEOBJECT AS TEEU");

                TESelectQuery.AppendFormat(" INNER JOIN {0} AS TE ON TE.{1} = TEEU.FILTEREDOBJID", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);

                if (HierarchyObj.AggregateAttrList.Count > 0)
                {
                    TESelectQuery.AppendFormat(" GROUP BY ");
                    TESelectQuery.AppendFormat(GroupByList.ToString().Substring(2));
                }

                // This is for appending nulls at the end

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormTEXMLStructureQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        private OMError FormSelectQuery_Scalar(ref StringBuilder SB_DynamicQuery)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            String CommaSeprator = null;
            String FieldAlias = null;

            try
            {
                SB_DynamicQuery.Append("SELECT ");
                foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    foreach (FieldCollection Field in HierarchyObj.FieldCollectionList)
                    {
                        if (String.IsNullOrEmpty(Field.m_Alias))
                            FieldAlias = Field.m_FieldName;
                        else
                            FieldAlias = Field.m_Alias;

                        switch (Field.m_FieldDataType)
                        {
                            case "CURRENCY":
                                SB_DynamicQuery.AppendFormat("{0}{1}.{2}_C AS {1}${3}_C", CommaSeprator, HierarchyObj.m_ObjectAliasName, Field.m_FieldName, FieldAlias);
                                SB_DynamicQuery.AppendFormat("{0}{1}.{2}_A AS {1}${3}_A", CommaSeprator, HierarchyObj.m_ObjectAliasName, Field.m_FieldName, FieldAlias);
                                break;

                            default:
                                SB_DynamicQuery.AppendFormat("{0}{1}.{2} AS {1}${3}", CommaSeprator, HierarchyObj.m_ObjectAliasName, Field.m_FieldName, FieldAlias);
                                break;
                        }

                        if (String.IsNullOrEmpty(CommaSeprator))
                            CommaSeprator = ",";
                    }
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormSelectQuery_Scalar",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        private OMError FormREandRFSelectQueries(HierarchyObject HierarchyObj, ref List<SortPartition> SortPartitionList, ref StringBuilder REandRFSelectQuery)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;

            try
            {
                foreach (HierarchyObject TempHierarchyObject in m_NSearchCommonObj.HierarchyObjectList)
                {

                    if (TempHierarchyObject.m_ParentID != 0)
                    {
                        if (TempHierarchyObject.m_RelationshipName.ToUpper() == "FILESYSTEM")
                        {
                            ErrorObj = FormFileSystemSelectQuery(TempHierarchyObject, ref REandRFSelectQuery);
                        }
                        else
                        {
                            ErrorObj = FormRESelectQuery(TempHierarchyObject, ref m_NSearchCommonObj.SortPartitionList, ref REandRFSelectQuery);
                        }
                        if (ErrorObj != null)
                            return ErrorObj;
                    }
                }
                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormREandRFSelectQueries",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// This function builds a select query for FileSystem.
        /// </summary>
        /// <param name="HierarchyObj">HierarchyObject pertaining to FileSystem</param>
        /// <param name="REandRFSelectQuery"></param>
        /// <returns>OMError </returns>

        private OMError FormFileSystemSelectQuery(HierarchyObject HierarchyObj, ref StringBuilder REandRFSelectQuery)
        {

            OMError                         ReturnErrorObj                      = null;
            OMError                         ErrorObj                            = null;
            long                            ParentTagID                         = 0;
            int                             TempCounter                         = 0;
            int                             CollectionNumberofNulls             = 0;
            int                             TempCollectionNumberofNulls         = 0;
            int                             NumberofNulls                       = 0;
            int                             SortValue                           = 0;
            int                             BinderStartColNumber                = 0;
            int                             FileStartColNumber                  = 0;
            int                             SortValuesCounter                   = 0;
            SortPartition                   FSSortPartitionObj                  = null;
            SortPartition                   ParentSortPartitionObj              = null;
            StringBuilder                   RECollectionSelects                 = null;
            StringBuilder                   TempRECollectionSelects             = null;
            StringBuilder                   SortColumnValues                    = null;
            StringBuilder                   BinderFileData                      = null;
            StringBuilder                   TempBinderFileData                  = null;
            StringBuilder                   FileData                            = null;
            StringBuilder                   TempFileData                        = null;
            StringBuilder                   InnerJoinList                       = null;
            StringBuilder                   TempJoinString                      = null;
            List<SortPartitionNamedValue>   SortedListofColswithValues          = null;
            HierarchyObject                 ParentHierarchyObject               = null;
            HierarchyObject                 TempHierarchyObject                 = null;
            Boolean                         IsBinderRequested                   = false;
            Boolean                         IsFileRequested                     = false;
            String                          BinderName                          = null;
            String                          EveryOneTableAlias                  = null;
            String                          EntityTableAlias                    = null;
            String                          FileFieldName                       = null;
            StringBuilder                   SortColumnValueForBinderField       = null;     //Stores sort column values (in comma separated format) for binder field. These values will be prepared to support fetching multiple binder fields.
            StringBuilder                   SortColumnValueForFileField         = null;     //Stores sort column values (in comma separated format) for file field. This will allow files to be fetched after all binder fields.
            bool                            b_SortColumnValuesPreparedForFileField = false; //Specifies whether sort column values are prepared for file field when binder and file fields are requested. This will avoid preparing sort column values multiple times.
            int                             NullCounter                         = 0;
            int                             SortColumnCounter                   = 0;
            int                             BinderCollectionNumberofNulls       = 0;
            int                             FieldCounter                        = 0;
            int                             IndexforValuetoDecrement            = 0;
            int                             SortColumnsrequested                = 0;
            int                             BinderFieldIndex                    = 0;        //Stores index of binder field (starting from 1), this will be used to form sort column value for binder field.
            int                             BinderFieldCount                    = 0;        //Stored number of binder fields requested in output xml. This will be used when both FILE and BINDER fields are requested.
            ArrayList                       ArrofSortValues                     = null;
            String                          TableNamewithAlias                  = null;
            Dictionary<String,int>          dictTableAlias                      = null;     //Stores table name and count to prepare alias (in case where there is a possibility of conflict in alias)

            try
            {
                // Select Query for Collection.
                // get the sort partition object corresponding to RE.
                foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                {
                    if (TempSortPartitionObj.m_NodeID == HierarchyObj.m_TagID)
                    {
                        FSSortPartitionObj = TempSortPartitionObj;
                        break;
                    }
                }

                RECollectionSelects = new StringBuilder();
                BinderFileData = new StringBuilder();
                FileData = new StringBuilder();
                InnerJoinList = new StringBuilder();
                TempJoinString = new StringBuilder();

                foreach (FieldCollection Field in HierarchyObj.CustomFieldCollectionList)
                {
                    if (Field.m_FieldDataType.ToUpper() == "BINDER")
                    {
                        BinderFieldCount++;
                        IsBinderRequested = true;
                    }

                    if (Field.m_FieldDataType.ToUpper() == "FILE")
                        IsFileRequested = true;
                }

                REandRFSelectQuery.Append(" UNION");
                RECollectionSelects.Append(" UNION");
                BinderFileData.Append(" UNION");
                FileData.Append(" UNION");

                // Get Parent actual tagid.
                foreach (HierarchyObject TempHierarchyObject1 in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObject1.m_TagID == HierarchyObj.m_ParentID)
                    {
                        ParentTagID = TempHierarchyObject1.m_ActualTagID;
                        ParentHierarchyObject = TempHierarchyObject1;
                        break;
                    }
                }

                // Check which all custom datatypes are requested.
                if ((IsBinderRequested) && (!IsFileRequested))
                {
                    // only Binder is requested
                    REandRFSelectQuery.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID - 1, ParentTagID);
                    RECollectionSelects.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID, HierarchyObj.m_ActualTagID - 1);
                    BinderFileData.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID + 1, HierarchyObj.m_ActualTagID);
                    REandRFSelectQuery.Append(", NULL, NULL");
                    RECollectionSelects.Append(", NULL, NULL");
                    BinderFileData.Append(", NULL, NULL");
                }

                if ((!IsBinderRequested) && (IsFileRequested))
                {
                    // Only File is requested
                    REandRFSelectQuery.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT ", HierarchyObj.m_ActualTagID - 1, ParentTagID);
                    REandRFSelectQuery.Append(", NULL, NULL");
                    FileData.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT ", HierarchyObj.m_ActualTagID, HierarchyObj.m_ActualTagID - 1);
                    FileData.Append(", NULL, NULL");

                }

                if ((IsBinderRequested) && (IsFileRequested))
                {
                    // Both File and Binder are requested.
                    REandRFSelectQuery.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID - 1, ParentTagID);
                    RECollectionSelects.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID, HierarchyObj.m_ActualTagID - 1);
                    BinderFileData.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID + 1, HierarchyObj.m_ActualTagID);
                    FileData.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID + 2, HierarchyObj.m_ActualTagID - 1);
                    REandRFSelectQuery.Append(", NULL, NULL");
                    RECollectionSelects.Append(", NULL, NULL");
                    BinderFileData.Append(", NULL, NULL");
                    FileData.Append(", NULL, NULL");

                }

                if (FSSortPartitionObj.PredecessorList.Count > 0)
                {
                    dictTableAlias = new Dictionary<string, int>(FSSortPartitionObj.PredecessorList.Count);
                    /* ParentHierarchyObject.m_TableName is added to dictionary so that the same alias will not be used again.
                     * EntityTableAlias is formed using ParentHierarchyObject.m_TableName + "_ALIAS", this is used at the end to replace
                     * inner join list with temp join
                     * */
                    dictTableAlias.Add(ParentHierarchyObject.m_TableName.ToUpper(), 0);
                }

                EveryOneTableAlias = ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS";
                EntityTableAlias = ParentHierarchyObject.m_TableName + "_ALIAS";

                foreach (Predecessor TempPredecessorObj in FSSortPartitionObj.PredecessorList)
                {
                    CollectionNumberofNulls = 0;
                    FieldCounter = 0;
                    TableNamewithAlias = null;
                    if (TempPredecessorObj.m_NumberofFields > 0)
                    {
                        // For TE
                        if (TempCounter == 0)
                        {
                            REandRFSelectQuery.AppendFormat(", {0}.{1} ", EveryOneTableAlias, "FILTEREDOBJID");
                            RECollectionSelects.AppendFormat(", {0}.{1} ", EveryOneTableAlias, "FILTEREDOBJID");
                            BinderFileData.AppendFormat(", {0}.{1} ", EveryOneTableAlias, "FILTEREDOBJID");
                            FileData.AppendFormat(", {0}.{1} ", EveryOneTableAlias, "FILTEREDOBJID");
                            // Add nulls to select query.

                            if (TempPredecessorObj.m_NumberofFields > 0)
                            {
                                ErrorObj = GetHierarchyObjectfromList(TempPredecessorObj.m_PredecessorID, ref TempHierarchyObject);
                                if (ErrorObj != null)
                                    return ErrorObj;

                                /* Alias is generated. 
                                 * When parent hierarchy object and predecessor hierarchy object actual tag ids are same, do not 
                                 * generate alias, since inner join taken when SortColumnCounter > 0 will be replaced at the end of this loop.
                                 * If alias is generated in this case, then there will be an additional join taken on table */
                                if(ParentHierarchyObject.m_ActualTagID != TempHierarchyObject.m_ActualTagID)
                                    TableNamewithAlias = PrepareAliasForFileSystemSelectQuery(TempHierarchyObject.m_TableName, ref dictTableAlias);
                                else
                                    TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS";

                                SortColumnCounter = 0;
                                foreach (FieldCollection TempField in TempHierarchyObject.FieldCollectionList)
                                {
                                    // Get the non primary key Fields which are requested for Sorting
                                    // as we need to propogate its values to root level.
                                    if ((!TempField.m_IsPrimaryKey) && TempField.m_IsSortRequested)
                                    {
                                        SortColumnCounter++;
                                        SortColumnsrequested++;
                                        // This is for TE

                                        //TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS";
                                        //Switch - case has been used instead of if..else
                                        switch (TempField.m_FieldDataType.ToUpper())
                                        {
                                            case "DATETIME":
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    REandRFSelectQuery.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2)  + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2)  + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    FileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2)  + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            //Case commented by : Mr. Sagar Dasalkar on 19 Nov 09
                                            //Sorting would not be done on CURRENCY field, instead it would be done on an additional column having currency amount.    
                                            //case "CURRENCY":
                                            //    {
                                            //        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        REandRFSelectQuery.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        REandRFSelectQuery.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        FileData.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        FileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");
                                            //    }
                                            //    break;

                                            case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "FLOAT":
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "INTEGER":
                                            case "BIGINT":
                                                {
                                                    RECollectionSelects.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            default:
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        //Add a null to Collection and select query.
                                        if (!TempField.m_IsPrimaryKey)
                                        {
                                            REandRFSelectQuery.AppendFormat(", NULL");
                                            RECollectionSelects.AppendFormat(", NULL");
                                            BinderFileData.AppendFormat(", NULL");
                                            FileData.AppendFormat(", NULL");
                                        }
                                    }
                                }

                                if (SortColumnCounter > 0)
                                {
                                    //InnerJoinList.AppendFormat(" INNER JOIN {0} {1} on {1}.{2} = {3}.FILTEREDOBJID ", TempHierarchyObject.m_TableName, TempHierarchyObject.m_TableName + "_ALIAS", TempHierarchyObject.m_PrimaryKeyName, HierarchyObj.m_EveryOneUserTableName.Trim());
                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {1}.{2} = {3}_ALIAS.FILTEREDOBJID ", TempHierarchyObject.m_TableName, TableNamewithAlias, TempHierarchyObject.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim());
                                }

                                // 4 because Parent,Tag,TECollection,TEData
                                NullCounter = 4 + TempPredecessorObj.m_NumberofFields;
                            }
                            CollectionNumberofNulls = TempPredecessorObj.m_StartColumnNumber + (TempPredecessorObj.m_NumberofFields - 1);
                        }
                        else
                        {
                            SortColumnsrequested = 0;
                            if (NullCounter < TempPredecessorObj.m_StartColumnNumber)
                            {
                                RECollectionSelects = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), RECollectionSelects);
                                REandRFSelectQuery = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), REandRFSelectQuery);
                                BinderFileData = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), BinderFileData);
                                FileData = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), FileData);
                            }

                            REandRFSelectQuery.AppendFormat(", {0}.{1}", EveryOneTableAlias, TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());
                            RECollectionSelects.AppendFormat(", {0}.{1}", EveryOneTableAlias, TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());
                            BinderFileData.AppendFormat(", {0}.{1}", EveryOneTableAlias, TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());
                            FileData.AppendFormat(", {0}.{1}", EveryOneTableAlias, TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());

                            if (TempPredecessorObj.m_NumberofFields > 0)
                            {
                                // Code for Sorting

                                ErrorObj = GetHierarchyObjectfromList(TempPredecessorObj.m_PredecessorID, ref TempHierarchyObject);
                                if (ErrorObj != null)
                                    return ErrorObj;

                                /* Alias is generated. 
                                 * When parent hierarchy object and predecessor hierarchy object actual tag ids are same, do not 
                                 * generate alias, since inner join taken when SortColumnCounter > 0 will be replaced at the end of this loop.
                                 * If alias is generated in this case, then there will be an additional join taken on table */
                                if (ParentHierarchyObject.m_ActualTagID != TempHierarchyObject.m_ActualTagID)
                                    TableNamewithAlias = PrepareAliasForFileSystemSelectQuery(TempHierarchyObject.m_TableName, ref dictTableAlias);
                                else
                                    TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS";

                                SortColumnCounter = 0;
                                foreach (FieldCollection TempField in TempHierarchyObject.FieldCollectionList)
                                {
                                    // Get the non primary key Fields which are requested for Sorting
                                    // as we need to propogate its values to root level.
                                    if ((!TempField.m_IsPrimaryKey) && TempField.m_IsSortRequested)
                                    {
                                        SortColumnCounter++;
                                        SortColumnsrequested++;
                                        // This is for TE
                                        //if ((!TempField.m_IsPrimaryKey) && TempField.m_IsSortRequested)
                                        //{
                                        //    SortColumnCounter++;
                                        // This is for TE
                                        //TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS";
                                        //Switch - case has been used instead of if..else
                                        switch (TempField.m_FieldDataType.ToUpper())
                                        {
                                            case "DATETIME":
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    REandRFSelectQuery.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    FileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2)  + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                }

                                                break;

                                            //Case commented by : Mr. Sagar Dasalkar on 19 Nov 09
                                            //Sorting would not be done on CURRENCY field, instead it would be done on an additional column having currency amount.    
                                            //case "CURRENCY":
                                            //    {
                                            //        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        REandRFSelectQuery.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        REandRFSelectQuery.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        FileData.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        FileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");
                                            //    }
                                            //    break;

                                            case OMInternalDataTypeForCurrency.CURRENY_AMOUNT :
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "FLOAT":
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "INTEGER":
                                            case "BIGINT":
                                                {
                                                    RECollectionSelects.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                }

                                                break;

                                            default:
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    BinderFileData.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    FileData.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        //Add a null to Collection and select query.
                                        if (!TempField.m_IsPrimaryKey)
                                        {
                                            FieldCounter++;
                                            REandRFSelectQuery.AppendFormat(", NULL");
                                            RECollectionSelects.AppendFormat(", NULL");
                                            BinderFileData.AppendFormat(", NULL");
                                            FileData.AppendFormat(", NULL");
                                        }
                                    }

                                }

                                /* FieldCounter is incremented for non-primary fields (for which sorting is not provided).
                                 * In case when sort list is provided, FieldCounter does not reach to the count of total number of fields.
                                 * So SortColumnCounter is added to FieldCounter since these fields are already added to query. This will avoid
                                 * adding extra column in query than expected.
                                 * */
                                if (FieldCounter + SortColumnCounter < TempPredecessorObj.m_NumberofFields)
                                {
                                    RECollectionSelects = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter - SortColumnCounter), RECollectionSelects);
                                    REandRFSelectQuery = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter - SortColumnCounter), REandRFSelectQuery);
                                    BinderFileData = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter - SortColumnCounter), BinderFileData);
                                    FileData = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter - SortColumnCounter), FileData);
                                }

                                if (SortColumnCounter > 0)
                                {
                                    //InnerJoinList.AppendFormat(" INNER JOIN {0}.{1} RE{2} on RE_ALIAS_{2}.{1} = @{3}.{4}{2} ", TempHierarchyObject.m_TableName, TempHierarchyObject.m_PrimaryKeyName, TempHierarchyObject.m_TagID, TempHierarchyObject.m_RelationshipName);
                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {5} ON {5}.{1} = {2}_ALIAS.{3}{4} ", TempHierarchyObject.m_TableName, TempHierarchyObject.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim(), TempHierarchyObject.m_RelationshipName, TempHierarchyObject.m_RelatedObjectLevel, TableNamewithAlias);
                                }
                            }
                            NullCounter = TempPredecessorObj.m_StartColumnNumber + TempPredecessorObj.m_NumberofFields;
                            //Since count of number of fields is added to CollectionNumberOfNulls, SortColumn counter is not added
                            CollectionNumberofNulls = TempPredecessorObj.m_StartColumnNumber + (TempPredecessorObj.m_NumberofFields);
                        }
                    }
                    TempCounter++;
                }

                if (CollectionNumberofNulls < FSSortPartitionObj.m_StartColumnNumber)
                {
                    TempCollectionNumberofNulls = 0;
                    TempCollectionNumberofNulls = (FSSortPartitionObj.m_StartColumnNumber) - CollectionNumberofNulls;
                    CollectionNumberofNulls = CollectionNumberofNulls + TempCollectionNumberofNulls;

                    REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                    RECollectionSelects = AddNullstoQuery(TempCollectionNumberofNulls, RECollectionSelects);
                    BinderFileData = AddNullstoQuery(TempCollectionNumberofNulls, BinderFileData);
                    FileData = AddNullstoQuery(TempCollectionNumberofNulls, FileData);
                }

                // Check if the sortColumn for this object is there in SortColumnList
                if (m_NSearchCommonObj.m_SortPartitionColumnsList.ContainsValue(FSSortPartitionObj.m_SortPartitionColumnName))
                {
                    // get the sort partition object corresponding to RE's Parent .
                    foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                    {
                        if (TempSortPartitionObj.m_NodeID == FSSortPartitionObj.m_ParentID)
                        {
                            ParentSortPartitionObj = TempSortPartitionObj;
                            break;
                        }
                    }
                    SortedListofColswithValues = null;
                    SortedListofColswithValues = new List<SortPartitionNamedValue>();

                    ErrorObj = GetSortPartitionValues(FSSortPartitionObj.m_SortPartitionID, FSSortPartitionObj.m_SortPartitionColumnName, ParentSortPartitionObj, ref SortedListofColswithValues);

                    if (ErrorObj != null)
                        return ErrorObj;

                    if (SortedListofColswithValues.Count > 0)
                    {
                        ArrofSortValues = new ArrayList();
                        ErrorObj = GetSortValuesSortedonSortColumns(ref SortedListofColswithValues, ref ArrofSortValues, ref IndexforValuetoDecrement);
                        if (ErrorObj != null)
                            return ErrorObj;
                    }
                }

                if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                {
                    TempCollectionNumberofNulls = 0;
                    if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                    {
                        SortColumnValues = new StringBuilder();
                        TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                        if (CollectionNumberofNulls > 0)
                            REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                        // Add Sort Partition Values.
                        TempCollectionNumberofNulls = TempCollectionNumberofNulls + CollectionNumberofNulls;

                        if (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                        {
                            SortValuesCounter = 0;
                            foreach (String SortColValue in ArrofSortValues)
                            {
                                TempCollectionNumberofNulls++;

                                if (SortValuesCounter == IndexforValuetoDecrement)
                                {
                                    SortValue = Convert.ToInt32(SortColValue);
                                    SortValue--;
                                    REandRFSelectQuery.AppendFormat(", {0}", SortValue.ToString());
                                    SortValue++;
                                    SortColumnValues.AppendFormat(", {0}", SortValue.ToString());
                                }
                                else
                                {
                                    SortColumnValues.AppendFormat(", {0}", SortColValue);
                                    REandRFSelectQuery.AppendFormat(", {0}", SortColValue);
                                }
                                SortValuesCounter++;
                            }
                            if (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                while (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                                {
                                    REandRFSelectQuery.Append(", NULL");
                                    SortColumnValues.Append(", NULL");
                                    TempCollectionNumberofNulls++;
                                }
                            }
                        }
                    }
                    else
                    {
                        TempCollectionNumberofNulls = m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls;
                        if (CollectionNumberofNulls > 0)
                            REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                    }
                }

                // Add FROM Clause to FileSystem Collection.
                REandRFSelectQuery.AppendFormat(" FROM @{0} AS {1} INNER JOIN {2} AS {3}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_TableName, EntityTableAlias);
                //REandRFSelectQuery.AppendFormat(" ON {0}.{1} = {2}.{3}", EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName);
                REandRFSelectQuery.AppendFormat(" ON {0}.{1} = {2}.{3}", EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName, EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                TempJoinString.AppendFormat(" INNER JOIN {0} {1} ON {2}.{3} = {4}.{5}", ParentHierarchyObject.m_TableName, EntityTableAlias, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName, EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                
                /* Join with respective table (entity/facet) is already taken in REandRFSelectQuery (before taking joins in TempJoinString).
                 * If sorting is provided for this object type (entity/facet), the join will get added to InnerJoinList.
                 * Now, TempJoinString is replaced with empty string in InnerJoinList to remove additional join on object table (entity/facet).
                 * e.g. Consider following hierarchy
                 *  TE (file/binder requested)
                 *    |->RE1 (sorting, file/binder requested)
                 *         |->RE2 (sorting, file/binder requested)
                 * Processing of above case : 
                 * 1. Processing (file system relationship for) TE - Since sorting is not provided on TE, join with TE table will not be included in InnerJoinList.
                 * 2. Processing (file system relationship for) RE1 - 
                 *      2.1 Sorting not provided for TE , so join with TE table will not be added to InnerJoinList. 
                 *      2.2 Sorting is provided for RE1 , so join with RE1 table will be added to InnerJoinList. In this case same join will get added
                 *          to REandRFSelectQuery. So while appending InnerJoinString, there will be multiple joins present in REandRFSelectQuery with same alias. 
                 *          So in this case join taken on RE1 table will be replaced by empty string in InnerJoinList to avoid multiple joins on same table with same alias.
                 * 3. Processing (file system relationship for) RE2 - 
                 *      3.1 Sorting not provided for TE , so join with TE table will not be added to InnerJoinList.
                 *      3.2 Sorting is provided for RE1 , so join with RE1 table will be added to InnerJoinList.
                 *      3.3 Sorting is provided for RE2 , so join with RE2 table will be added to InnerJoinList and it will be replaced by TempJoinString.
                 * 
                 * Note: Since TempJoinString is replaced with empty string in InnerJoinList, the join is formed for file/binder fields (in the following part) and then
                 * InnerJoinList is appended to it.
                 * */
                
                InnerJoinList.Replace(TempJoinString.ToString(), "");
                TempJoinString.Remove(0, TempJoinString.Length);
                REandRFSelectQuery.AppendFormat(InnerJoinList.ToString());

                NumberofNulls = CollectionNumberofNulls;

                #region Preparing File and Binder Select Query
                // Binder and File are requested.
                if ((IsBinderRequested) && (IsFileRequested))
                {
                    // this means that only Binder is selected.
                    // Start binder columns.
                    TempRECollectionSelects = new StringBuilder();
                    TempRECollectionSelects.AppendFormat(RECollectionSelects.ToString());
                    TempBinderFileData = new StringBuilder();
                    TempBinderFileData.AppendFormat(BinderFileData.ToString());
                    TempFileData = new StringBuilder();
                    TempFileData.AppendFormat(FileData.ToString());
                    BinderFieldIndex = 0;
                    b_SortColumnValuesPreparedForFileField = false;
                    BinderStartColNumber = NumberofNulls;
                    FileStartColNumber = NumberofNulls + NSearchCommon.m_OnlyBinderFields;

                    foreach (FieldCollection CustomField in HierarchyObj.CustomFieldCollectionList)
                    {
                        if (CustomField.m_FieldDataType.ToUpper() == "BINDER")
                        {
                            CollectionNumberofNulls = BinderStartColNumber;
                            RECollectionSelects.Remove(0, RECollectionSelects.Length);
                            RECollectionSelects.AppendFormat(TempRECollectionSelects.ToString());

                            BinderFileData.Remove(0, BinderFileData.Length);
                            BinderFileData.AppendFormat(TempBinderFileData.ToString());

                            //RECollectionSelects.AppendFormat(", NULL");
                            //CollectionNumberofNulls++;

                            BinderName = CustomField.m_FieldName;
                            BinderCollectionNumberofNulls = CollectionNumberofNulls;

                            BinderFieldIndex++;

                            foreach (DictionaryEntry FieldName in m_NSearchCommonObj.m_BinderFields)
                            {
                                CollectionNumberofNulls++;

                                if (FieldName.Value.ToString() == "DATETIME")
                                {
                                    RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSB.{0})) + ''</YEAR>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},101),1,2) + ''</MONTH>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},101),4,2) + ''</DAY>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),12,2) + ''</HOUR>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),15,2) + ''</MINUTE>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),18,2) + ''</SECOND>'','''')", FieldName.Key.ToString());
                                }
                                else
                                {
                                    // This code is added for Currency support    
                                    if (FieldName.Value.ToString() == "CURRENCY")
                                    {
                                        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSB.{0} + ''</CURRENCY>'','''')", FieldName.Key.ToString() + "_C");
                                        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSB.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FieldName.Key.ToString() + "_A");
                                    }
                                    else
                                    {
                                        if (FieldName.Key.ToString() == "FIELDNAME")
                                            RECollectionSelects.AppendFormat(", ISNULL(ALSC.{0},'''')", FieldName.Key.ToString());
                                        else
                                            RECollectionSelects.AppendFormat(", ISNULL(ALFSB.{0},'''')", FieldName.Key.ToString());
                                    }
                                }
                                //RECollectionSelects.AppendFormat(", ALFSB.{0}", FieldName.Key.ToString());
                            }

                            if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                TempCollectionNumberofNulls = 0;
                                if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                {
                                    TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                                    RECollectionSelects = AddNullstoQuery(TempCollectionNumberofNulls, RECollectionSelects);
                                    if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                    {
                                        /* Assigning Sort Column values for multiple binder fields: 
                                         * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                         *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                         * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                         *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                         *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                         * This will append FILE nodes to appropriate BINDER nodes. 
                                         * */
                                        if (SortColumnValueForBinderField == null)
                                            SortColumnValueForBinderField = new StringBuilder(50);
                                        else
                                            SortColumnValueForBinderField = SortColumnValueForBinderField.Remove(0, SortColumnValueForBinderField.Length);
                                        //Here sort column values will be generated only for Binder fields (not considering Binder files)
                                        ErrorObj = PrepareSortColumnValues("BINDER", BinderFieldIndex, FSSortPartitionObj, SortedListofColswithValues, ref SortColumnValueForBinderField);
                                        if (ErrorObj != null)
                                            return ErrorObj;
                                        RECollectionSelects.Append(SortColumnValueForBinderField);
                                    }
                                }
                                else
                                    RECollectionSelects = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls), RECollectionSelects);
                            }

                            // From Query.

                            RECollectionSelects.AppendFormat(" FROM ALFS_BINDER AS ALFSB INNER JOIN @{0} AS {1} ON ALFSB.OBJECTID = {1}.{2} INNER JOIN {3} AS {4}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, ParentHierarchyObject.m_TableName, EntityTableAlias);
                            RECollectionSelects.AppendFormat(" ON {0}.{1} =  {2}.{3} ", EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName);
                            RECollectionSelects.AppendFormat(InnerJoinList.ToString());
                            RECollectionSelects.AppendFormat(" INNER JOIN ALOM_SCHEMA AS ALSC ON ALFSB.SCHEMAID = ALSC.SCHEMAID WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1} ", BinderName, ParentHierarchyObject.m_ObjectTypeID);
                            REandRFSelectQuery.AppendFormat(RECollectionSelects.ToString());

                            // This is done because now there is no need to select Binder Columns and 1 for File Collection inserts.
                            BinderFileData = AddNullstoQuery(m_NSearchCommonObj.m_BinderFields.Count + 1, BinderFileData);

                            // Select Columns of AssoFileBinder.
                            foreach (DictionaryEntry AssoFileField in m_NSearchCommonObj.m_AssoFileBinderFields)
                            {
                                if (AssoFileField.Value.ToString() == "DATETIME")
                                {
                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ASSOFB.{0})) + ''</YEAR>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},101),1,2) + ''</MONTH>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22),ASSOFB.{0},101),4,2) + ''</DAY>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22),ASSOFB.{0},121),12,2) + ''</HOUR>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22),ASSOFB.{0},121),15,2) + ''</MINUTE>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22),ASSOFB.{0},121),18,2) + ''</SECOND>'','''')", AssoFileField.Key.ToString());
                                }
                                else
                                {
                                    if (AssoFileField.Value.ToString() == "CURRENCY")
                                    {
                                        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ASSOFB.{0} + ''</CURRENCY>'','''')", AssoFileField.Key.ToString() + "_C");
                                        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ASSOFB.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", AssoFileField.Key.ToString() + "_A");
                                    }
                                    else
                                        BinderFileData.AppendFormat(", ISNULL(ASSOFB.{0},'''')", AssoFileField.Key.ToString());
                                }
                                //BinderFileData.AppendFormat(", ASSOFB.{0}", AssoFileField.Key.ToString());
                            }

                            // Select Columns of File

                            foreach (DictionaryEntry FileField in m_NSearchCommonObj.m_FileFields)
                            {
                                if (FileField.Value.ToString() == "DATETIME")
                                {
                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSF.{0})) + ''</YEAR>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),1,2) + ''</MONTH>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),4,2) + ''</DAY>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),12,2) + ''</HOUR>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),15,2) + ''</MINUTE>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),18,2) + ''</SECOND>'','''')", FileField.Key.ToString());
                                }
                                else
                                {
                                    if (FileField.Value.ToString() == "CURRENCY")
                                    {
                                        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSF.{0} + ''</CURRENCY>'','''')", FileField.Key.ToString() + "_C");
                                        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSF.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FileField.Key.ToString() + "_A");
                                    }
                                    else
                                    {
                                        if (FileField.Key.ToString() == "FIELDNAME")
                                            BinderFileData.AppendFormat(", ISNULL(ALSC.{0},'''')", FileField.Key.ToString());
                                        else
                                            BinderFileData.AppendFormat(", ISNULL(ALFSF.{0},'''')", FileField.Key.ToString());
                                    }

                                }
                                //BinderFileData.AppendFormat(", ALFSF.{0}", FileField.Key.ToString());
                            }

                            // Append nulls till the end and check for Sort Columns.
                            BinderCollectionNumberofNulls = BinderCollectionNumberofNulls + m_NSearchCommonObj.m_BinderFields.Count + 1 + m_NSearchCommonObj.m_AssoFileBinderFields.Count + m_NSearchCommonObj.m_FileFields.Count;
                            if (BinderCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                TempCollectionNumberofNulls = 0;
                                if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                {
                                    TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - BinderCollectionNumberofNulls;
                                    BinderFileData = AddNullstoQuery(TempCollectionNumberofNulls, BinderFileData);
                                    if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                    {
                                        /* Assigning Sort Column values for multiple binder fields: 
                                         * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                         *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                         * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                         *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                         *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                         * This will append FILE nodes to appropriate BINDER nodes. 
                                         * */
                                        if (SortColumnValueForBinderField == null)
                                            SortColumnValueForBinderField = new StringBuilder(50);
                                        else
                                            SortColumnValueForBinderField = SortColumnValueForBinderField.Remove(0, SortColumnValueForBinderField.Length);
                                        //Here sort column values will be generated only for Binder Files
                                        ErrorObj = PrepareSortColumnValues("BINDERFILE", BinderFieldIndex, FSSortPartitionObj, SortedListofColswithValues, ref SortColumnValueForBinderField);
                                        if (ErrorObj != null)
                                            return ErrorObj;

                                        BinderFileData.Append(SortColumnValueForBinderField);
                                    }
                                }
                                else
                                    BinderFileData = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - BinderCollectionNumberofNulls), BinderFileData);
                            }

                            // Append FROM query.
                            BinderFileData.AppendFormat(" FROM ALFS_FILE AS ALFSF INNER JOIN @{0} AS {1} ON ALFSF.OBJECTID ={1}.{2}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            BinderFileData.AppendFormat(" INNER JOIN ALFS_ASSOFILEBINDER AS ASSOFB ON ALFSF.FILEID = ASSOFB.FILEID INNER JOIN ALFS_BINDER AS ALFSB ON ASSOFB.BINDERID = ALFSB.BINDERID AND ALFSF.SCHEMAID = ALFSB.SCHEMAID INNER JOIN {0} AS {1} ", ParentHierarchyObject.m_TableName, EntityTableAlias);
                            BinderFileData.AppendFormat(" ON {0}.{1} = {2}.{3} ", EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName);
                            BinderFileData.AppendFormat(InnerJoinList.ToString());
                            //BinderFileData.AppendFormat("WHERE ALFSF.SCHEMAID IN (SELECT ALSC.SCHEMAID FROM ALOM_SCHEMA AS ALSC WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1} )", BinderName, ParentHierarchyObject.m_ObjectTypeID);
                            BinderFileData.AppendFormat(" INNER JOIN ALOM_SCHEMA ALSC ON ALSC.SCHEMAID = ALFSF.SCHEMAID WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1}", BinderName, ParentHierarchyObject.m_ObjectTypeID);
                            REandRFSelectQuery.AppendFormat(BinderFileData.ToString());
                        }
                        // Processing for File Datatype
                        if (CustomField.m_FieldDataType.ToUpper() == "FILE")
                        {
                            //NumberofNulls = CollectionNumberofNulls;
                            // Insert Nulls For Binder selected till fileStartCOlNumber.

                            CollectionNumberofNulls = FileStartColNumber - 1;
                            FileData.Remove(0, FileData.Length);
                            FileData.AppendFormat(TempFileData.ToString());
                            FileData = AddNullstoQuery(((FileStartColNumber - 1) - BinderStartColNumber), FileData);
                            //FileData.Append(", NULL");
                            //CollectionNumberofNulls++;

                            if (CustomField.m_FieldDataType.ToUpper() == "FILE")
                            {
                                FileFieldName = CustomField.m_FieldName;
                                // Append actual file fields.
                                foreach (DictionaryEntry FileField in m_NSearchCommonObj.m_FileFields)
                                {
                                    CollectionNumberofNulls++;
                                    if (FileField.Value.ToString() == "DATETIME")
                                    {
                                        FileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSF.{0})) + ''</YEAR>''", FileField.Key.ToString());
                                        FileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),1,2) + ''</MONTH>''", FileField.Key.ToString());
                                        FileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),4,2) + ''</DAY>''", FileField.Key.ToString());
                                        FileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),12,2) + ''</HOUR>''", FileField.Key.ToString());
                                        FileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),15,2) + ''</MINUTE>''", FileField.Key.ToString());
                                        FileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),18,2) + ''</SECOND>'','''')", FileField.Key.ToString());
                                    }
                                    else
                                    {
                                        if (FileField.Value.ToString() == "CURRENCY")
                                        {
                                            FileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSF.{0} + ''</CURRENCY>'','''')", FileField.Key.ToString() + "_C");
                                            FileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSF.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FileField.Key.ToString() + "_A");
                                        }
                                        else
                                        {
                                            if (FileField.Key.ToString() == "FIELDNAME")
                                                FileData.AppendFormat(", ISNULL(ALSC.{0},'''')", FileField.Key.ToString());
                                            else
                                                FileData.AppendFormat(", ISNULL(ALFSF.{0},'''')", FileField.Key.ToString());
                                        }
                                    }
                                }

                                // this will add nulls to Filedata stringbuilder and sort values if present.
                                if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                                {
                                    TempCollectionNumberofNulls = 0;
                                    if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                    {
                                        TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                                        FileData = AddNullstoQuery(TempCollectionNumberofNulls, FileData);
                                        if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                        {
                                            /* Assigning Sort Column values for files when multiple binder and file fields are requested in output: 
                                             * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                             *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                             * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                             *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                             *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                             * 3. Now, sort column values will be prepared for binder fields as specified in step 1 and 2, 
                                             *    Step 3 will consider file fields. Sort column values for file fields will be assigned from next slot after processing binder fields by considering binder field count.
                                             *    e.g. 2 binder fields and 2 file fields are requested, sort column values will be
                                             *    3.1 Binder 1 - Sort column value is 2, for binder files value would be 3
                                             *    3.2 Binder 2 - Sort column value is 4, for binder files value would be 5
                                             *    3.3 file     - Sort column value is 6 (for all files)
                                             *    This is done so that all file (non binder) fields will be fetched at the end in FILESYSTEM node.
                                             * This will append FILE nodes to appropriate BINDER nodes. 
                                             * */
                                            if (!b_SortColumnValuesPreparedForFileField)
                                            {
                                                if (SortColumnValueForFileField == null)
                                                    SortColumnValueForFileField = new StringBuilder(50);
                                                else
                                                    SortColumnValueForFileField.Remove(0, SortColumnValueForFileField.Length);

                                                //Here sort column values will be prepared for FILE field. Binder field count is considered since the value would be assigned after assigning values to binder fields. 
                                                ErrorObj = PrepareSortColumnValues("FILE", BinderFieldCount, FSSortPartitionObj, SortedListofColswithValues, ref SortColumnValueForFileField);
                                                if (ErrorObj != null)
                                                    return ErrorObj;

                                                b_SortColumnValuesPreparedForFileField = true;
                                            }

                                            FileData.Append(SortColumnValueForFileField.ToString());
                                        }
                                    }
                                    else
                                        FileData = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls), FileData);
                                }

                                // FROM clause.
                                FileData.AppendFormat(" FROM ALFS_FILE AS ALFSF INNER JOIN @{0} AS {1} ON ALFSF.OBJECTID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                FileData.AppendFormat(" INNER JOIN {0} AS {1} ON {1}.{2} = {3}.{4} ", ParentHierarchyObject.m_TableName, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName, EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                FileData.AppendFormat(InnerJoinList.ToString());
                                FileData.AppendFormat(" INNER JOIN ALOM_SCHEMA ALSC ON 	ALSC.SCHEMAID = ALFSF.SCHEMAID WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1}", FileFieldName, ParentHierarchyObject.m_ObjectTypeID);

                                //FileData.AppendFormat(" WHERE ALFSF.SCHEMAID IN (SELECT ALSC.SCHEMAID FROM ALOM_SCHEMA AS ALSC WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1})", FileFieldName, ParentHierarchyObject.m_ObjectTypeID);

                                REandRFSelectQuery.AppendFormat(FileData.ToString());
                                FileData.Remove(0, FileData.Length);
                            }
                        }
                    }

                    b_SortColumnValuesPreparedForFileField = false;
                }

                #endregion

                #region Preparing Binder Select  Query.
                if ((IsBinderRequested) && (!IsFileRequested))
                {
                    // this means that only Binder is selected.
                    // Start binder columns.
                    //RECollectionSelects.AppendFormat(", NULL,NULL");
                    //CollectionNumberofNulls += 2;

                    TempRECollectionSelects = new StringBuilder();
                    TempRECollectionSelects.AppendFormat(RECollectionSelects.ToString());
                    TempBinderFileData = new StringBuilder();
                    TempBinderFileData.AppendFormat(BinderFileData.ToString());
                    TempFileData = new StringBuilder();
                    TempFileData.AppendFormat(FileData.ToString());
                    BinderStartColNumber = NumberofNulls;
                    BinderFieldIndex = 0;

                    foreach (FieldCollection CustomField in HierarchyObj.CustomFieldCollectionList)
                    {
                        if (CustomField.m_FieldDataType.ToUpper() == "BINDER")
                        {
                            CollectionNumberofNulls = BinderStartColNumber;
                            RECollectionSelects.Remove(0, RECollectionSelects.Length);
                            RECollectionSelects.AppendFormat(TempRECollectionSelects.ToString());

                            BinderFileData.Remove(0, BinderFileData.Length);
                            BinderFileData.AppendFormat(TempBinderFileData.ToString());

                            BinderName = CustomField.m_FieldName;
                            BinderCollectionNumberofNulls = CollectionNumberofNulls;

                            BinderFieldIndex++;

                            foreach (DictionaryEntry FieldName in m_NSearchCommonObj.m_BinderFields)
                            {
                                CollectionNumberofNulls++;
                                if (FieldName.Value.ToString() == "DATETIME")
                                {
                                    RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSB.{0})) + ''</YEAR>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},101),1,2) + ''</MONTH>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},101),4,2) + ''</DAY>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),12,2) + ''</HOUR>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),15,2) + ''</MINUTE>''", FieldName.Key.ToString());
                                    RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSB.{0},121),18,2) + ''</SECOND>'','''')", FieldName.Key.ToString());
                                }
                                else
                                {
                                    // This code is added for Currency support    
                                    if (FieldName.Value.ToString() == "CURRENCY")
                                    {
                                        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSB.{0} + ''</CURRENCY>'','''')", FieldName.Key.ToString() + "_C");
                                        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSB.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FieldName.Key.ToString() + "_A");
                                    }
                                    else
                                    {
                                        if (FieldName.Key.ToString() == "FIELDNAME")
                                            RECollectionSelects.AppendFormat(", ISNULL(ALSC.{0},'''')", FieldName.Key.ToString());
                                        else
                                            RECollectionSelects.AppendFormat(", ISNULL(ALFSB.{0},'''')", FieldName.Key.ToString());
                                    }
                                }
                                //RECollectionSelects.AppendFormat(", ALFSB.{0}", FieldName.Key.ToString());
                            }

                            if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                TempCollectionNumberofNulls = 0;
                                if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                {
                                    TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                                    RECollectionSelects = AddNullstoQuery(TempCollectionNumberofNulls, RECollectionSelects);
                                    if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                    {
                                        /* Assigning Sort Column values for multiple binder fields: 
                                        * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                        *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                        * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                        *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                        *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                        * This will append FILE nodes to appropriate BINDER nodes. 
                                        * */
                                        if (SortColumnValueForBinderField == null)
                                            SortColumnValueForBinderField = new StringBuilder(50);
                                        else
                                            SortColumnValueForBinderField = SortColumnValueForBinderField.Remove(0, SortColumnValueForBinderField.Length);
                                        //Here sort column values will be generated only for Binder fields (not considering Binder files)
                                        ErrorObj = PrepareSortColumnValues("BINDER", BinderFieldIndex, FSSortPartitionObj, SortedListofColswithValues, ref SortColumnValueForBinderField);
                                        if (ErrorObj != null)
                                            return ErrorObj;
                                        RECollectionSelects.Append(SortColumnValueForBinderField);
                                    }
                                }
                                else
                                    RECollectionSelects = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls), RECollectionSelects);
                            }

                            // From Query.

                            RECollectionSelects.AppendFormat(" FROM ALFS_BINDER AS ALFSB INNER JOIN @{0} AS {1} ON ALFSB.OBJECTID = {1}.{2} INNER JOIN {3} AS {4}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, ParentHierarchyObject.m_TableName, EntityTableAlias);
                            RECollectionSelects.AppendFormat(" ON {0}.{1} =  {2}.{3} ", EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName);
                            RECollectionSelects.AppendFormat(InnerJoinList.ToString());
                            RECollectionSelects.AppendFormat(" INNER JOIN ALOM_SCHEMA ALSC ON ALSC.SCHEMAID = ALFSB.SCHEMAID WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1}", BinderName, ParentHierarchyObject.m_ObjectTypeID);
                            REandRFSelectQuery.AppendFormat(RECollectionSelects.ToString());

                            // This is done because now there is no need to select Binder Columns and 1 for File Collection inserts.
                            BinderFileData = AddNullstoQuery(m_NSearchCommonObj.m_BinderFields.Count + 1, BinderFileData);

                            // Select Columns of AssoFileBinder.
                            foreach (DictionaryEntry AssoFileField in m_NSearchCommonObj.m_AssoFileBinderFields)
                            {
                                if (AssoFileField.Value.ToString() == "DATETIME")
                                {
                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ASSOFB.{0})) + ''</YEAR>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},101),1,2)  + ''</MONTH>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},101),4,2) + ''</DAY>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},121),12,2) + ''</HOUR>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},121),15,2) + ''</MINUTE>''", AssoFileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ASSOFB.{0},121),18,2) + ''</SECOND>'','''')", AssoFileField.Key.ToString());
                                }
                                else
                                {
                                    if (AssoFileField.Value.ToString() == "CURRENCY")
                                    {
                                        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ASSOFB.{0} + ''</CURRENCY>'','''')", AssoFileField.Key.ToString() + "_C");
                                        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ASSOFB.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", AssoFileField.Key.ToString() + "_A");
                                    }
                                    else
                                        BinderFileData.AppendFormat(", ISNULL(ASSOFB.{0},'''')", AssoFileField.Key.ToString());
                                }
                                //BinderFileData.AppendFormat(", ASSOFB.{0}", AssoFileField.Key.ToString());
                            }

                            // Select Columns of File

                            foreach (DictionaryEntry FileField in m_NSearchCommonObj.m_FileFields)
                            {
                                if (FileField.Value.ToString() == "DATETIME")
                                {
                                    BinderFileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSF.{0})) + ''</YEAR>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),1,2) + ''</MONTH>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),4,2) + ''</DAY>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),12,2) + ''</HOUR>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),15,2) + ''</MINUTE>''", FileField.Key.ToString());
                                    BinderFileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),18,2) + ''</SECOND>'','''')", FileField.Key.ToString());
                                }
                                else
                                {
                                    if (FileField.Value.ToString() == "CURRENCY")
                                    {
                                        BinderFileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSF.{0} + ''</CURRENCY>'','''')", FileField.Key.ToString() + "_C");
                                        BinderFileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSF.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FileField.Key.ToString() + "_A");
                                    }
                                    else
                                    {
                                        if (FileField.Key.ToString() == "FIELDNAME")
                                            BinderFileData.AppendFormat(", ISNULL(ALSC.{0},'''')", FileField.Key.ToString());
                                        else
                                            BinderFileData.AppendFormat(", ISNULL(ALFSF.{0},'''')", FileField.Key.ToString());
                                    }
                                }
                                //BinderFileData.AppendFormat(", ALFSF.{0}", FileField.Key.ToString());
                            }

                            // Append nulls till the end and check for Sort Columns.
                            BinderCollectionNumberofNulls = BinderCollectionNumberofNulls + m_NSearchCommonObj.m_BinderFields.Count + 1 + m_NSearchCommonObj.m_AssoFileBinderFields.Count + m_NSearchCommonObj.m_FileFields.Count;
                            if (BinderCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                TempCollectionNumberofNulls = 0;
                                if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                {
                                    TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - BinderCollectionNumberofNulls;
                                    BinderFileData = AddNullstoQuery(TempCollectionNumberofNulls, BinderFileData);
                                    if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                    {
                                        /* Assigning Sort Column values for multiple binder fields: 
                                         * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                         *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                         * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                         *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                         *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                         * This will append FILE nodes to appropriate BINDER nodes. 
                                         * */
                                        if (SortColumnValueForBinderField == null)
                                            SortColumnValueForBinderField = new StringBuilder(20);
                                        else
                                            SortColumnValueForBinderField.Remove(0, SortColumnValueForBinderField.Length);
                                        
                                        //Here sort column values will be generated only for Binder Files
                                        ErrorObj = PrepareSortColumnValues("BINDERFILE", BinderFieldIndex, FSSortPartitionObj, SortedListofColswithValues, ref SortColumnValueForBinderField);
                                        if (ErrorObj != null)
                                            return ErrorObj;

                                        BinderFileData.Append(SortColumnValueForBinderField);
                                    }
                                }
                                else
                                    BinderFileData = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - BinderCollectionNumberofNulls), BinderFileData);
                            }

                            // Append FROM query.
                            BinderFileData.AppendFormat(" FROM ALFS_FILE AS ALFSF INNER JOIN @{0} AS {1} ON ALFSF.OBJECTID ={1}.{2}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            BinderFileData.AppendFormat(" INNER JOIN ALFS_ASSOFILEBINDER AS ASSOFB ON ALFSF.FILEID = ASSOFB.FILEID INNER JOIN ALFS_BINDER AS ALFSB ON ASSOFB.BINDERID = ALFSB.BINDERID AND ALFSF.SCHEMAID = ALFSB.SCHEMAID INNER JOIN {0} AS {1} ", ParentHierarchyObject.m_TableName, EntityTableAlias);
                            BinderFileData.AppendFormat(" ON {0}.{1} = {2}.{3} ", EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName);
                            BinderFileData.AppendFormat(InnerJoinList.ToString());
                            BinderFileData.AppendFormat(" INNER JOIN ALOM_SCHEMA ALSC ON  ALSC.SCHEMAID = ALFSB.SCHEMAID WHERE ALSC.FIELDNAME = ''{0}''AND ALSC.OBJECTTYPEID = {1}", BinderName, ParentHierarchyObject.m_ObjectTypeID);
                            //BinderFileData.AppendFormat("WHERE ALFSF.SCHEMAID IN (SELECT ALSC.SCHEMAID FROM ALOM_SCHEMA AS ALSC WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1} )", BinderName, ParentHierarchyObject.m_ObjectTypeID);

                            REandRFSelectQuery.AppendFormat(BinderFileData.ToString());
                        }
                    }
                }

                #endregion

                #region Preparing Explicit File Query
                // If only File is requested.
                if ((!IsBinderRequested) && (IsFileRequested))
                {
                    TempFileData = new StringBuilder();
                    TempFileData.AppendFormat(FileData.ToString());
                    CollectionNumberofNulls = NumberofNulls;

                    foreach (FieldCollection CustomField in HierarchyObj.CustomFieldCollectionList)
                    {
                        CollectionNumberofNulls = NumberofNulls;
                        FileData.Remove(0, FileData.Length);
                        FileData.AppendFormat(TempFileData.ToString());

                        //FileData.Append(", NULL");
                        //CollectionNumberofNulls++;

                        if (CustomField.m_FieldDataType.ToUpper() == "FILE")
                        {
                            FileFieldName = CustomField.m_FieldName;
                            // Append actual file fields.
                            foreach (DictionaryEntry FileField in m_NSearchCommonObj.m_FileFields)
                            {
                                CollectionNumberofNulls++;
                                if (FileField.Value.ToString() == "DATETIME")
                                {
                                    FileData.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, ALFSF.{0})) + ''</YEAR>''", FileField.Key.ToString());
                                    FileData.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),1,2) + ''</MONTH>''", FileField.Key.ToString());
                                    FileData.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},101),4,2) + ''</DAY>''", FileField.Key.ToString());
                                    FileData.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),12,2) + ''</HOUR>''", FileField.Key.ToString());
                                    FileData.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),15,2) + ''</MINUTE>''", FileField.Key.ToString());
                                    FileData.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), ALFSF.{0},121),18,2) + ''</SECOND>'','''')", FileField.Key.ToString());
                                }
                                else
                                {
                                    if (FileField.Value.ToString() == "CURRENCY")
                                    {
                                        FileData.AppendFormat(",ISNULL(''<CURRENCY>'' + ALFSF.{0} + ''</CURRENCY>'','''')", FileField.Key.ToString() + "_C");
                                        FileData.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST(ALFSF.{0} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", FileField.Key.ToString() + "_A");
                                    }
                                    else
                                    {
                                        if (FileField.Key.ToString() == "FIELDNAME")
                                            FileData.AppendFormat(", ISNULL(ALSC.{0},'''')", FileField.Key.ToString());
                                        else
                                            FileData.AppendFormat(", ISNULL(ALFSF.{0},'''')", FileField.Key.ToString());
                                    }
                                }
                                //FileData.AppendFormat(",ALFSF.{0}",FileField.Key.ToString());
                            }

                            // this will add nulls to Filedata stringbuilder and sort values if present.
                            if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                TempCollectionNumberofNulls = 0;
                                if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                                {
                                    TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                                    FileData = AddNullstoQuery(TempCollectionNumberofNulls, FileData);
                                    if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                                        FileData.Append(SortColumnValues.ToString());
                                }
                                else
                                    FileData = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls), FileData);
                            }

                            // FROM clause.
                            FileData.AppendFormat(" FROM ALFS_FILE AS ALFSF INNER JOIN @{0} AS {1} ON ALFSF.OBJECTID = {1}.{2}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            FileData.AppendFormat(" INNER JOIN {0} AS {1} ON {1}.{2} = {3}.{4} ", ParentHierarchyObject.m_TableName, EntityTableAlias, ParentHierarchyObject.m_PrimaryKeyName, EveryOneTableAlias, ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            //FileData.AppendFormat(" WHERE ALFSF.SCHEMAID IN (SELECT ALSC.SCHEMAID FROM ALOM_SCHEMA AS ALSC WHERE ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1})", FileFieldName,ParentHierarchyObject.m_ObjectTypeID);
                            FileData.AppendFormat(InnerJoinList.ToString());
                            FileData.AppendFormat(" INNER JOIN ALOM_SCHEMA ALSC ON ALSC.SCHEMAID = ALFSF.SCHEMAID WHERE  ALSC.FIELDNAME = ''{0}'' AND ALSC.OBJECTTYPEID = {1} ", FileFieldName, ParentHierarchyObject.m_ObjectTypeID);

                            REandRFSelectQuery.AppendFormat(FileData.ToString());
                            FileData.Remove(0, FileData.Length);
                        }
                    }
                }

                if (dictTableAlias != null)
                    dictTableAlias.Clear();

                #endregion

                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormFileSystemSelectQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <Author>Mr Sagar Dasalkar</Author>
        /// <CreatedDate>3 February 2009</CreatedDate>
        /// <purpose>
        /// Generates alias name considering the situation where there is a possibility of having same alias for taking multiple joins.
        /// This function is specific for the case where select query gets generated for file system.
        /// </purpose>
        /// </summary>
        /// <param name="strTableName">Table name for which alias is to be generated.</param>
        /// <param name="dictTableAlias">Dictionary object to hold table name and alias counter details.</param>
        /// <returns>Returns alias for table.</returns>
        private String PrepareAliasForFileSystemSelectQuery(String strTableName, ref Dictionary<String, int> dictTableAlias)
        {
            bool                b_KeyAlreadyExists              = false;        //Identifies whether key already exists in dictionary

            strTableName = strTableName.ToUpper();

            //If dictionary does not contain table name, add it.
            if (!dictTableAlias.ContainsKey(strTableName))
            {
                dictTableAlias.Add(strTableName, 0);
            }
            else
                b_KeyAlreadyExists = true;

            //If table name already exists, increment the counter and generate the alias with counter appended to it
            if (b_KeyAlreadyExists)
            {
                //Update count and return alias
                dictTableAlias[strTableName] = dictTableAlias[strTableName] + 1;
                return strTableName + "_ALIAS_" + Convert.ToString(dictTableAlias[strTableName]);
            }

            return strTableName + "_ALIAS";
        }

        private OMError FormRESelectQuery(HierarchyObject HierarchyObj, ref List<SortPartition> SortPartitionList, ref StringBuilder REandRFSelectQuery)
        {
            OMError             ReturnErrorObj                  = null;
            OMError             ErrorObj                        = null;
            long                ParentTagID                     = 0;
            int                 TempCounter                     = 0;
            int                 CollectionNumberofNulls         = 0;
            int                 TempCollectionNumberofNulls     = 0;
            int                 NullCounter                     = 0;
            int                 SortValue                       = 0;
            int                 SortValuesCounter               = 0;
            int                 SortColumnCounter               = 0;
            int                 FieldCounter                    = 0;
            int                 IndexforValuetoDecrement        = 0;
            long                AliasID                         = 0;
            SortPartition       RESortPartitionObj              = null;
            SortPartition       ParentSortPartitionObj          = null;
            StringBuilder       RECollectionSelects             = null;
            StringBuilder       SortColumnValues                = null;
            //SortedList      SortedListofColswithValues      = null;
            List<SortPartitionNamedValue> SortedListofColswithValues = null;
            ArrayList           ArrofSortValues                 = null;
            HierarchyObject     TempHierarchyObject             = null;
            HierarchyObject     ParentHierarchyObject           = null;
            StringBuilder       InnerJoinList                   = null;
            String              TableNamewithAlias              = null;



            try
            {
                // Select Query for Collection.
                // get the sort partition object corresponding to RE.
                foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                {
                    if (TempSortPartitionObj.m_NodeID == HierarchyObj.m_TagID)
                    {
                        RESortPartitionObj = TempSortPartitionObj;
                        break;
                    }
                }

                InnerJoinList = new StringBuilder();
                RECollectionSelects = new StringBuilder();
                REandRFSelectQuery.Append(" UNION");
                RECollectionSelects.Append(" UNION");

                // Get Parent actual tagid.
                foreach (HierarchyObject TempHierarchyObject1 in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObject1.m_TagID == HierarchyObj.m_ParentID)
                    {
                        ParentTagID = TempHierarchyObject1.m_ActualTagID;
                        ParentHierarchyObject = TempHierarchyObject1;
                        break;
                    }
                }

                REandRFSelectQuery.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID - 1, ParentTagID);
                RECollectionSelects.AppendFormat(" SELECT {0} AS TAG, {1} AS PARENT", HierarchyObj.m_ActualTagID, HierarchyObj.m_ActualTagID - 1);
                REandRFSelectQuery.Append(", NULL, NULL");
                RECollectionSelects.Append(", NULL, NULL");

                foreach (Predecessor TempPredecessorObj in RESortPartitionObj.PredecessorList)
                {
                    CollectionNumberofNulls = 0;
                    FieldCounter = 0;
                    if (TempPredecessorObj.m_NumberofFields > 0)
                    {
                        // For TE
                        if (TempCounter == 0)
                        {
                            REandRFSelectQuery.AppendFormat(", {0}.{1} ", HierarchyObj.m_EveryOneUserTableName.Trim(), "FILTEREDOBJID");
                            RECollectionSelects.AppendFormat(", {0}.{1} ", HierarchyObj.m_EveryOneUserTableName.Trim(), "FILTEREDOBJID");
                            // Add nulls to select query.
                            if (TempPredecessorObj.m_NumberofFields > 0)
                            {
                                ErrorObj = GetHierarchyObjectfromList(TempPredecessorObj.m_PredecessorID, ref TempHierarchyObject);
                                if (ErrorObj != null)
                                    return ErrorObj;
                                SortColumnCounter = 0;
                                foreach (FieldCollection TempField in TempHierarchyObject.FieldCollectionList)
                                {
                                    // Get the non primary key Fields which are requested for Sorting
                                    // as we need to propogate its values to root level.
                                    if ((!TempField.m_IsPrimaryKey) && TempField.m_IsSortRequested)
                                    {
                                        SortColumnCounter++;
                                        TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS";
                                        // This is for TE
                                        //REandRFSelectQuery.AppendFormat(", {0}.{1}", TempHierarchyObject.m_TableName + "_ALIAS", TempField.m_FieldName);
                                        //RECollectionSelects.AppendFormat(", {0}.{1}", TempHierarchyObject.m_TableName + "_ALIAS", TempField.m_FieldName);
                                        //Switch - case has been used instead of if..else
                                        switch (TempField.m_FieldDataType.ToUpper())
                                        {
                                            case "DATETIME":
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                    REandRFSelectQuery.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            //Case commented by : Mr. Sagar Dasalkar on 19 Nov 09
                                            //Sorting would not be done on CURRENCY field, instead it would be done on an additional column having currency amount.    
                                            //case "CURRENCY":
                                            //    {
                                            //        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                            //        REandRFSelectQuery.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                            //        REandRFSelectQuery.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");
                                            //    }
                                            //    break;

                                                //For Currency Amount
                                            case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);

                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "BIGINT":
                                            case "INTEGER":
                                                {
                                                    RECollectionSelects.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            case "FLOAT":
                                                {
                                                    RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;

                                            default:
                                                {
                                                    RECollectionSelects.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                    REandRFSelectQuery.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                }
                                                break;
                                        }
                                    }
                                    else
                                    {
                                        // Add a null to Collection and select query.
                                        if (!TempField.m_IsPrimaryKey)
                                        {
                                            REandRFSelectQuery.AppendFormat(", NULL");
                                            RECollectionSelects.AppendFormat(", NULL");
                                        }
                                    }
                                }

                                if (SortColumnCounter > 0)
                                {
                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} on {1}.{2} = {3}.FILTEREDOBJID ", TempHierarchyObject.m_TableName, TempHierarchyObject.m_TableName + "_ALIAS", TempHierarchyObject.m_PrimaryKeyName, HierarchyObj.m_EveryOneUserTableName.Trim());
                                }

                                // 4 because Tag,Parent,TECollection & TEData node.
                                NullCounter = 4 + TempPredecessorObj.m_NumberofFields;
                            }
                            CollectionNumberofNulls = TempPredecessorObj.m_StartColumnNumber + (TempPredecessorObj.m_NumberofFields - 1);
                        }
                        else
                        {
                            if (NullCounter < TempPredecessorObj.m_StartColumnNumber)
                            {
                                RECollectionSelects = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), RECollectionSelects);
                                REandRFSelectQuery = AddNullstoQuery(((TempPredecessorObj.m_StartColumnNumber - 1) - NullCounter), REandRFSelectQuery);
                            }
                            REandRFSelectQuery.AppendFormat(", {0}.{1}", HierarchyObj.m_EveryOneUserTableName.Trim(), TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());
                            RECollectionSelects.AppendFormat(", {0}.{1}", HierarchyObj.m_EveryOneUserTableName.Trim(), TempPredecessorObj.m_RelationshipName + TempPredecessorObj.m_RelatedObjectLevel.ToString());

                            // Code for Sorting

                            ErrorObj = GetHierarchyObjectfromList(TempPredecessorObj.m_PredecessorID, ref TempHierarchyObject);
                            if (ErrorObj != null)
                                return ErrorObj;
                            SortColumnCounter = 0;

                            //****This code is commented by Kiran Abhang on 30th September 2011.
                            //Instead of generating unique ID from GUID, we are now using a sequential ID.
                            //This unique ID is used for temporary table variable names and for the field names from such tables.
                            ////UniqueID.Read(ref AliasID);
                            
                            //New Code.
                            GetUniqueId(ref AliasID);

                            foreach (FieldCollection TempField in TempHierarchyObject.FieldCollectionList)
                            {
                                // Get the non primary key Fields which are requested for Sorting
                                // as we need to propogate its values to root level.
                                if ((!TempField.m_IsPrimaryKey) && TempField.m_IsSortRequested)
                                {
                                    SortColumnCounter++;
                                    FieldCounter++;
                                    TableNamewithAlias = TempHierarchyObject.m_TableName + "_ALIAS" + AliasID.ToString();
                                    /*REandRFSelectQuery.AppendFormat(", {0}.{1}", TempHierarchyObject.m_TableName + "_ALIAS", TempField.m_FieldName);
                                    RECollectionSelects.AppendFormat(", {0}.{1}", TempHierarchyObject.m_TableName + "_ALIAS", TempField.m_FieldName);*/
                                    //Switch - case has been used instead of if..else
                                    switch (TempField.m_FieldDataType.ToUpper())
                                    {
                                        case "DATETIME":
                                            {
                                                RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);

                                                REandRFSelectQuery.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, {0}.{1})) + ''</YEAR>''", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),1,2) + ''</MONTH>''", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},101),4,2) + ''</DAY>''", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),12,2) + ''</HOUR>''", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),15,2) + ''</MINUTE>''", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), {0}.{1},121),18,2) + ''</SECOND>'','''')", TableNamewithAlias, TempField.m_FieldName);
                                            }

                                            break;

                                        //Case commented by : Mr. Sagar Dasalkar on 19 Nov 09
                                        //Sorting would not be done on CURRENCY field, instead it would be done on an additional column having currency amount.    
                                        //case "CURRENCY":
                                        //    {
                                        //        RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                        //        RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                        //        REandRFSelectQuery.AppendFormat(",ISNULL(''<CURRENCY>'' + {0}.{1} + ''</CURRENCY>'','''')", TableNamewithAlias, TempField.m_FieldName + "_C");
                                        //        REandRFSelectQuery.AppendFormat(" + ISNULL(''<AMOUNT>'' + CAST({0}.{1} AS DECIMAL(30,4))+ ''</AMOUNT>'','''')", TableNamewithAlias, TempField.m_FieldName + "_A");

                                        //    }
                                        //    break;

                                            //For currency amount
                                        case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                                            {
                                                RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);

                                                REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                            }
                                            break;


                                        case "BIGINT":
                                        case "INTEGER":
                                            {
                                                RECollectionSelects.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(", {0}.{1}", TableNamewithAlias, TempField.m_FieldName);
                                            }
                                            break;

                                        case "FLOAT":
                                            {
                                                RECollectionSelects.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(", CAST({0}.{1} AS DECIMAL(30,4))", TableNamewithAlias, TempField.m_FieldName);
                                            }
                                            break;

                                        default:
                                            {
                                                RECollectionSelects.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                                REandRFSelectQuery.AppendFormat(", ISNULL({0}.{1},'''')", TableNamewithAlias, TempField.m_FieldName);
                                            }
                                            break;

                                    }
                                }
                                else
                                {
                                    if (!TempField.m_IsPrimaryKey)
                                    {
                                        FieldCounter++;
                                        RECollectionSelects.AppendFormat(", NULL");
                                        REandRFSelectQuery.AppendFormat(", NULL");
                                    }
                                }
                            }

                            if (FieldCounter < TempPredecessorObj.m_NumberofFields)
                            {
                                RECollectionSelects = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter), RECollectionSelects);
                                REandRFSelectQuery = AddNullstoQuery((TempPredecessorObj.m_NumberofFields - FieldCounter), REandRFSelectQuery);
                            }
                            if (SortColumnCounter > 0)
                            {
                                InnerJoinList.AppendFormat(" INNER JOIN {0} {5} on {5}.{1} = {2}.{3}{4} ", TempHierarchyObject.m_TableName, TempHierarchyObject.m_PrimaryKeyName, HierarchyObj.m_EveryOneUserTableName.Trim(), TempHierarchyObject.m_RelationshipName, TempHierarchyObject.m_RelatedObjectLevel, TableNamewithAlias);
                            }


                            // Code for Sorting ends


                            //if (TempPredecessorObj.m_NumberofFields > 0)
                            //{
                            //    RECollectionSelects = AddNullstoQuery(TempPredecessorObj.m_NumberofFields, RECollectionSelects);
                            //    REandRFSelectQuery = AddNullstoQuery(TempPredecessorObj.m_NumberofFields, REandRFSelectQuery);
                            //}
                            NullCounter = TempPredecessorObj.m_StartColumnNumber + TempPredecessorObj.m_NumberofFields;
                            CollectionNumberofNulls = TempPredecessorObj.m_StartColumnNumber + (TempPredecessorObj.m_NumberofFields);
                        }
                    }
                    TempCounter++;
                }

                if (CollectionNumberofNulls < RESortPartitionObj.m_StartColumnNumber)
                {
                    TempCollectionNumberofNulls = 0;
                    TempCollectionNumberofNulls = (RESortPartitionObj.m_StartColumnNumber - 1) - CollectionNumberofNulls;
                    CollectionNumberofNulls = CollectionNumberofNulls + TempCollectionNumberofNulls;

                    REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                    RECollectionSelects = AddNullstoQuery(TempCollectionNumberofNulls, RECollectionSelects);
                }
                // Add the values for SortColumns.

                // Check if the sortColumn for this object is there in SortColumnList
                if (m_NSearchCommonObj.m_SortPartitionColumnsList.ContainsValue(RESortPartitionObj.m_SortPartitionColumnName))
                {
                    // get the sort partition object corresponding to RE's Parent .
                    foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                    {
                        if (TempSortPartitionObj.m_NodeID == RESortPartitionObj.m_ParentID)
                        {
                            ParentSortPartitionObj = TempSortPartitionObj;
                            break;
                        }
                    }
                    SortedListofColswithValues = null;
                    SortedListofColswithValues = new List<SortPartitionNamedValue>();
                    ErrorObj = GetSortPartitionValues(RESortPartitionObj.m_SortPartitionID, RESortPartitionObj.m_SortPartitionColumnName, ParentSortPartitionObj, ref SortedListofColswithValues);
                    if (ErrorObj != null)
                        return ErrorObj;

                    if (SortedListofColswithValues.Count > 0)
                    {
                        ArrofSortValues = new ArrayList();
                        ErrorObj = GetSortValuesSortedonSortColumns(ref SortedListofColswithValues, ref ArrofSortValues, ref IndexforValuetoDecrement);
                        if (ErrorObj != null)
                            return ErrorObj;
                    }


                }

                if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                {
                    TempCollectionNumberofNulls = 0;
                    if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                    {
                        SortColumnValues = new StringBuilder();
                        TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                        if (CollectionNumberofNulls > 0)
                            REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                        // Add Sort Partition Values.
                        TempCollectionNumberofNulls = TempCollectionNumberofNulls + CollectionNumberofNulls;
                        //TempCollectionNumberofNulls = CollectionNumberofNulls + m_NSearchCommonObj.m_SortPartitionColumnsList.Count;
                        if (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                        {
                            SortValuesCounter = 0;
                            foreach (String SortColValue in ArrofSortValues)
                            {
                                TempCollectionNumberofNulls++;

                                if (SortValuesCounter == IndexforValuetoDecrement)
                                {
                                    SortValue = Convert.ToInt32(SortColValue);
                                    SortValue--;
                                    REandRFSelectQuery.AppendFormat(", {0}", SortValue.ToString());
                                    SortValue++;
                                    SortColumnValues.AppendFormat(", {0}", SortValue.ToString());
                                }
                                else
                                {
                                    SortColumnValues.AppendFormat(", {0}", SortColValue);
                                    REandRFSelectQuery.AppendFormat(", {0}", SortColValue);
                                }

                                //else
                                //{
                                //    if (SortValuesCounter == ArrofSortValues.Count)
                                //    {
                                //        SortValue = Convert.ToInt32(SortColValue);
                                //        SortValue = --SortValue;
                                //        REandRFSelectQuery.AppendFormat(",''{0}''", SortValue.ToString());
                                //        SortValue = ++SortValue;
                                //        SortColumnValues.AppendFormat(",''{0}''", SortValue.ToString());
                                //    }
                                //    else
                                //    {
                                //        SortValue = Convert.ToInt32(SortColValue);
                                //        REandRFSelectQuery.AppendFormat(",''{0}''", SortValue.ToString());
                                //        SortColumnValues.AppendFormat(",''{0}''", SortValue.ToString());
                                //    }
                                //}
                                SortValuesCounter++;
                            }
                            if (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                            {
                                while (TempCollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                                {
                                    REandRFSelectQuery.Append(", NULL");
                                    SortColumnValues.Append(", NULL");
                                    TempCollectionNumberofNulls++;
                                }
                            }
                        }
                    }
                    else
                    {
                        TempCollectionNumberofNulls = m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls;
                        if (CollectionNumberofNulls > 0)
                            REandRFSelectQuery = AddNullstoQuery(TempCollectionNumberofNulls, REandRFSelectQuery);
                    }
                }

                // Select the actual fields of Related Entity.

                foreach (FieldCollection FieldObj in HierarchyObj.FieldCollectionList)
                {
                    CollectionNumberofNulls++;
                    //Switch - case has been used instead of if..else
                    switch (FieldObj.m_FieldDataType.ToUpper())
                    {
                        case "DATETIME":
                            {
                                RECollectionSelects.AppendFormat(", ISNULL(''<YEAR>'' + CONVERT(VARCHAR(22), DATEPART(YEAR, RE.{0})) + ''</YEAR>''", FieldObj.m_FieldName);
                                RECollectionSelects.AppendFormat(" + ''<MONTH>'' + SUBSTRING(CONVERT(VARCHAR(22), RE.{0},101),1,2) + ''</MONTH>''", FieldObj.m_FieldName);
                                RECollectionSelects.AppendFormat(" + ''<DAY>'' + SUBSTRING(CONVERT(VARCHAR(22), RE.{0},101),4,2)  + ''</DAY>''", FieldObj.m_FieldName);
                                RECollectionSelects.AppendFormat(" + ''<HOUR>'' + SUBSTRING(CONVERT(VARCHAR(22), RE.{0},121),12,2) + ''</HOUR>''", FieldObj.m_FieldName);
                                RECollectionSelects.AppendFormat(" + ''<MINUTE>'' + SUBSTRING(CONVERT(VARCHAR(22), RE.{0},121),15,2) + ''</MINUTE>''", FieldObj.m_FieldName);
                                RECollectionSelects.AppendFormat(" + ''<SECOND>'' + SUBSTRING(CONVERT(VARCHAR(22), RE.{0},121),18,2) + ''</SECOND>'','''')", FieldObj.m_FieldName);
                            }
                            break;

                        case "CURRENCY":
                            {
                                RECollectionSelects.AppendFormat(",ISNULL(''<CURRENCY>'' + RE.{0} + ''</CURRENCY>'','''')", FieldObj.m_FieldName + "_C");
                                RECollectionSelects.AppendFormat(" + ISNULL(''<AMOUNT>'' + RTRIM(CAST(RE.{0} AS DECIMAL(30,4))) + ''</AMOUNT>'','''')", FieldObj.m_FieldName + "_A");
                            }
                            break;

                        case OMInternalDataTypeForCurrency.CURRENY_AMOUNT:
                            {
                                RECollectionSelects.AppendFormat(",CAST(RE.{0} AS DECIMAL(30,4))", FieldObj.m_FieldName);
                            }
                            break;

                        case "FLOAT":
                            RECollectionSelects.AppendFormat(", CAST(RE.{0} AS DECIMAL(30,4))", FieldObj.m_FieldName);
                            break;

                        case "BIGINT":
                        case "INTEGER":
                            RECollectionSelects.AppendFormat(", RE.{0}", FieldObj.m_FieldName);
                            break;

                        default:
                            RECollectionSelects.AppendFormat(", ISNULL(RE.{0},'''')", FieldObj.m_FieldName);
                            break;
                    }
                }

                if (CollectionNumberofNulls < m_NSearchCommonObj.m_TotalNumberOfFields)
                {
                    TempCollectionNumberofNulls = 0;
                    if ((SortedListofColswithValues != null) && (SortedListofColswithValues.Count > 0))
                    {
                        TempCollectionNumberofNulls = (m_NSearchCommonObj.m_TotalNumberOfFields - m_NSearchCommonObj.m_SortPartitionColumnsList.Count) - CollectionNumberofNulls;
                        RECollectionSelects = AddNullstoQuery(TempCollectionNumberofNulls, RECollectionSelects);
                        if (m_NSearchCommonObj.m_SortPartitionColumnsList.Count > 0)
                            RECollectionSelects.Append(SortColumnValues.ToString());
                    }
                    else
                        RECollectionSelects = AddNullstoQuery((m_NSearchCommonObj.m_TotalNumberOfFields - CollectionNumberofNulls), RECollectionSelects);
                }
                REandRFSelectQuery.AppendFormat(" FROM {0} AS RE INNER JOIN {1} AS {2} ON RE.{3} = {4}.{5}", HierarchyObj.m_TableName, "@" + HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_PrimaryKeyforTableVariable);
                REandRFSelectQuery.AppendFormat(InnerJoinList.ToString());
                RECollectionSelects.AppendFormat(" FROM {0} AS RE INNER JOIN {1} AS {2} ON RE.{3} = {4}.{5}", HierarchyObj.m_TableName, "@" + HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_EveryOneUserTableName.Trim(), HierarchyObj.m_PrimaryKeyforTableVariable);
                RECollectionSelects.AppendFormat(InnerJoinList.ToString());

                REandRFSelectQuery.Append(RECollectionSelects.ToString());

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormRESelectQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// This function will return HierarchyObject from HierarchyObjectList
        /// </summary>
        /// <param name="TagID">Unique ID for that HierarchyObject</param>
        /// <param name="ParentID">ParentID for that HierarchyObject</param>
        /// <param name="TempHierarchyObject">ref HierarchyObject</param>
        /// <returns>NULL if successful else OMERROR Object</returns>
        private OMError GetHierarchyObjectfromList(long TagID, ref HierarchyObject TempHierarchyObject)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;

            try
            {
                foreach (HierarchyObject HObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (HObj.m_TagID == TagID)
                    {
                        TempHierarchyObject = HObj;
                        break;
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetHierarchyObjectfromList",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// This is a recursive function called which returns a hashtable which contains SortPartitionColName with its value.
        /// </summary>
        /// <param name="SortPartitionID">SortPartitionID which is to be searched in sibling List of Parent SortPartition Object</param>
        /// <param name="SortPartitionColName">SortPartition Column Name</param>
        /// <param name="ParentSortPartitionObj">ParentSortPartition Object</param>
        /// <param name="SortedListofColswithValues">A Hash Table of Columns with Sort Values</param>
        /// <param name="SortPartitionList">SortPartitionList which contains all the list of SortPartition Objects</param>
        /// <returns></returns>
        private OMError GetSortPartitionValues(long SortPartitionID, String SortPartitionColName, SortPartition ParentSortPartitionObj, ref List<SortPartitionNamedValue> SortedListofColswithValues)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            SortPartition NewParentSortPartitionObj = null;

            try
            {

                foreach (Sibling TempSibling in ParentSortPartitionObj.SiblingList)
                {
                    if (TempSibling.m_SortPartitionID == SortPartitionID)
                    {
                        SortPartitionNamedValue SPItem = new SortPartitionNamedValue();
                        SPItem.SortPartitionName = SortPartitionColName;
                        SPItem.SortPartitionValue = TempSibling.m_SortValue.ToString();
                        SortedListofColswithValues.Add(SPItem);
                        //SortedListofColswithValues.Add(SortPartitionColName,TempSibling.m_SortValue);
                        break;
                    }
                }

                // Get the parent of passed Sort Partition Object.
                foreach (SortPartition TempSortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                {
                    if (TempSortPartitionObj.m_NodeID == ParentSortPartitionObj.m_ParentID)
                    {
                        NewParentSortPartitionObj = TempSortPartitionObj;
                        if (m_NSearchCommonObj.m_SortPartitionColumnsList.ContainsValue(ParentSortPartitionObj.m_SortPartitionColumnName))
                        {
                            ErrorObj = GetSortPartitionValues(ParentSortPartitionObj.m_SortPartitionID, ParentSortPartitionObj.m_SortPartitionColumnName, NewParentSortPartitionObj, ref SortedListofColswithValues);
                            if (ErrorObj != null)
                                return ErrorObj;
                        }
                        break;
                    }
                }

                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetSortPartitionValues",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                return ReturnErrorObj;
            }

        }

        /// <summary>
        /// this function arranges Sort Values accoring to Sort Partition Columns
        /// </summary>
        /// <param name="SortedListofColswithValues"></param>
        /// <param name="ArrofSortValues"></param>
        /// <returns></returns>
        private OMError GetSortValuesSortedonSortColumns(ref List<SortPartitionNamedValue> SortedListofColswithValues, ref ArrayList ArrofSortValues, ref int IndexforValuetoDecrement)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            int SortPartitionCounter = 0;
            bool IsSortColinValueList = false;
            String IndexColName = null;

            try
            {
                // This will get index at which the sort Value has to be decremented.
                // i.e relationship at which sort values are processed.

                foreach (SortPartitionNamedValue ActualSPColwithValue in SortedListofColswithValues)
                {
                    IndexColName = ActualSPColwithValue.SortPartitionName;
                    break;
                }

                foreach (DictionaryEntry SPCol in m_NSearchCommonObj.m_SortPartitionColumnsList)
                {

                    if (SPCol.Value.ToString() == IndexColName)
                        IndexforValuetoDecrement = SortPartitionCounter;
                    SortPartitionCounter++;
                }

                foreach (DictionaryEntry SPCol in m_NSearchCommonObj.m_SortPartitionColumnsList)
                {

                    IsSortColinValueList = false;
                    foreach (SortPartitionNamedValue ActualSPColwithValue in SortedListofColswithValues)
                    {
                        if (SPCol.Value.ToString() == ActualSPColwithValue.SortPartitionName)
                        {
                            IsSortColinValueList = true;
                            ArrofSortValues.Add(ActualSPColwithValue.SortPartitionValue);
                            break;
                        }

                    }
                    if (!IsSortColinValueList)
                    {
                        ArrofSortValues.Add("NULL");
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetSortValuesSortedonSortColumns",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private StringBuilder AddNullstoQuery(int NumberofNulls, StringBuilder SelectQuery)
        {
            int TempCounter = 0;
            for (TempCounter = 0; TempCounter < NumberofNulls; TempCounter++)
            {
                SelectQuery.Append(", NULL");
            }
            return SelectQuery;
        }

        private OMError AddSortColumnstoStructureQuery(ref List<SortPartition> SortPartitionList, ref StringBuilder TEXMLStructQuery)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            long[] ProcessedParentIDList = new long[SortPartitionList.Count];
            String StructureName = null;

            try
            {
                m_NSearchCommonObj.m_StructureSortPartitionList = new SortedList();

                foreach (SortPartition SortPartitionObj in m_NSearchCommonObj.SortPartitionList)
                {
                    if (SortPartitionObj.SiblingList.Count >= 1)
                    {
                        SortPartitionColumnTagIDPair SPColTagIDObj = new SortPartitionColumnTagIDPair();

                        m_NSearchCommonObj.m_TotalNumberOfFields++;
                        //m_NSearchCommonObj.m_SortPartitionColumnsList.Add(m_NSearchCommonObj.m_TotalNumberOfFields, "SPCol" + SortPartitionObj.m_NodeID.ToString());
                        m_NSearchCommonObj.m_SortPartitionColumnsList.Add(m_NSearchCommonObj.m_TotalNumberOfFields, "SP" + SortPartitionObj.m_NodeID.ToString());
                        // This is done in order to preserve the SortSequence.
                        //SPColTagIDObj.SortPartitionColumnName = "SPCol" + SortPartitionObj.m_NodeID.ToString();
                        SPColTagIDObj.SortPartitionColumnName = "SP" + SortPartitionObj.m_NodeID.ToString();
                        SPColTagIDObj.TagID = SortPartitionObj.m_NodeID;
                        SPColTagIDObj.ParentID = SortPartitionObj.m_ParentID;
                        m_NSearchCommonObj.m_SPColTagIDList.Add(SPColTagIDObj);
                    }
                }

                foreach (DictionaryEntry SortPartCol in m_NSearchCommonObj.m_SortPartitionColumnsList)
                {
                    m_NSearchCommonObj.m_ParentNodeNumber++;
                    StructureName = null;
                    StructureName = "[SORTPARTITION!" + m_NSearchCommonObj.m_ParentNodeNumber.ToString() + "!" + SortPartCol.Value.ToString() + "!ELEMENT]";
                    TEXMLStructQuery.AppendFormat(", CAST(NULL AS INT) AS [SORTPARTITION!{0}!{1}!ELEMENT]", m_NSearchCommonObj.m_ParentNodeNumber, SortPartCol.Value.ToString());
                    m_NSearchCommonObj.m_StructureSortPartitionList.Add(SortPartCol.Value, StructureName);
                    foreach (SortPartitionColumnTagIDPair TempObj in m_NSearchCommonObj.m_SPColTagIDList)
                    {
                        if (TempObj.SortPartitionColumnName == SortPartCol.Value.ToString())
                        {
                            TempObj.SortPartitionStructColumnName = StructureName;
                            break;
                        }
                    }
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in AddSortColumnstoStructureQuery",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError GetFilterClause(HierarchyObject HierarchyObj, ref String ReturnFilterClause)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            ArrayList ProcessedRelationships = null;
            XmlNode ObjectTypeNode = null;
            XmlNode RelationshipNode = null;
            XmlNode PrimaryKeyNode = null;
            StringBuilder InnerJoinList = null;
            StringBuilder FilterClause = null;
            StringBuilder WhereClause = null;
            SortedList ClauseCollection = null;
            ulong RELATIONSHIPTYPEID = 0;
            ulong CONTAINERTYPEID = 0;
            ulong CONTAINSTYPEID = 0;
            ulong RelatedTypeID = 0;
            long TempUniqueID = 0;
            Boolean IsInnerJoinClauseAdded = false;
            Boolean IsCyclicRelation = false;
            String RelatedType = null;
            String RelatedTypeName = null;
            String RelatedObjectTypeName = null;
            String RelatedTypeAlias = null;
            String AssemblyColumnName = null;
            String RelatedTypePKName = null;
            String CONTAINSTYPE = null;
            String CONTAINERTYPE = null;
            String AssemblyAliasName = null;
            String AssoAliasName = null;
            List<RelationshipCollection> RelationshipCollectionListObj = null;
            RelationshipCollection RelationshipCollectionObj = null;
            bool b_IsRelationship_RF_RE = false;                // Specifies whether Entity instance data is expected below Facet instance data within Entity-Facet relationship 
            bool b_IsRelationship_RF_RF = false;                // Specifies whether relationship specified is between Facet-Facet
            HierarchyObject obj_ParentHierarchyObject = null;   // Used to hold parent hierarchy object for a hierarchy object

            try
            {
                // Check if Hierarchy Object has any Filterlist given.
                WhereClause = new StringBuilder();

                if (HierarchyObj.FilterCollectionList.Count > 0)
                {
                    // Get ObjectType Node.
                    InnerJoinList = new StringBuilder();
                    if (HierarchyObj.m_FilterCombination != null && HierarchyObj.m_FilterCombination != "" && HierarchyObj.FilterCollectionList.Count > 1)
                    {
                        ErrorObj = GetFilterClauseonFilterCombination(ref HierarchyObj, ref WhereClause);
                        if (ErrorObj != null)
                            return ErrorObj;
                    }
                    else
                    {
                        ClauseCollection = new SortedList();
                        ProcessedRelationships = new ArrayList();
                        RelationshipCollectionListObj = new List<RelationshipCollection>();
                        foreach (FilterCollection TempFilterObj in HierarchyObj.FilterCollectionList)
                        {
                            if (TempFilterObj.m_RelationshipName != null && TempFilterObj.m_RelationshipName != "")
                            {
                                if (!ProcessedRelationships.Contains(TempFilterObj.m_RelationshipName))
                                    ProcessedRelationships.Add(TempFilterObj.m_RelationshipName);
                            }
                        }

                        //Get Parent Hierarchy Object
                        ErrorObj = GetParentHierarchyObject(HierarchyObj, ref obj_ParentHierarchyObject);
                        if (ErrorObj != null)
                            return ErrorObj;

                        FilterClause = new StringBuilder();

                        foreach (String TempRelationshipName in ProcessedRelationships)
                        {

                            // Read Relation Information
                            RelationshipNode = null;
                            IsCyclicRelation = false;
                            b_IsRelationship_RF_RE = false;
                            b_IsRelationship_RF_RF = false;

                            if (ReadByRelationShipNameEx(TempRelationshipName, ref RelationshipNode) != null)
                            {
                                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                                                    ErrorObj.ErrorMessage, ErrorObj.ObjectType, ErrorObj.ObjectTypeName,
                                                    ErrorObj.ErrorType, ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                                return ReturnErrorObj;
                            }
                            // Resolve EntityType or Facet Type.

                            RELATIONSHIPTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("RELATIONSHIPTYPEID").InnerText.ToString());
                            CONTAINERTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINERTYPEID").InnerText.ToString());
                            CONTAINSTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINSTYPEID").InnerText.ToString());
                            CONTAINERTYPE = RelationshipNode.SelectSingleNode("CONTAINERTYPE").InnerText.ToString();
                            CONTAINSTYPE = RelationshipNode.SelectSingleNode("CONTAINSTYPE").InnerText.ToString();


                            // Processing for Relationship where ContainerTypeID,ContainsTypeID and ObjectTypeID are same.
                            if (CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "ENTITY")
                            {
                                if ((HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID) && (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID))
                                    IsCyclicRelation = true;

                                if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                                {
                                    RelatedTypeID = CONTAINSTYPEID;
                                    RelatedType = "ENTITY";
                                    AssemblyColumnName = "CONTAINSID";
                                }
                                else
                                {
                                    if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                                    {
                                        RelatedTypeID = CONTAINERTYPEID;
                                        RelatedType = "ENTITY";
                                        AssemblyColumnName = "CONTAINERID";
                                    }
                                }
                                RelatedTypeName = "RELATEDENTITY";

                                RelationshipCollectionObj = new RelationshipCollection();
                                RelationshipCollectionObj.m_RelationshionTypeId = RELATIONSHIPTYPEID;
                                RelationshipCollectionObj.m_RelationshipName = TempRelationshipName;
                                RelationshipCollectionListObj.Add(RelationshipCollectionObj);
                                RelationshipCollectionObj = null;
                            }
                            else if (CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "FACET")
                            {
                                if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                                {
                                    RelatedTypeID = CONTAINSTYPEID;
                                    // This is because Facet cannot be container for Entity.
                                    RelatedType = "FACET";
                                }
                                else
                                {
                                    if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                                    {
                                        RelatedTypeID = CONTAINERTYPEID;
                                        RelatedType = "ENTITY";
                                        b_IsRelationship_RF_RE = true;
                                    }
                                }
                                RelatedTypeName = "RELATEDFACET";
                            }
                            else if (CONTAINERTYPE.ToUpper() == "FACET" && CONTAINSTYPE.ToUpper() == "FACET")
                            {
                                if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                                {
                                    RelatedTypeID = CONTAINSTYPEID;
                                    AssemblyColumnName = "CONTAINSID";
                                }
                                else if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                                {
                                    RelatedTypeID = CONTAINERTYPEID;
                                    AssemblyColumnName = "CONTAINERID";
                                }

                                RelatedType = "FACET";
                                RelatedTypeName = "RELATEDFACET";
                                b_IsRelationship_RF_RF = true;

                                RelationshipCollectionObj = new RelationshipCollection();
                                RelationshipCollectionObj.m_RelationshionTypeId = RELATIONSHIPTYPEID;
                                RelationshipCollectionObj.m_RelationshipName = TempRelationshipName;
                                RelationshipCollectionListObj.Add(RelationshipCollectionObj);
                                RelationshipCollectionObj = null;

                            }

                            // Get ObjectType Details.
                            if (ReadTypeDetails(RelatedType, RelatedTypeID, ref ObjectTypeNode) != null)
                            {
                                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                                                ErrorObj.ErrorMessage, ErrorObj.ObjectType, ErrorObj.ObjectTypeName,
                                                ErrorObj.ErrorType, ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                                return ReturnErrorObj;
                            }
                            RelatedObjectTypeName = ObjectTypeNode.SelectSingleNode("TABLENAME").InnerText;
                            PrimaryKeyNode = ObjectTypeNode.SelectSingleNode("//FIELDCOLLECTION/FIELD[ISPRIMARYKEY='TRUE']");
                            RelatedTypePKName = PrimaryKeyNode.SelectSingleNode("FIELDNAME").InnerText;

                            // Add Inner Join Clause.
                            IsInnerJoinClauseAdded = false;

                            //****This code is commented by Kiran Abhang on 30th September 2011.
                            //Instead of generating unique ID from GUID, we are now using a sequential ID.
                            //This unique ID is used for temporary table variable names and for the field names from such tables.
                            //Assetlink.UniqueID.Read(ref TempUniqueID);

                            //New Code.
                            GetUniqueId(ref TempUniqueID);

                            // Iterate FilterCollection for matching relationships.
                            foreach (FilterCollection TempFilterObj in HierarchyObj.FilterCollectionList)
                            {
                                if (TempFilterObj.m_RelationshipName == TempRelationshipName)
                                {
                                    FilterClause = new StringBuilder();

                                    if (!IsInnerJoinClauseAdded)
                                    {
                                        IsInnerJoinClauseAdded = true;

                                        //RelatedTypeAlias = RelatedObjectTypeName + TempFilterObj.m_FilterID.ToString();
                                        RelatedTypeAlias = RelatedObjectTypeName + TempUniqueID;
                                        AssemblyAliasName = "ASSEMBLY" + TempUniqueID;

                                        if (RelatedTypeName == "RELATEDENTITY")
                                        {
                                            if (HierarchyObj.m_ParentID == 0)
                                            {
                                                if (TempFilterObj.m_IsParent == "TOPDOWN" && (IsCyclicRelation))
                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                else
                                                {
                                                    if (TempFilterObj.m_IsParent == "BOTTOMUP" && (IsCyclicRelation))
                                                        InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                    else
                                                    {
                                                        if (AssemblyColumnName == "CONTAINSID")
                                                            InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                        else
                                                            InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                    }
                                                }
                                            }
                                            else
                                            {
                                                if ((TempFilterObj.m_IsParent == "BOTTOMUP") && (IsCyclicRelation))
                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                if ((TempFilterObj.m_IsParent == "TOPDOWN") && (IsCyclicRelation))
                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = TE.{1} ", AssemblyAliasName, HierarchyObj.m_PrimaryKeyName);
                                                else
                                                {
                                                    //Identify the Parent Hierarchy Object Type and prepare query accordingly
                                                    switch (obj_ParentHierarchyObject.m_ObjectType.ToUpper())
                                                    {
                                                        /* This case specifies that parent is ENTITY (hierarchy object type), 
                                                         * and the previous join has been taken on ASSEMBLY table between Parent hierarchy object
                                                         * and current hierarchy object, so while taking join on ALOM_ASSEMBLY for filter on related object,
                                                         * consider ASSEMBLY.CONTAINERID OR ASSEMBLY.CONTAINSID
                                                         * */
                                                        case "ENTITY":
                                                            if (AssemblyColumnName == "CONTAINSID")
                                                                //InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON  ASSEMBLY{0}.CONTAINERID = {1}.{2} ", TempFilterObj.m_FilterID, HierarchyObj.m_EveryOneUserTableName.Trim() , HierarchyObj.m_PrimaryKeyforTableVariable);
                                                                InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = {1}.{2} ", AssemblyAliasName, "ASSEMBLY", HierarchyObj.m_ContainerName);
                                                            else
                                                                //InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON  ASSEMBLY{0}.CONTAINSID = {1}.{2} ", TempFilterObj.m_FilterID, HierarchyObj.m_EveryOneUserTableName.Trim() , HierarchyObj.m_PrimaryKeyforTableVariable);
                                                                InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = {1}.{2} ", AssemblyAliasName, "ASSEMBLY", HierarchyObj.m_ContainerName);

                                                            break;

                                                        /* This case specifies that parent is FACET (hierarchy object type), 
                                                         * and the previous join has been taken on ALOM_ASSOENTITYFACET table between Parent hierarchy object
                                                         * and current hierarchy object, so while taking join on ALOM_ASSEMBLY for filter on related object,
                                                         * consider ASSOENTFCT.ENTITYID
                                                         * */
                                                        case "FACET":

                                                            if (AssemblyColumnName == "CONTAINSID")
                                                                InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = {1}.{2} ", AssemblyAliasName, "ASSOENTFCT", "ENTITYID");
                                                            else
                                                                InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = {1}.{2} ", AssemblyAliasName, "ASSOENTFCT", "ENTITYID");

                                                            break;
                                                    }
                                                }
                                            }

                                            //InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON ASSEMBLY{2}.{3} = {4}.{5}", RelatedObjectTypeName, RelatedTypeAlias, TempFilterObj.m_FilterID, AssemblyColumnName, RelatedTypeAlias, RelatedTypePKName);
                                            if (TempFilterObj.m_IsParent == "TOPDOWN" && (IsCyclicRelation))
                                                InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.CONTAINERID = {3}.{4}", RelatedObjectTypeName, RelatedTypeAlias, AssemblyAliasName, RelatedTypeAlias, RelatedTypePKName);
                                            else
                                                if (TempFilterObj.m_IsParent == "BOTTOMUP" && (IsCyclicRelation))
                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.CONTAINSID = {3}.{4}", RelatedObjectTypeName, RelatedTypeAlias, AssemblyAliasName, RelatedTypeAlias, RelatedTypePKName);
                                                else
                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.{3} = {4}.{5}", RelatedObjectTypeName, RelatedTypeAlias, AssemblyAliasName, AssemblyColumnName, RelatedTypeAlias, RelatedTypePKName);

                                            //Add AssemblyAliasName in RelationshipCollectionListObj
                                            AddAssemblyAliasName(RelationshipCollectionListObj, TempRelationshipName, AssemblyAliasName);

                                        }
                                        else
                                        {
                                            if (RelatedTypeName == "RELATEDFACET")
                                            {
                                                AssoAliasName = "ASSOENTFCT" + TempUniqueID;
                                                if (HierarchyObj.m_ParentID == 0)
                                                {
                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET {0} ON {0}.ENTITYID = TE.{1} AND ({0}.RELATIONSHIPTYPEID = {2} OR {0}.RELATIONSHIPTYPEID IS NULL)", AssoAliasName, HierarchyObj.m_PrimaryKeyName, RELATIONSHIPTYPEID);
                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.FACETID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                                }
                                                else
                                                {
                                                    if (b_IsRelationship_RF_RF)
                                                    {
                                                        //Support for Facet-Facet relationship
                                                        //Identify the Parent Hierarchy Object Type and prepare query accordingly
                                                        switch (obj_ParentHierarchyObject.m_ObjectType.ToUpper())
                                                        {
                                                            /* This case specifies that parent is ENTITY (hierarchy object type), 
                                                             * and the previous join has been taken on ASSEMBLY table between Parent hierarchy object
                                                             * and current hierarchy object, so while taking join on ALOM_ASSEMBLY for filter on related object,
                                                             * consider ASSEMBLY.CONTAINERID OR ASSEMBLY.CONTAINSID
                                                             * */
                                                            case "ENTITY":
                                                                if (AssemblyColumnName == "CONTAINSID")
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = {1}.{2} ", AssemblyAliasName, "ASSOENTFCT", "FACETID");
                                                                else
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = {1}.{2} ", AssemblyAliasName, "ASSOENTFCT", "FACETID");

                                                                break;

                                                            /* This case specifies that parent is FACET (hierarchy object type), 
                                                             * and the previous join has been taken on ALOM_ASSOENTITYFACET table between Parent hierarchy object
                                                             * and current hierarchy object, so while taking join on ALOM_ASSEMBLY for filter on related object,
                                                             * consider ASSOENTFCT.ENTITYID
                                                             * */
                                                            case "FACET":

                                                                if (AssemblyColumnName == "CONTAINSID")
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINERID = {1}.{2} ", AssemblyAliasName, "ALASSEMBLY", HierarchyObj.m_ContainerName);
                                                                else
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSEMBLY {0} ON  {0}.CONTAINSID = {1}.{2} ", AssemblyAliasName, "ALASSEMBLY", HierarchyObj.m_ContainerName);

                                                                break;
                                                        }

                                                        InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.{3} = {4}.{5}", RelatedObjectTypeName, RelatedTypeAlias, AssemblyAliasName, AssemblyColumnName, RelatedTypeAlias, RelatedTypePKName);

                                                        //Add AssemblyAliasName in RelationshipCollectionListObj
                                                        AddAssemblyAliasName(RelationshipCollectionListObj, TempRelationshipName, AssemblyAliasName);
                                                    }
                                                    else
                                                    {
                                                        if (b_IsRelationship_RF_RE)
                                                        {
                                                            /* While taking join on ALOM_ASSOENTITYFACET, it is mandatory to know object type of Parent Hierarchy Object,
                                                             * so that the join can be taken on appropriate column from (i.e. ALASSEMLBY.CONTAINERID or ALASSEMBLY.CONTAINSID or ASSOENTFCT.FACETID
                                                             * */
                                                            switch (obj_ParentHierarchyObject.m_ObjectType.ToUpper())
                                                            {
                                                                case "FACET":
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET {0} ON {0}.FACETID = {1}.{2}", AssoAliasName, "ALASSEMBLY", HierarchyObj.m_ContainerName);
                                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.ENTITYID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                                                    break;

                                                                case "ENTITY":
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET {0} ON {0}.FACETID = {1}.{2}", AssoAliasName, "ASSOENTFCT", "FACETID");
                                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.ENTITYID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                                                    break;
                                                            }
                                                        }
                                                        else
                                                        {
                                                            /* While taking join on ALOM_ASSOENTITYFACET, it is mandatory to know object type of Parent Hierarchy Object,
                                                             * so that the join can be taken on appropriate column from (i.e. ALASSEMLBY.CONTAINERID or ALASSEMBLY.CONTAINSID or ASSOENTFCT.FACETID
                                                             * */
                                                            switch (obj_ParentHierarchyObject.m_ObjectType.ToUpper())
                                                            {
                                                                case "FACET":
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET {0} ON {0}.ENTITYID = {1}.{2}", AssoAliasName, "ASSOENTFCT", "ENTITYID");//HierarchyObj.m_ContainerName);
                                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.FACETID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                                                    break;

                                                                case "ENTITY":
                                                                    //InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET ASSOENTFCT{0} ON ASSOENTFCT{0}.ENTITYID = {1}.{2}", TempFilterObj.m_FilterID, HierarchyObj.m_EveryOneUserTableName.Trim() , HierarchyObj.m_PrimaryKeyforTableVariable);
                                                                    InnerJoinList.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET {0} ON {0}.ENTITYID = {1}.{2}", AssoAliasName, "ASSEMBLY", HierarchyObj.m_ContainerName);//HierarchyObj.m_ContainerName);
                                                                    InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.FACETID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                                                    break;
                                                            }
                                                        }
                                                    }
                                                }
                                                //InnerJoinList.AppendFormat(" INNER JOIN {0} {1} ON {2}.FACETID = {3}.{4} AND ({2}.RELATIONSHIPTYPEID = {5} OR {2}.RELATIONSHIPTYPEID IS NULL)", RelatedObjectTypeName, RelatedTypeAlias, AssoAliasName, RelatedTypeAlias, RelatedTypePKName, RELATIONSHIPTYPEID);
                                            }
                                        }
                                    }

                                    ErrorObj = GetFilterString(RelatedTypeAlias, TempFilterObj, ref FilterClause, ref ObjectTypeNode, false);
                                    if (ErrorObj != null)
                                        return ErrorObj;
                                    if (ClauseCollection.ContainsKey(TempRelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName))
                                    {
                                        string query = ClauseCollection[TempRelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName].ToString();
                                        FilterClause.AppendFormat(" OR {0}", query);
                                        ClauseCollection.Remove(TempRelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName);
                                    }
                                    ClauseCollection.Add(TempRelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName, FilterClause);

                                    // Code which prepares where clause ends.
                                }
                            }

                        }
                        // Get all the filters where relationship Name is empty or null.
                        // Get ObjectType Details for TE Level

                        if (HierarchyObj.m_IsRelatedEntity)
                            RelatedType = "ENTITY";
                        else
                            RelatedType = "FACET";

                        ObjectTypeNode = null;

                        if ((ReturnErrorObj = ReadTypeDetails(HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeID, ref ObjectTypeNode)) != null)
                        {
                            ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                                            ErrorObj.ErrorMessage, ErrorObj.ObjectType, ErrorObj.ObjectTypeName,
                                            ErrorObj.ErrorType, ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                            return ReturnErrorObj;
                        }

                        foreach (FilterCollection TempFilterObj in HierarchyObj.FilterCollectionList)
                        {
                            if (TempFilterObj.m_RelationshipName == null || TempFilterObj.m_RelationshipName == "")
                            {
                                FilterClause = new StringBuilder();
                                if (HierarchyObj.m_ParentID == 0)
                                    RelatedTypeAlias = "TE";
                                else
                                    RelatedTypeAlias = HierarchyObj.m_TableName + "_ALIAS";
                                ErrorObj = GetFilterString(RelatedTypeAlias, TempFilterObj, ref FilterClause, ref ObjectTypeNode, false);
                                if (ErrorObj != null)
                                    return ErrorObj;
                                if (ClauseCollection.ContainsKey(HierarchyObj.m_RelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName))
                                {
                                    string query = ClauseCollection[HierarchyObj.m_RelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName].ToString();
                                    FilterClause.AppendFormat(" OR {0}", query);
                                    ClauseCollection.Remove(HierarchyObj.m_RelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName);
                                }
                                ClauseCollection.Add(HierarchyObj.m_RelationshipName + "." + RelatedTypeAlias + "." + TempFilterObj.m_FieldName, FilterClause);
                            }
                        }

                        ReturnErrorObj = PrepareWhereClause(ClauseCollection, RelationshipCollectionListObj, ref WhereClause);
                        if (ReturnErrorObj != null)
                            return ReturnErrorObj;

                    }
                }

                if (!String.IsNullOrEmpty(HierarchyObj.m_FilterCombination))
                    ReturnFilterClause = WhereClause.ToString();
                else
                {
                    if (String.IsNullOrEmpty(WhereClause.ToString()))
                        ReturnFilterClause = null;
                    else
                    {
                        if ((HierarchyObj.FilterCollectionList.Count > 0) && (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString)) && (HierarchyObj.m_ParentID == 0))
                            WhereClause.AppendFormat(" AND TE.{0} IN ( SELECT SSOBJECTID FROM  @SSFILTEREDTABLE )", HierarchyObj.m_PrimaryKeyName);  //= " WHERE TE." + HierarchyObj.m_PrimaryKeyName + " IN ( SELECT SSOBJECTID FROM @SSFILTEREDTABLE) ";
                        ReturnFilterClause = InnerJoinList.ToString() + " WHERE ( " + WhereClause.ToString() + " )";
                    }
                }

                // Processing for Simple Search
                // This means that there are no Filter's on TE and RE or RF.
                if (HierarchyObj.FilterCollectionList.Count == 0 && (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString)) && HierarchyObj.m_ParentID == 0)
                {
                    ReturnFilterClause = " WHERE TE." + HierarchyObj.m_PrimaryKeyName + " IN ( SELECT SSOBJECTID FROM @SSFILTEREDTABLE) ";
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetFilterClause",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError GetFilterClauseonFilterCombination(ref HierarchyObject HierarchyObj, ref StringBuilder WhereClause)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;

            try
            {

                ErrorObj = PrepareQueryFromFilterCombination(HierarchyObj, ref WhereClause);
                if (ErrorObj != null)
                    return ErrorObj;

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetFilterClauseonFilterCombination",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        public override OMError GetFilterString(String TableAliasName, FilterCollection TempFilterObj, ref StringBuilder FilterClause, 
                                                ref XmlNode ObjectTypeNode, bool IsINToBeOptimized)
        {
            XmlNode DataTypeNode = null;
            String DataType = null;
            XmlNode PrimaryKeyNode = null;
            bool    IsPrimaryKey = false;
            XmlNode MultiSelectField = null;
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            String SearchOperator = null;
            String FieldValue = null;
            String Value = null;
            ulong Value1 = 0;
            ulong Value2 = 0;
            Decimal decimalValue1 = 0;
            Decimal decimalValue2 = 0;
            float floatValue1 = 0.0f;
            float floatValue2 = 0.0f;
            XmlNode ExpressionValueNode = null;
            XmlNode ValueNode = null;
            XmlNode dataTypeNode = null;
            decimal MoneyValue = 0;
            decimal CurrencyValue1 = 0;
            decimal CurrencyValue2 = 0;
            String Parameter_1;
            String Parameter_2;
            String Parameter_3;
            XmlNode DataTypeSizeNode = null;
            String DataTypeSize = null;

            try
            {
                // Code which will prepare Where Clause.
                DataTypeNode = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDNAME/text() = '" + TempFilterObj.m_FieldName + "']/FIELDTYPE");
                if (DataTypeNode == null)
                {
                    ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "FieldName " + TempFilterObj.m_FieldName + " Not Found",
                        OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.ERROR, null, "");
                    return ErrorObj;
                }
                DataType = DataTypeNode.InnerText.ToString();
                if (DataTypeNode == null)
                {
                    ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "FieldName " + TempFilterObj.m_FieldName + " Not Found",
                        OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.ERROR, null, "");
                    return ErrorObj;
                }
                DataType = DataTypeNode.InnerText.ToString();

                DataTypeSizeNode = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDNAME/text() = '" + TempFilterObj.m_FieldName + "']/FIELDSIZE");
                if (DataTypeSizeNode == null)
                {
                    ErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "FieldName " + TempFilterObj.m_FieldName + " Not Found",
                        OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.ERROR, null, "");
                    return ErrorObj;
                }
                DataTypeSize = DataTypeSizeNode.InnerText.ToString();

                //Doesn't need to be a failure
                PrimaryKeyNode = ObjectTypeNode.SelectSingleNode("PRIMARYKEY");
                if (PrimaryKeyNode != null)
                {
                    if (string.Compare(PrimaryKeyNode.InnerText.ToString(), TempFilterObj.m_FieldName, true) == 0)
                        IsPrimaryKey = true;
                }

                if (TempFilterObj.m_Operator.ToUpper() != "DATEMACRO")
                    SearchOperator = NSearchCommon.GetSearchOperatorValue(TempFilterObj.m_Operator);
                FilterClause.Remove(0, FilterClause.Length);

                switch (DataType.ToUpper())
                {
                    case "TEXT":
                    case "STRING":
                        string seperatorIndexes = string.Empty;
                        Parameter_1 = GetNextParameterizedVariable();
                        Parameter_3 = null;
                        MultiSelectField = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDNAME/text() = '" + TempFilterObj.m_FieldName + "' and ISNAMEDHIERARCHY/text()='TRUE' and FIELDUITYPE/text()='MULTISELECT']");
                        if (MultiSelectField != null)
                        {
                            FieldValue = GetStringValue(String.Format("$AL${0}$AL$", TempFilterObj.m_FieldValue), "CONTAINS");
                            SearchOperator = " LIKE";
                        }
                        else
                            FieldValue = GetStringValue(TempFilterObj.m_FieldValue, TempFilterObj.m_Operator);

                        if (SearchOperator.Trim().ToUpper() == "IN" || SearchOperator.Trim().ToUpper() == "NOT IN")
                        {
                            //FieldValue = FieldValue.Replace("(", "").Replace(")", "").Replace("'',''", ",").Replace("'', ''", ",");
                            FieldValue = FieldValue.Trim();
                            if (FieldValue.Length >= 2)
                            {
                                if (FieldValue[0] == '(' && FieldValue[FieldValue.Length - 1] == ')')
                                    FieldValue = FieldValue.Substring(1, FieldValue.Length - 2); 
                            }
                            FieldValue = FieldValue.Replace("'', ''", "'',''");

                            String strVal = FieldValue;

                            IList<int> iCntSingleQuote = new List<int>();
                            IList<int> iFinalIndexes = new List<int>();
                            String[] strActualValues = strVal.Split(new string[] { "'',''" }, StringSplitOptions.None);
                            for (int i = 0; i < strActualValues.Length; i++)
                            {
                                String strOrg = strActualValues[i];
                                strOrg = strOrg.Replace("''''", string.Empty);
                                int iDiff = strActualValues[i].Length - strOrg.Length;
                                iCntSingleQuote.Add(iDiff / 4);

                            }
                            //FieldValue = FieldValue.Replace("'',''", ",").Replace("'', ''", ",");
                            
                            string pattern = "'',''";
                            string sentence = FieldValue;
                            IList<int> indexes = new List<int>();
                            foreach (Match match in Regex.Matches(sentence, pattern))
                            {
                                indexes.Add(match.Index);
                            }

                            int iIteratorId = 0;
                            int iOffset_singleQuote = 0;
                            Int16 iOffset = 2;
                            foreach (Int16 index in indexes)
                            {
                                if (string.IsNullOrEmpty(seperatorIndexes))
                                {
                                    iOffset_singleQuote = iOffset_singleQuote + iCntSingleQuote[iIteratorId];
                                    seperatorIndexes += "'" + (Convert.ToInt16(index.ToString()) - iOffset + 1 - (iOffset_singleQuote * 3)).ToString();
                                    iOffset += 4;
                                    iIteratorId++;
                                }
                                else
                                {
                                    iOffset_singleQuote = iOffset_singleQuote + iCntSingleQuote[iIteratorId];
                                    seperatorIndexes += "," + (Convert.ToInt16(index.ToString()) - iOffset + 1 - (iOffset_singleQuote * 3)).ToString();
                                    iOffset += 4;
                                    iIteratorId++;
                                }
                            }
                            if (string.IsNullOrEmpty(seperatorIndexes))
                                seperatorIndexes = "''''";
                            else
                                seperatorIndexes += "'";
                            
                            FieldValue = FieldValue.Replace("'',''", ",");
                            Parameter_3 = GetNextParameterizedVariable();
                            
                            if(FieldValue.Length > 0)
                                FieldValue = "N" + FieldValue;
                            else
                                FieldValue = "N" + @"''''";

                            //FilterClause = GetCaseDependentClause(TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, FieldValue, TempFilterObj.m_IsCaseSensitive.ToString());
                            if (TempFilterObj.m_IsCaseSensitive.ToString().Equals("TRUE")) //Here the sequence of COLLATE is changed since <FIELD NAME> IS NULL COLLATE.. gives syntax error
                                FilterClause.AppendFormat("{0} COLLATE SQL_Latin1_General_CP1_CS_AS {1} ( Select Data From dbo.splitEx( {2} ,'','', {3} ) ) ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, Parameter_3);
                            else
                                FilterClause.AppendFormat("{0} {1} ( Select Data From dbo.splitEx( {2} ,'','', {3} ) ) ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, Parameter_3);

                        }
                        else
                        {
                            if ((TempFilterObj.m_Operator == "IS" || TempFilterObj.m_Operator == "IS NOT") && FieldValue == "NULL")
                                FilterClause = GetCaseDependentClause(TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, "NULL", TempFilterObj.m_IsCaseSensitive.ToString());
                            else
                                FilterClause = GetCaseDependentClause(TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, TempFilterObj.m_IsCaseSensitive.ToString());
                        }

                        if (string.Compare(SearchOperator, " NOT LIKE", true) == 0)
                        {
                            FilterClause.AppendFormat(" or {0} is null ", TableAliasName + "." + TempFilterObj.m_FieldName);
                        }

                        if (FieldValue != "NULL" && (TempFilterObj.m_Operator != "IS" || TempFilterObj.m_Operator != "IS NOT"))
                        {
                            TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)"; // IN CLAUSE CUASES PROBLEM
                            TempFilterObj.m_FieldValueParameterized.Add(FieldValue);

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);

                            if (SearchOperator.Trim().ToUpper() == "IN" || SearchOperator.Trim().ToUpper() == "NOT IN")
                            {
                                TempFilterObj.m_FieldValueParameterized.Add(seperatorIndexes);
                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_3);
                            }
                        }
                        break;
                    case "BOOLEAN":
                        Parameter_1 = GetNextParameterizedVariable();
                        switch (SearchOperator.Trim().ToUpper())
                        {
                            //Special handling is required for IS and IS NOT operator (because single quotes are not required around value), default implemenation will be applied for other operators
                            case "IS":
                            case "IS NOT":
                                FilterClause.AppendFormat(" {0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, TempFilterObj.m_FieldValue);
                                break;

                            default:
                                FilterClause.AppendFormat(" {0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                                break;
                        }
                        if (SearchOperator.Trim().ToUpper() != "IS" || SearchOperator.Trim().ToUpper() != "IS NOT")
                        {
                            TempFilterObj.m_FieldDataType = "BIT";
                            TempFilterObj.m_FieldValueParameterized.Add(TempFilterObj.m_FieldValue);
                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        break;
                    case "DATETIME":
                        if (TempFilterObj.m_Operator.ToUpper() == "DATEMACRO")
                            FilterClause = PrepareDateMacroFilterEx(TempFilterObj, TableAliasName + "." + TempFilterObj.m_FieldName, TempFilterObj.m_FieldValue, TempFilterObj.m_RelationshipName);
                        else
                            FilterClause = PrepareDateFilterEx(TempFilterObj, TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, TempFilterObj.m_XMLNode);
                        break;
                    case "INTEGER":
                    case "BIGINT":
                        if (SearchOperator.Equals("BETWEEN"))
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Parameter_2 = GetNextParameterizedVariable();
                            Value1 = Convert.ToUInt64(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='1']").InnerText);
                            Value2 = Convert.ToUInt64(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']").InnerText);
                            FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, Parameter_2);


                            TempFilterObj.m_FieldDataType = "BIGINT";
                            TempFilterObj.m_FieldValueParameterized.Add(Value1.ToString());
                            TempFilterObj.m_FieldValueParameterized.Add(Value2.ToString());
                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                        }
                        else if (SearchOperator.Trim().ToUpper() == "IN")
                        {
                            if (IsPrimaryKey && IsINToBeOptimized)
                            {
                                Value = TempFilterObj.m_FieldValue.Trim();
                                if (Value.Length >= 2)
                                {
                                    if (Value[0] == '(' && Value[Value.Length - 1] == ')')
                                        Value = Value.Substring(1, Value.Length - 2);
                                }
                                Value = "N''" + Value.Trim() + "''";

                                StringBuilder sbSelectUnionAll = new StringBuilder();
                                string[] IDs = Value.Split(',');

                                if (IDs.Length <= CONST_MAX_IDS_FOR_IN)
                                {
                                    //In this case, there is no performance benefit from optimizing the IN. So we will not optimize it
                                    Parameter_1 = GetNextParameterizedVariable();
                                    //Value = TempFilterObj.m_FieldValue;
                                    TempFilterObj.m_IsINOptimized = false;
                                    FilterClause.AppendFormat(" {0} {1} ( Select Data From dbo.split( {2} ,'','') )", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                                    //if(Value.Length < 8000) // MAXIMU ALLOWED VALUE FOR NVARCHAR
                                    //    TempFilterObj.m_FieldDataType = "NVARCHAR(" + Value.Length + ")";
                                    //else
                                    //    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                                    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)"; // commented clause will generate different query each time which is not acceptable
                                    TempFilterObj.m_FieldValueParameterized.Add(Value.ToString());

                                    TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                                    break;
                                }

                                //int IdCounter = 0;
                                sbSelectUnionAll.Append("<ROOT>");
                                foreach (string Id in IDs)
                                {
                                    sbSelectUnionAll.Append("<IDLIST FILTEREDOBJID=\"");
                                    sbSelectUnionAll.Append(Id.Trim());
                                    sbSelectUnionAll.Append("\"/>");
                                }
                                sbSelectUnionAll.Append("</ROOT>");

                                TempFilterObj.m_IsINOptimized = true;
                                FilterClause.Append(sbSelectUnionAll.ToString());

                                sbSelectUnionAll.Remove(0, sbSelectUnionAll.Length);
                                sbSelectUnionAll = null;
                            }
                            else
                            {
                                Parameter_1 = GetNextParameterizedVariable();
                                Value = TempFilterObj.m_FieldValue.Trim();
                                if (Value.Length >= 2)
                                {
                                    if (Value[0] == '(' && Value[Value.Length - 1] == ')')
                                        Value = Value.Substring(1, Value.Length - 2);
                                }
                                Value = "N''" + Value.Trim() + "''";
                                TempFilterObj.m_IsINOptimized = false;
                                FilterClause.AppendFormat(" {0} {1} ( Select Data From dbo.split( {2} ,'','') ) ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                                //if (Value.Length < 8000) // MAXIMU ALLOWED VALUE FOR NVARCHAR
                                //    TempFilterObj.m_FieldDataType = "NVARCHAR(" + Value.Length + ")";
                                //else
                                //    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                                TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                                TempFilterObj.m_FieldValueParameterized.Add(Value.ToString());

                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            }
                        }
                        else if (SearchOperator.Trim().ToUpper() == "NOT IN")
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Value = TempFilterObj.m_FieldValue.Trim();
                            if (Value.Length >= 2)
                            {
                                if (Value[0] == '(' && Value[Value.Length - 1] == ')')
                                    Value = Value.Substring(1, Value.Length - 2);
                            }
                            Value = "N''" + Value.Trim() + "''";
                            FilterClause.AppendFormat(" {0} {1} ( Select Data From dbo.split( {2} ,'','') ) ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                            //if (Value.Length < 8000) // MAXIMU ALLOWED VALUE FOR NVARCHAR
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(" + Value.Length + ")";
                            //else
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldValueParameterized.Add(Value.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else if (SearchOperator.Trim().ToUpper() == "IS" || SearchOperator.Trim().ToUpper() == "IS NOT")
                        {
                            //Parameter_1 = GetNextParameterizedVariable();
                            //Special case for IS and IS NOT operator
                            FilterClause.AppendFormat(" {0} {1} {2}", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, TempFilterObj.m_FieldValue);
                            
                            //TempFilterObj.m_FieldDataType = "BIGINT";
                            //TempFilterObj.m_FieldValueParameterized.Add(TempFilterObj.m_FieldValue);

                            //TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Value1 = Convert.ToUInt64(TempFilterObj.m_FieldValue);
                            FilterClause.AppendFormat(" {0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);

                            TempFilterObj.m_FieldDataType = "BIGINT";
                            TempFilterObj.m_FieldValueParameterized.Add(Value1.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        break;
                    case "DECIMAL":
                        decimalValue1 = Convert.ToDecimal(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='1']").InnerText);
                        if (SearchOperator.Equals("BETWEEN"))
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Parameter_2 = GetNextParameterizedVariable();
                            decimalValue2 = Convert.ToDecimal(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']").InnerText);
                            FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, Parameter_2);

                            TempFilterObj.m_FieldDataType = "DECIMAL";
                            TempFilterObj.m_FieldValueParameterized.Add(decimalValue1.ToString());
                            TempFilterObj.m_FieldValueParameterized.Add(decimalValue2.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                        }
                        else
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            FilterClause.AppendFormat("{0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                            TempFilterObj.m_FieldDataType = "DECIMAL";
                            TempFilterObj.m_FieldValueParameterized.Add(decimalValue1.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        break;
                    case "FLOAT":
                        if (SearchOperator.Equals("BETWEEN"))
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Parameter_2 = GetNextParameterizedVariable();
                            floatValue1 = Convert.ToSingle(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='1']").InnerText);
                            floatValue2 = Convert.ToSingle(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']").InnerText);
                            FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1, Parameter_2);

                            TempFilterObj.m_FieldDataType = "FLOAT";
                            TempFilterObj.m_FieldValueParameterized.Add(floatValue1.ToString());
                            TempFilterObj.m_FieldValueParameterized.Add(floatValue2.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                        }
                        else if (SearchOperator.Trim().ToUpper() == "IN")
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Value = TempFilterObj.m_FieldValue.Trim();
                            if (Value.Length >= 2)
                            {
                                if (Value[0] == '(' && Value[Value.Length - 1] == ')')
                                    Value = Value.Substring(1, Value.Length - 2);
                            }
                            Value = "N''" + Value.Trim() + "''";
                            //Value = "N" + TempFilterObj.m_FieldValue;
                            FilterClause.AppendFormat(" {0} {1} ( Select Data From dbo.split( {2} ,'','') ) ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                            //if (Value.Length < 8000) // MAXIMU ALLOWED VALUE FOR NVARCHAR
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(" + Value.Length + ")";
                            //else
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldValueParameterized.Add(Value);

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else if (SearchOperator.Trim().ToUpper() == "NOT IN")
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            Value = TempFilterObj.m_FieldValue.Trim();
                            if (Value.Length >= 2)
                            {
                                if (Value[0] == '(' && Value[Value.Length - 1] == ')')
                                    Value = Value.Substring(1, Value.Length - 2);
                            }
                            Value = "N''" + Value.Trim() + "''";
                            //Value = "N" + TempFilterObj.m_FieldValue;
                            FilterClause.AppendFormat(" {0} {1} ( Select Data From dbo.split( {2} ,'','') )", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);
                            //if (Value.Length < 8000) // MAXIMU ALLOWED VALUE FOR NVARCHAR
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(" + Value.Length + ")";
                            //else
                            //    TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                            TempFilterObj.m_FieldValueParameterized.Add(Value);

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else if (SearchOperator.Trim().ToUpper() == "IS" || SearchOperator.Trim().ToUpper() == "IS NOT")
                        {
                            //Parameter_1 = GetNextParameterizedVariable();
                            //Special handling for IS and IS NOT operator
                            FilterClause.AppendFormat(" {0} {1} {2}", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, TempFilterObj.m_FieldValue);

                            //TempFilterObj.m_FieldDataType = "FLOAT";
                            //TempFilterObj.m_FieldValueParameterized.Add(Value);

                            //TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            floatValue1 = Convert.ToSingle(TempFilterObj.m_FieldValue);
                            FilterClause.AppendFormat(" {0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName, SearchOperator, Parameter_1);

                            TempFilterObj.m_FieldDataType = "FLOAT";
                            TempFilterObj.m_FieldValueParameterized.Add(floatValue1.ToString());

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        break;

                    case "CURRENCY":
                        // The handling of currency type data in preparing the filter string is done by accepting the entire filter node in the
                        // m_XMLNode. The EXPRESSIONVALUE node contains TYPE of data (either type of currency or amount of currency) and VALUE 
                        // of data. The field name of the TempFilterObj is changed to FieldName_C(for TYPE) or FieldName_A(for VALUE) so as 
                        // to prevent formation of an OR condition between TYPE and VALUE (The system makes an OR condition by default if field 
                        // names are same.).


                        ExpressionValueNode = TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE");
                        dataTypeNode = ExpressionValueNode.SelectSingleNode("TYPE");

                        if (dataTypeNode.InnerText.ToUpper() == "CURRENCY")
                        {
                            Parameter_1 = GetNextParameterizedVariable();
                            ValueNode = ExpressionValueNode.SelectSingleNode("VALUE");
                            Value = ValueNode.InnerText.Trim();

                            if (SearchOperator.Trim().ToUpper() == "IS" || SearchOperator.Trim().ToUpper() == "IS NOT")
                            {
                                FilterClause.AppendFormat("{0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName + "_C", SearchOperator, Parameter_1);
                                TempFilterObj.m_FieldName = TempFilterObj.m_FieldName + "_C";
                            }
                            else
                            {
                                FilterClause.AppendFormat("{0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName + "_C", SearchOperator,  Parameter_1);
                                TempFilterObj.m_FieldName = TempFilterObj.m_FieldName + "_C";
                                Value = "N''" + Value + "''";
                            }
                            TempFilterObj.m_FieldDataType = "NVARCHAR(25)";
                            TempFilterObj.m_FieldValueParameterized.Add(Value);

                            TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                        }
                        else if (dataTypeNode.InnerText.ToUpper() == "CURRENCY_AMOUNT")
                        {
                            if (SearchOperator.Equals("BETWEEN"))
                            {
                                Parameter_1 = GetNextParameterizedVariable();
                                Parameter_2 = GetNextParameterizedVariable();
                                CurrencyValue1 = Convert.ToDecimal(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='1']").InnerText.Trim());
                                CurrencyValue2 = Convert.ToDecimal(TempFilterObj.m_XMLNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']").InnerText.Trim());
                                FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", TableAliasName + "." + TempFilterObj.m_FieldName + "_A", SearchOperator, Parameter_1, Parameter_2);
                                TempFilterObj.m_FieldName = TempFilterObj.m_FieldName + "_A";

                                TempFilterObj.m_FieldDataType = "MONEY";
                                TempFilterObj.m_FieldValueParameterized.Add(CurrencyValue1.ToString());
                                TempFilterObj.m_FieldValueParameterized.Add(CurrencyValue2.ToString());

                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);

                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                            }
                            else if (SearchOperator.Trim().ToUpper() == "IS" || SearchOperator.Trim().ToUpper() == "IS NOT")
                            {
                                Parameter_1 = GetNextParameterizedVariable();
                                ValueNode = ExpressionValueNode.SelectSingleNode("VALUE");
                                Value = ValueNode.InnerText.Trim();

                                FilterClause.AppendFormat("{0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName + "_A", SearchOperator, Parameter_1);
                                TempFilterObj.m_FieldName = TempFilterObj.m_FieldName + "_A";

                                TempFilterObj.m_FieldDataType = "MONEY";
                                TempFilterObj.m_FieldValueParameterized.Add(Value);

                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            }
                            else
                            {
                                Parameter_1 = GetNextParameterizedVariable();
                                ValueNode = ExpressionValueNode.SelectSingleNode("VALUE");
                                Value = ValueNode.InnerText.Trim();
                                MoneyValue = Convert.ToDecimal(Value);
                                FilterClause.AppendFormat("{0} {1} {2} ", TableAliasName + "." + TempFilterObj.m_FieldName + "_A", SearchOperator, Parameter_1);
                                TempFilterObj.m_FieldName = TempFilterObj.m_FieldName + "_A";

                                TempFilterObj.m_FieldDataType = "MONEY";
                                TempFilterObj.m_FieldValueParameterized.Add(MoneyValue.ToString());

                                TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                            }
                        }
                        
                        break;                        
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetFilterString",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }

        }

        private OMError GetParentHierarchyObject(HierarchyObject obj_HierarchyObject, ref HierarchyObject obj_ParentHierarchyObject)
        {
            OMError             obj_OMError             = null;
            try
            {
                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObj.m_TagID == obj_HierarchyObject.m_ParentID)
                    {
                        obj_ParentHierarchyObject = TempHierarchyObj;
                        break;
                    }
                }
            }
            catch (Exception obj_Exception)
            {
                obj_OMError = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetParentHierarchyObject",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, obj_Exception, "");

                
            }
            return obj_OMError;
        }

        private ArrayList GetFilterIDList(String InputString)
        {
            int Count = 0;
            ArrayList NumberArray = null;
            String StrNumber = null;

            NumberArray = new ArrayList();
            StrNumber = "";

            for (int i = 0; i < InputString.Length; i++)
            {
                if (InputString[i] >= '0' && InputString[i] <= '9')
                {
                    StrNumber = StrNumber + InputString[i].ToString();
                    Count++;
                }
                else
                {
                    if (StrNumber != "")
                    {
                        NumberArray.Add(Convert.ToInt32(StrNumber));
                        Count = 0;
                    }
                    StrNumber = "";
                }
            }
            return NumberArray;
        }

        private OMError GetBinderandFileFields()
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;

            try
            {
                m_NSearchCommonObj.m_BinderFields = new SortedList();
                m_NSearchCommonObj.m_FileFields = new SortedList();
                m_NSearchCommonObj.m_AssoFileBinderFields = new SortedList();

                m_NSearchCommonObj.m_BinderFields = GetListOfFieldsFromSchema("ALFS_BINDER");
                // This will add FIELDNAME as default 
                m_NSearchCommonObj.m_BinderFields.Add("FIELDNAME", "STRING");
                m_NSearchCommonObj.m_FileFields = GetListOfFieldsFromSchema("ALFS_FILE");
                m_NSearchCommonObj.m_FileFields.Add("FIELDNAME", "STRING");
                m_NSearchCommonObj.m_AssoFileBinderFields = GetListOfFieldsFromSchema("ALFS_ASSOFILEBINDER");

                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetBinderandFileFields",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// This method prepares the Query to fetch the result set, based on the sort col and sort order..
        /// </summary>
        /// 
        /// 
        /// 
        /// <param name="HierarchyObj">HierarchyObject Instance</param>
        /// <param name="Query">Query which contains filterclauses.</param>
        /// <param name="FilterQuery">Query which contains filterclauses and select clause.</param>
        /// <returns>OMError if any error is returned from OM</returns>

        private OMError PrepareQuerys(HierarchyObject HierarchyObj, ref String Query)
        {
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            StringBuilder ResultQuery = new StringBuilder();
            String PagingSortField = null;

            try
            {
                //Below code will be called in case no max results OR max results as well as paging
                if (HierarchyObj.m_MaxResultSize == 0 || (HierarchyObj.m_MaxResultSize > 0 && HierarchyObj.m_DoPaging))
                {
                    if (HierarchyObj.m_DoPaging)
                    {
                        if (!m_NSearchCommonObj.IsAuthorizationRequired)
                        {
                            PagingCountFromClause = string.Format(" FROM {1} AS TE ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);
                        }
                        else
                        {
                            PagingCountFromClause = string.Format(" FROM {1} AS TE ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);
                        }
                    }

                    if (!m_NSearchCommonObj.IsAuthorizationRequired && HierarchyObj.m_DoPaging &&
                        (string.IsNullOrEmpty(HierarchyObj.m_FilterCombination) || HierarchyObj.FilterCollectionList.Count == 1))
                    {
                        if (!string.IsNullOrEmpty(HierarchyObj.m_PagingSortField))
                            PagingSortField = HierarchyObj.m_PagingSortField;
                        else
                            PagingSortField = HierarchyObj.m_PrimaryKeyName;


                        if (HierarchyObj.SortSpecificationList.Count == 0)
                        {
                            ResultQuery.AppendFormat("select {0} from (SELECT DISTINCT({0}), ROW_NUMBER() OVER (ORDER BY {2} {3}) AS ROWNUMBER FROM (select distinct(TE.{0}) from {1} as TE ",
                                                        HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, PagingSortField,
                                                        HierarchyObj.m_PagingSortFieldOrder);
                        }
                        else
                        {
                            string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;
                            
                            GetSortFields(HierarchyObj.SortSpecificationList, "TE", HierarchyObj.m_PrimaryKeyName,
                                                ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                            ResultQuery.AppendFormat("select {0} from (SELECT DISTINCT({0}), ROW_NUMBER() OVER (ORDER BY {2}) AS ROWNUMBER {3}  FROM (select distinct(TE.{0}) {3} from {1} as TE ",
                                                        HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, SortFieldNamesWithOrder, SortFieldNamesOnly);
                        }

                        PagingCountFromClause = string.Format(" FROM {1} AS TE ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);

                    }
                    else
                    {
                        ResultQuery.AppendFormat("SELECT DISTINCT(TE.{0}) FROM {1} AS TE ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);
                    }

                }
                else
                {
                    ResultQuery.AppendFormat("SELECT DISTINCT TOP {0} TE.{1} FROM {2} AS TE ", HierarchyObj.m_MaxResultSize, HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);
                }

                Query = ResultQuery.ToString();

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareQuerys",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// Formates the value of the filter in case the data type of the field is datetime. This function handles
        /// the case when the time parameters are not specified, EQUALTO and Not EQUALTO operators have special handling
        /// </summary>
        /// <param name="FieldName">Name of the date field</param>
        /// <param name="SearchOperator">Value of the search operator</param>
        /// <param name="FilterNode">Filter node</param>
        /// <returns>Clause for the date filter</returns>
        public StringBuilder PrepareDateFilter(string FieldName, string SearchOperator, XmlNode FilterNode)
        {
            XmlNode DayNode = null;	    //This contains the Day node which specifies the day in the date in InputXML
            XmlNode MonthNode = null;	//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode YearNode = null;    //This contains the Year node which specifies the Year in the date in InputXML
            XmlNode HourNode = null;	//This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode MinuteNode = null;	//This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode SecondNode = null;	//This contains the Second node which specifies the Second in the date in InputXML

            string DateValue1 = "";
            string DateValue2 = "";
            //bool IsOMDataManagerInitialized = true;

            StringBuilder DateFormat = new StringBuilder();
            StringBuilder FilterClause = new StringBuilder();

            DayNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/DAY");
            MonthNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/MONTH");
            YearNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/YEAR");
            HourNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/HOUR");
            MinuteNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/MINUTE");
            SecondNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/SECOND");

            switch (SearchOperator)
            {
                case " =":
                case " <>":
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                    {
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                        DateValue1 = ConvertDate(DateFormat.ToString());
                        FilterClause.AppendFormat(" {0} {1} {2}", FieldName, SearchOperator, DateValue1);
                    }
                    else
                    {
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                        DateValue1 = ConvertDate(DateFormat.ToString());
                        DateFormat.Length = 0;

                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                        DateValue2 = ConvertDate(DateFormat.ToString());
                        if (SearchOperator.Equals(" <>"))
                            FilterClause.AppendFormat(" {0} NOT BETWEEN {1} AND {2} ", FieldName, DateValue1, DateValue2);
                        else
                            FilterClause.AppendFormat(" {0} BETWEEN {1} AND {2} ", FieldName, DateValue1, DateValue2);
                    }
                    break;
                case " <=":
                case " >":
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue1 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2}", FieldName, SearchOperator, DateValue1);
                    break;
                case " >=":
                case " <":
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue1 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2}", FieldName, SearchOperator, DateValue1);
                    break;
                case "BETWEEN":
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

                    DateValue1 = ConvertDate(DateFormat.ToString());
                    DateFormat.Length = 0;

                    DayNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/DAY");
                    MonthNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/MONTH");
                    YearNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/YEAR");
                    HourNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/HOUR");
                    MinuteNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/MINUTE");
                    SecondNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/SECOND");

                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue2 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", FieldName, SearchOperator, DateValue1, DateValue2);
                    break;
                case " IS":
                case " IS NOT":
                    FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE").InnerText);
                    break;
            }

            return FilterClause;
        }

        public StringBuilder PrepareDateFilterEx(FilterCollection TempFilterObj, string FieldName, string SearchOperator, XmlNode FilterNode)
        {
            XmlNode DayNode = null;	    //This contains the Day node which specifies the day in the date in InputXML
            XmlNode MonthNode = null;	//This contains the Month node which specifies the Month in the date in InputXML
            XmlNode YearNode = null;    //This contains the Year node which specifies the Year in the date in InputXML
            XmlNode HourNode = null;	//This contains the Hour node which specifies the Hour in the date in InputXML
            XmlNode MinuteNode = null;	//This contains the Minute node which specifies the Minute in the date in InputXML
            XmlNode SecondNode = null;	//This contains the Second node which specifies the Second in the date in InputXML

            string DateValue1 = "";
            string DateValue2 = "";
            //bool IsOMDataManagerInitialized = true;

            StringBuilder DateFormat = new StringBuilder();
            StringBuilder FilterClause = new StringBuilder();

            String Parameter_1;
            String Parameter_2;

            DayNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/DAY");
            MonthNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/MONTH");
            YearNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/YEAR");
            HourNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/HOUR");
            MinuteNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/MINUTE");
            SecondNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE/SECOND");

            switch (SearchOperator)
            {
                case " =":
                case " <>":
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                    {
                        Parameter_1 = GetNextParameterizedVariable();
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                        DateValue1 = ConvertDate(DateFormat.ToString());
                        FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, Parameter_1);

                        TempFilterObj.m_FieldDataType = "DATETIME";
                        TempFilterObj.m_FieldValueParameterized.Add(DateValue1);

                        TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                    }
                    else
                    {
                        Parameter_1 = GetNextParameterizedVariable();
                        Parameter_2 = GetNextParameterizedVariable();
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                        DateValue1 = ConvertDate(DateFormat.ToString());
                        DateFormat.Length = 0;

                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                        DateValue2 = ConvertDate(DateFormat.ToString());
                        if (SearchOperator.Equals(" <>"))
                            FilterClause.AppendFormat(" {0} NOT BETWEEN {1} AND {2} ", FieldName, Parameter_1, Parameter_2);
                        else
                            FilterClause.AppendFormat(" {0} BETWEEN {1} AND {2} ", FieldName, Parameter_1, Parameter_2);

                        TempFilterObj.m_FieldDataType = "DATETIME";
                        TempFilterObj.m_FieldValueParameterized.Add(DateValue1);
                        TempFilterObj.m_FieldValueParameterized.Add(DateValue2);

                        TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);

                        TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                    }
                    break;
                case " <=":
                case " >":
                    Parameter_1 = GetNextParameterizedVariable();
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue1 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, Parameter_1);

                    TempFilterObj.m_FieldDataType = "DATETIME";
                    TempFilterObj.m_FieldValueParameterized.Add(DateValue1);

                    TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);

                    break;
                case " >=":
                case " <":
                    Parameter_1 = GetNextParameterizedVariable();
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue1 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, Parameter_1);


                    TempFilterObj.m_FieldDataType = "DATETIME";
                    TempFilterObj.m_FieldValueParameterized.Add(DateValue1);

                    TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                    break;
                case "BETWEEN":
                    Parameter_1 = GetNextParameterizedVariable();
                    Parameter_2 = GetNextParameterizedVariable();
                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 0:0:0'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);

                    DateValue1 = ConvertDate(DateFormat.ToString());
                    DateFormat.Length = 0;

                    DayNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/DAY");
                    MonthNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/MONTH");
                    YearNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/YEAR");
                    HourNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/HOUR");
                    MinuteNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/MINUTE");
                    SecondNode = FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE[@Order='2']/SECOND");

                    if ((HourNode != null) && (MinuteNode != null) && (SecondNode != null))
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} {3}:{4}:{5}'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText, HourNode.InnerText, MinuteNode.InnerText, SecondNode.InnerText);
                    else
                        //SINGLE QUOTE (') is added by Kiran in the following statement,
                        // because it is required for execution of Stored Procedure.
                        DateFormat.AppendFormat("'{0}/{1}/{2} 23:59:59'", MonthNode.InnerText, DayNode.InnerText, YearNode.InnerText);
                    DateValue2 = ConvertDate(DateFormat.ToString());
                    FilterClause.AppendFormat(" {0} {1} {2} AND {3} ", FieldName, SearchOperator, Parameter_1, Parameter_2);

                    TempFilterObj.m_FieldDataType = "DATETIME";
                    TempFilterObj.m_FieldValueParameterized.Add(DateValue1);
                    TempFilterObj.m_FieldValueParameterized.Add(DateValue2);

                    TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);

                    TempFilterObj.m_ParameterizedVariables.Add(Parameter_2);
                    break;
                case " IS":
                case " IS NOT":
                    //Parameter_1 = GetNextParameterizedVariable();
                    //FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, Parameter_1);
                    //TempFilterObj.m_FieldDataType = "NVARCHAR(MAX)";
                    //TempFilterObj.m_FieldValueParameterized.Add(FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE").InnerText);

                    //TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                    if (FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE").InnerText != "NULL")
                    {
                        Parameter_1 = GetNextParameterizedVariable();
                        FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, Parameter_1);
                        TempFilterObj.m_FieldDataType = "DATETIME";
                        TempFilterObj.m_FieldValueParameterized.Add(FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE").InnerText);

                        TempFilterObj.m_ParameterizedVariables.Add(Parameter_1);
                    }
                    else
                    {
                        FilterClause.AppendFormat(" {0} {1} {2} ", FieldName, SearchOperator, FilterNode.SelectSingleNode("EXPRESSIONVALUE/VALUE").InnerText);
                    }
                    break;
            }

            return FilterClause;
        }

        #region Functions for Authorization check.

        private OMError FormTEInsertQuriesWithAccessControl(HierarchyObject HierarchyObj, String FilterQuery, int PrivilegeValue, ref StringBuilder TEInsertQuery)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;

            try
            {


                // New Authorization for USER,GROUP and EVERYONE with DENY ACCESS.
                TEInsertQuery.Append(" DECLARE @EVERYONEID NUMERIC");
                TEInsertQuery.Append(" SELECT @EVERYONEID = (SELECT ACCESSORROLEID FROM ALOM_ACCESSORROLE");
                TEInsertQuery.Append(" WHERE ACCESSORROLENAME = ''EveryOne'')");

                if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                {
                    TEInsertQuery.Append(" DECLARE @SSFILTEREDTABLE TABLE (SSOBJECTID NUMERIC) ");

                    //The following inserts the Document Search Query
                    
                    if (HierarchyObj.m_IsSearchInDocuments)
                        TEInsertQuery.Append(HierarchyObj.DocumentSearchQueryBuilder.ToString());

                    TEInsertQuery.Append(HierarchyObj.SimpleSearchQueryBuilder.ToString());
                }

                TEInsertQuery.Append(" DECLARE @TEOBJECT TABLE (FILTEREDOBJID NUMERIC)");
                //Check whether FilterCombination is sepcified, if yes suppress INSERT because it has already been included in FilterQuery
                if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                    TEInsertQuery.Append(" INSERT INTO @TEOBJECT (FILTEREDOBJID) ");

                if (HierarchyObj.m_FilterCombination != null && HierarchyObj.FilterCollectionList.Count == 1)
                    TEInsertQuery.Append(" INSERT INTO @TEOBJECT (FILTEREDOBJID) ");

                if (HierarchyObj.m_DoPaging)
                {
                    int StartRecordOnThisPage, EndRecordOnThisPage;

                    GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage, 
                                                out StartRecordOnThisPage, out EndRecordOnThisPage);

                    if (string.IsNullOrEmpty(HierarchyObj.m_FilterCombination) || HierarchyObj.FilterCollectionList.Count == 1)
                    {
                        if (!m_NSearchCommonObj.IsAuthorizationRequired)
                        {
                            if (HierarchyObj.m_MaxResultSize == 0)
                            {
                                //TEInsertQuery.AppendFormat(" SELECT {0} FROM ( ", HierarchyObj.m_PrimaryKeyName);
                                TEInsertQuery.Append(FilterQuery);

                                TEInsertQuery.AppendFormat(" ) AS TEOBJECTWITHROWNUMBERS ) AS TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1} ",
                                                        StartRecordOnThisPage, EndRecordOnThisPage);

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                    //Do Nothing
                                }
                                else
                                {
                                    TEInsertQuery.Append(" order by ");
                                    string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                                    GetSortFields(HierarchyObj.SortSpecificationList, "", HierarchyObj.m_PrimaryKeyName,
                                                    ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                                    TEInsertQuery.Append(SortFieldNamesWithOrder);
                                    TEInsertQuery.Append(" ");
                                }
                            }
                            else
                            {
                                TEInsertQuery.Append(FilterQuery);

                                TEInsertQuery.AppendFormat(" ) AS TEOBJECTWITHROWNUMBERS ) AS TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1} ",
                                                        StartRecordOnThisPage, EndRecordOnThisPage);

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                    //Do Nothing
                                }
                                else
                                {
                                    TEInsertQuery.Append(" order by ");
                                    string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                                    GetSortFields(HierarchyObj.SortSpecificationList, "", HierarchyObj.m_PrimaryKeyName,
                                                    ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                                    TEInsertQuery.Append(SortFieldNamesWithOrder);
                                    TEInsertQuery.Append(" ");
                                }

                            }
                        }
                        else
                        {
                            //The paging stuff will come with authorization values
                            TEInsertQuery.Append(FilterQuery);
                        }
                    }
                    else
                    {
                        TEInsertQuery.Append(FilterQuery);
                    }
                }
                else
                {
                    TEInsertQuery.Append(FilterQuery);
                }

                if (m_NSearchCommonObj.IsAuthorizationRequired)
                {
                    TEInsertQuery.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER1 TABLE (OBJECTID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                                INSERT INTO @TEAUTHEVERYONEUSER1 (OBJECTID, PRIVILEGEVALUE, ACCESSORID)
                                                SELECT TEO.FILTEREDOBJID, MAX(AUTH.PRIVILEGEVALUE) AS PRIVILEGEVALUE,{0}
                                                FROM @TEOBJECT TEO INNER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = TEO.FILTEREDOBJID
                                                AND AUTH.ACCESSORID IN ({1})
                                                GROUP BY TEO.FILTEREDOBJID HAVING MAX(AUTH.PRIVILEGEVALUE) >= {2} AND MAX(AUTH.PRIVILEGEVALUE) <> 5

                                                DECLARE @TEAUTHEVERYONEUSER2 TABLE (OBJECTID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                                INSERT INTO @TEAUTHEVERYONEUSER2 (OBJECTID, PRIVILEGEVALUE, ACCESSORID)
                                                SELECT TEO.FILTEREDOBJID, MAX(AUTH.PRIVILEGEVALUE) AS PRIVILEGEVALUE,AUTH.ACCESSORID
                                                FROM @TEOBJECT TEO
                                                LEFT OUTER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = TEO.FILTEREDOBJID
                                                AND AUTH.ACCESSORID IN ({1})
                                                GROUP BY TEO.FILTEREDOBJID, AUTH.ACCESSORID HAVING MAX(AUTH.PRIVILEGEVALUE) IS NULL", m_NSearchCommonObj.UserID, m_NSearchCommonObj.GroupIDList, PrivilegeValue);

                    if (!HierarchyObj.m_DoPaging)
                    {
                        TEInsertQuery.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER TABLE (FILTEREDOBJID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)
                                                INSERT INTO @TEAUTHEVERYONEUSER (FILTEREDOBJID, PRIVILEGEVALUE, ACCESSORID)
                                                SELECT TEAG1.OBJECTID,TEAG1.PRIVILEGEVALUE,TEAG1.ACCESSORID FROM @TEAUTHEVERYONEUSER1 TEAG1

                                                UNION

                                                SELECT TEG2.OBJECTID, MAX(AUTH.PRIVILEGEVALUE), @EVERYONEID 
                                                FROM @TEAUTHEVERYONEUSER2 TEG2
                                                INNER JOIN ALOM_AUTHORIZATION AUTH ON TEG2.OBJECTID = AUTH.OBJECTID
                                                AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                GROUP BY TEG2.OBJECTID
                                                HAVING MAX(AUTH.PRIVILEGEVALUE) >= {2} and MAX(AUTH.PRIVILEGEVALUE) <> 5", m_NSearchCommonObj.UserID, m_NSearchCommonObj.GroupIDList, PrivilegeValue);
                    }
                    else
                    {
                        StringBuilder RowNumberClause = null, PagingOrderbyClause = null;
                        int StartRecordOnThisPage, EndRecordOnThisPage;
                        string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;
                        string Separator = "";

                        GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage, 
                                                out StartRecordOnThisPage, out EndRecordOnThisPage);

                        if (HierarchyObj.SortSpecificationList.Count == 0)
                        {
                            RowNumberClause = new StringBuilder(" ROW_NUMBER() OVER (ORDER BY OBJECTID ASC) AS ROWNUMBER ");
                            PagingOrderbyClause = new StringBuilder("");
                        }
                        else
                        {

                            RowNumberClause = new StringBuilder(500);
                            RowNumberClause.Append(" ROW_NUMBER() OVER (ORDER BY ");

                            GetSortFields(HierarchyObj.SortSpecificationList, "", null, 
                                            ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                            RowNumberClause.Append(SortFieldNamesWithOrder);
                            RowNumberClause.Append(") AS ROWNUMBER ");

                            PagingOrderbyClause = new StringBuilder(200);
                            PagingOrderbyClause.Append(" order by ");
                            PagingOrderbyClause.Append(SortFieldNamesWithOrder);

                        }

                        TEInsertQuery.AppendFormat(@" DECLARE @TEAUTHEVERYONEUSER TABLE (FILTEREDOBJID NUMERIC, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC)

                                                INSERT INTO @TEAUTHEVERYONEUSER (FILTEREDOBJID, PRIVILEGEVALUE, ACCESSORID)

                                                SELECT OBJECTID, PRIVILEGEVALUE, ACCESSORID FROM (
                                                    SELECT OBJECTID, PRIVILEGEVALUE, ACCESSORID, {3} {10} {4} FROM (
                                                        SELECT TEAG1.OBJECTID,TEAG1.PRIVILEGEVALUE,TEAG1.ACCESSORID FROM @TEAUTHEVERYONEUSER1 TEAG1

                                                        UNION

                                                        SELECT TEG2.OBJECTID, MAX(AUTH.PRIVILEGEVALUE), @EVERYONEID 
                                                        FROM @TEAUTHEVERYONEUSER2 TEG2
                                                        INNER JOIN ALOM_AUTHORIZATION AUTH ON TEG2.OBJECTID = AUTH.OBJECTID
                                                        AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                        GROUP BY TEG2.OBJECTID
                                                        HAVING MAX(AUTH.PRIVILEGEVALUE) >= {2} and MAX(AUTH.PRIVILEGEVALUE) <> 5
                                                    ) as TEOBJECTWITHROWNUMBERS 
                                                    inner join {8} {8}_ALIAS on {8}_ALIAS.{9} = OBJECTID 
                                                ) as TEOBJECTWITHROWNUMBERS 
                                                WHERE ROWNUMBER BETWEEN {5} AND {6} {7}",
                                                    m_NSearchCommonObj.UserID, m_NSearchCommonObj.GroupIDList, PrivilegeValue,
                                                    RowNumberClause.ToString(), SortFieldNamesOnly, 
                                                    StartRecordOnThisPage, EndRecordOnThisPage,
                                                    PagingOrderbyClause.ToString(),
                                                    HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, Separator);

                        PagingCountFromClause = string.Format(@" FROM (
                                                SELECT TEAG1.OBJECTID,TEAG1.PRIVILEGEVALUE,TEAG1.ACCESSORID FROM @TEAUTHEVERYONEUSER1 TEAG1

                                                UNION

                                                SELECT TEG2.OBJECTID, MAX(AUTH.PRIVILEGEVALUE), @EVERYONEID 
                                                FROM @TEAUTHEVERYONEUSER2 TEG2
                                                INNER JOIN ALOM_AUTHORIZATION AUTH ON TEG2.OBJECTID = AUTH.OBJECTID
                                                AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                GROUP BY TEG2.OBJECTID
                                                HAVING MAX(AUTH.PRIVILEGEVALUE) >= {2} and MAX(AUTH.PRIVILEGEVALUE) <> 5) as PagingQuery ",
                                                m_NSearchCommonObj.UserID, m_NSearchCommonObj.GroupIDList, PrivilegeValue);
                    }

                    HierarchyObj.m_EveryOneUserTableName = "TEAUTHEVERYONEUSER";
                    HierarchyObj.m_PrimaryKeyforTableVariable = "FILTEREDOBJID";
                    HierarchyObj.m_ColumnList = "FILTEREDOBJID NUMERIC ";
                }
                else
                {
                    HierarchyObj.m_EveryOneUserTableName = "TEOBJECT";
                    HierarchyObj.m_PrimaryKeyforTableVariable = "FILTEREDOBJID";
                    HierarchyObj.m_ColumnList = "FILTEREDOBJID NUMERIC ";
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private void GetSortFields(List<SortSpecification> SortSpecificationList, string TableNameAlias,
                                    string TablePrimaryKeyName,
                                    ref string SortFieldNamesOnly, ref string SortFieldNamesWithOrder, 
                                    ref string SortFieldNamesAliasedWithOrder)
        {
            StringBuilder sbSortFieldNamesOnly = new StringBuilder(200);
            StringBuilder sbSortFieldNamesWithOrder = new StringBuilder(200);
            StringBuilder sbSortFieldNamesAliasedWithOrder = new StringBuilder(200);

            if (TableNameAlias.Length > 0)
                TableNameAlias = TableNameAlias + ".";

            foreach (SortSpecification ss in SortSpecificationList)
            {
                if (TablePrimaryKeyName == null || !TablePrimaryKeyName.Equals(ss.m_SortFieldName))
                {
                    sbSortFieldNamesOnly.AppendFormat(" {0},", ss.m_SortFieldName);
                }
                    
                sbSortFieldNamesWithOrder.AppendFormat(" {1} {2},", TableNameAlias, ss.m_SortFieldName, ss.m_SortOrder);
                sbSortFieldNamesAliasedWithOrder.AppendFormat(" {0}{1} {2},", TableNameAlias, ss.m_SortFieldName, ss.m_SortOrder);
            }

            if (sbSortFieldNamesOnly.Length != 0)
                SortFieldNamesOnly = ", " + sbSortFieldNamesOnly.Remove(sbSortFieldNamesOnly.Length - 1, 1).ToString();
            if (sbSortFieldNamesWithOrder.Length != 0)
                SortFieldNamesWithOrder = sbSortFieldNamesWithOrder.Remove(sbSortFieldNamesWithOrder.Length - 1, 1).ToString();
            if (sbSortFieldNamesAliasedWithOrder.Length != 0)
                SortFieldNamesAliasedWithOrder = sbSortFieldNamesAliasedWithOrder.Remove(sbSortFieldNamesAliasedWithOrder.Length - 1, 1).ToString();

            sbSortFieldNamesOnly.Remove(0, sbSortFieldNamesOnly.Length);
            sbSortFieldNamesOnly = null;

            sbSortFieldNamesWithOrder.Remove(0, sbSortFieldNamesWithOrder.Length);
            sbSortFieldNamesWithOrder = null;

            sbSortFieldNamesAliasedWithOrder.Remove(0, sbSortFieldNamesAliasedWithOrder.Length);
            sbSortFieldNamesAliasedWithOrder = null;
        }

        private void GetPagingRecordNumbers(int PageNumber, int RecordsPerPage, 
                                            out int StartRecordNumber, out int EndRecordNumber)
        {
            EndRecordNumber = RecordsPerPage * PageNumber;
            StartRecordNumber = EndRecordNumber - RecordsPerPage + 1;
        }

        private OMError FormREandRFInsertQueriesWithAccessControl(HierarchyObject HierarchyObj, ref StringBuilder REandRFInsertQuery)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;

            try
            {

                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObj.m_ParentID != 0)
                    {
                        if (TempHierarchyObj.m_IsRelatedEntity)
                        {
                            // This is a related Entity.
                            if (!TempHierarchyObj.m_RelationshipName.Contains("FILESYSTEM"))
                            {
                                ErrorObj = FormREInsertQueriesWithAccessControl(TempHierarchyObj, ref REandRFInsertQuery);
                                if (ErrorObj != null)
                                    return ErrorObj;
                            }
                        }
                        else
                        {
                            if (!TempHierarchyObj.m_RelationshipName.Contains("FILESYSTEM"))
                            {
                                // This is a related facet.
                                ErrorObj = FormRFInsertQueries(TempHierarchyObj, ref REandRFInsertQuery);
                                if (ErrorObj != null)
                                    return ErrorObj;
                            }
                        }
                    }
                }

                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError FormREInsertQueriesWithAccessControl(HierarchyObject HierarchyObj, ref StringBuilder REandRFInsertQuery)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            HierarchyObject ParentHierarchyObject = null;
            String REPrimaryColumnName = null;
            String FilterClause = null;
            String REInsertColumnNames = null;
            String[] REActualInsertColumnNames = null;
            String TempREInsertColumnNames = null;
            String TempParentInsertColumns = null;
            String[] ParentActualInsertColumns = null;
            int TempREColCounter = 0;
            int PrivilegeValueForRE = -1;
            Char[] delimiterChars = { ',' };

            StringBuilder sbPagingQuery = new StringBuilder();
            StringBuilder sbREandRFInsertQuery = new StringBuilder();
            StringBuilder sbPagingCountQuery = new StringBuilder(200);

            //StringBuilder sbSortFieldsWithSort = new StringBuilder(100);
            //StringBuilder sbSortFieldsWithSortNoAlias = new StringBuilder(100);
            //StringBuilder sbSortFieldNames = new StringBuilder(100);

            try
            {

                if (!HierarchyObj.m_RelationshipName.Contains("FILESYSTEM"))
                {
                    // Get Parent HierarchyObject for given hierarchyobject.
                    // This is done for getting columns for preparing REObjectTable.
                    foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                    {
                        if (TempHierarchyObj.m_TagID == HierarchyObj.m_ParentID)
                        {
                            ParentHierarchyObject = TempHierarchyObj;
                            break;
                        }
                    }

                    string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;
                    GetSortFields(HierarchyObj.SortSpecificationList,
                                    "", null, 
                                    ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                    PrivilegeValueForRE = NSearchCommon.GetPrivilegeValueFromName(HierarchyObj.m_PrivilegeLevel);
                    REPrimaryColumnName = HierarchyObj.m_RelationshipName + HierarchyObj.m_RelatedObjectLevel + " NUMERIC, ";
                    HierarchyObj.m_PrimaryKeyforTableVariable = HierarchyObj.m_RelationshipName + HierarchyObj.m_RelatedObjectLevel;
                    REInsertColumnNames = REPrimaryColumnName + ParentHierarchyObject.m_ColumnList;
                    TempParentInsertColumns = ParentHierarchyObject.m_ColumnList.Replace("NUMERIC", "");
                    TempREInsertColumnNames = REInsertColumnNames.Replace("NUMERIC", "");
                    REActualInsertColumnNames = TempParentInsertColumns.Split(delimiterChars, StringSplitOptions.RemoveEmptyEntries);

                    //If FilterCombination is null then construct table variable, otherwise the table variable will be constructed in GetFilterClause
                    if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                    {
                        REandRFInsertQuery.AppendFormat(" DECLARE @RE{0}{1} TABLE ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, REInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" INSERT INTO @RE{0}{1} ({2},{3})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns);
                        

                        if (HierarchyObj.m_DoPaging && !m_NSearchCommonObj.IsAuthorizationRequired)
                        {
                            sbPagingQuery.AppendFormat(" SELECT {0}, ", HierarchyObj.m_ContainerName);
                        }

                        sbREandRFInsertQuery.AppendFormat(" SELECT ASSEMBLY.{0}, ", HierarchyObj.m_ContainerName);

                        foreach (String TempColumnName in REActualInsertColumnNames)
                        {
                            TempREColCounter++;

                            if (HierarchyObj.m_DoPaging && !m_NSearchCommonObj.IsAuthorizationRequired)
                            {
                                if (TempREColCounter == REActualInsertColumnNames.Length)
                                {
                                    //The last column


                                    sbPagingQuery.AppendFormat(" {0} FROM ( ", TempColumnName.Trim());

                                    if (HierarchyObj.SortSpecificationList.Count == 0)
                                    {
                                        sbREandRFInsertQuery.AppendFormat("{0}.{1}, ROW_NUMBER() OVER (ORDER BY {0}.{1} {2}) AS ROWNUMBER FROM ALOM_ASSEMBLY ASSEMBLY ",
                                            ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim(),
                                            "ASC");

                                    }
                                    else
                                    {

                                        sbREandRFInsertQuery.AppendFormat("{0}.{1}, ROW_NUMBER() OVER (ORDER BY ", 
                                            ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());

                                        sbREandRFInsertQuery.Append(SortFieldNamesAliasedWithOrder);
                                        sbREandRFInsertQuery.Append(" ) AS ROWNUMBER");
                                        sbREandRFInsertQuery.Append(SortFieldNamesOnly);
                                        sbREandRFInsertQuery.AppendFormat(" from ALOM_ASSEMBLY ASSEMBLY ");
                                        
                                    }

                                    sbPagingCountQuery.Append(" from ALOM_ASSEMBLY ASSEMBLY ");
                                }
                                else
                                {
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} , ",
                                        ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());

                                    sbPagingQuery.AppendFormat(" {0}, ", TempColumnName.Trim());
                                }
                            }
                            else
                            {
                                if (TempREColCounter == REActualInsertColumnNames.Length)
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} FROM ALOM_ASSEMBLY ASSEMBLY ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                else
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                            }
                        }


                        sbREandRFInsertQuery.AppendFormat(" INNER JOIN @{0} {1}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS");
                        if (HierarchyObj.m_ContainerName.ToUpper() == "CONTAINERID")
                        {
                            sbREandRFInsertQuery.AppendFormat(" ON ASSEMBLY.CONTAINSID = {0}.{1}",
                                ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS",
                                ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                        }
                        else
                        {
                            sbREandRFInsertQuery.AppendFormat(" ON ASSEMBLY.CONTAINERID = {0}.{1}", 
                                ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", 
                                ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                        }
                        sbREandRFInsertQuery.AppendFormat(" AND ASSEMBLY.RELATIONSHIPTYPEID = {0}", HierarchyObj.m_RelationshipTypeID);

                        if (HierarchyObj.m_DoPaging && !m_NSearchCommonObj.IsAuthorizationRequired)
                        {
                            sbPagingCountQuery.AppendFormat(" INNER JOIN @{0} {1}", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS");
                            if (HierarchyObj.m_ContainerName.ToUpper() == "CONTAINERID")
                                sbPagingCountQuery.AppendFormat(" ON ASSEMBLY.CONTAINSID = {0}.{1}", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            else
                                sbPagingCountQuery.AppendFormat(" ON ASSEMBLY.CONTAINERID = {0}.{1}", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            sbPagingCountQuery.AppendFormat(" AND ASSEMBLY.RELATIONSHIPTYPEID = {0}", HierarchyObj.m_RelationshipTypeID);
                        }
                    }

                    // Prepare FilterClause for HierarchyObject
                    ErrorObj = GetFilterClause(HierarchyObj, ref FilterClause);

                    //If FilterCombination is null then construct table variable, otherwise the table variable will be constructed in GetFilterClause
                    if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                    {
                        if (FilterClause != null && FilterClause != "")
                        {
                            if (HierarchyObj.m_ContainerName.ToUpper() == "CONTAINERID")
                            {
                                sbREandRFInsertQuery.AppendFormat(" INNER JOIN {0} {1} ON {1}.{2} = ASSEMBLY.CONTAINERID ",
                                    HierarchyObj.m_TableName, HierarchyObj.m_TableName + "_ALIAS",
                                    HierarchyObj.m_PrimaryKeyName);
                            }
                            else
                            {
                                sbREandRFInsertQuery.AppendFormat(" INNER JOIN {0} {1} ON {1}.{2} = ASSEMBLY.CONTAINSID ",
                                    HierarchyObj.m_TableName, HierarchyObj.m_TableName + "_ALIAS",
                                    HierarchyObj.m_PrimaryKeyName);
                            }

                            if (HierarchyObj.m_DoPaging && !m_NSearchCommonObj.IsAuthorizationRequired)
                            {
                                if (HierarchyObj.m_ContainerName.ToUpper() == "CONTAINERID")
                                {
                                    sbPagingCountQuery.AppendFormat(" INNER JOIN {0} {1} ON {1}.{2} = ASSEMBLY.CONTAINERID ",
                                        HierarchyObj.m_TableName, HierarchyObj.m_TableName + "_ALIAS", HierarchyObj.m_PrimaryKeyName);
                                }
                                else
                                {
                                    sbPagingCountQuery.AppendFormat(" INNER JOIN {0} {1} ON {1}.{2} = ASSEMBLY.CONTAINSID ",
                                        HierarchyObj.m_TableName, HierarchyObj.m_TableName + "_ALIAS", HierarchyObj.m_PrimaryKeyName);
                                }
                            }
                        }
                        else
                        {
                            if (HierarchyObj.m_DoPaging && !m_NSearchCommonObj.IsAuthorizationRequired)
                            {
                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                }
                                else
                                {
                                    sbREandRFInsertQuery.AppendFormat(" inner join {0} {0}_ALIAS on assembly.{1} = {0}_ALIAS.{2}",
                                        HierarchyObj.m_TableName, HierarchyObj.m_ContainerName.ToUpper(),
                                        HierarchyObj.m_PrimaryKeyName);
                                }

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                }
                                else
                                {
                                    sbPagingCountQuery.AppendFormat(" inner join {0} {0}_ALIAS on assembly.{1} = {0}_ALIAS.{2}",
                                        HierarchyObj.m_TableName, HierarchyObj.m_ContainerName.ToUpper(),
                                        HierarchyObj.m_PrimaryKeyName);
                                }

                            }
                        }
                    }

                    sbREandRFInsertQuery.Append(FilterClause);
                    sbPagingCountQuery.Append(FilterClause);

                    //The FilterClause also needs to be added for the paging query, but under a certain
                    //condition only

                    //Add sbREandRFInsertQuery to the REandRFInsertQuery
                    if (HierarchyObj.m_DoPaging && string.IsNullOrEmpty(HierarchyObj.m_FilterCombination) && 
                            !m_NSearchCommonObj.IsAuthorizationRequired)
                    {
                        int StartRecordOnThisPage, EndRecordOnThisPage;

                        GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage,
                                                    out StartRecordOnThisPage, out EndRecordOnThisPage);

                        REandRFInsertQuery.Append(sbPagingQuery.ToString());
                        REandRFInsertQuery.Append(sbREandRFInsertQuery.ToString());
                        REandRFInsertQuery.AppendFormat(" ) AS TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1} ", 
                            StartRecordOnThisPage, EndRecordOnThisPage);

                        if (HierarchyObj.SortSpecificationList.Count == 0)
                        {
                        }
                        else
                        {
                            REandRFInsertQuery.Append(" order by ");

                            REandRFInsertQuery.Append(SortFieldNamesWithOrder);
                            REandRFInsertQuery.Append(" ");
                        }

                        PagingCountFromClause = sbPagingCountQuery.ToString();
                    }
                    else
                    {
                        REandRFInsertQuery.Append(sbREandRFInsertQuery.ToString());

                    }

                    sbREandRFInsertQuery.Remove(0, sbREandRFInsertQuery.Length);
                    sbREandRFInsertQuery = null;
                    sbPagingQuery.Remove(0, sbPagingQuery.Length);
                    sbPagingQuery = null;
                    sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                    sbPagingCountQuery = null;

                    if (m_NSearchCommonObj.IsAuthorizationRequired)
                    {
                        // New Authorization for USER,GROUP and EVERYONE starts.
                        //1
                        REandRFInsertQuery.AppendFormat(" DECLARE @REAUTHGROUP1{0} TABLE ({1}, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC) ", HierarchyObj.m_ActualTagID, REInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" INSERT INTO @REAUTHGROUP1{0} ({1}, PRIVILEGEVALUE, ACCESSORID) ", HierarchyObj.m_ActualTagID, TempREInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" SELECT REO.{0} ,{1}, MAX(AUTH.PRIVILEGEVALUE) , {2} FROM @RE{3}{4} REO ", HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns, m_NSearchCommonObj.UserID, HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID);
                        REandRFInsertQuery.AppendFormat(" INNER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = REO.{0} ", HierarchyObj.m_PrimaryKeyforTableVariable);
                        REandRFInsertQuery.AppendFormat(" AND AUTH.ACCESSORID IN ({0})", m_NSearchCommonObj.GroupIDList);
                        REandRFInsertQuery.AppendFormat(" GROUP BY REO.{0} ", HierarchyObj.m_PrimaryKeyforTableVariable.Trim());

                        ParentActualInsertColumns = TempParentInsertColumns.Split(delimiterChars, StringSplitOptions.RemoveEmptyEntries);
                        foreach (String TempParentCol in ParentActualInsertColumns)
                        {
                            REandRFInsertQuery.AppendFormat(",REO.{0}", TempParentCol.Trim());
                        }
                        REandRFInsertQuery.AppendFormat(" HAVING MAX(AUTH.PRIVILEGEVALUE) >= {0} AND MAX(AUTH.PRIVILEGEVALUE) <> 5 ", PrivilegeValueForRE);

                        //2
                        REandRFInsertQuery.AppendFormat(" DECLARE @REAUTHGROUP2{0} TABLE ({1}, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC) ", HierarchyObj.m_ActualTagID, REInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" INSERT INTO @REAUTHGROUP2{0} ({1}, PRIVILEGEVALUE, ACCESSORID) ", HierarchyObj.m_ActualTagID, TempREInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" SELECT REO.{0} ,{1}, MAX(AUTH.PRIVILEGEVALUE) , {2} FROM @RE{3}{4} REO ", HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns, m_NSearchCommonObj.UserID, HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID);
                        REandRFInsertQuery.AppendFormat(" LEFT OUTER JOIN ALOM_AUTHORIZATION AUTH ON AUTH.OBJECTID = REO.{0} ", HierarchyObj.m_PrimaryKeyforTableVariable);
                        REandRFInsertQuery.AppendFormat(" AND AUTH.ACCESSORID IN ({0})", m_NSearchCommonObj.GroupIDList);
                        REandRFInsertQuery.AppendFormat(" GROUP BY REO.{0},AUTH.ACCESSORID ", HierarchyObj.m_PrimaryKeyforTableVariable.Trim());

                        //ParentActualInsertColumns = TempParentInsertColumns.Split(delimiterChars, StringSplitOptions.RemoveEmptyEntries);
                        foreach (String TempParentCol in ParentActualInsertColumns)
                        {
                            REandRFInsertQuery.AppendFormat(",REO.{0}", TempParentCol.Trim());
                        }
                        REandRFInsertQuery.AppendFormat(" HAVING MAX(AUTH.PRIVILEGEVALUE) IS NULL ");

                        if (!HierarchyObj.m_DoPaging)
                        {
                            //CHECK IF USER HAS DENY ACCESS FOR GROUP1
                            REandRFInsertQuery.AppendFormat(" DECLARE @REAUTHEVERYONEUSER{0} TABLE ({1}, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC ) ", HierarchyObj.m_ActualTagID, REInsertColumnNames);
                            REandRFInsertQuery.AppendFormat(" INSERT INTO @REAUTHEVERYONEUSER{0} ( {1} , PRIVILEGEVALUE,ACCESSORID ) ", HierarchyObj.m_ActualTagID, TempREInsertColumnNames);
                            REandRFInsertQuery.AppendFormat(" SELECT {0},{1}, PRIVILEGEVALUE,ACCESSORID FROM @REAUTHGROUP1{2} ", HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns, HierarchyObj.m_ActualTagID, m_NSearchCommonObj.UserID);

                            REandRFInsertQuery.AppendFormat(" UNION ");

                            REandRFInsertQuery.AppendFormat(" SELECT REAG2.{0}", HierarchyObj.m_PrimaryKeyforTableVariable.Trim());
                            foreach (String TempParentCol in ParentActualInsertColumns)
                            {
                                REandRFInsertQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                            }

                            REandRFInsertQuery.AppendFormat(@",AUTH.PRIVILEGEVALUE, @EVERYONEID 
                                                           FROM @REAUTHGROUP2{0} REAG2
                                                           INNER JOIN ALOM_AUTHORIZATION AUTH ON REAG2.{1} = AUTH.OBJECTID
                                                           AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                           GROUP BY REAG2.{1}, AUTH.PRIVILEGEVALUE ", HierarchyObj.m_ActualTagID, HierarchyObj.m_PrimaryKeyforTableVariable.Trim());

                            foreach (String TempParentCol in ParentActualInsertColumns)
                            {
                                REandRFInsertQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                            }
                            REandRFInsertQuery.AppendFormat(" HAVING MAX(AUTH.PRIVILEGEVALUE) >= 2 and MAX(AUTH.PRIVILEGEVALUE) <> 5 ");
                        }
                        else
                        {
                            StringBuilder RowNumberClause = null, PagingOrderbyClause = null;
                            int StartRecordOnThisPage, EndRecordOnThisPage;

                            if (HierarchyObj.SortSpecificationList.Count == 0)
                            {
                                RowNumberClause = new StringBuilder(" ROW_NUMBER() OVER (ORDER BY FILTEREDOBJID ASC) AS ROWNUMBER ");
                                PagingOrderbyClause = new StringBuilder("");
                            }
                            else
                            {

                                RowNumberClause = new StringBuilder(500);
                                RowNumberClause.Append(" ROW_NUMBER() OVER (ORDER BY ");

                                RowNumberClause.Append(SortFieldNamesWithOrder);
                                RowNumberClause.AppendFormat(") AS ROWNUMBER {0} ", SortFieldNamesOnly);

                                PagingOrderbyClause = new StringBuilder(200);
                                PagingOrderbyClause.Append(" order by ");
                                PagingOrderbyClause.Append(SortFieldNamesWithOrder);

                            }

                            //CHECK IF USER HAS DENY ACCESS FOR GROUP1
                            REandRFInsertQuery.AppendFormat(" DECLARE @REAUTHEVERYONEUSER{0} TABLE ({1}, PRIVILEGEVALUE NUMERIC,ACCESSORID NUMERIC ) ", HierarchyObj.m_ActualTagID, REInsertColumnNames);
                            REandRFInsertQuery.AppendFormat(" INSERT INTO @REAUTHEVERYONEUSER{0} ( {1} , PRIVILEGEVALUE,ACCESSORID ) ", HierarchyObj.m_ActualTagID, TempREInsertColumnNames);
                            
                            
                            REandRFInsertQuery.AppendFormat(" SELECT {0},{1}, PRIVILEGEVALUE,ACCESSORID FROM ( ", 
                                                                HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns);
                            REandRFInsertQuery.AppendFormat(" SELECT {0},{1}, PRIVILEGEVALUE,ACCESSORID, ",
                                                                HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns);
                            REandRFInsertQuery.Append(RowNumberClause);
                            REandRFInsertQuery.Append(" from ( ");

                            //Clear it and make the PagingCountFromClause
                            if (sbPagingCountQuery == null)
                            {
                                sbPagingCountQuery = new StringBuilder(200);
                            }
                            else
                            {
                                sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                            }

                            REandRFInsertQuery.AppendFormat(" SELECT {0},{1}, PRIVILEGEVALUE,ACCESSORID FROM @REAUTHGROUP1{2} ", HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns, HierarchyObj.m_ActualTagID, m_NSearchCommonObj.UserID);
                            sbPagingCountQuery.AppendFormat(" from (SELECT {0},{1}, PRIVILEGEVALUE,ACCESSORID FROM @REAUTHGROUP1{2} ", HierarchyObj.m_PrimaryKeyforTableVariable, TempParentInsertColumns, HierarchyObj.m_ActualTagID, m_NSearchCommonObj.UserID);

                            REandRFInsertQuery.AppendFormat(" UNION ");
                            sbPagingCountQuery.AppendFormat(" UNION ");

                            REandRFInsertQuery.AppendFormat(" SELECT REAG2.{0}", HierarchyObj.m_PrimaryKeyforTableVariable.Trim());
                            sbPagingCountQuery.AppendFormat(" SELECT REAG2.{0}", HierarchyObj.m_PrimaryKeyforTableVariable.Trim());
                            foreach (String TempParentCol in ParentActualInsertColumns)
                            {
                                REandRFInsertQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                                sbPagingCountQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                            }

                            REandRFInsertQuery.AppendFormat(@",AUTH.PRIVILEGEVALUE, @EVERYONEID 
                                                           FROM @REAUTHGROUP2{0} REAG2
                                                           INNER JOIN ALOM_AUTHORIZATION AUTH ON REAG2.{1} = AUTH.OBJECTID
                                                           AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                           GROUP BY REAG2.{1}, AUTH.PRIVILEGEVALUE ", HierarchyObj.m_ActualTagID, HierarchyObj.m_PrimaryKeyforTableVariable.Trim());
                            sbPagingCountQuery.AppendFormat(@",AUTH.PRIVILEGEVALUE, @EVERYONEID 
                                                           FROM @REAUTHGROUP2{0} REAG2
                                                           INNER JOIN ALOM_AUTHORIZATION AUTH ON REAG2.{1} = AUTH.OBJECTID
                                                           AND AUTH.ACCESSORID IN (@EVERYONEID)
                                                           GROUP BY REAG2.{1}, AUTH.PRIVILEGEVALUE ", HierarchyObj.m_ActualTagID, HierarchyObj.m_PrimaryKeyforTableVariable.Trim());

                            foreach (String TempParentCol in ParentActualInsertColumns)
                            {
                                REandRFInsertQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                                sbPagingCountQuery.AppendFormat(",REAG2.{0}", TempParentCol.Trim());
                            }
                            REandRFInsertQuery.Append(" HAVING MAX(AUTH.PRIVILEGEVALUE) >= 2 and MAX(AUTH.PRIVILEGEVALUE) <> 5 ");
                            sbPagingCountQuery.Append(" HAVING MAX(AUTH.PRIVILEGEVALUE) >= 2 and MAX(AUTH.PRIVILEGEVALUE) <> 5 ");
                            sbPagingCountQuery.Append(" ) as PagingQuery ");

                            REandRFInsertQuery.Append(") as teobjectwithrownumbers ");
                            REandRFInsertQuery.AppendFormat("INNER JOIN {0} {0}_ALIAS on {0}_ALIAS.{1} = FILTEREDOBJID ", 
                                                            HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);

                            GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage,
                                                    out StartRecordOnThisPage, out EndRecordOnThisPage);
                            
                            REandRFInsertQuery.Append(") as teobjectwithrownumbers ");
                            REandRFInsertQuery.AppendFormat("where rownumber between {0} and {1} ", StartRecordOnThisPage, EndRecordOnThisPage);
                            REandRFInsertQuery.Append(PagingOrderbyClause);

                            PagingCountFromClause = sbPagingCountQuery.ToString();
                        }


                        HierarchyObj.m_ColumnList = REInsertColumnNames;
                        HierarchyObj.m_EveryOneUserTableName = "REAUTHEVERYONEUSER" + HierarchyObj.m_ActualTagID;
                    }
                    else
                    {
                        HierarchyObj.m_ColumnList = REInsertColumnNames;
                        HierarchyObj.m_EveryOneUserTableName = "RE" + HierarchyObj.m_RelationshipName + HierarchyObj.m_ActualTagID.ToString();
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareDynamicSQL",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        private OMError FormRFInsertQueries(HierarchyObject HierarchyObj, ref StringBuilder REandRFInsertQuery)
        {
            OMError ErrorObj = null;
            OMError ReturnErrorObj = null;
            HierarchyObject ParentHierarchyObject = null;
            int PrivilegeValueForRE = -1;
            String RFPrimaryColumnName = null;
            String FilterClause = null;
            String RFInsertColumnNames = null;
            String[] RFActualInsertColumnNames = null;
            String TempRFInsertColumnNames = null;
            String TempParentInsertColumns = null;
            int TempRFColCounter = 0;
            Char[] delimiterChars = { ',' };

            StringBuilder sbPagingQuery = new StringBuilder();
            StringBuilder sbPagingCountQuery = new StringBuilder(200);
            StringBuilder sbREandRFInsertQuery = new StringBuilder(500);

            //StringBuilder sbSortFieldsWithSort = new StringBuilder(100);
            //StringBuilder sbSortFieldNames = new StringBuilder(100);
            
            try
            {

                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (TempHierarchyObj.m_TagID == HierarchyObj.m_ParentID)
                    {
                        ParentHierarchyObject = TempHierarchyObj;
                        break;
                    }
                }

                PrivilegeValueForRE = NSearchCommon.GetPrivilegeValueFromName(HierarchyObj.m_PrivilegeLevel);

                RFPrimaryColumnName = HierarchyObj.m_RelationshipName + HierarchyObj.m_RelatedObjectLevel + " NUMERIC, ";
                HierarchyObj.m_PrimaryKeyforTableVariable = HierarchyObj.m_RelationshipName + HierarchyObj.m_RelatedObjectLevel;
                RFInsertColumnNames = RFPrimaryColumnName + ParentHierarchyObject.m_ColumnList;
                TempParentInsertColumns = ParentHierarchyObject.m_ColumnList.Replace("NUMERIC", "");
                TempRFInsertColumnNames = RFInsertColumnNames.Replace("NUMERIC", "");
                RFActualInsertColumnNames = TempParentInsertColumns.Split(delimiterChars, StringSplitOptions.RemoveEmptyEntries);

                //Step 1 :
                //If FilterCombination is null then construct table variable, otherwise the table variable will be constructed in GetFilterClause
                if (HierarchyObj.m_FilterCombination == null || HierarchyObj.m_FilterCombination == "")
                {
                    if (ParentHierarchyObject.m_ObjectType.ToUpper() == "ENTITY")
                    {
                        REandRFInsertQuery.AppendFormat(" DECLARE @RF{0}{1} TABLE ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, RFInsertColumnNames);
                        REandRFInsertQuery.AppendFormat(" INSERT INTO @RF{0}{1} ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, TempRFInsertColumnNames);

                        if (HierarchyObj.m_DoPaging)
                        {
                            sbPagingQuery.Append(" SELECT FACETID, ");
                        }

                        sbREandRFInsertQuery.Append(" SELECT ASSOENTFCT.FACETID, ");

                        foreach (String TempColumnName in RFActualInsertColumnNames)
                        {
                            TempRFColCounter++;

                            if (HierarchyObj.m_DoPaging)
                            {
                                if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                {
                                    if (HierarchyObj.SortSpecificationList.Count == 0)
                                    {
                                        sbREandRFInsertQuery.AppendFormat("{0}.{1}, row_number() OVER (ORDER BY (ASSOENTFCT.FACETID) {2}) AS ROWNUMBER  FROM ALOM_ASSOENTITYFACET ASSOENTFCT ",
                                            ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim(), "ASC");
                                        sbPagingQuery.AppendFormat("{0} FROM ( ", TempColumnName.Trim());

                                        sbPagingCountQuery.Append(" FROM ALOM_ASSOENTITYFACET ASSOENTFCT ");
                                    }
                                    else
                                    {
                                        string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                                        sbREandRFInsertQuery.AppendFormat("{0}.{1}, row_number() OVER (ORDER BY ", 
                                            ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());

                                        GetSortFields(HierarchyObj.SortSpecificationList,
                                                HierarchyObj.m_TableName + "_ALIAS", null, 
                                                ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                                        sbREandRFInsertQuery.Append(SortFieldNamesAliasedWithOrder);
                                        sbREandRFInsertQuery.Append(" ) AS ROWNUMBER");
                                        sbREandRFInsertQuery.Append(SortFieldNamesOnly);
                                        sbREandRFInsertQuery.AppendFormat(" FROM ALOM_ASSOENTITYFACET ASSOENTFCT ");

                                        sbPagingQuery.AppendFormat("{0} FROM ( ", TempColumnName.Trim());

                                        sbPagingCountQuery.Append(" FROM ALOM_ASSOENTITYFACET ASSOENTFCT ");
                                    }
                                }
                                else
                                {
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                    sbPagingQuery.AppendFormat("{0} , ", TempColumnName.Trim());
                                }
                            }
                            else
                            {
                                if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                {
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} FROM ALOM_ASSOENTITYFACET ASSOENTFCT ",
                                        ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                }
                                else
                                {
                                    sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                }
                            }

                        }

                        sbREandRFInsertQuery.AppendFormat(" INNER JOIN @{0} {1} ON ASSOENTFCT.ENTITYID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                        sbPagingCountQuery.AppendFormat(" INNER JOIN @{0} {1} ON ASSOENTFCT.ENTITYID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);

                        
                    }
                    else if (ParentHierarchyObject.m_ObjectType.ToUpper() == "FACET")//New code for facet facet series
                    {
                        switch(HierarchyObj.m_ObjectType.ToUpper())
                        {
                            //Case for Facet-Facet relationship
                            case "FACET":
                                {
                                    REandRFInsertQuery.AppendFormat(" DECLARE @RF{0}{1} TABLE ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, RFInsertColumnNames);
                                    REandRFInsertQuery.AppendFormat(" INSERT INTO @RF{0}{1} ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, TempRFInsertColumnNames);

                                    if (HierarchyObj.m_DoPaging)
                                    {
                                        sbPagingQuery.Append(" SELECT CONTAINSID, ");
                                    }

                                    sbREandRFInsertQuery.Append(" SELECT ALASSEMBLY.CONTAINSID, ");

                                    foreach (String TempColumnName in RFActualInsertColumnNames)
                                    {
                                        TempRFColCounter++;

                                        if (HierarchyObj.m_DoPaging)
                                        {
                                            if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                            {
                                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                                {
                                                    sbREandRFInsertQuery.AppendFormat("{0}.{1}, row_number() OVER (ORDER BY (ALASSEMBLY.CONTAINSID) {2}) AS ROWNUMBER FROM ALOM_ASSEMBLY ALASSEMBLY ",
                                                        ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());

                                                    sbPagingQuery.AppendFormat("{0} FROM ( ", TempColumnName.Trim());

                                                    sbPagingCountQuery.Append(" FROM ALOM_ASSEMBLY ALASSEMBLY ");
                                                }
                                                else
                                                {
                                                    string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                                                    sbREandRFInsertQuery.AppendFormat("{0}.{1}, row_number() OVER (ORDER BY ",
                                                        ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());

                                                    GetSortFields(HierarchyObj.SortSpecificationList,
                                                                    ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", null, 
                                                                    ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                                                    sbREandRFInsertQuery.Append(SortFieldNamesAliasedWithOrder);
                                                    sbREandRFInsertQuery.Append(" ) AS ROWNUMBER");
                                                    sbREandRFInsertQuery.Append(SortFieldNamesOnly);
                                                    sbREandRFInsertQuery.AppendFormat(" FROM ALOM_ASSEMBLY ALASSEMBLY ");

                                                    sbPagingQuery.AppendFormat("{0} FROM ( ", TempColumnName.Trim());

                                                    sbPagingCountQuery.Append(" FROM ALOM_ASSEMBLY ALASSEMBLY ");
                                                }
                                            }
                                            else
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                                sbPagingQuery.AppendFormat("{1} , ", TempColumnName.Trim());
                                            }
                                        }
                                        else
                                        {
                                            if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} FROM ALOM_ASSEMBLY ALASSEMBLY ",
                                                    ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                            }
                                            else
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                            }

                                        }
                                    }

                                    sbREandRFInsertQuery.AppendFormat(" INNER JOIN @{0} {1} ON ALASSEMBLY.CONTAINERID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                    sbPagingCountQuery.AppendFormat(" INNER JOIN @{0} {1} ON ALASSEMBLY.CONTAINERID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);

                                    
                                }
                                break;

                            /* Special case for Entity-Facet relationship where facet data is expected on top of entity data. 
                             * e.g. Consider following hierarchy
                             * Hierarchy                                      | Search Xml
                             * ----------                                     | ----------
                             * TE                                             | TE   
                             *  |->RE1                                        |   |->RE
                             *      |->RF1.1                                  |        |->RF1.1
                             *  |->RF2                                        |   |->RF2  
                             *                                                |        |->RE2 
                             * RE2                                            |  
                             *  |->RF2                                        |  
                             * ----------------------------------------------------------------------------------------------------
                             * Note: Here TE and RE2 are associated with RF2. While retrieving RF2 user may want to retrieve RE2 instances.
                             * So the RF2 instance will be retrieved first and then RE2 instance.
                             */ 
                            case "ENTITY":
                                {
                                    REandRFInsertQuery.AppendFormat(" DECLARE @RF{0}{1} TABLE ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, RFInsertColumnNames);
                                    REandRFInsertQuery.AppendFormat(" INSERT INTO @RF{0}{1} ({2})", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID, TempRFInsertColumnNames);

                                    if (HierarchyObj.m_DoPaging)
                                    {
                                        sbPagingQuery.Append(" SELECT ENTITYID, ");
                                    }
                                    sbREandRFInsertQuery.Append(" SELECT ASSOENTFCT.ENTITYID, ");

                                    foreach (String TempColumnName in RFActualInsertColumnNames)
                                    {
                                        TempRFColCounter++;

                                        if (HierarchyObj.m_DoPaging)
                                        {
                                            if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} FROM ALOM_ASSOENTITYFACET ASSOENTFCT ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                                sbPagingQuery.AppendFormat("{0} FROM (", TempColumnName.Trim());
                                                sbPagingCountQuery.AppendFormat("{0}.{1} FROM ALOM_ASSOENTITYFACET ASSOENTFCT ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                            }
                                            else
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                                sbPagingQuery.AppendFormat("{0} , ", TempColumnName.Trim());
                                            }
                                        }
                                        else
                                        {
                                            if (TempRFColCounter == RFActualInsertColumnNames.Length)
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} FROM ALOM_ASSOENTITYFACET ASSOENTFCT ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                            }
                                            else
                                            {
                                                sbREandRFInsertQuery.AppendFormat("{0}.{1} , ", ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", TempColumnName.Trim());
                                            }
                                        }
                                    }

                                    sbREandRFInsertQuery.AppendFormat(" INNER JOIN @{0} {1} ON ASSOENTFCT.FACETID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                    sbPagingCountQuery.AppendFormat(" INNER JOIN @{0} {1} ON ASSOENTFCT.FACETID = {1}.{2} ", ParentHierarchyObject.m_EveryOneUserTableName.Trim(), ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);

                                    
                                }
                                break;
                        }

                    }
                }

                // For filter clause

                ErrorObj = GetFilterClause(HierarchyObj, ref FilterClause);

                //If FilterCombination is null then take the inner join on ALOM_ASSOENTITYFACET and specify WHERE clause if Step 1 executes
                if (string.IsNullOrEmpty(HierarchyObj.m_FilterCombination))
                {
                    if (ParentHierarchyObject.m_ObjectType.ToUpper() == "ENTITY")
                    {
                        if (FilterClause != null && FilterClause != "")
                        {
                            sbREandRFInsertQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ASSOENTFCT.FACETID ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            sbREandRFInsertQuery.AppendFormat(" AND ASSOENTFCT.ENTITYTYPEID = {0} AND  ASSOENTFCT.FACETTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);

                            sbPagingCountQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ASSOENTFCT.FACETID ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                            sbPagingCountQuery.AppendFormat(" AND ASSOENTFCT.ENTITYTYPEID = {0} AND  ASSOENTFCT.FACETTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);

                        }
                        else
                        {
                            if (HierarchyObj.m_DoPaging)
                            {
                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {

                                }
                                else
                                {
                                    sbREandRFInsertQuery.AppendFormat(" inner join {0} {0}_ALIAS on {0}_ALIAS.{1} = ASSOENTFCT.FACETID ",
                                                                        HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                }
                            }
                            

                            sbREandRFInsertQuery.AppendFormat(" WHERE ASSOENTFCT.ENTITYTYPEID = {0} AND  ASSOENTFCT.FACETTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                            sbPagingCountQuery.AppendFormat(" WHERE ASSOENTFCT.ENTITYTYPEID = {0} AND  ASSOENTFCT.FACETTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                        }
                    }
                    else if (ParentHierarchyObject.m_ObjectType.ToUpper() == "FACET")//New code for facet facet series
                    {
                        switch (HierarchyObj.m_ObjectType.ToUpper())
                        {
                            //Case for FACET-FACET relationship
                            case "FACET":
                                if (FilterClause != null && FilterClause != "")
                                {
                                    sbREandRFInsertQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ALASSEMBLY.{2} ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_ContainerName);
                                    sbREandRFInsertQuery.AppendFormat(" AND ALASSEMBLY.CONTAINERTYPEID = {0} AND  ALASSEMBLY.CONTAINSTYPEID = {1} AND (ALASSEMBLY.RELATIONSHIPTYPEID = {2} OR ALASSEMBLY.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);

                                    sbPagingCountQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ALASSEMBLY.{2} ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_ContainerName);
                                    sbPagingCountQuery.AppendFormat(" AND ALASSEMBLY.CONTAINERTYPEID = {0} AND  ALASSEMBLY.CONTAINSTYPEID = {1} AND (ALASSEMBLY.RELATIONSHIPTYPEID = {2} OR ALASSEMBLY.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);

                                }
                                else
                                {
                                    if (HierarchyObj.m_DoPaging)
                                    {
                                        if (HierarchyObj.SortSpecificationList.Count == 0)
                                        {

                                        }
                                        else
                                        {
                                            sbREandRFInsertQuery.AppendFormat(" inner join {0} {0}_ALIAS on {0}_ALIAS.{1} = ALASSEMBLY.CONTAINSID ",
                                                                                HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                        }
                                    }

                                    sbREandRFInsertQuery.AppendFormat(" WHERE ALASSEMBLY.CONTAINERTYPEID = {0} AND  ALASSEMBLY.CONTAINSTYPEID = {1} AND (ALASSEMBLY.RELATIONSHIPTYPEID = {2} OR ALASSEMBLY.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                                    sbPagingCountQuery.AppendFormat(" WHERE ALASSEMBLY.CONTAINERTYPEID = {0} AND  ALASSEMBLY.CONTAINSTYPEID = {1} AND (ALASSEMBLY.RELATIONSHIPTYPEID = {2} OR ALASSEMBLY.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                                }
                                break;

                            //Special case for Entity-Facet relationship where facet data is expected on top of entity data.
                            case "ENTITY":

                                if (FilterClause != null && FilterClause != "")
                                {
                                    sbREandRFInsertQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ASSOENTFCT.ENTITYID ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                    sbREandRFInsertQuery.AppendFormat(" AND ASSOENTFCT.FACETTYPEID = {0} AND  ASSOENTFCT.ENTITYTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);

                                    sbPagingCountQuery.AppendFormat(" INNER JOIN {0} {0}_ALIAS ON {0}_ALIAS.{1} = ASSOENTFCT.ENTITYID ", HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName, ParentHierarchyObject.m_EveryOneUserTableName.Trim() + "_ALIAS", ParentHierarchyObject.m_PrimaryKeyforTableVariable);
                                    sbPagingCountQuery.AppendFormat(" AND ASSOENTFCT.FACETTYPEID = {0} AND  ASSOENTFCT.ENTITYTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                                }
                                else
                                {
                                    if (HierarchyObj.m_DoPaging)
                                    {
                                        if (HierarchyObj.SortSpecificationList.Count == 0)
                                        {

                                        }
                                        else
                                        {
                                            sbREandRFInsertQuery.AppendFormat(" inner join {0} {0}_ALIAS on {0}_ALIAS.{1} = ASSOENTFCT.ENTITYID ",
                                                                                HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                        }
                                    }

                                    sbREandRFInsertQuery.AppendFormat(" WHERE ASSOENTFCT.FACETTYPEID = {0} AND  ASSOENTFCT.ENTITYTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                                    sbPagingCountQuery.AppendFormat(" WHERE ASSOENTFCT.FACETTYPEID = {0} AND  ASSOENTFCT.ENTITYTYPEID = {1} AND (ASSOENTFCT.RELATIONSHIPTYPEID = {2} OR ASSOENTFCT.RELATIONSHIPTYPEID IS NULL)", ParentHierarchyObject.m_ObjectTypeID, HierarchyObj.m_ObjectTypeID, HierarchyObj.m_RelationshipTypeID);
                                }

                                break;
                        }
                    }
                }


                if (HierarchyObj.m_DoPaging && string.IsNullOrEmpty(HierarchyObj.m_FilterCombination))
                {
                    int StartRecordOnThisPage, EndRecordOnThisPage;

                    GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage,
                                                out StartRecordOnThisPage, out EndRecordOnThisPage);

                    REandRFInsertQuery.Append(sbPagingQuery.ToString());
                    REandRFInsertQuery.Append(sbREandRFInsertQuery.ToString());
                    REandRFInsertQuery.Append(FilterClause);
                    REandRFInsertQuery.AppendFormat(") as TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1} ",
                                                StartRecordOnThisPage, EndRecordOnThisPage);

                    if (HierarchyObj.SortSpecificationList.Count == 0)
                    {
                    }
                    else
                    {
                        string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                        REandRFInsertQuery.Append(" order by ");

                        GetSortFields(HierarchyObj.SortSpecificationList,
                                        "", null, 
                                        ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                        REandRFInsertQuery.Append(SortFieldNamesWithOrder);
                        REandRFInsertQuery.Append(" ");
                    }

                    PagingCountFromClause = sbPagingCountQuery.ToString() + FilterClause;
                }
                else
                {
                    REandRFInsertQuery.Append(sbREandRFInsertQuery.ToString());
                    REandRFInsertQuery.Append(FilterClause);
                }

                sbPagingQuery.Remove(0, sbPagingQuery.Length);
                sbPagingQuery = null;
                sbREandRFInsertQuery.Remove(0, sbREandRFInsertQuery.Length);
                sbREandRFInsertQuery = null;
                sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                sbPagingCountQuery = null;

                HierarchyObj.m_EveryOneUserTableName = "RF" + HierarchyObj.m_RelationshipName + HierarchyObj.m_ActualTagID;
                HierarchyObj.m_ColumnList = RFInsertColumnNames;

                return ErrorObj;

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in FormRFInsertQuries",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// <Purpose>
        /// Generate uniqueid.
        /// It uses GUID to generate the nextid.
        /// It creates a GUId and remove the first 3 bytes and
        /// convert the last byte into integer        ///
        /// </Purpose>
        /// </summary>
        /// <param name="UniqueID">Returns a long UnqiueID</param>
        /// <returns></returns>
        private OMError GetUniqueId(ref long UniqueID)
        {
            #region Original Code
            //****This code is commented by Kiran Abhang on 30th September 2011.
            //Instead of generating unique ID from GUID, we are now using a sequential ID.
            //This unique ID is used for temporary table variable names and for the field names from such tables.

            //OMError ErrorObj = null;
            //OMError ReturnErrorObj = null;
            //String GUID = null;
            //OMError ReturnErrObj = null;
            //Object Lock = new Object();

            //try
            //{
            //    lock (Lock)
            //    {
            //        GUID = Guid.NewGuid().ToString();
            //        GUID = GUID.Remove(0, 24);
            //        UniqueID = Convert.ToInt64(GUID, 16);
            //    }
            //    return ErrorObj;
            //}
            //catch (Exception ExceptionObj)
            //{
            //    ReturnErrObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetUniqueId",
            //        OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
            //    return ReturnErrorObj;
            //}
            #endregion
        
            //New Code.
            UniqueID = ++int_UniqueIDCount;
            return (null);
        }

        /// <summary>
        /// This function prepares a filter clause based on Date Macro.
        /// </summary>
        /// <param name="FieldName">FieldName on which filter is specified</param>
        /// <param name="InputFieldValue">Contains Date macros value. e.g : THIS_WEEK,THIS_YEAR</param>
        /// <param name="InputFilter">XML Node which has all information for Filter Node</param>
        /// <returns></returns>
        public override StringBuilder PrepareDateMacroFilter(String FieldName, String InputFieldValue, String RelationshipName)
        {
            String DateSearchOperator = null;
            String SearchOperator = null;
            StringBuilder FilterValueBuilder = null;


            DateTime FromDate = new DateTime();
            DateTime ToDate = new DateTime();
            DateTime TodaysDate = DateTime.Now;

            XmlDocument FilterXMLDoc = null;
            XmlNode FilterNode = null;

            int CurrentDay = 0;
            int Year = 0;
            int QuarterNum = 0;
            int StartMonth = 0;
            int EndDay = 0;
            int EndMonth = 0;

            FilterXMLDoc = new XmlDocument();
            FilterNode = FilterXMLDoc.CreateNode(XmlNodeType.Element, "FILTER", "");
            FilterValueBuilder = new StringBuilder();

            switch (InputFieldValue)
            {
                #region TODAY
                case "TODAY":
                    DateSearchOperator = "EQUALTO";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region YESTERDAY
                case "YESTERDAY":
                    DateSearchOperator = "EQUALTO";
                    TodaysDate = DateTime.Now.AddDays(-1);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_WEEK
                case "THIS_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    ToDate = FromDate.AddDays(6);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_WEEK_TO_DATE
                case "THIS_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_WEEK
                case "LAST_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    ToDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = ToDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_WEEK_TO_DATE
                case "LAST_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = FromDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_MONTH
                case "THIS_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    ToDate = FromDate.AddMonths(1);
                    ToDate = ToDate.AddDays(-1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_MONTH_TO_DATE
                case "THIS_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_MONTH
                case "LAST_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    ToDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = ToDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_MONTH_TO_DATE
                case "LAST_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = FromDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_QUARTER
                case "THIS_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            break;
                        case "1":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            break;
                        case "2":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            break;
                        case "3":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_QUARTER_TO_DATE
                case "THIS_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            break;
                        case "1":
                            StartMonth = 4;
                            break;
                        case "2":
                            StartMonth = 7;
                            break;
                        case "3":
                            StartMonth = 10;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_QUARTER
                case "LAST_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_QUARTER_TO_DATE
                case "LAST_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_YEAR
                case "THIS_YEAR":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_YEAR_TO_DATE
                case "THIS_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_YEAR
                case "LAST_YEAR":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_YEAR_TO_DATE
                case "LAST_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");

                    break;
                #endregion

                #region LESS_THAN_TODAY
                case "LESS_THAN_TODAY":
                    DateSearchOperator = "LESSTHAN";
                    TodaysDate = TodaysDate.AddDays(-1);
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                  <OPERATOR>{1}</OPERATOR>
                                                  <EXPRESSIONVALUE>
                                                    <VALUE>
                                                        <DAY>{2}</DAY>
                                                        <MONTH>{3}</MONTH>
                                                        <YEAR>{4}</YEAR>
                                                        <HOUR>23</HOUR>
                                                        <MINUTE>59</MINUTE>
                                                        <SECOND>59</SECOND>
                                                    </VALUE>
                                                  </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    break;

                #endregion

                #region LESS_THAN_EQUALTO_TODAY
                case "LESS_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "LESSTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);


                    break;

                #endregion

                #region GREATER_THAN_TODAY
                case "GREATER_THAN_TODAY":
                    DateSearchOperator = "GREATERTHAN";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    break;

                #endregion

                #region GREATER_THAN_EQUALTO_TODAY
                case "GREATER_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "GREATERTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>00</HOUR>
                                                            <MINUTE>00</MINUTE>
                                                            <SECOND>00</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);

                    break;

                #endregion
            }
            if (!String.IsNullOrEmpty(RelationshipName))
                FilterValueBuilder.AppendFormat("<RELATIONSHIPNAME>{0}</RELATIONSHIPNAME>", RelationshipName);
            SearchOperator = NSearchCommon.GetSearchOperatorValue(DateSearchOperator);
            FilterNode.InnerXml = FilterValueBuilder.ToString();
            return (PrepareDateFilter(FieldName, SearchOperator, FilterNode));
        }

        public StringBuilder PrepareDateMacroFilterEx(FilterCollection TempFilterObj, String FieldName, String InputFieldValue, String RelationshipName)
        {
            String DateSearchOperator = null;
            String SearchOperator = null;
            StringBuilder FilterValueBuilder = null;


            DateTime FromDate = new DateTime();
            DateTime ToDate = new DateTime();
            DateTime TodaysDate = DateTime.Now;

            XmlDocument FilterXMLDoc = null;
            XmlNode FilterNode = null;

            int CurrentDay = 0;
            int Year = 0;
            int QuarterNum = 0;
            int StartMonth = 0;
            int EndDay = 0;
            int EndMonth = 0;

            FilterXMLDoc = new XmlDocument();
            FilterNode = FilterXMLDoc.CreateNode(XmlNodeType.Element, "FILTER", "");
            FilterValueBuilder = new StringBuilder();

            switch (InputFieldValue)
            {
                #region TODAY
                case "TODAY":
                    DateSearchOperator = "EQUALTO";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region YESTERDAY
                case "YESTERDAY":
                    DateSearchOperator = "EQUALTO";
                    TodaysDate = DateTime.Now.AddDays(-1);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_WEEK
                case "THIS_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    ToDate = FromDate.AddDays(6);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_WEEK_TO_DATE
                case "THIS_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_WEEK
                case "LAST_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    ToDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = ToDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_WEEK_TO_DATE
                case "LAST_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = FromDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_MONTH
                case "THIS_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    ToDate = FromDate.AddMonths(1);
                    ToDate = ToDate.AddDays(-1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_MONTH_TO_DATE
                case "THIS_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_MONTH
                case "LAST_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    ToDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = ToDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_MONTH_TO_DATE
                case "LAST_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = FromDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_QUARTER
                case "THIS_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            break;
                        case "1":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            break;
                        case "2":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            break;
                        case "3":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_QUARTER_TO_DATE
                case "THIS_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            break;
                        case "1":
                            StartMonth = 4;
                            break;
                        case "2":
                            StartMonth = 7;
                            break;
                        case "3":
                            StartMonth = 10;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_QUARTER
                case "LAST_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_QUARTER_TO_DATE
                case "LAST_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_YEAR
                case "THIS_YEAR":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region THIS_YEAR_TO_DATE
                case "THIS_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_YEAR
                case "LAST_YEAR":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;

                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");
                    break;
                #endregion

                #region LAST_YEAR_TO_DATE
                case "LAST_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;
                    FilterValueBuilder.AppendFormat("<EXPRESSION>{0}</EXPRESSION>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<EXPRESSIONVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</EXPRESSIONVALUE>");

                    break;
                #endregion

                #region LESS_THAN_TODAY
                case "LESS_THAN_TODAY":
                    DateSearchOperator = "LESSTHAN";
                    TodaysDate = TodaysDate.AddDays(-1);
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                  <OPERATOR>{1}</OPERATOR>
                                                  <EXPRESSIONVALUE>
                                                    <VALUE>
                                                        <DAY>{2}</DAY>
                                                        <MONTH>{3}</MONTH>
                                                        <YEAR>{4}</YEAR>
                                                        <HOUR>23</HOUR>
                                                        <MINUTE>59</MINUTE>
                                                        <SECOND>59</SECOND>
                                                    </VALUE>
                                                  </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    break;

                #endregion

                #region LESS_THAN_EQUALTO_TODAY
                case "LESS_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "LESSTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);


                    break;

                #endregion

                #region GREATER_THAN_TODAY
                case "GREATER_THAN_TODAY":
                    DateSearchOperator = "GREATERTHAN";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    break;

                #endregion

                #region GREATER_THAN_EQUALTO_TODAY
                case "GREATER_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "GREATERTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<EXPRESSION>{0}</EXPRESSION>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <EXPRESSIONVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>00</HOUR>
                                                            <MINUTE>00</MINUTE>
                                                            <SECOND>00</SECOND>
                                                        </VALUE>
                                                      </EXPRESSIONVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);

                    break;

                #endregion
            }
            if (!String.IsNullOrEmpty(RelationshipName))
                FilterValueBuilder.AppendFormat("<RELATIONSHIPNAME>{0}</RELATIONSHIPNAME>", RelationshipName);
            SearchOperator = NSearchCommon.GetSearchOperatorValue(DateSearchOperator);
            FilterNode.InnerXml = FilterValueBuilder.ToString();
            return (PrepareDateFilterEx(TempFilterObj, FieldName, SearchOperator, FilterNode));
        }
        #endregion

        private OMError PrepareQueryFromFilterCombination(HierarchyObject HierarchyObj, 
                            ref StringBuilder WhereClause)
        {
            #region Variables

            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            String FilterCombinationString = null;
            String TempString = null;
            String TableVariableName = null;
            bool IsOperatorPushed = false;
            bool IsFilterCombinationValid = false;
            bool IsMultipleFilter = false;
            int j = 0;
            Stack<String> FilterStack = null;
            Char ch = ' ';
            List<TableVariableCollection> TableVariableCollectionListObj = null;
            String ParsedFilterCombination = null;

            #endregion Variables

            try
            {
                FilterCombinationString = HierarchyObj.m_FilterCombination;

                if (FilterCombinationString.Length == 0)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                       "Filter Combination is Empty", OMErrorObjectType.GETDATAQUERYBUILDER,
                       "NSearch", OMErrorType.ERROR, null, null);
                    return ReturnErrorObj;
                }

                //Validate FilterCombination for well formness of brackets
                IsFilterCombinationValid = ValidateFilterCombination(FilterCombinationString);
                if (!IsFilterCombinationValid)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Filter Combination is Invalid", OMErrorObjectType.GETDATAQUERYBUILDER,
                    "NSearch", OMErrorType.ERROR, null, null);
                }

                ErrorObj = ParseFilterCombination(FilterCombinationString, ref ParsedFilterCombination);
                if (ErrorObj != null)
                    return ErrorObj;

                //Assign the parsed FilterCombinationString in FilterCombinationString
                FilterCombinationString = ParsedFilterCombination;
                WhereClause.Append(" ");

                FilterStack = new Stack<String>();

                // Process FilterCombination.

                IsOperatorPushed = false;
                TableVariableCollectionListObj = new List<TableVariableCollection>();

                #region Process Filter Combination String

                for (int i = 0; i < FilterCombinationString.Length; i++)
                {
                    ch = FilterCombinationString[i];

                    switch (ch)
                    {
                        case '(':
                            {
                                //Push ( in Stack
                                FilterStack.Push(ch.ToString());
                                IsOperatorPushed = false;
                            }
                            break;

                        case ')':
                            {
                                String Operator = null;
                                String Operand1 = null;
                                String Operand2 = null;

                                //Pop the intermediate Table Variable from Stack
                                TempString = FilterStack.Pop();

                                //Pop the (  i.e opening bracket from Stack
                                FilterStack.Pop();

                                //The operand is already pushed in Stack, pop operator and FilterId
                                if (FilterStack.Count == 0)
                                {
                                    //Only ( bracket is found in Stack
                                    FilterStack.Push(TempString);
                                }
                                else
                                {
                                    //Check whether FilterStack has '(' inside it, if yes insert the popped Table Variable into Stack
                                    if (FilterStack.Peek().Equals("("))
                                    {
                                        //FilterStack.Pop();
                                        FilterStack.Push(TempString);
                                    }
                                    else
                                    {
                                        //Pop the operator
                                        Operator = FilterStack.Pop();
                                        Operand1 = FilterStack.Pop();
                                        Operand2 = TempString;
                                        TableVariableName = "";

                                        //Combine the  Table Variables with the Operator
                                        ErrorObj = CombineTableVariable(Operator, Operand1, Operand2, TableVariableCollectionListObj, ref WhereClause, ref TableVariableName);
                                        if (ErrorObj != null)
                                            return ErrorObj;

                                        FilterStack.Push(TableVariableName);
                                    }
                                }
                                IsOperatorPushed = false;
                            }
                            break;

                        case 'A':
                        case 'a':
                            {
                                //Push AND operator in Stack
                                TempString = "";
                                TempString = ch.ToString() + FilterCombinationString[i + 1].ToString() + FilterCombinationString[i + 2].ToString();

                                if (TempString.ToUpper().CompareTo("AND") == 0)
                                {
                                    i = i + 2;
                                    FilterStack.Push(TempString);
                                    IsOperatorPushed = true;
                                }
                            }
                            break;

                        case 'O':
                        case 'o':
                            {
                                //Push OR operator in Stack
                                TempString = "";
                                TempString = ch.ToString() + FilterCombinationString[i + 1].ToString();

                                if (TempString.ToUpper().CompareTo("OR") == 0)
                                {
                                    i = i + 1;
                                    FilterStack.Push(TempString);
                                    IsOperatorPushed = true;
                                }
                            }
                            break;

                        default:
                            {
                                String Operator = null;
                                String Operand1 = null;
                                String Operand2 = null;

                                //Cases
                                //1. Push operand in Stack if no operator is identified
                                //2. Pop operator and FilterId and prepare Query
                                if (ch >= '0' && ch <= '9')
                                {
                                    TempString = ch.ToString();
                                    j = i + 1;

                                    for (; j < FilterCombinationString.Length; j++)
                                    {
                                        ch = FilterCombinationString[j];
                                        if (ch >= '0' && ch <= '9')
                                        {
                                            TempString = TempString + ch.ToString();
                                            i++;
                                            continue;
                                        }

                                        if (ch == ')' || ch == ' ')
                                        {
                                            ErrorObj = PrepareTableVariable(TempString, HierarchyObj, TableVariableCollectionListObj, ref TableVariableName);
                                            if (ErrorObj != null)
                                                return ErrorObj;

                                            if (IsOperatorPushed == true)
                                            {
                                                //The operand is already pushed in Stack, pop operator and FilterId

                                                //Pop the operator
                                                Operator = FilterStack.Pop();
                                                Operand1 = FilterStack.Pop();
                                                Operand2 = TableVariableName;
                                                TableVariableName = "";

                                                ErrorObj = CombineTableVariable(Operator, Operand1, Operand2, TableVariableCollectionListObj, ref WhereClause, ref TableVariableName);
                                                if (ErrorObj != null)
                                                    return ErrorObj;

                                                FilterStack.Push(TableVariableName);
                                                IsOperatorPushed = false;
                                                IsMultipleFilter = true;
                                            }
                                            else
                                            {
                                                //Push operator in Stack
                                                FilterStack.Push(TableVariableName);
                                            }
                                            break;
                                        }
                                    }

                                    if (j == FilterCombinationString.Length)
                                    {
                                        ErrorObj = PrepareTableVariable(TempString, HierarchyObj, TableVariableCollectionListObj, ref TableVariableName);
                                        if (ErrorObj != null)
                                            return ErrorObj;

                                        if (IsOperatorPushed == true)
                                        {
                                            //The operand is already pushed in Stack, pop operator and FilterId

                                            //Pop the operator
                                            Operator = FilterStack.Pop();
                                            Operand1 = FilterStack.Pop();
                                            Operand2 = TableVariableName;

                                            ErrorObj = CombineTableVariable(Operator, Operand1, Operand2, TableVariableCollectionListObj, ref WhereClause, ref TableVariableName);
                                            if (ErrorObj != null)
                                                return ErrorObj;

                                            FilterStack.Push(TableVariableName);
                                            IsOperatorPushed = false;
                                            IsMultipleFilter = true;
                                        }
                                        else
                                        {
                                            //Push operator in Stack
                                            FilterStack.Push(TableVariableName);
                                        }
                                    }
                                }
                            }
                            break;
                    }
                }

                #endregion Process Filter Combination String

                //Pop the last Table Variable from Stack
                TableVariableName = FilterStack.Pop();

                //Replace @ with "" so that it can be used as alias
                TempString = TableVariableName;
                TempString = TempString.Replace("@", "");

                if (HierarchyObj.m_ParentID == 0)
                {
                    #region Insert the result in to @TEOBJECT

                    //If FilterCombination contains only one filter, add the Query in Where clause
                    if (IsMultipleFilter == false)
                    {
                        WhereClause.Append(" ");
                        WhereClause.Append(TableVariableCollectionListObj[0].m_Query);
                    }

                    //Insert the result of the final Table Variable in Where Clause - INSERT IT INTO @TEOBJECT
                    WhereClause.Append(" ");

                    if (HierarchyObj.m_DoPaging)
                    {
                        int StartRecordOnThisPage, EndRecordOnThisPage;

                        GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage,
                                                    out StartRecordOnThisPage, out EndRecordOnThisPage);

                        String PagingSortField = null;

                        //StringBuilder sbSortFieldsWithSort = new StringBuilder(100);
                        //StringBuilder sbSortFieldNames = new StringBuilder(100);

                        if (!string.IsNullOrEmpty(HierarchyObj.m_PagingSortField))
                            PagingSortField = HierarchyObj.m_PagingSortField;
                        else
                            PagingSortField = HierarchyObj.m_PrimaryKeyName;

                        if (!m_NSearchCommonObj.IsAuthorizationRequired)
                        {
                            string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;
                            
                            GetSortFields(HierarchyObj.SortSpecificationList,
                                                HierarchyObj.m_TableName + "_ALIAS", HierarchyObj.m_PrimaryKeyName, 
                                                ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                            WhereClause.Append("INSERT INTO @TEOBJECT (FILTEREDOBJID)  ");

                            if (string.IsNullOrEmpty(HierarchyObj.m_FilterCombination))
                            {
                                WhereClause.Append(" SELECT FILTEREDOBJID FROM ( ");
                                WhereClause.AppendFormat("SELECT DISTINCT  ");

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                    WhereClause.AppendFormat("{1}.FILTEREDOBJID, ROW_NUMBER() OVER (ORDER BY {1}.FILTEREDOBJID {3}) AS ROWNUMBER FROM {0} {1} ",
                                                                TableVariableName, TempString, PagingSortField, HierarchyObj.m_PagingSortFieldOrder);
                                }
                                else
                                {

                                    WhereClause.AppendFormat("{1}.FILTEREDOBJID, ROW_NUMBER() OVER (ORDER BY ", TableVariableName, TempString);

                                    WhereClause.Append(SortFieldNamesAliasedWithOrder);
                                    WhereClause.Append(" ) AS ROWNUMBER");
                                    WhereClause.Append(SortFieldNamesOnly);
                                    WhereClause.AppendFormat(" FROM {0} {1} ", TableVariableName, TempString);
                                }

                                WhereClause.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                                WhereClause.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                                    WhereClause.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);

                                WhereClause.AppendFormat(") AS TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1}",
                                                            StartRecordOnThisPage, EndRecordOnThisPage);

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                }
                                else
                                {
                                    WhereClause.Append(" order by ");

                                    WhereClause.Append(SortFieldNamesWithOrder);
                                    WhereClause.Append(" ");
                                }

                                StringBuilder sbPagingCountQuery = new StringBuilder(200);

                                sbPagingCountQuery.AppendFormat(" FROM {0} {1}  ", TableVariableName, TempString);
                                sbPagingCountQuery.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                                sbPagingCountQuery.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                                    sbPagingCountQuery.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);

                                PagingCountFromClause = sbPagingCountQuery.ToString();

                                sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                                sbPagingCountQuery = null;
                            }
                            else
                            {
                                WhereClause.Append(" SELECT FILTEREDOBJID FROM ( SELECT FILTEREDOBJID ");

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                    WhereClause.Append(", ROW_NUMBER() OVER (ORDER BY  FILTEREDOBJID DESC ) AS ROWNUMBER FROM (   ");
                                    WhereClause.AppendFormat(" SELECT DISTINCT {1}.FILTEREDOBJID FROM {0} {1} ",
                                                                TableVariableName, TempString, PagingSortField, HierarchyObj.m_PagingSortFieldOrder);
                                }
                                else
                                {

                                    WhereClause.Append(", ROW_NUMBER() OVER (ORDER BY ");
                                    //WhereClause.AppendFormat("{1}.FILTEREDOBJID, ROW_NUMBER() OVER (ORDER BY ", TableVariableName, TempString);

                                    WhereClause.Append(SortFieldNamesWithOrder);
                                    WhereClause.Append(" ) AS ROWNUMBER");
                                    WhereClause.Append(SortFieldNamesOnly);
                                    WhereClause.AppendFormat(" from (select distinct {0}.FILTEREDOBJID {1} ", TempString, SortFieldNamesOnly);
                                    WhereClause.AppendFormat(" FROM {0} {1} ", TableVariableName, TempString);
                                }

                                WhereClause.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                                WhereClause.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                                    WhereClause.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);

                                WhereClause.AppendFormat(") AS TEOBJECTWITHROWNUMBERS) AS TEOBJECTWITHROWNUMBERS WHERE ROWNUMBER BETWEEN {0} AND {1}",
                                                            StartRecordOnThisPage, EndRecordOnThisPage);

                                if (HierarchyObj.SortSpecificationList.Count == 0)
                                {
                                }
                                else
                                {
                                    WhereClause.Append(" order by ");

                                    WhereClause.Append(SortFieldNamesWithOrder);
                                    WhereClause.Append(" ");
                                }

                                StringBuilder sbPagingCountQuery = new StringBuilder(200);

                                sbPagingCountQuery.AppendFormat(" FROM {0} {1}  ", TableVariableName, TempString);
                                sbPagingCountQuery.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                                sbPagingCountQuery.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                                    sbPagingCountQuery.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);

                                PagingCountFromClause = sbPagingCountQuery.ToString();

                                sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                                sbPagingCountQuery = null;
                            }
                        }
                        else
                        {
                            //Supposed to be filled by authorization???
                            //PagingCountFromClause = sbPagingCountQuery.ToString();

                            WhereClause.AppendFormat("INSERT INTO @TEOBJECT (FILTEREDOBJID) ");
                            WhereClause.AppendFormat("SELECT DISTINCT {0} ", (HierarchyObj.m_MaxResultSize != 0 ? "TOP " + HierarchyObj.m_MaxResultSize.ToString() : ""));
                            WhereClause.AppendFormat("{1}.FILTEREDOBJID FROM {0} {1} ", TableVariableName, TempString);
                            WhereClause.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                            WhereClause.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                                WhereClause.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);
                        }

                    }
                    else
                    {
                        WhereClause.AppendFormat("INSERT INTO @TEOBJECT (FILTEREDOBJID) ");
                        WhereClause.AppendFormat("SELECT DISTINCT {0} ", (HierarchyObj.m_MaxResultSize != 0 ? "TOP " + HierarchyObj.m_MaxResultSize.ToString() : ""));
                        WhereClause.AppendFormat("{1}.FILTEREDOBJID FROM {0} {1} ", TableVariableName, TempString);
                        WhereClause.AppendFormat("INNER JOIN {0} {0}_ALIAS ", HierarchyObj.m_TableName);
                        WhereClause.AppendFormat("ON {0}.FILTEREDOBJID = {1}_ALIAS.{2} ", TempString, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                        if (!String.IsNullOrEmpty(HierarchyObj.m_SimpleSearchString))
                            WhereClause.AppendFormat(" AND {0}.FILTEREDOBJID IN( SELECT SSOBJECTID from @SSFILTEREDTABLE)  ", TempString);
                    }
                    #endregion Insert the result in to @TEOBJECT
                }
                else
                {
                    StringBuilder TEWithREWhereClause = null;
                    ErrorObj = CombineTEWithRE(HierarchyObj, TempString, ref TEWithREWhereClause);
                    if (ErrorObj != null)
                        return ErrorObj;
                    WhereClause.AppendFormat(" {0}", TEWithREWhereClause.ToString());
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareQueryFromFilterCombination",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }


        /// <summary>
        /// This function combines TE with RE
        /// </summary>
        /// <param name="HierarchyObj">The HierarchyObj of type HierarchyObject</param>
        /// <param name="TEWithREWhereClause">The TEWithREWhereClause is the WhereClause which is returned</param>
        /// <returns>OMError Object</returns>
        private OMError CombineTEWithRE(HierarchyObject HierarchyObj, String ProcessedFilterTableVariableName, ref StringBuilder TEWithREWhereClause)
        {
            #region Variables

            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            XmlNode RelationshipNode = null;
            ulong RELATIONSHIPTYPEID = 0;
            ulong CONTAINERTYPEID = 0;
            ulong CONTAINSTYPEID = 0;
            ulong RelatedTypeID = 0;
            String CONTAINERTYPE = null;
            String CONTAINSTYPE = null;
            //String                  RelatedType                 = null;
            String AssemblyColumnName = null;
            String RelatedTypeName = null;
            StringBuilder TableVariableName = null;
            StringBuilder IntermediateTableVarName = null;
            StringBuilder IntermediateTableVarPkName = null;
            bool IsParentFound = false;
            HierarchyObject ParentHierarchyObj = null;
            String[] ArrParentColumns = null;

            #endregion Variables

            try
            {
                //Read Relationship
                ErrorObj = ReadByRelationShipNameEx(HierarchyObj.m_RelationshipName, ref RelationshipNode);
                if (ErrorObj != null)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                                        ErrorObj.ErrorMessage, ErrorObj.ObjectType, ErrorObj.ObjectTypeName,
                                        ErrorObj.ErrorType, ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                    return ReturnErrorObj;
                }

                // Resolve EntityType or Facet Type.

                RELATIONSHIPTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("RELATIONSHIPTYPEID").InnerText.ToString());
                CONTAINERTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINERTYPEID").InnerText.ToString());
                CONTAINSTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINSTYPEID").InnerText.ToString());
                CONTAINERTYPE = RelationshipNode.SelectSingleNode("CONTAINERTYPE").InnerText.ToString();
                CONTAINSTYPE = RelationshipNode.SelectSingleNode("CONTAINSTYPE").InnerText.ToString();

                // Processing for Relationship where ContainerTypeID,ContainsTypeID and ObjectTypeID are same. This should include FACET-FACET relationships as well
                // in which case the handling is done similar to the entity-entity case. 
                if ((CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "ENTITY") || 
                    (CONTAINERTYPE.ToUpper() == "FACET" && CONTAINSTYPE.ToUpper() == "FACET"))
                {
                    if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                    {
                        RelatedTypeID = CONTAINSTYPEID;
                        //RelatedType = "ENTITY";
                        AssemblyColumnName = "CONTAINERID";
                    }
                    else
                    {
                        if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                        {
                            RelatedTypeID = CONTAINERTYPEID;
                            //RelatedType = "ENTITY";
                            AssemblyColumnName = "CONTAINSID";
                        }
                    }

                    //A facet-facet relationship will need separate(but similar) handling compared to entity-entity 
                    //The case of facet-entity is handled separately in the code ahead
                    if (CONTAINERTYPE.ToUpper() == "ENTITY")
                        RelatedTypeName = "RELATEDENTITY";
                    else
                        RelatedTypeName = "FACETFACET";
                }
                else
                {
                    if (CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "FACET")
                    {
                        if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                        {
                            RelatedTypeID = CONTAINSTYPEID;
                            // This is because Facet cannot be container for Entity.
                            //RelatedType = "FACET";

                        }
                        else
                        {
                            if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                            {
                                RelatedTypeID = CONTAINERTYPEID;
                                //RelatedType = "ENTITY";
                            }
                        }
                                                
                        RelatedTypeName = "RELATEDFACET";                        

                        AssemblyColumnName = "CONTAINSID";
                    }
                }

                TEWithREWhereClause = new StringBuilder(1000);
                TableVariableName = new StringBuilder(100);
                IntermediateTableVarName = new StringBuilder(100);
                IntermediateTableVarPkName = new StringBuilder(100);

                //Prepare RE/RF INTERMEDIATE table variable
                TableVariableName.Append((RelatedTypeName.CompareTo("RELATEDENTITY") == 0 ? "RE" : "RF"));
                TableVariableName.AppendFormat("INTERMEDIATE{0}{1}", HierarchyObj.m_RelationshipName, HierarchyObj.m_TagID);

                IntermediateTableVarName.Append(TableVariableName.ToString());
                IntermediateTableVarPkName.AppendFormat("{0}{1}", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel);

                //Get Parent of this Relationship node
                foreach (HierarchyObject TempHierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
                {
                    if (HierarchyObj.m_ParentID == TempHierarchyObj.m_TagID)
                    {
                        if (TempHierarchyObj.m_EveryOneUserTableName.CompareTo("TEOBJECT") != 0)
                        {
                            IsParentFound = true;
                            ParentHierarchyObj = TempHierarchyObj;
                        }
                        break;
                    }
                }

                //Parent HierarchyObj is found, add columns to Intermediate table variable
                if (IsParentFound == true)
                {
                    TEWithREWhereClause.AppendFormat(" DECLARE @{0} TABLE ({1}{2} NUMERIC, {3}) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel, ParentHierarchyObj.m_ColumnList);
                    TEWithREWhereClause.AppendFormat("INSERT INTO @{0} ({1}{2}, {3}) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel, ParentHierarchyObj.m_ColumnList.Replace("NUMERIC", ""));
                }
                else
                {
                    TEWithREWhereClause.AppendFormat(" DECLARE @{0} TABLE ({1}{2} NUMERIC, FILTEREDOBJID NUMERIC) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel);
                    TEWithREWhereClause.AppendFormat("INSERT INTO @{0} ({1}{2}, FILTEREDOBJID) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel);
                }

                //Query formation for similar objecttypes
                if ((RelatedTypeName.CompareTo("RELATEDENTITY") == 0)||(RelatedTypeName.CompareTo("FACETFACET") == 0))
                {
                    //Parent HierarchyObj is found, add columns to Intermediate table variable
                    if (IsParentFound == true)
                    {
                        ArrParentColumns = ParentHierarchyObj.m_ColumnList.Split(',');

                        TEWithREWhereClause.AppendFormat("SELECT ASSEMBLY_{0}_ALIAS.{1} ", HierarchyObj.m_RelationshipName, AssemblyColumnName);

                        //Select Columns of ParentHierarchyObj from ColumnList
                        foreach (String Column in ArrParentColumns)
                        {
                            TEWithREWhereClause.AppendFormat(", {0}_ALIAS.{1} ", ParentHierarchyObj.m_EveryOneUserTableName, Column.Replace("NUMERIC", "").TrimStart(' '));
                        }

                        TEWithREWhereClause.AppendFormat("FROM ALOM_ASSEMBLY ASSEMBLY_{0}_ALIAS ", HierarchyObj.m_RelationshipName, AssemblyColumnName);
                        TEWithREWhereClause.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ParentHierarchyObj.m_EveryOneUserTableName);
                        TEWithREWhereClause.AppendFormat("ON {0}_ALIAS.{1} = ", ParentHierarchyObj.m_EveryOneUserTableName, ParentHierarchyObj.m_PrimaryKeyforTableVariable);
                        TEWithREWhereClause.AppendFormat("ASSEMBLY_{0}_ALIAS.{1} ", HierarchyObj.m_RelationshipName, (AssemblyColumnName.CompareTo("CONTAINERID") == 0 ? "CONTAINSID" : "CONTAINERID"));
                        TEWithREWhereClause.AppendFormat("AND ASSEMBLY_{0}_ALIAS.RELATIONSHIPTYPEID = {1}", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelationshipTypeID);
                    }
                    else
                    {
                        TEWithREWhereClause.AppendFormat("SELECT ASSEMBLY_{0}_ALIAS.{1}, TEOBJECT_{0}_ALIAS.FILTEREDOBJID FROM ALOM_ASSEMBLY ASSEMBLY_{0}_ALIAS ", HierarchyObj.m_RelationshipName, AssemblyColumnName);
                        TEWithREWhereClause.AppendFormat("INNER JOIN @TEOBJECT TEOBJECT_{0}_ALIAS ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("ON TEOBJECT_{0}_ALIAS.FILTEREDOBJID = ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("ASSEMBLY_{0}_ALIAS.{1} ", HierarchyObj.m_RelationshipName, (AssemblyColumnName.CompareTo("CONTAINSID") == 0 ? "CONTAINERID" : "CONTAINSID"));
                        TEWithREWhereClause.AppendFormat("AND ASSEMBLY_{0}_ALIAS.RELATIONSHIPTYPEID = {1}", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelationshipTypeID);
                    }
                }
                else
                {                    
                    if (IsParentFound == true)
                    {
                        
                        if (ParentHierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                        {
                            //Query formation for Entity-Facet relationship

                            ArrParentColumns = ParentHierarchyObj.m_ColumnList.Split(',');

                            TEWithREWhereClause.AppendFormat("SELECT ASSOENTFCT_{0}_ALIAS.FACETID ", HierarchyObj.m_RelationshipName);

                            //Select Columns of ParentHierarchyObj from ColumnList
                            foreach (String Column in ArrParentColumns)
                            {
                                TEWithREWhereClause.AppendFormat(", {0}_ALIAS.{1} ", ParentHierarchyObj.m_EveryOneUserTableName, Column.Replace("NUMERIC", "").TrimStart(' '));
                            }

                            //TEWithREWhereClause.AppendFormat("SELECT ASSOENTFCT_{0}_ALIAS.FACETID, {1}_ALIAS.FILTEREDOBJID, ASSOENTFCT_{0}_ALIAS.ENTITYID ", HierarchyObj.m_RelationshipName,ParentHierarchyObj.m_EveryOneUserTableName);
                            TEWithREWhereClause.AppendFormat("FROM ALOM_ASSOENTITYFACET ASSOENTFCT_{0}_ALIAS ", HierarchyObj.m_RelationshipName);

                            TEWithREWhereClause.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ParentHierarchyObj.m_EveryOneUserTableName);
                            TEWithREWhereClause.AppendFormat("ON {0}_ALIAS.{1} =  ", ParentHierarchyObj.m_EveryOneUserTableName, ParentHierarchyObj.m_PrimaryKeyforTableVariable);
                            TEWithREWhereClause.AppendFormat("ASSOENTFCT_{0}_ALIAS.ENTITYID ", HierarchyObj.m_RelationshipName);
                            TEWithREWhereClause.AppendFormat("AND (ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID = {1} OR ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID IS NULL)", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelationshipTypeID);
                        }
                        else if (ParentHierarchyObj.m_ObjectType.ToUpper() == "FACET")
                        {
                            //Query formation for Facet-Entity relationship

                            ArrParentColumns = ParentHierarchyObj.m_ColumnList.Split(',');

                            TEWithREWhereClause.AppendFormat("SELECT ASSOENTFCT_{0}_ALIAS.ENTITYID ", HierarchyObj.m_RelationshipName);

                            //Select Columns of ParentHierarchyObj from ColumnList
                            foreach (String Column in ArrParentColumns)
                            {
                                TEWithREWhereClause.AppendFormat(", {0}_ALIAS.{1} ", ParentHierarchyObj.m_EveryOneUserTableName, Column.Replace("NUMERIC", "").TrimStart(' '));
                            }

                            //TEWithREWhereClause.AppendFormat("SELECT ASSOENTFCT_{0}_ALIAS.FACETID, {1}_ALIAS.FILTEREDOBJID, ASSOENTFCT_{0}_ALIAS.ENTITYID ", HierarchyObj.m_RelationshipName,ParentHierarchyObj.m_EveryOneUserTableName);
                            TEWithREWhereClause.AppendFormat("FROM ALOM_ASSOENTITYFACET ASSOENTFCT_{0}_ALIAS ", HierarchyObj.m_RelationshipName);

                            TEWithREWhereClause.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ParentHierarchyObj.m_EveryOneUserTableName);
                            TEWithREWhereClause.AppendFormat("ON {0}_ALIAS.{1} =  ", ParentHierarchyObj.m_EveryOneUserTableName, ParentHierarchyObj.m_PrimaryKeyforTableVariable);
                            TEWithREWhereClause.AppendFormat("ASSOENTFCT_{0}_ALIAS.FACETID ", HierarchyObj.m_RelationshipName);
                            TEWithREWhereClause.AppendFormat("AND (ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID = {1} OR ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID IS NULL)", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelationshipTypeID);
                        }
                    }
                    else
                    {
                        TEWithREWhereClause.AppendFormat("SELECT ASSOENTFCT_{0}_ALIAS.FACETID, TEOBJECT_{0}_ALIAS.FILTEREDOBJID FROM ALOM_ASSOENTITYFACET ASSOENTFCT_{0}_ALIAS ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("INNER JOIN @TEOBJECT TEOBJECT_{0}_ALIAS ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("ON TEOBJECT_{0}_ALIAS.FILTEREDOBJID =  ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("ASSOENTFCT_{0}_ALIAS.ENTITYID ", HierarchyObj.m_RelationshipName);
                        TEWithREWhereClause.AppendFormat("AND (ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID = {1} OR ASSOENTFCT_{0}_ALIAS.RELATIONSHIPTYPEID IS NULL)", HierarchyObj.m_RelationshipName, HierarchyObj.m_RelationshipTypeID);
                    }
                }

                //Prepare RE{relationshipname} table variable
                TableVariableName.Remove(0, TableVariableName.Length);
                TableVariableName.Append(RelatedTypeName.CompareTo("RELATEDENTITY") == 0 ? "RE" : "RF");
                TableVariableName.AppendFormat("{0}{1}", HierarchyObj.m_RelationshipName, HierarchyObj.m_ActualTagID);

                if (IsParentFound == true)
                {
                    TEWithREWhereClause.AppendFormat(" DECLARE @{0} TABLE ({1}{2} NUMERIC, {3}) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel, ParentHierarchyObj.m_ColumnList);
                    TEWithREWhereClause.AppendFormat("INSERT INTO @{0} ({1}{2}, {3}) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel, ParentHierarchyObj.m_ColumnList.Replace("NUMERIC", ""));

                    StringBuilder sbSelectStmtForPaging = new StringBuilder();
                    StringBuilder sbSelectStmt = new StringBuilder();

                    StringBuilder sbPagingCountQuery = new StringBuilder(200);

                    //sbCountQueryForPaging.AppendFormat("SELECT count({0}) as TotalRecordCount ", IntermediateTableVarPkName);
                    sbSelectStmtForPaging.AppendFormat("SELECT {0} ", IntermediateTableVarPkName);
                    sbSelectStmt.AppendFormat("SELECT {0}_ALIAS.{1} ", IntermediateTableVarName, IntermediateTableVarPkName);
                    
                    //TEWithREWhereClause.AppendFormat("SELECT {0}_ALIAS.{1} ", IntermediateTableVarName, IntermediateTableVarPkName);

                    //Select Columns of ParentHierarchyObj from ColumnList
                    foreach (String Column in ArrParentColumns)
                    {
                        sbSelectStmtForPaging.AppendFormat(", {0} ", Column.Replace("NUMERIC", "").TrimStart(' '));
                        sbSelectStmt.AppendFormat(", {0}_ALIAS.{1} ", IntermediateTableVarName, Column.Replace("NUMERIC", "").TrimStart(' '));
                    }

                    sbSelectStmtForPaging.Append(" from ( ");

                    string SortFieldNamesOnly = null, SortFieldNamesWithOrder = null, SortFieldNamesAliasedWithOrder = null;

                    GetSortFields(HierarchyObj.SortSpecificationList,
                                            HierarchyObj.m_TableName + "_ALIAS", null, 
                                            ref SortFieldNamesOnly, ref SortFieldNamesWithOrder, ref SortFieldNamesAliasedWithOrder);

                    if (HierarchyObj.m_DoPaging)
                    {
                        

                        if (HierarchyObj.SortSpecificationList.Count == 0)
                        {
                            sbSelectStmt.AppendFormat(", row_number() over (order by {0}_ALIAS.{1} asc) as rownumber ",
                                                    IntermediateTableVarName,
                                                    ArrParentColumns[ArrParentColumns.Length - 1].Replace("NUMERIC", "").TrimStart(' '));
                        }
                        else
                        {
                            
                            sbSelectStmt.Append(", row_number() over (order by ");

                            sbSelectStmt.Append(SortFieldNamesAliasedWithOrder);
                            sbSelectStmt.Append(" ) AS ROWNUMBER");
                            sbSelectStmt.Append(SortFieldNamesOnly);

                        }
                    }

                    sbSelectStmt.AppendFormat(" FROM @{0} {0}_ALIAS ", IntermediateTableVarName);
                    sbPagingCountQuery.AppendFormat(" FROM @{0} {0}_ALIAS ", IntermediateTableVarName);

                    sbSelectStmt.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ProcessedFilterTableVariableName);
                    sbSelectStmt.AppendFormat("ON {0}_ALIAS.FILTEREDOBJID = {1}_ALIAS.{2} ", ProcessedFilterTableVariableName, IntermediateTableVarName, IntermediateTableVarPkName);

                    sbPagingCountQuery.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ProcessedFilterTableVariableName);
                    sbPagingCountQuery.AppendFormat("ON {0}_ALIAS.FILTEREDOBJID = {1}_ALIAS.{2} ", ProcessedFilterTableVariableName, IntermediateTableVarName, IntermediateTableVarPkName);

                    if (HierarchyObj.m_DoPaging)
                    {
                        int StartRecordOnThisPage, EndRecordOnThisPage;

                        GetPagingRecordNumbers(HierarchyObj.m_PageNumber, HierarchyObj.m_RecordsPerPage,
                                                    out StartRecordOnThisPage, out EndRecordOnThisPage);

                        TEWithREWhereClause.Append(sbSelectStmtForPaging);
                        TEWithREWhereClause.Append(sbSelectStmt);

                        TEWithREWhereClause.AppendFormat(" inner join {0} {0}_ALIAS on {1}_ALIAS.{2} = {0}_ALIAS.{3}",
                            HierarchyObj.m_TableName, ProcessedFilterTableVariableName,
                            ArrParentColumns[ArrParentColumns.Length - 1].Replace("NUMERIC", "").TrimStart(' '),
                            HierarchyObj.m_PrimaryKeyName);

                        TEWithREWhereClause.Append(" ) as TEOBJECTWITHROWNUMBERS ");
                        TEWithREWhereClause.AppendFormat(" where rownumber between {0} AND {1} ",
                                                StartRecordOnThisPage, EndRecordOnThisPage);

                        if (HierarchyObj.SortSpecificationList.Count == 0)
                        {
                        }
                        else
                        {
                            TEWithREWhereClause.Append(" order by ");

                            TEWithREWhereClause.Append(SortFieldNamesWithOrder);
                            TEWithREWhereClause.Append(" ");

                        }
                        
                        PagingCountFromClause = sbPagingCountQuery.ToString();
                    }
                    else
                    {
                        TEWithREWhereClause.Append(sbSelectStmt);
                    }

                    sbSelectStmtForPaging.Remove(0, sbSelectStmtForPaging.Length);
                    sbSelectStmtForPaging = null;
                    sbSelectStmt.Remove(0, sbSelectStmt.Length);
                    sbSelectStmt = null;
                    sbPagingCountQuery.Remove(0, sbPagingCountQuery.Length);
                    sbPagingCountQuery = null;

                    
                }
                else
                {
                    TEWithREWhereClause.AppendFormat(" DECLARE @{0} TABLE ({1}{2} NUMERIC, FILTEREDOBJID NUMERIC) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel);
                    TEWithREWhereClause.AppendFormat("INSERT INTO @{0} ({1}{2}, FILTEREDOBJID) ", TableVariableName, HierarchyObj.m_RelationshipName, HierarchyObj.m_RelatedObjectLevel);
                    TEWithREWhereClause.AppendFormat("SELECT {0}_ALIAS.{1},{0}_ALIAS.FILTEREDOBJID FROM @{0} {0}_ALIAS ", IntermediateTableVarName, IntermediateTableVarPkName);

                    TEWithREWhereClause.AppendFormat("INNER JOIN @{0} {0}_ALIAS ", ProcessedFilterTableVariableName);
                    TEWithREWhereClause.AppendFormat("ON {0}_ALIAS.FILTEREDOBJID = {1}_ALIAS.{2} ", ProcessedFilterTableVariableName, IntermediateTableVarName, IntermediateTableVarPkName);

                }

            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in CombineTEWithRE",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;

        }

        /// <summary>
        /// This function parses the FilterCombination to remove additional outer brackets and brackets around the FilterID 
        /// </summary>
        /// <param name="FilterCombination">The FilterCombination of type String which is to be parsed</param>
        /// <param name="ParsedFilterCombinationString">The ParsedFilterCombinationString is the parsed FilterCombination which is returned</param>
        /// <returns>OMError Object</returns>
        private OMError ParseFilterCombination(String FilterCombination, ref String ParsedFilterCombinationString)
        {
            #region Variables

            OMError ReturnErrorObj = null;
            String TempFilterCombination = null;
            bool IsClosingBracket = false;
            int j = 0;
            Char ch = ' ';
            int OpeningBracketIndex = 0;
            int ClosingBracketIndex = 0;
            int BracketRemovedCount = 0;

            #endregion Variables

            try
            {
                #region Parse Filter Combination String

                FilterCombination = FilterCombination.Replace("AND", " AND ");
                FilterCombination = FilterCombination.Replace("OR", " OR ");

                FilterCombination = FilterCombination.Replace("and", " and ");
                FilterCombination = FilterCombination.Replace("or", " or ");

                TempFilterCombination = FilterCombination;

                #region Remove additional brackets around the Filter Id

                //Remove brackets around the filterid
                for (int i = 0; i < TempFilterCombination.Length; i++)
                {
                    ch = TempFilterCombination[i];

                    if (ch >= '0' && ch <= '9')
                    {
                        //The opening bracket is not given for FilterID
                        if (i == 0)
                            continue;

                        OpeningBracketIndex = i - 1;

                        if (TempFilterCombination[OpeningBracketIndex] == '(')
                        {
                            j = i + 1;

                            for (; j < TempFilterCombination.Length; j++)
                            {
                                ch = TempFilterCombination[j];
                                if (ch >= '0' && ch <= '9')
                                    continue;
                                else if (ch == ')')
                                {
                                    ClosingBracketIndex = j;
                                    IsClosingBracket = true;
                                    break;
                                }
                                else if (ch == ' ' || ch == '(')
                                    break;
                            }
                            if (IsClosingBracket == true)
                            {
                                //Remove the opening and closing brackets around the filterid
                                FilterCombination = FilterCombination.Remove(OpeningBracketIndex - BracketRemovedCount, 1);
                                BracketRemovedCount++;
                                FilterCombination = FilterCombination.Remove(ClosingBracketIndex - BracketRemovedCount, 1);
                                BracketRemovedCount++;
                                IsClosingBracket = false;
                            }
                        }
                    }
                }

                #endregion Remove additional brackets around the Filter Id

                ParsedFilterCombinationString = FilterCombination;

                #endregion Parse Filter Combination String
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in ParseFilterCombination",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                ParsedFilterCombinationString = null;
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function returns the object of TableVariableCollection based on TableVariableName 
        /// </summary>
        /// <param name="TableVariableName">The TableVariableName of type String for which the TableVariableCollection object is to be returned</param>
        /// <param name="TableVariableCollectionListObj">The TableVariableCollectionListObj of type List from which the TableVariableCollection is to be returned</param>
        /// <param name="TableVariableCollectionObj">The TableVariableCollectionObj is returned</param>
        /// <returns>OMError Object</returns>
        private OMError GetTableVariableFromCollection(String TableVariableName, List<TableVariableCollection> TableVariableCollectionListObj, ref TableVariableCollection TableVariableCollectionObj)
        {

            #region Variables

            OMError ReturnErrorObj = null;

            #endregion Variables

            try
            {
                foreach (TableVariableCollection TempTableVariableObj in TableVariableCollectionListObj)
                {
                    if (TempTableVariableObj.m_TableVariableName.CompareTo(TableVariableName) == 0)
                    {
                        TableVariableCollectionObj = TempTableVariableObj;
                        break;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in GetTableVariableFromCollection",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function sets the flag IsTableVariableInWhereClause of object of TableVariableCollection based on TableVariableName 
        /// </summary>
        /// <param name="TableVariableName">The TableVariableName of type String for which the flag is to be set</param>
        /// <param name="TableVariableCollectionListObj">The TableVariableCollectionListObj is type List, which has collection of TableVariableCollection objects</param>
        /// <param name="IsTableVariableInWhereClause">The IsTableVariableInWhereClause of type bool specifies the value to be set</param>
        /// <returns>OMError Object</returns>
        private OMError SetTableVariableFlagInCollection(String TableVariableName, List<TableVariableCollection> TableVariableCollectionListObj, bool IsTableVariableInWhereClause)
        {

            #region Variables

            OMError ReturnErrorObj = null;

            #endregion Variables

            try
            {
                foreach (TableVariableCollection TempTableVariableObj in TableVariableCollectionListObj)
                {
                    if (TempTableVariableObj.m_TableVariableName.CompareTo(TableVariableName) == 0)
                    {
                        TempTableVariableObj.m_IsTableVariableInWhereClause = IsTableVariableInWhereClause;
                        break;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in SetTableVariableFlagInCollection",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function removes the object of TableVariableCollection from TableVariableCollectionList based on TableVariableName 
        /// </summary>
        /// <param name="TableVariableName">The TableVariableName of type String for which the TableVariableCollection object is to be deleted</param>
        /// <param name="TableVariableCollectionListObj">The TableVariableCollectionListObj is type List, which has collection of TableVariableCollection objects</param>
        /// <returns>OMError Object</returns>
        private OMError RemoveTableVariableFromCollection(String TableVariableName, List<TableVariableCollection> TableVariableCollectionListObj)
        {

            #region Variables

            OMError ReturnErrorObj = null;

            #endregion Variables

            try
            {
                foreach (TableVariableCollection TempTableVariableObj in TableVariableCollectionListObj)
                {
                    if (TempTableVariableObj.m_TableVariableName.CompareTo(TableVariableName) == 0)
                    {
                        TableVariableCollectionListObj.Remove(TempTableVariableObj);
                        break;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in RemoveTableVariableFromCollection",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function combines TableVariables based on Operator 
        /// </summary>
        /// <param name="Operator">The Operator specifies the how the TableVariables are to be combined</param>
        /// <param name="Operand1">The Operand1 specifies the TableVariableName</param>
        /// <param name="Operand2">The Operand2 specifies the TableVariableName</param>
        /// <param name="TableVariableCollectionListObj">The TableVariableCollectionListObj is type List, which has collection of TableVariableCollection objects</param>
        /// <param name="WhereClause">The WhereClause of type StringBuilder is returned</param>
        /// <param name="TableVariableName">The TableVariableName of type String is returned which is the new TableVariableName result of combining Operator1 and Operator2</param>
        /// <returns>OMError Object</returns>
        private OMError CombineTableVariable(String Operator, String Operand1, String Operand2, List<TableVariableCollection> TableVariableCollectionListObj, ref StringBuilder WhereClause, ref String TableVariableName)
        {
            #region Variables

            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            TableVariableCollection TableVariableCollectionObj1 = null;         //TableVariableCollection For Operand2
            TableVariableCollection TableVariableCollectionObj2 = null;         //TableVariableCollection For Operand1
            TableVariableCollection TableVariableCollectionObj = null;         //Result of Combining Operand1 and Operand2
            StringBuilder TableVariableQuery = null;         //To generate Table Variable Query for result of combining operand1, operand2
            long UniqueId = 0;

            #endregion Variables

            try
            {
                //Fetch query for TableVariable1 (Operand1)
                ErrorObj = GetTableVariableFromCollection(Operand1, TableVariableCollectionListObj, ref TableVariableCollectionObj1);
                if (ErrorObj != null)
                    return ErrorObj;

                //Fetch query for TableVariable2 (Operand2)
                ErrorObj = GetTableVariableFromCollection(Operand2, TableVariableCollectionListObj, ref TableVariableCollectionObj2);
                if (ErrorObj != null)
                    return ErrorObj;

                ErrorObj = GetUniqueId(ref UniqueId);
                if (ErrorObj != null)
                    return ErrorObj;

                TableVariableCollectionObj = new TableVariableCollection();
                TableVariableQuery = new StringBuilder();
                TableVariableName = "@FILTER_";
                TableVariableName = TableVariableName + ((Operator.CompareTo("AND") == 0 || Operator.CompareTo("and") == 0) ? "INNERJOIN_" : "UNION_");
                //Following code is commented due to the issue reported into Bugzilla 
                // (Bug- 9081: More than 28 Filters on Activity Types doesn't work on Calendars). Following ID generation is not required.
                //TableVariableName = TableVariableName + TableVariableCollectionObj1.m_FilterID + TableVariableCollectionObj2.m_FilterID;
                TableVariableName = TableVariableName + UniqueId.ToString();

                //Add the Table Variable only once in Where Clause
                if (TableVariableCollectionObj1.m_IsTableVariableInWhereClause == false)
                {
                    WhereClause.Append(TableVariableCollectionObj1.m_Query);
                    WhereClause.Append(" ");
                }

                //Add the Table Variable only once in Where Clause
                if (TableVariableCollectionObj2.m_IsTableVariableInWhereClause == false)
                {
                    WhereClause.Append(TableVariableCollectionObj2.m_Query);
                    WhereClause.Append(" ");
                }

                //Set IsTableVariableInWhereClause to true for this Table Variable since it has been added in Where Clause
                ErrorObj = SetTableVariableFlagInCollection(TableVariableCollectionObj1.m_TableVariableName, TableVariableCollectionListObj, true);
                if (ErrorObj != null)
                    return ErrorObj;

                //Set IsTableVariableInWhereClause to true for this Table Variable since it has been added in Where Clause
                ErrorObj = SetTableVariableFlagInCollection(TableVariableCollectionObj2.m_TableVariableName, TableVariableCollectionListObj, true);
                if (ErrorObj != null)
                    return ErrorObj;

                TableVariableQuery.AppendFormat("DECLARE {0} TABLE (FILTEREDOBJID NUMERIC) ", TableVariableName);
                TableVariableQuery.AppendFormat("INSERT INTO {0} (FILTEREDOBJID) ", TableVariableName);

                switch (Operator)
                {
                    case "AND":
                    case "and":
                        {
                            String AliasTableVariable1 = null;
                            String AliasTableVariable2 = null;

                            //Remove @ from TableVariableName to create Alias for TableVariableCollectionObj1.m_TableVariableName
                            AliasTableVariable1 = TableVariableCollectionObj1.m_TableVariableName;
                            AliasTableVariable1 = AliasTableVariable1.Replace("@", "");
                            AliasTableVariable1 = AliasTableVariable1 + "_ALIAS";

                            //Remove @ from TableVariableName to create Alias for TableVariableCollectionObj2.m_TableVariableName
                            AliasTableVariable2 = TableVariableCollectionObj2.m_TableVariableName;
                            AliasTableVariable2 = AliasTableVariable2.Replace("@", "");
                            AliasTableVariable2 = AliasTableVariable2 + "_ALIAS";

                            //Take inner join 
                            TableVariableQuery.AppendFormat("SELECT {1}.FILTEREDOBJID FROM {0} {1} ", TableVariableCollectionObj1.m_TableVariableName, AliasTableVariable1);
                            TableVariableQuery.AppendFormat("INNER JOIN {0} {1} ON {1}.{2} = {3}.{4}", TableVariableCollectionObj2.m_TableVariableName, AliasTableVariable2, TableVariableCollectionObj2.m_TableVariablePrimaryKey, AliasTableVariable1, TableVariableCollectionObj1.m_TableVariablePrimaryKey);

                            AliasTableVariable1 = null;
                            AliasTableVariable2 = null;
                        }
                        break;

                    case "OR":
                    case "or":

                        //Take Union 

                        TableVariableQuery.AppendFormat("SELECT FILTEREDOBJID FROM {0} UNION ", TableVariableCollectionObj1.m_TableVariableName);
                        TableVariableQuery.AppendFormat("SELECT FILTEREDOBJID FROM {0} ", TableVariableCollectionObj2.m_TableVariableName);

                        break;

                }

                TableVariableCollectionObj.m_FilterID = TableVariableCollectionObj1.m_FilterID + TableVariableCollectionObj2.m_FilterID;
                TableVariableCollectionObj.m_TableVariableName = TableVariableName;
                TableVariableCollectionObj.m_Query = TableVariableQuery.ToString();
                TableVariableCollectionObj.m_TableVariablePrimaryKey = "FILTEREDOBJID";
                TableVariableCollectionObj.m_IsTableVariableInWhereClause = true;
                TableVariableCollectionListObj.Add(TableVariableCollectionObj);

                WhereClause.Append(TableVariableCollectionObj.m_Query);
                WhereClause.Append(" ");

                //Remove TableVariables of Operand1 from TableVariableCollection
                ErrorObj = RemoveTableVariableFromCollection(TableVariableCollectionObj1.m_TableVariableName, TableVariableCollectionListObj);
                if (ErrorObj != null)
                    return ErrorObj;

                //Remove TableVariables of Operand2 from TableVariableCollection
                ErrorObj = RemoveTableVariableFromCollection(TableVariableCollectionObj2.m_TableVariableName, TableVariableCollectionListObj);
                if (ErrorObj != null)
                    return ErrorObj;

                TableVariableCollectionObj = null;
                TableVariableQuery = null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in CombineTableVariable",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function combines TableVariables based on Operator 
        /// </summary>
        /// <param name="FilterID">The FilterID specifies the FILTERID for that filter from FilterList Xml</param>
        /// <param name="HierarchyObj">The HierarchyObj specifies the details of the Target Entity</param>
        /// <param name="TableVariableCollectionListObj">The TableVariableCollectionListObj is type List, which has collection of TableVariableCollection objects</param>
        /// <param name="TableVariableName">The TableVariableName of type String is returned which is the new TableVariableName generated for FilterID</param>
        /// <returns>OMError Object</returns>
        private OMError PrepareTableVariable(String FilterID, HierarchyObject HierarchyObj, List<TableVariableCollection> TableVariableCollectionListObj, ref String TableVariableName)
        {
            #region Variables
            XmlNode ObjectTypeNode = null;
            XmlNode RelationshipNode = null;
            XmlNode PrimaryKeyNode = null;
            OMError ReturnErrorObj = null;
            OMError ErrorObj = null;
            StringBuilder FilterClause = null;
            StringBuilder TableVariableQuery = null;
            String RelationshipName = null;
            ulong RELATIONSHIPTYPEID = 0;
            ulong CONTAINERTYPEID = 0;
            ulong CONTAINSTYPEID = 0;
            ulong RelatedTypeID = 0;
            String CONTAINERTYPE = null;
            String CONTAINSTYPE = null;
            String RelatedType = null;
            String AssemblyColumnName = null;
            String RelatedTypeName = null;
            String TableName = null;
            bool IsFilterPresent = false;
            bool IsCyclicRelation = false;
            FilterCollection TempFilterCollection = null;
            TableVariableCollection TableVariableCollectionObj = null;
            long UniqueId = 0;
            string InsertStatement = null;

            #endregion Variables

            try
            {
                IsFilterPresent = false;

                foreach (FilterCollection FilterCollection in HierarchyObj.FilterCollectionList)
                {
                    if (FilterCollection.m_FilterID.ToString() == FilterID)
                    {
                        IsFilterPresent = true;
                        TempFilterCollection = FilterCollection;
                        break;
                    }
                }

                if (IsFilterPresent == false)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                      "Filter (" + FilterID + ") not found", OMErrorObjectType.GETDATAQUERYBUILDER,
                      "NSearch", OMErrorType.ERROR, null, null);
                    return ReturnErrorObj;
                }

                //Get UniqueId
                ErrorObj = GetUniqueId(ref UniqueId);
                if (ErrorObj != null)
                    return ErrorObj;

                TableVariableQuery = new StringBuilder();

                RelationshipName = TempFilterCollection.m_RelationshipName;
                if (RelationshipName == null || RelationshipName == "")
                {
                    //The Filter is on TE
                    bool IsINToBeOptimized = HierarchyObj.m_IsFilterCombinationAdded;

                    TableVariableName = "@FILTER_" + HierarchyObj.m_TableName + "_" + FilterID + UniqueId.ToString();

                    TableVariableQuery.AppendFormat("DECLARE {0} TABLE ", TableVariableName);
                    TableVariableQuery.Append("(FILTEREDOBJID NUMERIC) ");

                    //Read TE details
                    ErrorObj = ReadTypeDetails(HierarchyObj.m_ObjectType, HierarchyObj.m_ObjectTypeID, ref ObjectTypeNode);
                    if (ErrorObj != null)
                        return ErrorObj;

                    FilterClause = new StringBuilder();

                    ErrorObj = GetFilterString(HierarchyObj.m_TableName, TempFilterCollection, ref FilterClause, ref ObjectTypeNode, IsINToBeOptimized);
                    if (ErrorObj != null)
                        return ErrorObj;

                    //Separate handling for IN clause - IN optimization feature
                    if (TempFilterCollection.m_IsINOptimized)
                    {
                        ErrorObj = GetUniqueId(ref UniqueId);
                        if (ErrorObj != null)
                            return ErrorObj;                                               

                        string TempXmlDocumentVarName = "@xmlDocument" + UniqueId;
                        string TempDocHandleVarName = "@docHandle" + UniqueId;

                        TableVariableQuery.Append("DECLARE ");
                        TableVariableQuery.Append(TempDocHandleVarName);
                        TableVariableQuery.Append(" int ");

                        TableVariableQuery.Append("declare ");
                        TableVariableQuery.Append(TempXmlDocumentVarName);
                        TableVariableQuery.Append(" nvarchar(max) ");
                        TableVariableQuery.Append("set " + TempXmlDocumentVarName + " = N''");

                        TableVariableQuery.Append(FilterClause);

                        TableVariableQuery.Append("''");

                        TableVariableQuery.Append("EXEC sp_xml_preparedocument " + TempDocHandleVarName + " OUTPUT, " + TempXmlDocumentVarName + " ");
                        TableVariableQuery.Append("INSERT ");
                        TableVariableQuery.Append(TableVariableName);
                        TableVariableQuery.Append(" SELECT *  ");
                        TableVariableQuery.Append("FROM OPENXML(" + TempDocHandleVarName + ", N''/ROOT/IDLIST'')  ");
                        TableVariableQuery.Append("  WITH (FILTEREDOBJID NUMERIC) ");
                        TableVariableQuery.Append("EXEC sp_xml_removedocument " + TempDocHandleVarName + " "); 
                    }
                    else
                    {
                        InsertStatement = string.Format("INSERT INTO {0} (FILTEREDOBJID) ", TableVariableName);

                        TableVariableQuery.AppendFormat(InsertStatement);
                        TableVariableQuery.AppendFormat("SELECT {0} FROM {1} ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName);
                        TableVariableQuery.AppendFormat("WHERE {0}", FilterClause);
                    }

                    TableVariableCollectionObj = new TableVariableCollection();
                    TableVariableCollectionObj.m_TableVariableName = TableVariableName;
                    TableVariableCollectionObj.m_Query = TableVariableQuery.ToString();
                    TableVariableCollectionObj.m_TableVariablePrimaryKey = "FILTEREDOBJID";
                    TableVariableCollectionObj.m_FilterID = FilterID;
                    TableVariableCollectionObj.m_IsTableVariableInWhereClause = false;
                    TableVariableCollectionListObj.Add(TableVariableCollectionObj);
                }
                else
                {
                    TableVariableName = "@FILTER_" + RelationshipName + "_" + FilterID + UniqueId.ToString();

                    TableVariableQuery.AppendFormat("DECLARE {0} TABLE ", TableVariableName);
                    TableVariableQuery.Append("(FILTEREDOBJID NUMERIC) ");
                    TableVariableQuery.AppendFormat("INSERT INTO {0} (FILTEREDOBJID) ", TableVariableName);

                    StringBuilder sbSelectStatement = new StringBuilder();

                    ErrorObj = ReadByRelationShipNameEx(RelationshipName, ref RelationshipNode);
                    if (ErrorObj != null)
                    {
                        ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                                            ErrorObj.ErrorMessage, ErrorObj.ObjectType, ErrorObj.ObjectTypeName,
                                            ErrorObj.ErrorType, ErrorObj.ExceptionObject, ErrorObj.ErrorXML);
                        return ReturnErrorObj;
                    }

                    // Resolve EntityType or Facet Type.

                    RELATIONSHIPTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("RELATIONSHIPTYPEID").InnerText.ToString());
                    CONTAINERTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINERTYPEID").InnerText.ToString());
                    CONTAINSTYPEID = UInt64.Parse(RelationshipNode.SelectSingleNode("CONTAINSTYPEID").InnerText.ToString());
                    CONTAINERTYPE = RelationshipNode.SelectSingleNode("CONTAINERTYPE").InnerText.ToString();
                    CONTAINSTYPE = RelationshipNode.SelectSingleNode("CONTAINSTYPE").InnerText.ToString();

                    // Processing for Relationship where ContainerTypeID,ContainsTypeID and ObjectTypeID are same.
                    if (CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "ENTITY")
                    {

                        if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                        {
                            RelatedTypeID = CONTAINSTYPEID;
                            RelatedType = "ENTITY";
                            AssemblyColumnName = "CONTAINSID";
                        }
                        else
                        {
                            if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                            {
                                RelatedTypeID = CONTAINERTYPEID;
                                RelatedType = "ENTITY";
                                AssemblyColumnName = "CONTAINERID";
                            }
                        }

                        //Cyclic Relationship
                        if (CONTAINERTYPEID == HierarchyObj.m_ObjectTypeID && CONTAINSTYPEID == HierarchyObj.m_ObjectTypeID)
                        {
                            IsCyclicRelation = true;
                            if (TempFilterCollection.m_IsParent != null)
                            {
                                if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("BOTTOMUP") == 0)
                                    AssemblyColumnName = "CONTAINERID";
                            }
                        }

                        RelatedTypeName = "RELATEDENTITY";
                    }
                    else if (CONTAINERTYPE.ToUpper() == "FACET" && CONTAINSTYPE.ToUpper() == "FACET")
                    {
                        //Processing for facet-facet relationship present in the filter 

                        if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                        {
                            RelatedTypeID = CONTAINSTYPEID;
                            RelatedType = "FACET";
                            AssemblyColumnName = "CONTAINSID";
                        }
                        else
                        {
                            if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                            {
                                RelatedTypeID = CONTAINERTYPEID;
                                RelatedType = "FACET";
                                AssemblyColumnName = "CONTAINERID";
                            }
                        }

                        //Cyclic Relationship
                        if (CONTAINERTYPEID == HierarchyObj.m_ObjectTypeID && CONTAINSTYPEID == HierarchyObj.m_ObjectTypeID)
                        {
                            IsCyclicRelation = true;
                            if (TempFilterCollection.m_IsParent != null)
                            {
                                if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("BOTTOMUP") == 0)
                                    AssemblyColumnName = "CONTAINERID";
                            }
                        }

                        RelatedTypeName = "FACETFACET";
                    }

                    else
                    {
                        if (CONTAINERTYPE.ToUpper() == "ENTITY" && CONTAINSTYPE.ToUpper() == "FACET")
                        {
                            if (HierarchyObj.m_ObjectTypeID == CONTAINERTYPEID)
                            {
                                RelatedTypeID = CONTAINSTYPEID;
                                // This is because Facet cannot be container for Entity.
                                RelatedType = "FACET";

                            }
                            else
                            {
                                if (HierarchyObj.m_ObjectTypeID == CONTAINSTYPEID)
                                {
                                    RelatedTypeID = CONTAINERTYPEID;
                                    RelatedType = "ENTITY";
                                }
                            }
                            RelatedTypeName = "RELATEDFACET";
                        }
                    }

                    //Read Related Entity/Facet Type Details
                    ErrorObj = ReadTypeDetails(RelatedType, RelatedTypeID, ref ObjectTypeNode);
                    if (ErrorObj != null)
                        return ErrorObj;

                    PrimaryKeyNode = ObjectTypeNode.SelectSingleNode("PRIMARYKEY");



                    

                    if (RelatedTypeName == "RELATEDENTITY")
                    {
                        TableVariableQuery.AppendFormat("SELECT {1}{2}.{0} FROM {1} {1}{2} ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, FilterID);

                        if (IsCyclicRelation == true && TempFilterCollection.m_IsParent != null)
                        {
                            if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("BOTTOMUP") == 0)
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            else if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("TOPDOWN") == 0)
                            {
                                if (AssemblyColumnName == "CONTAINSID")
                                    TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINERID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                else //if (AssemblyColumnName == "CONTAINERID")
                                    TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            }
                        }
                        else
                        {
                            if (AssemblyColumnName == "CONTAINSID")
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINERID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            else //if (AssemblyColumnName == "CONTAINERID")
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                        }

                        TableVariableQuery.AppendFormat("AND ASSEMBLY{0}.RELATIONSHIPTYPEID = {1} ", FilterID, RELATIONSHIPTYPEID);

                        //Check whether the Filter is on the primary key of the RE
                        //If it is no primary key of the RE, there is no need to take join on RE
                        //Otherwise take the join with RE

                        //if (PrimaryKeyNode.InnerText.CompareTo(TempFilterCollection.m_FieldName.ToUpper()) != 0)
                        {
                            TableName = "ALENT_" + ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText;

                            if (AssemblyColumnName == "CONTAINSID")
                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSEMBLY{1}.CONTAINSID = {2}.{3} ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);
                            else //if(AssemblyColumnName == "CONTAINERID")
                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSEMBLY{1}.CONTAINERID = {2}.{3}  ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);

                            FilterClause = new StringBuilder();

                            ErrorObj = GetFilterString(TableName, TempFilterCollection, ref FilterClause, ref ObjectTypeNode, false);

                            if (ErrorObj != null)
                                return ErrorObj;

                            TableVariableQuery.AppendFormat("WHERE {0}", FilterClause);
                        }
                        /*else
                        {
                            //If the filter is on PrimaryKey include filter in where clause
                            //AssemblyColumnName = CONTAINSID specifies that the where clause is to be taken on CONTAINS
                            if (AssemblyColumnName == "CONTAINSID")
                                TableVariableQuery.AppendFormat("WHERE ASSEMBLY{0}.CONTAINSID = {1} ", FilterID, TempFilterCollection.m_FieldValue);
                            else
                                TableVariableQuery.AppendFormat("WHERE ASSEMBLY{0}.CONTAINERID = {1} ", FilterID,TempFilterCollection.m_FieldValue);
                        }*/
                    }
                    else if (RelatedTypeName == "FACETFACET")
                    {
                        //Query formation for facet-facet relationships

                        TableVariableQuery.AppendFormat("SELECT {1}{2}.{0} FROM {1} {1}{2} ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, FilterID);

                        if (IsCyclicRelation == true && TempFilterCollection.m_IsParent != null)
                        {
                            if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("BOTTOMUP") == 0)
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            else if (TempFilterCollection.m_IsParent.ToUpper().CompareTo("TOPDOWN") == 0)
                            {
                                if (AssemblyColumnName == "CONTAINSID")
                                    TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINERID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                                else //if (AssemblyColumnName == "CONTAINERID")
                                    TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            }
                        }
                        else
                        {
                            if (AssemblyColumnName == "CONTAINSID")
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINERID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            else //if (AssemblyColumnName == "CONTAINERID")
                                TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSEMBLY ASSEMBLY{0} ON {1}{0}.{2} = ASSEMBLY{0}.CONTAINSID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                        }

                        TableVariableQuery.AppendFormat("AND ASSEMBLY{0}.RELATIONSHIPTYPEID = {1} ", FilterID, RELATIONSHIPTYPEID);

                        //Check whether the Filter is on the primary key of the RE
                        //If it is no primary key of the RE, there is no need to take join on RE
                        //Otherwise take the join with RE
                        
                        {
                            TableName = "ALFCT_" + ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText;

                            if (AssemblyColumnName == "CONTAINSID")
                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSEMBLY{1}.CONTAINSID = {2}.{3} ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);
                            else //if(AssemblyColumnName == "CONTAINERID")
                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSEMBLY{1}.CONTAINERID = {2}.{3}  ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);

                            FilterClause = new StringBuilder();

                            ErrorObj = GetFilterString(TableName, TempFilterCollection, ref FilterClause, ref ObjectTypeNode, false);

                            if (ErrorObj != null)
                                return ErrorObj;

                            TableVariableQuery.AppendFormat("WHERE {0}", FilterClause);
                        }
                       
                    }
                    else //if (RelatedTypeName == "RELATEDFACET")
                    {
                        //In case of entity-facet relationships separate handling done for entity as parent or facet as parent
                        if (HierarchyObj.m_ObjectType.ToUpper() == "ENTITY")
                        {
                            TableVariableQuery.AppendFormat("SELECT {1}{2}.{0} FROM {1} {1}{2} ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, FilterID);

                            //Since Entity would always be CONTAINER for Facet, the CONTAINS case is not handled

                            TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSOENTITYFACET ASSOENTFCT{0} ON {1}{0}.{2} = ASSOENTFCT{0}.ENTITYID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            TableVariableQuery.AppendFormat("AND (ASSOENTFCT{0}.RELATIONSHIPTYPEID = {1} OR ASSOENTFCT{0}.RELATIONSHIPTYPEID IS NULL)  ", FilterID, RELATIONSHIPTYPEID);

                            //if (PrimaryKeyNode.InnerText.CompareTo(TempFilterCollection.m_FieldName.ToUpper()) != 0)
                            {
                                TableName = "ALFCT_" + ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText;

                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSOENTFCT{1}.FACETID = {2}.{3} ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);
                                FilterClause = new StringBuilder();

                                ErrorObj = GetFilterString(TableName, TempFilterCollection, ref FilterClause, ref ObjectTypeNode, false);

                                if (ErrorObj != null)
                                    return ErrorObj;

                                TableVariableQuery.AppendFormat("WHERE {0}", FilterClause);
                            }
                            /*else
                            {
                                TableVariableQuery.AppendFormat("WHERE ASSOENTFCT{0}.FACETID = {1} ",FilterID,TempFilterCollection.m_FieldValue);
                            }*/
                        }
                        else if (HierarchyObj.m_ObjectType.ToUpper() == "FACET")
                        {
                            TableVariableQuery.AppendFormat("SELECT {1}{2}.{0} FROM {1} {1}{2} ", HierarchyObj.m_PrimaryKeyName, HierarchyObj.m_TableName, FilterID);
                            
                            TableVariableQuery.AppendFormat("INNER JOIN ALOM_ASSOENTITYFACET ASSOENTFCT{0} ON {1}{0}.{2} = ASSOENTFCT{0}.FACETID ", FilterID, HierarchyObj.m_TableName, HierarchyObj.m_PrimaryKeyName);
                            TableVariableQuery.AppendFormat("AND (ASSOENTFCT{0}.RELATIONSHIPTYPEID = {1} OR ASSOENTFCT{0}.RELATIONSHIPTYPEID IS NULL)  ", FilterID, RELATIONSHIPTYPEID);
                            
                            {
                                TableName = "ALENT_" + ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText;

                                TableVariableQuery.AppendFormat("INNER JOIN {0} ON ASSOENTFCT{1}.ENTITYID = {2}.{3} ", TableName, FilterID, TableName, PrimaryKeyNode.InnerText);
                                FilterClause = new StringBuilder();

                                ErrorObj = GetFilterString(TableName, TempFilterCollection, ref FilterClause, ref ObjectTypeNode, false);

                                if (ErrorObj != null)
                                    return ErrorObj;

                                TableVariableQuery.AppendFormat("WHERE {0}", FilterClause);
                            }                            
                        }
                                                    
                    }

                    TableVariableCollectionObj = new TableVariableCollection();
                    TableVariableCollectionObj.m_TableVariableName = TableVariableName;
                    TableVariableCollectionObj.m_Query = TableVariableQuery.ToString();
                    TableVariableCollectionObj.m_TableVariablePrimaryKey = "FILTEREDOBJID";
                    TableVariableCollectionObj.m_FilterID = FilterID;
                    TableVariableCollectionObj.m_IsTableVariableInWhereClause = false;
                    TableVariableCollectionListObj.Add(TableVariableCollectionObj);
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareTableVariable",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function validates Filtercombination
        /// FilterString is as follows : (1(2)3)
        /// </summary>
        /// <param name="FilterCombinationString">String of filter combination</param>
        /// <returns></returns>
        private bool ValidateFilterCombination(String FilterCombinationString)
        {

            Stack FilterCombinationStack = null;

            try
            {
                if (FilterCombinationString != null && FilterCombinationString != "")
                {
                    FilterCombinationStack = new Stack();
                    foreach (Char ch in FilterCombinationString)
                    {
                        switch (ch)
                        {
                            case '(':
                                FilterCombinationStack.Push(ch.ToString());
                                break;
                            case ')':
                                FilterCombinationStack.Pop();
                                break;
                        }
                    }
                    if (FilterCombinationStack.Count == 0)
                        return true;
                }

                return false;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// This function adds AssemblyAliasName in RelationshipCollection.
        /// </summary>
        /// <param name="RelationshipCollection">RelationshipCollection List object</param>
        /// <param name="RelationshipName">RelationshipName for which AssemblyAliasName is to be associated.</param>
        /// <param name="AssemblyAliasName">AssemblyAliasName to be associated with RelationshipName.</param>
        /// <returns></returns>
        private OMError AddAssemblyAliasName(List<RelationshipCollection> RelationshipCollectionListObj, String RelationshipName, String AssemblyAliasName)
        {
            #region Variables

            RelationshipCollection RelationshipCollectionObj = null;
            IEnumerator IEnumeratorObj = null;
            OMError ReturnErrorObj = null;

            #endregion Variables

            try
            {
                if (RelationshipCollectionListObj.Count == 0)
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in AddAssemblyAliasName : Relationship not found in collection",
                     OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, null, "");
                    
                    return ReturnErrorObj;
                }

                IEnumeratorObj = RelationshipCollectionListObj.GetEnumerator();

                while (IEnumeratorObj.MoveNext())
                {
                    RelationshipCollectionObj = IEnumeratorObj.Current as RelationshipCollection;

                    if (RelationshipCollectionObj != null)
                    {
                        if (RelationshipCollectionObj.m_RelationshipName.ToUpper().CompareTo(RelationshipName.ToUpper()) == 0)
                        {
                            RelationshipCollectionObj.m_AssemblyAliasName = AssemblyAliasName;
                            return null;
                        }
                    }
                    RelationshipCollectionObj = null;
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in AddAssemblyAliasName",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// This function returns RelationshipCollection matching to the RelationshipName in RelationshipCollection.
        /// </summary>
        /// <param name="RelationshipCollection">RelationshipCollection List object</param>
        /// <param name="RelationshipName">RelationshipName for which AssemblyAliasName is to be associated.</param>
        /// <param name="AssemblyAliasName">AssemblyAliasName to be associated with RelationshipName.</param>
        /// <returns></returns>
        private OMError GetRelationshipCollectionByName(List<RelationshipCollection> RelationshipCollectionListObj, String RelationshipName, ref RelationshipCollection RetRelationshipCollectionObj)
        {
            #region Variables

            RelationshipCollection RelationshipCollectionObj = null;
            IEnumerator IEnumeratorObj = null;
            OMError ReturnErrorObj = null;

            #endregion Variables

            try
            {
                if (RelationshipCollectionListObj.Count == 0)
                {
                    return null;
                }

                IEnumeratorObj = RelationshipCollectionListObj.GetEnumerator();

                while (IEnumeratorObj.MoveNext())
                {
                    RelationshipCollectionObj = IEnumeratorObj.Current as RelationshipCollection;

                    if (RelationshipCollectionObj != null)
                    {
                        if (RelationshipCollectionObj.m_RelationshipName.ToUpper().CompareTo(RelationshipName.ToUpper()) == 0)
                        {
                            RetRelationshipCollectionObj = RelationshipCollectionObj;
                            return null;
                        }
                    }
                    RelationshipCollectionObj = null;
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in AddAssemblyAliasName",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        private OMError PrepareWhereClause(SortedList ClauseCollection, List<RelationshipCollection> RelationshipCollectionListObj, ref StringBuilder WhereClause)
        {
            #region Variables

            RelationshipCollection      RelationshipCollectionObj               = null;
            IEnumerator                 IEnumeratorObj                          = null;
            OMError                     ReturnErrorObj                          = null;
            String                      TmpRelationTypeName                     = null;
            int                         RelationshipNameIndexInClauseCollection = 0;
            ArrayList                   ClauseArrayList                         = null;
            int                         Counter                                 = 0;
            bool                        IsFirstTime                             = true;
            String                      StrDeleteClause                         = null;
            int                         LoopCounter                             = 0;
            ArrayList                   FilterListtoDelete                      = null;

            #endregion Variables

            try
            {
                if (RelationshipCollectionListObj.Count == 0)
                {
                    //Append the TE filters in Where Clause
                    for (Counter = 0; Counter < ClauseCollection.Count; Counter++)
                    {
                        if (ClauseCollection.GetByIndex(Counter) != null)
                        {
                            if (Counter != ClauseCollection.Count - 1)
                            {
                                WhereClause.AppendFormat("({0}) AND ", ClauseCollection.GetByIndex(Counter).ToString());
                            }
                            else
                                WhereClause.AppendFormat("({0})", ClauseCollection.GetByIndex(Counter).ToString());

                        }
                    }

                    return null;
                }

                IEnumeratorObj = RelationshipCollectionListObj.GetEnumerator();
                IsFirstTime = true;

                while (IEnumeratorObj.MoveNext())
                {
                    RelationshipCollectionObj = IEnumeratorObj.Current as RelationshipCollection;

                    if (RelationshipCollectionObj != null)
                    {
                        ClauseArrayList = new ArrayList();
                        StrDeleteClause = null;
                        for (Counter = 0; Counter < ClauseCollection.Count; Counter++)
                        {
                            //Check whether RelationshipName is present in ClauseCollection list
                            RelationshipNameIndexInClauseCollection = 0;
                            TmpRelationTypeName = null;

                            RelationshipNameIndexInClauseCollection = ClauseCollection.GetKey(Counter).ToString().IndexOf(".");

                            //Extract RelationshipName from ClauseCollection and check whether it is present in RelationshipTypeName_IDList
                            TmpRelationTypeName = ClauseCollection.GetKey(Counter).ToString().Substring(0, RelationshipNameIndexInClauseCollection);

                            if (RelationshipCollectionObj.m_RelationshipName.CompareTo(TmpRelationTypeName) == 0)
                            {
                                ClauseArrayList.Add(ClauseCollection.GetByIndex(Counter).ToString());
                                ClauseCollection.SetByIndex(Counter, null);

                                if (String.IsNullOrEmpty(StrDeleteClause))
                                    StrDeleteClause = Counter.ToString() + ";";
                                else
                                    StrDeleteClause = StrDeleteClause + Counter.ToString() + ";";
                            }
                        }

                        if (ClauseArrayList.Count > 1)
                        {
                            if (IsFirstTime == false)
                                WhereClause.Append(" AND ");

                            WhereClause.Append("(");

                            foreach (String Clause in ClauseArrayList)
                            {
                                if (Clause.CompareTo(ClauseArrayList[ClauseArrayList.Count - 1]) != 0)
                                {
                                    WhereClause.AppendFormat("({0}) AND ", Clause);

                                }
                                else
                                    WhereClause.AppendFormat("({0}) ", Clause);

                            }

                            if (RelationshipCollectionObj.m_AssemblyAliasName != null || RelationshipCollectionObj.m_AssemblyAliasName != "")
                                WhereClause.AppendFormat(" AND ({0}.RELATIONSHIPTYPEID = {1})", RelationshipCollectionObj.m_AssemblyAliasName, RelationshipCollectionObj.m_RelationshionTypeId);

                            WhereClause.Append(")");
                        }
                        else
                        {
                            if (ClauseArrayList.Count == 1)
                            {
                                if (IsFirstTime == false)
                                    WhereClause.Append(" AND ");

                                if (RelationshipCollectionObj.m_AssemblyAliasName != null || RelationshipCollectionObj.m_AssemblyAliasName != "")
                                {
                                    WhereClause.AppendFormat("(({0}) AND ", ClauseArrayList[0].ToString());
                                    WhereClause.AppendFormat("({0}.RELATIONSHIPTYPEID = {1}))", RelationshipCollectionObj.m_AssemblyAliasName, RelationshipCollectionObj.m_RelationshionTypeId);
                                }
                                else
                                    WhereClause.AppendFormat("({0})", ClauseArrayList[0].ToString());
                            }
                        }
                        IsFirstTime = false;
                    }
                    RelationshipCollectionObj = null;
                }

                IsFirstTime = true;

                //if (!String.IsNullOrEmpty(StrDeleteClause))
                //{
                //    int deletedCount = 0;
                //    String[] DeleteClauseArr = StrDeleteClause.Split(';');

                //    //Remove the Clauses from the ClauseCollection for which the processing is completed
                //    foreach (String DeleteClauseID in DeleteClauseArr)
                //    {
                //        if (!String.IsNullOrEmpty(DeleteClauseID))
                //        {
                //            Counter = Convert.ToInt32(DeleteClauseID);
                //            ClauseCollection.RemoveAt(Counter - deletedCount);
                //            deletedCount++;
                //        }
                //    }
                //}

                // This loop deletes all the processed Filters whose value is NULL.
                FilterListtoDelete = new ArrayList();
                LoopCounter = 0;
                foreach (DictionaryEntry FilterEntry in ClauseCollection)
                {
                    if (FilterEntry.Value == null)
                    {
                        FilterListtoDelete.Add(LoopCounter);
                    }
                    LoopCounter++;
                }

                if (FilterListtoDelete.Count > 0)
                    FilterListtoDelete.Reverse();
 
                foreach (Int32 IndexCounter in FilterListtoDelete)
                {
                    ClauseCollection.RemoveAt(IndexCounter);
                }

                //Append the TE filters in Where Clause
                for (Counter = 0; Counter < ClauseCollection.Count; Counter++)
                {
                    if (ClauseCollection.GetByIndex(Counter) != null)
                    {
                        //if (ClauseArrayList.Count > 0)
                        if (IsFirstTime == true)
                            WhereClause.Append(" AND ");

                        if (Counter != ClauseCollection.Count - 1)
                            WhereClause.AppendFormat("({0}) AND ", ClauseCollection.GetByIndex(Counter).ToString());
                        else
                            WhereClause.AppendFormat("({0})", ClauseCollection.GetByIndex(Counter).ToString());

                        IsFirstTime = false;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareWhereClause",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
                
                return ReturnErrorObj;
            }
            return null;
        }

        /// <summary>
        /// Prepares sort column values. This function will prepare sort column values keeping parent sort column values as it is and increasing
        /// sort column value for binder to support fetching multiple binder fields (will files).
        /// </summary>
        /// <param name="FieldType">Data type of filed (BINDER/FILE).</param>
        /// <param name="FieldIndex">Index of field from list (starting from 1). e.g. Index of Binder field from number of binder fields requested.</param>
        /// <param name="FileSystemSortPartition">Sort partition object for File System (relationship).</param>
        /// <param name="SortPartitionNamedValueList">Sort partition named value list containing sort partition name and value.</param>
        /// <param name="SortColumnValue">Output parameter, sort column values will be returned (separated by comma)</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError PrepareSortColumnValues(String FieldType, int FieldIndex, SortPartition FileSystemSortPartition, 
                                        List<SortPartitionNamedValue> SortPartitionNamedValueList, ref StringBuilder SortColumnValue)
        {
            OMError                     obj_OMError                     = null;     //Used to prepare OM Error
            SortPartitionNamedValue     obj_TempSortPartitionNamedValue = null;     //Used to hold sort partition named value
            int                         SortPartitionValue              = 0;        //Used to store Sort Partition value for File System Sort Partition (input param FileSystemSortPartition)
            bool                        b_OnlyBinderFile                = false;    //Used to identify whether the sort column values are to be prepared for binder files along with binder
            
            try
            {
                if (SortColumnValue == null)
                    SortColumnValue = new StringBuilder(100);
                else
                    SortColumnValue.Remove(0, SortColumnValue.Length);

                switch (FieldType.ToUpper())
                {
                    #region case "BINDERFILE"
                    //This case is specific for binder files to get sort column values.
                    case "BINDERFILE":
                        b_OnlyBinderFile = true;
                        goto case "BINDER";

                    #endregion case "BINDERFILE"

                    #region case "BINDER"
                    //This should be called when binder fields requested.
                    case "BINDER":
                        //Iterate over sort partition column list (which is for entire search)
                        foreach (DictionaryEntry obj_SortPartition in m_NSearchCommonObj.m_SortPartitionColumnsList)
                        {
                            obj_TempSortPartitionNamedValue = null;
                            //Fetch the sort partition value for each sort partition column
                            foreach (SortPartitionNamedValue obj_SortPartitionNamedValue in SortPartitionNamedValueList)
                            {
                                if (obj_SortPartitionNamedValue.SortPartitionName.ToUpper().CompareTo(obj_SortPartition.Value.ToString()) == 0)
                                {
                                    obj_TempSortPartitionNamedValue = obj_SortPartitionNamedValue;
                                    break;
                                }
                            }

                            //If sort partition value is found, append it to SortColumnValue, otherwise append NULL
                            if (obj_TempSortPartitionNamedValue != null)
                            {
                                /* Assigning Sort Column values for multiple binder fields: 
                                 * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                 *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                 * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                 *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                 *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                 * This will append FILE nodes to appropriate BINDER nodes. 
                                 * */
                                if (obj_TempSortPartitionNamedValue.SortPartitionName.ToUpper().CompareTo(FileSystemSortPartition.m_SortPartitionColumnName.ToUpper()) == 0)
                                {
                                    SortColumnValue.Append(", ");
                                    SortPartitionValue = Convert.ToInt32(obj_TempSortPartitionNamedValue.SortPartitionValue);

                                    /* For Binder File : The sort partition value needs to be incremented. This value will be not be equal to sort column value for binder field but it will be +1.
                                     * This is done so that the implementation will be uniform across databases.
                                     * */
                                    if (b_OnlyBinderFile)
                                        SortColumnValue.Append((SortPartitionValue * FieldIndex) + 1);
                                    else
                                        SortColumnValue.Append(SortPartitionValue * FieldIndex);
                                }
                                else
                                {
                                    SortColumnValue.Append(", ");
                                    SortColumnValue.Append(obj_TempSortPartitionNamedValue.SortPartitionValue);
                                }
                            }
                            else
                            {
                                SortColumnValue.Append(", NULL");
                            }
                        }

                        break;

                    #endregion case "BINDER"

                    #region case "FILE"
                    //This should be called only when multiple file and binder fields requested and not in case when only file fields are requested.
                    case "FILE":
                        //Iterate over sort partition column list (which is for entire search)
                        foreach (DictionaryEntry obj_SortPartition in m_NSearchCommonObj.m_SortPartitionColumnsList)
                        {
                            obj_TempSortPartitionNamedValue = null;
                            //Fetch the sort partition value for each sort partition column
                            foreach (SortPartitionNamedValue obj_SortPartitionNamedValue in SortPartitionNamedValueList)
                            {
                                if (obj_SortPartitionNamedValue.SortPartitionName.ToUpper().CompareTo(obj_SortPartition.Value.ToString()) == 0)
                                {
                                    obj_TempSortPartitionNamedValue = obj_SortPartitionNamedValue;
                                    break;
                                }
                            }

                            //If sort partition value is found, append it to SortColumnValue, otherwise append NULL
                            if (obj_TempSortPartitionNamedValue != null)
                            {
                                /* Assigning Sort Column values for files when multiple binder and file fields are requested in output: 
                                 * 1. For the first binder field, the value will be assigned which is assigned for FILESYSTEM relationship. 
                                 *    For binder files, sort column value for binder field + 1 will be assigned. This is done so that the implementation will be uniform across databases.
                                 * 2. For next binder fields, the value would be incremented in multiples of 2. 
                                 *    e.g. If sort column value assigned for FILESYSTEM relationship is 2, then first binder field will get value 2 and binder files will get value 3;
                                 *    whereas second binder field will get value 4 and its files will get value 5 and so on.
                                 * 3. Now, sort column values will be prepared for binder fields as specified in step 1 and 2, 
                                 *    Step 3 will consider file fields. Sort column values for file fields will be assigned from next slot after processing binder fields by considering binder field count.
                                 *    e.g. 2 binder fields and 2 file fields are requested, sort column values will be
                                 *    3.1 Binder 1 - Sort column value is 2, for binder files value would be 3
                                 *    3.2 Binder 2 - Sort column value is 4, for binder files value would be 5
                                 *    3.3 file     - Sort column value is 6 (for all files)
                                 *    This is done so that all file (non binder) fields will be fetched at the end in FILESYSTEM node.
                                 * This will append FILE nodes to appropriate BINDER nodes. 
                                 * */
                                if (obj_TempSortPartitionNamedValue.SortPartitionName.ToUpper().CompareTo(FileSystemSortPartition.m_SortPartitionColumnName.ToUpper()) == 0)
                                {
                                    SortColumnValue.Append(", ");
                                    SortPartitionValue = Convert.ToInt32(obj_TempSortPartitionNamedValue.SortPartitionValue);
                                    SortColumnValue.Append(SortPartitionValue * (FieldIndex + 1));
                                }
                                else
                                {
                                    SortColumnValue.Append(", ");
                                    SortColumnValue.Append(obj_TempSortPartitionNamedValue.SortPartitionValue);
                                }
                            }
                            else
                            {
                                SortColumnValue.Append(", NULL");
                            }
                        }

                        break;

                    #endregion case "FILE"
                }
            }
            catch (Exception ExceptionObj)
            {
                obj_OMError = new OMError(OMErrorCodes.GETDATA_FAILED, "Exception in PrepareSortColumnValue",
                    OMErrorObjectType.GETDATAQUERYBUILDER, "", OMErrorType.EXCEPTION, ExceptionObj, "");
            }

            return obj_OMError;
        }

        /// <summary>
        /// This is a method which reads Relation ship information
        /// </summary>
        /// <param name="RelationShipName">Relationship Name</param>
        /// <param name="RootNode">returns a Relationship XML node</param>
        /// <returns>OMError object if an error is encountered else NULL</returns>
        public OMError ReadByRelationShipNameEx(String RelationShipName, ref XmlNode RootNode)
        {
            OMError         ReadErrObj          = null;   //OMError object for reading data from DB.
            OMError         ReturnErrorObj      = null;   //OMError object for returning from DAL.
            String          AssociationXML      = null;   //String variable storing the AssociationXML.
            XmlDocument     OutputXMLDocument   = null;   //The XMLDocument representing output XML.

            try
            {
                if ((ReadErrObj = AddClause("RELATIONSHIPNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, RelationShipName, OMConjunction.NOTHING)) != null)
                    return ReadErrObj;

                try
                {
                    if ((ReadErrObj = ReadRecord("RELATIONSHIPTYPE", "ALOM_RELATIONSHIPTYPES", null, ref AssociationXML)) != null)
                        return ReadErrObj;
                }
                catch
                {
                }
                finally
                {
                    if (DataReaderObj != null && !DataReaderObj.IsClosed)
                        DataReaderObj.Close();
                }
                if (ReadErrObj != null || String.IsNullOrEmpty(AssociationXML))
                {
                    ReturnErrorObj = new OMError(OMErrorCodes.ASSOCIATION_NOT_FOUND,
                        "Relationship Name { " + RelationShipName + " } Not Found", OMErrorObjectType.ASSOCIATION,
                        "Association", ReadErrObj.ErrorType, ReadErrObj.ExceptionObject, null);
                    return ReturnErrorObj;
                }

                if (!String.IsNullOrEmpty(AssociationXML))
                {
                    OutputXMLDocument = new XmlDocument();
                    OutputXMLDocument.LoadXml(AssociationXML);
                    RootNode = OutputXMLDocument.DocumentElement;
                }
                return ReadErrObj;
            }
            catch (Exception ExceptionObj)
            {
                ReadErrObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                           ExceptionObj.Message, OMErrorObjectType.TABLE,
                           null, OMErrorType.EXCEPTION, ExceptionObj, null);
                return ReadErrObj;
            }
        }


        /// <summary>
        /// Helper function used to format the field value for the given operator
        /// </summary>
        /// <param name="FieldValue">Field Value</param>
        /// <param name="Operator">Value of the operator</param>
        /// <returns>Formatted value of the field, based on the operator value</returns>
        public override string GetStringValue(string FieldValue, string Operator)
        {
            StringBuilder NewFieldValue = new StringBuilder();

            FieldValue = ReplaceEscapeCharacters(FieldValue, Operator);

            switch (Operator.ToUpper())
            {
                //SINGLE QUOTE (') is added by Kiran in the following statement,
                // because it is required for execution of Stored Procedure.
                case "ENDSWITH":
                    NewFieldValue.AppendFormat("N''%{0}''", FieldValue);
                    break;
                case "STARTSWITH":
                    NewFieldValue.AppendFormat("N''{0}%''", FieldValue);
                    break;
                case "LIKE":
                case "CONTAINS":
                case "NOTLIKE":
                    NewFieldValue.AppendFormat("N''%{0}%''", FieldValue);
                    break;
                case "EQUALTO":
                    NewFieldValue.AppendFormat("N''{0}''", FieldValue);
                    break;
                case "IN":
                case "NOTIN":
                    NewFieldValue.AppendFormat("{0}", FieldValue);
                    break;
                case "GREATERTHAN":
                case "LESSTHAN":
                case "GREATERTHANEQUALTO":
                case "LESSTHANEQUALTO":
                case "NOTEQUALTO":
                case "INRANGE":
                    NewFieldValue.AppendFormat("N''{0}''", FieldValue);
                    break;
                case "IS":
                case "IS NOT":
                    NewFieldValue.AppendFormat("{0}", FieldValue);
                    break;
            }
            return NewFieldValue.ToString();
        }
        /// <summary>
        /// Helper function which prepares the clause based on the IsCaseSensitive value
        /// </summary>
        /// <param name="FieldName">Name of the field</param>
        /// <param name="SearchOperator">Operator value</param>
        /// <param name="FieldValue">Value of the field</param>
        /// <param name="IsCaseSensitive">IsCaseSensitive</param>
        /// <returns>Case dependent Clause</returns>
        public override StringBuilder GetCaseDependentClause(string FieldName, string SearchOperator, string FieldValue, string IsCaseSensitive)
        {
            StringBuilder FilterClause = new StringBuilder();
            //String DataBaseType = "";

            IsCaseSensitive = IsCaseSensitive.ToUpper();

            if (IsCaseSensitive.Equals("TRUE")) //Here the sequence of COLLATE is changed since <FIELD NAME> IS NULL COLLATE.. gives syntax error
                FilterClause.AppendFormat("{0} COLLATE SQL_Latin1_General_CP1_CS_AS {1} {2} ", FieldName, SearchOperator, FieldValue);
            else
                FilterClause.AppendFormat("{0} {1} {2} ", FieldName, SearchOperator, FieldValue);

            return FilterClause;
        }

        /// <summary>
        /// This function prepares a filter clause based on Date Macro.
        /// </summary>
        /// <param name="FieldName">FieldName on which filter is specified</param>
        /// <param name="InputFieldValue">Contains Date macros value. e.g : THIS_WEEK,THIS_YEAR</param>
        /// <param name="InputFilter">XML Node which has all information for Filter Node</param>
        /// <returns></returns>
        public StringBuilder PrepareDateMacroFilter(String FieldName, String InputFieldValue, XmlNode InputFilter)
        {
            String DateSearchOperator = null;
            String SearchOperator = null;
            StringBuilder FilterValueBuilder = null;


            DateTime FromDate = new DateTime();
            DateTime ToDate = new DateTime();
            DateTime TodaysDate = DateTime.Now;

            XmlDocument FilterXMLDoc = null;
            XmlNode FilterNode = null;
            XmlNode NavigationPath = null;

            int CurrentDay = 0;
            int Year = 0;
            int QuarterNum = 0;
            int StartMonth = 0;
            int EndDay = 0;
            int EndMonth = 0;


            FilterXMLDoc = new XmlDocument();
            FilterNode = FilterXMLDoc.CreateNode(XmlNodeType.Element, "FILTER", "");
            FilterValueBuilder = new StringBuilder();

            switch (InputFieldValue)
            {
                #region TODAY
                case "TODAY":
                    DateSearchOperator = "EQUALTO";
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region YESTERDAY
                case "YESTERDAY":
                    DateSearchOperator = "EQUALTO";
                    TodaysDate = DateTime.Now.AddDays(-1);
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>EQUALTO</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_WEEK
                case "THIS_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    ToDate = FromDate.AddDays(6);
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_WEEK_TO_DATE
                case "THIS_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_WEEK
                case "LAST_WEEK":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    ToDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = ToDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_WEEK_TO_DATE
                case "LAST_WEEK_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = (int)TodaysDate.DayOfWeek;
                    CurrentDay += 1;
                    FromDate = TodaysDate.AddDays(-CurrentDay);
                    FromDate = FromDate.AddDays(-6);

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_MONTH
                case "THIS_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    ToDate = FromDate.AddMonths(1);
                    ToDate = ToDate.AddDays(-1);

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_MONTH_TO_DATE
                case "THIS_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    CurrentDay = CurrentDay - 1;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_MONTH
                case "LAST_MONTH":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    ToDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = ToDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", ToDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", ToDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", ToDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_MONTH_TO_DATE
                case "LAST_MONTH_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    CurrentDay = TodaysDate.Day;
                    FromDate = TodaysDate.AddDays(-(CurrentDay));
                    FromDate = FromDate.AddMonths(-1);
                    FromDate = FromDate.AddDays(1);

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", FromDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", FromDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", FromDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_QUARTER
                case "THIS_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            break;
                        case "1":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            break;
                        case "2":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            break;
                        case "3":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_QUARTER_TO_DATE
                case "THIS_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 1;
                            break;
                        case "1":
                            StartMonth = 4;
                            break;
                        case "2":
                            StartMonth = 7;
                            break;
                        case "3":
                            StartMonth = 10;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_QUARTER
                case "LAST_QUARTER":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            EndDay = 31;
                            EndMonth = 12;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            EndDay = 31;
                            EndMonth = 3;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            EndDay = 30;
                            EndMonth = 6;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            EndDay = 30;
                            EndMonth = 9;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", EndDay.ToString());
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", EndMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_QUARTER_TO_DATE
                case "LAST_QUARTER_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    QuarterNum = (TodaysDate.Month - 1) / 3;
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");

                    switch (QuarterNum.ToString())
                    {
                        case "0":
                            StartMonth = 10;
                            Year = TodaysDate.Year - 1;
                            break;
                        case "1":
                            StartMonth = 1;
                            Year = TodaysDate.Year;
                            break;
                        case "2":
                            StartMonth = 4;
                            Year = TodaysDate.Year;
                            break;
                        case "3":
                            StartMonth = 7;
                            Year = TodaysDate.Year;
                            break;
                    }
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", StartMonth.ToString());
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_YEAR
                case "THIS_YEAR":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region THIS_YEAR_TO_DATE
                case "THIS_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_YEAR
                case "LAST_YEAR":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;

                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>31</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>12</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LAST_YEAR_TO_DATE
                case "LAST_YEAR_TO_DATE":
                    DateSearchOperator = "INRANGE";
                    Year = TodaysDate.Year;
                    Year = Year - 1;
                    FilterValueBuilder.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                    FilterValueBuilder.Append("<OPERATOR>INRANGE</OPERATOR>");
                    FilterValueBuilder.Append("<FIELDVALUE>");
                    FilterValueBuilder.Append("<VALUE>");
                    FilterValueBuilder.AppendFormat("<DAY>1</DAY>");
                    FilterValueBuilder.AppendFormat("<MONTH>1</MONTH>");
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", Year.ToString());
                    FilterValueBuilder.Append("<HOUR>00</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>00</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>00</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("<VALUE Order=\"2\">");
                    FilterValueBuilder.AppendFormat("<DAY>{0}</DAY>", TodaysDate.Day);
                    FilterValueBuilder.AppendFormat("<MONTH>{0}</MONTH>", TodaysDate.Month);
                    FilterValueBuilder.AppendFormat("<YEAR>{0}</YEAR>", TodaysDate.Year);
                    FilterValueBuilder.Append("<HOUR>23</HOUR>");
                    FilterValueBuilder.Append("<MINUTE>59</MINUTE>");
                    FilterValueBuilder.Append("<SECOND>59</SECOND>");
                    FilterValueBuilder.Append("</VALUE>");
                    FilterValueBuilder.Append("</FIELDVALUE>");
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;
                #endregion

                #region LESS_THAN_TODAY
                case "LESS_THAN_TODAY":
                    DateSearchOperator = "LESSTHAN";
                    TodaysDate = TodaysDate.AddDays(-1);
                    FilterValueBuilder.AppendFormat(@"<FIELDNAME>{0}</FIELDNAME>
                                                  <OPERATOR>{1}</OPERATOR>
                                                  <FIELDVALUE>
                                                    <VALUE>
                                                        <DAY>{2}</DAY>
                                                        <MONTH>{3}</MONTH>
                                                        <YEAR>{4}</YEAR>
                                                        <HOUR>23</HOUR>
                                                        <MINUTE>59</MINUTE>
                                                        <SECOND>59</SECOND>
                                                    </VALUE>
                                                  </FIELDVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;

                #endregion

                #region LESS_THAN_EQUALTO_TODAY
                case "LESS_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "LESSTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<FIELDNAME>{0}</FIELDNAME>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <FIELDVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </FIELDVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;

                #endregion

                #region GREATER_THAN_TODAY
                case "GREATER_THAN_TODAY":
                    DateSearchOperator = "GREATERTHAN";
                    FilterValueBuilder.AppendFormat(@"<FIELDNAME>{0}</FIELDNAME>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <FIELDVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>23</HOUR>
                                                            <MINUTE>59</MINUTE>
                                                            <SECOND>59</SECOND>
                                                        </VALUE>
                                                      </FIELDVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;

                #endregion

                #region GREATER_THAN_EQUALTO_TODAY
                case "GREATER_THAN_EQUALTO_TODAY":
                    DateSearchOperator = "GREATERTHANEQUALTO";
                    FilterValueBuilder.AppendFormat(@"<FIELDNAME>{0}</FIELDNAME>
                                                      <OPERATOR>{1}</OPERATOR>
                                                      <FIELDVALUE>
                                                        <VALUE>
                                                            <DAY>{2}</DAY>
                                                            <MONTH>{3}</MONTH>
                                                            <YEAR>{4}</YEAR>
                                                            <HOUR>00</HOUR>
                                                            <MINUTE>00</MINUTE>
                                                            <SECOND>00</SECOND>
                                                        </VALUE>
                                                      </FIELDVALUE>", FieldName, DateSearchOperator, TodaysDate.Day, TodaysDate.Month, TodaysDate.Year);
                    NavigationPath = null;
                    NavigationPath = InputFilter.SelectSingleNode("NAVIGATIONPATH");
                    if (NavigationPath != null)
                        FilterValueBuilder.AppendFormat("<NAVIGATIONPATH>{0}</NAVIGATIONPATH>", NavigationPath.InnerText);
                    break;

                #endregion
            }
            SearchOperator = NSearchCommon.GetSearchOperatorValue(DateSearchOperator);
            FilterNode.InnerXml = FilterValueBuilder.ToString();
            return (PrepareDateFilter(FieldName, SearchOperator, FilterNode));
        }

        //It returns comma seperated list of fields of passed table from ALOM_SCHEMA table.
        /// <summary>
        /// 
        /// </summary>
        /// <param name="TableName">TableName of which the fields are to be returned</param>
        /// <returns>Sorted List of Fields with its datatype</returns>
        public SortedList GetListOfFieldsFromSchema(String TableName)
        {
            OMError ReadErrorObj = null;
            XmlDocument OutputXMLDocument = null;
            XmlNodeList FieldNodes = null;
            String OutputXML = null;
            SortedList FieldList = null;

            try
            {
                

                FieldList = new SortedList();
                if((ReadErrorObj = AddClause("TABLENAME", OMFieldDataType.STRING,OMOperator.EQUALTO, TableName, OMConjunction.NOTHING)) != null)
                    return null;

                if ((ReadErrorObj = ReadAllRecord("SCHEMA", "ALOM_SCHEMA", null, ref OutputXML)) != null)
                    return null;

                if(!String.IsNullOrEmpty(OutputXML)) 
                {
                    OutputXMLDocument = new XmlDocument();
                    OutputXMLDocument.LoadXml(OutputXML);
                    FieldNodes = OutputXMLDocument.SelectNodes("SCHEMACOLLECTION/SCHEMA");

                    if (FieldNodes != null)
                    {
                        foreach (XmlNode Field in FieldNodes)
                        {
                            FieldList.Add(Field.SelectSingleNode("FIELDNAME").InnerText.Trim().ToUpper(), Field.SelectSingleNode("FIELDTYPE").InnerText.Trim().ToUpper());
                        }
                        return (FieldList);
                    }
                }
                return (null);
            }
            catch
            {
                return null;
            }
        }

        #region Processing for Simple Search

        /// <summary>
        /// Public method which prepares the query for a Search object for simple search and 
        /// returns the Target object list after executing that query.
        /// Query is prepared either from the Dummy query string by simply replacing the dummy field 
        /// name with actual search field name in the Search object or from the Post fix string 
        /// in case the field is a named list field.
        /// </summary>
        /// <param name="ObjectTypeNode">Root node of the XML containing Object schema information</param>
        /// <param name="TargetObjects">List of Target objects to be returned back</param>
        /// <returns>OMError Object, null incase no error is occured</returns>
        public override OMError SearchTargetObjectForQueryString(XmlNode ObjectTypeNode, String DummyQuery, String DummyField, ref string TEQuery)
        {
            XmlNodeList             SearchFieldList         = null;
            StringBuilder           QueryString             = null;
            ArrayList               ObjectIDList            = null;
            OMError                 ReturnErrorObj          = null;
            string                  SelectClause            = null;
            string                  WhereClause             = null;
            string                  ObjectType              = null;
            string                  ObjectTypeName          = null;

            try
            {
                ObjectType = ObjectTypeNode.SelectSingleNode("OBJECTTYPE").InnerText.ToString();
                ObjectTypeName = ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText.ToString();
                SearchFieldList = ObjectTypeNode.SelectNodes("FIELDCOLLECTION/FIELD[ISSEARCHFIELD='TRUE' and FIELDTYPE='STRING']");
                if (SearchFieldList.Count > 0)
                {
                    QueryString = new StringBuilder();
                    ObjectIDList = new ArrayList();
                    WhereClause = GetWhereClauseForObject(SearchFieldList, DummyQuery, DummyField);
                    if (!WhereClause.Equals(""))
                    {
                        SelectClause = " INSERT INTO @SSFILTEREDTABLE (SSOBJECTID) ";
                        SelectClause = SelectClause + GetSelectClauseForObject(ObjectTypeNode);
                        QueryString.AppendFormat("{0} where ( {1} )", SelectClause, WhereClause);
                        TEQuery = QueryString.ToString();
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Exception in Searching TargetObject For QueryString", OMErrorObjectType.TABLE, ObjectTypeName, OMErrorType.EXCEPTION,
                    ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// Public method which has to be derived in derived class. This method gets all the related facets and prepares a Simple Search Query 
        /// </summary>
        /// <param name="ObjectTypeNode">Facet Type Node</param>
        /// <param name="TargetObjectType">Object Type</param>
        /// <param name="TargetObjectTypeID">Object Type ID</param>
        /// <param name="RFQuery">returns a simple search query </param>
        /// <returns></returns>
        public override OMError SearchSubFacetsForQueryString(XmlNode ObjectTypeNode, string TargetObjectType, 
                                        ulong TargetObjectTypeID, String DummyQuery, String DummyField,
                                        ref string RFQuery)
        {
            XmlNodeList                     SearchFieldList                 = null;
            StringBuilder                   QueryString                     = null;
            ArrayList                       ObjectIDList                    = null;
            OMError                         ReturnErrorObj                  = null;
            string                          SelectClause                    = null;
            string                          WhereClause                     = null;
            string                          ObjectType                      = null;
            string                          ObjectTypeName                  = null;
            

            try
            {
                ObjectType          = ObjectTypeNode.SelectSingleNode("OBJECTTYPE").InnerText.ToString();
                ObjectTypeName      = ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText.ToString();
                SearchFieldList     = ObjectTypeNode.SelectNodes("FIELDCOLLECTION/FIELD[ISSEARCHFIELD='TRUE' and FIELDTYPE='STRING']");
                if (SearchFieldList.Count > 0)
                {
                    QueryString = new StringBuilder();
                    ObjectIDList = new ArrayList();
                    WhereClause = GetWhereClauseForObject(SearchFieldList, DummyQuery, DummyField);
                    
                    if (!WhereClause.Equals(""))
                    {
                        switch (TargetObjectType.ToUpper())
                        {
                            case "ENTITY":
                                SelectClause = GetSelectClauseFromAssociation(ObjectTypeNode, TargetObjectTypeID);
                                QueryString.AppendFormat("{0} and ( {1} )", SelectClause, WhereClause);
                                break;
                            case "FACET":
                                SelectClause = GetSelectClauseForObject(ObjectTypeNode);
                                QueryString.AppendFormat("Select CONTAINERID from ALOM_ASSEMBLY where CONTAINERTYPEID = {0} and CONTAINSID in ( {1} where {2} )", TargetObjectTypeID, SelectClause, WhereClause);
                                break;
                        }
                        RFQuery = QueryString.ToString();
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Exception in Searching TargetObject For QueryString", OMErrorObjectType.FACETTYPE, ObjectTypeName, OMErrorType.EXCEPTION,
                    ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }


        /// <summary>
        /// Public method which has to be derived in derived class. This method gets 
        /// all the related facets and prepares a Simple Search Query. Its an overloaded method
        /// which takes an additional parameter for simple search string and check whehter
        /// SQL Server Full Text is enabled. If yes then prepares the full text query.
        /// </summary>
        /// <param name="ObjectTypeNode"></param>
        /// <param name="TargetObjectType"></param>
        /// <param name="TargetObjectTypeID"></param>
        /// <param name="DummyQuery"></param>
        /// <param name="DummyField"></param>
        /// <param name="simpleSearchQueryString"></param>
        /// <param name="RFQuery"></param>
        /// <returns></returns>
        public override OMError SearchSubFacetsForQueryString(XmlNode ObjectTypeNode, string TargetObjectType, 
                                        ulong TargetObjectTypeID, String DummyQuery, String DummyField,
                                        String simpleSearchQueryString, ref string RFQuery)
        {
            XmlNodeList                     SearchFieldList                 = null;
            StringBuilder                   QueryString                     = null;
            ArrayList                       ObjectIDList                    = null;
            OMError                         ReturnErrorObj                  = null;
            string                          SelectClause                    = null;
            string                          WhereClause                     = null;
            string                          ObjectType                      = null;
            string                          ObjectTypeName                  = null;
            String                          IsFullTextEnabled               = null;

            try
            {
                ObjectType          = ObjectTypeNode.SelectSingleNode("OBJECTTYPE").InnerText.ToString();

                if(ObjectTypeNode.SelectSingleNode("FULLTEXTENABLED")!= null)
                    IsFullTextEnabled = ObjectTypeNode.SelectSingleNode("FULLTEXTENABLED").InnerText.ToString();

                ObjectTypeName      = ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText.ToString();
                SearchFieldList     = ObjectTypeNode.SelectNodes("FIELDCOLLECTION/FIELD[ISSEARCHFIELD='TRUE' and FIELDTYPE='STRING']");
               
                if (SearchFieldList.Count > 0)
                {
                    QueryString     = new StringBuilder();
                    ObjectIDList    = new ArrayList();

                    if (String.IsNullOrEmpty(IsFullTextEnabled) == true ||
                        IsFullTextEnabled.ToUpper() == "FALSE")
                        WhereClause = GetWhereClauseForObject(SearchFieldList, DummyQuery, DummyField);
                    else if (IsFullTextEnabled.ToUpper() == "TRUE")
                        WhereClause = getFreeTextQuery(SearchFieldList, simpleSearchQueryString);
                    else //Default
                        WhereClause = GetWhereClauseForObject(SearchFieldList, DummyQuery, DummyField);


                    if (!WhereClause.Equals(""))
                    {
                        switch (TargetObjectType.ToUpper())
                        {
                            case "ENTITY":
                                SelectClause = GetSelectClauseFromAssociation(ObjectTypeNode, TargetObjectTypeID);
                                QueryString.AppendFormat("{0} and ( {1} )", SelectClause, WhereClause);
                                break;
                            case "FACET":
                                SelectClause = GetSelectClauseForObject(ObjectTypeNode);
                                QueryString.AppendFormat("Select CONTAINERID from ALOM_ASSEMBLY where CONTAINERTYPEID = {0} and CONTAINSID in ( {1} where {2} )", TargetObjectTypeID, SelectClause, WhereClause);
                                break;
                        }
                        RFQuery = QueryString.ToString();
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Exception in Searching TargetObject For QueryString", OMErrorObjectType.FACETTYPE, ObjectTypeName, OMErrorType.EXCEPTION,
                    ExceptionObj, "");
                
                return ReturnErrorObj;
            }
        }


        /// <summary>
        /// Prepares the Full Text Search query.
        /// </summary>
        /// <param name="SearchFieldList"></param>
        /// <param name="searchString"></param>
        /// <returns></returns>
        private string getFreeTextQuery(XmlNodeList SearchFieldList, string searchString)
        {
            StringBuilder   WhereClause = null;
            String          Clause      = "";

            WhereClause = new StringBuilder();
            for (int Counter = 0; Counter < SearchFieldList.Count; Counter++)
            {
                XmlNode SearchFieldNode = SearchFieldList.Item(Counter); 
                String FieldName        = SearchFieldNode.SelectSingleNode("FIELDNAME").InnerText;
                Clause                 += FieldName + ",";
            }
            Clause          = Clause.Substring(0,Clause.Length-1);
            searchString    = prepareSearchString(searchString);
            WhereClause.AppendFormat("CONTAINS(({0}),N''{1}'')", Clause, searchString);
            return WhereClause.ToString();
           
        }

        /// <summary>
        /// The method prepares the full text search string for each word.
        /// </summary>
        /// <param name="searchString"></param>
        /// <returns></returns>
        private string prepareSearchString(string searchString)
        {

            String query = "";
            String DummyQuery = "\"*$STR$*\"";

            if (FTSLogicalOperator == null)
                FTSLogicalOperator = "AND";

            bool StartedAssembling;
            StartedAssembling = false;
            //Split the search string and put the quoted strings together
            String[] tokenString = searchString.Split(' ');
            for (int splitStringCounter = 0; splitStringCounter < tokenString.Length; splitStringCounter++)
            {
                if (tokenString[splitStringCounter].Length == 0)
                    continue;

                if (tokenString[splitStringCounter].StartsWith("\""))
                {
                    StartedAssembling = true;
                }

                int Location = -1;
                if (StartedAssembling)
                {
                    for (int m = splitStringCounter + 1; m < tokenString.Length; m++)
                    {
                        if (tokenString[m].EndsWith("\""))
                        {
                            Location = m;
                            break;
                        }
                    }

                    for (int m = splitStringCounter + 1; m <= Location; m++)
                    {
                        tokenString[splitStringCounter] = tokenString[splitStringCounter] + " " + tokenString[m];
                        tokenString[m] = "";
                    }
                    StartedAssembling = false;
                }
            }

            foreach (String token in tokenString)
            {
                if (!String.IsNullOrEmpty(token))
                {
                    String queryString = ReplaceEscapeCharacters(token.Trim(), "CONTAINS");

                    query += " (" + DummyQuery.Replace("$STR$", queryString.Replace("\"", "")) + ") " + FTSLogicalOperator;
                }
            }
            return query.Substring(0, query.Length - FTSLogicalOperator.Length);
        }

        /// <summary>
        /// Helper Function which prepares the Where Clasue for a Search Object
        /// </summary>
        /// <param name="SearchFieldList">List of search fields</param>
        /// <returns>Returns Where Clasue</returns>
        string GetWhereClauseForObject(XmlNodeList SearchFieldList, String DummyQuery, String DummyField)
        {
            StringBuilder   WhereClause = null;
            string          Clause      = null;

            WhereClause = new StringBuilder();
            for (int Counter = 0; Counter < SearchFieldList.Count; Counter++)
            {
                Clause = ProcessWhereClause(SearchFieldList.Item(Counter), DummyQuery, DummyField);
                if (!Clause.Equals(""))
                {
                    if (WhereClause.Length > 0)
                        WhereClause.Append(" OR ");
                    WhereClause.Append(Clause);
                }
            }
            return WhereClause.ToString();
        }

        /// <summary>
        /// Helper Function which prepares the Where clause for a given field and and dummy where clause
        /// by replacing the DummyField name with the actual field name.
        /// Handles the named heirarchy fields in specific manner.
        /// PostFix string is changed to prepare the query for the named list values, since 
        /// key word in the PostFix string is treated as named list value's display text.
        /// there for keyword is replaced with displaytext id.
        /// </summary>
        /// <param name="SearchFieldNode">XML containing Field information</param>
        /// <returns>returns the where clause</returns>
        string ProcessWhereClause(XmlNode SearchFieldNode, String DummyQuery, String DummyField)
        {
            //StringCollection        PostFixForNamedListQuery = null;
            string          IsNamedList             = null;
            string          FieldName               = null;
            //	StringCollection ValueQuery	= null;

            IsNamedList = SearchFieldNode.SelectSingleNode("ISNAMEDHIERARCHY").InnerText;
            FieldName = SearchFieldNode.SelectSingleNode("FIELDNAME").InnerText;
            if (FieldName != "")
            {
                //PostFixForNamedListQuery = new StringCollection();

                //if (!IsNamedList.Equals("TRUE"))
                return DummyQuery.Replace(DummyField, FieldName);
                /*	else
                    {
                        //int NamedListID = GetNamedListID(FieldName);
                        foreach (string word in this.m_PostFixString)
                        {
                            if (!IsOperator(word))
                            {
                                ValueQuery = new StringCollection();
                                //GetNamedListValueID(1, word, ref ValueQuery);
                                if (ValueQuery.Count==0)
                                    return "";
                                foreach (string str in ValueQuery)
                                    PostFixForNamedListQuery.Add(str);
                            }
                            else
                                PostFixForNamedListQuery.Add(word);
                        }
                        return (PrepareDummyQuery(PostFixForNamedListQuery, FieldName));
                    }
                 */
            }
            return "";
        }

        /// <summary>
        /// Prepares the Select clause for an Object
        /// </summary>
        /// <param name="EntityTypeNode">Root Node of the XML containing Entity Information</param>
        /// <returns>returns Select Clause</returns>
        public string GetSelectClauseForObject(XmlNode EntityTypeNode)
        {
            StringBuilder       SelectClause        = null;
            string              TableName           = null;
            string              PrimaryKey          = null;

            TableName = EntityTypeNode.SelectSingleNode("TABLENAME").InnerText;
            PrimaryKey = EntityTypeNode.SelectSingleNode("PRIMARYKEY").InnerText;
            SelectClause = new StringBuilder();

            SelectClause.AppendFormat("SELECT {0} FROM {1}", PrimaryKey, TableName);
            return SelectClause.ToString();
        }

        /// <summary>
        /// Prepares Select Clause for the Facet Objects
        /// </summary>
        /// <param name="FacetTypeNode">Root Node of the XML containing Facet Object Information</param>
        /// <param name="TargetEntityTypeID">target Entity TypeID. Required to fetch the records from 
        /// Entity, Facet Association table</param>
        /// <returns>Returns Select Clause</returns>
        string GetSelectClauseFromAssociation(XmlNode FacetTypeNode, ulong TargetEntityTypeID)
        {
            StringBuilder       SelectClause        = null;
            string              TableName           = null;
            string              PrimaryKey          = null;
            string              TypeID              = null;

            TableName = FacetTypeNode.SelectSingleNode("TABLENAME").InnerText;
            PrimaryKey = FacetTypeNode.SelectSingleNode("PRIMARYKEY").InnerText;
            TypeID = FacetTypeNode.SelectSingleNode("TYPEID").InnerText;
            SelectClause = new StringBuilder();
            SelectClause.AppendFormat(" INSERT INTO @SSFILTEREDTABLE (SSOBJECTID) Select ALOM_ASSOENTITYFACET.ENTITYID from ALOM_ASSOENTITYFACET inner join {0} on ALOM_ASSOENTITYFACET.FACETID = {0}.{1} where ALOM_ASSOENTITYFACET.ENTITYTYPEID = {2} ", TableName, PrimaryKey, TargetEntityTypeID);

            return SelectClause.ToString();
        }

        /// <summary>
        /// Prepares Select Clause for the Facet Objects. This method gets objectids of facet types associated with Target Entity.
        /// </summary>
        /// <param name="FacetTypeNode">Root Node of the XML containing Facet Object Information</param>
        /// <param name="TargetEntityTypeID">target Entity TypeID. Required to fetch the records from 
        /// Entity, Facet Association table</param>
        /// <returns>Returns Select Clause</returns>
        string GetSelectClauseForAssociatedFacetInFile(XmlNode FacetTypeNode, ulong TargetEntityTypeID)
        {
            StringBuilder SelectClause = null;
            string TableName = null;
            string PrimaryKey = null;
            string TypeID = null;

            TableName = FacetTypeNode.SelectSingleNode("TABLENAME").InnerText;
            PrimaryKey = FacetTypeNode.SelectSingleNode("PRIMARYKEY").InnerText;
            TypeID = FacetTypeNode.SelectSingleNode("TYPEID").InnerText;
            SelectClause = new StringBuilder();
            SelectClause.AppendFormat(" INSERT INTO @SSFILTEREDTABLE (SSOBJECTID) Select ALOM_ASSOENTITYFACET.ENTITYID from ALOM_ASSOENTITYFACET inner join {0} on ALOM_ASSOENTITYFACET.FACETID = {0}.{1} AND ALOM_ASSOENTITYFACET.ENTITYTYPEID = {2} ", TableName, PrimaryKey, TargetEntityTypeID);

            return SelectClause.ToString();
        }

        /// <summary>
        /// Prepares Select Clause for the Facet Objects
        /// </summary>
        /// <param name="FacetTypeNode">Root Node of the XML containing Facet Object Information</param>
        /// <param name="TargetEntityTypeID">target Entity TypeID. Required to fetch the records from 
        /// Entity, Facet Association table</param>
        /// <returns>Returns Select Clause</returns>
        string GetSelectClauseFromAssembly(XmlNode FacetTypeNode, int TargetEntityTypeID)
        {
            StringBuilder       SelectClause        = null;
            string              TableName           = null;
            string              PrimaryKey          = null;

            TableName = FacetTypeNode.SelectSingleNode("TABLENAME").InnerText;
            PrimaryKey = FacetTypeNode.SelectSingleNode("PRIMARYKEY").InnerText;
            SelectClause = new StringBuilder();

            SelectClause.AppendFormat("Select ALOM_ASSEMBLY.CONTAINERID from ALOM_ASSEMBLY inner join {0} on ALOM_ASSOENTITYFACET.FACETID = {0}.{1} where ALOM_ASSOENTITYFACET.ENTITYTYPEID = {2} ", TableName, PrimaryKey, TargetEntityTypeID);
            return SelectClause.ToString();
        }

        /// <summary>
        /// Prepares Document Search Query for Entity Objects
        /// </summary>
        /// <param name="ObjectTypeNode">Root Node of the XML containing Entity Object Information</param>
        /// <param name="SearchString">string to be searched</param>
        /// <param name="RelationshipName">Entire set of RelationshipNames separated by $ALSEP$</param>
        /// <param name="TEQuery">Resultant Query</param>
        /// <returns>Returns OMError if an error occurs</returns>
        public override OMError SearchTargetObjectForQueryStringInFile(XmlNode ObjectTypeNode, String SearchString, String RelationshipName, ref string TEQuery)
        {           
            StringBuilder           QueryString             = null;
            ArrayList               ObjectIDList            = null;
            OMError                 ReturnErrorObj          = null;
            string                  SelectClause            = null;            
            string                  TableName               = null;
            string                  PrimaryKey              = null;
            string                  ObjectTypeName          = null;
            string                  FileTextQuery           = null;
            string[]                RelationshipArr         = null;
            bool                    IsFileFieldPresent      = false;
            XmlNode                 FileField               = null;
            string                  FullTextSearchString    = null;

            try
            {
                #region Remove Noise words from search string

                if (!String.IsNullOrEmpty(SearchString))
                {
                    SearchString = RemoveNoiseWords(SearchString);
                }

                //Return null if search string is empty (especially when all word are noise words)
                if (String.IsNullOrEmpty(SearchString))
                    return null;

                #endregion

                TableName = ObjectTypeNode.SelectSingleNode("TABLENAME").InnerText.ToString();
                PrimaryKey = ObjectTypeNode.SelectSingleNode("PRIMARYKEY").InnerText.ToString();
                ObjectTypeName = ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText.ToString();
                FileField = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDTYPE = 'FILE']");

                //No support for binder fields as yet.
                //if (FileField == null)
                //    FileField = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDTYPE = 'BINDER']");
                if (FileField != null)
                    IsFileFieldPresent = true;
                
                    QueryString = new StringBuilder();
                    ObjectIDList = new ArrayList();
                    if (RelationshipName != null)
                    {
                        RelationshipArr = RelationshipName.Split(new string[] { "$ALSEP$" }, StringSplitOptions.None);
                    }
                    else
                    {
                        RelationshipArr = new string[1];
                        RelationshipArr[0] = "";
                    }
                    
                    //FileText table join need to be created to get the objectID corresponding to the resulting file 
                    //and then obtain the entityID matching this file.
                    foreach (string Relation in RelationshipArr)
                    {
                        if (!(String.IsNullOrEmpty(Relation) && (IsFileFieldPresent == false)))
                        {
                            FileTextQuery = GetFileTextQuery(Relation, TableName, PrimaryKey);

                            if (!String.IsNullOrEmpty(FileTextQuery))
                            {
                                SelectClause = " INSERT INTO @SSFILTEREDTABLE (SSOBJECTID) ";
                                SelectClause = SelectClause + GetSelectClauseForObject(ObjectTypeNode);
                                QueryString.AppendFormat("{0} {1}", SelectClause, FileTextQuery);                                
                                FullTextSearchString = prepareSearchString(SearchString);
                                QueryString.AppendFormat(" AND CONTAINS ((FILETEXT),N''{0}'')", FullTextSearchString);
                            }
                        }
                    }

                    TEQuery = QueryString.ToString();

                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Exception in Searching TargetObject For QueryString", OMErrorObjectType.TABLE, ObjectTypeName, OMErrorType.EXCEPTION,
                    ExceptionObj, "");

                return ReturnErrorObj;
            }
        }

        /// <summary>
        /// Prepares Join Clauses for the Entity Objects
        /// </summary>
        /// <param name="RelationshipName">Name of Relationship</param>
        /// <param name="TableName">TableName of TargetEntity</param>
        /// <param name="PrimaryKey">PrimaryKey of TargetEntity</param>
        /// <returns>Returns File Text Query</returns>
        string GetFileTextQuery(string RelationshipName, string TableName, string PrimaryKey)
        {
            OMError ReadErrObj = null;
            XmlNode RootNode = null;
            StringBuilder query = null;
            XmlNode ContainerTypeID = null;
            XmlNode ContainsTypeID = null;
            string ContainerTypeXML = null;
            string ContainsTypeXML = null;
            XmlDocument ContainerInfo = null;
            XmlDocument ContainsInfo = null;
            string ContainerTableName = null;
            string ContainsTableName = null;
            string ContainerPrimaryKey = null;
            string ContainsPrimaryKey = null;
            string ContainsType = null;

            query = new StringBuilder();
            try
            {
                if (!String.IsNullOrEmpty(RelationshipName))
                {
                    //Get ContainerTypeID and ContainsTypeID from the RelationshipName
                    ReadByRelationShipNameEx(RelationshipName, ref RootNode);
                    ContainerTypeID = RootNode.SelectSingleNode("CONTAINERTYPEID");
                    ContainsTypeID = RootNode.SelectSingleNode("CONTAINSTYPEID");
                    ContainsType = RootNode.SelectSingleNode("CONTAINSTYPE").InnerText.ToString();
                    
                    //Get TableName and PrimaryKey for the ContainerType
                    ReadErrObj = AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                        OMOperator.EQUALTO, ContainerTypeID.InnerText.ToString(),
                                                        OMConjunction.NOTHING);

                    if (ReadErrObj != null)
                        return null;

                    ReadErrObj = ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                    ref ContainerTypeXML);                    

                    if (ReadErrObj != null)
                        return null;

                    if (!String.IsNullOrEmpty(ContainerTypeXML))
                    {
                        ContainerInfo = new XmlDocument();
                        ContainerInfo.LoadXml(ContainerTypeXML);
                        ContainerTableName = ContainerInfo.SelectSingleNode("ENTITYTYPE/TABLENAME").InnerText.ToString();
                        ContainerPrimaryKey = ContainerInfo.SelectSingleNode("ENTITYTYPE/PRIMARYKEY").InnerText.ToString();
                    }

                    //Get TableName and PrimaryKey for the ContainsType
                    if (ContainsType.ToUpper().Equals("ENTITY"))
                    {
                        ReadErrObj = AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER,
                                                            OMOperator.EQUALTO, ContainsTypeID.InnerText.ToString(),
                                                            OMConjunction.NOTHING);

                        if (ReadErrObj != null)
                            return null;

                        ReadErrObj = ReadRecord("ENTITYTYPE", "ALOM_ENTITYTYPE", null,
                                                                        ref ContainsTypeXML);

                        if (ReadErrObj != null)
                            return null;

                        if (!String.IsNullOrEmpty(ContainsTypeXML))
                        {
                            ContainsInfo = new XmlDocument();
                            ContainsInfo.LoadXml(ContainsTypeXML);
                            ContainsTableName = ContainsInfo.SelectSingleNode("ENTITYTYPE/TABLENAME").InnerText.ToString();
                            ContainsPrimaryKey = ContainsInfo.SelectSingleNode("ENTITYTYPE/PRIMARYKEY").InnerText.ToString();
                        }

                        //Introduce a join between ALOM_ASSEMBLY and the Container
                        if ((ContainerTableName != null) && (ContainerPrimaryKey != null))
                        {
                            query.AppendFormat(" INNER JOIN ALOM_ASSEMBLY ON {0}.{1} = ALOM_ASSEMBLY.CONTAINERID ", ContainerTableName, ContainerPrimaryKey);
                        }
                        else
                            return null;

                        //Introduce a join between Contains and the above join
                        if ((ContainsTableName != null) && (ContainsPrimaryKey != null))
                        {
                            query.AppendFormat(" INNER JOIN {0} ON ALOM_ASSEMBLY.CONTAINSID = {0}.{1} ", ContainsTableName, ContainsPrimaryKey);
                        }
                        else
                            return null;


                    }
                    else if (ContainsType.ToUpper().Equals("FACET"))
                    {

                        ReadErrObj = AddClause("FACETTYPEID", OMFieldDataType.INTEGER,
                                                            OMOperator.EQUALTO, ContainsTypeID.InnerText.ToString(),
                                                            OMConjunction.NOTHING);

                        if (ReadErrObj != null)
                            return null;

                        ReadErrObj = ReadRecord("FACETTYPE", "ALOM_FACETTYPE", null,
                                                                        ref ContainsTypeXML);

                        if (ReadErrObj != null)
                            return null;

                        if (!String.IsNullOrEmpty(ContainsTypeXML))
                        {
                            ContainsInfo = new XmlDocument();
                            ContainsInfo.LoadXml(ContainsTypeXML);
                            ContainsTableName = ContainsInfo.SelectSingleNode("FACETTYPE/TABLENAME").InnerText.ToString();
                            ContainsPrimaryKey = ContainsInfo.SelectSingleNode("FACETTYPE/PRIMARYKEY").InnerText.ToString();
                        }

                        //Introduce a join between ALOM_ASSOENTITYFACET and the Container
                        if ((ContainerTableName != null) && (ContainerPrimaryKey != null))
                        {
                            query.AppendFormat(" INNER JOIN ALOM_ASSOENTITYFACET ON {0}.{1} = ALOM_ASSOENTITYFACET.ENTITYID ", ContainerTableName, ContainerPrimaryKey);
                        }
                        else
                            return null;

                        //Introduce a join between above and the Contains
                        if ((ContainsTableName != null) && (ContainsPrimaryKey != null))
                        {
                            query.AppendFormat(" INNER JOIN {0} ON ALOM_ASSOENTITYFACET.FACETID = {0}.{1} ", ContainsTableName, ContainsPrimaryKey);
                        }
                        else
                            return null;
                    }
 
                    
                    //Finally a join with the ALFCT_SEARCHABLE_FILE_TEXT table
                    query.AppendFormat("INNER JOIN ALFCT_SEARCHABLE_FILE_TEXT ON {0}.{1} = ALFCT_SEARCHABLE_FILE_TEXT.OBJECTID", ContainsTableName, ContainsPrimaryKey);

                }
                else
                {
                    query.AppendFormat("INNER JOIN ALFCT_SEARCHABLE_FILE_TEXT ON {0}.{1} = ALFCT_SEARCHABLE_FILE_TEXT.OBJECTID", TableName, PrimaryKey); 
                }
                return query.ToString();
            }
            catch
            {
                return null;
            }
        }

        public override OMError SearchSubFacetsForQueryStringInFile(XmlNode ObjectTypeNode,
                                        ulong TargetObjectTypeID, String simpleSearchQueryString, ref string RFQuery)
        {            
            StringBuilder QueryString = null;
            ArrayList ObjectIDList = null;
            OMError ReturnErrorObj = null;
            string SelectClause = null;            
            string ObjectType = null;
            string ObjectTypeName = null;            
            string FacetFileQuery = null;
            string ObjectTableName = null;
            string ObjectPrimaryKey = null;
            bool IsFileFieldPresent = false;
            XmlNode FileField = null;
            string FullTextSearchString = null;

            try
            {
                #region Remove Noise words from search string

                if (!String.IsNullOrEmpty(simpleSearchQueryString))
                {
                    simpleSearchQueryString = RemoveNoiseWords(simpleSearchQueryString);
                }

                //Return null if search string is empty (especially when all word are noise words)
                if (String.IsNullOrEmpty(simpleSearchQueryString))
                    return null;

                #endregion

                ObjectType = ObjectTypeNode.SelectSingleNode("OBJECTTYPE").InnerText.ToString();
                ObjectTableName = ObjectTypeNode.SelectSingleNode("TABLENAME").InnerText.ToString();
                ObjectPrimaryKey = ObjectTypeNode.SelectSingleNode("PRIMARYKEY").InnerText.ToString();
                ObjectTypeName = ObjectTypeNode.SelectSingleNode("TYPENAME").InnerText.ToString();
                FileField = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDTYPE = 'FILE']");

                //No support for binder as yet.
                //if (FileField == null)
                //    FileField = ObjectTypeNode.SelectSingleNode("FIELDCOLLECTION/FIELD[FIELDTYPE = 'BINDER']");
                if (FileField != null)
                    IsFileFieldPresent = true;

                if (IsFileFieldPresent)
                {
                    QueryString = new StringBuilder();
                    ObjectIDList = new ArrayList();                    

                    FacetFileQuery = GetFacetFileQuery(ObjectTableName, ObjectPrimaryKey);

                    if (!String.IsNullOrEmpty(FacetFileQuery))
                    {
                        SelectClause = GetSelectClauseForAssociatedFacetInFile(ObjectTypeNode, TargetObjectTypeID);
                        QueryString.AppendFormat("{0} {1} ", SelectClause, FacetFileQuery);                        
                        FullTextSearchString = prepareSearchString(simpleSearchQueryString);
                        QueryString.AppendFormat(" AND CONTAINS ((FILETEXT),N''{0}'')", FullTextSearchString);
                        RFQuery = QueryString.ToString();
                    }
                }
               
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrorObj = new OMError(OMErrorCodes.GETDATA_FAILED,
                    "Exception in Searching TargetObject For QueryString", OMErrorObjectType.FACETTYPE, ObjectTypeName, OMErrorType.EXCEPTION,
                    ExceptionObj, "");

                return ReturnErrorObj;
            }
        }

        string GetFacetFileQuery(string ObjectTableName, string ObjectPrimaryKey)
        {
            StringBuilder query = null;
            try
            {
                query = new StringBuilder();
                if ((!String.IsNullOrEmpty(ObjectTableName)) && (!String.IsNullOrEmpty(ObjectPrimaryKey)))
                    query.AppendFormat(" INNER JOIN ALFCT_SEARCHABLE_FILE_TEXT ON {0}.{1} = ALFCT_SEARCHABLE_FILE_TEXT.OBJECTID ", ObjectTableName, ObjectPrimaryKey);
                else
                    return null;

                return query.ToString();
            }
            catch
            {
                return null; 
            }
        }

        string RemoveNoiseWords(string SearchString)
        {
            string[] arrNoiseWord = null;
            string strNoiseWords = " 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 0 | $ | ! | @ | # | $ | % | ^ | & | * | ( | ) | - | _ | + | = | [ | ] | { | } | about | after | all | also | an | and | another | any | are | as | at | be | because | been | before | being | between | both | but | by | came | can | come | could | did | do | does | each | else | for | from | get | got | has | had | he | have | her | here | him | himself | his | how | if | in | into | is | it | its | just | like | make | many | me | might | more | most | much | must | my | never | now | of | on | only | or | other | our | out | over | re | said | same | see | should | since | so | some | still | such | take | than | that | the | their | them | then | there | these | they | this | those | through | to | too | under | up | use | very | want | was | way | we | well | were | what | when | where | which | while | who | will | with | would | you | your | a | b | c | d | e | f | g | h | i | j | k | l | m | n | o | p | q | r | s | t | u | v | w | x | y | z ";
            string NoiseWordList = null;
            
            string TenantXMLPath = ALCommon.GetTenantXMLPath(this.TenantName);
            XmlDocument TenantXML = new XmlDocument();
            TenantXML.Load(TenantXMLPath);

            SearchString = SearchString.ToUpper();

            //For full text queries single and double quotes need to be removed 
            //from query string as this is a base imlpementation of the document 
            //search feature.
            //SearchString = SearchString.Replace("\""," ");
            SearchString = SearchString.Replace("'", " ");

            if(TenantXML.SelectSingleNode("CONFIGURATION/FULLTEXTSEARCH/NOISEWORDS") != null)
                NoiseWordList = TenantXML.SelectSingleNode("CONFIGURATION/FULLTEXTSEARCH/NOISEWORDS").InnerText;

            if (NoiseWordList != null)
                strNoiseWords = NoiseWordList;

            //Blank spaces are appended to the begining and end to ensure that if first/last word 
            //or character is a noise word it gets replaced by a blank space 
            SearchString = " " + SearchString + " ";
            arrNoiseWord = strNoiseWords.Split('|');
            try
            {
                foreach (string NoiseWord in arrNoiseWord)
                {
                    SearchString = SearchString.Replace(NoiseWord.ToUpper(), " ");
                }
                return SearchString.Trim();
            }
            catch
            {
                return null;
            }
        }
        #endregion

        #endregion

        private void ParameterizeDynamicSQL(NSearchCommon NSearchCommonObj, StringBuilder DynamicSQL, ref StringBuilder OutPutSQL)
        {
            StringBuilder strSQLBatch = new StringBuilder();
            StringBuilder strParameterDeclaration = new StringBuilder();
            StringBuilder strParameterValues = new StringBuilder();
            StringBuilder strParameterList = new StringBuilder();
            int iVariableCounter = 0;
            String sVariableName = "@Var";
            OutPutSQL.Append("DECLARE @SQLString nvarchar(MAX); DECLARE @ParmDefinition nvarchar(MAX);");
            OutPutSQL.AppendFormat(System.Environment.NewLine);
            String sNewBatch = DynamicSQL.ToString();

            foreach (HierarchyObject HierarchyObj in m_NSearchCommonObj.HierarchyObjectList)
            {
                foreach (FilterCollection filter in HierarchyObj.FilterCollectionList)
                {
                    for (int i = 0; i < filter.m_ParameterizedVariables.Count; i++)
                    {
                        if (DynamicSQL.ToString().Contains(" " + filter.m_ParameterizedVariables[i] + " "))
                        {
                            sVariableName = "@Var" + iVariableCounter.ToString();
                            OutPutSQL.AppendFormat("DECLARE {0} {1};", sVariableName, filter.m_FieldDataType);
                            OutPutSQL.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterDeclaration.ToString()))
                                strParameterDeclaration.AppendFormat("{0} {1}", sVariableName, filter.m_FieldDataType);
                            else
                                strParameterDeclaration.AppendFormat(" ,{0} {1}", sVariableName, filter.m_FieldDataType);

                            strParameterValues.AppendFormat("SET {0} = {1};", sVariableName, filter.m_FieldValueParameterized[i].Replace(@"''", "'"));
                            strParameterValues.AppendFormat(System.Environment.NewLine);

                            if (string.IsNullOrEmpty(strParameterList.ToString()))
                            {
                                strParameterList.AppendFormat("{0}", sVariableName);
                            }
                            else
                            {
                                strParameterList.AppendFormat(",{0}", sVariableName);
                            }
                            iVariableCounter++;
                            if (string.IsNullOrEmpty(sNewBatch))
                                sNewBatch = sNewBatch.Replace(" " + filter.m_ParameterizedVariables[i] + " ", " " + sVariableName + " ");
                            else
                                sNewBatch = sNewBatch.Replace(" " + filter.m_ParameterizedVariables[i] + " ", " " + sVariableName + " ");
                        }

                    }
                }
            }
            OutPutSQL.AppendFormat("SET @SQLString = N'{0}';", sNewBatch);
            OutPutSQL.AppendFormat(System.Environment.NewLine);
            OutPutSQL.AppendFormat("SET @ParmDefinition = N'");
            OutPutSQL.AppendFormat(strParameterDeclaration.ToString());
            OutPutSQL.AppendFormat("'");
            OutPutSQL.AppendFormat(System.Environment.NewLine);

            OutPutSQL.Append(strParameterValues);

            if (strParameterList.Length > 0)
                OutPutSQL.AppendFormat("EXECUTE sp_executesql @SQLString, @ParmDefinition, {0}", strParameterList);
            else
                OutPutSQL.AppendFormat("EXECUTE sp_executesql @SQLString, @ParmDefinition");
        }

    }
}
