using System;
using System.Text;
using System.Transactions;
using Assetlink;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using Assetlink.DataAccessLayer;
using System.Collections.Specialized;
using System.Reflection;
using System.Threading;
using System.Collections;


namespace Assetlink.FileAccessLayer
{
    /// <summary>
    /// 
    /// </summary>

    /* Summary of changes : 
     * Date : 8 July 09
     * Changed by : Mr. Sagar Dasalkar
     * Description: All operations from LogDMObj except with those related with ALFS_DURABLELOG table have been moved to FileDMObj.
     * This is required because LogDMObj is intended only for those operations dealing with ALFS_DURABLELOG table.
     * Changes for Bug Fix: 6702, 6526, 6287
     * 
     * Date : 11 September 09
     * Changed by : Mr. Sagar Dasalkar
     * Description: Support added for Move File and Copy File.
     * 
     * Date : 6 November 09 * Changed by : Mr. Sagar Dasalkar
     * Description: Support added for document search for Move File and Copy File 
     * */
    public class FileSystemManager : IEnlistmentNotification
    {
        private FileTransferDelegate.OnFileTransfer m_delFileTransfer = null;

        /// <summary>
        /// Delegate used for asyncronous file Transfer
        /// </summary>
        public FileTransferDelegate.OnFileTransfer FileTransfer
        {
            set
            {
                m_delFileTransfer = value;
            }
        }

        static string Lock = "";
        DataManager FILEDMObj = null;
        /// <summary>
        /// Log Data Manager should be used only for those operations using ALFS_DURABLELOG table.
        /// </summary>
        DataManager LOGDMObj = null;
        ulong TransactionID = 0;
        string MountPoint = null;
        String UserProvidedMountPoint = null;
        string FileSystemType = null;
        ulong MaxFileCount = 0;
        ulong UserID = 0;
        String OutBox = null;
        Guid Gid = Guid.NewGuid();
        String m_TenantName = null;
        bool m_CopyFiles = false;    //Specifies whether a copy of existing file is to created
        bool m_MoveFiles = false;    //Specifies whether a file is to be moved
        String m_UserMountPointForFileSystem = null;     //Stores MountPoint attribute value specified at FILESYSTEM node
        String SHADOW_FOLDER = "SHADOW_FOLDER";
        String SYSYTEM_MOUNT_POINT = "SYSYTEM_MOUNT_POINT";

        /// <summary>
        /// Operations performed between OM Mount Point and Media Server Mount Point.
        /// </summary>
        private enum MediaServerOperationType : int
        {
            PUSHFILETOMEDIASERVER = 0,
            PULLFILEFROMMEDIASERVER = 1
        }

        /// <summary>
        /// File operations.
        /// </summary>
        private enum FileOperations : int
        {
            COPY = 0,
            MOVE = 1,
        }

        /// <summary>
        /// Manages files on user-given mount point (provided as MountPoint attribute for FILESYSTEM node)
        /// </summary>
        public String UserMountPointForFileSystem
        {
            get { return m_UserMountPointForFileSystem; }
            set { m_UserMountPointForFileSystem = value; }
        }

        /// <summary>
        /// Specifies whether a copy of existing file is to be created for Create File operation for an instance.
        /// </summary>
        public bool CopyFiles
        {
            get { return m_CopyFiles; }
            set { m_CopyFiles = value; }
        }

        /// <summary>
        /// Specifies whether existing files can be moved to another location within mount point.
        /// </summary>
        public bool MoveFiles
        {
            get { return m_MoveFiles; }
            set { m_MoveFiles = value; }
        }

        public ulong TranscationId
        {
            get { return TransactionID; }
            set { TransactionID = value; }
        }

        public DataManager LogDataManager
        {
            get { return LOGDMObj; }
            set { LOGDMObj = value; }
        }

        readonly FileSystemInterface LocalFileSystemInterfaceObj = null;
        readonly FileSystemInterface RemoteFileSystemInterfaceObj = null;
        readonly String DefaultMediaServerUrl = null;
        /// <summary>
        /// COnstructor for FileSystemManager
        /// </summary>
        /// <param name="UserID">Logged-In UserID</param>
        /// <param name="TenantName">Tenant Name</param>
        /// <param name="EnlistInTransaction">Specifies whether transaction is TRUE/FALSE</param>
        /// <param name="DMObj">DataManager Object</param>
        public FileSystemManager(ulong UserID, string TenantName, bool EnlistInTransaction, DataManager DMObj)
        {
            FILEDMObj = DMObj;
            m_TenantName = TenantName;
            LOGDMObj = new DataManager(TenantName);
            //LOGDMObj.FileSystemManager = this;
            UniqueID.Read(ref TransactionID);
            this.UserID = UserID;
            Transaction currentTx = Transaction.Current;
            InitializeConfigInfo(out LocalFileSystemInterfaceObj, out RemoteFileSystemInterfaceObj, out DefaultMediaServerUrl);
            //if (EnlistInTransaction)
            //{
            //    if (currentTx != null)
            //    {
            //        currentTx.EnlistDurable(Gid, this, EnlistmentOptions.None);
            //    }
            //}
        }

        /// <summary>
        /// All the changes are committed to Database
        /// </summary>
        /// <param name="enlistment"></param>
        public void Commit(Enlistment enlistment)
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = CommitOperations(LogNodes);
                }
                enlistment.Done();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// This function commits all the FileSystem Operations to Database 
        /// </summary>
        /// <returns>OMError Object</returns>

        public OMError CommitFileSystemOperations()
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ErrorObj = ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null && ErrorObj == null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = CommitOperations(LogNodes);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        public OMError RollbackFileSystemOperations()
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ErrorObj = ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null && ErrorObj == null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    if(LogNodes != null)
                        ErrorObj = RollBackOperations(LogNodes);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        public OMError HandleInDoubtTransactions(String strInfo)
        {
            OMError ErrorObj = null;
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""HandleInDoubtTransactions()"",""HandleInDoubtTransactions() called for TransactionID:" + TransactionID.ToString() + @"""");
                DurableLogDoc = new XmlDocument();
                ErrorObj = ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null && ErrorObj == null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    ErrorObj = ExecuteInDoubtTransactions(LogNodes, strInfo);
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }


        /// <summary>
        /// Method overridden which states the phase of Transaction
        /// </summary>
        /// <param name="enlistment"></param>
        public void InDoubt(Enlistment enlistment)
        {
            enlistment.Done();
        }

        /// <summary>
        /// Method overridden building phase for Transaction
        /// </summary>
        /// <param name="preparingEnlistment"></param>
        public void Prepare(PreparingEnlistment preparingEnlistment)
        {
            preparingEnlistment.Prepared();
        }

        /// <summary>
        /// If any error is occured, all the changes are rolled back
        /// </summary>
        /// <param name="enlistment"></param>
        public void Rollback(Enlistment enlistment)
        {
            string DurableLogXML = null;
            XmlDocument DurableLogDoc = null;
            XmlNodeList LogNodes = null;

            try
            {
                DurableLogDoc = new XmlDocument();
                ReadDurableLog(TransactionID, ref DurableLogXML);
                if (DurableLogXML != null)
                {
                    DurableLogDoc.LoadXml(DurableLogXML);
                    LogNodes = DurableLogDoc.SelectNodes("//LOG");
                    RollBackOperations(LogNodes);
                }
                enlistment.Done();
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        /// <summary>
        /// <CreatedOn>18 Februaty 2009</CreatedOn>
        /// <Author>Sagar Dasalkar</Author>
        /// <Purpose>
        /// Close database connection used by Data Manager(LogDM).
        /// </Purpose>
        /// </summary>
        public void CloseDatabaseConnection()
        {
            try
            {
                if (LOGDMObj != null)
                    LOGDMObj.DataProvider.Close();
            }
            catch
            {
                throw;
            }
        }

        private OMError ReadDurableLog(ulong TransactionID, ref string DurableLogXML)
        {
            OMError ErrorObj = null;
            try
            {
                if ((ErrorObj = FILEDMObj.DataProvider.AddClause("TRANSACTIONID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, TransactionID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.AddClause("ISXMLOPERATION", OMFieldDataType.INTEGER, OMOperator.EQUALTO, "0", OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = FILEDMObj.DataProvider.ReadAllRecordEx("LOG", "ALFS_DURABLELOG", null, ref DurableLogXML)) != null)
                        {
                            return ErrorObj;
                        }
                    }
                }
                return ErrorObj;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private OMError RollBackOperations(XmlNodeList LogNodes)
        {
            OMError ErrObj = null;
            string Operation = null;
            StringCollection Parameters = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            string OnMediaServer = null;
            string MediaServerUrl = null;
            bool bLocalFile = false;
            bool bRemoteFile = false;
            IFileSystem IFileSystemObjRemote = null;
            FileSystemInterface FileSystemInterfaceObjRemote = null;
            IFileSystem IFileSystemObjLocal = null;
            FileSystemInterface FileSystemInterfaceObjLocal = null;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;
            try
            {
                Parameters = new StringCollection();
                foreach (XmlNode LogNode in LogNodes)
                {
                    bCaseHit = false;
                    Parameters.Clear();
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                    OnMediaServer = LogNode.SelectSingleNode("ONMEDIASERVER").InnerText;
                    MediaServerUrl = LogNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                    if (OnMediaServer.ToUpper() == "FALSE")
                    {
                        if (IFileSystemObjLocal == null)
                        {
                            FileSystemInterfaceObjLocal = LocalFileSystemInterfaceObj;
                            IFileSystemObjLocal = FileSystemInterfaceObjLocal.IFileSystemInstance;
                        }
                        bLocalFile = true;
                    }
                    else if (OnMediaServer.ToUpper() == "TRUE")
                    {
                        if (IFileSystemObjRemote == null)
                        {
                            if (string.IsNullOrEmpty(MediaServerUrl))
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;
                            }
                            else
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = MediaServerUrl;
                            }
                            FileSystemInterfaceObjRemote = RemoteFileSystemInterfaceObj;
                            IFileSystemObjRemote = FileSystemInterfaceObjRemote.IFileSystemInstance;
                        }
                        bRemoteFile = true;
                    }
                    switch (Operation.ToUpper())
                    {
                        case "CREATEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            Parameters.Add(ParamsNodes[1].InnerText);
                            PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance);
                            //DecrementFolderCount(ParamsNodes[1].InnerText);
                            bCaseHit = true;
                            break;

                        case "MOVEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM"); //RESTORE MOVED FILE
                            Parameters.Add(ParamsNodes[1].InnerText);
                            Parameters.Add(ParamsNodes[0].InnerText);
                            if ((ErrObj = PerformOperation("MoveFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance)) != null)
                            {
                                if (ErrObj.Serialize().Contains("System.IO.IOException: Cannot create a file when that file already exists."))
                                {
                                    //Delete source file
                                    IFileSystem iFileSystem = GetFileSystemInterface(LogNode).IFileSystemInstance;
                                    if (iFileSystem.IsFileExists(ParamsNodes[1].InnerText))
                                    {
                                        Parameters.Clear();
                                        Parameters.Add(ParamsNodes[1].InnerText);
                                        PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance);
                                    }
                                    else
                                        return ErrObj;
                                }
                                else if (ErrObj.Serialize().Contains("System.IO.FileNotFoundException: Could not find file"))
                                {
                                    IFileSystem iFileSystem = GetFileSystemInterface(LogNode).IFileSystemInstance;
                                    if (iFileSystem.IsFileExists(ParamsNodes[1].InnerText)) // file at source is present
                                    {
                                        if ((ErrObj = iFileSystem.CopyFile(ParamsNodes[1].InnerText, ParamsNodes[0].InnerText)) == null)
                                        {
                                            if ((ErrObj = iFileSystem.DeleteFile(ParamsNodes[1].InnerText)) != null)
                                                return ErrObj;
                                        }
                                        else
                                            return ErrObj;
                                        //try CopyFiles and then delete
                                    }
                                    else
                                    {
                                        return ErrObj;
                                    }
                                }
                                else
                                    return ErrObj;
                            }
                            bCaseHit = true;
                            break;

                        case "UPDATEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM"); //DELETE FILE FROM DESTINATION LOCATION
                            Parameters.Add(ParamsNodes[0].InnerText);
                            Parameters.Add(ParamsNodes[1].InnerText);
                            PerformOperation("CopyFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance);

                            Parameters.Clear();
                            Parameters.Add(ParamsNodes[0].InnerText);
                            if ((ErrObj = PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance)) != null) //DELETE FILE AT SHADOW FOLDER PATH
                            {
                                return ErrObj;
                            }

                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;

                        case "DELETEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM"); //DELETE FILE FROM DESTINATION LOCATION
                            Parameters.Add(ParamsNodes[1].InnerText);
                            Parameters.Add(ParamsNodes[0].InnerText);
                            if ((ErrObj = PerformOperation("MoveFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance)) != null)
                            {
                                if (ErrObj.Serialize().Contains("System.IO.IOException: Cannot create a file when that file already exists."))
                                {
                                    // Delete file from shadow folder
                                    IFileSystem iFileSystem = GetFileSystemInterface(LogNode).IFileSystemInstance;
                                    if (iFileSystem.IsFileExists(ParamsNodes[1].InnerText))
                                    {
                                        Parameters.Clear();
                                        Parameters.Add(ParamsNodes[1].InnerText);
                                        PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance);
                                    }
                                    else
                                        return ErrObj;
                                }
                                else if (ErrObj.Serialize().Contains("System.IO.FileNotFoundException: Could not find file"))
                                {
                                    IFileSystem iFileSystem = GetFileSystemInterface(LogNode).IFileSystemInstance;
                                    if (iFileSystem.IsFileExists(ParamsNodes[1].InnerText)) // file at source is present
                                    {
                                        if ((ErrObj = iFileSystem.CopyFile(ParamsNodes[1].InnerText, ParamsNodes[0].InnerText)) == null)
                                        {
                                            if ((ErrObj = iFileSystem.DeleteFile(ParamsNodes[1].InnerText)) != null)
                                                return ErrObj;
                                        }
                                        else
                                            return ErrObj;
                                        //try CopyFiles and then delete
                                    }
                                    else
                                    {
                                        return ErrObj;
                                    }
                                }
                                else
                                    return ErrObj;
                            }
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;

                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO ROLLEDBACK
                        if ((ErrObj = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, "ROLLEDBACK", string.Empty)) != null)
                        {
                            return ErrObj;
                        }
                    }
                }
                if (bLocalFile)
                {
                    foreach (string strFolder in LocalFolders)
                    {
                        IFileSystemObjLocal.DeleteFolder(strFolder);
                    }
                }
                if (bRemoteFile)
                {
                    foreach (string strFolder in RemoteFolders)
                    {
                        IFileSystemObjRemote.DeleteFolder(strFolder);
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError DecrementFolderCount(string FilePath)
        {
            OMError ErrorObj = null;
            string FolderPath = null;
            string FolderXML = null;
            XmlDocument FolderDocument = null;
            ulong NoOfFiles = 0;
            StringBuilder Query = null;

            try
            {
                FolderDocument = new XmlDocument();
                Query = new StringBuilder();
                FolderPath = FilePath.Substring(0, FilePath.LastIndexOf('\\'));

                Monitor.Enter(Lock);
                if ((ErrorObj = FILEDMObj.DataProvider.AddClause("PATH", OMFieldDataType.STRING, OMOperator.EQUALTO, FolderPath, OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.ReadRecord("FOLDER", "ALFS_FOLDER", null, ref FolderXML)) == null)
                    {
                        if (FolderXML != null)
                        {
                            FolderDocument.LoadXml(FolderXML);
                            NoOfFiles = Convert.ToUInt64(FolderDocument.SelectSingleNode("//NOOFFILES").InnerText);
                            //Defencive code added.
                            if (NoOfFiles > 0)
                            {
                                NoOfFiles--;
                                if (NoOfFiles < MaxFileCount)
                                {
                                    Query.AppendFormat("Update ALFS_FOLDER set NOOFFILES = {0}, STATUS = 'AVAILABLE' where PATH = '{1}'", NoOfFiles, FolderPath);
                                }
                                else
                                {
                                    Query.AppendFormat("Update ALFS_FOLDER set NOOFFILES = {0} where PATH = '{1}'", NoOfFiles, FolderPath);
                                }

                                if ((ErrorObj = FILEDMObj.DataProvider.Execute(Query.ToString())) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            finally
            {
                Monitor.Exit(Lock);
            }

        }

        private OMError CommitOperations(XmlNodeList LogNodes)
        {
            OMError ErrObj = null;
            string Operation = null;
            StringCollection Parameters = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            string OnMediaServer = null;
            string MediaServerUrl = null;
            bool bLocalFile = false;
            bool bRemoteFile = false;
            IFileSystem IFileSystemObjRemote = null;
            FileSystemInterface FileSystemInterfaceObjRemote = null;
            IFileSystem IFileSystemObjLocal = null;
            FileSystemInterface FileSystemInterfaceObjLocal = null;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;

            try
            {
                Parameters = new StringCollection();
                foreach (XmlNode LogNode in LogNodes)
                {
                    bCaseHit = false;
                    Parameters.Clear();
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                    OnMediaServer = LogNode.SelectSingleNode("ONMEDIASERVER").InnerText;
                    MediaServerUrl = LogNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                    if (OnMediaServer.ToUpper() == "FALSE")
                    {
                        if (IFileSystemObjLocal == null)
                        {
                            FileSystemInterfaceObjLocal = LocalFileSystemInterfaceObj;
                            IFileSystemObjLocal = FileSystemInterfaceObjLocal.IFileSystemInstance;
                        }
                        bLocalFile = true;
                    }
                    else if (OnMediaServer.ToUpper() == "TRUE")
                    {
                        if (IFileSystemObjRemote == null)
                        {
                            if (string.IsNullOrEmpty(MediaServerUrl))
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;
                            }
                            else
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = MediaServerUrl;
                            }
                            FileSystemInterfaceObjRemote = RemoteFileSystemInterfaceObj;
                            IFileSystemObjRemote = FileSystemInterfaceObjRemote.IFileSystemInstance;
                        }
                        bRemoteFile = true;
                    }
                    switch (Operation.ToUpper())
                    {
                        case "DELETEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            Parameters.Add(ParamsNodes[1].InnerText);
                            if ((ErrObj = PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance)) != null)
                            {
                                return ErrObj;
                            }
                            //if ((ErrObj = DecrementFolderCount(ParamsNodes[0].InnerText)) != null)
                            //{
                            //    return ErrObj;
                            //}
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;
                        case "UPDATEFILE": ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            Parameters.Add(ParamsNodes[0].InnerText);
                            if ((ErrObj = PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(LogNode).IFileSystemInstance)) != null) //DELETE FILE AT SHADOW FOLDER PATH
                            {
                                return ErrObj;
                            }
                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;

                        case "CREATEFILE":
                            bCaseHit = true;
                            break;

                        case "MOVEFILE":
                            bCaseHit = true;
                            break;

                        case "COPYFILE":
                            bCaseHit = true;
                            break;

                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO COMMITED
                        if ((ErrObj = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, "COMMITED", string.Empty)) != null)
                        {
                            return ErrObj;
                        }
                    }
                }
                if (bLocalFile)
                {
                    foreach (string strFolder in LocalFolders)
                    {
                        IFileSystemObjLocal.DeleteFolder(strFolder);
                    }
                }
                if (bRemoteFile)
                {
                    foreach (string strFolder in RemoteFolders)
                    {
                        IFileSystemObjRemote.DeleteFolder(strFolder);
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError ExecuteInDoubtTransactions(XmlNodeList LogNodes, String strInfo)
        {
            OMError ErrObj = null;
            string Operation = null;
            StringCollection Parameters = null;
            XmlNodeList ParamsNodes = null;
            bool bCaseHit = false;
            string OnMediaServer = null;
            string MediaServerUrl = null;
            bool bLocalFile = false;
            bool bRemoteFile = false;
            IFileSystem IFileSystemObjRemote = null;
            FileSystemInterface FileSystemInterfaceObjRemote = null;
            IFileSystem IFileSystemObjLocal = null;
            FileSystemInterface FileSystemInterfaceObjLocal = null;
            StringCollection LocalFolders = new StringCollection();
            StringCollection RemoteFolders = new StringCollection();
            string FolderPath = null;
            string sOperationStatus = string.Empty;

            try
            {
                Parameters = new StringCollection();
                foreach (XmlNode LogNode in LogNodes)
                {
                    IFileSystem iFSMInstance = GetFileSystemInterface(LogNode).IFileSystemInstance;
                    bCaseHit = false;
                    Parameters.Clear();
                    Operation = LogNode.SelectSingleNode("OPERATION").InnerText;
                    OnMediaServer = LogNode.SelectSingleNode("ONMEDIASERVER").InnerText;
                    MediaServerUrl = LogNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                    if (OnMediaServer.ToUpper() == "FALSE")
                    {
                        if (IFileSystemObjLocal == null)
                        {
                            FileSystemInterfaceObjLocal = LocalFileSystemInterfaceObj;
                            IFileSystemObjLocal = FileSystemInterfaceObjLocal.IFileSystemInstance;
                        }
                        bLocalFile = true;
                    }
                    else if (OnMediaServer.ToUpper() == "TRUE")
                    {
                        if (IFileSystemObjRemote == null)
                        {
                            if (string.IsNullOrEmpty(MediaServerUrl))
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;
                            }
                            else
                            {
                                RemoteFileSystemInterfaceObj.MediaServerUrl = MediaServerUrl;
                            }
                            FileSystemInterfaceObjRemote = RemoteFileSystemInterfaceObj;
                            IFileSystemObjRemote = FileSystemInterfaceObjRemote.IFileSystemInstance;
                        }
                        bRemoteFile = true;
                    }
                    switch (Operation.ToUpper())
                    {
                        case "DELETEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "ROLLEDBACK";
                            if (!iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    if ((ErrObj = PerformOperation("MoveFile", Parameters, iFSMInstance)) != null) //MOVE FILE FROM SHADOW FOLDER PATH
                                    {
                                        return ErrObj;
                                    }
                                }
                            }

                            bCaseHit = true;
                            FolderPath = ParamsNodes[1].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;
                        case "UPDATEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "COMMITED";
                            bCaseHit = true;
                            break;

                        case "CREATEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "COMMITED";
                            if (!iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    if ((ErrObj = PerformOperation("CreateFile", Parameters, iFSMInstance)) != null) //MOVE FILE FROM SHADOW FOLDER PATH
                                    {
                                        return ErrObj;
                                    }
                                }
                            }
                            bCaseHit = true;
                            break;

                        case "MOVEFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "COPIED";
                            if (!iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    if ((ErrObj = PerformOperation("CopyFile", Parameters, iFSMInstance)) != null) //MOVE FILE FROM SHADOW FOLDER PATH
                                    {
                                        return ErrObj;
                                    }
                                }
                            }
                            if (!iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    if ((ErrObj = PerformOperation("CopyFile", Parameters, iFSMInstance)) != null) //MOVE FILE FROM SHADOW FOLDER PATH
                                    {
                                        return ErrObj;
                                    }
                                }
                            }
                            bCaseHit = true;
                            FolderPath = ParamsNodes[0].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;

                        case "COPYFILE":
                            ParamsNodes = LogNode.SelectNodes("PARAMETERINFO/PARAMETERS/PARAM");
                            sOperationStatus = "COMMITED";
                            if (!iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    if ((ErrObj = PerformOperation("CopyFile", Parameters, iFSMInstance)) != null)
                                    {
                                        return ErrObj;
                                    }
                                }
                            }
                            if (!iFSMInstance.IsFileExists(ParamsNodes[0].InnerText))
                            {
                                if (iFSMInstance.IsFileExists(ParamsNodes[1].InnerText))
                                {
                                    Parameters.Clear();
                                    Parameters.Add(ParamsNodes[1].InnerText);
                                    Parameters.Add(ParamsNodes[0].InnerText);
                                    if ((ErrObj = PerformOperation("CopyFile", Parameters, iFSMInstance)) != null)
                                    {
                                        return ErrObj;
                                    }
                                }
                            }
                            bCaseHit = true;
                            FolderPath = ParamsNodes[0].InnerText;
                            FolderPath = FolderPath.Substring(0, FolderPath.LastIndexOf(@"\"));
                            if (OnMediaServer.ToUpper() == "FALSE")
                            {
                                if (!LocalFolders.Contains(FolderPath))
                                    LocalFolders.Add(FolderPath);
                            }
                            else if (OnMediaServer.ToUpper() == "TRUE")
                            {
                                if (!RemoteFolders.Contains(FolderPath))
                                    RemoteFolders.Add(FolderPath);
                            }
                            break;

                    }
                    if (bCaseHit)
                    {
                        //UPDATE DURABLE LOG STATUS TO COMMITED
                        if ((ErrObj = UpdateDurableLogStatus(LogNode.SelectSingleNode("LOGID").InnerText, sOperationStatus, strInfo)) != null)
                        {
                            return ErrObj;
                        }
                    }
                }
                if (bLocalFile)
                {
                    foreach (string strFolder in LocalFolders)
                    {
                        IFileSystemObjLocal.DeleteFolder(strFolder);
                    }
                }
                if (bRemoteFile)
                {
                    foreach (string strFolder in RemoteFolders)
                    {
                        IFileSystemObjRemote.DeleteFolder(strFolder);
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private OMError PerformOperation(string Operation, StringCollection Parameters, IFileSystem IFileSystemObj)
        {
            OMError ErrObj = null;
            string[] Parameter = new string[Parameters.Count];
            try
            {
                for (int cnt = 0; cnt < Parameters.Count; cnt++)
                {
                    Parameter[cnt] = Parameters[cnt];
                }
                if ((ErrObj = (OMError)IFileSystemObj.GetType().InvokeMember(Operation, BindingFlags.InvokeMethod, null, IFileSystemObj, Parameter)) != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }
        /// <summary>
        /// This function will process all the Filesystem Node for an Entity instance
        /// </summary>
        /// <param name="ObjectID">Entity/Facet ID</param>
        /// <param name="ObjectType">ObjectType</param>
        /// <param name="ObjectTypeID">ObjectTypeID</param>
        /// <param name="AssetNode"></param>
        /// <returns></returns>
        public OMError ProcessFileSystem(ulong ObjectID, string ObjectType, ulong ObjectTypeID, string AssetNode)
        {
            OMError ReturnErrObj = null;
            XmlDocument AssetDoc = null;
            XmlNodeList BinderNodes = null;
            XmlNodeList FileNodes = null;
            OMError ErrorObj = null;
            try
            {
                //Return error when COPYFILES=TRUE and MOVEFILES=TRUE, only one flag can be set to true
                if (CopyFiles && MoveFiles)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_PROCESSFILESYSTEM_FAILED,
                                                   "Both COPYFILES and MOVEFILES cannot be set to true.",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", OMErrorType.ERROR,
                                                   null, null);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }

                AssetDoc = new XmlDocument();
                AssetDoc.LoadXml(AssetNode);
                BinderNodes = AssetDoc.SelectNodes("FILESYSTEM/BINDER");
                FileNodes = AssetDoc.SelectNodes("FILESYSTEM/FILE");

                if ((ErrorObj = ProcessBinders(ObjectID, ObjectType, ObjectTypeID, BinderNodes)) != null)
                {
                    ErrorObj.TransactionId = TransactionID.ToString();
                    return ErrorObj;
                }

                if ((ErrorObj = ProcessFiles(ObjectID, ObjectType, ObjectTypeID, FileNodes)) != null)
                {
                    ErrorObj.TransactionId = TransactionID.ToString();
                    return ErrorObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_PROCESSFILESYSTEM_FAILED,
                                                   "Exception Thrown in ProcessFileSystem API",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", OMErrorType.EXCEPTION,
                                                   ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        /// <summary>
        /// This function retrives Files from MountPoint and copies it to OutBxx
        /// </summary>
        /// <param name="ASyncronousFieldList">Fields i.e files which are to be transferred asyncronously</param>
        /// <param name="AvoidFileTransferFieldList"></param>
        /// <param name="FileSystemXML">FileSystem node retrived from Stored proc execution</param>
        /// <returns></returns>
        public OMError RetriveFileSystem(StringCollection ASyncronousFieldList, StringCollection AvoidFileTransferFieldList, ref string FileSystemXML)
        {
            XmlDocument FileSystemXMLDoc = null;
            ulong FolderID = 0;
            StringBuilder Date = null;
            StringBuilder DestinationPath = null;
            StringBuilder FilePath = null;
            ulong FileID = 0;
            XmlNode FileIDNode = null;
            StringBuilder SourcePath = null;
            String SourcePathString = null;
            XmlNode SourcePathNode = null;
            XmlNode ObjectIDNode = null;
            XmlNodeList BinderFiles = null;
            XmlNodeList FileSystemNodes = null;
            XmlNodeList FileNodes = null;
            XmlNodeList FileIDNodes = null;
            XmlNodeList BinderNodes = null;
            string FileName = null;
            string BinderName = null;
            string FieldName = null;
            XmlAttribute ObjectIDAttribute = null;
            ulong ObjectID = 0;
            StringBuilder FolderPath = null;
            ulong SchemaID = 0;
            ulong BinderID = 0;
            ulong SequenceOrder = 0;
            //XmlNode                     BinderIDNode                    = null;
            bool isFolderCreated = false;
            XmlNode SchemaIDNode = null;
            String SchemaIDString = null;
            OMError ErrorObj = null;

            FileSystemInterface FileSystemInterfaceObj = null;

            try
            {
                Date = new StringBuilder();
                SourcePath = new StringBuilder();
                DestinationPath = new StringBuilder();
                FilePath = new StringBuilder();
                FolderPath = new StringBuilder();
                FileSystemXMLDoc = new XmlDocument();
                FileSystemXMLDoc.LoadXml(FileSystemXML);
                FileSystemNodes = FileSystemXMLDoc.SelectNodes("//FILESYSTEM");


                foreach (XmlNode FileSystemNode in FileSystemNodes)
                {

                    ObjectIDAttribute = null;
                    UniqueID.Read(ref FolderID);
                    DestinationPath.Remove(0, DestinationPath.Length);
                    DestinationPath.AppendFormat("{0}\\{1}", OutBox, FolderID);
                    FileNodes = FileSystemNode.SelectNodes("FILE");
                    BinderNodes = FileSystemNode.SelectNodes("BINDER");

                    //IFileSystemObj.CreateFolder(DestinationPath.ToString());
                    ObjectIDAttribute = (XmlAttribute)FileSystemNode.Attributes.GetNamedItem("ObjectID");
                    if (ObjectIDAttribute != null)
                        ObjectID = Convert.ToUInt64(ObjectIDAttribute.InnerText);
                    else
                    {
                        if (FileSystemNode.SelectSingleNode("OBJECTID") != null)
                            ObjectID = Convert.ToUInt64(FileSystemNode.SelectSingleNode("OBJECTID").InnerText);
                    }

                    foreach (XmlNode FileNode in FileNodes)
                    {
                        #region Get the FileSystem intreface for the Current File Operation
                        FileSystemInterfaceObj = GetFileSystemInterface(FileNode);
                        #endregion

                        isFolderCreated = false;
                        FilePath.Remove(0, FilePath.Length);
                        SourcePath.Remove(0, SourcePath.Length);
                        FolderPath.Remove(0, FolderPath.Length);
                        // This code change is added because if multiple file nodes are present in the system, then same OBJECTID was taken.
                        ObjectIDNode = FileNode.SelectSingleNode("OBJECTID");
                        if (ObjectIDNode != null)
                            ObjectID = Convert.ToUInt64(ObjectIDNode.InnerText);
                        FileName = FileNode.SelectSingleNode("NAME").InnerText;
                        // Anand : This code is added because if schemaid is provided we use SchemaID else 
                        // FieldName. As Uniqueness is not garanteed incase of field name. i.e. two entities can have
                        // same field name.(Earlier schemaid was not getting passed by getdata/gethierarchy thus we could not 
                        // use this scheme but as getdata/gethierarchy is changed to pass schemaid, we can use this scheme).
                        SchemaIDString = null;
                        if ((SchemaIDNode = FileNode.SelectSingleNode("SCHEMAID")) != null)
                            SchemaIDString = SchemaIDNode.InnerText;
                        FieldName = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                        // Anand : This code is moved here to improve performance. If file is not requested by the user
                        // then we do not query on file table.
                        if (!String.IsNullOrEmpty(SchemaIDString))
                        {
                            // Here we are not removing the PATH node because
                            // in future search will changed so that PATH will not be returned from search.
                            if (AvoidFileTransferFieldList.Contains(SchemaIDString))
                                continue;
                        }
                        else
                        {
                            if (AvoidFileTransferFieldList.Contains(FieldName))
                                continue;
                        }
                        if (ObjectID != 0) // This part is added because file request can cum from outside.Other than search.
                        {
                            SchemaID = Convert.ToUInt64(SchemaIDString);
                            ErrorObj = GetFileInfo(FileName, SchemaID, ObjectID, ref FileID, ref SourcePathString);
                            if (ErrorObj != null)
                            {
                                ErrorObj.TransactionId = TransactionID.ToString();
                                return ErrorObj;
                            }
                            if (FileNode.SelectNodes("PATH").Count > 0)
                                SourcePathNode = FileNode.SelectSingleNode("PATH");
                            else
                            {
                                SourcePathNode = FileSystemXMLDoc.CreateElement("PATH");
                                FileNode.AppendChild(SourcePathNode);
                            }

                        }
                        else
                        {
                            FileIDNode = FileNode.SelectSingleNode("FILEID");
                            if (FileIDNode != null)
                            {
                                FileID = Convert.ToUInt64(FileIDNode.InnerText);
                                SourcePathNode = FileNode.SelectSingleNode("PATH");
                                SourcePathString = SourcePathNode.InnerText;

                            }
                            else
                            {
                                if (FileNode.SelectSingleNode("OBJECTID") != null)
                                    ObjectID = Convert.ToUInt64(FileNode.SelectSingleNode("OBJECTID").InnerText);

                                if (ObjectID != 0) // This part is added because file request can cum from outside.Other than search.
                                {
                                    SchemaID = Convert.ToUInt64(FileNode.SelectSingleNode("SCHEMAID").InnerText);
                                    ErrorObj = GetFileInfo(FileName, SchemaID, ObjectID, ref FileID, ref SourcePathString);
                                    if (ErrorObj != null)
                                    {
                                        ErrorObj.TransactionId = TransactionID.ToString();
                                        return ErrorObj;
                                    }
                                    if (FileNode.SelectNodes("PATH").Count > 0)
                                        SourcePathNode = FileNode.SelectSingleNode("PATH");
                                    else
                                    {
                                        SourcePathNode = FileSystemXMLDoc.CreateElement("PATH");
                                        FileNode.AppendChild(SourcePathNode);
                                    }
                                }
                            }
                            FileIDNodes = FileNode.SelectNodes("FILEID");
                            if (FileIDNodes != null)
                            {
                                foreach (XmlNode FileIDNodeinList in FileIDNodes)
                                {
                                    FileNode.RemoveChild(FileIDNodeinList);
                                }
                            }
                        }

                        SourcePath.AppendFormat("{0}\\{1}", SourcePathString, FileID);
                        FolderPath.AppendFormat("{0}\\{1}", DestinationPath.ToString(), FieldName);
                        //IFileSystemObj.CreateFolder(FolderPath.ToString());
                        FilePath.AppendFormat("{0}\\{1}", FolderPath.ToString(), FileName);

                        SourcePathNode.InnerText = FilePath.ToString();

                        /* Code moved above.
                        if (AvoidFileTransferFieldList.Contains(FieldName))
                        {
                            FileNode.RemoveChild(SourcePathNode);
                            continue;
                        }*/
                        //Code added on 12 Feb 2007
                        //The following code will create folder hierarchy only if the files are requested
                        if (isFolderCreated == false)
                        {
                            FileSystemInterfaceObj.IFileSystemInstance.CreateFolder(FolderPath.ToString());
                            isFolderCreated = true;
                        }
                        // Anand : This code is added because if schemaid is provided we use SchemaID else 
                        // FieldName. As Uniqueness is not garanteed incase of field name. i.e. two entities can have
                        // same field name.(Earlier schemaid was not getting passed by getdata/gethierarchy thus we could not 
                        // use this scheme but as getdata/gethierarchy is changed to pass schemaid, we can use this scheme).

                        if (!String.IsNullOrEmpty(SchemaIDString))
                        {
                            if (ASyncronousFieldList.Contains(SchemaIDString))
                            {
                                if ((ErrorObj = ASyncronousFileTransfer(SourcePath.ToString(),
                                    FilePath.ToString(), FileSystemInterfaceObj.IFileSystemInstance)) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                            else
                            {
                                if ((ErrorObj = FileSystemInterfaceObj.IFileSystemInstance.CopyFile(SourcePath.ToString(),
                                    FilePath.ToString())) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                        }
                        else
                        {
                            if (ASyncronousFieldList.Contains(FieldName))
                            {
                                if ((ErrorObj = ASyncronousFileTransfer(SourcePath.ToString(),
                                    FilePath.ToString(), FileSystemInterfaceObj.IFileSystemInstance)) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                            else
                            {
                                if ((ErrorObj = FileSystemInterfaceObj.IFileSystemInstance.CopyFile(SourcePath.ToString(),
                                    FilePath.ToString())) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                        }

                    }
                    foreach (XmlNode BinderNode in BinderNodes)
                    {
                        SchemaIDString = null;

                        BinderName = BinderNode.SelectSingleNode("NAME").InnerText;
                        FieldName = BinderNode.SelectSingleNode("FIELDNAME").InnerText;
                        DestinationPath.AppendFormat("\\{0}", FieldName);
                        //IFileSystemObj.CreateFolder(DestinationPath.ToString());
                        // This change is done because same ObjectID was used for Multiple Binder Nodes
                        ObjectIDNode = BinderNode.SelectSingleNode("OBJECTID");
                        if (ObjectIDNode != null)
                            ObjectID = Convert.ToUInt64(ObjectIDNode.InnerText);

                        if (ObjectID != 0)
                        {
                            if (BinderNode.SelectSingleNode("SCHEMAID") != null)
                            {
                                SchemaID = Convert.ToUInt64(BinderNode.SelectSingleNode("SCHEMAID").InnerText);
                                SchemaIDString = BinderNode.SelectSingleNode("SCHEMAID").InnerText;

                                GetBinderID(ObjectID, SchemaID, ref BinderID);
                            }
                            else
                            {
                                if (BinderNode.SelectSingleNode("BINDERID") != null)
                                {
                                    BinderID = Convert.ToUInt64(BinderNode.SelectSingleNode("BINDERID").InnerText);
                                }
                            }
                        }

                        //Code commented by : Sagar Dasalkar, Date : 13 November 2009
                        //Reason: BINDERID would be returned in output XML even if FileRequired='YES' set for binder field
                        //if (BinderNode.SelectNodes("BINDERID").Count > 0)
                        //{
                        //    BinderIDNode = BinderNode.SelectSingleNode("BINDERID");
                        //    BinderNode.RemoveChild(BinderIDNode);
                        //}

                        BinderFiles = BinderNode.SelectNodes("FILE");
                        foreach (XmlNode BinderFile in BinderFiles)
                        {
                            #region Get the FileSystem intreface for the Current File Operation
                            FileSystemInterfaceObj = GetFileSystemInterface(BinderFile);
                            #endregion

                            FilePath.Remove(0, FilePath.Length);
                            SourcePath.Remove(0, SourcePath.Length);
                            FolderPath.Remove(0, FolderPath.Length);

                            FileName = BinderFile.SelectSingleNode("NAME").InnerText;
                            SequenceOrder = Convert.ToUInt64(BinderFile.SelectSingleNode("SEQUENCEORDER").InnerText);
                            if (ObjectID != 0) // This part is added because file request can cum from outside.Other than search.
                            {
                                SchemaID = Convert.ToUInt64(BinderFile.SelectSingleNode("SCHEMAID").InnerText);
                                SchemaIDString = BinderFile.SelectSingleNode("SCHEMAID").InnerText;

                                GetBinderFileInfo(FileName, SchemaID, ObjectID, BinderID, SequenceOrder, ref FileID, ref SourcePathString);
                                if (BinderFile.SelectNodes("PATH").Count > 0)
                                    SourcePathNode = BinderFile.SelectSingleNode("PATH");
                                else
                                {
                                    SourcePathNode = FileSystemXMLDoc.CreateElement("PATH");
                                    BinderFile.AppendChild(SourcePathNode);
                                }
                            }
                            else
                            {
                                FileIDNode = BinderFile.SelectSingleNode("FILEID");
                                if (FileIDNode != null)
                                {
                                    FileID = Convert.ToUInt64(FileIDNode.InnerText);
                                    SourcePathNode = BinderFile.SelectSingleNode("PATH");
                                    SourcePathString = SourcePathNode.InnerText;
                                }
                                else
                                {
                                    if (BinderFile.SelectSingleNode("OBJECTID") != null)
                                        ObjectID = Convert.ToUInt64(BinderFile.SelectSingleNode("OBJECTID").InnerText);
                                    if (ObjectID != 0)
                                    {
                                        SchemaID = Convert.ToUInt64(BinderFile.SelectSingleNode("SCHEMAID").InnerText);
                                        SchemaIDString = BinderFile.SelectSingleNode("SCHEMAID").InnerText;

                                        GetBinderFileInfo(FileName, SchemaID, ObjectID, BinderID, SequenceOrder, ref FileID, ref SourcePathString);
                                        if (BinderFile.SelectNodes("PATH").Count > 0)
                                            SourcePathNode = BinderFile.SelectSingleNode("PATH");
                                        else
                                        {
                                            SourcePathNode = FileSystemXMLDoc.CreateElement("PATH");
                                            BinderFile.AppendChild(SourcePathNode);
                                        }
                                    }
                                }
                                //BinderFile.RemoveChild(FileIDNode);

                                FileIDNodes = BinderFile.SelectNodes("FILEID");
                                if (FileIDNodes != null)
                                {
                                    foreach (XmlNode FileIDNodeinList in FileIDNodes)
                                    {
                                        BinderFile.RemoveChild(FileIDNodeinList);
                                    }
                                }
                            }
                            SourcePath.AppendFormat("{0}\\{1}", SourcePathString, FileID);
                            FolderPath.AppendFormat("{0}\\{1}", DestinationPath.ToString(), SequenceOrder);
                            //IFileSystemObj.CreateFolder(FolderPath.ToString());

                            FilePath.AppendFormat("{0}\\{1}", FolderPath.ToString(), FileName);

                            SourcePathNode.InnerText = FilePath.ToString();

                            //Remove the file to transfer. Previously was using FieldName to remove,
                            //but now we use SchemaID, so removing based on that
                            if (!String.IsNullOrEmpty(SchemaIDString))
                            {
                                if (AvoidFileTransferFieldList.Contains(SchemaIDString))
                                {
                                    BinderFile.RemoveChild(SourcePathNode);
                                    continue;
                                }
                            }
                            else
                            {
                                if (AvoidFileTransferFieldList.Contains(FieldName))
                                {
                                    BinderFile.RemoveChild(SourcePathNode);
                                    continue;
                                }
                            }

                            //Code added on 12 Feb 2007
                            //The following code will create folder hierarchy only if the files are requested
                            FileSystemInterfaceObj.IFileSystemInstance.CreateFolder(FolderPath.ToString());

                            if (ASyncronousFieldList.Contains(FieldName))
                            {
                                if ((ErrorObj = ASyncronousFileTransfer(SourcePath.ToString(),
                                     FilePath.ToString(), FileSystemInterfaceObj.IFileSystemInstance)) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                            else
                            {
                                if ((ErrorObj = FileSystemInterfaceObj.IFileSystemInstance.CopyFile(SourcePath.ToString(),
                                    FilePath.ToString())) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }
                        }

                    }
                }
                FileSystemXML = FileSystemXMLDoc.OuterXml;
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError GetFileInfo(string FileName, ulong SchemaID, ulong ObjectID, ref ulong FileID, ref string SourcePath)
        {
            OMError ErrObj = null;
            string FileXML = null;
            XmlDocument FileDoc = null;
            XmlNode FileIDNode = null;
            XmlNode SourcePathNode = null;

            if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
            {
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("NAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FileName, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("FILE", "ALFS_FILE", null, ref FileXML)) == null)
                        {
                            if (String.IsNullOrEmpty(FileXML))
                            {
                                //File not Found
                                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILE_NOT_FOUND, "File not found", OMErrorObjectType.FILESYSTEM, "FileSystemManager", OMErrorType.ERROR, null, null);
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            else
                            {
                                FileDoc = new XmlDocument();
                                FileDoc.LoadXml(FileXML);
                                FileIDNode = FileDoc.SelectSingleNode("//FILEID");
                                SourcePathNode = FileDoc.SelectSingleNode("//PATH");
                                FileID = Convert.ToUInt64(FileIDNode.InnerText);
                                SourcePath = SourcePathNode.InnerText;
                            }
                        }
                    }
                }
            }
            return null;
        }

        /// <summary>
        /// <CreatedOn>11 September 2009</CreatedOn>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <purpose>
        /// Fetches file information.
        /// </purpose>
        /// </summary>
        /// <param name="ulFileID">Specify File Id for which information is to be fetched.</param>
        /// <param name="strFieldCollection">Specify which fields are to be fetched.</param>
        /// <param name="strFileDetails">Output parameter, in which file details will be returned in Xml format.</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError GetFileInfo(ulong ulFileID, String strFieldCollection, ref String strFileDetails)
        {
            OMError obj_OMError = null;     //Used to prepare OMError object

            //Add clause on FILEID and fetch file details (fetch requested fields)
            if ((obj_OMError = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ulFileID.ToString(), OMConjunction.NOTHING)) == null)
            {
                obj_OMError = FILEDMObj.DataProvider.ReadRecord("FILE", "ALFS_FILE", strFieldCollection, ref strFileDetails);
            }
            if (obj_OMError != null)
                obj_OMError.TransactionId = TransactionID.ToString();
            return obj_OMError;
        }

        /// <summary>
        /// Fetches file details of existing file and updates File Node for copy operation.
        /// </summary>
        /// <param name="ulObjectId">Specify Object Id for which a copy of existing file is to be created.</param>
        /// <param name="strObjectType">Specify Object Type.</param>
        /// <param name="ulObjectTypeId">Specify Object Type Id.</param>
        /// <param name="xmlNode_File">Output parameter, which will be updated with the existing file details.</param>
        /// <param name="bIsCopyFileOperation">Output parameter, whose value will specify whether the operation is COPY (otherwise CREATE).</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError GetExistingFileInfoForCopy(ulong ulObjectId, String strObjectType, ulong ulObjectTypeId, ref XmlNode xmlNode_File, ref bool bIsCopyFileOperation)
        {
            #region Variables

            ulong ul_FileId = 0;        //Used to store FILEID to copy files
            ulong ul_SchemaId = 0;        //Used to store SCHEMAID of field
            OMError obj_OMError = null;     //Used to prepare OM Error if an error or exception occurs
            bool b_ConversionResult = false;    //Used to identify whether FILEID could be parsed and converted to ulong
            String str_FileDetailsXml = null;     //Used to store FILE details (for given FILEID)
            String str_FieldCollectionXml = null;     //Used to prepare field collection to fetch FILE fields (for existing file)
            XmlDocument xmlDoc_FileDetails = null;     //Used to load file details Xml
            XmlNode xmlNode_FileDetails = null;     //Xml node used to get FILE details
            XmlNode xmlNode_ObjectId = null;     //Xml node used to get OBJECTID details
            XmlNode xmlNode_FieldNameFromInput = null;     //Xml node used to get FIELDNAME from input Xml
            XmlNode xmlNode_FieldName = null;     //Xml node used to add FIELDNAME node if it is not present file details Xml (fetched from database)
            XmlNode xmlNode_ObjectType = null;     //Used to fetch object type of file to be created (i.e. ENTITY/FACET)
            XmlNode xmlNode_FileId = null;     //Used to get FILEID of existing file
            XmlNode xmlNode_SchemaId = null;     //Xml node used to change SCHEMAID in input Xml
            bool b_IsOperationAttrValueCopy = false;    //Used to identify whether Operation=COPY
            String str_Operation = null;     //Used to get Operation attribute value

            #endregion Variables

            #region Get file information for given FILEID and update FileNode for copy operation

            try
            {
                #region Implementation Cases
                /* There can be following cases
                 * 1. Operation=COPY specified - Copy file operation to be performed
                 * 2. Operation attribute not specified, FILEID is specified with COPYFILES=TRUE - Copy file operation is to be performed
                 * 3. Operation attribute not specified, FILEID is specified with COPYFILES=FALSE - Create file operation is to be performed
                 * 4. Operation=CREATE specified - Create file operation is to be performed
                 * 5. Operation=CREATE specified, FILEID is specified with COPYFILES=TRUE - Create file operation is to be performed
                 * 6. Operation=CREATE specified, FILEID is specified with COPYFILES=FALSE - Create file operation is to be performed
                 * */
                #endregion Implementation Cases

                //Fetch operation attribute value. This will allow CreateFile method to distinguish between COPY and CREATE case
                obj_OMError = GetFileOperation(xmlNode_File, out str_Operation);
                if (obj_OMError != null)
                {
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                switch (str_Operation.ToUpper())
                {
                    case "COPY":
                        b_IsOperationAttrValueCopy = true;
                        break;

                    case "CREATE":
                        //When Operation attribute value is Create, then CreateFile operation will always be performed even if FILEID is specified.
                        return null;
                }

                //Operation=COPY
                if (b_IsOperationAttrValueCopy)
                {
                    xmlNode_FileId = xmlNode_File.SelectSingleNode("FILEID");

                    if (xmlNode_FileId == null)
                    {
                        //Return error when Operation="COPY" but FILEID is not specified
                        obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                   "Invalid File Id",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", OMErrorType.ERROR,
                                                   null, null);
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;

                    }

                    b_ConversionResult = ulong.TryParse(xmlNode_FileId.InnerText, out ul_FileId);

                    //Return error if there is an error parsing FILEID value
                    if (!b_ConversionResult && ul_FileId == 0)
                    {
                        obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                     "Invalid File Id",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", OMErrorType.ERROR,
                                                     null, null);
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    str_FieldCollectionXml = @"<FIELDCOLLECTION>
                                                <FIELD><FIELDNAME>FILEID</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>NAME</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>DISPLAYNAME</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>DESCRIPTION</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>PATH</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>ONMEDIASERVER</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>MEDIASERVERURL</FIELDNAME></FIELD>
                                                <FIELD><FIELDNAME>FILESIZE</FIELDNAME></FIELD>
                                          </FIELDCOLLECTION>";
                    //Get file information for given FILEID
                    obj_OMError = GetFileInfo(ul_FileId, str_FieldCollectionXml, ref str_FileDetailsXml);

                    if (obj_OMError != null)
                    {
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    //If file details are not found, return an error
                    if (String.IsNullOrEmpty(str_FileDetailsXml))
                    {
                        obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                    "Invalid File Id",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", OMErrorType.ERROR,
                                                    null, null);
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    xmlDoc_FileDetails = new XmlDocument();
                    xmlDoc_FileDetails.LoadXml(str_FileDetailsXml);
                    xmlNode_FileDetails = xmlDoc_FileDetails.SelectSingleNode("FILE");

                    //Create OBJECTID node (if it is not returned in file details XML) and assign Object Id of the new instance to be created
                    xmlNode_ObjectId = xmlNode_FileDetails.SelectSingleNode("OBJECTID");
                    if (xmlNode_ObjectId == null)
                    {
                        xmlNode_ObjectId = xmlDoc_FileDetails.CreateElement("OBJECTID");
                        xmlNode_FileDetails.AppendChild(xmlNode_ObjectId);
                    }
                    xmlNode_ObjectId.InnerText = ulObjectId.ToString();

                    //Update OBJECTTYPE since the existing file can belong to ENTITY/FACET and 
                    xmlNode_ObjectType = xmlNode_FileDetails.SelectSingleNode("OBJECTTYPE");
                    if (xmlNode_ObjectType == null)
                    {
                        xmlNode_ObjectType = xmlDoc_FileDetails.CreateElement("OBJECTTYPE");
                        xmlNode_FileDetails.AppendChild(xmlNode_ObjectType);
                    }
                    xmlNode_ObjectType.InnerText = strObjectType;

                    //Add FIELDNAME node to Xml (which is not available in ALFS_FILE table)
                    xmlNode_FieldNameFromInput = xmlNode_File.SelectSingleNode("FIELDNAME");
                    if (xmlNode_FieldNameFromInput != null)
                    {
                        xmlNode_FieldName = xmlDoc_FileDetails.CreateElement("FIELDNAME");
                        xmlNode_FieldName.InnerText = xmlNode_FieldNameFromInput.InnerText;
                        xmlNode_FileDetails.AppendChild(xmlNode_FieldName);
                    }
                    else
                    {
                        obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                    "Invalid Field Name",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", OMErrorType.ERROR,
                                                    null, null);
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    //Change SCHEMAID as per the target object field
                    obj_OMError = GetSchemaID(ulObjectTypeId, xmlNode_File.SelectSingleNode("FIELDNAME").InnerText, ref ul_SchemaId);
                    if (obj_OMError != null)
                    {
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    if (ul_SchemaId == 0)
                    {
                        obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                    "Invalid Field Name",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", OMErrorType.ERROR,
                                                    null, null);
                        obj_OMError.TransactionId = TransactionID.ToString();
                        return obj_OMError;
                    }

                    xmlNode_SchemaId = xmlNode_FileDetails.SelectSingleNode("SCHEMAID");
                    if (xmlNode_SchemaId == null)
                    {
                        xmlNode_SchemaId = xmlDoc_FileDetails.CreateElement("SCHEMAID");
                        xmlNode_FileDetails.AppendChild(xmlNode_SchemaId);
                    }
                    xmlNode_SchemaId.InnerText = ul_SchemaId.ToString();

                    //Since file details have been fetched and modified for copy operation, copy the details to FileNode
                    xmlNode_File.InnerXml = xmlNode_FileDetails.InnerXml;

                    bIsCopyFileOperation = true;
                }
                if (obj_OMError != null)
                    obj_OMError.TransactionId = TransactionID.ToString();
                return obj_OMError;
            }
            finally
            {
                #region Cleanup

                xmlNode_FileId = null;
                xmlNode_ObjectId = null;
                xmlNode_FieldNameFromInput = null;
                xmlNode_SchemaId = null;
                xmlNode_FieldName = null;
                xmlNode_FileDetails = null;
                str_FileDetailsXml = null;
                xmlDoc_FileDetails = null;

                #endregion Cleanup
            }

            #endregion Get file information for given FILEID and update FileNode for copy operation
        }

        /// <summary>
        /// Fetches file details of existing file and updates File Node for move operation.
        /// </summary>
        /// <param name="xmlNode_File">
        /// File node specifying file details specified in input Xml. 
        /// Existing file details will be fetched, updated for move operation and copied to File Node.
        /// </param>
        /// <param name="ulFileId">Output parameter, returns Id of the file to be moved</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError GetExistingFileInfoForMove(ref XmlNode xmlNode_File, out ulong ulFileId)
        {
            #region Variables

            ulong ul_FileId = 0;        //Stores FILEID of the file to be moved
            OMError obj_OMError = null;     //Used to prepare OM Error if an error or exception occurs
            OMError obj_ReturnOMError = null;     //Used to prepare an OM Error from OMError object
            bool b_ConversionResult = false;    //Used to identify whether FILEID could be parsed and converted to ulong
            String str_FileDetailsXml = null;     //Used to store FILE details (for given FILEID)
            XmlNode xmlNode_FileId = null;     //Used to store FILEID

            XPathDocument xpDoc_FileDetails = null;     //Used to load file details
            XPathNavigator xpNav_File = null;     //Used to read file node details

            #endregion Variables

            #region Get file information for given FILEID and update FileNode for move operation

            try
            {
                #region Implementation Cases
                /* There can be following cases
                 * 1. Operation=MOVE specified - Move file operation to be performed
                 * 2. Operation attribute not specified, FILEID is specified with MOVEFILES=TRUE - Move file operation is to be performed
                 * 3. Operation attribute not specified, FILEID is specified with MOVEFILES=FALSE - Create file operation is to be performed
                 * 4. Operation=CREATE specified - Create file operation is to be performed
                 * 5. Operation=CREATE specified, FILEID is specified with MOVEFILES=TRUE - Create file operation is to be performed
                 * 6. Operation=CREATE specified, FILEID is specified with MOVEFILES=FALSE - Create file operation is to be performed
                 * */
                #endregion Implementation Cases

                ulFileId = 0;
                xmlNode_FileId = xmlNode_File.SelectSingleNode("FILEID");
                if (xmlNode_FileId == null)
                {
                    //Return error when Operation="MOVE" but FILEID is not specified
                    obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                            "Invalid File Id",
                                            OMErrorObjectType.FILESYSTEM,
                                            "FileSystemManager", OMErrorType.ERROR,
                                            null, null);
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;

                }

                b_ConversionResult = ulong.TryParse(xmlNode_FileId.InnerText, out ul_FileId);

                //Return error if there is an error parsing FILEID value
                if (!b_ConversionResult && ul_FileId == 0)
                {
                    obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Invalid File Id",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", OMErrorType.ERROR,
                                                 null, null);
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                //Get file information for given FILEID
                obj_OMError = GetFileInfo(ul_FileId, null, ref str_FileDetailsXml);
                if (obj_OMError != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "Failed to get file details",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                //If file details are not found, return an error
                if (String.IsNullOrEmpty(str_FileDetailsXml))
                {
                    obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "Invalid File Id",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                null, null);
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                ulFileId = ul_FileId;

                xpDoc_FileDetails = new XPathDocument(new StringReader(str_FileDetailsXml));
                xpNav_File = xpDoc_FileDetails.CreateNavigator().SelectSingleNode("FILE");

                //Copy File details from database to input Xml
                xmlNode_File.InnerXml = xpNav_File.InnerXml;
                if (obj_OMError != null)
                    obj_OMError.TransactionId = TransactionID.ToString();
                return obj_OMError;
            }
            finally
            {
                #region Cleanup

                xmlNode_FileId = null;
                str_FileDetailsXml = null;
                xpNav_File = null;
                xpDoc_FileDetails = null;

                #endregion Cleanup
            }

            #endregion Get file information for given FILEID and update FileNode for move operation

        }

        private OMError GetBinderFileInfo(string FileName, ulong SchemaID, ulong ObjectID, ulong BinderID, ulong SequenceOrder, ref ulong FileID, ref string SourcePath)
        {
            OMError ErrObj = null;
            string FileXML = null;
            XmlDocument FileDoc = null;
            XmlNode FileIDNode = null;
            XmlNode SourcePathNode = null;
            StringBuilder SelectQuery = null;


            SelectQuery = new StringBuilder();
            //SelectQuery.Append("select ALFS_FILE.FILEID,PATH from ALFS_FILE,ALFS_ASSOFILEBINDER ");
            //SelectQuery.AppendFormat(" where ALFS_FILE.OBJECTID = {0} and ALFS_FILE.SCHEMAID ={1} and ALFS_FILE.NAME ='{2}'", ObjectID, SchemaID, FileName);
            //SelectQuery.AppendFormat(" and ALFS_ASSOFILEBINDER.BINDERID = {0} and ALFS_ASSOFILEBINDER.SEQUENCEORDER = {1}", BinderID, SequenceOrder);

            // Commented because above query was not giving expected result. As it does a cartesian product 
            // of ALFS_FILE and ALFS_ASSOFILEBINDER table result was not as expected.

            FileName = ReplaceSingleQuote(FileName);
            SelectQuery.AppendFormat(@"  SELECT ALFS_FILE_ALIAS.FILEID,ALFS_FILE_ALIAS.PATH 
                                        FROM ALFS_FILE ALFS_FILE_ALIAS INNER JOIN ALFS_ASSOFILEBINDER ALFS_ASSOFILEBINDER_ALIAS 
                                        ON ALFS_FILE_ALIAS.FILEID = ALFS_ASSOFILEBINDER_ALIAS.FILEID 
                                        WHERE ALFS_FILE_ALIAS.OBJECTID = {0} AND  ALFS_FILE_ALIAS.SCHEMAID ={1}
                                        AND ALFS_FILE_ALIAS.NAME =N'{2}' AND ALFS_ASSOFILEBINDER_ALIAS.BINDERID = {3}
                                        AND ALFS_ASSOFILEBINDER_ALIAS.SEQUENCEORDER = {4}", ObjectID, SchemaID, FileName, BinderID, SequenceOrder);



            if ((ErrObj = FILEDMObj.DataProvider.ExecuteQuery(SelectQuery.ToString(), "FILE", ref FileXML)) == null)
            {
                FileDoc = new XmlDocument();
                FileDoc.LoadXml(FileXML);
                FileIDNode = FileDoc.SelectSingleNode("//FILEID");
                SourcePathNode = FileDoc.SelectSingleNode("//PATH");
                FileID = Convert.ToUInt64(FileIDNode.InnerText);
                SourcePath = SourcePathNode.InnerText;
            }
            return null;
        }

        /// <summary>
        /// Special characters replacement
        /// </summary>
        /// <param name="FieldValue">InputValue</param>
        /// <returns>Returns a string in which single quotes are replaced with two single quotes</returns>
        public String ReplaceSingleQuote(string FieldValue)
        {
            FieldValue = FieldValue.Replace("'", "''");
            return FieldValue;
        }

        private OMError GetBinderID(ulong ObjectID, ulong SchemaID, ref ulong BinderID)
        {
            OMError ErrObj = null;
            string BinderXML = null;
            XmlDocument BinderDoc = null;
            XmlNode BinderIDNode = null;

            if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
            {
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("BINDER", "ALFS_BINDER", null, ref BinderXML)) == null)
                    {
                        BinderDoc = new XmlDocument();
                        BinderDoc.LoadXml(BinderXML);
                        BinderIDNode = BinderDoc.SelectSingleNode("//BINDERID");
                        BinderID = Convert.ToUInt64(BinderIDNode.InnerText);
                    }
                }
            }
            return null;
        }

        private OMError ASyncronousFileTransfer(string SourcePath, string DestinationPath, IFileSystem IFileSystemObj)
        {
            StringCollection ParameterCollection = null;
            Hashtable ThreadParameters = null;
            object Parameters = null;
            try
            {
                ThreadParameters = new Hashtable();
                ParameterCollection = new StringCollection();
                ParameterCollection.Add(SourcePath);
                ParameterCollection.Add(DestinationPath);

                Parameters = (Object)ParameterCollection;
                Thread AsyncronousThread = new Thread(new ParameterizedThreadStart(FileCopyThread));

                #region Preparing thread Parameters
                ThreadParameters.Add("IFileSystemObj", IFileSystemObj);
                ThreadParameters.Add("FileOperationParameters", Parameters);
                #endregion
                AsyncronousThread.Start(ThreadParameters);
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private void FileCopyThread(object ParameterObj)
        {
            OMError ErrorObj = null;
            Hashtable ThreadParameters = null;
            StringCollection Parameters = null;
            IFileSystem IFileSystemObj = null;
            Parameters = new StringCollection();

            try
            {
                ThreadParameters = (Hashtable)ParameterObj;
                Parameters = (StringCollection)ThreadParameters["FileOperationParameters"];
                IFileSystemObj = (IFileSystem)ThreadParameters["IFileSystemObj"];

                if ((ErrorObj = PerformOperation("CopyFile", Parameters, IFileSystemObj)) == null)
                {
                    m_delFileTransfer(true, Parameters[1]);
                }
                else
                {
                    m_delFileTransfer(false, ErrorObj.Serialize());
                }
            }
            catch (Exception ExcepObj)
            {
                throw ExcepObj;
            }
        }

        /// <summary>
        /// This function deletes a File associated with Entity/Facet Instance
        /// </summary>
        /// <param name="ObjectID">Entity/Facet ID</param>
        /// <returns></returns>

        public OMError DeleteFileSystem(ulong ObjectID)
        {
            OMError ErrorObj = null;
            string FileCollectionXML = null;
            XmlDocument FileCollectionXMLDoc = null;
            XmlNodeList FileNodes = null;
            string FileID = null;
            string Path = null;
            StringBuilder FilePath = null;
            StringCollection Parameters = null;
            string InsertQuery = null;
            StringBuilder BatchQuery = null;
            string Seprator = null;
            string ParameterXML = null;
            StringBuilder FileIDs = null;
            string IDSeprator = null;
            string DurableLogXML = null;
            string DeleteQuery = null;

            try
            {
                FileCollectionXMLDoc = new XmlDocument();
                FilePath = new StringBuilder();
                Parameters = new StringCollection();
                BatchQuery = new StringBuilder();
                FileIDs = new StringBuilder();

                if ((ErrorObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.ReadAllRecord("FILE", "ALFS_FOLDER", null, ref FileCollectionXML)) == null)
                    {
                        if (FileCollectionXML != null)
                        {
                            FileCollectionXMLDoc.LoadXml(FileCollectionXML);
                            FileNodes = FileCollectionXMLDoc.SelectNodes("//FILE");

                            FileIDs.Append("{");
                            foreach (XmlNode FileNode in FileNodes)
                            {
                                Parameters.Clear();
                                InsertQuery = null;
                                FilePath.Remove(0, FilePath.Length);
                                ParameterXML = null;

                                FileID = FileNode.SelectSingleNode("//FILEID").InnerText;
                                Path = FileNode.SelectSingleNode("//PATH").InnerText;
                                Path = Path.Substring(0, Path.LastIndexOf('\\'));
                                FileIDs.AppendFormat("{0}{1}", IDSeprator, FileID);
                                FilePath.AppendFormat("{0}\\{1}", Path, FileID);
                                Parameters.Add(FilePath.ToString());
                                FormPamareterXML(Parameters, ref ParameterXML);
                                // Log into DurableLog file
                                FormDurableLogXML(TransactionID, "DeleteFile", GetFileSystemInterface(FileNode), ParameterXML, null, ref DurableLogXML);
                                if ((ErrorObj = LOGDMObj.DataProvider.PrepareInsertQuery("ALFS_DURABLELOG", DurableLogXML, ref InsertQuery)) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                                BatchQuery.AppendFormat("{0}{1}", Seprator, InsertQuery);
                                Seprator = ";";
                                IDSeprator = ",";
                            }
                            FileIDs.Append("}");
                            if (BatchQuery.Length > 0)
                            {
                                if ((ErrorObj = LOGDMObj.DataProvider.Execute(BatchQuery.ToString())) != null)
                                {
                                    ErrorObj.TransactionId = TransactionID.ToString();
                                    return ErrorObj;
                                }
                            }

                        }
                    }
                }
                BatchQuery.Remove(0, BatchQuery.Length);
                Seprator = null;
                if (FileIDs.Length > 2)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = FILEDMObj.DataProvider.PrepareDeleteQuery("ALFS_FILE", ref DeleteQuery)) == null)
                        {
                            BatchQuery.AppendFormat("{0}{1}", Seprator, DeleteQuery);
                            DeleteQuery = null;
                            Seprator = ";";
                        }
                    }

                    if ((ErrorObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.IN, FileIDs.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrorObj = FILEDMObj.DataProvider.PrepareDeleteQuery("ALFS_ASSOFILEBINDER", ref DeleteQuery)) == null)
                        {
                            BatchQuery.AppendFormat("{0}{1}", Seprator, DeleteQuery);
                            DeleteQuery = null;
                            Seprator = ";";
                        }
                    }
                }

                if ((ErrorObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.PrepareDeleteQuery("ALFS_FOLDER", ref DeleteQuery)) == null)
                    {
                        BatchQuery.AppendFormat("{0}{1}", Seprator, DeleteQuery);
                        DeleteQuery = null;
                        Seprator = ";";
                    }
                }

                if (BatchQuery.Length > 0)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.Execute(BatchQuery.ToString())) != null)
                    {
                        ErrorObj.TransactionId = TransactionID.ToString();
                        return ErrorObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }

        }

        private OMError ProcessBinders(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNodeList BinderNodes)
        {
            string Operation = null;
            OMError ErrObj = null;
            try
            {
                foreach (XmlNode BinderNode in BinderNodes)
                {
                    Operation = null;

                    //Get Operation attribute value if specified
                    if (BinderNode.Attributes.GetNamedItem("Operation") != null)
                    {
                        Operation = BinderNode.Attributes.GetNamedItem("Operation").Value;
                    }

                    //If Operation attribute is not specified, set default value as CREATE
                    if (String.IsNullOrEmpty(Operation))
                        Operation = "CREATE";

                    switch (Operation.ToUpper())
                    {
                        case "DELETE":
                            if ((ErrObj = DeleteBinder(ObjectID, ObjectType, ObjectTypeID, BinderNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;
                        case "CREATE":
                        default:
                            if ((ErrObj = CreateBinder(ObjectID, ObjectType, ObjectTypeID, BinderNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        case "UPDATE":
                            if ((ErrObj = UpdateBinder(ObjectID, ObjectType, ObjectTypeID, BinderNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;
                    }


                }
                return null;
            }
            catch (Exception)
            {

                throw;
            }
        }

        private OMError ProcessFiles(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNodeList FileNodes)
        {
            OMError ErrObj = null;
            string Operation = null;
            ulong FileID = 0;

            try
            {
                foreach (XmlNode FileNode in FileNodes)
                {
                    Operation = null;

                    //Identify operation to be performed
                    ErrObj = GetFileOperation(FileNode, out Operation);
                    if (ErrObj != null)
                        return ErrObj;

                    switch (Operation.ToUpper())
                    {
                        case "COPY":
                        case "CREATE":
                        default:

                            UniqueID.Read(ref FileID);
                            if ((ErrObj = CreateFile(FileID, ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        case "UPDATE":
                            if ((ErrObj = UpdateFile(ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        case "DELETE":
                            if ((ErrObj = DeleteFile(ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        case "MOVE":
                            if ((ErrObj = MoveFile(ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;
                    }
                }
                return null;
            }
            catch (Exception)
            {

                throw;
            }
        }

        /// <summary>
        /// Creates file on mount point.
        /// </summary>
        /// <param name="FileID"></param>
        /// <param name="ObjectID"></param>
        /// <param name="ObjectType"></param>
        /// <param name="ObjectTypeID"></param>
        /// <param name="FileNode"></param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError CreateFile(ulong FileID, ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string DestinationPath = null;
            StringBuilder DestinationFilePath = null;
            StringBuilder SourcePath = null;
            string ParameterXML = null;
            String str_SourceFolderPath = null;     //Specifies folder path of existing file
            StringCollection Parameters = null;
            StringBuilder InsertQuery = null;
            string DurableLogXML = null;
            string FileName = null;
            ulong SchemaID = 0;
            string Description = null;
            string FileXML = null;
            string FieldName = null;
            string FileDisplayName = null;
            string FileSize = null;
            bool b_IsFileOnMediaServer = false;    //Specifies whether file resides on Media Server (Used for Copy operation)
            bool b_IsDestinationFolderOnMediaServer = false;    //Specifies whether file is to be copied to Media Server (Used for Copy operation)
            bool b_IsCopyFileOperation = false;    //Specifies whether operation is Copy file
            FileSystemInterface FileSystemInterfaceObj = null;

            try
            {
                Parameters = new StringCollection();
                InsertQuery = new StringBuilder();
                SourcePath = new StringBuilder();
                DestinationFilePath = new StringBuilder();

                //Check whether a copy of existing file is to be created for new instance. This function will get exising file details.
                ErrObj = GetExistingFileInfoForCopy(ObjectID, ObjectType, ObjectTypeID, ref FileNode, ref b_IsCopyFileOperation);
                if (ErrObj != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }

                #region Identify whether operation is CopyFile
                //File Copy operation
                if (b_IsCopyFileOperation)
                {
                    //Read ONMEDIASERVER node value for file
                    b_IsFileOnMediaServer = CheckForMediaServer(FileNode);

                    //Read OnMediaServer attribute value for destination folder
                    if (FileNode.Attributes.GetNamedItem("OnMediaServer") != null)
                    {
                        Boolean.TryParse(FileNode.Attributes.GetNamedItem("OnMediaServer").Value, out b_IsDestinationFolderOnMediaServer);
                    }
                    else
                    {
                        /* If ONMEDIASERVER attribute is not specified to FILE node, that means, the file is to be copied 
                         * within Media Server mount point or OM mount point (according to the location where file resides)
                         */
                        b_IsDestinationFolderOnMediaServer = b_IsFileOnMediaServer;
                    }

                    #region Important Comment - Copy file Cases
                    /* There can be following cases
                    * |-------------------------------------------------------------------------------------------------------------------------|
                    * |b_IsFileOnMediaServer    |b_IsDestinationFolderOnMediaServer     |Operation to be performed                              |  
                    * |-------------------------------------------------------------------------------------------------------------------------|
                    * |TRUE                     |TRUE                                   |File copy operation within media server mount point.   |
                    * |TRUE                     |FALSE                                  |File Pull operation.                                   |
                    * |FALSE                    |TRUE                                   |File Push operation.                                   |
                    * |FALSE                    |FALSE                                  |File copy operation within OM mount point.             |
                    * |-------------------------------------------------------------------------------------------------------------------------|
                    * */
                    #endregion Important Comment - Copy file Cases

                    //Consider a case where both file and destination folder are on different machine (OM Mount Point and Media Server Mount Point)
                    if (b_IsFileOnMediaServer != b_IsDestinationFolderOnMediaServer)
                    {
                        if (b_IsDestinationFolderOnMediaServer)
                        {
                            //File Push operation is required.
                            ErrObj = PerformMediaServerOperation(MediaServerOperationType.PUSHFILETOMEDIASERVER, FileOperations.COPY, FileID, ObjectID, ObjectTypeID, ref FileNode);
                        }
                        else
                        {
                            //File Pull operation is required.
                            ErrObj = PerformMediaServerOperation(MediaServerOperationType.PULLFILEFROMMEDIASERVER, FileOperations.COPY, FileID, ObjectID, ObjectTypeID, ref FileNode);
                        }
                        if (ErrObj != null)
                            ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }
                    else
                    {
                        /* In this case a local copy is to be created for file (either on OM mount point or on Media Server mount point).
                         * Since this operation is copy operation, FILE node contains mount point folder path. In order to create a copy of file,
                         * the path node should have full path (including FILEID). So add the FILEID to PATH node.
                         */
                        str_SourceFolderPath = FileNode.SelectSingleNode("PATH").InnerText;
                        FileNode.SelectSingleNode("PATH").InnerText += "\\" + FileNode.SelectSingleNode("FILEID").InnerText;
                    }
                }

                #endregion Identify whether operation is CopyFile

                FileSystemInterfaceObj = GetFileSystemInterface(FileNode);
                SourcePath.AppendFormat("{0}", FileNode.SelectSingleNode("PATH").InnerText);

                #region Comments for MountPoint to use for file operation
                /* MountPoint attribute can be specified at FILESYSTEM and FILE nodes. Following table specifies which MountPoint attributes 
                 * will be used for processing.
                 * --------------------------------------------------------------------------------------------------------------------------   
                 * |Mount Point @FILESYSTEM node|Mount Point @FILE node  |MountPoint attribute to use                                       |
                 * |------------------------------------------------------------------------------------------------------------------------|  
                 * |YES                         |YES                     |MountPoint @FILE node will be used (override).                    |
                 * |YES                         |NO                      |MountPoint @FILESYSETM node will be used.                         |
                 * |NO                          |YES                     |MountPoint @FILE node will be used.                               |
                 * |NO                          |NO                      |MountPoint configured for Tenant will be used.                    |   
                 * --------------------------------------------------------------------------------------------------------------------------  
                 * */
                #endregion Comments for MountPoint to use for file operation

                //Check whether MountPoint attribute is specified @FILE node
                if (FileNode.Attributes.GetNamedItem("MountPoint") != null)
                    UserProvidedMountPoint = FileNode.Attributes.GetNamedItem("MountPoint").Value;
                //If MountPoint attribute is not specified @FILE node, use MountPoint attribute specified @FILESYSTEM node. 
                if (String.IsNullOrEmpty(UserProvidedMountPoint))
                    UserProvidedMountPoint = m_UserMountPointForFileSystem;

                if ((ErrObj = GetFolderPath(ref DestinationPath, FileSystemInterfaceObj)) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                 "Getting destination folder path Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", ErrObj.ErrorType,
                                                 ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }

                //Commented By: Sagar Dasalkar Date: 14 Oct 09 
                //File can be copied within same location (using different fileid). So the following code is commented
                ////In case of copy file operation, source and destination path should be different
                //if (b_IsCopyFileOperation)
                //{
                //    //Return error if source and destination path are same
                //    if (DestinationPath.ToUpper().CompareTo(str_SourceFolderPath.ToUpper()) == 0)
                //    {
                //        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                //                                     "Destination path must be different from source path for copy file operation.",
                //                                     OMErrorObjectType.FILESYSTEM,
                //                                     "FileSystemManager", OMErrorType.ERROR,
                //                                     null, null);
                //        return ReturnErrObj;
                //    }
                //}

                if (string.IsNullOrEmpty(DestinationPath))
                {
                    if (FileNode.Attributes.GetNamedItem("OnMediaServer") != null)
                    {
                        bool b_IsMediaServer = false;
                        Boolean.TryParse(FileNode.Attributes.GetNamedItem("OnMediaServer").Value, out b_IsMediaServer);
                        if (b_IsMediaServer)
                        {
                            DestinationPath = ALCommon.GetInstallationDir() + SYSYTEM_MOUNT_POINT;
                            if (!FileSystemInterfaceObj.IFileSystemInstance.IsFolderExists(DestinationPath))
                                FileSystemInterfaceObj.IFileSystemInstance.CreateFolder(DestinationPath);
                        }
                    }
                    else if (FileNode.SelectSingleNode("ONMEDIASERVER") != null && FileNode.SelectSingleNode("ONMEDIASERVER").InnerText.ToUpper() == "TRUE")
                    {
                        DestinationPath = ALCommon.GetInstallationDir() + SYSYTEM_MOUNT_POINT;
                        if (!FileSystemInterfaceObj.IFileSystemInstance.IsFolderExists(DestinationPath))
                            FileSystemInterfaceObj.IFileSystemInstance.CreateFolder(DestinationPath);
                    }
                    else
                    {
                        if (!string.IsNullOrEmpty(MountPoint))
                        {
                            DestinationPath = MountPoint;
                        }
                        //else
                        //{
                        //    WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""CreateFile()"",""MountPoint is NULL OR EMPTY. Not changing the value of DestinationPath""");
                        //}
                    }
                }
                DestinationFilePath.AppendFormat("{0}\\{1}", DestinationPath, FileID);

                // Put File Entry in ALFS_FILE
                FieldName = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                FileName = FileNode.SelectSingleNode("NAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);

                if (FileNode.SelectNodes("DISPLAYNAME").Count > 0)
                {
                    FileDisplayName = FileNode.SelectSingleNode("DISPLAYNAME").InnerText;
                }

                if (FileNode.SelectNodes("DESCRIPTION").Count > 0)
                {
                    Description = FileNode.SelectSingleNode("DESCRIPTION").InnerText;
                }

                if (FileNode.SelectNodes("FILESIZE").Count > 0)
                {
                    FileSize = FileNode.SelectSingleNode("FILESIZE").InnerText;
                }

                FormFileXMLForCreate(FileID, ObjectID, ObjectType, SchemaID.ToString(), FileName, FileDisplayName, Description, DestinationPath, UserID, FileSystemInterfaceObj, FileSize, ref FileXML);
                if ((ErrObj = FILEDMObj.DataProvider.CreateRecord("ALFS_FILE", FileXML)) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                 "Writing into ALFS_FILE Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", ErrObj.ErrorType,
                                                 ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }


                #region For introducing alerts in the ALENT_SEARCHABLE_FILE_ALERT table for document search

                ErrObj = AddSearchableTextForFile(FileID, ObjectID, ObjectType, ObjectTypeID, FileName, SchemaID, FieldName, FileSystemInterfaceObj);
                if (ErrObj != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }

                #endregion
                if ((ErrObj = FileSystemInterfaceObj.IFileSystemInstance.CreateFile(SourcePath.ToString(), DestinationFilePath.ToString())) != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }

                Parameters.Add(SourcePath.ToString());
                Parameters.Add(DestinationFilePath.ToString());
                FormPamareterXML(Parameters, ref ParameterXML);

                // Log into DurableLog file
                FormDurableLogXML(TransactionID, (b_IsCopyFileOperation ? "CopyFile" : "CreateFile"), FileSystemInterfaceObj, ParameterXML, "PREPARED", ref DurableLogXML);
                if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                 "Writing into ALFS_DURABLELOG Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", ErrObj.ErrorType,
                                                 ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                   "Exception thrown in CreateFile",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", OMErrorType.EXCEPTION,
                                                   ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }



        }

        /// <summary>
        /// <purpose>
        /// Inserts an entry in ALENT_SEARCHABLE_FILE_ALERT table for file (only for supported file extensions).
        /// </purpose>
        /// <CreatedDate>6 November 2009</CreatedDate>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// </summary>
        /// <param name="FileID">File Id for which entry is to be added in searchable file table.</param>
        /// <param name="ObjectID">Object Id (instance id).</param>
        /// <param name="ObjectType">Object Type (ENTITY/FACET).</param>
        /// <param name="ObjectTypeID">Object Type id.</param>
        /// <param name="FileName">Name of the file.</param>
        /// <param name="SchemaID">Schema Id of file field.</param>
        /// <param name="FieldName">Field Name.</param>
        /// <param name="FileSystemInterfaceObj">Interface to File System.</param>
        /// <returns>OMError if any error occurs, otherwise null</returns>
        private OMError AddSearchableTextForFile(ulong FileID, ulong ObjectID, string ObjectType, ulong ObjectTypeID,
            string FileName, ulong SchemaID, string FieldName, FileSystemInterface FileSystemInterfaceObj)
        {
            string AlertXML = null;     //Used to prepare input xml for inserting an entry in ALENT_SEARCHABLE_FILE_ALERT
            string ObjectName = null;     //Used to Object Name
            bool IsFileSearchable = false;    //Used to identify whether the file field is searchable
            string FileExtension = null;     //Used to store file extension
            string AssetlinkXMLPath = null;     //Used to prepare path to Assetlink.xml
            XmlDocument AssetlinkXML = null;     //Used to load Assetlink.xml
            XmlNodeList ExtensionsList = null;     //Used to store extension list (from Assetlink.xml)
            bool FileTypeSupported = false;    //Used to identify whether file (extension) is supported as searchable
            OMError ErrObj = null;     //Used to prepare errors
            OMError ReturnErrObj = null;     //Used to prepare errors

            try
            {
                AssetlinkXMLPath = ALCommon.GetConfigurationXMLPath() + "Assetlink.xml";
                FileExtension = FileName.Substring(FileName.LastIndexOf(".") + 1).ToUpper();
                AssetlinkXML = new XmlDocument();
                AssetlinkXML.Load(AssetlinkXMLPath);
                ExtensionsList = AssetlinkXML.SelectNodes("Assetlink/TEXTCATALOGER/TEXTPROCESSORS/TEXTPROCESSOR/EXTENSION");
                if (ExtensionsList.Count != 0)
                {
                    foreach (XmlNode Extension in ExtensionsList)
                    {
                        string[] ProcessableExts = Extension.InnerText.Split(',');
                        foreach (string ProcessableExt in ProcessableExts)
                        {
                            if (String.Compare(FileExtension, ProcessableExt) == 0)
                            {
                                FileTypeSupported = true;
                                break;
                            }
                        }

                        if (FileTypeSupported)
                            break;
                    }
                }

                if (FileTypeSupported)
                {
                    ErrObj = GetFileSearchability(ObjectTypeID, FieldName, ref IsFileSearchable);
                    if (ErrObj != null)
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }

                    if (IsFileSearchable)
                    {
                        ErrObj = GetObjectName(ObjectTypeID, ObjectType, ref ObjectName);
                        if (ErrObj != null)
                        {
                            ErrObj.TransactionId = TransactionID.ToString();
                            return ErrObj;
                        }

                        ErrObj = FormSearchableFileAlert(FileID, ObjectID, ObjectType, ObjectName, "NEW",
                                                                     SchemaID.ToString(), FieldName, FileName, FileSystemInterfaceObj, ref AlertXML);
                        if (ErrObj != null)
                        {
                            ErrObj.TransactionId = TransactionID.ToString();
                            return ErrObj;
                        }

                        if ((ErrObj = FILEDMObj.DataProvider.CreateRecord("ALENT_SEARCHABLE_FILE_ALERT", AlertXML)) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                         "Writing into ALENT_SEARCHABLE_FILE_ALERT Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", ErrObj.ErrorType,
                                                         ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                         "Writing into ALENT_SEARCHABLE_FILE_ALERT Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", OMErrorType.EXCEPTION,
                                                         ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }

            return null;
        }


        /// <summary>
        /// <purpose>
        /// Updates an entry in ALENT_SEARCHABLE_FILE_ALERT table for file.
        /// </purpose>
        /// <CreatedDate>6 November 2009</CreatedDate>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// </summary>
        /// <param name="FileID">File Id for which entry is to be updated.</param>
        /// <param name="FileSystemInterfaceObj">Interface to File System.</param>
        /// <returns>OMError if any error occurs, otherwise null</returns>
        private OMError UpdateSearchableTextForFile(ulong FileID, FileSystemInterface FileSystemInterfaceObj)
        {
            String str_AlertXml = null;     //Used to prepare input xml for updating entry in ALENT_SEARCHABLE_FILE_ALERT
            OMError obj_OMError = null;     //Used to preapare error
            OMError obj_ReturnError = null;     //Used to preapare error

            XPathDocument xpDoc_AlertDetails = null;     //Used to load existing alert details for the file
            XPathNavigator xpNav_SearchableTextRoot = null;     //Used to navigate through alert details document

            try
            {
                //Update an entry only if it is already present
                if ((obj_OMError = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    obj_OMError = FILEDMObj.DataProvider.ReadRecord("SEARCHABLE_FILE_ALERT", "ALENT_SEARCHABLE_FILE_ALERT", null, ref str_AlertXml);
                }
                if (obj_OMError != null)
                {
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                if (String.IsNullOrEmpty(str_AlertXml))
                    return null;

                xpDoc_AlertDetails = new XPathDocument(new StringReader(str_AlertXml));
                xpNav_SearchableTextRoot = xpDoc_AlertDetails.CreateNavigator().SelectSingleNode("SEARCHABLE_FILE_ALERT");
                if (xpNav_SearchableTextRoot == null)
                    return null;

                str_AlertXml = null;

                obj_OMError = FormSearchableFileAlertForUpdate(FileID, FileSystemInterfaceObj, ref str_AlertXml);
                if (obj_OMError != null)
                {
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                if ((obj_OMError = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((obj_OMError = FILEDMObj.DataProvider.UpdateRecord("ALENT_SEARCHABLE_FILE_ALERT", str_AlertXml)) != null)
                    {
                        obj_ReturnError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                     "Updating searchable alert into ALENT_SEARCHABLE_FILE_ALERT Failed",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", obj_OMError.ErrorType,
                                                     obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                        obj_ReturnError.TransactionId = TransactionID.ToString();
                        return obj_ReturnError;
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                obj_ReturnError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                         "Updating searchable alert into ALENT_SEARCHABLE_FILE_ALERT Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", OMErrorType.EXCEPTION,
                                                         ExceptionObj, null);
                obj_ReturnError.TransactionId = TransactionID.ToString();
                return obj_ReturnError;
            }

            if (obj_OMError != null)
                obj_OMError.TransactionId = TransactionID.ToString();
            return obj_OMError;
        }

        /// <summary>
        /// <CreatedOn>11 September 2009</CreatedOn>
        /// <Author>Mr. Sagar Dasalkar</Author>
        /// <purpose>
        /// Moves file to specified mount point.
        /// </purpose>
        /// </summary>
        /// <param name="ObjectID">Object Id to which the file to be moved belongs.</param>
        /// <param name="ObjectType">Object Type.</param>
        /// <param name="ObjectTypeID">Object Type Id.</param>
        /// <param name="FileNode">File Node having file details for file move operation.</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError MoveFile(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            #region Variables

            OMError obj_OMError = null;     //Used to prepare OM Error
            OMError obj_ReturnOMError = null;     //Used to prepare error using previous error (OMError object)

            FileSystemInterface obj_FileSystemInterface = null;     //Used to get File System Interfacet for file operations

            String str_DestinationPath = null;     //Used to prepare destination path for Move file operation
            String str_ParameterXml = null;     //Used to prepare parameter Xml for creating durable log entry
            String str_DurableLogXml = null;     //Used to prepare Xml for creating durable log entry

            StringBuilder sb_DestinationFilePath = null;     //Used to prepare destination path (including FILEID)
            StringBuilder sb_SourceFilePath = null;     //Used to prepare source file path (including FILEID)
            StringBuilder sb_FileDetailsXmlForUpdate = null;     //Used to prepare Xml for Updating File details after move operation
            StringCollection strColl_Parameters = null;     //Used to maintain parameters, and later used to get prepare Xml

            bool b_IsFileOnMediaServer = false;    //Used to identify whether file resides on Media Server
            bool b_IsDestinationFolderOnMediaServer = false;    //Used to identify whether file is to moved on Media Server

            ulong ul_ObjectIdOfExistingFile = 0;        //Used to store Object Id of the file to be moved
            ulong ul_ExistingFileId = 0;        //Used to store FILEID of existing file

            #endregion Variables

            #region Move File

            try
            {
                strColl_Parameters = new StringCollection();
                sb_DestinationFilePath = new StringBuilder(100);
                sb_SourceFilePath = new StringBuilder(100);

                //Get file details for given FILEID
                obj_OMError = GetExistingFileInfoForMove(ref FileNode, out ul_ExistingFileId);
                if (obj_OMError != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "Failed to get file details",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();

                    return obj_ReturnOMError;
                }

                //Check whether file to be moved belongs to the same Object whose details are being updated
                ulong.TryParse(FileNode.SelectSingleNode("OBJECTID").InnerText, out ul_ObjectIdOfExistingFile);
                if (ObjectID != ul_ObjectIdOfExistingFile)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                            "File belonging to another instance cannot be moved.",
                            OMErrorObjectType.FILESYSTEM,
                            "FileSystemManager", OMErrorType.ERROR,
                            null, null);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();

                    return obj_ReturnOMError;
                }

                //Read ONMEDIASERVER node value for file
                b_IsFileOnMediaServer = CheckForMediaServer(FileNode);

                //Read OnMediaServer attribute value for destination folder
                if (FileNode.Attributes.GetNamedItem("OnMediaServer") != null)
                {
                    Boolean.TryParse(FileNode.Attributes.GetNamedItem("OnMediaServer").Value, out b_IsDestinationFolderOnMediaServer);
                }
                else
                {
                    /* If ONMEDIASERVER attribute is not specified to FILE node, that means, the file is to be copied 
                     * within Media Server mount point or OM mount point (according to the location where file resides)
                     */
                    b_IsDestinationFolderOnMediaServer = b_IsFileOnMediaServer;
                }

                #region Important Comment - Move file Cases
                /* There can be following cases
                * |-----------------------------------------------------------------------------------------------------------------|
                * |b_IsFileOnMediaServer    |b_IsDestinationFolderOnMediaServer         |Operation to be performed                  |  
                * |-----------------------------------------------------------------------------------------------------------------|
                * |TRUE                     |TRUE                                       |File move within media server mount point. |
                * |TRUE                     |FALSE                                      |File Pull operation.                       |
                * |FALSE                    |TRUE                                       |File Push operation.                       |
                * |FALSE                    |FALSE                                      |File move within OM mount point.           |
                * |-----------------------------------------------------------------------------------------------------------------|
                * */
                #endregion Important Comment - Move file Cases

                //Consider a case where both file and destination folder are on different machine (OM Mount Point and Media Server Mount Point)
                if (b_IsFileOnMediaServer != b_IsDestinationFolderOnMediaServer)
                {
                    if (b_IsDestinationFolderOnMediaServer)
                    {
                        //File Push operation is required.
                        obj_OMError = PerformMediaServerOperation(MediaServerOperationType.PUSHFILETOMEDIASERVER, FileOperations.MOVE, 0, ObjectID, ObjectTypeID, ref FileNode);
                    }
                    else
                    {
                        //File Pull operation is required.
                        obj_OMError = PerformMediaServerOperation(MediaServerOperationType.PULLFILEFROMMEDIASERVER, FileOperations.MOVE, 0, ObjectID, ObjectTypeID, ref FileNode);
                    }
                    if (obj_OMError != null)
                        obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                #region Identify MountPoint attribute to use for move file operation
                /* MountPoint attribute can be specified at FILESYSTEM and FILE nodes. Following table specifies which MountPoint attributes 
                 * will be used for processing.
                 * |------------------------------------------------------------------------------------------------------------------------|   
                 * |Mount Point @FILESYSTEM node|Mount Point @FILE node  |MountPoint to use                                                 |
                 * |------------------------------------------------------------------------------------------------------------------------|  
                 * |YES                         |YES                     |MountPoint @FILE node will be used.                               |
                 * |YES                         |NO                      |MountPoint @FILESYSETM node will be used.                         |
                 * |NO                          |YES                     |MountPoint @FILE node will be used.                               |
                 * |NO                          |NO                      |MountPoint configured for Tenant will be used.                    |   
                 * |------------------------------------------------------------------------------------------------------------------------|  
                 * */
                #endregion Identify MountPoint attribute to use for move file operation

                //Chech whether MountPoint attribute is specified @FILE node
                if (FileNode.Attributes.GetNamedItem("MountPoint") != null)
                    UserProvidedMountPoint = FileNode.Attributes.GetNamedItem("MountPoint").Value;
                //If MountPoint attribute is not specified @FILE node, use MountPoint attribute specified @FILESYSTEM node. 
                if (String.IsNullOrEmpty(UserProvidedMountPoint))
                    UserProvidedMountPoint = m_UserMountPointForFileSystem;

                //Get File System Interface
                obj_FileSystemInterface = GetFileSystemInterface(FileNode);

                if ((obj_OMError = GetFolderPath(ref str_DestinationPath, obj_FileSystemInterface)) != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Failed to get destination folder path",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                //Return error if source and destination path are same
                if (str_DestinationPath.ToUpper().CompareTo(FileNode.SelectSingleNode("PATH").InnerText.ToUpper()) == 0)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Destination path must be different from source path for move file operation.",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", OMErrorType.ERROR,
                                                 null, null);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                //Prepare source and destination path
                sb_DestinationFilePath.AppendFormat("{0}\\{1}", str_DestinationPath, ul_ExistingFileId);
                sb_SourceFilePath.AppendFormat("{0}\\{1}", FileNode.SelectSingleNode("PATH").InnerText, ul_ExistingFileId);

                strColl_Parameters.Add(sb_SourceFilePath.ToString());
                strColl_Parameters.Add(sb_DestinationFilePath.ToString());

                FormPamareterXML(strColl_Parameters, ref str_ParameterXml);

                //Create an entry in ALFS_DURABLELOG for MoveFile operation
                FormDurableLogXML(TransactionID, "MoveFile", obj_FileSystemInterface, str_ParameterXml, "PREPARED", ref str_DurableLogXml);
                if ((obj_OMError = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", str_DurableLogXml)) != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Writing into ALFS_DURABLELOG Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }
                //Prepare Xml to update file PATH (since file will be moved to another location)
                sb_FileDetailsXmlForUpdate = new StringBuilder(200);
                sb_FileDetailsXmlForUpdate.Append(@"<FILE>
                                                        <PATH>" + ALCommon.TrimAndReplaceSpecialChars(str_DestinationPath) + @"</PATH>
                                                    </FILE>");

                //Update ALFS_FILE record
                if ((obj_OMError = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ul_ExistingFileId.ToString(), OMConjunction.NOTHING)) == null)
                {
                    obj_OMError = FILEDMObj.DataProvider.UpdateRecord("ALFS_FILE", sb_FileDetailsXmlForUpdate.ToString());
                }
                if (obj_OMError != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                     "Failed to update file details",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", obj_OMError.ErrorType,
                                                     obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                //Since the file would be moved to another location, reduce the file count for previous folder
                obj_OMError = DecrementFolderCount(sb_SourceFilePath.ToString());
                if (obj_OMError != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Failed to update folder details",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                //Move File
                if ((obj_OMError = obj_FileSystemInterface.IFileSystemInstance.MoveFile(sb_SourceFilePath.ToString(), sb_DestinationFilePath.ToString())) != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "File move failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                   "Exception thrown in MoveFile",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", OMErrorType.EXCEPTION,
                                                   ExceptionObj, null);
                obj_ReturnOMError.TransactionId = TransactionID.ToString();
                return obj_ReturnOMError;
            }
            finally
            {
                #region Cleanup

                obj_FileSystemInterface = null;
                str_DestinationPath = null;
                str_ParameterXml = null;
                str_DurableLogXml = null;

                if (strColl_Parameters != null)
                {
                    strColl_Parameters.Clear();
                    strColl_Parameters = null;
                }

                if (sb_DestinationFilePath != null)
                {
                    sb_DestinationFilePath = sb_DestinationFilePath.Remove(0, sb_DestinationFilePath.Length);
                    sb_DestinationFilePath = null;
                }

                if (sb_SourceFilePath != null)
                {
                    sb_SourceFilePath = sb_SourceFilePath.Remove(0, sb_SourceFilePath.Length);
                    sb_SourceFilePath = null;
                }

                if (sb_FileDetailsXmlForUpdate != null)
                {
                    sb_FileDetailsXmlForUpdate = sb_FileDetailsXmlForUpdate.Remove(0, sb_FileDetailsXmlForUpdate.Length);
                    sb_FileDetailsXmlForUpdate = null;
                }

                #endregion Cleanup
            }

            #endregion Move File
        }

        /// <summary>
        /// Performs push or pull file operations across OM mount point and Media Server mount point.
        /// </summary>
        /// <param name="enumMoveFileOperations">Operation to be performed i.e. PUSH/PULL.</param>
        /// <param name="enumFileOperations">File operation i.e. COPY/MOVE.</param>
        /// <param name="ulNewFileId">File Id to be used to create a copy of existing file in COPY file operation. Specify 0 for MOVE file operation</param>
        /// <param name="ulObjectId">Object Id for which file operations are being performed.</param>
        /// <param name="ulObjectTypeId">Object type Id</param>
        /// <param name="FileNode">File Node with file details for COPY/MOVE file operation.</param>
        /// <returns>OMError if any error occurs, otherwise null.</returns>
        private OMError PerformMediaServerOperation(MediaServerOperationType enumMoveFileOperations, FileOperations enumFileOperations,
            ulong ulNewFileId, ulong ulObjectId, ulong ulObjectTypeId, ref XmlNode FileNode)
        {
            #region Variables

            XmlNode xmlNode_DestinationPath = null;     //Used to prepare destination path (with FILEID) for COPY/MOVE file operation

            String str_FileCollectionXml = null;     //Used to prepare file collection Xml for Push or Pull operation
            String str_ErrorXml = null;     //Used to prepare error Xml
            String str_DestinationPath = null;     //Used to prepare destination folder path (without FILEID)
            String str_ParameterXml = null;     //Used to prepare parameter Xml for creating durable log entry
            String str_DurableLogXml = null;     //Used to prepare Xml for creating durable log entry
            String str_FileXmlForCopy = null;     //Used to prepare file details Xml to create a record in ALFS_FILE table in case of COPY operation

            StringBuilder sb_FileDetailsXmlForUpdate = null;     //Used to prepare file details Xml to update file details
            StringCollection strColl_Parameters = null;     //Used to store parameters and preapare parameter Xml

            FileTransferInfrastructure obj_FileTransferInfrastructure = null;     //Used to perform file transfer
            FileSystemInterface obj_FileSystemInterface = null;     //Used to get store File System Interface

            OMError obj_OMError = null;     //Used to prepare OM Error object
            OMError obj_ReturnOMError = null;     //Used to prepare error from previous one (OMError object)

            ulong ul_ExistingFileId = 0;        //Used to store FILEID of existing file (which will be either copied or moved)
            String str_FileSize = null;     //Used to store the FILESIZE of the uploaded file in the database.

            #endregion Variables

            #region Push file to Media Server or Pull file from Media Server

            try
            {

                #region Get File System Interface, File Id (from FILE node), prepare Destination Folder path

                ulong.TryParse(FileNode.SelectSingleNode("FILEID").InnerText, out ul_ExistingFileId);
                if (ul_ExistingFileId == 0)
                {
                    obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "Invalid File Id",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                null, null);
                    obj_OMError.TransactionId = TransactionID.ToString();
                    return obj_OMError;
                }

                //Get File System Interface
                obj_FileSystemInterface = GetFileSystemInterface(FileNode);

                #region Identify MountPoint attribute to use
                /* MountPoint attribute can be specified at FILESYSTEM and FILE nodes. Following table specifies which MountPoint attributes 
                 * will be used for processing.
                 * --------------------------------------------------------------------------------------------------------------------------   
                 * |Mount Point @FILESYSTEM node|Mount Point @FILE node  |MountPoint attribute to use                                       |
                 * |------------------------------------------------------------------------------------------------------------------------|  
                 * |YES                         |YES                     |MountPoint @FILE node will be used (override).                    |
                 * |YES                         |NO                      |MountPoint @FILESYSETM node will be used.                         |
                 * |NO                          |YES                     |MountPoint @FILE node will be used.                               |
                 * |NO                          |NO                      |MountPoint configured for Tenant will be used.                    |   
                 * --------------------------------------------------------------------------------------------------------------------------  
                 * */
                #endregion Identify MountPoint attribute to use

                //Check whether MountPoint attribute is specified @FILE node
                if (FileNode.Attributes.GetNamedItem("MountPoint") != null)
                    UserProvidedMountPoint = FileNode.Attributes.GetNamedItem("MountPoint").Value;
                //If MountPoint attribute is not specified @FILE node, use MountPoint attribute specified @FILESYSTEM node. 
                if (String.IsNullOrEmpty(UserProvidedMountPoint))
                    UserProvidedMountPoint = m_UserMountPointForFileSystem;

                //Get Folder Path (destination folder)
                if ((obj_OMError = GetFolderPath(ref str_DestinationPath, obj_FileSystemInterface)) != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Failed to get destination folder path",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }
                //Create DESTINATIONFPATH node
                xmlNode_DestinationPath = FileNode.OwnerDocument.CreateElement("DESTINATIONPATH");

                //Update destination path including FILEID
                if (enumFileOperations == FileOperations.COPY)
                    xmlNode_DestinationPath.InnerText = str_DestinationPath + @"\" + ulNewFileId;
                else
                    xmlNode_DestinationPath.InnerText = str_DestinationPath + @"\" + ul_ExistingFileId;

                FileNode.AppendChild(xmlNode_DestinationPath);

                //Change the source path since it requires entire path upto FILEID
                FileNode.SelectSingleNode("PATH").InnerText = FileNode.SelectSingleNode("PATH").InnerText + @"\" + ul_ExistingFileId;

                #endregion Get File System Interface, File Id (from FILE node), prepare Destination Folder path

                #region Insert an entry into ALFS_DURABLELOG

                strColl_Parameters = new StringCollection();

                //Add source and destination path to parameters
                strColl_Parameters.Add(FileNode.SelectSingleNode("PATH").InnerText);
                strColl_Parameters.Add(FileNode.SelectSingleNode("DESTINATIONPATH").InnerText);

                FormPamareterXML(strColl_Parameters, ref str_ParameterXml);

                //Create an entry in ALFS_DURABLELOG for Copy or Move File operation
                FormDurableLogXML(TransactionID, (enumFileOperations == FileOperations.COPY ? "CopyFile" : "MoveFile"), obj_FileSystemInterface, str_ParameterXml, "PREPARED", ref str_DurableLogXml);
                if ((obj_OMError = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", str_DurableLogXml)) != null)
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                 "Writing into ALFS_DURABLELOG Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", obj_OMError.ErrorType,
                                                 obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                #endregion Insert an entry into ALFS_DURABLELOG

                #region Push file to Media Server or Pull file from Media Server

                str_FileCollectionXml = "<FILECOLLECTION>" + FileNode.OuterXml + "</FILECOLLECTION>";
                //Perform operations when Media Server is activated for tenant.
                if (ALCommon.IsMediaServerActivated(m_TenantName))
                {
                    obj_FileTransferInfrastructure = new FileTransferInfrastructure(null, UserID.ToString(), m_TenantName, null);
                    obj_FileTransferInfrastructure.UseMediaServer = true;

                    if (enumMoveFileOperations == MediaServerOperationType.PUSHFILETOMEDIASERVER)
                        str_ErrorXml = obj_FileTransferInfrastructure.SaveFile(ref str_FileCollectionXml);
                    else
                        str_ErrorXml = obj_FileTransferInfrastructure.GetFile(ref str_FileCollectionXml);
                }
                else
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "Media Server is not activated",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                null, str_ErrorXml);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                if (String.IsNullOrEmpty(str_ErrorXml))
                {
                    if (enumFileOperations == FileOperations.COPY)
                    {
                        #region Copy File case

                        if (FileNode.SelectSingleNode("FILESIZE") != null)
                        {
                            str_FileSize = FileNode.SelectSingleNode("FILESIZE").InnerText;
                        }

                        //Create an entry in ALFS_FILE table for newly created file
                        FormFileXMLForCreate(ulNewFileId, ulObjectId, FileNode.SelectSingleNode("OBJECTTYPE").InnerText,
                                            FileNode.SelectSingleNode("SCHEMAID").InnerText, FileNode.SelectSingleNode("NAME").InnerText,
                                            FileNode.SelectSingleNode("DISPLAYNAME").InnerText, FileNode.SelectSingleNode("DESCRIPTION").InnerText,
                                            str_DestinationPath, UserID, obj_FileSystemInterface, str_FileSize, ref str_FileXmlForCopy);

                        if ((obj_OMError = FILEDMObj.DataProvider.CreateRecord("ALFS_FILE", str_FileXmlForCopy)) != null)
                        {
                            obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                         "Writing into ALFS_FILE Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", obj_OMError.ErrorType,
                                                         obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                            obj_ReturnOMError.TransactionId = TransactionID.ToString();
                            return obj_ReturnOMError;
                        }

                        obj_OMError = AddSearchableTextForFile(ulNewFileId, ulObjectId, FileNode.SelectSingleNode("OBJECTTYPE").InnerText,
                                ulObjectTypeId, FileNode.SelectSingleNode("NAME").InnerText,
                                Convert.ToUInt64(FileNode.SelectSingleNode("SCHEMAID").InnerText),
                                FileNode.SelectSingleNode("FIELDNAME").InnerText, obj_FileSystemInterface);
                        if (obj_OMError != null)
                        {
                            obj_OMError.TransactionId = TransactionID.ToString();
                            return obj_OMError;
                        }

                        #endregion Copy File case
                    }
                    else if (enumFileOperations == FileOperations.MOVE)
                    {
                        #region Move File case

                        /* Steps Required: 
                         * 1. Since the file has been moved to specified location, ALFS_FILE table needs to be updated.
                         * 2. Source file needs to be deleted since move operation will be considered as copy in case of Pull or Push file.
                         * 3. Decrement folder count since file wil be deleted
                         * */

                        #region Update File details (Path, OnMediaServer, MediaServerUrl)

                        sb_FileDetailsXmlForUpdate = new StringBuilder(200);
                        //Update PATH, ONMEDIASERVER and MEDIASERVERURL as per operation performed for file
                        if (enumMoveFileOperations == MediaServerOperationType.PUSHFILETOMEDIASERVER)
                        {
                            sb_FileDetailsXmlForUpdate.Append(@"<FILE>
                                                        <PATH>" + ALCommon.TrimAndReplaceSpecialChars(str_DestinationPath) + @"</PATH>
                                                        <ONMEDIASERVER>TRUE</ONMEDIASERVER>
                                                        <MEDIASERVERURL>" + obj_FileSystemInterface.MediaServerUrl + @"</MEDIASERVERURL>
                                                    </FILE>");
                        }
                        else
                        {
                            sb_FileDetailsXmlForUpdate.Append(@"<FILE>
                                                        <PATH>" + ALCommon.TrimAndReplaceSpecialChars(str_DestinationPath) + @"</PATH>
                                                        <ONMEDIASERVER>FALSE</ONMEDIASERVER>
                                                        <MEDIASERVERURL></MEDIASERVERURL>
                                                    </FILE>");
                        }

                        //Update ALFS_FILE record
                        if ((obj_OMError = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ul_ExistingFileId.ToString(), OMConjunction.NOTHING)) == null)
                        {
                            obj_OMError = FILEDMObj.DataProvider.UpdateRecord("ALFS_FILE", sb_FileDetailsXmlForUpdate.ToString());
                        }
                        if (obj_OMError != null)
                        {
                            obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                             "Failed to update file details",
                                                             OMErrorObjectType.FILESYSTEM,
                                                             "FileSystemManager", obj_OMError.ErrorType,
                                                             obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                            obj_ReturnOMError.TransactionId = TransactionID.ToString();
                            return obj_ReturnOMError;
                        }

                        //Updating alerts in the ALENT_SEARCHABLE_FILE_ALERT table for document search
                        obj_OMError = UpdateSearchableTextForFile(ul_ExistingFileId, obj_FileSystemInterface);

                        if (obj_OMError != null)
                            return obj_OMError;

                        #endregion Update File details (Path, OnMediaServer, MediaServerUrl)

                        #region Delete file from NTFS

                        #region Push File and Pull File implementation details
                        /* File needs to be deleted from source in following cases 
                         * PUSHFILETOMEDIASERVER: If the operation is push file, then the file from OM Mount Point needs to be deleted after push operation is successful.
                         * PULLFILEFROMMEDIASERVER: If the operation is pull file, then the file from Media Server Mount Point needs to be deleted after pull operation is successful.
                         * 1. In case of PUSH file operation, Remote File System Interface object will be required to push file from OM Mount Point to
                         *    Media Server Mount Point. Now, while deleting file from OM Mount Point, Remote File System Interface object could not be 
                         *    used since the file to be deleted is sitting under OM Mount Point, so Local File System Interface object will be requied.
                         * 
                         * 2. In case of PULL file operation, Local File System Interface object will be required to pull file from Media Server Mount Point 
                         *    to OM Mount Point. Now, while deleting file from Media Server Mount Point, Local File System Interface object could not be 
                         *    used since the file to be deleted is sitting under Media Server Mount Point, so Remote File System Interface object will be requied.
                         */
                        #endregion Push File and Pull File implementation details

                        //Get File System Interface object which will be used for delete file operation
                        obj_FileSystemInterface = GetFileSystemInterface(true, FileNode);

                        //Delete file from source
                        obj_OMError = obj_FileSystemInterface.IFileSystemInstance.DeleteFile(FileNode.SelectSingleNode("PATH").InnerText);
                        if (obj_OMError != null)
                        {
                            obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                   "Failed to delete file",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", obj_OMError.ErrorType,
                                                   obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                            obj_ReturnOMError.TransactionId = TransactionID.ToString();
                            return obj_ReturnOMError;
                        }

                        #endregion Delete file from NTFS

                        #region Decrement folder count (since file is moved to another folder)

                        //Since the file has got deleted, reduce the file count for its folder
                        obj_OMError = DecrementFolderCount(FileNode.SelectSingleNode("PATH").InnerText);
                        if (obj_OMError != null)
                        {
                            obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                         "Failed to update folder details",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", obj_OMError.ErrorType,
                                                         obj_OMError.ExceptionObject, obj_OMError.ErrorXML);
                            obj_ReturnOMError.TransactionId = TransactionID.ToString();
                            return obj_ReturnOMError;
                        }

                        #endregion Decrement folder count (since file is moved to another folder)

                        #endregion Move File case
                    }
                }
                else
                {
                    obj_ReturnOMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "File move operation failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "FileSystemManager", OMErrorType.ERROR,
                                                null, str_ErrorXml);
                    obj_ReturnOMError.TransactionId = TransactionID.ToString();
                    return obj_ReturnOMError;
                }

                #endregion Push file to Media Server or Pull file from Media Server
            }
            finally
            {
                #region Cleanup

                xmlNode_DestinationPath = null;
                str_FileCollectionXml = null;
                str_ErrorXml = null;
                str_DestinationPath = null;
                str_ParameterXml = null;
                str_DurableLogXml = null;
                strColl_Parameters = null;
                obj_FileTransferInfrastructure = null;
                obj_OMError = null;
                obj_ReturnOMError = null;
                obj_FileSystemInterface = null;

                if (sb_FileDetailsXmlForUpdate != null)
                {
                    sb_FileDetailsXmlForUpdate = sb_FileDetailsXmlForUpdate.Remove(0, sb_FileDetailsXmlForUpdate.Length);
                    sb_FileDetailsXmlForUpdate = null;
                }

                #endregion Cleanup
            }

            return null;

            #endregion Push file to Media Server or Pull file from Media Server
        }

        /// <summary>
        /// Identifies Operation to be performed on file and returns Operation attribute value.
        /// </summary>
        /// <param name="xmlNode_File">File Node with details of operation to be performed.</param>
        /// <param name="strOperation">Output parameter, which specifies operation to be performed on file.</param>
        /// <returns>Returns OMError if any error occurs, otherwise null.</returns>
        private OMError GetFileOperation(XmlNode xmlNode_File, out String strOperation)
        {
            strOperation = null;
            //Read Operation attribute value
            if (xmlNode_File.Attributes.GetNamedItem("Operation") != null)
            {
                strOperation = xmlNode_File.Attributes.GetNamedItem("Operation").Value;
            }

            //Cases when Operation atribute is not specified (or value is not specified)
            if (String.IsNullOrEmpty(strOperation))
            {
                if (MoveFiles || CopyFiles)
                {
                    if (xmlNode_File.SelectSingleNode("FILEID") == null)
                    {
                        //When FILEID is not specified, consider the operation as CREATE
                        strOperation = "CREATE";
                    }
                    else
                    {
                        //When FILEID is specified, set the operation according to MoveFiles or CopyFiles flag value
                        if (MoveFiles)
                            strOperation = "MOVE";
                        else if (CopyFiles)
                            strOperation = "COPY";
                    }
                }
                else
                {
                    //When MoveFiles = false and CopyFiles = false, consider the operation as CREATE regardless of whether FILEID is specified
                    strOperation = "CREATE";
                }
            }

            //Non null values for operation will be returned as it is (i.e. CREATE/UPDATE/DELETE/COPY/MOVE)
            return null;
        }

        private OMError GetSchemaID(ulong ObjectTypeID, string FieldName, ref ulong SchemaID)
        {
            OMError ErrObj = null;
            string SchemaXML = null;
            XmlDocument SchemaXMLDoc = null;
            try
            {
                SchemaXMLDoc = new XmlDocument();
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FIELDNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FieldName, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("SCHEMA", "ALOM_SCHEMA", null, ref SchemaXML)) == null)
                        {
                            if (SchemaXML != null)
                            {
                                SchemaXMLDoc.LoadXml(SchemaXML);
                                SchemaID = Convert.ToUInt64(SchemaXMLDoc.SelectSingleNode("//SCHEMAID").InnerText);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception)
            {

                throw;
            }
        }

        private OMError GetObjectName(ulong ObjectTypeID, string ObjectType, ref string ObjectName)
        {
            OMError ErrObj = null;
            string TypeXML = null;
            XmlDocument TypeXMLDoc = null;
            try
            {
                TypeXMLDoc = new XmlDocument();
                if (String.Equals(ObjectType.ToUpper(), "ENTITY"))
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("ENTITYTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("ENTITYTYPEDATA", "ALOM_ENTITYTYPE", null, ref TypeXML)) == null)
                        {
                            if (TypeXML != null)
                            {
                                TypeXMLDoc.LoadXml(TypeXML);
                                ObjectName = TypeXMLDoc.SelectSingleNode("ENTITYTYPEDATA/ENTITYTYPENAME").InnerText;
                            }
                        }
                    }
                }
                else if (String.Equals(ObjectType.ToUpper(), "FACET"))
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FACETTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("FACETTYPEDATA", "ALOM_FACETTYPE", null, ref TypeXML)) == null)
                        {
                            if (TypeXML != null)
                            {
                                TypeXMLDoc.LoadXml(TypeXML);
                                ObjectName = TypeXMLDoc.SelectSingleNode("FACETTYPEDATA/FACETTYPENAME").InnerText;
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception)
            {
                ErrObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                                                         "Read Record For Type Data Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", ErrObj.ErrorType,
                                                         ErrObj.ExceptionObject, ErrObj.ErrorXML);
                ErrObj.TransactionId = TransactionID.ToString();
                return ErrObj;
            }
        }

        private OMError GetFileSearchability(ulong ObjectTypeID, string FieldName, ref bool IsFileSearchable)
        {
            OMError ErrObj = null;
            string SchemaXML = null;
            XmlDocument SchemaXMLDoc = null;
            try
            {
                SchemaXMLDoc = new XmlDocument();
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTTYPEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectTypeID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FIELDNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FieldName, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("SCHEMA", "ALOM_SCHEMA", null, ref SchemaXML)) == null)
                        {
                            if (SchemaXML != null)
                            {
                                SchemaXMLDoc.LoadXml(SchemaXML);
                                IsFileSearchable = Convert.ToBoolean(SchemaXMLDoc.SelectSingleNode("SCHEMA/ISSEARCHFIELD").InnerText);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception e)
            {
                ErrObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                                     "Reading Searchability Details From Schema Table Failed",
                                     OMErrorObjectType.FILESYSTEM, "FileSystemManager", OMErrorType.EXCEPTION, e, "");
                ErrObj.TransactionId = TransactionID.ToString();
                return ErrObj;
            }
        }

        /*private OMError GetSchemaID(string FieldName, string TableName, ref ulong SchemaID)
        {
            OMError ErrObj = null;
            string SchemaXML = null;
            XmlDocument SchemaXMLDoc = null;
            try
            {
                SchemaXMLDoc = new XmlDocument();
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("TABLENAME", OMFieldDataType.STRING, OMOperator.EQUALTO, TableName, OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FIELDNAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FieldName, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("SCHEMA", "ALOM_SCHEMA", null, ref SchemaXML)) == null)
                        {
                            if (SchemaXML != null)
                            {
                                SchemaXMLDoc.LoadXml(SchemaXML);
                                SchemaID = Convert.ToUInt64(SchemaXMLDoc.SelectSingleNode("//SCHEMAID").InnerText);
                            }
                        }
                    }
                }
                return null;
            }
            catch (Exception)
            {
                throw;
            }
        }*/

        private OMError UpdateFile(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string DestinationPath = null;
            StringBuilder DestinationFilePath = null;
            StringBuilder SourcePath = null;
            string ParameterXML = null;
            StringCollection Parameters = null;
            StringBuilder InsertQuery = null;
            string DurableLogXML = null;
            string FileName = null;
            ulong SchemaID = 0;
            string Description = null;
            ulong FileID = 0;
            string FileXML = null;
            string FieldName = null;
            XmlDocument FileXMLDoc = null;
            string FileDisplayName = null;

            try
            {
                // check whether write possible.
                Parameters = new StringCollection();
                InsertQuery = new StringBuilder();
                SourcePath = new StringBuilder();
                DestinationFilePath = new StringBuilder();
                FileXMLDoc = new XmlDocument();


                FileName = FileNode.SelectSingleNode("NAME").InnerText;
                FieldName = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("FILE", "ALFS_FILE", null, ref FileXML)) == null)
                        {
                            if (FileXML != null)
                            {
                                FileXMLDoc.LoadXml(FileXML);
                                FileID = Convert.ToUInt64(FileXMLDoc.SelectSingleNode("//FILEID").InnerText);
                                DestinationPath = FileXMLDoc.SelectSingleNode("//PATH").InnerText;
                                DestinationFilePath.AppendFormat("{0}\\{1}", DestinationPath, FileID);
                            }
                        }
                        else
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                     "Reading ALFS_FILE Failed",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", ErrObj.ErrorType,
                                                     ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }

                if (FileNode.SelectNodes("DISPLAYNAME").Count > 0)
                {
                    FileDisplayName = FileNode.SelectSingleNode("DISPLAYNAME").InnerText;
                }
                if (FileNode.SelectNodes("DESCRIPTION").Count > 0)
                {
                    Description = FileNode.SelectSingleNode("DESCRIPTION").InnerText;
                }

                if (FileNode.SelectNodes("PATH").Count > 0)
                {
                    SourcePath.AppendFormat("{0}", FileNode.SelectSingleNode("PATH").InnerText);

                    StringBuilder ShadowFolderFilePath = new StringBuilder();
                    String strShadowFolder = GetShadowFolderPath(DestinationPath, FileXMLDoc.DocumentElement);
                    ShadowFolderFilePath.AppendFormat("{0}\\{1}", strShadowFolder, FileID);
                    Parameters.Clear();
                    Parameters.Add(DestinationFilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    if ((ErrObj = PerformOperation("CopyFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // COPY ORIGINAL FILE TO SHADOW LOCATION
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }
                    Parameters.Clear();
                    Parameters.Add(SourcePath.ToString());
                    Parameters.Add(DestinationFilePath.ToString());
                    if ((ErrObj = PerformOperation("CopyFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // REPLACE ORIGINAL FILE WITH NEW FILE
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }

                    Parameters.Clear();
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    Parameters.Add(DestinationFilePath.ToString());
                    FormPamareterXML(Parameters, ref ParameterXML);

                    // Log into DurableLog file
                    FormDurableLogXML(TransactionID, "UpdateFile", GetFileSystemInterface(FileNode), ParameterXML, "PREPARED", ref DurableLogXML);

                    if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                    "Writing into ALFS_DURABLELOG Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }

                FormFileXMLForUpdate(FileName, FileDisplayName, Description, UserID, ref FileXML);
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.UpdateRecord("ALFS_FILE", FileXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                   "Updating ALFS_FILE Failed",
                                                   OMErrorObjectType.FILESYSTEM,
                                                   "FileSystemManager", ErrObj.ErrorType,
                                                   ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                           "Exception thrown in UpdateFile API",
                                           OMErrorObjectType.FILESYSTEM,
                                           "FileSystemManager", OMErrorType.EXCEPTION,
                                           ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError UpdateFile(ulong FileID, ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string DestinationPath = null;
            StringBuilder DestinationFilePath = null;
            StringBuilder SourcePath = null;
            string ParameterXML = null;
            StringCollection Parameters = null;
            StringBuilder InsertQuery = null;
            string DurableLogXML = null;
            string FileName = null;
            ulong SchemaID = 0;
            string Description = null;
            string FileXML = null;
            string FieldName = null;
            XmlDocument FileXMLDoc = null;
            string FileDisplayName = null;

            try
            {
                // check whether write possible.
                Parameters = new StringCollection();
                InsertQuery = new StringBuilder();
                SourcePath = new StringBuilder();
                DestinationFilePath = new StringBuilder();
                FileXMLDoc = new XmlDocument();


                FileName = FileNode.SelectSingleNode("NAME").InnerText;
                FieldName = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("FILE", "ALFS_FILE", null, ref FileXML)) == null)
                    {
                        if (FileXML != null)
                        {
                            FileXMLDoc.LoadXml(FileXML);
                            DestinationPath = FileXMLDoc.SelectSingleNode("PATH").InnerText;
                            DestinationFilePath.AppendFormat("{0}\\{1}", DestinationPath, FileID);
                        }
                        else
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                     "Reading ALFS_FILE Failed",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", ErrObj.ErrorType,
                                                     ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }

                if (FileNode.SelectNodes("DISPLAYNAME").Count > 0)
                {
                    FileDisplayName = FileNode.SelectSingleNode("DISPLAYNAME").InnerText;
                }

                if (FileNode.SelectNodes("DESCRIPTION").Count > 0)
                {
                    Description = FileNode.SelectSingleNode("DESCRIPTION").InnerText;
                }

                if (FileNode.SelectNodes("PATH").Count > 0)
                {
                    SourcePath.AppendFormat("{0}", FileNode.SelectSingleNode("PATH").InnerText);
                    StringBuilder ShadowFolderFilePath = new StringBuilder();
                    String strShadowFolder = GetShadowFolderPath(DestinationPath, FileXMLDoc.DocumentElement);
                    ShadowFolderFilePath.AppendFormat("{0}\\{1}", strShadowFolder, FileID);
                    Parameters.Clear();
                    Parameters.Add(DestinationFilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    if ((ErrObj = PerformOperation("CopyFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // COPY ORIGINAL FILE TO SHADOW LOCATION
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }
                    Parameters.Clear();
                    Parameters.Add(SourcePath.ToString());
                    Parameters.Add(DestinationFilePath.ToString());
                    if ((ErrObj = PerformOperation("CopyFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // REPLACE ORIGINAL FILE WITH NEW FILE
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }

                    Parameters.Clear();
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    Parameters.Add(DestinationFilePath.ToString());
                    FormPamareterXML(Parameters, ref ParameterXML);

                    // Log into DurableLog file
                    FormDurableLogXML(TransactionID, "UpdateFile", GetFileSystemInterface(FileNode), ParameterXML, "PREPARED", ref DurableLogXML);
                    if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                    "Writing into ALFS_DURABLELOG Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }

                FormFileXMLForUpdate(FileName, FileDisplayName, Description, UserID, ref FileXML);
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.UpdateRecord("ALFS_FILE", FileXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                                  "Updating ALFS_FILE Failed",
                                                  OMErrorObjectType.FILESYSTEM,
                                                  "FileSystemManager", ErrObj.ErrorType,
                                                  ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                                               "Exception thrown in UpdateFile API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError DeleteFile(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string Path = null;
            StringBuilder FilePath = null;
            StringBuilder SourcePath = null;
            string ParameterXML = null;
            StringCollection Parameters = null;
            string DurableLogXML = null;
            string FileName = null;
            ulong SchemaID = 0;
            string FieldName = null;
            string FileID = null;
            string FileXML = null;
            XmlDocument FileXMLDoc = null;

            try
            {
                // check whether write possible.

                SourcePath = new StringBuilder();
                FilePath = new StringBuilder();
                FileXMLDoc = new XmlDocument();
                Parameters = new StringCollection();

                if (FileNode.SelectSingleNode("NAME") != null)
                    FileName = FileNode.SelectSingleNode("NAME").InnerText;



                //Code changed by pallavi, while deleting binder FieldName is not present in XML
                //on dated 25th Aug 2006
                if (FileNode.SelectSingleNode("FIELDNAME") != null)
                {
                    FieldName = FileNode.SelectSingleNode("FIELDNAME").InnerText;
                    GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);
                }
                else
                    SchemaID = Convert.ToUInt64(FileNode.SelectSingleNode("SCHEMAID").InnerText);
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
                {
                    if (!String.IsNullOrEmpty(FileName))
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.AND)) == null)
                            ErrObj = FILEDMObj.DataProvider.AddClause("NAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FileName, OMConjunction.NOTHING);
                    }
                    else
                        ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING);

                    if (ErrObj == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("ROOT", "ALFS_FILE", null, ref FileXML)) == null)
                        {
                            if (FileXML != null)
                            {
                                FileXMLDoc.LoadXml(FileXML);
                                FileID = FileXMLDoc.SelectSingleNode("//FILEID").InnerText;
                                Path = FileXMLDoc.SelectSingleNode("//PATH").InnerText;
                                FilePath.AppendFormat("{0}\\{1}", Path, FileID);
                            }
                        }
                        else
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                 "Reading ALFS_FILE Failed",
                                                 OMErrorObjectType.FILESYSTEM,
                                                 "FileSystemManager", ErrObj.ErrorType,
                                                 ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }

                if (FilePath != null)
                {
                    StringBuilder ShadowFolderFilePath = new StringBuilder();
                    String strShadowFolder = GetShadowFolderPath(Path, FileXMLDoc.DocumentElement);
                    ShadowFolderFilePath.AppendFormat("{0}\\{1}", strShadowFolder, FileID);
                    Parameters.Clear();
                    Parameters.Add(FilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    if ((ErrObj = PerformOperation("MoveFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // MOVE ORIGINAL FILE TO SHADOW LOCATION
                    {
                        if (ErrObj.Serialize().Contains("System.IO.IOException: Cannot create a file when that file already exists."))
                        {
                            IFileSystem iFileSystem = GetFileSystemInterface(FileNode).IFileSystemInstance;
                            if (iFileSystem.IsFileExists(FilePath.ToString()))
                            {
                                Parameters.Clear();
                                Parameters.Add(FilePath.ToString());
                                PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance);
                            }
                            else
                            {
                                if (ErrObj != null)
                                    ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            // delete source file
                        }
                        else if (ErrObj.Serialize().Contains("System.IO.FileNotFoundException: Could not find file"))
                        {
                            IFileSystem iFileSystem = GetFileSystemInterface(FileNode).IFileSystemInstance;
                            if (iFileSystem.IsFileExists(FilePath.ToString())) // file at source is present 
                            {
                                if ((ErrObj = iFileSystem.CopyFile(FilePath.ToString(), ShadowFolderFilePath.ToString())) == null)
                                {
                                    if ((ErrObj = iFileSystem.DeleteFile(FilePath.ToString())) != null)
                                    {
                                        if (ErrObj != null)
                                            ErrObj.TransactionId = TransactionID.ToString();
                                        return ErrObj;
                                    }
                                }
                                else
                                {
                                    if (ErrObj != null)
                                        ErrObj.TransactionId = TransactionID.ToString();
                                    return ErrObj;
                                }
                                //try CopyFiles and then delete
                            }
                            else
                            {
                                if (ErrObj != null)
                                    ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                        }
                        else
                        {
                            if (ErrObj != null)
                                ErrObj.TransactionId = TransactionID.ToString();
                            return ErrObj;
                        }
                    }

                    Parameters.Clear();
                    Parameters.Add(FilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    FormPamareterXML(Parameters, ref ParameterXML);
                    // Log into DurableLog file
                    FormDurableLogXML(TransactionID, "DeleteFile", GetFileSystemInterface(FileNode), ParameterXML, "PREPARED", ref DurableLogXML);
                    if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                    "Writing into ALFS_DURABLELOG Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }

                    // Remove Entry from ALFS_File table.
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.DeleteRecord("ALFS_FILE")) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                    "Delete on ALFS_FILE Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                    DecrementFolderCount(FilePath.ToString());
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                               "Exception thrown in DeleteFile API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError CreateBinder(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode BinderNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            ulong SchemaID = 0;
            string Description = null;
            ulong BinderID = 0;
            string BinderName = null;
            string FieldName = null;
            string BinderXML = null;
            XmlNodeList FileNodes = null;
            XmlNode FileNode = null;
            XmlNode SequenceOrderNode = null;

            try
            {
                UniqueID.Read(ref BinderID);
                BinderName = BinderNode.SelectSingleNode("NAME").InnerText;
                FieldName = BinderNode.SelectSingleNode("FIELDNAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);
                if (BinderNode.SelectNodes("DESCRIPTION").Count > 0)
                {
                    Description = BinderNode.SelectSingleNode("DESCRIPTION").InnerText;
                }
                FormBinderXML(BinderID, BinderName, ObjectID, ObjectType, Description, SchemaID, UserID, 0, ref BinderXML);
                if ((ErrObj = FILEDMObj.DataProvider.CreateRecord("ALFS_BINDER", BinderXML)) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERCREATION_FAILED,
                                                     "Writing into ALFS_BINDER Failed",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", ErrObj.ErrorType,
                                                     ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }
                FileNodes = BinderNode.SelectNodes("FILE");
                if (BinderNode.SelectNodes("//SEQUENCEORDER").Count == 0)
                {
                    for (int FileCount = 1; FileCount <= FileNodes.Count; FileCount++)
                    {
                        FileNode = FileNodes[FileCount - 1];
                        SequenceOrderNode = FileNode.OwnerDocument.CreateElement("SEQUENCEORDER");
                        SequenceOrderNode.InnerText = FileCount.ToString();
                        FileNode.AppendChild(SequenceOrderNode);
                    }
                }
                if ((ErrObj = FileBinderOperations(BinderID, ObjectID, ObjectType, ObjectTypeID, FileNodes)) != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERCREATION_FAILED,
                                               "Exception thrown in CreateBinder API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }

        }

        private OMError UpdateBinder(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode BinderNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            ulong SchemaID = 0;
            string Description = null;
            ulong BinderID = 0;
            string BinderName = null;
            string FieldName = null;
            string BinderXML = null;
            XmlNodeList FileNodes = null;
            XmlDocument BinderXMLDoc = null;
            ulong SequenceOrder = 0;
            string Operation = null;
            XmlNode SequenceOrderNode = null;

            try
            {
                BinderXMLDoc = new XmlDocument();
                FieldName = BinderNode.SelectSingleNode("FIELDNAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("BINDER", "ALFS_BINDER", null, ref BinderXML)) == null)
                        {
                            if (BinderXML != null)
                            {
                                BinderXMLDoc.LoadXml(BinderXML);
                                BinderID = Convert.ToUInt64(BinderXMLDoc.SelectSingleNode("//BINDERID").InnerText);
                            }
                            else
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERUPDATION_FAILED,
                                                         "Reading ALFS_BINDER Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", ErrObj.ErrorType,
                                                         ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                        }
                    }
                }


                BinderName = BinderNode.SelectSingleNode("NAME").InnerText;
                if (BinderNode.SelectNodes("DESCRIPTION").Count > 0)
                {
                    Description = BinderNode.SelectSingleNode("DESCRIPTION").InnerText;
                }

                FormBinderXML(BinderID, BinderName, ObjectID, ObjectType, Description, SchemaID, 0, UserID, ref BinderXML);
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.UpdateRecord("ALFS_BINDER", BinderXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERUPDATION_FAILED,
                                                  "Updating ALFS_BINDER Failed",
                                                  OMErrorObjectType.FILESYSTEM,
                                                  "FileSystemManager", ErrObj.ErrorType,
                                                  ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }

                FileNodes = BinderNode.SelectNodes("FILE");

                if ((ErrObj = GetMaxSequenceOrder(BinderID, ref SequenceOrder)) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERUPDATION_FAILED,
                                                     "Updating ALFS_BINDER Failed",
                                                     OMErrorObjectType.FILESYSTEM,
                                                     "FileSystemManager", ErrObj.ErrorType,
                                                     ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }

                foreach (XmlNode FileNode in FileNodes)
                {
                    Operation = null;
                    //Check whether Operation attribute is specified
                    if (FileNode.Attributes.GetNamedItem("Operation") != null)
                        Operation = FileNode.Attributes.GetNamedItem("Operation").Value;
                    if (Operation == null || Operation.ToUpper() == "CREATE")
                    {
                        if (FileNode.SelectNodes("SEQUENCEORDER").Count == 0)
                        {
                            SequenceOrder++;
                            SequenceOrderNode = FileNode.OwnerDocument.CreateElement("SEQUENCEORDER");
                            SequenceOrderNode.InnerText = SequenceOrder.ToString();
                            FileNode.AppendChild(SequenceOrderNode);
                        }
                    }
                }
                if ((ErrObj = FileBinderOperations(BinderID, ObjectID, ObjectType, ObjectTypeID, FileNodes)) != null)
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERCREATION_FAILED,
                                               "Exception thrown in UpdateBinder API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }

        }

        private OMError DeleteBinder(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode BinderNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            ulong SchemaID = 0;
            string FieldName = null;
            XmlDocument FileXMLDoc = null;
            string FileIDs = null;
            XmlDocument FileCollectionDoc = null;
            XmlNodeList FileNodes = null;
            string BinderID = null;
            string BinderName = null;
            string BinderXML = null;
            XmlDocument BinderXMLDoc = null;
            XmlDocument AssoFileBinderXMLDoc = null;
            string FileCollectionXML = null;
            try
            {
                // check whether write possible.

                FileXMLDoc = new XmlDocument();
                BinderXMLDoc = new XmlDocument();
                FileCollectionDoc = new XmlDocument();
                AssoFileBinderXMLDoc = new XmlDocument();

                BinderName = BinderNode.SelectSingleNode("NAME").InnerText;
                FieldName = BinderNode.SelectSingleNode("FIELDNAME").InnerText;
                GetSchemaID(ObjectTypeID, FieldName, ref SchemaID);
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("OBJECTID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, ObjectID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("SCHEMAID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SchemaID.ToString(), OMConjunction.AND)) == null)
                    {
                        //  if ((ErrObj = FILEDMObj.DataProvider.AddClause("NAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FileName, OMConjunction.NOTHING)) == null)
                        if ((ErrObj = FILEDMObj.DataProvider.AddClause("NAME", OMFieldDataType.STRING, OMOperator.EQUALTO, BinderName, OMConjunction.NOTHING)) == null)
                        {
                            if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("ROOT", "ALFS_BINDER", null, ref BinderXML)) == null)
                            {
                                if (BinderXML != null)
                                {
                                    BinderXMLDoc.LoadXml(BinderXML);
                                    // BinderID = FileXMLDoc.SelectSingleNode("//BINDERID").InnerText;
                                    BinderID = BinderXMLDoc.SelectSingleNode("//BINDERID").InnerText;
                                }
                                else
                                {
                                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERDELETION_FAILED,
                                                         "Reading ALFS_BINDER Failed",
                                                         OMErrorObjectType.FILESYSTEM,
                                                         "FileSystemManager", ErrObj.ErrorType,
                                                         ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                    ReturnErrObj.TransactionId = TransactionID.ToString();
                                    return ReturnErrObj;
                                }
                            }
                        }
                    }
                }

                GetBinderFileIDs(Convert.ToUInt64(BinderID), ref FileIDs);

                if (FileIDs.Length > 0)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.IN, FileIDs.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadAllRecord("FILE", "ALFS_FILE", null, ref FileCollectionXML)) == null)
                        {
                            if (FileCollectionXML != null)
                            {
                                FileCollectionDoc.LoadXml(FileCollectionXML);
                                FileNodes = FileCollectionDoc.SelectNodes("//FILE");
                                if (FileNodes.Count > 0)
                                {
                                    foreach (XmlNode FileNode in FileNodes)
                                    {
                                        if ((ErrObj = DeleteFile(ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                                        {
                                            ErrObj.TransactionId = TransactionID.ToString();
                                            return ErrObj;
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERDELETION_FAILED,
                                                        "Reading ALFS_FILE Failed",
                                                        OMErrorObjectType.FILESYSTEM,
                                                        "FileSystemManager", ErrObj.ErrorType,
                                                        ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID, OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.DeleteRecord("ALFS_ASSOFILEBINDER")) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERDELETION_FAILED,
                                                        "Delete on ALFS_ASSOFILEBINDER Failed",
                                                        OMErrorObjectType.FILESYSTEM,
                                                        "FileSystemManager", ErrObj.ErrorType,
                                                        ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID, OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.DeleteRecord("ALFS_BINDER")) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERDELETION_FAILED,
                                                        "Delete on ALFS_BINDER Failed",
                                                        OMErrorObjectType.FILESYSTEM,
                                                        "FileSystemManager", ErrObj.ErrorType,
                                                        ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERDELETION_FAILED,
                                               "Exception thrown in DeleteBinder API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }

        }

        private OMError GetMaxSequenceOrder(ulong BinderID, ref ulong MaxSequenceOrder)
        {
            OMError ErrObj = null;
            StringBuilder SelectQuery = null;
            string AssoFileBinderXML = null;
            XmlDocument AssoFileBinderXMLDoc = null;

            try
            {

                SelectQuery = new StringBuilder();
                SelectQuery.AppendFormat("Select MAX(SEQUENCEORDER) as MAXSEQUENCEORDER from ALFS_ASSOFILEBINDER where BINDERID = {0}", BinderID);
                if ((ErrObj = FILEDMObj.DataProvider.ExecuteQuery(SelectQuery.ToString(), "ASSOFILEBINDER", ref AssoFileBinderXML)) == null)
                {
                    if (AssoFileBinderXML != null)
                    {
                        AssoFileBinderXMLDoc = new XmlDocument();
                        AssoFileBinderXMLDoc.LoadXml(AssoFileBinderXML);
                        if ((AssoFileBinderXMLDoc.SelectSingleNode("//MAXSEQUENCEORDER").InnerText != null) && (AssoFileBinderXMLDoc.SelectSingleNode("//MAXSEQUENCEORDER").InnerText != ""))
                            MaxSequenceOrder = Convert.ToUInt64(AssoFileBinderXMLDoc.SelectSingleNode("//MAXSEQUENCEORDER").InnerText);
                        else
                            MaxSequenceOrder = 0;
                    }
                    else
                    {
                        MaxSequenceOrder = 0;
                    }
                }
                else
                {
                    ErrObj.TransactionId = TransactionID.ToString();
                    return ErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private OMError GetBinderFileIDs(ulong BinderID, ref string FileIDs)
        {
            OMError ErrObj = null;
            StringBuilder FileID = null;
            string AssoFileBinderXML = null;
            XmlNodeList FileIDNodes = null;
            string Seprator = null;
            XmlDocument AssoFileBinderXMLDoc = null;
            try
            {
                FileID = new StringBuilder();
                AssoFileBinderXMLDoc = new XmlDocument();
                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID.ToString(), OMConjunction.NOTHING)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.ReadAllRecord("ROOT", "ALFS_ASSOFILEBINDER", null, ref AssoFileBinderXML)) == null)
                    {
                        if (AssoFileBinderXML != null)
                        {
                            AssoFileBinderXMLDoc.LoadXml(AssoFileBinderXML);
                            FileIDNodes = AssoFileBinderXMLDoc.SelectNodes("//FILEID");
                            if (FileIDNodes.Count > 0)
                            {
                                FileID.Append("(");
                                foreach (XmlNode FileIDNode in FileIDNodes)
                                {
                                    FileID.AppendFormat("{0}{1}", Seprator, FileIDNode.InnerText);
                                    Seprator = ",";
                                }
                                FileID.Append(")");
                            }
                        }
                    }
                }
                FileIDs = FileID.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }
        }

        private OMError FileBinderOperations(ulong BinderID, ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNodeList FileNodes)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string Operation = null;
            ulong FileID = 0;
            XmlNode SequenceOrderNode = null;
            ulong SequenceOrder = 0;

            try
            {
                foreach (XmlNode FileNode in FileNodes)
                {
                    SequenceOrder = 0;
                    SequenceOrderNode = FileNode.SelectSingleNode("SEQUENCEORDER");
                    if (SequenceOrderNode != null)
                    {
                        //SEQUENCEORDER may not be available in MOVE file case, so 
                        ulong.TryParse(SequenceOrderNode.InnerText, out SequenceOrder);
                        FileNode.RemoveChild(SequenceOrderNode);
                    }

                    Operation = null;

                    //Get file Operation (to be performed)
                    ErrObj = GetFileOperation(FileNode, out Operation);
                    if (ErrObj != null)
                    {
                        ErrObj.TransactionId = TransactionID.ToString();
                        return ErrObj;
                    }

                    switch (Operation.ToUpper())
                    {
                        case "COPY":
                        case "CREATE":
                        default:

                            //When Sequence Order = 0, return error since the value could not be converted from input Xml
                            if (SequenceOrder == 0)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                   "File creation within Binder Failed - Sequence Order is invalid",
                                   OMErrorObjectType.FILESYSTEM,
                                   "FileSystemManager", OMErrorType.ERROR,
                                   null, null);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }

                            UniqueID.Read(ref FileID);
                            if ((ErrObj = CreateFile(FileID, ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                   "File creation within Binder Failed",
                                   OMErrorObjectType.FILESYSTEM,
                                   "FileSystemManager", ErrObj.ErrorType,
                                   ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                            if ((ErrObj = AssociateFileWithBinder(BinderID, FileID, SequenceOrder)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        /* Changed by    : Mr. Sagar Dasalkar
                         * Date          : 6 August 2009
                         * Reason        : Update binder file case has been commented due to following reasons
                         * 1. There are issues with UpdateBinderFile() functionality. Fixes will be required.
                         * 2. User Interface does not allow updating binder files.
                            
                    case "UPDATE": 
                           
                        if ((ErrObj = UpdateBinderFile(BinderID, ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEUPDATION_FAILED,
                               "Updating file within Binder Failed",
                               OMErrorObjectType.FILESYSTEM,
                               "FileSystemManager", ErrObj.ErrorType,
                               ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            return ReturnErrObj;
                        }
                        if ((ErrObj = UpdateSequenceOrder(BinderID, FileID, SequenceOrder)) != null)
                        {
                            return ErrObj;
                        }
                        break;
                             
                         */

                        case "DELETE":

                            //When Sequence Order = 0, return error since the value could not be converted from input Xml
                            if (SequenceOrder == 0)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                   "Deleting file from Binder Failed - Sequence Order is invalid",
                                   OMErrorObjectType.FILESYSTEM,
                                   "FileSystemManager", OMErrorType.ERROR,
                                   null, null);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }

                            if ((ErrObj = DeleteBinderFile(ObjectID, ObjectType, ObjectTypeID, FileNode, BinderID, SequenceOrder)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                   "Deleting file from Binder Failed",
                                   OMErrorObjectType.FILESYSTEM,
                                   "FileSystemManager", ErrObj.ErrorType,
                                   ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                            if ((ErrObj = DeleteFileBinderAssociation(BinderID, SequenceOrder)) != null)
                            {
                                ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            break;

                        case "MOVE":

                            if ((ErrObj = MoveFile(ObjectID, ObjectType, ObjectTypeID, FileNode)) != null)
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                  "File move within Binder failed",
                                                  OMErrorObjectType.FILESYSTEM,
                                                  "FileSystemManager", ErrObj.ErrorType,
                                                  ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                            break;

                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDEROPERATION_FAILED,
                               "Exception thrown in FileBinderOperations API",
                               OMErrorObjectType.FILESYSTEM,
                               "FileSystemManager", OMErrorType.EXCEPTION,
                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        /// <summary>
        /// <Author>Tina C. Jawale</Author>
        /// <CreatedON>10 April 2007</CreatedON>
        /// <Purpose>Deletes the file uploaded using binder.</Purpose>
        /// </summary>
        /// <param name="ObjectID">Id of the enitity or facet instance</param>
        /// <param name="ObjectType">object type enitity/facet </param>
        /// <param name="ObjectTypeID">enititytype id or facet type id</param>
        /// <param name="FileNode">Information node about the file.</param>
        /// <param name="BinderID">Id of the Binder which is the same as that of in ALFS_BINDER</param>
        /// <param name="SequenceOrder">Sequence order of the file</param>
        /// <returns>Null - if file is successfully deleted
        ///          Error - if file deletion is unsuccessfull.         
        /// </returns>
        /// 
        private OMError DeleteBinderFile(ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode, ulong BinderID, ulong SequenceOrder)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string Path = null;
            string ParameterXML = null;
            string DurableLogXML = null;
            string FileID = null;
            string FileXML = null;
            XmlDocument FileXMLDoc = null;
            StringBuilder FilePath = null;
            StringCollection Parameters = null;

            try
            {
                FilePath = new StringBuilder();
                Parameters = new StringCollection();

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("SEQUENCEORDER", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SequenceOrder.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("ROOT", "ALFS_ASSOFILEBINDER", null, ref FileXML)) == null)
                        {
                            if (FileXML != null)
                            {
                                //retrive file - id
                                FileXMLDoc = new XmlDocument();
                                FileXMLDoc.LoadXml(FileXML);
                                FileID = FileXMLDoc.SelectSingleNode("//FILEID").InnerText;

                                FileXML = null;

                                if (FileID != null)
                                {
                                    //retrive file details
                                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID, OMConjunction.NOTHING)) == null)
                                    {
                                        if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("ROOT", "ALFS_FILE", null, ref FileXML)) == null)
                                        {
                                            FileXMLDoc.LoadXml(FileXML);
                                            Path = FileXMLDoc.SelectSingleNode("//PATH").InnerText;
                                            FilePath.AppendFormat("{0}\\{1}", Path, FileID);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (FilePath != null)
                {
                    StringBuilder ShadowFolderFilePath = new StringBuilder();
                    String strShadowFolder = GetShadowFolderPath(Path, FileXMLDoc.DocumentElement);
                    ShadowFolderFilePath.AppendFormat("{0}\\{1}", strShadowFolder, FileID);
                    Parameters.Clear();
                    Parameters.Add(FilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    if ((ErrObj = PerformOperation("MoveFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance)) != null) // MOVE ORIGINAL FILE TO SHADOW LOCATION
                    {
                        if (ErrObj.Serialize().Contains("System.IO.IOException: Cannot create a file when that file already exists."))
                        {
                            IFileSystem iFileSystem = GetFileSystemInterface(FileNode).IFileSystemInstance;
                            if (iFileSystem.IsFileExists(FilePath.ToString()))
                            {
                                Parameters.Clear();
                                Parameters.Add(FilePath.ToString());
                                PerformOperation("DeleteFile", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance);
                            }
                            else
                            {
                                if (ErrObj != null)
                                    ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                            // delete source file
                        }
                        else if (ErrObj.Serialize().Contains("System.IO.FileNotFoundException: Could not find file"))
                        {
                            IFileSystem iFileSystem = GetFileSystemInterface(FileNode).IFileSystemInstance;
                            if (iFileSystem.IsFileExists(FilePath.ToString())) // file at source is present
                            {
                                if ((ErrObj = iFileSystem.CopyFile(FilePath.ToString(), ShadowFolderFilePath.ToString())) == null)
                                {
                                    if ((ErrObj = iFileSystem.DeleteFile(FilePath.ToString())) != null)
                                    {
                                        ErrObj.TransactionId = TransactionID.ToString();
                                        return ErrObj;
                                    }
                                }
                                else
                                {
                                    if (ErrObj != null)
                                        ErrObj.TransactionId = TransactionID.ToString();
                                    return ErrObj;
                                }
                                //try CopyFiles and then delete
                            }
                            else
                            {
                                if (ErrObj != null)
                                    ErrObj.TransactionId = TransactionID.ToString();
                                return ErrObj;
                            }
                        }
                        else
                        {
                            if (ErrObj != null)
                                ErrObj.TransactionId = TransactionID.ToString();
                            return ErrObj;
                        }
                    }

                    Parameters.Clear();
                    Parameters.Add(FilePath.ToString());
                    Parameters.Add(ShadowFolderFilePath.ToString());
                    FormPamareterXML(Parameters, ref ParameterXML);
                    // Log into DurableLog file
                    FormDurableLogXML(TransactionID, "DeleteFile", GetFileSystemInterface(FileNode), ParameterXML, "PREPARED", ref DurableLogXML);
                    if ((ErrObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                    {
                        ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                    "Writing into ALFS_DURABLELOG Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                        ReturnErrObj.TransactionId = TransactionID.ToString();
                        return ReturnErrObj;
                    }

                    // Remove Entry from ALFS_File table.
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID, OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.DeleteRecord("ALFS_FILE")) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                    "Delete on ALFS_FILE Failed",
                                                    OMErrorObjectType.FILESYSTEM,
                                                    "FileSystemManager", ErrObj.ErrorType,
                                                    ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                    DecrementFolderCount(FilePath.ToString());
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                               "Exception thrown in DeleteBinderFile API",
                                               OMErrorObjectType.FILESYSTEM,
                                               "FileSystemManager", OMErrorType.EXCEPTION,
                                               ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError UpdateBinderFile(ulong BinderID, ulong ObjectID, string ObjectType, ulong ObjectTypeID, XmlNode FileNode)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            string FileName = null;
            string FileIDs = null;
            string FileXML = null;
            ulong FileID = 0;
            XmlDocument FileXMLDoc = null;
            try
            {
                FileXMLDoc = new XmlDocument();
                GetBinderFileIDs(BinderID, ref FileIDs);
                FileName = FileNode.SelectSingleNode("//NAME").InnerText;
                if (FileIDs.Length > 0)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.IN, FileIDs, OMConjunction.AND)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.AddClause("NAME", OMFieldDataType.STRING, OMOperator.EQUALTO, FileName, OMConjunction.NOTHING)) == null)
                        {
                            if ((ErrObj = FILEDMObj.DataProvider.ReadRecord("FILE", "ALFS_FILE", null, ref FileXML)) == null)
                            {
                                if (FileXML != null)
                                {
                                    FileXMLDoc.LoadXml(FileXML);
                                    FileID = Convert.ToUInt64(FileXMLDoc.SelectSingleNode("//FILEID").InnerText);
                                    UpdateFile(FileID, ObjectID, ObjectType, ObjectTypeID, FileNode);
                                }
                                else
                                {
                                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERFILEUPDATE_FAILED,
                                                  "Reading ALFS_FILE Failed",
                                                  OMErrorObjectType.FILESYSTEM,
                                                  "FileSystemManager", ErrObj.ErrorType,
                                                  ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                    ReturnErrObj.TransactionId = TransactionID.ToString();
                                    return ReturnErrObj;
                                }
                                //else
                                //{
                                //    UniqueID.Read(ref FileID);
                                //    CreateFile(FileID, ObjectID, ObjectType, ObjectTypeID, FileNode);
                                //}
                            }
                            else
                            {
                                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERFILEUPDATE_FAILED,
                                              "Reading ALFS_FILE Failed",
                                              OMErrorObjectType.FILESYSTEM,
                                              "FileSystemManager", ErrObj.ErrorType,
                                              ErrObj.ExceptionObject, ErrObj.ErrorXML);
                                ReturnErrObj.TransactionId = TransactionID.ToString();
                                return ReturnErrObj;
                            }
                        }
                    }

                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_BINDERFILEUPDATE_FAILED,
                                              "Exception Thrown in UpdateBinderFile",
                                              OMErrorObjectType.FILESYSTEM,
                                              "FileSystemManager", ErrObj.ErrorType,
                                              ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }

        }

        private OMError AssociateFileWithBinder(ulong BinderID, ulong FileID, ulong SequenceOrder)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            StringBuilder AssoFileBinderXML = null;
            ulong AssociationID = 0;
            try
            {
                UniqueID.Read(ref AssociationID);
                AssoFileBinderXML = new StringBuilder();
                AssoFileBinderXML.Append("<ASSOFILEBINDER>");
                AssoFileBinderXML.AppendFormat("<ASSOCIATIONID>{0}</ASSOCIATIONID>", AssociationID);
                AssoFileBinderXML.AppendFormat("<BINDERID>{0}</BINDERID>", BinderID);
                AssoFileBinderXML.AppendFormat("<FILEID>{0}</FILEID>", FileID);
                AssoFileBinderXML.AppendFormat("<SEQUENCEORDER>{0}</SEQUENCEORDER>", SequenceOrder);
                AssoFileBinderXML.Append("</ASSOFILEBINDER>");

                if ((ErrObj = FILEDMObj.DataProvider.CreateRecord("ALFS_ASSOFILEBINDER", AssoFileBinderXML.ToString())) != null)
                {
                    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_ASSOCIATINGFILEWITHBINDER_FAILED,
                                              "Writing into ALFS_ASSOFILEBINDER Failed",
                                              OMErrorObjectType.FILESYSTEM,
                                              "FileSystemManager", ErrObj.ErrorType,
                                              ErrObj.ExceptionObject, ErrObj.ErrorXML);
                    ReturnErrObj.TransactionId = TransactionID.ToString();
                    return ReturnErrObj;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_ASSOCIATINGFILEWITHBINDER_FAILED,
                                             "Exception thrown in AssociateFileWithBinder API",
                                             OMErrorObjectType.FILESYSTEM,
                                             "FileSystemManager", OMErrorType.EXCEPTION,
                                             ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError DeleteFileBinderAssociation(ulong BinderID, ulong SequenceOrder)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;

            try
            {

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("SEQUENCEORDER", OMFieldDataType.INTEGER, OMOperator.EQUALTO, SequenceOrder.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.DeleteRecord("ALFS_ASSOFILEBINDER")) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_DELETING_FILEBINDERASSOCIATION_FAILED,
                                                      "Deliting FileBinder Association from ALFS_ASSOFILEBINDER Failed",
                                                      OMErrorObjectType.FILESYSTEM,
                                                      "FileSystemManager", ErrObj.ErrorType,
                                                      ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }
                // commented because when multiple files are deleted from binder sequence number from xml 
                // were not insync with database - Tina

                //UpdateQuery = new StringBuilder();
                //UpdateQuery.Append("Update ALFS_ASSOFILEBINDER Set SEQUENCEORDER = SEQUENCEORDER - 1 ");
                //UpdateQuery.AppendFormat("Where BINDERID = {0} and SEQUENCEORDER > {1}",BinderID,SequenceOrder);
                //if ((ErrObj = FILEDMObj.DataProvider.Execute(UpdateQuery.ToString())) != null)
                //{
                //    ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_DELETING_FILEBINDERASSOCIATION_FAILED,
                //                              "Updating SequenceOrder of other files in Binder Failed",
                //                              OMErrorObjectType.FILESYSTEM,
                //                              "FileSystemManager", ErrObj.ErrorType,
                //                              ErrObj.ExceptionObject, ErrObj.ErrorXML);
                //    return ReturnErrObj;
                //}
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_DELETING_FILEBINDERASSOCIATION_FAILED,
                                             "Exception thrown in DeleteFileBinderAssociation API",
                                             OMErrorObjectType.FILESYSTEM,
                                             "FileSystemManager", OMErrorType.EXCEPTION,
                                             ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError UpdateSequenceOrder(ulong BinderID, ulong FileID, ulong SequenceOrder)
        {
            OMError ErrObj = null;
            OMError ReturnErrObj = null;
            StringBuilder AssoFileBinderXML = null;

            try
            {
                AssoFileBinderXML = new StringBuilder();
                AssoFileBinderXML.Append("<ASSOFILEBINDER>");
                AssoFileBinderXML.AppendFormat("<SEQUENCEORDER>{0}</SEQUENCEORDER>", SequenceOrder);
                AssoFileBinderXML.Append("</ASSOFILEBINDER>");

                if ((ErrObj = FILEDMObj.DataProvider.AddClause("BINDERID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, BinderID.ToString(), OMConjunction.AND)) == null)
                {
                    if ((ErrObj = FILEDMObj.DataProvider.AddClause("FILEID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, FileID.ToString(), OMConjunction.NOTHING)) == null)
                    {
                        if ((ErrObj = FILEDMObj.DataProvider.UpdateRecord("ALFS_ASSOFILEBINDER", AssoFileBinderXML.ToString())) != null)
                        {
                            ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_UPDATING_FILEBINDERSEQUENCEORDER_FAILED,
                                                      "Updating ALFS_ASSOFILEBINDER Failed",
                                                      OMErrorObjectType.FILESYSTEM,
                                                      "FileSystemManager", ErrObj.ErrorType,
                                                      ErrObj.ExceptionObject, ErrObj.ErrorXML);
                            ReturnErrObj.TransactionId = TransactionID.ToString();
                            return ReturnErrObj;
                        }
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                ReturnErrObj = new OMError(OMErrorCodes.FILESYSTEM_UPDATING_FILEBINDERSEQUENCEORDER_FAILED,
                                             "Exception thrown in UpdateSequenceOrder API",
                                             OMErrorObjectType.FILESYSTEM,
                                             "FileSystemManager", OMErrorType.EXCEPTION,
                                             ExceptionObj, null);
                ReturnErrObj.TransactionId = TransactionID.ToString();
                return ReturnErrObj;
            }
        }

        private OMError FormBinderXML(ulong BinderID, string BinderName, ulong ObjectID, string ObjectType, string Description, ulong SchemaID, ulong CreatedBy, ulong ModifiedBy, ref string BinderXML)
        {
            StringBuilder SBBinderXML = null;

            try
            {
                SBBinderXML = new StringBuilder();
                SBBinderXML.Append("<BINDER>");
                SBBinderXML.AppendFormat("<BINDERID>{0}</BINDERID>", BinderID);

                if ((BinderName != null) && (BinderName != ""))
                    BinderName = ALCommon.TrimAndReplaceSpecialChars(BinderName);
                SBBinderXML.AppendFormat("<NAME>{0}</NAME>", BinderName);

                SBBinderXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ObjectID);
                SBBinderXML.AppendFormat("<OBJECTTYPE>{0}</OBJECTTYPE>", ObjectType);

                if ((Description != null) && (Description != ""))
                    Description = ALCommon.TrimAndReplaceSpecialChars(Description);
                SBBinderXML.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", Description);

                SBBinderXML.AppendFormat("<SCHEMAID>{0}</SCHEMAID>", SchemaID);
                if (CreatedBy != 0)
                {
                    SBBinderXML.AppendFormat("<CREATEDBY>{0}</CREATEDBY>", CreatedBy);
                    SBBinderXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                }
                if (ModifiedBy != 0)
                {
                    SBBinderXML.AppendFormat("<MODIFIEDBY>{0}</MODIFIEDBY>", ModifiedBy);
                    SBBinderXML.AppendFormat("<MODIFIEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                }
                SBBinderXML.Append("</BINDER>");
                BinderXML = SBBinderXML.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError FormFileXMLForCreate(ulong FileID, ulong ObjectID, string ObjectType, string SchemaID, string FileName, string FileDisplayName, string Desc, string Path, ulong CreatedBy, FileSystemInterface FileSystemInterfaceObj, string FileSize, ref string FileXML)
        {
            StringBuilder SBFileXML = null;

            try
            {
                SBFileXML = new StringBuilder();
                SBFileXML.Append("<FILE>");
                SBFileXML.AppendFormat("<FILEID>{0}</FILEID>", FileID);
                SBFileXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ObjectID);
                SBFileXML.AppendFormat("<OBJECTTYPE>{0}</OBJECTTYPE>", ObjectType);
                SBFileXML.AppendFormat("<SCHEMAID>{0}</SCHEMAID>", SchemaID);
                if ((FileName != null) && (FileName != ""))
                    FileName = ALCommon.TrimAndReplaceSpecialChars(FileName);
                SBFileXML.AppendFormat("<NAME>{0}</NAME>", FileName);

                if ((FileDisplayName != null) && (FileDisplayName != ""))
                    FileDisplayName = ALCommon.TrimAndReplaceSpecialChars(FileDisplayName);
                SBFileXML.AppendFormat("<DISPLAYNAME>{0}</DISPLAYNAME>", FileDisplayName);

                if ((Desc != null) && (Desc != ""))
                    Desc = ALCommon.TrimAndReplaceSpecialChars(Desc);
                SBFileXML.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", Desc);

                if ((Path != null) && (Path != ""))
                    Path = ALCommon.TrimAndReplaceSpecialChars(Path);
                SBFileXML.AppendFormat("<PATH>{0}</PATH>", Path);
                if (CreatedBy != 0)
                {
                    SBFileXML.AppendFormat("<CREATEDBY>{0}</CREATEDBY>", CreatedBy);
                    SBFileXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                }

                SBFileXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", FileSystemInterfaceObj.UseMediaServer.ToString().ToUpper());
                if (FileSystemInterfaceObj.UseMediaServer)
                    SBFileXML.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", FileSystemInterfaceObj.MediaServerUrl);

                SBFileXML.Append("<FILESIZE>" + FileSize + "</FILESIZE>");
                SBFileXML.Append("</FILE>");
                FileXML = SBFileXML.ToString();

                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError FormSearchableFileAlert(ulong FileID, ulong ObjectID, string ObjectType, string ObjectName,
                                                 string Status, string SchemaID, string FieldName, string FileName, FileSystemInterface FileSystemInterfaceObj, ref string AlertXML)
        {
            OMError ErrObj = null;
            StringBuilder SBFileXML = null;
            try
            {
                SBFileXML = new StringBuilder();
                SBFileXML.Append("<FILE>");
                SBFileXML.AppendFormat("<FILEID>{0}</FILEID>", FileID);
                SBFileXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ObjectID);
                SBFileXML.AppendFormat("<OBJECTTYPE>{0}</OBJECTTYPE>", ObjectType);
                SBFileXML.AppendFormat("<OBJECTNAME>{0}</OBJECTNAME>", ObjectName);
                SBFileXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
                SBFileXML.AppendFormat("<SCHEMAID>{0}</SCHEMAID>", SchemaID);
                SBFileXML.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                //BugFix 6733: Special character handling for file name is required.
                if ((FileName != null) && (FileName != ""))
                    FileName = ALCommon.TrimAndReplaceSpecialChars(FileName);

                SBFileXML.AppendFormat("<FILENAME>{0}</FILENAME>", FileName);
                SBFileXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", FileSystemInterfaceObj.UseMediaServer.ToString().ToUpper());
                if (FileSystemInterfaceObj.UseMediaServer)
                    SBFileXML.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", FileSystemInterfaceObj.MediaServerUrl);

                SBFileXML.Append("</FILE>");
                AlertXML = SBFileXML.ToString();

                return null;
            }
            catch (Exception e)
            {
                ErrObj = new OMError(OMErrorCodes.READ_RECORD_FAILED,
                                     "Creating Input XML for ALENT_SEARCHABLE_FILE_ALERT Failed",
                                     OMErrorObjectType.FILESYSTEM, "FileSystemManager", OMErrorType.EXCEPTION, e, "");
                ErrObj.TransactionId = TransactionID.ToString();
                return ErrObj;
            }

        }

        /// <summary>
        /// <purpose>Forms input xml for updating searchable file alert (ALENT_SEARCHABLE_FILE_ALERT)</purpose>
        /// <author>Mr. Sagar Dasalkar</author>
        /// <CreatedDate>6 November 2009</CreatedDate>
        /// </summary>
        /// <param name="FileID">File Id for whic the searchable entry is to be updated.</param>
        /// <param name="FileSystemInterfaceObj">Interface to File System Manager.</param>
        /// <param name="AlertXML">Output parameter, xml for updating searchable file alert will be returned.</param>
        /// <returns>OMError if any error/exception occurs, otherwise null</returns>
        private OMError FormSearchableFileAlertForUpdate(ulong FileID, FileSystemInterface FileSystemInterfaceObj, ref string AlertXML)
        {
            OMError obj_OMError = null;
            StringBuilder sb_FileXml = null;
            try
            {
                //Prepare input xml for updating searchable alert for file. ONMEDIASERVER and MEDIASERVERURL are the only candidate for update.
                sb_FileXml = new StringBuilder(200);
                sb_FileXml.Append("<FILE>");
                sb_FileXml.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", FileSystemInterfaceObj.UseMediaServer.ToString().ToUpper());
                if (FileSystemInterfaceObj.UseMediaServer)
                    sb_FileXml.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", FileSystemInterfaceObj.MediaServerUrl);
                else
                    sb_FileXml.Append("<MEDIASERVERURL></MEDIASERVERURL>");
                sb_FileXml.Append("</FILE>");
                AlertXML = sb_FileXml.ToString();

                return null;
            }
            catch (Exception obj_Exception)
            {
                obj_OMError = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                     "Creating Input XML for update ALENT_SEARCHABLE_FILE_ALERT table failed",
                                     OMErrorObjectType.FILESYSTEM, "FileSystemManager", OMErrorType.EXCEPTION, obj_Exception, null);
                obj_OMError.TransactionId = TransactionID.ToString();
                return obj_OMError;
            }
        }

        private OMError FormFileXMLForUpdate(String FileName, string FileDisplayName, string Desc, ulong ModifiedBy, ref string FileXML)
        {
            StringBuilder SBFileXML = null;

            try
            {
                SBFileXML = new StringBuilder();
                SBFileXML.Append("<FILE>");

                if (!String.IsNullOrEmpty(FileName))
                {
                    FileName = ALCommon.TrimAndReplaceSpecialChars(FileName);
                    SBFileXML.AppendFormat("<NAME>{0}</NAME>", FileName);
                }

                if (!String.IsNullOrEmpty(FileDisplayName))
                {
                    FileDisplayName = ALCommon.TrimAndReplaceSpecialChars(FileDisplayName);
                    SBFileXML.AppendFormat("<DISPLAYNAME>{0}</DISPLAYNAME>", FileDisplayName);
                }

                if (!String.IsNullOrEmpty(Desc))
                {
                    Desc = ALCommon.TrimAndReplaceSpecialChars(Desc);
                    SBFileXML.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", Desc);
                }

                if (ModifiedBy != 0)
                {
                    SBFileXML.AppendFormat("<MODIFIEDBY>{0}</MODIFIEDBY>", ModifiedBy);
                    SBFileXML.AppendFormat("<MODIFIEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></MODIFIEDON>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                }
                SBFileXML.Append("</FILE>");
                FileXML = SBFileXML.ToString();
                return null;
            }
            catch (Exception ex)
            {

                throw ex;
            }

        }

        private OMError FormDurableLogXML(ulong TransactionID, string Operation, FileSystemInterface FileSystemInterfaceObj, string ParameterXML, string Status, ref string DurableLogXML)
        {
            StringBuilder LogXML = null;

            try
            {
                LogXML = new StringBuilder();

                LogXML.Append("<INPUTXML>");
                LogXML.AppendFormat("<TRANSACTIONID>{0}</TRANSACTIONID>", TransactionID);
                LogXML.AppendFormat("<OPERATION>{0}</OPERATION>", Operation);
                LogXML.AppendFormat("<PARAMETERINFO>{0}</PARAMETERINFO>", ParameterXML);
                LogXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", FileSystemInterfaceObj.UseMediaServer.ToString().ToUpper());
                if (FileSystemInterfaceObj.UseMediaServer)
                    LogXML.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", FileSystemInterfaceObj.MediaServerUrl);
                LogXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
                LogXML.AppendFormat("<ISXMLOPERATION>0</ISXMLOPERATION>");
                LogXML.AppendFormat(@"<CREATEDON> 
                                                <YEAR>{0}</YEAR> 
                                                <MONTH>{1}</MONTH> 
                                                <DAY>{2}</DAY> 
                                                <HOUR>{3}</HOUR> 
                                                <MINUTE>{4}</MINUTE> 
                                                <SECOND>{5}</SECOND> 
                                                </CREATEDON>", DateTime.Now.Year, DateTime.Now.Month,
                                                DateTime.Now.Day, DateTime.Now.Hour,
                                                DateTime.Now.Minute, DateTime.Now.Second);


                LogXML.Append("</INPUTXML>");
                DurableLogXML = LogXML.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError FormDurableLogXMLForUpdate(string Status, string strInfo, ref string DurableLogXML)
        {
            StringBuilder LogXML = null;

            try
            {
                LogXML = new StringBuilder();

                LogXML.Append("<INPUTXML>");
                LogXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
                LogXML.AppendFormat("<MOREINFO>{0}</MOREINFO>", strInfo);

                LogXML.AppendFormat(@"<MODIFIEDON> 
                                                <YEAR>{0}</YEAR> 
                                                <MONTH>{1}</MONTH> 
                                                <DAY>{2}</DAY> 
                                                <HOUR>{3}</HOUR> 
                                                <MINUTE>{4}</MINUTE> 
                                                <SECOND>{5}</SECOND> 
                                                </MODIFIEDON>", DateTime.Now.Year, DateTime.Now.Month,
                                                DateTime.Now.Day, DateTime.Now.Hour,
                                                DateTime.Now.Minute, DateTime.Now.Second);

                LogXML.Append("</INPUTXML>");
                DurableLogXML = LogXML.ToString();
                return null;
            }
            catch (Exception)
            {

                throw;
            }

        }

        private OMError FormPamareterXML(StringCollection Parameters, ref string ParameterXML)
        {
            StringBuilder OutputXML = null;
            String value = null;

            try
            {
                OutputXML = new StringBuilder();
                OutputXML.Append("<PARAMETERS>");
                foreach (string Parameter in Parameters)
                {
                    value = Assetlink.ALCommon.TrimAndReplaceSpecialChars(Parameter);
                    OutputXML.AppendFormat("<PARAM>{0}</PARAM>", value);
                }
                OutputXML.Append("</PARAMETERS>");
                ParameterXML = OutputXML.ToString();
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private OMError FormFolderXML(ulong FolderID, string Path, int NoOfFiles, string Status, DateTime CreationDate, string MountPoint, FileSystemInterface FileSystemInterfaceObj, ref string FolderXML)
        {
            StringBuilder FolderQueryXML = null;

            FolderQueryXML = new StringBuilder();
            FolderQueryXML.Append("<ALFS_FOLDER>");
            FolderQueryXML.AppendFormat("<FOLDERID>{0}</FOLDERID>", FolderID);
            FolderQueryXML.AppendFormat("<PATH>{0}</PATH>", ALCommon.TrimAndReplaceSpecialChars(Path));
            FolderQueryXML.AppendFormat("<NOOFFILES>{0}</NOOFFILES>", NoOfFiles);
            FolderQueryXML.AppendFormat("<STATUS>{0}</STATUS>", Status);
            FolderQueryXML.AppendFormat("<MOUNTPOINT>{0}</MOUNTPOINT>", ALCommon.TrimAndReplaceSpecialChars(MountPoint));
            FolderQueryXML.Append("<CREATIONDATE>");
            FolderQueryXML.AppendFormat("<DAY>{0}</DAY>", CreationDate.Day);
            FolderQueryXML.AppendFormat("<MONTH>{0}</MONTH>", CreationDate.Month);
            FolderQueryXML.AppendFormat("<YEAR>{0}</YEAR>", CreationDate.Year);
            FolderQueryXML.AppendFormat("<HOUR>{0}</HOUR>", CreationDate.Hour);
            FolderQueryXML.AppendFormat("<MINUTE>{0}</MINUTE>", CreationDate.Minute);
            FolderQueryXML.AppendFormat("<SECOND>{0}</SECOND>", CreationDate.Second);
            FolderQueryXML.Append("</CREATIONDATE>");
            FolderQueryXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", FileSystemInterfaceObj.UseMediaServer.ToString().ToUpper());
            if (FileSystemInterfaceObj.UseMediaServer)
                FolderQueryXML.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", FileSystemInterfaceObj.MediaServerUrl);
            FolderQueryXML.Append("</ALFS_FOLDER>");

            FolderXML = FolderQueryXML.ToString();
            return null;
        }

        private OMError GetFolderPath(ref string FolderPath, FileSystemInterface FileSystemInterfaceObj)
        {
            string FolderXML = null;
            XmlDocument FolderDocument = null;
            ulong FolderID = 0;
            ulong NoOfFiles = 0;
            StringBuilder Query = null;
            StringBuilder Path = null;
            OMError ErrorObj = null;
            StringCollection Parameters = null;
            string ParameterXML = null;
            string DurableLogXML = null;
            String TempMountPoint = null;
            IFileSystem IFileSystemObj = null;

            try
            {
                IFileSystemObj = FileSystemInterfaceObj.IFileSystemInstance;

                FolderDocument = new XmlDocument();
                Query = new StringBuilder();
                Path = new StringBuilder();
                Parameters = new StringCollection();

                if ((UserProvidedMountPoint != null) && (UserProvidedMountPoint.Length > 0))
                {
                    TempMountPoint = UserProvidedMountPoint;
                    UserProvidedMountPoint = null;
                }
                else
                {
                    TempMountPoint = MountPoint;
                }

                Monitor.Enter(Lock);
                if (MaxFileCount > 0)
                {
                    if ((ErrorObj = FILEDMObj.DataProvider.AddClause("MOUNTPOINT", OMFieldDataType.STRING, OMOperator.EQUALTO, TempMountPoint, OMConjunction.AND)) == null)
                    {
                        if ((ErrorObj = FILEDMObj.DataProvider.AddClause("NOOFFILES", OMFieldDataType.INTEGER, OMOperator.LESSTHAN, MaxFileCount.ToString(), OMConjunction.NOTHING)) == null)
                        {

                            if ((ErrorObj = FILEDMObj.DataProvider.ReadRecord("ROOT", "ALFS_FOLDER", null, ref FolderXML)) == null)
                            {
                                if (FolderXML != null)
                                {
                                    FolderDocument.LoadXml(FolderXML);
                                    FolderID = Convert.ToUInt64(FolderDocument.SelectSingleNode("//FOLDERID").InnerText);
                                    FolderPath = FolderDocument.SelectSingleNode("//PATH").InnerText;
                                    if (!IFileSystemObj.IsFolderExists(FolderPath))
                                    {
                                        IFileSystemObj.CreateFolder(FolderPath);
                                        //FormFolderXML(FolderID, FolderPath, 1, "AVAILABLE", DateTime.Now, TempMountPoint, ref FolderXML);
                                        //if ((ErrorObj = FILEDMObj.DataProvider.CreateRecord("ALFS_FOLDER", FolderXML)) != null)
                                        //{
                                        //    FolderPath = null;
                                        //    // Return Error.
                                        //}

                                        // Log into DurableLog file
                                        Parameters.Add(FolderPath.ToString());
                                        FormPamareterXML(Parameters, ref ParameterXML);
                                        FormDurableLogXML(TransactionID, "CreateFolder", FileSystemInterfaceObj, ParameterXML, "", ref DurableLogXML);
                                        if ((ErrorObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                                        {
                                            return ErrorObj;
                                        }
                                    }

                                    NoOfFiles = Convert.ToUInt64(FolderDocument.SelectSingleNode("//NOOFFILES").InnerText);
                                    NoOfFiles++;
                                    if (NoOfFiles < MaxFileCount)
                                    {
                                        Query.AppendFormat("Update ALFS_FOLDER set NOOFFILES = {0} where FOLDERID = {1}", NoOfFiles, FolderID);
                                    }
                                    else
                                    {
                                        Query.AppendFormat("Update ALFS_FOLDER set NOOFFILES = {0},STATUS = 'FULL' where FOLDERID = {1}", NoOfFiles, FolderID);
                                    }
                                    if ((ErrorObj = FILEDMObj.DataProvider.Execute(Query.ToString())) != null)
                                    {
                                        FolderPath = null;
                                        ErrorObj.TransactionId = TransactionID.ToString();
                                        return ErrorObj;
                                        // Return Error.
                                    }

                                }
                                else
                                {
                                    // All Folders are full or no folder created uptill now
                                    UniqueID.Read(ref FolderID);
                                    Path.AppendFormat("{0}\\{1}", TempMountPoint, FolderID);
                                    FormFolderXML(FolderID, Path.ToString(), 1, "AVAILABLE", DateTime.Now, TempMountPoint, FileSystemInterfaceObj, ref FolderXML);
                                    if ((ErrorObj = FILEDMObj.DataProvider.CreateRecord("ALFS_FOLDER", FolderXML)) != null)
                                    {
                                        FolderPath = null;
                                        ErrorObj.TransactionId = TransactionID.ToString();
                                        return ErrorObj; 
                                        // Return Error.
                                    }
                                    IFileSystemObj.CreateFolder(Path.ToString());
                                    FolderPath = Path.ToString();

                                    // Log into DurableLog file
                                    Parameters.Add(Path.ToString());
                                    FormPamareterXML(Parameters, ref ParameterXML);
                                    FormDurableLogXML(TransactionID, "CreateFolder", FileSystemInterfaceObj, ParameterXML, "", ref DurableLogXML);
                                    if ((ErrorObj = LOGDMObj.DataProvider.CreateRecord("ALFS_DURABLELOG", DurableLogXML)) != null)
                                    {
                                        ErrorObj.TransactionId = TransactionID.ToString();
                                        return ErrorObj; // Return error.
                                        // Return error.
                                    }

                                }
                            }

                        }
                    }

                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
            finally
            {
                Monitor.Exit(Lock);
            }


        }

        private OMError InitializeConfigInfo(out FileSystemInterface LocalFileSystemInterfaceObj, out FileSystemInterface RemoteFileSystemInterfaceObj, out String DefaultMediaServerUrl)
        {
            string InstalledPath = null;
            StringBuilder AppXMLPath = null;
            XmlDocument TenantXMLDocument = null;
            string AssemblyName = null;
            Type ClassType = null;
            StringBuilder AssemblyPath = null;
            Assembly LoaclAssembly = null;

            XmlNode MediaServerNode = null;
            StringBuilder RemoteAssemblyPath = null;
            String RemoteAssemblyName = null;
            String RemoteServerUrl = null;
            Assembly RemoteAssembly = null;

            IFileSystem ILocalFileSystemObj = null;
            IFileSystem IRemoteFileSystemObj = null;

            LocalFileSystemInterfaceObj = null;
            RemoteFileSystemInterfaceObj = null;
            DefaultMediaServerUrl = null;

            try
            {
                InstalledPath = RegistrySettings.ReadValue(@"SOFTWARE\ASSETLINK",
                                                                    "Installed Path");
                AppXMLPath = new StringBuilder();
                AppXMLPath.AppendFormat("{0}", ALCommon.GetTenantXMLPath(m_TenantName));

                TenantXMLDocument = new XmlDocument();
                TenantXMLDocument.Load(AppXMLPath.ToString());

                if (TenantXMLDocument.SelectNodes("CONFIGURATION/FILESYSTEM/MOUNTPOINT").Count > 0)
                    MountPoint = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/MOUNTPOINT").InnerText;
                else
                    MountPoint = TenantXMLDocument.SelectSingleNode("Configuration/FileSystem/MountPoint").InnerText;

                if (TenantXMLDocument.SelectNodes("CONFIGURATION/FILESYSTEM/TYPE").Count > 0)
                {
                    FileSystemType = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/TYPE").InnerText;
                }
                else if (TenantXMLDocument.SelectNodes("Configuration/FileSystem/Type").Count > 0)
                {
                    FileSystemType = TenantXMLDocument.SelectSingleNode("Configuration/FileSystem/Type").InnerText;
                }
                else
                    FileSystemType = "WINDOWS";

                if (TenantXMLDocument.SelectNodes("CONFIGURATION/FILESYSTEM/MAXFILECOUNT").Count > 0)
                {
                    MaxFileCount = Convert.ToUInt64(TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/MAXFILECOUNT").InnerText);
                }
                else if (TenantXMLDocument.SelectNodes("Configuration/FileSystem/MaxFileCount").Count > 0)
                {
                    MaxFileCount = Convert.ToUInt64(TenantXMLDocument.SelectSingleNode("Configuration/FileSystem/MaxFileCount").InnerText);
                }
                else
                {
                    MaxFileCount = 0;
                }

                if (TenantXMLDocument.SelectNodes("CONFIGURATION/FILESYSTEM/OUTBOX").Count > 0)
                    OutBox = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/OUTBOX").InnerText;
                else
                    OutBox = TenantXMLDocument.SelectSingleNode("Configuration/FileSystem/OutBox").InnerText;

                #region creating local FileSystem Obj
                AssemblyPath = new StringBuilder();
                if (TenantXMLDocument.SelectNodes("CONFIGURATION/FILESYSTEM/ASSEMBLYNAME").Count > 0)
                {
                    AssemblyName = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/ASSEMBLYNAME").InnerText;
                }
                else
                {
                    AssemblyName = TenantXMLDocument.SelectSingleNode("Configuration/FileSystem/AssemblyName").InnerText;
                }
                AssemblyPath.AppendFormat("{0}Binaries\\{1}", InstalledPath, AssemblyName);
                LoaclAssembly = Assembly.LoadFrom(AssemblyPath.ToString());

                ClassType = GetAssemblyType(LoaclAssembly);
                ILocalFileSystemObj = (IFileSystem)Activator.CreateInstance(ClassType);
                LocalFileSystemInterfaceObj = new FileSystemInterface(ILocalFileSystemObj, false, null);
                #endregion

                #region Creating Remote FileSystem Obj
                RemoteAssemblyPath = new StringBuilder();
                MediaServerNode = TenantXMLDocument.SelectSingleNode("CONFIGURATION/FILESYSTEM/MEDIASERVER");
                if (MediaServerNode != null)
                {
                    #region Reading the Settings
                    RemoteAssemblyName = MediaServerNode.SelectSingleNode("ASSEMBLYNAME").InnerText;
                    RemoteServerUrl = MediaServerNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                    RemoteAssemblyPath.AppendFormat("{0}Binaries\\{1}", InstalledPath, RemoteAssemblyName);
                    DefaultMediaServerUrl = RemoteServerUrl;
                    #endregion

                    #region creating the Instance
                    RemoteAssemblyName = RemoteAssemblyName.ToUpper().Substring(0, RemoteAssemblyName.IndexOf("."));
                    RemoteAssembly = Assembly.LoadFrom(RemoteAssemblyPath.ToString());
                    ClassType = GetAssemblyType(RemoteAssembly);
                    IRemoteFileSystemObj = (IFileSystem)Activator.CreateInstance(ClassType, new object[3] { UserID, m_TenantName, RemoteServerUrl });
                    #endregion
                }
                else
                    IRemoteFileSystemObj = null;

                RemoteFileSystemInterfaceObj = new FileSystemInterface(IRemoteFileSystemObj, true, RemoteServerUrl);
                #endregion
                return null;
            }
            catch (Exception ExceptionObj)
            {

                throw ExceptionObj;
            }
        }

        private Type GetAssemblyType(Assembly Assembly)
        {
            foreach (Type ClassType in Assembly.GetTypes())
            {
                if (ClassType.GetInterface("IFileSystem") != null)
                    return ClassType;
            }
            return null;
        }

        private FileSystemInterface GetFileSystemInterface(XmlNode FileNode)
        {
            #region Important Comment
            /* Comment details
             * Author:  Mr Sagar Dasalkar
             * Date:    11 August 2009
             * 
             * ONMEDIASERVER can be specified to following
             * 1. As an attribute to FILE node for Copy/Move file operations (newly introduced)
             * 2. As a child node of FILE node for Create file operation
             * 
             * In most of the cases, value of ONMEDIASERVER node decides whether the operation is to be performed on Media Server or OM.
             * But, consider following case
             * Move file Case: OM and File System Manager are enhanced to support Move file operation. In this case, solution needs to specify following
             * 1. FILEID node (to specify which file is to be moved)
             * 2. FIELDNAME node 
             * 
             * In this case File System Manager will read FILE details from database using given FILEID. Now, consider a case where file resides on 
             * OM Mount Point, and it is to be moved to Media Server Mount Point. Here, since the file is sitting under OM Mount Point FILE node will 
             * have ONMEDIASERVER node with value as FALSE. OM Mount Point is the source and destination is Media Server Mount Point. 
             * For this case, destination is to be identified to decide whether the file is to be moved to Media Server Mount Point. So, ONMEDIASERVER attribute 
             * (given to FILE node) should be considered instead of ONMEDIASERVER node. So the implementation of GetFileSystemInterface() is changed to read ONMEDIASERVER attribute 
             * (from FILE node) first and take a decision of returning RemoteFileSystemInterface/LocalFileSystemInterface object.
             * If ONMEDIASERVER attribute is not specified to FILE node, then ONMEDIASERVER node will be read and decision will be taken depending
             * on its value.
             * */
            #endregion Important Comment

            bool b_IsMediaServer = false;        //Used to read ONMEDIASERVER attribute value

            //Read OnMediaServer attribute value if attribute is specified to FILE node
            if (FileNode.Attributes.GetNamedItem("OnMediaServer") != null)
            {
                Boolean.TryParse(FileNode.Attributes.GetNamedItem("OnMediaServer").Value, out b_IsMediaServer);

                if (b_IsMediaServer)
                {
                    /* Since MEDIASERVERURL is not available for existing file (since file is sitting under OM Mount Point), default 
                     * media server URL will be considered.
                    */
                    RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;
                    return RemoteFileSystemInterfaceObj;
                }
            }
            else if (FileNode.SelectSingleNode("ONMEDIASERVER") != null && FileNode.SelectSingleNode("ONMEDIASERVER").InnerText.ToUpper() == "TRUE")
            {
                //Read ONMEDIASERVER node value if node is specified under FILE node
                if (FileNode.SelectSingleNode("MEDIASERVERURL") != null && !String.IsNullOrEmpty(FileNode.SelectSingleNode("MEDIASERVERURL").InnerText))
                    RemoteFileSystemInterfaceObj.MediaServerUrl = FileNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                else
                    RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;

                return RemoteFileSystemInterfaceObj;
            }

            return LocalFileSystemInterfaceObj;
        }

        /// <summary>
        /// <purpose>
        /// Returns Interface to File System.
        /// </purpose>
        /// <CreatedDate>6 November 2009</CreatedDate>
        /// <author>Mr. Sagar Dasalkar</author>
        /// </summary>
        /// <param name="ignoreMediaServerAttr">Specifies whether to ignore OnMediaServer attribute specified to FILE node.</param>
        /// <param name="FileNode">File Node containing file details.</param>
        /// <returns>Interface to File System.</returns>
        private FileSystemInterface GetFileSystemInterface(bool IgnoreOnMediaServerAttr, XmlNode FileNode)
        {
            if (IgnoreOnMediaServerAttr)
            {
                if (FileNode.SelectSingleNode("ONMEDIASERVER") != null && FileNode.SelectSingleNode("ONMEDIASERVER").InnerText.ToUpper() == "TRUE")
                {
                    //Read ONMEDIASERVER node value if node is specified under FILE node
                    if (FileNode.SelectSingleNode("MEDIASERVERURL") != null && !String.IsNullOrEmpty(FileNode.SelectSingleNode("MEDIASERVERURL").InnerText))
                        RemoteFileSystemInterfaceObj.MediaServerUrl = FileNode.SelectSingleNode("MEDIASERVERURL").InnerText;
                    else
                        RemoteFileSystemInterfaceObj.MediaServerUrl = DefaultMediaServerUrl;

                    return RemoteFileSystemInterfaceObj;
                }
                return LocalFileSystemInterfaceObj;
            }
            return GetFileSystemInterface(FileNode);
        }

        private Boolean CheckForMediaServer(XmlNode FileNode)
        {
            if (FileNode.SelectSingleNode("ONMEDIASERVER") != null)
            {
                if (FileNode.SelectSingleNode("ONMEDIASERVER").InnerText.ToUpper() == "TRUE")
                    return true;
            }
            return false;
        }

        private class FileSystemInterface
        {
            public IFileSystem IFileSystemInstance;
            public Boolean UseMediaServer;

            String m_MediaServerUrl;
            public String MediaServerUrl
            {
                set
                {
                    IFileSystemInstance.MediaServerUrl = value;
                    m_MediaServerUrl = value;
                }

                get
                {
                    return m_MediaServerUrl;
                }
            }

            public FileSystemInterface(IFileSystem IFileSystemObj, Boolean UseMediaServer, String MediaServerUrl)
            {
                this.IFileSystemInstance = IFileSystemObj;
                this.UseMediaServer = UseMediaServer;
                this.MediaServerUrl = MediaServerUrl;
            }
        }

        private string GetShadowFolderPath(string strFolderPath, XmlNode FileNode)
        {
            string strShadowFolder = string.Empty;
            OMError ErrObj = null;
            IFileSystem IFileSystemObj = null;
            FileSystemInterface FileSystemInterfaceObj = null;

            FileSystemInterfaceObj = GetFileSystemInterface(FileNode);
            IFileSystemObj = FileSystemInterfaceObj.IFileSystemInstance;
            if (!string.IsNullOrEmpty(strFolderPath))
            {
                //if (strFolderPath.Contains(@""))
                //{
                strShadowFolder = strFolderPath.Substring(0, strFolderPath.LastIndexOf(@"\")) + @"\" + SHADOW_FOLDER;
                //}
                //else
                //    strShadowFolder = MountPoint + @"\" + SHADOW_FOLDER;
            }
            else
            {
                if (FileNode.Attributes.GetNamedItem("OnMediaServer") != null)
                {
                    bool b_IsMediaServer = false;
                    Boolean.TryParse(FileNode.Attributes.GetNamedItem("OnMediaServer").Value, out b_IsMediaServer);
                    if (b_IsMediaServer)
                    {
                        strShadowFolder = ALCommon.GetInstallationDir() + SYSYTEM_MOUNT_POINT;
                    }
                    else
                        strShadowFolder = MountPoint + @"\" + SHADOW_FOLDER;
                }
                else if (FileNode.SelectSingleNode("ONMEDIASERVER") != null && FileNode.SelectSingleNode("ONMEDIASERVER").InnerText.ToUpper() == "TRUE")
                {
                    strShadowFolder = ALCommon.GetInstallationDir() + SYSYTEM_MOUNT_POINT;
                }
                else
                {
                    strShadowFolder = MountPoint + @"\" + SHADOW_FOLDER;
                }
            }
            if (!IFileSystemObj.IsFolderExists(strShadowFolder))
            {
                try
                {
                    StringCollection Parameters = new StringCollection();
                    Parameters.Clear();
                    Parameters.Add(strShadowFolder);
                    ErrObj = PerformOperation("CreateFolder", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance);
                }
                catch (Exception ex)
                {
                    //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetShadowFolderPath()"",""Exception in Creating SHADOW FOLDER:" + ex.Message.Replace(@"""", @"""""") + @"""");
                    throw ex;
                }
            }

            strShadowFolder = strShadowFolder + @"\" + TransactionID;
            if (!IFileSystemObj.IsFolderExists(strShadowFolder))
            {
                try
                {
                    StringCollection Parameters = new StringCollection();
                    Parameters.Clear();
                    Parameters.Add(strShadowFolder);
                    ErrObj = PerformOperation("CreateFolder", Parameters, GetFileSystemInterface(FileNode).IFileSystemInstance);
                }
                catch (Exception ex)
                {
                    //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""GetShadowFolderPath()"",""Exception in Creating SHADOW FOLDER:" + ex.Message.Replace(@"""", @"""""") + @"""");
                    throw ex;
                }
            }

            return strShadowFolder;

            // MAKE CREATEFOLDER CALL TO CREATE SHADOW FOLDER
        }

        private OMError UpdateDurableLogStatus(string LogId, string Status, string strInfo)
        {
            OMError ErrorObj = null;
            try
            {
                String strUpdateXML = string.Empty;
                FormDurableLogXMLForUpdate(Status, strInfo, ref strUpdateXML);
                if ((ErrorObj = LOGDMObj.DataProvider.AddClause("LOGID", OMFieldDataType.INTEGER, OMOperator.EQUALTO, LogId, OMConjunction.NOTHING)) == null)
                {
                    if ((ErrorObj = LOGDMObj.DataProvider.UpdateRecord("ALFS_DURABLELOG", strUpdateXML)) != null)
                    {
                        ErrorObj.TransactionId = TransactionID.ToString();
                        return ErrorObj;
                    }
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                //WriteLog(m_TenantName, "INSTR", System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), System.Threading.Thread.CurrentThread.ManagedThreadId.ToString(), @"""" + System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + @""",""" + DateTime.Now.ToString("MM/dd/yyyy HH:MM:ss.fff") + @""",""UpdateDurableLogStatus()"",""Exception in UpdateDurableLogStatus() for LogId:" + LogId + " and Status:" + Status + " with Exception:" + ExceptionObj.Message.Replace(@"""", @"""""") + @"""");
                throw ExceptionObj;
            }

        }
    }
}
