# region System Includes

using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.XPath;
using System.Configuration;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Data;
using System.Data.SqlClient;
using Teradata.Client.Provider;
using Oracle.DataAccess.Client; 
using System.Web.Services;
using System.Web;
using System.Drawing;
using System.IO.Packaging;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;

# endregion

# region Assetlink Includes

using Assetlink;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.Services;
using Assetlink.Services.Engines.InfrastructureEngine;
using Assetlink.WebServices.ALMSOfficeProcessorWS;

# endregion

namespace Assetlink.Services.Engines.ReportEngine
{
    using InputXML  = System.String;
    using ERRORXML  = System.String;
    using XML       = System.String;

    class FieldInfo
    {
        public String RelationShipName;
        public String ObjectName;
        public String FieldName;
        public String DataType;
    }

    public class Report_Engine: ALBaseEngine
    {
        # region Member Variables
        private ALEngineError   ALEngineErrorObj    = null;
        private XML             OperationXML        = null;
        private ERRORXML        ErrorXML            = null;
        private ERRORXML        ReturnErrorXML      = null;
        private OperationQueue  QMQueueObj          = null;
        private Operation       OperObj             = null;
        private OMInterface     OMIObj              = null;

        private String          m_DatabaseType          = null;
        private String          m_ConnectionString      = null;
        private String          m_ConnectionStringSP    = null;
        private String          strOrderByClause        = null;
        private String          LogRptPath              = null;

        Func<List<String>, XElement, List<String>>  AggregateValues     = null;
        Func<String, decimal>                       ConvertToDecimal    = null;
        Func<List<String>, decimal>                 GetSum              = null;
        Func<List<String>, decimal>                 GetMax              = null;
        Func<List<String>, decimal>                 GetMin              = null;
        Func<List<String>, decimal>                 GetAvg              = null;
        Func<List<String>, decimal>                 GetCount            = null;
        Func<GroupByQueryResult, String>            groupSortFunc       = null;

        String                  str_FormatXMLPath                       = null;
        String                  str_CultureInfo                         = null;
        CultureInfo             customculture_Object                    = null;
        # endregion

        #region Member Classes
        class GroupByQueryResult
        {
            public String GroupByColumnValue { get; set; }
            public IGrouping<string, XElement> groupData { get; set; }
        }

        class AggregationConfig
        {
            public String ColumnName { get; set; }
            public String NodeName { get; set; }
            public String RelationshipName { get; set; }
            public String FieldDataType { get; set; }
            public String AggregationFunc { get; set; }
        }

        class SortConfig
        {
            public String ColumnName { get; set; }
            public String NodeName { get; set; }
            public String RelationshipName { get; set; }
            public String FieldDataType { get; set; }
            public String SortOrder { get; set; }
            public String GroupByLevel { get; set; }
        }

        class Constants
        {
            public const string DATA_TYPE_TEXT      = "TEXT";
            public const string DATA_TYPE_NUMBER    = "NUMBER";
            public const string DATA_TYPE_FLOAT     = "FLOAT";
            public const string DATA_TYPE_CURRENCY  = "CURRENCY";
            public const string DATA_TYPE_DATE      = "DATE";
            public const string DATA_TYPE_DATETIME  = "DATETIME";
            public const string DATA_TYPE_MULTILINE = "MULTILINE";
            public const string SORTORDER_ASC       = "ASCENDING";
            public const string SORTORDER_DESC      = "DESCENDING";
            public const string AGGREGATEFUNC_SUM   = "SUM";
            public const string AGGREGATEFUNC_MAX   = "MAX";
            public const string AGGREGATEFUNC_MIN   = "MIN";
            public const string AGGREGATEFUNC_AVG   = "AVG";
            public const string AGGREGATEFUNC_COUNT = "COUNT";
        }

        class XElementComparer : IEqualityComparer<XElement>
        {
            private String PK = "";

            public XElementComparer(string PKVal)
            {
                PK = PKVal;
            }

            public bool Equals(XElement x, XElement y)
            {
                String xKeyVal = x.Element(PK).Value;
                String yKeyVal = y.Element(PK).Value;

                if (xKeyVal == yKeyVal)
                    return true;
                return false;
            }

            public int GetHashCode(XElement xEle)
            {
                int hCode = 0;
                return hCode.GetHashCode();
            }
        }
        #endregion

        #region Constructors

        public Report_Engine() : base ()
        {
            ALEngineErrorObj = new ALEngineError();
        }

        public Report_Engine(UInt64 UserID, String Language, String TenantName)
            : base(UserID, Language, TenantName)
        {
            ALEngineErrorObj = new ALEngineError();
        }

        public Report_Engine(UInt64 SessionID, UInt64 UserID, String Language, String TenantName)
            : base(SessionID, UserID, Language, TenantName)
        {
            ALEngineErrorObj = new ALEngineError();
        }

        public Report_Engine(UInt64 SessionID, UInt64 UserID, String Language, String TenantName, String ReportType)
            : base(SessionID, UserID, Language, TenantName)
        {
            if (ReportType.ToUpper() == "CRYSTAL REPORT")
            {
                XPathDocument XpDoc_TenantXML = null;
                XPathNavigator XpNav_Tenant = null;
                StringBuilder AppXMLPath = new StringBuilder();

                ALEngineErrorObj = new ALEngineError();

                // Load Tenant ConfigurationXML
                AppXMLPath.Append(ALCommon.GetTenantXMLPath(TenantName));
                XpDoc_TenantXML = new XPathDocument(AppXMLPath.ToString());
                XpNav_Tenant = XpDoc_TenantXML.CreateNavigator();

                // Set Class Variables
                this.m_DatabaseType = XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/DATABASETYPE").Value.ToUpper();
                this.m_ConnectionString = Assetlink.ALCommon.Decrypt(XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRING").Value);
                this.m_ConnectionStringSP = Assetlink.ALCommon.Decrypt(XpNav_Tenant.SelectSingleNode("CONFIGURATION/DATABASE/CONNECTIONSTRINGFORSP").Value);
            }
        }

        #endregion

        # region Member/Public Functions For Reports

        /// <summary>
        /// <CreatedBy>Nachiket V. Deshpande</CreatedBy>
        /// <CreatedDate>08 Feburary 2007</CreatedDate>
        /// <Purpose>To add an entry of new report into the database.</Purpose>
        /// <returns> 1: Nothing on Successful creation of the record.
        ///           2: ErrorXML on Failure to create the record
        /// </returns>
        /// </summary>
        public ERRORXML CreateReport(XML InputXML)
        {

            #region Local Variables
            AuthorizationHelperClass    obj_AuthorizationClass      = null;
            String                      AuthorziationXML            = null;
            String                      ReportID                    = null;
            #endregion

            try
            {
                # region Add new report entry into database
                
                QMQueueObj      = new OperationQueue(false);
                OperObj         = new Operation("ENTITY", "CREATEDATA");

                OperObj.AddParameter("INPUTXML", InputXML);
                QMQueueObj.AddOperation(OperObj);

                OperationXML    = QMQueueObj.Xml;
                OMIObj          = new OMInterface(UserID, Language, TenantName);
                ErrorXML        = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_CREATEREPORT_REPORTCREATIONFAILED", this.GetType().ToString(), "CreateReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                # endregion

                #region Adding Authorization Entry
                ReportID = ALCommon.GetInnerValue(ref OperationXML, "<REPORTID>", "</REPORTID>");
                if (!String.IsNullOrEmpty(ReportID))
                {
                    AuthorziationXML        = "<AUTHORIZATION><GROUPPRIVILEGES><GROUP><GROUPID>104</GROUPID><PRIVILEGENAME>DELETE</PRIVILEGENAME></GROUP></GROUPPRIVILEGES></AUTHORIZATION>";
                    obj_AuthorizationClass  = new AuthorizationHelperClass(SessionID, UserID, Language, TenantName);
                    ErrorXML                = obj_AuthorizationClass.AddAuthorizationEntries(AuthorziationXML, ReportID, "ENTITY", "ANALYTICALREPORT");
                }
                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_CREATEREPORT_AUTHORIZATIONCREATIONFAILED", this.GetType().ToString(), "CreateReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
                #endregion
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_CREATEREPORT_REPORTCREATIONFAILED", this.GetType().ToString(), "CreateReport", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return null;
        }
        
        /// <summary>
        /// <CreatedBy>Nachiket V. Deshpande</CreatedBy>
        /// <CreatedDate>08 Feburary 2007</CreatedDate>
        /// <Purpose>To delete an existing entry of report from the database</Purpose>
        /// <returns>   1: Nothing on Successful deletion of the record.
        ///             2: ErrorXML on failuer of the delete operation.
        /// </returns>
        /// </summary>
        public ERRORXML DeleteReport(String ReportID)
        {

            try
            {
                # region Delete report entry from database

                QMQueueObj      = new OperationQueue(false);
                OperObj         = new Operation("ENTITY", "DELETE");
                OperObj.AddParameter("ENTITYID", ReportID);
                OMIObj          = new OMInterface(UserID, Language, TenantName); 
                
                OperObj.AddParameter("ENTITYTYPENAME", "ANALYTICALREPORT");
                QMQueueObj.AddOperation(OperObj);

                OperationXML    = QMQueueObj.Xml;
                ErrorXML        = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_DELETEREPORT_REPORTDELETIONFAILED", this.GetType().ToString(), "DeleteReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                # endregion
            }

            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_DELETEREPORT_REPORTDELETIONFAILED", this.GetType().ToString(), "DeleteReport", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return null;
        }
        
        /// <summary>
        /// <CreatedBy>Nachiket V. Deshpande</CreatedBy>
        /// <CreatedDate>08 Feburary 2007</CreatedDate>
        /// <Purpose>To update an existing Report entry into the database</Purpose>
        /// <returns> 1: All entities in XML format.
        ///           2: ErrorXML on Failure
        /// </returns>
        /// </summary>
        public ERRORXML UpdateReport(XML InputXML)
        {
            try
            {
                # region Update report details into database

                QMQueueObj      = new OperationQueue(false);
                OperObj         = new Operation("ENTITY", "UPDATEDATA");
                
                OperObj.AddParameter("INPUTXML", InputXML);
                QMQueueObj.AddOperation(OperObj);
                OperationXML    = QMQueueObj.Xml;

                OMIObj          = new OMInterface(UserID, Language, TenantName);
                ErrorXML        = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORT_REPORTUPDATIONFAILED", this.GetType().ToString(), "UpdateReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }

                # endregion
            }

            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORT_REPORTUPDATIONFAILED", this.GetType().ToString(), "UpdateReport", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return null;
        }

        # endregion

        #region Member/Public Functions For AdHoc Reports

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to create the new instance for AdHocReport Entity.</Purpose>
        /// <returns>On Success it will create a record into the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ERRORXML CreateAdHocReport(ref XML AdHocReportInputXML)
        {
            XPathDocument   xpathdoc_OperationXML   = null;
            XPathNavigator  xpathnav_OperationXML   = null;
            XPathNavigator  xpathnav_ParamInputXML  = null;
            XmlDocument     AdHocReportXMLDoc       = null;
            XmlNode         AdHocReportNode         = null;
            

            try
            {
                AdHocReportXMLDoc = new XmlDocument();
                AdHocReportXMLDoc.LoadXml(AdHocReportInputXML);

                AdHocReportNode = AdHocReportXMLDoc.DocumentElement;

                Common.PrepareDateTimeNode(ref AdHocReportXMLDoc, AdHocReportNode, "CREATEDON");
                Common.PrepareDateTimeNode(ref AdHocReportXMLDoc, AdHocReportNode, "LASTUPDATEDON");
                Common.PrepareRelationNode(ref AdHocReportXMLDoc, AdHocReportNode, "ADHOCREPORT_CREATEDBY", UserID.ToString(), false, false);
                Common.PrepareRelationNode(ref AdHocReportXMLDoc, AdHocReportNode, "ADHOCREPORT_MODIFIEDBY", UserID.ToString(), false, false);

                ReturnErrorXML = null;
                OperObj = new Operation("ENTITY", "CREATEDATA");
                OperObj.AddParameter("INPUTXML", AdHocReportXMLDoc.OuterXml);
                QMQueueObj = new OperationQueue(false);
                QMQueueObj.AddOperation(OperObj);
                OperationXML = QMQueueObj.Xml;

                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.Execute(ref OperationXML);
                if (!(String.IsNullOrEmpty(ErrorXML)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_CREATEFAILED", this.GetType().ToString(), 
                            "CreateAdHocReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                }
                else
                {
                    //If an WorkItem is created successfully, then get the required output XML from execute result.
                    xpathdoc_OperationXML   = new XPathDocument(new StringReader(OperationXML));
                    xpathnav_OperationXML   = xpathdoc_OperationXML.CreateNavigator();
                    xpathnav_ParamInputXML  = xpathnav_OperationXML.SelectSingleNode("INPUTXML/OPERATION/PARAM[@Name='INPUTXML']");
                    if (xpathnav_ParamInputXML != null)
                        AdHocReportInputXML = xpathnav_ParamInputXML.InnerXml;
                }

            }
            catch (Exception ExceptionObject)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_CREATEFAILED", this.GetType().ToString(), 
                        "CreateAdHocReport", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }
            return (ReturnErrorXML);
        }
        
        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to update the existing instance for AdHocReport Entity.</Purpose>
        /// <returns>On Success it will update a record into the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ERRORXML UpdateAdHocReport(ref XML AdHocReportInputXML)
        {
            XPathDocument   xpathdoc_OperationXML   = null;
            XPathNavigator  xpathnav_OperationXML   = null;
            XPathNavigator  xpathnav_ParamInputXML  = null;
            XmlDocument     AdHocReportXMLDoc       = null;
            XmlNode         AdHocReportNode         = null;
            String          LastModifiedByUserID    = null;

            try
            {
                AdHocReportXMLDoc = new XmlDocument();
                AdHocReportXMLDoc.LoadXml(AdHocReportInputXML);

                AdHocReportNode = AdHocReportXMLDoc.DocumentElement;

                Common.PrepareDateTimeNode(ref AdHocReportXMLDoc, AdHocReportNode, "LASTUPDATEDON");

                if (AdHocReportNode.SelectSingleNode("LASTMODIFIEDBYUSERID") != null)
                    LastModifiedByUserID = AdHocReportNode.SelectSingleNode("LASTMODIFIEDBYUSERID").InnerText;
                else
                    ErrorXML = GetAdHocReportLastModifiedByUserID(AdHocReportNode.SelectSingleNode("REPORTID").InnerText, out LastModifiedByUserID);

                if (!(String.IsNullOrEmpty(ErrorXML)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_UPDATEFAILED", this.GetType().ToString(),
                            "UpdateAdHocReport", null, ErrorXML);
                    return ALEngineErrorObj.ErrorCollectionXML;
                }
                else
                {
                    if (LastModifiedByUserID != null)
                    {
                        if ((LastModifiedByUserID != UserID.ToString()))
                        {
                            Common.PrepareRelationNode(ref AdHocReportXMLDoc, AdHocReportNode,
                                "ADHOCREPORT_MODIFIEDBY", LastModifiedByUserID, false, true);
                            Common.PrepareRelationNode(ref AdHocReportXMLDoc, AdHocReportNode,
                                "ADHOCREPORT_MODIFIEDBY", UserID.ToString(), false, false);
                        }
                    }
                    else
                        Common.PrepareRelationNode(ref AdHocReportXMLDoc, AdHocReportNode,
                         "ADHOCREPORT_MODIFIEDBY", UserID.ToString(), false, false);
                }

                ReturnErrorXML = null;
                OperObj = new Operation("ENTITY", "UPDATEDATA");
                OperObj.AddParameter("INPUTXML", AdHocReportXMLDoc.OuterXml);
                QMQueueObj = new OperationQueue(false);
                QMQueueObj.AddOperation(OperObj);
                OperationXML = QMQueueObj.Xml;

                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.Execute(ref OperationXML);
                if (!(String.IsNullOrEmpty(ErrorXML)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_UPDATEFAILED", this.GetType().ToString(), 
                            "UpdateAdHocReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                }
                else
                {
                    //If an WorkItem is created successfully, then get the required output XML from execute result.
                    xpathdoc_OperationXML   = new XPathDocument(new StringReader(OperationXML));
                    xpathnav_OperationXML   = xpathdoc_OperationXML.CreateNavigator();
                    xpathnav_ParamInputXML  = xpathnav_OperationXML.SelectSingleNode("INPUTXML/OPERATION/PARAM[@Name='INPUTXML']");
                    if (xpathnav_ParamInputXML != null)
                        AdHocReportInputXML = xpathnav_ParamInputXML.InnerXml;
                }
            }

            catch (Exception ExceptionObject)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_UPDATEFAILED", this.GetType().ToString(), 
                        "UpdateAdHocReport", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }

            return (ReturnErrorXML);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>21st October 2010</CreatedDate>
        /// <Purpose>This method is used to delete an instance of AdHocReport Entity.</Purpose>
        /// <returns>On Success it will delete a record from the database and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ERRORXML DeleteAdHocReports(String CommaSeparatedReportIDList)
        {
            String[] strarr_AdHocReportIDs = null;

            try
            {
                ReturnErrorXML = null;
                strarr_AdHocReportIDs = CommaSeparatedReportIDList.Split(',');
                if (strarr_AdHocReportIDs.Length <= 0)
                    return (null);

                QMQueueObj = new OperationQueue(false);
                foreach (String str_AdHocReportID in strarr_AdHocReportIDs)
                {
                    OperObj = new Operation("ENTITY", "DELETE");
                    OperObj.AddParameter("ENTITYID", str_AdHocReportID);
                    OperObj.AddParameter("ENTITYTYPENAME", "ADHOCREPORT");
                    QMQueueObj.AddOperation(OperObj);
                }
                OperationXML = QMQueueObj.Xml;

                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.Execute(ref OperationXML);
                if (!(String.IsNullOrEmpty(ErrorXML)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DELETEFAILED", this.GetType().ToString(), 
                            "DeleteAdHocReport", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                }
            }

            catch (Exception ExceptionObject)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DELETEFAILED", this.GetType().ToString(), 
                        "DeleteAdHocReport", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }

            return (ReturnErrorXML);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>06th December 2010</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for the ad-hoc report. Internally it reads the configured info
        /// for ad-hoc reporting and retrieves the entity data. Then using it it generates the Excel.
        /// THIS API WILL BE MOSTLY USED FROM SOLUTION LEVEL TO VIEW EXCEL.
        /// </Purpose>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        /// <param name="AdHocReportID">Ad-hoc report instance ID to get the configuration for Excel generation.</param>
        /// <param name="ReportSchedulerID">Scheduler ID which is used while generating the Excel file path.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        /// <param name="ExportFormat">Format in which the data to be exported for Ad-hoc report.</param>
        public ERRORXML ExportAdhocReport(String AdHocReportID, String ReportSchedulerID, String ExportFormat, out String FileCollectionXML)
        {            
            FileCollectionXML = null;

            try
            {
                ReturnErrorXML = null;
                switch (ExportFormat.ToUpper())
                {
                    case "EXCEL":
                        return (AdHocReportExportToExcelForReportScheduler(AdHocReportID, ReportSchedulerID, out FileCollectionXML));

                    case "PDF":
                        break;
                }
            }
            catch (Exception ExceptionObject)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_EXPORTFAILED", this.GetType().ToString(), 
                        "ExportAdhocReport", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }
            return (ReturnErrorXML);
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>06th December 2010</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for the ad-hoc report. Internally it reads the configured info
        /// for ad-hoc reporting and retrieves the entity data. Then using it it generates the Excel.
        /// THIS API WILL BE MOSTLY USED FROM SOLUTION LEVEL TO VIEW EXCEL.
        /// </Purpose>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        /// <param name="AdHocReportID">Ad-hoc report instance ID to get the configuration for Excel generation.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        public ERRORXML AdHocReportExportToExcelForView(String AdHocReportID, out String FileCollectionXML)
        {
            #region Method Variables

            UInt64      unit_UniqueID               = 0;
            String      str_AdHocReportName         = null;
            String      str_TargetObjectTypeName    = null;
            String      str_AdHocReportFileName     = null;
            XML         xml_AdHocConfigForExcel     = null;
            XML         xml_DisplayColumnsConfig    = null;
            ERRORXML    errorxml_Output             = null;
            String      FilterPersonalizationXML    = null;
            UserPersonalization ObjUserPers         = null;
            DataSet     DS_Result                   = null;
            #endregion

            try
            {
                FileCollectionXML = null;

                #region get PersonalizedFilterXML for Report
                ObjUserPers         = new UserPersonalization(SessionID, UserID, Language, TenantName);
                errorxml_Output     = ObjUserPers.GetUserPersonalization("ADHOCREPORTFILTERPERSONALIZATION", "RPT_" + AdHocReportID, out FilterPersonalizationXML);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_READPERSONALIZATION_FAILED", this.GetType().ToString(),
                            "AdHocReportExportToExcelForView", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                if (!String.IsNullOrEmpty(FilterPersonalizationXML))
                {
                    FilterPersonalizationXML = ALCommon.GetInnerValue(ref FilterPersonalizationXML, "<ADHOCREPORTFILTERS>", "</ADHOCREPORTFILTERS>");
                    if (!String.IsNullOrEmpty(FilterPersonalizationXML))
                        FilterPersonalizationXML = String.Format("<SEARCH><TARGETENTITY>{0}</TARGETENTITY></SEARCH>", FilterPersonalizationXML);
                }
                #endregion

                //Retrieve the ad-hoc report details and configuration info for Excel generation.
                errorxml_Output = getAdHocReportDetailsForExcelGeneration(AdHocReportID,FilterPersonalizationXML, out str_AdHocReportName, out str_TargetObjectTypeName,
                        out xml_DisplayColumnsConfig, ref DS_Result);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(), 
                            "AdHocReportExportToExcelForView", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }
                else if (String.IsNullOrEmpty(xml_DisplayColumnsConfig))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(), 
                            "AdHocReportExportToExcelForView", null, null);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                ALCommon.GenerateUniqueID(ref unit_UniqueID);
                str_AdHocReportFileName = AdHocReportID + "_" + unit_UniqueID.ToString() + ".xlsx";
                xml_AdHocConfigForExcel = "<ADHOCCONFIGFOREXCEL>" + "<ADHOCTARGETOBJECTTYPENAME>" + str_TargetObjectTypeName
                        + "</ADHOCTARGETOBJECTTYPENAME><ADHOCDISPLAYCOLUMNSCONFIG>" + xml_DisplayColumnsConfig
                        + "</ADHOCDISPLAYCOLUMNSCONFIG><ADHOCREPORTNAME>" + str_AdHocReportName
                        + "</ADHOCREPORTNAME><ADHOCREPORTEXCELFILENAME>" + str_AdHocReportFileName
                        + "</ADHOCREPORTEXCELFILENAME></ADHOCCONFIGFOREXCEL>";
          
                //Generate the Excel for ad-hoc reports using configuration details.
                errorxml_Output = generateExcelForAdHocReport(AdHocReportID, xml_AdHocConfigForExcel, ref DS_Result, out FileCollectionXML);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForView", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }
                else if (String.IsNullOrEmpty(FileCollectionXML))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForView", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                return (errorxml_Output);
            }
            catch (Exception ExceptionObject)
            {
                FileCollectionXML = null;
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(), 
                        "AdHocReportExportToExcelForView", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return (ReturnErrorXML);
            }
        }

        /// <summary>
        /// <CreatedBy>Kiran Abhang</CreatedBy>
        /// <CreatedDate>06th December 2010</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for the ad-hoc report. Internally it reads the configured info
        /// for ad-hoc reporting and retrieves the entity data. Then using it it generates the Excel.
        /// THIS API WILL BE MOSTLY USED FROM REPORT SCHEDULER TO GENERATE AND ATTACH AN EXCEL.
        /// </Purpose>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        /// </summary>
        /// <param name="AdHocReportID">Ad-hoc report instance ID to get the configuration for Excel generation.</param>
        /// <param name="ReportSchedulerID">Scheduler ID which is used while generating the Excel file path.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        public ERRORXML AdHocReportExportToExcelForReportScheduler(String AdHocReportID, String ReportSchedulerID, out String FileCollectionXML)
        {
            #region Method Variables

            UInt64      unit_UniqueID               = 0;
            String      str_AdHocReportName         = null;
            String      str_TargetObjectTypeName    = null;
            String      str_AdHocReportFileName     = null;
            XML         xml_AdHocConfigForExcel     = null;
            XML         xml_DisplayColumnsConfig    = null;
            ERRORXML    errorxml_Output             = null;
            String      FilterPersonalizationXML    = null;
            DataSet     DS_Result                   = null;
            #endregion

            try
            {
                FileCollectionXML = null;

                #region get PersonalizedFilterXML for Report             
                errorxml_Output = GetFilterPersonalizationXMLForScheduleInstance(ReportSchedulerID, ref FilterPersonalizationXML);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_READSCHEDULEFILTERPERSONALIZATION_FAILED", this.GetType().ToString(),
                            "AdHocReportExportToExcelForView", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                if (!String.IsNullOrEmpty(FilterPersonalizationXML))
                {
                    FilterPersonalizationXML = ALCommon.GetInnerValue(ref FilterPersonalizationXML, "<ADHOCREPORTFILTERS>", "</ADHOCREPORTFILTERS>");
                    if (!String.IsNullOrEmpty(FilterPersonalizationXML))
                        FilterPersonalizationXML = String.Format("<SEARCH><TARGETENTITY>{0}</TARGETENTITY></SEARCH>", FilterPersonalizationXML);
                }
                #endregion

                //Retrieve the ad-hoc report details and configuration info for Excel generation.
                errorxml_Output = getAdHocReportDetailsForExcelGeneration(AdHocReportID, FilterPersonalizationXML, out str_AdHocReportName, out str_TargetObjectTypeName,
                        out xml_DisplayColumnsConfig, ref DS_Result);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForReportScheduler", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }
                else if (String.IsNullOrEmpty(xml_DisplayColumnsConfig))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForReportScheduler", null, null);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                //Report name is taken outside the generate excel API() because generate excel API can be used standard without any change.
                if (!(String.IsNullOrEmpty(str_AdHocReportName)))
                    str_AdHocReportFileName = ALCommon.ReplaceSpecialChars(str_AdHocReportName) + ".xls";
                else
                {
                    ALCommon.GenerateUniqueID(ref unit_UniqueID);
                    str_AdHocReportFileName = AdHocReportID + "_" + unit_UniqueID.ToString() + ".xls";
                }

                xml_AdHocConfigForExcel = "<ADHOCCONFIGFOREXCEL>" + "<ADHOCTARGETOBJECTTYPENAME>" + str_TargetObjectTypeName
                        + "</ADHOCTARGETOBJECTTYPENAME><ADHOCDISPLAYCOLUMNSCONFIG>" + xml_DisplayColumnsConfig
                        + "</ADHOCDISPLAYCOLUMNSCONFIG><ADHOCREPORTNAME>" + str_AdHocReportName
                        + "</ADHOCREPORTNAME><ADHOCREPORTEXCELFILENAME>" + str_AdHocReportFileName
                        + "</ADHOCREPORTEXCELFILENAME><REPORTSCHEDULERID>" + ReportSchedulerID 
                        + "</REPORTSCHEDULERID></ADHOCCONFIGFOREXCEL>";

                //Generate the Excel for ad-hoc reports using configuration details.
                errorxml_Output = generateExcelForAdHocReport(AdHocReportID, xml_AdHocConfigForExcel,ref DS_Result, out FileCollectionXML);
                if (!(String.IsNullOrEmpty(errorxml_Output)))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForReportScheduler", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }
                else if (String.IsNullOrEmpty(FileCollectionXML))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(),
                            "AdHocReportExportToExcelForReportScheduler", null, errorxml_Output);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                return (errorxml_Output);
            }
            catch (Exception ExceptionObject)
            {
                FileCollectionXML = null;
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_GENERATEEXCEL", this.GetType().ToString(),
                        "AdHocReportExportToExcelForReportScheduler", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return (ReturnErrorXML);
            }
        }

        /// <summary>
        /// <CreatedBy>Ankit Agrawal</CreatedBy>
        /// <CreatedDate>19th January 2011</CreatedDate>
        /// <Purpose>This method is used to delete an instance of AdHocReport Entity.</Purpose>
        /// <returns>On Success it will pass by reference the Ad hoc Report Grouped Data XML; whereas on failure an Error will be returned.</returns>
        /// </summary>
        public ERRORXML GetAdHocReportGroupedDataXML(XML ReportConfiguration, XML xml_SearchOutput, ref XML GroupCollectionXML)
        {
            XDocument               xDoc_ReportConfig           = null;
            XElement                xEle_ReportConfig           = null;
            List<String>            relationshipList            = new List<string>();
            IEnumerable<XElement>   xEles_Relationship          = null;
            String                  relationshipName            = null;
            String                  str_TargetObjectTypeName    = null;
            String                  str_TargetObjectPK          = null;
            XML                     DisplayConfiguration        = null;
            XDocument               xDoc_DisplayConfig          = null;
            StringBuilder           sb_GroupCollectionXML       = new StringBuilder();
            ERRORXML                ErrorXML                    = null;
            int                     TotalGroupLevels            = 0;

            xDoc_ReportConfig = XDocument.Parse(ReportConfiguration);
            xEle_ReportConfig = xDoc_ReportConfig.Root;
            xEles_Relationship = xEle_ReportConfig.XPathSelectElements("SEARCHINPUT/INPUTXML/SEARCHCOLLECTION/SEARCH/TARGETENTITY/RELATIONSHIP");

            foreach (XElement xEle_Relationship in xEles_Relationship)
            {
                relationshipName = xEle_Relationship.Attribute("NAME").Value;
                relationshipList.Add(relationshipName);
            }

            str_TargetObjectTypeName = xEle_ReportConfig.Element("SearchTarget").Attribute("ObjectName").Value;
            str_TargetObjectPK = xEle_ReportConfig.Element("SearchTarget").Attribute("ObjectPrimaryKey").Value;

            DisplayConfiguration = xEle_ReportConfig.XPathSelectElement("DisplayLayout/SeriesCollection/AdHocReportsDisplayColumns/CONFIGCOLUMNSCOLLECTION").ToString();

            xDoc_ReportConfig = null;

            xDoc_DisplayConfig = XDocument.Parse(DisplayConfiguration);

            initializeAggregationFunctions();
            var aggregationColumns = GetAggregationColumns(xDoc_DisplayConfig);

            initializeSortingFunctions();
            var sortColumns = GetSortColumns(xDoc_DisplayConfig);

            TotalGroupLevels = GetGroupByColumnsCount(xDoc_DisplayConfig);

            ErrorXML = GroupData(xDoc_DisplayConfig, relationshipList, TotalGroupLevels, aggregationColumns, sortColumns, xml_SearchOutput, 1, str_TargetObjectTypeName, str_TargetObjectPK, ref sb_GroupCollectionXML);
            if (!String.IsNullOrEmpty(ErrorXML))
                return ErrorXML;

            GroupCollectionXML = "<GROUPDATACOLLECTION>" + sb_GroupCollectionXML.ToString() + "</GROUPDATACOLLECTION>";

            return ErrorXML;
        }

        #endregion

        # region Member/Public Functions For Crystal Reports

        /// <summary>
        /// ExportReportToExcel() method is used to export crystal report data into excel format.
        /// </summary>
        /// <param name="TenantName">Name of client/tenant.</param>
        /// <param name="ParametersXML">It contains all parameters required for stored procedure.</param>
        /// <param name="StoredProcedureName">Name of stored procedure, which is to be executed.</param>
        /// <param name="ExcelFilePath">File path at which an excel file to be generated.</param>
        public ERRORXML ExportCrystalReportToExcel(String TenantName, String ParametersXML, String StoredProcedureName, String ExcelFilePath, String TableName)
        {
            SqlCommand      SqlCommandObject    = null;
            XmlDocument     ParametersDoc       = null;
            DataSet         dsReport            = null;
            DataTableReader ObjDataTableReader  = null;

            try
            {
                ReturnErrorXML = null;
                switch (this.m_DatabaseType)
                {
                    case "SQLSERVER":
                    case "SQLEXPRESS":
                        break;

                    case "TERADATA":
                    case "ORACLE":
                    case "ORACLEEXPRESS":
                        dsReport = getStoredProcedureInformation(StoredProcedureName, ParametersXML, TableName);
                        ObjDataTableReader = dsReport.CreateDataReader();
                        ReturnErrorXML = exportCrystalReportDataToExcel(ObjDataTableReader, ExcelFilePath);
                        return ReturnErrorXML;                    
                }

                //Set stored procedure paramters to SQL's command object.
                SqlCommandObject = new SqlCommand();
                SqlCommandObject.CommandType = CommandType.StoredProcedure;
                if (!String.IsNullOrEmpty(ParametersXML))
                {
                    ParametersDoc = new XmlDocument();
                    ParametersDoc.LoadXml(ParametersXML);
                    foreach (XmlNode ParameterNode in ParametersDoc.SelectNodes("//PARAM"))
                    {
                        SqlCommandObject.Parameters.AddWithValue(ParameterNode.SelectSingleNode("NAME").InnerText,
                                ParameterNode.SelectSingleNode("VALUE").InnerText.Trim());
                    }
                }
                SqlCommandObject.CommandText = StoredProcedureName;
                SqlCommandObject.Connection = new SqlConnection(this.m_ConnectionString);
                SqlCommandObject.Connection.Open();

                SqlDataReader SqlDataReaderObject = null;
                SqlDataReaderObject = SqlCommandObject.ExecuteReader();
                ReturnErrorXML = exportCrystalReportDataToExcel(SqlDataReaderObject, ExcelFilePath);
            }
            catch (Exception ExceptionObject)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("ERR_RPT_REPORTENGINE_EXPORTFAILED", this.GetType().ToString(),
                        "ExportCrystalReportToExcel", ExceptionObject, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }
            return ReturnErrorXML;
        }

        #endregion

        #region Member/Public Functions For Asset Metadate Report

        public ERRORXML ExportAssetMetadataReport(String ExportType, XML UserInputXml, XML SearchInputXml, XML SortFieldListXml, XML FormConfigXml, int SortType, out String ResultExportFilePath)
        {
            # region Local Variables
            OMInterface             omi                 = null;
            XML                     outputXML           = null;
            XPathDocument           xpDocSearchResult   = null;
            XPathNavigator          xpNavSearchResult   = null;
            XPathDocument           xpDocUserInput      = null;
            XPathNavigator          xpNavUserInput      = null;
            XPathDocument           xpDocFormConfig     = null;
            XPathNavigator          xpNavFormConfig     = null;
            int                     docHeight           = 0;
            ulong                   randomID            = 0;
            StringBuilder           TargetPath          = null;
            String                  ResultHtmlPath      = null;
            String                  ResultMHtmlPath     = null;
            String                  MHtmlNetworkPath    = null;
            ALMSOfficeProcessorWS   MSOfficeWSObj       = null;
            String                  str_MSOfficeProcessorLocalPath      = null;
            String                  str_MSOfficeProcessorNetworkPath    = null;
            String                  ExportFileLocalPath                 = null;
            String                  ResultExportFileName                = null;
            # endregion

            ResultExportFilePath = null;
            try
            {                
                omi = new OMInterface(UserID, Language, TenantName);
                omi.SessionID = SessionID;

                ErrorXML = omi.GetHierarchialData(SearchInputXml, ref outputXML);
                if ((String.IsNullOrEmpty(ErrorXML) == true) && (String.IsNullOrEmpty(outputXML) == false))
                {
                    if (SortType > 0)
                        outputXML = sortResultXml(outputXML, SortFieldListXml, SortType);

                    outputXML = getAllThumbnailsToServicesMachine(outputXML);

                    xpDocSearchResult = new XPathDocument(new StringReader(outputXML));
                    xpNavSearchResult = xpDocSearchResult.CreateNavigator();

                    ALCommon.GenerateUniqueID(ref randomID);

                    //ResultHtmlPath = ALCommon.GetDownloadFilePath(TenantName, SessionID.ToString()) + "AssetMetadata_" + randomID.ToString() + ".htm";
                    //ResultMHtmlPath = ALCommon.GetDownloadFilePath(TenantName, SessionID.ToString()) + "AssetMetadata_" + randomID.ToString() + ".mht";

                    ErrorXML = getMSOfficeProcessorFolderPath(out str_MSOfficeProcessorLocalPath, out str_MSOfficeProcessorNetworkPath);
                    if ((String.IsNullOrEmpty(str_MSOfficeProcessorLocalPath)) || (String.IsNullOrEmpty(str_MSOfficeProcessorNetworkPath)))
                    {
                        ALEngineErrorObj.AppendError("ERR_MSOFFICEPROCESSORPATHNOTGENERATE_SHARED", this.GetType().ToString(),
                                "ExportAssetMetadataReport", null, ErrorXML);
                        return (ALEngineErrorObj.ErrorCollectionXML);
                    }

                    ResultHtmlPath = str_MSOfficeProcessorLocalPath + "\\AssetMetadata_" + randomID.ToString() + ".htm";
                    ResultMHtmlPath = str_MSOfficeProcessorLocalPath + "\\AssetMetadata_" + randomID.ToString() + ".mht";
                    MHtmlNetworkPath = str_MSOfficeProcessorNetworkPath + "\\AssetMetadata_" + randomID.ToString() + ".mht";
                    if (String.Compare(ExportType, "PDF", true) == 0)
                        docHeight = 700;

                    xpDocUserInput = new XPathDocument(new StringReader(UserInputXml));
                    xpNavUserInput = xpDocUserInput.CreateNavigator();

                    if (String.IsNullOrEmpty(FormConfigXml) == false)
                    {
                        xpDocFormConfig = new XPathDocument(new StringReader(FormConfigXml));
                        xpNavFormConfig = xpDocFormConfig.CreateNavigator();
                    }                    

                    ErrorXML = prepareHtml(xpNavSearchResult, xpNavUserInput, xpNavFormConfig, ResultHtmlPath, docHeight);                    

                    if (String.IsNullOrEmpty(ErrorXML) == true)
                    {
                        ErrorXML = generateMHtmlFromHtml(ResultHtmlPath, ResultMHtmlPath);

                        if (String.IsNullOrEmpty(ErrorXML) == true)
                        {
                            MSOfficeWSObj = new ALMSOfficeProcessorWS();
                            MSOfficeWSObj.Url = ALCommon.GetWebServiceURL("ALMSOfficeProcessorWS");
                            MSOfficeWSObj.Timeout = ALBaseEngine.GetWebServiceTimeOut();
                            TargetPath = new StringBuilder(50);
                            

                            if (String.Compare(ExportType, "PDF", true) == 0)
                            {
                                ResultExportFileName = "AssetMetadata_" + randomID.ToString() + ".pdf";
                                TargetPath.AppendFormat("{0}\\{1}", str_MSOfficeProcessorNetworkPath, ResultExportFileName);

                                ErrorXML = MSOfficeWSObj.GeneratePDFFromMHtml(MHtmlNetworkPath, TargetPath.ToString());
                            }
                            else if (String.Compare(ExportType, "EXCEL", true) == 0)
                            {
                                ResultExportFileName = "AssetMetadata_" + randomID.ToString() + ".xls";
                                TargetPath.AppendFormat("{0}\\{1}", str_MSOfficeProcessorNetworkPath, ResultExportFileName);

                                ErrorXML = MSOfficeWSObj.GenerateExcelFromMHtml(MHtmlNetworkPath, TargetPath.ToString());
                            }

                            if (String.IsNullOrEmpty(ErrorXML) == false)
                                return ErrorXML;

                            # region Moving exported file to download folder
                            
                            //Copy file only in case of Load Balance setup
                            if (String.IsNullOrEmpty(ConfigurationManager.AppSettings["SHAREDDRIVEROOT"]) == false)
                            {
                                ResultExportFilePath = ALCommon.GetDownloadFilePath(TenantName, SessionID.ToString()) + ResultExportFileName;
                                ExportFileLocalPath = str_MSOfficeProcessorLocalPath + "\\" + ResultExportFileName;
                                File.Copy(ExportFileLocalPath, ResultExportFilePath);
                            }
                            else
                            {
                                ResultExportFilePath = TargetPath.ToString();
                            }
                            # endregion
                        }
                    }
                }

            }
            catch (Exception ExceptionObject)
            {
                ErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ErrorXML);
                ALEngineErrorObj.AppendError("ERR_REPORT_REPORTENGINE_EXPORTASSETMETADATAREPORTFAILED", this.GetType().ToString(),
                        "ExportAssetMetadataReport", ExceptionObject, null);
                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }

            return ErrorXML;
        }

        #endregion

        #region Private Methods

        private void PrepareWeeklyOption(ref String str_FormData, String Mode)
        {
            String[] Array = null;
            String[] Seperator = { "$AL$" };
            StringBuilder sb_WeeklyOptions = null;
            sb_WeeklyOptions = new StringBuilder();
            if (Mode.ToUpper() == "DATABASE")
            {
                Array = str_FormData.Split(Seperator, StringSplitOptions.RemoveEmptyEntries);
                for (int i = 0; i < Array.Length; i++)
                {
                    switch (Array[i])
                    {
                        case "Sunday": sb_WeeklyOptions.Append("0");
                            break;
                        case "Monday": sb_WeeklyOptions.Append("1");
                            break;
                        case "Tuesday": sb_WeeklyOptions.Append("2");
                            break;
                        case "Wednesday": sb_WeeklyOptions.Append("3");
                            break;
                        case "Thursday": sb_WeeklyOptions.Append("4");
                            break;
                        case "Friday": sb_WeeklyOptions.Append("5");
                            break;
                        case "Saturday": sb_WeeklyOptions.Append("6");
                            break;
                        default: break;

                    }
                }
            }
            str_FormData = sb_WeeklyOptions.ToString();


        }

        private void PrepareMonthlyOption(ref String str_FormData, String Mode)
        {
            String[] Array = null;
            String[] Seperator = { "$AL$" };
            StringBuilder sb_MonthlyOptions = null;
            sb_MonthlyOptions = new StringBuilder();
            if (Mode.ToUpper() == "DATABASE")
            {
                Array = str_FormData.Split(Seperator, StringSplitOptions.RemoveEmptyEntries);
                if (Array.Length > 0)
                {
                    sb_MonthlyOptions.Append("$AL$");
                    for (int i = 0; i < Array.Length; i++)
                    {
                        switch (Array[i].ToUpper())
                        {
                            case "JANUARY": sb_MonthlyOptions.Append("1$AL$");
                                break;
                            case "FEBRUARY": sb_MonthlyOptions.Append("2$AL$");
                                break;
                            case "MARCH": sb_MonthlyOptions.Append("3$AL$");
                                break;
                            case "APRIL": sb_MonthlyOptions.Append("4$AL$");
                                break;
                            case "MAY": sb_MonthlyOptions.Append("5$AL$");
                                break;
                            case "JUNE": sb_MonthlyOptions.Append("6$AL$");
                                break;
                            case "JULY": sb_MonthlyOptions.Append("7$AL$");
                                break;
                            case "AUGUST": sb_MonthlyOptions.Append("8$AL$");
                                break;
                            case "SEPTEMBER": sb_MonthlyOptions.Append("9$AL$");
                                break;
                            case "OCTOBER": sb_MonthlyOptions.Append("10$AL$");
                                break;
                            case "NOVEMBER": sb_MonthlyOptions.Append("11$AL$");
                                break;
                            case "DECEMBER": sb_MonthlyOptions.Append("12$AL$");
                                break;
                            default: break;

                        }
                    }
                }
            }
            str_FormData = sb_MonthlyOptions.ToString();
        }

        private String PrepareYearlyOption(ref String str_yrMonth, String str_yrDay)
        {
            StringBuilder sb_FormData = null;
           
            sb_FormData = new StringBuilder();
            sb_FormData.Append(str_yrDay);
            switch (str_yrMonth)
            {
                case "January": sb_FormData.Append("01");
                    str_yrMonth = "01";
                    break;
                case "February": sb_FormData.Append("02");
                    str_yrMonth = "02";
                    break;
                case "March": sb_FormData.Append("03");
                    str_yrMonth = "03";
                    break;
                case "April": sb_FormData.Append("04");
                    str_yrMonth = "04";
                    break;
                case "May": sb_FormData.Append("05");
                    str_yrMonth = "05";
                    break;
                case "June": sb_FormData.Append("06");
                    str_yrMonth = "06";
                    break;
                case "July": sb_FormData.Append("07");
                    str_yrMonth = "07";
                    break;
                case "August": sb_FormData.Append("08");
                    str_yrMonth = "08";
                    break;
                case "September": sb_FormData.Append("09");
                    str_yrMonth = "09";
                    break;
                case "October": sb_FormData.Append("10");
                    str_yrMonth = "10";
                    break;
                case "November": sb_FormData.Append("11");
                    str_yrMonth = "11";
                    break;
                case "December": sb_FormData.Append("12");
                    str_yrMonth = "12";
                    break;
                default: break;
            }
            return sb_FormData.ToString();            
        }

        private DateTime XmlToDateTime(XPathNavigator xmlNode_Data)
        {
            XPathNavigator xmlNode_Day = null;         //Used to navigate through DAY node
            XPathNavigator xmlNode_Month = null;         //Used to navigate through MONTH node
            XPathNavigator xmlNode_Year = null;         //Used to navigate through YEAR node
            XPathNavigator xmlNode_Hour = null;         //Used to navigate through HOUR node
            XPathNavigator xmlNode_Minute = null;         //Used to navigate through MINUTE node
            XPathNavigator xmlNode_Second = null;         //Used to navigate through SECOND node

            if (xmlNode_Data != null)
            {
                xmlNode_Day = xmlNode_Data.SelectSingleNode("DAY");
                xmlNode_Month = xmlNode_Data.SelectSingleNode("MONTH");
                xmlNode_Year = xmlNode_Data.SelectSingleNode("YEAR");
                xmlNode_Hour = xmlNode_Data.SelectSingleNode("HOUR");
                xmlNode_Minute = xmlNode_Data.SelectSingleNode("MINUTE");
                xmlNode_Second = xmlNode_Data.SelectSingleNode("SECOND");

                if (xmlNode_Day != null && xmlNode_Month != null && xmlNode_Year != null)
                {
                    if (xmlNode_Hour != null && xmlNode_Minute != null && xmlNode_Second != null)
                        return new DateTime(Convert.ToInt32(xmlNode_Year.Value), Convert.ToInt32(xmlNode_Month.Value), Convert.ToInt32(xmlNode_Day.Value),
                            Convert.ToInt32(xmlNode_Hour.Value), Convert.ToInt32(xmlNode_Minute.Value), Convert.ToInt32(xmlNode_Second.Value));
                    else
                        return new DateTime(Convert.ToInt32(xmlNode_Year.Value), Convert.ToInt32(xmlNode_Month.Value), Convert.ToInt32(xmlNode_Day.Value));
                }
            }

            return DateTime.Now;
        }        

        private ERRORXML CheckWhetherInstanceTimeExist(String ScheduleReportID, DateTime objDate, ref bool IsInstanceExist)
        {
            String                      strSearchOutputXML                  = null;
            String                      SCHRPTInstanceID                    = null;
            StringBuilder               sbReportInstancesSearchXML          = new StringBuilder(16);
            OMInterface                 omi                                 = null;
                
            sbReportInstancesSearchXML.AppendFormat(@"<SEARCH>
                                                            <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />
                                                            <TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>
                                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                            <TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>
                                                                <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <ALIAS>SCHEDREPORTID</ALIAS>
                                                                        <FIELDNAME>SCHEDREPORTID</FIELDNAME>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                                <FILTERLIST>    
                                                                    <FILTER>
                                                                        <EXPRESSION>INSTANCEDATETIME</EXPRESSION>
                                                                        <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>
                                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                                        <EXPRESSIONVALUE>
                                                                            <VALUE>{4}</VALUE>
                                                                        </EXPRESSIONVALUE>
                                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                    </FILTER>
                                                                    <FILTER>
                                                                        <EXPRESSION>SCHEDREPORTID</EXPRESSION>                                                                        
                                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                                        <EXPRESSIONVALUE>{5}</EXPRESSIONVALUE>
                                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                    </FILTER>
                                                                </FILTERLIST>
                                                                <RELATIONSHIP NAME='SCHEDULEDREPORT_INSTANCES'>
                                                                    <SELECTLIST>
                                                                        <EXPRESSION>
                                                                            <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                                                            <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                                                        </EXPRESSION>
                                                                    </SELECTLIST>
                                                                </RELATIONSHIP>
                                                            </TARGETENTITY>
                                                        </SEARCH>", SessionID, UserID, TenantName, Language, GetDateInXML(objDate), ScheduleReportID);

            omi = new OMInterface();

            omi.CurrentLanguage = Language;
            omi.SessionID = SessionID;
            omi.UserID = UserID;
            omi.TenantName = TenantName;

            ErrorXML = omi.GetHierarchialData(sbReportInstancesSearchXML.ToString(), ref strSearchOutputXML);
            if (ErrorXML == null)
            {
                SCHRPTInstanceID = ALCommon.GetInnerValue(ref strSearchOutputXML, "<SCHEDREPORTINSTANCEID>", "</SCHEDREPORTINSTANCEID>");
                if (!String.IsNullOrEmpty(SCHRPTInstanceID))
                {
                    IsInstanceExist = true;
                }
                else
                {
                    IsInstanceExist = false;
                }
            }
            return ErrorXML;
        }

        private ERRORXML GetLastRunNextRunInstanceID(String ScheduleReportID, out String LastRunInstanceID, out String NextRunInstanceID)
        {
            String                      strSearchOutputXML                  = null;            
            StringBuilder               sbReportInstancesSearchXML          = new StringBuilder(16);
            OMInterface                 omi                                 = null;

            LastRunInstanceID = null;
            NextRunInstanceID = null;

            sbReportInstancesSearchXML.AppendFormat(@"<SEARCH>
                                                            <SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />
                                                            <TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>
                                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                            <TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>
                                                                <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <ALIAS>SCHEDREPORTID</ALIAS>
                                                                        <FIELDNAME>SCHEDREPORTID</FIELDNAME>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                                <FILTERLIST>                                                                        
                                                                    <FILTER>
                                                                        <EXPRESSION>SCHEDREPORTID</EXPRESSION>                                                                        
                                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                                        <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
                                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                    </FILTER>
                                                                </FILTERLIST>                                                                
                                                                <RELATIONSHIP NAME='SCHRPT_LASTRUN'>
                                                                    <SELECTLIST>
                                                                        <EXPRESSION>
                                                                            <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                                                            <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                                                        </EXPRESSION>
                                                                    </SELECTLIST>
                                                                </RELATIONSHIP>
                                                                <RELATIONSHIP NAME='SCHRPT_NEXTRUN'>
                                                                    <SELECTLIST>
                                                                        <EXPRESSION>
                                                                            <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                                                            <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                                                        </EXPRESSION>
                                                                    </SELECTLIST>
                                                                </RELATIONSHIP>
                                                            </TARGETENTITY>
                                                        </SEARCH>", SessionID, UserID, TenantName, Language, ScheduleReportID);

            omi = new OMInterface();

            omi.CurrentLanguage = Language;
            omi.SessionID = SessionID;
            omi.UserID = UserID;
            omi.TenantName = TenantName;

            ErrorXML = omi.GetHierarchialData(sbReportInstancesSearchXML.ToString(), ref strSearchOutputXML);
            if (ErrorXML == null && strSearchOutputXML != null)
            {
                XmlDocument SearchResultXmlDoc = new XmlDocument();
                SearchResultXmlDoc.LoadXml(strSearchOutputXML);

                XmlNode InstanceIDNode = SearchResultXmlDoc.SelectSingleNode("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS/SCHRPT_LASTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID");
                if (InstanceIDNode != null)
                    LastRunInstanceID = InstanceIDNode.InnerText;

                InstanceIDNode = SearchResultXmlDoc.SelectSingleNode("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS/SCHRPT_NEXTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID");
                if (InstanceIDNode != null)
                    NextRunInstanceID = InstanceIDNode.InnerText;
            }

            return ErrorXML;
        }

        private string GetDateInXML(DateTime dt)
        {
            return string.Format("<YEAR>{0}</YEAR><MONTH>{1}</MONTH><DAY>{2}</DAY><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND>",
                                    dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
        }

        #region To be Remove
        //private void PrepareEditMode_WeeklyOption(XmlNode xNodeWeeklyOption, out String strUpdatedWeeklyOption)
        //{
        //    String WeeklyOption = null;
        //    String strDay = null;
        //    StringBuilder sbWeeklyOptions = null;

        //    strUpdatedWeeklyOption = null;
        //    if (xNodeWeeklyOption != null)
        //    {
        //        WeeklyOption = xNodeWeeklyOption.InnerXml;
        //        if (WeeklyOption != null)
        //        {
        //            sbWeeklyOptions = new StringBuilder(16);
        //            sbWeeklyOptions.Append("$AL$");
        //            for (int cnt = 0; cnt < WeeklyOption.Length; cnt++)
        //            {
        //                strDay = WeeklyOption.Substring(cnt, 1);
        //                switch (strDay.ToUpper())
        //                {
        //                    case "0":
        //                        sbWeeklyOptions.Append("Sunday$AL$");
        //                        break;
        //                    case "1":
        //                        sbWeeklyOptions.Append("Monday$AL$");
        //                        break;
        //                    case "2":
        //                        sbWeeklyOptions.Append("Tuesday$AL$");
        //                        break;
        //                    case "3":
        //                        sbWeeklyOptions.Append("Wednesday$AL$");
        //                        break;
        //                    case "4":
        //                        sbWeeklyOptions.Append("Thursday$AL$");
        //                        break;
        //                    case "5":
        //                        sbWeeklyOptions.Append("Friday$AL$");
        //                        break;
        //                    case "6":
        //                        sbWeeklyOptions.Append("Saturday$AL$");
        //                        break;
        //                }
        //            }
        //            strUpdatedWeeklyOption = sbWeeklyOptions.ToString();
        //        }
        //    }
        //}

        //private void PrepareEditMode_TimeOption(XmlNode xNodeSCHTime, out String strUpdatedSCHTime, out String strUpdatedSCHPeriod)
        //{
        //    String strSCHTime = null;

        //    strUpdatedSCHTime = null;
        //    strUpdatedSCHPeriod = null;
        //    if (xNodeSCHTime != null)
        //    {
        //        strSCHTime = xNodeSCHTime.InnerXml;
        //        if (strSCHTime != null)
        //        {
        //            strUpdatedSCHTime = strSCHTime.Substring(0, 2);
        //            if (strUpdatedSCHTime.Contains("0"))
        //            {
        //                if (strUpdatedSCHTime.IndexOf("0") == 0)
        //                {
        //                    strUpdatedSCHTime = strUpdatedSCHTime.Remove(0, 1);
        //                }
        //            }
        //            strUpdatedSCHPeriod = strSCHTime.Substring(4);
        //        }
        //    }
        //}

        //private void PrepareEditMode_YearlyOption(XmlNode xNodeYearlyOption, out String strUpdatedYearlyOption, out String strUpdatedDayOfMonth)
        //{
        //    String strYearlyOption = null;

        //    strUpdatedYearlyOption = null;
        //    strUpdatedDayOfMonth = null;
        //    if (xNodeYearlyOption != null)
        //    {
        //        strYearlyOption = xNodeYearlyOption.InnerXml;
        //        if (!String.IsNullOrEmpty(strYearlyOption))
        //        {
        //            strUpdatedDayOfMonth = strYearlyOption.Substring(0, 2);

        //            strUpdatedYearlyOption = strYearlyOption.Substring(2, 2);
        //            switch (strUpdatedYearlyOption)
        //            {
        //                case "01":
        //                    strUpdatedYearlyOption = "January";
        //                    break;
        //                case "02":
        //                    strUpdatedYearlyOption = "February";
        //                    break;
        //                case "03":
        //                    strUpdatedYearlyOption = "March";
        //                    break;
        //                case "04":
        //                    strUpdatedYearlyOption = "April";
        //                    break;
        //                case "05":
        //                    strUpdatedYearlyOption = "May";
        //                    break;
        //                case "06":
        //                    strUpdatedYearlyOption = "June";
        //                    break;
        //                case "07":
        //                    strUpdatedYearlyOption = "July";
        //                    break;
        //                case "08":
        //                    strUpdatedYearlyOption = "August";
        //                    break;
        //                case "09":
        //                    strUpdatedYearlyOption = "September";
        //                    break;
        //                case "10":
        //                    strUpdatedYearlyOption = "October";
        //                    break;
        //                case "11":
        //                    strUpdatedYearlyOption = "November";
        //                    break;
        //                case "12":
        //                    strUpdatedYearlyOption = "December";
        //                    break;
        //            }                    
        //        }
        //    }
        //}
        #endregion

        private String SaveScheduleReportInstanceDetails(String ScheduleReportID, String FilterXML, String FormOutputXML, bool IsInstanceReq)
        {
            #region Member Variables
            String                      ErrorXml                        = null;            
            String                      SCHTypeOption                   = null;
            String                      RunCycleOption                  = null;
            String                      str_yrMonth                     = null;
            String                      str_yrDay                       = null;
            String                      str_SCHRptDetails               = null;
            String                      str_FormData                    = null;
            StringBuilder               sb_SCHRptDetailsColumns         = null;
            XPathDocument               xpDoc_FormDataXML               = null;
            XPathNavigator              xpNav_FormDataXML               = null;
            XPathNavigator              xpNav_FormDataNode              = null;
            InfrastructureEngineClass   objInfEngine                    = null;
            bool                        IsTypeRunOnce                   = false;
            Boolean                     IsOperationCreate               = false;
            #endregion

            if (!String.IsNullOrEmpty(FormOutputXML))
            {
                sb_SCHRptDetailsColumns = new StringBuilder(1024);
                xpDoc_FormDataXML = new XPathDocument(new StringReader(FormOutputXML));
                xpNav_FormDataXML = xpDoc_FormDataXML.CreateNavigator();

                #region Prepare Schedule Details Column
                if (String.IsNullOrEmpty(ScheduleReportID))
                {
                    IsOperationCreate = true;
                    ulong NewUniqueID = 0;
                    Assetlink.UniqueID.Read(ref NewUniqueID);
                    ScheduleReportID = NewUniqueID.ToString();
                }

                sb_SCHRptDetailsColumns.AppendFormat("<SCHEDREPORTID>{0}</SCHEDREPORTID>", ScheduleReportID);

                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHEDREPORTINSTANCENAME");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SCHEDREPORTINSTANCENAME>{0}</SCHEDREPORTINSTANCENAME>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/DESCRIPTION");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", xpNav_FormDataNode.InnerXml);
                }

                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHEDTYPE");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SCHEDTYPE>{0}</SCHEDTYPE>", xpNav_FormDataNode.InnerXml);
                    SCHTypeOption = xpNav_FormDataNode.InnerXml;                    
                }

                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/STARTDATE");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<STARTDATE>{0}</STARTDATE>", xpNav_FormDataNode.InnerXml);
                }

                if (SCHTypeOption != null && SCHTypeOption.ToUpper() == "RUN ONCE")
                {
                    IsTypeRunOnce  = true;
                    RunCycleOption = "DAILY";
                    sb_SCHRptDetailsColumns.AppendFormat("<RUNCYCLE>{0}</RUNCYCLE>", RunCycleOption);

                    sb_SCHRptDetailsColumns.AppendFormat("<ENDDATE>{0}</ENDDATE>", xpNav_FormDataNode.InnerXml);                    
                }
                else
                {
                    xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/ENDDATE");
                    if (xpNav_FormDataNode != null)
                    {
                        sb_SCHRptDetailsColumns.AppendFormat("<ENDDATE>{0}</ENDDATE>", xpNav_FormDataNode.InnerXml);
                    }
                }

                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHEDTIME");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SCHEDTIME>{0}</SCHEDTIME>", xpNav_FormDataNode.InnerXml);                              
                }              

                if (!IsTypeRunOnce)
                {
                    xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/RUNCYCLE");
                    if (xpNav_FormDataNode != null)
                    {
                        sb_SCHRptDetailsColumns.AppendFormat("<RUNCYCLE>{0}</RUNCYCLE>", xpNav_FormDataNode.InnerXml);
                        RunCycleOption = xpNav_FormDataNode.InnerXml;

                        switch (RunCycleOption.ToUpper())
                        {
                            case "DAILY":
                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/DAILYOPTION");
                                if (xpNav_FormDataNode != null)
                                    str_FormData = xpNav_FormDataNode.InnerXml;

                                sb_SCHRptDetailsColumns.AppendFormat("<DAILYOPTION>{0}</DAILYOPTION>", str_FormData);
                                break;

                            case "WEEKLY":
                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/WEEKLYOPTION");
                                if (xpNav_FormDataNode != null)
                                    str_FormData = xpNav_FormDataNode.InnerXml;

                                PrepareWeeklyOption(ref str_FormData, "DATABASE");
                                sb_SCHRptDetailsColumns.AppendFormat("<WEEKLYOPTION>{0}</WEEKLYOPTION>", str_FormData);
                                break;

                            case "YEARLY":
                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/YEARLYOPTION");
                                if (xpNav_FormDataNode != null)
                                    str_yrMonth = xpNav_FormDataNode.InnerXml;

                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/DayOFMonth");
                                if (xpNav_FormDataNode != null)
                                    str_yrDay = xpNav_FormDataNode.InnerXml;

                                str_FormData = PrepareYearlyOption(ref str_yrMonth, str_yrDay);

                                sb_SCHRptDetailsColumns.AppendFormat("<YEARLYOPTION>{0}</YEARLYOPTION>", str_FormData);
                                break;
                            case "MONTHLY":
                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/MONTHLYOPTIONTYPE");
                                if (xpNav_FormDataNode != null)
                                {
                                    str_FormData = xpNav_FormDataNode.InnerXml;
                                }

                                sb_SCHRptDetailsColumns.AppendFormat("<MONTHLYOPTIONTYPE>{0}</MONTHLYOPTIONTYPE>", str_FormData);

                                if (str_FormData.ToUpper() == "DATE")
                                {
                                    xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/MONTHLYOPTIONDATE");
                                    if (xpNav_FormDataNode != null)
                                        str_FormData = xpNav_FormDataNode.InnerXml;
                                    sb_SCHRptDetailsColumns.AppendFormat("<MONTHLYOPTIONDATE>{0}</MONTHLYOPTIONDATE>", str_FormData);
                                }
                                else if (str_FormData.ToUpper() == "DAY")
                                {
                                    xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/MONTHLYOPTIONDAY");
                                    if (xpNav_FormDataNode != null)
                                        str_FormData = xpNav_FormDataNode.InnerXml;

                                    sb_SCHRptDetailsColumns.AppendFormat("<MONTHLYOPTIONDAY>{0}</MONTHLYOPTIONDAY>", str_FormData);

                                    xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/MONTHLYOPTIONWEEKDAY");
                                    if (xpNav_FormDataNode != null)
                                        str_FormData = xpNav_FormDataNode.InnerXml;

                                    sb_SCHRptDetailsColumns.AppendFormat("<MONTHLYOPTIONDATE>{0}</MONTHLYOPTIONDATE>", str_FormData);
                                }

                                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SELECTEDMONTHS");
                                if (xpNav_FormDataNode != null)
                                    str_FormData = xpNav_FormDataNode.InnerXml;

                                PrepareMonthlyOption(ref str_FormData, "DATABASE");
                                sb_SCHRptDetailsColumns.AppendFormat("<SELECTEDMONTHS>{0}</SELECTEDMONTHS>", str_FormData);
                                break;
                            case "QUARTERLY":
                                break;
                        }
                    }
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/DELIVERYMODE");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<DELIVERYMODE>{0}</DELIVERYMODE>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/EXPORTFORMAT");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<EXPORTFORMAT>{0}</EXPORTFORMAT>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/NOTIFYME");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<NOTIFYME>{0}</NOTIFYME>", xpNav_FormDataNode.InnerXml);
                }
               
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHEDULEREPORTTYPE");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SCHEDULEREPORTTYPE>{0}</SCHEDULEREPORTTYPE>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SUBJECT");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SUBJECT>{0}</SUBJECT>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/MESSAGE");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<MESSAGE>{0}</MESSAGE>", xpNav_FormDataNode.InnerXml);
                }
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHEDULEREPORTFILTERXML");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetailsColumns.AppendFormat("<SCHEDULEREPORTFILTERXML>{0}</SCHEDULEREPORTFILTERXML>", xpNav_FormDataNode.InnerXml);
                }

                #region Append CreatedOn and LastUpdatedOn node
                StringBuilder sbDateNode = new StringBuilder(16);
                sbDateNode.AppendFormat(@"<DAY>{0}</DAY>
                                        <MONTH>{1}</MONTH>
                                        <YEAR>{2}</YEAR>
                                        <HOUR>{3}</HOUR>
                                        <MINUTE>{4}</MINUTE>
                                        <SECOND>{5}</SECOND>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                sb_SCHRptDetailsColumns.AppendFormat(@"<CREATEDON>{0}</CREATEDON>
                                                <LASTUPDATEDON>{0}</LASTUPDATEDON>", sbDateNode.ToString());
                #endregion

                #endregion

                #region Prepare Scheduler Details Xml
                StringBuilder sb_SCHRptDetails = new StringBuilder(1024);
                sb_SCHRptDetails.Append("<SCHEDULEDREPORTDETAILS>");
                sb_SCHRptDetails.Append(sb_SCHRptDetailsColumns);

                #region Prepare Schedule Instance Details
                /*if (IsInstanceReq)
                {
                    StringBuilder Sb_ScheduleInstanceXml = null;
                    String FirstInstnceUniqueID          = null;
                    PrepareScheduleInsanceXml("<SCHEDULEDREPORTDETAILS>" + sb_SCHRptDetailsColumns + "</SCHEDULEDREPORTDETAILS>", true, GetBaseDate(), out Sb_ScheduleInstanceXml);
                    
                    if (Sb_ScheduleInstanceXml != null)
                    {
                        String ScheduleInstanceXml = Sb_ScheduleInstanceXml.ToString();
                        SetFirstScheduleInsanceUniqueID(ref ScheduleInstanceXml, out FirstInstnceUniqueID);
                        sb_SCHRptDetails.Append(ScheduleInstanceXml);

                        sb_SCHRptDetails.AppendFormat(@"<SCHRPT_NEXTRUN TYPE='RELATION'>{0}</SCHRPT_NEXTRUN>", FirstInstnceUniqueID);
                    }
                    
                }*/
                #endregion

                #region Prepare reated Entity Node
                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/SCHRPT_ANALYTICALRPT");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetails.AppendFormat(xpNav_FormDataNode.OuterXml);
                }

                xpNav_FormDataNode = xpNav_FormDataXML.SelectSingleNode("SCHEDULEDREPORTDETAILS/ADHOCREPORT_SCHRPTDETAILS");
                if (xpNav_FormDataNode != null)
                {
                    sb_SCHRptDetails.AppendFormat(xpNav_FormDataNode.OuterXml);
                }
                #endregion

                #region Append Report Receivers node
                XPathNodeIterator xpItr_TOUser = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_TOUSER");
                if (xpItr_TOUser != null && xpItr_TOUser.Count > 0)
                {
                    while (xpItr_TOUser.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_TOUser.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_CCUser = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_CCUSER");
                if (xpItr_CCUser != null && xpItr_CCUser.Count > 0)
                {
                    while (xpItr_CCUser.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_CCUser.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_BCCUser = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_BCCUSER");
                if (xpItr_BCCUser != null && xpItr_BCCUser.Count > 0)
                {
                    while (xpItr_BCCUser.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_BCCUser.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_TOGroup = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_TOGROUP");
                if (xpItr_TOGroup != null && xpItr_TOGroup.Count > 0)
                {
                    while (xpItr_TOGroup.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_TOGroup.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_CCGroup = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_CCGROUP");
                if (xpItr_CCGroup != null && xpItr_CCGroup.Count > 0)
                {
                    while (xpItr_CCGroup.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_CCGroup.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_BCCGroup = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_BCCGROUP");
                if (xpItr_BCCGroup != null && xpItr_BCCGroup.Count > 0)
                {
                    while (xpItr_BCCGroup.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_BCCGroup.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_TORecipient = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_TOEMAILRECIPIENT");
                if (xpItr_TORecipient.Count>0)
                {
                    while (xpItr_TORecipient.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_TORecipient.Current.OuterXml);
                    }
                }
                XPathNodeIterator xpItr_CCRecipient = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_CCEMAILRECIPIENT");
                if (xpItr_CCRecipient.Count > 0)
                {
                    while (xpItr_CCRecipient.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_CCRecipient.Current.OuterXml);
                    }
                }

                XPathNodeIterator xpItr_BCCRecipient = xpNav_FormDataXML.Select("SCHEDULEDREPORTDETAILS/SCHRPT_BCCEMAILRECIPIENT");
                if (xpItr_BCCRecipient.Count > 0)
                {
                    while (xpItr_BCCRecipient.MoveNext())
                    {
                        sb_SCHRptDetails.Append(xpItr_BCCRecipient.Current.OuterXml);
                    }
                }

                #endregion

                #region Append Report Filter input xml
                if (!String.IsNullOrEmpty(FilterXML))
                {
                    sb_SCHRptDetails.AppendFormat(@"<FILTERVALUES>{0}</FILTERVALUES>", FilterXML);
                }
                #endregion

                if (IsOperationCreate)
                    sb_SCHRptDetails.AppendFormat(@"<SCHRPT_OWNER TYPE='RELATION'>{0}</SCHRPT_OWNER>", UserID);

                sb_SCHRptDetails.Append("</SCHEDULEDREPORTDETAILS>");
                #endregion

                objInfEngine = new InfrastructureEngineClass(SessionID, UserID, Language, TenantName);
                str_SCHRptDetails = sb_SCHRptDetails.ToString();

                if (IsOperationCreate)
                    ErrorXml = objInfEngine.CreateEntityInstance(ref str_SCHRptDetails);                    
                else
                    ErrorXml = objInfEngine.UpdateEntityInstance(ref str_SCHRptDetails);

                #region Prepare Schedule Instance Details
                if (ErrorXml == null && IsInstanceReq)
                    ErrorXml = UpdateReportScheduleInstanceForNextRun(ScheduleReportID, null, null, null, 0);
                #endregion
            }
            return ErrorXml;
        }

        private void PrepareScheduleInsanceXml(String ScheduleDetailsXml, Boolean PrepareOnlyOneScheduleInstnaceXml, DateTime NextOfDateTime, out StringBuilder ScheduleInstanceXml)
        {
            String Startdate                = null;
            String Endtdate                 = null;
            String ScheduleTime             = null;

            Boolean IsTypeRunOnce           = false;
            String RunCycleOption           = null;
            String DailyOption              = null;
            String WeeklyOption             = null;
            String MonthlyOption            = null;
            String MonthlyDateOption        = null;
            String MonthlyDayOption         = null;
            String MonthlyDayOfaWeekOption  = null;
            String SelectedMonths           = null;
            String YearMonthDay             = null;
            String YearMonth                = null;
            String FiscalMonth              = null;

            XPathDocument   xpDoc_ScheduleDetailsXml    = new XPathDocument(new StringReader(ScheduleDetailsXml));
            XPathNavigator  xpNav_ScheduleDetailsNode   = xpDoc_ScheduleDetailsXml.CreateNavigator().SelectSingleNode("SCHEDULEDREPORTDETAILS");

            XPathNavigator StartDateColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("STARTDATE");
            if (StartDateColumn != null)
            {
                DateTime StartdateTime = XmlToDateTime(StartDateColumn);
                Startdate = StartdateTime.ToString("s");
                Startdate = Startdate.Substring(0, 11) + "00:00:00";
            }

            XPathNavigator ScheduleTypeColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("SCHEDTYPE");
            DateTime EnddateTime;
            if (ScheduleTypeColumn != null && ScheduleTypeColumn.InnerXml.ToUpper() == "RUN ONCE")
            {
                IsTypeRunOnce = true;
                RunCycleOption = "DAILY";
                DailyOption = "ALLDAYS";

                EnddateTime = XmlToDateTime(StartDateColumn);
                Endtdate = EnddateTime.ToString("s");
                Endtdate = Endtdate.Substring(0, 11) + "23:59:59";
            }
            else
            {
                XPathNavigator EndtDateColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("ENDDATE");
                if (EndtDateColumn != null)
                {
                    EnddateTime = XmlToDateTime(EndtDateColumn);
                    Endtdate = EnddateTime.ToString("s");
                    Endtdate = Endtdate.Substring(0, 11) + "23:59:59";
                }
            }

            XPathNavigator ScheduleTimeColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("SCHEDTIME");
            if (ScheduleTimeColumn != null)
                ScheduleTime = ScheduleTimeColumn.InnerXml;

            if (!IsTypeRunOnce)
            {
                XPathNavigator ScheduleRunCycleOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("RUNCYCLE");
                if (ScheduleRunCycleOptionColumn != null)
                {
                    RunCycleOption = ScheduleRunCycleOptionColumn.InnerXml;

                    switch (RunCycleOption)
                    {
                        case "DAILY":
                            XPathNavigator ScheduleDailyOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("DAILYOPTION");
                            if (ScheduleDailyOptionColumn != null)
                                DailyOption = ScheduleDailyOptionColumn.InnerXml;
                            break;

                        case "WEEKLY":
                            XPathNavigator ScheduleWeeklyOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("WEEKLYOPTION");
                            if (ScheduleWeeklyOptionColumn != null)
                                WeeklyOption = ScheduleWeeklyOptionColumn.InnerXml;
                            break;

                        case "YEARLY":
                            XPathNavigator ScheduleYearlyOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("YEARLYOPTION");
                            if (ScheduleYearlyOptionColumn != null)
                            {
                                //Handling Pending
                                YearMonthDay = ScheduleYearlyOptionColumn.InnerXml.Substring(0, 2);
                                YearMonth = ScheduleYearlyOptionColumn.InnerXml.Substring(2, 2);
                            }
                            break;
                        case "MONTHLY":
                            XPathNavigator ScheduleMonthlyOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("MONTHLYOPTIONTYPE");
                            if (ScheduleMonthlyOptionColumn != null)
                            {
                                MonthlyOption = ScheduleMonthlyOptionColumn.InnerXml;

                                switch (MonthlyOption.ToUpper())
                                {
                                    case "DATE":
                                        XPathNavigator ScheduleMonthlyDateOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("MONTHLYOPTIONDATE");
                                        if (ScheduleMonthlyDateOptionColumn != null)
                                            MonthlyDateOption = ScheduleMonthlyDateOptionColumn.InnerXml;

                                        break;
                                    case "DAY":
                                        XPathNavigator ScheduleMonthlyDayOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("MONTHLYOPTIONDAY");
                                        if (ScheduleMonthlyDayOptionColumn != null)
                                            MonthlyDayOption = ScheduleMonthlyDayOptionColumn.InnerXml;

                                        XPathNavigator ScheduleMonthlyDayOfaWeekOptionColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("MONTHLYOPTIONDATE");
                                        if (ScheduleMonthlyDayOfaWeekOptionColumn != null)
                                            MonthlyDayOfaWeekOption = ScheduleMonthlyDayOfaWeekOptionColumn.InnerXml;
                                        break;
                                }
                            }

                            XPathNavigator ScheduleSelectedMonthsColumn = xpNav_ScheduleDetailsNode.SelectSingleNode("SELECTEDMONTHS");
                            if (ScheduleSelectedMonthsColumn != null)
                                SelectedMonths = ScheduleSelectedMonthsColumn.InnerXml;

                            break;
                        case "QUARTERLY":
                            GetConfigurationSettings(out FiscalMonth);
                            break;
                    }
                }
            }


            PrepareScheduleInsanceXml(Startdate, Endtdate, ScheduleTime, RunCycleOption, FiscalMonth, DailyOption, WeeklyOption,
                MonthlyOption, MonthlyDateOption, MonthlyDayOption, MonthlyDayOfaWeekOption, SelectedMonths, YearMonthDay, YearMonth, PrepareOnlyOneScheduleInstnaceXml, NextOfDateTime, out ScheduleInstanceXml);
        }


        private void PrepareScheduleInsanceXml(String StartDate, String EndDate, String ScheduleTime, String RunCycleOption, String FiscalMonth, String DailyOption, String WeeklyOption,
                                                String MonthlyOption, String MonthlyDate, String MonthlyOptionDay, String DayOfaWeek, String SelectedMonths, String YearDay, String YearMonth, 
                                                Boolean PrepareOnlyOneScheduleInstnaceXml, DateTime NextOfDateTime, out StringBuilder ScheduleInstanceXml)
        {
            ScheduleInstanceXml = null;

            //First create the necessary structures
            ScheduleInstancesCalculator sic = new ScheduleInstancesCalculator();
            SchedulerTimeStructure stc      = new SchedulerTimeStructure();
            stc.StartDate   = DateTime.Parse(StartDate);
            stc.EndDate     = DateTime.Parse(EndDate);

            stc.TimeOfSendingReport = ScheduleTime;

            if (!String.IsNullOrEmpty(RunCycleOption))
            {
                switch (RunCycleOption.ToUpper())
                {
                    case "FORTNIGHTLY":
                        stc.RunCycle = EnumRunCycleTypes.FORTNIGHTLY;

                        FortnightlySchedule fs = new FortnightlySchedule();
                        stc.fortnightlySchedule = fs;
                        break;

                    case "QUARTERLY":
                        stc.RunCycle = EnumRunCycleTypes.QUARTERLY;

                        QuarterlySchedule qs = new QuarterlySchedule();
                        if (!String.IsNullOrEmpty(FiscalMonth))
                        {
                            qs.StartingMonthOfYear = Int32.Parse(FiscalMonth);
                        }
                        stc.quarterlySchedule = qs;
                        break;

                    case "DAILY":
                        stc.RunCycle = EnumRunCycleTypes.DAILY;

                        DailySchedule ds = new DailySchedule();

                        if (DailyOption.ToUpper() == "ALLDAYS")
                        {
                            ds.DailyScheduleType = EnumDailyScheduleTypes.ALL;
                        }
                        else if (DailyOption.ToUpper() == "WEEKDAYSONLY")
                        {
                            String          TenantConfigPath    = ALCommon.GetTenantXMLPath(TenantName);
                            XPathDocument   xpDoc_TenantXml     = new XPathDocument(TenantConfigPath);
                            XPathNavigator  xpNav_TenantXml     = xpDoc_TenantXml.CreateNavigator();
                            xpNav_TenantXml = xpNav_TenantXml.SelectSingleNode("CONFIGURATION/DATECALCULATION/WORKINGDAYS");
                            if (xpNav_TenantXml != null)
                            {
                                ds.WorkingDays = xpNav_TenantXml.InnerXml;
                            }
                            ds.DailyScheduleType = EnumDailyScheduleTypes.WEEKDAYS;
                        }

                        stc.dailySchedule = ds;
                        break;

                    case "WEEKLY":
                        stc.RunCycle = EnumRunCycleTypes.WEEKLY;

                        WeeklySchedule ws = new WeeklySchedule();
                        String[] Array = Regex.Split(WeeklyOption, "^|");
                        for (int i = 0; i < Array.Length; i++)
                        {
                            if (Array[i] == "0")
                            {
                                ws.SendOnSunday = true;
                            }
                            else if (Array[i] == "1")
                            {
                                ws.SendOnMonday = true;
                            }
                            else if (Array[i] == "2")
                            {
                                ws.SendOnTuesday = true;
                            }
                            else if (Array[i] == "3")
                            {
                                ws.SendOnWednesday = true;
                            }
                            else if (Array[i] == "4")
                            {
                                ws.SendOnThursday = true;
                            }
                            else if (Array[i] == "5")
                            {
                                ws.SendOnFriday = true;
                            }
                            else if (Array[i] == "6")
                            {
                                ws.SendOnSaturday = true;
                            }
                        }

                        stc.weeklySchedule = ws;
                        break;

                    case "YEARLY":
                        stc.RunCycle = EnumRunCycleTypes.YEARLY;

                        YearlySchedule ys = new YearlySchedule();
                        ys.DateMonth = string.Format("{0:00}{1:00}", Int32.Parse(YearDay), YearMonth);

                        stc.yearlySchedule = ys;
                        break;
                    case "MONTHLY":
                        stc.RunCycle = EnumRunCycleTypes.MONTHLY;

                        MonthlySchedule ms = new MonthlySchedule();
                       
                        if (MonthlyOption.ToUpper() == "DATE")
                        {
                            ms.MonthlyScheduleType = EnumMonthlyScheduleTypes.DATE;
                            if (MonthlyDate.ToUpper() == "LAST")
                            {
                                ms.DayOfMonthType = EnumMonthlyDayOfMonthTypes.LAST_DAY_OF_MONTH;
                            }
                            else
                            {
                                ms.DayOfMonthType = EnumMonthlyDayOfMonthTypes.ACTUAL_DAY_OF_MONTH;
                                ms.DayOfMonth = Int32.Parse(MonthlyDate);
                            }
                        }
                        else if (MonthlyOption.ToUpper() == "DAY")
                        {
                            ms.MonthlyScheduleType = EnumMonthlyScheduleTypes.DAY;

                            if (MonthlyOptionDay.ToUpper() == "FIRST")
                                ms.DayOfWeekValue = EnumMonthlyDayOfWeekValues.FIRST;
                            else if (MonthlyOptionDay.ToUpper() == "LAST")
                                ms.DayOfWeekValue = EnumMonthlyDayOfWeekValues.LAST;

                            if (DayOfaWeek.ToUpper() == "SUNDAY")
                                ms.DayOfWeek = DayOfWeek.Sunday;
                            else if (DayOfaWeek.ToUpper() == "MONDAY")
                                ms.DayOfWeek = DayOfWeek.Monday;
                            else if (DayOfaWeek.ToUpper() == "TUESDAY")
                                ms.DayOfWeek = DayOfWeek.Tuesday;
                            else if (DayOfaWeek.ToUpper() == "WEDNESDAY")
                                ms.DayOfWeek = DayOfWeek.Wednesday;
                            else if (DayOfaWeek.ToUpper() == "THURSDAY")
                                ms.DayOfWeek = DayOfWeek.Thursday;
                            else if (DayOfaWeek.ToUpper() == "FRIDAY")
                                ms.DayOfWeek = DayOfWeek.Friday;
                            else if (DayOfaWeek.ToUpper() == "SATURDAY")
                                ms.DayOfWeek = DayOfWeek.Saturday;
                        }
                        else
                        {
                            //Default Handling for Month Day not Selected
                            ms.MonthlyScheduleType  = EnumMonthlyScheduleTypes.DATE;
                            ms.DayOfMonthType       = EnumMonthlyDayOfMonthTypes.ACTUAL_DAY_OF_MONTH;
                            ms.DayOfMonth           = stc.StartDate.Day;
                        }

                        String[] ArrayMonths = null;
                        String[] Seperator = { "$AL$" };
                        ArrayMonths = SelectedMonths.Split(Seperator, StringSplitOptions.RemoveEmptyEntries);
                        if (ArrayMonths.Length > 0)
                        {
                            for (int i = 0; i < ArrayMonths.Length; i++)
                            {
                                if (ArrayMonths[i] == "1")
                                {
                                    ms.SendOnJan = true;
                                }
                                else if (ArrayMonths[i] == "2")
                                {
                                    ms.SendOnFeb = true;
                                }
                                else if (ArrayMonths[i] == "3")
                                {
                                    ms.SendOnMarch = true;
                                }
                                else if (ArrayMonths[i] == "4")
                                {
                                    ms.SendOnApr = true;
                                }
                                else if (ArrayMonths[i] == "5")
                                {
                                    ms.SendOnMay = true;
                                }
                                else if (ArrayMonths[i] == "6")
                                {
                                    ms.SendOnJune = true;
                                }
                                else if (ArrayMonths[i] == "7")
                                {
                                    ms.SendOnJuly = true;
                                }
                                else if (ArrayMonths[i] == "8")
                                {
                                    ms.SendOnAug = true;
                                }
                                else if (ArrayMonths[i] == "9")
                                {
                                    ms.SendOnSept = true;
                                }
                                else if (ArrayMonths[i] == "10")
                                {
                                    ms.SendOnOct = true;
                                }
                                else if (ArrayMonths[i] == "11")
                                {
                                    ms.SendOnNov = true;
                                }
                                else if (ArrayMonths[i] == "12")
                                {
                                    ms.SendOnDec = true;
                                }
                            }
                        }

                        stc.monthlySchedule = ms;
                        break;
                }
            }
            try
            {
                List<ScheduleInstance> sc = sic.GetScheduleInstances(stc);
                if (sc != null && sc.Count > 0)
                {
                    ScheduleInstanceXml = new StringBuilder(1024);
                    ScheduleInstanceXml.Append("<SCHEDULEDREPORT_INSTANCES TYPE='RELATION'>");
                    foreach (ScheduleInstance si in sc)
                    {
                        if (PrepareOnlyOneScheduleInstnaceXml)
                        {
                            if (!Helper.IsPastDate(si.ScheduledDateTime) && si.ScheduledDateTime > NextOfDateTime)
                            {
                                ScheduleInstanceXml.Append(si.ToXML());
                                break;
                            }
                        }
                        else
                            ScheduleInstanceXml.Append(si.ToXML());
                    }

                    ScheduleInstanceXml.Append("</SCHEDULEDREPORT_INSTANCES>");
                }
            }
            catch
            {
                ScheduleInstanceXml = null;
            }
        }

        private void SetFirstScheduleInsanceUniqueID(ref String ScheduleInstanceXml, out String FirstInstnceUniqueID)
        {
            FirstInstnceUniqueID = null;
            if (ScheduleInstanceXml != null)
            {
                ulong FirstinstnceUniqueID = 0;
                Assetlink.UniqueID.Read(ref FirstinstnceUniqueID);
                FirstInstnceUniqueID    = FirstinstnceUniqueID.ToString();
                ScheduleInstanceXml     = ScheduleInstanceXml.Replace("<SCHEDULEDREPORT_INSTANCES TYPE='RELATION'><SCHEDULEDREPORTINSTANCES>", "<SCHEDULEDREPORT_INSTANCES TYPE='RELATION'><SCHEDULEDREPORTINSTANCES><SCHEDREPORTINSTANCEID>" + FirstInstnceUniqueID + "</SCHEDREPORTINSTANCEID>");
            }
        }

        private DateTime GetBaseDate()
        {
           return  new DateTime(1985, 1, 1);
        }

        private void GetConfigurationSettings(out String FiscalMonth)
        {
            AssetlinkSearch AssetlinkSearchobj  = null;
            XPathDocument   xpDoc_Config        = null;
            XPathNavigator  xpNav_Config        = null;
            XPathNavigator  xpNav_ConfigNodes   = null;
            StringBuilder   sb_SearchXML        = null;
            String          str_SearchOutput    = null;
            FiscalMonth                         = "1";

            #region Prepare Search Xml
            sb_SearchXML = new StringBuilder(512);
            sb_SearchXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>CONFIGURATION</TARGETOBJECT>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'/>
                                            <TARGETENTITY>
                                                <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>CONFIGURATIONID</FIELDNAME>
                                                        <ALIAS>CONFIGURATIONID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>CONFIGNAME</FIELDNAME>
                                                        <ALIAS>CONFIGNAME</ALIAS>
                                                    </EXPRESSION>                                                                                             
                                                    <EXPRESSION>
                                                        <FIELDNAME>CONFIGVALUE</FIELDNAME>
                                                        <ALIAS>CONFIGVALUE</ALIAS>
                                                    </EXPRESSION>  
                                                </SELECTLIST>                                                
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <FILTERID>1</FILTERID>
                                                        <EXPRESSION>CONFIGNAME</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>FISCALMONTH</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                            </TARGETENTITY>
                                        </SEARCH>", Language, TenantName, UserID, SessionID);
            #endregion
            AssetlinkSearchobj = new AssetlinkSearch(UserID, Language, TenantName);
            ErrorXML = AssetlinkSearchobj.GetHierarchialData(sb_SearchXML.ToString(), ref str_SearchOutput);
            if (String.IsNullOrEmpty(ErrorXML))
            {
                xpDoc_Config = new XPathDocument(new StringReader(str_SearchOutput));
                xpNav_Config = xpDoc_Config.CreateNavigator();

                xpNav_ConfigNodes = xpNav_Config.SelectSingleNode("CONFIGURATIONCOLLECTION/CONFIGURATION[CONFIGNAME[text()='FISCALMONTH']]/CONFIGVALUE");
                if (xpNav_ConfigNodes != null)
                    FiscalMonth = xpNav_ConfigNodes.InnerXml;
            }
        }

        private ERRORXML GetAllScheduledInstances(String ScheduledReportID,out String ScheduledInstanceIDList, out String DisNextRunID)
        {
            #region Member Variables
            StringBuilder                                   sbSessionInfo               = null;
            StringBuilder                                   sbInputXML                  = null;
            StringBuilder                                   SCHInstancesIDList          = null;
            String                                          SCHInstanceXML              = null;
            XPathDocument                                   xpDoc_InstanceXML           = null;
            XPathNavigator                                  xpNav_InstanceXML           = null;
            XPathNodeIterator                               xpItr_InstanceIDList        = null;
            Assetlink.Services.Engines.AssetlinkSearch      objSearch                   = null;
            #endregion

            ScheduledInstanceIDList = null;
            DisNextRunID = null;
            objSearch               = new AssetlinkSearch(UserID, Language, TenantName);
            sbSessionInfo           = new StringBuilder(16);
            sbSessionInfo.AppendFormat("<SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'></SESSIONINFO>", Language, TenantName, UserID, SessionID);

            sbInputXML = new StringBuilder(16);
            sbInputXML.AppendFormat(@"<SEARCH>
	                                    <TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>
	                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                    <TARGETENTITY>
		                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>SCHEDREPORTID</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>		                                                        
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>                                            
                                            </FILTERLIST>                                             
                                            <RELATIONSHIP NAME='SCHEDULEDREPORT_INSTANCES'>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>STATUS</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>PENDING</EXPRESSIONVALUE>		                                                        
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>                                            
                                                </FILTERLIST>	
                                                <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
			                                            <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
		                                            </EXPRESSION>	
                                                </SELECTLIST>	                                                                                    
		                                    </RELATIONSHIP> 
                                            <RELATIONSHIP NAME='SCHRPT_NEXTRUN'>
                                                <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
			                                            <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
		                                            </EXPRESSION>	
                                                </SELECTLIST>
                                            </RELATIONSHIP>                                           
                                        </TARGETENTITY>{0}
                                    </SEARCH>", sbSessionInfo.ToString(), ScheduledReportID);

            ErrorXML = objSearch.GetHierarchialData(sbInputXML.ToString(), ref SCHInstanceXML);
            if (ErrorXML == null)
            {
                xpDoc_InstanceXML = new XPathDocument(new StringReader(SCHInstanceXML));
                xpNav_InstanceXML = xpDoc_InstanceXML.CreateNavigator();
                xpItr_InstanceIDList = xpNav_InstanceXML.Select("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS/SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID");
                if (xpItr_InstanceIDList != null && xpItr_InstanceIDList.Count > 0)
                {
                    SCHInstancesIDList = new StringBuilder(16);
                    while (xpItr_InstanceIDList.MoveNext())
                    {
                        SCHInstancesIDList.Append(xpItr_InstanceIDList.Current.InnerXml);
                        if (xpItr_InstanceIDList.CurrentPosition != xpItr_InstanceIDList.Count)
                        {
                            SCHInstancesIDList.Append(",");
                        }
                    }
                    ScheduledInstanceIDList = SCHInstancesIDList.ToString();
                }
                XPathNavigator xpNav_NextRun = xpNav_InstanceXML.SelectSingleNode("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS/SCHRPT_NEXTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID");
                if (xpNav_NextRun != null)
                {
                    DisNextRunID = xpNav_NextRun.InnerXml;
                }

            }
            return ErrorXML;
        }

        private String GetMonth(String Month)
        {
            String strMonthNumber = null;

            switch (Month.ToString())
            {
                case "JANUARY": strMonthNumber = "01";
                    break;
                case "FEBRUARY": strMonthNumber = "02";
                    break;
                case "MARCH": strMonthNumber = "03";
                    break;
                case "APRIL": strMonthNumber = "04";
                    break;
                case "MAY": strMonthNumber = "05";
                    break;
                case "JUNE": strMonthNumber = "06";
                    break;
                case "JULY": strMonthNumber = "07";
                    break;
                case "AUGUST": strMonthNumber = "08";
                    break;
                case "SEPTEMBER":strMonthNumber = "09";
                    break;
                case "OCTOBER": strMonthNumber = "10";
                    break;
                case "NOVEMBER":strMonthNumber = "11";
                    break;
                case "DECEMBER":strMonthNumber = "12";
                    break;
                default: break;
            }
            return strMonthNumber;
        }

        private ERRORXML GetAdHocReportLastModifiedByUserID(String AdHocReportID, out String LastModifiedByUserID)
        {
            ERRORXML        ErrorXML            = null;
            StringBuilder   SearchInputXML      = null;
            AssetlinkSearch AssetlinkSearchObj  = null;
            String          SearchOutputXml     = null;
            XmlDocument     SearchOutputXmlDoc  = null;
            LastModifiedByUserID                = null;

            #region Perform Search
                
            SearchInputXML      = new StringBuilder();
            AssetlinkSearchObj  = new AssetlinkSearch(UserID, Language, TenantName);
            SearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>ADHOCREPORT</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <SESSIONINFO Language='{0}' UserID='{1}' TenantName='{2}' SessionID='{3}'/>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
                                                <FILTERLIST>
	                                                <FILTER>
		                                                <EXPRESSION>REPORTID</EXPRESSION>
		                                                <OPERATOR>EQUALTO</OPERATOR>
		                                                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
		                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
	                                                </FILTER>
                                                </FILTERLIST>
                                                <RELATIONSHIP NAME='ADHOCREPORT_MODIFIEDBY' >
                                                <SELECTLIST>
			                                        <EXPRESSION>
				                                        <FIELDNAME>USERID</FIELDNAME>
			                                            </EXPRESSION>
                                                </SELECTLIST>
                                              </RELATIONSHIP>
                                            </TARGETENTITY>
                                        </SEARCH>", Language, UserID.ToString(), TenantName, SessionID, AdHocReportID);

            ErrorXML = AssetlinkSearchObj.GetHierarchialData(SearchInputXML.ToString(), ref SearchOutputXml);
            if (ErrorXML == null)
            {
                SearchOutputXmlDoc = new XmlDocument();
                SearchOutputXmlDoc.LoadXml(SearchOutputXml);

                if (SearchOutputXmlDoc.DocumentElement.FirstChild.SelectSingleNode("ADHOCREPORT_MODIFIEDBY/USER/USERID") != null)
                    LastModifiedByUserID = SearchOutputXmlDoc.DocumentElement.FirstChild.SelectSingleNode("ADHOCREPORT_MODIFIEDBY/USER/USERID").InnerText;
            }
            
            return ErrorXML;
            #endregion
        }

        //getAdHocReportDetailsForExcelGeneration() method is used to retrieve the ad-hoc report details and configuration info.
        //This requiers for generating an Excel.
        private ERRORXML getAdHocReportDetailsForExcelGeneration(String AdHocReportID,String FilterPersonalizationXML,out String AdHocReportName,
                out String TargetObjectTypeName, out XML DisplayColumnsConfigXML, ref DataSet DS_Result)
        {
            #region Method Variables

            AssetlinkSearch search_EngineObject     = null;
            XPathDocument   xpathdoc_AdHocReport    = null;
            XPathNavigator  xpathnav_AdHocReport    = null;
            XPathNavigator  xpathnav_Config         = null;
            String          str_InsertString        = null;
            StringBuilder   strbldr_SearchInputXML  = null;
            XML             xml_SearchInput         = null;
            XML             xml_SearchOutput        = null;
            ERRORXML        errorxml_Output         = null;            

            #endregion

            DS_Result               = null;
            AdHocReportName         = null;
            TargetObjectTypeName    = null;
            DisplayColumnsConfigXML = null;

            strbldr_SearchInputXML  = new StringBuilder();
            strbldr_SearchInputXML.AppendFormat(@"<SEARCH>
	                <TARGETOBJECT>ADHOCREPORT</TARGETOBJECT>
	                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
	                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                <SESSIONINFO UserID='{0}' Language='{1}' TenantName='{2}' SessionID='{3}'/>
	                <TARGETENTITY>
		                <SELECTLIST>
			                <EXPRESSION>
				                <FIELDNAME>REPORTID</FIELDNAME>
				                <ALIAS>REPORTID</ALIAS>
			                </EXPRESSION>
			                <EXPRESSION>
				                <FIELDNAME>REPORTNAME</FIELDNAME>
				                <ALIAS>REPORTNAME</ALIAS>
			                </EXPRESSION>
			                <EXPRESSION>
				                <FIELDNAME>REPORTCONFIGXML</FIELDNAME>
				                <ALIAS>REPORTCONFIGXML</ALIAS>
			                </EXPRESSION>
		                </SELECTLIST>
                        <FILTERLIST>
			                <FILTER>
				                <FILTERID>1</FILTERID>
				                <EXPRESSION>REPORTID</EXPRESSION>
				                <OPERATOR>EQUALTO</OPERATOR>
				                <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
				                <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                </FILTER>
                        </FILTERLIST>
	                </TARGETENTITY>
                </SEARCH>", UserID, Language, TenantName, SessionID, AdHocReportID);
            search_EngineObject = new AssetlinkSearch();
            errorxml_Output = search_EngineObject.GetHierarchialData(strbldr_SearchInputXML.ToString(), ref xml_SearchOutput);
            if ((!String.IsNullOrEmpty(errorxml_Output)) || (String.IsNullOrEmpty(xml_SearchOutput)))
            {
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_NOCONFIG", this.GetType().ToString(),
                        "getAdHocReportDetailsForExcelGeneration", null, errorxml_Output);
                return (ALEngineErrorObj.ErrorCollectionXML);
            }

            AdHocReportName = ALCommon.GetInnerValue(ref xml_SearchOutput, "<REPORTNAME>", "</REPORTNAME>");

            //Retrieve the search input XML from REPORTCONFIGXML.            
            xpathdoc_AdHocReport = new XPathDocument(new StringReader(xml_SearchOutput));
            xpathnav_Config = xpathdoc_AdHocReport.CreateNavigator().SelectSingleNode("ADHOCREPORTCOLLECTION/ADHOCREPORT/REPORTCONFIGXML/CONFIGURATION");
            xpathnav_AdHocReport = xpathnav_Config.SelectSingleNode("SEARCHINPUT/INPUTXML/SEARCHCOLLECTION/SEARCH");            
            if (xpathnav_AdHocReport != null)
            {
                xml_SearchInput = xpathnav_AdHocReport.OuterXml;
                if (String.IsNullOrEmpty(xml_SearchInput))
                {
                    ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_NOCONFIG", this.GetType().ToString(),
                            "getAdHocReportDetailsForExcelGeneration", null, null);
                    return (ALEngineErrorObj.ErrorCollectionXML);
                }

                str_InsertString = "<SESSIONINFO UserID='" + UserID + "' Language='" + Language
                        + "' TenantName='" + TenantName + "' SessionID='" + SessionID + "'/>";
                xml_SearchInput = ALCommon.InsertBefore(ref xml_SearchInput, "<TARGETENTITY>", ref str_InsertString);

				#region Add "FROMODS" attribute to "SEARCH" node so that search operation will be performed on appropriate Table

				XmlDocument tmpDoc = null;
				XPathNavigator xpNav = null;

				tmpDoc = new XmlDocument();
				tmpDoc.LoadXml(xml_SearchInput);

                //Apply Personalized filters to the report search XML
                ALReportHelper.MergePersonalizedFiltersIntoSearchXML(ref tmpDoc, FilterPersonalizationXML, "", null);

				xpNav = tmpDoc.CreateNavigator();

				if (xpNav.SelectSingleNode("SEARCH") != null)
					xpNav.SelectSingleNode("SEARCH").CreateAttribute("", "FromODS", "", "True");

				#endregion
				xml_SearchInput = tmpDoc.OuterXml;
            }

            //Retrieve the search input XML from REPORTCONFIGXML.
            xpathnav_AdHocReport = xpathnav_Config.SelectSingleNode("DisplayLayout/SeriesCollection/AdHocReportsDisplayColumns");            
            if (xpathnav_AdHocReport != null)
            {
                DisplayColumnsConfigXML = xpathnav_AdHocReport.InnerXml;
            }
            if (String.IsNullOrEmpty(DisplayColumnsConfigXML))
            {
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_NOCONFIG", this.GetType().ToString(),
                        "getAdHocReportDetailsForExcelGeneration", null, null);
                return (ALEngineErrorObj.ErrorCollectionXML);
            }

            //Retrieve the name of the target object type.
            xpathnav_AdHocReport = xpathnav_Config.SelectSingleNode("SearchTarget");            
            if (xpathnav_AdHocReport != null)
            {
                TargetObjectTypeName = xpathnav_AdHocReport.GetAttribute("ObjectName", "");
            }
            if (String.IsNullOrEmpty(TargetObjectTypeName))
            {
                ALEngineErrorObj.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(),
                        "getAdHocReportDetailsForExcelGeneration", null, null);
                return (ALEngineErrorObj.ErrorCollectionXML);
            }

            errorxml_Output = search_EngineObject.GetHierarchialData(xml_SearchInput, ref DS_Result);
            
            return (errorxml_Output);
        }

        private ERRORXML generateExcelForAdHocReport(String AdHocReportID, XML AdHocConfigForExcel, ref DataSet DS_Result, out XML FileCollectionXML)
        {
            #region Method Variables
            String      str_StringToBeInserted              = null;
            String      str_TenantFormatXMLPath             = null;
            String      str_CultureInfo                     = null;
            ERRORXML    errorxml_Output                     = null;

            #endregion

            FileCollectionXML       = null;
            #region STEP01: Copy FORMATS.XML file to shared folder and add this new path into the configuration

            str_TenantFormatXMLPath = ALCommon.GetFormatDetails(TenantName, out str_CultureInfo);
            str_StringToBeInserted  = "<FORMATXMLPATH>" + str_TenantFormatXMLPath + "</FORMATXMLPATH>";
            AdHocConfigForExcel     = ALCommon.InsertBefore(ref AdHocConfigForExcel, "</ADHOCCONFIGFOREXCEL>", ref str_StringToBeInserted);

            str_StringToBeInserted  = "<FORMATCULTURENAME>" + str_CultureInfo + "</FORMATCULTURENAME>";
            AdHocConfigForExcel     = ALCommon.InsertBefore(ref AdHocConfigForExcel, "</ADHOCCONFIGFOREXCEL>", ref str_StringToBeInserted);

            #endregion
            

            #region STEP02: Generate the Excel

            errorxml_Output         = generateExcel(AdHocConfigForExcel, ref DS_Result, TenantName, out FileCollectionXML);

            #endregion
            return (errorxml_Output);
        }

        //getMediaProcessorSharedFolderPath() method is used to retrieve the shared folder path for the Media Processor
        // to work with MSOfficeProcessor.
        private ERRORXML getMSOfficeProcessorFolderPath(out String MSOfficeProcessorLocalPath, out String MSOfficeProcessorNetworkPath)
        {
            #region Method Variables

            MediaRequestRouter  mediarequestrouter_Object   = null;
            String              str_NetworkPath             = null;
            String              str_LocalPath               = null;
            ERRORXML            errorxml_Output             = null;
            UInt64              unit_UniqueID               = 0;

            #endregion

            MSOfficeProcessorLocalPath      = null;
            MSOfficeProcessorNetworkPath    = null;

            //Get the MSOfficeProcessor's folder paths from the media request router XML.
            mediarequestrouter_Object = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), Language, "ASSETLINKSYSTEM");
            errorxml_Output = mediarequestrouter_Object.GetNetworkPath("MSOFFICEPROCESSOR", out str_NetworkPath, out str_LocalPath);
            if (!(String.IsNullOrEmpty(errorxml_Output)))
            {
                ALEngineErrorObj.AppendError("ERR_MSOFFICEPROCESSORPATHNOTGENERATE_SHARED", this.GetType().ToString(),
                        "getMSOfficeProcessorFolderPath", null, errorxml_Output);
                return (ALEngineErrorObj.ErrorCollectionXML);
            }
            else if ((String.IsNullOrEmpty(str_NetworkPath)) || (String.IsNullOrEmpty(str_LocalPath)))
            {
                ALEngineErrorObj.AppendError("ERR_MSOFFICEPROCESSORPATHNOTGENERATE_SHARED", this.GetType().ToString(),
                        "getMSOfficeProcessorFolderPath", null, errorxml_Output);
                return (ALEngineErrorObj.ErrorCollectionXML);
            }

            //Generate local machine folder path.
            ALCommon.GenerateUniqueID(ref unit_UniqueID);
            MSOfficeProcessorLocalPath = str_LocalPath + @"\MSOfficeProcessor\" + TenantName + @"\" + unit_UniqueID.ToString();
            if (!Directory.Exists(MSOfficeProcessorLocalPath))
                Directory.CreateDirectory(MSOfficeProcessorLocalPath);

            //Generate shared drive folder path on network.
            MSOfficeProcessorNetworkPath = str_NetworkPath + @"\MSOfficeProcessor\" + TenantName + @"\" + unit_UniqueID.ToString();
            //if (!Directory.Exists(MSOfficeProcessorNetworkPath))
            //    Directory.CreateDirectory(MSOfficeProcessorNetworkPath);

            return (errorxml_Output);
        }

        #region Private methods for Crystal Reports
        
        //exportCrystalReportDataToExcel() method is used to generate an excel file using passed DataReader.
        private ERRORXML exportCrystalReportDataToExcel(IDataReader SqlDataReaderObject, String ExcelFilePath)
        {
            # region Local Variable

            ERRORXML                ErrorXML        = null;
            ALMSOfficeProcessorWS   MSOfficeWSObj   = null;
            String                  NetworkPath     = null;
            String                  ReplacablePath  = null;
            String                  DataXMLPath     = null;
            String                  OutputFilePath  = null;
            ulong                   randomID        = 0;
            XML                     ftXML           = null;

            # endregion
            try
            {
                MediaRequestRouter mediaRequestRouterObj = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), Language, "ASSETLINKSYSTEM");
                mediaRequestRouterObj.GetNetworkPath("MSOFFICEPROCESSOR", out NetworkPath, out ReplacablePath);

                DataXMLPath = ReplacablePath + @"\MSOfficeProcessor";
                                
                if (!Directory.Exists(DataXMLPath))
                    Directory.CreateDirectory(DataXMLPath);

                DataXMLPath += @"\" + TenantName;
                
                if (!Directory.Exists(DataXMLPath))
                    Directory.CreateDirectory(DataXMLPath);
                
                ALCommon.GenerateUniqueID(ref randomID);

                DataXMLPath += @"\" + randomID.ToString();
                if (!Directory.Exists(DataXMLPath))
                    Directory.CreateDirectory(DataXMLPath);

                OutputFilePath = DataXMLPath.Replace(ReplacablePath, NetworkPath) + @"\ExportedData.xls";
                DataXMLPath+= @"\ExportedData.xml";

                System.Data.DataTable dt = new System.Data.DataTable("ExportedData");                
                dt.Load(SqlDataReaderObject);
                dt.WriteXml(DataXMLPath, XmlWriteMode.WriteSchema);
                SqlDataReaderObject.Close();
                SqlDataReaderObject.Dispose();

                DataXMLPath = DataXMLPath.Replace(ReplacablePath, NetworkPath);
                MSOfficeWSObj = new ALMSOfficeProcessorWS();
                MSOfficeWSObj.Url = ALCommon.GetWebServiceURL("ALMSOfficeProcessorWS");
                MSOfficeWSObj.Timeout = ALBaseEngine.GetWebServiceTimeOut();
                ErrorXML = MSOfficeWSObj.ExportCrystalReportDataToExcel(DataXMLPath, OutputFilePath);

                if (!String.IsNullOrEmpty(ErrorXML))
                    return ErrorXML;

                OutputFilePath = OutputFilePath.Replace(NetworkPath, ReplacablePath);
                FileTransferInfrastructure ftObj = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                ftObj.ToUseServiceURL = ALCommon.GetWebServiceURL("ALSOLUTIONMTOMWS");
                ftXML = "<FILECOLLECTION><FILE><PATH>" + OutputFilePath + "</PATH><DESTINATIONPATH>" + ExcelFilePath + "</DESTINATIONPATH><NAME>ExportedData.xls</NAME></FILE></FILECOLLECTION>";
                ErrorXML = ftObj.SaveFile(ref ftXML);
            }
            catch (Exception ExceptionObject)
            {
                ErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ErrorXML);
                ALEngineErrorObj.AppendError("ERR_RPT_REPORTENGINE_EXPORTFAILED", this.GetType().ToString(),
                        "exportCrystalReportDataToExcel", ExceptionObject, null);
                ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        /// <summary>
        /// This function executes Stored Procedure with Set of Parameters provided. For Teradata after a Stored Procedure is executed global Temporary
        /// table variable is populated with resultset. 
        /// </summary>
        /// <param name="StoreProcedureName">Name of Stored Procedure to be executed</param>
        /// <param name="SPDynamicParams">Parametes for Stored Procedure</param>
        /// <param name="TableName">Name of Global Temporary Table variable used in case of Teradata</param>
        /// <returns>Returns a Dataset object which has a resultset.</returns>
        public DataSet getStoredProcedureInformation(String StoreProcedureName, XML SPDynamicParams, String TableName)
        {
            #region Local Variables
            SqlConnection SQLConnObj = null;
            SqlCommand SQLCmdObj = null;
            SqlDataAdapter SQLDataAdapterObj = null;
            TdConnection TdConnection = null;
            TdCommand TdCCommandObj = null;
            TdDataAdapter TdDataAdapter = null;
            TdCommand TdCCommandforRead = null;
            DataSet dsProject = null;
            OracleConnection OracleConnObj = null;
            OracleCommand OracleCmdObj = null;
            OracleCommand OracleCmdforRead = null;
            OracleDataAdapter OrclDataAdapter = null;
            #endregion

            try
            {

                switch (this.m_DatabaseType)
                {
                    case "SQLSERVER":
                    case "SQLEXPRESS":
                        SQLConnObj = new System.Data.SqlClient.SqlConnection(this.m_ConnectionString);
                        SQLCmdObj = new System.Data.SqlClient.SqlCommand();
                        dsProject = new DataSet();
                        SQLCmdObj.Connection = SQLConnObj;
                        SQLCmdObj.CommandType = CommandType.StoredProcedure;
                        SQLCmdObj.CommandText = StoreProcedureName;
                        SQLCmdObj.CommandTimeout = 300;

                        // Assign Dynamic Parameters
                        AssignDynamicParamsforSQL(SPDynamicParams, ref SQLCmdObj);

                        SQLDataAdapterObj = new SqlDataAdapter((SqlCommand)SQLCmdObj);
                        SQLDataAdapterObj.Fill(dsProject);

                        SQLCmdObj.Dispose();
                        SQLConnObj.Close();
                        SQLConnObj.Dispose();
                        break;
                    case "TERADATA":
                        TdConnection = new TdConnection(this.m_ConnectionStringSP);
                        TdConnection.Open();
                        TdCCommandObj = new TdCommand();
                        dsProject = new DataSet();
                        TdCCommandObj.Connection = TdConnection;
                        TdCCommandObj.CommandType = CommandType.StoredProcedure;
                        TdCCommandObj.CommandText = StoreProcedureName;
                        TdCCommandObj.CommandTimeout = 300;

                        // Assign Dynamic Parameters
                        AssignDynamicParamsforTD(SPDynamicParams, ref TdCCommandObj);

                        TdCCommandObj.ExecuteNonQuery();
                        TdCCommandforRead = new TdCommand();
                        TdCCommandforRead.Connection = TdConnection;
                        TdCCommandforRead.CommandType = CommandType.Text;
                        if (!String.IsNullOrEmpty(strOrderByClause))
                        {
                            TdCCommandforRead.CommandText = " SELECT * FROM " + TableName + " " + strOrderByClause;
                            strOrderByClause = null;
                        }
                        else
                            TdCCommandforRead.CommandText = " SELECT * FROM " + TableName;
                        TdCCommandforRead.CommandTimeout = 300;
                        TdDataAdapter = new TdDataAdapter((TdCommand)TdCCommandforRead);
                        TdDataAdapter.Fill(dsProject);

                        TdCCommandObj.Dispose();
                        TdCCommandforRead.Dispose();
                        TdConnection.Close();
                        TdConnection.Dispose();
                        break;
                    case "ORACLE":
                    case "ORACLEEXPRESS":
                        OracleConnObj = new OracleConnection(this.m_ConnectionStringSP);
                        OracleConnObj.Open();
                        OracleCmdObj = new OracleCommand();
                        dsProject = new DataSet();
                        OracleCmdObj.CommandTimeout = 300;
                        OracleCmdObj.Connection = OracleConnObj;
                        OracleCmdObj.CommandType = CommandType.StoredProcedure;
                        OracleCmdObj.CommandText = StoreProcedureName;

                        LogRptPath = ALCommon.GetInstallationDir() + StoreProcedureName + ".txt";
                        if (!File.Exists(LogRptPath))
                            File.WriteAllText(LogRptPath, "");
                        else
                        {
                            File.AppendAllText(LogRptPath, "------------------------------------------------------------------------" + Environment.NewLine);
                            File.AppendAllText(LogRptPath, DateTime.Now.Day + "/" + DateTime.Now.Month + "/" + DateTime.Now.Year + ":" + DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second + Environment.NewLine);
                        }
                        File.AppendAllText(LogRptPath, "StoredProcedureName:" + StoreProcedureName + Environment.NewLine);

                        // Assign Dynamic Parameters
                        AssignDynamicParamsforOracle(SPDynamicParams, ref OracleCmdObj);

                        OracleCmdObj.ExecuteNonQuery();
                        OracleCmdforRead = new OracleCommand();
                        OracleCmdforRead.Connection = OracleConnObj;
                        OracleCmdforRead.CommandType = CommandType.Text;
                        if (!String.IsNullOrEmpty(strOrderByClause))
                        {
                            OracleCmdforRead.CommandText = " SELECT * FROM " + TableName + " " + strOrderByClause;
                            strOrderByClause = null;
                        }
                        else
                            OracleCmdforRead.CommandText = " SELECT * FROM " + TableName;
                        OracleCmdforRead.CommandTimeout = 300;
                        OrclDataAdapter = new OracleDataAdapter((OracleCommand)OracleCmdforRead);
                        OrclDataAdapter.Fill(dsProject);

                        OracleCmdObj.Dispose();
                        OracleCmdforRead.Dispose();
                        OracleConnObj.Close();
                        OracleConnObj.Dispose();

                        break;
                }

                return dsProject;
            }
            catch (SystemException exception)
            {
                throw exception;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="SPDynamicParams"></param>
        /// <param name="SQLCmdObj"></param>
        private void AssignDynamicParamsforSQL(XML SPDynamicParams, ref SqlCommand SQLCmdObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;
                        SQLCmdObj.Parameters.AddWithValue(strName, strVal);
                    }
                }
            }
            catch (Exception Ex)
            {
                throw Ex;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="SPDynamicParams"></param>
        /// <param name="TdCCommandObj"></param>
        private void AssignDynamicParamsforTD(XML SPDynamicParams, ref TdCommand TdCCommandObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;

                        //Separateout ORDER BY clause
                        if (strVal.ToUpper().Contains("ORDER BY"))
                        {
                            strOrderByClause = strVal.Substring(strVal.ToUpper().IndexOf("ORDER BY"));
                            strVal = strVal.Remove(strVal.ToUpper().IndexOf("ORDER BY"));
                        }
                        if (!String.IsNullOrEmpty(strName))
                        {
                            TdParameter ParamObj = TdCCommandObj.CreateParameter();
                            if (strName.Contains("@"))
                                strName = strName.Replace("@", "");
                            ParamObj.ParameterName = strName.ToUpper();
                            ParamObj.Value = strVal;
                            ParamObj.TdType = TdType.VarChar;
                            ParamObj.Direction = ParameterDirection.Input;
                            TdCCommandObj.Parameters.Add(ParamObj);
                        }
                    }
                }
            }
            catch (Exception Ex)
            {
                throw Ex;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="SPDynamicParams"></param>
        /// <param name="TdCCommandObj"></param>
        private void AssignDynamicParamsforOracle(XML SPDynamicParams, ref OracleCommand OracleCmdObj)
        {
            XPathDocument XpDoc_DynamicParams = null;
            XPathNavigator XpNav_DynamicParams = null;
            XPathNodeIterator XpNode_Iterator = null;
            String strName = null;
            String strVal = null;

            try
            {
                if (!String.IsNullOrEmpty(SPDynamicParams))
                {
                    XpDoc_DynamicParams = new XPathDocument(new StringReader(SPDynamicParams));
                    XpNav_DynamicParams = XpDoc_DynamicParams.CreateNavigator();
                    XpNode_Iterator = XpNav_DynamicParams.Select("PARAMLIST/PARAM");
                    while (XpNode_Iterator.MoveNext())
                    {
                        strName = XpNode_Iterator.Current.SelectSingleNode("NAME").Value;
                        strVal = XpNode_Iterator.Current.SelectSingleNode("VALUE").Value;
                        //if (String.IsNullOrEmpty(strVal))
                        //    strVal = " ";
                        //Separateout ORDER BY clause
                        if (strVal.ToUpper().Contains("ORDER BY"))
                        {
                            strOrderByClause = strVal.Substring(strVal.ToUpper().IndexOf("ORDER BY"));
                            strVal = strVal.Remove(strVal.ToUpper().IndexOf("ORDER BY"));
                        }
                        if (!String.IsNullOrEmpty(strName))
                        {
                            OracleParameter ParamObj = OracleCmdObj.CreateParameter();
                            if (strName.Contains("@"))
                                strName = strName.Replace("@", "");
                            ParamObj.ParameterName = strName.ToUpper();
                            File.AppendAllText(LogRptPath, "Parameter Name:" + strName.ToUpper() + Environment.NewLine);
                            //check for to_timestamp function
                            if (strVal.Contains("to_timestamp"))
                            {
                                strVal = strVal.Replace("'to_timestamp", "to_timestamp");
                                strVal = strVal.Replace("DD-MON-YY HH24.MI.SSXFF')'", "DD-MON-YY HH24.MI.SSXFF')");
                            }
                            File.AppendAllText(LogRptPath, "Parameter Value:" + strVal + Environment.NewLine);
                            File.AppendAllText(LogRptPath, "Parameter DataType:" + OracleDbType.NVarchar2.ToString() + Environment.NewLine);
                            File.AppendAllText(LogRptPath, "------------------------------------------------------------------------" + Environment.NewLine);
                            ParamObj.Value = strVal;
                            ParamObj.OracleDbType = OracleDbType.NVarchar2;
                            ParamObj.Direction = ParameterDirection.Input;
                            OracleCmdObj.Parameters.Add(ParamObj);
                        }
                    }
                }
            }
            catch (Exception Ex)
            {
                throw Ex;
            }
        }

        #endregion

        #region Private Methods For Asset Metadate Report

        private XML sortResultXml(String SearchResultXml, String SortFieldListXml, int SortType)
        {
            # region Local Variables
            XPathDocument       SearchResultXDoc    = null;
            XPathNavigator      SearchResultXNav    = null;
            XPathExpression     SortXExpr           = null;
            XmlDataType         XmlDataTypeObj      = XmlDataType.Text;
            XmlSortOrder        XmlSortOrderObj     = XmlSortOrder.Ascending;
            XPathNodeIterator   SortedDataXItr      = null;
            StringBuilder       xPath               = null;
            StringBuilder       SortedXml           = null;
            String              ObjectName          = null;
            String              RelationName        = null;
            String              FieldName           = null;
            String              DataType            = null;
            String              SortValue           = null;
            XPathDocument       SortFieldXDoc       = null;
            XPathNavigator      SortFieldXNav       = null;
            # endregion

            try
            {
                SortFieldXDoc = new XPathDocument(new StringReader(SortFieldListXml));
                SortFieldXNav = SortFieldXDoc.CreateNavigator();

                SearchResultXDoc = new XPathDocument(new StringReader(SearchResultXml));
                SearchResultXNav = SearchResultXDoc.CreateNavigator();

                SortXExpr = SearchResultXNav.Compile("DIGITALASSETCOLLECTION/DIGITALASSET");
                xPath = new StringBuilder(10);
                foreach (XPathNavigator SortFiedXNav in SortFieldXNav.Select("SortFields/FIELD"))
                {
                    SortValue       = SortFiedXNav.GetAttribute("Name", "");
                    xPath.Remove(0, xPath.Length);
                    RelationName    = SortFiedXNav.GetAttribute("Relation", "");
                    FieldName       = SortFiedXNav.GetAttribute("FieldName", "");
                    DataType        = SortFiedXNav.GetAttribute("DataType", "").ToUpper();
                    ObjectName      = SortFiedXNav.GetAttribute("ObjectName", "");

                    if ((String.IsNullOrEmpty(DataType) == false) && ((String.Compare(DataType, "DATETIME") == 0) || (String.Compare(DataType, "INTEGER") == 0) || (String.Compare(DataType, "FLOAT") == 0)))
                        XmlDataTypeObj = XmlDataType.Number;

                    if (String.Compare(RelationName, "DIGITALASSET", true) != 0)
                        xPath.AppendFormat("{0}/{1}/", RelationName, ObjectName);

                    xPath.Append(FieldName);

                    switch (DataType)
                    {
                        case "DATETIME":
                            SortXExpr.AddSort(xPath.ToString() + "/YEAR", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            SortXExpr.AddSort(xPath.ToString() + "/MONTH", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            SortXExpr.AddSort(xPath.ToString() + "/DAY", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            SortXExpr.AddSort(xPath.ToString() + "/HOUR", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            SortXExpr.AddSort(xPath.ToString() + "/MINUTE", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            SortXExpr.AddSort(xPath.ToString() + "/SECOND", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            break;
                        case "CURRENCY":
                            SortXExpr.AddSort(xPath.ToString() + "/AMOUNT", XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            break;
                        default:
                            SortXExpr.AddSort(xPath.ToString(), XmlSortOrderObj, XmlCaseOrder.None, "", XmlDataTypeObj);
                            break;
                    }
                    if (SortType == 1) break;
                }
                SortedDataXItr = SearchResultXNav.Select(SortXExpr);
                SortedXml = new StringBuilder(100);
                SortedXml.Append("<DIGITALASSETCOLLECTION>");
                while (SortedDataXItr.MoveNext())
                {
                    //tempNav = SortedDataXItr.Current;
                    SortedXml.Append(SortedDataXItr.Current.OuterXml);
                }
                SortedXml.Append("</DIGITALASSETCOLLECTION>");
            }
            catch (Exception)
            {
                return SearchResultXml;
            }
            return SortedXml.ToString();
        }

        private String prepareHtml(XPathNavigator xPthNav, XPathNavigator xPthRefNav, XPathNavigator xPthFormConfigXml, String ResultHtmlPath, int DocHeight)
        {
            # region Local Variables
            //String TemplateId = null;
            //XPathNavigator TemplateFieldNav = null;
            //String AssetMetadataTemplateFilePath = ALSolutionCommon.GetConfigurationPath(TenantName, "DAM") + "AssetMetadataTemplates/";
            StringBuilder HtmlString = new StringBuilder(50);
            XPathNodeIterator xFieldNodeItr = null;
            XPathNodeIterator xPthNodeItr = xPthNav.Select("DIGITALASSETCOLLECTION/DIGITALASSET");
            XPathNavigator xPthNodeNavigator = null;
            //TemplateFieldNav = xPthRefNav.SelectSingleNode("REPORT/SECTION[@Name='DisplayLayout']/FIELD[@Name='Template']");
            //TemplateId = TemplateFieldNav.GetAttribute("ID", "");
            XPathNavigator ThumbnailNode = null;
            String RValue = null;
            String ThumbFileFormat = null;
            String ThumbName = null;
            String SelectedSections = null;
            StringBuilder FileName = null;
            StringBuilder ThumbFileName = null;
            string[] separator = { "$AL$" };
            String[] SelectedSectionsList = null;
            Boolean ShowAssetSmry = false;
            Boolean ShowBI = false;
            Boolean ShowVersion = false;
            XPathNavigator GrByNavNode = null;
            String SpacerGifPath = null;
            String ThumbHeight = null;
            String ThumbWidth = null;
            
            String RelationName = null;
            String GrByNavNodeVal = null;
            String GrByRelation = null;
            String GrByObjectName = null;
            String GrByFieldName = null;
            String GrByDataType = null;
            String GrByValue = null;
            String Year = null;
            String Mnth = null;
            String Day = null;
            String ContentRevId = null;
            XPathNodeIterator xContentRevNodes = null;
            XPathNavigator xContentRevNodeNav = null;
            String DefaultImagePath = null;
            String GroupBy = null;
            Localization LocalizationObj = null;
            
            String AssetName = Localization.GetStringByPath("ALResource", TenantName, Language, "DAM_AssetName_Shared");
            String AssetDesc = Localization.GetStringByPath("ALResource", TenantName, Language, "Description_Shared");
            String ExpiryDate = Localization.GetStringByPath("ALResource", TenantName, Language, "DAM_ExpiryDate_Shared");
            String AssetSummary = Localization.GetStringByPath("ALResource", TenantName, Language, "DAM_AssetMetdata_AssetSummary");
            String NoShared = Localization.GetStringByPath("ALResource", TenantName, Language, "No_Shared");
            String YesShared = Localization.GetStringByPath("ALResource", TenantName, Language, "Yes_Shared");
            String BusinessInfo = Localization.GetStringByPath("ALResource", TenantName, Language, "DAM_BusinessInformation_Shared");
            String VersionInfo = Localization.GetStringByPath("ALResource", TenantName, Language, "DAM_AssetMetaData_VersionInformation");

            string[] Month = {  Localization.GetStringByPath("ALResource",TenantName,Language,"January_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"Febuary_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"March_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"April_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"May_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"June_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"July_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"August_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"September_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"October_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"November_Shared"),
                                          Localization.GetStringByPath("ALResource",TenantName,Language,"December_Shared") };

            # endregion

            try
            {
                DefaultImagePath = ALCommon.GetInstallationDir() + TenantName + "\\App_Themes\\Default\\Images\\";
                ThumbFileName = new StringBuilder(60);
                SpacerGifPath = DefaultImagePath + "spacer.gif";
                LocalizationObj = new Localization(TenantName);

                HtmlString.Append("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
                HtmlString.Append("<html xmlns=\"http://www.w3.org/1999/xhtml\">");

                #region Head Region
                HtmlString.Append("<head>");
                HtmlString.Append("<title>Asset Metadatea</title>");
                HtmlString.Append("<meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\">");
                HtmlString.Append("</head>");
                #endregion

                #region Body Region
                HtmlString.Append("<body>");
                HtmlString.Append("<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">");

                #region Asset Metadata Title bar
                //HtmlString.Append("<tr class=\"PageTitleBar\" valign=\"top\">");
                HtmlString.Append("<tr style=\"height:20px;	background-color:#8E9BC4;color:#FFFFFF\" valign=\"top\">"); //RRPageTitle
                HtmlString.Append("<td style=\"font-family:Verdana,Arial,Helvetica,sans-serif;font-size:12px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none \">&nbsp;&nbsp;Asset Metadata</td>");
                HtmlString.Append("</tr>");
                #endregion

                GrByNavNode = xPthRefNav.SelectSingleNode("REPORT/SECTION[@Name='DisplayLayout']/FIELD[@Name='GroupBy']");
                if (GrByNavNode != null)
                {
                    GrByNavNodeVal = GrByNavNode.Value;
                    GrByRelation = GrByNavNode.GetAttribute("Relation", "");
                    GrByObjectName = GrByNavNode.GetAttribute("ObjectName", "");
                    GrByFieldName = GrByNavNode.GetAttribute("FieldName", "");
                    GrByDataType = GrByNavNode.GetAttribute("DataType", "");
                }

                while (xPthNodeItr.MoveNext())
                {                   
                    xPthNodeNavigator = xPthNodeItr.Current;
                    ThumbHeight = null;
                    

                    if (DocHeight > 0)
                        HtmlString.AppendFormat("<tr style=\"height: {0}px\" valign=\"top\">", DocHeight);
                    else
                        HtmlString.Append("<tr valign=\"top\">");

                    HtmlString.Append("<td>");
                    HtmlString.Append("<table border=\"0\" width=\"100%\">");

                    #region GroupBy

                    if (GrByNavNode != null)
                    {
                        if (GrByRelation.ToUpper() == GrByObjectName.ToUpper())
                        {
                            if (GrByDataType.ToUpper() == "DATETIME")
                            {
                                if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName).InnerXml))
                                {
                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/YEAR") != null)
                                        Year = xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/YEAR").Value;
                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/MONTH") != null)
                                        Mnth = xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/MONTH").Value;
                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/DAY") != null)
                                        Day = xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "/DAY").Value;

                                    if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                                        GrByValue = "--";
                                    else
                                        GrByValue = Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year;
                                }
                            }
                            else
                                if (GrByDataType.ToUpper() == "BOOLEAN")
                                {
                                    if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "").Value))
                                    {
                                        if (xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "").Value == "0")
                                            GrByValue = "No";
                                        else
                                            GrByValue = "Yes";
                                    }
                                }
                                else
                                {
                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName) != null)
                                        GrByValue = getFormatedString(xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + ""), xPthNodeNavigator.SelectSingleNode("descendant::" + GrByFieldName + "").Value, GrByDataType, LocalizationObj);
                                }
                        }
                        else
                        {
                            if (GrByDataType.ToUpper() == "DATETIME")
                            {
                                if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName).InnerXml))
                                {
                                    if (xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/YEAR") != null)
                                        Year = xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/YEAR").Value;
                                    if (xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/MONTH") != null)
                                        Mnth = xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/MONTH").Value;
                                    if (xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/DAY") != null)
                                        Day = xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName + "/DAY").Value;

                                    if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                                        GrByValue = "--";
                                    else
                                        GrByValue = Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year;
                                }
                            }
                            else
                                if (GrByDataType.ToUpper() == "BOOLEAN")
                                {
                                    if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName).Value))
                                    {
                                        if (xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName).Value == "0")
                                            GrByValue = "No";
                                        else
                                            GrByValue = "Yes";
                                    }
                                }
                                else
                                {
                                    if (xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName) != null)
                                        GrByValue = getFormatedString(xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName), xPthNodeNavigator.SelectSingleNode(GrByRelation + "/" + GrByObjectName + "/" + GrByFieldName).Value, GrByDataType, LocalizationObj);
                                }

                        }

                        if ((String.IsNullOrEmpty(GroupBy) == true) || (String.Compare(GrByValue, GroupBy, true) != 0))
                        {
                            GroupBy = GrByValue;

                            HtmlString.Append("<tr style=\"height:20px;background-color:#8EA5F4;color:#000000\" valign=\"top\">");  //RRFirstHeading                      
                            if (!String.IsNullOrEmpty(GrByValue))
                                HtmlString.AppendFormat("<td style=\"font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-decoration:none; white-space:nowrap;\">&nbsp;&nbsp;{0}: {1}</td>", GrByNavNodeVal, GrByValue);
                            else
                                HtmlString.AppendFormat("<td style=\"font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-decoration:none\">&nbsp;&nbsp;{0}: {1}</td>", GrByNavNodeVal, "--");

                            HtmlString.Append("</tr>");
                        }
                    }
                    #endregion

                    #region Image section
                    HtmlString.Append("<tr><td>");
                    HtmlString.Append("<table border=\"0\" width=\"100%\">");
                    HtmlString.Append("<tr><td></td></tr>");
                    HtmlString.Append("<tr><td><table border=\"0\" width=\"100%\"><tr>");

                    #region Add Image start
                    if (xPthNodeNavigator.SelectSingleNode("descendant::NAME") != null)
                    {
                        // If thumbnail not found, file format of original will be used to get default thumbnail
                        ThumbFileFormat = getFileFomat(xPthNodeNavigator.SelectSingleNode("descendant::NAME").Value);
                    }
                    else
                        ThumbFileFormat = "";

                    FileName = new StringBuilder(50);
                    if (xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISIONID") == null)
                        FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                    else
                    {
                        ContentRevId = xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISIONID").Value;
                        xContentRevNodes = xPthNodeNavigator.Select("DIGITALASSET_CONTENTREVISION/CONTENTREVISION");
                        if (xContentRevNodes.Count > 0)
                        {
                            while (xContentRevNodes.MoveNext())
                            {
                                ThumbFileName.Remove(0, ThumbFileName.Length);
                                FileName.Remove(0, FileName.Length);
                                xContentRevNodeNav = xContentRevNodes.Current;
                                if (xContentRevNodeNav.SelectSingleNode("CONTENTREVISIONID").Value == ContentRevId)
                                {
                                    if (xContentRevNodeNav.SelectSingleNode("CONTENTREVISION_RENDITION") != null)
                                    {
                                        ThumbnailNode = xContentRevNodeNav.SelectSingleNode("CONTENTREVISION_RENDITION");
                                        if ((ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID") != null) &&
                                            (ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME")) != null)
                                        {
                                            RValue = ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID").Value;
                                            ThumbName = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME").Value;
                                            //ThumbFileName.AppendFormat("{0}{1}_{2}", ALSolutionCommon.GetDownloadFilePath(TenantName), RValue, ThumbName);
                                            ThumbFileName.Append(ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/PATH").Value);

                                            if (!File.Exists(ThumbFileName.ToString()))
                                            {
                                                ThumbFileFormat = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/FILETYPE").Value;
                                                if (String.IsNullOrEmpty(ThumbFileFormat) == true)
                                                    ThumbFileFormat = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILEFORMAT").Value;

                                                FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                                ThumbHeight = "125";
                                            }
                                            else
                                            {
                                                if (ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT") != null)
                                                    ThumbHeight = ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT").Value;
                                                if (ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT") != null)
                                                    ThumbWidth = ThumbnailNode.SelectSingleNode("ASSETRENDITION/WIDTH").Value;

                                                FileName.Append(getThumbnailInfo(ThumbFileName.ToString(), RValue + "_" + ThumbName, ref ThumbHeight, ref ThumbWidth, ThumbFileFormat));
                                            }
                                            
                                        }
                                        else
                                        {
                                            FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                        }
                                    }
                                    else
                                    {
                                        FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                    }

                                    break;
                                }
                            }
                        }
                        else 
                        {
                            ThumbFileName.Remove(0, ThumbFileName.Length);
                            FileName.Remove(0, FileName.Length);

                            ThumbnailNode = xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTTHUMBNAILRENDITION");
                            if (ThumbnailNode != null)
                            {
                                if ((ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID") != null) &&
                                            (ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME")) != null)
                                {
                                    ThumbFileFormat = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/FILETYPE").Value;
                                    RValue = ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID").Value;
                                    ThumbName = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME").Value;
                                    //ThumbFileName.AppendFormat("{0}{1}_{2}", ALSolutionCommon.GetDownloadFilePath(TenantName), RValue, ThumbName);
                                    ThumbFileName.Append(ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/PATH").Value);
                                    if (!File.Exists(ThumbFileName.ToString()))
                                    {
                                        FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                        ThumbHeight = "125";
                                    }
                                    else
                                    {
                                        if (ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT") != null)
                                            ThumbHeight = ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT").Value;
                                        if (ThumbnailNode.SelectSingleNode("ASSETRENDITION/WIDTH") != null)
                                            ThumbWidth = ThumbnailNode.SelectSingleNode("ASSETRENDITION/WIDTH").Value;
                                        FileName.Append(getThumbnailInfo(ThumbFileName.ToString(), RValue + "_" + ThumbName, ref ThumbHeight, ref ThumbWidth, ThumbFileFormat));
                                    }
                                }
                                else
                                {
                                    FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                }
                            }
                            else
                                FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                        }
                        
                    }

                    if ((String.IsNullOrEmpty(ThumbHeight) == true) || (String.Compare(ThumbHeight, "0") == 0) || (ThumbHeight.StartsWith("-")==true))
                    {
                        ThumbHeight = "125";
                    }

                    HtmlString.AppendFormat("<td  align=\"left\" style=\"height: {0}px;\">", ThumbHeight);
                    HtmlString.AppendFormat("<img alt=\"\" src=\"{0}\" height=\"125px\" width=\"125px\"/>", FileName.ToString());
                    HtmlString.Append("</td>");
                    #endregion

                    HtmlString.Append("<td align=\"left\" style=\"width: 5%;\">");
                    HtmlString.AppendFormat("<img alt=\"\" src=\"{0}\"/>", SpacerGifPath);
                    HtmlString.Append("</td>");

                    #region Add Image details start
                    HtmlString.Append("<td><table border=\"0\" width=\"100%\">");

                    //put condition of Asset Name req
                    if (xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Asset Summary']/FIELD[@Name='NAME']") != null)
                    {
                        HtmlString.Append("<tr valign=\"top\">");

                        #region Add Asset name Label start
                        //HtmlString.Append("<td class=\"LabelText\" style=\"width: 20%;\" valign=\"top\" align=\"right\">");
                        HtmlString.Append("<td style=\"width: 20%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none;\" valign=\"top\" align=\"right\">");
                        HtmlString.Append(AssetName +":");
                        HtmlString.Append("</td>");
                        #endregion

                        #region Add Asset name starts
                        //HtmlString.Append("<td class=\"Text\" align=\"left\" style=\"width: 68%;\" valign=\"top\">");
                        HtmlString.Append("<td align=\"left\" style=\"width: 68%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::NAME").Value))
                            HtmlString.AppendFormat("{0}", xPthNodeNavigator.SelectSingleNode("descendant::NAME").Value);
                        else
                            HtmlString.Append("--");
                        HtmlString.Append("</td>");
                        #endregion

                        HtmlString.Append("</tr>");
                    }
                    //put condition of Asset Name req ends

                    //put condition of Asset Desc req
                    if (xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Asset Summary']/FIELD[@Name='DESCRIPTION']") != null)
                    {
                        HtmlString.Append("<tr valign=\"top\">");

                        #region Add Asset description Label start
                        HtmlString.Append("<td style=\"width: 20%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none\" valign=\"top\" align=\"right\">");
                        HtmlString.Append(AssetDesc + ":");
                        HtmlString.Append("</td>");
                        #endregion

                        #region Add Asset description starts
                        HtmlString.Append("<td  align=\"left\" style=\"width: 68%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::DESCRIPTION").Value))
                            HtmlString.AppendFormat("{0}", xPthNodeNavigator.SelectSingleNode("descendant::DESCRIPTION").Value);
                        else
                            HtmlString.Append("--");
                        HtmlString.Append("</td>");
                        #endregion

                        HtmlString.Append("</tr>");
                    }
                    //put condition of Asset Desc req ends

                    //put condition of Asset Exp Date req
                    if (xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Asset Summary']/FIELD[@Name='EXPIRYDATE']") != null)
                    {
                        HtmlString.Append("<tr valign=\"top\">");

                        #region Add Asset Expiry Date Label start
                        HtmlString.Append("<td style=\"width: 20%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none\" valign=\"top\" align=\"right\">");
                        HtmlString.Append(ExpiryDate + ":");
                        HtmlString.Append("</td>");
                        #endregion

                        #region Add Asset Expiry Date starts
                        HtmlString.Append("<td align=\"left\" style=\"width: 68%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        //HtmlString.Append("27 August 2009");
                        if ((xPthNodeNavigator.SelectSingleNode("descendant::EXPIRYDATE") != null) && !String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::EXPIRYDATE").InnerXml))
                        {
                            Year = xPthNodeNavigator.SelectSingleNode("descendant::EXPIRYDATE/YEAR").Value;
                            Mnth = xPthNodeNavigator.SelectSingleNode("descendant::EXPIRYDATE/MONTH").Value;
                            Day = xPthNodeNavigator.SelectSingleNode("descendant::EXPIRYDATE/DAY").Value;

                            if (!String.IsNullOrEmpty(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year))
                                HtmlString.Append(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year);
                            else
                                HtmlString.Append("--");
                        }
                        else
                            HtmlString.Append("--");
                        HtmlString.Append("</td>");
                        #endregion

                        HtmlString.Append("</tr>");
                    }
                    //put condition of Asset Exp Date req ends

                    HtmlString.Append("<tr><td colspan=\"2\" align=\"left\">");
                    HtmlString.AppendFormat("<img alt=\"\" src=\"{0}\"/>", SpacerGifPath);
                    HtmlString.Append("</td></tr>");

                    HtmlString.Append("</table></td>");
                    #endregion

                    HtmlString.Append("</tr></table></td></tr>");
                    HtmlString.Append("<tr><td style=\"height: 5px;\"></td></tr>");
                    HtmlString.Append("</table>");
                    HtmlString.Append("</td></tr>");
                    #endregion

                    #region Section Selection Check
                    if (xPthRefNav.SelectSingleNode("REPORT/SECTION[@Name='DisplayLayout']/FIELD[@Name='SelectedSections']") != null)
                    {
                        SelectedSections = xPthRefNav.SelectSingleNode("REPORT/SECTION[@Name='DisplayLayout']/FIELD[@Name='SelectedSections']").Value;
                        SelectedSectionsList = SelectedSections.Split(separator, StringSplitOptions.RemoveEmptyEntries);
                        for (int cnt = 0; cnt < SelectedSectionsList.Length; cnt++)
                        {
                            if (SelectedSectionsList[cnt].ToUpper() == "ASSET SUMMARY")
                                ShowAssetSmry = true;

                            if (SelectedSectionsList[cnt].ToUpper() == "BUSINESS INFORMATION")
                                ShowBI = true;

                            if (SelectedSectionsList[cnt].ToUpper() == "VERSION")
                                ShowVersion = true;
                        }
                    }
                    #endregion

                    //put condition of Asset Summary req
                    XPathNavigator xFieldNav = null;
                    String Relation = null;
                    String DataType = null;
                    String ObjectName = null;
                    String Name = null;
                    XPathNodeIterator TempXPathNodeItr = null;

                    if (ShowAssetSmry)
                    {
                        TempXPathNodeItr = xPthRefNav.Select("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Asset Summary']/FIELD");
                        if (TempXPathNodeItr.Count > 0)
                        {
                            #region Asset Summary Section

                            HtmlString.Append("<tr>");
                            HtmlString.Append("<td>");
                            HtmlString.Append("<table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">");

                            #region Asset Summary Section bar
                            HtmlString.Append("<tr style=\"background-color:#A8BEF7;color:#000000;height: 20px\">");//RRSecondHeading
                            HtmlString.Append("<td style=\"white-space: nowrap;font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none\" align=\"left\">&nbsp;&nbsp;"+AssetSummary+"</td>");//MainTitleText
                            HtmlString.Append("<td></td>");
                            HtmlString.Append("</tr>");
                            #endregion

                            HtmlString.Append("<tr><td style=\"width: 50%;\"><table border=\"0\" width=\"100%\" cellpadding=\"1\" cellspacing=\"1\">");

                            xFieldNodeItr = xPthRefNav.Select("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Asset Summary']/FIELD");
                            int TdCnt = 0;
                            String TR = "closed";
                            while (xFieldNodeItr.MoveNext())
                            {
                                xFieldNav = xFieldNodeItr.Current;

                                if (TdCnt == 2)
                                    TdCnt = 0;

                                if (!String.IsNullOrEmpty(xFieldNav.Value) && xFieldNav.Value != "Asset Name" && xFieldNav.Value != "Asset Description" && xFieldNav.Value != "Expiry Date")
                                {
                                    if (TdCnt == 0)
                                    {
                                        HtmlString.Append("<tr>");
                                        TR = "created";
                                    }

                                    #region Add Label start

                                    if (TdCnt == 0)
                                    {
                                        HtmlString.Append("<td style=\"width: 10%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none\" valign=\"top\" align=\"right\">");
                                        HtmlString.AppendFormat("{0}:", xFieldNav.Value);
                                        HtmlString.Append("</td>");
                                    }
                                    else
                                    {
                                        HtmlString.Append("<td style=\"width: 1%;\"></td>");
                                        HtmlString.Append("<td style=\"width: 10%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none\" valign=\"top\" align=\"right\">");
                                        HtmlString.AppendFormat("{0}:", xFieldNav.Value);
                                        HtmlString.Append("</td>");
                                    }

                                    #endregion

                                    #region Add Label Text starts

                                    Relation = xFieldNav.GetAttribute("Relation", "");
                                    DataType = xFieldNav.GetAttribute("DataType", "");
                                    ObjectName = xFieldNav.GetAttribute("ObjectName", "");
                                    Name = xFieldNav.GetAttribute("Name", "");

                                    if (Relation.ToUpper() == ObjectName.ToUpper())
                                    {
                                        if (DataType.ToUpper() == "DATETIME")
                                        {
                                            if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::" + Name).InnerXml))
                                            {
                                                Year = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/YEAR").Value;
                                                Mnth = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/MONTH").Value;
                                                Day = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/DAY").Value;

                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (!String.IsNullOrEmpty(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year))
                                                    HtmlString.Append(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year);
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");
                                            }
                                            //else
                                            //    HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                            //    HtmlString.Append("--");
                                            //    HtmlString.Append("</td>");

                                        }
                                        else
                                            if (DataType.ToUpper() == "BOOLEAN")
                                            {
                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value))
                                                {
                                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value == "0")
                                                        HtmlString.AppendFormat("{0}", NoShared);
                                                    else
                                                        HtmlString.AppendFormat("{0}", YesShared);
                                                }
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");
                                            }
                                            else
                                            {
                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name) != null)
                                                {
                                                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value != "")
                                                        HtmlString.AppendFormat("{0}", xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value);
                                                    else
                                                        HtmlString.Append("--");
                                                }
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");
                                            }
                                    }
                                    else
                                    {
                                        if (DataType.ToUpper() == "DATETIME")
                                        {
                                            if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).InnerXml))
                                            {
                                                Year = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/YEAR").Value;
                                                Mnth = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/MONTH").Value;
                                                Day = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/DAY").Value;

                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (!String.IsNullOrEmpty(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year))
                                                    HtmlString.Append(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year);
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");
                                            }
                                            //else
                                            //    HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                            //    HtmlString.Append("--");
                                            //    HtmlString.Append("</td>");
                                        }
                                        else
                                            if (DataType.ToUpper() == "BOOLEAN")
                                            {
                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value))
                                                {
                                                    if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value == "0")
                                                        HtmlString.AppendFormat("{0}", NoShared);
                                                    else
                                                        HtmlString.AppendFormat("{0}", YesShared);
                                                }
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");

                                            }
                                            else
                                            {
                                                HtmlString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                                                if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name) != null)
                                                {
                                                    if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value != "")
                                                        HtmlString.AppendFormat("{0}", xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value);
                                                    else
                                                        HtmlString.Append("--");
                                                }
                                                else
                                                    HtmlString.Append("--");
                                                HtmlString.Append("</td>");
                                            }

                                    }


                                    #endregion

                                    TdCnt++;

                                    if (TdCnt == 2)
                                    {
                                        HtmlString.Append("</tr>");
                                        TR = "closed";
                                    }
                                }
                            }
                            if (TR == "created")
                                HtmlString.Append("</tr>");

                            HtmlString.Append("</table></td></tr>");

                            HtmlString.Append("</table>");
                            HtmlString.Append("</td>");
                            HtmlString.Append("</tr>");
                            #endregion
                        }
                    }
                    //put condition of Asset Summary req end

                    XPathNodeIterator xFrmCnfgSectionNodes = null;
                    XPathNavigator xFrmCnfgSectionNode = null;
                    XPathNodeIterator xSectionColmNodes = null;
                    XPathNavigator xSectionColNode = null;
                    XPathNodeIterator xColFieldNodes = null;
                    XPathNavigator xColFieldNodeNav = null;
                    XPathNavigator xFacetSeriesColNodeNav = null;
                    XPathNodeIterator xFacetSeriesColNodes = null;
                    String BINameAttr = null;
                    String BIRelationAttr = null;
                    String BIObjectNameAttr = null;
                    XPathNavigator xBIFieldNodeNav = null;
                    XPathNavigator TempXPathNodeNav = null;
                    Boolean Flag = false;


                    if (ShowBI && xPthFormConfigXml != null)
                    {
                        #region Space
                        if (ShowAssetSmry)
                            HtmlString.AppendFormat("<tr style=\"height:10px;\"><td><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);
                        #endregion

                        TempXPathNodeItr = null;
                        TempXPathNodeItr = xPthRefNav.Select("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Business Information']/FIELD");
                        if (TempXPathNodeItr.Count > 0)
                        {
                            while (TempXPathNodeItr.MoveNext())
                            {
                                TempXPathNodeNav = TempXPathNodeItr.Current;
                                if (xPthNodeNavigator.SelectSingleNode(TempXPathNodeNav.GetAttribute("Relation", "")) != null)
                                    Flag = true;
                            }

                            if (Flag)
                            {
                                #region BI Section
                                HtmlString.Append("<tr>");
                                HtmlString.Append("<td>");
                                HtmlString.Append("<table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">");//start

                                #region Business Information Section bar
                                HtmlString.Append("<tr style=\"background-color:#A8BEF7;color:#000000;height: 20px\">");//RRSecondHeading
                                HtmlString.Append("<td style=\"white-space: nowrap;font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none\" align=\"left\" colspan=\"2\">&nbsp;&nbsp;"+ BusinessInfo+"</td>");//MainTitleText
                                HtmlString.Append("</tr>");
                                #endregion

                                HtmlString.AppendFormat("<tr><td colspan=\"2\"><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);

                                XPathExpression xp_Exp_Section = null;
                                xp_Exp_Section = XPathExpression.Compile("Form/Section");
                                xp_Exp_Section.AddSort("@Order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);

                                xFrmCnfgSectionNodes = xPthFormConfigXml.Select(xp_Exp_Section);
                                String SectionHeading = null;
                                String ColumnLayout = "2";
                                Boolean SectionChanged = false;
                                Boolean ColChanged = false;
                                Boolean SectionTRCreated = false;
                                Boolean ColTDCreated = false;
                                //StringBuilder TempHTMLString = new StringBuilder();

                                while (xFrmCnfgSectionNodes.MoveNext())
                                {
                                    xFrmCnfgSectionNode = xFrmCnfgSectionNodes.Current;
                                    SectionHeading = xFrmCnfgSectionNode.GetAttribute("Heading", "");
                                    ColumnLayout = xFrmCnfgSectionNode.GetAttribute("ColumnLayout", "");
                                    SectionChanged = true;

                                    XPathExpression xp_Exp_Column = null;
                                    xp_Exp_Column = XPathExpression.Compile("Column");
                                    xp_Exp_Column.AddSort("@Order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);

                                    xSectionColmNodes = xFrmCnfgSectionNode.Select(xp_Exp_Column);
                                    while (xSectionColmNodes.MoveNext())
                                    {
                                        xSectionColNode = xSectionColmNodes.Current;
                                        ColChanged = true;

                                        XPathExpression xp_Exp_Field = null;
                                        xp_Exp_Field = XPathExpression.Compile("Field");
                                        xp_Exp_Field.AddSort("@Order", System.Xml.XPath.XmlSortOrder.Ascending, XmlCaseOrder.None, "", XmlDataType.Number);

                                        xColFieldNodes = xSectionColNode.Select(xp_Exp_Field);
                                        while (xColFieldNodes.MoveNext())
                                        {
                                            xColFieldNodeNav = xColFieldNodes.Current;

                                            xFacetSeriesColNodeNav = xColFieldNodeNav.SelectSingleNode("Configuration/DisplayLayout/List/DisplayColumns/TargetObject");

                                            if (xFacetSeriesColNodeNav == null)
                                            {
                                                #region Facet Field

                                                BINameAttr = xColFieldNodeNav.GetAttribute("Name", "");
                                                BIRelationAttr = xColFieldNodeNav.GetAttribute("RELATIONSHIPNAME", "");
                                                BIObjectNameAttr = xColFieldNodeNav.GetAttribute("OBJECTTYPENAME", "");

                                                //check if field is selected field
                                                if (xPthNodeNavigator.SelectSingleNode(xColFieldNodeNav.GetAttribute("RELATIONSHIPNAME", "")) != null)
                                                {
                                                    if (xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Business Information']/FIELD[@Name='" + BINameAttr + "' and @Relation='" + BIRelationAttr + "' and @ObjectName='"+ BIObjectNameAttr +"']") != null)
                                                    {
                                                        if (SectionChanged)
                                                        {
                                                            #region Display Section Bar
                                                            HtmlString.Append("<tr style=\"background-color:#DAE8FD;color:#000000;height: 20px\">");
                                                            HtmlString.AppendFormat("<td style=\"white-space: nowrap;font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none\" align=\"left\" colspan=\"2\">&nbsp;&nbsp;{0}</td>", SectionHeading);//MainTitleText
                                                            HtmlString.Append("</tr>");
                                                            #endregion
                                                            //start <tr> for each section.
                                                            HtmlString.Append("<tr>");
                                                            SectionTRCreated = true;
                                                            SectionChanged = false;
                                                        }

                                                        if (ColChanged)
                                                        {
                                                            //start <td> for each section.decision to be taken.
                                                            if (ColumnLayout == "2")
                                                                HtmlString.Append("<td valign=\"top\" style=\"width:50%;\"><table width=\"100%\" cellpadding=\"1\" cellspacing=\"1\">");
                                                            else
                                                                HtmlString.Append("<td valign=\"top\" style=\"width:100%;\" colspan=\"2\"><table width=\"100%\" cellpadding=\"1\" cellspacing=\"1\">");
                                                            ColTDCreated = true;
                                                            ColChanged = false;
                                                        }

                                                        //check if it is selected field is Facet series
                                                        xBIFieldNodeNav = xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Business Information']/FIELD[@Name='" + BINameAttr + "' and @Relation='" + BIRelationAttr + "' and @ObjectName='" + BIObjectNameAttr + "']");
                                                                                                     
                                                        handleFacet(ref HtmlString, xBIFieldNodeNav, xPthNodeNavigator,Month, LocalizationObj);

                                                    }

                                                }
                                                //when 1st field is not selected and second is selected
                                                //else
                                                //{
                                                //    HtmlString.Append("<tr><td colspan=\"2\"></td></tr>");
                                                //}
                                                #endregion
                                            }
                                            else
                                            {
                                                #region Facet Series

                                                xFacetSeriesColNodes = xFacetSeriesColNodeNav.Select("Column");

                                                XmlDocument XmlDoc = new XmlDocument();
                                                XmlNode HeaderNode = null;
                                                XmlNode RowNode = null;
                                                XmlNode DataNode = null;

                                                RowNode = (XmlElement)XmlDoc.CreateElement("ROW");
                                                HeaderNode = (XmlElement)XmlDoc.CreateElement("HEADER");
                                                DataNode = (XmlElement)XmlDoc.CreateElement("DATA");

                                                RowNode.AppendChild(HeaderNode);
                                                RowNode.AppendChild(DataNode);
                                                XmlDoc.AppendChild(RowNode);

                                                while (xFacetSeriesColNodes.MoveNext())
                                                {
                                                    xColFieldNodeNav = xFacetSeriesColNodes.Current;
                                                    if (String.IsNullOrEmpty(xColFieldNodeNav.GetAttribute("RELATIONSHIPNAME", "")) == false)
                                                        RelationName = xColFieldNodeNav.GetAttribute("RELATIONSHIPNAME", "");

                                                    if (xColFieldNodeNav.GetAttribute("DisplayState", "").ToUpper() == "YES")
                                                    {
                                                        BINameAttr = xColFieldNodeNav.GetAttribute("Name", "");

                                                        //check if field is selected field
                                                        if (xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Business Information']/FIELD[@Name='" + BINameAttr + "' and @Relation='" + RelationName + "']") != null)                                                                                       
                                                        {
                                                            //check if it is selected field is Facet series
                                                            xBIFieldNodeNav = xPthRefNav.SelectSingleNode("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Business Information']/FIELD[@Name='" + BINameAttr + "' and @Relation='" + RelationName + "']");
                                                            if (xBIFieldNodeNav.GetAttribute("IsSeries", "").ToUpper() == "YES")
                                                            {
                                                                handleFacetSeries(ref HtmlString, xBIFieldNodeNav, xPthNodeNavigator, ref HeaderNode, ref DataNode, ref XmlDoc, Month, LocalizationObj);
                                                            }
                                                            else
                                                                handleFacet(ref HtmlString, xBIFieldNodeNav, xPthNodeNavigator, Month, LocalizationObj);


                                                            if (SectionChanged)
                                                            {
                                                                if (!String.IsNullOrEmpty(XmlDoc.SelectSingleNode("ROW/DATA").InnerXml))
                                                                {
                                                                    #region Display Section Bar
                                                                    HtmlString.Append("<tr style=\"background-color:#DAE8FD;color:#000000;height: 20px\">");
                                                                    HtmlString.AppendFormat("<td style=\"white-space: nowrap;font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none\" align=\"left\" colspan=\"2\">&nbsp;&nbsp;{0}</td>", SectionHeading);//MainTitleText
                                                                    HtmlString.Append("</tr>");
                                                                    #endregion
                                                                }
                                                                //start <tr> for each section.
                                                                HtmlString.Append("<tr>");
                                                                SectionTRCreated = true;
                                                                SectionChanged = false;
                                                            }

                                                            if (ColChanged)
                                                            {
                                                                //start <td> for each section.
                                                                HtmlString.Append("<td valign=\"top\" style=\"width: 100%;\" colspan=\"2\"><table width=\"100%\" cellpadding=\"1\" cellspacing=\"1\">");
                                                                ColTDCreated = true;
                                                                ColChanged = false;
                                                            }

                                                        }
                                                    }

                                                }

                                                if (!String.IsNullOrEmpty(XmlDoc.SelectSingleNode("ROW/DATA").InnerXml))
                                                    prepareTabularHTML(ref HtmlString, XmlDoc);
                                                #endregion
                                            }
                                        }
                                        //end <td> for each section.
                                        if (ColTDCreated)
                                        {
                                            HtmlString.Append("</table></td>");
                                            ColTDCreated = false;
                                        }
                                    }

                                    //end <tr> for each section.
                                    if (SectionTRCreated)
                                    {
                                        HtmlString.Append("</tr>");
                                        SectionTRCreated = false;
                                    }
                                }

                                HtmlString.Append("</table>");//end
                                HtmlString.Append("</td>");
                                HtmlString.Append("</tr>");
                                #endregion
                            }
                        }
                    }

                    if (ShowVersion)
                    {
                        XPathNodeIterator xPthVerionFileds = null;
                        XPathNavigator xPthVerionFiledNav = null;
                        XmlDocument XmlDoc = new XmlDocument();
                        XmlNode HeaderNode = null;
                        XmlNode RowNode = null;
                        XmlNode DataNode = null;
                        TempXPathNodeItr = null;

                        #region Space
                        if (ShowBI)
                            HtmlString.AppendFormat("<tr style=\"height:10px;\"><td><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);
                        else
                            if (ShowAssetSmry)
                                HtmlString.AppendFormat("<tr style=\"height:10px;\"><td><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);
                        #endregion

                        TempXPathNodeItr = xPthRefNav.Select("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Version']/FIELD");
                        if (TempXPathNodeItr.Count > 0)
                        {
                            #region Version Section

                            HtmlString.Append("<tr>");//start
                            HtmlString.Append("<td>");
                            HtmlString.Append("<table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">");//start

                            #region Version Information Section bar
                            HtmlString.Append("<tr style=\"background-color:#A8BEF7;color:#000000;height: 20px\">");//RRSecondHeading
                            HtmlString.Append("<td style=\"white-space: nowrap;font-family:Verdana,Arial,Helvetica,sans-serif;font-size:11px;font-weight:bold;color:#333333;vertical-align:middle;text-align:left;text-decoration:none\" align=\"left\" colspan=\"2\">&nbsp;&nbsp;"+ VersionInfo +"</td>");//MainTitleText
                            HtmlString.Append("</tr>");
                            #endregion

                            HtmlString.AppendFormat("<tr><td colspan=\"2\"><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);

                            xPthVerionFileds = xPthRefNav.Select("REPORT/SECTIONLIST[@Name='DisplayFieldList']/SECTION[@Name='Version']/FIELD");
                            if (xPthVerionFileds.Count > 0)
                            {

                                RowNode = (XmlElement)XmlDoc.CreateElement("ROW");
                                HeaderNode = (XmlElement)XmlDoc.CreateElement("HEADER");
                                DataNode = (XmlElement)XmlDoc.CreateElement("DATA");

                                RowNode.AppendChild(HeaderNode);
                                RowNode.AppendChild(DataNode);
                                XmlDoc.AppendChild(RowNode);

                                HtmlString.Append("<tr>");
                                HtmlString.Append("<td valign=\"top\" style=\"width:100%; \" colspan=\"2\"><table width=\"100%\" cellpadding=\"1\" cellspacing=\"1\">");

                                addVersionThumbnail(xPthNodeNavigator, ref HeaderNode, ref DataNode, ref XmlDoc, ThumbFileFormat);

                                while (xPthVerionFileds.MoveNext())
                                {
                                    xPthVerionFiledNav = xPthVerionFileds.Current;
                                    handleVerion(xPthVerionFiledNav, xPthNodeNavigator, ref HeaderNode, ref DataNode, ref XmlDoc, Month, YesShared, NoShared);
                                }

                                prepareTabularHTML(ref HtmlString, XmlDoc);

                                HtmlString.Append("</table>");//end
                                HtmlString.Append("</td>");
                                HtmlString.Append("</tr>");
                            }


                            HtmlString.Append("</table>");//end
                            HtmlString.Append("</td>");
                            HtmlString.Append("</tr>");

                            #endregion
                        }
                    }

                    HtmlString.Append("</table>");
                    HtmlString.Append("</td>");
                    HtmlString.Append("</tr>");

                    if (DocHeight == 0)
                        HtmlString.AppendFormat("<tr style=\"height:50px;\"><td><img alt=\"\" src=\"{0}\" /></td></tr>", SpacerGifPath);                    
                }

                HtmlString.Append("</table>");
                HtmlString.Append("</body>");
                #endregion

                HtmlString.Append("</html>");

                File.WriteAllText(ResultHtmlPath, HtmlString.ToString());
            }
            catch (Exception ExcepObj)
            {
                return ExcepObj.Message;
            }
            return null;
        }

        private void addVersionThumbnail(XPathNavigator xPthNodeNavigator, ref XmlNode HeaderNode, ref XmlNode DataNode, ref XmlDocument XmlDoc, String OrgFileFormat)
        {
            # region Local Variables
            XmlNode             FieldNode           = null;
            XmlNode             RowNode             = null;
            XPathNodeIterator   xFacetSeriesItr     = null;
            XPathNavigator      xFacetSeriesNav     = null;
            XmlAttribute        XmlAttr             = null;
            XPathNavigator      ThumbnailNode       = null;
            String              RValue              = null;
            String              ThumbFileFormat     = "";
            String              ThumbName           = null;
            StringBuilder       FileName            = null;
            StringBuilder       ThumbFileName       = null;
            String              ThumbHeight         = null;
            String              ThumbWidth          = null;
            # endregion
            
            FieldNode = (XmlElement)XmlDoc.CreateElement("FIELD");
            FieldNode.InnerText = "VersionThumbnail";
            HeaderNode.AppendChild(FieldNode);

            int cnt = 1;

            if (xPthNodeNavigator.Select("DIGITALASSET_CONTENTREVISION/CONTENTREVISION").Count > 0)
            {
                ThumbFileName = new StringBuilder(60);
                FileName = new StringBuilder(60);
                xFacetSeriesItr = xPthNodeNavigator.Select("DIGITALASSET_CONTENTREVISION/CONTENTREVISION");
                while (xFacetSeriesItr.MoveNext())
                {
                    FileName.Remove(0, FileName.Length);
                    ThumbFileName.Remove(0, ThumbFileName.Length);
                    xFacetSeriesNav = xFacetSeriesItr.Current;
                    XmlAttr = XmlDoc.CreateAttribute("VersionThumbnail");
                    XmlAttr.Value = "Yes";
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    RowNode.Attributes.Append(XmlAttr);

                    #region Prepare Thumbnail file path
                    
                    if (xFacetSeriesNav.SelectSingleNode("CONTENTREVISION_RENDITION") != null)
                    {
                        ThumbnailNode = xFacetSeriesNav.SelectSingleNode("CONTENTREVISION_RENDITION");
                        ThumbFileFormat = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/FILETYPE").Value;
                        if ((ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID") != null) &&
                            (ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME")) != null)
                        {
                            RValue = ThumbnailNode.SelectSingleNode("ASSETRENDITION/ASSETRENDITIONID").Value;
                            ThumbName = ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/NAME").Value;
                            //ThumbFileName.AppendFormat("{0}{1}_{2}", ALSolutionCommon.GetDownloadFilePath(TenantName), RValue, ThumbName);
                            ThumbFileName.Append(ThumbnailNode.SelectSingleNode("ASSETRENDITION/FILESYSTEM/FILE/PATH").Value);
                            if (!File.Exists(ThumbFileName.ToString()))
                            {
                                FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                                ThumbHeight = "125";
                                ThumbWidth = "125";
                            }
                            else
                            {
                                ThumbHeight = ThumbnailNode.SelectSingleNode("ASSETRENDITION/HEIGHT").Value;
                                ThumbWidth = ThumbnailNode.SelectSingleNode("ASSETRENDITION/WIDTH").Value;
                                FileName.Append(getThumbnailInfo(ThumbFileName.ToString(), RValue + "_" + ThumbName, ref ThumbHeight, ref ThumbWidth, ThumbFileFormat));
                            }
                            if (String.IsNullOrEmpty(ThumbHeight) == false)
                            {
                                XmlAttr = XmlDoc.CreateAttribute("ThumbnailHeight");
                                XmlAttr.Value = ThumbHeight;
                                RowNode.Attributes.Append(XmlAttr);
                            }

                            if (String.IsNullOrEmpty(ThumbWidth) == false)
                            {
                                XmlAttr = XmlDoc.CreateAttribute("ThumbnailWidth");
                                XmlAttr.Value = ThumbWidth;
                                RowNode.Attributes.Append(XmlAttr);
                            }
                        }
                        else
                            FileName.Append(getDefaultThumbnail(ThumbFileFormat));
                    }
                    else
                        FileName.Append(getDefaultThumbnail(OrgFileFormat));


                    #endregion

                    RowNode.InnerText = FileName.ToString();
                    DataNode.AppendChild(RowNode);
                    cnt++;
                }
            }
        }

        private void handleVerion(XPathNavigator xBIFieldNodeNav, XPathNavigator xPthNodeNavigator, ref XmlNode HeaderNode, ref XmlNode DataNode, ref XmlDocument XmlDoc, String[] Month, String YesShared, String NoShared)
        {
            # region Local Variables
            String      Relation    = null;
            String      DataType    = null;
            String      ObjectName  = null;
            String      Name        = null;
            String      Year        = null;
            String      Mnth        = null;
            String      Day         = null;
            String      CntRevID    = null;
            XmlNode     FieldNode   = null;
            XmlNode     RowNode     = null;
            //string[]    Month       = { Localization.GetStringByPath("ALResource",TenantName,Language,"January_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"Febuary_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"March_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"April_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"May_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"June_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"July_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"August_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"September_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"October_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"November_Shared"),
            //                              Localization.GetStringByPath("ALResource",TenantName,Language,"December_Shared")};
            int         cnt         = 0;
            XPathNodeIterator       xFacetSeriesItr     = null;
            XPathNavigator          xFacetSeriesNav     = null;
            XmlAttribute            XmlAttr             = null;
            
            # endregion
            
            Relation    = xBIFieldNodeNav.GetAttribute("Relation", "");
            DataType    = xBIFieldNodeNav.GetAttribute("DataType", "");
            ObjectName  = xBIFieldNodeNav.GetAttribute("ObjectName", "");
            Name        = xBIFieldNodeNav.GetAttribute("Name", "");


            if (Name != "CURRENT" && Name != "LOGINNAME")
            {
                FieldNode           = (XmlElement)XmlDoc.CreateElement("FIELD");
                FieldNode.InnerText = xBIFieldNodeNav.Value;
                HeaderNode.AppendChild(FieldNode);
                xFacetSeriesItr = xPthNodeNavigator.Select(Relation + "/" + ObjectName + "/" + Name);

                if (DataType.ToUpper() == "DATETIME")
                {
                    while (xFacetSeriesItr.MoveNext())
                    {
                        ++cnt;
                        RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                        DataNode.AppendChild(RowNode);
                        xFacetSeriesNav = xFacetSeriesItr.Current;

                        if (xFacetSeriesNav.SelectSingleNode("YEAR") != null)
                            Year = xFacetSeriesNav.SelectSingleNode("YEAR").Value;
                        if (xFacetSeriesNav.SelectSingleNode("MONTH") != null)
                            Mnth = xFacetSeriesNav.SelectSingleNode("MONTH").Value;
                        if (xFacetSeriesNav.SelectSingleNode("DAY") != null)
                            Day = xFacetSeriesNav.SelectSingleNode("DAY").Value;

                        if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                            RowNode.InnerText = "--";
                        else
                            RowNode.InnerText = Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year;
                    }
                }
                else if (DataType.ToUpper() == "BOOLEAN")
                {
                    xFacetSeriesItr = xPthNodeNavigator.Select(Relation + "/" + ObjectName + "/" + Name);
                    while (xFacetSeriesItr.MoveNext())
                    {
                        ++cnt;
                        RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                        DataNode.AppendChild(RowNode);
                        xFacetSeriesNav = xFacetSeriesItr.Current;
                        if (xFacetSeriesNav.Value == "1")
                        {
                            RowNode.InnerText = "Yes";
                        }
                        else
                        {
                            RowNode.InnerText = "No";
                        }
                    }
                }
                else
                {
                     xFacetSeriesItr = xPthNodeNavigator.Select(Relation + "/" + ObjectName + "/" + Name);
                     while (xFacetSeriesItr.MoveNext())
                     {
                         ++cnt;
                         RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                         DataNode.AppendChild(RowNode);
                         xFacetSeriesNav = xFacetSeriesItr.Current;
                         RowNode.InnerText = xFacetSeriesNav.Value;
                     }
                }
            }
            else if (Name == "CURRENT")
            {
                FieldNode = (XmlElement)XmlDoc.CreateElement("FIELD");
                FieldNode.InnerText = xBIFieldNodeNav.Value;
                HeaderNode.AppendChild(FieldNode);

                if (xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISIONID") != null)
                {
                    XmlAttr = XmlDoc.CreateAttribute("ContentRevisionId");
                    XmlAttr.Value = xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISIONID").Value;
                    CntRevID = xPthNodeNavigator.SelectSingleNode("DIGITALASSET_CURRENTREVISION/CONTENTREVISION/CONTENTREVISIONID").Value;
                    DataNode.Attributes.Append(XmlAttr);
                }

                xFacetSeriesItr = xPthNodeNavigator.Select("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISIONID");
                while (xFacetSeriesItr.MoveNext())
                {
                    ++cnt;
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    DataNode.AppendChild(RowNode);
                    xFacetSeriesNav = xFacetSeriesItr.Current;

                    if (xFacetSeriesNav.Value == CntRevID)
                        RowNode.InnerText = YesShared;
                    else
                        RowNode.InnerText = NoShared;
                }
            }
            else if (Name == "LOGINNAME")
            {
                FieldNode = (XmlElement)XmlDoc.CreateElement("FIELD");
                FieldNode.InnerText = xBIFieldNodeNav.Value;
                HeaderNode.AppendChild(FieldNode);

                xFacetSeriesItr = xPthNodeNavigator.Select("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/" + Relation + "/" + ObjectName + "/" + Name);
                while (xFacetSeriesItr.MoveNext())
                {
                    ++cnt;
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    DataNode.AppendChild(RowNode);
                    xFacetSeriesNav = xFacetSeriesItr.Current;
                    RowNode.InnerText = xFacetSeriesNav.Value;
                }
            }

            ((XmlElement)DataNode).SetAttribute("RecordCount", cnt.ToString());
        }

        private void prepareTabularHTML(ref StringBuilder HtmlString, XmlDocument XmlDoc)
        {
            # region Local Variables
            XmlNodeList HeaderFieldList = XmlDoc.SelectNodes("ROW/HEADER/FIELD");
            XmlNodeList RowFieldList = null;
            String ThumbHeight = null;
            String ThumbWidth = null;
            String RecordCount = null;
            int iRecordCount = 0;
            # endregion
            
            if (HeaderFieldList.Count > 0)
            {
                HtmlString.Append("<tr style=\"font-weight:normal; color:#000000;vertical-align:middle;text-align:left;text-decoration:none;background-color:#F7F7F7;height:17px\">");
                foreach (XmlNode Node in HeaderFieldList)
                {
                    //HtmlString.AppendFormat("<td class=\"Text\" style=\"width: 10%; font-weight: bold;\" align=\"center\">{0}</td>", Node.InnerText);
                    HtmlString.AppendFormat("<td style=\"font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none;width: 10%;\" align=\"center\">{0}</td>", Node.InnerText);
                }
                HtmlString.Append("</tr>");
            }

            try
            {
                RecordCount = ((XmlElement)XmlDoc.SelectSingleNode("ROW/DATA")).GetAttribute("RecordCount");
                iRecordCount = int.Parse(RecordCount);            
            }
            catch (Exception)
            {
                iRecordCount = 0;
            }


            if (!String.IsNullOrEmpty(XmlDoc.SelectSingleNode("ROW/DATA").InnerXml))
            {
                for (int RowCnt = 1; RowCnt <= iRecordCount; RowCnt++)
                {
                    RowFieldList = XmlDoc.SelectNodes("ROW/DATA/ROW" + RowCnt);
                    if (RowFieldList.Count > 0)
                    {
                        HtmlString.Append("<tr>");
                        foreach (XmlNode Node in RowFieldList)
                        {
                            if (String.IsNullOrEmpty(((XmlElement)Node).GetAttribute("VersionThumbnail")))
                            {
                                if (!String.IsNullOrEmpty(Node.InnerText))
                                    HtmlString.AppendFormat("<td style=\"width: 10%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px;vertical-align:middle;\" align=\"center\">{0}</td>", Node.InnerText);
                                else
                                    HtmlString.Append("<td style=\"width: 10%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px;vertical-align:middle;\" align=\"center\">--</td>");
                            }
                            else
                            {
                                ThumbHeight = ((XmlElement)Node).GetAttribute("ThumbnailHeight");
                                if ((String.IsNullOrEmpty(ThumbHeight) == true) || (String.Compare(ThumbHeight, "0") == 0) || (ThumbHeight.StartsWith("-") == true))
                                {
                                    ThumbHeight = "125";
                                }
                                else
                                {
                                    try
                                    {
                                        if (Convert.ToUInt64(ThumbHeight) > 125)
                                            ThumbHeight = "125";
                                    }
                                    catch(Exception)
                                    {
                                        ThumbHeight = "125";
                                    }
                                }

                                //ThumbnailWidth
                                ThumbWidth = ((XmlElement)Node).GetAttribute("ThumbnailWidth");
                                if ((String.IsNullOrEmpty(ThumbWidth) == true) || (String.Compare(ThumbWidth, "0") == 0) || (ThumbWidth.StartsWith("-") == true))
                                {
                                    ThumbWidth = "125";
                                }
                                else
                                {
                                    try
                                    {
                                        if (Convert.ToUInt64(ThumbWidth) > 125)
                                            ThumbWidth = "125";
                                    }
                                    catch (Exception)
                                    {
                                        ThumbWidth = "125";
                                    }
                                }
                                HtmlString.AppendFormat("<td style=\"height:{1}px;\" align=\"center\"><img alt=\"\" src=\"{0}\" height=\"{1}px\" width=\"{2}px\" /></td>", Node.InnerText, ThumbHeight, ThumbWidth);
                            }
                        }
                        HtmlString.Append("</tr>");
                    }
                }
            }
        }

        private void handleFacetSeries(ref StringBuilder HtmlString, XPathNavigator xBIFieldNodeNav, XPathNavigator xPthNodeNavigator, ref XmlNode HeaderNode, ref XmlNode DataNode, ref XmlDocument XmlDoc, String[] Month, Localization LocalizationObj)
        {
            # region Local variable
            String              Relation        = null;
            String              DataType        = null;
            String              ObjectName      = null;
            String              Name            = null;
            String              Year            = null;
            String              Mnth            = null;
            String              Day             = null;
            XmlNode             FieldNode       = null;
            XmlNode             RowNode         = null;
            XPathNodeIterator   xFacetSeriesItr = null;
            XPathNavigator      xFacetSeriesNav = null;
            int                 cnt             = 0;
            # endregion

            Relation    = xBIFieldNodeNav.GetAttribute("Relation", "");
            DataType    = xBIFieldNodeNav.GetAttribute("DataType", "");
            ObjectName  = xBIFieldNodeNav.GetAttribute("ObjectName", "");
            Name        = xBIFieldNodeNav.GetAttribute("Name", "");

            FieldNode           = (XmlElement)XmlDoc.CreateElement("FIELD");
            FieldNode.InnerText = xBIFieldNodeNav.Value;
            HeaderNode.AppendChild(FieldNode);
            xFacetSeriesItr 	= xPthNodeNavigator.Select(Relation + "/" + ObjectName + "/" + Name);

            if (DataType.ToUpper() == "DATETIME")
            {
                while (xFacetSeriesItr.MoveNext())
                {
                    ++cnt;
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    DataNode.AppendChild(RowNode);
                    xFacetSeriesNav = xFacetSeriesItr.Current;

                    if (xFacetSeriesNav.SelectSingleNode("YEAR") != null)
                        Year = xFacetSeriesNav.SelectSingleNode("YEAR").Value;
                    if (xFacetSeriesNav.SelectSingleNode("MONTH") != null)
                        Mnth = xFacetSeriesNav.SelectSingleNode("MONTH").Value;
                    if (xFacetSeriesNav.SelectSingleNode("DAY") != null)
                        Day = xFacetSeriesNav.SelectSingleNode("DAY").Value;

                    if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                        RowNode.InnerText = "--";
                    else
                        RowNode.InnerText = Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year;
                }
            }
            else if (DataType.ToUpper() == "BOOLEAN")
            {
                while (xFacetSeriesItr.MoveNext())
                {
                    ++cnt;
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    DataNode.AppendChild(RowNode);
                    xFacetSeriesNav = xFacetSeriesItr.Current;

                    if (xFacetSeriesItr.Current.SelectSingleNode(Name).Value == "1")
                    {
                        RowNode.InnerText = "Yes";
                    }
                    else
                    {
                        RowNode.InnerText = "No";
                    }
                }
            }
            else
            {
                while (xFacetSeriesItr.MoveNext())
                {
                    ++cnt;
                    RowNode = (XmlElement)XmlDoc.CreateElement("ROW" + cnt);
                    DataNode.AppendChild(RowNode);
                    xFacetSeriesNav = xFacetSeriesItr.Current;
                    //RowNode.InnerText = getFormatedStringForMultiSelect(xFacetSeriesNav.Value);
                    RowNode.InnerText = getFormatedString(xFacetSeriesNav, xFacetSeriesNav.Value, DataType, LocalizationObj);
                }
            }

            ((XmlElement)DataNode).SetAttribute("RecordCount", cnt.ToString());
        }

        private void handleFacet(ref StringBuilder TempHTMLString, XPathNavigator xBIFieldNodeNav, XPathNavigator xPthNodeNavigator, String[] Month, Localization LocalizationObj)
        {
            # region Local Variables
            String      Relation    = null;
            String      DataType    = null;
            String      ObjectName  = null;
            String      Name        = null;
            String      Year        = null;
            String      Mnth        = null;
            String      Day         = null;            
            # endregion

            Relation = xBIFieldNodeNav.GetAttribute("Relation", "");
            DataType = xBIFieldNodeNav.GetAttribute("DataType", "");
            ObjectName = xBIFieldNodeNav.GetAttribute("ObjectName", "");
            Name = xBIFieldNodeNav.GetAttribute("Name", "");

            TempHTMLString.Append("<tr valign=\"top\">");
            TempHTMLString.AppendFormat("<td style=\"width: 10%;font-family:Arial;font-size:11px;font-weight:bold;color:#000000;text-decoration:none\" align=\"right\" valign=\"top\">{0}: </td>", xBIFieldNodeNav.Value);

            if (Relation.ToUpper() == ObjectName.ToUpper())
            {
                if (DataType.ToUpper() == "DATETIME")
                {
                    TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                    if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name) != null)
                    {
                        if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/YEAR") != null)
                            Year = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/YEAR").Value;
                        if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/MONTH") != null)
                            Mnth = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/MONTH").Value;
                        if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/DAY") != null)
                            Day = xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "/DAY").Value;

                        if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                            TempHTMLString.Append("--");
                        else
                            TempHTMLString.Append(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year);
                    }
                    else
                        TempHTMLString.Append("--");

                    TempHTMLString.Append("</td>");
                }
                else
                    if (DataType.ToUpper() == "BOOLEAN")
                    {
                        TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value))
                        {
                            if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value == "0")
                                TempHTMLString.AppendFormat("{0}", "No");
                            else
                                TempHTMLString.AppendFormat("{0}", "Yes");

                        }
                        else
                            TempHTMLString.Append("--");
                        TempHTMLString.Append("</td>");
                    }
                    else
                    {
                        TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name) != null)
                        {
                            if (xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value != "")
                            {
                                TempHTMLString.AppendFormat("{0}", getFormatedString(xPthNodeNavigator.SelectSingleNode("descendant::" + Name + ""), xPthNodeNavigator.SelectSingleNode("descendant::" + Name + "").Value, DataType, LocalizationObj));
                            }
                            else
                                TempHTMLString.Append("--");
                        }
                        else
                            TempHTMLString.Append("--");

                        TempHTMLString.Append("</td>");
                    }
            }
            else
            {
                if (DataType.ToUpper() == "DATETIME")
                {
                    TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                    if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name) != null)
                    {
                        if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/YEAR") != null)
                            Year = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/YEAR").Value;
                        if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/MONTH") != null)
                            Mnth = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/MONTH").Value;
                        if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/DAY") != null)
                            Day = xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name + "/DAY").Value;

                        if (String.IsNullOrEmpty(Day) || String.IsNullOrEmpty(Mnth) || String.IsNullOrEmpty(Year))
                            TempHTMLString.Append("--");
                        else
                            TempHTMLString.Append(Day + " " + Month[int.Parse(Mnth) - 1] + " " + Year);
                    }
                    else
                        TempHTMLString.Append("--");

                    TempHTMLString.Append("</td>");
                }
                else
                    if (DataType.ToUpper() == "BOOLEAN")
                    {
                        TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (!String.IsNullOrEmpty(xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value))
                        {
                            if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value == "0")
                                TempHTMLString.AppendFormat("{0}", "No");
                            else
                                TempHTMLString.AppendFormat("{0}", "Yes");

                        }
                        else
                            TempHTMLString.Append("--");
                        TempHTMLString.Append("</td>");

                    }
                    else
                    {
                        TempHTMLString.Append("<td align=\"left\" style=\"width: 20%;font-family:Verdana,Arial,Helvetica,sans-serif;text-decoration:none;font-weight:normal;font-size:11px\" valign=\"top\">");
                        if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name) != null)
                        {
                            if (xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value != "")
                            {
                                TempHTMLString.AppendFormat("{0}", getFormatedString(xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name), xPthNodeNavigator.SelectSingleNode(Relation + "/" + ObjectName + "/" + Name).Value, DataType, LocalizationObj));
                            }
                            else
                                TempHTMLString.Append("--");
                        }
                        else
                            TempHTMLString.Append("--");
                        TempHTMLString.Append("</td>");
                    }

            }

            TempHTMLString.Append("</tr>");
        }

        private String getDefaultThumbnail(String FileFormat)
        {
            # region Local Variables
            StringBuilder   sbFilePath          = null;
            String          FilePath            = null;
            # endregion

            sbFilePath = new StringBuilder();
            sbFilePath.AppendFormat("{0}{1}\\App_Themes\\Default\\Images\\", ALCommon.GetInstallationDir(), TenantName);

            if (!String.IsNullOrEmpty(FileFormat))
            {
                switch (FileFormat.ToUpper())
                {
                    case "DOC":
                        sbFilePath.Append("WinWord.gif");
                        break;
                    case "DOCX":
                        sbFilePath.Append("Icon_DOCX.gif");
                        break;
                    case "EPS":
                        sbFilePath.Append("PShopEPS.gif");
                        break;
                    case "XLS":
                        sbFilePath.Append("XLS.gif");
                        break;
                    case "XLSX":
                        sbFilePath.Append("Icon_XLSX.gif");
                        break;
                    case "MPG":
                    case "AVI":
                        sbFilePath.Append("Mplayer.gif");
                        break;
                    case "PDF":
                        sbFilePath.Append("AcroExch.gif");
                        break;
                    case "TIFF":
                        sbFilePath.Append("PShopTIFF.gif");
                        break;
                    case "JPEG":
                    case "JPG":
                        sbFilePath.Append("PShopJPG.gif");
                        break;
                    case "TXT":
                    case "TEXT":
                        sbFilePath.Append("NotePad.gif");
                        break;
                    case "PPT":
                        sbFilePath.Append("PPT.gif");
                        break;
                    case "PPTX":
                        sbFilePath.Append("Icon_PPTX.gif");
                        break;
                    case "HTM":
                    case "HTML":
                        sbFilePath.Append("Iexplore.gif");
                        break;
                    case "BMP":
                        sbFilePath.Append("Pbrush.gif");
                        break;
                    case "GIF":
                        sbFilePath.Append("PShopGIF.gif");
                        break;
                    case "PSD":
                        sbFilePath.Append("PShopPHOTO.gif");
                        break;
                    case "PICT":
                        sbFilePath.Append("PShopPICT.gif");
                        break;
                    default:
                        sbFilePath.Append("Default_AssetThumbnail.gif");
                        break;
                }
            }
            else
            {
                sbFilePath.Append("Default_AssetThumbnail.gif");
            }

            FilePath = sbFilePath.ToString();
            sbFilePath.Remove(0, sbFilePath.Length);
            return FilePath;
        }

        private String getFileFomat(String FileName)
        {
            int ExtentionIndex = 0;

            if ((String.IsNullOrEmpty(FileName) == false) && FileName.Contains(".") == true)
            {
                ExtentionIndex = FileName.LastIndexOf('.') + 1;
                return (FileName.Substring(ExtentionIndex));
            }
            return ("");
        }

        private Boolean getAspectRatio(String DispHeight, String DispWidth, ref int ImageHeight, ref int ImageWidth)
        {
            # region Local Variables
            int         ImgHt           = 0;
            int         ImgWdth         = 0;
            int         ThumbMaxHeight  = 150;
            int         ThumbMaxWidth   = 150;
            double      wfactor, hfactor, factor;
            Boolean     ReturnValue = false;
            # endregion

            try
            {
                ImgHt   = Convert.ToInt32(DispHeight);
                ImgWdth = Convert.ToInt32(DispWidth);
            }
            catch (Exception)
            {
                ImageHeight = 125;
                ImageWidth  = 125;
                return false;
            }

            if (ImgHt <= ThumbMaxHeight && ImgWdth <= ThumbMaxWidth)
            {
                ImageHeight = ImgHt;
                ImageWidth  = ImgWdth;
                return false;
            }

            ThumbMaxHeight  = 125;
            ThumbMaxWidth   = 125;

            hfactor = ThumbMaxHeight / (double)ImgHt;
            wfactor = ThumbMaxWidth / (double)ImgWdth;
            factor  = (wfactor < hfactor) ? wfactor : hfactor;
            if (factor <= 1)
            {
                ImageHeight = (int)(factor * ImgHt);
                ImageWidth  = (int)(factor * ImgWdth);
                ReturnValue = true;
            }
            else
            {
                ImageHeight = ImgHt;
                ImageWidth  = ImgWdth;
                ReturnValue = false;
            }

            return ReturnValue;        
        }

        private String getThumbnailInfo(String ThumbFilePath, String FileName, ref String DispHeight, ref String DispWidth, String ThumbFileFormat)
        {
            # region Local Variables
            StringBuilder       ThumbRenditionPath  = null;
            String              Error               = null;
            Bitmap              BitmapObj           = null;
            int                 ImageHeight         = 0;
            int                 ImageWidth          = 0;
            Boolean             RenderThumbnail     = false;
            String              TempSessionIDPath   = null;
            # endregion

            try
            {
                ThumbRenditionPath = new StringBuilder(60);
                RenderThumbnail = getAspectRatio(DispHeight, DispWidth, ref ImageHeight, ref ImageWidth);
                if (RenderThumbnail == true)
                {
                    TempSessionIDPath = ALCommon.GetTempPath() + SessionID.ToString() + "\\";
                    TempSessionIDPath = TempSessionIDPath.Replace("\\\\", "\\");
                    if (Directory.Exists(TempSessionIDPath) == false)
                        Directory.CreateDirectory(TempSessionIDPath);

                    ThumbRenditionPath.AppendFormat("{0}AMD_{1}", TempSessionIDPath, FileName);
                    
                    if (File.Exists(ThumbRenditionPath.ToString()) == false)
                    {
                        BitmapObj = new Bitmap(ThumbFilePath);
                        Error = Assetlink.Services.MediaProcessor.ALMediaProcessor.ALBitmapProcessor.GenerateThumbnail(ref BitmapObj, true, ImageWidth, ImageHeight, 72, 24, 100, ThumbFileFormat, ThumbRenditionPath.ToString(), out ImageWidth, out ImageHeight);
                        if (!String.IsNullOrEmpty(Error))
                        {
                            ThumbRenditionPath.Remove(0, ThumbRenditionPath.Length);
                            ThumbRenditionPath.Append(ThumbFilePath);
                        }
                        else
                        {
                            DispHeight = ImageHeight.ToString();
                            DispWidth = ImageWidth.ToString();
                        }
                    }
                    else
                    {
                        DispHeight = ImageHeight.ToString();
                        DispWidth = ImageWidth.ToString();
                    }
                }
                else
                {
                    ThumbRenditionPath.Append(ThumbFilePath);
                }
            }
            catch (Exception)
            {
                ThumbRenditionPath.Remove(0, ThumbRenditionPath.Length);
                ThumbRenditionPath.Append(ThumbFilePath);
            }
            finally
            {
                BitmapObj = null;
            }

            return ThumbRenditionPath.ToString();
        }

        private String generateMHtmlFromHtml(String SourcePath, String TargetPath)
        {
            # region Local variables
            CDO.MessageClass    CDOObj          = null;
            ADODB.Stream        ADOStreamObj    = null;
            # endregion

            try
            {
                CDOObj = new CDO.MessageClass();
                CDOObj.MimeFormatted = true;
                CDOObj.CreateMHTMLBody(SourcePath, CDO.CdoMHTMLFlags.cdoSuppressNone, "", "");
                ADOStreamObj = CDOObj.GetStream();
                ADOStreamObj.SaveToFile(TargetPath, ADODB.SaveOptionsEnum.adSaveCreateOverWrite);
            }
            catch (Exception ExceptionObject)
            {
                return ExceptionObject.Message;
            }
            return null;
        }

        /// <summary>
        /// This method will check whether Media Server and Services are installed
        /// on same machine or not
        /// </summary>
        /// <returns></returns>
        private bool IsMediaServerAndServicesTogether()
        {
            bool IsMSAndServicesTogether        = false;
            String str_WebServicesInstalled     = null;
            String str_MediaServer_Installed    = null;

            str_MediaServer_Installed   = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "MediaServer Installed");
            str_WebServicesInstalled    = ALCommon.ReadRegistryValue(@"SOFTWARE\ASSETLINK", "WebServicesInstalled");

            if (str_MediaServer_Installed == "1" && str_WebServicesInstalled == "1")
                IsMSAndServicesTogether = true;

            return IsMSAndServicesTogether;
        }

        private XML getAllThumbnailsToServicesMachine(XML outputXML)
        {
            # region Local Variables
            FileTransferInfrastructure  ObjFT_MS            = null;
            StringBuilder               sb_FileColXml       = null;
            String                      ConfigXmlPath       = null;
            String                      OutboxPath          = null;
            XPathDocument               xpDocConfigXml      = null;
            XPathNavigator              xpNavConfigXml      = null;
            XPathNavigator              xpNavOutbox         = null;
            XmlDocument                 OutputXmlDoc        = null;
            XmlNodeList                 FilePathXmlNodes    = null;
            String                      SourceFilePath      = null;
            StringBuilder               sbTargetFilePath    = null;
            String                      TempFileName        = null;
            String                      TempIDString        = null;
            String                      TempString          = null;
            String                      TempString2         = null;
            int                         TempIndex           = 0;
            # endregion

            try
            {
                if (ALCommon.IsMediaServerActivated(TenantName) && !IsMediaServerAndServicesTogether())
                {
                    ObjFT_MS = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                    ObjFT_MS.ToUseServiceURL = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");
                    ConfigXmlPath = ALCommon.GetTenantXMLPath(TenantName);
                    xpDocConfigXml = new XPathDocument(ConfigXmlPath);
                    xpNavConfigXml = xpDocConfigXml.CreateNavigator();
                    xpNavOutbox = xpNavConfigXml.SelectSingleNode("CONFIGURATION/FILESYSTEM/OUTBOX");
                    if (xpNavOutbox == null)
                        return outputXML;
                    if (String.IsNullOrEmpty(xpNavOutbox.Value) == true)
                        return outputXML;

                    OutputXmlDoc = new XmlDocument();
                    OutputXmlDoc.LoadXml(outputXML);
                    if (OutputXmlDoc == null)
                        return outputXML;

                    FilePathXmlNodes = OutputXmlDoc.SelectNodes("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION/FILESYSTEM/FILE/PATH");

                    //If Version information is not selected for Asset Metadata report then get Thumbnail file details from DIGITALASSET_CURRENTTHUMBNAILRENDITION 
                    if(FilePathXmlNodes.Count == 0)
                        FilePathXmlNodes = OutputXmlDoc.SelectNodes("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTTHUMBNAILRENDITION/ASSETRENDITION/FILESYSTEM/FILE/PATH");

                    if (FilePathXmlNodes.Count > 0)
                    {
                        sb_FileColXml = new StringBuilder(100);
                        sbTargetFilePath = new StringBuilder(50);
                        OutboxPath = xpNavOutbox.Value;
                        sb_FileColXml.Append("<FILECOLLECTION>");

                        foreach (XmlNode FilePathNode in FilePathXmlNodes)
                        {
                            sbTargetFilePath.Remove(0, sbTargetFilePath.Length);
                            SourceFilePath = FilePathNode.InnerText;
                            //File Name
                            TempIndex = SourceFilePath.LastIndexOf("\\");
                            TempFileName = SourceFilePath.Substring(TempIndex);

                            //Folder name ACTUALFILE/THUMBNAILFILE
                            TempString = SourceFilePath.Substring(0, TempIndex);
                            TempIndex = TempString.LastIndexOf("\\");
                            TempString2 = TempString.Substring(TempIndex);

                            //ID Folder path
                            TempString = TempString.Substring(0, TempIndex);
                            TempIndex = TempString.LastIndexOf("\\");
                            TempIDString = TempString.Substring(TempIndex);

                            sbTargetFilePath.AppendFormat("{0}\\{1}\\{2}\\{3}", OutboxPath, TempIDString, TempString2, TempFileName);


                            sb_FileColXml.AppendFormat(@"<FILE><SOURCEPATH>{0}</SOURCEPATH><DESTINATIONPATH>{1}</DESTINATIONPATH></FILE>", ALCommon.TrimAndReplaceSpecialChars(SourceFilePath), ALCommon.TrimAndReplaceSpecialChars(sbTargetFilePath.ToString()));
                            FilePathNode.InnerText = sbTargetFilePath.ToString();
                        }

                        sb_FileColXml.Append("</FILECOLLECTION>");

                        //Get File to Services outbox folder from Media server
                        TempString = sb_FileColXml.ToString();
                        ErrorXML = ObjFT_MS.GetFile(ref TempString);

                        if (String.IsNullOrEmpty(ErrorXML) == true)
                        {
                            outputXML = OutputXmlDoc.OuterXml;
                        }
                    }
                }
            }
            catch (Exception)
            {
                return outputXML;
            }

            return outputXML;
        }

        private String getFormatedStringForMultiSelect(String InputString)
        {
            StringBuilder sbReturnValue = null;
            String[] ArrValue = null;
            String[] Seperator = { "$AL$" };

            if (String.IsNullOrEmpty(InputString) == true)
                return "";

            sbReturnValue = new StringBuilder(50);
            ArrValue = InputString.Split(Seperator, StringSplitOptions.RemoveEmptyEntries);
            for (int i = 0; i < ArrValue.Length; ++i)
            {
                sbReturnValue.AppendFormat("{0},", ArrValue[i]);
            }
            sbReturnValue.Remove(sbReturnValue.Length - 1, 1);

            return sbReturnValue.ToString();
        }

        private String getFormatedString(XPathNavigator xPthFieldNode, String InputString, String Datatype, Localization LocalizationObj)
        {
            String returnValue = "";
            switch (Datatype.ToUpper())
            {
                case "INTEGER" :
                    returnValue = LocalizationObj.FormatNumericField(InputString, CultureInfo.GetCultureInfo(LocalizationObj.CurrentUICulture));
                    break;
                case "FLOAT" :
                    returnValue = LocalizationObj.FormatFloatField(InputString, CultureInfo.GetCultureInfo(LocalizationObj.CurrentUICulture));
                    break;
                case "CURRENCY":
                    returnValue = xPthFieldNode.SelectSingleNode("CURRENCY").Value + " " + LocalizationObj.FormatCurrencyField(xPthFieldNode.SelectSingleNode("AMOUNT").Value, CultureInfo.GetCultureInfo(LocalizationObj.CurrentUICulture));
                    break;
                default:
                    returnValue = getFormatedStringForMultiSelect(InputString);
                    break;
            }

            return (returnValue);
        }

        #endregion

        #region Helper Methods for Grouping, Aggregating & Sorting Ad hoc Report Data - Ankit Agrawal (19th January 2011)
        private void initializeAggregationFunctions()
        {
            AggregateValues = delegate(List<String> list, XElement xEle)
            {
                list.Add(xEle.Value);
                return list;
            };

            ConvertToDecimal = delegate(String s)
            {
                if (String.IsNullOrEmpty(s))
                    return 0;
                return Convert.ToDecimal(s);
            };

            GetSum = delegate(List<String> lst)
            {
                return lst.Sum(ConvertToDecimal);
            };

            GetMax = delegate(List<String> lst)
            {
                return lst.Max(ConvertToDecimal);
            };

            GetMin = delegate(List<String> lst)
            {
                return lst.Min(ConvertToDecimal);
            };

            GetAvg = delegate(List<String> lst)
            {
                return lst.Average(ConvertToDecimal);
            };

            GetCount = delegate(List<String> lst)
            {
                return lst.Count();
            };
        }

        private void initializeSortingFunctions()
        {
            groupSortFunc = delegate(GroupByQueryResult xEle)
            {
                return xEle.GroupByColumnValue;
            };
        }

        private int GetGroupByColumnsCount(XDocument xDoc_Config)
        {
            int TotalGroupByColumnCount = 0;

            var groupByColumns = xDoc_Config.Root.XPathSelectElements("CONFIGCOLUMNS[GROUPBYLEVEL != 'None']");
            if (groupByColumns != null && groupByColumns.Count() > 0)
                TotalGroupByColumnCount = groupByColumns.Count();

            return TotalGroupByColumnCount;
        }

        private IEnumerable<AggregationConfig> GetAggregationColumns(XDocument xDoc_Config)
        {
            var aggregationColumns = from fieldEle in xDoc_Config.Root.Elements("CONFIGCOLUMNS")
                                     where fieldEle.Element("AGGREGATION").Value.ToUpper() != "NONE"
                                     select new AggregationConfig
                                     {
                                         ColumnName = fieldEle.Element("FIELDNAME").Value,
                                         NodeName = fieldEle.Element("OBJECTNAME").Value,
                                         RelationshipName = fieldEle.Element("RELATIONSHIPNAME").Value,
                                         FieldDataType = fieldEle.Element("FIELDDATATYPE").Value,
                                         AggregationFunc = fieldEle.Element("AGGREGATION").Value.ToUpper()
                                     };
            return aggregationColumns;
        }

        private IEnumerable<SortConfig> GetSortColumns(XDocument xDoc_Config)
        {
            var sortColumns = from fieldEle in xDoc_Config.Root.Elements("CONFIGCOLUMNS")
                              where fieldEle.Element("SORTORDERLEVEL").Value.ToUpper() != "NONE"
                              orderby fieldEle.Element("SORTORDERLEVEL").Value
                              select new SortConfig
                              {
                                  ColumnName = fieldEle.Element("FIELDNAME").Value,
                                  NodeName = fieldEle.Element("OBJECTNAME").Value,
                                  RelationshipName = fieldEle.Element("RELATIONSHIPNAME").Value,
                                  FieldDataType = fieldEle.Element("FIELDDATATYPE").Value,
                                  SortOrder = fieldEle.Element("SORTORDER").Value.ToUpper(),
                                  GroupByLevel = fieldEle.Element("GROUPBYLEVEL").Value.ToUpper()
                              };
            return sortColumns;
        }

        private String FormatValueForGroupDataType(XElement xEle, String DataType)
        {
            String value = "";
            if (xEle != null)
            {
                switch (DataType.ToUpper())
                {
                    case Constants.DATA_TYPE_CURRENCY:
                        value = xEle.Element("AMOUNT").Value;
                        break;
                    case Constants.DATA_TYPE_DATETIME:
                        if (xEle.Element("YEAR") != null && !String.IsNullOrEmpty(xEle.Element("YEAR").Value))
                        {
                            value = xEle.Element("MONTH").Value + "/" + xEle.Element("DAY").Value + "/" + xEle.Element("YEAR").Value;
                            if (xEle.Element("HOUR") != null && !String.IsNullOrEmpty(xEle.Element("HOUR").Value))
                                value += " " + xEle.Element("HOUR").Value + ":" + xEle.Element("MINUTE").Value + ":" + xEle.Element("SECOND").Value;
                        }
                        else
                            value = "";
                        break;
                    case Constants.DATA_TYPE_DATE:
                        if (xEle.Element("YEAR") != null && !String.IsNullOrEmpty(xEle.Element("YEAR").Value))
                            value = xEle.Element("MONTH").Value + "/" + xEle.Element("DAY").Value + "/" + xEle.Element("YEAR").Value;
                        else
                            value = "";
                        break;
                    default:
                        value = xEle.Value;
                        break;
                }
            }
            return value;
        }

        private Object FormatValueForSortType(XElement xEle, String DataType)
        {
            Object value = "";

            switch (DataType.ToUpper())
            {
                case Constants.DATA_TYPE_CURRENCY:
                    if (xEle != null)
                        value = Convert.ToDouble(xEle.Element("AMOUNT").Value);
                    else
                        value = 0.0;
                    break;
                case Constants.DATA_TYPE_FLOAT:
                    if (xEle != null)
                        value = Convert.ToDouble(xEle.Value);
                    else
                        value = 0.0;
                    break;
                case Constants.DATA_TYPE_NUMBER:
                    if (xEle != null)
                        value = Convert.ToInt32(xEle.Value);
                    else
                        value = 0;
                    break;
                case Constants.DATA_TYPE_DATETIME:
                    if (xEle != null)
                    {
                        if (xEle.Element("YEAR") != null && !String.IsNullOrEmpty(xEle.Element("YEAR").Value))
                        {

                            int currYear = Convert.ToInt32(xEle.Element("YEAR").Value);
                            int currMth = Convert.ToInt32(xEle.Element("MONTH").Value);
                            int currDay = Convert.ToInt32(xEle.Element("DAY").Value);
                            int currHour = 0;
                            int currMinute = 0;
                            int currSec = 0;
                            if (xEle.Element("HOUR") != null && !String.IsNullOrEmpty(xEle.Element("HOUR").Value))
                                currHour = Convert.ToInt32(xEle.Element("HOUR").Value);
                            if (xEle.Element("MINUTE") != null && !String.IsNullOrEmpty(xEle.Element("MINUTE").Value))
                                currMinute = Convert.ToInt32(xEle.Element("MINUTE").Value);
                            if (xEle.Element("SECOND") != null && !String.IsNullOrEmpty(xEle.Element("SECOND").Value))
                                currSec = Convert.ToInt32(xEle.Element("SECOND").Value);

                            DateTime refDate = new DateTime(1990, 1, 1);
                            DateTime currDate = new DateTime(currYear, currMth, currDay, currHour, currMinute, currSec);
                            value = (currDate - refDate).TotalMilliseconds;
                        }
                        else
                            value = 0.0;
                    }
                    else
                        value = 0.0;
                    break;
                case Constants.DATA_TYPE_DATE:
                    if (xEle != null)
                    {
                        if (xEle.Element("YEAR") != null && !String.IsNullOrEmpty(xEle.Element("YEAR").Value))
                        {

                            int currYear = Convert.ToInt32(xEle.Element("YEAR").Value);
                            int currMth = Convert.ToInt32(xEle.Element("MONTH").Value);
                            int currDay = Convert.ToInt32(xEle.Element("DAY").Value);
                            DateTime refDate = new DateTime(1990, 1, 1);
                            DateTime currDate = new DateTime(currYear, currMth, currDay);
                            value = (currDate - refDate).TotalMilliseconds;
                        }
                        else
                            value = 0.0;
                    }
                    else
                        value = 0.0;
                    break;
                default:
                    if (xEle != null)
                        value = xEle.Value;
                    break;
            }
            return value;
        }

        private ERRORXML GroupData(XDocument xDoc_Config, List<String> relationshipList, int TotalGroupLevels, IEnumerable<AggregationConfig> aggregationColumns, IEnumerable<SortConfig> sortColumns, XML dataXML, int currLevel, String str_TargetObjectTypeName, String str_TargetObjectPK, ref StringBuilder sb_HierarichalXML)
        {
            #region Member Variables
            String groupByColumnName = null;
            Boolean lastLevel = false;
            String relationShipName = null;
            String nodeName = null;
            XDocument xDoc_SearchOutput = XDocument.Parse(dataXML);
            StringBuilder sb_TempXML = null;
            String TempXML = null;
            String groupByName = null;
            String groupByFieldDataType = null;
            String groupBySortOrder = null;
            String groupByColumnValue = null;
            Boolean isRelated = false;
            ERRORXML ErrorXML = null;
            String GroupDataNodesXML = null;
            XElement dataNode = null;
            #endregion

            try
            {
                #region Get Group By Columns
                var groupByColumns = from field in xDoc_Config.Root.Elements("CONFIGCOLUMNS")
                                     where field.Element("GROUPBYLEVEL").Value == currLevel.ToString()
                                     select new
                                     {
                                         ColumnName = field.Element("FIELDNAME").Value,
                                         NodeName = field.Element("OBJECTNAME").Value,
                                         RelationShipName = field.Element("RELATIONSHIPNAME").Value,
                                         FieldDataType = field.Element("FIELDDATATYPE").Value,
                                         SortOrder = field.Element("SORTORDER").Value.ToUpper()
                                     };
                #endregion

                #region Identify lastlevel or not and populate member variables for grouping
                if (groupByColumns != null && groupByColumns.ToArray().Length > 0)
                {
                    foreach (var groupByColumn in groupByColumns)
                    {
                        groupByColumnName = groupByColumn.ColumnName;
                        nodeName = groupByColumn.NodeName;                        
                        relationShipName = groupByColumn.RelationShipName;
                        groupByName = relationShipName + "." + groupByColumnName;
                        groupByFieldDataType = groupByColumn.FieldDataType;
                        groupBySortOrder = groupByColumn.SortOrder;

                        if (relationShipName != nodeName)
                            isRelated = true;
                        else
                            isRelated = false;
                    }
                    currLevel++;
                }
                else
                    lastLevel = true;
                #endregion

                if (!lastLevel)
                {
                    #region Handle Grouping Levels

                    HandleEmptyNodes(str_TargetObjectTypeName, relationShipName, nodeName, groupByColumnName, ref xDoc_SearchOutput);

                    var items = xDoc_SearchOutput.Root.Elements(str_TargetObjectTypeName);
                    if (isRelated)
                        items = items.Elements(relationShipName).Elements(nodeName);
                    items = items.Elements(groupByColumnName);

                    var groups =
                        from item in items
                        group item by FormatValueForGroupDataType(item, groupByFieldDataType)
                            into groupData
                            select new GroupByQueryResult()
                            {
                                GroupByColumnValue = groupData.Key,
                                groupData = groupData,
                            };

                    if (groupBySortOrder != "NONE")
                    {
                        ErrorXML = GetSortedGroup(groupBySortOrder, ref groups);
                        if (!String.IsNullOrEmpty(ErrorXML))
                            return ErrorXML;
                    }

                    if (groups != null && groups.ToArray().Length > 0)
                    {
                        foreach (var group in groups)
                        {
                            groupByColumnValue = ALCommon.TrimAndReplaceSpecialChars(group.GroupByColumnValue);                                                        
                            sb_HierarichalXML.AppendFormat("<GROUPDATA NAME='{0}' VALUE='{1}'", groupByName, groupByColumnValue);

                            sb_TempXML = new StringBuilder();

                            foreach (XElement p in group.groupData)
                            {
                                dataNode = p.Parent;
                                XElement parentNode = null;
                                XElement relNode = null;
                                XElement requiredNode = null;
                                IEnumerable<XElement> nodesList = null;
                                String currValue = null;
                                int nodeCount = 0;
                                GroupDataNodesXML = null;

                                if (isRelated)
                                    dataNode = dataNode.Parent.Parent;

                                parentNode = new XElement(dataNode);

                                if (isRelated)
                                {
                                    //If grouping is done on fields such that one to many grouping occurs
                                    nodesList = parentNode.Elements(relationShipName).Elements(nodeName).Elements(groupByColumnName);
                                    if (nodesList != null && nodesList.Count() > 1)
                                    {
                                        relNode = parentNode.Element(relationShipName);
                                        nodeCount = nodesList.Count();
                                        for (int n = 0; n < nodeCount; n++)
                                        {
                                            XElement tempNode = nodesList.ElementAt(n);
                                            currValue = FormatValueForGroupDataType(tempNode, groupByFieldDataType);
                                            if (currValue == group.GroupByColumnValue)
                                            {
                                                requiredNode = new XElement(tempNode.Parent);
                                                break;
                                            }
                                        }
                                        relNode.ReplaceAll(requiredNode);
                                    }
                                }

                                //parentNode.Element(groupByColumnName).Remove();

                                ErrorXML = GetDataNodesForGroup(parentNode, relationshipList, currLevel, TotalGroupLevels, ref GroupDataNodesXML);
                                if (!String.IsNullOrEmpty(ErrorXML))
                                    return ErrorXML;
                                sb_TempXML.Append(GroupDataNodesXML);
                            }

                            TempXML = "<GROUPDATA>" + sb_TempXML.ToString() + "</GROUPDATA>";

                            #region Aggregation Related Logic Here
                            ErrorXML = AggregateData(aggregationColumns, TempXML, str_TargetObjectTypeName, str_TargetObjectPK, ref sb_HierarichalXML);
                            if (!String.IsNullOrEmpty(ErrorXML))
                                return ErrorXML;
                            #endregion

                            sb_HierarichalXML.Append(">");


                            ErrorXML = GroupData(xDoc_Config, relationshipList, TotalGroupLevels, aggregationColumns, sortColumns, TempXML, currLevel, str_TargetObjectTypeName, str_TargetObjectPK, ref sb_HierarichalXML);
                            if (!String.IsNullOrEmpty(ErrorXML))
                                return ErrorXML;
                            sb_HierarichalXML.Append("</GROUPDATA>");
                        }
                    }
                    else
                    {
                        sb_HierarichalXML.AppendFormat("<GROUPDATA NAME='{0}' VALUE='{1}'", groupByName, groupByColumnValue);

                        sb_TempXML = new StringBuilder();

                        foreach (XElement myNode in xDoc_SearchOutput.Root.Elements())
                        {
                            ErrorXML = GetDataNodesForGroup(myNode, relationshipList, currLevel, TotalGroupLevels, ref GroupDataNodesXML);
                            if (!String.IsNullOrEmpty(ErrorXML))
                                return ErrorXML;
                            sb_TempXML.Append(GroupDataNodesXML);
                        }

                        TempXML = "<GROUPDATA>" + sb_TempXML.ToString() + "</GROUPDATA>";

                        //TempXML = dataXML;
                        #region Aggregation Related Logic Here
                        ErrorXML = AggregateData(aggregationColumns, TempXML, str_TargetObjectTypeName, str_TargetObjectPK, ref sb_HierarichalXML);
                        if (!String.IsNullOrEmpty(ErrorXML))
                            return ErrorXML;
                        #endregion

                        sb_HierarichalXML.Append(">");

                        ErrorXML = GroupData(xDoc_Config, relationshipList, TotalGroupLevels, aggregationColumns, sortColumns, TempXML, currLevel, str_TargetObjectTypeName, str_TargetObjectPK, ref sb_HierarichalXML);
                        if (!String.IsNullOrEmpty(ErrorXML))
                            return ErrorXML;
                        sb_HierarichalXML.Append("</GROUPDATA>");
                    }
                    #endregion
                }
                else
                {
                    #region Handle Last Level

                    ErrorXML = GetNLevelSortedData(sortColumns, ref dataXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                        return ErrorXML;

                    if (currLevel > 1)
                        TempXML = ALCommon.GetInnerValue(ref dataXML, "<GROUPDATA>", "</GROUPDATA>");
                    else
                    {
                        sb_TempXML = new StringBuilder();

                        xDoc_SearchOutput = XDocument.Parse(dataXML);
                        foreach (XElement myNode in xDoc_SearchOutput.Root.Elements())
                        {
                            ErrorXML = GetDataNodesForGroup(myNode, relationshipList, currLevel, TotalGroupLevels, ref GroupDataNodesXML);
                            if (!String.IsNullOrEmpty(ErrorXML))
                                return ErrorXML;
                            sb_TempXML.Append(GroupDataNodesXML);
                        }

                        TempXML = sb_TempXML.ToString();
                    }

                    if (!String.IsNullOrEmpty(TempXML))
                        sb_HierarichalXML.Append(TempXML);
                    #endregion
                }
            }
            catch (Exception)
            {                
                ErrorXML = "Err_AdhocReport_Report_Group_Data";
            }
            return ErrorXML;
        }

        private void HandleEmptyNodes(String str_TargetObjectTypeName, String relationShipName, String nodeName, String groupByColumnName, ref XDocument xDoc_SearchOutput)
        {
            IEnumerable<XElement> emptyItems = null;

            if (relationShipName != nodeName) //implies isRelated = true
            {
                emptyItems = xDoc_SearchOutput.Root.XPathSelectElements(str_TargetObjectTypeName + "[not(" + relationShipName + ")]");
                if (emptyItems != null)
                {
                    foreach (XElement emptyItem in emptyItems)
                    {
                        emptyItem.SetElementValue(relationShipName, "");
                    }
                }

                emptyItems = xDoc_SearchOutput.Root.XPathSelectElements(str_TargetObjectTypeName + "/" + relationShipName + "[not(" + nodeName + ")]");
                if (emptyItems != null)
                {
                    foreach (XElement emptyItem in emptyItems)
                    {
                        emptyItem.SetElementValue(nodeName, "");
                    }
                }

                emptyItems = xDoc_SearchOutput.Root.XPathSelectElements(str_TargetObjectTypeName + "/" + relationShipName + "/" + nodeName + "[not(" + groupByColumnName + ")]");
                if (emptyItems != null)
                {
                    foreach (XElement emptyItem in emptyItems)
                    {
                        emptyItem.SetElementValue(groupByColumnName, "");
                    }
                }
            }
            else
            {
                emptyItems = xDoc_SearchOutput.Root.XPathSelectElements(str_TargetObjectTypeName + "[not(" + groupByColumnName + ")]");
                if (emptyItems != null)
                {
                    foreach (XElement emptyItem in emptyItems)
                    {
                        emptyItem.SetElementValue(groupByColumnName, "");
                    }
                }
            }
        }

        private ERRORXML GetDataNodesForGroup(XElement parentNode, List<String> relationshipList, int currLevel, int totalLevels, ref String dataXML)
        {
            IEnumerable<XElement> nodesList = null;
            String resultVal = null;
            int nodeCount = 0;
            ERRORXML ErrorXML = null;
            List<string> OneToManyRelations = new List<string>();

            try
            {
                if (currLevel - 1 == totalLevels)
                {
                    foreach (string currRelation in relationshipList)
                    {
                        nodesList = parentNode.Elements(currRelation);
                        if (nodesList != null && nodesList.Count() > 0)
                        {
                            nodesList = nodesList.Elements();
                            if (nodesList != null && nodesList.Count() > 1)
                            {
                                OneToManyRelations.Add(currRelation);
                            }
                        }
                    }
                }

                if (OneToManyRelations.Count > 0)
                {
                    nodesList = null;
                    StringBuilder sb_OneToManyNodes = new StringBuilder();
                    //List<String> tempRelList = new List<string>(relationshipList);
                    foreach (string currRelationName in OneToManyRelations)
                    {
                        XElement cloneNode = null;
                        XElement tempCloneNode = null;
                        XElement cloneRelNode = null;
                        nodesList = parentNode.Elements(currRelationName);
                        if (nodesList != null && nodesList.Count() > 0)
                        {
                            nodesList = nodesList.Elements();
                            if (nodesList != null)
                            {
                                nodeCount = nodesList.Count();
                                for (int j = 0; j < nodeCount; j++)
                                {
                                    cloneNode = new XElement(parentNode);

                                    cloneRelNode = cloneNode.Element(currRelationName);
                                    cloneRelNode.ReplaceAll(nodesList.ElementAt(j));
                                    foreach (string tempRelationName in OneToManyRelations)
                                    {
                                        if (tempRelationName != currRelationName)
                                        {
                                            tempCloneNode = cloneNode.Element(tempRelationName);
                                            if (tempCloneNode != null)
                                                tempCloneNode.Remove();
                                        }
                                    }
                                    sb_OneToManyNodes.Append(cloneNode.ToString());
                                }
                            }
                        }
                    }
                    resultVal = sb_OneToManyNodes.ToString();
                }
                else
                    resultVal = parentNode.ToString();

                dataXML = resultVal;
            }
            catch (Exception)
            {
                ErrorXML = "Err_AdhocReport_Report_Group_Data";
            }
            return ErrorXML;
        }

        private ERRORXML AggregateData(IEnumerable<AggregationConfig> aggregationColumns, XML GroupXML, String str_TargetObjectTypeName, String str_TargetObjectPK, ref StringBuilder sb_HierarichalXML)
        {
            String aggregationName = null;
            decimal aggregationValue = 0;
            ERRORXML ErrorXML = null;
            XElement xEle_GroupData = null;
            try
            {
                xEle_GroupData = XElement.Parse(GroupXML);
                foreach (var aggregationColumn in aggregationColumns)
                {
                    Func<List<String>, decimal> aggregationFunc = null;
                    IEnumerable<XElement> aggregationList = null;

                    if (aggregationColumn.NodeName == aggregationColumn.RelationshipName)
                        aggregationList = xEle_GroupData.Elements(str_TargetObjectTypeName).Distinct(new XElementComparer(str_TargetObjectPK)).Elements(aggregationColumn.ColumnName);
                    else
                        aggregationList = xEle_GroupData.Elements(str_TargetObjectTypeName).Elements(aggregationColumn.RelationshipName).Elements(aggregationColumn.NodeName).Distinct().Elements(aggregationColumn.ColumnName);

                    if (aggregationColumn.FieldDataType == "CURRENCY")
                        aggregationList = aggregationList.Elements("AMOUNT");

                    switch (aggregationColumn.AggregationFunc)
                    {
                        case Constants.AGGREGATEFUNC_SUM:
                            aggregationFunc = GetSum;
                            break;
                        case Constants.AGGREGATEFUNC_MAX:
                            aggregationFunc = GetMax;
                            break;
                        case Constants.AGGREGATEFUNC_MIN:
                            aggregationFunc = GetMin;
                            break;
                        case Constants.AGGREGATEFUNC_AVG:
                            aggregationFunc = GetAvg;
                            break;
                        case Constants.AGGREGATEFUNC_COUNT:
                            aggregationFunc = GetCount;
                            break;
                        default:
                            break;
                    }

                    aggregationName = aggregationColumn.RelationshipName + "." + aggregationColumn.ColumnName;
                    aggregationValue = aggregationList.Aggregate(new List<String>(), AggregateValues, aggregationFunc);
                    sb_HierarichalXML.AppendFormat(" {0}='{1}'", aggregationName, aggregationValue.ToString());
                }
            }
            catch (Exception)
            {   
                ErrorXML = "Err_AdhocReport_Report_Aggregate_Data";
            }
            return ErrorXML;
        }

        private ERRORXML GetSortedGroup(String groupBySortOrder, ref IEnumerable<GroupByQueryResult> groups)
        {
            ERRORXML ErrorXML = null;
            try
            {
                switch (groupBySortOrder)
                {
                    case "ASCENDING":
                        groups = groups.OrderBy(groupSortFunc);
                        break;
                    case "DESCENDING":
                        groups = groups.OrderByDescending(groupSortFunc);
                        break;
                }
            }
            catch (Exception)
            {
                ErrorXML = "Err_AdhocReport_Report_Sorting_Groups";
            }
            return ErrorXML;
        }

        private ERRORXML GetNLevelSortedData(IEnumerable<SortConfig> sortColumns, ref String dataXML)
        {
            IOrderedEnumerable<XElement> orderedList = null;
            ERRORXML ErrorXML = null;
            XDocument xDoc_dataXML = null;
            try
            {
                xDoc_dataXML = XDocument.Parse(dataXML);
                foreach (var sortColumn in sortColumns)
                {
                    if (sortColumn.GroupByLevel == "NONE")
                    {
                        //The .NET internal sorting for IOrderedEnumerable<XElement> using sort function is done by refernece
                        //and thus the variables below should be initialised on each instance to perform multiple level sorting
                        String tempRelationName = sortColumn.RelationshipName;
                        String tempColumnName = sortColumn.ColumnName;
                        String tempNodeName = sortColumn.NodeName;
                        String tempFieldType = sortColumn.FieldDataType;

                        Func<XElement, Object> tempFunc = delegate(XElement xEle)
                        {
                            Object sortKey = "";
                            XElement item = null;
                            if (tempRelationName == tempNodeName)
                                item = xEle.Element(tempColumnName);
                            else
                            {
                                item = xEle.Element(tempRelationName);
                                if (item != null)
                                    item = item.Element(tempNodeName);
                                if (item != null)
                                    item = item.Element(tempColumnName);
                            }

                            sortKey = FormatValueForSortType(item, tempFieldType);
                            return sortKey;
                        };

                        if (orderedList == null)
                        {
                            switch (sortColumn.SortOrder)
                            {
                                case "ASCENDING":
                                    orderedList = xDoc_dataXML.Root.Elements().OrderBy(tempFunc);
                                    break;
                                case "DESCENDING":
                                    orderedList = xDoc_dataXML.Root.Elements().OrderByDescending(tempFunc);
                                    break;
                            }
                        }
                        else
                        {
                            switch (sortColumn.SortOrder)
                            {
                                case "ASCENDING":
                                    orderedList = orderedList.ThenBy(tempFunc);
                                    break;
                                case "DESCENDING":
                                    orderedList = orderedList.ThenByDescending(tempFunc);
                                    break;
                            }
                        }
                    }
                }

                if (orderedList != null)
                {   
                    xDoc_dataXML.Root.ReplaceNodes(orderedList);
                    dataXML = xDoc_dataXML.ToString();
                }
            }
            catch (Exception)
            {
                ErrorXML = "Err_AdhocReport_Report_Sorting_Data";
            }
            return ErrorXML;
        }
        #endregion

        #region Private Methods for excel preparation
         /// <summary>
        /// <CreatedBy>Anand Mahadik</CreatedBy>
        /// <CreatedDate>10th July 2012</CreatedDate>
        /// <Purpose>This method is used to generate the Excel for adhoc report using the configuration and data provided.</Purpose>
        /// </summary>
        /// <param name="FilePathForAdHocConfig">This contains the configuration XML which is used for Excel generation.</param>
        /// <param name="FilePathForAdHocResultantData">This contains the resultant data XML which is used for Excel generation.</param>
        /// <param name="FileCollectionXML">This is used to store the details of generated Excel file.</param>
        /// <returns>On Success it will generate the Excel and return NULL; whereas on failure an Error will be returned.</returns>
        private ERRORXML generateExcel(String AdHocConfig, ref DataSet DS_Result, String tenantName, out XML FileCollectionXML)
        {
            #region Method Variables
            ALErrorCommon       errorcommon_Object          = null;
            String              str_GeneratedExcelFilePath  = null;
            String              str_AdHocReportName         = null;
            String              str_TargetObjectTypeName    = null; 
            String              str_ReportSchedulerID       = null;
            String              str_ExcelReportFileName     = null;            
            XML                 xml_DisplayColumnsConfig    = null;
            ERRORXML            errorxml_Output             = null;
            XDocument           xDoc_DisplayConfig          = null;
            XElement            xEle_DisplayConfig          = null;
            String              TempFolderPath              = null;                  
            #endregion

            FileCollectionXML = null;
            TenantName = tenantName;
            try
            {
                #region STEP00: Mandatory checks
                errorcommon_Object = new ALErrorCommon();
                #endregion

                #region STEP01: Retrieve mandatory configuration values
                
                //Retrieve the necessary details from the config XML.
                xml_DisplayColumnsConfig            = ALCommon.GetInnerValue(ref AdHocConfig, "<ADHOCDISPLAYCOLUMNSCONFIG>", "</ADHOCDISPLAYCOLUMNSCONFIG>");
                str_TargetObjectTypeName            = ALCommon.GetInnerValue(ref AdHocConfig, "<ADHOCTARGETOBJECTTYPENAME>", "</ADHOCTARGETOBJECTTYPENAME>");
                str_AdHocReportName                 = ALCommon.GetInnerValue(ref AdHocConfig, "<ADHOCREPORTNAME>", "</ADHOCREPORTNAME>");
                str_ExcelReportFileName             = ALCommon.GetInnerValue(ref AdHocConfig, "<ADHOCREPORTEXCELFILENAME>", "</ADHOCREPORTEXCELFILENAME>");
                str_ReportSchedulerID               = ALCommon.GetInnerValue(ref AdHocConfig, "<REPORTSCHEDULERID>", "</REPORTSCHEDULERID>");
                str_FormatXMLPath                   = ALCommon.GetInnerValue(ref AdHocConfig, "<FORMATXMLPATH>", "</FORMATXMLPATH>");
                str_CultureInfo                     = ALCommon.GetInnerValue(ref AdHocConfig, "<FORMATCULTURENAME>", "</FORMATCULTURENAME>");

                #endregion

                #region STEP02: Retrieve the required parameters and attributes from the list data XML

                //Use the display columns confguration XML to generate columns and rows for the Excel.
                xDoc_DisplayConfig                  = XDocument.Parse(xml_DisplayColumnsConfig);
                xEle_DisplayConfig                  = xDoc_DisplayConfig.Root;                
                if (xEle_DisplayConfig == null)
                {
                    errorcommon_Object.AppendError("ERR_ADHOCREPORT_REPORTENGINE_DISPLAYCONFIGNULL", this.GetType().ToString(),
                            "GenerateExcelForAdHocReport", null, errorxml_Output);
                    return (errorcommon_Object.ErrorCollectionXML);
                }

                #endregion

                #region STEP03: Process data and generate the Excel 
 
                TempFolderPath = ALCommon.GetTempPath() + "\\" + SessionID + "\\ExportedData";
                if (!Directory.Exists(TempFolderPath))
                    Directory.CreateDirectory(TempFolderPath);
                str_GeneratedExcelFilePath = TempFolderPath+ "\\" + str_ExcelReportFileName;

                PrepareSpreadSheetXML(str_ExcelReportFileName, "ExportedData", ref DS_Result, xEle_DisplayConfig, str_TargetObjectTypeName, str_GeneratedExcelFilePath);

                #endregion

                #region STEP04: Save Excel File And Release The Excel Objects

                //Preapre the FileCollectionXML which is an OUT parameter.
                if (String.IsNullOrEmpty(str_ReportSchedulerID))
                {
                    //Prepare file collection details for Excel view.
                    FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_GeneratedExcelFilePath + "</PATH><NAME>" + str_ExcelReportFileName
                            + "</NAME><REPORTNAME>" + str_AdHocReportName + "</REPORTNAME></FILE></FILECOLLECTION>";
                }
                else
                {
                    //Prepare file collection details for report scheduler.
                    FileCollectionXML = "<FILECOLLECTION><FILE><PATH>" + str_GeneratedExcelFilePath + "</PATH><NAME>" + str_ExcelReportFileName
                            + "</NAME></FILE></FILECOLLECTION>";
                }

                #endregion
            }
            catch (Exception ExceptionObject)
            {
                throw (ExceptionObject);
            }
            return (errorxml_Output);            
        }

        private String PrepareSpreadSheetXML(String str_WorkbookName, String WorkSheetName, ref DataSet DS_Result, XElement xEle_DisplayConfig, String str_TargetObjectTypeName, String str_DestinationPath)
        {
            #region member variables
            IOrderedEnumerable<XElement>    GroupByColumnList       = null;
            IOrderedEnumerable<XElement>    NonGroupByColumnList    = null;
            IEnumerable<XElement>           totalColumnsList        = null;
            XElement                        xEle_FieldTitle         = null;
            XElement                        xEle_ColWidth           = null;            
            Int32                           TotalColumnCnt          = 0;
            Int32                           int_ColumnNumber        = 0;
            String                          str_Width               = null;
            String                          str_DataFieldType       = null;
            String                          ErrorXml                = null;  
            int                             columnIndex             = 0;
            int                             int_WidthInPixel        = 0;
            Double                          ColumnWidth             = 0.0;
            #endregion

            try
            {
                #region Initialization
                customculture_Object        = new CultureInfo(str_CultureInfo);
                #endregion

                using (SpreadsheetDocument SpreadsheetDocumentObj = SpreadsheetDocument.Create(
                        str_DestinationPath, SpreadsheetDocumentType.Workbook))
                {
                    WorkbookPart            workbookpart        = null;
                    WorksheetPart           worksheetpart       = null; 
                    OpenXmlWriter           writer              = null;
                    WorkbookStylesPart      stylesPart          = null;
                    List<OpenXmlAttribute>  CellAttrs           = null;
                    Sheets                  sheets              = null;    
                    Sheet                   sheet               = null;

                    SpreadsheetDocumentObj.CompressionOption    = CompressionOption.SuperFast;

                    #region Prepare SpreadSheet XML
                    workbookpart            = SpreadsheetDocumentObj.AddWorkbookPart();
                    workbookpart.Workbook   = new Workbook();

                    worksheetpart           = workbookpart.AddNewPart<WorksheetPart>();
                    
                    sheets                  = workbookpart.Workbook.AppendChild<Sheets>(new Sheets());
                    sheet                   = new Sheet() { Id = SpreadsheetDocumentObj.WorkbookPart.GetIdOfPart(worksheetpart), SheetId = 1, Name = WorkSheetName };
                    sheets.AppendChild(sheet);

                    stylesPart              = workbookpart.AddNewPart<WorkbookStylesPart>();
                    stylesPart.Stylesheet   = CreateStylesheet(customculture_Object);

                    writer                  = OpenXmlWriter.Create(worksheetpart);
                    CellAttrs               = new List<OpenXmlAttribute>();
                    #region Prepare Column Header
                    Func<XElement, Object> sortFunc = delegate(XElement xEle)
                    {
                        return Convert.ToInt32(xEle.Element("ROWID").Value);
                    };
                    Func<XElement, Object> grpSortFunc = delegate(XElement xEle)
                    {
                        return Convert.ToInt32(xEle.Element("GROUPBYLEVEL").Value);
                    };

                    GroupByColumnList       = xEle_DisplayConfig.XPathSelectElements("CONFIGCOLUMNS[GROUPBYLEVEL != 'None']").OrderBy(grpSortFunc);
                    NonGroupByColumnList    = xEle_DisplayConfig.XPathSelectElements("CONFIGCOLUMNS[GROUPBYLEVEL = 'None']").OrderBy(sortFunc);

                    totalColumnsList        = GroupByColumnList;

                    if (totalColumnsList != null)
                        totalColumnsList    = totalColumnsList.Concat(NonGroupByColumnList);
                    else
                        totalColumnsList    = NonGroupByColumnList;
                    
                    TotalColumnCnt = totalColumnsList.Count();
                    writer.WriteStartElement(new Worksheet());
                    #region Add Columns
                    writer.WriteStartElement(new Columns());
                    foreach (XElement columnEle in totalColumnsList)
                    {
                        xEle_FieldTitle = columnEle.Element("FIELDTITLE");
                        if (xEle_FieldTitle != null)
                        {
                            xEle_ColWidth = columnEle.Element("WIDTH");
                            if (xEle_ColWidth != null)
                                str_Width = xEle_ColWidth.Value;

                            if (String.IsNullOrEmpty(str_Width)) //Apply Default Width
                            {
                                str_DataFieldType = columnEle.Element("FIELDDATATYPE").Value;
                                switch (str_DataFieldType.ToUpper())
                                {
                                    case "DATE":
                                    case "FLOAT":
                                    case "NUMBER":
                                    case "CURRENCY":
                                        str_Width = "120";
                                        break;
                                    case "DATETIME":
                                        str_Width = "150";
                                        break;
                                    default:
                                        str_Width = "200";
                                        break;
                                }
                            }

                            int_WidthInPixel    = Convert.ToInt32(str_Width);
                            ColumnWidth         = (int_WidthInPixel - 5)/ 5; // This formula is mentioned in Microsoft forumn but it also
                            columnIndex++;                                   // Says that its not very accurate as it depends on font type.
                            CellAttrs.Clear();
                            CellAttrs.Add(new OpenXmlAttribute("min", null, columnIndex.ToString()));
                            CellAttrs.Add(new OpenXmlAttribute("max", null, columnIndex.ToString()));
                            CellAttrs.Add(new OpenXmlAttribute("width", null, ColumnWidth.ToString()));
                            CellAttrs.Add(new OpenXmlAttribute("customWidth", null, "1"));
                            writer.WriteStartElement(new Column(), CellAttrs);
                            writer.WriteEndElement();
                        }
                    }
                    writer.WriteEndElement();
                    #endregion
                    
                    writer.WriteStartElement(new SheetData());
                    writer.WriteStartElement(new Row());
                    foreach (XElement columnEle in totalColumnsList)
                    {
                        int_ColumnNumber++;
                        xEle_FieldTitle = columnEle.Element("FIELDTITLE");
                        if (xEle_FieldTitle != null)
                        {                        
                            ErrorXml += printCellData_Xml(xEle_FieldTitle.Value, "String", true, true, writer,CellAttrs, true);
                        }
                    }
                    writer.WriteEndElement();
                    #endregion

                    #region Prepare Column Data
                    CreateExcelDataRows(xEle_DisplayConfig, totalColumnsList, ref DS_Result, str_TargetObjectTypeName, false, writer, CellAttrs);
                    #endregion
                    
                    #endregion
                    writer.WriteEndElement();
                    writer.WriteEndElement();
                    writer.Close();
                    SpreadsheetDocumentObj.Close();
                }
            }
            catch (Exception ObjException)
            {
                throw ObjException;
            }
            finally
            {
            }
            return null;
        }

        private void CreateExcelDataRows(XElement xEle_DisplayConfig, IEnumerable<XElement> ColumnList, ref DataSet DS_Result, String objectName, bool FromGroupData,
            OpenXmlWriter writer, List<OpenXmlAttribute> CellAttrs)
        {
            #region member variables
            String                  str_FieldValue                  = "";
            StringBuilder           DataSetFieldName                = null;
            List<FieldInfo>         FieldInfoCollectionObj          = new List<FieldInfo>();
            FieldInfo               FieldInfoObj                    = null;
            StringBuilder           sb_DateTime                     = null;
            Hashtable               FieldnameValueMapping           = null;
            String                  FieldDataType                   = null;
            DataTable               DT_EntityCollection             = null;
            DataRow                 CurrentRow                      = null;
            object                  FieldValue                      = null;
            StringBuilder           SB_SortByClause                 = null;
            IEnumerable<XElement>   xEle_SortOnColumns              = null;
            String[]                SortOnColumnArray               = null; 
            String                  ObjectName                      = null;
            String                  FieldName                       = null;
            String                  SortSequence                    = null;
            String                  SortOrder                       = null;
            int                     int_SortOrder                   = 0;
            String                  CommaSep                        = null;

            #endregion
            
            DataSetFieldName                = new StringBuilder(50);
            sb_DateTime                     = new StringBuilder(50);
            FieldnameValueMapping           = new Hashtable(10);
            
            foreach (XElement column in ColumnList)
            {
                FieldInfoObj                    = new FieldInfo();
                FieldInfoObj.RelationShipName   = column.Element("RELATIONSHIPNAME").Value;
                FieldInfoObj.ObjectName         = column.Element("OBJECTNAME").Value;
                FieldInfoObj.FieldName          = column.Element("FIELDNAME").Value;
                FieldInfoObj.DataType           = column.Element("FIELDDATATYPE").Value;
                FieldInfoCollectionObj.Add(FieldInfoObj);
            }

            if((DS_Result.Tables.Count > 0) && (DS_Result.Tables[0].Rows.Count > 0))
            {
                DT_EntityCollection     = DS_Result.Tables[0];

                #region Form sort by clause
                SB_SortByClause     = new StringBuilder(50);
                xEle_SortOnColumns  = xEle_DisplayConfig.XPathSelectElements("CONFIGCOLUMNS[SORTORDER != 'None']");
                if (xEle_SortOnColumns.Count() > 0)
                {
                    SortOnColumnArray = new String[xEle_SortOnColumns.Count()];
                    foreach (XElement xEle_SortOnColumn in xEle_SortOnColumns)
                    {
                        ObjectName      = xEle_SortOnColumn.Element("OBJECTNAME").Value;
                        FieldName       = xEle_SortOnColumn.Element("FIELDNAME").Value;
                        SortSequence    = xEle_SortOnColumn.Element("SORTORDERLEVEL").Value;
                        if (SortSequence.ToUpper() == "NONE")
                        {
                            SortSequence = xEle_SortOnColumn.Element("GROUPBYLEVEL").Value;
                        }
                        int_SortOrder = Convert.ToInt32(SortSequence);
                        int_SortOrder--;

                        SortOrder = xEle_SortOnColumn.Element("SORTORDER").Value;
                        if (SortOrder.ToUpper() == "ASCENDING")
                            SortOrder = "ASC";
                        else
                            SortOrder = "DESC";

                        SB_SortByClause.Remove(0, SB_SortByClause.Length);

                        SB_SortByClause.AppendFormat("{0}${1} {2}", ObjectName, FieldName, SortOrder);
                        SortOnColumnArray[int_SortOrder] = SB_SortByClause.ToString();
                    }

                    SB_SortByClause.Remove(0, SB_SortByClause.Length);
                    for (int Index = 0; Index < SortOnColumnArray.Length; Index++)
                    {
                        SB_SortByClause.Append(CommaSep);
                        SB_SortByClause.Append(SortOnColumnArray[Index]);
                        if (String.IsNullOrEmpty(CommaSep))
                            CommaSep = ",";
                    }
                }
                #endregion

                DataRow[] Rows = DT_EntityCollection.Select(null, SB_SortByClause.ToString());
                for (int Index = 0; Index < Rows.Length; Index++)
                {
                    CurrentRow = Rows[Index];
                    writer.WriteStartElement(new Row());
                    for(int ColID = 0; ColID < FieldInfoCollectionObj.Count; ColID++)
                    {
                        DataSetFieldName.Remove(0, DataSetFieldName.Length);
                        str_FieldValue  = null;
                        FieldInfoObj    = FieldInfoCollectionObj[ColID];
                        FieldDataType   = FieldInfoObj.DataType;

                        if (FieldDataType.ToUpper() != "CURRENCY")
                        {
                            DataSetFieldName.AppendFormat("{0}${1}", FieldInfoObj.ObjectName, FieldInfoObj.FieldName);
                            FieldValue = CurrentRow[DataSetFieldName.ToString()];
                            FieldValue = CurrentRow[DataSetFieldName.ToString()];
                            if (FieldValue != null)
                                str_FieldValue = FieldValue.ToString();
                            else
                                str_FieldValue = String.Empty;
                        }
                        else
                        {
                            DataSetFieldName.AppendFormat("{0}${1}_C", FieldInfoObj.ObjectName, FieldInfoObj.FieldName);
                            FieldValue      = CurrentRow[DataSetFieldName.ToString()];
                            if (FieldValue != null)
                            {
                                str_FieldValue  = FieldValue.ToString();
                                DataSetFieldName.AppendFormat("{0}${1}_A", FieldInfoObj.ObjectName, FieldInfoObj.FieldName);
                                FieldValue      = CurrentRow[DataSetFieldName.ToString()];
                                str_FieldValue  = str_FieldValue + "$" + FieldValue;
                            }
                            else
                                str_FieldValue  = String.Empty;
                        }
                        
                        if (String.Compare(FieldDataType, "DATETIME", true) != 0)
                        {
                            FormatValueforDataType(ref FieldDataType, ref str_FieldValue);
                        }

                       printCellData_Xml( str_FieldValue,  FieldDataType, false, false, writer,CellAttrs, true);
                    }
                    writer.WriteEndElement();
                }
            }
        }
        
        private ERRORXML printCellData_Xml( String data, String DataType, Boolean HeaderRowCell, bool bIsDataHtml, OpenXmlWriter writer,List<OpenXmlAttribute> CellAttrs, bool OverWriteStyle)
        {
            #region Member Varibales
            #endregion

            if (String.IsNullOrEmpty(DataType))
                DataType = "STRING";

            CellAttrs.Clear();
            switch (DataType.ToUpper())
            {
                case "DATETIME":
                        #region DATETIME
                        if (!string.IsNullOrEmpty(data))
                            data        = Convert.ToDateTime(data).ToOADate().ToString();

                        CellAttrs.Add(new OpenXmlAttribute("s", null, "2"));
                        CellAttrs.Add(new OpenXmlAttribute("t", null, "n"));
                        #endregion
                        break;

                case "BOOLEAN":
                    #region BOOLEAN
                    switch (data)
                    {
                        case "1": data = "Yes";
                            break;
                        case "0": data = "No";
                            break;
                        default:
                            break;
                    }
                    #endregion
                    CellAttrs.Add(new OpenXmlAttribute("s", null, "1"));
                    CellAttrs.Add(new OpenXmlAttribute("t", null, "str"));
                    break;

                case "FLOAT":
                            CellAttrs.Add(new OpenXmlAttribute("s", null, "4"));
                            break;
                case "INTEGER":
                case "NUMBER":
                    CellAttrs.Add(new OpenXmlAttribute("s", null, "3"));
                    CellAttrs.Add(new OpenXmlAttribute("t", null, "n"));
                    break;

                case "CURRENCY":
                    CellAttrs.Add(new OpenXmlAttribute("s", null, "1"));
                    CellAttrs.Add(new OpenXmlAttribute("t", null, "str"));
                    break;

                default:
                    if(HeaderRowCell)
                        CellAttrs.Add(new OpenXmlAttribute("s", null, "0"));
                    else
                        CellAttrs.Add(new OpenXmlAttribute("s", null, "1"));
                    CellAttrs.Add(new OpenXmlAttribute("t", null, "str"));
                    break;
            }

            #region prepare cell node structure
            //if (bIsDataHtml == false)
            //{
            //    data = ALCommon.TrimAndReplaceSpecialChars(data);
            //    data = data.Replace("\n", "&#10;");
            //}
            //else
            //{
            //    data = data.Replace("\n", "&#10;");
            //}

            writer.WriteStartElement(new Cell(), CellAttrs);
            if (!String.IsNullOrEmpty(data))
            {
                writer.WriteElement(new CellValue(data));
            }
            writer.WriteEndElement();
            #endregion

            return null;
        }

        private void FormatValueforDataType(ref String str_DataFieldType,ref String str_FieldValue)
        {
            String[] Curr_Amount            = null;
            String[] SplitBy                = {"$"};
            ALCommon ALCommonObj            = null;
            String[] MultiSelectvalueArray  = null;
            try
            {                
                switch (str_DataFieldType)
                {
                    case "FLOAT":
                        //str_FieldValue = ALCommon.FormatFloatField(str_FormatXMLPath,
                        //        str_FieldValue, customculture_Object);
                        break;

                    case "CURRENCY":
                        if (!String.IsNullOrEmpty(str_FieldValue))
                        {
                            Curr_Amount             = str_FieldValue.Split(SplitBy,StringSplitOptions.None);
                            String str_Currency     = "";
                            if (!String.IsNullOrEmpty(Curr_Amount[0]))
                                str_Currency = Curr_Amount[0];

                            str_FieldValue          = str_Currency + " " + ALCommon.FormatCurrencyField(str_FormatXMLPath,
                                                                                                        Curr_Amount[1],
                                                                                                        customculture_Object);
                        }
                        break;
                    
                    case "STRING":
                                ALSpecialCharHandling.DecodeSpecialCharacters(ref str_FieldValue);
                                if (str_FieldValue.Contains("$ALP$") || str_FieldValue.Contains("$AL$"))
                                {
                                    ALCommonObj = new ALCommon(TenantName, UserID.ToString(), SessionID.ToString(), Language);
                                    ALCommonObj.ConvertDelimiterSeparatorToSeparator(str_FieldValue, ",", out str_FieldValue, out MultiSelectvalueArray, false, TenantName);
                                    str_FieldValue = str_FieldValue.Replace("$ALP$", "-");
                                }
                                break;
                    default:
                        break;
                }
            }
            catch
            {
                //Ignore Exception for formatting of data value based on field data type.
            }
        }


        private static Stylesheet CreateStylesheet(CultureInfo CultureInfoObj)
        {
            Stylesheet                                          stylesheet              = null;
            Fonts                                               fonts                   = null;
            FontName                                            fontname                = null;
            FontSize                                            fontsize                = null;
            CellFormats                                         cellformats             = null;
            CellFormat                                          cellformat              = null;
            Bold                                                bold                    = null;
            CellStyles                                          cellstyles              = null;
            CellStyle                                           cellstyle               = null;
            Fills                                               fills                   = null; 
            Fill                                                fill                    = null;
            Borders                                             borders                 = null;
            Border                                              border                  = null;
            PatternFill                                         patternFill             = null;
            Alignment                                           alignment               = null;
            DocumentFormat.OpenXml.Spreadsheet.Font             font                    = null;
            DocumentFormat.OpenXml.Spreadsheet.FontFamily       fontfamily              = null;
            DocumentFormat.OpenXml.Spreadsheet.NumberingFormats numberingformats        = null;
            DocumentFormat.OpenXml.Spreadsheet.NumberingFormat  numberingformat         = null;
            String                                              str_FloatFormatCode     = null;

            stylesheet                      = new Stylesheet();

            //Fonts
            fonts                           = new Fonts();

            font                            = new DocumentFormat.OpenXml.Spreadsheet.Font();
            fontfamily                      = new DocumentFormat.OpenXml.Spreadsheet.FontFamily();
            fontfamily.Val                  = 3; // Swiss
            font.Append(fontfamily);
            fontname                        = new FontName();
            fontname.Val                    = "Calibri";
            fontsize                        = new FontSize();
            fontsize.Val                    = 11;
            bold                            = new Bold();
            bold.Val                        = true;
            font.Bold                       = bold;
            font.FontName                   = fontname;
            font.FontSize                   = fontsize;
            fonts.Append(font);

            font                            = (DocumentFormat.OpenXml.Spreadsheet.Font)font.CloneNode(true);
            font.Bold.Val                   = false;
            fonts.Append(font);

            fonts.Count                     = (uint)fonts.ChildElements.Count;

            //Fills
            fills                           = new Fills();
            fill                            = new Fill();
            patternFill                     = new PatternFill();
            patternFill.PatternType         = PatternValues.None;
            fill.PatternFill                = patternFill;
            fills.Append(fill);
            fills.Count                     = UInt32Value.FromUInt32((uint)fills.ChildElements.Count);

            //Borders
            borders                         = new Borders();
            border                          = new Border();
            border.LeftBorder               = new LeftBorder();
            border.RightBorder              = new RightBorder();
            border.TopBorder                = new TopBorder();
            border.BottomBorder             = new BottomBorder();
            border.DiagonalBorder           = new DiagonalBorder();
            borders.Append(border);
            borders.Count                   = UInt32Value.FromUInt32((uint)borders.ChildElements.Count);


            //Data formatting
            numberingformats                = new NumberingFormats();

            numberingformat                 = new NumberingFormat();
            numberingformat.FormatCode      = "";
            numberingformat.NumberFormatId  = 164;
            numberingformats.Append(numberingformat);

            //DateTime
            numberingformat                 = new NumberingFormat();
            numberingformat.NumberFormatId  = 165;
            numberingformat.FormatCode      = "dd/mm/yyyy hh:mm:ss";
            numberingformats.Append(numberingformat);

            //Numeric
            numberingformat                 = new NumberingFormat();
            numberingformat.NumberFormatId  = 166;
            numberingformat.FormatCode      = "#0";
            numberingformats.Append(numberingformat);

            //Float
            numberingformat                 = new NumberingFormat();
            numberingformat.NumberFormatId  = 167;
            str_FloatFormatCode             = "#,##0.0";
            for (int Index = 1; Index < CultureInfoObj.NumberFormat.NumberDecimalDigits; Index++)
            {
                str_FloatFormatCode += "0";
            }
            numberingformat.FormatCode      = str_FloatFormatCode;
            numberingformats.Append(numberingformat);

            numberingformats.Count          = (uint)numberingformats.ChildElements.Count;

            // Cell formatting
            cellformats                     = new CellFormats();

                // HeaderRow
            cellformat                      = new CellFormat();
            cellformat.NumberFormatId       = 164;
            cellformat.FontId               = 0;
            cellformat.FillId               = 0;
            cellformat.BorderId             = 0;
            alignment                       = new Alignment();
            alignment.WrapText              = BooleanValue.FromBoolean(true);
            cellformat.Alignment            = alignment;
            cellformat.ApplyNumberFormat    = BooleanValue.FromBoolean(true);
            cellformats.Append(cellformat);
                // Cell with string value 
            cellformat                      = new CellFormat();
            cellformat.NumberFormatId       = 164;
            cellformat.FontId               = 1;
            cellformat.FillId               = 0;
            cellformat.BorderId             = 0;
            alignment                       = new Alignment();
            alignment.WrapText              = BooleanValue.FromBoolean(true);
            cellformat.Alignment            = alignment;
            cellformat.ApplyNumberFormat    = BooleanValue.FromBoolean(true);
            cellformats.Append(cellformat);
                // Cell with datetime value
            cellformat                      = new CellFormat();
            cellformat.NumberFormatId       = 22; // Standard DateTime format
            cellformat.FontId               = 1;
            cellformat.FillId               = 0;
            cellformat.BorderId             = 0;
            cellformat.ApplyNumberFormat    = BooleanValue.FromBoolean(true);
            cellformats.Append(cellformat);
                // Cell with numeric value
            cellformat                      = new CellFormat();
            cellformat.NumberFormatId       = 166;
            cellformat.FontId               = 1;
            cellformat.FillId               = 0;
            cellformat.BorderId             = 0;
            cellformat.ApplyNumberFormat    = BooleanValue.FromBoolean(true);
            cellformats.Append(cellformat);
                // Cell with float value
            cellformat                      = new CellFormat();
            cellformat.NumberFormatId       = 167;
            cellformat.FontId               = 1;
            cellformat.FillId               = 0;
            cellformat.BorderId             = 0;
            cellformat.ApplyNumberFormat    = BooleanValue.FromBoolean(true);
            cellformats.Append(cellformat);

            cellformats.Count               = (uint)cellformats.ChildElements.Count;

            //Cell style
            cellstyles                      = new CellStyles();
            cellstyle                       = new CellStyle();
            cellstyle.Name                  = "Normal";
            cellstyle.FormatId              = 0;
            cellstyle.BuiltinId             = 0;
            cellstyles.Append(cellstyle);
            cellstyles.Count                = (uint)cellstyles.ChildElements.Count;

            stylesheet.Append(numberingformats);
            stylesheet.Append(fonts);
            stylesheet.Append(fills);
            stylesheet.Append(borders);
            stylesheet.Append(cellformats);
            stylesheet.Append(cellstyles);

            return stylesheet;
        }

        #endregion

        /// <summary>
        /// This method is used to get Filter personalization XML of the Schedule Report for given Schedule Instance
        /// </summary>
        /// <param name="ReportSchedulerInstanceID"></param>
        /// <param name="FilterPersonalizationXML"></param>
        /// <returns></returns>
        private ERRORXML GetFilterPersonalizationXMLForScheduleInstance(String ReportSchedulerInstanceID, ref String FilterPersonalizationXML)
        {
            #region member variables            
            StringBuilder       sbSessionInfo               = null;
            StringBuilder       sbInputXML                  = null;            
            String              SCHInstanceXML              = null;
            XPathDocument       xpDoc_InstanceXML           = null;
            XPathNavigator      xpNav_InstanceXML           = null;
            XPathNavigator      xpNav_FilterXML             = null;            
            AssetlinkSearch     objSearch                   = null;
            #endregion

           
            try
            {            
                objSearch               = new AssetlinkSearch(UserID, Language, TenantName);
                sbSessionInfo           = new StringBuilder(16);
                sbSessionInfo.AppendFormat("<SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'></SESSIONINFO>", Language, TenantName, UserID, SessionID);

                sbInputXML = new StringBuilder(16);
                sbInputXML.AppendFormat(@"<SEARCH>
	                                        <TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>
	                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                        <TARGETENTITY>
		                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>SCHEDREPORTINSTANCEID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>		                                                        
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>SCHEDULEDREPORT_INSTANCES</RELATIONSHIPNAME>
                                                    </FILTER>                                            
                                                </FILTERLIST> 
                                                <SELECTLIST>
	                                                <EXPRESSION>
		                                                <FIELDNAME>SCHEDREPORTID</FIELDNAME>
		                                                <ALIAS>SCHEDREPORTID</ALIAS>
	                                                </EXPRESSION>	
                                                    <EXPRESSION>
		                                                <FIELDNAME>SCHEDULEREPORTFILTERXML</FIELDNAME>
		                                                <ALIAS>SCHEDULEREPORTFILTERXML</ALIAS>
	                                                </EXPRESSION>	
                                                </SELECTLIST>	                                                                                                                                                                             
                                            </TARGETENTITY>{0}
                                        </SEARCH>", sbSessionInfo.ToString(), ReportSchedulerInstanceID);

                ErrorXML = objSearch.GetHierarchialData(sbInputXML.ToString(), ref SCHInstanceXML);
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    xpDoc_InstanceXML       = new XPathDocument(new StringReader(SCHInstanceXML));
                    xpNav_InstanceXML       = xpDoc_InstanceXML.CreateNavigator();
                    xpNav_FilterXML         = xpNav_InstanceXML.SelectSingleNode("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS/SCHEDULEREPORTFILTERXML");
                    if (xpNav_FilterXML != null && !String.IsNullOrEmpty(xpNav_FilterXML.InnerXml))
                    {
                        FilterPersonalizationXML = xpNav_FilterXML.InnerXml;
                    }
                }
            }
            catch (Exception)
            {
                FilterPersonalizationXML = null;
            }
            return ErrorXML;
        }
        #endregion

        #region Public Methods for Report Scheduler

        /// <summary>
        /// <CreatedBy>Bhavana Chavan</CreatedBy>
        /// <purpose> Creates instance of Schedule Report along with instances in SCHEDULEDREPORTINSTANCES table depending on Run Cycle. </purpose>
        /// </summary>
        /// <param name="FilterXML, InputDataXML"></param>
        /// <returns>ErrorXML</returns>

        public ERRORXML CreateReportSchedule(String FilterXML, String InputDataXML)
        {
            try
            {
                ErrorXML = SaveScheduleReportInstanceDetails(null, FilterXML, InputDataXML, true);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_CREATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "CreateReportSchedule", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_CREATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "CreateReportSchedule", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return ErrorXML;
        }

        /// <summary>
        /// <CreatedBy>Bhavana Chavan</CreatedBy>
        /// <purpose> This will first marked old instances as delete and 
        /// Creats new instance of Schedule Report along with instances in SCHEDULEDREPORTINSTANCES table depending on Run Cycle. </purpose>
        /// </summary>
        /// <param name="FilterXML, InputDataXML"></param>
        /// <returns>ErrorXML</returns>

        public ERRORXML UpdateReportSchedule(String ScheduleReportID, String FilterXML, String InputDataXML)
        {
            String ScheduledInstanceIDList = null;
            String strDisNextRunID = null;
            StringBuilder sbBatchUpdateInputXml = null;            

            try
            {
                //Update old instances as marked as delete whose status as pending.
                //first get all the instances which satisifies the criteria
                ErrorXML = GetAllScheduledInstances(ScheduleReportID, out ScheduledInstanceIDList, out strDisNextRunID);
                if (ErrorXML == null)
                {
                    if (!String.IsNullOrEmpty(ScheduledInstanceIDList))
                    {
                        //call for batch update
                        sbBatchUpdateInputXml = new StringBuilder(16);
                        sbBatchUpdateInputXml.AppendFormat(@"<ENTITYCOLLECTION>
                                                                <SCHEDULEDREPORTINSTANCES ForbidPartialBatchUpdate='FALSE'>
                                                                    <FILTERLIST>
						                                                <FILTER>
							                                                <EXPRESSION>SCHEDREPORTINSTANCEID</EXPRESSION>
							                                                <OPERATOR>IN</OPERATOR>
							                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                        </FILTER>
                                                                    </FILTERLIST>
                                                                    <MARKEDASDELETED>1</MARKEDASDELETED>
                                                                </SCHEDULEDREPORTINSTANCES>
                                                            </ENTITYCOLLECTION>", ScheduledInstanceIDList);

                        QMQueueObj = new OperationQueue(false);
                        OperObj = new Operation("ENTITY", "BATCHUPDATE");
                        OperObj.AddParameter("INPUTXML", sbBatchUpdateInputXml.ToString());

                        QMQueueObj.AddOperation(OperObj);

                        if (!String.IsNullOrEmpty(strDisNextRunID))
                        {
                            OperObj = new Operation("RELATION", "DISASSOCIATEINSTANCEEX");
                            OperObj.AddParameter("CONTAINERID", ScheduleReportID);
                            OperObj.AddParameter("CONTAINSID", strDisNextRunID);
                            OperObj.AddParameter("RELATIONSHIPNAME", "SCHRPT_NEXTRUN");
                            QMQueueObj.AddOperation(OperObj);
                        }

                        OperationXML = QMQueueObj.Xml;

                        OMIObj = new OMInterface(UserID, Language, TenantName);
                        ErrorXML = OMIObj.Execute(ref OperationXML);
                    }
                    if (ErrorXML == null)
                    {
                        //Stamp new instances
                        ErrorXML = SaveScheduleReportInstanceDetails(ScheduleReportID, FilterXML, InputDataXML, true);

                        if (ErrorXML != null)
                        {
                            ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", null, ErrorXML);
                            ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                            return ReturnErrorXML;
                        }
                    }
                    else
                    {
                        ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", null, ErrorXML);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }

                }
                else
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return ErrorXML;
        }

        /// <summary>
        /// <CreatedBy>Bhavana Chavan</CreatedBy>
        /// <purpose>This will update only the SCHEDULEREPORTDETAILS table,it will not update SCHEDULEDREPORTINSTANCES instances.</purpose>
        /// </summary>
        /// <param name="InputDataXML"></param>
        /// <returns>ErrorXML</returns>

        public ERRORXML UpdateReportSchedule(String InputDataXML, bool IsReschedule)
        {
            String ScheduleRptID = ALCommon.GetInnerValue(ref InputDataXML, "<SCHEDREPORTID>", "</SCHEDREPORTID>");            

            try
            {
                if (IsReschedule)
                {
                    String FilterXML = ALCommon.GetInnerValue(ref InputDataXML, "<FILTERVALUES>", "</FILTERVALUES>");
                    ErrorXML = SaveScheduleReportInstanceDetails(ScheduleRptID, FilterXML, InputDataXML, false);
                    if (ErrorXML != null)
                    {
                        ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", null, ErrorXML);
                        ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                        return ReturnErrorXML;
                    }
                }
                else
                {
                    QMQueueObj = new OperationQueue(false);
                    OperObj = new Operation("ENTITY", "UPDATEDATA");
                    OperObj.AddParameter("INPUTXML", InputDataXML);

                    QMQueueObj.AddOperation(OperObj);
                    OperationXML = QMQueueObj.Xml;

                    OMIObj = new OMInterface(UserID, Language, TenantName);
                    ErrorXML = OMIObj.Execute(ref OperationXML);
                }
                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_UPDATEREPORTSCHEDULE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "UpdateReportSchedule", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            return ErrorXML;
        }

         /// <summary>
        /// <CreatedBy>Yuvraj Deshmukh</CreatedBy>
        /// <purpose>This function will update the Current Schedule Instance Details and will create the next schedule instance record.
        /// It will also update the SCHRPT_LASTRUN and SCHRPT_NEXTRUN Relations</purpose>
        /// </summary>
        /// <param name="ScheduleDetailsID">Report Schedule Details ID</param>
        /// <param name="LastExecutedScheduleInstanceID">ID of Last Executed Report Schedule Instance for which record Details need to be update and next Schedule Instance 
        /// need to be calculated</param>
        /// <param name="ScheduleInstanceStatus">To Update Schedule Instance Status</param>
        /// <param name="ErrorXml">To Update Schedule Instance Error xml if any</param>
        /// <param name="NotificationID">Notification Stamped for the Instance</param>
        /// <returns>if error will return error xml or else return nill</returns>
        public ERRORXML UpdateReportScheduleInstanceForNextRun(String ScheduleDetailsID, String LastExecutedScheduleInstanceID, String ScheduleInstanceStatus, String ErrorXml, ulong NotificationID)
        {
            StringBuilder ScheduleSearchXml         = new StringBuilder(1024);
            StringBuilder ScheduleDetailsUdateXml   = new StringBuilder(1024);

            try
            {
                #region Prepare Serch Xml
                ScheduleSearchXml.Append("<SEARCH>");
                ScheduleSearchXml.AppendFormat("<SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />", SessionID, UserID, TenantName, Language);
                ScheduleSearchXml.Append(@"<TARGETOBJECT>SCHEDULEDREPORTDETAILS</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <TARGETENTITY><PRIVILEGENAME>READ</PRIVILEGENAME>
                                        <SELECTLIST>
                                              <EXPRESSION>
                                                <ALIAS>SCHEDREPORTID</ALIAS>
                                                <FIELDNAME>SCHEDREPORTID</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>STARTDATE</ALIAS>
                                                <FIELDNAME>STARTDATE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>ENDDATE</ALIAS>
                                                <FIELDNAME>ENDDATE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>SCHEDTIME</ALIAS>
                                                <FIELDNAME>SCHEDTIME</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>SCHEDTYPE</ALIAS>
                                                <FIELDNAME>SCHEDTYPE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>RUNCYCLE</ALIAS>
                                                <FIELDNAME>RUNCYCLE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>DAILYOPTION</ALIAS>
                                                <FIELDNAME>DAILYOPTION</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>WEEKLYOPTION</ALIAS>
                                                <FIELDNAME>WEEKLYOPTION</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>YEARLYOPTION</ALIAS>
                                                <FIELDNAME>YEARLYOPTION</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>MONTHLYOPTIONTYPE</ALIAS>
                                                <FIELDNAME>MONTHLYOPTIONTYPE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>MONTHLYOPTIONDATE</ALIAS>
                                                <FIELDNAME>MONTHLYOPTIONDATE</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>MONTHLYOPTIONDAY</ALIAS>
                                                <FIELDNAME>MONTHLYOPTIONDAY</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>MONTHLYOPTIONWEEKDAY</ALIAS>
                                                <FIELDNAME>MONTHLYOPTIONWEEKDAY</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>SELECTEDMONTHS</ALIAS>
                                                <FIELDNAME>SELECTEDMONTHS</FIELDNAME>
                                              </EXPRESSION>
                                        </SELECTLIST>
                                        <FILTERLIST>
                                            <FILTER>
                                              <FILTERID>1</FILTERID>
                                              <EXPRESSION>SCHEDREPORTID</EXPRESSION>
                                              <OPERATOR>EQUALTO</OPERATOR>
                                              <EXPRESSIONVALUE>" + ScheduleDetailsID + @"</EXPRESSIONVALUE>
                                              <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                            </FILTER>
                                        </FILTERLIST>");

                if (!String.IsNullOrEmpty(LastExecutedScheduleInstanceID))
                    ScheduleSearchXml.Append(@"<RELATIONSHIP NAME='SCHEDULEDREPORT_INSTANCES'>
                                            <FILTERLIST>
                                                <FILTER>
                                                  <EXPRESSION>SCHEDREPORTINSTANCEID</EXPRESSION>
                                                  <OPERATOR>EQUALTO</OPERATOR>
                                                  <EXPRESSIONVALUE>" + LastExecutedScheduleInstanceID + @"</EXPRESSIONVALUE>
                                                  <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                            </FILTERLIST>
                                          <SELECTLIST>
                                              <EXPRESSION>
                                                <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                                <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                              </EXPRESSION>
                                              <EXPRESSION>
                                                <ALIAS>INSTANCEDATETIME</ALIAS>
                                                <FIELDNAME>INSTANCEDATETIME</FIELDNAME>
                                              </EXPRESSION>
                                          </SELECTLIST>
                                        </RELATIONSHIP>

                                        <RELATIONSHIP NAME='SCHRPT_NEXTRUN'>
                                          <SELECTLIST>
                                            <EXPRESSION>
                                              <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                              <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                            </EXPRESSION>
                                          </SELECTLIST>
                                        </RELATIONSHIP>

                                        <RELATIONSHIP NAME='SCHRPT_LASTRUN'>
                                          <SELECTLIST>
                                            <EXPRESSION>
                                              <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
                                              <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
                                            </EXPRESSION>
                                          </SELECTLIST>
                                        </RELATIONSHIP>");

                ScheduleSearchXml.Append(@"</TARGETENTITY>
                                   </SEARCH>");
                #endregion

                String SearchResult = null;
                OMInterface OMIObj  = new OMInterface(UserID, Language, TenantName);
                String ErrorXML     = OMIObj.GetHierarchialData(ScheduleSearchXml.ToString(), ref SearchResult);

                if (ErrorXML == null && !String.IsNullOrEmpty(SearchResult))
                {
                    XmlNode ScheduledReportDetailsNode = null;
                    XmlDocument ScheduledReportDoc = new XmlDocument();
                    DateTime CurrentScheduleDateTime = GetBaseDate();

                    ScheduledReportDoc.LoadXml(SearchResult);
                    ScheduledReportDetailsNode = ScheduledReportDoc.SelectSingleNode("SCHEDULEDREPORTDETAILSCOLLECTION/SCHEDULEDREPORTDETAILS");

                    #region Prepare Update ScheduleDetails Xml
                    ScheduleDetailsUdateXml.Append("<SCHEDULEDREPORTDETAILS>");
                    ScheduleDetailsUdateXml.AppendFormat("<SCHEDREPORTID>{0}</SCHEDREPORTID>", ScheduleDetailsID);

                    #region Disassociate LastRun and NextRun
                    if (ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_LASTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID") != null)
                        ScheduleDetailsUdateXml.AppendFormat("<SCHRPT_LASTRUN TYPE=\"RELATION\" DISASSOCIATE=\"TRUE\">{0}</SCHRPT_LASTRUN>", ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_LASTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID").InnerText);

                    if (ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_NEXTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID") != null)
                        ScheduleDetailsUdateXml.AppendFormat("<SCHRPT_NEXTRUN TYPE=\"RELATION\" DISASSOCIATE=\"TRUE\">{0}</SCHRPT_NEXTRUN>", ScheduledReportDetailsNode.SelectSingleNode("SCHRPT_NEXTRUN/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID").InnerText);
                    #endregion

                    #region Update Current Schedule Instance
                    if (!String.IsNullOrEmpty(LastExecutedScheduleInstanceID))
                    {
                        ScheduleDetailsUdateXml.AppendFormat(@"<SCHEDULEDREPORT_INSTANCES TYPE='RELATION'>
                                                                    <SCHEDULEDREPORTINSTANCES MODIFIED='TRUE'>
                                                                        <SCHEDREPORTINSTANCEID>" + LastExecutedScheduleInstanceID + @"</SCHEDREPORTINSTANCEID>
                                                                        <STATUS>" + ScheduleInstanceStatus + @"</STATUS>
                                                                        <ERRORCONFIGURATIONXML>" + ErrorXml + @"</ERRORCONFIGURATIONXML>
                                                                        {0}
                                                                    </SCHEDULEDREPORTINSTANCES>
                                                                </SCHEDULEDREPORT_INSTANCES>",
                                                                        NotificationID > 0 ? "<SCHRPTINSTANCE_NOTIFICATION TYPE=\"RELATION\">" + NotificationID + "</SCHRPTINSTANCE_NOTIFICATION>" : "");

                        ScheduleDetailsUdateXml.AppendFormat("<SCHRPT_LASTRUN TYPE=\"RELATION\" >{0}</SCHRPT_LASTRUN>", LastExecutedScheduleInstanceID);
                    }
                    #endregion


                    #region Get Current Schedule date Time
                    if (!String.IsNullOrEmpty(LastExecutedScheduleInstanceID))
                    {
                        XmlNode ScheduleDateTimeNode = ScheduledReportDetailsNode.SelectSingleNode("SCHEDULEDREPORT_INSTANCES/SCHEDULEDREPORTINSTANCES/INSTANCEDATETIME");
                        if (ScheduleDateTimeNode != null)
                            CurrentScheduleDateTime = XmlToDateTime(ScheduleDateTimeNode.CreateNavigator());
                    }

                    #endregion

                    #region Prepare Next Schedule Instance Details
                    StringBuilder Sb_ScheduleInstanceXml = null;
                    PrepareScheduleInsanceXml(ScheduledReportDetailsNode.OuterXml, true, CurrentScheduleDateTime, out Sb_ScheduleInstanceXml);

                    if (Sb_ScheduleInstanceXml != null)
                    {
                        String NextInstnceUniqueID = null;
                        String ScheduleInstanceXml = Sb_ScheduleInstanceXml.ToString();
                        SetFirstScheduleInsanceUniqueID(ref ScheduleInstanceXml, out NextInstnceUniqueID);

                        if (!String.IsNullOrEmpty(NextInstnceUniqueID))
                        {
                            ScheduleDetailsUdateXml.Append(ScheduleInstanceXml);
                            ScheduleDetailsUdateXml.AppendFormat("<SCHRPT_NEXTRUN TYPE=\"RELATION\" >{0}</SCHRPT_NEXTRUN>", NextInstnceUniqueID);
                        }
                    }
                    #endregion

                    ScheduleDetailsUdateXml.Append("</SCHEDULEDREPORTDETAILS>");
                    #endregion

                    #region Update Datebase
                    OperationQueue QMQueueObj   = new OperationQueue(false);
                    Operation       OperObj     = new Operation("ENTITY", "UPDATEDATA");
                    OperObj.AddParameter("INPUTXML", ScheduleDetailsUdateXml.ToString());

                    QMQueueObj.AddOperation(OperObj);
                    OperationXML    = QMQueueObj.Xml;
                    ErrorXML        = OMIObj.Execute(ref OperationXML);
                    #endregion
                }

            }
            catch (Exception ExceptionObject)
            {
                ALEngineErrorObj.AppendError("REPORTENGINE_SCHEDULEREPORT_UPDATEREPORTSCHEDULEINSTANCEFORNEXTRUNFAILED", this.GetType().ToString(),
                    "UpdateReportScheduleInstanceForNextRun", ExceptionObject, null);

                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExceptionObject);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ErrorXML =  ALEngineErrorObj.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        /// <summary>
        /// <CreatedBy>Bhavana Chavan</CreatedBy>
        /// <purpose> Deletes instance of Schedule Report along with instances in SCHEDULEDREPORTINSTANCES. </purpose>
        /// </summary>
        /// <param name="FilterXML, InputDataXML"></param>
        /// <returns>ErrorXML</returns>

        public ERRORXML DeleteScheduleReport(String SCHRPT_IDList)
        {            
            String[]                    arrSCHRptIDList             = SCHRPT_IDList.Split(';');
            InfrastructureEngineClass   objInfEngine                = null;

            try
            {
                objInfEngine = new InfrastructureEngineClass(SessionID, UserID, Language, TenantName);
                for (int cnt = 0; cnt < arrSCHRptIDList.Length; cnt++)
                {
                    if (arrSCHRptIDList[cnt] != "" && arrSCHRptIDList[cnt] != null)
                    {
                        ErrorXML = objInfEngine.DeleteEntityInstance("SCHEDULEDREPORTDETAILS", arrSCHRptIDList[cnt]);
                        if (ErrorXML != null)
                        {
                            ALEngineErrorObj.AppendError("REPORTENGINE_DELETESCHEDULEREPORT_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "DeleteScheduleReport", null, ErrorXML);
                            ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                            return ReturnErrorXML;
                        }
                    }
                }
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_DELETESCHEDULEREPORT_SCHEDULEREPORTDELETIONFAILED", this.GetType().ToString(), "DeleteScheduleReport", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return ErrorXML;
        }

        /// <summary>
        /// <createddate> 15th Feb 2010 </createddate>
        /// <author> Archana Pisal </author>
        /// <purpose> This API is used to get all the schedule instances marked for deletion i.e. MARKASDELETED = 1 and 
        /// delete these records Permanently.
        /// </purpose>
        /// </summary>
        /// <purpose></purpose>
        /// <returns></returns>
        public ERRORXML DeleteScheduleReportInstanceDetails()
        {
            StringBuilder       SearchInputXML          = null;
            ERRORXML            ErrorXML                = null;
            StringBuilder       ErrorMessage            = null;
            XPathDocument       ScheduleInstanceXMLDoc  = null;
            XPathNavigator      RootNodeNav             = null;
            XPathNodeIterator   InstanceIDNodes         = null;
            XML                 InstanceDetailsXML      = null;
            String              OperationXML            = null;
            OMInterface         OMIObj                  = null;

            try
            {
                SearchInputXML      = new StringBuilder(1024);
                ErrorMessage        = new StringBuilder(100);
                OMIObj              = new OMInterface(UserID, Language, TenantName);

                #region Get  SCHEDULEDREPORTINSTANCES marked for delete
                SearchInputXML.AppendFormat(@"<SEARCH>
                                                    <TARGETOBJECT>SCHEDULEDREPORTINSTANCES</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}' SessionID='{3}'/> 
                                                    <TARGETENTITY>
                                                        <FILTERLIST>
                                                            <FILTER>
	                                                            <EXPRESSION>MARKEDASDELETED</EXPRESSION>
	                                                            <OPERATOR>EQUALTO</OPERATOR>
	                                                            <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
	                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>
                                                        <SELECTLIST>
		                                                    <EXPRESSION>
			                                                    <FIELDNAME>SCHEDREPORTINSTANCEID</FIELDNAME>
			                                                    <ALIAS>SCHEDREPORTINSTANCEID</ALIAS>
		                                                    </EXPRESSION>
	                                                    </SELECTLIST>
                                                    </TARGETENTITY>
                                                </SEARCH>", UserID, TenantName, Language, SessionID);

                ErrorXML = OMIObj.GetHierarchialData(SearchInputXML.ToString(), ref InstanceDetailsXML);
                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_SCHEDULEREPORT_SCHINSTANCEGETFAILED", this.GetType().ToString(), "DeleteScheduleReportInstanceDetails", null, ErrorXML);
                    ErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ErrorXML;
                }

                ScheduleInstanceXMLDoc  = new XPathDocument(new StringReader(InstanceDetailsXML));
                RootNodeNav             = ScheduleInstanceXMLDoc.CreateNavigator();
                if ((RootNodeNav.SelectSingleNode("SCHEDULEDREPORTINSTANCESCOLLECTION/SCHEDULEDREPORTINSTANCES") == null))
                {
                    return null;
                }
                #endregion

                #region Delete Instances details
                InstanceIDNodes = RootNodeNav.Select("SCHEDULEDREPORTINSTANCESCOLLECTION/SCHEDULEDREPORTINSTANCES/SCHEDREPORTINSTANCEID");
                QMQueueObj      = new OperationQueue(true);

                while (InstanceIDNodes.MoveNext()) 
                {
                    OperObj = new Operation("ENTITY", "DELETE");
                    OperObj.AddParameter("ENTITYID", InstanceIDNodes.Current.Value);
                    OperObj.AddParameter("ENTITYTYPENAME", "SCHEDULEDREPORTINSTANCES");
                    QMQueueObj.AddOperation(OperObj);
                }

                OMIObj       = new OMInterface(UserID, Language, TenantName);
                OperationXML = QMQueueObj.Xml;
                ErrorXML     = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_SCHEDULEREPORT_SCHINSTANCEDELETEFAILED", this.GetType().ToString(), "DeleteScheduleReportInstanceDetails", null, ErrorXML);
                    return ALEngineErrorObj.ErrorCollectionXML;
                }
                #endregion
            }
            catch (Exception ExcepObj)
            {
                ALEngineErrorObj.AppendError("REPORTENGINE_SCHEDULEREPORT_SCHINSTANCEDELETEFAILED", this.GetType().ToString(), "DeleteScheduleReportInstanceDetails", ExcepObj, null);
                
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                return ALEngineErrorObj.ErrorCollectionXML;
            }
            return ALEngineErrorObj.ErrorCollectionXML;
        }


        /// <summary>
        /// <CreatedBy>Bhavana Chavan</CreatedBy>
        /// <purpose> Restamp the instance in SCHEDULEDREPORTINSTANCES table for the instance which has status as ERROR. </purpose>
        /// </summary>
        /// <param name="ScheduleReportID, ref IsAlreadyExist"></param>
        /// <returns>ErrorXML</returns>
        public ERRORXML ResendErrorInstance(String ScheduleReportID, ref bool IsAlreadyExist)
        {            
            String                          str_SCHRptDetails           = null;
            String                          LastRunInstanceID           = null;
            String                          NextRunInstanceID           = null;
            DateTime                        objStampDate;
            InfrastructureEngineClass       objInfEngine                = null;
            StringBuilder                   sb_SCHRptDetails            = null;

            try
            {
                if (DateTime.Now.Minute < 25)
                {
                    objStampDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, 30, 0);                    
                }
                else if (DateTime.Now.Minute < 55)
                {
                    objStampDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour + 1, 0, 0);
                }
                else
                {
                    objStampDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour + 1, 30, 0);
                }

                ErrorXML = CheckWhetherInstanceTimeExist(ScheduleReportID, objStampDate, ref IsAlreadyExist);
                if (ErrorXML == null)
                {
                    if (!IsAlreadyExist)
                    {
                        ErrorXML = GetLastRunNextRunInstanceID(ScheduleReportID, out LastRunInstanceID, out NextRunInstanceID);
                        if (ErrorXML == null)
                        {
                            sb_SCHRptDetails = new StringBuilder(16);
                            sb_SCHRptDetails.AppendFormat(@"<SCHEDULEDREPORTDETAILS>
                                                            <SCHEDREPORTID>{0}</SCHEDREPORTID>
                                                            <SCHRPT_LASTRUN TYPE='RELATION'>
                                                                <SCHEDULEDREPORTINSTANCES MODIFIED='TRUE'>
                                                                    <SCHEDREPORTINSTANCEID>{1}</SCHEDREPORTINSTANCEID>                                                                    
                                                                    <STATUS>RESUBMITTED</STATUS>
                                                                </SCHEDULEDREPORTINSTANCES>
                                                            </SCHRPT_LASTRUN>", ScheduleReportID, LastRunInstanceID);
                            if(String.IsNullOrEmpty(NextRunInstanceID))
                            {
                                ulong NewUniqueID = 0;
                                Assetlink.UniqueID.Read(ref NewUniqueID);
                                sb_SCHRptDetails.AppendFormat(@"<SCHEDULEDREPORT_INSTANCES TYPE='RELATION'>
                                                                    <SCHEDULEDREPORTINSTANCES>
                                                                        <SCHEDREPORTINSTANCEID>{0}</SCHEDREPORTINSTANCEID>
                                                                        <INSTANCEDATETIME>{1}</INSTANCEDATETIME>
                                                                        <STATUS>PENDING</STATUS>
                                                                    </SCHEDULEDREPORTINSTANCES>
                                                               </SCHEDULEDREPORT_INSTANCES>
                                                               <SCHRPT_NEXTRUN TYPE='RELATION' >{0}</SCHRPT_NEXTRUN>", NewUniqueID.ToString(), GetDateInXML(objStampDate));
                            }
                            else
                            {
                                 sb_SCHRptDetails.AppendFormat(@"<SCHRPT_NEXTRUN TYPE='RELATION'>
                                                                    <SCHEDULEDREPORTINSTANCES MODIFIED='TRUE'>
                                                                        <SCHEDREPORTINSTANCEID>{0}</SCHEDREPORTINSTANCEID>                                                                    
                                                                        <INSTANCEDATETIME>{1}</INSTANCEDATETIME>
                                                                        <STATUS>PENDING</STATUS>
                                                                    </SCHEDULEDREPORTINSTANCES>
                                                                </SCHRPT_NEXTRUN>", NextRunInstanceID, GetDateInXML(objStampDate));
                            }

                            sb_SCHRptDetails.AppendFormat("</SCHEDULEDREPORTDETAILS>");

                            str_SCHRptDetails = sb_SCHRptDetails.ToString();

                            objInfEngine = new InfrastructureEngineClass(SessionID, UserID, Language, TenantName);
                            ErrorXML = objInfEngine.UpdateEntityInstance(ref str_SCHRptDetails);
                            if (ErrorXML != null)
                            {
                                ALEngineErrorObj.AppendError("REPORTENGINE_RESENDERRORINSTANCE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "ResendErrorInstance", null, ErrorXML);
                                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                                return ReturnErrorXML;
                            }
                        }
                        else
                        {
                            ALEngineErrorObj.AppendError("REPORTENGINE_RESENDERRORINSTANCE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "ResendErrorInstance", null, ErrorXML);
                            ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                            return ReturnErrorXML;
                        }
                    }
                }
                else
                {
                    ALEngineErrorObj.AppendError("REPORTENGINE_RESENDERRORINSTANCE_SCHEDULEREPORTCREATIONFAILED", this.GetType().ToString(), "ResendErrorInstance", null, ErrorXML);
                    ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            catch (Exception ExcepObj)
            {
                ReturnErrorXML = ALEngineError.PrepareExceptionXML(ExcepObj);
                ExceptionTracing.LogError(TenantName, ReturnErrorXML);
                ALEngineErrorObj.AppendError("REPORTENGINE_RESENDERRORINSTANCE_SCHEDULEREPORTDELETIONFAILED", this.GetType().ToString(), "ResendErrorInstance", ExcepObj, null);
                ReturnErrorXML = ALEngineErrorObj.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return ErrorXML;
        }
        #endregion        
    }
}
